/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.faces.FactoryFinder;
import javax.faces.application.Application;
import javax.faces.application.ApplicationFactory;
import javax.faces.context.FacesContext;



public class ResourceHandler {
	public final static String PROPERTIES_FILE = "runtimeibm"; //$NON-NLS-1$
	public static Boolean USING_CUSTOM = null;
	public static String CUSTOM_FILE = null;
	
	/*
	 * <p>Check if custom resource bundle is being used.</p>
	 */
	public static boolean isUsingCustomBundle(){
		if(USING_CUSTOM != null){
			return USING_CUSTOM.booleanValue();
		}
		FacesContext context = FacesContext.getCurrentInstance();
		Application app = null;
		if(context != null){
			app = context.getApplication();
		} else {
			//no context, try to retrieve application through factory
			ApplicationFactory af = (ApplicationFactory) FactoryFinder.getFactory(FactoryFinder.APPLICATION_FACTORY);
			app = af.getApplication();
		}
		if(app != null){
			String f = app.getMessageBundle();
			if(f != null){
				CUSTOM_FILE = f;
				USING_CUSTOM = Boolean.TRUE;
				return USING_CUSTOM.booleanValue();
			}
			USING_CUSTOM = Boolean.FALSE;
		}
		//just in case application object is not available, return false
		return false;
	}
	
	/*
	 * <p>Returns the runtimeibm bundle.</p>
	 */
	public static ResourceBundle getBundle(FacesContext context) {
		if(context == null) {
			context = FacesContext.getCurrentInstance();
		}
		if(context != null) {
			return ResourceBundle.getBundle(PROPERTIES_FILE, context.getViewRoot().getLocale());
		}
		return ResourceBundle.getBundle(PROPERTIES_FILE);
	}

	/**
	 * @param key
	 * @return return string for server locale
	 */
	public static String getServerString(String key) {
		try {
			return ResourceBundle.getBundle(PROPERTIES_FILE).getString(key);
		} catch (MissingResourceException e) {
			return "!" + key + "!";//$NON-NLS-2$//$NON-NLS-1$
		}
	}
	
	/*
	 *<p>Should be used to retrieve FacesMessage strings, this method checks the message-bundle first if it is 
	 * defined in the project's faces-config.xml file.</p> 
	 */
	public static String getFacesMessageString(ResourceBundle bundle, String key) {
		
		//first check in message-bundle if set
		if(isUsingCustomBundle()){
			ResourceBundle customBundle = null;
			FacesContext context = FacesContext.getCurrentInstance();
			if(context != null){
				customBundle = ResourceBundle.getBundle(CUSTOM_FILE, context.getViewRoot().getLocale());
			} else {
				customBundle = ResourceBundle.getBundle(CUSTOM_FILE);
			}
			if(customBundle != null){
				try{
					return customBundle.getString(key);
				} catch (MissingResourceException e){
					//do nothing, look for key in runtimeibm.properties
				}
			}
		}
		
		if(bundle == null) {
			bundle = getBundle(null);		
		}
		
		if (bundle != null) {
			try {
				return bundle.getString(key);
			} catch (MissingResourceException e) {
				return "!" + key + "!";//$NON-NLS-2$//$NON-NLS-1$
			}
		} 
		//key value was not founf in bundle
		return "!" + key + "!";//$NON-NLS-2$//$NON-NLS-1$
	}
	
	/*
	 * <p>Returns string for matching key in runtimeibm.properties, message-bundle is not checked.<p>
	 */
	public static String getString(ResourceBundle bundle, String key) {
		if(bundle == null) {
			bundle = getBundle(null);		
		}
		
		if (bundle != null) {
			try {
				return bundle.getString(key);
			} catch (MissingResourceException e) {
				return "!" + key + "!";//$NON-NLS-2$//$NON-NLS-1$
			}
		} 

		return "!" + key + "!";//$NON-NLS-2$//$NON-NLS-1$
	}
	
	
	
	public static String getString(ResourceBundle bundle, String key, Object[] args) {
		try {
			return MessageFormat.format(getString(bundle, key), args);
		}
		catch (IllegalArgumentException e) {
			return getString(bundle, key);
		}
	}

	/**
	 * @param string
	 * @param context
	 * @return
	 */
	public static String buildLocalizedLibName(ResourceBundle bundle, String lib) {
		if(bundle == null) {
			bundle = getBundle(null);		
		}
		if(bundle == null) {
			return lib;
		}
		String suffix = bundle.getString("Locale.suffix"); //$NON-NLS-1$
		if(suffix == null || suffix.trim().length() == 0 || suffix.equals("!Locale.suffix!") || suffix.equals("en")) { //$NON-NLS-1$
			return lib;
		}
		int n = lib.lastIndexOf(".js"); //$NON-NLS-1$
		return lib.substring(0, n) + "_" + suffix + ".js"; //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	public static String buildLocalizedLibName(String lib, Locale locale) {
		ResourceBundle bundle = ResourceBundle.getBundle(PROPERTIES_FILE, locale);
		return buildLocalizedLibName(bundle, lib);
	}

}
