/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.application;

import java.io.IOException;
import java.lang.reflect.Method;
import java.util.HashMap;

import javax.faces.application.StateManager;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;

public class DevelopmentStateManager extends StateManager {

	private StateManager stateManager = null;

	/**
	 * 
	 */
	public DevelopmentStateManager(StateManager stateManager) {
		this.stateManager = stateManager;
		System.out.println("Develoment State Manager Decorator loaded.");
	}

	/*
	 * @see javax.faces.application.StateManager#saveSerializedView(javax.faces.context.FacesContext)
	 */
	public SerializedView saveSerializedView(FacesContext fc) {
		return stateManager.saveSerializedView(fc);
	}

	/*
	 * @see javax.faces.application.StateManager#writeState(javax.faces.context.FacesContext,
	 *      javax.faces.application.StateManager.SerializedView)
	 */
	public void writeState(FacesContext fc, SerializedView sv)
			throws IOException {
		stateManager.writeState(fc, sv);
	}

	/*
	 * @see javax.faces.application.StateManager#getComponentStateToSave(javax.faces.context.FacesContext)
	 */
	protected Object getComponentStateToSave(FacesContext fc) {
		// should never be called
		return null;
	}

	/*
	 * @see javax.faces.application.StateManager#getTreeStructureToSave(javax.faces.context.FacesContext)
	 */
	protected Object getTreeStructureToSave(FacesContext fc) {
		// should never be called
		return null;
	}

	/*
	 * @see javax.faces.application.StateManager#restoreComponentState(javax.faces.context.FacesContext,
	 *      javax.faces.component.UIViewRoot, java.lang.String)
	 */
	protected void restoreComponentState(FacesContext fc, UIViewRoot arg1,
			String arg2) {
		// should never be called
	}

	/*
	 * @see javax.faces.application.StateManager#restoreTreeStructure(javax.faces.context.FacesContext,
	 *      java.lang.String, java.lang.String)
	 */
	protected UIViewRoot restoreTreeStructure(FacesContext fc, String arg1,
			String arg2) {
		// should never be called
		return null;
	}

	/*
	 * @see javax.faces.application.StateManager#restoreView(javax.faces.context.FacesContext,
	 *      java.lang.String, java.lang.String)
	 */
	public UIViewRoot restoreView(FacesContext fc, String viewId,
			String renderKitId) {
		/*
		 * Cannot just make a simple not POST check as there can be "get" requests caused
		 * by Ajax, graphicIageEx and outputLinkEx renderings
		 * 
		 * if (!("POST".equalsIgnoreCase(request.getMethod()))) { // if not a
		 * submit then start with a new tree System.out.println("Development
		 * State Manager: View request - skipping restore view."); return null; }
		 * 
		 */
		if (isFileModified(viewId)) {
			// this should catch the "POST" refresh
			System.out
					.println("Development State Manager: View modified - discarding restored view.");
			return null;
		}
		UIViewRoot viewRoot = stateManager.restoreView(fc, viewId, renderKitId);
		if (null == viewRoot) {
			storeFileInfo(fc, viewId);
		}
		return viewRoot;
	}

	private HashMap jspFileInfoMap = null;

	// class for storing jsp file info

	public class JspFileInfo {

		private String filePath;

		private long timestamp;

		public JspFileInfo(String path, long time) {
			filePath = path;
			timestamp = time;
		}

		public String getFilePath() {
			return filePath;
		}

		public long getTimestamp() {
			return timestamp;
		}

		public void setFilePath(String string) {
			filePath = string;
		}

		public void setTimestamp(long long1) {
			timestamp = long1;
		}

	}

	private void storeFileInfo(FacesContext context, String viewId) {

		synchronized (this) {
			if (jspFileInfoMap == null) {
				jspFileInfoMap = new HashMap();
			}
			if (jspFileInfoMap.containsKey(viewId)) {
				return;
			}

			String jspPath = getRealJSPPath(context, viewId);

			if (jspPath != null) {
				java.io.File jspFile = new java.io.File(jspPath);

				if (jspFile.exists() && viewId != null) {
					jspFileInfoMap.put(viewId, new JspFileInfo(jspPath, jspFile
							.lastModified()));
				}

			}
		}
	}

	private boolean isFileModified(String viewId) {

		synchronized (this) {
			if (jspFileInfoMap == null) {
				jspFileInfoMap = new HashMap();
			}
			JspFileInfo jspData = (JspFileInfo) jspFileInfoMap.get(viewId);

			if (jspData != null) {
				java.io.File jspFile = new java.io.File(jspData.getFilePath());

				if (jspData.getTimestamp() < jspFile.lastModified()) {
					jspData.setTimestamp(jspFile.lastModified());
					jspFileInfoMap.put(viewId, jspData);
					return true;
				}
			}
		}
		return false;
	}

	private String getRealJSPPath(FacesContext facesContext, String viewId) {
		if (null != viewId) {
			// we are counting on view id being the jsp file name
			String rjspPath = null;
			Object realContext = facesContext.getExternalContext().getContext();
			Class[] params = { String.class };
			try {
				Method m = realContext.getClass().getDeclaredMethod(
						"getRealPath", params);
				Object[] param = { viewId };
				rjspPath = (String) m.invoke(realContext, param);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return rjspPath;
		}
		return null;
	}

}
