/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.application;

import java.io.IOException;
import java.util.ArrayList;

import javax.faces.application.StateManager;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.servlet.ServletRequest;

import com.ibm.faces.util.ContextParams;


public class StateManagerImpl extends StateManager{
	
	private StateManager parentManager = null;
	
    private static Boolean savingEnhancedServerState = null;
    public static String ENHANCED_SERVER_STATE_SAVING = "com.ibm.faces.ENHANCED_SERVER_STATE_SAVING";
    private static String ENHANCED_SERVER_STATE_SAVING_VIEW_LIST = "com.ibm.faces.ENHANCED_SERVER_STATE_SAVING_VIEW_LIST"; 
	
    //add a parameter to change this later
    private int numberOfViewsToStore = -1;
    
    
	public StateManagerImpl(StateManager parent){
		this.parentManager = parent;
	}

	
	
	public SerializedView saveSerializedView(FacesContext context) {
		//here
		if(context != null && isSavingEnhancedServerState(context)){
			synchronized(this){
				String viewId = (String)context.getViewRoot().getAttributes().get(ENHANCED_SERVER_STATE_SAVING);
				if(viewId != null){
					//saving in enhanced server mode, save a serialized viewRoot
					context.getViewRoot().getAttributes().remove(ENHANCED_SERVER_STATE_SAVING);
					SerializedView serializedView = new SerializedView(getTreeStructureToSave(context), context.getViewRoot().processSaveState(context));
            	
					ArrayList views = (ArrayList) context.getExternalContext().getSessionMap().get(ENHANCED_SERVER_STATE_SAVING_VIEW_LIST);
					if(views == null){
						views = new ArrayList();
						context.getExternalContext().getSessionMap().put(ENHANCED_SERVER_STATE_SAVING_VIEW_LIST,views);
					}
					//check if a custom number of view to save has been set, default is 15
					if(numberOfViewsToStore == -1){
						String viewsToSave = context.getExternalContext().getInitParameter(ContextParams.ENHANCED_SERVER_STATE_SAVING_SESSION_STORED_VIEWS);
						if(viewsToSave != null){
							int num = Integer.parseInt(viewsToSave);
							if(num < 1){
								savingEnhancedServerState = Boolean.FALSE;
							} else {
								numberOfViewsToStore = num;
							}
						} else {
							numberOfViewsToStore = 15;
						}
					}
					
					//check amount of stored views, trim if greater than allowed
					if(views.size() >= numberOfViewsToStore){
						context.getExternalContext().getSessionMap().remove(views.remove(0));
					}

					views.add(viewId);
					context.getExternalContext().getSessionMap().put(viewId, serializedView);
				}
			}
		}
		return parentManager.saveSerializedView(context);
	}

	protected Object getTreeStructureToSave(FacesContext context) {
		ViewStructure structure = new ViewStructure(context.getViewRoot());
		structure.buildStructure(context, context.getViewRoot(),structure);
		return structure;
	}
	
	

	public void writeState(FacesContext context, SerializedView arg1) throws IOException {
		this.parentManager.writeState(context,arg1);
		
	}

	public UIViewRoot restoreView(FacesContext context, String viewId, String renderKitId) {
		
		if(isSavingEnhancedServerState(context)){
			synchronized(this){
				//begin: back button
				String postedViewId = obtainPostedServerViewId(context);
				if(null != postedViewId) {
    			//restore viewRoot from serialized view that was saved in enhanced mode
					SerializedView serializedView = (SerializedView)context.getExternalContext().getSessionMap().get(postedViewId);
					ViewStructure viewStructure = (ViewStructure)serializedView.getStructure();
					UIViewRoot viewRoot = (UIViewRoot) viewStructure.createComponent();
					viewStructure.restoreStructure(viewStructure,viewRoot);
					if(null != viewRoot) {
						viewRoot.processRestoreState(context,serializedView.getState());
						return viewRoot;
					} 
					return parentManager.restoreView(context,viewId,renderKitId);
					
				} 
				return parentManager.restoreView(context,viewId,renderKitId);
				
    		}
		} 
		return parentManager.restoreView(context,viewId,renderKitId);
	}
 
	protected UIViewRoot restoreTreeStructure(FacesContext context, String arg1, String arg2) {
		//not used
		return null;
	}

	protected void restoreComponentState(FacesContext context, UIViewRoot arg1, String arg2) {
		//not used
	}
	
	public static boolean isSavingEnhancedServerState(FacesContext context){
		
		if(null == savingEnhancedServerState){
			//only use this for ServletRequest, portlet runtime handles this param its own way
			if (context.getApplication().getStateManager().isSavingStateInClient(context)
				|| !(context.getExternalContext().getRequest() instanceof ServletRequest))
			{
				savingEnhancedServerState = Boolean.FALSE;
			} else {
				savingEnhancedServerState = Boolean.TRUE;
			//not using context-param anymore, now enabled by adding state-manager and view-handler lines in
			//application faces-config.xml
			/*
			else {
				String s=(String)context.getExternalContext().getInitParameter( ENHANCED_SERVER_STATE_SAVING);
				if(s!=null && s.equalsIgnoreCase("TRUE")){
					savingEnhancedServerState=Boolean.TRUE;
				} else {
					savingEnhancedServerState=Boolean.FALSE;
				}
			}
			*/
			}
		}
		return savingEnhancedServerState.booleanValue();
	}
	
    private String obtainPostedServerViewId (FacesContext context){
		ServletRequest request = (ServletRequest)context.getExternalContext().getRequest();
		return request.getParameter(ENHANCED_SERVER_STATE_SAVING);
	}

	protected Object getComponentStateToSave(FacesContext arg0) {
		//not used
		return null;
	}
    
}
