/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.component;

import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

/**
 * @author Brendan Murray
 *
 * This component simply packages AJAX behaviors. Since AJAX assumes
 * client-side code, the meat of any AJAX component is in its renderers.
 */
public class UIAjax extends UIComponentBase {

	/**
	 * Returns component type.
	 * 
	 * @return component type
	 */
	public String getComponentType() {
		return UIAjax.class.getName();
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponent#getFamily()
	 * <p>Return the standard component family for this component.</p>
	 */
	public static final String COMPONENT_FAMILY = "com.ibm.faces.Ajax"; //$NON-NLS-1$

	public String getFamily() {
		return (COMPONENT_FAMILY);
	}

	private String onstart; // this is the string that holds the javascript to be executed before the request is sent
	private String onerror; // this is the string that holds the javascript to be executed if an error has been encountered
	private String oncomplete; // this is the string that holds the javascript to be executed before the request is complete
	
	private String title; // this is the string the hold the (HTML) title of the component (provide audible/visible indication
							// indicating a request is in progress).
	
	private String target; 
	private String inProgress = "LOCKED"; // this is the default value for this.
											// npWhile the request is in progress, the JWL action to apply to the panel. 
											// Currently the only supported action is LOCKED.
	private boolean inProgress_set;
	
	
	/**
	 * @return Returns the oncomplete.
	 */
	public String getOncomplete() {
		
		ValueBinding vb = getValueBinding("oncomplete");
		if(vb != null){
			return (String)vb.getValue(FacesContext.getCurrentInstance());
		}
		
		if(this.oncomplete != null){
			return this.oncomplete;
		}

		//no value set
		return null;	
		
		//return oncomplete;
	}
	/**
	 * @param oncomplete The oncomplete to set.
	 */
	public void setOncomplete(String oncomplete) {
		this.oncomplete = oncomplete;
	}
	/**
	 * @return Returns the onerror.
	 */
	public String getOnerror() {
		
		ValueBinding vb = getValueBinding("onerror");
		if(vb != null){
			return (String)vb.getValue(FacesContext.getCurrentInstance());
		}
		
		if(this.onerror != null){
			return this.onerror;
		}

		//no value set
		return null;
		
		
		//return onerror;
	}
	/**
	 * @param onerror The onerror to set.
	 */
	public void setOnerror(String onerror) {
		this.onerror = onerror;
	}
	/**
	 * @return Returns the onstart.
	 */
	public String getOnstart() {
		
		ValueBinding vb = getValueBinding("onstart");
		if(vb != null){
			return (String)vb.getValue(FacesContext.getCurrentInstance());
		}
		
		if(this.onstart != null){
			return this.onstart;
		}

		//no value set
		return null;
		
		//return onstart;
	}
	/**
	 * @param onstart The onstart to set.
	 */
	public void setOnstart(String onstart) {
		this.onstart = onstart;
	}
	
	/**
	 * @return Returns the title.
	 */
	public String getTitle() {
		
		ValueBinding vb = getValueBinding("title");
		if(vb != null){
			return (String)vb.getValue(FacesContext.getCurrentInstance());
		}
		
		if(this.title != null){
			return this.title;
		}
		
		//no value set
		return null;		
		
		//return title;
	}
	/**
	 * @param title The title to set.
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	
	/**
	 * @return Returns the inProgess.
	 */
	public String getTarget() {
		
		ValueBinding vb = getValueBinding("target");
		if(vb != null){
			return (String)vb.getValue(FacesContext.getCurrentInstance());
		}
		
		if(this.target != null){
			return this.target;
		}
		
		//no value set
		return null;		
		
		//return title;
	}
	/**
	 * @param inProgress The inProgress to set.
	 */
	public void setTarget(String target) {
		this.target = target;
	}
	
	
	/**
	 * @return Returns the inProgess.
	 */
	public String getInProgress() {
		
		if(this.inProgress_set){
			return this.inProgress;
		}
		ValueBinding vb = getValueBinding("inProgress");
		if(vb != null){
			return (String)vb.getValue(FacesContext.getCurrentInstance());
		}
		//no value set
		return this.inProgress;		
	}
	/**
	 * @param inProgress The inProgress to set.
	 */
	public void setInProgress(String inProgress) {
		this.inProgress = inProgress;
		this.inProgress_set = true;
	}
	
	public Object saveState(FacesContext _context) {
		Object _values[] = new Object[7];
		_values[0] = super.saveState(_context);		
		_values[1] = onstart;
		_values[2] = onerror;
		_values[3] = oncomplete;
		_values[4] = title;
		_values[5] = target;
		_values[6] = inProgress;
		return _values;
	}

	public void restoreState(FacesContext _context, Object _state) {
		Object _values[] = (Object[]) _state;
		super.restoreState(_context, _values[0]);
		this.onstart = (java.lang.String) _values[1];
		this.onerror = (java.lang.String) _values[2];
		this.oncomplete = (java.lang.String) _values[3];
		this.title = (java.lang.String) _values[4];
		this.target = (java.lang.String) _values[5];
		this.inProgress = (java.lang.String) _values[6];
	}
	
}
