/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2003, 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.component;

import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

/**
 * @author Brendan Murray
 *
 * This component simply packages AJAX behaviors. Since AJAX assumes
 * client-side code, the meat of any AJAX component is in its renderers.
 */
public class UIBackchannel extends UIComponentBase {
	// The methods associated with the calls
	private String onError, onStart, onComplete;
	private boolean useiframe = false;
	private boolean useiframe_set = false;
	
	/**
	 * Returns component type.
	 * 
	 * @return component type
	 */
	public String getComponentType() {
		return UIAjax.class.getName();
	}
	
	/**
	 * @see javax.faces.component.UIComponent#getFamily()
	 * 
	 * @return component family
	 * 
	 * <p>Return the standard component family for this component.</p>
	 */
	public static final String COMPONENT_FAMILY = "com.ibm.faces.Backchannel"; //$NON-NLS-1$

	public String getFamily() {
		return (COMPONENT_FAMILY);
	}

	  /**
	   * <p>Return the value of the <code>onComplete</code> property.  Contents:</p><p>
	   * JavaScript to be executed after the response has been processed.
	   * </p>
	   */
	  public java.lang.String getOncomplete() {
	    if (null != this.onComplete) {
	      return this.onComplete;
	    }
	    ValueBinding _vb = getValueBinding("oncomplete");
	    if (_vb != null) {
	      return (java.lang.String) _vb.getValue(getFacesContext());
	    } else {
	      return null;
	    }
	  }
	/**
	 * @param onComplete The onComplete to set.
	 */
	public void setOncomplete(String onComplete) {
		this.onComplete = onComplete;
	}

	/**
	   * <p>Return the value of the <code>onError</code> property.  Contents:</p><p>
	   * JavaScript to be executed if an error has been encountered.
	   * </p>
	   */
	  public java.lang.String getOnerror() {
	    if (null != this.onError) {
	      return this.onError;
	    }
	    ValueBinding _vb = getValueBinding("onerror");
	    if (_vb != null) {
	      return (java.lang.String) _vb.getValue(getFacesContext());
	    } else {
	      return null;
	    }
	  }
	  
	/**
	 * @param onError The onError to set.
	 */
	public void setOnerror(String onError) {
		this.onError = onError;
	}
	  /**
	   * <p>Return the value of the <code>onStart</code> property.  Contents:</p><p>
	   * JavaScript to be executed before the request is sent.
	   * </p>
	   */
	  public java.lang.String getOnstart() {
	    if (null != this.onStart) {
	      return this.onStart;
	    }
	    ValueBinding _vb = getValueBinding("onstart");
	    if (_vb != null) {
	      return (java.lang.String) _vb.getValue(getFacesContext());
	    } else {
	      return null;
	    }
	  }

	/**
	 * @param onStart The onStart to set.
	 */
	public void setOnstart(String onStart) {
		this.onStart = onStart;
	}
	public Object saveState(FacesContext context) {
		Object _values[] = new Object[6];
		_values[0] = super.saveState(context);
		_values[1] = onStart;
		_values[2] = onComplete;
		_values[3] = onError;
	    _values[4] = (this.useiframe && this.useiframe_set) ? Boolean.TRUE : Boolean.FALSE;
	    _values[5] = this.useiframe_set ? Boolean.TRUE : Boolean.FALSE;		return _values;
	}
	
	public void restoreState(FacesContext context, Object state) {
		Object _values[] = (Object[])state;
		super.restoreState(context, _values[0]);
		this.onStart    = (String)_values[1];
		this.onComplete = (String)_values[2];
		this.onError    = (String)_values[3];
	    this.useiframe = ((Boolean) _values[8]).booleanValue();
	    this.useiframe_set = ((Boolean) _values[9]).booleanValue();	}
	
	
	  /**
	   * <p>Return the value of the <code>useiframe</code> property.  Contents:</p><p>
	   * Boolean value to specify if the connection should
	   * 				use a hidden iFrame instead of XmlHttpRequest.
	   * </p>
	   */
	  public boolean isUseiframe() {
	    if (this.useiframe_set) {
	      return this.useiframe;
	    }
	    ValueBinding _vb = getValueBinding("useiframe");
	    if (_vb != null) {
	      Object _result = _vb.getValue(getFacesContext());
	      if (_result == null) {
	        return false;
	      } else {
	        return ((Boolean) _result).booleanValue();
	      }
	    } else {
	        return this.useiframe;
	    }
	  }

	  /**
	   * <p>Set the value of the <code>useiframe</code> property.</p>
	   */
	  public void setUseiframe(boolean useiframe) {
	    this.useiframe = useiframe;
	    this.useiframe_set = true;
	  }
}

