/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/
package com.ibm.faces.component;

import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;


/**
 * @author pandrews
 *
 * <p>UIBehavior components add script actions to components or to JSF page using # identifiers, e.g. #body.</p> 
 */
public class UIBehavior extends UIComponentBase {

	
	public static final String COMPONENT_TYPE = "com.ibm.faces.Behavior"; //$NON-NLS-1$
	public static final String COMPONENT_FAMILY = "com.ibm.faces.Behavior"; //$NON-NLS-1$
	
	private String target;
	private String targetAction;
	private String behaviorAction;
	private String onActionFunction;
	
	public String getFamily() {
		return COMPONENT_FAMILY;
	}
	

	public String getBehaviorAction() {
		if(this.behaviorAction != null){
			return this.behaviorAction;
		}
		ValueBinding vb = getValueBinding("behaviorAction");
		if(vb != null){
			return (String)vb.getValue(FacesContext.getCurrentInstance());
		}
		//no value set
		return null;
	}

	public void setBehaviorAction(String behaviorAction) {
		this.behaviorAction = behaviorAction;	
	}

	public String getOnActionFunction() {
		if(this.onActionFunction != null){
			return this.onActionFunction;
		}
		ValueBinding vb = getValueBinding("onActionFunction");
		if(vb != null){
			return (String)vb.getValue(FacesContext.getCurrentInstance());
		}
		//no value set
		return null;
	}
	
	public void setOnActionFunction(String onActionFunction) {
		this.onActionFunction = onActionFunction;	
	}

	public String getTarget() {
		if(this.target != null){
			return this.target;
		}
		ValueBinding vb = getValueBinding("target");
		if(vb != null){
			return (String)vb.getValue(FacesContext.getCurrentInstance());
		}
		//no value set
		return null;
	}

	public void setTarget(String target) {
		this.target = target;	
	}

	public String getTargetAction() {
		if(this.targetAction != null){
			return this.targetAction;
		}
		ValueBinding vb = getValueBinding("targetAction");
		if(vb != null){
			return (String)vb.getValue(FacesContext.getCurrentInstance());
		}
		//no value set
		return null;
	}

	public void setTargetAction(String targetAction) {
		this.targetAction = targetAction;	
	}

	
	public Object saveState(FacesContext _context) {
		Object _values[] = new Object[5];
		_values[0] = super.saveState(_context);		
		_values[1] = target;
		_values[2] = targetAction;
		_values[3] = behaviorAction;
		_values[4] = onActionFunction;
		return _values;
	}

	public void restoreState(FacesContext _context, Object _state) {
		Object _values[] = (Object[]) _state;
		super.restoreState(_context, _values[0]);
		this.target = (java.lang.String) _values[1];
		this.targetAction = (java.lang.String) _values[2];
		this.behaviorAction = (java.lang.String) _values[3];
		this.onActionFunction = (java.lang.String) _values[4];
	}
	
}
