/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.component;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.faces.component.UICommand;
import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;

import com.ibm.faces.util.LifecycleUtil;

/**
 * Component associated with {@link UIDatagrid UIDatagrid}.
 * Used for row edit and row action.
 * Use this component to determine what rows actions occured on (via getParameters()).
 * <p>
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Row_ActionTag},
 * {@link com.ibm.faces.taglib.html_extended.Row_EditTag}
 * <br>
 * <b>Renderers:</b> n/a
 */
public class UICommandEx extends UICommand {

    /**
     * <p>The standard component family for this component.</p>
     */
    public static final String COMPONENT_FAMILY = "com.ibm.faces.CommandEx"; //$NON-NLS-1$

    private transient Map parameters;
    private transient Map rowParameters = null;
    private transient int rowIndex = -1;

    
    /**
     * <p>Returns a Map of component UIParameter children.</p>
     */
	public Map getParameters() {
		return parameters;
	}
	
    /**
     * @return
     */
    public int getRowIndex() {
        return rowIndex;
    }

    /**
     * @return
     */
    public Map getRowParameters() {
        if (this.rowParameters != null) {
            Map map = new HashMap();
            map.putAll(this.rowParameters);
            return map;
        }
        return null;
    }

    /**
     * @param i
     */
    public void setRowIndex(int i) {
        rowIndex = i;
    }

    /**
     * @param map
     */
    public void setRowParameters(Map map) {
        if (this.rowParameters == null) {
            this.rowParameters = new HashMap();
        }
        else {
            this.rowParameters.clear();
        }
        rowParameters.putAll(map);
    }

    public String getFamily() {
        return (COMPONENT_FAMILY);
    }

    /**
     * @see javax.faces.component.UIComponent#processDecodes(javax.faces.context.FacesContext)
     * @param context
     */
    public void processDecodes(FacesContext context) {
    	
        if (context == null) {
            throw new NullPointerException();
        }
        
        if (!isRendered()) {
            return;
        }

        //need to avoid decoding row edit rows if they haven't been clicked
        if (!"com.ibm.faces.RowEdit".equals(this.getRendererType())) { //$NON-NLS-1$
        	//set parameters
        	UIParameter param[] = LifecycleUtil.getParameters(this);
        	HashMap m = new HashMap();
        	for(int i =0; i< param.length; i++){
        		m.put(param[i].getName().toString(), param[i].getValue());
        	}
        	this.parameters = m;
            super.processDecodes(context);
            return;
        }
        //if we get here this is a Row Edit component
        decode(context);
        //if row not "save" clicked, do not decode children        
        if (this.getRowIndex() < 0) {
            return;
        }
        UIData data = findUIData(this);
        if ((data == null) || (data.getRowIndex() != this.getRowIndex())) {
            return;
        }
        // Process the children
        Iterator children = getFacetsAndChildren();
        while (children.hasNext()) {
            UIComponent child = (UIComponent) children.next();
            child.processDecodes(context);
        }
    }

    private static UIData findUIData(UIComponent component) {
        if (component == null) {
            return null;
        }
        if (component instanceof UIData) {
            return (UIData) component;
        }
        return findUIData(component.getParent());
    }

}
