/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.component;

import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

public class UIFormItem extends UIComponentBase {

	/* (non-Javadoc)
	 * @see javax.faces.component.UIComponent#getFamily()
	 * <p>Return the standard component family for this component.</p>
	 */
	public static final String COMPONENT_FAMILY = "com.ibm.faces.FormItem"; //$NON-NLS-1$

	public String getFamily() {
		return (COMPONENT_FAMILY);
	}

	private java.lang.String errorText;

	/**
	 * <p>Return the value of the <code>errorText</code> property.  Contents:</p><p>
	 * This text string provides an error message to alert the end 
	 * 				user that there is a problem with this particular formItem. 
	 * 				If no text is provided then the FacesMessage (created by default 
	 * 				lifecycle processing) will be used.
	 * </p>
	 */
	public java.lang.String getErrorText() {
		if (null != this.errorText) {
			return this.errorText;
		}
		ValueBinding _vb = getValueBinding("errorText");
		if (_vb != null) {
			return (java.lang.String) _vb.getValue(getFacesContext());
		} else {
			return null;
		}
	}

	/**
	 * <p>Set the value of the <code>errorText</code> property.</p>
	 */
	public void setErrorText(java.lang.String errorText) {
		this.errorText = errorText;
	}

	private java.lang.String infoText;

	/**
	 * <p>Return the value of the <code>infoText</code> property.  Contents:</p><p>
	 * This text string provides information to describe the 
	 * 				purpose of this particular formItem (or more specifically, 
	 * 				this formItem's contents).
	 * </p>
	 */
	public java.lang.String getInfoText() {
		if (null != this.infoText) {
			return this.infoText;
		}
		ValueBinding _vb = getValueBinding("infoText");
		if (_vb != null) {
			return (java.lang.String) _vb.getValue(getFacesContext());
		} else {
			return null;
		}
	}

	/**
	 * <p>
	 * Set the value of the <code>infoText</code> property.
	 * </p>
	 */
	public void setInfoText(java.lang.String infoText) {
		this.infoText = infoText;
	}

	private java.lang.String showHelp;

	/**
	 * <p>
	 * Return the value of the <code>showHelp</code> property. Contents:
	 * </p>
	 * <p>
	 * Determines whether this formItem's infoText, or errorText, or both, will
	 * be displayed next to the formItem (the exact layout position is
	 * determined by the helpPosition attribute on formBox). This message is
	 * shown in addition to the collection of error messages displayed by the
	 * formMessagesArea tag.
	 * </p>
	 */
	public java.lang.String getShowHelp() {
		if (null != this.showHelp) {
			return this.showHelp;
		}
		ValueBinding _vb = getValueBinding("showHelp");
		if (_vb != null) {
			return (java.lang.String) _vb.getValue(getFacesContext());
		} else {
			return null;
		}
	}

	/**
	 * <p>
	 * Set the value of the <code>showHelp</code> property.
	 * </p>
	 */
	public void setShowHelp(java.lang.String showHelp) {
		this.showHelp = showHelp;
	}

	public Object saveState(FacesContext _context) {
		Object _values[] = new Object[4];
		_values[0] = super.saveState(_context);
		_values[1] = errorText;
		_values[2] = infoText;
		_values[3] = showHelp;
		return _values;
	}

	public void restoreState(FacesContext _context, Object _state) {
		Object _values[] = (Object[]) _state;
		super.restoreState(_context, _values[0]);
		this.errorText = (java.lang.String) _values[1];
		this.infoText = (java.lang.String) _values[2];
		this.showHelp = (java.lang.String) _values[3];
	}
	
	private final String _LABEL_SUFFIX 		= "Label";
	private final String _REQUIRED_SUFFIX 	= "Required";
	private final String _ERROR_SUFFIX 		= "Error";
	private final String _INFO_SUFFIX 		= "Info";
	private final String CONNECTOR_U		= "_";
	private final String CONNECTOR_H		= "-";
	
	private final String TRUE				= "true";
	private final String INFO				= "info";
	private final String ALWAYS				= "always";
	private final String ERROR				= "error";
	private final String AS_NEEDED			= "as-needed";

	public String getErrorSpanId(FacesContext context) {
		String showHelp = getShowHelp();
		if (showHelp == null || showHelp.equals(ALWAYS) || showHelp.equals(ERROR) || showHelp.equals(AS_NEEDED)) {
			return getClientId(context) + CONNECTOR_U + _ERROR_SUFFIX.toLowerCase();
		}
		return null;
	}
	
	public String getInfoSpanId(FacesContext context) {
		String showHelp = getShowHelp();
		if (showHelp == null || showHelp.equals(ALWAYS) || showHelp.equals(INFO) || showHelp.equals(AS_NEEDED)) {
			return getClientId(context) + CONNECTOR_U + _INFO_SUFFIX.toLowerCase();
		}
		return null;
	}
	
	public String getLabelId(FacesContext context) {
		if (getChildren() == null || getChildren().size() < 1) {
			return null;
		}
		return ((UIComponent) getChildren().get(0)).getClientId(context) + CONNECTOR_U + _LABEL_SUFFIX.toLowerCase();
	}
	
	public String getLabelInfoClass() {
		
		if (getChildren() == null || getChildren().size() < 1) {
			return null;
		}
		UIComponent child = (UIComponent) getChildren().get(0);
		boolean required = (child instanceof UIInput) && ((UIInput)child).isRendered();

//		if (!(getParent() instanceof UIFormBox)) {
//			return null;
//		}
//		String parentStyleClass = ((UIFormBox) getParent()).getStyleClass();
		String parentStyleClass = (String) getParent().getAttributes().get("styleClass");
		if(parentStyleClass != null && parentStyleClass.indexOf(" ") != -1){
			// If multiple classes are defined just use the first one
			parentStyleClass = parentStyleClass.substring(0, parentStyleClass.indexOf(" ")).trim();
		}
		
		if (required) {
			return parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _REQUIRED_SUFFIX;
		} else {
			return parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX;
		}
	}
	
	public String getLabelErrorClass() {
//		if (!(getParent() instanceof UIFormBox)) {
//			return null;
//		}
//		String parentStyleClass = ((UIFormBox) getParent()).getStyleClass();
		String parentStyleClass = (String) getParent().getAttributes().get("styleClass");
		if(parentStyleClass != null && parentStyleClass.indexOf(" ") != -1){
			// If multiple classes are defined just use the first one
			parentStyleClass = parentStyleClass.substring(0, parentStyleClass.indexOf(" ")).trim();
		}
		return parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _ERROR_SUFFIX;
	}
	
	public String getSpanIsPopup() {
//		if (!(getParent() instanceof UIFormBox)) {
//			return null;
//		}
//		UIFormBox formBox = (UIFormBox)getParent();
//		String showInPopup = formBox.getShowInPopup();
		String showInPopup = (String) getParent().getAttributes().get("showInPopup");

		if (TRUE.equals(showInPopup)) {
//			return formBox.getHelpPosition();
			return (String) getParent().getAttributes().get("helpPosition");
		}
		return null;
	}
	
	public String getInfoSpanAlways() {
		String showHelp = getShowHelp();
		if (INFO.equals(showHelp) || ALWAYS.equals(showHelp) || showHelp == null) {
			return TRUE;
		}
		return null;
	}
	
	public String getErrorSpanAlways() {
		String showHelp = getShowHelp();
		if (showHelp == null || showHelp.equals(ALWAYS) || showHelp.equals(ERROR) || showHelp.equals(AS_NEEDED)) {
			if (getErrorText() != null && getErrorText().length() > 0) {
				return TRUE;
			}
		}
		return null;
	}

	public static UIFormItem find(UIComponent component) {
		while (null != (component =	component.getParent())) {
			if (component instanceof UIFormItem) {
				return (UIFormItem) component;
			}
		}
		return null;
	}
}
