/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2004 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.component;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import javax.faces.component.UIPanel;
import javax.faces.context.FacesContext;
import javax.servlet.jsp.PageContext;

import com.ibm.faces.renderkit.html_extended.Utils;
/**
 * <p>JspPanel.</p>
 */

public class UIJspPanel extends UIPanel {

    /**
     * <p>The component id string.</p>
     */
    public static final String COMPONENT_TYPE = "com.ibm.faces.JspPanel"; //$NON-NLS-1$

	/**
     * <p>The component family id string.</p>
	 */
	public static final String COMPONENT_FAMILY = "com.ibm.faces.JspPanel"; //$NON-NLS-1$

    /**
     * <p>Marker inserted in text to identify jsf tag location.</p>
     */
    public static final String JSF_COMP_MARKER = "!!jsftagmarker!!"; //$NON-NLS-1$


    /**
     * <p>Create a new {@link UIJspPanel} instance.</p>
     */
    public UIJspPanel() {

        super();
        setRendererType(null);

    }

    public String getFamily() {
        return (COMPONENT_FAMILY);
    }

    private PageContext pagecontext = null;

    /**
     * <p>Return the JSP page context for processing the page on whic
     * this component lives.</p>
     */
    public PageContext getPageContext() {
        return pagecontext;
    }

    /**
     * <p>Set the JSP page context for processing the page on whic
     * this component lives.</p>
     * 
     * @param pagecontext the pagecontext
     */
    public void setPageContext(PageContext pagecontext) {
        this.pagecontext = pagecontext;
    }

    /* (non-Javadoc)
     * @see javax.faces.component.UIComponent#getChildren()
     */
    public List getChildren() {
        if (children == null) {
            children = new ChildrenListEx(super.getChildren());
        }
        return (children);
    }
    private List children = null;
    // Private implementation of List that supports the functionality
    // required by UIComponent.getChildren()
    private class ChildrenListEx implements List {

        private List realChildList = null;
        public ChildrenListEx(List childList) {
            realChildList = childList;
        }

		// while pagecontext is not null we are processing the body tag.
		// so keep track of where in the text this comp was added by emitting a marker.
		// I only anticipate add being called during this phase.

        public void add(int index, Object element) {
            try {
                if (null != pagecontext) {
                    pagecontext.getOut().write(JSF_COMP_MARKER);
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            realChildList.add(index, element);
        }

        public boolean add(Object element) {
            try {
                if (null != pagecontext) {
                    pagecontext.getOut().write(JSF_COMP_MARKER);
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
            return realChildList.add(element);
        }

        public boolean addAll(Collection collection) {
			try {
				if (null != pagecontext) {
					for (int i = collection.size(); i > 0; --i){
						pagecontext.getOut().write(JSF_COMP_MARKER);
					}
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
            return realChildList.addAll(collection);
        }

        public boolean addAll(int index, Collection collection) {
			try {
				if (null != pagecontext) {
					for (int i = collection.size(); i > 0; --i){
						pagecontext.getOut().write(JSF_COMP_MARKER);
					}
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
            return realChildList.addAll(index, collection);
        }

        public void clear() {
            realChildList.clear();
        }

        public Iterator iterator() {
            return realChildList.iterator();
        }

        public ListIterator listIterator() {
            return realChildList.listIterator();
        }

        public ListIterator listIterator(int index) {
            return realChildList.listIterator(index);
        }

        public Object remove(int index) {
            return realChildList.remove(index);
        }

        public boolean remove(Object element) {
            return realChildList.remove(element);
        }

        public boolean removeAll(Collection collection) {
            return realChildList.removeAll(collection);
        }

        public boolean retainAll(Collection collection) {
            return realChildList.retainAll(collection);
        }

        public Object set(int index, Object element) {
            return realChildList.set(index, element);
        }

		/* (non-Javadoc)
		 * @see java.util.Collection#contains(java.lang.Object)
		 */
		public boolean contains(Object arg0) {
			return realChildList.contains(arg0);
		}

		/* (non-Javadoc)
		 * @see java.util.List#get(int)
		 */
		public Object get(int arg0) {
			return realChildList.get(arg0);
		}

		/* (non-Javadoc)
		 * @see java.util.List#indexOf(java.lang.Object)
		 */
		public int indexOf(Object arg0) {
			return realChildList.indexOf(arg0);
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#isEmpty()
		 */
		public boolean isEmpty() {
			return realChildList.isEmpty();
		}

		/* (non-Javadoc)
		 * @see java.util.List#lastIndexOf(java.lang.Object)
		 */
		public int lastIndexOf(Object arg0) {
			return realChildList.lastIndexOf(arg0);
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#size()
		 */
		public int size() {
			return realChildList.size();
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#toArray()
		 */
		public Object[] toArray() {
			return realChildList.toArray();
		}

		/* (non-Javadoc)
		 * @see java.util.Collection#toArray(java.lang.Object[])
		 */
		public Object[] toArray(Object[] arg0) {
			return realChildList.toArray(arg0);
		}
		/* (non-Javadoc)
		 * @see java.util.List#containsAll(java.util.Collection)
		 */
		public boolean containsAll(Collection arg0) {
			return realChildList.containsAll(arg0);
		}
		/* (non-Javadoc)
		 * @see java.util.List#subList(int, int)
		 */
		public List subList(int arg0, int arg1) {
			return realChildList.subList(arg0, arg1);
		}
    }

    /* (non-Javadoc)
     * @see javax.faces.component.UIComponent#getRendersChildren()
     */
    public boolean getRendersChildren() {
        return true;
    }

    /* (non-Javadoc)
     * @see javax.faces.component.UIComponent#encodeChildren(javax.faces.context.FacesContext)
     */
    public void encodeChildren(FacesContext context) throws IOException {
        if (!isRendered()) {
            return;
        }
        Utils.encodeChildren(context,this);
    }

}
