/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/
package com.ibm.faces.component;


import java.lang.reflect.Method;
import java.util.Map;

import javax.faces.component.UIOutput;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;

import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.faces.util.LifecycleUtil;

/**
 * @author pandrews
 * <p>UIRequest components generate request links that can execute a JSF Action</p>
 */
public class UIRequest extends UIOutput {
	
	public static final String COMPONENT_TYPE = "com.ibm.faces.Request"; //$NON-NLS-1$
	public static final String COMPONENT_FAMILY = "com.ibm.faces.Request"; //$NON-NLS-1$
	public static String ACTION_PARAMETER = "_rap";
	public static String VIEW_ID_PARAMETER = "_rvip";
	public static String OUTCOME_PREFIX = "!";
	public static String NO_NAVIGATION_VALUE = "_rlnn";
	//Dummy view id used for url, .jsp extension is required for portal server to encode the generated url...
	public static String REQUEST_VIEW_ID = "_rlvid.jsp";
	public static String PORTLET_MODE_PARAMETER = "com.ibm.faces.portlet.mode";
	
	private MethodBinding action;

	public MethodBinding getAction() {
		return action;
	}

	public void setAction(MethodBinding action) {
		this.action = action;
	}
	
	public String getFamily() {
		return COMPONENT_FAMILY;
	}

	/*
	 * <p>Generates encoded request URL with action parameter and child paramters appended.</p>
	 */
	public String generateRequestURL(FacesContext context){

		String url = context.getApplication().getViewHandler().getActionURL(context, context.getViewRoot().getViewId());
		StringBuffer href = null;
			/*
			 * use special viewid to prevent an unncessary view being restored. This is because RequestPhaseListener checks for UIRequest component
			 * parameters after restoreView phase, it cannot do this before restoreView phase as this will cause character corruption for non ascii 
			 * page encodings (the conversion to unicode characters by the server will not have happened at this point. 
			 */	
		href = new StringBuffer(url.substring(0,url.lastIndexOf("/") + 1));
		href.append(REQUEST_VIEW_ID);
		String fext = ".faces";
		//check for faces suffix
		if(url.endsWith(fext)){
			href.append(fext);
		}
		href.append("?");
		
		String actionStr = null;
		if(getAction() != null){
			actionStr = getAction().getExpressionString();
		}
		
		//actionStr can be ""
		if(actionStr == null){
			//no action value set, set value to 'no navigation value'
			actionStr = NO_NAVIGATION_VALUE;
		}  else if (actionStr.indexOf("#{") > -1 && actionStr.indexOf("}") > -1) {
			//action binding, strip away the #{}
			actionStr = actionStr.substring(actionStr.indexOf("#") + 2,actionStr.lastIndexOf("}"));
		}else {
			//this is a navigation outcome string
			actionStr = OUTCOME_PREFIX + actionStr;
		}
		
		/*
		String actionStr = getAction();
		// handle deprecated action attribute
		if(requestStr == null && getAction() != null){
			requestStr = getAction().getExpressionString();
		}
		
		if (requestStr == null || requestStr.length() < 0) {
			//no request value set, set default 'do nothing' value
			requestStr = NO_NAVIGATION_VALUE;
		} else if (requestStr.indexOf("#{") > -1 && requestStr.indexOf("}") > -1) {
			//action binding, strip away the #{}
			requestStr = requestStr.substring(requestStr.indexOf("#") + 2,requestStr.lastIndexOf("}"));
		}else {
			//this is a navigation outcome string
			requestStr = OUTCOME_PREFIX + requestStr;
		}
		*/
		
		href.append(ACTION_PARAMETER)
		.append("=")
		.append(actionStr);
		
		//add current view id parameter
		href.append("&")
		.append(VIEW_ID_PARAMETER)
		.append("=")
		.append(context.getViewRoot().getViewId());
			
		//add parameters to URL if they exist
		UIParameter[] params = LifecycleUtil.getParameters(this);
		if(params != null && params.length > 0){
			for(int i=0; i < params.length; i++){
				String name = params[i].getName();
				Object value = params[i].getValue();
				String strValue = null;
				if(value != null) {
					strValue = value.toString();
					strValue = strValue.replaceAll("&", "%26");
				}
				href.append("&")
				.append(name)
				.append("=")
				.append(strValue);
			}
		}
		
		// add portlet mode parameter if in Portal
		if(HxClientRenderUtil.isPortal(context.getExternalContext().getRequest())) {
			String portletMode = null;
            try {
            	Method method = context.getExternalContext().getRequest().getClass().getMethod("getPortletMode", null);
				portletMode = method.invoke(context.getExternalContext().getRequest(), null).toString();
			} catch (Exception e) {
			}

			if(portletMode != null) {
				href.append("&")
				.append(PORTLET_MODE_PARAMETER)
				.append("=")
				.append(portletMode);
			}			
		}
		
		/*
		 * REMOVING FUNCTIONALITY TO ADD ROW PARAMETER
		 * 
		//if the component is contained in a datatable add a rowIndex parameter
		if(!checkedForUIData.booleanValue()){
			data = Utils.findParentUIData(this);
			checkedForUIData = Boolean.TRUE;
		}
		if(data != null){
			href.append("&")
			.append(ROW_INDEX_PARAMETER)
			.append("=")
			.append(data.getRowIndex());
		}
		*/
		
		return context.getExternalContext().encodeResourceURL(href.toString());
	}
	
	/*
	 * <p>Static utility method to retrieve request parameters.</p>
	 */
	public static Object getParameter(String param){
		return FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap().get(param);
	}
	/*
	 * <p>Static utility method to retrieve request parameter map.</p>
	 */
	public static Map getParameterMap(){
		return FacesContext.getCurrentInstance().getExternalContext().getRequestParameterMap();
	}
	
	public void restoreState(FacesContext context, Object state) {
		Object values[] = (Object[])state;
		super.restoreState(context, values[0]);
		this.action = (MethodBinding) restoreAttachedState(context, values[1]);
	}
	
	public Object saveState(FacesContext context) {
		Object values[] = new Object[2];
		values[0] = super.saveState(context);
		values[1] = saveAttachedState(context, this.action);
		return values; 
	}
}
