/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>com.ibm.faces.ExternalRequest</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlAjaxExternalRequest extends com.ibm.faces.component.UIAjax {


  public HtmlAjaxExternalRequest() {
    super();
    setRendererType("com.ibm.faces.ExternalRequest"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.HtmlAjaxExternalRequest"; //$NON-NLS-1$


  private java.lang.String charset;

  /**
   * <p>Return the value of the <code>charset</code> property.  Contents:</p><p>
   * The (prefered) character encoding of page that is retrieved (the page the link resolves to).
   * </p>
   */
  public java.lang.String getCharset() {
    if (null != this.charset) {
      return this.charset;
    }
    ValueBinding _vb = getValueBinding("charset"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>charset</code> property.</p>
   */
  public void setCharset(java.lang.String charset) {
    this.charset = charset;
  }


  private java.lang.String href;

  /**
   * <p>Return the value of the <code>href</code> property.  Contents:</p><p>
   * The URL of the jsp that is going to respond to the request.
   * </p>
   */
  public java.lang.String getHref() {
    if (null != this.href) {
      return this.href;
    }
    ValueBinding _vb = getValueBinding("href"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>href</code> property.</p>
   */
  public void setHref(java.lang.String href) {
    this.href = href;
  }


  private java.lang.String hreflang;

  /**
   * <p>Return the value of the <code>hreflang</code> property.  Contents:</p><p>
   * The language of the page that is retrieved (the page that the link resolves to). 
   * 				This is advisory only so that a browser can do something (not known what this something is) 
   * 				in preparation of receiving the page.
   * </p>
   */
  public java.lang.String getHreflang() {
    if (null != this.hreflang) {
      return this.hreflang;
    }
    ValueBinding _vb = getValueBinding("hreflang"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>hreflang</code> property.</p>
   */
  public void setHreflang(java.lang.String hreflang) {
    this.hreflang = hreflang;
  }


  private java.lang.String params;

  /**
   * <p>Return the value of the <code>params</code> property.  Contents:</p><p>
   * semicolon separated list of component ids. The value of 
   * 				these components will be retrieved and appended as params 
   * 				to the URL used to get the new content. The name of the 
   * 				param will be the id of the component (not the clientId)..
   * </p>
   */
  public java.lang.String getParams() {
    if (null != this.params) {
      return this.params;
    }
    ValueBinding _vb = getValueBinding("params"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>params</code> property.</p>
   */
  public void setParams(java.lang.String params) {
    this.params = params;
  }


  private java.lang.String source;

  /**
   * <p>Return the value of the <code>source</code> property.  Contents:</p><p>
   * The id of a component in the targeted URL whose content will replace 
   * 				the content of the container this component is a child of.
   * </p>
   */
  public java.lang.String getSource() {
    if (null != this.source) {
      return this.source;
    }
    ValueBinding _vb = getValueBinding("source"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>source</code> property.</p>
   */
  public void setSource(java.lang.String source) {
    this.source = source;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[6];
    _values[0] = super.saveState(_context);
    _values[1] = charset;
    _values[2] = href;
    _values[3] = hreflang;
    _values[4] = params;
    _values[5] = source;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.charset = (java.lang.String) _values[1];
    this.href = (java.lang.String) _values[2];
    this.hreflang = (java.lang.String) _values[3];
    this.params = (java.lang.String) _values[4];
    this.source = (java.lang.String) _values[5];
  }


}
