/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>com.ibm.faces.BackchannelHttpRequestRenderer</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlBackchannelHttpRequest extends com.ibm.faces.component.UIBackchannel {


  public HtmlBackchannelHttpRequest() {
    super();
    setRendererType("com.ibm.faces.BackchannelHttpRequestRenderer"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.HtmlBackchannelHttpRequest"; //$NON-NLS-1$


  private boolean asynchronous = false;
  private boolean asynchronous_set = false;

  /**
   * <p>Return the value of the <code>asynchronous</code> property.  Contents:</p><p>
   * Boolean value to specify if the connection is
   * 				asynchronous, default is true.
   * </p>
   */
  public boolean isAsynchronous() {
    if (this.asynchronous_set) {
      return this.asynchronous;
    }
    ValueBinding _vb = getValueBinding("asynchronous"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.asynchronous;
    }
  }

  /**
   * <p>Set the value of the <code>asynchronous</code> property.</p>
   */
  public void setAsynchronous(boolean asynchronous) {
    this.asynchronous = asynchronous;
    this.asynchronous_set = true;
  }


  private java.lang.String mode;

  /**
   * <p>Return the value of the <code>mode</code> property.  Contents:</p><p>
   * The usage: "post" or "get"
   * </p>
   */
  public java.lang.String getMode() {
    if (null != this.mode) {
      return this.mode;
    }
    ValueBinding _vb = getValueBinding("mode"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>mode</code> property.</p>
   */
  public void setMode(java.lang.String mode) {
    this.mode = mode;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[4];
    _values[0] = super.saveState(_context);
    _values[1] = this.asynchronous ? Boolean.TRUE : Boolean.FALSE;
    _values[2] = this.asynchronous_set ? Boolean.TRUE : Boolean.FALSE;
    _values[3] = mode;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.asynchronous = ((Boolean) _values[1]).booleanValue();
    this.asynchronous_set = ((Boolean) _values[2]).booleanValue();
    this.mode = (java.lang.String) _values[3];
  }


}
