/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>com.ibm.faces.TableEx</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlDataTableEx extends javax.faces.component.UIData {


  public HtmlDataTableEx() {
    super();
    setRendererType("com.ibm.faces.TableEx"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.HtmlDataTableEx"; //$NON-NLS-1$


  private java.lang.String bgcolor;

  /**
   * <p>Return the value of the <code>bgcolor</code> property.  Contents:</p><p>
   * Color name or RGB code for table background color.
   * </p>
   */
  public java.lang.String getBgcolor() {
    if (null != this.bgcolor) {
      return this.bgcolor;
    }
    ValueBinding _vb = getValueBinding("bgcolor"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>bgcolor</code> property.</p>
   */
  public void setBgcolor(java.lang.String bgcolor) {
    this.bgcolor = bgcolor;
  }


  private int border = Integer.MIN_VALUE;
  private boolean border_set = false;

  /**
   * <p>Return the value of the <code>border</code> property.  Contents:</p><p>
   * Pixel width of table border.
   * </p>
   */
  public int getBorder() {
    if (this.border_set) {
      return this.border;
    }
    ValueBinding _vb = getValueBinding("border"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return Integer.MIN_VALUE;
      } else {
        return ((Integer) _result).intValue();
      }
    } else {
        return this.border;
    }
  }

  /**
   * <p>Set the value of the <code>border</code> property.</p>
   */
  public void setBorder(int border) {
    this.border = border;
    this.border_set = true;
  }


  private java.lang.String cellpadding;

  /**
   * <p>Return the value of the <code>cellpadding</code> property.  Contents:</p><p>
   * Space between cell contents and cell border.
   * </p>
   */
  public java.lang.String getCellpadding() {
    if (null != this.cellpadding) {
      return this.cellpadding;
    }
    ValueBinding _vb = getValueBinding("cellpadding"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>cellpadding</code> property.</p>
   */
  public void setCellpadding(java.lang.String cellpadding) {
    this.cellpadding = cellpadding;
  }


  private java.lang.String cellspacing;

  /**
   * <p>Return the value of the <code>cellspacing</code> property.  Contents:</p><p>
   * Space between cells.
   * </p>
   */
  public java.lang.String getCellspacing() {
    if (null != this.cellspacing) {
      return this.cellspacing;
    }
    ValueBinding _vb = getValueBinding("cellspacing"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>cellspacing</code> property.</p>
   */
  public void setCellspacing(java.lang.String cellspacing) {
    this.cellspacing = cellspacing;
  }


  private java.lang.String columnClasses;

  /**
   * <p>Return the value of the <code>columnClasses</code> property.  Contents:</p><p>
   * One or more CSS style classes (comma-separated)
   * 					applied respectively to table columns.
   * </p>
   */
  public java.lang.String getColumnClasses() {
    if (null != this.columnClasses) {
      return this.columnClasses;
    }
    ValueBinding _vb = getValueBinding("columnClasses"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>columnClasses</code> property.</p>
   */
  public void setColumnClasses(java.lang.String columnClasses) {
    this.columnClasses = columnClasses;
  }


  private java.lang.String dir;

  /**
   * <p>Return the value of the <code>dir</code> property.  Contents:</p><p>
   * Text direction for directionally neutral text,
   * 					values can be "LTR" (left to right) or "RTL" (right
   * 					to left).
   * </p>
   */
  public java.lang.String getDir() {
    if (null != this.dir) {
      return this.dir;
    }
    ValueBinding _vb = getValueBinding("dir"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>dir</code> property.</p>
   */
  public void setDir(java.lang.String dir) {
    this.dir = dir;
  }


  private java.lang.String footerClass;

  /**
   * <p>Return the value of the <code>footerClass</code> property.  Contents:</p><p>
   * One or more CSS style classes (space-separated)
   * 					applied to the table footer.
   * </p>
   */
  public java.lang.String getFooterClass() {
    if (null != this.footerClass) {
      return this.footerClass;
    }
    ValueBinding _vb = getValueBinding("footerClass"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>footerClass</code> property.</p>
   */
  public void setFooterClass(java.lang.String footerClass) {
    this.footerClass = footerClass;
  }


  private java.lang.String forceFixedWidths;

  /**
   * <p>Return the value of the <code>forceFixedWidths</code> property.  Contents:</p><p>
   * Force Fixed Column Widths.
   * </p>
   */
  public java.lang.String getForceFixedWidths() {
    if (null != this.forceFixedWidths) {
      return this.forceFixedWidths;
    }
    ValueBinding _vb = getValueBinding("forceFixedWidths"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>forceFixedWidths</code> property.</p>
   */
  public void setForceFixedWidths(java.lang.String forceFixedWidths) {
    this.forceFixedWidths = forceFixedWidths;
  }


  private java.lang.String frame;

  /**
   * <p>Return the value of the <code>frame</code> property.  Contents:</p><p>
   * Define the look of the frame around the table can be
   * 					none, above, below, hsides, vsides, lhs, rhs, box
   * 					and border.
   * </p>
   */
  public java.lang.String getFrame() {
    if (null != this.frame) {
      return this.frame;
    }
    ValueBinding _vb = getValueBinding("frame"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>frame</code> property.</p>
   */
  public void setFrame(java.lang.String frame) {
    this.frame = frame;
  }


  private java.lang.String headerClass;

  /**
   * <p>Return the value of the <code>headerClass</code> property.  Contents:</p><p>
   * One or more CSS style classes (space-separated)
   * 					applied to the table header.
   * </p>
   */
  public java.lang.String getHeaderClass() {
    if (null != this.headerClass) {
      return this.headerClass;
    }
    ValueBinding _vb = getValueBinding("headerClass"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>headerClass</code> property.</p>
   */
  public void setHeaderClass(java.lang.String headerClass) {
    this.headerClass = headerClass;
  }


  private java.lang.String lang;

  /**
   * <p>Return the value of the <code>lang</code> property.  Contents:</p><p>
   * Specifies the base language of attributes and text
   * 					context for this component
   * </p>
   */
  public java.lang.String getLang() {
    if (null != this.lang) {
      return this.lang;
    }
    ValueBinding _vb = getValueBinding("lang"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>lang</code> property.</p>
   */
  public void setLang(java.lang.String lang) {
    this.lang = lang;
  }


  private java.lang.String onclick;

  /**
   * <p>Return the value of the <code>onclick</code> property.  Contents:</p><p>
   * Javascript onclick event.
   * </p>
   */
  public java.lang.String getOnclick() {
    if (null != this.onclick) {
      return this.onclick;
    }
    ValueBinding _vb = getValueBinding("onclick"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onclick</code> property.</p>
   */
  public void setOnclick(java.lang.String onclick) {
    this.onclick = onclick;
  }


  private java.lang.String ondblclick;

  /**
   * <p>Return the value of the <code>ondblclick</code> property.  Contents:</p><p>
   * Javascript double click event.
   * </p>
   */
  public java.lang.String getOndblclick() {
    if (null != this.ondblclick) {
      return this.ondblclick;
    }
    ValueBinding _vb = getValueBinding("ondblclick"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>ondblclick</code> property.</p>
   */
  public void setOndblclick(java.lang.String ondblclick) {
    this.ondblclick = ondblclick;
  }


  private java.lang.String onkeydown;

  /**
   * <p>Return the value of the <code>onkeydown</code> property.  Contents:</p><p>
   * Javascript key down event.
   * </p>
   */
  public java.lang.String getOnkeydown() {
    if (null != this.onkeydown) {
      return this.onkeydown;
    }
    ValueBinding _vb = getValueBinding("onkeydown"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onkeydown</code> property.</p>
   */
  public void setOnkeydown(java.lang.String onkeydown) {
    this.onkeydown = onkeydown;
  }


  private java.lang.String onkeypress;

  /**
   * <p>Return the value of the <code>onkeypress</code> property.  Contents:</p><p>
   * Javascript key pressed event.
   * </p>
   */
  public java.lang.String getOnkeypress() {
    if (null != this.onkeypress) {
      return this.onkeypress;
    }
    ValueBinding _vb = getValueBinding("onkeypress"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onkeypress</code> property.</p>
   */
  public void setOnkeypress(java.lang.String onkeypress) {
    this.onkeypress = onkeypress;
  }


  private java.lang.String onkeyup;

  /**
   * <p>Return the value of the <code>onkeyup</code> property.  Contents:</p><p>
   * Javascript key up event.
   * </p>
   */
  public java.lang.String getOnkeyup() {
    if (null != this.onkeyup) {
      return this.onkeyup;
    }
    ValueBinding _vb = getValueBinding("onkeyup"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onkeyup</code> property.</p>
   */
  public void setOnkeyup(java.lang.String onkeyup) {
    this.onkeyup = onkeyup;
  }


  private java.lang.String onmousedown;

  /**
   * <p>Return the value of the <code>onmousedown</code> property.  Contents:</p><p>
   * Javascript mouse button press event.
   * </p>
   */
  public java.lang.String getOnmousedown() {
    if (null != this.onmousedown) {
      return this.onmousedown;
    }
    ValueBinding _vb = getValueBinding("onmousedown"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onmousedown</code> property.</p>
   */
  public void setOnmousedown(java.lang.String onmousedown) {
    this.onmousedown = onmousedown;
  }


  private java.lang.String onmousemove;

  /**
   * <p>Return the value of the <code>onmousemove</code> property.  Contents:</p><p>
   * Javascript mouse move event.
   * </p>
   */
  public java.lang.String getOnmousemove() {
    if (null != this.onmousemove) {
      return this.onmousemove;
    }
    ValueBinding _vb = getValueBinding("onmousemove"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onmousemove</code> property.</p>
   */
  public void setOnmousemove(java.lang.String onmousemove) {
    this.onmousemove = onmousemove;
  }


  private java.lang.String onmouseout;

  /**
   * <p>Return the value of the <code>onmouseout</code> property.  Contents:</p><p>
   * Javascript mouse out event.
   * </p>
   */
  public java.lang.String getOnmouseout() {
    if (null != this.onmouseout) {
      return this.onmouseout;
    }
    ValueBinding _vb = getValueBinding("onmouseout"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onmouseout</code> property.</p>
   */
  public void setOnmouseout(java.lang.String onmouseout) {
    this.onmouseout = onmouseout;
  }


  private java.lang.String onmouseover;

  /**
   * <p>Return the value of the <code>onmouseover</code> property.  Contents:</p><p>
   * Javascript mouse over event.
   * </p>
   */
  public java.lang.String getOnmouseover() {
    if (null != this.onmouseover) {
      return this.onmouseover;
    }
    ValueBinding _vb = getValueBinding("onmouseover"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onmouseover</code> property.</p>
   */
  public void setOnmouseover(java.lang.String onmouseover) {
    this.onmouseover = onmouseover;
  }


  private java.lang.String onmouseup;

  /**
   * <p>Return the value of the <code>onmouseup</code> property.  Contents:</p><p>
   * Javascript mouse button release event.
   * </p>
   */
  public java.lang.String getOnmouseup() {
    if (null != this.onmouseup) {
      return this.onmouseup;
    }
    ValueBinding _vb = getValueBinding("onmouseup"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onmouseup</code> property.</p>
   */
  public void setOnmouseup(java.lang.String onmouseup) {
    this.onmouseup = onmouseup;
  }


  private java.lang.String rowClasses;

  /**
   * <p>Return the value of the <code>rowClasses</code> property.  Contents:</p><p>
   * One or more CSS style classes (comma-separated)
   * 					applied respectively to table rows.
   * </p>
   */
  public java.lang.String getRowClasses() {
    if (null != this.rowClasses) {
      return this.rowClasses;
    }
    ValueBinding _vb = getValueBinding("rowClasses"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>rowClasses</code> property.</p>
   */
  public void setRowClasses(java.lang.String rowClasses) {
    this.rowClasses = rowClasses;
  }


  private java.lang.String rules;

  /**
   * <p>Return the value of the <code>rules</code> property.  Contents:</p><p>
   * Define the visible table rules, this can be none,
   * 					groups, rows, cols and all.
   * </p>
   */
  public java.lang.String getRules() {
    if (null != this.rules) {
      return this.rules;
    }
    ValueBinding _vb = getValueBinding("rules"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>rules</code> property.</p>
   */
  public void setRules(java.lang.String rules) {
    this.rules = rules;
  }


  private java.lang.String scrollSize;

  /**
   * <p>Return the value of the <code>scrollSize</code> property.  Contents:</p><p>
   * Scroll Area Height.
   * </p>
   */
  public java.lang.String getScrollSize() {
    if (null != this.scrollSize) {
      return this.scrollSize;
    }
    ValueBinding _vb = getValueBinding("scrollSize"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>scrollSize</code> property.</p>
   */
  public void setScrollSize(java.lang.String scrollSize) {
    this.scrollSize = scrollSize;
  }


  private java.lang.String style;

  /**
   * <p>Return the value of the <code>style</code> property.  Contents:</p><p>
   * CSS styles applied when rendering this component.
   * </p>
   */
  public java.lang.String getStyle() {
    if (null != this.style) {
      return this.style;
    }
    ValueBinding _vb = getValueBinding("style"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>style</code> property.</p>
   */
  public void setStyle(java.lang.String style) {
    this.style = style;
  }


  private java.lang.String styleClass;

  /**
   * <p>Return the value of the <code>styleClass</code> property.  Contents:</p><p>
   * List of space-separated CSS style classes to be
   * 					applied when this component is rendered.
   * </p>
   */
  public java.lang.String getStyleClass() {
    if (null != this.styleClass) {
      return this.styleClass;
    }
    ValueBinding _vb = getValueBinding("styleClass"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>styleClass</code> property.</p>
   */
  public void setStyleClass(java.lang.String styleClass) {
    this.styleClass = styleClass;
  }


  private java.lang.String summary;

  /**
   * <p>Return the value of the <code>summary</code> property.  Contents:</p><p>
   * Summary information about the table.
   * </p>
   */
  public java.lang.String getSummary() {
    if (null != this.summary) {
      return this.summary;
    }
    ValueBinding _vb = getValueBinding("summary"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>summary</code> property.</p>
   */
  public void setSummary(java.lang.String summary) {
    this.summary = summary;
  }


  private java.lang.String title;

  /**
   * <p>Return the value of the <code>title</code> property.  Contents:</p><p>
   * Title information to be added to markup of this
   * 					component when rendered.
   * </p>
   */
  public java.lang.String getTitle() {
    if (null != this.title) {
      return this.title;
    }
    ValueBinding _vb = getValueBinding("title"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>title</code> property.</p>
   */
  public void setTitle(java.lang.String title) {
    this.title = title;
  }


  private java.lang.String width;

  /**
   * <p>Return the value of the <code>width</code> property.  Contents:</p><p>
   * Table width.
   * </p>
   */
  public java.lang.String getWidth() {
    if (null != this.width) {
      return this.width;
    }
    ValueBinding _vb = getValueBinding("width"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>width</code> property.</p>
   */
  public void setWidth(java.lang.String width) {
    this.width = width;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[31];
    _values[0] = super.saveState(_context);
    _values[1] = bgcolor;
    _values[2] = new Integer(this.border);
    _values[3] = this.border_set ? Boolean.TRUE : Boolean.FALSE;
    _values[4] = cellpadding;
    _values[5] = cellspacing;
    _values[6] = columnClasses;
    _values[7] = dir;
    _values[8] = footerClass;
    _values[9] = forceFixedWidths;
    _values[10] = frame;
    _values[11] = headerClass;
    _values[12] = lang;
    _values[13] = onclick;
    _values[14] = ondblclick;
    _values[15] = onkeydown;
    _values[16] = onkeypress;
    _values[17] = onkeyup;
    _values[18] = onmousedown;
    _values[19] = onmousemove;
    _values[20] = onmouseout;
    _values[21] = onmouseover;
    _values[22] = onmouseup;
    _values[23] = rowClasses;
    _values[24] = rules;
    _values[25] = scrollSize;
    _values[26] = style;
    _values[27] = styleClass;
    _values[28] = summary;
    _values[29] = title;
    _values[30] = width;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.bgcolor = (java.lang.String) _values[1];
    this.border = ((Integer) _values[2]).intValue();
    this.border_set = ((Boolean) _values[3]).booleanValue();
    this.cellpadding = (java.lang.String) _values[4];
    this.cellspacing = (java.lang.String) _values[5];
    this.columnClasses = (java.lang.String) _values[6];
    this.dir = (java.lang.String) _values[7];
    this.footerClass = (java.lang.String) _values[8];
    this.forceFixedWidths = (java.lang.String) _values[9];
    this.frame = (java.lang.String) _values[10];
    this.headerClass = (java.lang.String) _values[11];
    this.lang = (java.lang.String) _values[12];
    this.onclick = (java.lang.String) _values[13];
    this.ondblclick = (java.lang.String) _values[14];
    this.onkeydown = (java.lang.String) _values[15];
    this.onkeypress = (java.lang.String) _values[16];
    this.onkeyup = (java.lang.String) _values[17];
    this.onmousedown = (java.lang.String) _values[18];
    this.onmousemove = (java.lang.String) _values[19];
    this.onmouseout = (java.lang.String) _values[20];
    this.onmouseover = (java.lang.String) _values[21];
    this.onmouseup = (java.lang.String) _values[22];
    this.rowClasses = (java.lang.String) _values[23];
    this.rules = (java.lang.String) _values[24];
    this.scrollSize = (java.lang.String) _values[25];
    this.style = (java.lang.String) _values[26];
    this.styleClass = (java.lang.String) _values[27];
    this.summary = (java.lang.String) _values[28];
    this.title = (java.lang.String) _values[29];
    this.width = (java.lang.String) _values[30];
  }


}
