/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>com.ibm.faces.FormItem</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlFormItem extends com.ibm.faces.component.UIFormItem {


  public HtmlFormItem() {
    super();
    setRendererType("com.ibm.faces.FormItem"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.HtmlFormItem"; //$NON-NLS-1$


  private java.lang.String accesskey;

  /**
   * <p>Return the value of the <code>accesskey</code> property.  Contents:</p><p>
   * Access key to put focus on this compoent.
   * </p>
   */
  public java.lang.String getAccesskey() {
    if (null != this.accesskey) {
      return this.accesskey;
    }
    ValueBinding _vb = getValueBinding("accesskey"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>accesskey</code> property.</p>
   */
  public void setAccesskey(java.lang.String accesskey) {
    this.accesskey = accesskey;
  }


  private java.lang.String binding;

  /**
   * <p>Return the value of the <code>binding</code> property.  Contents:</p><p>
   * ValueBinding to bind the component to a component
   * 				property of a JavaBean
   * </p>
   */
  public java.lang.String getBinding() {
    if (null != this.binding) {
      return this.binding;
    }
    ValueBinding _vb = getValueBinding("binding"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>binding</code> property.</p>
   */
  public void setBinding(java.lang.String binding) {
    this.binding = binding;
  }


  private boolean escape = false;
  private boolean escape_set = false;

  /**
   * <p>Return the value of the <code>escape</code> property.  Contents:</p><p>
   * Boolean value indicating whether or not label, infoText and errorText are to
   * 				be escaped.
   * </p>
   */
  public boolean isEscape() {
    if (this.escape_set) {
      return this.escape;
    }
    ValueBinding _vb = getValueBinding("escape"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.escape;
    }
  }

  /**
   * <p>Set the value of the <code>escape</code> property.</p>
   */
  public void setEscape(boolean escape) {
    this.escape = escape;
    this.escape_set = true;
  }


  private java.lang.String label;

  /**
   * <p>Return the value of the <code>label</code> property.  Contents:</p><p>
   * Label for input field. Html label tag is generated.
   * </p>
   */
  public java.lang.String getLabel() {
    if (null != this.label) {
      return this.label;
    }
    ValueBinding _vb = getValueBinding("label"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>label</code> property.</p>
   */
  public void setLabel(java.lang.String label) {
    this.label = label;
  }


  private boolean noClientValidation = false;
  private boolean noClientValidation_set = false;

  /**
   * <p>Return the value of the <code>noClientValidation</code> property.  Contents:</p><p>
   * Boolean value indicating whether or not client validation is enabled. Default valud is false.
   * </p>
   */
  public boolean isNoClientValidation() {
    if (this.noClientValidation_set) {
      return this.noClientValidation;
    }
    ValueBinding _vb = getValueBinding("noClientValidation"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.noClientValidation;
    }
  }

  /**
   * <p>Set the value of the <code>noClientValidation</code> property.</p>
   */
  public void setNoClientValidation(boolean noClientValidation) {
    this.noClientValidation = noClientValidation;
    this.noClientValidation_set = true;
  }


  private java.lang.String releaseMargin;

  /**
   * <p>Return the value of the <code>releaseMargin</code> property.  Contents:</p><p>
   * Determins whether label or help messages are released from aligning with other formItems.
   * 				Client can specify one of the values : label, help, or both.
   * 				"label" value is effective only if parent panelFormBox specifies left or right as labelPosition. 
   * 				In this case, label and child control will be released from alignment.
   * 				"help" value is effective only if parent panelFormBox specifies left or right as helpPosition. 
   * 				In this case, helper message and child control will be released from alignment.
   * 				"both" value is applicable in any case. The whole formItem area is released from alignment.
   * </p>
   */
  public java.lang.String getReleaseMargin() {
    if (null != this.releaseMargin) {
      return this.releaseMargin;
    }
    ValueBinding _vb = getValueBinding("releaseMargin"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>releaseMargin</code> property.</p>
   */
  public void setReleaseMargin(java.lang.String releaseMargin) {
    this.releaseMargin = releaseMargin;
  }


  private java.lang.String style;

  /**
   * <p>Return the value of the <code>style</code> property.  Contents:</p><p>
   * CSS style(s) to be applied when this component is
   * 				rendered.
   * </p>
   */
  public java.lang.String getStyle() {
    if (null != this.style) {
      return this.style;
    }
    ValueBinding _vb = getValueBinding("style"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>style</code> property.</p>
   */
  public void setStyle(java.lang.String style) {
    this.style = style;
  }


  private java.lang.String styleClass;

  /**
   * <p>Return the value of the <code>styleClass</code> property.  Contents:</p><p>
   * Space-separated list of CSS style class(es) to be
   * 				applied when this element is rendered. This value must
   * 				be passed through as the "class" property on generated
   * 				markup.
   * </p>
   */
  public java.lang.String getStyleClass() {
    if (null != this.styleClass) {
      return this.styleClass;
    }
    ValueBinding _vb = getValueBinding("styleClass"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>styleClass</code> property.</p>
   */
  public void setStyleClass(java.lang.String styleClass) {
    this.styleClass = styleClass;
  }


  private java.lang.String title;

  /**
   * <p>Return the value of the <code>title</code> property.  Contents:</p><p>
   * Rendered title information.
   * </p>
   */
  public java.lang.String getTitle() {
    if (null != this.title) {
      return this.title;
    }
    ValueBinding _vb = getValueBinding("title"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>title</code> property.</p>
   */
  public void setTitle(java.lang.String title) {
    this.title = title;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[12];
    _values[0] = super.saveState(_context);
    _values[1] = accesskey;
    _values[2] = binding;
    _values[3] = this.escape ? Boolean.TRUE : Boolean.FALSE;
    _values[4] = this.escape_set ? Boolean.TRUE : Boolean.FALSE;
    _values[5] = label;
    _values[6] = this.noClientValidation ? Boolean.TRUE : Boolean.FALSE;
    _values[7] = this.noClientValidation_set ? Boolean.TRUE : Boolean.FALSE;
    _values[8] = releaseMargin;
    _values[9] = style;
    _values[10] = styleClass;
    _values[11] = title;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.accesskey = (java.lang.String) _values[1];
    this.binding = (java.lang.String) _values[2];
    this.escape = ((Boolean) _values[3]).booleanValue();
    this.escape_set = ((Boolean) _values[4]).booleanValue();
    this.label = (java.lang.String) _values[5];
    this.noClientValidation = ((Boolean) _values[6]).booleanValue();
    this.noClientValidation_set = ((Boolean) _values[7]).booleanValue();
    this.releaseMargin = (java.lang.String) _values[8];
    this.style = (java.lang.String) _values[9];
    this.styleClass = (java.lang.String) _values[10];
    this.title = (java.lang.String) _values[11];
  }


}
