/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>com.ibm.faces.Dialog</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlPanelDialog extends javax.faces.component.UIPanel {


  public HtmlPanelDialog() {
    super();
    setRendererType("com.ibm.faces.Dialog"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.HtmlPanelDialog"; //$NON-NLS-1$


  private java.lang.String align;

  /**
   * <p>Return the value of the <code>align</code> property.  Contents:</p><p>
   * This attribute specifies the horizontal alignment of the dialog with respect to the surrounding context. Possible values: 
   * 					- left: the left edge of the dialog is aligned with the left edge of the browser window 
   * 					- right: the right edge of the dialog is aligned with the right edge of the browser window
   * 					- center: dialog is horizontally centered relative to the browser window.  
   * 					- relative: left edge of the dialog is aligned with the left edge of the object that caused the onShow event to fire OR it is relative to relativeTo if relativeTo is provided.
   * 				The default depends on the base text direction. For left to right text, the default is align=left, while for right to left text, the default is align=right.
   * </p>
   */
  public java.lang.String getAlign() {
    if (null != this.align) {
      return this.align;
    }
    ValueBinding _vb = getValueBinding("align"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>align</code> property.</p>
   */
  public void setAlign(java.lang.String align) {
    this.align = align;
  }


  private java.lang.String binding;

  /**
   * <p>Return the value of the <code>binding</code> property.  Contents:</p><p>
   * ValueBinding to bind the component to a component
   * 				property of a JavaBean
   * </p>
   */
  public java.lang.String getBinding() {
    if (null != this.binding) {
      return this.binding;
    }
    ValueBinding _vb = getValueBinding("binding"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>binding</code> property.</p>
   */
  public void setBinding(java.lang.String binding) {
    this.binding = binding;
  }


  private java.lang.String dir;

  /**
   * <p>Return the value of the <code>dir</code> property.  Contents:</p><p>
   * The DIR attribute specifies the directionality of text--left-to-right (DIR=ltr, the default) or right-to-left (DIR=rtl).
   * </p>
   */
  public java.lang.String getDir() {
    if (null != this.dir) {
      return this.dir;
    }
    ValueBinding _vb = getValueBinding("dir"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>dir</code> property.</p>
   */
  public void setDir(java.lang.String dir) {
    this.dir = dir;
  }


  private boolean enableMoveKey = true;
  private boolean enableMoveKey_set = false;

  /**
   * <p>Return the value of the <code>enableMoveKey</code> property.  Contents:</p><p>
   * Indicates whether specific key sequence (Ctrl+Shift+Arrow_KeyX) moves dialog around the
   * 				browser window whenever dialog has focus. Default value is 'true'.
   * </p>
   */
  public boolean isEnableMoveKey() {
    if (this.enableMoveKey_set) {
      return this.enableMoveKey;
    }
    ValueBinding _vb = getValueBinding("enableMoveKey"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.enableMoveKey;
    }
  }

  /**
   * <p>Set the value of the <code>enableMoveKey</code> property.</p>
   */
  public void setEnableMoveKey(boolean enableMoveKey) {
    this.enableMoveKey = enableMoveKey;
    this.enableMoveKey_set = true;
  }


  private java.lang.String _for;

  /**
   * <p>Return the value of the <code>for</code> property.  Contents:</p><p>
   * Optional.  If present, when this tag is "clicked", the dialog is shown. This is a convenience attribute.
   * 				If we don't have a "forId", then designers would have to ALWAYS manually assign a generic behavior to
   * 				something in the page to bring up the dialog box.
   * </p>
   */
  public java.lang.String getFor() {
    if (null != this._for) {
      return this._for;
    }
    ValueBinding _vb = getValueBinding("for"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>for</code> property.</p>
   */
  public void setFor(java.lang.String _for) {
    this._for = _for;
  }


  private java.lang.String initialFocus;

  /**
   * <p>Return the value of the <code>initialFocus</code> property.  Contents:</p><p>
   * The id of the field that gets focus when the dialog is shown. 
   * 				If omitted, the "first" field is given focus.
   * </p>
   */
  public java.lang.String getInitialFocus() {
    if (null != this.initialFocus) {
      return this.initialFocus;
    }
    ValueBinding _vb = getValueBinding("initialFocus"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>initialFocus</code> property.</p>
   */
  public void setInitialFocus(java.lang.String initialFocus) {
    this.initialFocus = initialFocus;
  }


  private boolean initiallyShow = false;
  private boolean initiallyShow_set = false;

  /**
   * <p>Return the value of the <code>initiallyShow</code> property.  Contents:</p><p>
   * Values are true, false. Default is 'false'. 
   * 				If true, when the page loads, the dialog is shown (and onShow is fired).
   * </p>
   */
  public boolean isInitiallyShow() {
    if (this.initiallyShow_set) {
      return this.initiallyShow;
    }
    ValueBinding _vb = getValueBinding("initiallyShow"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.initiallyShow;
    }
  }

  /**
   * <p>Set the value of the <code>initiallyShow</code> property.</p>
   */
  public void setInitiallyShow(boolean initiallyShow) {
    this.initiallyShow = initiallyShow;
    this.initiallyShow_set = true;
  }


  private java.lang.String lang;

  /**
   * <p>Return the value of the <code>lang</code> property.  Contents:</p><p>
   * This attribute specifies the base language of an element's attribute values and text content. The default value of this attribute is unknown.
   * </p>
   */
  public java.lang.String getLang() {
    if (null != this.lang) {
      return this.lang;
    }
    ValueBinding _vb = getValueBinding("lang"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>lang</code> property.</p>
   */
  public void setLang(java.lang.String lang) {
    this.lang = lang;
  }


  private boolean movable = true;
  private boolean movable_set = false;

  /**
   * <p>Return the value of the <code>movable</code> property.  Contents:</p><p>
   * Indicates weather users can move the dialog within the browser window. 
   * 				Default value is true.
   * </p>
   */
  public boolean isMovable() {
    if (this.movable_set) {
      return this.movable;
    }
    ValueBinding _vb = getValueBinding("movable"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.movable;
    }
  }

  /**
   * <p>Set the value of the <code>movable</code> property.</p>
   */
  public void setMovable(boolean movable) {
    this.movable = movable;
    this.movable_set = true;
  }


  private boolean noKeybindings = false;
  private boolean noKeybindings_set = false;

  /**
   * <p>Return the value of the <code>noKeybindings</code> property.  Contents:</p><p>
   * Values are true, false. Default is 'false'. 
   * 				If true, no keybindings will be associated with the dialog buttons.
   * </p>
   */
  public boolean isNoKeybindings() {
    if (this.noKeybindings_set) {
      return this.noKeybindings;
    }
    ValueBinding _vb = getValueBinding("noKeybindings"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.noKeybindings;
    }
  }

  /**
   * <p>Set the value of the <code>noKeybindings</code> property.</p>
   */
  public void setNoKeybindings(boolean noKeybindings) {
    this.noKeybindings = noKeybindings;
    this.noKeybindings_set = true;
  }


  private java.lang.String onclick;

  /**
   * <p>Return the value of the <code>onclick</code> property.  Contents:</p><p>
   * Javascript onclick event
   * </p>
   */
  public java.lang.String getOnclick() {
    if (null != this.onclick) {
      return this.onclick;
    }
    ValueBinding _vb = getValueBinding("onclick"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onclick</code> property.</p>
   */
  public void setOnclick(java.lang.String onclick) {
    this.onclick = onclick;
  }


  private java.lang.String ondblclick;

  /**
   * <p>Return the value of the <code>ondblclick</code> property.  Contents:</p><p>
   * Javascript double click event
   * </p>
   */
  public java.lang.String getOndblclick() {
    if (null != this.ondblclick) {
      return this.ondblclick;
    }
    ValueBinding _vb = getValueBinding("ondblclick"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>ondblclick</code> property.</p>
   */
  public void setOndblclick(java.lang.String ondblclick) {
    this.ondblclick = ondblclick;
  }


  private java.lang.String onhide;

  /**
   * <p>Return the value of the <code>onhide</code> property.  Contents:</p><p>
   * Function to run whenever the dialog is hidden. 
   * 				Signature is onHide(sourceId, event).  
   * 				SourceId is the id of the triggering DOM object or "CANCEL" if it was the CANCEL button or "OK" if it was the OK button.  
   * 				If it's the cancel button and saveState is true, state of the input fields in the dialog is restored to when the dialog was shown (before the handler is called).
   * </p>
   */
  public java.lang.String getOnhide() {
    if (null != this.onhide) {
      return this.onhide;
    }
    ValueBinding _vb = getValueBinding("onhide"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onhide</code> property.</p>
   */
  public void setOnhide(java.lang.String onhide) {
    this.onhide = onhide;
  }


  private java.lang.String onkeydown;

  /**
   * <p>Return the value of the <code>onkeydown</code> property.  Contents:</p><p>
   * Javascript key down event
   * </p>
   */
  public java.lang.String getOnkeydown() {
    if (null != this.onkeydown) {
      return this.onkeydown;
    }
    ValueBinding _vb = getValueBinding("onkeydown"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onkeydown</code> property.</p>
   */
  public void setOnkeydown(java.lang.String onkeydown) {
    this.onkeydown = onkeydown;
  }


  private java.lang.String onkeypress;

  /**
   * <p>Return the value of the <code>onkeypress</code> property.  Contents:</p><p>
   * Javascript key press event
   * </p>
   */
  public java.lang.String getOnkeypress() {
    if (null != this.onkeypress) {
      return this.onkeypress;
    }
    ValueBinding _vb = getValueBinding("onkeypress"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onkeypress</code> property.</p>
   */
  public void setOnkeypress(java.lang.String onkeypress) {
    this.onkeypress = onkeypress;
  }


  private java.lang.String onkeyup;

  /**
   * <p>Return the value of the <code>onkeyup</code> property.  Contents:</p><p>
   * Javascript key up event
   * </p>
   */
  public java.lang.String getOnkeyup() {
    if (null != this.onkeyup) {
      return this.onkeyup;
    }
    ValueBinding _vb = getValueBinding("onkeyup"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onkeyup</code> property.</p>
   */
  public void setOnkeyup(java.lang.String onkeyup) {
    this.onkeyup = onkeyup;
  }


  private java.lang.String onmousedown;

  /**
   * <p>Return the value of the <code>onmousedown</code> property.  Contents:</p><p>
   * Javascript mouse button press event
   * </p>
   */
  public java.lang.String getOnmousedown() {
    if (null != this.onmousedown) {
      return this.onmousedown;
    }
    ValueBinding _vb = getValueBinding("onmousedown"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onmousedown</code> property.</p>
   */
  public void setOnmousedown(java.lang.String onmousedown) {
    this.onmousedown = onmousedown;
  }


  private java.lang.String onmousemove;

  /**
   * <p>Return the value of the <code>onmousemove</code> property.  Contents:</p><p>
   * Javascript mouse move event
   * </p>
   */
  public java.lang.String getOnmousemove() {
    if (null != this.onmousemove) {
      return this.onmousemove;
    }
    ValueBinding _vb = getValueBinding("onmousemove"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onmousemove</code> property.</p>
   */
  public void setOnmousemove(java.lang.String onmousemove) {
    this.onmousemove = onmousemove;
  }


  private java.lang.String onmouseout;

  /**
   * <p>Return the value of the <code>onmouseout</code> property.  Contents:</p><p>
   * Javascript mouse out event
   * </p>
   */
  public java.lang.String getOnmouseout() {
    if (null != this.onmouseout) {
      return this.onmouseout;
    }
    ValueBinding _vb = getValueBinding("onmouseout"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onmouseout</code> property.</p>
   */
  public void setOnmouseout(java.lang.String onmouseout) {
    this.onmouseout = onmouseout;
  }


  private java.lang.String onmouseover;

  /**
   * <p>Return the value of the <code>onmouseover</code> property.  Contents:</p><p>
   * Javascript mouse over event
   * </p>
   */
  public java.lang.String getOnmouseover() {
    if (null != this.onmouseover) {
      return this.onmouseover;
    }
    ValueBinding _vb = getValueBinding("onmouseover"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onmouseover</code> property.</p>
   */
  public void setOnmouseover(java.lang.String onmouseover) {
    this.onmouseover = onmouseover;
  }


  private java.lang.String onmouseup;

  /**
   * <p>Return the value of the <code>onmouseup</code> property.  Contents:</p><p>
   * Javascript mouse button release event
   * </p>
   */
  public java.lang.String getOnmouseup() {
    if (null != this.onmouseup) {
      return this.onmouseup;
    }
    ValueBinding _vb = getValueBinding("onmouseup"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onmouseup</code> property.</p>
   */
  public void setOnmouseup(java.lang.String onmouseup) {
    this.onmouseup = onmouseup;
  }


  private java.lang.String onshow;

  /**
   * <p>Return the value of the <code>onshow</code> property.  Contents:</p><p>
   * Function to run whenever the dialog is made visible. 
   * 				Signature is onshow(sourceId, event). 
   * 				SourceId is the id of the triggering DOM object.
   * </p>
   */
  public java.lang.String getOnshow() {
    if (null != this.onshow) {
      return this.onshow;
    }
    ValueBinding _vb = getValueBinding("onshow"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>onshow</code> property.</p>
   */
  public void setOnshow(java.lang.String onshow) {
    this.onshow = onshow;
  }


  private java.lang.String relativeTo;

  /**
   * <p>Return the value of the <code>relativeTo</code> property.  Contents:</p><p>
   * If provided and one of the positioning attributes is set to relative, 
   * 				then positioning is relative to this ID instead of the ID of the tag which triggered the onShow event.
   * </p>
   */
  public java.lang.String getRelativeTo() {
    if (null != this.relativeTo) {
      return this.relativeTo;
    }
    ValueBinding _vb = getValueBinding("relativeTo"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>relativeTo</code> property.</p>
   */
  public void setRelativeTo(java.lang.String relativeTo) {
    this.relativeTo = relativeTo;
  }


  private boolean saveState = true;
  private boolean saveState_set = false;

  /**
   * <p>Return the value of the <code>saveState</code> property.  Contents:</p><p>
   * Values are true, false.  If true, the values all input, select and textAreas are remembered when onShow is fired.
   * 				Default value is 'true'.
   * </p>
   */
  public boolean isSaveState() {
    if (this.saveState_set) {
      return this.saveState;
    }
    ValueBinding _vb = getValueBinding("saveState"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.saveState;
    }
  }

  /**
   * <p>Set the value of the <code>saveState</code> property.</p>
   */
  public void setSaveState(boolean saveState) {
    this.saveState = saveState;
    this.saveState_set = true;
  }


  private boolean showTitleBar = true;
  private boolean showTitleBar_set = false;

  /**
   * <p>Return the value of the <code>showTitleBar</code> property.  Contents:</p><p>
   * Indicates whether title bar is displayed. Default is true.
   * 				Note that the actual string that is displayed in the title bar is configured via title attribute.
   * </p>
   */
  public boolean isShowTitleBar() {
    if (this.showTitleBar_set) {
      return this.showTitleBar;
    }
    ValueBinding _vb = getValueBinding("showTitleBar"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showTitleBar;
    }
  }

  /**
   * <p>Set the value of the <code>showTitleBar</code> property.</p>
   */
  public void setShowTitleBar(boolean showTitleBar) {
    this.showTitleBar = showTitleBar;
    this.showTitleBar_set = true;
  }


  private boolean showTitleCloseButton = true;
  private boolean showTitleCloseButton_set = false;

  /**
   * <p>Return the value of the <code>showTitleCloseButton</code> property.  Contents:</p><p>
   * Specifies weather to display close button in the title bar, if title bar is shown. Default is true. 
   * 				Designers explicitly need to set this attribute to false in order to hide close button.
   * </p>
   */
  public boolean isShowTitleCloseButton() {
    if (this.showTitleCloseButton_set) {
      return this.showTitleCloseButton;
    }
    ValueBinding _vb = getValueBinding("showTitleCloseButton"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.showTitleCloseButton;
    }
  }

  /**
   * <p>Set the value of the <code>showTitleCloseButton</code> property.</p>
   */
  public void setShowTitleCloseButton(boolean showTitleCloseButton) {
    this.showTitleCloseButton = showTitleCloseButton;
    this.showTitleCloseButton_set = true;
  }


  private java.lang.String style;

  /**
   * <p>Return the value of the <code>style</code> property.  Contents:</p><p>
   * CSS style(s) to be applied when this component is
   * 				rendered.
   * </p>
   */
  public java.lang.String getStyle() {
    if (null != this.style) {
      return this.style;
    }
    ValueBinding _vb = getValueBinding("style"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>style</code> property.</p>
   */
  public void setStyle(java.lang.String style) {
    this.style = style;
  }


  private java.lang.String styleClass;

  /**
   * <p>Return the value of the <code>styleClass</code> property.  Contents:</p><p>
   * Space-separated list of CSS style class(es) to be
   * 				applied when this element is rendered. This value must
   * 				be passed through as the "class" property on generated
   * 				markup.
   * </p>
   */
  public java.lang.String getStyleClass() {
    if (null != this.styleClass) {
      return this.styleClass;
    }
    ValueBinding _vb = getValueBinding("styleClass"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>styleClass</code> property.</p>
   */
  public void setStyleClass(java.lang.String styleClass) {
    this.styleClass = styleClass;
  }


  private java.lang.String title;

  /**
   * <p>Return the value of the <code>title</code> property.  Contents:</p><p>
   * The title to display in the title bar (assuming the chrome is set to display one).
   * </p>
   */
  public java.lang.String getTitle() {
    if (null != this.title) {
      return this.title;
    }
    ValueBinding _vb = getValueBinding("title"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>title</code> property.</p>
   */
  public void setTitle(java.lang.String title) {
    this.title = title;
  }


  private java.lang.String titleCloseButtonImage;

  /**
   * <p>Return the value of the <code>titleCloseButtonImage</code> property.  Contents:</p><p>
   * Specifies the URI of the image used to render close button. 
   * 				By default, we use 1x1.gif and populate it via CSS background-image property
   * </p>
   */
  public java.lang.String getTitleCloseButtonImage() {
    if (null != this.titleCloseButtonImage) {
      return this.titleCloseButtonImage;
    }
    ValueBinding _vb = getValueBinding("titleCloseButtonImage"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>titleCloseButtonImage</code> property.</p>
   */
  public void setTitleCloseButtonImage(java.lang.String titleCloseButtonImage) {
    this.titleCloseButtonImage = titleCloseButtonImage;
  }


  private java.lang.String type;

  /**
   * <p>Return the value of the <code>type</code> property.  Contents:</p><p>
   * Specifies the type of the dialog box.  
   * 				Values are modal and modeless. Default is modal.
   * </p>
   */
  public java.lang.String getType() {
    if (null != this.type) {
      return this.type;
    }
    ValueBinding _vb = getValueBinding("type"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>type</code> property.</p>
   */
  public void setType(java.lang.String type) {
    this.type = type;
  }


  private java.lang.String valign;

  /**
   * <p>Return the value of the <code>valign</code> property.  Contents:</p><p>
   * This attribute specifies the horizontal alignment of the dialog with respect to the surrounding context. Possible values: 
   * 					- top: the top edge of the dialog is aligned with the top edge of the browser window
   * 					- bottom: the bottom edge of the dialog is aligned with the bottom edge of the browser window
   * 					- middle: dialog is vertically centered relative to the browser window.  
   * 					- relative: dialog's top edge is aligned with the BOTTOM edge of the whatever object caused the onShow event to fire  OR it is relative to relativeTo if relativeTo is provided.
   * </p>
   */
  public java.lang.String getValign() {
    if (null != this.valign) {
      return this.valign;
    }
    ValueBinding _vb = getValueBinding("valign"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>valign</code> property.</p>
   */
  public void setValign(java.lang.String valign) {
    this.valign = valign;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[40];
    _values[0] = super.saveState(_context);
    _values[1] = align;
    _values[2] = binding;
    _values[3] = dir;
    _values[4] = this.enableMoveKey ? Boolean.TRUE : Boolean.FALSE;
    _values[5] = this.enableMoveKey_set ? Boolean.TRUE : Boolean.FALSE;
    _values[6] = _for;
    _values[7] = initialFocus;
    _values[8] = this.initiallyShow ? Boolean.TRUE : Boolean.FALSE;
    _values[9] = this.initiallyShow_set ? Boolean.TRUE : Boolean.FALSE;
    _values[10] = lang;
    _values[11] = this.movable ? Boolean.TRUE : Boolean.FALSE;
    _values[12] = this.movable_set ? Boolean.TRUE : Boolean.FALSE;
    _values[13] = this.noKeybindings ? Boolean.TRUE : Boolean.FALSE;
    _values[14] = this.noKeybindings_set ? Boolean.TRUE : Boolean.FALSE;
    _values[15] = onclick;
    _values[16] = ondblclick;
    _values[17] = onhide;
    _values[18] = onkeydown;
    _values[19] = onkeypress;
    _values[20] = onkeyup;
    _values[21] = onmousedown;
    _values[22] = onmousemove;
    _values[23] = onmouseout;
    _values[24] = onmouseover;
    _values[25] = onmouseup;
    _values[26] = onshow;
    _values[27] = relativeTo;
    _values[28] = this.saveState ? Boolean.TRUE : Boolean.FALSE;
    _values[29] = this.saveState_set ? Boolean.TRUE : Boolean.FALSE;
    _values[30] = this.showTitleBar ? Boolean.TRUE : Boolean.FALSE;
    _values[31] = this.showTitleBar_set ? Boolean.TRUE : Boolean.FALSE;
    _values[32] = this.showTitleCloseButton ? Boolean.TRUE : Boolean.FALSE;
    _values[33] = this.showTitleCloseButton_set ? Boolean.TRUE : Boolean.FALSE;
    _values[34] = style;
    _values[35] = styleClass;
    _values[36] = title;
    _values[37] = titleCloseButtonImage;
    _values[38] = type;
    _values[39] = valign;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.align = (java.lang.String) _values[1];
    this.binding = (java.lang.String) _values[2];
    this.dir = (java.lang.String) _values[3];
    this.enableMoveKey = ((Boolean) _values[4]).booleanValue();
    this.enableMoveKey_set = ((Boolean) _values[5]).booleanValue();
    this._for = (java.lang.String) _values[6];
    this.initialFocus = (java.lang.String) _values[7];
    this.initiallyShow = ((Boolean) _values[8]).booleanValue();
    this.initiallyShow_set = ((Boolean) _values[9]).booleanValue();
    this.lang = (java.lang.String) _values[10];
    this.movable = ((Boolean) _values[11]).booleanValue();
    this.movable_set = ((Boolean) _values[12]).booleanValue();
    this.noKeybindings = ((Boolean) _values[13]).booleanValue();
    this.noKeybindings_set = ((Boolean) _values[14]).booleanValue();
    this.onclick = (java.lang.String) _values[15];
    this.ondblclick = (java.lang.String) _values[16];
    this.onhide = (java.lang.String) _values[17];
    this.onkeydown = (java.lang.String) _values[18];
    this.onkeypress = (java.lang.String) _values[19];
    this.onkeyup = (java.lang.String) _values[20];
    this.onmousedown = (java.lang.String) _values[21];
    this.onmousemove = (java.lang.String) _values[22];
    this.onmouseout = (java.lang.String) _values[23];
    this.onmouseover = (java.lang.String) _values[24];
    this.onmouseup = (java.lang.String) _values[25];
    this.onshow = (java.lang.String) _values[26];
    this.relativeTo = (java.lang.String) _values[27];
    this.saveState = ((Boolean) _values[28]).booleanValue();
    this.saveState_set = ((Boolean) _values[29]).booleanValue();
    this.showTitleBar = ((Boolean) _values[30]).booleanValue();
    this.showTitleBar_set = ((Boolean) _values[31]).booleanValue();
    this.showTitleCloseButton = ((Boolean) _values[32]).booleanValue();
    this.showTitleCloseButton_set = ((Boolean) _values[33]).booleanValue();
    this.style = (java.lang.String) _values[34];
    this.styleClass = (java.lang.String) _values[35];
    this.title = (java.lang.String) _values[36];
    this.titleCloseButtonImage = (java.lang.String) _values[37];
    this.type = (java.lang.String) _values[38];
    this.valign = (java.lang.String) _values[39];
  }


}
