/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.component.html;


import java.io.IOException;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;


/**
 * <p>By default, the <code>rendererType</code> property must be set to "<code>com.ibm.faces.Bar</code>" This value can be changed by calling the <code>setRendererType()</code> method.</p>
*/
public class HtmlProgressBar extends com.ibm.faces.component.UIProgress {


  public HtmlProgressBar() {
    super();
    setRendererType("com.ibm.faces.Bar"); //$NON-NLS-1$
  }


  /*
   * <p>The standard component type for this component.</p>
   */
   public static final String COMPONENT_TYPE = "com.ibm.faces.HtmlProgressBar"; //$NON-NLS-1$


  private boolean auto = false;
  private boolean auto_set = false;

  /**
   * <p>Return the value of the <code>auto</code> property.  Contents:</p><p>
   * boolean - true or not set means progress bar shows something is taking place
   * 						- false progress bar is in percentages (e.g a percentage done or left to do).
   * </p>
   */
  public boolean isAuto() {
    if (this.auto_set) {
      return this.auto;
    }
    ValueBinding _vb = getValueBinding("auto"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.auto;
    }
  }

  /**
   * <p>Set the value of the <code>auto</code> property.</p>
   */
  public void setAuto(boolean auto) {
    this.auto = auto;
    this.auto_set = true;
  }


  private boolean initHidden = false;
  private boolean initHidden_set = false;

  /**
   * <p>Return the value of the <code>initHidden</code> property.  Contents:</p><p>
   * Whenther the progressBar is hidden when the page loads. 
   * 				progressBar would need to be unhidden using javascript calls.
   * </p>
   */
  public boolean isInitHidden() {
    if (this.initHidden_set) {
      return this.initHidden;
    }
    ValueBinding _vb = getValueBinding("initHidden"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.initHidden;
    }
  }

  /**
   * <p>Set the value of the <code>initHidden</code> property.</p>
   */
  public void setInitHidden(boolean initHidden) {
    this.initHidden = initHidden;
    this.initHidden_set = true;
  }


  private java.lang.String initValue;

  /**
   * <p>Return the value of the <code>initValue</code> property.  Contents:</p><p>
   * Only used where you are in percentage mode.
   * 			When used in conjunction with auto set to false or not set, the amount 
   * 			(percentage) of the bar initally displayed (default is 0%), 
   * 			e.g. a value of 15 means that 15% of the bar is displayed on the bar 
   * 			when it is intially displayed.
   * </p>
   */
  public java.lang.String getInitValue() {
    if (null != this.initValue) {
      return this.initValue;
    }
    ValueBinding _vb = getValueBinding("initValue"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>initValue</code> property.</p>
   */
  public void setInitValue(java.lang.String initValue) {
    this.initValue = initValue;
  }


  private java.lang.String message;

  /**
   * <p>Return the value of the <code>message</code> property.  Contents:</p><p>
   * The message to appear on the progressbar
   * </p>
   */
  public java.lang.String getMessage() {
    if (null != this.message) {
      return this.message;
    }
    ValueBinding _vb = getValueBinding("message"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>message</code> property.</p>
   */
  public void setMessage(java.lang.String message) {
    this.message = message;
  }


  private boolean outward = false;
  private boolean outward_set = false;

  /**
   * <p>Return the value of the <code>outward</code> property.  Contents:</p><p>
   * boolean - true means progress bar expands from centre outwards, 
   * 				false /not set means end to end expansion
   * 				used in conjunction with auto.
   * </p>
   */
  public boolean isOutward() {
    if (this.outward_set) {
      return this.outward;
    }
    ValueBinding _vb = getValueBinding("outward"); //$NON-NLS-1$
    if (_vb != null) {
      Object _result = _vb.getValue(getFacesContext());
      if (_result == null) {
        return false;
      } else {
        return ((Boolean) _result).booleanValue();
      }
    } else {
        return this.outward;
    }
  }

  /**
   * <p>Set the value of the <code>outward</code> property.</p>
   */
  public void setOutward(boolean outward) {
    this.outward = outward;
    this.outward_set = true;
  }


  private java.lang.String proportion;

  /**
   * <p>Return the value of the <code>proportion</code> property.  Contents:</p><p>
   * the amount (percentage) of the bar added in auto progress updates,  
   * 				when used in conjunction with auto set to true (default is 5 %), 
   * 				e.g. a value of 10 means the bar gets 10% wider each interval.
   * 				When used in conjunction with auto set to false or not set, the amount 
   * 				(percentage) of the bar initally displayed (default is 0 %), 
   * 				e.g. a value of 15 means that 15% of the bar is displayed on the bar 
   * 				when it is intially displayed.
   * </p>
   */
  public java.lang.String getProportion() {
    if (null != this.proportion) {
      return this.proportion;
    }
    ValueBinding _vb = getValueBinding("proportion"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>proportion</code> property.</p>
   */
  public void setProportion(java.lang.String proportion) {
    this.proportion = proportion;
  }


  private java.lang.String style;

  /**
   * <p>Return the value of the <code>style</code> property.  Contents:</p><p>
   * this is a html style string applied to the progress bar
   * </p>
   */
  public java.lang.String getStyle() {
    if (null != this.style) {
      return this.style;
    }
    ValueBinding _vb = getValueBinding("style"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>style</code> property.</p>
   */
  public void setStyle(java.lang.String style) {
    this.style = style;
  }


  private java.lang.String styleClass;

  /**
   * <p>Return the value of the <code>styleClass</code> property.  Contents:</p><p>
   * This is prefix of the styleClass to be applied to the progress Bar.
   * </p>
   */
  public java.lang.String getStyleClass() {
    if (null != this.styleClass) {
      return this.styleClass;
    }
    ValueBinding _vb = getValueBinding("styleClass"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>styleClass</code> property.</p>
   */
  public void setStyleClass(java.lang.String styleClass) {
    this.styleClass = styleClass;
  }


  private java.lang.String timeInterval;

  /**
   * <p>Return the value of the <code>timeInterval</code> property.  Contents:</p><p>
   * the time interval between auto progress updates.
   * 				time in milliseconds.
   * 				e.g. a value of 5000 means the bar changes every 5 seconds.
   * 				used in conjunction with auto.
   * </p>
   */
  public java.lang.String getTimeInterval() {
    if (null != this.timeInterval) {
      return this.timeInterval;
    }
    ValueBinding _vb = getValueBinding("timeInterval"); //$NON-NLS-1$
    if (_vb != null) {
      return (java.lang.String) _vb.getValue(getFacesContext());
    } else {
      return null;
    }
  }

  /**
   * <p>Set the value of the <code>timeInterval</code> property.</p>
   */
  public void setTimeInterval(java.lang.String timeInterval) {
    this.timeInterval = timeInterval;
  }


  public Object saveState(FacesContext _context) {
    Object _values[] = new Object[13];
    _values[0] = super.saveState(_context);
    _values[1] = this.auto ? Boolean.TRUE : Boolean.FALSE;
    _values[2] = this.auto_set ? Boolean.TRUE : Boolean.FALSE;
    _values[3] = this.initHidden ? Boolean.TRUE : Boolean.FALSE;
    _values[4] = this.initHidden_set ? Boolean.TRUE : Boolean.FALSE;
    _values[5] = initValue;
    _values[6] = message;
    _values[7] = this.outward ? Boolean.TRUE : Boolean.FALSE;
    _values[8] = this.outward_set ? Boolean.TRUE : Boolean.FALSE;
    _values[9] = proportion;
    _values[10] = style;
    _values[11] = styleClass;
    _values[12] = timeInterval;
    return _values;
  }


  public void restoreState(FacesContext _context, Object _state) {
    Object _values[] = (Object[]) _state;
    super.restoreState(_context, _values[0]);
    this.auto = ((Boolean) _values[1]).booleanValue();
    this.auto_set = ((Boolean) _values[2]).booleanValue();
    this.initHidden = ((Boolean) _values[3]).booleanValue();
    this.initHidden_set = ((Boolean) _values[4]).booleanValue();
    this.initValue = (java.lang.String) _values[5];
    this.message = (java.lang.String) _values[6];
    this.outward = ((Boolean) _values[7]).booleanValue();
    this.outward_set = ((Boolean) _values[8]).booleanValue();
    this.proportion = (java.lang.String) _values[9];
    this.style = (java.lang.String) _values[10];
    this.styleClass = (java.lang.String) _values[11];
    this.timeInterval = (java.lang.String) _values[12];
  }


}
