/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.context;

import javax.faces.context.ResponseWriter;

/**
 * External contexts that implement this interfaces must:
 * <ul>
 * <li>Wrap the response in the dispatch method to return an output stream that writes to null</li>
 * <li>Return <code>isWrappedDispatchRequest()==true</code> if <code>dispatch(String)</code> is called</li>
 * <li>Return a <code>ResponseWriter</code> wrapping the real output stream</li>
 * </ul>
 * This interface is used to delegate platform functions.
 * If an AJAX aware <code>ExternalContext</code> is not provided, servlet calls are made.
 * 
 * @author Alasdair McDowell
 */
public interface AjaxAwareExternalContext {
	
	/**
	 * Returns true if the dispatch method has been called and the external context has wrapped the request
	 * to send non-JSF output to a null response writer. Used to eliminated unwanted output in JSP AJAX responses.
	 */
	public boolean isWrappedDispatchRequest();
	
	/**
	 * If <code>isWrappedDispatchRequest()</code> returns true, then the <code>ResponseWriter</code> set in <code>FacesContext</code>
	 * will wrap the null stream passed by the dispatch method. 
	 */
	public ResponseWriter getRealResponseWriter();
	
	/**
	 * Generates a URL for making AJAX calls.
	 */
	public String encodeAjaxUrl(String uri);
	
}
