/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.context;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;

import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.Principal;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import javax.faces.FacesException;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.RenderKit;
//import javax.portlet.ActionRequest;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

import com.ibm.faces.util.ContextParams;
import com.ibm.faces.jwlajaxencoder.AjaxUrlEncoder;
import com.ibm.faces.jwlajaxencoder.AjaxUrlFactory;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;;
/**
 * @author ieu93313
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class AjaxExternalContext extends ExternalContext implements AjaxAwareExternalContext {

	private FacesContext _delegate;
	private ExternalContext _externalContext;
	private AjaxAwareExternalContext _ajaxExternalContext;
	
	private AjaxHttpServletResponseWrapper _responseWrapper = null;
	
	private boolean _isWrappedDispatchRequest = false;
	
	private ResponseWriter _responseWriter = null;
	
	public AjaxExternalContext(FacesContext delegate) {
		_delegate = delegate;
		_externalContext = delegate.getExternalContext();
		
		if(_externalContext instanceof AjaxAwareExternalContext) {
			_ajaxExternalContext = (AjaxAwareExternalContext) _externalContext;
		}
	}

	/* (non-Javadoc)
	 * @see com.ibm.faces.context.IAjaxExternalContext#getRealResponseWriter()
	 */
	public ResponseWriter getRealResponseWriter() {
		if(_ajaxExternalContext==null) {
			if(_responseWriter==null) {
				try {
					RenderKit renderKit = _delegate.getRenderKit();
					String charEncoding = ((ServletRequest) _externalContext.getRequest()).getCharacterEncoding();
					//TODO: middle arg
					_responseWriter = renderKit.createResponseWriter(((ServletResponse) _externalContext.getResponse()).getWriter(), null, charEncoding);
				} catch(IOException e) {
					throw new FacesException(e);
				}
			}
			return _responseWriter;
		} 
		
		return _ajaxExternalContext.getRealResponseWriter();
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.faces.context.IAjaxExternalContext#isWrappedDispatchRequest()
	 */
	public boolean isWrappedDispatchRequest() {
		if(_ajaxExternalContext==null) {
			return _isWrappedDispatchRequest;
		}
		return _ajaxExternalContext.isWrappedDispatchRequest();
	}
	

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#dispatch(java.lang.String)
	 */
	public void dispatch(String requestURI) throws IOException {
	
		if(_ajaxExternalContext != null) {
			// Preserve current behavior for Designer
			// if the delegate understands AJAX, trust it to do the right thing
			_externalContext.dispatch(requestURI);
			return;
		}
		
		if(HxClientRenderUtil.isPortal(_externalContext.getRequest())) {
			// if in Portal, delegate
			_externalContext.dispatch(requestURI);
			return;
		}
		
		HttpServletRequest servletRequest = (HttpServletRequest) _externalContext.getRequest();
		HttpServletResponse servletResponse = (HttpServletResponse) _externalContext.getResponse();
		
		if(_responseWrapper==null) {
			_responseWrapper = new AjaxHttpServletResponseWrapper(servletResponse);
		}
		
		RequestDispatcher requestDispatcher = servletRequest.getRequestDispatcher(requestURI);
		try {
			_isWrappedDispatchRequest = true;
			requestDispatcher.forward(servletRequest, _responseWrapper);				
		} catch (IOException ioe) {
			// e.printStackTrace();
			throw ioe;
		} catch (ServletException se) {
			throw new FacesException(se);
		}
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#encodeActionURL(java.lang.String)
	 */
	public String encodeActionURL(String arg0) {
		return _externalContext.encodeActionURL(arg0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#encodeNamespace(java.lang.String)
	 */
	public String encodeNamespace(String arg0) {
		return _externalContext.encodeNamespace(arg0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#encodeResourceURL(java.lang.String)
	 */
	public String encodeResourceURL(String arg0) {
		return _externalContext.encodeResourceURL(arg0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getApplicationMap()
	 */
	public Map getApplicationMap() {
		return _externalContext.getApplicationMap();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getAuthType()
	 */
	public String getAuthType() {
		return _externalContext.getAuthType();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getContext()
	 */
	public Object getContext() {
		return _externalContext.getContext();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getInitParameter(java.lang.String)
	 */
	public String getInitParameter(String arg0) {		
		return _externalContext.getInitParameter(arg0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getInitParameterMap()
	 */
	public Map getInitParameterMap() {
		return _externalContext.getInitParameterMap();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRemoteUser()
	 */
	public String getRemoteUser() {
		return _externalContext.getRemoteUser();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequest()
	 */
	public Object getRequest() {
		return _externalContext.getRequest();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestContextPath()
	 */
	public String getRequestContextPath() {
		return _externalContext.getRequestContextPath();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestCookieMap()
	 */
	public Map getRequestCookieMap() {
		return _externalContext.getRequestCookieMap();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestHeaderMap()
	 */
	public Map getRequestHeaderMap() {
		return _externalContext.getRequestHeaderMap();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestHeaderValuesMap()
	 */
	public Map getRequestHeaderValuesMap() {
		return _externalContext.getRequestHeaderValuesMap();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestLocale()
	 */
	public Locale getRequestLocale() {
		return _externalContext.getRequestLocale();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestLocales()
	 */
	public Iterator getRequestLocales() {
		return _externalContext.getRequestLocales();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestMap()
	 */
	public Map getRequestMap() {
		return _externalContext.getRequestMap();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestParameterMap()
	 */
	public Map getRequestParameterMap() {
		return _externalContext.getRequestParameterMap();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestParameterNames()
	 */
	public Iterator getRequestParameterNames() {
		return _externalContext.getRequestParameterNames();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestParameterValuesMap()
	 */
	public Map getRequestParameterValuesMap() {
		return _externalContext.getRequestParameterValuesMap();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestPathInfo()
	 */
	public String getRequestPathInfo() {
		return _externalContext.getRequestPathInfo();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getRequestServletPath()
	 */
	public String getRequestServletPath() {
		return _externalContext.getRequestServletPath();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getResource(java.lang.String)
	 */
	public URL getResource(String arg0) throws MalformedURLException {
		return _externalContext.getResource(arg0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getResourceAsStream(java.lang.String)
	 */
	public InputStream getResourceAsStream(String arg0) {
		return _externalContext.getResourceAsStream(arg0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getResourcePaths(java.lang.String)
	 */
	public Set getResourcePaths(String arg0) {
		return _externalContext.getResourcePaths(arg0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getResponse()
	 */
	public Object getResponse() {
		return _externalContext.getResponse(); // TODO should this return the wrapper?
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getSession(boolean)
	 */
	public Object getSession(boolean arg0) {
		return _externalContext.getSession(arg0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getSessionMap()
	 */
	public Map getSessionMap() {
		return _externalContext.getSessionMap();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#getUserPrincipal()
	 */
	public Principal getUserPrincipal() {
		return _externalContext.getUserPrincipal();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#isUserInRole(java.lang.String)
	 */
	public boolean isUserInRole(String arg0) {
		return _externalContext.isUserInRole(arg0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#log(java.lang.String)
	 */
	public void log(String arg0) {
		_externalContext.log(arg0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#log(java.lang.String, java.lang.Throwable)
	 */
	public void log(String arg0, Throwable arg1) {
		_externalContext.log(arg0,arg1);
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ExternalContext#redirect(java.lang.String)
	 */
	public void redirect(String arg0) throws IOException {
		_externalContext.redirect(arg0); // TODO what about this?
	}

	class AjaxHttpServletResponseWrapper extends HttpServletResponseWrapper {

		private NullPrintWriter _writer = null;
		
		protected AjaxHttpServletResponseWrapper(HttpServletResponse response) {
			super(response);
		}
		
		/* (non-Javadoc)
		 * @see javax.servlet.ServletResponseWrapper#getWriter()
		 */
		public PrintWriter getWriter() throws IOException {
			if(_writer==null) {
				_writer = new NullPrintWriter(new PrintWriter(new StringWriter()));
			}
			return _writer;
		}
		
	}

	/* (non-Javadoc)
	 * @see com.ibm.faces.context.AjaxAwareExternalContext#encodeAjaxUrl()
	 */
	public String encodeAjaxUrl(String uri) {
		if(_externalContext instanceof AjaxAwareExternalContext) {
			return ((AjaxAwareExternalContext) _externalContext).encodeAjaxUrl(uri);
		}
        
        FacesContext context = FacesContext.getCurrentInstance();
        AjaxUrlEncoder aue = AjaxUrlFactory.getAjaxUrlEncoder(context);
        return aue.encodeAjaxUrl(context, uri);
    }
	

	//new JSF 1.2 api methods
	 
	public String getRequestContentType() {
		if(_externalContext.getRequest() instanceof ServletRequest){
			return ((ServletRequest)_externalContext.getRequest()).getContentType();
		}
		return null;
	}

	/*public String getRequestCharacterEncoding() {
		if(_externalContext.getRequest() instanceof ServletRequest){
			return ((ServletRequest)_externalContext.getRequest()).getCharacterEncoding();
		}
		if(_externalContext.getRequest() instanceof ActionRequest){
			return ((ActionRequest)_externalContext.getRequest()).getCharacterEncoding();
		}
		return null;
	}*/

	public String getResponseCharacterEncoding() {
		if(_externalContext.getResponse() instanceof ServletResponse){
			return ((ServletResponse)_externalContext.getResponse()).getCharacterEncoding();
		}
		return null;
	}
	
	private String ENV_UNSUPPORTED = "This operation is not supported in this Java EE environment, JavaEE5 required.";
	
	/*
	 * <p>This method will only work with JavaEE5, otherwise returns null.</p> 
	 */
	public String getResponseContentType() {
		if(_externalContext.getResponse() instanceof ServletResponse){
			//if pre JavaEE5 this method will not be available on response object
			try {
				Method m = ((ServletResponse)_externalContext.getResponse()).getClass().getMethod("getContentType",null);
				return (String)m.invoke(_externalContext.getResponse(),null);
			} catch (Exception e) {
				//method not available pre JavaEE5, just return null
				throw new UnsupportedOperationException(ENV_UNSUPPORTED);
			} 
		}
		return null;
	}

	public void setRequest(Object req) {
		
		try{
			Method m = _externalContext.getClass().getMethod("setRequest",new Class[]{Object.class});
			m.invoke(_externalContext,new Object[]{req});
			
		} catch(Exception e){
			//method not available pre JavaEE5, just return null
			throw new UnsupportedOperationException(ENV_UNSUPPORTED);
		}
	}

	/*public void setRequestCharacterEncoding(String encoding) throws UnsupportedEncodingException {
		if(_externalContext.getRequest() instanceof ServletRequest){
			((ServletRequest)_externalContext.getRequest()).setCharacterEncoding(encoding);
		}
		if(_externalContext.getRequest() instanceof ActionRequest){
			((ActionRequest)_externalContext.getRequest()).setCharacterEncoding(encoding);
		}
	}*/

	public void setResponse(Object res) {
		//attempt to execute on passed in externalcontext
		try{
			Method m = _externalContext.getClass().getMethod("setResponse",new Class[]{Object.class});
			m.invoke(_externalContext,new Object[]{res});
			
		} catch(Exception e){
			//method not available pre JavaEE5, just return null
			throw new UnsupportedOperationException(ENV_UNSUPPORTED);
		}
	}
	
	/*
	 * <p>This method will only work with JavaEE5, otherwise returns null.</p> 
	 */
	public void setResponseCharacterEncoding(String encoding) {
		
		if(_externalContext.getResponse() instanceof ServletResponse){
			//if pre JavaEE5 this method will not be available on response object
			try {
				Method m = ((ServletResponse)_externalContext.getResponse()).getClass().getMethod("setCharacterEncoding",new Class[]{String.class});
				m.invoke(_externalContext.getResponse(),new Object[]{encoding});
			} catch (Exception e) {
				//method not available pre JavaEE5, just return null
				throw new UnsupportedOperationException(ENV_UNSUPPORTED);
			} 
		}
	}
	//end of new JSF 1.2 methods.
	
}
