/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.context;

import javax.faces.FacesException;
import javax.faces.context.FacesContext;
import javax.faces.context.FacesContextFactory;
import javax.faces.lifecycle.Lifecycle;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;


import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.faces.util.AjaxUtil;


import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;

import javax.faces.application.ViewHandler;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import com.ibm.faces.util.ContextParams;



/**
 * AjaxFacesContextFactory is an implementation of FacesContextFactory which will decorate the default
 * FacesContext with the AjaxContext defined behaviour.
 * 
 * @author mark_wallace@ie.ibm.com
 */
public class AjaxFacesContextFactory extends FacesContextFactory {

	private FacesContextFactory _contextFactory;

	/**
	 * Construct an AjaxFacesContextFactory
	 */
	public AjaxFacesContextFactory() {
		/*
		 * This constructor should not get called, a context factory should 
		 * already exist before this one is created, however just in case use the
		 * only one we know that we can as we don't know what underlying ri is 
		 * being used... 
		 */
		_contextFactory = new MultipartFacesContextFactoryImpl();
	}

	/**
	 * Construct an AjaxFacesContextFactory
	 */
	public AjaxFacesContextFactory(FacesContextFactory delegate) {
		if (delegate == null) {
			throw new NullPointerException(
					"Invalid FacesContextFactory delegate");
		}
		_contextFactory = delegate;
	}

	/**
	 * @since 1.0
	 * @see javax.faces.context.FacesContextFactory#getFacesContext(java.lang.Object,
	 *      java.lang.Object, java.lang.Object, javax.faces.lifecycle.Lifecycle)
	 */
	public FacesContext getFacesContext(Object context, Object request,
			Object response, Lifecycle lifecycle) throws FacesException {
		
		try{
			if(!HxClientRenderUtil.isPortal(request)) // encoding sttring should be something line SHIFT_JIS
				((ServletRequest)request).setCharacterEncoding(getRequestEncoding(request, context));
		}catch(Exception e)
		{
			
		}
		
		// delegate to create a FacesContext instance
		FacesContext facesContext = _contextFactory.getFacesContext(context,
				request, response, lifecycle);
		if (facesContext == null) {
			return null;
		}
		
		//LifecycleUtil.getRequestEncoding(request, facesContext);
				
		// check is this an Ajax request
		int mode = AjaxUtil.getAjaxMode(facesContext);
        if (mode != AjaxUtil.AJAX_NONE) {
        	AjaxUtil.getAjaxComponentId(facesContext);
            facesContext = new AjaxFacesContext(facesContext);        
        }
        
		return facesContext;
	}

	
	
	public static String getRequestEncoding(Object req, Object sc) {
		// First, see if there's anything in the request
		String enc = ((ServletRequest) req).getCharacterEncoding();

		// If the request has no encoding info, search everywhere else
		if (null == enc) {
			if (req instanceof HttpServletRequest) {
				// Look for a charset in the Content-Type header
				String cType = (String) ((HttpServletRequest) req).getHeader("Content-Type");
				if (null != cType) {
					int iX = cType.indexOf("charset=");
					if (iX >= 0 && cType.length() > iX + 8) {
						enc = cType.substring(iX + 8).trim();
					}
				}

				// OK, still no encoding: see if it was set by the last request
				if (null == enc) {
					HttpSession sess = ((HttpServletRequest) req).getSession(false);
					if (null != sess)
					   enc = (String)sess.getAttribute(ViewHandler.CHARACTER_ENCODING_KEY);
				}
			}
			// Still none: use JVM settings
			if (null == enc) {
				try {
					enc = System.getProperty("file.encoding", "ISO8859_1");
				} catch (SecurityException se) {
					// Nothing - default to Latin-1
					enc = "ISO8859_1";
				}
			}
		}

		// If we've found it, map it according to the properties file
		if (null != enc) {
			enc = mapEncodingFromProperties(enc, sc);
		}
		return enc;
	}
	
	
	
	// This is done being *very* basic
	private static String[][] encNVPs = null;
	private static String mapEncodingFromProperties(String enc, Object sc) {
		String encoding = enc;
		int iX;
		// Find the mapping file
		if (null == encNVPs) {
			// Load the mappings into the table
			try {
				URL mapUrl = null;
				String NVPUrl = ((ServletContext) sc).getInitParameter(ContextParams.ENCODING_MAPPING);
				if (null != NVPUrl) { // If the param is set, use that value
					mapUrl = new URL(NVPUrl);
				} else {              // Otherwise default to WAS converter.properties
					mapUrl = Thread.currentThread().getContextClassLoader().getResource("converter.properties");
				}
				if (null != mapUrl) {
					InputStream inStrm = mapUrl.openStream();
					InputStreamReader inRdr = new InputStreamReader(inStrm);
					BufferedReader bufRdr = new BufferedReader(inRdr);
					String line;
					encNVPs = new String[100][2];
					int NVix = 0;
					while ((line = bufRdr.readLine()) != null) {
						iX = line.indexOf("=");
						if (iX > 0) {
							encNVPs[NVix][0] = new String(line.substring(0,iX).trim());
							encNVPs[NVix++][1] = new String(line.substring(iX+1).trim());
						}
					}
				}
			} catch (Exception e) {
				encNVPs = null;
			}
			// If we didn't find any, simply create a dummy one
			// to allow this to sun through OK.
			if (null == encNVPs) {
				encNVPs = new String[1][2];
			}
		}
		// Now that we've loaded the file (should only happen once), find the Name/Value entry
		for (iX = 0; iX<encNVPs.length && null != encNVPs[iX][0]; iX++) { 
			if (encNVPs[iX][0].equalsIgnoreCase(enc)) {
				encoding = encNVPs[iX][1];
				break;
			}
		}
		return encoding;
	}
	
	
}