/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/
package com.ibm.faces.context;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.FactoryFinder;
import javax.faces.application.Application;
import javax.faces.application.ApplicationFactory;
import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;
import javax.faces.component.UIViewRoot;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseStream;
import javax.faces.context.ResponseWriter;
import javax.faces.render.RenderKit;
import javax.faces.render.RenderKitFactory;

/**
 * @author pandrews
 */

public class FacesContextImpl extends FacesContext {
	
	private Application application = null;
	private Map messages = null;
	private ExternalContext externalContext = null;
	private UIViewRoot viewRoot = null;
	private boolean renderResponse = false;
	private boolean responseComplete = false;
	private ResponseStream responseStream = null;
	private ResponseWriter responseWriter = null;
	
	
	public FacesContextImpl(ExternalContext context){
		this.externalContext = context;
		setCurrentInstance(this);
	}
	
	public void initialize(ExternalContext context){
		this.externalContext = context;
		setCurrentInstance(this);
	}
	
	public Application getApplication() {
		if(application != null){
			return application;
		}
		application = ((ApplicationFactory)FactoryFinder.getFactory(FactoryFinder.APPLICATION_FACTORY)).getApplication();
		return application;
	}


	public Iterator getClientIdsWithMessages() {
		if(messages == null){
			return Collections.EMPTY_LIST.iterator();
		} 
		return messages.keySet().iterator();
	}

	public ExternalContext getExternalContext() {
		return externalContext;
	}

	public Severity getMaximumSeverity() {
		int highest = 0;
		Severity mostSevere = null;
		
		Iterator allMessages = getMessages();
		while(allMessages.hasNext()){
			FacesMessage fm = (FacesMessage)allMessages.next();
			if(fm.getSeverity().getOrdinal() > highest){
				highest = fm.getSeverity().getOrdinal();
				mostSevere = fm.getSeverity();
			}
		}
		return mostSevere;
	}

	public Iterator getMessages() {
		ArrayList messageList = new ArrayList();
		if(messages == null){
			return messageList.iterator();
		}
		Iterator allLists = messages.values().iterator();
		while (allLists.hasNext()){
			List list = (List)allLists.next();
			Iterator contents = list.iterator();
			while(contents.hasNext()){
				messageList.add(contents.next());
			}
		}
		return messageList.iterator();
	}

	public Iterator getMessages(String clientId) {
		if(messages == null){
			return Collections.EMPTY_LIST.iterator();
		}
		List messageList = (List)messages.get(clientId);
		if(messageList == null){
			return Collections.EMPTY_LIST.iterator();
		}
		return messageList.iterator();
	}

	public RenderKit getRenderKit() {
		UIViewRoot root = getViewRoot();
		if(root == null || root.getRenderKitId() == null){
			return null;
		}
		return ((RenderKitFactory)FactoryFinder.getFactory(FactoryFinder.RENDER_KIT_FACTORY)).getRenderKit(this,root.getRenderKitId());
	}

	public boolean getRenderResponse() {
		return renderResponse;
	}


	public boolean getResponseComplete() {
		return responseComplete;
	}

	public ResponseStream getResponseStream() {
		return responseStream;
	}

	public void setResponseStream(ResponseStream stream) {
		if(stream == null){
			throw new NullPointerException();
		}
		this.responseStream = stream;
	}

	public ResponseWriter getResponseWriter() {
		return responseWriter;
	}

	public void setResponseWriter(ResponseWriter writer) {
		if(writer == null){
			throw new NullPointerException();
		}
		this.responseWriter = writer;
	}

	public UIViewRoot getViewRoot() {
		return viewRoot;
	}

	public void setViewRoot(UIViewRoot viewRoot) {
		this.viewRoot = viewRoot;
	}

	public void addMessage(String clientId, FacesMessage message) {
		if(message == null){
			throw new NullPointerException();
		}
		if(messages == null){
			messages = new HashMap();
		}
		List componentMessages = (List)messages.get(clientId);
		if (componentMessages == null){
			componentMessages = new ArrayList();
			componentMessages.add(message);
			messages.put(clientId,componentMessages);
		} else {
			componentMessages.add(message);
		}
	}


	public void release() {
		
		setCurrentInstance(null);
		
		application = null;
		messages = null;
		externalContext = null;
		viewRoot = null;
		renderResponse = false;
		responseComplete = false;
		responseStream = null;
		responseWriter = null;
		
		//put back in pool if there is room
		if(MultipartFacesContextFactoryImpl.FACES_CONTEXT_POOL.size() < MultipartFacesContextFactoryImpl.FACES_CONTEXT_POOL_SIZE){
			MultipartFacesContextFactoryImpl.FACES_CONTEXT_POOL.add(this);
		}
	}

	public void renderResponse() {
		renderResponse = true;
		
	}

	public void responseComplete() {
		responseComplete = true;
	}

}
