/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.context;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.AbstractMap;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.faces.FacesException;
import javax.faces.application.ViewHandler;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletRequestWrapper;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.ibm.faces.fileupload.util.ContentElement;
import com.ibm.faces.fileupload.util.MultipartContent;
import com.ibm.faces.util.ContextParams;



// System.setProperty("javax.faces.context.FacesContextFactory", FileUploadFacesContextFactory.class.getName());

public class MultipartExternalContextImpl extends ExternalContext {
	public static final String FILE_ITEM_MAP_PARAM = "com.ibm.faces.FileItemMap";
	
	private ServletContext servletContext = null;
	private ServletRequest request = null;
	private ServletResponse response = null;

	private ApplicationMap applicationMap = null;
	private SessionMap sessionMap = null;
	private RequestMap requestMap = null;
	private RequestParameterMap requestParameterMap = null;
	private RequestParameterValuesMap requestParameterValuesMap = null;
	private RequestHeaderMap requestHeaderMap = null;
	private RequestHeaderValuesMap requestHeaderValuesMap = null;
	private RequestCookieMap cookieMap = null;
	private InitParameterMap initParameterMap = null;

	public MultipartExternalContextImpl(
		ServletContext sc,
		ServletRequest request,
		ServletResponse response) {

		if(sc == null || request == null || response == null){
			throw new NullPointerException();
		}

		// Save references to our context, request, and response
		this.servletContext = sc;
		this.request        = request; // Needed if we call back from the wrapper

		//String enc = getRequestEncoding(this.request);
		
		if (request instanceof HttpServletRequest) {
			this.request = new MultiPartServletRequestWrapper((HttpServletRequest) request);
		} else {
			this.request = new MyServletRequestWrapper(request);
		}

		this.response = response;
	}

	//	 RATLC01099383 (originially RATLC00289948) 	
	private String getRequestEncoding(ServletRequest req) {
		// First, see if there's anything in the request
		String enc = req.getCharacterEncoding();
		
		// If the request has no encoding info, search everywhere else
		if (null == enc) {
			// Look for a charset in the Content-Type header
           String cType = (String) getRequestHeaderMap().get("Content-Type");
           if (null != cType) {
              int iX = cType.indexOf("charset=");
              if (iX >= 0 && cType.length() > iX + 8) {
                 enc = cType.substring(iX + 8).trim();
              }
           }

           // OK, still no encoding: see if it was set by the last request
           if (null == enc) {
              enc = (String) getSessionMap().get(ViewHandler.CHARACTER_ENCODING_KEY);
           }

           // Still none: use JVM settings
           if (null == enc) {
              try {
                 enc = System.getProperty("file.encoding", "ISO8859_1");
              } catch (SecurityException se) {
                 // Nothing - default to Latin-1
                 enc = "ISO8859_1";
              }
           }
		}

        // If we've found it, map it according to the properties file
        if (null != enc) {
        	enc = mapEncodingFromProperties(enc);
        }
        return enc;
	}
	
	
	
	// This is done being *very* basic
	private String[][] encNVPs = null;
	private String mapEncodingFromProperties(String enc) {
		String encoding = enc;
		int iX;
		// Find the mapping file
		if (null == encNVPs) {
			// Load the mappings into the table
			try {
				URL mapUrl = null;
				String NVPUrl = getInitParameter(ContextParams.ENCODING_MAPPING);
				if (null != NVPUrl) { // If the param is set, use that value
					mapUrl = new URL(NVPUrl);
				} else {              // Otherwise default to WAS converter.properties
					mapUrl = Thread.currentThread().getContextClassLoader().getResource("converter.properties");
				}
				if (null != mapUrl) {
					InputStream inStrm = mapUrl.openStream();
					InputStreamReader inRdr = new InputStreamReader(inStrm);
					BufferedReader bufRdr = new BufferedReader(inRdr);
					String line;
					encNVPs = new String[100][2];
					int NVix = 0;
					while ((line = bufRdr.readLine()) != null) {
						iX = line.indexOf("=");
						if (iX > 0) {
							encNVPs[NVix][0] = new String(line.substring(0,iX).trim());
							encNVPs[NVix++][1] = new String(line.substring(iX+1).trim());
						}
					}
				}
			} catch (Exception e) {
				encNVPs = null;
			}
			// If we didn't find any, simply create a dummy one
			// to allow this to sun through OK.
			if (null == encNVPs) {
				encNVPs = new String[1][2];
			}
		}
		// Now that we've loaded the file (should only happen once), find the Name/Value entry
		for (iX = 0; iX<encNVPs.length && null != encNVPs[iX][0]; iX++) { 
			if (encNVPs[iX][0].equalsIgnoreCase(enc)) {
				encoding = encNVPs[iX][1];
				break;
			}
		}
		return encoding;
	}
	
	public Object getSession(boolean create) {
		return (((HttpServletRequest) request).getSession(create));
	}

	public Object getContext() {
		return this.servletContext;
	}

	public Object getRequest() {
		return this.request;
	}

	public Object getResponse() {
		return this.response;
	}

	public Map getApplicationMap() {
		if (applicationMap == null) {
			applicationMap = new ApplicationMap(servletContext);
		}
		return applicationMap;
	}

	public Map getSessionMap() {
        //TODO - check this is OK - getSessionMap()
        //commented out code below + replaced with latest from ExternalContextImpl
//		HttpSession session = (HttpSession) getSession(false);
//		if (session != null) {
//			if (sessionMap == null) {
//				sessionMap = new SessionMap(session);
//			}
//			return sessionMap;
//		} else {
//			return (null);
//		}
        
        if (sessionMap == null)
            sessionMap = new SessionMap((HttpSession) getSession(true));
        return sessionMap;
	}

	public Map getRequestMap() {
		if (requestMap == null) {
			requestMap = new RequestMap(this.request);
		}
		return requestMap;
	}

	public Map getRequestHeaderMap() {
		if (null == requestHeaderMap) {
			requestHeaderMap = new RequestHeaderMap((HttpServletRequest) request);
		}
		return requestHeaderMap;
	}

	public Map getRequestHeaderValuesMap() {
		if (null == requestHeaderValuesMap) {
			requestHeaderValuesMap = new RequestHeaderValuesMap((HttpServletRequest) request);
		}
		return requestHeaderValuesMap;
	}

	public Map getRequestCookieMap() {
		if (null == cookieMap) {
			cookieMap = new RequestCookieMap((HttpServletRequest) request);
		}
		return cookieMap;
	}

	public Map getInitParameterMap() {
		if (null == initParameterMap) {
			initParameterMap = new InitParameterMap(servletContext);
		}
		return initParameterMap;
	}

	public Map getRequestParameterMap() {
		if (null == requestParameterMap) {
			requestParameterMap = new RequestParameterMap(request);
		}
		return requestParameterMap;
	}

	public Map getRequestParameterValuesMap() {
		if (null == requestParameterValuesMap) {
			requestParameterValuesMap = new RequestParameterValuesMap(request);
		}
		return requestParameterValuesMap;
	}

	public Iterator getRequestParameterNames() {
		final Enumeration namEnum = request.getParameterNames();

		Iterator result = new Iterator() {
			public boolean hasNext() {
				return namEnum.hasMoreElements();
			}
			public Object next() {
				return namEnum.nextElement();
			}
			public void remove() {
				throw new UnsupportedOperationException();
			}
		};

		return result;
	}

	public Locale getRequestLocale() {
		return request.getLocale();
	}

	public Iterator getRequestLocales() {
		return (new LocalesIterator(request.getLocales()));
	}

	public String getRequestPathInfo() {
		return (((HttpServletRequest) request).getPathInfo());
	}

	public String getRealPath(String basePath) {
		return servletContext.getRealPath(basePath);
	}

	public Cookie[] getRequestCookies() {
		return (((HttpServletRequest) request).getCookies());
	}

	public String getRequestContextPath() {
		return (((HttpServletRequest) request).getContextPath());
	}

	public String getRequestServletPath() {
		return (((HttpServletRequest) request).getServletPath());
	}

	/**
	 * <p>Manage attributes associated with the <code>ServletContext</code>
	 * instance associated with the current request.</p>
	 */
	public String getInitParameter(String name) {
		return servletContext.getInitParameter(name);
	}

	public Set getResourcePaths(String path) {
		return servletContext.getResourcePaths(path);
	}

	public InputStream getResourceAsStream(String path) {
		return servletContext.getResourceAsStream(path);
	}

	public URL getResource(String path) {
		URL url = null;
		try {
			url = servletContext.getResource(path);
		} catch (MalformedURLException e) {
			return null;
		}
		return url;
	}

	/**
	 * <p>Force any URL that causes an action to work within a portal/portlet. 
	 * This causes the URL to have the required redirection for the specific
	 * portal to be included</p>
	 *
	 * @param sb The input URL to be reformatted
	 */
	public String encodeActionURL(String sb) {
		return ((HttpServletResponse) response).encodeURL(sb);
	}

	/**
	 * <p>Force any URL that references a resource to work within a
	 * portal/portlet. This causes the URL to have the required
	 * redirection for the specific portal to be included. In reality,
	 * it simply returns an absolute URL.</p>
	 *
	 * @param sb The input URL to be reformatted
	 */
	public String encodeResourceURL(String sb) {
		return ((HttpServletResponse) response).encodeURL(sb);
	}

	public String encodeNamespace(String aValue) {
		return aValue; // Do nothing for servlets
	}

	public String encodeURL(String url) {
		return ((HttpServletResponse) response).encodeURL(url);
	}

	/**
	 * <p>Dispatch a request to the apropriate context. In the
	 * case of servlets, this is done via "forward", but for
	 * portlets, it must use "include".</p>
	 *
	 * @param requestURI The input URI of the request tree.
	 */
	public void dispatch(String requestURI)
		throws IOException, FacesException {
		RequestDispatcher requestDispatcher =
			request.getRequestDispatcher(requestURI);
		try {
			requestDispatcher.forward(this.request, this.response);
		} catch (IOException ioe) {
			// e.printStackTrace();
			throw ioe;
		} catch (ServletException se) {
			throw new FacesException(se);
		}
	}

	public void redirect(String requestURI) throws IOException {
	((HttpServletResponse) response).sendRedirect(requestURI);
	FacesContext.getCurrentInstance().responseComplete();
	}

	public void log(String message) {
		servletContext.log(message);
	}

	public void log(String message, Throwable throwable) {
		servletContext.log(message, throwable);
	}

	public String getAuthType() {
		return ((HttpServletRequest) request).getAuthType();
	}

	public String getRemoteUser() {
		return ((HttpServletRequest) request).getRemoteUser();
	}

	public java.security.Principal getUserPrincipal() {
		return ((HttpServletRequest) request).getUserPrincipal();
	}

	public boolean isUserInRole(String role) {
		return ((HttpServletRequest) request).isUserInRole(role);
	}
	
	//new JSF 1.2 api methods
	 
	public String getRequestContentType() {
		return this.request.getContentType();
	}

	public String getRequestCharacterEncoding() {
		return this.request.getCharacterEncoding();
	}

	public String getResponseCharacterEncoding() {
		return this.response.getCharacterEncoding();
	}
	
	private String ENV_UNSUPPORTED = "This operation is not supported in this Java EE environment, JavaEE5 required.";
	
	/*
	 * <p>This method will only work with JavaEE5, otherwise returns null.</p> 
	 */
	public String getResponseContentType() {
		//if pre JavaEE5 this method will not be available on response object
		try {
			Method m = this.response.getClass().getMethod("getContentType",null);
			return (String)m.invoke(this.response,null);
		} catch (Exception e) {
			//method not available pre JavaEE5, just return null
			throw new UnsupportedOperationException(ENV_UNSUPPORTED);
		} 
	}

	public void setRequest(Object req) {
		if(req instanceof ServletRequest){
			this.request = (ServletRequest)req;
		}
	}

	public void setRequestCharacterEncoding(String encoding) throws UnsupportedEncodingException {
		this.request.setCharacterEncoding(encoding);
	}

	public void setResponse(Object res) {
		if(res instanceof ServletResponse){
			this.response = (ServletResponse)res;
		}
	}
	
	/*
	 * <p>This method will only work with JavaEE5, otherwise returns null.</p> 
	 */
	public void setResponseCharacterEncoding(String encoding) {
		//if pre JavaEE5 this method will not be available on response object
		try {
			Method m = this.response.getClass().getMethod("setCharacterEncoding",new Class[]{String.class});
			m.invoke(this.response,new Object[]{encoding});
		} catch(Exception e){
			throw new UnsupportedOperationException(ENV_UNSUPPORTED);
		}
		
	}
	//end of new JSF 1.2 methods.


	class MultiPartServletRequestWrapper extends HttpServletRequestWrapper {
		private Map parameters;

		public MultiPartServletRequestWrapper(HttpServletRequest request) {
			super(request);
			if (request.getParameterMap().size() > 0) {
				Enumeration names = request.getParameterNames();
				while (names.hasMoreElements()) {
					String name = (String) names.nextElement();
					String[] values = request.getParameterValues(name);
					this.addParameter(name, values);
				}
			}
			String contentType = request.getContentType();
			if(contentType!=null){			
				if (contentType.startsWith(	"multipart/form-data")){
					try{
						MultipartContent mpc = new MultipartContent(request.getInputStream());

						Vector form_content = mpc.getContents(getRequestEncoding(request));
						Enumeration elems=form_content.elements();
						Map fileItemMap = (Map) getRequestMap().get(FILE_ITEM_MAP_PARAM);
						while (elems.hasMoreElements()) {
							ContentElement item = (ContentElement) elems.nextElement();
							String name = item.getName();
							String fname = item.getFileName();							
							if(name == null) {
								throw new FacesException("Error parsing multipart/form-data. Uploaded item has no name.");
							}
							name = name.substring(1, name.length() - 1) ;
							if(fname == null || fname.trim().equals("")) {
								this.addParameter(name, item.getContentValueAsString());
							} else {
								fname = fname.substring(1, fname.length() - 1);							
								if (null == fileItemMap) { 
									fileItemMap = new HashMap(); 
								}
								fileItemMap.put(name, item);
							}	
							if(fileItemMap != null) {
								getRequestMap().put(FILE_ITEM_MAP_PARAM, fileItemMap);
							}
						}
					}
					catch(IOException ioe){
						throw new FacesException("Error parsing multipart/form-data",ioe);	
					}
				}
			}
		}

		/*
		 * all parameters on this parameter map should be String[]
		 */
		private void addParameter(String name, String[] values) {
			
			if (this.parameters == null) {
				parameters = new HashMap();
			}
			
			Object val = this.parameters.get(name);
			if(val == null){
				//not already set, add directly 
				this.parameters.put(name,values);
			} else {
				for (int i = 0, len = values.length; i < len; i++) {
					this.addParameter(name, values[i]);
				}
			}

			
		}

		private void addParameter(String name, String value) {

			if (this.parameters == null) {
				parameters = new HashMap();
			}

			Object objExisting = this.parameters.get(name);
			if (objExisting == null) {
				this.parameters.put(name, new String[]{value});
			} else if (objExisting instanceof String[]) {
				String[] arr = combine((String[]) objExisting, value);
				this.parameters.put(name, arr);
				//should only contain String[], however checking to be safe...
			} else if (objExisting instanceof String) {
				String[] arr = combine((String) objExisting, value);
				this.parameters.put(name, arr);
			}
		}

		private String[] combine(String str1, String str2) {
			String[] ret = new String[2];
			ret[0] = str1;
			ret[1] = str2;
			return ret;
		}

		private String[] combine(String[] arr, String str) {
			String[] ret = new String[arr.length + 1];
			ret[arr.length] = str;
			System.arraycopy(arr, 0, ret, 0, arr.length);
			return ret;
		}

		/**
		 * @see javax.servlet.ServletRequest#getParameter(java.lang.String)
		 * @param arg0
		 * @return
		 */
		public String getParameter(String arg0) {
			if (parameters == null) {
				return null;
			}
			Object obj = parameters.get(arg0);

			if (obj instanceof String[] && ((String[])obj).length > 0) {
				//if multiple values api specifies first should be returned
				return ((String[]) obj)[0];
			}
			//there should not be String values, left in just in case. All values should be String[]
			if (obj instanceof String) {
				return (String) obj;
			}
			return null;
		}

		/**
		 * @see javax.servlet.ServletRequest#getParameterNames()
		 * @return
		 */
		public Enumeration getParameterNames() {
            //TODO - check this; was getting null pointer
			//return Collections.enumeration(parameters.keySet());
            
            if(parameters==null) {
                return new Enumeration() {
                    public Object nextElement() {
                        return null;
                    }
                    public boolean hasMoreElements() {
                        return false;
                    }
                };
            }

            return Collections.enumeration(parameters.keySet());
		}

		/**
		 * @see javax.servlet.ServletRequest#getParameterValues(java.lang.String)
		 * @param arg0
		 * @return
		 */
		public String[] getParameterValues(String arg0) {
			if (null == parameters) {
				return null;
			}
			Object obj = parameters.get(arg0);
			if (obj == null) {
				return null;
			}
			if (obj instanceof String[]) {
				String[] arr = (String[]) obj;
				String[] ret = new String[arr.length];
				System.arraycopy(arr, 0, ret, 0, arr.length);
				return ret;
			}
			//should not be of String type, left in just in case. All values should be String[]
			if (obj instanceof String) {
				String[] ret = new String[1];
				ret[0] = (String) obj;
				return ret;
			}
			return null;
		}

		/**
		 * @see javax.servlet.ServletRequest#getParameterMap()
		 * @return
		 */
		public Map getParameterMap() {
			if(parameters == null){
				return Collections.EMPTY_MAP;
			}
			return Collections.unmodifiableMap(parameters);
		}

		public Object getAttribute(String key) {
			Object result = super.getAttribute(key);
			if (result == null) {
				ServletRequest wrapped = this.getRequest();
				if ((wrapped != null)
					&& (wrapped instanceof ServletRequestWrapper)) {
					wrapped = ((ServletRequestWrapper) wrapped).getRequest();
				}
				if ((wrapped != null)
					&& (wrapped instanceof HttpServletRequestWrapper)) {
					wrapped =
						((HttpServletRequestWrapper) wrapped).getRequest();
				}
				if (wrapped != null) {
					result = wrapped.getAttribute(key);
				}
			}
			return (result);
		}

	}  // End MultiPartServletRequestWrapper class

	private class LocalesIterator implements Iterator {

		public LocalesIterator(Enumeration locales) {
			this.locales = locales;
		}
		private Enumeration locales;

		public boolean hasNext() {
			return locales.hasMoreElements();
		}
		public Object next() {
			return locales.nextElement();
		}
		public void remove() {
			throw new UnsupportedOperationException();
		}

	}
} // End MultipartExternalContextImpl class

abstract class BaseContextMap extends AbstractMap {

	// Unsupported by all Maps.
	public void clear() {
		throw new UnsupportedOperationException();
	}

	// Unsupported by all Maps.
	public void putAll(Map t) {
		throw new UnsupportedOperationException();
	}

	// Supported by maps if overridden
	public Object remove(Object key) {
		throw new UnsupportedOperationException();
	}

	static class Entry implements Map.Entry {
		// immutable Entry
		private final Object key;
		private final Object value;

		Entry(Object key, Object value) {
			this.key = key;
			this.value = value;
		}

		public Object getKey() {
			return key;
		}

		public Object getValue() {
			return value;
		}

		// No support of setting the value
		public Object setValue(Object value) {
			throw new UnsupportedOperationException();
		}

		public int hashCode() {
			return (
				(key == null ? 0 : key.hashCode())
					^ (value == null ? 0 : value.hashCode()));
		}

		public boolean equals(Object obj) {
			if (obj == null || !(obj instanceof Map.Entry))
				return false;

			Map.Entry input = (Map.Entry) obj;
			Object inputKey = input.getKey();
			Object inputValue = input.getValue();

			if (inputKey == key
				|| (inputKey != null && inputKey.equals(key))) {
				if (inputValue == value
					|| (inputValue != null && inputValue.equals(value))) {
					return true;
				}
			}
			return false;
		}
	}
} // End BaseContextMap

class ApplicationMap extends BaseContextMap {
	private ServletContext servletContext = null;

	ApplicationMap(ServletContext servletContext) {
		this.servletContext = servletContext;
	}

	public Object get(Object key) {
		if (key == null) {
			throw new NullPointerException();
		}
		return servletContext.getAttribute(key.toString());
	}

	public Object put(Object key, Object value) {
		if (key == null) {
			throw new NullPointerException();
		}
		String keyString = key.toString();
		Object result = servletContext.getAttribute(keyString);
		servletContext.setAttribute(keyString, value);
		return (result);
	}

	public Object remove(Object key) {
		if (key == null) {
			return null;
		}
		String keyString = key.toString();
		Object result = servletContext.getAttribute(keyString);
		servletContext.removeAttribute(keyString);
		return (result);
	}

	public Set entrySet() {
		Set entries = new HashSet();
		for (Enumeration e = servletContext.getAttributeNames();
			e.hasMoreElements();
			) {
			String key = (String) e.nextElement();
			entries.add(new Entry(key, servletContext.getAttribute(key)));
		}
		return entries;
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof ApplicationMap))
			return false;
		return super.equals(obj);
	}

} // END ApplicationMap


class SessionMap extends BaseContextMap {
	private HttpSession session = null;

	SessionMap(HttpSession session) {
		this.session = session;
	}

	public Object get(Object key) {
		if (key == null) {
			throw new NullPointerException();
		}
		return session.getAttribute(key.toString());
	}

	public Object put(Object key, Object value) {
		if (key == null) {
			throw new NullPointerException();
		}
		String keyString = key.toString();
		Object result = session.getAttribute(keyString);
		session.setAttribute(keyString, value);
		return (result);
	}

	public Object remove(Object key) {
		if (key == null) {
			return null;
		}
		String keyString = key.toString();
		Object result = session.getAttribute(keyString);
		session.removeAttribute(keyString);
		return (result);
	}

	public Set entrySet() {
		Set entries = new HashSet();
		for (Enumeration e = session.getAttributeNames();
			 e.hasMoreElements(); ) {
			String key = (String) e.nextElement();
			entries.add(new Entry(key, session.getAttribute(key)));
		}
		return entries;
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof SessionMap))
			return false;
		return super.equals(obj);
	}
} // END SessionMap


class RequestMap extends BaseContextMap {
	private ServletRequest request = null;

	RequestMap(ServletRequest request) {
		this.request = request;
	}

	public Object get(Object key) {
		if (key == null) {
			throw new NullPointerException();
		}
		return request.getAttribute(key.toString());
	}

	public Object put(Object key, Object value) {
		if (key == null) {
			throw new NullPointerException();
		}
		String keyString = key.toString();
		Object result = request.getAttribute(keyString);
		request.setAttribute(keyString, value);
		return (result);
	}

	public Object remove(Object key) {
		if (key == null) {
			return null;
		}
		String keyString = key.toString();
		Object result = request.getAttribute(keyString);
		request.removeAttribute(keyString);
		return (result);
	}

	public Set entrySet() {
		Set entries = new HashSet();
		for (Enumeration e = request.getAttributeNames();
			 e.hasMoreElements();) {
			String key = (String) e.nextElement();
			entries.add(new Entry(key, request.getAttribute(key)));
		}
		return entries;
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof RequestMap))
			return false;
		return super.equals(obj);
	}
} // END RequestMap

class RequestParameterMap extends BaseContextMap {
	private ServletRequest request = null;

	RequestParameterMap(ServletRequest request) {
		this.request = request;
	}

	public Object get(Object key) {
		if (key == null) {
			throw new NullPointerException();
		}
		return request.getParameter(key.toString());
	}

	public Set entrySet() {
		Set entries = new HashSet();
		for (Enumeration e = request.getParameterNames();
		   e.hasMoreElements(); ) {
			String paramName = (String) e.nextElement();
			entries.add(new Entry(paramName, request.getParameter(paramName)));
		}
		return entries;
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof RequestParameterMap))
			return false;
		return super.equals(obj);
	}
} // END RequestParameterMap

class RequestParameterValuesMap extends BaseContextMap {
	private ServletRequest request = null;

	RequestParameterValuesMap(ServletRequest request) {
		this.request = request;
	}

	public Object get(Object key) {
		if (key == null) {
			throw new NullPointerException();
		}
		return request.getParameterValues(key.toString());
	}

	public Set entrySet() {
		Set entries = new HashSet();
		for (Enumeration e = request.getParameterNames();
			 e.hasMoreElements();) {
			String paramName = (String) e.nextElement();
			entries.add(
				new Entry(paramName, request.getParameterValues(paramName)));
		}
		return entries;
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof RequestParameterValuesMap))
			return false;
		return super.equals(obj);
	}
} // END RequestParameterValuesMap


class RequestHeaderMap extends BaseContextMap {
	private HttpServletRequest request = null;

	RequestHeaderMap(HttpServletRequest request) {
		this.request = request;
	}

	public Object get(Object key) {
		if (key == null) {
			throw new NullPointerException();
		}
		return (request.getHeader(key.toString()));
	}

	public Set entrySet() {
		Set entries = new HashSet();
		for (Enumeration e = request.getHeaderNames();
			 e.hasMoreElements();) {
			String headerName = (String) e.nextElement();
			entries.add(new Entry(headerName, request.getHeader(headerName)));
		}
		return entries;
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof RequestHeaderMap))
			return false;
		return super.equals(obj);
	}
} // END RequestHeaderMap

class RequestHeaderValuesMap extends BaseContextMap {
	private HttpServletRequest request = null;

	RequestHeaderValuesMap(HttpServletRequest request) {
		this.request = request;
	}

	public Object get(Object key) {
		if (key == null) {
			throw new NullPointerException();
		}
		return (request).getHeaders(key.toString());
	}

	public Set entrySet() {
		Set entries = new HashSet();
		for (Enumeration e = request.getHeaderNames();
			 e.hasMoreElements();) {
			String headerName = (String) e.nextElement();
			entries.add(new Entry(headerName, request.getHeaders(headerName)));
		}
		return entries;
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof RequestHeaderValuesMap))
			return false;
		return super.equals(obj);
	}

	// Override of containsValue was necessary as Enumeration.equals(Enumeration)
	// returned false.
	public boolean containsValue(Object value) {
		if (value == null || !(value instanceof Enumeration))
			return false;

		int valHash = 0;
		int valCount = 0;

		// get sum of the hashcode for all elements for the
		// input value.
		Enumeration val = (Enumeration) value;
		while (val.hasMoreElements()) {
			valHash += val.nextElement().hashCode();
			valCount++;
		}

		// For each Map.Entry within this instance, compute
		// the hash for each value and compare against the
		// sum computed above.  Ensure that the number of elements
		// in each enumeration is the same as well.
		for (Iterator i = entrySet().iterator(); i.hasNext(); ) {
			int thisHash = 0;
			int thisCount = 0;
			Map.Entry entry = (Map.Entry) i.next();
			Enumeration thisMap = (Enumeration) entry.getValue();

			while (thisMap.hasMoreElements()) {
				thisHash += thisMap.nextElement().hashCode();
				thisCount++;
			}
			if (thisCount == valCount && thisHash == valHash)
				return true;
		}
		return false;
	}

	// necessary to break the rules somewhat here as it couldn't be
	// guaranteed that the hashCode of the Enumeration would
	// be the same from call to call even if the underlying values contained
	// within are the same.
	public int hashCode() {
		int hashSum = 0;
		for (Iterator i = entrySet().iterator(); i.hasNext();) {
			Map.Entry entry = (Map.Entry) i.next();
			hashSum += entry.getKey().hashCode();
			for (Enumeration e = (Enumeration) entry.getValue();
			   e.hasMoreElements();) {
				hashSum += e.nextElement().hashCode();
			}
		}
		return hashSum;
	}
} // END RequestHeaderValuesMap

class RequestCookieMap extends BaseContextMap {
	private HttpServletRequest request = null;

	RequestCookieMap(HttpServletRequest newRequest) {
		this.request = newRequest;
	}

	public Object get(Object key) {
		if (key == null) {
			throw new NullPointerException();
		}
		Cookie[] cookies = request.getCookies();
		if (null == cookies) {
			return null;
		}

		String keyString = key.toString();
		Object result = null;

		for (int i = 0; i < cookies.length; i++) {
			if (cookies[i].getName().equals(keyString)) {
				result = cookies[i];
				break;
			}
		}
		return result;
	}

	public Set entrySet() {
		Set entries = new HashSet();
		Cookie[] cookies = request.getCookies();

		if (cookies != null) {
			for (int i = 0; i < cookies.length; i++) {
				entries.add(new Entry(cookies[i].getName(), cookies[i]));
			}
		}
		return entries;
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof RequestCookieMap))
			return false;
		return super.equals(obj);
	}
} // END RequestCookiesMap

class InitParameterMap extends BaseContextMap {
	private ServletContext servletContext;

	InitParameterMap(ServletContext newServletContext) {
		servletContext = newServletContext;
	}

	public Object get(Object key) {
		if (key == null) {
			throw new NullPointerException();
		}
		String keyString = key.toString();
		return servletContext.getInitParameter(keyString);
	}

	public Set entrySet() {
		Set entries = new HashSet();

		for (Enumeration e = servletContext.getInitParameterNames();
			 e.hasMoreElements(); ) {
			String initParamName = (String) e.nextElement();
			entries.add(new Entry(initParamName,
				servletContext.getInitParameter(initParamName)));
		}
		return entries;
	}

	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof InitParameterMap))
			return false;
		return super.equals(obj);
	}
} // END InitParameterMap


class MyServletRequestWrapper extends ServletRequestWrapper {

	public MyServletRequestWrapper(ServletRequest request) {
		super(request);
	}

	public Object getAttribute(String key) {
		Object result = super.getAttribute(key);
		if (result == null) {
			ServletRequest wrapped = getRequest();
			if ((wrapped != null) &&
				(wrapped instanceof ServletRequestWrapper)) {
				wrapped = ((ServletRequestWrapper) wrapped).getRequest();
			}
			if (wrapped != null) {
				result = wrapped.getAttribute(key);
			}
		}
		return (result);
	}

}


class MyHttpServletRequestWrapper extends HttpServletRequestWrapper {

	public MyHttpServletRequestWrapper(HttpServletRequest request) {
		super(request);
	}

	public Object getAttribute(String key) {
		Object result = super.getAttribute(key);
		if (result == null) {
			ServletRequest wrapped = getRequest();
			if ((wrapped != null) &&
				(wrapped instanceof ServletRequestWrapper)) {
				wrapped = ((ServletRequestWrapper) wrapped).getRequest();
			}
			if (wrapped != null) {
				result = wrapped.getAttribute(key);
			}
		}
		return (result);
	}

}
