/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/
package com.ibm.faces.context;


import java.util.ArrayList;

import javax.faces.FacesException;
import javax.faces.context.FacesContext;
import javax.faces.context.FacesContextFactory;
import javax.faces.lifecycle.Lifecycle;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import com.ibm.faces.util.ContextParams;

public class MultipartFacesContextFactoryImpl extends FacesContextFactory {

	
	protected static ArrayList FACES_CONTEXT_POOL = new ArrayList();
	protected static int FACES_CONTEXT_POOL_SIZE = -1;
	
	
	private FacesContextFactory superFactory = null;
	private Boolean multipartDisabled = null;
	//
	// Constructors and Initializers    
	//

	public MultipartFacesContextFactoryImpl() {
		super();
	}
	
	public MultipartFacesContextFactoryImpl(FacesContextFactory factory){
		super();
		this.superFactory = factory;
	}

	//
	// Methods from FacesContextFactory
	//
	public FacesContext getFacesContext(
		Object sc,
		Object request,
		Object response,
		Lifecycle lifecycle)
		throws FacesException {
		
		if(sc == null || request == null || response == null || lifecycle == null) {
			throw new NullPointerException();
		}
		
		if(multipartDisabled == null){
			//get com.ibm.faces.DISABLE_JWL_MULTIPART_CONTEXT context param value
			if("true".equalsIgnoreCase(((ServletContext)sc).getInitParameter(ContextParams.DISABLE_JWL_MULTIPART_CONTEXT))){
				multipartDisabled = Boolean.TRUE; 
			} else {
				multipartDisabled = Boolean.FALSE;
			}
		}
		
		if(//jwl multipart context disabled by context param
			(this.superFactory != null && multipartDisabled.booleanValue())
			||
			//content type is not multipart/form-data
			(this.superFactory != null && (((ServletRequest)request).getContentType() == null || !((ServletRequest)request).getContentType().startsWith("multipart/form-data")))	
		){
			//get facescontext from super factory
			return this.superFactory.getFacesContext(sc,request,response,lifecycle);
		} 
			
		MultipartExternalContextImpl ec = new MultipartExternalContextImpl((ServletContext) sc,(ServletRequest) request,(ServletResponse) response);
		
			if(FACES_CONTEXT_POOL_SIZE == -1){
				String poolSize = ec.getInitParameter(ContextParams.FACES_CONTEXT_POOL_SIZE);
				if(poolSize != null){
					FACES_CONTEXT_POOL_SIZE = Integer.parseInt(poolSize.trim());
					if(FACES_CONTEXT_POOL_SIZE < 1){
						FACES_CONTEXT_POOL_SIZE = 1;
					}
				} else {
					//default to 20
					FACES_CONTEXT_POOL_SIZE = 20;
				}
			}
			/*
			 * To support new JSF 1.2 (requires JavaEE5) getELContext(), if underlying
			 * FacesContextFactory is available then creating the FacesContext with that and
			 * storing it and ExternalContext for 1.2 getELContext calls (API impl method checks these params).
			 * Small overhead as this section of code reached only when this page contains fileupload or form is
			 * multipart. 
			 * CODESCAN NOTE : these referals com.sun.faces are not a problem.
			 */
			if(this.superFactory != null){
				FacesContext superContext = this.superFactory.getFacesContext(sc,request,response,lifecycle);
				ec.getRequestMap().put("com.sun.faces.FacesContextImpl",superContext);
				ec.getRequestMap().put("com.sun.faces.ExternalContextImpl",superContext.getExternalContext());
			}
			//end of JSF1.2 support
			
			if(FACES_CONTEXT_POOL.isEmpty()){
				return new FacesContextImpl(ec);
			} 
				
			FacesContextImpl context = (FacesContextImpl)FACES_CONTEXT_POOL.remove(FACES_CONTEXT_POOL.size() - 1);
			context.initialize(ec);
			return context;
	}

} // end of class MultipartFacesContextFactoryImpl
