/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.context;

import java.io.IOException;
import java.io.Writer;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * NullResponseWriter is a ResponseWriter which will caused nothing to be written to the underlying
 * output mechanism for charcter-based output.
 * 
 * @author mark_wallace@ie.ibm.com
 */
public class NullResponseWriter extends ResponseWriter {
	
	private FacesContext _context;
	
	public NullResponseWriter(FacesContext context) {
		_context = context;
	}
	
	private ResponseWriter getResponseWriter() {
		return _context.getResponseWriter();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#getContentType()
	 */
	public String getContentType() {
		return getResponseWriter().getContentType();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#getCharacterEncoding()
	 */
	public String getCharacterEncoding() {
		return getResponseWriter().getCharacterEncoding();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#flush()
	 */
	public void flush() throws IOException {
		getResponseWriter().flush();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#startDocument()
	 */
	public void startDocument() throws IOException {
		getResponseWriter().startDocument();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#endDocument()
	 */
	public void endDocument() throws IOException {
		getResponseWriter().endDocument();
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#startElement(java.lang.String, javax.faces.component.UIComponent)
	 */
	public void startElement(String name, UIComponent component) throws IOException {
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#endElement(java.lang.String)
	 */
	public void endElement(String name) throws IOException {
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#writeAttribute(java.lang.String, java.lang.Object, java.lang.String)
	 */
	public void writeAttribute(String name, Object value, String property) throws IOException {
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#writeURIAttribute(java.lang.String, java.lang.Object, java.lang.String)
	 */
	public void writeURIAttribute(String name, Object value, String property) throws IOException {
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#writeComment(java.lang.Object)
	 */
	public void writeComment(Object comment) throws IOException {
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#writeText(java.lang.Object, java.lang.String)
	 */
	public void writeText(Object text, String property) throws IOException {
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#writeText(char[], int, int)
	 */
	public void writeText(char[] text, int off, int len) throws IOException {
	}

	/* (non-Javadoc)
	 * @see javax.faces.context.ResponseWriter#cloneWithWriter(java.io.Writer)
	 */
	public ResponseWriter cloneWithWriter(Writer writer) {
		return getResponseWriter().cloneWithWriter(writer);
	}

	/* (non-Javadoc)
	 * @see java.io.Writer#close()
	 */
	public void close() throws IOException {
		getResponseWriter().close();
	}

	/* (non-Javadoc)
	 * @see java.io.Writer#write(char[], int, int)
	 */
	public void write(char[] cbuff, int off, int len) throws IOException {
	}

}
