/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2004 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.converter;

import java.util.Locale;
import java.util.TimeZone;

import javax.faces.context.FacesContext;
import javax.faces.convert.DateTimeConverter;

/**
 * DateTimeConverterEx
 * 
 * @author doconnor
 * @Version 0.1
 * 
 * Feb 27, 2004 - 1:17:05 PM
 */
public class DateTimeConverterEx extends DateTimeConverter {
	
	private boolean dateStyleSet 	= false;
	private boolean timeStyleSet 	= false;
	private boolean timeZoneSet 	= false;
	private boolean localeSet 		= false;
	private boolean patternSet 		= false;
	private boolean typeSet			= false;

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#getDateStyle()
	 */
	public String getDateStyle() {		
		return super.getDateStyle();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#getLocale()
	 */
	public Locale getLocale() {		
		if(isLocaleSet()){
			return super.getLocale();
		}
		return getLocale(FacesContext.getCurrentInstance());
	}
	
	/*
	 * Fix for api problem where locale is set for the life of the converter, this is a problem for dynamically 
	 * changing locales (e.g. portal, locale can be changed by a drop down selection).
	 * With this locale will be calculated each time with getViewRoot().getLocale() if locale attribute is not set.
	 */
	private Locale getLocale(FacesContext context){
		if(isLocaleSet()){
			return super.getLocale();
		}
		return context.getViewRoot().getLocale();
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#getPattern()
	 */
	public String getPattern() {		
		return super.getPattern();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#getTimeStyle()
	 */
	public String getTimeStyle() {		
		return super.getTimeStyle();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#getTimeZone()
	 */
	public TimeZone getTimeZone() {		
		return super.getTimeZone();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#getType()
	 */
	public String getType() {		
		return super.getType();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setDateStyle(java.lang.String)
	 */
	public void setDateStyle(String dateStyle) {		
		super.setDateStyle(dateStyle);
		this.setDateStyleSet(dateStyle != null && dateStyle.length() > 0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setLocale(java.util.Locale)
	 */
	public void setLocale(Locale locale) {	
		if(locale != null){
			super.setLocale(locale);
			this.setLocaleSet(true);	
		}
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setPattern(java.lang.String)
	 */
	public void setPattern(String pattern) {
		super.setPattern(pattern);
		this.setPatternSet(pattern != null && pattern.length() > 0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setTimeStyle(java.lang.String)
	 */
	public void setTimeStyle(String timeStyle) {		
		super.setTimeStyle(timeStyle);
		this.setTimeStyleSet(timeStyle != null && timeStyle.length() > 0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setTimeZone(java.util.TimeZone)
	 */
	public void setTimeZone(TimeZone timeZone) {		
		super.setTimeZone(timeZone);
		this.setTimeZoneSet(true);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setType(java.lang.String)
	 */
	public void setType(String type) {		
		super.setType(type);
		this.setTypeSet(type != null && type.length() > 0);
	}

	/**
	 * @return
	 */
	public boolean isDateStyleSet() {
		return dateStyleSet;
	}

	/**
	 * @return
	 */
	public boolean isLocaleSet() {
		return localeSet;
	}

	/**
	 * @return
	 */
	public boolean isPatternSet() {
		return patternSet;
	}

	/**
	 * @return
	 */
	public boolean isTimeStyleSet() {
		return timeStyleSet;
	}

	/**
	 * @return
	 */
	public boolean isTimeZoneSet() {
		return timeZoneSet;
	}

	/**
	 * @param b
	 */
	public void setDateStyleSet(boolean b) {
		dateStyleSet = b;
	}

	/**
	 * @param b
	 */
	public void setLocaleSet(boolean b) {
		localeSet = b;
	}

	/**
	 * @param b
	 */
	public void setPatternSet(boolean b) {
		patternSet = b;
	}

	/**
	 * @param b
	 */
	public void setTimeStyleSet(boolean b) {
		timeStyleSet = b;
	}

	/**
	 * @param b
	 */
	public void setTimeZoneSet(boolean b) {
		timeZoneSet = b;
	}

	/**
	 * @return
	 */
	public boolean isTypeSet() {
		return typeSet;
	}

	/**
	 * @param b
	 */
	public void setTypeSet(boolean b) {
		typeSet = b;
	}

	
	 /*JSF state saving/restoration methods*/
	 
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#restoreState(javax.faces.context.FacesContext, java.lang.Object)
	 */
	public void restoreState(FacesContext context, Object state) {
        Object values[] = (Object[]) state;
        super.restoreState(context, values[0]);
        
    	dateStyleSet 	= ((Boolean)values[1]).booleanValue();
    	timeStyleSet 	= ((Boolean)values[2]).booleanValue();
    	timeZoneSet 	= ((Boolean)values[3]).booleanValue();
    	localeSet 		= ((Boolean)values[4]).booleanValue();
    	patternSet 		= ((Boolean)values[5]).booleanValue();
    	typeSet 		= ((Boolean)values[6]).booleanValue(); 
        
	}
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#saveState(javax.faces.context.FacesContext)
	 */
	public Object saveState(FacesContext context) {
        Object values[] = new Object[7];
        values[0] = super.saveState(context);
        
        values[1] =  Boolean.valueOf(dateStyleSet);
        values[2] =  Boolean.valueOf(timeStyleSet);
        values[3] =  Boolean.valueOf(timeZoneSet);
        values[4] =  Boolean.valueOf(localeSet);
        values[5] =  Boolean.valueOf(patternSet);
        values[6] =  Boolean.valueOf(typeSet);
                
        return (values);
	}
	 		
}
