/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2004 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.converter;

import java.util.Locale;
import com.ibm.faces.util.ICU4JUtil;
import com.ibm.icu.text.DateFormat;
import com.ibm.icu.text.SimpleDateFormat;
import com.ibm.icu.util.TimeZone;
import com.ibm.icu.util.Calendar;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.ConverterException;
import javax.faces.convert.DateTimeConverter;


public class DateTimeConverterExICU extends DateTimeConverter /*implements Converter*/ {
	
	private boolean dateStyleSet 	= false;
	private boolean timeStyleSet 	= false;
	private boolean timeZoneSet 	= false;
	private boolean localeSet 		= false;
	private boolean patternSet 		= false;
	private boolean typeSet			= false;
	private boolean calendarSet 	= false;
	
	//these need to be saved when client side state saving is enabled
	//so should be covered in the saveState and restoreState methods
	private Calendar calendar	= null;
	private TimeZone timeZoneEx	= null;
	

	/**
	 * The converter id for this converter.
	 */
	public static final String CONVERTER_ID = "com.ibm.faces.DateTime"; //$NON-NLS-1$
	
	/* (non-Javadoc)
	 * @see com.ibm.faces.DateTimeConverterEx#getTimeZoneEx()
	 */
	public TimeZone getTimeZoneEx() 
	{		
		return timeZoneEx;
	}

	/* (non-Javadoc)
	 * @see com.ibm.faces.convert.DateTimeConverterEx#getCalendar()
	 * calendar string should be one of gregorian|islamic|hebrew|japanese|buddhist	(default: gregorian)
	 */
	public Calendar getCalendar() 
	{		
		return this.calendar;
	}	
		
	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setDateStyle(java.lang.String)
	 */
	public void setDateStyle(String dateStyle) 
	{		
		super.setDateStyle(dateStyle);
		this.setDateStyleSet(dateStyle != null && dateStyle.length() > 0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setLocale(java.util.Locale)
	 */
	public void setLocale(Locale locale) 
	{		
		super.setLocale(locale);
		this.setLocaleSet(locale != null);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setPattern(java.lang.String)
	 */
	public void setPattern(String pattern) 
	{
		super.setPattern(pattern);
		this.setPatternSet(pattern != null && pattern.length() > 0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setTimeStyle(java.lang.String)
	 */
	public void setTimeStyle(String timeStyle) 
	{		
		super.setTimeStyle(timeStyle);
		this.setTimeStyleSet(timeStyle != null && timeStyle.length() > 0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setTimeZoneEx(com.ibm.icu.util.TimeZone)
	 */
	public void setTimeZoneEx(TimeZone timeZone) 
	{		
		this.timeZoneEx = timeZone;
		this.setTimeZoneSet(timeZone != null);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.DateTimeConverter#setType(java.lang.String)
	 */
	public void setType(String type) 
	{		
		super.setType(type);
		this.setTypeSet(type != null && type.length() > 0);
	}

	
	/* (non-Javadoc)
	 * @see com.ibm.faces.convert.DateTimeConverterEx#setCalendar(com.ibm.icu.util.Calendar)
	 * calendar must be one of gregorian|islamic|hebrew|japanese|buddhist	(default: gregorian) 
	 * @see com.ibm.icu.util.Calendar
	 */
	public void setCalendar(Calendar calendar) 
	{		
		this.calendar = calendar;
		this.setCalendarSet(calendar != null);
	}	
	
	/**
	 * @return
	 */
	public boolean isDateStyleSet() 
	{
		return dateStyleSet;
	}

	/**
	 * @return
	 */
	public boolean isLocaleSet() 
	{
		return localeSet;
	}

	/**
	 * @return
	 */
	public boolean isPatternSet() 
	{
		return patternSet;
	}

	/**
	 * @return
	 */
	public boolean isTimeStyleSet() 
	{
		return timeStyleSet;
	}

	/**
	 * @return
	 */
	public boolean isTimeZoneSet() 
	{
		return timeZoneSet;
	}

	/**
	 * @param b
	 */
	public void setDateStyleSet(boolean b) 
	{
		dateStyleSet = b;
	}

	/**
	 * @param b
	 */
	public void setLocaleSet(boolean b) 
	{
		localeSet = b;
	}

	/**
	 * @param b
	 */
	public void setPatternSet(boolean b) 
	{
		patternSet = b;
	}

	/**
	 * @param b
	 */
	public void setTimeStyleSet(boolean b) 
	{
		timeStyleSet = b;
	}

	/**
	 * @param b
	 */
	public void setTimeZoneSet(boolean b) 
	{
		timeZoneSet = b;
	}

	/**
	 * @return
	 */
	public boolean isTypeSet() 
	{
		return typeSet;
	}

	/**
	 * @param b
	 */
	public void setTypeSet(boolean b) 
	{
		typeSet = b;
	}

	/**
	 * @return
	 */
	public boolean isCalendarSet() 
	{
		return calendarSet;
	}

	/**
	 * @param b
	 */
	public void setCalendarSet(boolean b) 
	{
		calendarSet = b;
	}
	

	/** (non-Javadoc)
	 * @see javax.faces.convert.Converter#getAsObject(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.String)
	 * 	
	 * Passed in some string with date and/or time and should return a date object
	 */
	public Object getAsObject(FacesContext context,	UIComponent component, String value){
		if (value == null) {
			return null;
		} 
		
		value = value.trim();
		if (value.length() < 1) {
			return null;
		}	
	
		try{
			return getDateFormat(context).parse(value);
		} catch(Exception e){
			throw new ConverterException(e);
		}
	}
	

	/**(non-Javadoc)
	 * @see javax.faces.convert.Converter#getAsString(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.Object)
	 * 	 
	 * Passed in a date object and should return a string with date and/or time	 
	 */	
	 public String getAsString(FacesContext context, UIComponent component, Object value) 
	 { 	
		if (value == null) {
			return ""; //$NON-NLS-1$
		}
	
		if(value instanceof String){
			return (String)value;
		}

		try{
			
			return getDateFormat(context).format(value);
			
		} catch(Exception e){
			throw new ConverterException(e);
		}
	 }
	 
	 private DateFormat getDateFormat(FacesContext context){

	 	DateFormat df = null;
	 	
	 	Locale locale = setupLocale(context);
	 	
	 	if(isPatternSet() && !isCalendarSet()) 
	 	{ // if you only set a pattern and NOT a calendar
	 			df = new SimpleDateFormat(getPattern(), locale);
	 	}
	 	else if(isCalendarSet())
	 	{ //if you set a calendar
	 		
			if(isTimeZoneSet()){
				this.calendar.setTimeZone(timeZoneEx);
			}
					 
			if (getType().equalsIgnoreCase("both"))
			{ //$NON-NLS-1$
				df = DateFormat.getDateTimeInstance(this.calendar, ICU4JUtil.getDateStyle(getDateStyle()), ICU4JUtil.getDateStyle(getTimeStyle()), locale);				
			} else if (getType().equalsIgnoreCase("date")) 
			{ //$NON-NLS-1$
				df = DateFormat.getDateInstance(this.calendar, ICU4JUtil.getDateStyle(getDateStyle()), locale);				
			} else if (getType().equalsIgnoreCase("time"))
			{ //$NON-NLS-1$
				df = DateFormat.getTimeInstance(this.calendar, ICU4JUtil.getDateStyle(getTimeStyle()), locale);				
			} else 
			{
				df = DateFormat.getDateInstance(this.calendar, ICU4JUtil.getDateStyle(getDateStyle()), locale);			
			}
	 		/**/
			if(df != null) // if you set a pattern as well, apply it
				if(isPatternSet())
					//((SimpleDateFormat)df).applyLocalizedPattern(getPattern());
					((SimpleDateFormat)df).applyPattern(getPattern());
			/**/
	 	} 
	 	return df;
	}
	 	
	
	 
	 private Locale setupLocale(FacesContext context){
	 	if(!isLocaleSet())
	 	{	 		
 			Locale l = context.getViewRoot().getLocale();
 			if(l != null)
 				return l; 			
	 	}
	 	return getLocale();
	 }
	 
	 //client side state methods
	 public Object saveState(FacesContext context) {
	 	Object values[] = new Object[7];
	 	values[0] = this.calendar;
	 	values[1] = this.timeZoneEx;
	 	values[2] = getLocale();
	 	values[3] = getDateStyle();
	 	values[4] = getTimeStyle();
	 	values[5] = getPattern();
	 	values[6] = getType();
	 	return (values);
	 }
	 
	 public void restoreState(FacesContext context, Object state) {
	 	Object values[] = (Object[]) state;
	 	setCalendar((Calendar)values[0]);
	 	setTimeZoneEx((TimeZone)values[1]);
	 	setLocale((Locale)values[2]);
	 	setDateStyle((String)values[3]);
	 	setTimeStyle((String)values[4]);
	 	setPattern((String)values[5]);
	 	setType((String)values[6]);
	 }
}
