/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2004 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.converter;

import java.lang.reflect.Constructor;
import java.text.MessageFormat;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.component.StateHolder;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;
import javax.faces.el.ValueBinding;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.util.InputAssistNames;

public class MaskConverter implements Converter, StateHolder {

	private String mask;
	private boolean asis;
	private String protectMask;
	private String char1RegEx;
	private String char2RegEx;
	private String char3RegEx;
	private String char4RegEx;
	private String sourceObjType;
	
	public String getChar1RegEx() {
		return char1RegEx;
	}
	public void setChar1RegEx(String char1RegEx) {
		this.char1RegEx = char1RegEx;
	}
	public String getChar2RegEx() {
		return char2RegEx;
	}
	public void setChar2RegEx(String char2RegEx) {
		this.char2RegEx = char2RegEx;
	}
	public String getChar3RegEx() {
		return char3RegEx;
	}
	public void setChar3RegEx(String char3RegEx) {
		this.char3RegEx = char3RegEx;
	}
	public String getChar4RegEx() {
		return char4RegEx;
	}
	public void setChar4RegEx(String char4RegEx) {
		this.char4RegEx = char4RegEx;
	}

	public String getProtectMask() {
		return protectMask;
	}
	public void setProtectMask(String protectMask) {
		this.protectMask = protectMask;
	}
	public MaskConverter() {
	}

	/**
	 * The converter id for this converter.
	 */
	public static final String CONVERTER_ID = "com.ibm.faces.Mask"; //$NON-NLS-1$

	/* (non-Javadoc)
	 * @see javax.faces.convert.Converter#getAsObject(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.String)
	 */
	public Object getAsObject(FacesContext context, UIComponent component, String value) {
		ResourceBundle rb = ResourceHandler.getBundle(context);
		
		if (context == null || component == null) {
			throw new NullPointerException(ResourceHandler.getString(rb, "ConstraintValidator.FacesContext_or_UIComponent_is_null._5")); //$NON-NLS-1$
		}

		if (value == null || value.length() < 1) {
			return null;
		}

		try {
			if (asis) {
				// Validate with mask pattern and return the String value. No modification.
				getStrippedValue(value, mask, rb);
				return value;
			} 

			//Remove the non data string. Type change to source object type.
			String modValue = getStrippedValue(value, mask, rb);
			if (sourceObjType == null) {
				ValueBinding valuebinding = component.getValueBinding("value"); //$NON-NLS-1$
				if (valuebinding != null) {
					String type = null;
					Class tClass = valuebinding.getType(context);
					if(tClass != null)
						type = tClass.getName();
					if (type != null) {
						sourceObjType = type;
					} else {
						MessageFormat format = new MessageFormat(ResourceHandler.getString(rb, "MaskConverter.Failed_to_convert___{0}___to_the_object_due_to_unknown_data_type._1")); //$NON-NLS-1$
						Object[] obj = {value};
						throw new ConverterException(format.format(obj));
						//return ""; //$NON-NLS-1$
					}
				} else {
					// If not having VBL expression, return type of String.
					String returnValue = getStrippedValue(value, mask, rb);
					return returnValue;
				}
			}
			return convertToOrignType(sourceObjType, modValue, rb);
			
		} catch (Exception e) {
			throw new ConverterException(e);
		}
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.Converter#getAsString(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.Object)
	 */
	public String getAsString(FacesContext context, UIComponent component, Object value) {
		ResourceBundle rb = ResourceHandler.getBundle(context);

		if (context == null || component == null) {
			throw new NullPointerException(ResourceHandler.getString(rb, "ConstraintValidator.FacesContext_or_UIComponent_is_null._5")); //$NON-NLS-1$
		}

		if (value == null) {
			return ""; //$NON-NLS-1$
		}

		String sourceString;
		try {
			if (value instanceof String) {
				sourceString = (String) value;
			} else if(value instanceof Number) {
				sourceString = value.toString();
				int nDigits = getNumberOfDigitsInMask();
				if(sourceString.length() < nDigits) {
					StringBuffer str = new StringBuffer();
					for(int i = 0; i < nDigits - sourceString.length(); i++) {
						str.append("0"); //$NON-NLS-1$
					}
					str.append(sourceString);
					sourceString = str.toString();
				}
			} else {
				sourceString = value.toString();
			}
			sourceObjType = value.getClass().getName();
			return getMaskedString(mask, sourceString, rb, context, component);
		} catch (Exception e) {
			throw new ConverterException(e);
		}
	}

	private String getStrippedValue(String data, String mask, ResourceBundle rb){
		
		//is the value matches the maskchars return the entered value, otherwise call removeNonDataCharacter
		char[] dataArray = data.toCharArray();
		char[] maskArray = mask.toCharArray();
		StringBuffer maskChars = new StringBuffer();
		StringBuffer dataChars = new StringBuffer();
		int count = 0;
		
		for(int i = 0; i < maskArray.length; i++){
			String c = String.valueOf(maskArray[i]);
			
				
			if (c.equals(InputAssistNames.MASK_ALPHABET_PLACEHOLDER) || 
					c.equals(InputAssistNames.MASK_DIGIT_PLACEHOLDER) ||
					(c.equals(InputAssistNames.MASK_CUSTOM_EXCLAMATION_PLACEHOLDER) && getChar3RegEx() != null && getChar3RegEx().length() > 0) ||
					(c.equals(InputAssistNames.MASK_CUSTOM_TILDA_PLACEHOLDER) && getChar4RegEx() != null && getChar4RegEx().length() > 0)){
		
					maskChars.append(c);
					if(count < dataArray.length){
						if(isValidType(c,dataArray[count])){
							dataChars.append(dataArray[count]);
						} else {
							//not valid
							break;
						}
					} else {
						if(isValidType(c,' ')){
							//add spaces
							dataChars.append(' ');
						} else {
							throw new ConverterException(ResourceHandler.getString(rb, "MaskConverter.Data_String_and_mask_pattern_are_mismatched._1")); //$NON-NLS-1$
						}
					}
					++count;
			}
		}
		
		if(maskChars.length() == dataChars.length()){
			return dataChars.toString();
		} 
		
		return removeNonDataCharacter(data,mask,rb);
	}
	
	private String removeNonDataCharacter(String data, String mask, ResourceBundle rb) {
		boolean b = true;
		char[] dataArray = data.toCharArray();
		char[] maskArray = mask.toCharArray();
		StringBuffer result = new StringBuffer();
		Boolean fill = Boolean.FALSE;
		//if (dataArray.length == maskArray.length) {
			for (int i = 0; i < maskArray.length; i++) {
				if(!fill.booleanValue()){
					//check if value needs to be filled with spaces
					if(i >= dataArray.length){
						fill = Boolean.TRUE;
					}
				}
				String temp = String.valueOf(maskArray[i]);
				if (temp.equals(InputAssistNames.MASK_ALPHABET_PLACEHOLDER) || 
						temp.equals(InputAssistNames.MASK_DIGIT_PLACEHOLDER) ||
						(temp.equals(InputAssistNames.MASK_CUSTOM_EXCLAMATION_PLACEHOLDER) && getChar3RegEx() != null && getChar3RegEx().length() > 0) ||
						(temp.equals(InputAssistNames.MASK_CUSTOM_TILDA_PLACEHOLDER) && getChar4RegEx() != null && getChar4RegEx().length() > 0)
						/*||
						temp.equals(InputAssistNames.MASK_DIGIT_OR_SPACE_PLACEHOLDER)*/) {
					boolean valid;
					
					if(fill.booleanValue()){
						valid = isValidType(temp,' ');
					} else {
						valid = isValidType(temp, dataArray[i]);
					}
					if (!valid) {
						b = false;
						break;
					}
//					if(temp.equals(InputAssistNames.MASK_DIGIT_OR_SPACE_PLACEHOLDER) && dataArray[i] == ' ') { //$NON-NLS-1$
//						result.append("0"); //$NON-NLS-1$
//					}
					
					if(fill.booleanValue()){
						result.append(' ');
					} else {
						result.append(dataArray[i]);	
					}
					
				} else {
					if(!fill.booleanValue()){
						if (!(maskArray[i] == dataArray[i])) {
							b = false;
							break;
						}
					}
				}
			}
			/*
		} else {
			b = false;
		}
		*/

		if (b) {
			return result.toString();
		} 
		throw new ConverterException(ResourceHandler.getString(rb, "MaskConverter.Data_String_and_mask_pattern_are_mismatched._1")); //$NON-NLS-1$
		//return data;
	}
	
	protected Object convertToOrignType(String type, String value, ResourceBundle rb) {
		Class[] classes = { String.class };
		String[] values = { value };
		try {
			Constructor construtor = Class.forName(type).getConstructor(classes);
			Object obj = construtor.newInstance(values);
			return obj;
		} catch (Exception e) {
			throw new ConverterException(ResourceHandler.getString(rb, "MaskConverter.Error__Fail_to_covert_original_Object_type._2")); //$NON-NLS-1$
			//return value;
		}
	}

	private String getMaskedString(String mask, String source, ResourceBundle rb, FacesContext context, UIComponent component) {
		StringBuffer tmp = new StringBuffer();
		char[] maskChars = mask.toCharArray();
		char[] sourceChars = source.toCharArray();
		int sourceCharIndex = 0;
		int sourceLength = sourceChars.length;
		int maskLength = maskChars.length;
		for (int i = 0; i < maskLength; i++) {
			String temp = String.valueOf(maskChars[i]);
			if (temp.equals(InputAssistNames.MASK_ALPHABET_PLACEHOLDER) || 
					temp.equals(InputAssistNames.MASK_DIGIT_PLACEHOLDER) || 
					(temp.equals(InputAssistNames.MASK_CUSTOM_EXCLAMATION_PLACEHOLDER) && getChar3RegEx() != null && getChar3RegEx().length() > 0) ||
					(temp.equals(InputAssistNames.MASK_CUSTOM_TILDA_PLACEHOLDER) && getChar4RegEx() != null && getChar4RegEx().length() > 0)
					/*||
					temp.equals(InputAssistNames.MASK_DIGIT_OR_SPACE_PLACEHOLDER)*/) {
				if (sourceLength > sourceCharIndex) {
					//check all chars
					//if (Character.isLetterOrDigit(sourceChars[sourceCharIndex])) {
						boolean b = isValidType(temp, sourceChars[sourceCharIndex]);
						if (b) {
//							if(temp.equals(InputAssistNames.MASK_DIGIT_OR_SPACE_PLACEHOLDER) && sourceChars[sourceCharIndex] == '0') { //$NON-NLS-1$
//								if(tmp.length() == 0 || !Character.isDigit(tmp.charAt(tmp.length() - 1))) {
//									tmp.append(' '); //$NON-NLS-1$
//								}
//								else {
//									tmp.append(sourceChars[sourceCharIndex]);
//								}
//							}
//							else {
								tmp.append(sourceChars[sourceCharIndex]);
//							}
							sourceCharIndex++;
						} else {
								FacesMessage errMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, 
										ResourceHandler.getString(rb, "MaskConverter.Conversion_Error__Source_string_is_unmatched_with_mask_pattern._3"), //$NON-NLS-1$
										ResourceHandler.getString(rb, "MaskConverter.Conversion_Error__Source_string_is_unmatched_with_mask_pattern._3")); //$NON-NLS-1$
							context.addMessage(component.getClientId(context), errMsg);
							//throw new ConverterException(ResourceHandler.getString(rb, "MaskConverter.Conversion_Error__Source_string_is_unmatched_with_mask_pattern._3")); //$NON-NLS-1$
							return source;
						}
						/* 
					} else if (Character.isSpaceChar(sourceChars[sourceCharIndex])) {
						return source;
						
					} else {
							FacesMessage errMsg = new FacesMessage(FacesMessage.SEVERITY_ERROR, 
									ResourceHandler.getString(rb, "MaskConverter.Conversion_Error__Source_string_is_unmatched_with_mask_pattern._4"), //$NON-NLS-1$
									ResourceHandler.getString(rb, "MaskConverter.Conversion_Error__Source_string_is_unmatched_with_mask_pattern._4")); //$NON-NLS-1$
						context.addMessage(component.getClientId(context), errMsg);
						//throw new ConverterException(ResourceHandler.getString(rb, "MaskConverter.Conversion_Error__Source_string_is_unmatched_with_mask_pattern._4")); //$NON-NLS-1$
						return source;
					}
					*/
				} else {
					return source;
				}
			} else {
				/*
				if (sourceLength > sourceCharIndex) {
					if (Character.isSpaceChar(sourceChars[sourceCharIndex])) {
						tmp.append(maskChars[i]);
						//sourceCharIndex++;
					} else if (Character.isLetterOrDigit(sourceChars[sourceCharIndex])) {
						tmp.append(maskChars[i]);
					} else {
						tmp.append(maskChars[i]);
						sourceCharIndex++;
					}
				} else {
					tmp.append(maskChars[i]);
				}
				*/
				
				//now that all chars can be allow with custom regEx...
				tmp.append(maskChars[i]);
			}
		}
		return tmp.toString();
	}
	//RegEx key chars
	private char[] regExChars = {'-','\\'};
	private boolean isValidForRegEx(String regex,char targetChar){
		
		if(regex.equals(".")){
			return true;
		}
		
		//inside brackets
		if(regex.trim().startsWith("[") && regex.trim().endsWith("]") ){
			
			Boolean negated = Boolean.FALSE; 
			//is bracket contents negated
			if(regex.trim().startsWith("[^")){
				negated = Boolean.TRUE;
			}
			
			//check regEx keychars have been escaped
			for(int x = 0; x < regExChars.length; x++){
				if(targetChar == regExChars[x]){
					int index = 0;
					while(index > -1){
						index = regex.indexOf(targetChar,index + 1);
						if(index > -1 && regex.charAt(index - 1) == '\\'){
							//found the char to be escaped
							if(negated.booleanValue()){
								return false;
							} 
							return true;
						}
					}
					//key character not escaped
					if(negated.booleanValue()){
						return true;
					} 
					return false;
					
				}
			}
			
			//check for literal char first
			if(regex.indexOf(targetChar) > -1){
				if(negated.booleanValue()){
					return false;
				} 
				return true;	
			} 
			//check char is in regex ranges
			int sep = -1;
			sep = regex.indexOf("-");
			while(sep > -1){
				if(targetChar >= regex.charAt(sep-1) && targetChar <= regex.charAt(sep+1)){
					if(negated.booleanValue()){
						return false;
					} 
					return true;	
				}
				sep = regex.indexOf("-",sep+1);
				if(sep > -1 && regex.charAt(sep - 1)=='\\'){
					sep = regex.indexOf("-",sep+1);
				}
			}
			
			//if we get here char not in regex
			if(negated.booleanValue()){
				return true;
			} 
			return false;
		}
		
		//if here char fails regex
		return false;

	}

	private boolean isValidType(String placeHolder, char targetChar) {

		if (placeHolder.equals(InputAssistNames.MASK_ALPHABET_PLACEHOLDER)) {
			if(getChar2RegEx() != null && getChar2RegEx().length() > 0){
				return isValidForRegEx(getChar2RegEx(),targetChar);
			}
			if (Character.isLetter(targetChar)) {
				return true;
			} 
			return false;
			
		} else if (placeHolder.equals(InputAssistNames.MASK_DIGIT_PLACEHOLDER)) {
			if(getChar1RegEx() != null && getChar1RegEx().length() > 0){
				return isValidForRegEx(getChar1RegEx(),targetChar);
			}
			if (Character.isDigit(targetChar)) {
				return true;
			} 
			return false;
			
			//char3RegEx and char4RegEx values are checked in removeNonDataCharacters
			//wouldn't be here if they were
		} else if (placeHolder.equals(InputAssistNames.MASK_CUSTOM_EXCLAMATION_PLACEHOLDER)){
				return isValidForRegEx(getChar3RegEx(),targetChar);
			
		} else if (placeHolder.equals(InputAssistNames.MASK_CUSTOM_TILDA_PLACEHOLDER)){
				return isValidForRegEx(getChar4RegEx(),targetChar);
			
//		} else if (placeHolder.equals(InputAssistNames.MASK_DIGIT_OR_SPACE_PLACEHOLDER)) {
//			if (Character.isDigit(targetChar) || targetChar == ' ') {
//				return true;
//			} else {
//				return false;
//			}
		} else {
			return false;
		}
		
	}

	private int getNumberOfDigitsInMask() {
		int result = 0;
		char[] maskChars = mask.toCharArray();
		int maskLength = maskChars.length;
		for (int i = 0; i < maskLength; i++) {
			String temp = String.valueOf(maskChars[i]);
			if (temp.equals(InputAssistNames.MASK_DIGIT_PLACEHOLDER) /*||
				temp.equals(InputAssistNames.MASK_DIGIT_OR_SPACE_PLACEHOLDER)*/) {
				result++;
			}
		}
		
		return result;
	}

	/**
	 * @return
	 */
	public boolean isAsis() {
		return asis;
	}

	/**
	 * @return
	 */
	public String getMask() {
		return mask;
	}

	/**
	 * @param b
	 */
	public void setAsis(boolean b) {
		asis = b;
	}

	/**
	 * @param string
	 */
	public void setMask(String string) {
		mask = string;
	}

	// --------------------------------------------------------------------- StateHolder Methods

	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#restoreState(javax.faces.context.FacesContext, java.lang.Object)
	 */
	public void restoreState(FacesContext context, Object state) {

		Object values[] = (Object[]) state;
		mask = (String) values[0];
		//		promptCharacter = (String) values[1];
		asis = ((Boolean) values[1]).booleanValue();
		sourceObjType = (String) values[2];
		
		char1RegEx = (String) values[3];
		char2RegEx = (String) values[4];
		char3RegEx = (String) values[5];
		char4RegEx = (String) values[6];
		protectMask = (String) values[7];
	}

	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#saveState(javax.faces.context.FacesContext)
	 */
	public Object saveState(FacesContext context) {

		Object values[] = new Object[8];
		values[0] = mask;
		//		values[1] = promptCharacter;
		values[1] = new Boolean(asis);
		values[2] = sourceObjType;
		
		values[3] = char1RegEx;
		values[4] = char2RegEx;
		values[5] = char3RegEx;
		values[6] = char4RegEx;
		values[7] = protectMask;
		
		return (values);
	}

	private boolean transientFlag = false;
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#isTransient()
	 */
	public boolean isTransient() {
		return transientFlag;
	}

	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#setTransient(boolean)
	 */
	public void setTransient(boolean newTransientValue) {
		this.transientFlag = newTransientValue;
	}
}
