/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2004 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.converter;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.util.HashMap;
import java.util.Locale;

import javax.faces.application.Application;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.NumberConverter;
import javax.faces.el.ValueBinding;

/**
 * NumberConverterEx
 * 
 * @author doconnor
 * @Version 0.1
 * 
 * Feb 27, 2004 - 11:56:36 AM
 */
public class NumberConverterEx extends NumberConverter {
	/**
	 * 
	 */
	private boolean currencyCodeSet 		= false;
	private boolean currencySymbolSet 		= false;
	private boolean localeSet 				= false;
	private boolean maxFractionDigitsSet 	= false;
	private boolean maxIntegerDigitsSet 	= false; 
	private boolean minFractionDigitsSet 	= false;
	private boolean minIntegerDigitsSet 	= false;
	private boolean patternSet 				= false;
	private boolean typeSet 				= false;
	private boolean integerOnlySet 			= false;
	private boolean groupingUsedSet			= false;
	
	private static final char SEPARATOR = '/';
	
	private static char DOT = '.';
	private static char ZERO = '0';
	private static char THOUSANDSEPARATOR = ',';
	private char PADCHAR= '0';
	private char DECPADCHAR = '0';
	private boolean symbolsSetup = false;
	private boolean padCharsAreEqual = false;
	
	private String patternExtension = null;
	private boolean patternExtensionSet = false;
	
	
	
	/**
	 * @return Returns the patternExtensionSet.
	 */
	public boolean isPatternExtensionSet() {
		return patternExtensionSet;
	}
	/**
	 * @param patternExtensionSet The patternExtensionSet to set.
	 */
	public void setPatternExtensionSet(boolean patternExtensionSet) {
		this.patternExtensionSet = patternExtensionSet;
	}
	/**
	 * @return Returns the patternExtension.
	 */
	public String getPatternExtension() {
		return patternExtension;
	}
	/**
	 * @param patternExtension The patternExtension to set.
	 */
	public void setPatternExtension(String patternExtension) {
		this.patternExtension = patternExtension;
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#getCurrencyCode()
	 */
	public String getCurrencyCode() {		
		return super.getCurrencyCode();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#getCurrencySymbol()
	 */
	public String getCurrencySymbol() {		
		return super.getCurrencySymbol();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#getLocale()
	 */
	public Locale getLocale() {		
		if(isLocaleSet()){
			return super.getLocale();
		}
		return getLocale(FacesContext.getCurrentInstance());
	}
	
	/*
	 * Fix for api problem where locale is set for the life of the converter, this is a problem for dynamically 
	 * changing locales (e.g. portal, locale can be changed by a drop down selection).
	 * With this locale will be calculated each time with getViewRoot().getLocale() if locale attribute is not set.
	 */
	private Locale getLocale(FacesContext context){
		if(isLocaleSet()){
			return super.getLocale();
		}
		return context.getViewRoot().getLocale();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#getMaxFractionDigits()
	 */
	public int getMaxFractionDigits() {		
		return super.getMaxFractionDigits();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#getMaxIntegerDigits()
	 */
	public int getMaxIntegerDigits() {		
		return super.getMaxIntegerDigits();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#getMinFractionDigits()
	 */
	public int getMinFractionDigits() {		
		return super.getMinFractionDigits();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#getMinIntegerDigits()
	 */
	public int getMinIntegerDigits() {		
		return super.getMinIntegerDigits();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#getPattern()
	 */
	public String getPattern() {		
		return super.getPattern();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#getType()
	 */
	public String getType() {		
		return super.getType();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#isGroupingUsed()
	 */
	public boolean isGroupingUsed() {		
		return super.isGroupingUsed();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#isIntegerOnly()
	 */
	public boolean isIntegerOnly() {		
		return super.isIntegerOnly();
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setCurrencyCode(java.lang.String)
	 */
	public void setCurrencyCode(String currencyCode) {		
		super.setCurrencyCode(currencyCode);
		this.setCurrencyCodeSet((currencyCode != null && currencyCode.length() > 0));
		
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setCurrencySymbol(java.lang.String)
	 */
	public void setCurrencySymbol(String currencySymbol) {		
		super.setCurrencySymbol(currencySymbol);
		this.setCurrencySymbolSet((currencySymbol != null && currencySymbol.length() > 0));
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setGroupingUsed(boolean)
	 */
	public void setGroupingUsed(boolean groupingUsed) {		
		super.setGroupingUsed(groupingUsed);
		this.setGroupingUsedSet(true);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setIntegerOnly(boolean)
	 */
	public void setIntegerOnly(boolean integerOnly) {		
		super.setIntegerOnly(integerOnly);
		this.setIntegerOnlySet(true);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setLocale(java.util.Locale)
	 */
	public void setLocale(Locale locale) {
		if(locale != null){
			super.setLocale(locale);
			this.setLocaleSet(locale != null);
		}
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setMaxFractionDigits(int)
	 */
	public void setMaxFractionDigits(int maxFractionDigits) {		
		super.setMaxFractionDigits(maxFractionDigits);
		this.setMaxFractionDigitsSet(true);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setMaxIntegerDigits(int)
	 */
	public void setMaxIntegerDigits(int maxIntegerDigits) {		
		super.setMaxIntegerDigits(maxIntegerDigits);
		this.setMaxIntegerDigitsSet(true);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setMinFractionDigits(int)
	 */
	public void setMinFractionDigits(int minFractionDigits) {		
		super.setMinFractionDigits(minFractionDigits);
		this.setMinFractionDigitsSet(true);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setMinIntegerDigits(int)
	 */
	public void setMinIntegerDigits(int minIntegerDigits) {		
		super.setMinIntegerDigits(minIntegerDigits);
		this.setMinIntegerDigitsSet(true);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setPattern(java.lang.String)
	 */
	public void setPattern(String pattern) {
		if(isPaddedPattern(pattern)){
			//just save the chars after the SEPARATOR
			setPatternExtension(pattern.substring(pattern.lastIndexOf(SEPARATOR)));
			setPatternExtensionSet(true);
			
			//change to normal pattern
			pattern = pattern.substring(0,pattern.lastIndexOf(SEPARATOR));
		}
		
		super.setPattern(pattern);
		this.setPatternSet(pattern != null && pattern.length() > 0);
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.NumberConverter#setType(java.lang.String)
	 */
	public void setType(String type) {			
		super.setType(type);
		this.setTypeSet(type != null && type.length() > 0);
	}
	/**
	 * @return
	 */
	public boolean isCurrencyCodeSet() {
		return currencyCodeSet;
	}

	/**
	 * @return
	 */
	public boolean isCurrencySymbolSet() {
		return currencySymbolSet;
	}

	/**
	 * @return
	 */
	public boolean isGroupingUsedSet() {
		return groupingUsedSet;
	}

	/**
	 * @return
	 */
	public boolean isIntegerOnlySet() {
		return integerOnlySet;
	}

	/**
	 * @return
	 */
	public boolean isLocaleSet() {
		return localeSet;
	}

	/**
	 * @return
	 */
	public boolean isMaxFractionDigitsSet() {
		return maxFractionDigitsSet;
	}

	/**
	 * @return
	 */
	public boolean isMaxIntegerDigitsSet() {
		return maxIntegerDigitsSet;
	}

	/**
	 * @return
	 */
	public boolean isMinFractionDigitsSet() {
		return minFractionDigitsSet;
	}

	/**
	 * @return
	 */
	public boolean isMinIntegerDigitsSet() {
		return minIntegerDigitsSet;
	}

	/**
	 * @return
	 */
	public boolean isPatternSet() {
		return patternSet;
	}

	/**
	 * @return
	 */
	public boolean isTypeSet() {
		return typeSet;
	}

	/**
	 * @param b
	 */
	public void setCurrencyCodeSet(boolean b) {
		currencyCodeSet = b;
	}

	/**
	 * @param b
	 */
	public void setCurrencySymbolSet(boolean b) {
		currencySymbolSet = b;
	}

	/**
	 * @param b
	 */
	public void setGroupingUsedSet(boolean b) {
		groupingUsedSet = b;
	}

	/**
	 * @param b
	 */
	public void setIntegerOnlySet(boolean b) {
		integerOnlySet = b;
	}

	/**
	 * @param b
	 */
	public void setLocaleSet(boolean b) {
		localeSet = b;
	}

	/**
	 * @param b
	 */
	public void setMaxFractionDigitsSet(boolean b) {
		maxFractionDigitsSet = b;
	}

	/**
	 * @param b
	 */
	public void setMaxIntegerDigitsSet(boolean b) {
		maxIntegerDigitsSet = b;
	}

	/**
	 * @param b
	 */
	public void setMinFractionDigitsSet(boolean b) {
		minFractionDigitsSet = b;
	}

	/**
	 * @param b
	 */
	public void setMinIntegerDigitsSet(boolean b) {
		minIntegerDigitsSet = b;
	}

	/**
	 * @param b
	 */
	public void setPatternSet(boolean b) {
		patternSet = b;
	}

	/**
	 * @param b
	 */
	public void setTypeSet(boolean b) {
		typeSet = b;
	}
	
	private void setupSymbols(){
		if(!symbolsSetup){
			DecimalFormatSymbols symbols = new DecimalFormatSymbols(this.getLocale());
			DOT = symbols.getDecimalSeparator();
			ZERO = symbols.getZeroDigit();
			PADCHAR = ZERO;
			DECPADCHAR = ZERO;
			THOUSANDSEPARATOR = symbols.getGroupingSeparator();
			symbolsSetup = true;			
		}	
	}
	
	private boolean isPaddedPattern(String pattern){
		if(pattern != null && pattern.indexOf(ZERO) != -1){
			int index = 0;
			if((index = pattern.lastIndexOf(SEPARATOR)) != -1){ // If the pattern contains '0' and '/'
				if(pattern.indexOf('\'', index) == -1){	// and the pattern does not contain ' (quotation mark) following
					return true;			// then this is a 'padding pattern'.
				} 		
			}
		}		
		return false;
	}
	
	/*
	 * Removes the IBM extended pattern chars form the value.
	 */
	protected String decodeExtensionPatternValue(FacesContext context, String value){
		StringBuffer paddedValAfterDot = null;
		StringBuffer paddedValBeforeDot = new StringBuffer(value);
	
		if(value.indexOf(DOT) != -1){
			paddedValBeforeDot = new StringBuffer(value.substring(0, value.indexOf(DOT)));
			paddedValAfterDot = new StringBuffer(value.substring(value.indexOf(DOT) + 1, value.length()));
		}
		
		if(getPatternExtension().length() > 1){
			 //PADCHAR = pattern.charAt(pattern.lastIndexOf(SEPARATOR) + 1);
			PADCHAR = getPatternExtension().charAt(1);
		}
		if(getPatternExtension().length() > 2){
			DECPADCHAR = getPatternExtension().charAt(2);				
		}
		
		boolean containsDigits = false;
		
		for(int i = 0; i < paddedValBeforeDot.length(); i++){
			char c = paddedValBeforeDot.charAt(i);
			if(!Character.isDigit(c)){
				if(c == PADCHAR){
					paddedValBeforeDot.deleteCharAt(i);
					i--;
				} else if(PADCHAR != '0' && getPattern().indexOf(PADCHAR) > -1){
					//check in chase PADCHAR is in the pattern, if this is deleted we'll get a converter exception
					//not perfect as not checking positive and negative cases but in most cases these would be the same i guess
					int index = getPattern().indexOf(c);
					if(index > -1){
						int x = index + 1;
						while(x < getPattern().length() && getPattern().charAt(x) == PADCHAR){
							//there is one or more padchars to the right of this char in the pattern - append padchar
							paddedValBeforeDot.insert(paddedValBeforeDot.toString().lastIndexOf(c) + 1,PADCHAR);
							i++;
							x++;
						}
						//no else, maybe the case the char has a padchar on both sides in the pattern
						x = index - 1;
						while(x > -1 && getPattern().charAt(x) == PADCHAR){
							//there is a padchar to the left of this char in the pattern - prepend padchar
							paddedValBeforeDot.insert(paddedValBeforeDot.toString().indexOf(c),PADCHAR);
							i++;
							x--;
						}	
					}
				}
			}else{
				containsDigits = true;
				break;
			}								
		}
			
		if(paddedValAfterDot != null){
			for(int i = paddedValAfterDot.length() - 1; i >= 0; i--){
				char c = paddedValAfterDot.charAt(i);
				if(!Character.isDigit(c)){
					if(c == DECPADCHAR){
						paddedValAfterDot.deleteCharAt(i);
					} else if(DECPADCHAR != '0' && getPattern().indexOf(DECPADCHAR) > -1){
						//check in chase PADCHAR is in the pattern, if this is deleted we'll get a converter exception
						//not perfect as not checking positive and negative cases but in most cases these would be the same i guess
						int index = getPattern().indexOf(c);
						if(index > -1){
							int x = index + 1;
							while(x < getPattern().length() && getPattern().charAt(x) == PADCHAR){
								//there is one or more padchars to the right of this char in the pattern - append padchar
								paddedValAfterDot.insert(paddedValAfterDot.toString().lastIndexOf(c) + 1,PADCHAR);
								x++;
							}
							//no else, maybe the case the char has a padchar on both sides in the pattern
							x = index - 1;
							while(x > -1 && getPattern().charAt(x) == PADCHAR){
								//there is a padchar to the left of this char in the pattern - prepend padchar
								paddedValAfterDot.insert(paddedValAfterDot.toString().indexOf(c),PADCHAR);
								x--;
							}	
						}
					}						
				}else{
					containsDigits = true;
					break;
				}
			}
		}
			
		String parsedVal = paddedValBeforeDot.toString();
		if(paddedValAfterDot != null){
			parsedVal += DOT;
			parsedVal += paddedValAfterDot.toString();
		}
		value = parsedVal;
		if((value != null && value.length() == 0) || !containsDigits){
			//setting a zero char causes conversion error in case of currency symbol being used,need to format based on pattern
			//value += new String(new char[]{ZERO});
			NumberFormat nf = NumberFormat.getInstance(context.getViewRoot().getLocale());
			if(nf instanceof DecimalFormat) {
				((DecimalFormat) nf).applyPattern(getPattern());
				value = ((DecimalFormat)nf).format(0);
			} else {
				value = "";
			}
		}
		return value;
	}

	/* (non-Javadoc)
	 * @see javax.faces.convert.Converter#getAsObject(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.String)
	 */
	public Object getAsObject(
		FacesContext context,
		UIComponent component,
		String value) {
		
		setupSymbols();
		
		if (value.indexOf(' ') > -1
	 			&& (THOUSANDSEPARATOR == '\u00A0' || DOT == '\u00A0')){
	 		// fix input so conversion will work
	 		value = value.replace(' ', '\u00A0');
	 	}
	 	
		if(isPatternExtensionSet()){			
			value = decodeExtensionPatternValue(context,value);			
		}		

		Object newValue = super.getAsObject(context, component, value);
		ValueBinding valueBinding = component.getValueBinding("value"); //$NON-NLS-1$
		Converter converter = null;
		if (null == newValue || null == valueBinding) {
			return newValue;
		}
		Class resultType = valueBinding.getType(context);
		
		if (resultType == newValue.getClass()
			|| resultType == null // ?
			|| resultType == Object.class) {
			return newValue;
		}
		
		// this is a cheap way of picking up any conversion errors
		// when converting to the new type
		// attempt to get an the appropriate converter

		try {
			Application application = context.getApplication();
			converter = application.createConverter(resultType);
		} catch (Exception e) {
			// TODO should log error
			return (null);
		}

		if (converter != null) {			
			newValue = converter.getAsObject(context, component, newValue.toString());
			return newValue;
		} 
		
		return newValue;
		
	}
	
	/*
	 * Adds the IBM pattern extension chars to the value.
	 */
	protected String encodeExtensionPatternValue(Object value, String objectAsString){
 		/*
 		 * this means our pattern is of the form:
 		 * #,##0.00 (#,##0.00) /**
 		 * or something similar 
 		 */	 			 	
 		
	 	padCharsAreEqual = false;
	 	/*
	 	 * Get the pad character for before and after the decimal point (.)
	 	 */
 		if(getPatternExtension().length() > 1){
 			 PADCHAR = getPatternExtension().charAt(1);
 		}
 		
 		if(getPatternExtension().length() > 2){
 			DECPADCHAR = getPatternExtension().charAt(2);
 			if(DECPADCHAR == PADCHAR){
				padCharsAreEqual = true;
			}
 		}
 		
 		
	 	
	 	if(objectAsString != null && objectAsString.length() > 0){
	 		Double d = new Double(1);
	 		if(value instanceof String){
	 			d = new Double(Double.parseDouble((String)value));
	 		}else{
	 			Number n = (Number)value;
		 		d = new Double(n.doubleValue());		 			
	 		}
	 		
	 		if(d.doubleValue() != 0 || !padCharsAreEqual){
	 			String newVal = objectAsString;
		 		StringBuffer beforeDot = new StringBuffer(objectAsString);
		 		StringBuffer afterDot = null;
		 		if(objectAsString.indexOf(DOT) != -1){
		 			beforeDot = new StringBuffer(objectAsString.substring(0, objectAsString.indexOf(DOT)));
		 			if((objectAsString.indexOf(DOT) + 1) <= objectAsString.length()){
		 				afterDot = new StringBuffer(objectAsString.substring(objectAsString.indexOf(DOT) + 1));		 				
		 			}
		 		}
		 		
		 		if(beforeDot != null && beforeDot.length() > 0){
		 			for(int i = 0; i < beforeDot.length(); i++){
		 				char c = beforeDot.charAt(i);
		 				
		 				if(Character.isDigit(c) || c == THOUSANDSEPARATOR){
		 					if(c == ZERO  || c == THOUSANDSEPARATOR){
		 						beforeDot.setCharAt(i, PADCHAR);
		 					}else{
		 						break;
		 					}
		 				}		 				
		 			}
		 			newVal = beforeDot.toString();		 			
		 		}	
		 		if(afterDot != null && afterDot.length() > 0){		 			
		 			for(int i = afterDot.length(); i > 0; i--){
		 				char c = afterDot.charAt(i - 1);
		 				if(Character.isDigit(c)){
		 					if(c == ZERO){
		 						afterDot.setCharAt(i - 1, DECPADCHAR);
		 					}else{
			 					break;
			 				}		 					
		 				}	
		 			}
		 			newVal += DOT;
		 			newVal += afterDot.toString();		 			
		 		}
		 		return newVal;		 			
	 		}

	 		StringBuffer sb = new StringBuffer(objectAsString);
 			
 			for(int i = 0; i < objectAsString.length(); i++){
 				if(objectAsString.charAt(i) == ZERO || objectAsString.charAt(i) == DOT || objectAsString.charAt(i) == THOUSANDSEPARATOR){
 					sb.setCharAt(i, PADCHAR);		 					
 				}else{
 					sb.setCharAt(i, objectAsString.charAt(i));
 				}
 			}		 			
 			return sb.toString();
	 		
	 	}
	 	return objectAsString;

		
	}
	
	 public String getAsString(FacesContext context, UIComponent component,
            Object value) { 	
	 	
	 	//String pattern = getPattern();
	 	setupSymbols();

	 	String valueStr = null;
	 	if(isPatternExtensionSet()){
	 		valueStr = encodeExtensionPatternValue(value, super.getAsString(context, component, value));
	 	} else {
		 	valueStr = super.getAsString(context, component, value);	 		
	 	}
	 	if (component instanceof javax.faces.component.EditableValueHolder)
	 		valueStr = valueStr.replace('\u00A0', ' ');

	 	return valueStr;
	 }
	 
	 /*JSF state saving/restoration methods*/
	 
		/* (non-Javadoc)
		 * @see javax.faces.component.StateHolder#restoreState(javax.faces.context.FacesContext, java.lang.Object)
		 */
		public void restoreState(FacesContext context, Object state) {
	        Object values[] = (Object[]) state;
	        super.restoreState(context, values[0]);
	        
	    	currencyCodeSet 		= ((Boolean)values[1]).booleanValue();
	    	currencySymbolSet 		= ((Boolean)values[2]).booleanValue();
	    	localeSet 				= ((Boolean)values[3]).booleanValue();
	    	maxFractionDigitsSet 	= ((Boolean)values[4]).booleanValue();
	    	maxIntegerDigitsSet 	= ((Boolean)values[5]).booleanValue(); 
	    	minFractionDigitsSet 	= ((Boolean)values[6]).booleanValue();
	    	minIntegerDigitsSet 	= ((Boolean)values[7]).booleanValue();
	    	patternSet 				= ((Boolean)values[8]).booleanValue();
	    	typeSet 				= ((Boolean)values[9]).booleanValue();
	    	integerOnlySet 			= ((Boolean)values[10]).booleanValue();
	    	groupingUsedSet			= ((Boolean)values[11]).booleanValue();	    	
	    	patternExtension 		= (String)values[12];
	    	patternExtensionSet 	= ((Boolean)values[13]).booleanValue();
	        
		}
		/* (non-Javadoc)
		 * @see javax.faces.component.StateHolder#saveState(javax.faces.context.FacesContext)
		 */
		public Object saveState(FacesContext context) {
	        Object values[] = new Object[14];
	        values[0] = super.saveState(context);
	        
	        values[1] =  Boolean.valueOf(currencyCodeSet);
	        values[2] =  Boolean.valueOf(currencySymbolSet);
	        values[3] =  Boolean.valueOf(localeSet);
	        values[4] =  Boolean.valueOf(maxFractionDigitsSet);
	        values[5] =  Boolean.valueOf(maxIntegerDigitsSet);
	        values[6] =  Boolean.valueOf(minFractionDigitsSet);
	        values[7] =  Boolean.valueOf(minIntegerDigitsSet);
	        values[8] =  Boolean.valueOf(patternSet);
	        values[9] =  Boolean.valueOf(typeSet);
	        values[10] =  Boolean.valueOf(integerOnlySet);
	        values[11] =  Boolean.valueOf(groupingUsedSet);
	        values[12] =  patternExtension;
	        values[13] =  Boolean.valueOf(patternExtensionSet);
	        
	        return (values);
		}
	 
	 
}
