/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2004 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.converter;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Locale;

import javax.faces.application.Application;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.ConverterException;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;

import com.ibm.faces.renderkit.html_extended.Utils;
import com.ibm.icu.text.DecimalFormat;
import com.ibm.icu.text.DecimalFormatSymbols;
import com.ibm.icu.text.NumberFormat;

/**
 * NumberConverterExICU
 * 
 * @author womahony
 * @Version 0.1
 * 
 *  This is the ICU4J -ished version of the NumberConverterEx , it overrides getAsString and getAsObject
 * 
 * Feb 13, 2005 - 11:56:36 AM
 */
public class NumberConverterExICU extends NumberConverterEx {
	private static final char NBSP = '\u00A0';
	private static final char SPACE = ' ';
	/**
	 * The converter id for this converter.
	 */
	public static final String CONVERTER_ID = "com.ibm.faces.Number"; //$NON-NLS-1$
	

	
	
	private Locale setupLocale(FacesContext context)
	{
		if(!isLocaleSet())
		{
			Locale l = context.getViewRoot().getLocale();
			if(l != null)
				return l;			
		}
		return getLocale(); 
	}
	
	public DecimalFormat getNumberFormat(FacesContext context)	{
		return getNumberFormat(context, false);
	}	

	private DecimalFormatSymbols fixNbspSymbols(DecimalFormatSymbols symbols){
		if (symbols.getDecimalSeparator() == NBSP)
			symbols.setDecimalSeparator(SPACE);
		if (symbols.getGroupingSeparator() == NBSP)
			symbols.setGroupingSeparator(SPACE);
		if (symbols.getCurrencySymbol() != null)
			symbols.setCurrencySymbol(symbols.getCurrencySymbol().replace(NBSP,	SPACE));
		return symbols;
	}
	
	public DecimalFormat getNumberFormat(FacesContext context, boolean isInput)	{
			
		Locale locale = setupLocale(context);
		DecimalFormat df = null;
		DecimalFormatSymbols symbols = null;
	 	if(isPatternSet()){ 
	 		symbols = new DecimalFormatSymbols(locale);
		 	
		 	//currency
 			if(isCurrencyCodeSet()){
	 			symbols.setInternationalCurrencySymbol(getCurrencyCode());
	 		}
	 		if(isCurrencySymbolSet()){
	 			symbols.setCurrencySymbol(getCurrencySymbol());
	 		}
	 		if (isInput){
	 			fixNbspSymbols(symbols);
	 		}
	 		df = new DecimalFormat(getPattern(), symbols);
		 	return df;
	 	} 
	 	
	 	//pattern not set....
	 	symbols = new DecimalFormatSymbols(locale);
	 	
	 	if(isCurrencyCodeSet()){
	 		symbols.setInternationalCurrencySymbol(getCurrencyCode());
	 	}
	 	if(isCurrencySymbolSet()){
	 		symbols.setCurrencySymbol(getCurrencySymbol());
	 	}
 		if (isInput){
 			fixNbspSymbols(symbols);
 		}
	 	if((Utils.NUMBERSTYLE_NUMBER.equalsIgnoreCase(getType())) 
				|| (!isTypeSet() && !isPatternSet())){
	 		// if you set number, decimal or you set nothing
	 		df = (DecimalFormat)NumberFormat.getInstance(locale);	 		
	 		
	 	} else if(Utils.NUMBERSTYLE_CURRENCY.equalsIgnoreCase(getType())){
	 		
 			df = (DecimalFormat)NumberFormat.getCurrencyInstance(locale);
	 		
	 	} else if(Utils.NUMBERSTYLE_PERCENT.equalsIgnoreCase(getType())){	 		
	 		 			
	 		df = (DecimalFormat)NumberFormat.getPercentInstance(locale);
	 		
	 	} else {
	 		throw new ConverterException("Cannot resolve a NumberFormat from attributes set");
	 	}
 		df.setDecimalFormatSymbols(symbols);
 		return df;
	}

	
	/* (non-Javadoc)
	 * @see javax.faces.convert.Converter#getAsObject(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.String)
	 */
	public Object getAsObject(FacesContext context,	UIComponent component,String value) 
	{
		
		if(isPatternExtensionSet()){			
			value = decodeExtensionPatternValue(context,value);			
		}	
		
		if(value == null)
			return null;
		
		value = value.trim();
		
		if(value.length() <= 0)
			return null;
		
	 	try{
	 		NumberFormat nf = getNumberFormat(context, component instanceof javax.faces.component.EditableValueHolder);
	 		if(isIntegerOnlySet()){
	 			nf.setParseIntegerOnly(isIntegerOnly());	
	 		}
	 		
	 		Number num = nf.parse(value);
	 		
	 		ValueBinding vb = component.getValueBinding("value");
	 		if(vb != null){
	 			Class vbClass = vb.getType(context);
	 			if(vbClass == Number.class || vbClass == num.getClass() || vbClass == Object.class){
	 				return num;
	 			} else if(vbClass == Short.class || vbClass == short.class) {
	 				return new Short(num.shortValue());
	 			} else if(vbClass == Integer.class || vbClass == int.class) {
	 				return new Integer(num.intValue());
	 			} else if(vbClass == Long.class || vbClass == long.class){
	 				return new Long(num.longValue());
	 			} else if(vbClass == Double.class || vbClass == double.class){
	 				return new Double(num.doubleValue());
	 			} else if(vbClass == Float.class || vbClass == float.class){
	 				return new Float(num.floatValue());
	 			} else if(vbClass == BigDecimal.class){
	 				return new BigDecimal(num.toString());
	 			}else if(vbClass == BigInteger.class){
	 				return new BigInteger(num.toString());
	 			} else if(vbClass == com.ibm.icu.math.BigDecimal.class){ // this is the one icu4j number case we need to cover
	 				return new com.ibm.icu.math.BigDecimal(num.toString());
	 			}
	 			
	 			// this is a cheap way of picking up any conversion errors
	 			// when converting to the new type
	 			// attempt to get an the appropriate converter
	 			Converter converter = null;
	 			try {
	 				Application application = context.getApplication();
	 				converter = application.createConverter(vbClass);
	 			} catch (Exception e) {
	 				// TODO should log error
	 				return (null);
	 			}

	 			if (converter != null) {			
	 				Object numobj = converter.getAsObject(context, component, num.toString());
	 				return numobj;
	 			} 
	 			throw new ConverterException("Valuebinding type not supported.");
	 		}
	 		return num;
	 		
		}catch(Exception e)
		{
			throw new ConverterException(e);
		}
	 	
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.convert.Converter#getAsStringjavax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.Object)
	 */
	
	 public String getAsString(FacesContext context, UIComponent component, Object value) 
	 { 	
		
	 	String valueStr = null;
	 	
	 	if (value == null) {
			valueStr = ""; //$NON-NLS-1$
		} else if(value instanceof String) {// not sure about this
			valueStr = (String)value;
		} else {
			
	 	try {
				NumberFormat nf = getNumberFormat(context, component instanceof javax.faces.component.EditableValueHolder);
				//apply max, min and grouping settings
				if (isMaxFractionDigitsSet())
					nf.setMaximumFractionDigits(getMaxFractionDigits());
				if (isMinFractionDigitsSet())
					nf.setMinimumFractionDigits(getMinFractionDigits());
				if (isMaxIntegerDigitsSet())
					nf.setMaximumIntegerDigits(getMaxIntegerDigits());
				if (isMinIntegerDigitsSet())
					nf.setMinimumIntegerDigits(getMinIntegerDigits());
				if (isGroupingUsedSet())
					nf.setGroupingUsed(isGroupingUsed());
				//format the value object
				valueStr = nf.format(value);
			} catch (Exception e) {
				throw new ConverterException(e);
			}
		}
	 	
	 	if(isPatternExtensionSet()){
	 		return encodeExtensionPatternValue(value, valueStr);
	 	} 
	 		
	 	return valueStr;
	
	 }
	 
	 //client side state methods
	public Object saveState(FacesContext context) {
	 	return super.saveState(context);
	 }
	 
	 public void restoreState(FacesContext context, Object state) {
	 	super.restoreState(context,state);
	 }	 
}
