/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.databind;

import javax.faces.el.PropertyNotFoundException;
import javax.faces.el.PropertyResolver;

/**
 * Property resolver for the expression after the "selectitems" variable
 * 
 * @author spaxton
 */

public class SelectItemsPropResolver extends PropertyResolver {

	private static final String TOARRAY = "toArray";
	private PropertyResolver originalResolver=null;
	
	public SelectItemsPropResolver(PropertyResolver originalResolver) {
		this.originalResolver = originalResolver;
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getValue(java.lang.Object, java.lang.String)
	 */
	public Object getValue(Object base, Object name) throws PropertyNotFoundException {
		if(base instanceof SelectItemsWrapper) {
			SelectItemsWrapper wrapper = (SelectItemsWrapper) base;
			String segment = (String) name;
			if(!TOARRAY.equals(segment)) {
				wrapper.addExpressionSegment((String) name);
				return wrapper;
			} else {
				return wrapper.toArray();
			}
		} else {
			return originalResolver.getValue(base, name);
		}
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getValue(java.lang.Object, int)
	 */
	public Object getValue(Object base, int index) throws PropertyNotFoundException {
		if(base instanceof SelectItemsWrapper) {
			throw new PropertyNotFoundException();
		}
		return originalResolver.getValue(base, index);
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#setValue(java.lang.Object, java.lang.String, java.lang.Object)
	 */
	public void setValue(Object base, Object name, Object value)
		throws PropertyNotFoundException {
			if(!(base instanceof SelectItemsWrapper)) {
				originalResolver.setValue(base, name, value);
				return;
			}
			throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#setValue(java.lang.Object, int, java.lang.Object)
	 */
	public void setValue(Object base, int index, Object value)
		throws PropertyNotFoundException {
			if(!(base instanceof SelectItemsWrapper)) {
				originalResolver.setValue(base, index, value);
				return;
			}
			throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#isReadOnly(java.lang.Object, java.lang.String)
	 */
	public boolean isReadOnly(Object base, Object name)	throws PropertyNotFoundException {
			if(base instanceof SelectItemsWrapper) {
				return true;
			} else {
				return originalResolver.isReadOnly(base, name);
			}
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#isReadOnly(java.lang.Object, int)
	 */
	public boolean isReadOnly(Object base, int index)
		throws PropertyNotFoundException {
			if(!(base instanceof SelectItemsWrapper)) {
				return originalResolver.isReadOnly(base, index);
			}
		return true;
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getType(java.lang.Object, java.lang.String)
	 */
	public Class getType(Object base, Object name)
		throws PropertyNotFoundException {
			if(!(base instanceof SelectItemsWrapper)) {
				return originalResolver.getType(base, name);
			} else {
				return SelectItemsWrapper.class;
			}
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getType(java.lang.Object, int)
	 */
	public Class getType(Object base, int index)
		throws PropertyNotFoundException {
		if(!(base instanceof SelectItemsWrapper)) {
			return originalResolver.getType(base, index);
		} else {
			throw new PropertyNotFoundException();
		}
	}

}
