/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2003 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.databind;

import javax.faces.context.FacesContext;
import javax.faces.el.VariableResolver;



/**
 * A variable resolver for producing JSF selectitem components from non-Map data.  The resolver
 * is triggered by the special String "selectitems" in an EL expression of the form
 * "selectitems.beanName.labelField.valueField".
 * 
 * @author spaxton
 */
public class SelectItemsVarResolver extends VariableResolver {
	private static final String SELECTITEMS_VAR = "selectitems"; //$NON-NLS-1$
	
	private VariableResolver originalResolver = null;
	
	public SelectItemsVarResolver(VariableResolver parent) {
		this.originalResolver = parent;
	}


	public Object resolveVariable(FacesContext context, String name) {
		if(SELECTITEMS_VAR.equals(name)) {
			// matched the selectitems variable name
			// create a new wrapper object
			SelectItemsWrapper wrapperObj = new SelectItemsWrapper();
			return wrapperObj;
		}
		return originalResolver.resolveVariable(context, name);
	}

}
