/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.fileupload.util;

import java.io.UnsupportedEncodingException;

/**
 * Content line implementation. 
 * For the Multipart Content parser, it is required to save both String and binary representation of a 
 * line that has been parsed of the content stream. This is handled by this object.
 * 
 * @author Andreas Nagel
 */
public class ContentLine
    {
 
    /**
     * Byte representation of the content.
     */
    private byte [] byt;
    /**
     * String representation of the content.
     */
    private String str;

    protected static String trimByteArray (byte [] line, String enc)
       {
       String lineAsString;
       int offset = 0;
       if (line.length > 0 && line [line.length -1] == 0x0a)
          {
          if (line.length > 1 && line [line.length -2] == 0x0d)
             offset = 2;
          else
             offset = 1;
          }
       try {
          lineAsString = new String (line, 0, line.length - offset, enc);
          }
       catch (UnsupportedEncodingException e)
          { // try to use the default method.
          lineAsString = new String (line, 0, line.length - offset);
          }
       return lineAsString;
       }

    /**
     * Constructor method
     * 
     * @param b      Line content. Usually, the last characters of the line are CR+LF.
     */
    public ContentLine (byte [] b, String enc)
       {
       byt = b;
       str = trimByteArray (b, enc);
       }

    /**
     * Return the length of this line, including CRLF characters
     * 
     * @return int Line length, including CRLF characters
     */
    public int size ()
       {
       return byt.length;
       }

    /**
     * Return the line as a byte array, including CRLF characters
     * 
     * @return byte[] binary representation of the line
     */
    public byte [] getBytes () 
       {
       return byt;
       }

    /**
     * Return a String representation of the line (excluding CRLDF characters).
     * 
     * @return String The content  of this line
     */
    public String getString ()
       {
       return str;
       }

    /**
     * Compare the value of the line with another value.
     * 
     * @param s      Perform a String compare: check if the lines are identical
     * @return boolean true if the strings match, false if the strings are different.
     */
    public boolean equals (String s)
       {
       return s.equals (str);
       }

    /**
     * Checks if a line matches to another
     * 
     * @param s      String to compare
     * @return boolean true if the strings match, false if the strings do not match
     */
    public boolean equalsIgnoreCase (String s)
       {
       return s.equalsIgnoreCase (str);
       }

    }
  
