/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.fileupload.util;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Implementation of a content line reader.
 * The class is required to parse multipart contents.
 * 
 * @author Andreas Nagel
 */
public class ContentLineReader
    {
    protected InputStream reader;
    private boolean eof;
    private static final char lf= '\n';
    private boolean debug;
    private String encoding;
    private ByteArrayOutputStream dump;

    /**
     * Constructor method
     * 
     * @param r      Input stream reader
     * @param enc     Encoding
     */
    public ContentLineReader (InputStream r, String enc)
       {
       reader = r;
       eof = false;
       encoding=enc;
       debug = false;
       }

    /**
     * Constructor method
     * 
     * @param r      Input stream reader
     * @param isDebug If the flag is set true, the data that is read from the input stream is also logged in a dump, which can be retrieved later on
     * @param enc     Encoding
     *                using the getDump method.
     */
    public ContentLineReader (InputStream r, String enc, boolean isDebug)
       {
       reader = r;
       debug = isDebug;
       encoding = enc;
       eof = false;
       if (debug)
          dump = new ByteArrayOutputStream ();
       }


    /**
     * Read one line of content information
     * 
     * @return ContentLine content line.
     * @exception IOException
     */
    public ContentLine readLine () throws IOException
       {
       boolean eol = false; // end-of-line mark
       ByteArrayOutputStream os = new ByteArrayOutputStream ();

       if (eof == true) return null;
       while (!eol && !eof)
          {
          int i = reader.read ();
          if (i == -1) 
             eof = true;
          else
             {
             if (debug) dump.write (i);
             os.write (i);
             if (i == lf) 
                eol = true;
             }
          }
       return new ContentLine (os.toByteArray(), encoding);
       }

    /**
     * Return the content of the input stream that has been read so far.
     * null is returned, if the debugging mode was disabled.
     * The function can be used for tracing and debugging purposes.
     * 
     * @return byte [] Input stream contents
     */
    public byte [] getDump ()
       {
       if (debug == false)
          return null;
       else
          return (dump.toByteArray ());
       }

    }
  
