/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.fileupload.util;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;


/**
 * Implementation of the Multipart Content reader.
 * 
 * @author Andreas Nagel
 */
public class MultipartContent
    {
    private InputStream is;
    private boolean debug;
    private byte [] dump;
    private Vector contents;

/**
 * Default constructor method
 * 
 * @param i      Content input stream
 */
    public MultipartContent (InputStream i)
       {
       debug = false;
       is = i;
       dump = null;
       }

    /**
     * Set the debugging mode.
     * If the debugging mode is activated, the content of the input stream that is read by this object is also logged in a dump. The dump 
     * can be retrieved by the caller, in using the getDump() method. The purpose of this dump is
     * to allow tracing.
     * 
     * @param isDebug true if a dump shall be created, false if no dump shall be created.
     */
    public void setDebug (boolean isDebug)
       {
       debug = isDebug;
       }

    /**
     * Return the input stream dump.
     * 
     * @return byte [] input stream dump
     */
    public byte [] getDump ()
       {
       return dump;
       }


    /**
     * Return the Multipart content elements in a Vector list.
     * 
     * @param encoding value.
     * @return Vector list of ContentElement objects. Each ContentElement object represents one piece of the multipart content.
     * @exception IOException
     */
    public Vector getContents (String encoding) throws IOException
       { 
       if (contents == null)
          {
          contents = new Vector ();
          ContentLineReader processOutputReader = new ContentLineReader (new BufferedInputStream(is), encoding, debug);
          
          ContentLine line = processOutputReader.readLine();
          
          if (line == null) return contents;
          
          // the first line must be the content separator!
          
          String mark     = line.getString ();
          String endmark  = mark + "--";       // end mark
          
          boolean eoc = false; // end of content
          ContentElement elem = null;
          
          // loop through the content stream
          while (line != null && eoc == false)
              {
              if (line.equals (mark))
                 {
                 if (elem != null) 
                    {
                    contents.addElement (elem);
                    }
                 elem = new ContentElement (encoding);
                 }
          
              else if (line.equals (endmark))
                 {
          
                 if (elem != null) 
                    contents.addElement (elem);
                 eoc = true;
                 }
          
              else 
                 {
                 elem.addToContent (line);
                 }
              // read the next line;
              line = processOutputReader.readLine();
              }
          
          // save a dump of the content, if required
          if (debug)
             {
             dump = processOutputReader.getDump ();
             }
          }
       return contents;
       }
    }
  
