/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.jwlajaxencoder;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.portlet.PortletURL;
import javax.portlet.RenderResponse;

/**
 * @deprecated Use IscAjaxUrlEncoder instead.
 */
public class NonIBMBridgeNonPortalAjaxUrlEncoder implements AjaxUrlEncoder {

  /**
   * Encoding a url involves creating a Portlet URL and adding all additional
   * parameters to the url.
   * @param facesContext The FacesContext object for this JSF environment.
   * @param url The String representation of the url to encode.
   * @return The encoded String URL object.
   * @see AjaxUrlEncoder#encodeAjaxUrl(javax.faces.context.FacesContext, java.lang.String)
   */
  public String encodeAjaxUrl( FacesContext facesContext, String url )
  {
    ExternalContext externalContext = facesContext.getExternalContext();
    // Get the Response object.
    Object object = externalContext.getResponse();
    // If the Response object is a RenderResponse.
    if ( object instanceof RenderResponse )
    {
      // RenderResponse is a portlet class so we are in the portlet environment.
      RenderResponse response = ( RenderResponse )object;

      String params = null;
      
      // Get the LAST index of ? in cases where we have multiple question marks.
      int n = url.lastIndexOf( '?' );
      if ( n > 0 )
      {
        params = url.substring( n + 1 );
        url = url.substring( 0, n );
      }

      // If the url starts with the portlet context path, remove it.
      if ( url.startsWith( externalContext.getRequestContextPath() ) )
      {
        url = url.substring( externalContext.getRequestContextPath().length() );
      }
      // If the url starts with the servlet path, remove it.
      if ( url.startsWith( externalContext.getRequestServletPath() ) )
      {
        url = url.substring( externalContext.getRequestServletPath().length() );
      }
      // Create the Portlet action url from the response.
      PortletURL actionURL = response.createActionURL();

      // If parameters are not null.
      if ( params != null )
      {
        // Add the parameters to the action url.
        addParams( params, actionURL );
      }
      // Return the String representation of the Portlet action url.
      return actionURL.toString();
    }
    else
    {
      return externalContext.encodeActionURL( url );
      //return context.getExternalContext().encodeActionURL(url);
    }
  }

  /**
   * Parses the additional parameter strings and adds name value pairs to the
   * Portlet action URL.
   * @param params The String containing the name and value pairs.
   * @param url The Portlet URL object to add these parameters to.
   */
  private void addParams( String params, PortletURL url )
  {
    if ( params != null && url != null )
    {
      if ( ( params.indexOf( "=" ) ) > -1 )
      {
        int pos1 = 0;
        int pos2 = 0;
        String name = null;
        String value = null;
        while ( pos1 > -1 )
        {
          pos2 = params.indexOf( "=", pos1 );
          name = params.substring( pos1, pos2 );
          pos1 = params.indexOf( "&", pos2 );
          if ( pos1 == -1 )
          {
            value = params.substring( pos2 + 1 );
          }
          else
          {
            value = params.substring( pos2 + 1, pos1 );
            ++pos1;
          }
          url.setParameter( name, value );
        }
      }
    }
  }
}
