/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit;

import java.io.OutputStream;
import java.io.Writer;
import java.util.Map;
import java.util.HashMap;

import javax.faces.context.FacesContext;
import javax.faces.context.ResponseStream;
import javax.faces.context.ResponseWriter;
import javax.faces.render.RenderKit;
import javax.faces.render.Renderer;
import javax.faces.render.ResponseStateManager;

import com.ibm.faces.application.AjaxResponseStateManagerImpl;
import com.ibm.faces.util.AjaxUtil;

/**
 * AjaxRenderKit is an implementation of RenderKit which will return an implementation of
 * AjaxRenderer when asked for a Renderer and one exists in the underlying RenderKit.
 * 
 * @author mark_wallace@ie.ibm.com
 */
public class AjaxRenderKit extends RenderKit {
	
	private RenderKit _renderKit;
	private AjaxResponseStateManagerImpl RSM;
	
	private Map _rendererCache; // this guy holds the references to the wrapped renderers
		
	AjaxRenderKit(RenderKit renderKit) {		
		_renderKit = renderKit;
		RSM = new AjaxResponseStateManagerImpl(renderKit.getResponseStateManager());
		_rendererCache = new HashMap();
	}

	/* (non-Javadoc)
	 * @see javax.faces.render.RenderKit#addRenderer(java.lang.String, java.lang.String, javax.faces.render.Renderer)
	 */
/*	public void addRenderer(String family, String rendererType, Renderer renderer) {
		if (!(renderer instanceof AjaxRenderer)) {
			renderer = new DefaultAjaxRenderer(renderer);
		}
		_renderKit.addRenderer(family, rendererType, renderer);
	}*/

	public void addRenderer(String family, String rendererType, Renderer renderer) {
		
		_renderKit.addRenderer(family, rendererType, renderer);	
		renderer = _renderKit.getRenderer(family, rendererType);
		if (!(renderer instanceof AjaxRenderer)) 
		{
			renderer = new DefaultAjaxRenderer(renderer);
		}
		String ajaxKey = family + rendererType;
		synchronized(_rendererCache)
		{
			_rendererCache.put(ajaxKey, renderer);
		}
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.render.RenderKit#getRenderer(java.lang.String, java.lang.String)
	 */
/*	public Renderer getRenderer(String family, String rendererType) {
		Renderer renderer = _renderKit.getRenderer(family, rendererType);
		FacesContext fc = FacesContext.getCurrentInstance();
		if ((renderer != null) && !(renderer instanceof AjaxRenderer)) {
			renderer = new DefaultAjaxRenderer(renderer);
		}
		return renderer;
	}*/

	public Renderer getRenderer(String family, String rendererType) {
		
		Renderer renderer = null;

		String ajaxKey = family + rendererType;
		renderer = (Renderer) _rendererCache.get(ajaxKey); // we'll assume it is there cause it 
														// should have been set in teh addRenderer
		
		// this would only be needed if somehow a renderer gets added to 
		//the real renderkit without using addrenderer
		if (renderer == null) 
		{
			renderer = _renderKit.getRenderer(family, rendererType);
			if (renderer != null) 
			{
				if (!(renderer instanceof AjaxRenderer)) 
				{
					renderer = new DefaultAjaxRenderer(renderer);
				}
				synchronized(_rendererCache)
				{
					_rendererCache.put(ajaxKey, renderer);
				}
			}
		}

		
		return renderer;
	}
	
	
	
	/* (non-Javadoc)
	 * @see javax.faces.render.RenderKit#getResponseStateManager()
	 */
	public ResponseStateManager getResponseStateManager() {
		//return _renderKit.getResponseStateManager();
		return this.RSM;
	}

	/* (non-Javadoc)
	 * @see javax.faces.render.RenderKit#createResponseWriter(java.io.Writer, java.lang.String, java.lang.String)
	 */
	public ResponseWriter createResponseWriter(Writer writer, String contentTypeList, String characterEncoding) {
		return _renderKit.createResponseWriter(writer, contentTypeList, characterEncoding);
	}

	/* (non-Javadoc)
	 * @see javax.faces.render.RenderKit#createResponseStream(java.io.OutputStream)
	 */
	public ResponseStream createResponseStream(OutputStream out) {
		return _renderKit.createResponseStream(out);
	}

}
