/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit;

import java.util.Iterator;

import java.util.Map;
import java.util.HashMap;

import javax.faces.context.FacesContext;
import javax.faces.render.RenderKit;
import javax.faces.render.RenderKitFactory;




/**
 * AjaxRenderKitFactory is an implementation of RenderKitFactory which will return an implementation of
 * AjaxRenderKit when the current request is a Ajax one.
 * 
 * @author mark_wallace@ie.ibm.com
 */
public class AjaxRenderKitFactory extends RenderKitFactory {

	private RenderKitFactory _renderKitFactory;
	
	private Map _renderKits;

	/**
	 * Construct an AjaxFacesContextFactory
	 */
	public AjaxRenderKitFactory(RenderKitFactory delegate) {
		if (delegate == null) {
			throw new NullPointerException(
					"Invalid FacesContextFactory delegate");
		}
		_renderKitFactory = delegate;
		_renderKits = new HashMap();
	}

	
	/* (non-Javadoc)
	 * @see javax.faces.render.RenderKitFactory#addRenderKit(java.lang.String, javax.faces.render.RenderKit)
	 */
	public void addRenderKit(String renderKitId, RenderKit renderKit) {
		_renderKitFactory.addRenderKit(renderKitId, renderKit);
		if(!(renderKit instanceof AjaxRenderKit))
		{
			renderKit = new AjaxRenderKit(renderKit);
			synchronized(_renderKits)
			{
				_renderKits.put(renderKitId, renderKit);
			}
		}
	}

	/* (non-Javadoc)
	 * @see javax.faces.render.RenderKitFactory#getRenderKit(javax.faces.context.FacesContext, java.lang.String)
	 */
	public RenderKit getRenderKit(FacesContext context, String renderKitId) {
		// TODO perhaps this should be optimized to wrap all RenderKits just once
		
		RenderKit renderKit = (RenderKit)_renderKits.get(renderKitId); // we'll asume it is there 

		// this would only be needed if somehow a renderkit gets added to 
		// after teh ajaxrenderkitfactory is instancicated
		if (renderKit == null)
		{
			renderKit = _renderKitFactory.getRenderKit(context, renderKitId);
			if(renderKit != null)
			{
				renderKit = new AjaxRenderKit(renderKit);
				synchronized(_renderKits)
				{
					_renderKits.put(renderKitId, renderKit);
				}
			}
		}

		return renderKit;
	}

	/* (non-Javadoc)
	 * @see javax.faces.render.RenderKitFactory#getRenderKitIds()
	 */
	public Iterator getRenderKitIds() {
		return _renderKitFactory.getRenderKitIds();
	}

}
