/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.component.UIMessage;
import javax.faces.component.UIMessages;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.ConverterException;
import javax.faces.render.Renderer;


import com.ibm.faces.application.ViewHandlerImpl;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.faces.util.AjaxUtil;


import javax.faces.application.Application;
import javax.faces.application.StateManager;
import javax.faces.application.ViewHandler;


/**
 * Default implemntation of an Ajax aware Renderer which will change the rendering state to false if
 * the current request is an Ajax one and the associated UI Component should not be rendered.
 * 
 * @author mark_wallace@ie.ibm.com
 */
public class DefaultAjaxRenderer extends Renderer implements AjaxRenderer {
	
	private Renderer _renderer;
	private boolean _isAjaxRenderer;
	
	private static String hidden_state_input_done = "com.ibm.faces.JWLAJAX.StateInputFieldDone";
	
	
	public DefaultAjaxRenderer(Renderer renderer) {
		_renderer = renderer;
		_isAjaxRenderer = (renderer instanceof AjaxRenderer);
	}
	
	public Renderer getRealRenderer()
	{
		return _renderer;
	}
	
	
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {
		//checkRendering(context, component);
		checkBeginRendering(context, component);
		_renderer.encodeBegin(context, component);
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#encodeChildren(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeChildren(FacesContext context, UIComponent component)
			throws IOException {
		//checkRendering(context, component); // children should be encoded on their own merits
		_renderer.encodeChildren(context, component);
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {
		//checkRendering(context, component);	
		checkForFormEnd(context, component);
		_renderer.encodeEnd(context, component);		
		checkEndRendering(context, component);
	}

	/*
	 * Check the rendering state is correct for an Ajax request for the encode begin of a component
	 * used to turn on rendering at the start of the target
	 */
	protected void checkBeginRendering(FacesContext context, UIComponent component) {
		if (!(AjaxUtil.isAjaxRequest(context))) {
			return;
		}
		
		
		if(AjaxUtil.isRendering(context)) 						// if you started rendering 
			AjaxUtil.setRendering(context,true);					// then continue rendering (children)
		else if(AjaxUtil.isRendered(context, component)) // if you've come to the end of the target component
			AjaxUtil.setRendering(context,true);					// every child component should render based on it's own merits
		
				
		String del_start = (String) context.getExternalContext().getRequestMap().get(HxClientRenderUtil.DELIM_START);
		
		
		if(AjaxUtil.isRendering(context) && HxClientRenderUtil.isPortal(context.getExternalContext().getRequest()))
			if(del_start == null)
			{
				try{
					ResponseWriter writer = context.getResponseWriter();
					writer.write("<head title=\"PORTALcontent\">");
					context.getExternalContext().getRequestMap().put(HxClientRenderUtil.DELIM_START, "true");
				}catch(IOException ioe)
				{
					
				}
			}					
		
	}
	
	/*
	 * Check the rendering state is correct for an Ajax request for the encode end of a component
	 * used to turn of the rendering at the end of the target
	 */
	protected void checkEndRendering(FacesContext context, UIComponent component) {
		if (!(AjaxUtil.isAjaxRequest(context))) {
			return;
		}
		
		if(AjaxUtil.isRendering(context))						// if you're rendereing and 
			if(AjaxUtil.isRendered(context, component)) // if you've come to the end of the target component
				AjaxUtil.setRendering(context,false);			// the everything after this shouldn't render

	}	
	
	/*
	 * Check to see if the current encode end is of a form
	 * */
	protected void checkForFormEnd(FacesContext context, UIComponent component)
	{
		if (!(AjaxUtil.isAjaxRequest(context))) {
			return;
		}
		
		
		if(component instanceof UIForm && (context.getExternalContext().getRequestMap().get(hidden_state_input_done) == null))
		{			
			AjaxUtil.setRendering(context, true);
		
			// how do we get these objects
			// serialize view via StateManager
			
			Application app = context.getApplication();
			StateManager asm = app.getStateManager();
			ViewHandler vh = app.getViewHandler();
			
			// write the serialiased view via AjaxResponseStateManagerImpl.writeState
			try{
				if(vh instanceof ViewHandlerImpl) // our jwl enhanced server saving one 
					vh.writeState(context);
				else
				{
					StateManager.SerializedView sv = asm.saveSerializedView(context);
					asm.writeState(context, sv);
				}
			}catch(IOException ioe)
			{
				ioe.printStackTrace(System.out);
			}
			
			AjaxUtil.setRendering(context,false);
			
			context.getExternalContext().getRequestMap().put(hidden_state_input_done, "DONE");
		}
	}
	
	
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#getRendersChildren()
	 */
	public boolean getRendersChildren() {
		return _renderer.getRendersChildren();
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#convertClientId(javax.faces.context.FacesContext, java.lang.String)
	 */
	public String convertClientId(FacesContext context, String arg1) {
		return _renderer.convertClientId(context, arg1);
	}
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void decode(FacesContext context, UIComponent component)  {
		_renderer.decode(context, component);
	}
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#getConvertedValue(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.Object)
	 */
	public Object getConvertedValue(FacesContext context, UIComponent component,
			Object convertee) throws ConverterException {
		return _renderer.getConvertedValue(context, component, convertee);
	}
}
