/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.io.Writer;

import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.context.FacesContext;

import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.util.HtmlUtil;
import com.ibm.faces.util.JavaScriptUtil;
import javax.faces.render.Renderer;

/**
 * Abstract base class for selection control renderers.
 */
abstract class AbstractSelectRenderer extends Renderer {

    /**
     * Gets the JavaScript variable name associated with the selection group.
     * @param context
     * @param component
     * @return
     */
    protected String getJsSelectionGroupVariable(
        FacesContext context,
        UIComponent component) {
        UIData data = findDataComponent(component);
        if (data != null) {
            String id = data.getClientId(context);
            return RowSelectRenderer.getJsSelectionGroupVariable(id);
        }
        return null;
    }

    /**
     * Locates the data component on which this renderer operates.
     * Uses either the "for" attribute or an appropriate parent.
     * @param component
     * @return  the data component or null if not found
     */
    private UIData findDataComponent(UIComponent component) {
        String dataId = (String) component.getAttributes().get("for"); //$NON-NLS-1$

        UIComponent data = null;

        if (dataId != null) {
            data = component.findComponent(dataId);
        }
        if (data == null) {
            data = findDataParent(component);
        }

        return (UIData) data;
    }

    private UIComponent findDataParent(UIComponent component) {
        if (component == null) {
            return null;
        }
        if (component instanceof UIData) {
            return component;
        }
        return findDataParent(component.getParent());
    }

    protected void writeJavaScript(
        FacesContext context,
        UIComponent component,
        String script)
        throws IOException {

        UIScriptCollector scriptCollector = UIScriptCollector.find(component);

        if (scriptCollector != null) {
			HxClientRenderUtil.initJSLibraries(scriptCollector, context);
            scriptCollector.addScriptOnce(script);
        }
        else {
            //failsafe
            Writer writer = context.getResponseWriter();
            writer.write(JavaScriptUtil.getStartTag());
            writer.write(script);
            writer.write(JavaScriptUtil.getEndTag());
        }
    }

    protected void writeInputSubmit(Writer writer, String id, String label, String onclick, String styleClass) throws IOException {
        HtmlUtil.writeTagStartOpen(writer, "input", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "type", "button"); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "name", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "value", label); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "onclick", onclick); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
        HtmlUtil.writeTagStartCloseLn(writer, true);
    }

}
