/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;


import java.io.IOException;
import java.util.Iterator;

import javax.faces.application.Application;
import javax.faces.application.ViewHandler;
import javax.faces.component.UIComponent;
import javax.faces.component.UIParameter;
import javax.faces.component.UIViewRoot;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.render.Renderer;

import com.ibm.faces.component.IClientValueHolder;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.context.AjaxAwareExternalContext;
import com.ibm.faces.jwlajaxencoder.AjaxUrlEncoder;
import com.ibm.faces.jwlajaxencoder.AjaxUrlFactory;
import com.ibm.faces.renderkit.AjaxRenderer;
import com.ibm.faces.util.AjaxUtil;
import com.ibm.faces.util.StringUtil;

/**
 * Renderer for {@link com.ibm.faces.component.UIAjax UIAjax} component.
 * 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.AjaxContentTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIAjax}
 */
public class AjaxBaseRenderer extends Renderer{

	protected final char paramsdelimiter = ';';

	protected final String DEFAULTACTION = "LOCKED";
    public static final String AJAX_ROWID = "$$AJAXROW$$"; //$NON-NLS-1$
    
    /*
     * Private method to find a component in the component tree using it's server id and then
     * getting it to generate it's own client id.
     * */
    protected String getTargetsClientId(FacesContext context, UIComponent component, String target)
    {
    	String resolvedTarget = target;
		// resolve the JSF id into a rendered id
    	if(target != null && (target.trim().length() > 0))
    	{
    		String id = target;
    		// Special case for components in dataTable. If target id begins with a special marker -- remove the marker first
    		if(target.startsWith(AJAX_ROWID)) {
    			id = target.substring(AJAX_ROWID.length());
    		}
    		resolvedTarget = HxClientRenderUtil.convertComponentIdToClientId(id, component);
    		// Special case for components in dataTable. If target id begins with a special marker -- put the marker back into the resolved id    			
			if(target.startsWith(AJAX_ROWID)) {
				resolvedTarget = AJAX_ROWID + resolvedTarget;
			}
    	}
    	else
    	{
    		UIComponent parent = component.getParent();
			resolvedTarget = parent.getClientId(context);
    	}
    	
		return resolvedTarget;
    }
 
    /*
     * Private method to find any f:params of teh behavior
     * and turn it into a URL type string &name=value, etc.
     * */    
	public String dofparams(FacesContext context, UIComponent component, String existingHref)
	{
		StringBuffer hrefBuff = new StringBuffer();
		Iterator children = component.getChildren().iterator();
		UIComponent child;
		boolean first = true;
		if(existingHref.indexOf("?") > 0)
			first = false;
		
		while(children.hasNext()){
			child = (UIComponent)children.next();
			if(child instanceof UIParameter){
				if(first)
				{
					hrefBuff.append("?");
					first = false;
				}
				else
					hrefBuff.append("&");				
				hrefBuff.append(((UIParameter)child).getName()).append("=").append((((UIParameter)child).getValue()));
			}
		}
		return hrefBuff.toString();
	}

    
    
     /*
     * Private method to find any params atrribute of teh behavior
     * and turn it into a name value (clientID) pair.
     * */    
    protected String manipulationParamsString(FacesContext context, UIComponent component, String params)
    {
    	if(params == null || (params!=null && params.trim().length() == 0))
    		return null;
    	
		String [] ps = StringUtil.splitToArray(params,paramsdelimiter);
		
		String [] psr = new String [ps.length];
		
		for (int i = 0; i < ps.length; i++)
		{		
			String id = ps[i];
			// Special case for components in dataTable. If target id begins with a special marker -- remove the marker first
    		if(ps[i].startsWith(AJAX_ROWID)) {
    			id = ps[i].substring(AJAX_ROWID.length());
    		}
			UIComponent targetComponent = Utils.findNearestComponent(component, id);
			if(targetComponent != null) {
				if(targetComponent instanceof IClientValueHolder) {
					psr[i] = ((IClientValueHolder)targetComponent).getValueHolderClientId(context, targetComponent);
				}
				else {
					Renderer renderer = context.getRenderKit().getRenderer(targetComponent.getFamily(), targetComponent.getRendererType());
					if(renderer instanceof AjaxRenderer) {
						renderer = ((AjaxRenderer)renderer).getRealRenderer();
					}
					if(renderer instanceof IClientValueHolder) {
						psr[i] = ((IClientValueHolder)renderer).getValueHolderClientId(context, targetComponent);
					}
					else {
						psr[i] = targetComponent.getClientId(context);
					}
				}
			}
			else {
	    		UIComponent parent = component.getParent();
				psr[i] = parent.getClientId(context);
			}

    		// Special case for components in dataTable. If target id begins with a special marker -- put the marker back into the resolved id    			
			if(ps[i].startsWith(AJAX_ROWID)) {
				psr[i] = AJAX_ROWID + psr[i];
			}
		}
		
		StringBuffer pbuff = new StringBuffer();
		
		for (int j = 0; j < psr.length; j++)
		{
			pbuff.append(psr[j]);
			if((psr.length > 1) && (j < (psr.length -1)))
				pbuff.append(paramsdelimiter);
		}		
		
		return pbuff.toString();
    }
 
    /*
     * private
     */
    protected void doAddLibraryAndInit(FacesContext context, UIComponent component)
    {
    	UIScriptCollector scriptCollector = UIScriptCollector.find(component);
    	
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
		HxClientRenderUtil.addScriptLibrary("hxclient_bc.js", component);
		
		// Output the javascript library
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
    	
    }
    
 
    /*
     * private need to build url so that
     */
    protected String buildURL(FacesContext context, UIComponent component, String target, boolean dopost)
    {
		String d = null;
    	
		ExternalContext ec = context.getExternalContext();
		
    	UIViewRoot vr = context.getViewRoot();
    	String vid = vr.getViewId();
		
    	Application app = context.getApplication();
    	ViewHandler vh = app.getViewHandler();
    	String url = vh.getActionURL(context, vid);
    	
    	StringBuffer hrefbuff = new StringBuffer(url);    
		hrefbuff.append("?$$ajaxid=");
		hrefbuff.append(target);
		if(dopost)
			hrefbuff.append("&$$ajaxmode=axfull");
		else
			hrefbuff.append("&$$ajaxmode=axpartial");
		
		String extraparams = null;
		
		extraparams = dofparams(context, component, hrefbuff.toString());
		
		if(extraparams != null && extraparams.length() >0)
			hrefbuff.append(extraparams);
		
		//hrefbuff.append("&jwluniquemarker=");
		//hrefbuff.append((new GregorianCalendar()).getTimeInMillis());
		
		/* EXPERIMENTAL*/
		
		if(ec instanceof AjaxAwareExternalContext)
			d = ((AjaxAwareExternalContext)ec).encodeAjaxUrl(hrefbuff.toString());
		else
		{
			AjaxUrlEncoder aue = AjaxUrlFactory.getAjaxUrlEncoder(context);
			d = aue.encodeAjaxUrl(context, hrefbuff.toString());
			
			//System.out.println(">>>>>>>>>>>>>>>>>>> type - "+aue.getClass().getName());
		}	
		
		/* EXPERIMENTAL*/
		/*
		if(ec instanceof AjaxAwareExternalContext)
			d = ((AjaxAwareExternalContext)ec).encodeAjaxUrl(hrefbuff.toString());
		else
			d = ec.encodeActionURL(hrefbuff.toString());
		*/
		
		if((d != null) && (d.lastIndexOf("#") >0))
		{
			String dr = d.substring(0, d.lastIndexOf("#")); // this is a fix to get IE and Portal working together
			
			return dr;
		}
		else
			return d;
    	
    }
    
	protected void ajaxReRendering(FacesContext context, UIComponent component, boolean dopost, boolean isparent) throws IOException {
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		
		// InitialiseDefaults
		String id			= null;
		String title		= null;			//Defaults ->
		String onStart		= null;			//TODO: Display an animated gif in the top right corner, something like gmail, but without text (no need to localize it)
		String onComplete	= null;			//TODO: Remove the above display
		String onError		= null;			//TODO: Display an error in the error-message field (see below), otherwise display a popup.
		String target		= null;		// Mandatory
		String inprogress	= null;
		
		String temp 		= null;


		//get attribute values from the component
		target = (String)component.getAttributes().get("target");
	

		target = getTargetsClientId(context, component, target);
	
		String href = buildURL(context, component, target, dopost);
		
		
		StringBuffer ajaxScript = new StringBuffer("if(");
		// construct JavaScript String
		ajaxScript.append(HxClientRenderUtil.HX_VAR_NAME) 
					.append(".getBehaviorById(\"")					
					.append(target);
		
		if(dopost)			
			ajaxScript.append("\", \"ajaxsubmit\"");
		else			
			ajaxScript.append("\", \"ajaxget\"");
					
        ajaxScript.append(", \"onget\"))") 
					.append(HxClientRenderUtil.HX_VAR_NAME)		
					.append(".getBehaviorById(\"")					
					.append(target);
        
		if(dopost)			
			ajaxScript.append("\", \"ajaxsubmit\"");
		else			
			ajaxScript.append("\", \"ajaxget\"");
		
		ajaxScript.append(", \"onget\").setAttribute(\"href:") ;

		ajaxScript.append(href);
		ajaxScript.append("\"");			
	
		ajaxScript.append(");\n");
		
		// render JavaScript to page
		if(!isparent)
			AjaxUtil.setRendering(context,true);
		scriptCollector.addScript(ajaxScript.toString());
		if(!isparent)
			AjaxUtil.setRendering(context,false);
	}
    
    
}
