/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlAjaxExternalRequest;

/**
 * Renderer for {@link com.ibm.faces.component.UIAjax UIAjax} component.
 * 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.AjaxContentTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIAjax}
 */
public class AjaxExternalRequestRenderer extends AjaxBaseRenderer implements IScriptContributor {

	/**
	 * Renders the beginning of the component.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}

		if(!component.isRendered())
			return;

	}
	
	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		encode(context, component);
	}

	/**
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 * @param context
	 * @param component
	 * @throws IOException
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}

		if(!component.isRendered())
			return;
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);

		doAddLibraryAndInit(context, component);
		if (HxClientRenderUtil.isContainedInPanel(component.getParent())) {
			encode(context, component);
		} else {
			scriptCollector.register(this, component);
		}
		
	}

	private void encode(FacesContext context, UIComponent component) throws IOException {
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		
		HtmlAjaxExternalRequest acomponent = null;
		
		if (component instanceof HtmlAjaxExternalRequest) // check if AjaxUpdateContent Component
			acomponent = (HtmlAjaxExternalRequest)component;
		
		// InitialiseDefaults		
		String title		= null;			//Defaults ->
		String onStart		= null;			//TODO: Display an animated gif in the top right corner, something like gmail, but without text (no need to localize it)
		String onComplete	= null;			//TODO: Remove the above display
		String onError		= null;			//TODO: Display an error in the error-message field (see below), otherwise display a popup.
		String href			= null;
		String hreflang		= null;
		String source 		= null;  // manditory
		String charset		= null;
		String target		= null;		// Mandatory
		String inprogress	= null;
		String params		= null;
		String paramsfields = null;
		
		String temp 		= null;
		
		//get attribute values from the component

		
		if (null != acomponent) 
		{			
			if (null != (temp = acomponent.getTitle())) {
				title = temp;
			}
			if (null != (temp = acomponent.getOnstart())) {
				onStart = temp;
			}
			if (null != (temp = acomponent.getOncomplete())) {
				onComplete = temp;
			}
			if (null != (temp = acomponent.getOnerror())) {
				onError = temp;
			}
			if (null != (temp = acomponent.getHref())) {
				href = temp;
			}
			if (null != (temp = acomponent.getHreflang())) {
				hreflang = temp;
			}
			if (null != (temp = acomponent.getSource())) {
				source = temp;
			}
			if (null != (temp = acomponent.getCharset())) {
				charset = temp;
			}			
			if (null != (temp = acomponent.getInProgress())) {
				inprogress = temp;
			}
			if (null != (temp = acomponent.getParams())) {
				params = temp;
			}
			
			target = acomponent.getTarget();
			
		} else 
		{
			if (null != (temp = (String)component.getAttributes().get("title"))) {
				title = temp;
			}
			if (null != (temp = (String)component.getAttributes().get("onstart"))) {
				onStart = temp;
			}
			if (null != (temp = (String)component.getAttributes().get("oncomplete"))) {
				onComplete = temp;
			}
			if (null != (temp = (String)component.getAttributes().get("onerror"))) {
				onError = temp;
			}
			if (null != (temp = (String)component.getAttributes().get("href"))) {
				href = temp;
			}
			if (null != (temp = (String)component.getAttributes().get("hreflang"))) {
				hreflang = temp;
			}
			if (null != (temp = (String)component.getAttributes().get("source"))) {
				source = temp;
			}
			if (null != (temp = (String)component.getAttributes().get("charset"))) {
				charset = temp;
			}
			if (null != (temp = (String)component.getAttributes().get("inProgress"))) {
				inprogress = temp;
			}
			if (null != (temp = (String)component.getAttributes().get("params"))) {
				params = temp;
			}
			
			target = (String)component.getAttributes().get("target");
			
		}
		
		// resolve target in a client id // assume target is valid after this
		target = getTargetsClientId(context, component, target);		

		// take params string and turn in the param-feilds string		
		paramsfields = manipulationParamsString(context, component, params);
	
		
		String extraparams = null;
		
		extraparams = dofparams(context, component, href);
		
		if(extraparams != null && extraparams.length() >0)
			href = href + extraparams;
		
		/*
		 * hX_4.addBehavior("form1:panelHelp", "onget", new hX_4.JSFBehaviorAjaxGetExternal("on-error:return doError(this, event);", 
		 * "source:form1:panelHelp", "params:lookUpText", "param-fields:form1:lookUpText", "on-start:return doPrep(this, event);", 
         * "href:JWL3-TEST-AJAX-External1-source1.jsp"));
		 */
		
		if(inprogress == null || (inprogress != null && inprogress.trim().length() < 0))
			inprogress = DEFAULTACTION;
		
		boolean needcomma = false;
		
		StringBuffer ajaxScript = new StringBuffer();
		// construct JavaScript String
		ajaxScript.append(HxClientRenderUtil.HX_VAR_NAME) 
					.append(".addBehavior(\"")
					.append(target)
					.append("\", \"onget\"")
					.append(", new ") 
					.append(HxClientRenderUtil.HX_VAR_NAME)		
					.append(".JSFBehaviorAjaxGetExternal(");
		
		if(inprogress != null && inprogress.trim().length() >0)
		{
			ajaxScript.append("\"progress-actions:");
			ajaxScript.append(inprogress);
			ajaxScript.append("\"");
			needcomma = true;
		}

		if(params != null && params.trim().length() >0)
		{
			if(needcomma)
				ajaxScript.append(", ");
			ajaxScript.append("\"params:");
			ajaxScript.append(params);
			ajaxScript.append("\", ");
			ajaxScript.append("\"param-fields:");
			ajaxScript.append(paramsfields);
			ajaxScript.append("\"");
			needcomma = true;
		}		
		
		if(title != null && title.trim().length() >0)
		{
			if(needcomma)
				ajaxScript.append(", ");
			ajaxScript.append("\"title:");
			ajaxScript.append(title);
			ajaxScript.append("\"");
			needcomma = true;
		}
		
		if(onError != null && onError.trim().length() >0)
		{
			if(needcomma)
				ajaxScript.append(", ");
			ajaxScript.append("\"on-error:");
			ajaxScript.append(onError);
			ajaxScript.append("\"");
			needcomma = true;
		}
		
		if(onComplete != null && onComplete.trim().length() >0)
		{
			if(needcomma)
				ajaxScript.append(", ");
			ajaxScript.append("\"on-complete:");
			ajaxScript.append(onComplete);
			ajaxScript.append("\"");
			needcomma = true;
		}		
		
		if(onStart != null && onStart.trim().length() >0)
		{
			if(needcomma)
				ajaxScript.append(", ");
			ajaxScript.append("\"on-start:");
			ajaxScript.append(onStart);
			ajaxScript.append("\"");
			needcomma = true;
		}

		if(href != null && href.trim().length() >0)
		{
			if(needcomma)
				ajaxScript.append(", ");
			ajaxScript.append("\"href:");
			ajaxScript.append(href);
			ajaxScript.append("\"");
			needcomma = true;
		}
		
		if(hreflang != null && hreflang.trim().length() >0)
		{
			if(needcomma)
				ajaxScript.append(", ");
			ajaxScript.append("\"hreflang:");
			ajaxScript.append(hreflang);
			ajaxScript.append("\"");
			needcomma = true;
		}

		if(source != null && source.trim().length() >0)
		{
			if(needcomma)
				ajaxScript.append(", ");
			ajaxScript.append("\"source:");
			ajaxScript.append(source);
			ajaxScript.append("\"");
			needcomma = true;
		}
		
		if(charset != null && charset.trim().length() >0)
		{
			if(needcomma)
				ajaxScript.append(", ");
			ajaxScript.append("\"charset:");
			ajaxScript.append(charset);
			ajaxScript.append("\"");			
		}		
		ajaxScript.append("));\n");
		
		// render JavaScript to page
		scriptCollector.addScript(ajaxScript.toString());
		
	}



}
