/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlBackchannelHttpRequest;
import com.ibm.faces.util.JavaScriptUtil;

/**
 * Renderer for {@link javax.faces.component.UIBackchannel} component, 
 * that creates markup on the page.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.backchannelHttpTag}<br>
 * <b>Components:</b> {@link javax.faces.component.UIBackchannel}
 */
public class BackchannelHttpRequestRenderer extends Renderer implements IScriptContributor {
	
	/** 
	 * Renders the component.
	 * <p>
	 * Creates the basic IFrame tag, referring to the constant initial page
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		
		if (context == null || component == null) {
			throw new NullPointerException();
		}
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		scriptCollector.register(this, component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		
		HtmlBackchannelHttpRequest backchannelComponent = null;
		if (component instanceof HtmlBackchannelHttpRequest) {
			backchannelComponent = (HtmlBackchannelHttpRequest) component;
		}
	}
	
	/**
	 * Renders the ending of the component.
	 * <p>
	 * Creates the closing TABLE tag.
	 * <p>
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		
		if (context == null || component == null) {
			throw new NullPointerException();
		}
		// Render the ending of this panel
		//ResponseWriter writer = context.getResponseWriter();
		//writer.endElement("iframe"); //$NON-NLS-1$
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#getRendersChildren()
	 */
	public boolean getRendersChildren() {
		return false;
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		
		HtmlBackchannelHttpRequest backchannelComponent = null;
		if (component instanceof HtmlBackchannelHttpRequest) {
			backchannelComponent = (HtmlBackchannelHttpRequest) component;
		}
		
		String callback, errorfn, startfn, method;
		Boolean async, post, iframe;
		if (backchannelComponent != null) {
			callback = backchannelComponent.getOncomplete();
			method = backchannelComponent.getMode();
			errorfn = backchannelComponent.getOnerror();
			startfn = backchannelComponent.getOnstart();
			async = new Boolean(backchannelComponent.isAsynchronous());
			iframe = new Boolean(backchannelComponent.isUseiframe());
		} else {
			callback = (String) component.getAttributes().get("oncomplete"); //$NON-NLS-1$
			method = (String) component.getAttributes().get("mode"); //$NON-NLS-1$
			errorfn = (String) component.getAttributes().get("onerror"); //$NON-NLS-1$
			startfn = (String) component.getAttributes().get("onstart"); //$NON-NLS-1$
			async = (Boolean) component.getAttributes().get("asynchronous"); //$NON-NLS-1$
			iframe = (Boolean) component.getAttributes().get("useiframe"); //$NON-NLS-1$
		}
		if (method != null && method.equalsIgnoreCase("post")) { //$NON-NLS-1$
			post = Boolean.TRUE;
		} else {
			post = Boolean.FALSE;
		}
		
		String id = JavaScriptUtil.makeVariableName(component
				.getClientId(context));
		StringBuffer params = new StringBuffer();
		
		params.append("\"") //$NON-NLS-1$
			  .append(id)
			  .append("\",") //$NON-NLS-1$
			  .append((callback != null) ? callback : "null") //$NON-NLS-1$
			  .append(",") //$NON-NLS-1$
			  .append((post == Boolean.TRUE) ? post : Boolean.FALSE)
			  .append(",") //$NON-NLS-1$
			  .append((async == Boolean.TRUE) ? async : Boolean.FALSE)
			  .append(","); //$NON-NLS-1$
		if (iframe == Boolean.TRUE) {
			params.append("\"__") //$NON-NLS-1$
				  .append(id)
				  .append("__iframe\""); //$NON-NLS-1$
		} else {
			params.append("null"); //$NON-NLS-1$
		}
		if (errorfn != null && errorfn.length() > 0) {
			params.append(",") //$NON-NLS-1$
				  .append(errorfn)
				  .append(""); //$NON-NLS-1$
		}
		
		StringBuffer scriptText = new StringBuffer();
		scriptText.append("var ") //$NON-NLS-1$
				  .append(id)
				  .append(" = new ") //$NON-NLS-1$
				  .append(HxClientRenderUtil.HX_VAR_NAME)
				  .append(".HTTPBackchannel(") //$NON-NLS-1$
				  .append(params).append(");\n"); //$NON-NLS-1$
		
		writer.write(scriptText.toString());
	}
	
}
