/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.render.Renderer;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlBehavior;
import com.ibm.faces.component.html.HtmlInputMiniCalendar;

/**
 * @author pandrews
 *
 */
public class BehaviorRenderer extends Renderer implements IScriptContributor{

	
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
		
		if(HxClientRenderUtil.isContainedInPanel(component)){
			//rendered by parent
			encode(context,component,false);
		} else {
			scriptCollector.register(this,component);
		}
		
	}


	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		encode(context,component,true);
		
	}
	
	private void encode(FacesContext context, UIComponent component, boolean writeNow) {
		
		HtmlBehavior behaviorComponent = null;
		if ( component instanceof HtmlBehavior ){
			behaviorComponent = (HtmlBehavior)component;
		}
		
		
		String target, targetFacet, event, action, targetAction, function;
		if ( behaviorComponent != null ){
			target = behaviorComponent.getTarget();
			targetFacet = behaviorComponent.getTargetFacet();
			event = behaviorComponent.getEvent();
			action = behaviorComponent.getBehaviorAction();
			targetAction = behaviorComponent.getTargetAction();
			function = behaviorComponent.getOnActionFunction();
		}else{
			target =  (String)component.getAttributes().get("target"); //$NON-NLS-1$
			targetFacet =  (String)component.getAttributes().get("targetFacet"); //$NON-NLS-1$
			event = (String)component.getAttributes().get("event"); //$NON-NLS-1$
			action = (String)component.getAttributes().get("behaviorAction"); //$NON-NLS-1$
			targetAction = (String)component.getAttributes().get("targetAction"); //$NON-NLS-1$
			function = (String)component.getAttributes().get("onActionFunction"); //$NON-NLS-1$
		}
		
		if(event == null || (action == null && function == null)){
			//no event or action has been defined, exit
			return;
		}
		
		UIComponent forComponent = null;
		if(target != null){
			if(!(target.startsWith("#"))){
				forComponent = component.findComponent(target);
				if(forComponent == null){
					//try from view root
					forComponent = Utils.findComponent(context.getViewRoot(),target);
				}
			}
		} else {
			//target is null so apply to parent component
			forComponent = component.getParent();
			if(forComponent == null){
				//we don't have a component to apply this click to, end it all
				return;
			}
		}
		//if facet is set and component for id is found , apply sctipt to facet component
		if(forComponent !=null && targetFacet != null){
			UIComponent facetComponent = forComponent.getFacet(targetFacet);
			if(facetComponent != null){
				target = facetComponent.getClientId(context);
			} else {
				//cannot find specified facet, nothing to apply script to so return
				return;
			}
		//facet not set so if a component has been found set target to it clientId
		} else if(forComponent != null) {
			
			if(forComponent instanceof HtmlInputMiniCalendar && ("onchange".equalsIgnoreCase(event) ||  "onblur".equalsIgnoreCase(event) || "onfocus".equalsIgnoreCase(event)))
				target = forComponent.getClientId(context)+MiniCalendarRenderer._INPUT;
			else
				target = forComponent.getClientId(context);
		}
		
		if ( targetAction != null) {
			targetAction = HxClientRenderUtil.convertComponentIdsToClientIds(targetAction, component);
		}
		
		StringBuffer script = new StringBuffer();
		
		script.append("\n"); //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".addBehavior(\"");
		script.append(target);
		script.append("\", \""); //$NON-NLS-1$
		script.append(event);
		script.append("\", new ");
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".JSFBehaviorGeneric(");
		Boolean started = Boolean.FALSE;
		if (null != action) {
			script.append("\"action:")
			.append(action)
			.append("\"");
			started = Boolean.TRUE;
		}
		if (null != targetAction) {
			if(started.booleanValue()){
				script.append(", ");
			} else {
				started = Boolean.TRUE;
			}
			script.append("\"target:")
			.append(targetAction)
			.append("\"");
		}
		if (null != function) {
			if(started.booleanValue()){
				script.append(", ");
			} else {
				started = Boolean.TRUE;
			}
			script.append("\"function:")
			.append(function)
			.append("\"");
		}
		script.append("));"); //$NON-NLS-1$
		
		//add script
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		if(writeNow){
			scriptCollector.writeScriptOnce(script.toString(),context,false);
		} else {
			scriptCollector.addScriptOnce(script.toString());
		}
	
	}
	
}
