/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Iterator;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;




import com.ibm.faces.component.html.HtmlPanelBox;

/**
 * Renderer for {@link javax.faces.component.UIPanel UIPanel} component, 
 * that aligns all children either vertically or horizontally.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Panel_BoxTag}<br>
 * <b>Components:</b> {@link javax.faces.component.UIPanel}
 */
public class BoxRenderer extends Renderer {
	
	/** 
	 * Renders the beginning of the component.
	 * <p>
	 * Creates the opening TABLE tag. Childrwn will be rendered
	 * in table cells later on in {@link BoxRenderer#encodeChildren encodeChildren}.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */   
    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {

        if (context == null || component == null) {
            throw new NullPointerException();
        }
        HtmlPanelBox boxComponent = null;
        if ( component instanceof HtmlPanelBox ){
        	boxComponent = (HtmlPanelBox)component;
        }
        
        String panelClass;
        if ( boxComponent != null ){
        	panelClass = boxComponent.getStyleClass();
        }else{
			panelClass = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
        }
        
		ResponseWriter writer = context.getResponseWriter();
               
        writer.startElement("table",component);
        
        writer.writeAttribute("id",component.getClientId(context),null);
        
        if (panelClass != null) {
            writer.writeAttribute("class",panelClass,"styleClass");
        }
        
		if ( boxComponent != null ) {
			
			if ( boxComponent.getAlign() != null) {
				writer.writeAttribute(
					"align",
					boxComponent.getAlign(),
					null);
			}

			if ( boxComponent.getBgcolor() != null) {
				writer.writeAttribute(
					"bgcolor",
					boxComponent.getBgcolor(),
					null);
			}
			
			if ( boxComponent.getBorder() != null) {
				writer.writeAttribute(
					"border",
					boxComponent.getBorder(),
					null);
			}
			
			if ( boxComponent.getCellpadding() != null) {
				writer.writeAttribute(
					"cellpadding",
					boxComponent.getCellpadding(),
					null);
			}									

			if ( boxComponent.getCellspacing() != null) {
				writer.writeAttribute(
					"cellspacing",
					boxComponent.getCellspacing(),
					null);
			}
			
			if ( boxComponent.getHeight() != null) {
				writer.writeAttribute(
					"height",
					boxComponent.getHeight(),
					null);
			}

			if ( boxComponent.getStyle() != null) {
				writer.writeAttribute(
					"style",
					boxComponent.getStyle(),
					null);
			}			
			if ( boxComponent.getTitle() != null) {
				writer.writeAttribute(
					"title",
					boxComponent.getTitle(),
					null);
			}
			
			if ( boxComponent.getWidth() != null) {
				writer.writeAttribute(
					"width",
					boxComponent.getWidth(),
					null);
			}
			
			if ( boxComponent.getLang() != null) {
				writer.writeAttribute(
					"lang",
					boxComponent.getLang(),
					null);
			}
			
			if ( boxComponent.getDir() != null) {
				writer.writeAttribute(
					"dir",
					boxComponent.getDir(),
					null);
			}
			
		}else{
			String align = (String)component.getAttributes().get("align");
			if(align != null){
				writer.writeAttribute("align",align,"align");
			}
			String bgcolor = (String)component.getAttributes().get("bgcolor");
			if(bgcolor != null){
				writer.writeAttribute("bgcolor",bgcolor,"bgcolor");
			}
			String border = (String)component.getAttributes().get("border");
			if(border != null){
				writer.writeAttribute("border",border,"border");
			}
			String cellpadding = (String)component.getAttributes().get("cellpadding");
			if(cellpadding != null){
				writer.writeAttribute("cellpadding",cellpadding,"cellpadding");
			}
			String cellspacing = (String)component.getAttributes().get("cellspacing");
			if(cellspacing != null ){
				writer.writeAttribute("cellspacing",cellspacing,"cellspacing");
			}
			String height = (String)component.getAttributes().get("height");
			if(height != null){
				writer.writeAttribute("height",height,"height");
			}
			String style = (String)component.getAttributes().get("style");
			if(style != null) {
				writer.writeAttribute("style",style,"style");
			}
			String title = (String)component.getAttributes().get("title");
			if(title != null) {
				writer.writeAttribute("title",title,"title");
			}
			String width = (String)component.getAttributes().get("width");
			if(width != null){
				writer.writeAttribute("width",width,"width");
			}
			
			String lang = (String)component.getAttributes().get("lang");
			if(lang != null){
				writer.writeAttribute("lang",lang,"lang");
			}
			String dir = (String)component.getAttributes().get("dir");
			if(dir != null){
				writer.writeAttribute("dir",dir,"dir");
			}
					
		}
		
    }

	/** 
	 * Renders component's children.
	 * <p>
	 * Renders all child components as table cells either vertically
	 * or horizontally, depending on <code>layout</code> attribute.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
    public void encodeChildren(FacesContext context, UIComponent component)
        throws IOException {

        if (context == null || component == null) {
            throw new NullPointerException();
        }
        
		HtmlPanelBox boxComponent = null;
		if ( component instanceof HtmlPanelBox ){
			boxComponent = (HtmlPanelBox)component;
		}

		String layout;
		String valign;
		
		if( boxComponent != null ){
			layout = boxComponent.getLayout();
			valign = boxComponent.getValign();
		}else{
			layout = (String) component.getAttributes().get("layout"); //$NON-NLS-1$
			valign = (String) component.getAttributes().get("valign"); //$NON-NLS-1$
		}
		
        ResponseWriter writer = context.getResponseWriter();

        //RATLC01079228
		Boolean firstTr = Boolean.TRUE;
		Boolean childRendered = Boolean.FALSE;
		Iterator children =  component.getChildren().iterator();
        while(children.hasNext()) {
			UIComponent child = (UIComponent)children.next();
			if(child.isRendered()){
				if((firstTr.booleanValue()) || (layout != null && layout.equalsIgnoreCase("pageDirection"))) { //$NON-NLS-1$
			        writer.startElement("tr",component); //$NON-NLS-1$
			        if(valign != null) {
			        	writer.writeAttribute("valign",valign,"valign");
		        	}
			        if(firstTr.booleanValue()) firstTr = Boolean.FALSE;
				}
        	    writer.startElement("td",component); //$NON-NLS-1$
            	Utils.encodeComponent(context, child);
	            writer.endElement("td"); //$NON-NLS-1$
	            if(!childRendered.booleanValue()) childRendered = Boolean.TRUE;

    	        if (layout != null && layout.equalsIgnoreCase("pageDirection")) { //$NON-NLS-1$
        	        writer.endElement("tr"); //$NON-NLS-1$
            	}
			}
			if(!children.hasNext() && childRendered.booleanValue() && (layout == null || !layout.equalsIgnoreCase("pageDirection"))){
				writer.endElement("tr"); //$NON-NLS-1$
			}
        }
        //RATLC01079228 end
    }

	/**
	 * Renders the ending of the component.
	 * <p>
	 * Creates the closing TABLE tag.
	 * <p>
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {

       if (context == null || component == null) {
            throw new NullPointerException();
        }
        // Render the ending of this panel
        ResponseWriter writer = context.getResponseWriter();
        writer.endElement("table");
	}

	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#getRendersChildren()
	 */
	public boolean getRendersChildren() {
		return true;
	}

}
