/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import java.util.Date;
import java.util.Map;

import javax.faces.application.Application;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.Converter;
import javax.faces.convert.DateTimeConverter;
import javax.faces.render.Renderer;
import javax.faces.el.ValueBinding;
import javax.faces.event.ActionEvent;

import com.ibm.faces.component.UICommandEx;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlCommandExButton;



public class ButtonRenderer extends Renderer {
	
	public void decode(FacesContext context, UIComponent component) {
		
		if(context == null)
			throw new NullPointerException();

		if(component == null)
			throw new NullPointerException();		

		HtmlCommandExButton bcomponent = null;
		
		if (component instanceof HtmlCommandExButton) // check if this is a button component
			bcomponent = (HtmlCommandExButton)component;
		
		
				
		boolean readonly = false;
		boolean disabled = false;
		boolean reset = false;
		
		if(bcomponent != null)	// if button component use getters cause it faster
		{
			readonly = bcomponent.isReadonly();
			disabled = bcomponent.isDisabled();
			String resetstr = bcomponent.getType();
			if("reset".equalsIgnoreCase(resetstr))
				reset = true;
		}
		else	// other access the attributes map
		{
			Map attrMap = component.getAttributes();
			Object readonlyObj = attrMap.get("readonly");
			Object disabledObj = attrMap.get("disabled");
			String resetStr = (String)attrMap.get("type");
			if(readonlyObj instanceof String){
				if("true".equalsIgnoreCase(readonlyObj.toString()))
					readonly = true;			
			}
			if(readonlyObj instanceof Boolean){
				readonly = ((Boolean)readonlyObj).booleanValue();
			}
			if(disabledObj instanceof String){
				if("true".equalsIgnoreCase(disabledObj.toString()))
					disabled = true;			
			}
			if(disabledObj instanceof Boolean){
				disabled = ((Boolean)disabledObj).booleanValue();
			}
			if("reset".equalsIgnoreCase(resetStr)){
					reset = true;			
			}
		}
		
		// if the button is read only or disabled or reset we do nothing and exit
		if( reset || disabled || readonly)
			return;
								//	 get the request parameters
		Map paramMap = context.getExternalContext().getRequestParameterMap(); 
								// get the client id
		String clientId = component.getClientId(context);
						// check if the clientid, etc. is set
		String clientid = (String)paramMap.get(clientId); // if it a button
		
		String clientidX = (String)paramMap.get(clientId+".x");// if it an image
		String clientidY = (String)paramMap.get(clientId+".y");// if it an image
		
								// if all are null
		if((clientid == null) && ((clientidX == null) && (clientidY == null))){
			return;				// do nothing and exit
		} 
		//if we're here clientid is not null, queue action
		component.queueEvent(new ActionEvent(component));	// otherwise there is an event to be queued.
	}

	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {
		
		if(context == null || component == null) 
			throw new NullPointerException();
		
	}

	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {
		
		if(context == null || component == null) 
			throw new NullPointerException();

		HtmlCommandExButton bcomponent = null;
		
		if (component instanceof HtmlCommandExButton) // check if this is a button component
			bcomponent = (HtmlCommandExButton)component;		// get and set methods used  in this case
				// cause it faster than the attributes map
		/*HxClientRenderUtil.initJSLibraries(component,context);
		HxClientRenderUtil.addScriptLibrary("hxclient_im.js", component);
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
		*/
		
		boolean isbutton = false;
		boolean isreset = false;
		boolean issubmit = false;
		
		String image = (String)component.getAttributes().get("image"); // image not defined on 	HtmlCommandExButton			
															// so get it from the atrrbiutes map
		//encode image string if set
		if(image != null && image.length() > 0){
			image = encodeImageSrc(context ,image);
		}
		
		ResponseWriter writer = context.getResponseWriter(); // create a writer
							
							// start of writting the input tag
		writer.startElement("input", component);
		
		String expectedtype = (String)component.getAttributes().get("type");
		
		if(expectedtype == null)
		{
			issubmit = true;
		}
		else
		{
			if("button".equalsIgnoreCase(expectedtype.trim()))
				isbutton = true;
			if("reset".equalsIgnoreCase(expectedtype.trim()))
				isreset = true;
			if(("submit".equalsIgnoreCase(expectedtype.trim())) || ("image".equalsIgnoreCase(expectedtype.trim()))
					|| ("".equalsIgnoreCase(expectedtype.trim())))
				issubmit = true;
		}
		
		//	 start of the attributes of the button
		if(isbutton || isreset)
		{				// do this if button is an button or reset
			writer.writeAttribute("type", expectedtype, "type");			
		}
		else if(issubmit)
		{			// do a submit			
			
			if( image == null)// if you set type to image and dob't set image then .......
			{
				writer.writeAttribute("type", "submit","type"); // default is submit				
			}
			else
			{
				writer.writeAttribute("type", "image","type");
				writer.writeAttribute("src", image, null);
			}
		}


		//value
		String value = getValue(context, component);
		writer.writeAttribute("value",value,"value");
		
		String clientid = component.getClientId(context);
		writer.writeAttribute("id", clientid,null);
		writer.writeAttribute("name", clientid,null);

		String style = null;
		if(bcomponent != null)
			style = bcomponent.getStyle();
		else		
			style = (String)component.getAttributes().get("style");				
		if( style!= null)
			writer.writeAttribute("style", style.trim(),"style");		

		String styleClass = null;
		if(bcomponent != null)
			styleClass = bcomponent.getStyleClass();
		else		
			styleClass = (String)component.getAttributes().get("styleClass");				
		if( styleClass!= null)
			writer.writeAttribute("class", styleClass,"class");

		
		Boolean disabled = Boolean.FALSE;
		if(bcomponent != null){
			if(bcomponent.isDisabled())
				disabled = Boolean.TRUE;
		} else {
			Object d = component.getAttributes().get("disabled");
			if(d instanceof String){
				if("true".equalsIgnoreCase(d.toString())){
					disabled = Boolean.TRUE;
				}
			} else if(d instanceof Boolean){
				disabled = (Boolean)d;
			}
		}
		if( disabled.booleanValue())
			writer.writeAttribute("disabled", "disabled",null);

		
		Boolean readonly = Boolean.FALSE;
		if(bcomponent != null){
			if(bcomponent.isReadonly())
				readonly = Boolean.TRUE;
		} else {		
			Object r = component.getAttributes().get("readonly");
			if(r instanceof String){
				if("true".equalsIgnoreCase(r.toString())){
					readonly = Boolean.TRUE;
				}
			} else if(r instanceof Boolean){
				readonly = (Boolean)r;
			}
		}
		if(readonly.booleanValue())
			writer.writeAttribute("readonly", "readonly",null);

		
		String alt = null;
		if(bcomponent != null)		
			alt= bcomponent.getAlt();		
		else		
			alt = (String)component.getAttributes().get("alt");		
		if(alt != null)
			writer.writeAttribute("alt", alt,"alt");

		String dir = null;
		if(bcomponent != null)		
			dir= bcomponent.getDir();		
		else		
			dir = (String)component.getAttributes().get("dir");		
		if(dir != null)
			writer.writeAttribute("dir", dir,"dir");

		String lang = null;
		if(bcomponent != null)		
			lang= bcomponent.getLang();		
		else		
			lang = (String)component.getAttributes().get("lang");			
		if(lang != null)
			writer.writeAttribute("lang", lang,"lang");

		String title = null;
		if(bcomponent != null)		
			title = bcomponent.getTitle();		
		else		
			title = (String)component.getAttributes().get("title");			
		if(title != null)
			writer.writeAttribute("title", title,"title");

		String tabindex = null;
		if(bcomponent != null)		
			tabindex= bcomponent.getTabindex();		
		else		
			tabindex = (String)component.getAttributes().get("tabindex");			
		if(tabindex != null)
			writer.writeAttribute("tabindex", tabindex,"tabindex");

		String accesskey = null;
		if(bcomponent != null)		
			accesskey= bcomponent.getAccesskey();		
		else		
			accesskey = (String)component.getAttributes().get("accesskey");			
		if(accesskey != null)
			writer.writeAttribute("accesskey", accesskey,"accesskey");		
		
							// start of the event section
		String onblur = null;
		if(bcomponent != null)		
			onblur= bcomponent.getOnblur();		
		else		
			onblur = (String)component.getAttributes().get("onblur");			
		if(onblur != null)
			writer.writeAttribute("onblur", onblur,"onblur");

		String onchange = null;
		if(bcomponent != null)		
			onchange= bcomponent.getOnchange();		
		else		
			onchange = (String)component.getAttributes().get("onchange");			
		if(onchange != null)
			writer.writeAttribute("onchange", onchange,"onchange");

		String onclick = null; 
		if(bcomponent != null)		
			onclick= bcomponent.getOnclick();		
		else		
			onclick = (String)component.getAttributes().get("onclick");		
		if(onclick != null)
			writer.writeAttribute("onclick", onclick,"onclick");

		String ondblclick = null;
		if(bcomponent != null)		
			ondblclick= bcomponent.getOndblclick();
		else
			ondblclick = (String)component.getAttributes().get("ondblclick");		
		if(ondblclick != null)
			writer.writeAttribute("ondblclick", ondblclick,"ondblclick");
		
		String onfocus = null;
		if(bcomponent != null)		
			onfocus= bcomponent.getOnfocus();		
		else		
			onfocus = (String)component.getAttributes().get("onfocus");
		if(onfocus != null)
			writer.writeAttribute("onfocus", onfocus,"onfocus");
		
		String onkeydown = null;
		if(bcomponent != null)		
			onkeydown= bcomponent.getOnkeydown();		
		else		
			onkeydown = (String)component.getAttributes().get("onkeydown");
		if(onkeydown != null)
			writer.writeAttribute("onkeydown", onkeydown,"onkeydown");
		
		String onkeypress = null;
		if(bcomponent != null)		
			onkeypress= bcomponent.getOnkeypress();		
		else		
			onkeypress = (String)component.getAttributes().get("onkeypress");
		if(onkeypress != null)
			writer.writeAttribute("onkeypress", onkeypress,"onkeypress");
		
		String onkeyup = null;
		if(bcomponent != null)		
			onkeyup= bcomponent.getOnkeyup();		
		else		
			onkeyup = (String)component.getAttributes().get("onkeyup");
		if(onkeyup != null)
			writer.writeAttribute("onkeyup", onkeyup,"onkeyup");
		
		String onmousedown = null;
		if(bcomponent != null)		
			onmousedown= bcomponent.getOnmousedown();		
		else		
			onmousedown = (String)component.getAttributes().get("onmousedown");		
		if(onmousedown != null)
			writer.writeAttribute("onmousedown", onmousedown,"onmousedown");
		
		String onmousemove = null;
		if(bcomponent != null)		
			onmousemove= bcomponent.getOnmousemove();		
		else		
			onmousemove = (String)component.getAttributes().get("onmousemove");
		if(onmousemove != null)
			writer.writeAttribute("onmousemove", onmousemove,"onmousemove");

		String onmouseout = null;
		if(bcomponent != null)		
			onmouseout= bcomponent.getOnmouseout();		
		else		
			onmouseout = (String)component.getAttributes().get("onmouseout");		
		if(onmouseout != null)
			writer.writeAttribute("onmouseout", onmouseout,"onmouseout");

		String onmouseover = null;
		if(bcomponent != null)		
			onmouseover= bcomponent.getOnmouseover();		
		else		
			onmouseover = (String)component.getAttributes().get("onmouseover");		
		if(onmouseover != null)
			writer.writeAttribute("onmouseover", onmouseover,"onmouseover");

		String onmouseup = null;
		if(bcomponent != null)		
			onmouseup= bcomponent.getOnmouseup();		
		else		
			onmouseup = (String)component.getAttributes().get("onmouseup");		
		if(onmouseup != null)
			writer.writeAttribute("onmouseup", onmouseup,"onmouseup");	

		String onselect = null;
		if(bcomponent != null)		
			onselect= bcomponent.getOnselect();		
		else		
			onselect = (String)component.getAttributes().get("onselect");		
		if(onselect != null)
			writer.writeAttribute("onselect", onselect,"onselect");	
							//		 end of the event section
							// end of the attributes of the button
		
		writer.endElement("input");
							// end of writting the input tag
		
		String confirm = null;
		if ( bcomponent != null ){
			confirm = bcomponent.getConfirm();
		}else{
			confirm = (String)component.getAttributes().get("confirm"); 
		}
		
		String iconNormal = null;
		if(bcomponent != null){
			iconNormal = bcomponent.getIconNormal();
		} else {
			iconNormal = (String)component.getAttributes().get("iconNormal"); 
		}
		String iconMoused = null;
		if(bcomponent != null){
			iconMoused = bcomponent.getIconMoused();
		} else {
			iconMoused = (String)component.getAttributes().get("iconMoused"); 
		}
		String iconDepressed = null;
		if(bcomponent != null){
			iconDepressed = bcomponent.getIconDepressed();
		} else {
			iconDepressed = (String)component.getAttributes().get("iconDepressed"); 
		}
		String iconDisabled = null;
		if(bcomponent != null){
			iconDisabled = bcomponent.getIconDisabled();
		} else {
			iconDisabled = (String)component.getAttributes().get("iconDisabled"); 
		}

		String himage = null;
		if(bcomponent != null){
			himage = bcomponent.getHimage();
		} else {
			himage = (String)component.getAttributes().get("himage"); 
		}
		String pimage = null;
		if(bcomponent != null){
			pimage = bcomponent.getPimage();
		} else {
			pimage = (String)component.getAttributes().get("pimage"); 
		}
		String dimage = null;
		if(bcomponent != null){
			dimage = bcomponent.getDimage();
		} else {
			dimage = (String)component.getAttributes().get("dimage"); 
		}		
		
		//if image is set check for a label otherwise label not used
		String label = null;
		if(image != null){
			label = value;
		}
		
			// RATLC01145202 -- to cover simple sumbit button with image and nothing else
		if((issubmit) &&  (label == null || (label != null && label.length() == 0)) && (image != null) && (himage == null) && (dimage == null) && (pimage == null) && (iconNormal == null) && (iconMoused == null) && (iconDepressed == null) && (iconDisabled == null) && (confirm == null))
			return; 
			
		// we output script from here to the end of the encodeEnd method
       if((image != null && label != null) ||  himage != null || pimage != null || dimage != null || confirm != null || iconNormal != null || iconMoused != null || iconDepressed != null || iconDisabled != null) { // add some JS script if needed
		//if(false){
        	
			UIScriptCollector scriptCollector = UIScriptCollector.find(component);
			/*
			 * Not registering as not using contributeScript method anymore, will leave
			 * IScriptContributor impl in place so we can tell this renderer is adding script.
			 */
			//scriptCollector.register(this, component);
			HxClientRenderUtil.initJSLibraries(component,context);
			HxClientRenderUtil.addScriptLibrary("hxclient_im.js", component);
			HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
			
			//instead of contribute script
//			 Get own attributes		
			
			himage =  encodeImageSrc(context, himage); //$NON-NLS-1$
			pimage =  encodeImageSrc(context, pimage); //$NON-NLS-1$
			dimage =  encodeImageSrc(context, dimage); //$NON-NLS-1$

			
			iconNormal = encodeImageSrc(context, iconNormal);
			iconMoused = encodeImageSrc(context, iconMoused);
			iconDepressed = encodeImageSrc(context, iconDepressed);
			iconDisabled = encodeImageSrc(context, iconDisabled);
			
			
			if( (image != null && image.length() > 0) ||
					(iconNormal != null && iconNormal.length() > 0) ||
					(iconMoused != null && iconMoused.length() > 0) ||
					(iconDepressed != null && iconDepressed.length() > 0) ||
					(iconDisabled != null && iconDisabled.length() > 0) 
			) {	
				
					boolean started = false;
					StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME);
					script.append(".addComponent(\"")
					.append(component.getClientId(context))
					.append("\", new ")
					.append(HxClientRenderUtil.HX_VAR_NAME)
					.append(".JSFImage(");
					if(image != null && image.length() > 0){
						script.append("\"normal:url('")
						.append(image)
						.append("')\"");
						started = true;
					}
					if(himage != null && himage.length() > 0){
						script.append((started)?",":"")
						.append("\"moused:url('")
						.append(himage)
						.append("')\"");
						started = true;
					}
					if(pimage != null && pimage.length() > 0){
						script.append((started)?",":"")
						.append("\"depressed:url('")
						.append(pimage)
						.append("')\"");
						started = true;
					}
					if(dimage != null && dimage.length() > 0){
						script.append((started)?",":"")
						.append("\"disabled:url('")
						.append(dimage)
						.append("')\"");
						started = true;
					}
					if(label != null && label.length() > 0){
						script.append((started)?",":"")
						.append("\"label:")
						.append(label)
						.append("\"");
						started = true;
					}
					if(iconNormal != null && iconNormal.length() > 0){
						script.append((started)?",":"")
						.append("\"icon-normal:url('")
						.append(iconNormal)
						.append("')\"");
						started = true;
					}				
					if(iconMoused != null && iconMoused.length() > 0){
						script.append((started)?",":"")
						.append("\"icon-moused:url('")
						.append(iconMoused)
						.append("')\"");
						started = true;
					}				
					if(iconDepressed != null && iconDepressed.length() > 0){
						script.append((started)?",":"")
						.append("\"icon-depressed:url('")
						.append(iconDepressed)
						.append("')\"");
						started = true;
					}				
					if(iconDisabled != null && iconDisabled.length() > 0){
						script.append((started)?",":"")
						.append("\"icon-disabled:url('")
						.append(iconDisabled)
						.append("')\"");
						started = true;
					}
					
				/*	if(isreset){
						script.append((started)?",":"")
						.append("\"reset:true\"");
						started = true;
					}
					if(isbutton){
						script.append((started)?",":"")
						.append("\"button:true\"");
						started = true;
					}
					if(issubmit){
						script.append((started)?",":"")
						.append("\"submit:true\"");
						started = true;
					}	*/
					
					script.append("));\n");
					
					//writer.write(script.toString());
					scriptCollector.addScript(script.toString());
				}

			if(confirm != null && confirm.length() > 0) {		
				StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME);
				script.append(".addBehavior(\"")
				.append(component.getClientId(context))
				.append("\", \"onclick\", new ")
				.append(HxClientRenderUtil.HX_VAR_NAME)
				.append(".JSFBehaviorGeneric(\"action:confirm\",\"target:")
				.append(confirm)
				.append("\"));\n");
				
				//writer.write(script.toString());
				scriptCollector.addScript(script.toString());
			}
        }
       
	}

	/**
	* @return the image src if this component is configured to display
	* an image label, null otherwise.
	*/
	private String encodeImageSrc(FacesContext context, String src) {
		
		String result = src;
		
		if (result == null) {
			return result;
		}

		return (Utils.getEncodedResourceURL(context, result));
	}

	/**
	* @return the image src if this component is configured to display
	* an image label, null otherwise.
	*/
	private String getValue(FacesContext context, UIComponent component) {
		
		Object value = null;
		String value_str = "";
		HtmlCommandExButton bcomponent = null;
		
		if (component instanceof HtmlCommandExButton) // check if this is a button component
			bcomponent = (HtmlCommandExButton)component;
		
		if(bcomponent != null)
			value = bcomponent.getValue();
		else		
			value = component.getAttributes().get("value");
		
		if(value != null)
		{
			if (value instanceof String) 
			{
				value_str = (String) value;
			}
			else
			{
				Converter converter = null;
				try {
	 				Application application = context.getApplication();
	 				converter = application.createConverter(value.getClass());
	 			} catch (Exception e) {
	 				// TODO should log error	 				
	 			}
				
				if(converter != null)
				{
					value_str = converter.getAsString( context, component, value);
				}
				else
				{
					value_str = value.toString();
				}
			}
		}
		
		return value_str;
		
	}
	
}
