/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.faces.el.ValueBinding;
import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.component.UISelectItem;
import javax.faces.component.UISelectItems;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.Converter;
import javax.faces.model.SelectItem;
import javax.faces.render.Renderer;


import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlSelectOneColor;

import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.faces.renderkit.html_extended.Utils;
//import com.ibm.faces.util.LifecycleUtil;


/**
 * Renderer for JWL progress bar component. 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.InputHelper_DatePickerTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIInputHelper}
 */

public class ColorPickerRenderer extends Renderer {

	final private String DEFAULT_SIZE = "10";
	
	public void decode(FacesContext context, UIComponent component) {
		
		if(context == null)
			throw new NullPointerException();

		if(component == null)
			throw new NullPointerException();		

		HtmlSelectOneColor col_comp = null;
		
		if (component instanceof HtmlSelectOneColor) // check if this is a button component
			col_comp = (HtmlSelectOneColor)component;
		
								//	 get the request parameters
		Map paramMap = context.getExternalContext().getRequestParameterMap(); 
								// get the client id
		String clientId = component.getClientId(context);		
		
		String sub_Value = (String)paramMap.get(clientId);// get value on the input
		// should be set if it is null, have someone cleared it, but I'm propably wrong 
		col_comp.setSubmittedValue(sub_Value);
								
	}	
	
	
	/**
	 * Renders the beginning of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}

		if(!(component.isRendered())) // if you're no meant to see anything at all.
			return;
	
		HxClientRenderUtil.addScriptLibrary("hxclient_cp.js", component);
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		// we need to output our javascript library		
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		
		HtmlSelectOneColor cp_comp = null;
		if(component instanceof HtmlSelectOneColor)
			cp_comp = (HtmlSelectOneColor)component;
		
		String id = component.getClientId(context);	
		String style = null;
		String size = null;
		String title = null;
		String sCSSPrefix = null;
		String tabindex = null; 
		String dir = null;
		String lang = null;
		String accesskey = null;
		boolean disabled = false;
		
		if(cp_comp != null)
		{	
			style = cp_comp.getStyle();
			size =  cp_comp.getSize();
			title = cp_comp.getTitle();
			sCSSPrefix = cp_comp.getStyleClass();	
			tabindex = cp_comp.getTabindex();
			dir = cp_comp.getDir();
			lang = cp_comp.getLang();
			accesskey = cp_comp.getAccesskey();
			disabled = cp_comp.isDisabled();
			
		}
		else
		{
			style = (String)component.getAttributes().get("style");
			size = (String)component.getAttributes().get("size");
			title = (String)component.getAttributes().get("title");
			sCSSPrefix = (String)component.getAttributes().get("styleClass");
			tabindex = (String)component.getAttributes().get("tabindex");
			dir = (String)component.getAttributes().get("dir");
			lang = (String)component.getAttributes().get("lang");
			accesskey = (String)component.getAttributes().get("accesskey");
			
			Object dis = component.getAttributes().get("disabled");	
			
			if(dis instanceof String) // if not set give default
				disabled = Boolean.valueOf((String)dis).booleanValue();

			if(dis instanceof Boolean)
				disabled = ((Boolean)dis).booleanValue();
		}
		
		ResponseWriter writer = context.getResponseWriter();
		
		/*
		 *  Rendered HTML should look like this
		 * <input type="text">
		 * 
		 * */
		
		
		writer.startElement("input",component); // open container div
		writer.writeAttribute("id", id, null);
		writer.writeAttribute("name", id, null);
		writer.writeAttribute("type", "text", null);
		
		if(sCSSPrefix != null && sCSSPrefix.trim().length()>0)
			writer.writeAttribute("class", sCSSPrefix, null);
		if(title != null && title.trim().length() > 0)
			writer.writeAttribute("title", title, null); 
		if(dir != null && dir.trim().length() > 0)
			writer.writeAttribute("dir", dir, null); 
		if(lang != null && lang.trim().length() > 0)
			writer.writeAttribute("lang", lang, null);
		if(style != null && style.trim().length() > 0)
			writer.writeAttribute("style", style, null);		
		if(size != null && size.trim().length() > 0)
			writer.writeAttribute("size", size, null);
		else
			writer.writeAttribute("size", DEFAULT_SIZE, null);
		if(disabled)
			writer.writeAttribute("disabled", "disabled", null); 
	
		if(tabindex != null && tabindex.trim().length() > 0)
			writer.writeAttribute("tabindex", tabindex, null); // tabindex
		if(accesskey != null && accesskey.trim().length() > 0)
			writer.writeAttribute("accesskey", accesskey, null); // tabindex	
		
		
		String value = null; //$NON-NLS-1$
		Object obj = null;
		ValueBinding vb = component.getValueBinding("value"); //$NON-NLS-1$
		if (vb != null) 
		{
			obj = vb.getValue(context);
			if (obj instanceof String) 
			{
				value = (String) obj;
			}
			
		} else 
		{
			if(cp_comp != null)
				value = (String)cp_comp.getValue(); 
			else
				value = (String)component.getAttributes().get("value");
		}
		
		Converter converter = cp_comp.getConverter();
		if(converter != null)
		{
			if(value != null)
				value = converter.getAsString( context, component, value);
			else if (obj != null)
				value = converter.getAsString( context, component, obj);
		}
				
		if((value != null) &&(value.length() > 0))
			writer.writeAttribute("value", value, null);	
		
		
		//START EVENTS
		
		String onchange = null; 
		if(cp_comp != null)		
			onchange= cp_comp.getOnchange();		
		else		
			onchange = (String)component.getAttributes().get("onchange");		
		if(onchange != null)
			writer.writeAttribute("onchange", onchange,"onchange");		
		
		String onselect = null; 
		if(cp_comp != null)		
			onselect= cp_comp.getOnselect();		
		else		
			onselect = (String)component.getAttributes().get("onselect");		
		if(onselect != null)
			writer.writeAttribute("onselect", onselect,"onselect");		

		String onblur = null; 
		if(cp_comp != null)		
			onblur= cp_comp.getOnblur();		
		else		
			onblur = (String)component.getAttributes().get("onblur");		
		if(onblur != null)
			writer.writeAttribute("onblur", onblur,"onblur");		
		
		String onclick = null; 
		if(cp_comp != null)		
			onclick= cp_comp.getOnclick();		
		else		
			onclick = (String)component.getAttributes().get("onclick");		
		if(onclick != null)
			writer.writeAttribute("onclick", onclick,"onclick");

		String ondblclick = null;
		if(cp_comp != null)		
			ondblclick= cp_comp.getOndblclick();
		else
			ondblclick = (String)component.getAttributes().get("ondblclick");		
		if(ondblclick != null)
			writer.writeAttribute("ondblclick", ondblclick,"ondblclick");

		String onfocus = null;
		if(cp_comp != null)		
			onfocus= cp_comp.getOnfocus();
		else
			onfocus = (String)component.getAttributes().get("onfocus");		
		if(onfocus != null)
			writer.writeAttribute("onfocus", onfocus,"onfocus");		
		
		String onkeydown = null;
		if(cp_comp != null)		
			onkeydown= cp_comp.getOnkeydown();		
		else		
			onkeydown = (String)component.getAttributes().get("onkeydown");
		if(onkeydown != null)
			writer.writeAttribute("onkeydown", onkeydown,"onkeydown");
		
		String onkeypress = null;
		if(cp_comp != null)		
			onkeypress= cp_comp.getOnkeypress();		
		else		
			onkeypress = (String)component.getAttributes().get("onkeypress");
		if(onkeypress != null)
			writer.writeAttribute("onkeypress", onkeypress,"onkeypress");
		
		String onkeyup = null;
		if(cp_comp != null)		
			onkeyup= cp_comp.getOnkeyup();		
		else		
			onkeyup = (String)component.getAttributes().get("onkeyup");
		if(onkeyup != null)
			writer.writeAttribute("onkeyup", onkeyup,"onkeyup");
		
		String onmousedown = null;
		if(cp_comp != null)		
			onmousedown= cp_comp.getOnmousedown();		
		else		
			onmousedown = (String)component.getAttributes().get("onmousedown");		
		if(onmousedown != null)
			writer.writeAttribute("onmousedown", onmousedown,"onmousedown");
		
		String onmousemove = null;
		if(cp_comp != null)		
			onmousemove= cp_comp.getOnmousemove();		
		else		
			onmousemove = (String)component.getAttributes().get("onmousemove");
		if(onmousemove != null)
			writer.writeAttribute("onmousemove", onmousemove,"onmousemove");

		String onmouseout = null;
		if(cp_comp != null)		
			onmouseout= cp_comp.getOnmouseout();		
		else		
			onmouseout = (String)component.getAttributes().get("onmouseout");		
		if(onmouseout != null)
			writer.writeAttribute("onmouseout", onmouseout,"onmouseout");

		String onmouseover = null;
		if(cp_comp != null)		
			onmouseover= cp_comp.getOnmouseover();		
		else		
			onmouseover = (String)component.getAttributes().get("onmouseover");		
		if(onmouseover != null)
			writer.writeAttribute("onmouseover", onmouseover,"onmouseover");

		String onmouseup = null;
		if(cp_comp != null)		
			onmouseup= cp_comp.getOnmouseup();		
		else		
			onmouseup = (String)component.getAttributes().get("onmouseup");		
		if(onmouseup != null)
			writer.writeAttribute("onmouseup", onmouseup,"onmouseup");	

		// END EVENTS
		
		writer.endElement("input");
		//end the input feild
		
	}

	

	
	/*
	 * 
	 * 
	 * */
	public boolean getRendersChildren() {
		return true;
	}
	
	/** 
	 * Renders component's children.
	 * <p>
	 * Renders all child components as it see them.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */ 
	public void encodeChildren(FacesContext context, UIComponent component)throws IOException 
	{
	/*	if (context == null || component == null) {
			throw new NullPointerException();
		}
		
		if(!(component.isRendered())) // if you're no meant to see anything at all.
			return;
		
		List children = component.getChildren();
		for (int i = 0; i < children.size(); i++) {
			UIComponent child = (UIComponent) children.get(i);
			Utils.encodeComponent(context, child);
		}	*/
	}
	
    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}
		
		if(!(component.isRendered())) // if you're no meant to see anything at all.
			return;
		
		List children = component.getChildren();
		for (int i = 0; i < children.size(); i++) {
			UIComponent child = (UIComponent) children.get(i);
			Utils.encodeComponent(context, child);
		}
		
		
		//look after script...
		
		HtmlSelectOneColor cp_comp = null;
		
		if(component instanceof HtmlSelectOneColor)
			cp_comp = (HtmlSelectOneColor)component;
	
		boolean showlabels = false;		
		String sCSSPrefix = null;
		String buttonBorder = null;
		String buttonColor = null;
		String colors = null;
		String colornames = null;
		
		
		if(cp_comp != null)
			sCSSPrefix = cp_comp.getDropdownClass();
		else
			sCSSPrefix = (String)component.getAttributes().get("dropdownClass");	

		if(cp_comp != null)
			buttonBorder = cp_comp.getButtonBorder();
		else
			buttonBorder = (String)component.getAttributes().get("buttonBorder");	

		if(cp_comp != null)
			buttonColor = cp_comp.getButtonColor();
		else
			buttonColor = (String)component.getAttributes().get("buttonColor");			
		
		
		if(cp_comp != null)		
			showlabels = cp_comp.isShowLabels();		
		else
		{
			Object ex = component.getAttributes().get("showLabels");	
		
			if(ex instanceof String) // if not set give default
				showlabels = Boolean.valueOf((String)ex).booleanValue();

			if(ex instanceof Boolean)
				showlabels = ((Boolean)ex).booleanValue();
		}		
		
		colors = getColors(component, context, "itemValue");
		colornames = getColors(component, context, "itemLabel");
		
		
		
	/*
	 * example script
	 * hX_4.addComponent("form1:section13", new hX_4.JSFSection("css-prefix:section", "expanded:true", 
	 * 		"diabled:false", "imgopened:form1:image1", "imgclosed:form1:image2", 
	 * 		"textopened:form1:text", "textclosed:form1:text2"));
	 * */
		
		StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME); //$NON-NLS-1$
		script.append(".addComponent(\"");  //$NON-NLS-1$//$NON-NLS-2$
		script.append(component.getClientId(context));
		script.append("\", new "); 	//$NON-NLS-1$//$NON-NLS-2$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".JSFColorPicker(");  //$NON-NLS-1$//$NON-NLS-2$

		if(buttonBorder !=null && buttonBorder.trim().length()>0) // done like this to make sure there is a first parameter passed to 
		{												// JSFColoPicker constructor.
			script.append("\"button-border:"); //$NON-NLS-1$
			script.append(buttonBorder);
			script.append("\""); //$NON-NLS-1$
		}
		else // default
		{			
			script.append("\"button-border:"); //$NON-NLS-1$
			script.append("-1");
			script.append("\""); //$NON-NLS-1$
		}
		
		if(sCSSPrefix != null && sCSSPrefix.trim().length() >0)
		{
			script.append(", \"css-prefix:"); //$NON-NLS-1$
			script.append(sCSSPrefix);
			script.append("\""); //$NON-NLS-1$ 
		}
		
		if(buttonColor !=null && buttonColor.trim().length()>0)  
		{	
			script.append(", \"button-color:"); //$NON-NLS-1$
			script.append(buttonColor);
			script.append("\""); //$NON-NLS-1$
		}
			
		if(showlabels) 
		{				
			script.append(", \"show-labels:"); //$NON-NLS-1$
			script.append("true");
			script.append("\""); //$NON-NLS-1$
		}
		else // default
		{			
			script.append(", \"show-labels:"); //$NON-NLS-1$
			script.append("false");
			script.append("\""); //$NON-NLS-1$
		}		
		
		if(colors !=null && colors.trim().length()>0 && !"null".equalsIgnoreCase(colors.trim()))  
		{	
			script.append(", \"colors:"); //$NON-NLS-1$
			script.append(colors);
			script.append("\""); //$NON-NLS-1$
		}
		
		if(colornames !=null && colornames.trim().length()>0 && !"null".equalsIgnoreCase(colornames.trim()))  
		{
			script.append(", \"color-names:"); //$NON-NLS-1$
			script.append(colornames);
			script.append("\""); //$NON-NLS-1$
		}
		
	
		script.append("));\n"); //$NON-NLS-1$
		
		//ResponseWriter writer = context.getResponseWriter();
		//writer.write(script.toString());
			
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		if(scriptCollector != null){
			scriptCollector.addScript(script.toString());
		} else {
			throw new FacesException("com.ibm.faces.SCRIPT_COLLECTOR_REQUIRED_FOR_COLORPICKER");
		}
    	
    }
    
    
    /*
     * get the children of selectonecolor and only get values and labels 
     * off of UISelectItem 's 
     * */
    private String getColors(UIComponent component, FacesContext context ,String type)
    {
    	StringBuffer allColors = new StringBuffer();
    	
		List children = component.getChildren();
		List childList = new ArrayList();
		
		// makes sure we only deal with UISelectItem
		for (int i = 0; i < children.size(); i++) 
		{
			UIComponent child = (UIComponent) children.get(i);
					
			if(child instanceof UISelectItem)
			{
				childList.add(child);
			}
			else if(child instanceof UISelectItems)
			{
				ValueBinding vb =((UISelectItems)child).getValueBinding("value");
				
				Object value = null;
				
				if(vb != null)
					value = vb.getValue(context);
				else
					value = ((UISelectItems)child).getValue();
				
				if(value != null)
				{
					if(value instanceof UISelectItem) // single
						childList.add((UISelectItem)value);					
					if(value instanceof SelectItem) // single
						childList.add((SelectItem)value);	
					
					if(value instanceof UISelectItem[])  //Array
					{
						UISelectItem[] varray = (UISelectItem[])value;
						for(int j = 0; j < varray.length ; j++)
							childList.add((UISelectItem)varray[j]);	
					}
					if(value instanceof SelectItem[])  //Array
					{
						SelectItem[] varray = (SelectItem[])value;
						for(int j = 0; j < varray.length ; j++)
							childList.add((SelectItem)varray[j]);	
					}					
					
					if(value instanceof Collection)  // collection
						childList.addAll((Collection)value);
	
	
					if(value instanceof Map) // Map
					{
						Map map = (Map)value;
												
						childList.addAll(createComponents(map));
					}
				}
				
			}
		}
		// get the Value or Label from the SelectItem 			
		for (int i = 0; i < childList.size(); i++) 
		{
			Object child = childList.get(i);
			
			if(child instanceof UISelectItem)
			{
				UISelectItem kid = (UISelectItem)child;
				if(type.equalsIgnoreCase("itemValue"))
					allColors.append(kid.getItemValue());
				else
					allColors.append(kid.getItemLabel());
				if(i < (childList.size()-1))
					allColors.append(";");
			}
			else
			{
				SelectItem kid = (SelectItem)child;
				if(type.equalsIgnoreCase("itemValue"))					
					allColors.append(kid.getValue());
				else
					allColors.append(kid.getLabel());
				if(i < (childList.size()-1))
					allColors.append(";");				
			}
			
		}
    	    	
    	return allColors.toString();
    }
    
    
    /*
     * This method creates SelectItem's based on the key value pairs of teh map 
     * 
     * */
    private ArrayList createComponents(Map map)
    {
    	ArrayList childList = new ArrayList();
		
    	SelectItem newsi = null;
    	
    	Set ks = map.keySet();
    	
    	Object key = null;
    	
		Iterator itr = ks.iterator();
				
		while(itr.hasNext())
		{
			key = itr.next();
								// value - value // label - key	
			newsi = new SelectItem(map.get(key), (String)key);
			
			childList.add(newsi);
		}
		
    	
    	return childList;
    }
    
}
