/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.faces.component.UIColumn;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.component.UIData;
import javax.faces.component.html.HtmlOutputText;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.html.HtmlSortHeader;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.faces.util.LifecycleUtil;

public class DataListRenderer extends Renderer implements IScriptContributor {


	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {
		
		if(context == null || component == null)
			throw new NullPointerException();
		
		HxClientRenderUtil.addScriptLibrary("datalist.js", component);
		
		Map atts = component.getAttributes();
		
		String containerId = (String)atts.get("containerId");
		
		boolean showHeader = true;
		String showHeaderObj = (String)atts.get("showHeader");
		System.out.println("showheader"+showHeaderObj);
		if ( showHeaderObj != null )
			showHeader = new Boolean(showHeaderObj).booleanValue();

		boolean altRow = false;
		String altRowObj = (String)atts.get("altRow");
		if ( altRowObj != null )
			altRow = new Boolean(altRowObj).booleanValue();
		
		boolean showUnread = true;
		String showUnreadObj = (String)atts.get("showUnread");
		if ( showUnreadObj != null )
			showUnread = new Boolean(showUnreadObj).booleanValue();	

		String onDeleteCB = null;
		onDeleteCB = (String)atts.get("onDeleteCB");

		String onDblclickCB = null;
		onDblclickCB = (String)atts.get("onDblclickCB");	
		
		String onFormatRowCB = null;
		onFormatRowCB = (String)atts.get("onFormatRowCB");
		
		String onLoadXmlCB = null;
		onLoadXmlCB = (String)atts.get("onLoadXmlCB");

		String headerClass = null;
		headerClass = (String)atts.get("headerClass");	

		String titleClass = null;
		titleClass = (String)atts.get("titleClass");	
		
		String rowClasses = null;
		rowClasses = (String)atts.get("rowClasses");	
		
		String readFontClasses = null;
		readFontClasses = (String)atts.get("readFontClasses");
		
		String unReadFontClasses = null;
		unReadFontClasses = (String)atts.get("unReadFontClasses");
		
		String oddRowClasses = null;
		oddRowClasses = (String)atts.get("oddRowClasses");
		
		String evenRowClasses = null;
		evenRowClasses = (String)atts.get("evenRowClasses");
		
		String categoryClasses = null;
		categoryClasses = (String)atts.get("categoryClasses");
		
		ArrayList columnList = getColumnList((UIData)component);
		ArrayList columnInfoList = new ArrayList(columnList.size());
		//column headers
		if(columnList != null && columnList.size() > 0 && columnFacetCheck(columnList.iterator(),"header").booleanValue())
		{
			Iterator cols = columnList.iterator();
			while(cols.hasNext())
			{
				UIColumn column = (UIColumn)cols.next();
				UIComponentBase columnFacet = (UIComponentBase)column.getFacet("header");
				HashMap columnInfo = new HashMap();
				columnInfoList.add(columnInfo);
				String value = null;
				String styleClass = null;
				Integer width = null;
				String widthStr = (String)(column.getAttributes().get("width"));
				if ( widthStr != null )
					width = new Integer(widthStr);
				Boolean isIcon = null;
				String isIconStr = (String)(column.getAttributes().get("bIsIcon"));
				if ( isIconStr != null )
					isIcon = new Boolean(isIconStr);				
				int sortOrder = 0;
				if(columnFacet != null)
				{
					if ( columnFacet instanceof  HtmlOutputText)
					{
						HtmlOutputText header = (HtmlOutputText)columnFacet;
						value = (String)(header.getValue());
						styleClass = header.getStyleClass();
					}
					else if ( columnFacet instanceof  HtmlSortHeader)
					{
						HtmlOutputText header = (HtmlOutputText)(columnFacet.getChildren().get(0));
						value = (String)(header.getValue());
						styleClass = header.getStyleClass();
						String sortStr = ((HtmlSortHeader)columnFacet).getDefaultSortOrder();
						if ( sortStr == null || sortStr.equals(""))
							sortOrder = 0;
						else if ( sortStr.equals("sortdesc" ))
							sortOrder = 1;
						else if ( sortStr.equals("sortasc" ))
							sortOrder = 2;
						else if ( sortStr.equals("sortbi" ))
							sortOrder = 5;
						else
							sortOrder = 0;
					}
				}
				if ( value != null )
					columnInfo.put("value", value);
				else
					columnInfo.put("value", "");
				columnInfo.put("name", column.getId());
				columnInfo.put("styleClass", styleClass);
				columnInfo.put("sortOrder", new Integer(sortOrder));
				columnInfo.put("width", width);
				columnInfo.put("isIcon", isIcon);
			}
		}
		StringBuffer scriptStr = new StringBuffer();
		String resourceDir = LifecycleUtil.getResourceServletContextPath(context);
		String tableId = component.getId();
		String dataTableName = "MyDataList_" + tableId;
		scriptStr.append( "\r\n" + "var " + dataTableName + " = new VirtualList(\"");
		scriptStr.append(containerId);
		scriptStr.append("\", " + columnList.size() + ", " + altRow + " , false, 0, " + showHeader + " );\r\n");

		scriptStr.append(dataTableName + ".setResourceDir(\"" + resourceDir + "\");\r\n");

		if ( headerClass != null ) scriptStr.append("    " + dataTableName + ".setHeaderFmt(\"" + headerClass + "\");\r\n");
		if ( titleClass != null ) scriptStr.append("    " + dataTableName + ".setTitleFmt(\"" + titleClass + "\");\r\n");
		if ( rowClasses != null ) scriptStr.append("    " + dataTableName + ".setRowFmt(\"" + rowClasses + "\");\r\n");
		if ( readFontClasses != null ) scriptStr.append("   " + dataTableName + ".setReadFmt(\"" + readFontClasses + "\");\r\n");
		if ( unReadFontClasses != null ) scriptStr.append("    " + dataTableName + ".setUnreadFmt(\"" + unReadFontClasses + "\");\r\n");
		if ( oddRowClasses != null ) scriptStr.append("    " + dataTableName + ".setOddRowFmt(\"" + oddRowClasses + "\");\r\n");
		if ( evenRowClasses != null ) scriptStr.append("    " + dataTableName + ".setEvenRowFmt(\"" + evenRowClasses + "\");\r\n");
		if ( categoryClasses != null ) scriptStr.append("    " + dataTableName + ".setCatFmt(\"" + categoryClasses + "\");\r\n");
		
		if ( !showUnread ) scriptStr.append("    " + dataTableName + ".setShowUnread(" + false + ");\r\n");

		if ( onDblclickCB != null ) scriptStr.append("    " + dataTableName + ".readDoc = \"" + onDblclickCB + "\";\r\n");
		if ( onDeleteCB != null ) scriptStr.append("    " + dataTableName + ".setDeleteCB(" + onDeleteCB + ");\r\n");
		if ( onFormatRowCB != null ) scriptStr.append("    " + dataTableName + ".setFormatRowCB(" + onFormatRowCB + ");\r\n");
		if ( onLoadXmlCB != null ) scriptStr.append("    " + dataTableName + ".setLoadXmlCB(" + onLoadXmlCB + ");\r\n");
		
		StringBuffer columnInfoListStr = new StringBuffer();
		columnInfoListStr.append("[");
		Iterator cols = columnInfoList.iterator();
		int i = 0;
		while(cols.hasNext())
		{
			HashMap columnInfo = (HashMap)cols.next();
			columnInfoListStr.append( "{sName : ").append("\"" +columnInfo.get("name")+ "\"" );
			Integer widthInteger = (Integer)(columnInfo.get("width"));
			Boolean isIconBoolean = (Boolean)(columnInfo.get("isIcon"));
			columnInfoListStr.append(", nWidth : ").append((widthInteger != null)?widthInteger.intValue():0).append(", bFixed : ").append(false)
				.append(", bChars: ").append(false).append(", bTwistie : ").append(false).append(", bResponse: ").append(false).append(", sTitle: ").
				append("\"" +columnInfo.get("value")+ "\"" ).append(", bSort: ").append(((Integer)columnInfo.get("sortOrder")).intValue()).
				append(", iViewSort: 0").append("},\r\n  ");
			if ( isIconBoolean != null )
				scriptStr.append(dataTableName + ".aColInfo[" + i + "].bIsIcon = " + isIconBoolean + ";\r\n");
			i++;
		}
		String finalString = columnInfoListStr.substring(0, columnInfoListStr.lastIndexOf(",")) + "];\r\n";
		scriptStr.append("var aInfo = " + finalString);
		scriptStr.append("for ( var i=0, iMax = ( aInfo? aInfo.length:0 ); i < iMax; i++ ) {\r\n");
		scriptStr.append("	oInfo = " + "aInfo[i];\r\n");
		scriptStr.append("	" + dataTableName + ".setColumnWidth (i, oInfo.nWidth, oInfo.bFixed, oInfo.bChars, oInfo.bTwistie, oInfo.bResponse);\r\n");
		scriptStr.append("	" + dataTableName + ".setColumnTitle (i, oInfo.sTitle, oInfo.bSort, oInfo.iViewSort, oInfo.sName);\r\n");
		scriptStr.append("};\r\n");
		this.encodeJavaScript(context, scriptStr.toString());		

	}

	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException 
	{
		if(context == null || component == null)
			throw new NullPointerException();
	}

	public boolean getRendersChildren() {
		return true;
	}


	private ArrayList getColumnList(UIData table){
		ArrayList list = new ArrayList();
		Iterator children = table.getChildren().iterator();
		while(children.hasNext()){
			UIComponent child = (UIComponent)children.next();
			if(child instanceof UIColumn && child.isRendered()){
				list.add(child);
				
		    	//rest facet ids so correct client ids are reset for facets - this fixes a bug in UIData
    			resetFacetIds(child);
			}
		}
		return list;
	}

	
	private Boolean columnFacetCheck(Iterator columns, String facetName){
		while(columns.hasNext()){
			UIComponent comp = (UIComponent)columns.next();
			if(null != (comp.getFacet(facetName))){
				return Boolean.TRUE;
			}
		}
		return Boolean.FALSE;
	}
	
	//bug in UIData where facet ids are not reset to set clientId to null causing 
	//facets not having the row index as part of their clientId
	private void resetFacetIds(UIComponent component){
		Iterator facets = component.getFacets().values().iterator();
		while(facets.hasNext()){
			UIComponent facet = (UIComponent)facets.next();
			facet.setId(facet.getId());
		}
		
		Iterator children = component.getChildren().iterator();
		while(children.hasNext()){
			resetFacetIds((UIComponent)children.next());
		}
	}

    private void encodeJavaScript(
            FacesContext context,
    		String columns)
            throws IOException {
                String src = JavaScriptUtil.getJavaScriptPath(context) + "dataTable.js"; //$NON-NLS-1$
                ResponseWriter writer = context.getResponseWriter();
                if (src == null) {
                    writer.write(JavaScriptUtil.getStartTag(src));
                    writer.write(JavaScriptUtil.getEndTag());
                }
                if (columns != null) {
                    writer.write(JavaScriptUtil.getStartTag());
                    writer.write(columns);
                    writer.write(JavaScriptUtil.getEndTag());
                }
            
        }
    

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		// TODO Auto-generated method stub
		
	}

}
