/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import javax.faces.FacesException;
import javax.faces.component.UIColumn;
import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.UIPager;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.faces.util.ResponseWriterBuffer;

public class DataTableRenderer extends Renderer{

	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {
		
		//get a response writer
		ResponseWriter writer = context.getResponseWriter();
		ResponseWriterBuffer sb = new ResponseWriterBuffer();
		
		//get information about the table
		boolean splitTables = false;
		boolean forceFixedWidths = false;
		String forceFixedWidthsHelper = null;
		String scrolling = null;
		String width = null;
		String style = null;
		String title = null;
		String styleClass = null;
		String dir = null;
		String lang = null;
		String border = null;
		String cellspacing = null;
		String cellpadding = null;
		String bgcolor = null;
		String frame = null;
		String rules = null;
		String summary = null;
		String onclick = null;
		String ondblclick = null;
		String onkeydown = null;
		String onkeypress = null;
		String onkeyup = null;
		String onmousedown = null;
		String onmousemove = null;
		String onmouseout = null;
		String onmouseover = null;
		String onmouseup = null;
		
		javax.faces.component.html.HtmlDataTable h_table = null;
		com.ibm.faces.component.html.HtmlDataTableEx hx_table = null;
		
		if (component instanceof javax.faces.component.html.HtmlDataTable){
			h_table = (javax.faces.component.html.HtmlDataTable)component;
		} else if (component instanceof com.ibm.faces.component.html.HtmlDataTableEx){
			hx_table = (com.ibm.faces.component.html.HtmlDataTableEx)component;
		}

		if(h_table != null){
			title = h_table.getTitle();
			styleClass = h_table.getStyleClass();
			dir = h_table.getDir();
			lang = h_table.getLang();
			border = Integer.toString(h_table.getBorder());
			cellpadding = h_table.getCellpadding();
			cellspacing = h_table.getCellspacing();
			bgcolor = h_table.getBgcolor();
			frame = h_table.getFrame();
			rules = h_table.getRules();
			summary = h_table.getSummary();
			width = h_table.getWidth();
			style = h_table.getStyle();
			onclick= h_table.getOnclick();
			ondblclick= h_table.getOndblclick();
			onkeydown= h_table.getOnkeydown();
			onkeypress= h_table.getOnkeypress();
			onkeyup= h_table.getOnkeyup();
			onmousedown= h_table.getOnmousedown();
			onmousemove= h_table.getOnmousemove();
			onmouseout= h_table.getOnmouseout();
			onmouseover= h_table.getOnmouseover();
			onmouseup= h_table.getOnmouseup();
			scrolling = (String)component.getAttributes().get("scrollSize");
			forceFixedWidthsHelper = (String)component.getAttributes().get("forceFixedWidths");			
		} else if (hx_table != null){
			title = hx_table.getTitle();
			styleClass = hx_table.getStyleClass();
			dir = hx_table.getDir();
			lang = hx_table.getLang();
			border = Integer.toString(hx_table.getBorder());
			cellpadding = hx_table.getCellpadding();
			cellspacing = hx_table.getCellspacing();
			bgcolor = hx_table.getBgcolor();
			frame = hx_table.getFrame();
			rules = hx_table.getRules();
			summary = hx_table.getSummary();
			width = hx_table.getWidth();
			style = hx_table.getStyle();
			onclick= hx_table.getOnclick();
			ondblclick= hx_table.getOndblclick();
			onkeydown= hx_table.getOnkeydown();
			onkeypress= hx_table.getOnkeypress();
			onkeyup= hx_table.getOnkeyup();
			onmousedown= hx_table.getOnmousedown();
			onmousemove= hx_table.getOnmousemove();
			onmouseout= hx_table.getOnmouseout();
			onmouseover= hx_table.getOnmouseover();
			onmouseup= hx_table.getOnmouseup();
			scrolling = hx_table.getScrollSize();
			forceFixedWidthsHelper = hx_table.getForceFixedWidths();
		} else {
			title = (String)component.getAttributes().get("title");
			styleClass = (String)component.getAttributes().get("styleClass");
			dir = (String)component.getAttributes().get("dir");
			lang = (String)component.getAttributes().get("lang");
			//border
			Object borderobj = component.getAttributes().get("border");
			if(borderobj != null)
			{
				if(borderobj instanceof String)
					border = (String)borderobj; // cast as string
				if(borderobj instanceof Integer)
					border = borderobj.toString(); // convert Integer value to a String
			}
			//border end
			cellpadding = (String)component.getAttributes().get("cellpadding");
			cellspacing = (String)component.getAttributes().get("cellspacing");
			bgcolor = (String)component.getAttributes().get("bgcolor");
			frame = (String)component.getAttributes().get("frame");
			rules = (String)component.getAttributes().get("rules");
			summary = (String)component.getAttributes().get("summary");
			width = (String)component.getAttributes().get("width");
			style = (String)component.getAttributes().get("style");
			onclick = (String)component.getAttributes().get("onclick");
			ondblclick = (String)component.getAttributes().get("ondblclick");
			onkeydown = (String)component.getAttributes().get("onkeydown");
			onkeypress = (String)component.getAttributes().get("onkeypress");
			onkeyup = (String)component.getAttributes().get("onkeyup");
			onmousedown = (String)component.getAttributes().get("onmousedown");
			onmousemove = (String)component.getAttributes().get("onmousemove");
			onmouseout = (String)component.getAttributes().get("onmouseout");
			onmouseover = (String)component.getAttributes().get("onmouseover");
			onmouseup = (String)component.getAttributes().get("onmouseup");
			scrolling = (String)component.getAttributes().get("scrollSize");
			forceFixedWidthsHelper = (String)component.getAttributes().get("forceFixedWidths");
		}
		
		if(forceFixedWidthsHelper != null)
			forceFixedWidths = forceFixedWidthsHelper.trim().equalsIgnoreCase("true");		

 		if(forceFixedWidths == true || scrolling != null)
			splitTables = true;
 		
		if(splitTables == true){
			if(width == null)
				width = "100%";
			
			if(style == null)
				style = "table-layout: fixed;";
			else
				style += "table-layout: fixed;";
		}
		
		//Start rendering the tables
		boolean renderHeaderFacet = (getNumberOfTables(context, component).charAt(1) == '1');
		boolean renderHeaderCells = (getNumberOfTables(context, component).charAt(2) == '1');
		boolean renderFooterCells = (getNumberOfTables(context, component).charAt(3) == '1');
		boolean renderFooterFacet = (getNumberOfTables(context, component).charAt(4) == '1');
		
		boolean captionRendered = false;
				
		//two paths through the code
		
		if(splitTables == true) {
						
			if(renderHeaderFacet == true) {
				//table1
				sb.startElement("table", component);
				sb.writeAttribute("id", component.getClientId(context) + "head1", null);		

				//if this facet is displayed the top border will always be shown
				//the bottom border should always be ommitted
				String headerFacetStyle = style;				
				headerFacetStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px;" + headerFacetStyle;
				
				if(headerFacetStyle!= null)
					sb.writeAttribute("style", headerFacetStyle, "style");
				
				if(style!= null)
					sb.writeAttribute("style", style,"style");		
						
				if(title!= null)
					sb.writeAttribute("title", title,"title");		
				
				if(styleClass!= null)
					sb.writeAttribute("class", styleClass, null);	
				
				if(dir != null)
					sb.writeAttribute("dir", dir,"dir");
	
				if(lang != null)
					sb.writeAttribute("lang", lang,"lang");
					
				if(width != null)
					sb.writeAttribute("width", width,"width");
				
				if(border != null)
					sb.writeAttribute("border", border,"border");
				
				if(cellpadding != null)
					sb.writeAttribute("cellpadding", cellpadding,"cellpadding");	
	
				if(cellspacing != null)
					sb.writeAttribute("cellspacing", cellspacing,"cellspacing");	
				
				if(bgcolor != null)
					sb.writeAttribute("bgcolor", bgcolor,"bgcolor");			
	
				if(frame != null)
					sb.writeAttribute("frame", frame,"frame");		
	
				if(rules != null)
					sb.writeAttribute("rules", rules,"rules");
	
				if(summary != null)
					sb.writeAttribute("summary", summary,"summary");		
						
				if(onclick != null)
					sb.writeAttribute("onclick", onclick,"onclick");
	
				if(ondblclick != null)
					sb.writeAttribute("ondblclick", ondblclick,"ondblclick");
				
				if(onkeydown != null)
					sb.writeAttribute("onkeydown", onkeydown,"onkeydown");
				
				if(onkeypress != null)
					sb.writeAttribute("onkeypress", onkeypress,"onkeypress");
				
				if(onkeyup != null)
					sb.writeAttribute("onkeyup", onkeyup,"onkeyup");
				
				if(onmousedown != null)
					sb.writeAttribute("onmousedown", onmousedown,"onmousedown");
				
				if(onmousemove != null)
					sb.writeAttribute("onmousemove", onmousemove,"onmousemove");
	
				if(onmouseout != null)
					sb.writeAttribute("onmouseout", onmouseout,"onmouseout");
	
				if(onmouseover != null)
					sb.writeAttribute("onmouseover", onmouseover,"onmouseover");
	
				if(onmouseup != null)
					sb.writeAttribute("onmouseup", onmouseup,"onmouseup");
				
				sb.checkTagOpenState();
				writer.write(sb.toString());
				sb.clearBuffer();
				
				if(captionRendered == false) {
					doCaptionFacet(context, component);
					captionRendered = true;
				}
				
				doHeaderFacet(context, component);
				
				sb.endElement("table");
				sb.writeText("\n",null);
				
				sb.checkTagOpenState();
				writer.write(sb.toString());
				sb.clearBuffer();
			}			
			
			if(renderHeaderCells == true){
				//table2
				sb.startElement("table", component);
				sb.writeAttribute("id", component.getClientId(context) + "head2", null);		
						
				//if there is a header facet the top border of the header cells should be ommitted
				//the bottom border should always be ommitted
				String headerCellsStyle = style;				
				if(renderHeaderFacet == true)
					headerCellsStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px; " + headerCellsStyle;
				else
					headerCellsStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px; " + headerCellsStyle;
				
				if(headerCellsStyle != null)
					sb.writeAttribute("style", headerCellsStyle,"style");		
						
				if(title!= null)
					sb.writeAttribute("title", title,"title");		
				
				if(styleClass!= null)
					sb.writeAttribute("class", styleClass, null);	
				
				if(dir != null)
					sb.writeAttribute("dir", dir,"dir");
	
				if(lang != null)
					sb.writeAttribute("lang", lang,"lang");
					
				if(width != null)
					sb.writeAttribute("width", width,"width");
				
				if(border != null)
					sb.writeAttribute("border", border,"border");
				
				if(cellpadding != null)
					sb.writeAttribute("cellpadding", cellpadding,"cellpadding");	
	
				if(cellspacing != null)
					sb.writeAttribute("cellspacing", cellspacing,"cellspacing");	
				
				if(bgcolor != null)
					sb.writeAttribute("bgcolor", bgcolor,"bgcolor");			
	
				if(frame != null)
					sb.writeAttribute("frame", frame,"frame");		
	
				if(rules != null)
					sb.writeAttribute("rules", rules,"rules");
	
				if(summary != null)
					sb.writeAttribute("summary", summary,"summary");		
						
				if(onclick != null)
					sb.writeAttribute("onclick", onclick,"onclick");
	
				if(ondblclick != null)
					sb.writeAttribute("ondblclick", ondblclick,"ondblclick");
				
				if(onkeydown != null)
					sb.writeAttribute("onkeydown", onkeydown,"onkeydown");
				
				if(onkeypress != null)
					sb.writeAttribute("onkeypress", onkeypress,"onkeypress");
				
				if(onkeyup != null)
					sb.writeAttribute("onkeyup", onkeyup,"onkeyup");
				
				if(onmousedown != null)
					sb.writeAttribute("onmousedown", onmousedown,"onmousedown");
				
				if(onmousemove != null)
					sb.writeAttribute("onmousemove", onmousemove,"onmousemove");
	
				if(onmouseout != null)
					sb.writeAttribute("onmouseout", onmouseout,"onmouseout");
	
				if(onmouseover != null)
					sb.writeAttribute("onmouseover", onmouseover,"onmouseover");
	
				if(onmouseup != null)
					sb.writeAttribute("onmouseup", onmouseup,"onmouseup");
				
				sb.checkTagOpenState();
				writer.write(sb.toString());
				sb.clearBuffer();
				
				if(captionRendered == false) {
					doCaptionFacet(context, component);
					captionRendered = true;
				}
				
				doHeaderCells(context, component);
	
				sb.endElement("table");
				sb.writeText("\n",null);
				
				sb.checkTagOpenState();
				writer.write(sb.toString());
				sb.clearBuffer();
			}
			
			//Table to contain main body of data - table3
			if (scrolling != null) {
				sb.startElement("div", component);
				sb.writeAttribute("id", component.getClientId(context) +"Div", null);
				
				if(styleClass != null)
					sb.writeAttribute("class", styleClass, null);
				
				String widthValue = Utils.checkMeasurementMetric(width);
				
				scrolling = Utils.checkMeasurementMetric(scrolling);
				
				String divStyle = "width:" + widthValue + "; height:" + scrolling + "; overflow-y:scroll;";
				//if there is a header facet or header cells then the top border should be ommitted
				if(renderHeaderFacet == true || renderHeaderCells == true)
					divStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px; " + divStyle;
				
				//if there is a footer facet or footer cells then the bottom border should be ommitted
				if(renderFooterFacet == true || renderFooterCells == true)
					divStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px; " + divStyle;
				
				sb.writeAttribute("style", divStyle, null);
			}
		}
		
		//table3 (main body of data) or this is the entire table if the table is not split up
		sb.startElement("table", component);
		sb.writeAttribute("id", component.getClientId(context), null);		
		
		String bodyStyle = style;
		
		//check if table is split
		if(splitTables == true) {
			//if there is a header facet or header cells then the top border should be ommitted
			if(renderHeaderFacet == true || renderHeaderCells == true)
				bodyStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px; " + bodyStyle;
			
			//if there is a footer facet or footer cells then the bottom border should be ommitted
			if(renderFooterFacet == true || renderFooterCells == true)
				bodyStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px; " + bodyStyle;
			
			//remove all border from table if there is a <div> around it
			if (scrolling != null)
				bodyStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px; " + bodyStyle;
		}
		
		if(bodyStyle != null)
			sb.writeAttribute("style", bodyStyle, "style");		
				
		if(title!= null)
			sb.writeAttribute("title", title,"title");		
		
		if(styleClass!= null)
			sb.writeAttribute("class", styleClass, null);	
		
		if(dir != null)
			sb.writeAttribute("dir", dir,"dir");

		if(lang != null)
			sb.writeAttribute("lang", lang,"lang");
			
		if(scrolling != null) {
			sb.writeAttribute("width", "100%", "width");
		}
		else {
			if(width != null)
				sb.writeAttribute("width", width,"width");
		}
		
		if(border != null)
			sb.writeAttribute("border", border,"border");
		
		if(cellpadding != null)
			sb.writeAttribute("cellpadding", cellpadding,"cellpadding");	

		if(cellspacing != null)
			sb.writeAttribute("cellspacing", cellspacing,"cellspacing");	
		
		if(bgcolor != null)
			sb.writeAttribute("bgcolor", bgcolor,"bgcolor");			

		if(frame != null)
			sb.writeAttribute("frame", frame,"frame");		

		if(rules != null)
			sb.writeAttribute("rules", rules,"rules");

		if(summary != null)
			sb.writeAttribute("summary", summary,"summary");		
				
		if(onclick != null)
			sb.writeAttribute("onclick", onclick,"onclick");

		if(ondblclick != null)
			sb.writeAttribute("ondblclick", ondblclick,"ondblclick");
		
		if(onkeydown != null)
			sb.writeAttribute("onkeydown", onkeydown,"onkeydown");
		
		if(onkeypress != null)
			sb.writeAttribute("onkeypress", onkeypress,"onkeypress");
		
		if(onkeyup != null)
			sb.writeAttribute("onkeyup", onkeyup,"onkeyup");
		
		if(onmousedown != null)
			sb.writeAttribute("onmousedown", onmousedown,"onmousedown");
		
		if(onmousemove != null)
			sb.writeAttribute("onmousemove", onmousemove,"onmousemove");

		if(onmouseout != null)
			sb.writeAttribute("onmouseout", onmouseout,"onmouseout");

		if(onmouseover != null)
			sb.writeAttribute("onmouseover", onmouseover,"onmouseover");

		if(onmouseup != null)
			sb.writeAttribute("onmouseup", onmouseup,"onmouseup");
		
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
		
		if(captionRendered == false ){
			doCaptionFacet(context, component);
			captionRendered = true;
		}
		
		if(splitTables == false) {
			if(renderHeaderFacet == true)
				doHeaderFacet(context, component);
			if(renderHeaderCells == true)
				doHeaderCells(context, component);
			if(renderFooterCells == true)
				doFooterCells(context, component);
			if(renderFooterFacet == true)
				doFooterFacet(context, component);			
		}
		
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
	}

	/**
	 * this method will render the caption for the table if one exists
	 **/	
	private void doCaptionFacet(FacesContext context, UIComponent component) throws IOException
	{
		ResponseWriterBuffer sb = new ResponseWriterBuffer();
		ResponseWriter writer = context.getResponseWriter(); // get a sb	
		
		//get table caption facet
		UIComponent captionFacet = component.getFacet("caption");

		if(captionFacet != null){
			sb.startElement("caption",captionFacet);
			sb.writeAttribute("id",captionFacet.getId(),null);
			String captionStyle = (String)captionFacet.getAttributes().get("style");
			String captionClass = (String)captionFacet.getAttributes().get("styleClass");
		
			if(captionStyle != null){
				sb.writeAttribute("style",captionStyle,null);
			}
			
			if(captionClass != null){
				sb.writeAttribute("class",captionClass,null);
			}
			
			sb.checkTagOpenState();
			writer.write(sb.toString());
			sb.clearBuffer();
			Utils.encodeComponent(context,captionFacet);
			writer.endElement("caption");
			writer.writeText("\n", null);
		}
	}
	
	/**
	 * this method will render the header for the table if one exists
	 **/	
	private void doHeaderFacet(FacesContext context, UIComponent component) throws IOException
	{
		boolean splitTables = false;
		boolean forceFixedWidths = false;
		
		String forceFixedWidthsHelper = null;
		String scrolling = null;
		String headerclass = null;
		
		javax.faces.component.html.HtmlDataTable h_table = null;
		com.ibm.faces.component.html.HtmlDataTableEx hx_table = null;
		
		if (component instanceof javax.faces.component.html.HtmlDataTable){
			h_table = (javax.faces.component.html.HtmlDataTable)component;
		} else if (component instanceof com.ibm.faces.component.html.HtmlDataTableEx){
			hx_table = (com.ibm.faces.component.html.HtmlDataTableEx)component;
		}

		if(h_table != null){
			headerclass= h_table.getHeaderClass();
			scrolling = (String)component.getAttributes().get("scrollSize");
			forceFixedWidthsHelper = (String)component.getAttributes().get("forceFixedWidths");
		} else if (hx_table != null){
			headerclass= hx_table.getHeaderClass();
			scrolling = hx_table.getScrollSize();
			forceFixedWidthsHelper = hx_table.getForceFixedWidths();
		} else {
			headerclass = (String)component.getAttributes().get("headerClass");
			scrolling = (String)component.getAttributes().get("scrollSize");
			forceFixedWidthsHelper = (String)component.getAttributes().get("forceFixedWidths");
		}
		
		if(forceFixedWidthsHelper != null)
			forceFixedWidths = forceFixedWidthsHelper.trim().equalsIgnoreCase("true");		

 		if(forceFixedWidths == true || scrolling != null)
			splitTables = true;
				
		//render content of the header facet
		ResponseWriterBuffer sb = new ResponseWriterBuffer();
		ResponseWriter writer = context.getResponseWriter(); // get a sb		
		
		ArrayList columnList = getColumnList((UIData)component);
		
		//table header
		UIComponent tableHeader = component.getFacet("header");
		
		//render the content of the table header facet
		sb.startElement("thead", component);

		sb.startElement("tr", component);
		sb.startElement("th", component);
		sb.writeAttribute("scope", "colgroup", null);
		
		if(headerclass != null)
			sb.writeAttribute("class", headerclass, null);

		if(splitTables == false && columnList != null){
			sb.writeAttribute("colspan", Integer.toString(columnList.size()), null);
		}
		
		if(scrolling != null)
			sb.writeAttribute("style", "border-right-width:0px", null);
		
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
		Utils.encodeComponent(context,tableHeader);
		
		sb.endElement("th");
		
		//add extra "TH" to header facet
		if(scrolling != null) {
			sb.startElement("th",component);
			sb.writeAttribute("scope", "colgroup", null);
			
			if(headerclass != null)			
				sb.writeAttribute("class", headerclass, null);
			
			//set borders and padding to 0pt
			//set left border to 0px as well
			sb.writeAttribute("style", "margin:0px; padding:0px; padding-left:0pt; padding-right:0pt; border-left-width:0px", null);
			
			sb.startElement("div",component);					
			sb.writeAttribute("style", "overflow: -moz-scrollbars-none; overflow-x: hidden;", null);
			sb.endElement("div");
			sb.endElement("th");
		}
		
		sb.endElement("tr");
		sb.writeText("\n",null);

		boolean renderHeaderCells = (getNumberOfTables(context, component).charAt(2) == '1');
		
		//if table split up, always need an end tfoot
		if(splitTables == true){
			sb.endElement("thead");
		}
		//if there are no header cells below this, then end the thead here
		else if(renderHeaderCells == false) {
			sb.endElement("thead");
		}
		
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
	}	

	/**
	 * this method will render the header cells for the table
	 **/	
	private void doHeaderCells(FacesContext context, UIComponent component) throws IOException
	{
		boolean splitTables = false;
		boolean forceFixedWidths = false;
		
		String forceFixedWidthsHelper = null;
		String scrolling = null;
		
		com.ibm.faces.component.html.HtmlDataTableEx hx_table = null;
		
		if (component instanceof com.ibm.faces.component.html.HtmlDataTableEx){
			hx_table = (com.ibm.faces.component.html.HtmlDataTableEx)component;
		}

		if (hx_table != null){
			scrolling = hx_table.getScrollSize();
			forceFixedWidthsHelper = hx_table.getForceFixedWidths();
		} else {
			scrolling = (String)component.getAttributes().get("scrollSize");
			forceFixedWidthsHelper = (String)component.getAttributes().get("forceFixedWidths");
		}		
		
		if(forceFixedWidthsHelper != null)
			forceFixedWidths = forceFixedWidthsHelper.trim().equalsIgnoreCase("true");		

 		if(forceFixedWidths == true || scrolling != null)
			splitTables = true;
 		
 		//render the header cells
		ResponseWriterBuffer sb = new ResponseWriterBuffer();
		ResponseWriter writer = context.getResponseWriter(); // get a sb		
		
		ArrayList columnList = getColumnList((UIData)component);
		boolean renderHeaderFacet = (getNumberOfTables(context, component).charAt(1) == '1');
		
		if(splitTables == true) {
			sb.startElement("thead", component);
		}
		//if there was no header facet above this start a new thead
		else if(renderHeaderFacet == false) {
			sb.startElement("thead", component);
		}
		
		String spacerImageSrc=JavaScriptUtil.getImagePath(context) + "1x1.gif"; // get the spacer image
		spacerImageSrc=context.getExternalContext().encodeResourceURL(spacerImageSrc);
			
		sb.startElement("tr",component);
		Iterator cols = columnList.iterator();
		
		String lastStyle = null;
		
		while(cols.hasNext()){
			UIColumn column = (UIColumn)cols.next();
			
			//get all the attribute for the column
			String align = null;
			String axis = null;
			String background = null;
			String bgcolor = null;
			String dir = null;
			String headerClass = null;
			String height = null;
			String nowrap = null;
			String scope = null;
			String style = null;
			String valign = null;
			String width = null;
			String title = null;
			
			com.ibm.faces.component.UIColumnEx hx_column = null;
			
			if (column instanceof com.ibm.faces.component.UIColumnEx){
				hx_column = (com.ibm.faces.component.UIColumnEx)column;
			}

			if (hx_column != null){
				align = hx_column.getAlign();
				axis = hx_column.getAxis();
				background = hx_column.getBackground();
				bgcolor = hx_column.getBgcolor();
				dir = hx_column.getDir();
				headerClass = hx_column.getHeaderClass();
				height = hx_column.getHeight();
				nowrap = hx_column.getNowrap();
				scope = hx_column.getScope();
				style = hx_column.getStyle();
				valign = hx_column.getValign();
				width = hx_column.getWidth();
				title = hx_column.getTitle();
			} else {
				align = (String)column.getAttributes().get("align");
				axis = (String)column.getAttributes().get("axis");
				background = (String)column.getAttributes().get("background");
				bgcolor = (String)column.getAttributes().get("bgcolor");
				dir = (String)column.getAttributes().get("dir");
				headerClass = (String)column.getAttributes().get("headerClass");
				height = (String)column.getAttributes().get("height");
				nowrap = (String)column.getAttributes().get("nowrap");
				scope = (String)column.getAttributes().get("scope");
				style = (String)column.getAttributes().get("style");
				valign = (String)column.getAttributes().get("valign");
				width = (String)column.getAttributes().get("width");
				title = (String)column.getAttributes().get("title");
			}
			
			//if headerClass is null use headerClass set on table
			if(headerClass == null){
				if(hx_table != null){
					headerClass = hx_table.getHeaderClass();
				} else {
					headerClass = (String)component.getAttributes().get("headerClass");
				}
			}
			
			//start the TH
			sb.startElement("th",column);
			
			//write the column attributes
			if (align != null)
				sb.writeAttribute("align", align, null);
			
			if (axis != null)
				sb.writeAttribute("axis", axis, null);
			
			if (background != null)
				sb.writeAttribute("background", background, null);
			
			if (bgcolor != null)
				sb.writeAttribute("bgcolor", bgcolor, null);
			
			if (dir != null)
				sb.writeAttribute("dir", dir, null);
			
			if (headerClass != null)
				sb.writeAttribute("class", headerClass, null);
			
			if (height != null)
				sb.writeAttribute("height", height, null);

			if (nowrap != null)
				sb.writeAttribute("nowrap", nowrap, null);
			
			if (scope != null)
				sb.writeAttribute("scope", scope, null);

			if (title != null)
				sb.writeAttribute("title", title, null);

			//if it is the last column
			if(cols.hasNext() == false && scrolling != null) {
				if(style != null)
					sb.writeAttribute("style", "border-right-width:0px; " + style, null);
				else
					sb.writeAttribute("style", "border-right-width:0px;", null);
				lastStyle = headerClass;
			}
			else if (style != null) {
				sb.writeAttribute("style", style, null);
			}
			
			if (valign != null)
				sb.writeAttribute("valign", valign, null);
			
			if (width != null)
				sb.writeAttribute("width", width, null);
			
			if(width != null) {
				sb.startElement("div",column);
				
				if(width.charAt(width.length()-1) == '%')
					sb.writeAttribute("style", "overflow: -moz-scrollbars-none; overflow-x: hidden; width: 100%;", null);
				else
				{
					width = Utils.checkMeasurementMetric(width);
					sb.writeAttribute("style", "overflow: -moz-scrollbars-none; overflow-x: hidden; width:" + width, null);
				}
			}
			
			UIComponent columnFacet = column.getFacet("header");
			if(columnFacet != null){
				sb.checkTagOpenState();
				writer.write(sb.toString());
				sb.clearBuffer();
				Utils.encodeComponent(context,columnFacet);
			} else {
				sb.startElement("img",columnFacet);
				sb.writeAttribute("src",spacerImageSrc,null);
				sb.endElement("img");				
			}
			
			if(width != null)
				sb.endElement("div");
			
			sb.endElement("th");
		}
		
		if(scrolling != null) {
			//add extra "TH" to header cells row
			sb.startElement("th",component);
			
			if (lastStyle != null)
				sb.writeAttribute("class", lastStyle, null);

			//set borders and padding to 0pt
			//set left border to 0px as well
			sb.writeAttribute("style", "margin:0px; padding:0px; padding-left:0pt; padding-right:0pt; border-left-width:0px", null);
			
			sb.startElement("div",component);					
			sb.writeAttribute("style", "overflow: -moz-scrollbars-none; overflow-x: hidden;", null);
			sb.endElement("div");
			sb.endElement("th");
		}
		
		sb.endElement("tr");
		sb.writeText("\n",null);
		
		sb.endElement("thead");
		
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
	}	

	public void encodeChildren(FacesContext context, UIComponent component)
			throws IOException {
		
		String rowclasses = null;
		String columnclasses = null;
		
		javax.faces.component.html.HtmlDataTable h_table = null;
		com.ibm.faces.component.html.HtmlDataTableEx hx_table = null;
		
		if (component instanceof javax.faces.component.html.HtmlDataTable){
			h_table = (javax.faces.component.html.HtmlDataTable)component;
		} else if (component instanceof com.ibm.faces.component.html.HtmlDataTableEx){
			hx_table = (com.ibm.faces.component.html.HtmlDataTableEx)component;
		}

		if(h_table != null){
			rowclasses = h_table.getRowClasses();
			columnclasses = h_table.getColumnClasses();
		} else if (hx_table != null){
			rowclasses = hx_table.getRowClasses();
			columnclasses = hx_table.getColumnClasses();
		}
		
		if(rowclasses == null)
			rowclasses = (String)component.getAttributes().get("rowClasses");
		if(columnclasses == null)
			columnclasses = (String)component.getAttributes().get("columnClasses");

		UIData dataComponent = (UIData)component;
		
		ResponseWriterBuffer sb = new ResponseWriterBuffer();
		ResponseWriter writer = context.getResponseWriter(); // get a sb
		
		//split styleclasses to arrays
		Object[] rowClassArray  = null;
		Object[] colClassArray = null;
		if(rowclasses != null){
			rowClassArray = getStyleClassIterator(rowclasses,",");
		}
		if(columnclasses != null){
			colClassArray = getStyleClassIterator(columnclasses,",");
		}

		//	write the table body
		sb.startElement("tbody", component);
		sb.writeText("\n", null);
		int rowsToDisplay = dataComponent.getRows();
		int rowIndex = dataComponent.getFirst();
		int processedRowCount = 0;
		
		Boolean processRow = Boolean.TRUE;
		
		dataComponent.setRowIndex(rowIndex);
		if(!dataComponent.isRowAvailable()){
			processRow = Boolean.FALSE;
		}
		
		int rowClassIndex = 0;
		int colClassIndex = 0;
		Iterator columns;
		Iterator columnChildren;
		
		while(processRow.booleanValue()){
			//sb.startElement("tr",component);
			 startRow(sb, context, dataComponent); //*********************
			//row class
			if(rowclasses != null && rowClassArray.length > 0){
				sb.writeAttribute("class", rowClassArray[rowClassIndex++],null);
				if(rowClassIndex >= rowClassArray.length){
					rowClassIndex = 0;
				}
			}
			
			//process columns for row
			columns = getColumnList(dataComponent).iterator();
			while(columns.hasNext()){
				UIColumn column = (UIColumn)columns.next();
				
                java.util.Map attrs = column.getAttributes();
                String dir = (String) attrs.get("dir");
                String scope = (String) attrs.get("scope");
                String axis = (String) attrs.get("axis");
                String width = (String)attrs.get("width");
                String height = (String)attrs.get("height");
                String nowrapString = (String)attrs.get("nowrap");
                boolean nowrap = ("true").equals( nowrapString );
                String align = (String)attrs.get("align");
                String valign = (String)attrs.get("valign");
                String bgcolor = (String)attrs.get("bgcolor");
                String background = (String)attrs.get("background");
                String style = (String)attrs.get("style");
                String title = (String)attrs.get("title");
                
                if(scope != null && scope.equalsIgnoreCase("row"))
                	sb.startElement("th",column);
                else
                	sb.startElement("td",column);
                	
                if(dir != null){
                	sb.writeAttribute("dir",dir,null);
                }
                if(scope != null){
                	sb.writeAttribute("scope",scope,null);
                }
                if(axis != null){
                	sb.writeAttribute("axis",axis,null);
                }
                if ( width != null ){
					sb.writeAttribute("width",width,null);
                }
				if ( height != null ){
					sb.writeAttribute("height",height,null);
				}
				if ( nowrap ){
					sb.writeAttribute("nowrap",Boolean.TRUE,null);
				}
				if ( align != null ){
					sb.writeAttribute("align",align,null);
				}
				if ( valign != null ){
					sb.writeAttribute("valign",valign,null);
				}
				if ( bgcolor != null ){
					sb.writeAttribute("bgcolor",bgcolor,null);
				}
				if ( background != null ){
					sb.writeAttribute("background",background,null);
				}
				if ( style != null ){
					sb.writeAttribute("style",style,null);
				}
				if ( title != null ){
					sb.writeAttribute("title",title,null);
				}

				if(columnclasses != null && colClassArray.length > 0){
					sb.writeAttribute("class",colClassArray[colClassIndex++],null);
					if(colClassIndex >= colClassArray.length){
						colClassIndex = 0;
					}
				}

				if(width != null) {
					sb.startElement("div",column);
					
					String columnDivStyle = getColumnDivStyle();
					
					if(width.charAt(width.length()-1) == '%')
					{
						sb.writeAttribute("style", columnDivStyle+"width: 100%;", null);
					}
					else
					{
						width = Utils.checkMeasurementMetric(width);
						sb.writeAttribute("style", columnDivStyle+"width:" + width , null);
					}
				}			
				
				//CHILDREN ENCODED HERE
				columnChildren = column.getChildren().iterator();
				while(columnChildren.hasNext()){
					sb.checkTagOpenState();
					writer.write(sb.toString());
					sb.clearBuffer();
					UIComponent child = (UIComponent)columnChildren.next();
					//if child is dataTable check for the inner table pager hashmap.
					if(child instanceof UIData){
						Map innerPagerMap = (Map)context.getExternalContext().getSessionMap().get(UIPager.PAGED_INNER_TABLE_IDS);
						if(innerPagerMap != null){
							int[] vals = (int[])innerPagerMap.get(child.getClientId(context));
							if(vals != null){
								((UIData)child).setFirst(vals[0]);	
							}
						}
					}
					Utils.encodeComponent(context, child);
				}
				
				if(width != null)
					sb.endElement("div");
				
				sb.endElement("td");
				sb.writeText("\n",null);
			}
			
			//sb.endElement("tr");
			//sb.writeText("\n",null);
			//++processedRowCount;
			processedRowCount = endRow(sb, context, dataComponent, processRow, processedRowCount, rowsToDisplay); //*********************
			colClassIndex = 0;
			dataComponent.setRowIndex(++rowIndex);
			//if(!dataComponent.isRowAvailable() || (rowsToDisplay > 0 && processedRowCount >= rowsToDisplay)){
			//	processRow = Boolean.FALSE;
			//}
			 if (!isRowAvailable(context, dataComponent, rowsToDisplay, processedRowCount)){ //********************
            	processRow = Boolean.FALSE;
            }

		}
		sb.endElement("tbody");
		sb.writeText("\n",null);
		
		//deselect row
		dataComponent.setRowIndex(-1);
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
	}

	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {

		//get a response writer
		ResponseWriter writer = context.getResponseWriter();
		ResponseWriterBuffer sb = new ResponseWriterBuffer();
		
		//get information about the table
		boolean splitTables = false;
		boolean forceFixedWidths = false;
		String forceFixedWidthsHelper = null;
		String scrolling = null;
		String width = null;
		String style = null;
		String title = null;
		String styleClass = null;
		String dir = null;
		String lang = null;
		String border = null;
		String cellspacing = null;
		String cellpadding = null;
		String bgcolor = null;
		String frame = null;
		String rules = null;
		String summary = null;
		String onclick = null;
		String ondblclick = null;
		String onkeydown = null;
		String onkeypress = null;
		String onkeyup = null;
		String onmousedown = null;
		String onmousemove = null;
		String onmouseout = null;
		String onmouseover = null;
		String onmouseup = null;
		
		javax.faces.component.html.HtmlDataTable h_table = null;
		com.ibm.faces.component.html.HtmlDataTableEx hx_table = null;
		
		if (component instanceof javax.faces.component.html.HtmlDataTable){
			h_table = (javax.faces.component.html.HtmlDataTable)component;
		} else if (component instanceof com.ibm.faces.component.html.HtmlDataTableEx){
			hx_table = (com.ibm.faces.component.html.HtmlDataTableEx)component;
		}

		if(h_table != null){
			title = h_table.getTitle();
			styleClass = h_table.getStyleClass();
			dir = h_table.getDir();
			lang = h_table.getLang();
			border = Integer.toString(h_table.getBorder());
			cellpadding = h_table.getCellpadding();
			cellspacing = h_table.getCellspacing();
			bgcolor = h_table.getBgcolor();
			frame = h_table.getFrame();
			rules = h_table.getRules();
			summary = h_table.getSummary();
			width = h_table.getWidth();
			style = h_table.getStyle();
			onclick= h_table.getOnclick();
			ondblclick= h_table.getOndblclick();
			onkeydown= h_table.getOnkeydown();
			onkeypress= h_table.getOnkeypress();
			onkeyup= h_table.getOnkeyup();
			onmousedown= h_table.getOnmousedown();
			onmousemove= h_table.getOnmousemove();
			onmouseout= h_table.getOnmouseout();
			onmouseover= h_table.getOnmouseover();
			onmouseup= h_table.getOnmouseup();
			scrolling = (String)component.getAttributes().get("scrollSize");
			forceFixedWidthsHelper = (String)component.getAttributes().get("forceFixedWidths");			
		} else if (hx_table != null){
			title = hx_table.getTitle();
			styleClass = hx_table.getStyleClass();
			dir = hx_table.getDir();
			lang = hx_table.getLang();
			border = Integer.toString(hx_table.getBorder());
			cellpadding = hx_table.getCellpadding();
			cellspacing = hx_table.getCellspacing();
			bgcolor = hx_table.getBgcolor();
			frame = hx_table.getFrame();
			rules = hx_table.getRules();
			summary = hx_table.getSummary();
			width = hx_table.getWidth();
			style = hx_table.getStyle();
			onclick= hx_table.getOnclick();
			ondblclick= hx_table.getOndblclick();
			onkeydown= hx_table.getOnkeydown();
			onkeypress= hx_table.getOnkeypress();
			onkeyup= hx_table.getOnkeyup();
			onmousedown= hx_table.getOnmousedown();
			onmousemove= hx_table.getOnmousemove();
			onmouseout= hx_table.getOnmouseout();
			onmouseover= hx_table.getOnmouseover();
			onmouseup= hx_table.getOnmouseup();
			scrolling = hx_table.getScrollSize();
			forceFixedWidthsHelper = hx_table.getForceFixedWidths();
		} else {
			title = (String)component.getAttributes().get("title");
			styleClass = (String)component.getAttributes().get("styleClass");
			dir = (String)component.getAttributes().get("dir");
			lang = (String)component.getAttributes().get("lang");
			//border
			Object borderobj = component.getAttributes().get("border");
			if(borderobj != null)
			{
				if(borderobj instanceof String)
					border = (String)borderobj; // cast as string
				if(borderobj instanceof Integer)
					border = borderobj.toString(); // convert Integer value to a String
			}
			//border end
			cellpadding = (String)component.getAttributes().get("cellpadding");
			cellspacing = (String)component.getAttributes().get("cellspacing");
			bgcolor = (String)component.getAttributes().get("bgcolor");
			frame = (String)component.getAttributes().get("frame");
			rules = (String)component.getAttributes().get("rules");
			summary = (String)component.getAttributes().get("summary");
			width = (String)component.getAttributes().get("width");
			style = (String)component.getAttributes().get("style");
			onclick = (String)component.getAttributes().get("onclick");
			ondblclick = (String)component.getAttributes().get("ondblclick");
			onkeydown = (String)component.getAttributes().get("onkeydown");
			onkeypress = (String)component.getAttributes().get("onkeypress");
			onkeyup = (String)component.getAttributes().get("onkeyup");
			onmousedown = (String)component.getAttributes().get("onmousedown");
			onmousemove = (String)component.getAttributes().get("onmousemove");
			onmouseout = (String)component.getAttributes().get("onmouseout");
			onmouseover = (String)component.getAttributes().get("onmouseover");
			onmouseup = (String)component.getAttributes().get("onmouseup");
			scrolling = (String)component.getAttributes().get("scrollSize");
			forceFixedWidthsHelper = (String)component.getAttributes().get("forceFixedWidths");
		}
		
		if(forceFixedWidthsHelper != null)
			forceFixedWidths = forceFixedWidthsHelper.trim().equalsIgnoreCase("true");		

 		if(forceFixedWidths == true || scrolling != null)
			splitTables = true;
 		
		if(splitTables == true){
			if(width == null)
				width = "100%";
			
			if(style == null)
				style = "table-layout: fixed;";
			else
				style += "table-layout: fixed;";
		}
		
		//end main body table - table3
		sb.endElement("table");
		if(scrolling != null)
			sb.endElement("div");
		sb.writeText("\n",null);
	
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
		
		//Start rendering the footer tables
		boolean renderFooterCells = (getNumberOfTables(context, component).charAt(3) == '1');
		boolean renderFooterFacet = (getNumberOfTables(context, component).charAt(4) == '1');
		
		if(splitTables == true) {
			
			if(renderFooterCells == true) {
				//table1
				sb.startElement("table", component);
				sb.writeAttribute("id", component.getClientId(context) + "foot1", null);		

				//if this facet is displayed the top border will always be ommitted
				//the bottom border should be ommitted if there is a footer facet
				String footerCellsStyle = style;				
				footerCellsStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px; " + footerCellsStyle;
				
				if(renderFooterFacet == true)
					footerCellsStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px; " + footerCellsStyle;
				
				if(footerCellsStyle!= null)
					sb.writeAttribute("style", footerCellsStyle, "style");
				
				if(style!= null)
					sb.writeAttribute("style", style,"style");		
						
				if(title!= null)
					sb.writeAttribute("title", title,"title");		
				
				if(styleClass!= null)
					sb.writeAttribute("class", styleClass, null);	
				
				if(dir != null)
					sb.writeAttribute("dir", dir,"dir");
	
				if(lang != null)
					sb.writeAttribute("lang", lang,"lang");
					
				if(width != null)
					sb.writeAttribute("width", width,"width");
				
				if(border != null)
					sb.writeAttribute("border", border,"border");
				
				if(cellpadding != null)
					sb.writeAttribute("cellpadding", cellpadding,"cellpadding");	
	
				if(cellspacing != null)
					sb.writeAttribute("cellspacing", cellspacing,"cellspacing");	
				
				if(bgcolor != null)
					sb.writeAttribute("bgcolor", bgcolor,"bgcolor");			
	
				if(frame != null)
					sb.writeAttribute("frame", frame,"frame");		
	
				if(rules != null)
					sb.writeAttribute("rules", rules,"rules");
	
				if(summary != null)
					sb.writeAttribute("summary", summary,"summary");		
						
				if(onclick != null)
					sb.writeAttribute("onclick", onclick,"onclick");
	
				if(ondblclick != null)
					sb.writeAttribute("ondblclick", ondblclick,"ondblclick");
				
				if(onkeydown != null)
					sb.writeAttribute("onkeydown", onkeydown,"onkeydown");
				
				if(onkeypress != null)
					sb.writeAttribute("onkeypress", onkeypress,"onkeypress");
				
				if(onkeyup != null)
					sb.writeAttribute("onkeyup", onkeyup,"onkeyup");
				
				if(onmousedown != null)
					sb.writeAttribute("onmousedown", onmousedown,"onmousedown");
				
				if(onmousemove != null)
					sb.writeAttribute("onmousemove", onmousemove,"onmousemove");
	
				if(onmouseout != null)
					sb.writeAttribute("onmouseout", onmouseout,"onmouseout");
	
				if(onmouseover != null)
					sb.writeAttribute("onmouseover", onmouseover,"onmouseover");
	
				if(onmouseup != null)
					sb.writeAttribute("onmouseup", onmouseup,"onmouseup");
				
				sb.checkTagOpenState();
				writer.write(sb.toString());
				sb.clearBuffer();
				
				doFooterCells(context, component);
				
				sb.endElement("table");
				sb.writeText("\n",null);
				
				sb.checkTagOpenState();
				writer.write(sb.toString());
				sb.clearBuffer();
			}
			
			if(renderFooterFacet == true) {
				//table1
				sb.startElement("table", component);
				sb.writeAttribute("id", component.getClientId(context) + "foot2", null);		

				//if this facet is displayed the top border will always be ommitted
				//the bottom border should always be shown
				String footerFacetStyle = style;				
				footerFacetStyle = "border-bottom-width:0px; border-top-width:0px; border-left-width:0px; border-right-width:0px; " + footerFacetStyle;
				
				if(footerFacetStyle!= null)
					sb.writeAttribute("style", footerFacetStyle, "style");
				
				if(style!= null)
					sb.writeAttribute("style", style,"style");		
						
				if(title!= null)
					sb.writeAttribute("title", title,"title");		
				
				if(styleClass!= null)
					sb.writeAttribute("class", styleClass, null);	
				
				if(dir != null)
					sb.writeAttribute("dir", dir,"dir");
	
				if(lang != null)
					sb.writeAttribute("lang", lang,"lang");
					
				if(width != null)
					sb.writeAttribute("width", width,"width");
				
				if(border != null)
					sb.writeAttribute("border", border,"border");
				
				if(cellpadding != null)
					sb.writeAttribute("cellpadding", cellpadding,"cellpadding");	
	
				if(cellspacing != null)
					sb.writeAttribute("cellspacing", cellspacing,"cellspacing");	
				
				if(bgcolor != null)
					sb.writeAttribute("bgcolor", bgcolor,"bgcolor");			
	
				if(frame != null)
					sb.writeAttribute("frame", frame,"frame");		
	
				if(rules != null)
					sb.writeAttribute("rules", rules,"rules");
	
				if(summary != null)
					sb.writeAttribute("summary", summary,"summary");		
						
				if(onclick != null)
					sb.writeAttribute("onclick", onclick,"onclick");
	
				if(ondblclick != null)
					sb.writeAttribute("ondblclick", ondblclick,"ondblclick");
				
				if(onkeydown != null)
					sb.writeAttribute("onkeydown", onkeydown,"onkeydown");
				
				if(onkeypress != null)
					sb.writeAttribute("onkeypress", onkeypress,"onkeypress");
				
				if(onkeyup != null)
					sb.writeAttribute("onkeyup", onkeyup,"onkeyup");
				
				if(onmousedown != null)
					sb.writeAttribute("onmousedown", onmousedown,"onmousedown");
				
				if(onmousemove != null)
					sb.writeAttribute("onmousemove", onmousemove,"onmousemove");
	
				if(onmouseout != null)
					sb.writeAttribute("onmouseout", onmouseout,"onmouseout");
	
				if(onmouseover != null)
					sb.writeAttribute("onmouseover", onmouseover,"onmouseover");
	
				if(onmouseup != null)
					sb.writeAttribute("onmouseup", onmouseup,"onmouseup");
				
				sb.checkTagOpenState();
				writer.write(sb.toString());
				sb.clearBuffer();
				
				doFooterFacet(context, component);
				
				sb.endElement("table");
				sb.writeText("\n",null);
				
				sb.checkTagOpenState();
				writer.write(sb.toString());
				sb.clearBuffer();
			}
		}
			
		//add javascript for scrolling
		if(scrolling != null) {
	    	StringBuffer script = new StringBuffer();
	    	script.append(HxClientRenderUtil.HX_VAR_NAME + ".addComponent(\"" + component.getClientId(context) + "\", new " + HxClientRenderUtil.HX_VAR_NAME + ".JSFDataTable(\"adjust-columns:true\"));");
	    	String scriptString  = script.toString(); 
			writeJavaScript(context, component, scriptString);
		}
	}

	public boolean getRendersChildren() {
		return true;
	}

	/**
	 * this method will render the footer cells for the table if one exists
	 **/	
	private void doFooterCells(FacesContext context, UIComponent component) throws IOException {

		boolean splitTables = false;
		boolean forceFixedWidths = false;
		
		String forceFixedWidthsHelper = null;
		String scrolling = null;
		
		com.ibm.faces.component.html.HtmlDataTableEx hx_table = null;
		
		if (component instanceof com.ibm.faces.component.html.HtmlDataTableEx){
			hx_table = (com.ibm.faces.component.html.HtmlDataTableEx)component;
		}

		if (hx_table != null){
			scrolling = hx_table.getScrollSize();
			forceFixedWidthsHelper = hx_table.getForceFixedWidths();
		} else {
			scrolling = (String)component.getAttributes().get("scrollSize");
			forceFixedWidthsHelper = (String)component.getAttributes().get("forceFixedWidths");
		}

		if(forceFixedWidthsHelper != null)
			forceFixedWidths = forceFixedWidthsHelper.trim().equalsIgnoreCase("true");		

 		if(forceFixedWidths == true || scrolling != null)
			splitTables = true;

 		//get a sb
		ResponseWriterBuffer sb = new ResponseWriterBuffer();
		ResponseWriter writer = context.getResponseWriter();

		ArrayList columnList = getColumnList((UIData)component);
		
		sb.startElement("tfoot", component);
		
		String spacerImageSrc=JavaScriptUtil.getImagePath(context) + "1x1.gif"; // get the spacer image
		spacerImageSrc=context.getExternalContext().encodeResourceURL(spacerImageSrc);
			
		sb.startElement("tr",component);
		Iterator cols = columnList.iterator();
		
		String lastStyle = null;
		
		while(cols.hasNext()){
			UIColumn column = (UIColumn)cols.next();
				
			//get all the attribute for the column
			String align = null;
			String axis = null;
			String background = null;
			String bgcolorCol = null;
			String dirCol = null;
			String footerClass = null;
			String height = null;
			String nowrap = null;
			String scope = null;
			String styleCol = null;
			String valign = null;
			String widthCol = null;
			
			com.ibm.faces.component.UIColumnEx hx_column = null;
			
			if (column instanceof com.ibm.faces.component.UIColumnEx){
				hx_column = (com.ibm.faces.component.UIColumnEx)column;
			}

			if (hx_column != null){
				align = hx_column.getAlign();
				axis = hx_column.getAxis();
				background = hx_column.getBackground();
				bgcolorCol = hx_column.getBgcolor();
				dirCol = hx_column.getDir();
				footerClass = hx_column.getFooterClass();
				height = hx_column.getHeight();
				nowrap = hx_column.getNowrap();
				scope = hx_column.getScope();
				styleCol = hx_column.getStyle();
				valign = hx_column.getValign();
				widthCol = hx_column.getWidth();
			} else {
				align = (String)column.getAttributes().get("align");
				axis = (String)column.getAttributes().get("axis");
				background = (String)column.getAttributes().get("background");
				bgcolorCol = (String)column.getAttributes().get("bgcolor");
				dirCol = (String)column.getAttributes().get("dir");
				footerClass = (String)column.getAttributes().get("footerClass");
				height = (String)column.getAttributes().get("height");
				nowrap = (String)column.getAttributes().get("nowrap");
				scope = (String)column.getAttributes().get("scope");
				styleCol = (String)column.getAttributes().get("style");
				valign = (String)column.getAttributes().get("valign");
				widthCol = (String)column.getAttributes().get("width");
			}
				
			//start the TD
            if(scope != null && scope.equalsIgnoreCase("row"))
            	sb.startElement("th",column);
            else
            	sb.startElement("td",column);
			
			//write the column attributes
			if (align != null)
				sb.writeAttribute("align", align, null);
			
			if (axis != null)
				sb.writeAttribute("axis", axis, null);
			
			if (background != null)
				sb.writeAttribute("background", background, null);
			
			if (bgcolorCol != null)
				sb.writeAttribute("bgcolor", bgcolorCol, null);
			
			if (dirCol != null)
				sb.writeAttribute("dir", dirCol, null);
			
			if (footerClass != null)
				sb.writeAttribute("class", footerClass, null);
			
			if (height != null)
				sb.writeAttribute("height", height, null);

			if (nowrap != null)
				sb.writeAttribute("nowrap", nowrap, null);
			
			if (scope != null)
				sb.writeAttribute("scope", scope, null);

			//if it is the last column
			if(cols.hasNext() == false && scrolling != null) {
				if(styleCol != null)
					sb.writeAttribute("style", "border-right-width:0px; " + styleCol, null);
				else
					sb.writeAttribute("style", "border-right-width:0px;", null);
				lastStyle = footerClass;
			}
			else if (styleCol != null) {
				sb.writeAttribute("style", styleCol, null);
			}
			
			if (valign != null)
				sb.writeAttribute("valign", valign, null);
			
			if (widthCol != null)
				sb.writeAttribute("width", widthCol, null);
			
			if(widthCol != null) {
				sb.startElement("div",column);
				
				if(widthCol.charAt(widthCol.length()-1) == '%')
					sb.writeAttribute("style", "overflow: -moz-scrollbars-none; overflow-x: hidden; width: 100%;", null);
				else
				{
					widthCol = Utils.checkMeasurementMetric(widthCol);
					sb.writeAttribute("style", "overflow: -moz-scrollbars-none; overflow-x: hidden; width:" + widthCol, null);
				}
			}
			
			UIComponent columnFacet = column.getFacet("footer");
			if(columnFacet != null){
				sb.checkTagOpenState();
				writer.write(sb.toString());
				sb.clearBuffer();
				Utils.encodeComponent(context,columnFacet);
			} else {
				sb.startElement("img",columnFacet);
				sb.writeAttribute("src",spacerImageSrc,null);
				sb.endElement("img");				
			}
			
			if(widthCol != null)
				sb.endElement("div");
			
			sb.endElement("td");
		}
			
		if(scrolling != null) {
			//add extra "TD" to footer cells row
			sb.startElement("td",component);
			
			if (lastStyle != null)
				sb.writeAttribute("class", lastStyle, null);

			//set borders and padding to 0pt
			//set left border to 0px as well
			sb.writeAttribute("style", "margin:0px; padding:0px; padding-left:0pt; padding-right:0pt; border-left-width:0px", null);
			
			sb.startElement("div",component);					
			sb.writeAttribute("style", "overflow: -moz-scrollbars-none; overflow-x: hidden;", null);
			sb.endElement("div");
			sb.endElement("td");
		}
		
		sb.endElement("tr");
		sb.writeText("\n",null);
		
		boolean renderFooterFacet = (getNumberOfTables(context, component).charAt(4) == '1');
		
		//if the table is split up end the tfoot
		if(splitTables == true){
			sb.endElement("tfoot");
		}
		//if there is no footer facet below this, end the tfoot
		else if(renderFooterFacet == false){
			sb.endElement("tfoot");
		}
		
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
	}

	/**
	 * this method will render the footer facet for the table if one exists
	 **/	
	private void doFooterFacet(FacesContext context, UIComponent component) throws IOException {
		
		boolean splitTables = false;
		boolean forceFixedWidths = false;
		
		String forceFixedWidthsHelper = null;
		String scrolling = null;
		String footerclass = null;
		
		javax.faces.component.html.HtmlDataTable h_table = null;
		com.ibm.faces.component.html.HtmlDataTableEx hx_table = null;
		
		if (component instanceof javax.faces.component.html.HtmlDataTable){
			h_table = (javax.faces.component.html.HtmlDataTable)component;
		} else if (component instanceof com.ibm.faces.component.html.HtmlDataTableEx){
			hx_table = (com.ibm.faces.component.html.HtmlDataTableEx)component;
		}

		if(h_table != null){
			footerclass= h_table.getFooterClass();
			scrolling = (String)component.getAttributes().get("scrollSize");
			forceFixedWidthsHelper = (String)component.getAttributes().get("forceFixedWidths");			
		} else if (hx_table != null){
			footerclass= hx_table.getFooterClass();
			scrolling = hx_table.getScrollSize();
			forceFixedWidthsHelper = hx_table.getForceFixedWidths();
		} else {
			footerclass = (String)component.getAttributes().get("footerClass");
			scrolling = (String)component.getAttributes().get("scrollSize");
			forceFixedWidthsHelper = (String)component.getAttributes().get("forceFixedWidths");
		}		
		
		if(forceFixedWidthsHelper != null)
			forceFixedWidths = forceFixedWidthsHelper.trim().equalsIgnoreCase("true");		

 		if(forceFixedWidths == true || scrolling != null)
			splitTables = true;
				
		//render content of the footer facet
		ResponseWriterBuffer sb = new ResponseWriterBuffer();
		ResponseWriter writer = context.getResponseWriter(); // get a sb		
		
		ArrayList columnList = getColumnList((UIData)component);
		
		//table footer
		UIComponent tableFooter = component.getFacet("footer");
		
		boolean renderFooterCells = (getNumberOfTables(context, component).charAt(3) == '1');
		
		//render the content of the table header facet
		if(splitTables == true) {
			sb.startElement("tfoot", component);
		}
		//if there were no footer cells above this then render the tfoot
		else if(renderFooterCells == false) {
			sb.startElement("tfoot", component);
		}
		

		sb.startElement("tr", component);
		sb.startElement("td", component);
		sb.writeAttribute("scope", "colgroup", null);
		
		if(footerclass != null)
			sb.writeAttribute("class", footerclass, null);

		if(splitTables == false && columnList != null){
			sb.writeAttribute("colspan", Integer.toString(columnList.size()), null);
		}
		
		if(scrolling != null)
			sb.writeAttribute("style", "border-right-width:0px", null);
		
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
		Utils.encodeComponent(context,tableFooter);
		
		sb.endElement("td");
		
		//add extra "TD" to footer facet
		if(scrolling != null) {
			sb.startElement("td",component);
			sb.writeAttribute("scope", "colgroup", null);
			
			if(footerclass != null)			
				sb.writeAttribute("class", footerclass, null);
			
			//set borders and padding to 0pt
			//set left border to 0px as well
			sb.writeAttribute("style", "margin:0px; padding:0px; padding-left:0pt; padding-right:0pt; border-left-width:0px", null);
			
			sb.startElement("div",component);					
			sb.writeAttribute("style", "overflow: -moz-scrollbars-none; overflow-x: hidden;", null);
			sb.endElement("div");
			sb.endElement("td");
		}
		
		sb.endElement("tr");
		sb.writeText("\n",null);

		sb.endElement("tfoot");
		
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
	
	}

	private ArrayList getColumnList(UIData table){
		ArrayList list = new ArrayList();
		Iterator children = table.getChildren().iterator();
		while(children.hasNext()){
			UIComponent child = (UIComponent)children.next();
			if(child instanceof UIColumn && child.isRendered()){
				list.add(child);
				
		    	//reset facet ids so correct client ids are reset for facets - this fixes a bug in UIData
    			resetFacetIds(child);
			}
		}
		return list;
	}

	
	private Boolean columnFacetCheck(Iterator columns, String facetName){
		while(columns.hasNext()){
			UIComponent comp = (UIComponent)columns.next();
			if(null != (comp.getFacet(facetName))){
				return Boolean.TRUE;
			}
		}
		return Boolean.FALSE;
	}
	
	private Object[] getStyleClassIterator(String classes, String delimeter){
		ArrayList list  = new ArrayList();
		StringTokenizer st = new StringTokenizer(classes,delimeter);
		while(st.hasMoreElements()){
			list.add(st.nextToken().trim());
		}
		return list.toArray();
	}
	
	//bug in UIData where facet ids are not reset to set clientId to null causing 
	//facets not having the row index as part of their clientId
	private void resetFacetIds(UIComponent component){
		Iterator facets = component.getFacets().values().iterator();
		while(facets.hasNext()){
			UIComponent facet = (UIComponent)facets.next();
			facet.setId(facet.getId());
		}
		
		Iterator children = component.getChildren().iterator();
		while(children.hasNext()){
			resetFacetIds((UIComponent)children.next());
		}
	}
	
    private void writeJavaScript(FacesContext context, UIComponent component, String script) throws IOException {

        UIScriptCollector scriptCollector = UIScriptCollector.find(component);
                
        if (scriptCollector != null) {
        	HxClientRenderUtil.initJSLibraries(component, context);
        	HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
    		HxClientRenderUtil.addScriptLibrary("hxclient_table.js",component);
            scriptCollector.addScriptOnce(script);
        }
        else 
        {
        	ResourceBundle rb = ResourceHandler.getBundle(context);	            	
        	throw new FacesException(ResourceHandler.getString(rb, "NoScriptCollectorFoundError"));
        }
    }
    
    private String getNumberOfTables (FacesContext context, UIComponent component) {
		
    	String tablesNeeded = "";
    	
    	//check for table caption
    	UIComponent tableCaption = component.getFacet("caption");
    	if(tableCaption != null)
			tablesNeeded += "1";
    	else
    		tablesNeeded += "0";
    	
    	//check for table header facet
    	UIComponent tableHeader = component.getFacet("header");
    	if(tableHeader != null)
			tablesNeeded += "1";
    	else
    		tablesNeeded += "0";
    	
    	//check for table header cells
		ArrayList columnList = getColumnList((UIData)component);
		if(columnList != null && columnList.size() > 0 && columnFacetCheck(columnList.iterator(),"header").booleanValue())		
			tablesNeeded += "1";
		else
			tablesNeeded += "0";

    	//check for table footer cells
		if(columnList != null && columnList.size() > 0 && columnFacetCheck(columnList.iterator(),"footer").booleanValue())		
			tablesNeeded += "1";
		else
			tablesNeeded += "0";

    	//check for table footer facet
    	UIComponent tableFooter = component.getFacet("footer");
    	if(tableFooter != null)
			tablesNeeded += "1";
    	else
    		tablesNeeded += "0";
		
		return tablesNeeded;
    }
    
    //Designer refactoring. We use the styles specified here but also need "white-space:nowrap" too. So we override this to
    //add that part on.
    protected String getColumnDivStyle() {
		return "overflow: -moz-scrollbars-none; overflow-x: hidden; ";
	}

    /*
     * isRowAvailable is provided as a separate method so that Designer's runtime 
     * can alter when a tables rendering is completed. In designer not all rows are treated equally.
     */
    protected boolean isRowAvailable(FacesContext context, UIData dataComponent, int rowsToDisplay, int processedRowCount) {
		if(!dataComponent.isRowAvailable() || (rowsToDisplay > 0 && processedRowCount >= rowsToDisplay)){
			return false;
		}else{
			return true;
		}
	}

    /*
     * endRow closes off the current row and increments the processedRowCount. The reason this is 
     * refactored out into its own method is so it can be overriden by the Designer runtime
     */
	protected int endRow(ResponseWriterBuffer sb, FacesContext context, UIData dataComponent, Boolean processRow, int processedRowCount, int rowsToDisplay) throws IOException {

		sb.endElement("tr");
		sb.writeText("\n",null);

        return ++processedRowCount;
	}

	/*
	 * startRow simply starts the row. This is extracted out into a separate method so it can be 
	 * overriden by Designer.
	 */
	protected void startRow(ResponseWriterBuffer sb, FacesContext context, UIData component) throws IOException {
    		sb.startElement("tr", component); // $NON-NLS-1$
	}

    
    
}