/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.io.Writer;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;

import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.UIPager;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.util.HtmlUtil;
import com.ibm.faces.util.JavaScriptUtil;

import com.ibm.faces.component.UIPager.PagerDataWrapper;
import com.ibm.faces.component.html.HtmlPagerDeluxe;

/**
 * @author Alasdair McDowell
 * Deluxe pager renderer.
 */
public class DeluxePagerRenderer extends AbstractPagerRenderer implements IScriptContributor{

    private static final String PAGER_PREVIOUS = "__pagerPrevious"; //$NON-NLS-1$
    private static final String PAGER_NEXT = "__pagerNext"; //$NON-NLS-1$
    private static final String PAGER_FIRST = "__pagerFirst"; //$NON-NLS-1$
    private static final String PAGER_LAST = "__pagerLast"; //$NON-NLS-1$
    private static final String PAGER_TEXT = "__pagerText"; //$NON-NLS-1$

    //private static final HxSubmitIcon ICON_PAGER_FIRST = new HxSubmitIcon("ibmjsfres/img/datagrid_pager_first.txt"); //$NON-NLS-1$
    //private static final HxSubmitIcon ICON_PAGER_PREVIOUS = new HxSubmitIcon("ibmjsfres/img/datagrid_pager_previous.txt"); //$NON-NLS-1$
    //private static final HxSubmitIcon ICON_PAGER_NEXT = new HxSubmitIcon("ibmjsfres/img/datagrid_pager_next.txt"); //$NON-NLS-1$
    //private static final HxSubmitIcon ICON_PAGER_LAST = new HxSubmitIcon("ibmjsfres/img/datagrid_pager_last.txt"); //$NON-NLS-1$
    
    private static final String PAGER_FIRST_IMG = Utils.versionizeFilename("datagrid_pager_first.gif");
    private static final String PAGER_PREVIOUS_IMG = Utils.versionizeFilename("datagrid_pager_previous.gif");
    private static final String PAGER_NEXT_IMG = Utils.versionizeFilename("datagrid_pager_next.gif");
    private static final String PAGER_LAST_IMG = Utils.versionizeFilename("datagrid_pager_last.gif");

    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {

    	if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
    	
		HtmlPagerDeluxe deluxeComponent = null;
		if ( component instanceof HtmlPagerDeluxe ){
			deluxeComponent = (HtmlPagerDeluxe)component;
		}
		

        Writer writer = context.getResponseWriter();
        ResourceBundle bundle = ResourceHandler.getBundle(context);
        
        PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
        int currentPage = ((UIPager)component).getCurrentPage(pdw);
        int lastPage = ((UIPager)component).getLastPage(pdw);

        Object[] args = { new Integer(currentPage + 1), new Integer(lastPage + 1)};
        String localizedPageOf = ResourceHandler.getString(bundle, "PagerRenderer.pageof", args); //$NON-NLS-1$
        String localizedFirst = ResourceHandler.getString(bundle, "PagerRenderer.first"); //$NON-NLS-1$
        String localizedPrevious = ResourceHandler.getString(bundle, "PagerRenderer.previous"); //$NON-NLS-1$
        String localizedNext = ResourceHandler.getString(bundle, "PagerRenderer.next"); //$NON-NLS-1$
        String localizedLast = ResourceHandler.getString(bundle, "PagerRenderer.last"); //$NON-NLS-1$

        String pagerId = component.getClientId(context);
        
        String style;
        String styleClass;
        Object useButtonsVal;
        Object showOnlyNextPreviousVal;
        Object hideDisabledButtonsVal;
        String title;
        String tabindex;
        
        if ( deluxeComponent != null ){
        	style = deluxeComponent.getStyle();
        	styleClass = deluxeComponent.getStyleClass();
        	useButtonsVal = deluxeComponent.getUseButtons();
        	showOnlyNextPreviousVal = deluxeComponent.getShowOnlyNextPrevious();
        	hideDisabledButtonsVal = deluxeComponent.getHideDisabledButtons();
        	title = deluxeComponent.getTitle();
        	tabindex = deluxeComponent.getTabindex();
        }else{
			style = (String) component.getAttributes().get("style");
        	styleClass = (String) component.getAttributes().get("styleClass");
        	useButtonsVal = component.getAttributes().get("useButtons");
        	showOnlyNextPreviousVal = component.getAttributes().get("showOnlyNextPrevious");
        	hideDisabledButtonsVal = component.getAttributes().get("hideDisabledButtons");
        	title = (String) component.getAttributes().get("title");
        	tabindex = (String) component.getAttributes().get("tabindex");
        }
    
        String idFirst = pagerId + PAGER_FIRST;
        String idPrevious = pagerId + PAGER_PREVIOUS;
        String idNext = pagerId + PAGER_NEXT;
        String idLast = pagerId + PAGER_LAST;

        boolean useButtons = "true".equals(useButtonsVal) || Boolean.TRUE.equals(useButtonsVal); //$NON-NLS-1$
        boolean showOnlyNextPrevious = "true".equals(showOnlyNextPreviousVal) || Boolean.TRUE.equals(showOnlyNextPreviousVal); //$NON-NLS-1$
        boolean hideDisabledButtons = "true".equals(hideDisabledButtonsVal) || Boolean.TRUE.equals(hideDisabledButtonsVal); //$NON-NLS-1$

        boolean canPrev = currentPage > 0;
        boolean canNext = currentPage < lastPage || lastPage < 0;
        //if lastPage < 0, allow it

        HtmlUtil.writeTagStartOpen(writer, "table", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", pagerId); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "style", style); //$NON-NLS-1$
        if(title != null)
        	HtmlUtil.writeTagAttribute(writer, "title", title); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "border", "0"); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "cellspacing", "0"); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "cellpadding", "0"); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagStartCloseLn(writer, false);
        HtmlUtil.writeTagStartOpenLn(writer, "tbody", true, false); //$NON-NLS-1$
        HtmlUtil.writeTagStartOpenLn(writer, "tr", true, false); //$NON-NLS-1$

        if (useButtons) {

        	if(!(!(!canPrev && hideDisabledButtons) && !showOnlyNextPrevious))
        		localizedFirst = "";
        	if(!canPrev && hideDisabledButtons)
        		localizedPrevious = "";
        	if(!canNext && hideDisabledButtons)
            	localizedNext = "";
        	if(!(!(!canNext && hideDisabledButtons) && !showOnlyNextPrevious))
            	localizedLast = "";
            	
	    	this.writeInputSubmit(
	    		component,
				writer,
				idFirst,
				localizedFirst,
				styleClass,
				canPrev,
				showOnlyNextPrevious,
				hideDisabledButtons,
				tabindex);

            this.writeInputSubmit(
                component,
                writer,
                idPrevious,
                localizedPrevious,
                styleClass,
				canPrev,
				false,
				hideDisabledButtons,
				tabindex);
            
            this.writeText(writer, pagerId, localizedPageOf, styleClass);
            
            this.writeInputSubmit(
                component,
                writer,
                idNext,
                localizedNext,
                styleClass,
                canNext,
				false,
				hideDisabledButtons,
				tabindex);				
            
            if (lastPage >= 0) {
                this.writeInputSubmit(
                    component,
                    writer,
                    idLast,
                    localizedLast,
                    styleClass,
	                canNext,
					showOnlyNextPrevious,
					hideDisabledButtons,
					tabindex);
            }
        }
        else {
        	String srcFirst;
        	String imgFirst;
        	String srcPrevious;
        	String imgPrevious;
        	String srcNext;
        	String imgNext;
        	String srcLast;
        	String imgLast;
        	
        	if ( deluxeComponent != null ){
        		srcFirst = deluxeComponent.getSrcFirst();
        		srcPrevious = deluxeComponent.getSrcPrevious();
        		srcNext = deluxeComponent.getSrcNext();
        		srcLast = deluxeComponent.getSrcLast();
        	}else{
				srcFirst = (String) component.getAttributes().get("srcFirst"); //$NON-NLS-1$
				srcPrevious = (String) component.getAttributes().get("srcPrevious"); //$NON-NLS-1$
				srcNext = (String) component.getAttributes().get("srcNext"); //$NON-NLS-1$
				srcLast = (String) component.getAttributes().get("srcLast"); //$NON-NLS-1$
        	}
        	imgFirst = this.encodeResource(context, srcFirst, PAGER_FIRST_IMG); 
        	imgPrevious = this.encodeResource(context, srcPrevious, PAGER_PREVIOUS_IMG); 
        	imgNext = this.encodeResource(context, srcNext, PAGER_NEXT_IMG);
        	imgLast = this.encodeResource(context, srcLast, PAGER_LAST_IMG);
        	
            String img1x1 = this.encodeResource(context, null, "1x1.gif"); //$NON-NLS-1$
    
            //determine whether to draw icons using script or use images
            boolean iconSupported = useScript(context);
            StringBuffer buffer = null;
            
            if (iconSupported && srcFirst == null) {
                imgFirst = img1x1;
                
                if(!(!canPrev && hideDisabledButtons) && !showOnlyNextPrevious)
                		buffer = this.bufferScript(buffer, idFirst, false, true);
            }
            else {
            	if((!canPrev && hideDisabledButtons) && !showOnlyNextPrevious)
            		imgFirst = img1x1;
            }
            if (iconSupported && srcPrevious == null) {
                imgPrevious = img1x1;
                
                if(!(!canPrev && hideDisabledButtons))
                	buffer = this.bufferScript(buffer, idPrevious, false, false);
            }
            else {
            	if((!canPrev && hideDisabledButtons))
                	imgPrevious = img1x1;
            }
            if (iconSupported && srcNext == null) {
                imgNext = img1x1;
                if(!(!canNext && hideDisabledButtons))
                	buffer = this.bufferScript(buffer, idNext, true, false);
            }
            else {
            	if((!canNext && hideDisabledButtons))
                	imgNext = img1x1;
            }
            if (iconSupported && srcLast == null) {
                imgLast = img1x1;

                if(!(!canNext && hideDisabledButtons) && !showOnlyNextPrevious)
                			buffer = this.bufferScript(buffer, idLast, true, true);
            }
            else {
            	if((!canNext && hideDisabledButtons) && !showOnlyNextPrevious)
                	imgLast = img1x1;
            }

            this.writeInputImage(
                component,
                writer,
                idFirst,
                localizedFirst,
                imgFirst,
                styleClass,
                canPrev,
				showOnlyNextPrevious,
				hideDisabledButtons,
				tabindex);					
            
            this.writeInputImage(
                component,
                writer,
                idPrevious,
                localizedPrevious,
                imgPrevious,
                styleClass,
                canPrev,
				false,
				hideDisabledButtons,
				tabindex);
            
            this.writeText(writer, pagerId, localizedPageOf, styleClass);
            
            this.writeInputImage(
                component,
                writer,
                idNext,
                localizedNext,
                imgNext,
                styleClass,
                canNext,
                false,
                hideDisabledButtons,
				tabindex);
            
            if (lastPage >= 0) {
                this.writeInputImage(
                    component,
                    writer,
                    idLast,
                    localizedLast,
                    imgLast,
                    styleClass,
	                canNext,
					showOnlyNextPrevious,
					hideDisabledButtons,
					tabindex);
            }

            //write script
            if (buffer != null) {
                this.writeJavaScript(context, component, buffer);
            }

        }

        HtmlUtil.writeTagEndLn(writer, "tr"); //$NON-NLS-1$
        HtmlUtil.writeTagEndLn(writer, "tbody"); //$NON-NLS-1$
        HtmlUtil.writeTagEndLn(writer, "table"); //$NON-NLS-1$
    }

    private String encodeResource(FacesContext context, String customImage, String defaultImage) {
    	String imageURL = null;
        if (customImage != null) {
        	imageURL =  Utils.getEncodedResourceURL(context, customImage);
        } else {
        	imageURL = context.getExternalContext().encodeResourceURL(JavaScriptUtil.getImagePath(context, defaultImage));
        }
        return imageURL;
    }

    private void writeText(
        Writer writer,
        String id,
        String localizedPageOf,
        String styleClass)
        throws IOException {

        String textId = id + PAGER_TEXT;

        writeTdStart(writer, styleClass);

        if (styleClass != null) {
            styleClass += "_text"; //$NON-NLS-1$
        }

        HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", textId); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
        HtmlUtil.writeTagStartClose(writer, false);
        HtmlUtil.writeText(writer, localizedPageOf);
        HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$

        writeTdEnd(writer);
    }

    /**
     * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     */
    public void decode(FacesContext context, UIComponent component) {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
         if (pdw == null) {
            return;
        }
        
        if (!((UIPager)component).isPaged(pdw)) {
            return;
        }
        
    	int newPage = -1;
        int currentPage = 0;
        int lastPage = 0;
        String id = component.getClientId(context);

        //        boolean iconSupported = useScript(context);
        //        String srcFirst = (String) component.getAttributes().get("srcFirst"); //$NON-NLS-1$
        //        String srcPrevious = (String) component.getAttributes().get("srcPrevious"); //$NON-NLS-1$
        //        String srcNext = (String) component.getAttributes().get("srcNext"); //$NON-NLS-1$
        //        String srcLast = (String) component.getAttributes().get("srcLast"); //$NON-NLS-1$

        if (this
            .wasInputClicked(
                context,
                component,
                id + PAGER_FIRST)) {
        	currentPage = ((UIPager)component).getCurrentPage(pdw);
            lastPage = ((UIPager)component).getLastPage(pdw);
            newPage = 0;
        }
        else if (
            this.wasInputClicked(
                context,
                component,
                id + PAGER_PREVIOUS)) {
        	currentPage = ((UIPager)component).getCurrentPage(pdw);
            lastPage = ((UIPager)component).getLastPage(pdw);
            newPage = currentPage - 1;
        }
        else if (
            this.wasInputClicked(
                context,
                component,
                id + PAGER_NEXT)) {
            currentPage = ((UIPager)component).getCurrentPage(pdw);
            lastPage = ((UIPager)component).getLastPage(pdw);
            newPage = currentPage + 1;
        }
        else if (
            (this
                .wasInputClicked(
                    context,
                    component,
                    id + PAGER_LAST))
                && (lastPage >= 0)) {
            currentPage = ((UIPager)component).getCurrentPage(pdw);
            lastPage = ((UIPager)component).getLastPage(pdw);
            newPage = lastPage;
        }
        else {
            return;
        }

        this.queueEvent(component, currentPage, newPage, lastPage);
    }

    protected void writeInputImage(
        UIComponent component,
        Writer writer,
        String id,
        String label,
        String image,
        String css,
        boolean enabled,
		boolean showOnlyNextPrevious,
		boolean hideDisabledButtons,
		String tabindex)		
        throws IOException {
    	//UIComponent parameter passed in not used here, but is used by xfaces
    	if(showOnlyNextPrevious || (!enabled && hideDisabledButtons))
    		//css = "pagerDeluxeHidden";
    		css = css+"Hidden";
    	else
    		//css = "pagerDeluxeActive";
    		css = css+"Active";
    	
    	writeTdStart(writer, css);

        if (css != null) {
            css += "_button"; //$NON-NLS-1$
        }

    	HtmlUtil.writeTagStartOpen(writer, "input", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
        if(tabindex != null)
        	HtmlUtil.writeTagAttribute(writer, "tabindex", tabindex); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "name", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", css); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "alt", label); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "title", label); //$NON-NLS-1$
        //if(css.equals("pagerDeluxeActive")) {
        	HtmlUtil.writeTagAttribute(writer, "src", image); //$NON-NLS-1$
        	HtmlUtil.writeTagAttribute(writer, "type", "image"); //$NON-NLS-1$ //$NON-NLS-2$
        //}
        
        if (!enabled  && !hideDisabledButtons) {
        	HtmlUtil.writeTagAttribute(writer, "disabled", "disabled"); //$NON-NLS-1$ //$NON-NLS-2$
        }

        if(showOnlyNextPrevious || (!enabled && hideDisabledButtons)) {
        	HtmlUtil.writeTagAttribute(writer, "disabled", "disabled"); //$NON-NLS-1$ //$NON-NLS-2$
        }
        
        HtmlUtil.writeTagStartCloseLn(writer, true);

        writeTdEnd(writer);
    }

    protected void writeInputSubmit(
        UIComponent component,
        Writer writer,
        String id,
        String label,
        String css,
        boolean enabled,
		boolean showOnlyNextPrevious,
		boolean hideDisabledButtons,
		String tabindex)
        throws IOException {
    	//UIComponent parameter passed in not used here, but is used by xfaces
    	
    	if(showOnlyNextPrevious || (!enabled && hideDisabledButtons))
    		//css = "pagerDeluxeHidden";
    		css = css+"Hidden";
    	else
    		//css = "pagerDeluxeActive";
    		css = css+"Active";
    		
    	writeTdStart(writer, css);

        if (css != null) {
            css += "_text"; //$NON-NLS-1$
        }

        HtmlUtil.writeTagStartOpen(writer, "input", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
        if(tabindex != null)
        	HtmlUtil.writeTagAttribute(writer, "tabindex", tabindex); //$NON-NLS-1$        
        HtmlUtil.writeTagAttribute(writer, "name", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", css); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "alt", label); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "title", label); //$NON-NLS-1$        
        HtmlUtil.writeTagAttribute(writer, "value", label); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "type", "submit"); //$NON-NLS-1$ //$NON-NLS-2$
        
        if (!enabled  && !hideDisabledButtons) {
        	HtmlUtil.writeTagAttribute(writer, "disabled", "disabled"); //$NON-NLS-1$ //$NON-NLS-2$
        }
        
        if(showOnlyNextPrevious || (!enabled && hideDisabledButtons)) {
        	HtmlUtil.writeTagAttribute(writer, "disabled", "disabled"); //$NON-NLS-1$ //$NON-NLS-2$
        }
        	
        HtmlUtil.writeTagStartCloseLn(writer, true);

        writeTdEnd(writer);
    }

    private void writeTdStart(Writer writer, String styleClass)
        throws IOException {
        /*if (styleClass != null) {
            styleClass += "_cell"; //$NON-NLS-1$
        }*/

        HtmlUtil.writeTagStartOpen(writer, "td", false, false); //$NON-NLS-1$ 
        HtmlUtil.writeTagAttribute(writer, "align", "left"); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "valign", "top"); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagStartCloseLn(writer, false);
    }

    private void writeTdEnd(Writer writer) throws IOException {
        HtmlUtil.writeTagEndLn(writer, "td"); //$NON-NLS-1$ 
    }

    private void writeJavaScript(
        FacesContext context,
        UIComponent component,
        StringBuffer buffer)
        throws IOException {

        if(buffer==null) {
            return;
        }
        
        final String onPageLoad = HxClientRenderUtil.HX_VAR_NAME + ".onPageLoad();\r\n"; //$NON-NLS-1$

        UIScriptCollector scriptCollector = UIScriptCollector.find(component);
        scriptCollector.register(this,component);
        ResponseWriter writer = context.getResponseWriter();
        
        if (scriptCollector == null) {
            ResourceBundle bundle = ResourceHandler.getBundle(context);
            String src = JavaScriptUtil.getJavaScriptPath(context) + ResourceHandler.buildLocalizedLibName(bundle, "hxclient_v1.js"); //$NON-NLS-1$
            writer.write(JavaScriptUtil.getStartTag(src));
            writer.write(JavaScriptUtil.getEndTag());
            writer.write(JavaScriptUtil.getStartTag());
            writer.write(buffer.toString());
            writer.write(JavaScriptUtil.getEndTag());
            writer.write(onPageLoad);
        }
        else {
        	HxClientRenderUtil.initJSLibraries(scriptCollector, context);
        	HxClientRenderUtil.addScriptLibrary("hxclient_im.js", component);
        	scriptCollector.addScript(buffer.toString());            
        }
    }

    private StringBuffer bufferScript(
        StringBuffer buffer,
        String id,
        boolean rightFacing,
        boolean drawBar) {
        /*
        hX_*.addComponent("form1:table1:deluxe1__pagerFirst",   new hX_*.JSFTwistie("direction:2", "bar-style:1"));
        hX_*.addComponent("form1:table1:deluxe1__pagerPrevious",    new hX_*.JSFTwistie("direction:2", "bar-style:0"));
        hX_*.addComponent("form1:table1:deluxe1__pagerNext",        new hX_*.JSFTwistie("direction:3", "bar-style:0"));
        hX_*.addComponent("form1:table1:deluxe1__pagerLast",    new hX_*.JSFTwistie("direction:3", "bar-style:1")); 
        */

        if (buffer == null) {
            buffer = new StringBuffer();
        }

        int direction = rightFacing ? 3 : 2;
        int barStyle = drawBar ? 1 : 0;
        buffer.append(HxClientRenderUtil.HX_VAR_NAME) //$NON-NLS-1$
		.append(".addComponent(\"")
        .append(id).append("\", new ") //$NON-NLS-1$
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".JSFTwistie(\"direction:")
        .append(direction).append("\", \"bar-style:") //$NON-NLS-1$ 
        .append(barStyle).append("\"));\r\n"); //$NON-NLS-1$ 
        return buffer;
    }

    protected boolean wasInputClicked(FacesContext context, UIComponent component, String name) {
    	//UIComponent parameter passed in not used here, but is used by xfaces
        return (context.getExternalContext().getRequestParameterMap().get(name) != null) || (context.getExternalContext().getRequestParameterMap().get(name + ".x") //$NON-NLS-1$
        != null);
    }

    private boolean useScript(FacesContext context) {
        Object obj = context.getExternalContext().getRequestHeaderMap().get("User-Agent"); //$NON-NLS-1$
        if (obj == null) {
            return false;
        }
        
        //System.err.println(obj);
        
        if (obj.toString().indexOf("MSIE") >= 0) { //$NON-NLS-1$
            if(obj.toString().indexOf("Windows") < 0) { //$NON-NLS-1$
                //We haven't tested Mac
                return false;
            }
            //IE versions we don't support anyway
            if (obj.toString().indexOf("MSIE 5.0") >= 0) { //$NON-NLS-1$
                //icon code works, but lib causes other errs
                return false;
            }
            if (obj.toString().indexOf("MSIE 4.") >= 0) { //$NON-NLS-1$
                return false;
            }
            if (obj.toString().indexOf("MSIE 3.") >= 0) { //$NON-NLS-1$
                return false;
            }
            return true;
        } else if(obj.toString().indexOf("Gecko") >=0) { //$NON-NLS-1$
            //Mozilla (Netscape over v4)
            
            return true;
        }

        return false;
    }

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		// TODO Auto-generated method stub
		
	}

}