/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlPanelDialog;

public class DialogRenderer extends Renderer implements IScriptContributor
{
	public static final String DLG_MODAL		= "modal";
	public static final String DLG_MODELESS		= "modeless";
	
	public static final String CONTENT_AREA_ID		= "_ca";
	public static final String TITLE_BAR_ID 	    = "_tb";
	public static final String CLOSE_BUTTON_ID 		= "_cb";
	public static final String CLOSE_BUTTON_IMG_ID 	= "_cbi";	
	public static final String IFRAME_ID        	= "_if";
	
	public static final String PANEL_DIALOG_CSS	= "panelDialog";
	public static final String CONTENT_AREA_CSS	= "_ContentArea";
	public static final String TITLE_BAR_CSS	= "_TitleBar";
	public static final String CLOSE_BUTTON_CSS	= TITLE_BAR_CSS + "-CloseButton";
	public static final String IMAGE_CSS 	    = "-img";	
	public static final String IFRAME_FILTER_STYLE  = " filter:progid:DXImageTransform.Microsoft.Alpha(style=0,opacity=0);";
	
	private static String COMMA_SEPARATOR = ", ";
	private static char CLOSE_QUOTE = '\"';
		
	
	/**
	 * When start of the <panelDialog> is encountered, this method is called.
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException 
	{
		if (context == null || component == null)
			throw new NullPointerException("NULL_PARAMETERS_ERROR_MESSAGE_ID");

		if(!component.isRendered())
			return;
			
		HtmlPanelDialog dialog = null;
		if(component instanceof HtmlPanelDialog)
			dialog = (HtmlPanelDialog )component;
		
		ResponseWriter writer = context.getResponseWriter();		
		String clientId = component.getClientId(context);
		
		String styleClass = null;
		String style_str = null;
		String dir = null;
		String lang	= null;
		String title = null;
		boolean showTitleBar = true; // default is 'true'
		boolean showTitleCloseButton = true; // default is 'true'
		String closeButtonImage = null;
		String onclick = null;
		String ondblclick = null;
		String onmousedown = null;
		String onmouseup = null;
		String onmouseover = null;
		String onmousemove = null;
		String onmouseout = null;
		String onkeypress = null;
		String onkeydown = null;
		String onkeyup = null;
		
		if(dialog != null)
		{
			styleClass = dialog.getStyleClass();
			style_str = dialog.getStyle();
			dir = dialog.getDir();
			lang = dialog.getLang();
			title = dialog.getTitle();
			showTitleBar = dialog.isShowTitleBar();
			showTitleCloseButton = dialog.isShowTitleCloseButton();
			closeButtonImage = dialog.getTitleCloseButtonImage();
			
			onclick = dialog.getOnclick();
			ondblclick = dialog.getOndblclick();
			onmousedown = dialog.getOnmousedown();
			onmouseup = dialog.getOnmouseup();
			onmouseover = dialog.getOnmouseover();
			onmousemove = dialog.getOnmousemove();
			onmouseout = dialog.getOnmouseout();
			onkeypress = dialog.getOnkeypress();
			onkeydown = dialog.getOnkeydown();
			onkeyup = dialog.getOnkeyup();
			
		}
		else
		{
			Map attrs = component.getAttributes();
			
			styleClass = (String) attrs.get("styleClass");
			style_str = (String) attrs.get("style");
			dir = (String) attrs.get("dir");
			lang = (String) attrs.get("lang");
			title = (String) attrs.get("title");
			
			Object bSTB = attrs.get("showTitleBar"); // only where to tell if attribaute was set
			if(bSTB != null && bSTB instanceof String){
				if("false".equalsIgnoreCase(bSTB.toString()))
					showTitleBar = false;			
			}
			if(bSTB != null && bSTB instanceof Boolean){
				showTitleBar = ((Boolean)bSTB).booleanValue();
			}

			Object bSTCB = attrs.get("showTitleCloseButton"); // only where to tell if attribaute was set
			if(bSTCB != null && bSTCB instanceof String){
				if("false".equalsIgnoreCase(bSTCB.toString()))
					showTitleCloseButton = false;			
			}
			if(bSTCB != null && bSTCB instanceof Boolean){
				showTitleCloseButton = ((Boolean)bSTCB).booleanValue();
			}
			
			closeButtonImage = (String) attrs.get("titleCloseButtonImage");
			
			onclick = (String) attrs.get("onclick");
			ondblclick = (String) attrs.get("ondblclick");
			onmousedown = (String) attrs.get("onmousedown");
			onmouseup = (String) attrs.get("onmouseup");
			onmouseover = (String) attrs.get("onmouseover");
			onmousemove = (String) attrs.get("onmousemove");
			onmouseout = (String) attrs.get("onmouseout");
			onkeypress = (String) attrs.get("onkeypress");
			onkeydown = (String) attrs.get("onkeydown");
			onkeyup = (String) attrs.get("onkeyup");
		}
		
		
		// ----------------------------------------------------------------------------
		// To solve IE 5.5 <SELECT> 'bleed through' problem, hxclient code will generate
		// and underlying IFrame object, size it according to dialog's DIV size,  
		// assigned appropriate z-index, and toggle it's visibility as dialog becomes
		// visible/hidden.
		// Client code also ensures that underlying IFrame object moves along with the
		// dialog's main DIV.
		// ----------------------------------------------------------------------------
		
		// ----------------------------------------------------------------------------
		// We need to generate the following HTML that preceeds all child components of
		// the dialog:
		// - start dialog main (enclosing) DIV
		// - dialog title bar (including close button)
		// - start dialog content area DIV
	    //
		//  <div id="dialog1" class="panelDialog" style="position:absolute; display:none;">
		//  	<div id="dialog1_tb" class="panelDialog_TitleBar">
	    //     		<a href="#" id="dialog1_cb" class="panelDialog_TitleBar-CloseButton">
	    //      		<img id="dialog1_cbi" src="ibmjsfres/panelDialog_Small_CloseButton-normal.gif"></img>
	    //       	</a>
		//       	Dialog Title
		//		</div>
		//		<div id="dialog1_ca" class="panelDialog_Content" onXXX="...">		
		//			Dialog Content, including buttions
		//		</div>
		//	</div>
		//
		// NOTE: only <base> CSS class will be generated by the renderer (for main DIV).
		//       all other CSS classes will be produced by the hxclient code. 
		// ----------------------------------------------------------------------------

		// ----------------------------------------------------------------------------		
		// Start dialog main DIV
		writer.startElement("div", dialog);	
		writer.writeAttribute("id", clientId, null);

		// Style class for the main DIV is specified via styleClass attribute. 
		// If designers don't supply their own style class, default is CSS_PANEL_DIALOG.
		// Note that styleClass will serve as a 'base' name for all other style classes.		
		if( null == styleClass ) // if designer didn't specify a style class, use default
			styleClass = PANEL_DIALOG_CSS;

		writer.writeAttribute("class", styleClass, "styleClass");
		
		String style = "position:absolute; display:none;"; // we fix position to 'aboslute' and display to 'none'		
		if( null != style_str )	// if user specified 'style' attribute, then append its content
			style = style + " " + style_str;
		
		writer.writeAttribute("style", style, "style"); // write out style for the main DIV		
	
		if( null != dir )
			writer.writeAttribute("dir", dir, "dir");
				
		if( null != lang )
			writer.writeAttribute("lang", lang, "lang");
				
		if( showTitleBar ) // dialog nees to show its title bar, so emmit one
		{
			// Title Bar DIV element:
			// 	<div id="dialog1_TitleBar" class="panelDialog_TitleBar">
			writer.startElement("div", dialog);

			// write out id of the title bar DIV
			writer.writeAttribute("id", clientId + TITLE_BAR_ID, null);
			
			// Title is specified and showTitleBar is 'true'. Set the 'title' attribute on the header div
			if( null != title )
				writer.writeAttribute("title", title, "title");		
			
			// Title Bar close button
		    // 	<a href="#" id="dialog1_cb" title="close">
		    //  	<img id="dialog1_cbi" src="ibmjsfres/panelDialog_Small_CloseButton-normal.gif" title="close" alt="close"></img>
		    //  </a>			
			
			if( true == showTitleCloseButton ) // user wants close button
			{
				ResourceBundle bundle = ResourceHandler.getBundle(context);
				String altText = ResourceHandler.getString(bundle, "DialogCloseButtonTitle");

				// emit close button <anchor> element
				writer.startElement("a", dialog);  
				writer.writeAttribute("href", "#", null);
				writer.writeAttribute("id", clientId + CLOSE_BUTTON_ID, null);
				writer.writeAttribute("title", altText, null);
				
				// -----------------------------------------------------
				// Emit close button <img> element	
				//  - if designers specify their own close button image, emit 'src' attribute
				//  - otherwise, emit 1x1.gif and realy on CSS class to supply image for that gif. 
				writer.startElement("img", dialog);
				writer.writeAttribute("id", clientId + CLOSE_BUTTON_IMG_ID, null);
				
				if( null != closeButtonImage ) { // designer specifies 'titleBarCloseImage' attribute 
					writer.writeAttribute("src", Utils.getEncodedResourceURL(context, closeButtonImage), null);
				}
				
				// NOTE: if closeButtonImage is not explicitly specified, hxclient code will take care
				//       of assigning 1x1.gif and producing appropriate CSS class for the <img> element

				writer.writeAttribute("title", altText, null);
				writer.writeAttribute("alt", altText, null);				
				writer.endElement("img"); // end close button IMG
				// -----------------------------------------------------
				
				writer.endElement("a");  // end close button <anchor> element
			} // end if(true == showTitleCloseButton)

			// Write out Title Bar title string and close Tible Bar DIV
			// 	Dialog Title</div>	
			//
			// Note: if user didn't specify title string, emmit placeholder '&nbsp' 
			if( null == title )
				title = "&nbsp";
			writer.write(title);
						
			writer.endElement("div"); // close title bar DIV			
		} // end if(showTitleBar is true)
		else { // dialog does not show its title bar, so 'title' attribute should be set on the main dialog DIV
			if( null != title )
				writer.writeAttribute("title", title, "title");		
		} // end else(dialog does not show its title bar)
		
		// --------------------------------------------------------------------------------
		//  <div id="dialog1_ContentArea" onXXX="...">		
		writer.startElement("div", dialog);
		writer.writeAttribute("id", clientId + CONTENT_AREA_ID, null);
		
		// --------------------------------------------------------------------------------
		// All user-defined handlers (except onhide/onshow) get attached to the content DIV
		// Since content DIV is a <DIV> HTML element, we will only support the following handler:
		// - ONCLICK, when the mouse button is clicked on an element; 
		// - ONDBLCLICK, when the mouse button is double-clicked on an element; 
		// - ONMOUSEDOWN, when the mouse button is pressed over an element; 
		// - ONMOUSEUP, when the mouse button is released over an element; 
		// - ONMOUSEOVER, when the mouse is moved onto an element; 
		// - ONMOUSEMOVE, when the mouse is moved while over an element; 
		// - ONMOUSEOUT, when the mouse is moved away from an element; 
		// - ONKEYPRESS, when a key is pressed and released over an element; 
		// - ONKEYDOWN, when a key is pressed down over an element; 
		// - ONKEYUP, when a key is released over an element.		
		if( null != onclick )
			writer.writeAttribute("onclick", onclick, "onclick");
		
		if( null != ondblclick )
			writer.writeAttribute("ondblclick", ondblclick, "ondblclick");

		if( null != onmousedown )
			writer.writeAttribute("onmousedown", onmousedown, "onmousedown");

		if( null != onmouseup )
			writer.writeAttribute("onmouseup", onmouseup, "onmouseup");

		if( null != onmouseover )
			writer.writeAttribute("onmouseover", onmouseover, "onmouseover");

		if( null != onmousemove )
			writer.writeAttribute("onmousemove", onmousemove, "onmousemove");

		if( null != onmouseout )
			writer.writeAttribute("onmouseout", onmouseout, "onmouseout");

		if( null != onkeypress )
			writer.writeAttribute("onkeypress", onkeypress, "onkeypress");
		
		if( null != onkeydown )
			writer.writeAttribute("onkeydown", onkeydown, "onkeydown");

		if( null != onkeyup )
			writer.writeAttribute("onkeyup", onkeyup, "onkeyup");
		// --------------------------------------------------------------------------------
				
		// Note: 
		// closing of the panelDialog content <DIV> and main <DIV> will be done in the encodeEnd method.
	} // end encodeBegin()

	/**
	 * When end of the <panelDialog> is encountered, this method is called.
	 */
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException 
	{
		if ((context == null) || (component == null))
			throw new NullPointerException("NULL_PARAMETERS_ERROR_MESSAGE_ID");
		
		if(!component.isRendered())
			return;
		
		ResponseWriter writer = context.getResponseWriter();		

		// In the encodeEnd() method we need to do the following:
		// - close up dialog content <DIV>
		// - close up dialog main (enclosing) <DIV>
		//
        //   </DIV>
        // </DIV>					
		writer.endElement("div"); // close dialog content <DIV>		
		writer.endElement("div"); // close dialog main (enclosing) <DIV>


		//------------------------------------------------------------------		
		// Add all required JavaScript to the ScriptCollector
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);		
		HxClientRenderUtil.addScriptLibrary("hxclient_dlg.js", component);
		// scriptCollector.addScript(getPanelDialogScript(context, component));
		
		if( HxClientRenderUtil.isContainedInPanel(component) ) // rendered by the parent
		{
			generatePanelDialogScript(context, component, false);
		} // end if(rendered by the parent) 
		else // just register with scriptCollector. ScriptCollector will ask to generate script at the end
		{
			scriptCollector.register(this, component);
		} // end else(just register with scriptCollector)
	} // end encodeEnd()

	
	/**
	 * <p>Return a flag indicating that this {@link Renderer} is responsible
	 * for rendering the children the component it is asked to render.
	 */
	public boolean getRendersChildren() 
	{
		return false;
	} // end getRendersChildren() 

	
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		generatePanelDialogScript(context, component, true);
		
	}
	
	/**
	 * Generate JavaScript required to add JSFDialog component to the 
	 * global list of JSFComponents maintained by the hxclient.
	 * 
	 * @param context
	 * @param component
	 * @return
	 */
	public void generatePanelDialogScript(FacesContext context, UIComponent component, boolean writeNow) 
	{		
		if ((context == null) || (component == null))
			throw new NullPointerException("NULL_PARAMETERS_ERROR_MESSAGE_ID");
		
		HtmlPanelDialog dialog = null;
		if(component instanceof HtmlPanelDialog)
			dialog = (HtmlPanelDialog )component;
			
		/** We need to generate the following sript:
		 * hX_4.addComponent("clientId", 
		 *                   new hX_4.JSFDialog("initially-show:false", "movable:true", "onhide:;", "onshow:;" ... ));
		 * 
		 *  where "clientId" is the clientID of the panelDialog component
		 */
		
		String align = null;
		String valign = null;
		String forId = null;
		String relativeTo = null;
		boolean initiallyShow = false; //default is 'false'
		String  initialFocus = null;
		boolean movable	= true; // default is 'true'
		boolean enableMoveKey = true; // default is 'true'
		boolean noKeybindings = false; // default is 'false'
		String type = null; // default is "modal"
		boolean saveState = true;	
		String onshow = null;
		String onhide = null;
		
		if(dialog != null)
		{
			align = dialog.getAlign();
			valign = dialog.getValign();
			forId = dialog.getFor();
			relativeTo = dialog.getRelativeTo();
			initiallyShow = dialog.isInitiallyShow();
			initialFocus = dialog.getInitialFocus();
			movable	= dialog.isMovable(); 
			enableMoveKey = dialog.isEnableMoveKey();
			noKeybindings = dialog.isNoKeybindings();
			type = dialog.getType();
			saveState = dialog.isSaveState();	
			onshow = dialog.getOnshow();
			onhide = dialog.getOnhide();
		}
		else
		{
			Map attrs = component.getAttributes();
			
			align = (String) attrs.get("align");
			valign = (String) attrs.get("valign");
			forId = (String) attrs.get("for");
			relativeTo = (String) attrs.get("relativeTo");
			
			Object bIS = attrs.get("initiallyShow"); // only where to tell if attribaute was set
			if(bIS != null && bIS instanceof String){
				if("true".equalsIgnoreCase(bIS.toString()))
					initiallyShow = true;			
			}
			if(bIS != null && bIS instanceof Boolean){
				initiallyShow = ((Boolean)bIS).booleanValue();
			}
			
			initialFocus = (String) attrs.get("initialFocus");
			
			Object bM = attrs.get("movable"); // only where to tell if attribaute was set
			if(bM != null && bM instanceof String){
				if("false".equalsIgnoreCase(bM.toString()))
					movable = false;			
			}
			if(bM != null && bM instanceof Boolean){
				movable = ((Boolean)bM).booleanValue();
			}
			
			Object bEMK = attrs.get("enableMoveKey"); // only where to tell if attribaute was set
			if(bEMK != null && bEMK instanceof String){
				if("false".equalsIgnoreCase(bEMK.toString()))
					enableMoveKey = false;			
			}
			if(bEMK != null && bEMK instanceof Boolean){
				enableMoveKey = ((Boolean)bEMK).booleanValue();
			}

			Object bNKB = attrs.get("noKeybindings"); // only where to tell if attribaute was set
			if(bNKB != null && bNKB instanceof String){
				if("true".equalsIgnoreCase(bNKB.toString()))
					noKeybindings = true;			
			}
			if(bNKB != null && bNKB instanceof Boolean){
				noKeybindings = ((Boolean)bNKB).booleanValue();
			}
			
			type = (String)attrs.get("type");
			
			Object bSS = attrs.get("saveState"); // only where to tell if attribaute was set
			if(bSS != null && bSS instanceof String){
				if("false".equalsIgnoreCase(bSS.toString()))
					saveState = false;			
			}
			if(bSS != null && bSS instanceof Boolean){
				saveState = ((Boolean)bSS).booleanValue();
			}
			
			onshow = (String)attrs.get("onshow");
			onhide = (String)attrs.get("onhide");
		}
		
		
		StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME); //$NON-NLS-1$
		script.append(".addComponent(\"");
		script.append(component.getClientId(context));
		script.append("\", new "); //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".JSFDialog("); 

		boolean needComma = false;
		
		if( null != align )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			script.append("\"align:").append(align).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()

		if( null != valign )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			script.append("\"valign:").append(valign).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()

		
		if( null != forId )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}

			// Find UIComponent indicated by the 'forId' attribute in the component tree
			// so that we could fix-up 'for-id' attribute to represent 'clientId' of the component
			UIComponent forIdComponent = component.findComponent(forId);
			if( null == forIdComponent )
			{
				forIdComponent = context.getViewRoot().findComponent(forId);				
				if( forIdComponent == null ) // check from view root didn't yeild succsess
				{
					if( forIdComponent == null ) // method to resursively search through all components
						forIdComponent = Utils.findComponent(context.getViewRoot(), forId);
				} // end if()
			} // end if()
			if( forIdComponent != null ) // we finally found component
				forId = forIdComponent.getClientId(context); // it is clientId that we are after
			
			script.append("\"for-id:").append(forId).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()

		
		if( null != relativeTo )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			
			// Find UIComponent indicated by the 'posId' attribute in the component tree
			// so that we could fix-up 'pos-id' attribute to represent 'clientId' of the component
			UIComponent posIdComponent = component.findComponent(relativeTo);
			if( null == posIdComponent )
			{			
				posIdComponent = context.getViewRoot().findComponent(relativeTo);
				if( posIdComponent == null ) // check from view root didn't yeild succsess
				{
					if( posIdComponent == null ) // method to resursively search through all components
						posIdComponent = Utils.findComponent(context.getViewRoot(), relativeTo);
				} // end if()
			} // end if()
			if( posIdComponent != null ) // we finally found component
				relativeTo = posIdComponent.getClientId(context); // it is clientId that we are after
			
			script.append("\"relative-to:").append(relativeTo).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()
			
		if( true == initiallyShow )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			script.append("\"initially-show:").append(initiallyShow).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()

		if( null != initialFocus )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			
			// Find UIComponent indicated by the 'initialFocus' attribute in the component tree
			// so that we could fix-up 'initial-focus' attribute to represent 'clientId' of the component
			UIComponent initialFocusComponent = component.findComponent(initialFocus);
			if( null == initialFocusComponent )
			{
				initialFocusComponent = context.getViewRoot().findComponent(initialFocus);				
				if( initialFocusComponent == null ) // check from view root didn't yeild succsess
				{
					if( initialFocusComponent == null ) // method to resursively search through all components
						initialFocusComponent = Utils.findComponent(context.getViewRoot(), initialFocus);
				} // end if()
			} // end if()
			if( initialFocusComponent != null ) // we finally found component
				initialFocus = initialFocusComponent.getClientId(context); // it is clientId that we are after
			
			script.append("\"initial-focus:").append(initialFocus).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()
		
		if( false == movable )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			script.append("\"movable:").append(movable).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()
				
		if( false == enableMoveKey )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			script.append("\"enable-movekey:").append(enableMoveKey).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()
		
		if( true == noKeybindings )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			script.append("\"no-keybindings:").append(noKeybindings).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()
				
		if( (type != null) && type.equalsIgnoreCase(DLG_MODELESS) )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			script.append("\"type:").append(type).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()
						
		if( false == saveState )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			script.append("\"save-state:").append(saveState).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()
		
		if( null != onshow )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			script.append("\"onshow:").append(onshow).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()

		if( null != onhide )
		{
			if (needComma){
				script.append(COMMA_SEPARATOR);
			} else {
				needComma = true;
			}
			script.append("\"onhide:").append(onhide).append(CLOSE_QUOTE); //$NON-NLS-1$
		} // end if()
						
		script.append( "));" ); // close this statement
		
		// finally add script to the scriptCollector
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		if(writeNow){
			scriptCollector.writeScriptOnce(script.toString(),context,false);
		} // end if() 
		else {
			scriptCollector.addScript(script.toString());
		} // end else()
	} // end getPanelDialogScript()
	
} // end class DialogRenderer
