/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import com.ibm.faces.component.UIPlayer;
import com.ibm.faces.component.html.HtmlPlayerFlash;
import com.ibm.faces.util.JavaScriptUtil;

/**
 * Renderer for {@link com.ibm.faces.component.UIPlayer UIPlayer} component, 
 * that creates a Micromedia Flash control.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Player_FlashTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIPlayer}
 */
public final class FlashRenderer extends PlayerRenderer
{
	// Flash parameters.. 
	public static final String PARAM_WMODE = "wmode"; //$NON-NLS-1$
	public static final String PARAM_LOOP = "loop"; //$NON-NLS-1$
	public static final String PARAM_QUALITY = "quality"; //$NON-NLS-1$
	public static final String PARAM_SALIGN = "salign"; //$NON-NLS-1$
	public static final String PARAM_MENU = "menu"; //$NON-NLS-1$
	public static final String PARAM_SCALE = "scale"; //$NON-NLS-1$
	

	private final static Param[] objectParams = {
		new Param("src",      "MOVIE"), //$NON-NLS-1$ //$NON-NLS-2$
		new Param(UIPlayer.ATTR_AUTOSTART,       "PLAY"), //$NON-NLS-1$
		new Param(PARAM_WMODE,      "WMODE"), //$NON-NLS-1$
		new Param(PARAM_LOOP,       "LOOP"), //$NON-NLS-1$
		new Param(PARAM_QUALITY,    "QUALITY"), //$NON-NLS-1$
		new Param(PARAM_SALIGN,     "SALIGN"), //$NON-NLS-1$
		new Param(PARAM_MENU,       "MENU"), //$NON-NLS-1$
		new Param(PARAM_SCALE,      "SCALE"), //$NON-NLS-1$
	};
	private final static Param[] pluginParams = {
		new Param("src",           "SRC"), //$NON-NLS-1$ //$NON-NLS-2$
		new Param(UIPlayer.ATTR_AUTOSTART,       "PLAY"), //$NON-NLS-1$
		new Param(PARAM_WMODE,      "WMODE"), //$NON-NLS-1$
		new Param(PARAM_LOOP,       "LOOP"), //$NON-NLS-1$
		new Param(PARAM_QUALITY,    "QUALITY"), //$NON-NLS-1$
		new Param(PARAM_SALIGN,     "SALIGN"), //$NON-NLS-1$
		new Param(PARAM_MENU,       "MENU"), //$NON-NLS-1$
		new Param(PARAM_SCALE,      "SCALE"), //$NON-NLS-1$
	};
	
	private static final String DEFAULT_CLASSID = "clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"; //$NON-NLS-1$
	private static final String DEFAULT_CODEBASE = "http://active.macromedia.com/flash5/cabs/swflash.cab#version=5,0,0,0"; //$NON-NLS-1$

	private static final String DEFAULT_PLUGINPAGE  = "http://www.macromedia.com/downloads/"; //$NON-NLS-1$
	private static final String DEFAULT_PLUGINTYPE = null;

	/**
	 * Returns Class ID for Flash control.
	 * 
	 * @return Class ID
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectClassId()
	 */
	public String getObjectClassId() {
		return DEFAULT_CLASSID;
	}

	/**
	 * Returns Code Base for Flash control.
	 * 
	 * @return Code Base
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectCodebase()
	 */
	public String getObjectCodebase() {
		return DEFAULT_CODEBASE;
	}

	/**
	 * Returns a list of parameters for Flash control.
	 * 
	 * @return array of parameters
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectParams()
	 */
	protected Param[] getObjectParams() {
		return objectParams;
	}

	/**
	 * Returns URL to Flash's web site
	 * 
	 * @return URL to Flash's web site
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginPage()
	 */
	protected String getPluginPage() {
		return DEFAULT_PLUGINPAGE;
	}

	/**
	 * Returns type of Flash plug-in
	 * 
	 * @return type of plugin
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginType()
	 */
	protected String getPluginType() {
		return DEFAULT_PLUGINTYPE;
	}

	/**
	 * Returns a list of parameters for Flash plugin.
	 * 
	 * @return array of parameters
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginParams()
	 */
	protected Param[] getPluginParams() {
		return pluginParams;
	}
	
	
	public void encodeBegin(FacesContext context, UIComponent component)throws IOException {
		super.encodeBegin(context,component);
		//workaround for defect RATLC00970989, problem loading page in IE when flash src is not set, so setting a dummy for that case.
		
		//fixes for stephen  - xfaces
		Object src = null;
		if(component instanceof HtmlPlayerFlash){
			src = ((HtmlPlayerFlash)component).getSrc();
		} else {
			src = component.getAttributes().get("src");
		}
		if(src==null){
			String dummySrc=JavaScriptUtil.getImagePath(context)+"dummy.swf";
			context.getExternalContext().encodeResourceURL(dummySrc);
			if(component instanceof HtmlPlayerFlash){
				((HtmlPlayerFlash)component).setSrc(dummySrc);
			} else {
				component.getAttributes().put("src",dummySrc);
			}
		}
	}
}
