/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Iterator;

import javax.faces.component.UIComponent;
import javax.faces.component.html.HtmlOutputLabel;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.component.UIFormItem;
import com.ibm.faces.component.html.HtmlPanelFormBox;

/**
 * @author koinuma
 */
public class FormBoxRenderer extends Renderer {
	
	private static final String TR 				= "tr"; //$NON-NLS-1$
	private static final String TD 				= "td"; //$NON-NLS-1$
	private static final String RIGHT 			= "right"; //$NON-NLS-1$
	private static final String LEFT 			= "left"; //$NON-NLS-1$
	private static final String TABLE 			= "table"; //$NON-NLS-1$
	private static final String CELLSPACING 	= "cellspacing"; //$NON-NLS-1$
	private static final String CELLPADDING 	= "cellpadding"; //$NON-NLS-1$
	private static final String CLASS 			= "class"; //$NON-NLS-1$
	private static final String CRLF 			= "\n"; //$NON-NLS-1$
	private static final String ID 				= "id"; //$NON-NLS-1$
	private static final String COLSPAN 		= "colspan"; //$NON-NLS-1$
	private static final String ROWSPAN			= "rowspan"; //$NON-NLS-1$
	private static final String STYLE 			= "style"; //$NON-NLS-1$
	private static final String LABEL 			= "label"; //$NON-NLS-1$
	
	private static final String _HEADER_SUFFIX	= "Header"; //$NON-NLS-1$
	private static final String _OTHER_SUFFIX	= "Other"; //$NON-NLS-1$
	private static final String _CELL_SUFFIX	= "Cell"; //$NON-NLS-1$
	private static final String _TOP_SUFFIX		= "Top"; //$NON-NLS-1$
	private static final String _BOTTOM_SUFFIX 	= "Bottom"; //$NON-NLS-1$
	private static final String _LEFT_SUFFIX	= "Left"; //$NON-NLS-1$
	private static final String _RIGHT_SUFFIX 	= "Right"; //$NON-NLS-1$
	private static final String _SPACER_SUFFIX 	= "Spacer"; //$NON-NLS-1$
	private static final String CONNECTOR_U		= "_"; //$NON-NLS-1$
	private static final String CONNECTOR_H		= "-"; //$NON-NLS-1$

	private static final String BOTTOM 			= "bottom"; //$NON-NLS-1$
	private static final String TOP				= "top"; //$NON-NLS-1$
	private static final String OVER 			= "over"; //$NON-NLS-1$
	private static final String VALUE			= "value"; //$NON-NLS-1$
	private static final String FOR				= "for"; //$NON-NLS-1$
	private static final String UNDER			= "under"; //$NON-NLS-1$
	private static final String TITLE			= "title"; //$NON-NLS-1$
	
	private static final String SPACER_STYLE	= "font-family:sans-serif; font-size:"; //$NON-NLS-1$
	private static final String NBSP			= Character.toString((char)0xA0);

	public boolean getRendersChildren() {
		return true;
	}

	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {
		if(!component.isRendered()){
			return;
		}
		
		HxClientRenderUtil.initJSLibraries(component, context);
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component); //$NON-NLS-1$
		
		ResponseWriter writer = context.getResponseWriter();
		HtmlPanelFormBox formBox = (HtmlPanelFormBox) component;
		
		//get the attributes from the component
		String styleClass = getStyleClass(component);
		String style = formBox.getStyle();
		String title = formBox.getTitle();
		String labelPosition = formBox.getLabelPosition();
		String helpPosition = formBox.getHelpPosition();
		
		int numCols = getColumnCount(component);
		
		//Start rendering
		writer.startElement(TABLE, null);		
		if(formBox.getId() != null){
			writer.writeAttribute(ID, formBox.getClientId(context), ID);
		}
		writer.writeAttribute(CELLSPACING, new Integer(0), CELLSPACING);		
		writer.writeAttribute(CELLPADDING, new Integer(0), CELLPADDING);

		if(styleClass != null && styleClass.length() > 0){
			// use raw style class attribute value
			writer.writeAttribute(CLASS, formBox.getStyleClass(), CLASS);
		}
		if(style != null && style.length() > 0){
			writer.writeAttribute(STYLE, style, STYLE);
		}
		if (title != null && title.length() >0) {
			writer.writeAttribute(TITLE, title, TITLE);
		}
		//\n
		writer.writeText(CRLF, null);

		if (formBox.getLabel() != null) {	//If a label was defined			
			HtmlOutputLabel label = new HtmlOutputLabel();
			label.setValue(formBox.getLabel());
			label.setFor(formBox.getClientId(context));
			label.setStyleClass(styleClass + CONNECTOR_U + _HEADER_SUFFIX);
			
			writer.startElement(TR, null);
			writer.writeText(CRLF, null);			
			writer.startElement(TD, null);
			writer.writeAttribute(COLSPAN, new Integer(numCols).toString(), COLSPAN);
			writer.writeAttribute(CLASS, label.getStyleClass(), CLASS);
			writer.writeText(CRLF, null);
			writer.startElement(LABEL, label);
			writer.writeAttribute(FOR, label.getFor(), FOR);
			
			if (formBox.isEscape()) {
				writer.writeText(label.getValue(), VALUE);
			} else {
				writer.write(label.getValue().toString());
			}
			writer.endElement(LABEL);
			writer.writeText(CRLF, null);
			writer.endElement(TD);
			writer.endElement(TR);
			writer.writeText(CRLF, null);
		}

		if (component.getFacet(TOP) != null) { // there is a 'top' facet
			writer.startElement(TR, null);
			writer.writeText(CRLF, null);
			writer.startElement(TD, null);
			writer.writeAttribute(COLSPAN, new Integer(numCols).toString(), COLSPAN);
			writer.writeAttribute(CLASS, styleClass + CONNECTOR_U + _TOP_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
			writer.writeText(CRLF, null);
			// render whatever is in the facet in the td
			// more than likely this is going to be a formMessagesArea component.
			encodeRecursive(context, component.getFacet(TOP));
			writer.endElement(TD);
			writer.writeText(CRLF, null);
			writer.endElement(TR);
			writer.writeText(CRLF, null);
		}
		
		int numRows = 2;
		if (OVER.equals(labelPosition) || OVER.equals(helpPosition) || labelPosition == null || helpPosition == null) {
			numRows++;
		}
		if (UNDER.equals(labelPosition) || UNDER.equals(helpPosition)) {
			numRows++;
		}
		numRows = numRows * component.getChildCount();

		if (component.getFacet(LEFT) != null) {
			writer.startElement(TR, null);
			writer.writeText(CRLF, null);
			writer.startElement(TD, null);
			writer.writeAttribute(ROWSPAN, new Integer(numRows + 1).toString(), ROWSPAN);
			writer.writeAttribute(CLASS, styleClass + CONNECTOR_U + _LEFT_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
			writer.writeText(CRLF, null);
			encodeRecursive(context, component.getFacet(LEFT));
			writer.endElement(TD);
			writer.writeText(CRLF, null);
		} else {
			writer.startElement(TR, null);
			writer.writeText(CRLF, null);
		}
		
		writer.startElement(TD, null);
		writer.writeAttribute(COLSPAN, new Integer(getColumnSpacerCount(component)).toString(), COLSPAN);
		writer.writeAttribute(CLASS, styleClass + CONNECTOR_U + _SPACER_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
		if (((HtmlPanelFormBox)component).getInterItemSpacing() != null) {
			writer.writeAttribute(STYLE, SPACER_STYLE + ((HtmlPanelFormBox)component).getInterItemSpacing(), STYLE);
		}
		writer.writeText(NBSP, null);
		writer.endElement(TD);
		
		if (component.getFacet(RIGHT) != null) {
			writer.writeText(CRLF, null);
			writer.startElement(TD, null);
			writer.writeAttribute(ROWSPAN, new Integer(numRows + 1).toString(), ROWSPAN);
			writer.writeAttribute(CLASS, styleClass + CONNECTOR_U + _RIGHT_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
			writer.writeText(CRLF, null);
			encodeRecursive(context, component.getFacet(RIGHT));
			writer.endElement(TD);
			writer.writeText(CRLF, null);
		}
		
		writer.endElement(TR);
		writer.writeText(CRLF, null);
	}

	public void encodeChildren(FacesContext context, UIComponent component)
			throws IOException {
		if(!component.isRendered()){
			return;
		}

		ResponseWriter writer = context.getResponseWriter();

		int numColSpacer = getColumnCount(component);
		if (component.getFacet(LEFT) != null) {
			numColSpacer--;
		}
		if (component.getFacet(RIGHT) != null) {
			numColSpacer--;
		}
		
		Iterator i = component.getChildren().iterator();
		
		String styleClass = getStyleClass(component);
		
		while (i.hasNext()) {
			UIComponent child = (UIComponent) i.next();

			writer.startElement(TR, null);
			writer.writeText(CRLF, null);
			if (child instanceof UIFormItem) {
				encodeRecursive(context, child);
			} else {
				writer.startElement(TD, null);
				writer.writeAttribute(COLSPAN, new Integer(numColSpacer).toString(), COLSPAN);
				writer.writeAttribute(CLASS, styleClass + CONNECTOR_U + _OTHER_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
				encodeRecursive(context, child);
				writer.endElement(TD);
				writer.writeText(CRLF, null);
			}
			
			writer.endElement(TR);
			writer.writeText(CRLF, null);
			
			writer.startElement(TR, null);
			writer.startElement(TD, null);
			writer.writeAttribute(COLSPAN, new Integer(numColSpacer).toString(), COLSPAN);
			writer.writeAttribute(CLASS, styleClass + CONNECTOR_U + _SPACER_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
			if (((HtmlPanelFormBox)component).getInterItemSpacing() != null) {
				writer.writeAttribute(STYLE, SPACER_STYLE + ((HtmlPanelFormBox)component).getInterItemSpacing(), STYLE);
			}
			writer.writeText(NBSP, null);
			writer.endElement(TD);
			writer.endElement(TR);
			writer.writeText(CRLF, null);			
			
		}
	}

	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {
		if(!component.isRendered()){
			return;
		}
		ResponseWriter writer = context.getResponseWriter();			

		if (component.getFacet(BOTTOM) != null) {
			writer.startElement(TR, null);
			writer.writeText(CRLF, null);
			writer.startElement(TD, null);
			writer.writeAttribute(COLSPAN, new Integer(getColumnCount(component)).toString(), COLSPAN);
			writer.writeAttribute(CLASS, getStyleClass(component) + CONNECTOR_U + _BOTTOM_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
			writer.writeText(CRLF, null);
			encodeRecursive(context, component.getFacet(BOTTOM));
			writer.endElement(TD);
			writer.writeText(CRLF, null);
			writer.endElement(TR);
			writer.writeText(CRLF, null);

		}
		writer.endElement(TABLE);
		writer.writeText(CRLF, null);
	}

	public void encodeRecursive(FacesContext context, UIComponent component)
			throws IOException {
		if (!component.isRendered()) {
			return;
		}
		component.encodeBegin(context);
		if (component.getRendersChildren()) {
			component.encodeChildren(context);
		} else {
			Iterator i = component.getChildren().iterator();
			while (i.hasNext()) {
				UIComponent child = (UIComponent) i.next();
				encodeRecursive(context, child);
			}
		}
		component.encodeEnd(context);
	}
	
	private String getStyleClass(UIComponent component) {
		HtmlPanelFormBox formBox = (HtmlPanelFormBox) component;
		
		String styleClass = formBox.getStyleClass();
		if(styleClass != null && styleClass.indexOf(" ") != -1){ //$NON-NLS-1$
			//Only save the first defined class if there are multiple defined.
			styleClass = styleClass.substring(0, styleClass.indexOf(" ")); //$NON-NLS-1$
		}		
		return styleClass;
	}
	
	private int getColumnCount(UIComponent component) {
		int numCols = getColumnSpacerCount(component);

		if (component.getFacet(LEFT) != null) {
			numCols++;
		}
		if (component.getFacet(RIGHT) != null) {
			numCols++;
		}
		return numCols;
	}
	
	private int getColumnSpacerCount(UIComponent component) {
		int numCols = 1;
		
		HtmlPanelFormBox formBox = (HtmlPanelFormBox) component;
		String labelPosition = formBox.getLabelPosition();
		String helpPosition = formBox.getHelpPosition();
		
		//Figure out what the colspan needs to be
		if(labelPosition != null){
			if(labelPosition.equals(RIGHT) || labelPosition.equals(LEFT)){
				numCols++;
			}
		}		
		if(helpPosition != null){
			if(helpPosition.equals(RIGHT) || helpPosition.equals(LEFT)){
				numCols++;
			}
		}
		return numCols;
	}
}
