/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Iterator;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.component.html.HtmlOutputLabel;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlFormItem;
import com.ibm.faces.component.html.HtmlInputHelperAssist;
import com.ibm.faces.component.html.HtmlPanelFormBox;
import com.ibm.faces.util.ScriptObject;

/**
 * @author koinuma
 */
public class FormItemRenderer extends Renderer implements IScriptContributor {
	
	//Constants	
	private static final String ERROR 				= "error"; //$NON-NLS-1$
	private static final String ALWAYS 				= "always"; //$NON-NLS-1$
	private static final String INFO 				= "info"; //$NON-NLS-1$
	private static final String AS_NEEDED			= "as-needed"; //$NON-NLS-1$
	private static final String OVER 				= "over"; //$NON-NLS-1$
	private static final String UNDER 				= "under"; //$NON-NLS-1$
	private static final String LEFT 				= "left"; //$NON-NLS-1$
	private static final String RIGHT 				= "right"; //$NON-NLS-1$
	private static final String HELP				= "help"; //$NON-NLS-1$
	private static final String BOTH				= "both"; //$NON-NLS-1$
	
	private static final String CR 					= "\n"; //$NON-NLS-1$
	
	private static final String _ERROR_SUFFIX 		= "Error"; //$NON-NLS-1$
	private static final String _INFO_SUFFIX 		= "Info"; //$NON-NLS-1$
	private static final String _LABEL_SUFFIX 		= "Label"; //$NON-NLS-1$
	private static final String _REQUIRED_SUFFIX 	= "Required"; //$NON-NLS-1$
	private static final String _RELEASED_SUFFIX	= "Released"; //$NON-NLS-1$
	private static final String _CONTENT_SUFFIX 	= "Content"; //$NON-NLS-1$
	private static final String _CELL_SUFFIX		= "Cell"; //$NON-NLS-1$
	private static final String _HELP_SUFFIX		= "Help"; //$NON-NLS-1$
	private static final String CONNECTOR_U			= "_"; //$NON-NLS-1$
	private static final String CONNECTOR_H			= "-"; //$NON-NLS-1$
	private static final String SPACER				= " "; //$NON-NLS-1$
	
	private static final String SPAN 				= "span"; //$NON-NLS-1$
	private static final String TABLE				= "table"; //$NON-NLS-1$
	private static final String TD 					= "td"; //$NON-NLS-1$
	private static final String TR 					= "tr"; //$NON-NLS-1$
	private static final String LABEL 				= "label"; //$NON-NLS-1$
	private static final String CLASS 				= "class"; //$NON-NLS-1$
	private static final String STYLE 				= "style"; //$NON-NLS-1$
	private static final String FOR 				= "for"; //$NON-NLS-1$
	private static final String ID 					= "id"; //$NON-NLS-1$
	private static final String DIV 				= "div"; //$NON-NLS-1$
	private static final String VALUE 				= "value"; //$NON-NLS-1$
	private static final String COLSPAN 			= "colspan"; //$NON-NLS-1$
	private static final String ACCESSKEY			= "accesskey"; //$NON-NLS-1$
	private static final String TITLE				= "title"; //$NON-NLS-1$
	private static final String WIDTH				= "width"; //$NON-NLS-1$
	private static final String DISPLAY_NONE		= "display:none"; //$NON-NLS-1$
	
	private static int SINGLE_CELL = 0;
	private static int SINGLE_CELL_WITH_JOINED_CELL = 1;
	private static int LABEL_CELL = 2;
	private static int HELP_CELL = 3;
	
	public boolean getRendersChildren() {
		return true;
	}

	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {
		
	}

	public void encodeChildren(FacesContext context, UIComponent component)
			throws IOException {
	}

	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {		
		
		if (!component.isRendered()) {
			return;
		}
		
		if(component.getChildCount() == 0) {
			// Nothing to render if there are no children
			return;
		}
		
		UIComponent child = (UIComponent) component.getChildren().get(0);
//		 ENCODE BEGIN -->
		HtmlFormItem formItem = (HtmlFormItem) component;
		ResponseWriter writer = context.getResponseWriter();
		//set up the renderer's constants
		Iterator errorMsgIter = context.getMessages(child.getClientId(context));
		final boolean errorOnPage = errorMsgIter.hasNext();
		final FacesMessage msg = (errorOnPage) ? (FacesMessage) errorMsgIter.next() : null;
		final String childErrorMessage = (msg != null) ? ( (msg.getDetail() != null) ? msg.getDetail() : msg.getSummary()) : null;
		final String showHelp = formItem.getShowHelp();
		final String releaseMargin = formItem.getReleaseMargin();
		final boolean releaseLabel = LABEL.equals(releaseMargin);
		final boolean releaseHelp = HELP.equals(releaseMargin);
		final boolean releaseBoth = BOTH.equals(releaseMargin);
		
		HtmlOutputLabel label = new HtmlOutputLabel();
		label.setValue(formItem.getLabel());
		label.setTitle(formItem.getTitle());
		label.setStyle(formItem.getStyle());
		label.setFor(child.getClientId(context));
		label.setAccesskey(formItem.getAccesskey());
		// Get the style of the formBox
		HtmlPanelFormBox formBox = (HtmlPanelFormBox) component.getParent();
		String parentStyleClass = formBox.getStyleClass();
		if(parentStyleClass != null && parentStyleClass.indexOf(" ") != -1){ //$NON-NLS-1$
			// If multiple classes are defined just use the first one
			parentStyleClass = parentStyleClass.substring(0, parentStyleClass.indexOf(" ")).trim(); //$NON-NLS-1$
		}
		// width attributes
		String widthContent = formBox.getWidthContent();

		boolean required = (child instanceof UIInput) && ((UIInput)child).isRequired();
		
		if (showHelp == null || showHelp.equals(ALWAYS) || showHelp.equals(INFO) || showHelp.equals(ERROR) || showHelp.equals(AS_NEEDED)) {
			if (!formItem.isNoClientValidation()) {
				UIScriptCollector scriptCollector = UIScriptCollector.find(component);
				// We need to output our javascript library
				HxClientRenderUtil.initJSLibraries(scriptCollector, context);
				scriptCollector.register(this, component);
			}
		}
		
		// styleClass attribute
		String classSuffix = ""; //$NON-NLS-1$
		if (formItem.getStyleClass() != null && formItem.getStyleClass().trim().length() > 0) {
			classSuffix = SPACER + formItem.getStyleClass();
		}
		if(errorOnPage){
			// If there is an error on the page change the style
			if (required) {
				label.setStyleClass(parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _REQUIRED_SUFFIX + CONNECTOR_H + _ERROR_SUFFIX + classSuffix);
			} else {
				label.setStyleClass(parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _ERROR_SUFFIX + classSuffix);
			}
		}else if (required) {
			label.setStyleClass(parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _REQUIRED_SUFFIX + classSuffix);
		}else{
			label.setStyleClass(parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX + classSuffix);
		}
		
		String pos = ((HtmlPanelFormBox)component.getParent()).getLabelPosition();
		
		boolean labelIsOver = true, labelIsLeft = false, labelIsRight = false, labelIsUnder = false;
		boolean helpIsOver = true, helpIsLeft = false, helpIsRight = false, helpIsUnder = false;
		
		//get the position of the label
		if (pos != null) {
			labelIsOver = pos.equals(OVER);
			labelIsLeft = pos.equals(LEFT);
			labelIsRight = pos.equals(RIGHT);
			labelIsUnder = pos.equals(UNDER);
		} else { // default value
			labelIsOver = true;
		}
		pos = ((HtmlPanelFormBox) component.getParent()).getHelpPosition();
		//get the position of the info/help text
		if (pos != null) {			
			helpIsOver = pos.equals(OVER);
			helpIsLeft = pos.equals(LEFT);
			helpIsRight = pos.equals(RIGHT);
			helpIsUnder = pos.equals(UNDER);
			pos = null;
		} else { // default value
			helpIsOver = true;
		}
		
		if (releaseBoth && (labelIsLeft || labelIsRight || helpIsLeft || helpIsRight)) {
			int span = 1;
			if (labelIsLeft || labelIsRight) span++;
			if (helpIsLeft || helpIsRight) span++;
			startNestedTable(writer, parentStyleClass, span);
		}

		// emit 7 candidate cells

		if (labelIsOver || helpIsOver) {
			// 1. left-top empty cell
			if (labelIsLeft || helpIsLeft) {
				writer.startElement(TD, null);
				writer.endElement(TD);
			}
		
			// 2. top-center cell
			writeLabelOrHelperCells(context, component, label, errorOnPage, childErrorMessage, labelIsOver, helpIsOver, (labelIsRight || helpIsRight) ? SINGLE_CELL_WITH_JOINED_CELL : SINGLE_CELL);
		
			// break row
			writer.endElement(TR);
			writer.writeText(CR, null);
			writer.startElement(TR, null);
		}

		if (releaseLabel && labelIsLeft) {
			startNestedTable(writer, parentStyleClass, (labelIsLeft&&helpIsLeft) ? 3 : 2);
		}
		
		// 3. left-center cell
		if (labelIsLeft || helpIsLeft) {
			writeLabelOrHelperCells(context, component, label, errorOnPage, childErrorMessage, labelIsLeft, helpIsLeft, LABEL_CELL);
			if (releaseHelp && helpIsLeft) {
				startNestedTable(writer, parentStyleClass, 2);
			}
			writeLabelOrHelperCells(context, component, label, errorOnPage, childErrorMessage, labelIsLeft, helpIsLeft, HELP_CELL);
		}
		
		if ((releaseLabel && labelIsRight) || (releaseHelp && helpIsRight)) {
			startNestedTable(writer, parentStyleClass, (labelIsRight&&helpIsRight&&releaseHelp) ? 3 : 2);
		}
		
		// 4. ENCODE CHILDREN -->
		writer.startElement(TD, null);
		if (widthContent != null && !(releaseBoth || (releaseLabel && (labelIsLeft || labelIsRight)) || (releaseHelp && (helpIsLeft || helpIsRight)))) {
			writer.writeAttribute(WIDTH, widthContent, WIDTH);
		}
		if (releaseBoth || (releaseLabel && (labelIsLeft || labelIsRight)) || (releaseHelp && (helpIsLeft || helpIsRight))) {
			writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _CONTENT_SUFFIX + CONNECTOR_H + _RELEASED_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
		} else {
			writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _CONTENT_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
		}
		if (component.getChildCount() > 0) {
			writeContent(context, component, errorOnPage, parentStyleClass);
		}
		writer.endElement(TD);
		writer.writeText(CR, null);
		
		if ((releaseLabel && labelIsLeft) || (releaseHelp && helpIsLeft)) {
			endNestedTable(writer);
		}

		// 5. right-center cell
		if (labelIsRight || helpIsRight) {
			writeLabelOrHelperCells(context, component, label, errorOnPage, childErrorMessage, labelIsRight, helpIsRight, LABEL_CELL);
			if (releaseLabel && labelIsRight) {
				endNestedTable(writer);
			}
			writeLabelOrHelperCells(context, component, label, errorOnPage, childErrorMessage, labelIsRight, helpIsRight, HELP_CELL);
		}

		if (releaseHelp && helpIsRight) {
			endNestedTable(writer);
		}
		
		if (labelIsUnder || helpIsUnder) {
			// break row
			writer.endElement(TR);
			writer.writeText(CR, null);
			writer.startElement(TR, null);
			
			// 6. left-bottom empty cell
			if (labelIsLeft || helpIsLeft) {
				writer.startElement(TD, null);
				writer.endElement(TD);
			}
			
			// 7. top-center cell
			writeLabelOrHelperCells(context, component, label, errorOnPage, childErrorMessage, labelIsUnder, helpIsUnder, (labelIsRight || helpIsRight) ? SINGLE_CELL_WITH_JOINED_CELL : SINGLE_CELL);
		}
		
		if (releaseBoth && (labelIsLeft || labelIsRight || helpIsLeft || helpIsRight)) {
			endNestedTable(writer);
		}

	}

	private void encodeCorrectMessage(FacesContext context,
			UIComponent component, boolean showInfoText, boolean showErrorText, boolean errorOnPage, Object childErrorMessage, String parentStyleClass) throws IOException {

		HtmlFormItem formItem = (HtmlFormItem) component;
		final String spanInfoId = formItem.getInfoSpanId(context);
		final String spanErrorId = formItem.getErrorSpanId(context);
		ResponseWriter writer = context.getResponseWriter();		

		// info text
		if (showInfoText) {
			writer.startElement(SPAN, null);
			writer.writeAttribute(ID, spanInfoId, ID);
			writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _INFO_SUFFIX, CLASS);
			if (errorOnPage || AS_NEEDED.equals(formItem.getShowHelp())) {
				writer.writeAttribute(STYLE, DISPLAY_NONE, STYLE);
			}
			if (formItem.getInfoText() != null) {
				if (formItem.isEscape()) {
					writer.writeText(formItem.getInfoText(), null);
				} else {
					writer.write(formItem.getInfoText());
				}
			}
			writer.endElement(SPAN);
			writer.writeText(CR, null);
		}
		
		// error text
		if (showErrorText) {
			writer.startElement(SPAN, null);
			writer.writeAttribute(ID, spanErrorId, ID);
			writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _ERROR_SUFFIX, CLASS);
			if (!errorOnPage) {
				writer.writeAttribute(STYLE, DISPLAY_NONE, STYLE);
			}
			if (formItem.getErrorText() != null) {
				if (formItem.isEscape()) {
					writer.writeText(formItem.getErrorText(), null);
				} else {
					writer.write(formItem.getErrorText());
				}
			} else if (childErrorMessage != null) {
				writer.writeText(childErrorMessage, null);
			}
			writer.endElement(SPAN);
			writer.writeText(CR, null);
		}
	}
	
	private void addClientValidatorParameters(FacesContext context, UIComponent component, StringBuffer script, String styleClassFamily, ScriptObject converter, ScriptObject validator, boolean required) {
		HtmlFormItem formItem = (HtmlFormItem) component;
//		script.append("\n\t\t"); // formatting
		// styleClass attribute
		String classSuffix = ""; //$NON-NLS-1$
		if (formItem.getStyleClass() != null && formItem.getStyleClass().trim().length() > 0) {
			classSuffix = SPACER + formItem.getStyleClass();
		}
		if (required) {
			script.append("\"label-success-class:" + styleClassFamily + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _REQUIRED_SUFFIX + classSuffix + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			script.append("\"label-success-class:" + styleClassFamily + CONNECTOR_U + _LABEL_SUFFIX + classSuffix + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (required) {
			script.append(", \"label-error-class:" + styleClassFamily + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _REQUIRED_SUFFIX + CONNECTOR_H + _ERROR_SUFFIX + classSuffix + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		} else {
			script.append(", \"label-error-class:" + styleClassFamily + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _ERROR_SUFFIX + classSuffix + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		String labelSpanId = formItem.getLabelId(context);
		if (labelSpanId != null) {
			script.append(", \"label-id:" + labelSpanId + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		String infoSpanId = formItem.getInfoSpanId(context);
		if (infoSpanId != null) {
			script.append(", \"info-span-id:" + infoSpanId + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		String errorSpanId = formItem.getErrorSpanId(context);
		if (errorSpanId != null) {
			script.append(", \"error-span-id:" + errorSpanId + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		String spanIsPopup = formItem.getSpanIsPopup();
		if (spanIsPopup != null) {
			script.append(", \"span-is-popup:" + spanIsPopup + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		String infoSpanAlways = formItem.getInfoSpanAlways();
		if (infoSpanAlways != null) {
			script.append(", \"info-span-always:" + infoSpanAlways + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		String errorSpanAlways = formItem.getErrorSpanAlways();
		if (errorSpanAlways != null) {
			script.append(", \"error-span-always:" + errorSpanAlways + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (converter != null) {
			script.append(", \"converter:" + converter.getId() + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (validator != null) {
			script.append(", \"validator:" + validator.getId() + "\""); //$NON-NLS-1$ //$NON-NLS-2$
		}
//		script.append("\n\t"); // formatting
		
	}
	
	private void addClientEventCode(FacesContext context, UIComponent component, StringBuffer script, String editableValueHolderId, String eventName, String styleClassFamily, ScriptObject converter, ScriptObject validator, boolean required) {
		script.append("if (" + HxClientRenderUtil.HX_VAR_NAME + ".getBehaviorById(\"" + editableValueHolderId + "\", \"validate\", \"" + eventName + "\")==null) ");   //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$//$NON-NLS-4$
		script.append("\n\t"); // formatting //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME + ".addBehavior(\"" + editableValueHolderId + "\", \"" + eventName + "\", new " + HxClientRenderUtil.HX_VAR_NAME + ".JSFBehaviorValidate(");  //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		addClientValidatorParameters(context, component, script, styleClassFamily, converter, validator, required);
		script.append("));"); //$NON-NLS-1$
		script.append("\n"); // formatting //$NON-NLS-1$
		script.append("else "); //$NON-NLS-1$
		script.append("\t"); // formatting //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME + ".getBehaviorById(\"" + editableValueHolderId + "\", \"validate\", \"" + eventName + "\").setAttribute(");  //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$
		addClientValidatorParameters(context, component, script, styleClassFamily, converter, validator, required);
		script.append(");"); //$NON-NLS-1$
		script.append("\n"); // formatting //$NON-NLS-1$
		
	}

	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		UIComponent child = (UIComponent) component.getChildren().get(0);
		boolean required = (child instanceof UIInput) && ((UIInput)child).isRequired();
		String editableValueHolderId = child.getClientId(context);
		
		// set parent styleClass
		String styleClassFamily = ((HtmlPanelFormBox) component.getParent()).getStyleClass();
		if(styleClassFamily != null && styleClassFamily.indexOf(" ") != -1){ //$NON-NLS-1$
			// If multiple classes are defined just use the first one
			styleClassFamily = styleClassFamily.substring(0, styleClassFamily.indexOf(" ")).trim(); //$NON-NLS-1$
		}
		StringBuffer javaScript = new StringBuffer();
		
		// validator and converter if necessary
		ScriptObject converter = null, validator = null;
		if ((child instanceof UIInput) && needValidator(context, (UIInput)child)) {
			converter = AssistRenderer.getConverterScript(context, (UIInput)child);
			validator = AssistRenderer.getValidatorScript(context, (UIInput)child, converter);
		}
		
		// onfocus
		addClientEventCode(context, component, javaScript, editableValueHolderId, "onfocus", styleClassFamily, converter, validator, required); //$NON-NLS-1$
		
		// onblur
		addClientEventCode(context, component, javaScript, editableValueHolderId, "onblur", styleClassFamily, converter, validator, required); //$NON-NLS-1$
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		if (null != converter) {
			scriptCollector.writeScriptOnce(converter.getScript(), context, false);
		}
		if (null != validator) {
			scriptCollector.writeScriptOnce(validator.getScript(), context, false);
		}
		scriptCollector.addScript(javaScript.toString());
		javaScript = null;
	}
	
	private void writeLabel(FacesContext context, UIComponent component, HtmlOutputLabel label) throws IOException {
		HtmlFormItem formItem = (HtmlFormItem) component;
		ResponseWriter writer = context.getResponseWriter();
		boolean escape = formItem.isEscape();
		String labelId = formItem.getLabelId(context);
		
		writer.startElement(LABEL, label);
		writer.writeAttribute(ID, labelId, ID);
		if (label.getFor() != null) {
			writer.writeAttribute(FOR, label.getFor(), FOR);
		}
		if (label.getStyleClass() != null) {
			writer.writeAttribute(CLASS, label.getStyleClass(), CLASS);
		}
		if (label.getAccesskey() != null) {
			writer.writeAttribute(ACCESSKEY, label.getAccesskey(), ACCESSKEY);
		}
		if(label.getStyle() != null){
			writer.writeAttribute(STYLE, label.getStyle(), STYLE);
		}
		if (label.getTitle() != null) {
			writer.writeAttribute(TITLE, label.getTitle(), TITLE);
		}
		
		if (label.getValue() != null) {
			if (escape) {
				writer.writeText(label.getValue(), VALUE);
			} else {
				writer.write(label.getValue().toString());
			}
		}
		writer.endElement(LABEL);		
		writer.writeText(CR, null);
	}
	
	private void writeContent(FacesContext context, UIComponent component, boolean errorOnPage, String parentStyleClass) throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		writer.startElement(DIV, null);
		writer.writeAttribute(ID, component.getClientId(context) + _CONTENT_SUFFIX.toLowerCase(), ID);
		
		if (errorOnPage){
			writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _CONTENT_SUFFIX + CONNECTOR_H + _ERROR_SUFFIX, CLASS);
//		} else if (required) {
//			writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _CONTENT_SUFFIX + CONNECTOR_H + _REQUIRED_SUFFIX, CLASS);
		}else{
			writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _CONTENT_SUFFIX, CLASS);
		}
		
		writer.writeText(CR, null);

		Utils.encodeChildren(context, component);
		
		writer.endElement(DIV);
		writer.writeText(CR, null);
	}
	
	
	private void writeLabelOrHelperCells(FacesContext context, UIComponent component, HtmlOutputLabel label, boolean errorOnPage, String childErrorMessage, boolean hasLabel, boolean hasHelp, int type) throws IOException {
		ResponseWriter writer = context.getResponseWriter();
		HtmlFormItem formItem = (HtmlFormItem) component;
		final String showHelp = formItem.getShowHelp();
		final boolean showErrorText = (showHelp == null || showHelp.equals(ALWAYS) || showHelp.equals(ERROR) || showHelp.equals(AS_NEEDED));
		final boolean showInfoText = (showHelp == null || showHelp.equals(ALWAYS) || showHelp.equals(INFO) || showHelp.equals(AS_NEEDED));
		// Get the style of the formBox
		HtmlPanelFormBox formBox = (HtmlPanelFormBox) component.getParent();
		String parentStyleClass = formBox.getStyleClass();
		if(parentStyleClass != null && parentStyleClass.indexOf(" ") != -1){ //$NON-NLS-1$
			// If multiple classes are defined just use the first one
			parentStyleClass = parentStyleClass.substring(0, parentStyleClass.indexOf(" ")).trim(); //$NON-NLS-1$
		}
		final String widthHelp = formBox.getWidthHelp();
		final String widthLabel = formBox.getWidthLabel();
		final String releaseMargin = formItem.getReleaseMargin();
		
		if (type == SINGLE_CELL || type == SINGLE_CELL_WITH_JOINED_CELL) {
			writer.startElement(TD, null);
			boolean release = BOTH.equals(releaseMargin);
			if (hasLabel && widthLabel != null && !release) {
				writer.writeAttribute(WIDTH, widthLabel, WIDTH);
			} else if (hasHelp && widthHelp != null && !release) {
				writer.writeAttribute(WIDTH, widthHelp, WIDTH);
			}
			String labelClass = parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _CELL_SUFFIX;
			String helpClass  = parentStyleClass + CONNECTOR_U + _HELP_SUFFIX  + CONNECTOR_H + _CELL_SUFFIX;
			if (release) {
				labelClass = parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _RELEASED_SUFFIX + CONNECTOR_H + _CELL_SUFFIX;
				helpClass  = parentStyleClass + CONNECTOR_U + _HELP_SUFFIX  + CONNECTOR_H + _RELEASED_SUFFIX + CONNECTOR_H + _CELL_SUFFIX;
			}
			
			if (hasLabel && hasHelp) {
				writer.writeAttribute(CLASS, labelClass + SPACER + helpClass, CLASS);
			} else if (hasLabel) {
				writer.writeAttribute(CLASS, labelClass , CLASS);
			} else { // hasHelp
				writer.writeAttribute(CLASS,helpClass , CLASS);
			}
			if (type == SINGLE_CELL_WITH_JOINED_CELL) {
				writer.writeAttribute(COLSPAN, new Integer(2).toString(), COLSPAN);
			}
			writer.writeText(CR, null);
			if (hasLabel) {
				writeLabel(context, component, label);
			}
			if (hasHelp) {
				encodeCorrectMessage(context, component, showInfoText, showErrorText, errorOnPage, childErrorMessage, parentStyleClass);
			}
			writer.endElement(TD);
			writer.writeText(CR, null);
		} else if (type == LABEL_CELL) {
			final boolean releaseLabel = LABEL.equals(releaseMargin) || BOTH.equals(releaseMargin);
			if (hasLabel) {
				writer.startElement(TD, null);
				if (widthLabel != null && !releaseLabel) {
					writer.writeAttribute(WIDTH, widthLabel, WIDTH);
				}
				if (releaseLabel) {
					writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _RELEASED_SUFFIX + CONNECTOR_H + _CELL_SUFFIX, CLASS);
				} else {
					writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _LABEL_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
				}
				writer.writeText(CR, null);
				writeLabel(context, component, label);
				writer.endElement(TD);
				writer.writeText(CR, null);
			}
		} else if (type == HELP_CELL) {
			final boolean releaseHelp = HELP.equals(releaseMargin) || BOTH.equals(releaseMargin);
			if (hasHelp) {
				writer.startElement(TD, null);
				if (widthHelp != null && !releaseHelp) {
					writer.writeAttribute(WIDTH, widthHelp, WIDTH);
				}
				if (releaseHelp) {
					writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _HELP_SUFFIX + CONNECTOR_H + _RELEASED_SUFFIX + CONNECTOR_H + _CELL_SUFFIX, CLASS);
				} else {
					writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _HELP_SUFFIX + CONNECTOR_H + _CELL_SUFFIX , CLASS);
				}
				writer.writeText(CR, null);
				encodeCorrectMessage(context, component, showInfoText, showErrorText, errorOnPage, childErrorMessage, parentStyleClass);
				writer.endElement(TD);
				writer.writeText(CR, null);
			}
		}
	}
	
	private void startNestedTable(ResponseWriter writer, String parentStyleClass, int span) throws IOException {
		writer.startElement(TD, null);
		if (span > 1) {
			writer.writeAttribute(COLSPAN, (new Integer(span)).toString(), COLSPAN);
		}
		writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _RELEASED_SUFFIX + CONNECTOR_H + _CELL_SUFFIX, CLASS);
		writer.writeText(CR, null);
		writer.startElement(TABLE, null);
		writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _RELEASED_SUFFIX, CLASS);
		writer.startElement(TR, null);
	}
	
	private void endNestedTable(ResponseWriter writer) throws IOException {
		writer.endElement(TR);
		writer.endElement(TABLE);
		writer.endElement(TD);
		writer.writeText(CR, null);
	}
	
	protected boolean needValidator(FacesContext context, UIInput input) {
		Iterator iter = input.getChildren().iterator();
		while(iter.hasNext()) {
			UIComponent child = (UIComponent) iter.next();
			if (child instanceof HtmlInputHelperAssist) {
				// this input has input assist node
				return false;
			}
		}
		return true;
	}	
	
}
