/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.component.html.HtmlPanelFormBox;
import com.ibm.faces.component.html.HtmlFormItem;
import com.ibm.faces.component.html.HtmlFormMessagesArea;

/**
 * @author koinuma
 */
public class FormMessagesAreaRenderer extends Renderer {

	//constants
	private static final String ERROR 				= "error";	 //$NON-NLS-1$
	private static final String ALWAYS 				= "always"; //$NON-NLS-1$
	private static final String INFO 				= "info"; //$NON-NLS-1$
	private static final String BOTH				= "both"; //$NON-NLS-1$
	private static final String ERRORS				= "errors"; //$NON-NLS-1$
	private static final String CR 					= "\n"; //$NON-NLS-1$
//	private static final String VALUE 				= "value";
//	private static final String COLSPAN 			= "colspan";
	
	private static final String DIV 				= "div"; //$NON-NLS-1$
	private static final String TABLE 				= "table"; //$NON-NLS-1$
//	private static final String LABEL 				= "label";
	private static final String TD 					= "td"; //$NON-NLS-1$
	private static final String TR 					= "tr";	 //$NON-NLS-1$
	private static final String UL					= "ul"; //$NON-NLS-1$
	private static final String LI					= "li"; //$NON-NLS-1$
	private static final String SPAN 				= "span"; //$NON-NLS-1$

	private static final String _MESSAGESAREA_SUFFIX= "MessagesArea"; //$NON-NLS-1$
	private static final String _ERROR_SUFFIX 		= "Error"; //$NON-NLS-1$
	private static final String _INFO_SUFFIX 		= "Info"; //$NON-NLS-1$
	private static final String _ERRORS_SUFFIX		= "Errors"; //$NON-NLS-1$
	private static final String CONNECTOR_U			= "_"; //$NON-NLS-1$
	private static final String CONNECTOR_H			= "-"; //$NON-NLS-1$
	private static final String SPACER				= " "; //$NON-NLS-1$
	
	private static final String ERRORS_TABLE		= "table"; //$NON-NLS-1$
	private static final String ERRORS_LIST			= "list"; //$NON-NLS-1$
//	private static final String ERRORS_NONE			= "none";
	
	private static final String CLASS 				= "class"; //$NON-NLS-1$
	private static final String STYLE 				= "style"; //$NON-NLS-1$
	private static final String TITLE				= "title"; //$NON-NLS-1$
	private static final String ID 					= "id";	 //$NON-NLS-1$
	
	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {

		if (!component.isRendered()) {
			return;
		}

		ResponseWriter writer = context.getResponseWriter();
		

		HtmlFormMessagesArea messages = (HtmlFormMessagesArea) component;
		
		String infoSpanId = component.getClientId(context) + CONNECTOR_U + _INFO_SUFFIX.toLowerCase();
		String errorSpanId = component.getClientId(context) + CONNECTOR_U + _ERROR_SUFFIX.toLowerCase();
		String messagesId = component.getClientId(context) + CONNECTOR_U + _ERRORS_SUFFIX.toLowerCase();
		
		String showHelp = messages.getShowHelp();
		boolean showErrorText = (showHelp == null || showHelp.equals(ALWAYS) || showHelp.equals(ERROR) || showHelp.equals(BOTH) || showHelp.equals(ERRORS));
		boolean showInfoText = (showHelp == null || showHelp.equals(ALWAYS) || showHelp.equals(INFO) || showHelp.equals(BOTH));
		boolean showChildErrors = (showHelp == null || showHelp.equals(ALWAYS) || showHelp.equals(ERRORS));
		
		writer.startElement(DIV, null);
		writer.writeAttribute(ID, messages.getClientId(context), ID);		

		// Get the style of the formBox
		String parentStyleClass = ((HtmlPanelFormBox) component.getParent()).getStyleClass();
		if(parentStyleClass != null && parentStyleClass.indexOf(" ") != -1){ //$NON-NLS-1$
			// If multiple classes are defined just use the first one
			parentStyleClass = parentStyleClass.substring(0, parentStyleClass.indexOf(" ")).trim(); //$NON-NLS-1$
		}
		// styleClass attribute
		String classSuffix = ""; //$NON-NLS-1$
		if (messages.getStyleClass() != null && messages.getStyleClass().trim().length() > 0) {
			classSuffix = SPACER + messages.getStyleClass();
		}
		if (parentStyleClass != null) {		
			writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _MESSAGESAREA_SUFFIX + classSuffix, CLASS);
		}

		if (messages.getStyle() != null) {
			writer.writeAttribute(STYLE, messages.getStyle(), STYLE);
		}
		if (messages.getTitle() != null && messages.getTitle().length() >0) {
			writer.writeAttribute(TITLE, messages.getTitle(), TITLE);
		}
		writer.writeText(CR, null);
		writer.startElement(TABLE, null);		
		writer.writeText(CR, null);

		if (showInfoText) {
			if (component.getFacet(INFO) != null) {
				writer.startElement(TR, null);
				writer.writeText(CR, null);
				writer.startElement(TD, null);
				writer.writeText(CR, null);			
				encodeRecursive(context, component.getFacet(INFO));
				writer.writeText(CR, null);
				writer.endElement(TD);
				writer.endElement(TR);
			} else if (messages.getInfoText() != null) {
				writer.writeText(CR, null);
				writer.startElement(TR, null);
				writer.writeText(CR, null);
				writer.startElement(TD, null);
				writer.writeText(CR, null);
				// write error
				writer.startElement(SPAN, null);
				writer.writeAttribute(ID, infoSpanId, ID);
				writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _MESSAGESAREA_SUFFIX + CONNECTOR_H + _INFO_SUFFIX, CLASS);
				if (messages.isEscape()) {
					writer.writeText(messages.getInfoText(), null);
				} else {
					writer.write(messages.getInfoText());
				}
				writer.endElement(SPAN);
				
				writer.writeText(CR, null);
				writer.endElement(TD);
				writer.endElement(TR);
				writer.writeText(CR, null);			
			}
		}
		
		List errors = getErrors(context, component.getParent());
		if (errors != null) {
			if (showErrorText) {
				if (component.getFacet(ERROR) != null) {					
					writer.writeText(CR, null);
					writer.startElement(TR, null);
					writer.writeText(CR, null);
					writer.startElement(TD, null);
					writer.writeText(CR, null);			
					encodeRecursive(context, component.getFacet(ERROR));
					writer.writeText(CR, null);
					writer.endElement(TD);
					writer.endElement(TR);
					writer.writeText(CR, null);			
				} else if (messages.getErrorText() != null) {
					writer.writeText(CR, null);
					writer.startElement(TR, null);
					writer.writeText(CR, null);
					writer.startElement(TD, null);
					writer.writeText(CR, null);
					// write error
					writer.startElement(SPAN, null);
					writer.writeAttribute(ID, errorSpanId, ID);
					writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _MESSAGESAREA_SUFFIX + CONNECTOR_H + _ERROR_SUFFIX, CLASS);
					if (messages.isEscape()) {
						writer.writeText(messages.getErrorText(), null);
					} else {
						writer.write(messages.getErrorText());
					}
					writer.endElement(SPAN);
					
					writer.writeText(CR, null);
					writer.endElement(TD);
					writer.endElement(TR);
					writer.writeText(CR, null);			
				}
			}
			if (showChildErrors && errors != null && errors.size() > 0) {
				if (messages.getShowErrors() == null || messages.getShowErrors().equals(ERRORS_LIST)) {
					// list all errors with list style
					writer.writeText(CR, null);
					writer.startElement(TR, null);
					writer.writeText(CR, null);
					writer.startElement(TD, null);
					writer.writeText(CR, null);
	
					writer.startElement(UL, null);
					writer.writeAttribute(ID, messagesId, ID);
					int errorNum = errors.size();
					for(int i=0;i<errorNum;i++) {
						writer.writeText(CR, null);
						writer.startElement(LI, null);
						writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _ERROR_SUFFIX, CLASS);
						String errorText = (String) ((Object[])errors.get(i))[0];
						Boolean escape = (Boolean) ((Object[])errors.get(i))[1];
						if (escape.booleanValue()) {
							writer.writeText(errorText, null);
						} else {
							writer.write(errorText);
						}
						writer.endElement(LI);
					}
					writer.writeText(CR, null);
					writer.endElement(UL);
	
					writer.writeText(CR, null);
					writer.endElement(TD);
					writer.endElement(TR);
					writer.writeText(CR, null);			
				} else if (messages.getShowErrors().equals(ERRORS_TABLE)) {
					// list all error with table style
					writer.writeText(CR, null);
					writer.startElement(TR, null);
					writer.writeText(CR, null);
					writer.startElement(TD, null);
					writer.writeText(CR, null);
					
					writer.startElement(TABLE, null);
					writer.writeAttribute(ID, messagesId, ID);
					int errorNum = errors.size();
					for(int i=0;i<errorNum;i++) {
						writer.writeText(CR, null);
						writer.startElement(TR, null);
						writer.writeText(CR, null);
						writer.startElement(TD, null);
						writer.writeAttribute(CLASS, parentStyleClass + CONNECTOR_U + _ERROR_SUFFIX, CLASS);
						String errorText = (String) ((Object[])errors.get(i))[0];
						Boolean escape = (Boolean) ((Object[])errors.get(i))[1];
						if (escape.booleanValue()) {
							writer.writeText(errorText, null);
						} else {
							writer.write(errorText);
						}
						writer.endElement(TD);
						writer.writeText(CR, null);
						writer.endElement(TR);
					}
					writer.writeText(CR, null);
					writer.endElement(TABLE);
	
					writer.writeText(CR, null);
					writer.endElement(TD);
					writer.endElement(TR);
					writer.writeText(CR, null);			
				}
			}
		}
		writer.writeText(CR, null);
		writer.endElement(TABLE);
		writer.writeText(CR, null);
		writer.endElement(DIV);
		writer.writeText(CR, null);
		
		
	}

	public void encodeRecursive(FacesContext context, UIComponent component)
			throws IOException {
		if (!component.isRendered()) {
			return;
		}

		component.encodeBegin(context);
		if (component.getRendersChildren()) {
			component.encodeChildren(context);
		} else {
			Iterator i = component.getChildren().iterator();
			while (i.hasNext()) {
				UIComponent child = (UIComponent) i.next();
				encodeRecursive(context, child);
			}
		}
		component.encodeEnd(context);
	}

	private List getErrors(FacesContext context, UIComponent component) throws IOException {

		if (component.getChildren() == null || component.getChildren().size() < 1) {
			return null;
		}

		List children = component.getChildren();
		List errors = null;

		for (int i = 0; i < children.size(); i++) {
			if (children.get(i) instanceof HtmlFormItem) {
				HtmlFormItem child = (HtmlFormItem) children.get(i);
				List components = child.getChildren();

				for (int z = 0; z < components.size(); z++) {
					if (components.get(z) instanceof UIInput) {
						String clientId = ((UIComponent) components.get(z)).getClientId(context);
						Iterator itr = context.getMessages(clientId);
						// store the error messages from the child tags.
						if (itr != null && itr.hasNext()) {
							if (errors == null) {
								errors = new ArrayList();
							}
							if (child.getErrorText() != null && child.getErrorText().length() > 0) {
								errors.add(new Object[] { child.getErrorText(), (child.isEscape() ? Boolean.TRUE : Boolean.FALSE) });
							} else {
								while(itr.hasNext()) {
									FacesMessage msg = (FacesMessage) itr.next();
									if (msg != null) {
										String error = msg.getDetail();
										if (error == null) {
											error = msg.getSummary();
										}
										errors.add(new Object[] { error, Boolean.TRUE } );
									}
								}
							}
							break;
						}
					}
				}
			}
		}
		return errors;
	}
}
