/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlInputHelperFormTracker;

/**
 * @author pandrews
 *
 */
public class FormTrackerRenderer extends Renderer implements IScriptContributor {
	
	public static final String POST_LASTFIELD = ":POST_LASTFIELD";
	public static final String POST_LASTCURSOR = ":POST_LASTCURSOR";
	
	public void decode(FacesContext context, UIComponent component) {
		if (context == null || component == null) {
			throw new NullPointerException();
		}
		HtmlInputHelperFormTracker comp = null;
		if(component instanceof HtmlInputHelperFormTracker){
			comp = (HtmlInputHelperFormTracker)component;
		} else {
			return;
		}
		
		//need to be in a uiform, if not do nothing
		UIComponent parent = component.getParent();
		while(parent != null && !(parent instanceof UIForm)){
			parent = parent.getParent();
		}
		if(parent == null){
			//couldn't find parent form
			return;
		}
		
		Map requestMap = context.getExternalContext().getRequestParameterMap();

		//no need to check if the parameters are there, field correctly set to null if they're not
		comp.setLastFieldId((String)requestMap.get(parent.getClientId(context) + POST_LASTFIELD));
		comp.setLastCursorPosition((String)requestMap.get(parent.getClientId(context) + POST_LASTCURSOR));
	}
	
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}

		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		scriptCollector.register(this, component);
		
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
	}
	
	
	
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException{
		
		if (context == null || component == null) {
			throw new NullPointerException();
		}
		//need to render hidden fields
		
		//need to be in a uiform, if not do nothing
		UIComponent parent = component.getParent();
		while(parent != null && !(parent instanceof UIForm)){
			parent = parent.getParent();
		}
		if(parent == null){
			//couldn't find parent form, leaving now
			return;
		}
		
		ResponseWriter writer = context.getResponseWriter();
		writer.write("\n");
		writer.startElement("input type=\"hidden\"",component);
		writer.writeAttribute("id",parent.getClientId(context) + POST_LASTFIELD,null);
		writer.writeAttribute("name",parent.getClientId(context) + POST_LASTFIELD,null);
		writer.endElement("input");
		writer.write("\n");
		writer.startElement("input type=\"hidden\"",component);
		writer.writeAttribute("id",parent.getClientId(context) + POST_LASTCURSOR,null);
		writer.writeAttribute("name",parent.getClientId(context) + POST_LASTCURSOR,null);
		writer.endElement("input");
		writer.write("\n");
	}
	
	public void contributeScript(FacesContext context, UIComponent component)
			throws IOException {
		
		//get container form id
		UIComponent parent = component.getParent();
		while(parent != null && !(parent instanceof UIForm)){
			parent = parent.getParent();
		}
		if(parent == null){
			//uiform parent not found
			return;
		}
		
		//emit the script
		StringBuffer script = new StringBuffer();
		
		script.append("\n"); //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".addBehavior(\"");
		script.append(parent.getClientId(context));
		script.append("\", \"oninput\", new "); //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".JSFBehaviorTrack());");

		context.getResponseWriter().write(script.toString());
		
	}

	
	
	
	
	
}
