/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;

import javax.faces.component.UIComponent;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.ValueBinding;
import javax.faces.render.Renderer;

import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.webapp.ValueResourcePhaseListener;



/**
 * Renderer for {@link com.ibm.faces.component.UIPlayer UIPlayer} component, 
 * that creates a Generic Player control.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Player_GenericPlayerTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIPlayer}
 */
public final class GenericPlayerRenderer extends Renderer {
	public static final String MIMETYPE_ATTR = "mimeType";
	
	public static final String PARAM_WIDTH  		= "width"; //$NON-NLS-1$
	public static final String PARAM_HEIGHT  		= "height"; //$NON-NLS-1$
	public static final String PARAM_HIDDEN  		= "hidden"; //$NON-NLS-1$
	public static final String PARAM_ACCESSKEY  	= "accesskey"; //$NON-NLS-1$
	public static final String PARAM_ALIGN  		= "align"; //$NON-NLS-1$
	public static final String PARAM_ALT  			= "alt"; //$NON-NLS-1$
	public static final String PARAM_STYLECLASS  	= "styleClass"; //$NON-NLS-1$
	public static final String PARAM_STYLE  		= "style"; //$NON-NLS-1$
	public static final String PARAM_TABINDEX  		= "tabindex"; //$NON-NLS-1$
	public static final String PARAM_TITLE  		= "title"; //$NON-NLS-1$
	public static final String PARAM_DIR  			= "dir"; //$NON-NLS-1$
	public static final String PARAM_LANG  			= "lang"; //$NON-NLS-1$
	public static final String PARAM_ID  			= "id"; //$NON-NLS-1$
	public static final String PARAM_PLUGINSPAGE	= "pluginsPage"; //$NON-NLS-1$
	public static final String PARAM_PLUGINURL  	= "pluginUrl"; //$NON-NLS-1$
	public static final String PARAM_BORDER  		= "border"; //$NON-NLS-1$
	public static final String PARAM_HSPACE  		= "hspace"; //$NON-NLS-1$
	public static final String PARAM_VSPACE  		= "vspace"; //$NON-NLS-1$
	public static final String PARAM_ONBLUR  		= "onblur"; //$NON-NLS-1$
	public static final String PARAM_ONFOCUS  		= "onfocus"; //$NON-NLS-1$
	
	private static final Param[] objectParams = {
		new Param(PARAM_WIDTH, "width"), //$NON-NLS-1$
		new Param(PARAM_HEIGHT, "height"), //$NON-NLS-1$
		new Param(PARAM_HIDDEN, "hidden"), //$NON-NLS-1$
		new Param(PARAM_ACCESSKEY, "accesskey"), //$NON-NLS-1$
		new Param(PARAM_ALIGN, "align"), //$NON-NLS-1$
		new Param(PARAM_ALT, "alt"), //$NON-NLS-1$
		new Param(PARAM_STYLECLASS, "class"), //$NON-NLS-1$
		new Param(PARAM_STYLE, "style"), //$NON-NLS-1$
		new Param(PARAM_TABINDEX, "tabindex"), //$NON-NLS-1$
		new Param(PARAM_TITLE, "title"), //$NON-NLS-1$
		new Param(PARAM_DIR, "dir"), //$NON-NLS-1$
		new Param(PARAM_LANG, "lang"), //$NON-NLS-1$
		new Param(PARAM_ID, "name"), //$NON-NLS-1$
		new Param(PARAM_PLUGINSPAGE,"pluginspage"), //$NON-NLS-1$
		new Param(PARAM_PLUGINURL,"pluginurl"), //$NON-NLS-1$
		new Param(PARAM_BORDER,"border"), //$NON-NLS-1$
		new Param(PARAM_HSPACE,"hspace"), //$NON-NLS-1$
		new Param(PARAM_VSPACE,"vspace"), //$NON-NLS-1$
		new Param(PARAM_ONBLUR,"onblur"),//$NON-NLS-1$
		new Param(PARAM_ONFOCUS,"onfocus") //$NON-NLS-1$
	};
	
	
	/**
	 * Player's parameter.
	 * <p>Matches JSF tag's attribute name to a player's parameter name.
	 */
	public static class Param {
		/** JSF tag attribute */
		protected String attrName;
		/** Player's parameter */
		protected String paramName;

		/**
		 * Constructor.
		 * 
		 * @param attrName attribute name
		 * @param paramName parameter name
		 */
		public Param(String attrName, String paramName){
			this.attrName = attrName;
			this.paramName = paramName;
		}

		public String getName() {
			return attrName;
		}

		public String getValue() {
			return paramName;
		}
	}
	
	

	/** 
	 * Renders the ending of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */   	
	public void encodeEnd(FacesContext context, UIComponent component)
		throws IOException 
	{
		if (context == null || component == null) {
			throw new NullPointerException();
		}
		
		StringBuffer embedAttr=new StringBuffer();
		
		HashMap attrMap= new HashMap();
		Iterator paramKeys;
		ResponseWriter writer=context.getResponseWriter();
		
		
		/* Encode predefinied params (src, width, height) */
		encodeAttributes(context, component, attrMap,objectParams);  
		
		//paramKeys=attrMap.keySet().iterator();
		//while (paramKeys.hasNext()) {
		//	String key=(String)paramKeys.next();
		//	embedAttr.append(key+"=\""+attrMap.get(key)+"\""+" ");
		//}
		writer.startElement("span",component);
		writer.writeAttribute("id",component.getClientId(context)+"span","id");
		writer.endElement("span");
		
		/* Removing noscript section, not necessary as we expect js to be enabled for our components 
		 * to work.
		 * See defect 278225
		writer.write("\n<noscript>\n<embed " + embedAttr.toString() + "></embed>\n</noscript>");
		*/
		
		//UIScriptCollector.find(component).addScriptOnce(constructPlayerScript(context,component,embedAttr));
		UIScriptCollector.find(component).addScript(constructPlayerScript(context,component,attrMap));
	 }
     
	/** 
	 * Renders control's predefined parameters (src, width, height).
	 * The src attribute can point to a file via uri/url or bound to a back end data.
	 * When binding data to src the 'mimeType' attribute must also be set, the 'filename' attribute is optional. 
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * @param htmlOutput String buffer to append HTML to
	 * @param params Array of parameters to be rendered
	 * 
	 */
	protected void encodeAttributes(FacesContext context, UIComponent component, HashMap attrMap, Param[] params)
	{
		Object objValue;
		attrMap.put("src",src(context,component));

		//get the rest of the attr
		int len = params.length;
		for (int i = 0; i < len; i++) {
			if (null != (objValue = component.getAttributes().get(params[i].attrName))) {
				attrMap.put(params[i].paramName,objValue.toString());
			}
		}
		
		//get UIParmater children if any... 
		Iterator children = component.getChildren().iterator();
		while (children.hasNext()){
			UIComponent c = (UIComponent)children.next();
			if(c instanceof UIParameter){
				Object value = ((UIParameter)c).getValue();
				attrMap.put(((UIParameter)c).getName(),value == null? null : value.toString());
			}
		}
	}

	/** 
	 * Gets the list of parameters (specific to player) added by
	 * the page developer).
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *	 
	 */   
	/** 
	 * Renders the beginning of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */   	
	public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {
			if (context == null || component == null) {
				throw new NullPointerException();
			}
			UIScriptCollector scriptCollector = UIScriptCollector.find(component);
			HxClientRenderUtil.initJSLibraries(scriptCollector, context);
	}

	private String constructPlayerScript(FacesContext context, UIComponent component,/*StringBuffer*/ HashMap embedAttr){

		StringBuffer scriptOut=new StringBuffer();
		String spanId = component.getClientId(context)+"span";

		scriptOut.append("if(null != "); //$NON-NLS-1$
		scriptOut.append(HxClientRenderUtil.HX_VAR_NAME);
		scriptOut.append(".getElementById('").append(spanId).append("')){\n");
		scriptOut.append("var elem = document.createElement(");
		scriptOut.append(HxClientRenderUtil.HX_VAR_NAME);
		scriptOut.append(".TAG_EMBED + '');");		

		Iterator paramKeys;		
		paramKeys=embedAttr.keySet().iterator();
		while (paramKeys.hasNext()) {
			String key=(String)paramKeys.next();
			if(!("name".equalsIgnoreCase(key)))
				scriptOut.append("elem.setAttribute(\""+key+"\",\""+embedAttr.get(key)+"\");");
			else
				scriptOut.append("elem."+key+" = \""+embedAttr.get(key)+"\";");
		}
		
		
		scriptOut.append("(");
		scriptOut.append(HxClientRenderUtil.HX_VAR_NAME);
		scriptOut.append(".getElementById('");
		scriptOut.append(spanId);
		scriptOut.append("')).appendChild(elem);");
		
		scriptOut.append("}\n");
		
		/*scriptOut.append("')).innerHTML='<' + ");
		scriptOut.append(HxClientRenderUtil.HX_VAR_NAME);
		scriptOut.append(".TAG_EMBED + ' ");
		scriptOut.append(embedAttr.toString());
		scriptOut.append(" />';}\n");*/

		return(scriptOut.toString());
	}
	
	private String src(FacesContext context, UIComponent component) {
		String value = ""; //$NON-NLS-1$
		ValueBinding vb = component.getValueBinding("src"); //$NON-NLS-1$

		if ((component.getValueBinding(MIMETYPE_ATTR) != null)
		|| (component.getAttributes().get(MIMETYPE_ATTR) != null)) {
			
			//need to set 'value' valuebinding for ValueResourceListener
			component.setValueBinding("value",vb);						
			
			value = ValueResourcePhaseListener.getUrl(context,component);
		} else {
			if (vb != null) {
				Object obj = vb.getValue(context);
				if (obj instanceof String) {
					value = (String) obj;
				}
		} else {
			value = (String) component.getAttributes().get("src"); //$NON-NLS-1$
		}
		if (value == null) {
			value = ""; //$NON-NLS-1$
		}
		}
		value =	context.getApplication().getViewHandler().getResourceURL(context,value);
		return (context.getExternalContext().encodeResourceURL(value));
	}
}
