/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.ResourceBundle;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import com.ibm.faces.ResourceHandler;
import com.ibm.faces.util.HtmlUtil;
import com.ibm.faces.util.ScriptObject;
import com.ibm.faces.validator.LongRangeValidatorEx;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIPager;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.UIPager.PagerDataWrapper;
import com.ibm.faces.component.html.HtmlPagerGoto;
import com.ibm.faces.converter.NumberConverterEx;

/**
 * @author Alasdair McDowell
 * <p>Goto Pager</p>
 */
public class GotoPagerRenderer extends AbstractPagerRenderer implements IScriptContributor{

    protected static final String PAGER_GO = "__pagerGoButton"; //$NON-NLS-1$
    protected static final String PAGER_TEXT = "__pagerGoText"; //$NON-NLS-1$

    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }

		HtmlPagerGoto gotoComponent = null;		
		if ( component instanceof HtmlPagerGoto ){
			gotoComponent = (HtmlPagerGoto)component;
		}


        ResponseWriter writer = context.getResponseWriter();
		ResourceBundle bundle = ResourceHandler.getBundle(context);

        String localizedGo = ResourceHandler.getString(bundle, "PagerRenderer.go"); //$NON-NLS-1$
        
        PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
 		int currentPage = ((UIPager)component).getCurrentPage(pdw);
 		
        String id = component.getClientId(context);
        
        String style;
        String styleClass;
        //RATLC01081538
        String type;
        String imageSrc;
        String title;
        String tabindex;
        String alt;
        
        if ( gotoComponent != null ){
        	style = gotoComponent.getStyle();
        	styleClass = gotoComponent.getStyleClass();
        	type = gotoComponent.getType();
        	imageSrc = gotoComponent.getButtonImageSrc();
        	title = gotoComponent.getTitle();
        	tabindex = gotoComponent.getTabindex();
        	alt = gotoComponent.getAlt();
        }else{
        	style = (String) component.getAttributes().get("style");
			styleClass = (String) component.getAttributes().get("styleClass");
			type = (String) component.getAttributes().get("type");
			imageSrc = (String)component.getAttributes().get("buttonImageSrc");
			title = (String)component.getAttributes().get("title");
			tabindex = (String)component.getAttributes().get("tabindex");
			alt = (String)component.getAttributes().get("alt");
        }
        
        if(imageSrc != null){
        	imageSrc =  Utils.getEncodedResourceURL(context, imageSrc);
        }        
    	//write style span
    	writer.startElement("span",component);
    	writer.writeAttribute("id",id,null);
    	if(style != null)
    		writer.writeAttribute("style",style,"style");
    	if(styleClass != null)
    		writer.writeAttribute("class",styleClass,"styleClass");
    	if(title != null)
    		writer.writeAttribute("title",title,"title");
        
    	
        if(type == null || type.equalsIgnoreCase("text") || !isTypeValid(type).booleanValue()){
        	writeTextInput(writer,id,currentPage, getFieldSize(component), styleClass, tabindex);
        } else if (type.equalsIgnoreCase("combobox")) {
        	writeCombobox(writer,component,id,currentPage);
        } else if (type.equalsIgnoreCase("spinner") || type.equalsIgnoreCase("slider")){
        	int size = getFieldSize(component);
        	//problem displaying in IE, button appears under control, putting in table to fix
        	writer.startElement("table",component);
        	writer.startElement("tr",component);
        	writer.startElement("td",component);
        	writer.writeAttribute("width",String.valueOf(size * 8 + 20),null);
        	writeSpinnerSlider(context,writer,component,id,currentPage,type,size, styleClass, tabindex);
        	writer.endElement("td");
        	writer.startElement("td",component);
        }
            
        writeSubmitControl(component, writer, localizedGo, id, styleClass, imageSrc, tabindex, alt);
        
        if (type != null && (type.equalsIgnoreCase("spinner") || type.equalsIgnoreCase("slider"))){
            //problem displaying in IE, close the table
            writer.endElement("td");
            writer.endElement("tr");
            writer.endElement("table");
        }
        
        writer.endElement("span");
        
    }

    /**
     * @param component
     * @param writer
     * @param localizedGo
     * @param id
     * @param styleClass
     * @param imageSrc
     * @throws IOException
     */
    protected void writeSubmitControl(UIComponent component, ResponseWriter writer, String localizedGo, String id, String styleClass, String imageSrc, String tabindex, String alt) throws IOException {
        writer.startElement("span",component);
        
        writer.write("\n");
        writer.startElement("input",component);

        if(imageSrc != null){
            writer.writeAttribute("type","image",null);
            writer.writeAttribute("src",imageSrc,null);
        } else {
            writer.writeAttribute("type","submit",null);
        }
        writer.writeAttribute("id",id + PAGER_GO,null);
        writer.writeAttribute("name",id + PAGER_GO,null);
        writer.writeAttribute("value",localizedGo,null);
        if(styleClass != null){
            writer.writeAttribute("class",styleClass,null);
        }
        if(tabindex != null){
        	writer.writeAttribute("tabindex",tabindex,null);
        }
        if(alt != null){
        	writer.writeAttribute("alt",alt,"alt");
        }
        writer.endElement("input");
        writer.write("\n");
        
        writer.endElement("span");
    }

    /**
     * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     */
    public void decode(FacesContext context, UIComponent component) {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        
        PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
        if (!((UIPager)component).isPaged(pdw)) {
            return;
        }

        String id = component.getClientId(context);
        String keyButton = id + PAGER_GO;
        if (this.wasInputClicked(context, component, keyButton)) {

            String keyText = id + PAGER_TEXT;
            String valueText =
                (String) context
                    .getExternalContext()
                    .getRequestParameterMap()
                    .get(
                    keyText);
            if (valueText != null) {
                try {
                    Integer objI = new Integer(valueText);
                    int newPage = objI.intValue() - 1; //0-index
                    int currentPage = ((UIPager)component).getCurrentPage(pdw);
                    int lastPage = ((UIPager)component).getLastPage(pdw);
                    this.queueEvent(
                        component,
                        currentPage,
                        newPage,
                        lastPage);
                }
                catch (NumberFormatException e) {
                    //FAILSAFE
                }
            }

            return;
        }
    }
    
    private void writeTextInput(ResponseWriter writer, String id, int currentPage, int size, String styleClass, String tabindex) throws IOException{
    	//original code for writing goto pager input
        String textId = id + PAGER_TEXT;
        HtmlUtil.writeTagStartOpen(writer, "input", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "type", "text"); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "size", String.valueOf(size)); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "id", textId); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "name", textId); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "value", "" + (currentPage + 1)); //$NON-NLS-1$ //$NON-NLS-2$
        if(tabindex != null)
        	HtmlUtil.writeTagAttribute(writer, "tabindex", tabindex); //$NON-NLS-1$ //$NON-NLS-2$
        if(styleClass != null){
        	HtmlUtil.writeTagAttribute(writer, "class", styleClass + "_inputText"); //$NON-NLS-1$ //$NON-NLS-2$
        }
        HtmlUtil.writeTagStartCloseLn(writer, true);
    }
    
    private void writeCombobox(ResponseWriter writer, UIComponent component, String id, int currentPage) throws IOException{
    	
    	//write select tag
    	String textId = id + PAGER_TEXT;
    	writer.startElement("select",component);
    	writer.writeAttribute("id",textId,null);
    	writer.writeAttribute("name",textId,null);
    	
    	PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
    	//get total no. of pages
    	int total = ((UIPager)component).getLastPage(pdw) + 1;
    	if(total < 1) total = 1;
    	//write options
    	for(int i = 1; i <= total;i++) {
    		writer.startElement("option",component);
    		if(i == (currentPage + 1)){
    			writer.writeAttribute("selected",Boolean.TRUE,null);
    		}
    		writer.writeAttribute("value",String.valueOf(i),null);
    		writer.writeText(String.valueOf(i),null);
    		writer.endElement("option");
    		
    	}
    	writer.endElement("select");
    }
    
    private void writeSpinnerSlider(FacesContext context, ResponseWriter writer, UIComponent component, String id, int currentPage, String type, int size, String styleClass, String tabindex) throws IOException{

    	String textId = id + PAGER_TEXT;
    	//write input
    	writer.startElement("input",component);
        writer.writeAttribute("type","text",null);
        writer.writeAttribute("size",String.valueOf(size),null);
        writer.writeAttribute("id",textId,null);
        writer.writeAttribute("name",textId,null);
        writer.writeAttribute("value",String.valueOf(currentPage + 1),null);
        if(styleClass != null){
        	writer.writeAttribute("class", styleClass + "_inputText", null); //$NON-NLS-1$ //$NON-NLS-2$
        }
        if(tabindex != null){
        	writer.writeAttribute("tabindex", tabindex, null); //$NON-NLS-1$ //$NON-NLS-2$
        }        
        writer.endElement("input");
        
        PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
        //get total no. of pages
    	int total = ((UIPager)component).getLastPage(pdw) + 1;
    	if(total < 1) total = 1;
        
    	//add slider script to the input
    	UIScriptCollector scriptCollector = UIScriptCollector.find(component);
    	HxClientRenderUtil.initJSLibraries(scriptCollector, context);
    	HxClientRenderUtil.addScriptLibrary("hxclient_sl.js", component);
    	HxClientRenderUtil.addScriptLibrary("hxclient_sp.js", component);
		    	
    	//create suitable number converter
    	NumberConverterEx nc = new NumberConverterEx();
    	nc.setIntegerOnly(true);
    	//create suitable validator
    	LongRangeValidatorEx lrv = new LongRangeValidatorEx();
    	lrv.setMinimum(1);
    	lrv.setMaximum(total);
    	//add script to collector
    	ScriptObject converterScript =HxClientRenderUtil.buildJsNumberConverter(context, nc);
    	ScriptObject validatorScript = HxClientRenderUtil.buildJsRangeValidator(context,nc, lrv, false, null, null);
    	scriptCollector.addScriptOnce(converterScript.getScript());
    	scriptCollector.addScriptOnce(validatorScript.getScript());
    	StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME);  //$NON-NLS-1$
    	script.append(".addComponent(\"");
    	script.append(textId);
		
    	//slider or spinner?
    	if(type.equalsIgnoreCase("slider")) {
    		script.append("\", new ");  //$NON-NLS-1$
    		script.append(HxClientRenderUtil.HX_VAR_NAME);
    		script.append(".JSFSlider(\"increment:1\"");
    	} else {
    		script.append("\", new ");  //$NON-NLS-1$
    		script.append(HxClientRenderUtil.HX_VAR_NAME);
    		script.append(".JSFSpinner(\"increment:1\"");
    	}
    	script.append(",\"converter:");  //$NON-NLS-1$
    	script.append(converterScript.getId());
    	script.append('\"'); //$NON-NLS-1$
    	script.append(",\"validator:");  //$NON-NLS-1$
    	script.append(validatorScript.getId());
    	script.append('\"'); //$NON-NLS-1$
    	script.append("));");  //$NON-NLS-1$
    	scriptCollector.addScript(script.toString());
        
    }

    
    /*
    	private void writeInputSubmit(
    		Writer writer,
			String id,
			String label,
			String css,
			boolean enabled)
        throws IOException {

        HtmlUtil.writeTagStartOpen(writer, "input", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "type", "submit"); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "name", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "value", label); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", css); //$NON-NLS-1$
        if (!enabled) {
            HtmlUtil.writeTagAttribute(writer, "disabled", "disabled"); //$NON-NLS-1$ //$NON-NLS-2$
        }
        HtmlUtil.writeTagStartCloseLn(writer, true);
    }
    */

    protected boolean wasInputClicked(FacesContext context, UIComponent component, String name) {
    	//UIComponent parameter passed in not used here, but is used by xfaces
        return (context.getExternalContext().getRequestParameterMap().get(name) != null) || (context.getExternalContext().getRequestParameterMap().get(name + ".x") //$NON-NLS-1$
        != null);
    }
    
    private Boolean isTypeValid(String type){
    	String[] types = {"text","combobox","spinner","slider"};
    	Boolean valid = Boolean.FALSE;
    	for(int x = 0; x < types.length; x++){
    		if(type.equalsIgnoreCase(types[x])){
    			valid = Boolean.TRUE;
    			break;
    		}
    	}
    	return valid;
    }
    
    private int getFieldSize(UIComponent component){
    	int size = 4;
    	PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
    	if(pdw == null){
    		return size;
    	}
    	if (pdw.getRows() > 0) {
    		size = String.valueOf(pdw.getRowCount() / pdw.getRows()).length();
    	} else {
    		size = 0;
    	}
    	if(size == 0)
    		size = 1;
    	++size;
    	return size;
    }

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		// TODO Auto-generated method stub
		
	}
}
