/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;


import java.lang.reflect.Field;
import java.util.Locale;

import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import com.ibm.faces.converter.DateTimeConverterExICU;
import com.ibm.faces.converter.NumberConverterExICU;
import com.ibm.faces.util.ICU4JUtil;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.icu.text.DateFormat;
import com.ibm.icu.text.DateFormatSymbols;
import com.ibm.icu.text.DecimalFormat;
import com.ibm.icu.text.DecimalFormatSymbols;
import com.ibm.icu.text.NumberFormat;
import com.ibm.icu.text.SimpleDateFormat;
import com.ibm.icu.util.BuddhistCalendar;
import com.ibm.icu.util.Calendar;
import com.ibm.icu.util.GregorianCalendar;
import com.ibm.icu.util.HebrewCalendar;
import com.ibm.icu.util.IslamicCalendar;
import com.ibm.icu.util.JapaneseCalendar;

/**
 * @author pandrews

 */
public class HxClientICU {


	private static int STANDALONE = 0;
	private static int NARROW = 0;
	
	/*
	 * <p>Need to get these ICU4J constants by reflection as they are static final values 
	 * that have varied in different versions of ICU4J (e.g. changed values 3.4.1 and 3.4.4) 
	 * as they are static final ints they are not looked up again after this class has been 
	 * compiled, so we use reflection to get the latest values.
	 */
	
	protected static final void initICU4JConstants() throws Exception
	{
		Class clazz = Class.forName("com.ibm.icu.text.DateFormatSymbols");
		Field f = clazz.getField("STANDALONE");
		STANDALONE = f.getInt(null);
		f = clazz.getField("NARROW");
		Field g = clazz.getField("NARROW");
		NARROW = g.getInt(null);
	}
	
	
	protected static final String buildJsDateTimeConverterICU(DateTimeConverterExICU datetimeConverter) {

		String pattern = datetimeConverter.getPattern();
		String type = datetimeConverter.getType();
		String dateStyle = datetimeConverter.getDateStyle();
		String timeStyle = datetimeConverter.getTimeStyle();
		String format = null;
		String epoch = new String("");
		int fdow = 0;
		
		Locale locale = datetimeConverter.getLocale();
		
		Calendar calendar = null;
		calendar = datetimeConverter.getCalendar();
		
				
		if (pattern != null && !pattern.equals("") && calendar == null) //$NON-NLS-1$
			format = pattern;
		else if (calendar != null)
		{		

				// GregorionCalendar is the default anyways
				if(calendar instanceof BuddhistCalendar)
					epoch = new String("b");				
				if(calendar instanceof HebrewCalendar)
					epoch = new String("h");
				if(calendar instanceof IslamicCalendar)
					epoch = new String("i");
				if(calendar instanceof JapaneseCalendar)
					epoch = new String("j");	
				/*if(calendar instanceof CopticCalendar)
					epoch = new String("c");
				if(calendar instanceof EthiopticCalendar)
					epoch = new String("e");*/ // if hxclient js ever deal with Coptic and Ethioptic
				fdow = calendar.getFirstDayOfWeek();
			
			
			DateFormat df_icu = null;
			if(calendar != null)
			{
				if (type.equalsIgnoreCase("both")) { //$NON-NLS-1$
					df_icu = DateFormat.getDateTimeInstance(calendar, ICU4JUtil.getDateStyle(dateStyle), ICU4JUtil.getDateStyle(timeStyle), locale);
				} else if (type.equalsIgnoreCase("date")) { //$NON-NLS-1$
					df_icu = DateFormat.getDateInstance(calendar, ICU4JUtil.getDateStyle(dateStyle), locale);
				} else if (type.equalsIgnoreCase("time")) { //$NON-NLS-1$
					df_icu = DateFormat.getTimeInstance(calendar, ICU4JUtil.getDateStyle(timeStyle), locale);
				} else {
					df_icu = DateFormat.getDateInstance(calendar, ICU4JUtil.getDateStyle(dateStyle), locale);
				}
			}
			else
			{
				if (type.equalsIgnoreCase("both")) { //$NON-NLS-1$
					df_icu = DateFormat.getDateTimeInstance(ICU4JUtil.getDateStyle(dateStyle), ICU4JUtil.getDateStyle(timeStyle), locale);
				} else if (type.equalsIgnoreCase("date")) { //$NON-NLS-1$
					df_icu = DateFormat.getDateInstance( ICU4JUtil.getDateStyle(dateStyle), locale);
				} else if (type.equalsIgnoreCase("time")) { //$NON-NLS-1$
					df_icu = DateFormat.getTimeInstance( ICU4JUtil.getDateStyle(timeStyle), locale);
				} else {
					df_icu = DateFormat.getDateInstance(ICU4JUtil.getDateStyle(dateStyle), locale);
				}
			}
			
			if (pattern != null && !pattern.equals(""))
			{
				((SimpleDateFormat)df_icu).applyPattern(pattern);
				format = ((SimpleDateFormat)df_icu).toPattern();
			}
			else
				format = ((SimpleDateFormat) df_icu).toPattern();
		}

		
		if (null != format) 
		{
			StringBuffer javascript = new StringBuffer("new " + HxClientRenderUtil.HX_VAR_NAME + ".DateTimeConverter(");
			// extra ICU stuff
			javascript.append("\"ICU4J\", \"epoch:"+epoch+"\", \"first-day-of-week:"+ fdow+"\",");
			// extra end
			javascript.append(" \"strict:" + HxClientRenderUtil.getDateStrictnessLevel() +"\", \"format:"); //$NON-NLS-1$ //$NON-NLS-2$
			javascript.append(format);
			if(!locale.toString().equals(FacesContext.getCurrentInstance().getViewRoot().getLocale().toString())) {
				javascript.append("\", \"locale:").append(locale);
			}
			javascript.append("\")"); //$NON-NLS-1$
			return javascript.toString();
		}
		return null;
	}

	/***
	 * TODO Number Converter code ICU  
	 */
	
	
	/**
	 * Builds the javascript statement to create a new NumberConverter. 
	 *  
	 * @param numConverter
	 * @param locale
	 * @return the javascript new converter statement without trailing ';'
	 */
	protected static final String buildJsNumberConverterICU(NumberConverterExICU numConverter) {

		String converterPattern = null;
		NumberFormat formatter = numConverter.getNumberFormat(FacesContext.getCurrentInstance());
		
		try {
			// Configure the formatter
			configureFormatterICU(numConverter, formatter);
			
			String pattern = numConverter.getPattern();
			if (formatter instanceof DecimalFormat) {
				if(pattern != null && !pattern.equals("")){ //$NON-NLS-1$
					((DecimalFormat) formatter).applyPattern(pattern);
					converterPattern=pattern;
				} else {
					converterPattern = ((DecimalFormat) formatter).toPattern();	
				}
				
				//RATLC00670050 - output cannot handle the currency pattern character in japanese locale(there maybe others locales affected) 
				//replacing it with unicode string for the character
				while(converterPattern.indexOf("\u00A4")>-1){ //$NON-NLS-1$
					int pos=converterPattern.indexOf("\u00A4"); //$NON-NLS-1$
					StringBuffer b=new StringBuffer();
					b.append(converterPattern.substring(0,pos));
					b.append("\\u00A4"); //$NON-NLS-1$
					b.append(converterPattern.substring(pos+1,converterPattern.length()));
					converterPattern=b.toString();
				}
				
			} else {
				String formattedResult = formatter.format(new Double(111111111111111111.111111111111111111));
				converterPattern = formattedResult.replace('1', '#');
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		if (null != converterPattern && converterPattern.length() > 0) {
			StringBuffer javascript = new StringBuffer("new ");
			javascript.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".NumberConverter(\"ICU4J\", ")
			.append("\"strict:")
			.append(HxClientRenderUtil.getNumberStrictnessLevel())
			.append("\", \"pattern:") //$NON-NLS-1$ //$NON-NLS-2$
			.append(converterPattern);
			//append ibm pattern extension to pattern if set
			if(numConverter.isPatternExtensionSet()){
				javascript.append(numConverter.getPatternExtension());
			}
			//RATLC00976099 fix, client side validation not working correctly when currency symbol set
			//This section of code was not using the symbols from the configured formatter set up previously in this method 
			DecimalFormatSymbols symbols=null;
			if(formatter instanceof DecimalFormat){
				symbols=((DecimalFormat)formatter).getDecimalFormatSymbols();
			} else {
				symbols = new DecimalFormatSymbols(numConverter.getLocale());
			}
			char zero_digit = symbols.getZeroDigit();
			
			String w = Character.toString(zero_digit);
			if(!"0".equalsIgnoreCase(w)) // if you are not a western zero character
			{
				int i = zero_digit;				
				javascript.append("\", \"digits:"+i); // set set you.
			}
			javascript.append("\", \"locale:"); //$NON-NLS-1$
			
			StringBuffer localeBuf = new StringBuffer();
			localeBuf.append(convertNbspToSpace(symbols.getGroupingSeparator()));
			localeBuf.append(convertNbspToSpace(symbols.getDecimalSeparator()));
			localeBuf.append(symbols.getPercent());
			localeBuf.append(symbols.getPerMill());
			localeBuf.append(symbols.getMinusSign());
			localeBuf.append(Utils.escapeBackslash(symbols.getCurrencySymbol()));
			
			//hebrew currency symbol contains a " so encode complete string for this language code, it can be "iw" (deprecated) or "he"
			Locale loc = FacesContext.getCurrentInstance().getViewRoot().getLocale();
			if(loc.getLanguage().equalsIgnoreCase("he") || loc.getLanguage().equalsIgnoreCase("iw")){
				javascript.append(JavaScriptUtil.encodeScriptResourceString(localeBuf.toString(), true));	
			} else {
				//just encode chars outside ascii range
				javascript.append(JavaScriptUtil.encodeScriptResourceString(localeBuf.toString(), false));
			}
			
			javascript.append("\")"); //$NON-NLS-1$		
			return javascript.toString();
		}
		return null;
	}
	
	// RATLC01133770 user cannot easily type an nbsp so convert and expect converter to handle
	private static char convertNbspToSpace(char c){
 		// if input then we want to change the thousands separator from an nbsp to space
		return (c == '\u00A0') ? ' ' : c;
	}
	

	//private static final Class[] GET_INSTANCE_PARAM_TYPES = new Class[] { String.class };

	/**
	 * <p>Configure the specified <code>NumberFormat</code> based on the
	 * formatting properties that have been set.</p>
	 *
	 * @param formatter The <code>NumberFormat</code> instance to configure
	 */
	private static void configureFormatterICU(NumberConverterExICU numConverter, NumberFormat formatter) {

		formatter.setGroupingUsed(numConverter.isGroupingUsed());
		if (numConverter.isMaxIntegerDigitsSet()) {
			formatter.setMaximumIntegerDigits(numConverter.getMaxIntegerDigits());
		}
		if (numConverter.isMinIntegerDigitsSet()) {
			formatter.setMinimumIntegerDigits(numConverter.getMinIntegerDigits());
		}
		if (numConverter.isMaxFractionDigitsSet()) {
			formatter.setMaximumFractionDigits(numConverter.getMaxFractionDigits());
		}
		if (numConverter.isMinFractionDigitsSet()) {
			formatter.setMinimumFractionDigits(numConverter.getMinFractionDigits());
		}
		if (numConverter.isIntegerOnly()) {
			formatter.setMaximumFractionDigits(0);
		}

	}
	
	/*
	 * Fields used by compileHxICUResources.
	 */
	private static byte SHORT_MONTH = 0;
	private static byte DAY = 1;
	private static byte SHORT_DAY = 2;
	private static byte ABBREVIATED_DAY = 3;
	private static byte HEBREW = 4;
	private static byte ISLAMIC = 5;
	
	/*
	 * Compiles hx resources from ICU strings.
	 */
	protected static String compileHxICUResources(Locale locale) {
		
		try {
			initICU4JConstants();
		} catch (Exception e) {
			//TODO - we should report error... 
			//Problem determining icu4j constants, we can carry on though methods using these
			//constants will be be incorrent
		}
		
		StringBuffer buf = new StringBuffer();
		
		DateFormatSymbols gregorianDFS = new DateFormatSymbols(new GregorianCalendar(locale),locale);
		DateFormatSymbols islamicDFS = new DateFormatSymbols(new IslamicCalendar(locale),locale);
		DateFormatSymbols hebrewDFS = new DateFormatSymbols(new HebrewCalendar(locale),locale);
		DateFormatSymbols buddhistDFS = new DateFormatSymbols(new BuddhistCalendar(locale),locale);
		DateFormatSymbols japaneseDFS = new DateFormatSymbols(new JapaneseCalendar(locale),locale);
		
		String valueArray[] = null;
		String tempArray[] = null;
		
		/*
		 * LONG MONTH NAMES
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_MONTHS = \n")
		.append("//Gregorian...\n");
		//valueArray = new String[]{gCal.get(gCal.JANUARY),};
		buf.append("[[")
		.append(Utils.appendStrings(gregorianDFS.getMonths()))
		.append("],\n");
		
		buf.append("//Islamic...\n")
		.append("[")
		.append(Utils.appendStrings(islamicDFS.getMonths()))
		.append("],\n");
		
		buf.append("//Hebrew...\n")
		.append("[");
		//need to reorder month sequenc as in bill resource file
		tempArray = hebrewDFS.getMonths();
		valueArray = new String[13];
		for(int i = 0; i < tempArray.length; i++){
			if(i >= 7){
				valueArray[(i - 7)] = tempArray[i];
			} else{
				valueArray[(i + 6)] = tempArray[i];
			}
		}
		buf.append(Utils.appendStrings(valueArray))
		.append("]];\n");
		
		/*
		 * SHORT MONTH NAMES
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_SHMONTHS = \n")
		.append("//Gregorian...\n");
		buf.append("[[")
		.append(Utils.appendStrings(gregorianDFS.getShortMonths()))
		.append("],\n");
		
		buf.append("//Islamic...\n")
		.append("[");
		//appendStrings(buf,islamicDFS.getShortMonths());
		checkBeforeAppendStrings(buf,islamicDFS.getShortMonths(),SHORT_MONTH,ISLAMIC,gregorianDFS);
		buf.append("],\n");
		
		buf.append("//Hebrew...\n")
		.append("[");
		//appendStrings(buf,hebrewDFS.getShortMonths());
		checkBeforeAppendStrings(buf,hebrewDFS.getShortMonths(),SHORT_MONTH,HEBREW,gregorianDFS);
		buf.append("]];\n");
		
		/*
		 * LONG DAYS OF THE WEEK
		 * (currenlty icu4j returns 8 element in the array, first one empty)
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_DAYS = \n")
		.append("//Gregorian...\n");
		buf.append("[[")
		.append(Utils.appendStrings(HxClientRenderUtil.removeEmptyStringArrayItems(gregorianDFS.getWeekdays())))
		.append("],\n");
		
		buf.append("//Islamic...\n")
		.append("[");
		//appendStrings(buf,removeWeekdayBlank(islamicDFS.getWeekdays()));
		checkBeforeAppendStrings(buf,HxClientRenderUtil.removeEmptyStringArrayItems(islamicDFS.getWeekdays()),DAY,ISLAMIC,gregorianDFS);
		buf.append("],\n");
		
		buf.append("//Hebrew...\n")
		.append("[");
		//appendStrings(buf,removeWeekdayBlank(hebrewDFS.getWeekdays()));
		checkBeforeAppendStrings(buf,HxClientRenderUtil.removeEmptyStringArrayItems(hebrewDFS.getWeekdays()),DAY,HEBREW,gregorianDFS);
		buf.append("]];\n");
		
		/*
		 * SHORT DAYS OF THE WEEK
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_SHDAYS = \n")
		.append("//Gregorian...\n");
		buf.append("[[")
		.append(Utils.appendStrings(HxClientRenderUtil.removeEmptyStringArrayItems(gregorianDFS.getShortWeekdays())))
		.append("],\n");
		
		buf.append("//Islamic...\n")
		.append("[");
		//appendStrings(buf,removeWeekdayBlank(islamicDFS.getShortWeekdays()));
		checkBeforeAppendStrings(buf,HxClientRenderUtil.removeEmptyStringArrayItems(islamicDFS.getShortWeekdays()),SHORT_DAY,ISLAMIC,gregorianDFS);
		buf.append("],\n");
		
		buf.append("//Hebrew...\n")
		.append("[");
		//appendStrings(buf,removeWeekdayBlank(hebrewDFS.getShortWeekdays()));
		checkBeforeAppendStrings(buf,HxClientRenderUtil.removeEmptyStringArrayItems(hebrewDFS.getShortWeekdays()),SHORT_DAY,HEBREW,gregorianDFS);
		buf.append("]];\n");
		
		/*
		 * ABBREVIATED DAYS OF THE WEEK
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_SHSHDAYS = \n")
		.append("//Gregorian...\n");
		buf.append("[[")
		.append(Utils.appendStrings(HxClientRenderUtil.removeEmptyStringArrayItems(gregorianDFS.getWeekdays(STANDALONE, NARROW))))
		.append("],\n");
		
		buf.append("//Islamic...\n")
		.append("[");
		//appendStrings(buf,removeWeekdayBlank(islamicDFS.getWeekdays(DateFormatSymbols.STANDALONE, DateFormatSymbols.NARROW)));
		checkBeforeAppendStrings(buf,HxClientRenderUtil.removeEmptyStringArrayItems(islamicDFS.getWeekdays(STANDALONE, NARROW)),ABBREVIATED_DAY,ISLAMIC,gregorianDFS);
		buf.append("],\n");
		
		buf.append("//Hebrew...\n")
		.append("[");
		//appendStrings(buf, removeWeekdayBlank(hebrewDFS.getWeekdays(DateFormatSymbols.STANDALONE, DateFormatSymbols.NARROW)));
		checkBeforeAppendStrings(buf,HxClientRenderUtil.removeEmptyStringArrayItems(hebrewDFS.getWeekdays(STANDALONE, NARROW)),ABBREVIATED_DAY,HEBREW,gregorianDFS);
		buf.append("]];\n");
		
		/*
		 * ERAS
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_EPOCHS = [\n")
		.append("//Christian Gregorian Calendar, two epochs -- religious form\n");
		
		//these match locale, eras match calendar type so they don't match up, going to 
		//emit eraNames as "--"
		//String eraNames[] = gregorianDFS.getEraNames();
		String eras[] = gregorianDFS.getEras();
		buf.append("[");
		tempArray = new String[2];
		for(int i = 0 ; i < eras.length; i++){
			buf.append("[");
			tempArray[0] = "--";
			tempArray[1] = eras[i];
			buf.append(Utils.appendStrings(tempArray))
			.append("]");
			if(eras.length != (i + 1)){
				buf.append(",");
			}
		}
		buf.append("],\n")
		
		.append("//Hijiri (Islamic) Calendar, one epoch -- the date of the haj\n");
		
		eras = islamicDFS.getEras();
		buf.append("[");
		//put in a blank like bills first
		tempArray[1] = "--";
		
		buf.append("[")
		.append(Utils.appendStrings(tempArray))
		.append("]");
		if(eras.length > 0){
			buf.append(",");
		}
		
		for(int i = 0 ; i < eras.length; i++){
			buf.append("[");
			tempArray[1] = eras[i];
			buf.append(Utils.appendStrings(tempArray))
			.append("]");
			if(eras.length != (i + 1)){
				buf.append(",");
			}
		}
		buf.append("],\n")
		
		.append("// Hebrew Calendar, one epoch -- the date of the creation of the world\n");
		
		eras = hebrewDFS.getEras();
		buf.append("[");
		//put in a blank like bills first
		tempArray[1] = "--";
		
		buf.append("[")
		.append(Utils.appendStrings(tempArray))
		.append("]");
		if(eras.length > 0){
			buf.append(",");
		}
		
		
		String enc_str = null;
		for(int i = 0 ; i < eras.length; i++){
			buf.append("[");
			//	hebrew character breaking script so encode string for this language code, it can be "iw" (deprecated) or "he"
			if(locale.getLanguage().equalsIgnoreCase("he") || locale.getLanguage().equalsIgnoreCase("iw")){
				enc_str = JavaScriptUtil.encodeScriptResourceString(eras[i], true);	
			} else {
				//just encode chars outside ascii range
				enc_str = JavaScriptUtil.encodeScriptResourceString(eras[i], false);
			}
			tempArray[1] = enc_str;
			//tempArray[1] = eras[i];
			buf.append(Utils.appendStrings(tempArray))
			.append("]");
			if(eras.length != (i + 1)){
				buf.append(",");
			}
		}
		buf.append("],\n")		
		

		.append("//Buddhist Gregorian Calendar, one epoch -- the birth of The Buddha\n");
		
		eras = buddhistDFS.getEras();
		buf.append("[");
		//put in a blank like bills first
		tempArray[1] = "--";
		
		buf.append("[");
		buf.append(Utils.appendStrings(tempArray))
		.append("]");
		if(eras.length > 0){
			buf.append(",");
		}
		
		for(int i = 0 ; i < eras.length; i++){
			buf.append("[");
			tempArray[1] = eras[i];
			buf.append(Utils.appendStrings(tempArray))
			.append("]");
			if(eras.length != (i + 1)){
				buf.append(",");
			}
		}
		buf.append("],\n")

		.append("//Japanese Gregorian Calendar, reigns of the emperors -- Meiji period and later\n");
		
		eras = japaneseDFS.getEras();
		buf.append("[");
		//do the latest 4
		for(int i = 4; i > 0; i--){
			buf.append("[");
			tempArray[1] = eras[eras.length - i];
			buf.append(Utils.appendStrings(tempArray))
			.append("]");
			if(i > 1){
				buf.append(",");
			}
		}
		buf.append("]\n];\n");
		
		/*
		 * DEFAULT AM AND PM SUFFIXES
		 */
		tempArray = gregorianDFS.getAmPmStrings();
		buf.append("//Default AM and PM suffixes\n")
		.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STR_AM_SUFFIX = \"")
		.append(tempArray[0])
		.append("\";\n")
		.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STR_PM_SUFFIX = \"")
		.append(tempArray[1])
		.append("\";\n");
		
		/*
		 * TIME ZONE SCHEMES
		 */
		//skip this for now - need to figure out a reliable way to do this for all
		//locale situations, at time of writing not possible in ICU
		
		/*
		 * NUMBERS AND CURRENCY
		 * 
		 * These are no longer used
		
		DecimalFormatSymbols dfs = new DecimalFormatSymbols(locale);
		buf.append("//Numbers and currency\n")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".STR_NUMBER_DECIMAL = \"")
		.append(dfs.getDecimalSeparator())
		.append("\";\n")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".STR_NUMBER_THOUSANDS = \"")
		.append(dfs.getGroupingSeparator())
		.append("\";\n")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".STR_NUMBER_CURRENCY = \"")
		.append(dfs.getCurrencySymbol())
		.append("\";\n\n");
		 */
		
		//return buf.toString();
		//or encode instead...
		return JavaScriptUtil.encodeScriptResourceString(buf.toString(), false);
	}
	
	private static void checkBeforeAppendStrings(StringBuffer buf, String[] values, byte type, byte calendar, DateFormatSymbols gregorianDFS){
		
		if(type > -1){
			String ERR = "ERR";
			String week[] = new String[]{ERR,ERR,ERR,ERR,ERR,ERR,ERR};
			String month[] = new String[]{ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR};
			/*
			 * Covering for ICU bug, the strings should be hebrew of islamic, but ICU gives 
			 * out locale strings in these cases.
			 */
			if(type == SHORT_MONTH){
				String gsm[] = gregorianDFS.getShortMonths();
				//check second and third strings in array
				if((gsm[1].equals(values[1]) && gsm[2].equals(values[2]))){
					//emit ERR strings for now - may decide later to provide defaults
					//13 months in hebrew calendar
					if(calendar == HEBREW){
						buf.append(Utils.appendStrings(new String[]{ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR,ERR}));
						
					} else {
						buf.append(Utils.appendStrings(month));
					}
				}
				else
				{
					buf.append(Utils.appendStrings(values));
				}
				
			} else if(type == DAY){
				String gsm[] = HxClientRenderUtil.removeEmptyStringArrayItems(gregorianDFS.getWeekdays());
				if(gsm[1].equals(values[1]) && gsm[2].equals(values[2])){
					buf.append(Utils.appendStrings(week));
				}
				
			} else if(type == SHORT_DAY){
				String gsm[] = HxClientRenderUtil.removeEmptyStringArrayItems(gregorianDFS.getShortWeekdays());
				if(gsm[1].equals(values[1]) && gsm[2].equals(values[2])){
					buf.append(Utils.appendStrings(week));
				}
				
			} else if(type == ABBREVIATED_DAY){
				String gsm[] = HxClientRenderUtil.removeEmptyStringArrayItems(gregorianDFS.getWeekdays(STANDALONE, NARROW));
				if(gsm[1].equals(values[1]) && gsm[2].equals(values[2])){
					buf.append(Utils.appendStrings(week));
				}
			}
		}

	}
}
