/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;


import java.io.InputStream;
import java.lang.reflect.Method;
import java.text.DateFormatSymbols;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;
import javax.faces.convert.DateTimeConverter;
import javax.faces.validator.Validator;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.UIDataIterator;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.converter.DateTimeConverterExICU;
import com.ibm.faces.converter.MaskConverter;
import com.ibm.faces.converter.NumberConverterEx;
import com.ibm.faces.converter.NumberConverterExICU;
import com.ibm.faces.util.AjaxUtil;
import com.ibm.faces.util.CacheHashMap;
import com.ibm.faces.util.ContextParams;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.faces.util.LifecycleUtil;
import com.ibm.faces.util.ScriptObject;
import com.ibm.faces.util.StringUtil;
import com.ibm.faces.validator.DateTimeRangeValidator;
import com.ibm.faces.validator.DoubleRangeValidatorEx;
import com.ibm.faces.validator.ExpressionValidator;
import com.ibm.faces.validator.LongRangeValidatorEx;
import com.ibm.faces.validator.ModulusSelfCheckValidator;



/**
 * @author Bob Gallagher
 *
 */
public class HxClientRenderUtil {
	public static final String HX_VERSION = "v3_0_4"; 
	public static final String HX_LIB_FILENAME = "hxclient_v3_0_4.js";
	public static final String HX_VAR_NAME = "hX_5";
	public static final String HXG_FUNCTION_NAME = "HxG_5";
	
	public static final String HX_STRING_RESOURCES = "com.ibm.faces.HX_STRING_RESOURCES";
	public static final String HX_VIEW_LOCALE = "viewLocale";
	
	//TODO BILL
	public static final String HX_CAL_MAGICNAME_RESOURCES = "_JWLCALSTRING";
	//TODO BILL
	public static final String HX_PORTAL_ENCODING = "com.ibm.faces.JWL.HX_PORTAL_ENCODING";
	
	public static final String DELIM_END = "com.ibm.faces.JWLAJAX.DELIMITER_END";
	public static final String DELIM_START = "com.ibm.faces.JWLAJAX.DELIMITER_START";
	
	private static final ClassLoader hxClientClassLoader = (new HxClientRenderUtil()).getClass().getClassLoader();

	// generate an id
	private static int idSeed = 0;
	private synchronized static String genScriptId(FacesContext context) {
		return context.getExternalContext().encodeNamespace(Integer.toHexString(idSeed++));
	}

	// Some maps to hold on to converters/validators we may already have generated
	private static CacheHashMap dateConverterJs = new CacheHashMap(25);
	private static CacheHashMap numberConverterJs = new CacheHashMap(25);
	private static CacheHashMap rangeValidatorJs = new CacheHashMap(25);
	private static CacheHashMap constraintValidatorJs = new CacheHashMap(25);
	//removed as part of a fix for cs bug #50729
	//private static CacheHashMap libraryInclude = new CacheHashMap(12);
	
	private static String numberStrictnessLevel = null;
	private static String dateStrictnessLevel = null;
	
	
	
	public static final String BODY_SCOPE = "#body"; //$NON-NLS-1$
	/**
	 * Builds the javascript statement(s) to generate the setResourceServer 
	 * call to init resources in hxclient.js. This could return null if this call is not
	 * required.
	 *  
	 * @param context FacesContext
	 * @return return null if is not need else the setResourceServer call to init resource in hxclient.js
	 */
	public static final String buildSetResourcePathCall(FacesContext context) {
		StringBuffer script = new StringBuffer();
		script.append("if (")
			.append(HX_VAR_NAME)
			.append(") ")
			.append(HX_VAR_NAME)
			.append(".setResourceServer(\"");
		//RATLC01083006
		script.append(LifecycleUtil.getResourceServletContextPath(context));
		
		script.append("\");\n"); //$NON-NLS-1$
		return script.toString();
	}

	public static final String buildLocaleCall(FacesContext context) {
		StringBuffer script = new StringBuffer();

		// if (hX_5 && hX_5.setLocale) hX_5.setLocale("en");
		script.append("if(").append(HX_VAR_NAME).append(" && ").append(HX_VAR_NAME).append(".setLocale) ").
		append(HX_VAR_NAME).append(".setLocale(\"").append(context.getViewRoot().getLocale()).append("\");\n");
		
		return script.toString();
	}

	/**
	 * Builds the javascript statement(s) to generate the setPortalEncoding 
	 * call to Turn on Portal code paths in hxclient js. This could return blank String if this call is not
	 * required.
	 *  
	 * @param context FacesContext
	 * @return return blank string if is not need else the setPortalEncoding call
	 */
	public static final String buildPortalFlag(FacesContext context) {
		
		boolean inportal = false;	
		
		String stored = (String) context.getExternalContext().getSessionMap().get(HX_PORTAL_ENCODING);
		
		if(stored != null && "true".equalsIgnoreCase(stored))
		{
			inportal = true;
		}
		else if(stored != null && "false".equalsIgnoreCase(stored))
		{
			inportal = false;
		}
		else if(stored == null)
		{
			Object req = context.getExternalContext().getRequest();
			
			inportal = isPortal(req);
			
			if(inportal)
				context.getExternalContext().getSessionMap().put(HX_PORTAL_ENCODING,"true");
			else
				context.getExternalContext().getSessionMap().put(HX_PORTAL_ENCODING,"false");
		}
		
		
		if(inportal)
		{
			StringBuffer script = new StringBuffer("\n"); //$NON-NLS-1$
			script.append("if (")
				.append(HX_VAR_NAME)
				.append(" && ")
				.append(HX_VAR_NAME)
				.append(".setPortalEncoding) ")
				.append(HX_VAR_NAME)
				.append(".setPortalEncoding(); // turns on Portal code paths for HxClient.\n");
			
			return script.toString();
		}
		return "";
	}	
	
	
	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * The javascript is cached to be available for requests for a converter configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same converter being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param converter DateTimeConverter
	 * @return ScriptObject containing the id and javascript of the converter.
	 */
	public synchronized static final ScriptObject buildJsMaskConverter(
		FacesContext context,
		MaskConverter maskConverter) {

		if (null == maskConverter) {
			return null;
		}
		String pattern = maskConverter.getMask();
		// let's see if we already have one
		// pandrews : with new functionality same pattern may not mean same script object,
		//may now have custom regex - taking this check out for now
		/*
		ScriptObject cachedScript = (ScriptObject) maskConverterJs.get(pattern);
		if (null != cachedScript) {
			return cachedScript;
		}
		*/
		// ok, build one

		String converterId = genScriptId(context);

		if (pattern != null && !pattern.equals("")) { //$NON-NLS-1$
			StringBuffer javascript = new StringBuffer(HX_VAR_NAME); //$NON-NLS-1$
			javascript.append(".addConverter(\"");
			javascript.append(converterId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(buildJsMaskConverter(maskConverter));
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(converterId, javascript.toString());
			//pandrews: see note above - abandoning storing mask converter script objects for now
			//maskConverterJs.put(pattern, so);
			return so;
		}
		return null;
	}
	/**
	 * Builds the javascript statement to create a new MaskConverter. 
	 *  
	 * @param MaskConverter
	 * @return the javascript new converter statement without trailing ';'
	 */
	public static final String buildJsMaskConverter(MaskConverter maskConverter) {
		if (null == maskConverter) {
			return null;
		}
		String pattern = maskConverter.getMask();
		if (pattern != null && !pattern.equals("")) { //$NON-NLS-1$
			StringBuffer javascript = new StringBuffer("new "); //$NON-NLS-1$
			javascript.append(HX_VAR_NAME);
			javascript.append(".MaskConverter(\"mask:");
			javascript.append(pattern)
			.append("\"");
			
			if(maskConverter.getProtectMask() != null){
				javascript.append(",\"protect-mask:")
				.append(maskConverter.getProtectMask())
				.append("\"");
			}
			if(maskConverter.getChar1RegEx() != null){
				javascript.append(",\"char1-regexpression:")
				.append(maskConverter.getChar1RegEx())
				.append("\"");
			}
			if(maskConverter.getChar2RegEx() != null){
				javascript.append(",\"char2-regexpression:")
				.append(maskConverter.getChar2RegEx())
				.append("\"");
			}
			if(maskConverter.getChar3RegEx() != null){
				javascript.append(",\"char3-regexpression:")
				.append(maskConverter.getChar3RegEx())
				.append("\"");
			}
			if(maskConverter.getChar4RegEx() != null){
				javascript.append(",\"char4-regexpression:")
				.append(maskConverter.getChar4RegEx())
				.append("\"");
			}
			javascript.append(")"); //$NON-NLS-1$
			return javascript.toString();
		}
		return null;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * The javascript is cached to be available for requests for a converter configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same converter being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param converter DateTimeConverter
	 * @return ScriptObject containing the id and javascript of the converter.
	 */
	public synchronized static final ScriptObject buildJsDateConverter(
		FacesContext context,
		DateTimeConverter datetimeConverter) {

		if (null == datetimeConverter) {
			datetimeConverter = defaultDateTimeConverter;
		}
		String converterKey = createDateConverterKey(datetimeConverter);
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) dateConverterJs.get(converterKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		// ok, build one
		
		setupStrictnessLevel(context);

		String converterId = genScriptId(context);
		String newConverter = buildJsDateTimeConverter(datetimeConverter);
		if (null != newConverter) {
			StringBuffer javascript = new StringBuffer(HX_VAR_NAME); //$NON-NLS-1$
			javascript.append(".addConverter(\"");
			javascript.append(converterId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(newConverter);
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(converterId, javascript.toString());
			dateConverterJs.put(converterKey, so);
			return so;
		}
		return null;
	}
	private static DateTimeConverter defaultDateTimeConverter = new DateTimeConverter();

	private static String createDateConverterKey(DateTimeConverter converter) {
		StringBuffer key = new StringBuffer(converter.getLocale().toString());
		if(converter instanceof DateTimeConverterExICU)
		{
			if(((DateTimeConverterExICU)converter).getCalendar() != null)
				key.append(((DateTimeConverterExICU)converter).getCalendar().getType());
		}
		key.append(converter.getType());
		key.append(converter.getPattern());
		key.append(converter.getDateStyle());
		key.append(converter.getTimeStyle());
		key.append(converter.getLocale());
		return key.toString();
	}
	/**
	 * Builds the javascript statement to create a new DateTimeConverter. 
	 *  
	 * @param datetimeConverter
	 * @return the javascript new converter statement without trailing ';'
	 */
	public static final String buildJsDateTimeConverter(DateTimeConverter datetimeConverter) {
		
		if(datetimeConverter instanceof DateTimeConverterExICU){
			return HxClientICU.buildJsDateTimeConverterICU((DateTimeConverterExICU)datetimeConverter);
		}
		
		String pattern = datetimeConverter.getPattern();
		String type = datetimeConverter.getType();
		String dateStyle = datetimeConverter.getDateStyle();
		String timeStyle = datetimeConverter.getTimeStyle();
		Locale locale = datetimeConverter.getLocale();
		String format = null;
				
		if (pattern != null && !pattern.equals("")) //$NON-NLS-1$
			format = pattern;
		else 
		{
			java.text.DateFormat df = null;
			if (type.equalsIgnoreCase("both")) { //$NON-NLS-1$
				df = java.text.DateFormat.getDateTimeInstance(Utils.getDateStyle(dateStyle), Utils.getDateStyle(timeStyle), locale);
			} else if (type.equalsIgnoreCase("date")) { //$NON-NLS-1$
				df = java.text.DateFormat.getDateInstance(Utils.getDateStyle(dateStyle), locale);
			} else if (type.equalsIgnoreCase("time")) { //$NON-NLS-1$
				df = java.text.DateFormat.getTimeInstance(Utils.getDateStyle(timeStyle), locale);
			} else {
				df = java.text.DateFormat.getDateInstance(Utils.getDateStyle(dateStyle), locale);
			}
			format = ((java.text.SimpleDateFormat) df).toPattern();
		}

		
		if (null != format) 
		{
			StringBuffer javascript = new StringBuffer("new " + HX_VAR_NAME + ".DateTimeConverter(\"strict:" + dateStrictnessLevel +"\", \"format:"); //$NON-NLS-1$ //$NON-NLS-2$
			javascript.append(format);
			if(!locale.toString().equals(FacesContext.getCurrentInstance().getViewRoot().getLocale().toString())) {
				javascript.append("\", \"locale:").append(locale);
			}
			javascript.append("\")"); //$NON-NLS-1$
			return javascript.toString();
		}
		return null;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * The javascript is cached to be available for requests for a converter configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same converter being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param converter NumberConverterEx
	 * @return ScriptObject containing the id and javascript of the converter.
	 */
	public synchronized static final ScriptObject buildJsNumberConverter(
		FacesContext context,
		NumberConverterEx numConverter) {

	/*	if (null == numConverter) {
			numConverter = defaultNumberConverter;
		}*/
		
		String numConverterKey = createNumberConverterKey(numConverter);
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) numberConverterJs.get(numConverterKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		// ok, build one
		
		setupStrictnessLevel(context);

		String converterId = genScriptId(context);

		StringBuffer javascript = new StringBuffer();		
		//TODO bill
		String newConverter = null;
		if(numConverter instanceof NumberConverterExICU)	// ICU4J way
			newConverter = HxClientICU.buildJsNumberConverterICU((NumberConverterExICU)numConverter);
		else  // normal way
			newConverter = buildJsNumberConverter(numConverter);

		//TODO bill
		if (null != newConverter && newConverter.length() > 0) {
			javascript.append(HX_VAR_NAME); //$NON-NLS-1$
			javascript.append(".addConverter(\"");
			javascript.append(converterId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(newConverter);
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(converterId, javascript.toString());
			numberConverterJs.put(numConverterKey, so);
			return so;
		}
		return null;
	}
	
	/*
	 * for JWL use only
	 * */
	private static NumberConverterEx defaultNumberConverter = new NumberConverterEx();	
	public static NumberConverterEx getDefaultNumberConverter()
	{
		return defaultNumberConverter;
	}
		
	private static String createNumberConverterKey(NumberConverterEx numConverter) {
		StringBuffer key = new StringBuffer(numConverter.getLocale().toString());
		key.append(numConverter.getPattern());
		//add ibm extension if exists
		if(numConverter.isPatternExtensionSet()){
			key.append(numConverter.getPatternExtension());
		}
		key.append(numConverter.getType());
		key.append(numConverter.isCurrencySymbolSet() ? numConverter.getCurrencySymbol() : null);
		key.append(numConverter.isCurrencyCodeSet() ? numConverter.getCurrencyCode() : null);
		key.append(numConverter.isGroupingUsed());
		key.append(numConverter.isIntegerOnly());
		key.append(
			numConverter.isMaxFractionDigitsSet() ? Integer.toString(numConverter.getMaxFractionDigits()) : null);
		key.append(
			numConverter.isMinFractionDigitsSet() ? Integer.toString(numConverter.getMinFractionDigits()) : null);
		key.append(numConverter.isMaxIntegerDigitsSet() ? Integer.toString(numConverter.getMaxIntegerDigits()) : null);
		key.append(numConverter.isMinIntegerDigitsSet() ? Integer.toString(numConverter.getMinIntegerDigits()) : null);
		return key.toString();
	}
	private static String createNumberValidatorKey(Object a, Object b, Object c, Object d, boolean required, String expression, String modulus) {
		StringBuffer key = new StringBuffer("n"); //$NON-NLS-1$
		key.append(a);
		key.append(b);
		key.append(c);
		key.append(d);
		key.append(required);
		key.append(expression);
		key.append(modulus);
		return key.toString();
	}
	private static String createDateTimeValidatorKey(Object a, Object b, Object c, Object d, boolean required, String expression) {
		StringBuffer key = new StringBuffer("d"); //$NON-NLS-1$
		key.append(a);
		key.append(b);
		key.append(c);
		key.append(d);
		key.append(required);
		key.append(expression);
		return key.toString();
	}
	private static String createStringValidatorKey(String constraint, String minLength, String maxLength, boolean required, String expression) {
		StringBuffer key = new StringBuffer();
		key.append(constraint);
		key.append(minLength);
		key.append(maxLength);
		key.append(required);
		key.append(expression);
		return key.toString();
	}
	/**
	 * @deprecated - use buildJsNumberConverter(NumberConverterEx)
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * It returns the id of the converter so it can be looked up in the converter map.
	 *  
	 * @param numConverter
	 * @param locale
	 * @return the javascript new converter statement
	 */
	public static final String buildJsNumberConverter(NumberConverterEx numConverter, Locale locale) {
		return buildJsNumberConverter(numConverter);
	}
	/**
	 * Builds the javascript statement to create a new NumberConverter. 
	 *  
	 * @param numConverter
	 * @param locale
	 * @return the javascript new converter statement without trailing ';'
	 */
	public static final String buildJsNumberConverter(NumberConverterEx numConverter) {

		String converterPattern = null;
		Locale locale = numConverter.getLocale();
		NumberFormat formatter = getNumberFormat(numConverter, locale);
		try {
			// Configure the formatter
			String pattern = numConverter.getPattern();
			String type = numConverter.getType();
			if (((pattern != null) && !pattern.equals("")) || Utils.NUMBERSTYLE_CURRENCY.equalsIgnoreCase(type)) { //$NON-NLS-1$
				configureCurrency(numConverter, formatter);
			}
			configureFormatter(numConverter, formatter);
			if (formatter instanceof DecimalFormat) {
				if(pattern != null && !pattern.equals("")){ //$NON-NLS-1$
					((DecimalFormat) formatter).applyPattern(pattern);
					converterPattern=pattern;
				} else {
					converterPattern = ((DecimalFormat) formatter).toPattern();	
				}
				
				//RATLC00670050 - output cannot handle the currency pattern character in japanese locale(there maybe others locales affected) 
				//replacing it with unicode string for the character
				while(converterPattern.indexOf("\u00A4")>-1){ //$NON-NLS-1$
					int pos=converterPattern.indexOf("\u00A4"); //$NON-NLS-1$
					StringBuffer b=new StringBuffer();
					b.append(converterPattern.substring(0,pos));
					b.append("\\u00A4"); //$NON-NLS-1$
					b.append(converterPattern.substring(pos+1,converterPattern.length()));
					converterPattern=b.toString();
				}
				
			} else {
				String formattedResult = formatter.format(new Double(111111111111111111.111111111111111111));
				converterPattern = formattedResult.replace('1', '#');
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		if (null != converterPattern && converterPattern.length() > 0) {
			StringBuffer javascript = new StringBuffer("new " + HX_VAR_NAME + ".NumberConverter(\"strict:" + numberStrictnessLevel + "\", \"pattern:"); //$NON-NLS-1$ //$NON-NLS-2$
			javascript.append(converterPattern);
			//append ibm pattern extension to pattern if set
			if(numConverter.isPatternExtensionSet()){
				javascript.append(numConverter.getPatternExtension());
			}
			javascript.append("\", \"locale:"); //$NON-NLS-1$
			//RATLC00976099 fix, client side validation not working correctly when currency symbol set
			//This section of code was not using the symbols from the configured formatter set up previously in this method 
			DecimalFormatSymbols symbols=null;
			if(formatter instanceof DecimalFormat){
				symbols=((DecimalFormat)formatter).getDecimalFormatSymbols();
			} else {
				symbols = new DecimalFormatSymbols(locale);
			}
			StringBuffer localeBuf = new StringBuffer();
			localeBuf.append(convertNbspToSpace(symbols.getGroupingSeparator()));
			localeBuf.append(convertNbspToSpace(symbols.getDecimalSeparator()));
			localeBuf.append(symbols.getPercent());
			localeBuf.append(symbols.getPerMill());
			localeBuf.append(symbols.getMinusSign());
			localeBuf.append(Utils.escapeBackslash(symbols.getCurrencySymbol()));
			
			//hebrew currency symbol contains a " so encode complete string for this language code, it can be "iw" (deprecated) or "he"
			Locale loc = FacesContext.getCurrentInstance().getViewRoot().getLocale();
			if(loc.getLanguage().equalsIgnoreCase("he") || loc.getLanguage().equalsIgnoreCase("iw")){
				javascript.append(JavaScriptUtil.encodeScriptResourceString(localeBuf.toString(), true));	
			} else {
				//just encode chars outside ascii range
				javascript.append(JavaScriptUtil.encodeScriptResourceString(localeBuf.toString(), false));
			}
			
			javascript.append("\")"); //$NON-NLS-1$		
			return javascript.toString();
		}
		return null;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a validator and register it. 
	 * The javascript is cached to be available for requests for a validator configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same validator being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param validator Range validator
	 * @return ScriptObject containing the id and javascript of the validator.
	 */
	public synchronized static final ScriptObject buildJsRangeValidator(FacesContext context, Converter converter, Validator validator, boolean required, ExpressionValidator expressionValidator, ModulusSelfCheckValidator modulusValidator) {

		if (converter instanceof DateTimeConverter || validator instanceof DateTimeRangeValidator) {
			return buildJsAddDateTimeValidator(context, (DateTimeRangeValidator) validator, required, expressionValidator);
		} else if ((converter instanceof NumberConverterEx) ||(validator instanceof LongRangeValidatorEx) || (validator instanceof DoubleRangeValidatorEx)) {
			return buildJsAddNumberValidator(context, validator, required, expressionValidator, modulusValidator);
		}
		//validator may be null, 
		return null;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a datetime validator and register it. 
	 * The javascript is cached to be available for requests for a validator configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same validator being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param validator Range validator
	 * @return ScriptObject containing the id and javascript of the validator.
	 */
	public synchronized static final ScriptObject buildJsAddDateTimeValidator(FacesContext context, DateTimeRangeValidator validator, boolean required, ExpressionValidator expressionValidator) {
		
		if(validator == null && required == false && expressionValidator == null){
			//validator not emitted
			return null;
		}
		
		Object minBound = null;
		Object maxBound = null;
	
		String validatorKey = null;
		String validatorId = null;
		String newValidator = null;
	
		String expression = null;
		if(expressionValidator != null) {
			expression = JavaScriptUtil.convertEL2JavaScript(expressionValidator.getExpression());
		}

		validatorKey =
			createDateTimeValidatorKey(
				null == validator ? null:(validator).getMinimum(),
				null == validator ? null:(validator).getMaximum(),
				minBound,
				maxBound,
				required,
				expression);
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) rangeValidatorJs.get(validatorKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		newValidator =
			buildJsDateRangeValidator(validator, (Date) minBound, (Date) maxBound, required, expression);
		validatorId = genScriptId(context);
		if (null != newValidator) {
			StringBuffer javascript = new StringBuffer(HX_VAR_NAME); //$NON-NLS-1$
			javascript.append(".addValidator(\"");
			javascript.append(validatorId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(newValidator);
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(validatorId, javascript.toString());
			rangeValidatorJs.put(validatorKey, so);
			return so;
		}
		return null;
	}


	/**
	 * Builds the javascript statement(s) to instanciate a number validator and register it. 
	 * The javascript is cached to be available for requests for a validator configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same validator being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param validator Range validator
	 * @return ScriptObject containing the id and javascript of the validator.
	 */
	public synchronized static final ScriptObject buildJsAddNumberValidator(FacesContext context, Validator validator, boolean required, ExpressionValidator expressionValidator, ModulusSelfCheckValidator modulusValidator) {
		
		if(validator == null && required == false && expressionValidator == null && modulusValidator == null){
			//don't emit validator script
			return null;
		}
		Object minBound = null;
		Object maxBound = null;

		String validatorKey = null;
		String validatorId = null;
		String newValidator = null;

		String expression = null;
		String modulus = null;
		if(expressionValidator != null) {
			expression = JavaScriptUtil.convertEL2JavaScript(expressionValidator.getExpression());
		}
		if(modulusValidator != null){
			modulus = modulusValidator.getModulus();
		}

		if ((validator instanceof LongRangeValidatorEx) || (validator instanceof DoubleRangeValidatorEx)) {
			if (validator instanceof LongRangeValidatorEx) {
				validatorKey =
					createNumberValidatorKey(
						null == validator ? null:new Long(((LongRangeValidatorEx) validator).getMinimum()),
						null == validator ? null:new Long(((LongRangeValidatorEx) validator).getMaximum()),
						minBound,
						maxBound,
						required,
						expression,
						modulus);
			} else {
				validatorKey =
					createNumberValidatorKey(
						null == validator ? null:new Double(((DoubleRangeValidatorEx) validator).getMinimum()),
						null == validator ? null:new Double(((DoubleRangeValidatorEx) validator).getMaximum()),
						minBound,
						maxBound,
						required,
						expression,
						modulus);
			}
		} else {
			validatorKey =
				createNumberValidatorKey(
					null,
					null,
					null,
					null,
					required,
					expression,
					modulus);
		}
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) rangeValidatorJs.get(validatorKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		newValidator = buildJsNumberRangeValidator(validator, (Number) minBound, (Number) maxBound, required, expression, modulus);
		validatorId = genScriptId(context);
		if (null != newValidator) {
			StringBuffer javascript = new StringBuffer(HX_VAR_NAME); //$NON-NLS-1$
			javascript.append(".addValidator(\"");
			javascript.append(validatorId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(newValidator);
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(validatorId, javascript.toString());
			rangeValidatorJs.put(validatorKey, so);
			return so;
		}
		return null;
	}

	public static final Validator getRangeValidator(UIInput input) {
		// Check if the component has a validation tag
		Validator validators[] = input.getValidators();
		for (int i = 0; i < validators.length; i++) {
			Validator validator = validators[i];
			if ((validator instanceof DateTimeRangeValidator)
				|| (validator instanceof LongRangeValidatorEx)
				|| (validator instanceof DoubleRangeValidatorEx)) {
				return validator;
			}
		}
		return null;
	}

	public static final Validator getNumberRangeValidator(UIInput input) {
		// Check if the component has a validation tag
		Validator validators[] = input.getValidators();
		for (int i = 0; i < validators.length; i++) {
			Validator validator = validators[i];
			if ((validator instanceof LongRangeValidatorEx)
				|| (validator instanceof DoubleRangeValidatorEx)) {
				return validator;
			}
		}
		return null;
	}

	public static final DateTimeRangeValidator getDateRangeValidator(UIInput input) {
		// Check if the component has a validation tag
		Validator validators[] = input.getValidators();
		for (int i = 0; i < validators.length; i++) {
			Validator validator = validators[i];
			if (validator instanceof DateTimeRangeValidator) {
				return (DateTimeRangeValidator)validator;
			}
		}
		return null;
	}
	
	public static final ExpressionValidator getExpressionValidator(UIInput input) {
		// Check if the component has a validation tag
		Validator validators[] = input.getValidators();
		for (int i = 0; i < validators.length; i++) {
			Validator validator = validators[i];
			if (validator instanceof ExpressionValidator) {
				return (ExpressionValidator)validator;
			}
		}
		return null;
	}
	
	public static final ModulusSelfCheckValidator getModulusValidator(UIInput input) {
		// Check if the component has a validation tag
		Validator validators[] = input.getValidators();
		for (int i = 0; i < validators.length; i++) {
			Validator validator = validators[i];
			if (validator instanceof ModulusSelfCheckValidator) {
				return (ModulusSelfCheckValidator)validator;
			}
		}
		return null;
	}

	

	/**
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsNumberRangeValidator(Validator validator) {
		return buildJsNumberRangeValidator(validator, null, null, false, null, null);
	}

	/**
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsNumberRangeValidator(Validator validator, Number minBound, Number maxBound,boolean required,String expression, String modulus) {

		StringBuffer javascript = new StringBuffer();
		if (validator instanceof LongRangeValidatorEx) {
			LongRangeValidatorEx longValidator = (LongRangeValidatorEx) validator;
			if (null == minBound && longValidator.isMinimumSet()) {
				minBound = new Long(longValidator.getMinimum());
			}
			if (null == maxBound && longValidator.isMaximumSet()) {
				maxBound = new Long(longValidator.getMaximum());
			}
		}
		if (validator instanceof DoubleRangeValidatorEx) {
			DoubleRangeValidatorEx doubleValidator = (DoubleRangeValidatorEx) validator;
			if (null == minBound && doubleValidator.isMinimumSet()) {
				minBound = new Double(doubleValidator.getMinimum());
			}
			if (null == maxBound && doubleValidator.isMaximumSet()) {
				maxBound = new Double(doubleValidator.getMaximum());
			}

		}
		javascript.append("new "); //$NON-NLS-1$
		javascript.append(HX_VAR_NAME);
		javascript.append(".NumberValidator(");
		int nArgs = 0;
		if (minBound != null) {
			javascript.append(" \"min-bound:"); //$NON-NLS-1$
			javascript.append(minBound);
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
			nArgs++;
		}
		if (maxBound != null) {
			if (nArgs++ > 0) {
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"max-bound:"); //$NON-NLS-1$
			javascript.append(maxBound);
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if(required){
			if (nArgs++ > 0){
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"required:true\""); //$NON-NLS-1$
		}
		if(expression != null) {
			if (nArgs++ > 0){
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"constraint-expression:"); //$NON-NLS-1$
			javascript.append(expression);
			javascript.append('\"'); //$NON-NLS-1$ 
		}
		if(modulus != null) {
			if (nArgs++ > 0){
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"modulus:"); //$NON-NLS-1$
			javascript.append(modulus);
			javascript.append('\"'); //$NON-NLS-1$ 
		}
		
		javascript.append(")"); //$NON-NLS-1$		
		return javascript.toString();
	}
	/**
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsDateRangeValidator(DateTimeRangeValidator validator) {
		return buildJsDateRangeValidator(validator, null, null,false, null);
	}

	/**
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsDateRangeValidator(
		DateTimeRangeValidator validator,
		Date minBound,
		Date maxBound,
		boolean required,
		String expression) {

		StringBuffer javascript = new StringBuffer();
		if (null != validator){
			if (null == minBound) {
				minBound = validator.getMinimum(); // override component's own settings
			}
			if (null == maxBound) {
				maxBound = validator.getMaximum(); // override component's own settings
			}
		}
		javascript.append("new "); //$NON-NLS-1$
		javascript.append(HX_VAR_NAME);
		javascript.append(".DateTimeValidator(");
		SimpleDateFormat sd = new SimpleDateFormat("yyyyMMddHHmmss"); //$NON-NLS-1$
		int nArgs = 0;
		if (minBound != null) {
			javascript.append(" \"min-bound:"); //$NON-NLS-1$
			javascript.append(sd.format(minBound));
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
			nArgs++;
		}
		if (maxBound != null) {
			if (nArgs++ > 0) {
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"max-bound:"); //$NON-NLS-1$
			javascript.append(sd.format(maxBound));
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if(required){
			if (nArgs++ > 0){
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"required:true\""); //$NON-NLS-1$
		}
		if(expression != null) {
			if (nArgs++ > 0){
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"constraint-expression:"); //$NON-NLS-1$
			javascript.append(expression);
			javascript.append('\"'); //$NON-NLS-1$ 
		}

		javascript.append(")"); //$NON-NLS-1$	
		return javascript.toString();
	}

	/**
	 * <p>Return a <code>NumberFormat</code> instance to use for formatting
	 * and parsing in this {@link Converter}.</p>
	 *          
	 * @param locale The <code>Locale</code> used to select formatting
	 *  and parsing conventions
	 *
	 * @exception ConverterException if no instance can be created
	 */
	static private NumberFormat getNumberFormat(NumberConverterEx numConverter, Locale locale) {
		NumberFormat formatter = null;
		String pattern = numConverter.getPattern();
		String type = numConverter.getType();
		if (pattern == null && type == null) {
			throw new IllegalArgumentException("Either pattern or type must" + " be specified."); //$NON-NLS-1$ //$NON-NLS-2$
		}


		// If pattern is specified, type is ignored
		if (pattern != null) {
			DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
			formatter = new DecimalFormat(pattern, symbols);
		}

		// Create an instance based on the specified type
		else if (type.equalsIgnoreCase(Utils.NUMBERSTYLE_CURRENCY)) {
			formatter = NumberFormat.getCurrencyInstance(locale);
		} else if (type.equalsIgnoreCase(Utils.NUMBERSTYLE_NUMBER)) {
			formatter = NumberFormat.getNumberInstance(locale);
		} else if (type.equalsIgnoreCase(Utils.NUMBERSTYLE_PERCENT)) {
			formatter = NumberFormat.getPercentInstance(locale);
		} else {
			throw new ConverterException(new IllegalArgumentException(type));
		}
		if (numConverter.isIntegerOnly()) {
			formatter.setMaximumFractionDigits(0);
		}
		return formatter;
	}
	
	// RATLC01133770 user cannot easily type an nbsp so convert and expect converter to handle
	private static char convertNbspToSpace(char c){
 		// if input then we want to change the thousands separator from an nbsp to space
		return (c == '\u00A0') ? ' ' : c;
	}
	
	private static Boolean curClassAvailable = null;
	private static Class curClass = null;
	private static Class[] stringArg = new Class[]{String.class};
	
	private static void configureCurrency(NumberConverterEx numConverter, NumberFormat formatter) throws Exception {
		//if code or symbol not set do nothing
		if(numConverter.getCurrencyCode() == null && numConverter.getCurrencySymbol() == null){
			return;
		}
		if(curClassAvailable == null) {
				try {
					curClass = Class.forName("java.util.Currency");
					curClassAvailable = Boolean.TRUE;
				} catch (ClassNotFoundException e) {
					//jre version < 1.4
					curClassAvailable = Boolean.FALSE;
				}
		}
		
		//if currencycode is set and currency class is available
		if(numConverter.getCurrencyCode() != null && curClassAvailable.booleanValue()){
			
			Method method = curClass.getMethod("getInstance",stringArg);
			Object cur = method.invoke(null, new Object[]{numConverter.getCurrencyCode()});
			
			Class numFormatClass = Class.forName("java.text.NumberFormat");
			method = numFormatClass.getMethod("setCurrency",new Class[]{curClass});
			method.invoke(formatter, new Object[]{cur});
			return;
		}
		String curSymbol = null;
		
		if (numConverter.getCurrencySymbol() != null){
			curSymbol = numConverter.getCurrencySymbol();
		} else if (numConverter.getCurrencyCode() != null){
			curSymbol = numConverter.getCurrencyCode();
			
		}

		if (curSymbol != null){
			DecimalFormat df = (DecimalFormat) formatter;
			DecimalFormatSymbols dfs = df.getDecimalFormatSymbols();
			dfs.setCurrencySymbol(curSymbol);
			df.setDecimalFormatSymbols(dfs);
		}
	}

	
	/**
	 * <p>Configure the specified <code>NumberFormat</code> based on the
	 * formatting properties that have been set.</p>
	 *
	 * @param formatter The <code>NumberFormat</code> instance to configure
	 */
	private static void configureFormatter(NumberConverterEx numConverter, NumberFormat formatter) {

		formatter.setGroupingUsed(numConverter.isGroupingUsed());
		if (numConverter.isMaxIntegerDigitsSet()) {
			formatter.setMaximumIntegerDigits(numConverter.getMaxIntegerDigits());
		}
		if (numConverter.isMinIntegerDigitsSet()) {
			formatter.setMinimumIntegerDigits(numConverter.getMinIntegerDigits());
		}
		if (numConverter.isMaxFractionDigitsSet()) {
			formatter.setMaximumFractionDigits(numConverter.getMaxFractionDigits());
		}
		if (numConverter.isMinFractionDigitsSet()) {
			formatter.setMinimumFractionDigits(numConverter.getMinFractionDigits());
		}

	}

	/**
	 * @param component
	 * @param context
	 */
	public static void initJSLibraries(UIComponent component, FacesContext context) {
		initJSLibraries(component, context, true);
	}

	/**
	 * @deprecated use initJSLibraries with version
	 * @param component
	 * @param context
	 * @param bWriteNow
	 */
	public static void initJSLibraries(UIComponent component, FacesContext context, boolean bWriteNow) {
		
		//Check if this is a faces porject. If it is, check if it is portal.
		//If it is portal then add the viewId to the Apllication Map so that we
		//can encode variables at any point in the lifecycle.
		if(StringUtil.checkEncoding())
		{
			boolean inPortal = isPortal(context.getExternalContext().getRequest());
			if(inPortal)
			{
				context.getExternalContext().getApplicationMap().put("PortalEncoding", context.getExternalContext().encodeNamespace(""));	
			}
		}
		
		if(!(AjaxUtil.isAjaxRequest(context)))
		{
			bWriteNow = true;  //need this always to be true for the js files to be output in the correct order
			UIScriptCollector scriptCollector = null;
			if (component instanceof UIScriptCollector) {
				scriptCollector = (UIScriptCollector) component;
			} else {
				scriptCollector = UIScriptCollector.find(component);
			}
			
			if(scriptCollector != null)
			{
				// Include the strings and libs as libraries, not as a block of text
				/*moved so that it is output after hxclient_core to avoid js errors.
				StringBuffer strbuf = new StringBuffer();
				strbuf.append(LifecycleUtil.getContextPath(context))
					  .append("/.")
					  .append(ResourceHandler.buildLocalizedLibName(ResourceHandler.getBundle(context), "ibmjsfres/hxclient_S_" + HX_VERSION + ".js")); //$NON-NLS-1$ //$NON-NLS-2$
				scriptCollector.addLibrary(strbuf.toString());*/
			
				/*
				 *Get hxclient additional resource string from runtime
				 *Compile once and store on application map 
				 */
				Locale locale = context.getViewRoot().getLocale();
				//check if already compiled and saved on app map
				Map resMap = (Map)context.getExternalContext().getApplicationMap();
				
				generateStringResources(locale, resMap);			

				//	Include the rest of the stuff do it here. so we know that the strings should be available
				
				ScriptObject script = buildLibraryIncludeBlock(context);
				
				if (bWriteNow) {
					scriptCollector.writeLibraryReference(context, script.getId(), script.getScript());
				} else {
					scriptCollector.addLibrary(script.getId(), script.getScript()); 
				}
		
				//rebuild prototype in portal after includes
				String useFullHxVersion = context.getExternalContext().getInitParameter(ContextParams.USE_HXCLIENT_FULL);
				if(useFullHxVersion == null || useFullHxVersion.equalsIgnoreCase("false"))
				{
					Object req = context.getExternalContext().getRequest();			
					if(isPortal(req))
						scriptCollector.addScriptOnce("if (hX_5) hX_5.AJAX_prototypeloader();", UIScriptCollector.ADD_AT_TOP);
				}	
				
				//Now called for both HX and ODC components. 
				//Needed for ODC stuff to attach onPageUnload for memory cleanup
				scriptCollector.addScriptOnce(HX_VAR_NAME + ".onPageLoad();", UIScriptCollector.ADD_AT_VERY_BOTTOM);
			}
			else
			{
				ResourceBundle rb = ResourceHandler.getBundle(context);	  
				throw new FacesException(ResourceHandler.getString(rb, "NoScriptCollectorFoundError"));
			}
		}

	}
	
	/*
	 * <p>Resets the stored hx client string resources for a particular locale, forcing them to be regenerated. If locale is null
	 * all stored locale strings are reset.</p>
	 */
	public static void resetHxClientStringResources(FacesContext context, Locale l){
		Map resMap = (Map)context.getExternalContext().getApplicationMap().get(HX_STRING_RESOURCES);
		if(resMap != null){
			if(l != null){
				resMap.put(l.toString(),null);
			} else {
				resMap.clear();
			}
			
		}
		
	}
	
	/*
	 * DateFormatSymbols set the first week array item as a blank, also some locales return extra blank items - this method removes blanks from the array
	 */
	protected static String[] removeEmptyStringArrayItems(String[] array){
		
		ArrayList list = new ArrayList();
		for(int i = 0 ; i < array.length; i++){
			if(array[i] != null && !array[i].equals("") ){
				list.add(array[i]);
			}
		}
		
		String newarray[] = new String[list.size()];
		list.toArray(newarray);
		
		return newarray;
	}
	
	/*
	 * This call builds the icu4j or java.text version of hte for the specfied locale and places it 
	 *  a hashmap held in the applicationMap
	 * */
	public static String generateStringResources(Locale locale , Map applicationMap)
	{
		/*
		 *Get hxclient additional resource string from runtime
		 *Compile once and store on application map 
		 */
		
		//check if already compiled and saved on app map
		Map resMap = null;
		if(applicationMap != null)
			resMap = (Map)applicationMap.get(HX_STRING_RESOURCES);
		String localeRes = null;
		if(resMap != null){
			localeRes = (String)resMap.get(locale.toString());
		}
		
		 
		
		if(localeRes == null){
			//store resources not found, need to compile them
			
			Boolean useICU = Boolean.FALSE;
			//are we using ICU
			try {
				Class d = Class.forName("com.ibm.icu.text.DateFormatSymbols");						
				//	no ICU version 4.3.1 // fix for portal 6.0 xfaces
				Class[] dp = new Class[] {int.class, int.class};
				Method f = d.getMethod("getWeekdays",dp);					
				if(f != null)
					//	no ICU version 4.3.1 // fix for portal 6.0 xfaces
					useICU = Boolean.TRUE;
			} catch (ClassNotFoundException e){
				//no ICU
			}catch (NoSuchMethodException e){
				//no ICU version 4.3.1 // fix for portal 6.0 xfaces
			}
			
			if(useICU.booleanValue()){
				localeRes = HxClientICU.compileHxICUResources(locale);
			} else {
				localeRes = compileHxResources(locale);
			}

			String localeArrayIndex = ".L[\"" + locale + "\"]";
			
			String library = "ibmjsfres/hxclient_S_" + HX_VERSION + ".js";
			library = ResourceHandler.buildLocalizedLibName(library, locale);
			String strings = "";
			InputStream stream = hxClientClassLoader.getResourceAsStream(library);
			if (stream != null) {
				byte[] buffer = new byte[10000];
				byte[] bytes = new byte[0];
				int read = 0;
				try {
					while ((read = stream.read(buffer)) > 0) {
						int newSize = bytes.length + read;
						byte[] newBytes = new byte[newSize];
						System.arraycopy(bytes, 0, newBytes, 0, bytes.length);
						System.arraycopy(buffer, 0, newBytes, bytes.length, read);
						bytes = newBytes;
					}
					stream.close();
				} catch (Exception e) {
					e.printStackTrace();
				}
				strings = new String(bytes);
			}
			strings = strings.replaceAll(HxClientRenderUtil.HX_VAR_NAME, HxClientRenderUtil.HX_VAR_NAME + localeArrayIndex);

			StringBuffer buf = new StringBuffer();
			// Put a special code string (new in JWL 302 in front of all other strings
			// 	if(hX_5.L["en"]==null)hX_5.L["en"]=new Object();
			buf.append("if(").
			append(HxClientRenderUtil.HX_VAR_NAME).append(localeArrayIndex).
			append("==null)").
			append(HxClientRenderUtil.HX_VAR_NAME).append(localeArrayIndex).
			append("=new Object();\n");

			buf.append(strings);
			buf.append(localeRes);

			localeRes = buf.toString();

			//store localeRes on app map
			if(resMap == null){
				resMap = new HashMap();
				if(applicationMap != null)
					applicationMap.put(HX_STRING_RESOURCES,resMap);
			}
			resMap.put(locale.toString(),localeRes);
		}
		
		return localeRes;
	}
	
	
	
	/*
	 *Returns a compilation of hx resource strings from java.text formatters. 
	 */
	private static String compileHxResources(Locale locale){
		
		StringBuffer buf = new StringBuffer();
		
		DateFormatSymbols dfs = new DateFormatSymbols(locale);
		
		String err = "ERR";
		String errWeek[] = new String[]{err,err,err,err,err,err,err};
		String errMonth[] = new String[]{err,err,err,err,err,err,err,err,err,err,err,err};
		String errHebrewMonth[] = new String[]{err,err,err,err,err,err,err,err,err,err,err,err,err};
		
		String tempArray[] = null;
		
		/*
		 * LONG MONTH NAMES
		 */
		buf.append("\n//Note : Non-Gregorian calendar strings cannot be retrieved as ICU4J library is not available to this project.\n//Strings here retrieved from java.text.DateFormatSymbols in JRE\n");
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_MONTHS = \n")
		.append("//Gregorian...\n");
		//valueArray = new String[]{gCal.get(gCal.JANUARY),};
		buf.append("[[")
		.append(Utils.appendStrings(removeEmptyStringArrayItems(dfs.getMonths())))
		.append("],\n");
		
		buf.append("//Islamic...\n")
		.append("[")
		.append(Utils.appendStrings(errMonth))
		.append("],\n");
		
		buf.append("//Hebrew...\n")
		.append("[")
		.append(Utils.appendStrings(errHebrewMonth))
		.append("]];\n");
		
		/*
		 * SHORT MONTH NAMES
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_SHMONTHS = \n")
		.append("//Gregorian...\n");
		buf.append("[[")
		.append(Utils.appendStrings(removeEmptyStringArrayItems(dfs.getShortMonths())))
		.append("],\n");
		
		buf.append("//Islamic...\n")
		.append("[")
		.append(Utils.appendStrings(errMonth))
		.append("],\n");
		
		buf.append("//Hebrew...\n")
		.append("[")
		.append(Utils.appendStrings(errHebrewMonth))
		.append("]];\n");
		
		/*
		 * LONG DAYS OF THE WEEK
		 * (currenlty icu4j returns 8 element in the array, first one empty)
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_DAYS = \n")
		.append("//Gregorian...\n");
		buf.append("[[")
		//.append(Utils.appendStrings(removeWeekdayBlank(gregorianDFS.getWeekdays())))
		.append(Utils.appendStrings(removeEmptyStringArrayItems(dfs.getWeekdays())))
		.append("],\n");
		
		buf.append("//Islamic...\n")
		.append("[")
		.append(Utils.appendStrings(errWeek))
		.append("],\n");
		
		buf.append("//Hebrew...\n")
		.append("[")
		.append(Utils.appendStrings(errWeek))
		.append("]];\n");
		
		/*
		 * SHORT DAYS OF THE WEEK
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_SHDAYS = \n")
		.append("//Gregorian...\n");
		buf.append("[[")
		.append(Utils.appendStrings(removeEmptyStringArrayItems(dfs.getShortWeekdays())))
		.append("],\n");
		
		buf.append("//Islamic...\n")
		.append("[")
		.append(Utils.appendStrings(errWeek))
		.append("],\n");
		
		buf.append("//Hebrew...\n")
		.append("[")
		.append(Utils.appendStrings(errWeek))
		.append("]];\n");
		
		/*
		 * ABBREVIATED DAYS OF THE WEEK
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_SHSHDAYS = \n")
		.append("//Gregorian...\n");
		buf.append("[[");
		//.append(Utils.appendStrings(removeWeekdayBlank(gregorianDFS.getWeekdays(DateFormatSymbols.STANDALONE, DateFormatSymbols.NARROW))))
		String days[] = removeEmptyStringArrayItems(dfs.getShortWeekdays());
		for (int i= 0 ; i < days.length; i++){
			days[i] = days[i].substring(0,1);
		}
		buf.append(Utils.appendStrings(days))
		.append("],\n");
		
		buf.append("//Islamic...\n")
		.append("[")
		.append(Utils.appendStrings(errWeek))
		.append("],\n");
		
		buf.append("//Hebrew...\n")
		.append("[")
		.append(Utils.appendStrings(errWeek))
		.append("]];\n");
		
		/*
		 * ERAS
		 */
		buf.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STRA_DATE_EPOCHS = [\n")
		.append("//Christian Gregorian Calendar, two epochs -- religious form\n");
		
		//these match locale, eras match calendar type so they don't match up, going to 
		//emit eraNames as "--"
		//String eraNames[] = gregorianDFS.getEraNames();
		String eras[] = dfs.getEras();
		buf.append("[");
		tempArray = new String[2];
		for(int i = 0 ; i < eras.length; i++){
			buf.append("[");
			tempArray[0] = "--";
			tempArray[1] = eras[i];
			buf.append(Utils.appendStrings(tempArray))
			.append("]");
			if(eras.length != (i + 1)){
				buf.append(",");
			}
		}
		buf.append("],\n")
		
		.append("//Hijiri (Islamic) Calendar, one epoch -- the date of the haj\n");
		
		//eras = islamicDFS.getEras();
		//dummy
		eras = new String[]{err};
		buf.append("[");
		//put in a blank like bills first
		tempArray[1] = "--";
		
		buf.append("[")
		.append(Utils.appendStrings(tempArray))
		.append("]");
		if(eras.length > 0){
			buf.append(",");
		}
		
		for(int i = 0 ; i < eras.length; i++){
			buf.append("[");
			tempArray[1] = eras[i];
			buf.append(Utils.appendStrings(tempArray))
			.append("]");
			if(eras.length != (i + 1)){
				buf.append(",");
			}
		}
		buf.append("],\n")

		.append("//Buddhist Gregorian Calendar, one epoch -- the birth of The Buddha\n");
		
		//eras already set to dummy
		//eras = buddhistDFS.getEras();
		
		buf.append("[");
		//put in a blank like bills first
		tempArray[1] = "--";
		
		buf.append("[");
		buf.append(Utils.appendStrings(tempArray))
		.append("]");
		if(eras.length > 0){
			buf.append(",");
		}
		
		for(int i = 0 ; i < eras.length; i++){
			buf.append("[");
			tempArray[1] = eras[i];
			buf.append(Utils.appendStrings(tempArray))
			.append("]");
			if(eras.length != (i + 1)){
				buf.append(",");
			}
		}
		buf.append("],\n")

		.append("//Japanese Gregorian Calendar, reigns of the emperors -- Meiji period and later\n");
		
		//eras already set to dummy
		//eras = japaneseDFS.getEras();
		buf.append("[");
		/*
		//do the latest 4
		for(int i = 4; i > 0; i--){
			buf.append("[");
			tempArray[1] = eras[eras.length - i];
			buf.append(Utils.appendStrings(tempArray))
			.append("]");
			if(i > 1){
				buf.append(",");
			}
		}
		*/
		for(int i = 0 ; i < eras.length; i++){
			buf.append("[");
			tempArray[1] = eras[i];
			buf.append(Utils.appendStrings(tempArray))
			.append("]");
			if(eras.length != (i + 1)){
				buf.append(",");
			}
		}
		buf.append("]\n];\n");
		
		/*
		 * DEFAULT AM AND PM SUFFIXES
		 */
		tempArray = dfs.getAmPmStrings();
		buf.append("//Default AM and PM suffixes\n")
		.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STR_AM_SUFFIX = \"")
		.append(tempArray[0])
		.append("\";\n")
		.append(HxClientRenderUtil.HX_VAR_NAME).append(".L[\"").append(locale).append("\"]")
		.append(".STR_PM_SUFFIX = \"")
		.append(tempArray[1])
		.append("\";\n");
		
		/*
		 * TIME ZONE SCHEMES
		 */
		//skip this for now - need to figure out a reliable way to do this for all
		//locale situations, at time of writing not possible in ICU
		
		/*
		 * NUMBERS AND CURRENCY
		 * 
		 * These are no longer used
		
		DecimalFormatSymbols dfs = new DecimalFormatSymbols(locale);
		buf.append("//Numbers and currency\n")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".STR_NUMBER_DECIMAL = \"")
		.append(dfs.getDecimalSeparator())
		.append("\";\n")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".STR_NUMBER_THOUSANDS = \"")
		.append(dfs.getGroupingSeparator())
		.append("\";\n")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".STR_NUMBER_CURRENCY = \"")
		.append(dfs.getCurrencySymbol())
		.append("\";\n\n");
		 */

		//return buf.toString();
		//or encode instead...
		return JavaScriptUtil.encodeScriptResourceString(buf.toString(), false);
	}
	
	/**
	 * Build hxclient library include block
	 * @param context
	 * @param version
	 * @return ScriptObject where the Id is the library name and the script is the library include block
	 * which includes the script file include and string definitions and call to resource init.
	 */
	public synchronized static ScriptObject buildLibraryIncludeBlock(FacesContext context) {

		Locale locale = context.getViewRoot().getLocale();		
		/* Removed as part of a fix for cs bug #50729
		ScriptObject script = (ScriptObject)libraryInclude.get(libKey);
		if (null != script){
			return script;
		}
		*/
		/* Don't include the scripts any more
		ResourceBundle bundle = ResourceHandler.getBundle(context);
		String libStrings = ResourceHandler.buildLocalizedLibName(bundle, "ibmjsfres/hxclient_S_" + HX_VERSION + ".js"); //$NON-NLS-1$ //$NON-NLS-2$
	
		String strings = null;
		InputStream stream = hxClientClassLoader.getResourceAsStream(libStrings);
		if (stream != null) {
			byte[] buffer = new byte[10000];
			byte[] bytes = new byte[0];
			int read = 0;
			try {
				while ((read = stream.read(buffer)) > 0) {
					int newSize = bytes.length + read;
					byte[] newBytes = new byte[newSize];
					System.arraycopy(bytes, 0, newBytes, 0, bytes.length);
					System.arraycopy(buffer, 0, newBytes, bytes.length, read);
					bytes = newBytes;
				}
				stream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}

			try {
				String encoding = bundle.getString("Locale.encoding"); //$NON-NLS-1$
				if (encoding == null || encoding.trim().length() == 0 || encoding.equalsIgnoreCase("!Locale.encoding!")) { //$NON-NLS-1$
					encoding = "ISO-8859-1"; //$NON-NLS-1$
				}
				strings = new String(bytes, encoding);
			} catch (UnsupportedEncodingException e1) {
				// Shouldn't happen!
				e1.printStackTrace();
				strings = new String(bytes);
			}
		}
		*/	
		

		StringBuffer buffer = new StringBuffer();
		String theEncodedURL;
		String useFullHxVersion = context.getExternalContext().getInitParameter(ContextParams.USE_HXCLIENT_FULL);
		String theContextPath = LifecycleUtil.getResourceServletContextPath(context);
		String rebuild_proto_portal = "";
		
		if(useFullHxVersion != null && useFullHxVersion.equalsIgnoreCase("true"))
		{
			theEncodedURL = context.getExternalContext().encodeResourceURL(theContextPath + "/" + HX_LIB_FILENAME);
		}
		else
		{
			theEncodedURL = context.getExternalContext().encodeResourceURL(theContextPath + "/" + Utils.versionizeFilename("hxclient_core.js"));
			
			//rebuild prototype after includes in portal
			Object req = context.getExternalContext().getRequest();			
			if(isPortal(req))
				rebuild_proto_portal = "\nif (hX_5) hX_5.AJAX_prototypeloader();";
		
		}
		/*		
		buffer.append("<script type=\"text/JavaScript\" language=\"JavaScript\">\n"); //$NON-NLS-1$
		if (null != strings){
			buffer.append(strings);
		}
		*/
	
		StringBuffer resUrl = new StringBuffer(theContextPath);
		resUrl.append("/hxclient_S_")
		.append(HX_VERSION)
		.append(".js");
		resUrl = new StringBuffer(ResourceHandler.buildLocalizedLibName(ResourceHandler.getBundle(context), resUrl.toString()));
		resUrl.append("?")
		.append(HX_VIEW_LOCALE)
		.append("=")
		.append(locale.toString());
		String theEncodedStringURL = context.getExternalContext().encodeResourceURL(resUrl.toString()); //$NON-NLS-1$ //$NON-NLS-2$
		
	
		buffer.append("\n<script type=\"text/JavaScript\" language=\"JavaScript\" src=\"") //$NON-NLS-1$
		      .append(theEncodedURL +"\""+"></script>\n") //$NON-NLS-1$
			  .append("\n<script type=\"text/JavaScript\" language=\"JavaScript\" src=\"") //$NON-NLS-1$
			  .append(theEncodedStringURL +"\""+"></script>\n") 
			  .append("<script type=\"text/JavaScript\" language=\"JavaScript\">\n") //$NON-NLS-1$
			  .append(rebuild_proto_portal)
			  .append(buildSetResourcePathCall(context))
			  .append(buildPortalFlag(context))
			  .append(buildLocaleCall(context))
		      .append("\n</script>"); //$NON-NLS-1$
		
		//removed as part of fix for cs bug #50729
		//libraryInclude.put(libKey, script = (new ScriptObject(libKey, buffer.toString())));
		ScriptObject script = new ScriptObject(locale.toString(), buffer.toString());
		return script;
	}

	/**
	* <p>Return <code>true</code> if the component is contained in a DATA
	* component that renders its children.</p>
	*/
	public static boolean isContainedInPanel(UIComponent component) {

		while (component != null) {
			if (component instanceof UIData || component instanceof UIDataIterator) {
				return true;
			}
			component = component.getParent();
		}
		return false;

	}

	/**
        * @deprecated - use buildJsAddStringValidator()
		*  
		* @param context FacesContext
		* @param validators validators on component
		* @return ScriptObject containing the id and javascript of the validator.
		*/
	public synchronized static final ScriptObject buildJsStringValidator(
		FacesContext context,
		String constraint,
		String minLength,
		String maxLength,
		boolean required,
		ExpressionValidator expressionValidator) {
			return buildJsAddStringValidator(context, constraint, minLength, maxLength, required, expressionValidator);
		}

	/**
		* Builds the javascript statement(s) to instanciate a validator and register it. 
		* The javascript is cached to be available for requests for a validator configure 
		* the same.
		* This method is synchronized to avoid multiple copies of same validator being added.
		* It is banking on getting a high hit rate for cache otherwise it performance may
		* be poor due to blocking for the creationg of coverter code. The hit rate will
		* come from the same page being served up to multiple users and use in a dataTable
		*  
		* @param context FacesContext
		* @param validators validators on component
		* @return ScriptObject containing the id and javascript of the validator.
		*/
	public synchronized static final ScriptObject buildJsAddStringValidator(
		FacesContext context,
		String regexp,
		String minLength,
		String maxLength,
		boolean required,
		ExpressionValidator expressionValidator) {

		String validatorKey = null;
		String validatorId = null;
		String expression = null;
		if(expressionValidator != null) {
			expression = JavaScriptUtil.convertEL2JavaScript(expressionValidator.getExpression());
		}
		validatorKey = createStringValidatorKey(regexp, minLength, maxLength, required, expression);
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) constraintValidatorJs.get(validatorKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		String[] args = new String[5];
		int iArg = 0;
		if (null != minLength) {
			args[iArg++] = "min-length:" + minLength; //$NON-NLS-1$
		}
		if (null != maxLength) {
			args[iArg++] = "max-length:" + maxLength; //$NON-NLS-1$
		}
		if (required){
			args[iArg++] = "required:true"; //$NON-NLS-1$
		}
		if (null != expression) {
			args[iArg++] = "constraint-expression:" + expression; //$NON-NLS-1$
		}
		if (null != regexp) {
			if(regexp.equalsIgnoreCase("AlphabetOnly") || regexp.equalsIgnoreCase("DigitOnly") || regexp.equalsIgnoreCase("AlnumOnly") ||  //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			   regexp.equalsIgnoreCase("AlphabetExOnly") || regexp.equalsIgnoreCase("AlnumExOnly")) { //$NON-NLS-1$ //$NON-NLS-2$

				args[iArg++] = "constraint:" + regexp; //$NON-NLS-1$
			}
			else {
				// JavaScript can't handle back slashes in regular expressions correctly.
				// We need to replace \ with \u005c for JS to work.
				regexp = regexp.replaceAll("\\\\", "\\\\u005c"); //$NON-NLS-1$ //$NON-NLS-2$
				
				args[iArg++] = "constraint-regexpression:" + regexp; //$NON-NLS-1$
			}
		}

		if (iArg > 0) {
			validatorId = genScriptId(context);
			StringBuffer javascript = new StringBuffer(HX_VAR_NAME); //$NON-NLS-1$
			javascript.append(".addValidator(\"");
			javascript.append(validatorId);
			javascript.append("\", "); //$NON-NLS-1$

			javascript.append("new ") //$NON-NLS-1$
			.append(HX_VAR_NAME)
			.append(".StringValidator(\"")
			.append(args[--iArg]).append('\"');
			while (iArg > 0) {
				javascript.append(", \"") //$NON-NLS-1$
				.append(args[--iArg]).append('\"');
			}
			javascript.append("));"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(validatorId, javascript.toString());
			constraintValidatorJs.put(validatorKey, so);
			return so;
		}
		return null;
	}
	
	public static void setupStrictnessLevel(FacesContext context){
		if(dateStrictnessLevel == null){
			dateStrictnessLevel = context.getExternalContext().getInitParameter(ContextParams.DATETIME_ASSIST_STRICTNESS);
			if(dateStrictnessLevel == null || (!dateStrictnessLevel.equals("1") && !dateStrictnessLevel.equals("2"))){ //$NON-NLS-1$ //$NON-NLS-2$
				dateStrictnessLevel = "1"; //$NON-NLS-1$
			}
		}
		if(numberStrictnessLevel == null){
			numberStrictnessLevel = context.getExternalContext().getInitParameter(ContextParams.NUMBER_ASSIST_STRICTNESS);
			if(numberStrictnessLevel == null || (!numberStrictnessLevel.equals("1") && !numberStrictnessLevel.equals("2"))){ //$NON-NLS-1$ //$NON-NLS-2$
				numberStrictnessLevel = "1"; //$NON-NLS-1$
			}
		}
	}
	
	/**
	 * @return Returns the dateStrictnessLevel.
	 */
	public static String getDateStrictnessLevel() {
		if(dateStrictnessLevel == null){
			setupStrictnessLevel(FacesContext.getCurrentInstance());
		}
		return dateStrictnessLevel;
	}
	/**
	 * @return Returns the numberStrictnessLevel.
	 */
	public static String getNumberStrictnessLevel() {
		if(numberStrictnessLevel == null){
			setupStrictnessLevel(FacesContext.getCurrentInstance());
		}
		return numberStrictnessLevel;
	}
	
	public static final void addScriptLibrary(String library,UIComponent component)
	{
		FacesContext context = FacesContext.getCurrentInstance();
		if(context != null)  //check to make sure that this is a faces app
		{
			String useFullHxVersion = context.getExternalContext().getInitParameter(ContextParams.USE_HXCLIENT_FULL);
			if(useFullHxVersion == null || (useFullHxVersion!=null && (!(useFullHxVersion.equals("true")))))
			{
				if(library != null)
				{
					UIScriptCollector scriptCollector;
					if(component==null)
					{
						scriptCollector = getScriptCollector(FacesContext.getCurrentInstance().getViewRoot().getFacetsAndChildren());
					}
					else
					{
						if(component instanceof UIScriptCollector)
						{
							scriptCollector = (UIScriptCollector) component;
						}
						else
						{
							scriptCollector = UIScriptCollector.find(component);
						}
					}
					if(scriptCollector.isRendering())
						scriptCollector.addLibraryReference(Utils.versionizeFilename(library),context);
				}	
			}
		}
		else
		{
			//if there is no FacesContext then the ScriptCollector cannot be used. 
			//Do nothing, leave it up to the developer to output the script manually
		}
	}
	
	private static UIScriptCollector getScriptCollector(Iterator components)
	{
	   
		while(components.hasNext()) //for each component in the viewroot
		{
			UIComponent comp = (UIComponent)components.next();
			
			if((comp.getFacetsAndChildren()).hasNext()) // if the component has children
			{
				getScriptCollector(comp.getFacetsAndChildren()); //recursively call this method with their  children
			}
			
			
			if(comp instanceof UIScriptCollector)
			{
				return (UIScriptCollector)comp;
			}
		}
		return null;
	}

	public static final void addLocalizedStringLibraryIfNeeded(String locale, UIComponent component) {
		FacesContext context = FacesContext.getCurrentInstance();
		if(context == null) {
			return;
		}
		if(locale.toString().equals(context.getViewRoot().getLocale().toString())) {
			return;
		}
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		if(scriptCollector == null || !scriptCollector.isRendering()) {
			return;
		}
		StringBuffer resUrl = new StringBuffer(LifecycleUtil.getResourceServletContextPath(context));
		resUrl.append("/hxclient_S_")
		.append(HxClientRenderUtil.HX_VERSION)
		.append("_").append(locale)
		.append(".js?")
		.append(HxClientRenderUtil.HX_VIEW_LOCALE)
		.append("=")
		.append(locale.toString());
		String theEncodedStringURL = context.getExternalContext().encodeResourceURL(resUrl.toString());
		scriptCollector.addLibrary(theEncodedStringURL);
	}
	
	/*
	 * detecting if we are in portal code
	 * */
	
    private static final Class PORTLET_REQ = getPortletRequestClass();
    
    private static Class getPortletRequestClass() {
    	ClassLoader cl = Thread.currentThread().getContextClassLoader();
    	if(cl==null) {
    		cl = HxClientRenderUtil.class.getClassLoader();
    	}
    	if(cl==null) {
    		cl = ClassLoader.getSystemClassLoader();
    	}
    	
    	try {
    		return cl.loadClass("javax.portlet.PortletRequest");
    	} catch(ClassNotFoundException e) {
    		return null;
    	}
    }
    
    
    private static final Class LEG_PORTLET_REQ = getLegacyPortletRequestClass();
    
    private static Class getLegacyPortletRequestClass() {
    	ClassLoader cl = Thread.currentThread().getContextClassLoader();
    	if(cl==null) {
    		cl = HxClientRenderUtil.class.getClassLoader();
    	}
    	if(cl==null) {
    		cl = ClassLoader.getSystemClassLoader();
    	}
    	
    	try {
    		return cl.loadClass("org.apache.jetspeed.portlet.PortletRequest");
    	} catch(ClassNotFoundException e) {
    		return null;
    	}
    }
    
    
    public static boolean isPortal(Object request) {
    	
    	if(PORTLET_REQ != null && PORTLET_REQ.isInstance(request)) {
    		return true;
    	}
    	
    	if(LEG_PORTLET_REQ != null && LEG_PORTLET_REQ.isInstance(request)) {
    		return true;
    	}
    	
    	return false;
    }
	
    /**
     * Converts a component ID into component's client ID
     * For example, given component ID "text1", the method would return a client ID
     * such as "form1:text1".
     * 
     * Note: Client ID resolution is possible only if the component already exists in the View tree.
     * This is not always the case. When the component hasn't been created yet by the JSP processor,
     * there is no client ID to be found. In this case the original component ID will be returned.
     *  
     * @param id of the component whose client id is to be returned
     * @param component that requests the client id search
     * @return client id of the component, if found. Unmodified component id otherwise.
     */
    public static String convertComponentIdToClientId(String id, UIComponent component) {
    	String clientId = id;
    	
		UIComponent targetComponent = Utils.findNearestComponent(component, id);
		if(targetComponent != null)
		{
			clientId = targetComponent.getClientId(FacesContext.getCurrentInstance());
		}    	
		
    	return clientId;
    }
    
    /**
     * Converts a list of compoent IDs into a list of components' client IDs.
     * 
     * The lists are semicolon-separated strings.
     * 
     * Each ID is converted using convertComponentIdToClientId method.
     * 
     * @see convertComponentIdToClientId
     * @param ids of components whose client IDs are to be returned. Semicolon-separated.
     * @param component that requests the client id search
     * @return client IDs of the components, if found. Unmodified component IDs otherwise. Semicolon-separated.
     */
    public static String convertComponentIdsToClientIds(String ids, UIComponent component) {
    	StringBuffer clientIds = new StringBuffer();
		
    	StringTokenizer st = new StringTokenizer(ids, ";"); //$NON-NLS-1$
		while(st.hasMoreTokens()) {
			String currentId = st.nextToken();
			String clientId = convertComponentIdToClientId(currentId, component);
			clientIds.append(clientId);
			if(st.hasMoreTokens()){
				clientIds.append(";");//$NON-NLS-1$
			}
		}
    	
    	return clientIds.toString();
    }
    
}
