/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.lang.reflect.Method;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;
import javax.faces.convert.DateTimeConverter;
import javax.faces.validator.Validator;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.converter.DateTimeConverterEx;
import com.ibm.faces.converter.MaskConverter;
import com.ibm.faces.converter.NumberConverterEx;
import com.ibm.faces.util.CacheHashMap;
import com.ibm.faces.util.LifecycleUtil;
import com.ibm.faces.util.ScriptObject;
import com.ibm.faces.validator.DateTimeRangeValidator;
import com.ibm.faces.validator.DoubleRangeValidatorEx;
import com.ibm.faces.validator.LongRangeValidatorEx;

/**
*
* @deprecated Use HxClientRenderUtil object.  
*/
public class HxClientUtil {
	
	private static final ClassLoader hxClientClassLoader = (new HxClientUtil()).getClass().getClassLoader();

	//    static private final String DEFAULT_BASE_YEAR_OFFSET = "80"; //$NON-NLS-1$
	private static final String DATESTYLE_SHORT = "short"; //$NON-NLS-1$
	private static final String DATESTYLE_MEDIUM = "medium"; //$NON-NLS-1$
	private static final String DATESTYLE_LONG = "long"; //$NON-NLS-1$
	private static final String DATESTYLE_FULL = "full"; //$NON-NLS-1$

	private static final String NUMBERSTYLE_NUMBER = "number"; //$NON-NLS-1$
	private static final String NUMBERSTYLE_CURRENCY = "currency"; //$NON-NLS-1$
	private static final String NUMBERSTYLE_PERCENT = "percent"; //$NON-NLS-1$

	// generate an id
	private static int idSeed = 0;
	private synchronized static String genScriptId(FacesContext context) {
		return context.getExternalContext().encodeNamespace(Integer.toHexString(idSeed++));
	}

	// Some maps to hold on to converters/validators we may already have generated
	private static CacheHashMap maskConverterJs = new CacheHashMap(25);
	private static CacheHashMap dateConverterJs = new CacheHashMap(25);
	private static CacheHashMap numberConverterJs = new CacheHashMap(25);
	private static CacheHashMap rangeValidatorJs = new CacheHashMap(25);
	private static CacheHashMap libraryInclude = new CacheHashMap(12);
	/**
	 * Builds the javascript statement(s) to generate the setResourceServer 
	 * call to init resources in hxclient.js. This could return null if this call is not
	 * required.
	 *  
	 * @param context FacesContext
	 * @return return null if is not need else the setResourceServer call to init resource in hxclient.js
	 */
	public static final String buildSetResourcePathCall(FacesContext context) {
		StringBuffer script = new StringBuffer();
		script.append("if (hX_1)  hX_1.setResourceServer(\""); //$NON-NLS-1$

		script.append(LifecycleUtil.getResourceServletContextPath(context));
		
		script.append("\");"); //$NON-NLS-1$
		return script.toString();
	}

	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * It returns the id of the converter so it can be looked up in the converter map.
	 *  
	 * @deprecated Use specific converter build
	 * @param javascript
	 * @param context
	 * @param component
	 * @return the id of the converter so it can be looked up in the converter map.
	 */
	public static final String buildJsConverter(StringBuffer javascript, FacesContext context, UIOutput component) {

		Converter converter = component.getConverter();
		if (null == converter) {
			return null;
		}
		if (converter instanceof DateTimeConverter) {
			ScriptObject js = buildJsDateConverter(context, (DateTimeConverterEx) converter);
			javascript.append(js.getScript());
			return js.getId();
		} else if (converter instanceof NumberConverterEx) {
			ScriptObject js = buildJsNumberConverter(context, (NumberConverterEx) converter);
			javascript.append(js.getScript());
			return js.getId();
		}
		return null;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * The javascript is cached to be available for requests for a converter configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same converter being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param converter DateTimeConverter
	 * @return ScriptObject containing the id and javascript of the converter.
	 */
	public synchronized static final ScriptObject buildJsMaskConverter(
		FacesContext context,
		MaskConverter maskConverter) {

		if (null == maskConverter) {
			return null;
		}
		String pattern = maskConverter.getMask();
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) maskConverterJs.get(pattern);
		if (null != cachedScript) {
			return cachedScript;
		}
		// ok, build one

		String converterId = genScriptId(context);

		if (pattern != null && !pattern.equals("")) { //$NON-NLS-1$
			StringBuffer javascript = new StringBuffer("hX_1.addConverter(\""); //$NON-NLS-1$
			javascript.append(converterId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append("new hX_1.MaskConverter(\"mask:"); //$NON-NLS-1$
			javascript.append(pattern);
			javascript.append("\")"); //$NON-NLS-1$
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(converterId, javascript.toString());
			maskConverterJs.put(pattern, so);
			return so;
		}
		return null;
	}

	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * The javascript is cached to be available for requests for a converter configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same converter being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param converter DateTimeConverter
	 * @return ScriptObject containing the id and javascript of the converter.
	 */
	public synchronized static final ScriptObject buildJsDateConverter(
		FacesContext context,
		DateTimeConverter datetimeConverter) {

		if (null == datetimeConverter) {
			datetimeConverter = defaultDateTimeConverter;
		}
		Locale locale = datetimeConverter.getLocale();
		String converterKey = createDateConverterKey(datetimeConverter);
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) dateConverterJs.get(converterKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		// ok, build one

		String converterId = genScriptId(context);
		String pattern = datetimeConverter.getPattern();
		String type = datetimeConverter.getType();
		String dateStyle = datetimeConverter.getDateStyle();
		String timeStyle = datetimeConverter.getTimeStyle();
		String format = null;

		if (pattern != null && !pattern.equals("")) //$NON-NLS-1$
			format = pattern;
		else {
			DateFormat df = null;
			if (type.equals("both")) { //$NON-NLS-1$
				df = DateFormat.getDateTimeInstance(getStyleCode(dateStyle), getStyleCode(timeStyle), locale);
			} else if (type.equals("date")) { //$NON-NLS-1$
				df = DateFormat.getDateInstance(getStyleCode(dateStyle), locale);
			} else if (type.equals("time")) { //$NON-NLS-1$
				df = DateFormat.getTimeInstance(getStyleCode(timeStyle), locale);
			} else {
				df = DateFormat.getDateInstance(getStyleCode(dateStyle), locale);
			}
			format = ((SimpleDateFormat) df).toPattern();
		}
		if (null != format) {
			StringBuffer javascript = new StringBuffer("hX_1.addConverter(\""); //$NON-NLS-1$
			javascript.append(converterId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append("new hX_1.DateTimeConverter(\"strict:1\", \"format:"); //$NON-NLS-1$
			javascript.append(format);
			javascript.append("\")"); //$NON-NLS-1$
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(converterId, javascript.toString());
			dateConverterJs.put(converterKey, so);
			return so;
		}
		return null;
	}
	private static DateTimeConverter defaultDateTimeConverter = new DateTimeConverter();

	private static String createDateConverterKey(DateTimeConverter converter) {
		StringBuffer key = new StringBuffer(converter.getLocale().toString());
		key.append(converter.getType());
		key.append(converter.getPattern());
		key.append(converter.getDateStyle());
		key.append(converter.getTimeStyle());
		return key.toString();
	}
	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * The javascript is cached to be available for requests for a converter configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same converter being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param converter NumberConverterEx
	 * @return ScriptObject containing the id and javascript of the converter.
	 */
	public synchronized static final ScriptObject buildJsNumberConverter(
		FacesContext context,
		NumberConverterEx numConverter) {

		Locale locale = numConverter.getLocale();
		String pattern = numConverter.getPattern();
		String type = numConverter.getType();
		String numConverterKey = createNumberConverterKey(numConverter);
		// let's see if we already have one
		ScriptObject cachedScript = (ScriptObject) numberConverterJs.get(numConverterKey);
		if (null != cachedScript) {
			return cachedScript;
		}
		// ok, build one

		String converterId = genScriptId(context);

		StringBuffer javascript = new StringBuffer();
		String converterPattern = null;
		try {
			// Create and configure the formatter to be used
			NumberFormat formatter = getNumberFormat(numConverter, locale);
			if (((pattern != null) && !pattern.equals("")) || NUMBERSTYLE_CURRENCY.equals(type)) { //$NON-NLS-1$
				configureCurrency(numConverter, formatter);
			}
			configureFormatter(numConverter, formatter);
			if (formatter instanceof DecimalFormat) {
				converterPattern = ((DecimalFormat) formatter).toPattern();
			} else {
				String formattedResult = formatter.format(new Double(111111111111111111.111111111111111111));
				converterPattern = formattedResult.replace('1', '#');
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		if (null != converterPattern && converterPattern.length() > 0) {
			javascript.append("hX_1.addConverter(\""); //$NON-NLS-1$
			javascript.append(converterId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append("new hX_1.NumberConverter(\"strict:1\", \"pattern:"); //$NON-NLS-1$
			javascript.append(converterPattern);
			javascript.append("\", \"locale:"); //$NON-NLS-1$
			DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
			javascript.append(symbols.getGroupingSeparator());
			javascript.append(symbols.getDecimalSeparator());
			javascript.append(symbols.getPercent());
			javascript.append(symbols.getPerMill());
			javascript.append(symbols.getMinusSign());
		    javascript.append(Utils.escapeBackslash(symbols.getCurrencySymbol()));
			//javascript.append(symbols.getCurrencySymbol());
			javascript.append("\")"); //$NON-NLS-1$		
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(converterId, javascript.toString());
			numberConverterJs.put(numConverterKey, so);
			return so;
		}
		return null;
	}
	private static String createNumberConverterKey(NumberConverterEx numConverter) {
		StringBuffer key = new StringBuffer(numConverter.getLocale().toString());
		key.append(numConverter.getPattern());
		key.append(numConverter.getType());
		key.append(numConverter.isCurrencySymbolSet() ? numConverter.getCurrencySymbol() : null);
		key.append(numConverter.isCurrencyCodeSet() ? numConverter.getCurrencyCode() : null);
		key.append(numConverter.isGroupingUsed());
		key.append(numConverter.isIntegerOnly());
		key.append(
			numConverter.isMaxFractionDigitsSet() ? Integer.toString(numConverter.getMaxFractionDigits()) : null);
		key.append(
			numConverter.isMinFractionDigitsSet() ? Integer.toString(numConverter.getMinFractionDigits()) : null);
		key.append(numConverter.isMaxIntegerDigitsSet() ? Integer.toString(numConverter.getMaxIntegerDigits()) : null);
		key.append(numConverter.isMinIntegerDigitsSet() ? Integer.toString(numConverter.getMinIntegerDigits()) : null);
		return key.toString();
	}
	private static String createRangeValidatorKey(Object a, Object b, Object c, Object d) {
		StringBuffer key = new StringBuffer();
		key.append(a);
		key.append(b);
		key.append(c);
		key.append(d);
		return key.toString();
	}
	private static String createRangeValidatorKey(double a, double b, Object c, Object d) {
		StringBuffer key = new StringBuffer();
		key.append(a);
		key.append(b);
		key.append(c);
		key.append(d);
		return key.toString();
	}
	private static String createRangeValidatorKey(long a, long b, Object c, Object d) {
		StringBuffer key = new StringBuffer();
		key.append(a);
		key.append(b);
		key.append(c);
		key.append(d);
		return key.toString();
	}
	/*
	private static String createStringValidatorKey(String constraint, String minLength, String maxLength) {
		StringBuffer key = new StringBuffer(constraint);
		key.append(minLength);
		key.append(maxLength);
		return key.toString();
	}
	*/
	
	/**
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * It returns the id of the converter so it can be looked up in the converter map.
	 *  
	 * @param numConverter
	 * @param locale
	 * @return the javascript new converter statement
	 */
	public static final String buildJsNumberConverter(NumberConverterEx numConverter, Locale locale) {

		String converterPattern = null;
		if (null != numConverter.getLocale()) {
			// over-ride default locale
			locale = numConverter.getLocale();
		}
		try {
			// Create and configure the formatter to be used
			NumberFormat formatter = getNumberFormat(numConverter, locale);
			String pattern = numConverter.getPattern();
			String type = numConverter.getType();
			if (((pattern != null) && !pattern.equals("")) || NUMBERSTYLE_CURRENCY.equals(type)) { //$NON-NLS-1$
				configureCurrency(numConverter, formatter);
			}
			configureFormatter(numConverter, formatter);
			if (formatter instanceof DecimalFormat) {
				converterPattern = ((DecimalFormat) formatter).toPattern();
			} else {
				String formattedResult = formatter.format(new Double(111111111111111111.111111111111111111));
				converterPattern = formattedResult.replace('1', '#');
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
		if (null != converterPattern && converterPattern.length() > 0) {
			StringBuffer javascript = new StringBuffer();
			javascript.append("new hX_1.NumberConverter(\"strict:1\", \"pattern:"); //$NON-NLS-1$
			javascript.append(converterPattern);
			javascript.append("\", \"locale:"); //$NON-NLS-1$
			DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
			javascript.append(symbols.getGroupingSeparator());
			javascript.append(symbols.getDecimalSeparator());
			javascript.append(symbols.getPercent());
			javascript.append(symbols.getPerMill());
			javascript.append(symbols.getMinusSign());
		    javascript.append(Utils.escapeBackslash(symbols.getCurrencySymbol()));
			//javascript.append(symbols.getCurrencySymbol());
			javascript.append("\")"); //$NON-NLS-1$		
			return javascript.toString();
		}
		return null;
	}

	/**
	 * @deprecated
	 * Builds the javascript statement(s) to instanciate a converter and register it. 
	 * It returns the id of the converter so it can be looked up in the converter map.
	 *  
	 * @param datetimeConverter
	 * @param locale
	 * @return the javascript new converter statement
	 */
	public static final String buildJsDateConverter(DateTimeConverter datetimeConverter, Locale locale) {

		String format = null;
		if (null != datetimeConverter.getLocale()) {
			// over-ride default locale
			locale = datetimeConverter.getLocale();
		}
		String pattern = datetimeConverter.getPattern();
		if (pattern != null && !pattern.equals("")) //$NON-NLS-1$
			format = pattern;
		else {
			String type = datetimeConverter.getType();
			DateFormat df = null;
			if (type.equals("both")) { //$NON-NLS-1$
				df =
					DateFormat.getDateTimeInstance(
						getStyleCode(datetimeConverter.getDateStyle()),
						getStyleCode(datetimeConverter.getTimeStyle()),
						locale);
			} else if (type.equals("date")) { //$NON-NLS-1$
				df = DateFormat.getDateInstance(getStyleCode(datetimeConverter.getDateStyle()), locale);
			} else if (type.equals("time")) { //$NON-NLS-1$
				df = DateFormat.getTimeInstance(getStyleCode(datetimeConverter.getTimeStyle()), locale);
			} else {
				df = DateFormat.getDateInstance(getStyleCode(datetimeConverter.getDateStyle()), locale);
			}
			format = ((SimpleDateFormat) df).toPattern();
		}
		if (null != format) {
			StringBuffer javascript = new StringBuffer();
			javascript.append("new hX_1.DateTimeConverter(\"strict:1\", \"format:"); //$NON-NLS-1$
			javascript.append(format);
			javascript.append("\")"); //$NON-NLS-1$
			return javascript.toString();
		}
		return null;
	}

	private static int getStyleCode(String s) {
		int dateCode = DateFormat.DEFAULT;
		if (s != null && s != "") { //$NON-NLS-1$
			if (DATESTYLE_SHORT.equalsIgnoreCase(s)) {
				dateCode = DateFormat.SHORT;
			} else if (DATESTYLE_MEDIUM.equalsIgnoreCase(s)) {
				dateCode = DateFormat.MEDIUM;
			} else if (DATESTYLE_LONG.equalsIgnoreCase(s)) {
				dateCode = DateFormat.LONG;
			} else if (DATESTYLE_FULL.equalsIgnoreCase(s)) {
				dateCode = DateFormat.FULL;
			}
		}
		return dateCode;
	}

	/**
	 * @deprecated
	 * Builds the javascript statement(s) to instanciate a validator and register it. 
	 * It returns the id of the validator so it can be looked up in the validator map.
	 *  
	 * @param javascript StringBuffer to fill in with javascript string
	 * @param context FacesContext
	 * @param input input component being processed
	 * @return the id of the validator so it can be looked up in the validator map or null if no validator
	 */
	public static final String buildJsRangeValidator(StringBuffer javascript, FacesContext context, UIInput input) {

		ScriptObject js = buildJsRangeValidator(context, getRangeValidator(input));
		javascript.append(js.getScript());
		return js.getId();
	}

	/**
	 * Builds the javascript statement(s) to instanciate a validator and register it. 
	 * The javascript is cached to be available for requests for a validator configure 
	 * the same.
	 * This method is synchronized to avoid multiple copies of same validator being added.
	 * It is banking on getting a high hit rate for cache otherwise it performance may
	 * be poor due to blocking for the creationg of coverter code. The hit rate will
	 * come from the same page being served up to multiple users and use in a dataTable
	 *  
	 * @param context FacesContext
	 * @param validator Range validator
	 * @return ScriptObject containing the id and javascript of the validator.
	 */
	public synchronized static final ScriptObject buildJsRangeValidator(FacesContext context, Validator validator) {

		// TODO remove old override support?
		Object minBound = null;
		Object maxBound = null;

		String validatorKey = null;
		String validatorId = null;
		String newValidator = null;

		if (validator instanceof DateTimeRangeValidator) {
			validatorKey =
				createRangeValidatorKey(
					((DateTimeRangeValidator) validator).getMinimum(),
					((DateTimeRangeValidator) validator).getMaximum(),
					minBound,
					maxBound);
			// let's see if we already have one
			ScriptObject cachedScript = (ScriptObject) rangeValidatorJs.get(validatorKey);
			if (null != cachedScript) {
				return cachedScript;
			}
			newValidator =
				buildJsDateRangeValidator((DateTimeRangeValidator) validator, (Date) minBound, (Date) maxBound);
		} else if ((validator instanceof LongRangeValidatorEx) || (validator instanceof DoubleRangeValidatorEx)) {
			if (validator instanceof LongRangeValidatorEx) {
				validatorKey =
					createRangeValidatorKey(
						((LongRangeValidatorEx) validator).getMinimum(),
						((LongRangeValidatorEx) validator).getMaximum(),
						minBound,
						maxBound);
			} else {
				validatorKey =
					createRangeValidatorKey(
						((DoubleRangeValidatorEx) validator).getMinimum(),
						((DoubleRangeValidatorEx) validator).getMaximum(),
						minBound,
						maxBound);
			}
			// let's see if we already have one
			ScriptObject cachedScript = (ScriptObject) rangeValidatorJs.get(validatorKey);
			if (null != cachedScript) {
				return cachedScript;
			}
			newValidator = buildJsNumberRangeValidator(validator, (Number) minBound, (Number) maxBound);
		}
		validatorId = genScriptId(context);
		if (null != newValidator) {
			StringBuffer javascript = new StringBuffer("hX_1.addValidator(\""); //$NON-NLS-1$
			javascript.append(validatorId);
			javascript.append("\", "); //$NON-NLS-1$
			javascript.append(newValidator);
			javascript.append(");"); //$NON-NLS-1$
			ScriptObject so = new ScriptObject(validatorId, javascript.toString());
			rangeValidatorJs.put(validatorKey, so);
			return so;
		}
		return null;
	}

	public static final Validator getRangeValidator(UIInput input) {
		// Check if the component has a validation tag
		Validator validators[] = input.getValidators();
		for (int i = 0; i < validators.length; i++) {
			Validator validator = validators[i];
			if ((validator instanceof DateTimeRangeValidator)
				|| (validator instanceof LongRangeValidatorEx)
				|| (validator instanceof DoubleRangeValidatorEx)) {
				return validator;
			}
		}
		return null;
	}
	/**
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsNumberRangeValidator(Validator validator) {
		return buildJsNumberRangeValidator(validator, null, null);
	}

	/**
	 * @deprecated
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsNumberRangeValidator(Validator validator, Number minBound, Number maxBound) {

		StringBuffer javascript = new StringBuffer();
		if (validator instanceof LongRangeValidatorEx) {
			LongRangeValidatorEx longValidator = (LongRangeValidatorEx) validator;
			if (null == minBound && longValidator.isMinimumSet()) {
				minBound = new Long(longValidator.getMinimum());
			}
			if (null == maxBound && longValidator.isMaximumSet()) {
				maxBound = new Long(longValidator.getMaximum());
			}
		}
		if (validator instanceof DoubleRangeValidatorEx) {
			DoubleRangeValidatorEx doubleValidator = (DoubleRangeValidatorEx) validator;
			if (null == minBound && doubleValidator.isMinimumSet()) {
				minBound = new Double(doubleValidator.getMinimum());
			}
			if (null == maxBound && doubleValidator.isMaximumSet()) {
				maxBound = new Double(doubleValidator.getMaximum());
			}

		}
		javascript.append("new hX_1.NumberValidator("); //$NON-NLS-1$
		if (minBound != null) {
			javascript.append(" \"min-bound:"); //$NON-NLS-1$
			javascript.append(minBound);
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (maxBound != null) {
			if (minBound != null) {
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"max-bound:"); //$NON-NLS-1$
			javascript.append(maxBound);
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		javascript.append(")"); //$NON-NLS-1$		
		return javascript.toString();
	}
	/**
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsDateRangeValidator(DateTimeRangeValidator validator) {
		return buildJsDateRangeValidator(validator, null, null);
	}

	/**
	 * @deprecated
	 * Builds the javascript statement(s) to instanciate a number validator. 
	 *  
	 * @param validator
	 * @param minBound override of validator min
	 * @param maxBound override of validator max
	 * @return the javascript new validator statement
	 */
	public static final String buildJsDateRangeValidator(
		DateTimeRangeValidator validator,
		Date minBound,
		Date maxBound) {

		StringBuffer javascript = new StringBuffer();
		if (null == minBound) {
			minBound = validator.getMinimum(); // override component's own settings
		}
		if (null == maxBound) {
			maxBound = validator.getMaximum(); // override component's own settings
		}
		javascript.append("new hX_1.DateTimeValidator("); //$NON-NLS-1$
		SimpleDateFormat sd = new SimpleDateFormat("yyyyMMddhhmmss"); //$NON-NLS-1$
		if (minBound != null) {
			javascript.append(" \"min-bound:"); //$NON-NLS-1$
			javascript.append(sd.format(minBound));
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		if (maxBound != null) {
			if (minBound != null) {
				javascript.append(", "); //$NON-NLS-1$
			}
			javascript.append(" \"max-bound:"); //$NON-NLS-1$
			javascript.append(sd.format(maxBound));
			javascript.append('\"'); //$NON-NLS-1$ //$NON-NLS-2$
		}
		javascript.append(")"); //$NON-NLS-1$	
		return javascript.toString();
	}

	/**
	 * <p>Return a <code>NumberFormat</code> instance to use for formatting
	 * and parsing in this {@link Converter}.</p>
	 *          
	 * @param locale The <code>Locale</code> used to select formatting
	 *  and parsing conventions
	 *
	 * @exception ConverterException if no instance can be created
	 */
	static private NumberFormat getNumberFormat(NumberConverterEx numConverter, Locale locale) {
		NumberFormat formatter = null;
		String pattern = numConverter.getPattern();
		String type = numConverter.getType();
		if (pattern == null && type == null) {
			throw new IllegalArgumentException("Either pattern or type must" + " be specified."); //$NON-NLS-1$ //$NON-NLS-2$
		}


		// If pattern is specified, type is ignored
		if (pattern != null) {
			DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);
			formatter = new DecimalFormat(pattern, symbols);
		}

		// Create an instance based on the specified type
		else if (type.equals(NUMBERSTYLE_CURRENCY)) {
			formatter = NumberFormat.getCurrencyInstance(locale);
		} else if (type.equals(NUMBERSTYLE_NUMBER)) {
			formatter = NumberFormat.getNumberInstance(locale);
		} else if (type.equals(NUMBERSTYLE_PERCENT)) {
			formatter = NumberFormat.getPercentInstance(locale);
		} else {
			throw new ConverterException(new IllegalArgumentException(type));
		}
		if (numConverter.isIntegerOnly()) {
			formatter.setMaximumFractionDigits(0);
		}
		return formatter;
	}

	private static Boolean curClassAvailable = null;
	private static Class curClass = null;
	private static Class[] stringArg = new Class[]{String.class};
	
	private static void configureCurrency(NumberConverterEx numConverter, NumberFormat formatter) throws Exception {
		//if code or symbol not set do nothing
		if(numConverter.getCurrencyCode() == null && numConverter.getCurrencySymbol() == null){
			return;
		}
		if(curClassAvailable == null) {
				try {
					curClass = Class.forName("java.util.Currency");
					curClassAvailable = Boolean.TRUE;
				} catch (ClassNotFoundException e) {
					//jre version < 1.4
					curClassAvailable = Boolean.FALSE;
				}
		}
		
		//if currencycode is set and currency class is available
		if(numConverter.getCurrencyCode() != null && curClassAvailable.booleanValue()){
			
			Method method = curClass.getMethod("getInstance",stringArg);
			Object cur = method.invoke(null, new Object[]{numConverter.getCurrencyCode()});
			
			Class numFormatClass = Class.forName("java.text.NumberFormat");
			method = numFormatClass.getMethod("setCurrency",new Class[]{curClass});
			method.invoke(formatter, new Object[]{cur});
			return;
		}
		String curSymbol = null;
		
		if (numConverter.getCurrencySymbol() != null){
			curSymbol = numConverter.getCurrencySymbol();
		} else if (numConverter.getCurrencyCode() != null){
			curSymbol = numConverter.getCurrencyCode();
			
		}

		if (curSymbol != null){
			DecimalFormat df = (DecimalFormat) formatter;
			DecimalFormatSymbols dfs = df.getDecimalFormatSymbols();
			dfs.setCurrencySymbol(curSymbol);
			df.setDecimalFormatSymbols(dfs);
		}
	}

	
	/**
	 * <p>Configure the specified <code>NumberFormat</code> based on the
	 * formatting properties that have been set.</p>
	 *
	 * @param formatter The <code>NumberFormat</code> instance to configure
	 */
	private static void configureFormatter(NumberConverterEx numConverter, NumberFormat formatter) {

		formatter.setGroupingUsed(numConverter.isGroupingUsed());
		if (numConverter.isMaxIntegerDigitsSet()) {
			formatter.setMaximumIntegerDigits(numConverter.getMaxIntegerDigits());
		}
		if (numConverter.isMinIntegerDigitsSet()) {
			formatter.setMinimumIntegerDigits(numConverter.getMinIntegerDigits());
		}
		if (numConverter.isMaxFractionDigitsSet()) {
			formatter.setMaximumFractionDigits(numConverter.getMaxFractionDigits());
		}
		if (numConverter.isMinFractionDigitsSet()) {
			formatter.setMinimumFractionDigits(numConverter.getMinFractionDigits());
		}

	}

	/**
	 * @param component
	 * @param context
	 */
	public static void initJSLibraries(UIComponent component, FacesContext context) {
		UIScriptCollector scriptCollector = null;
		if (component instanceof UIScriptCollector) {
			scriptCollector = (UIScriptCollector) component;
		} else {
			scriptCollector = UIScriptCollector.find(component);
		}
		ScriptObject script = buildLibraryIncludeBlock(context);
		scriptCollector.writeLibraryReference(context, script.getId(), script.getScript());
	}

	/**
	 * @deprecated use initJSLibraries(UIComponent,FacesContext)
	 * @param component
	 * @param context
	 * @param bWriteNow
	 */
	public static void initJSLibraries(UIComponent component, FacesContext context, boolean bWriteNow) {
		UIScriptCollector scriptCollector = null;
		if (component instanceof UIScriptCollector) {
			scriptCollector = (UIScriptCollector) component;
		} else {
			scriptCollector = UIScriptCollector.find(component);
		}
		ScriptObject script = buildLibraryIncludeBlock(context);
		if (bWriteNow) {
			scriptCollector.writeLibraryReference(context, script.getId(), script.getScript());
		} else {
			scriptCollector.addLibrary(script.getId(), script.getScript());
			scriptCollector.addScriptOnce("hX_1.onPageLoad();"); //$NON-NLS-1$ 
		}
	}

	/**
	 * Build hxclient library include block
	 * @param context
	 * @param version
	 * @return ScriptObject where the Id is the library name and the script is the library include block
	 * which includes the script file include and string definitions and call to resource init.
	 */
	public synchronized static ScriptObject buildLibraryIncludeBlock(FacesContext context) {

		String libKey = context.getViewRoot().getLocale().toString();		
		ScriptObject script = (ScriptObject)libraryInclude.get(libKey);
		if (null != script){
			return script;
		}
		ResourceBundle bundle = ResourceHandler.getBundle(context);
		String libStrings = ResourceHandler.buildLocalizedLibName(bundle, "ibmjsfres/hxclient_S_1.js"); //$NON-NLS-1$ //$NON-NLS-2$
	
		String strings = null;
		InputStream stream = hxClientClassLoader.getResourceAsStream(libStrings);
		if (stream != null) {
			byte[] buffer = new byte[10000];
			byte[] bytes = new byte[0];
			int read = 0;
			try {
				while ((read = stream.read(buffer)) > 0) {
					int newSize = bytes.length + read;
					byte[] newBytes = new byte[newSize];
					System.arraycopy(bytes, 0, newBytes, 0, bytes.length);
					System.arraycopy(buffer, 0, newBytes, bytes.length, read);
					bytes = newBytes;
				}
				stream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}

			try {
				String encoding = bundle.getString("Locale.encoding"); //$NON-NLS-1$
				if (encoding == null || encoding.trim().length() == 0 || encoding.equals("!Locale.encoding!")) { //$NON-NLS-1$
					encoding = "ISO-8859-1"; //$NON-NLS-1$
				}
				strings = new String(bytes, encoding);
			} catch (UnsupportedEncodingException e1) {
				// Shouldn't happen!
				e1.printStackTrace();
				strings = new String(bytes);
			}
		}
		StringBuffer buffer = new StringBuffer();
		buffer.append("<script type=\"text/JavaScript\" language=\"JavaScript\""); //$NON-NLS-1$
		buffer.append(" src=\""); //$NON-NLS-1$
		
		//RATLC00285401
		buffer.append(LifecycleUtil.getContextPath(context)).append('/');
		
		buffer.append(".ibmjsfres/hxclient_v1.js\"></script>\n"); //$NON-NLS-1$
		buffer.append("<script type=\"text/JavaScript\" language=\"JavaScript\">\n"); //$NON-NLS-1$
		if (null != strings){
			buffer.append(strings);
		}
		buffer.append(buildSetResourcePathCall(context));
		buffer.append("\n</script>"); //$NON-NLS-1$
		libraryInclude.put(libKey, script = (new ScriptObject(libKey, buffer.toString())));
		return script;
	}

	/**
	* <p>Return <code>true</code> if the component is contained in a another
	* component that renders its children.</p>
	*/
	public static boolean isContainedInPanel(UIComponent component) {

		while (component != null) {
			if (component.getRendersChildren()) {
				return (true);
			}
			component = component.getParent();
		}
		return (false);

	}
}
