/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Iterator;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.render.Renderer;

import com.ibm.faces.component.UIDataIterator;



/**
 * <p>Iterator renderer, looks after the rendering of iterator components children.</p>
 * @author pandrews
 */
public class IteratorRenderer extends Renderer {
	
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#encodeChildren(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeChildren(FacesContext context, UIComponent component)
			throws IOException {
		
		if(!component.isRendered() || !(component instanceof UIDataIterator) ){
			return;
		}
		
		UIDataIterator iterator = (UIDataIterator) component;
		
		iterator.setIndex(iterator.getFirst());
		int rowCount = 1;
		int numberOfRows = iterator.getRows();

		while(iterator.isRowAvailable()){
			//check if reached number of rows to be displayed, if zero all rows to be displayed
			if(numberOfRows > 0 && rowCount > numberOfRows){
				break;
			}
			//process iterator children
			Iterator children = iterator.getChildren().iterator();
			while(children.hasNext()){
				Utils.encodeComponent(context, (UIComponent)children.next());
			}
			iterator.setIndex(iterator.getIndex() + 1);
			++rowCount;
		}
		//reset
		iterator.setIndex(-1);
	}
}
