/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;


import com.ibm.faces.component.html.HtmlPanelLayout;

/**
 * Renderer for {@link javax.faces.component.UIPanel UIPanel} component, 
 * that arranges all children along the component's borders.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Panel_LayoutTag}<br>
 * <b>Components:</b> {@link javax.faces.component.UIPanel}
 */

public class LayoutRenderer extends Renderer {
    

	/** 
	 * Renders the beginning of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */   
    
    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {

        if (context == null || component == null) {
            throw new NullPointerException();
        }

		HtmlPanelLayout layoutComponent = null;
		if ( component instanceof HtmlPanelLayout ){
			layoutComponent = (HtmlPanelLayout)component;
		}
		
		String panelClass;
		if ( layoutComponent != null ){
			panelClass = layoutComponent.getStyleClass();
		}else{
			panelClass = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
		}
		
		String width;
		if( layoutComponent !=null) {
			width = layoutComponent.getWidth();
		}else{
			width = (String)component.getAttributes().get("width");
		}

		String height;
		if( layoutComponent !=null) {
			height = layoutComponent.getHeight();
		}else{
			height = (String)component.getAttributes().get("height");
		}
		
		String border;
		if( layoutComponent !=null) {
			border = layoutComponent.getBorder();
		}else{
			border = (String)component.getAttributes().get("border");
		}
		String bgcolor;
		if( layoutComponent !=null) {
			bgcolor = layoutComponent.getBgcolor();
		}else{
			bgcolor = (String)component.getAttributes().get("bgcolor");
		}		
		String frame;
		if( layoutComponent !=null) {
			frame = layoutComponent.getFrame();
		}else{
			frame = (String)component.getAttributes().get("frame");
		}			
		String summary;
		if( layoutComponent !=null) {
			summary = layoutComponent.getSummary();
		}else{
			summary = (String)component.getAttributes().get("summary");
		}
        // Render the beginning of this panel
        ResponseWriter writer = context.getResponseWriter();
        writer.startElement("table",component);
        writer.writeAttribute("id",component.getClientId(context),null);
        writer.writeAttribute("cellspacing","0",null);
        writer.writeAttribute("cellpadding","0",null);
        if (panelClass != null) {
            writer.writeAttribute("class",panelClass,"styleClass");
        }
        if(width != null){
        	writer.writeAttribute("width",width,"width");
        }
        if(height != null){
        	writer.writeAttribute("height",height,"height");
        }
        if(border != null){
        	writer.writeAttribute("border",border,"border");
        }
        if(bgcolor != null){
        	writer.writeAttribute("bgcolor",bgcolor,"bgcolor");
        }
        if(frame != null){
        	writer.writeAttribute("frame",frame,"frame");
        }
        if(summary != null){
        	writer.writeAttribute("summary",summary,"summary");
        }

		if ( layoutComponent != null ) {
			
			if (layoutComponent.getDir() != null) {
				writer.writeAttribute("dir", layoutComponent.getDir(), null);
			}
			if (layoutComponent.getLang() != null) {
				writer.writeAttribute("lang", layoutComponent.getLang(), null);
			}
			if (layoutComponent.getStyle() != null) {
				writer.writeAttribute(
					"style",
					layoutComponent.getStyle(),
					null);
			}
			if (layoutComponent.getTitle() != null) {
				writer.writeAttribute(
					"title",
					layoutComponent.getTitle(),
					null);
			}
			
		} else {
			String dir = (String)component.getAttributes().get("dir");
			if(dir != null){
				writer.writeAttribute("dir",dir,"dir");
			}
			String lang = (String)component.getAttributes().get("lang");
			if(lang != null){
				writer.writeAttribute("lang",lang,"lang");
			}
			String style = (String)component.getAttributes().get("style");
			if(style != null){
				writer.writeAttribute("style",style,"style");
			}
			String title = (String)component.getAttributes().get("title");
			if(title != null){
				writer.writeAttribute("title",title,"title");
			}
		}

		String alignment;
		if ( layoutComponent != null ){
			alignment = layoutComponent.getAlign();
		}else{
			alignment = (String) component.getAttributes().get("align");	//$NON-NLS-1$
		}

		if (alignment != null) {
			writer.writeAttribute("align",alignment,"align");
		}
    }

	/** 
	 * Renders component's children.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeChildren(FacesContext context, UIComponent component)
		throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}
		
		HtmlPanelLayout layoutComponent = null;
		if ( component instanceof HtmlPanelLayout ){
			layoutComponent = (HtmlPanelLayout)component;
		}
		
		ResponseWriter writer = context.getResponseWriter();

		UIComponent topPanel, bottomPanel, leftPanel, rightPanel, bodyPanel;
		topPanel = component.getFacet("top"); //$NON-NLS-1$
		bottomPanel = component.getFacet("bottom"); //$NON-NLS-1$
		leftPanel = component.getFacet("left"); //$NON-NLS-1$
		rightPanel = component.getFacet("right"); //$NON-NLS-1$
		bodyPanel = component.getFacet("body"); //$NON-NLS-1$
		
		if (null != topPanel){
			// defect#267260
			writer.startElement("tr",topPanel);
			writer.startElement("td",topPanel);
			writer.writeAttribute("valign","top",null);
			writer.writeAttribute("width","100%",null);
			Utils.encodeComponent(context, topPanel);
			writer.endElement("td");
			writer.endElement("tr");
		}
		if(null != leftPanel ||  null != bodyPanel ||  null != rightPanel){
			int panelCount = ((null != leftPanel)?1:0) + ((null != rightPanel)?1:0) + ((null != bodyPanel)?1:0);
			if (1 < panelCount){
				writer.startElement("tr",component);
				writer.startElement("td",component);				
				writer.startElement("table",component);
				writer.writeAttribute("width","100%",null);
				writer.writeAttribute("border","0",null);
				writer.writeAttribute("cellspacing","0",null);
				writer.writeAttribute("cellpadding","0",null);
			}
			writer.startElement("tr",component);
			if (null != leftPanel){
				// defect#267260
				writer.startElement("td",leftPanel);
				writer.writeAttribute("align","left",null);
				Utils.encodeComponent(context, leftPanel);
				writer.endElement("td");
			}
			if (null != bodyPanel){
				writer.startElement("td",bodyPanel);
				Utils.encodeComponent(context, bodyPanel);
				writer.endElement("td");
			}
			if (null != rightPanel){
				/*
				String width;
				if ( layoutComponent != null ){
					width = layoutComponent.getWidth();
				}else{
					width = (String) rightPanel.getAttributes().get("width"); //$NON-NLS-1$
				}
				*/
				// defect#267260
				writer.startElement("td",rightPanel);
				writer.writeAttribute("align","right",null);
				/*
				if (null != width){
					writer.writeAttribute("width",width,"width");
				}
				*/
				
				Utils.encodeComponent(context, rightPanel);
				writer.endElement("td");
			}
			writer.endElement("tr");
			if (1 < panelCount){
				writer.endElement("table");
				writer.endElement("td");
				writer.endElement("tr");
			}
		}

		if (null != bottomPanel){
			// defect#267260
			writer.startElement("tr",bottomPanel);
			writer.startElement("td",bottomPanel);
			writer.writeAttribute("valign","bottom",null);
			writer.writeAttribute("width","100%",null);
			Utils.encodeComponent(context, bottomPanel);
			writer.endElement("td");
			writer.endElement("tr");
		}
	}

	/**
	 * Renders the ending of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {

        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        // Render the ending of this panel
        context.getResponseWriter().endElement("table"); //$NON-NLS-1$
    }


    /*
    * Returns an array of stylesheet classes to be applied to
    * each column in the list in the order specified. Every column may or
    * may not have a stylesheet
    
    private String[] getColumnClasses(UIComponent component) {
        String values = (String) component.getAttributes().get("columnClasses"); //$NON-NLS-1$
        if (values == null) {
            return (new String[0]);
        }
        values = values.trim();
        ArrayList list = new ArrayList();
        while (values.length() > 0) {
            int comma = values.indexOf(","); //$NON-NLS-1$
            if (comma >= 0) {
                list.add(values.substring(0, comma).trim());
                values = values.substring(comma + 1);
            } else {
                list.add(values.trim());
                values = ""; //$NON-NLS-1$
            }
        }
        String results[] = new String[list.size()];
        return ((String[]) list.toArray(results));
    }
	*/
   /*
    * Returns an array of stylesheet classes to be applied to
    * each row in the list in the order specified. Every row may or 
    * may not have a stylesheet
    
    private String[] getRowClasses(UIComponent component) {
        String values = (String) component.getAttributes().get("rowClasses"); //$NON-NLS-1$
        if (values == null) {
            return (new String[0]);
        }
        values = values.trim();
        ArrayList list = new ArrayList();
        while (values.length() > 0) {
            int comma = values.indexOf(","); //$NON-NLS-1$
            if (comma >= 0) {
                list.add(values.substring(0, comma).trim());
                values = values.substring(comma + 1);
            } else {
                list.add(values.trim());
                values = ""; //$NON-NLS-1$
            }
        }
        String results[] = new String[list.size()];
        return ((String[]) list.toArray(results));
    }
	*/
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#getRendersChildren()
	 */
	public boolean getRendersChildren() {
		return true;
	}

}
