/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import java.util.Iterator;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import javax.faces.component.UIParameter;

import com.ibm.faces.component.html.HtmlOutputLinkEx;

import com.ibm.faces.webapp.ValueResourcePhaseListener; // not sure about using this

public class LinkRenderer extends Renderer {

	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {
			
		if(context == null || component == null)
			throw new NullPointerException();

		HtmlOutputLinkEx lcomponent = null;
		if (component instanceof HtmlOutputLinkEx) // check if this is a button component
			lcomponent = (HtmlOutputLinkEx)component;		

		
		Boolean doParams = Boolean.TRUE;
		String href = null;
		
		String mimeType = null;
		if ( lcomponent != null )
			mimeType = lcomponent.getMimeType();
		else
			mimeType = (String)component.getAttributes().get("mimeType");
		
		
		if ((component.getValueBinding("mimeType") != null) || ( mimeType != null && mimeType.length() > 0)) {
			href = ValueResourcePhaseListener.getUrl(context, component); 
			doParams = Boolean.FALSE; // getUrl method will look after params	
		} else {
			if ( lcomponent != null )
				href = (String)lcomponent.getValue();
			else
				href = (String)component.getAttributes().get("value");
		}

		if(doParams.booleanValue() && ((href!= null) && (href.length() > 0))) // we process the params for the href
			href = doParameterString(component, href);
		
					//	END	 we process the params for the href

		ResponseWriter writer = context.getResponseWriter(); // get a writer
		
				// start of writting the anchor tag
		writer.startElement("a", component);
				
				// start of attributes
		String clientid = component.getClientId(context);
		writer.writeAttribute("id", clientid, null);
		
		if((href != null) && (href.length() > 0)){ // we already worked out href
			writer.writeURIAttribute("href", context.getExternalContext().encodeResourceURL(href), "href");
		} else {
			//this else was added by the xfaces team - we use the outputLink as a commandLink too (by adding command behaviour to it, so sometimes we need href to be blank but present
			writer.writeAttribute("href","#",null);
		}
			
		
		String style = null;
		if(lcomponent != null)
			style = lcomponent.getStyle();
		else		
			style = (String)component.getAttributes().get("style");				
		if(style!= null)
			writer.writeAttribute("style", style.trim(),"style");		

		String styleClass = null;
		if(lcomponent != null)
			styleClass = lcomponent.getStyleClass();
		else		
			styleClass = (String)component.getAttributes().get("styleClass");				
		if(styleClass!= null)
			writer.writeAttribute("class", styleClass,"class");	

		String dir = null;
		if(lcomponent != null)		
			dir= lcomponent.getDir();		
		else		
			dir = (String)component.getAttributes().get("dir");		
		if(dir != null)
			writer.writeAttribute("dir", dir,"dir");

		String lang = null;
		if(lcomponent != null)		
			lang= lcomponent.getLang();		
		else		
			lang = (String)component.getAttributes().get("lang");			
		if(lang != null)
			writer.writeAttribute("lang", lang,"lang");

		String title = null;
		if(lcomponent != null)		
			title = lcomponent.getTitle();		
		else		
			title = (String)component.getAttributes().get("title");			
		if(title != null)
			writer.writeAttribute("title", title,"title");

		String tabindex = null;
		if(lcomponent != null)		
			tabindex= lcomponent.getTabindex();		
		else		
			tabindex = (String)component.getAttributes().get("tabindex");			
		if(tabindex != null)
			writer.writeAttribute("tabindex", tabindex,"tabindex");

		String accesskey = null;
		if(lcomponent != null)		
			accesskey= lcomponent.getAccesskey();		
		else		
			accesskey = (String)component.getAttributes().get("accesskey");			
		if(accesskey != null)
			writer.writeAttribute("accesskey", accesskey,"accesskey");
									 

		String charset = null;
		if(lcomponent != null)		
			charset= lcomponent.getCharset();		
		else		
			charset = (String)component.getAttributes().get("charset");			
		if(charset != null)
			writer.writeAttribute("charset", charset,"charset");

		String coords = null;
		if(lcomponent != null)		
			coords= lcomponent.getCoords();		
		else		
			coords = (String)component.getAttributes().get("coords");			
		if(coords != null)
			writer.writeAttribute("coords", coords,"coords");		
		
		String hreflang = null;
		if(lcomponent != null)		
			hreflang= lcomponent.getHreflang();		
		else		
			hreflang = (String)component.getAttributes().get("hreflang");			
		if(hreflang != null)
			writer.writeAttribute("hreflang", hreflang,"hreflang");

		String rel = null;
		if(lcomponent != null)		
			rel= lcomponent.getRel();		
		else		
			rel = (String)component.getAttributes().get("rel");			
		if(rel != null)
			writer.writeAttribute("rel", rel,"rel");		
		
		String rev = null;
		if(lcomponent != null)		
			rev= lcomponent.getRev();		
		else		
			rev = (String)component.getAttributes().get("rev");			
		if(rev != null)
			writer.writeAttribute("rev", rev,"rev");

		String type = null;
		if(lcomponent != null)		
			type = lcomponent.getType();		
		else		
			type = (String)component.getAttributes().get("type");			
		if(type != null)
			writer.writeAttribute("type", type,"type");
	
		String target = null;
		if(lcomponent != null)		
			target = lcomponent.getTarget();		
		else		
			target = (String)component.getAttributes().get("target");			
		if(target != null)
			writer.writeAttribute("target", target,"target");

		String shape = null;
		if(lcomponent != null)		
			shape= lcomponent.getShape();		
		else		
			shape = (String)component.getAttributes().get("shape");			
		if(shape != null)
			writer.writeAttribute("shape", shape,"shape");		
									
									//		 start of the event section
		String onblur = null;
		if(lcomponent != null)		
			onblur= lcomponent.getOnblur();		
		else		
			onblur = (String)component.getAttributes().get("onblur");			
		if(onblur != null)
			writer.writeAttribute("onblur", onblur,"onblur");

		String onclick = null; 
		if(lcomponent != null)		
			onclick= lcomponent.getOnclick();		
		else		
			onclick = (String)component.getAttributes().get("onclick");		
		if(onclick != null)
			writer.writeAttribute("onclick", onclick,"onclick");

		String ondblclick = null;
		if(lcomponent != null)		
			ondblclick= lcomponent.getOndblclick();
		else
			ondblclick = (String)component.getAttributes().get("ondblclick");		
		if(ondblclick != null)
			writer.writeAttribute("ondblclick", ondblclick,"ondblclick");
		
		String onfocus = null;
		if(lcomponent != null)		
			onfocus= lcomponent.getOnfocus();		
		else		
			onfocus = (String)component.getAttributes().get("onfocus");
		if(onfocus != null)
			writer.writeAttribute("onfocus", onfocus,"onfocus");
		
		String onkeydown = null;
		if(lcomponent != null)		
			onkeydown= lcomponent.getOnkeydown();		
		else		
			onkeydown = (String)component.getAttributes().get("onkeydown");
		if(onkeydown != null)
			writer.writeAttribute("onkeydown", onkeydown,"onkeydown");
		
		String onkeypress = null;
		if(lcomponent != null)		
			onkeypress= lcomponent.getOnkeypress();		
		else		
			onkeypress = (String)component.getAttributes().get("onkeypress");
		if(onkeypress != null)
			writer.writeAttribute("onkeypress", onkeypress,"onkeypress");
		
		String onkeyup = null;
		if(lcomponent != null)		
			onkeyup= lcomponent.getOnkeyup();		
		else		
			onkeyup = (String)component.getAttributes().get("onkeyup");
		if(onkeyup != null)
			writer.writeAttribute("onkeyup", onkeyup,"onkeyup");
		
		String onmousedown = null;
		if(lcomponent != null)		
			onmousedown= lcomponent.getOnmousedown();		
		else		
			onmousedown = (String)component.getAttributes().get("onmousedown");		
		if(onmousedown != null)
			writer.writeAttribute("onmousedown", onmousedown,"onmousedown");
		
		String onmousemove = null;
		if(lcomponent != null)		
			onmousemove= lcomponent.getOnmousemove();		
		else		
			onmousemove = (String)component.getAttributes().get("onmousemove");
		if(onmousemove != null)
			writer.writeAttribute("onmousemove", onmousemove,"onmousemove");

		String onmouseout = null;
		if(lcomponent != null)		
			onmouseout= lcomponent.getOnmouseout();		
		else		
			onmouseout = (String)component.getAttributes().get("onmouseout");		
		if(onmouseout != null)
			writer.writeAttribute("onmouseout", onmouseout,"onmouseout");

		String onmouseover = null;
		if(lcomponent != null)		
			onmouseover= lcomponent.getOnmouseover();		
		else		
			onmouseover = (String)component.getAttributes().get("onmouseover");		
		if(onmouseover != null)
			writer.writeAttribute("onmouseover", onmouseover,"onmouseover");

		String onmouseup = null;
		if(lcomponent != null)		
			onmouseup= lcomponent.getOnmouseup();		
		else		
			onmouseup = (String)component.getAttributes().get("onmouseup");		
		if(onmouseup != null)
			writer.writeAttribute("onmouseup", onmouseup,"onmouseup");
								//		end of the event section
							// end of the attributes of the button		
	}
	
	public void encodeChildren(FacesContext context, UIComponent component)
			throws IOException {
		Utils.encodeChildren(context,component);
	}
	
	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {
		
		if(context == null || component == null)
			throw new NullPointerException();										 // just exit	
		
		ResponseWriter writer = context.getResponseWriter(); // get a writer
		
		// end of writting the anchor tag
		writer.endElement("a");
	}
	
	public boolean getRendersChildren() {
		return true;
	}
	
	/* 
	 * we process the params for the href 
	 */	
	private String doParameterString(UIComponent component, String hrefStart)
	{
		StringBuffer hrefBuff = new StringBuffer(hrefStart);
		Iterator children = component.getChildren().iterator();
		UIComponent child;
		Boolean queryStarted = Boolean.FALSE;
		while(children.hasNext()){
			child = (UIComponent)children.next();
			if(child instanceof UIParameter){
				if(!queryStarted.booleanValue()){
					hrefBuff.append("?");
					queryStarted = Boolean.TRUE;
				} else {
					hrefBuff.append("&");
				}
				String name = ((UIParameter)child).getName();
				Object value = ((UIParameter)child).getValue();
				String strValue = null;
				if(value != null) {
					strValue = value.toString();
					strValue = strValue.replaceAll("&", "%26");
				}
				hrefBuff.append(name).append("=").append((strValue));
			}
		}
		return hrefBuff.toString();				
	}
	
}
