/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import com.ibm.faces.component.UIPlayer;

/**
 * Renderer for {@link com.ibm.faces.component.UIPlayer UIPlayer} component
 * that creates a Media Player control.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Player_MediaPlayerTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIPlayer}
 */
public final class MediaPlayerRenderer extends PlayerRenderer
{
	// Media Player parameters...
	public static final String PARAM_VOLUME	= "volume"; //$NON-NLS-1$
	public static final String PARAM_BASEURL = "baseURL"; //$NON-NLS-1$
	public static final String PARAM_PLAYCOUNT = "playCount"; //$NON-NLS-1$
	public static final String PARAM_UIMODE	= "uiMode"; //$NON-NLS-1$
	public static final String PARAM_STRETCHTOFIT = "stretchToFit"; //$NON-NLS-1$
	public static final String PARAM_ENABLECONTEXTMENU = "enableContextMenu"; //$NON-NLS-1$
			
	private static final String DEFAULT_CLASSID = "CLSID:22d6f312-b0f6-11d0-94ab-0080c74c7e95"; //$NON-NLS-1$
	private static final String DEFAULT_CODEBASE = "http://activex.microsoft.com/activex/controls/mplayer/en/nsmp2inf.cab#Version=6,4,5,715"; //$NON-NLS-1$
	private static final String DEFAULT_TYPE = "application/x-oleobject"; //$NON-NLS-1$

	private final static Param[] objectParams = {
		new Param("src"  				   		, "fileName"), //$NON-NLS-1$ //$NON-NLS-2$
		new Param(UIPlayer.ATTR_AUTOSTART	   	, "autoStart"), //$NON-NLS-1$
		new Param(PARAM_VOLUME  		   		, "volume"), //$NON-NLS-1$
		new Param(PARAM_BASEURL  			   	, "baseURL"), //$NON-NLS-1$
		new Param(PARAM_PLAYCOUNT  		   		, "playCount"), //$NON-NLS-1$
		new Param(PARAM_UIMODE 	   				, "uiMode"), //$NON-NLS-1$
		new Param(PARAM_STRETCHTOFIT  		   	, "stretchToFit"), //$NON-NLS-1$
		new Param(PARAM_ENABLECONTEXTMENU 		, "enableContextMenu") //$NON-NLS-1$
	};
	
	private final static Param[] pluginParams = {
		new Param("src"  	  		    		, "src"), //$NON-NLS-1$ //$NON-NLS-2$
		new Param(UIPlayer.ATTR_AUTOSTART		, "autoStart"), //$NON-NLS-1$
		new Param(PARAM_VOLUME  		   		, "volume"), //$NON-NLS-1$
		new Param(PARAM_BASEURL  			   	, "baseURL"), //$NON-NLS-1$
		new Param(PARAM_PLAYCOUNT  		   		, "playCount"), //$NON-NLS-1$
		new Param(PARAM_UIMODE 	   				, "uiMode"), //$NON-NLS-1$
		new Param(PARAM_STRETCHTOFIT  		   	, "stretchToFit"), //$NON-NLS-1$
		new Param(PARAM_ENABLECONTEXTMENU 		, "enableContextMenu") //$NON-NLS-1$
	};
	
	private static final String PLUGIN_PAGE = "http://www.microsoft.com/windows/windowsmedia/download/default.asp"; //$NON-NLS-1$
	private static final String PLUGIN_TYPE = "application/x-mplayer2"; //$NON-NLS-1$

	/**
	 * Returns Class ID for Media Player control.
	 * 
	 * @return Class ID
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectClassId()
	 */	
	public String getObjectClassId() {
		return DEFAULT_CLASSID;
	}

	/**
	 * Returns Code Base for Media Player control.
	 * 
	 * @return Code Base
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectCodebase()
	 */
	public String getObjectCodebase() {
		return DEFAULT_CODEBASE;
	}

	/**
	 * Returns type of Media Player control.
	 * 
	 * @return type of Media Player control
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectType()
	 */
	public String getObjectType() {
		return DEFAULT_TYPE;
	}

	/**
	 * Returns a list of parameters for Media Player control.
	 * 
	 * @return array of parameters
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectParams()
	 */
	protected Param[] getObjectParams() {
		return objectParams;
	}

	/**
	 * Returns URL to Media Player's web site.
	 * 
	 * @return URL to Media Player's web site
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginPage()
	 */
	protected String getPluginPage() {
		return PLUGIN_PAGE;
	}

	/**
	 * Returns type of Media Player plug-in.
	 * 
	 * @return type of plugin
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginType()
	 */
	protected String getPluginType() {
		return PLUGIN_TYPE;
	}

	/**
	 * Returns a list of parameters for Media Player plugin.
	 * 
	 * @return array of parameters
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginParams()
	 */
	protected Param[] getPluginParams() {
		return pluginParams;
	}
}
