/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.FacesException;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;
import javax.faces.convert.DateTimeConverter;
import javax.faces.el.ValueBinding;
import javax.faces.render.Renderer;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.IClientValueHolder;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlInputMiniCalendar;
import com.ibm.faces.util.ScriptObject;



/**
 * Renderer for {@link com.ibm.faces.component.html.HtmlInputMiniCalendar InputMiniCalendar} component, 
 * that creates a calendar control.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.InputMiniCalendarTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.html.HtmlInputMiniCalendar}
 */

public class MiniCalendarRenderer extends Renderer implements IClientValueHolder {
	
	protected static final String _INPUT = "_INPUT"; //$NON-NLS-1$
	
	/* (non-Javadoc)
	 * @see javax.faces.render.Renderer#getConvertedValue(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.Object)
	 */
	public Object getConvertedValue(FacesContext context, UIComponent component,
			Object convertee) throws ConverterException {

		Object converted = null;

		if(convertee != null)
		{
			String convertee_str = null;
			
			if(convertee instanceof String)
				convertee_str = (String)convertee;
						
			Converter con = ((UIInput)component).getConverter();
				
			if(con != null)
				converted = con.getAsObject(context, component, convertee_str);
		}
		
		return converted;
	}
	/*
	 * The component passed in there does need to extend UIInput
	 * 
	 * */
	public void decode(FacesContext context, UIComponent component) {
		
		if(context == null)
			throw new NullPointerException();

		if(component == null)
			throw new NullPointerException();		
		
								//	 get the request parameters
		Map paramMap = context.getExternalContext().getRequestParameterMap(); 
								// get the client id
		String clientId = component.getClientId(context);		
		
		String sub_Value = (String)paramMap.get(clientId+_INPUT);// get value on the input
		// should be set if it is null, have someone cleared it, but I'm propably wrong 
		((UIInput)component).setSubmittedValue(sub_Value);
								
	}	
	
	/**
	 * Renders the beginning of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
	}
	
	public boolean getRendersChildren() {
		return false;
	}

	
	/**
	 * Renders the ending of the component.
	 * <p>
	 * Creates the closing div tag.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeEnd(FacesContext context, UIComponent component)throws IOException {

		if (context == null || component == null) {
			throw new NullPointerException();
		}
		
		if(!(component.isRendered()))
			return;
		
		HtmlInputMiniCalendar calendar_comp = null;
		if(component instanceof HtmlInputMiniCalendar)
			calendar_comp = (HtmlInputMiniCalendar)component;
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		HxClientRenderUtil.addScriptLibrary("hxclient_dp.js", component);
		HxClientRenderUtil.addScriptLibrary("hxclient_cal.js", component);
		
		DateTimeConverter dtConverter = (DateTimeConverter)((UIInput)component).getConverter();
		if(dtConverter != null) {
			Locale locale = dtConverter.getLocale();
			HxClientRenderUtil.addLocalizedStringLibraryIfNeeded(locale.toString(), component);			
		}
		
		
		String id = component.getClientId(context);
		
		ResponseWriter writer = context.getResponseWriter();
		
		/*
		 *  Rendered HTML should look like this
		 *  <input id="blaa_input" onchange="func"><div id="blaa"></div>
		 * 
		 * */
		
		String lang = null;
		String dir = null;
		boolean disabled = false;
		
		String user_method_change = null;
		String user_method_focus = null;
		String user_method_blur = null;
		
		String tabindex = null;
		String ackey = null;
		String title = null;
		
		String style = null;
		String styleClass = null; 
		if(calendar_comp != null)
		{
			lang = calendar_comp.getLang();
			dir = calendar_comp.getDir(); 
			disabled = calendar_comp.isDisabled(); 
			
			user_method_change = calendar_comp.getOnchange();
			user_method_focus = calendar_comp.getOnfocus();
			user_method_blur = calendar_comp.getOnblur(); 
			
			tabindex = calendar_comp.getTabindex(); 
			ackey = calendar_comp.getAccesskey(); 
			title = calendar_comp.getTitle();
			
			style = calendar_comp.getStyle(); 
			styleClass = calendar_comp.getStyleClass();
		}
		else
		{
			lang = (String)component.getAttributes().get("lang");
			dir = (String)component.getAttributes().get("dir");
			
			Object bDisabled = component.getAttributes().get("disabled");
			if(bDisabled instanceof String){
				if("true".equalsIgnoreCase(bDisabled.toString()))
					disabled = true;			
			}
			if(bDisabled instanceof Boolean){
				disabled = ((Boolean)bDisabled).booleanValue();
			}
			
			user_method_change = (String)component.getAttributes().get("onchange");
			user_method_focus = (String)component.getAttributes().get("onfocus");
			user_method_blur = (String)component.getAttributes().get("onblur");
			
			tabindex  = (String)component.getAttributes().get("tabindex");
			ackey  = (String)component.getAttributes().get("accesskey");
			title = (String)component.getAttributes().get("title");
			
			style = (String)component.getAttributes().get("style");
			styleClass = (String)component.getAttributes().get("styleClass");
		}

			// START INPUT 
		writer.startElement("input",component);
		writer.writeAttribute("id", id+_INPUT, null);
		writer.writeAttribute("type", "text", null);
		writer.writeAttribute("name", id+_INPUT, null);
		
		// hid the input
		writer.writeAttribute("style", "display:none;", null);
		writer.writeAttribute("autocomplete", "off", null); // need to to this or inout shows up

		if(disabled)
			writer.writeAttribute("disabled", "disabled", null);		

		if(dir != null && dir.length() > 0)
			writer.writeAttribute("dir", dir, null);
		
		if(lang != null && lang.length() > 0)
			writer.writeAttribute("lang", lang, null);

		// EVENTS	
		if((user_method_change != null) &&(user_method_change.length() > 0))
			writer.writeAttribute("onchange", user_method_change, null);

		if((user_method_focus != null) &&(user_method_focus.length() > 0))
			writer.writeAttribute("onfocus", user_method_focus, null);		

		if((user_method_blur != null) &&(user_method_blur.length() > 0))
			writer.writeAttribute("onblur", user_method_blur, null);	
		//a11y
		if((tabindex != null) &&(tabindex.length() > 0))				
			writer.writeAttribute("tabindex", tabindex, null);

		if((ackey != null) &&(ackey.length() > 0))				
			writer.writeAttribute("accesskey", ackey, null);
		
		
		String value = null; //$NON-NLS-1$
		ValueBinding vb = component.getValueBinding("value"); //$NON-NLS-1$
		Object obj = null;
		if (vb != null) 
		{
			obj = vb.getValue(context);
			
		} else 
		{
			if(calendar_comp != null)
				obj = calendar_comp.getValue(); 
			else
				obj = component.getAttributes().get("value");
		}
		
		if (obj instanceof String) 
		{
			value = (String) obj;
		}
		if (obj instanceof Date) 
		{
			if(dtConverter != null)
				value = dtConverter.getAsString( context, component, obj);
		}
		
		
		if((value != null) &&(value.length() > 0))
			writer.writeAttribute("value", value, null);		
				
		writer.endElement("input");

		// END INPUT
		// START DIV
		writer.startElement("div",component);
		writer.writeAttribute("id", id, "id");

		if((style != null) &&(style.length() > 0))
			writer.writeAttribute("style", style, null);
	
		if(disabled)
			writer.writeAttribute("disabled", "disabled", null);
		
		if(dir != null && dir.length() > 0)
			writer.writeAttribute("dir", dir, null);
		
		if(lang != null && lang.length() > 0)
			writer.writeAttribute("lang", lang, null);

		if((title != null) &&(title.length() > 0))
			writer.writeAttribute("title", title, null);			
		
		if((styleClass != null) &&(styleClass.length() > 0))
			writer.writeAttribute("class", styleClass, null);	
				
		writer.endElement("div"); //END of div //$NON-NLS-1$		
		
		doscript(context, component); // this will write out the JS for the calendar
		
	}
	
	/*
	 * This writes the JS script contructor call for JSFCalendar, also get the script for the converter and validator.
	 * Script is written to the scriptcollector, which teh JSFCalendar call going last 
	 * */
	private void doscript(FacesContext context, UIComponent component) {
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		
		DateTimeConverter dtConverter = (DateTimeConverter)((UIInput)component).getConverter();
		
        if(dtConverter == null)
        {
        	ResourceBundle rb = ResourceHandler.getBundle(context);	            	
        	throw new FacesException(ResourceHandler.getString(rb, "MiniCalendarNoConveterFoundError"));
        }
		
		ScriptObject converterScript = HxClientRenderUtil.buildJsDateConverter(context, dtConverter);
		ScriptObject validatorScript = null;
		if (null != converterScript) {
			 // We can't have a validator without a converter so only get the 
			 // validator (as follows) if we've found a converter. 
			validatorScript = HxClientRenderUtil.buildJsRangeValidator(context,
																dtConverter,
																HxClientRenderUtil.getRangeValidator(((UIInput)component)), 
																((UIInput)component).isRequired(), 
																HxClientRenderUtil.getExpressionValidator(((UIInput)component)), null);
		}
		String calendarScript = null;
		
		calendarScript = getCalendarScript(context, component,dtConverter.getLocale(), 
											converterScript!=null?converterScript.getId():null,
											validatorScript!=null?validatorScript.getId():null);

		if (null != converterScript) 
			scriptCollector.addScriptOnce(converterScript.getScript());
		
		if (null != validatorScript) 
			scriptCollector.addScriptOnce(validatorScript.getScript());
		
		if (null != calendarScript) // This shouldn't be null, if it is then something is broken
			scriptCollector.addScript(calendarScript);		
		
	}
	
	/*
	 * This generates the JS script for teh JSFCalendar constructor
	 * */
	private String getCalendarScript(FacesContext context, UIComponent component, Locale converterLocale, String converterId, String validatorId) {
				
		HtmlInputMiniCalendar calendarComponent = null;
		if ( component instanceof HtmlInputMiniCalendar ){
			calendarComponent = (HtmlInputMiniCalendar)component;
		}

		String sRequired = null;
		
		boolean disabled = false;
		
		boolean multiLine = true; // This is the default
		
		String sCSSPrefix = null;	
		
		if(calendarComponent != null)
		{	
			sCSSPrefix = calendarComponent.getStyleClass();	
			multiLine = calendarComponent.isMultiLine();
			disabled = calendarComponent.isDisabled();
			boolean bRequired = calendarComponent.isRequired();
			if (bRequired)
				sRequired = "true"; //$NON-NLS-1$
		}
		else
		{
			Map attrs = component.getAttributes();
			
			sCSSPrefix = (String) attrs.get("styleClass"); //$NON-NLS-1$
			
			Object bDisabled = attrs.get("disabled");
			if(bDisabled instanceof String){
				if("true".equalsIgnoreCase(bDisabled.toString()))
					disabled = true;			
			}
			if(bDisabled instanceof Boolean){
				disabled = ((Boolean)bDisabled).booleanValue();
			}
			
			Object ml = attrs.get("multiLine"); // only where to tell if attribaute was set
			if(ml != null && ml instanceof String){
				if("false".equalsIgnoreCase(ml.toString()))
					multiLine = false;			
			}
			if(ml != null && ml instanceof Boolean){
				multiLine = ((Boolean)ml).booleanValue();
			}
			
			Boolean bRequired = (Boolean) attrs.get("required"); //$NON-NLS-1$
			if (bRequired.booleanValue())
				sRequired = "true"; //$NON-NLS-1$
			
		}		
	
		/* Get a default value of the first day of week - locale sensitive 
		 * Use converter locale if set, otherwise use viewRoot locale
		 * */
		Locale locale;
		if(converterLocale==null){
			locale = context.getViewRoot().getLocale();
		} else {
			locale=converterLocale;
		}
		//Locale locale = context.getViewRoot().getLocale();
		Calendar cal = Calendar.getInstance(locale);
		int DefaultFirstDay = cal.getFirstDayOfWeek();
		DefaultFirstDay--;

			
		String firstDay = null;
	/*	if ( calendarComponent != null )
			firstDay = calendarComponent.getFirstDay();
		else
			firstDay = (String)component.getAttributes().get("firstDay"); //$NON-NLS-1$*/
					
		String sFirstDay =
			getNumericAttribute(
				firstDay, //$NON-NLS-1$
				"first-day-of-week:", //$NON-NLS-1$
				new Integer(DefaultFirstDay).toString(),
				new Integer(0),
				new Integer(6));
						

		StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME) //$NON-NLS-1$
		.append(".addComponent(\"")
		 .append(component.getClientId(context))
		  .append("\", new ")
			.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".JSFCalendar(");  //$NON-NLS-1$//$NON-NLS-2$
		if (disabled) {
			script.append("\"disabled:true\", "); //$NON-NLS-1$			
		}		
		if (null != converterId) {
			script.append("\"converter:"); //$NON-NLS-1$
			script.append(converterId);
			script.append('\"'); //$NON-NLS-1$ 
		}
		if (null != validatorId) {
			script.append(", \""); //$NON-NLS-1$
			script.append("validator:"); //$NON-NLS-1$
			script.append(validatorId);
			script.append('\"'); //$NON-NLS-1$ 
		}
		if (sCSSPrefix != null && sCSSPrefix != "") { //$NON-NLS-1$
			script.append(", \"css-prefix:"); //$NON-NLS-1$
			script.append(sCSSPrefix);
			script.append('\"'); //$NON-NLS-1$ 
		}

		if (sRequired != null && sRequired != "") { //$NON-NLS-1$
			script.append(", \"required:"); //$NON-NLS-1$
			script.append(sRequired);
			script.append('\"'); //$NON-NLS-1$
		}
		
		if (multiLine) { //$NON-NLS-1$ // this is the normal case
			script.append(", \"multi-line:"); //$NON-NLS-1$
			script.append("true");
			script.append('\"'); //$NON-NLS-1$
		}
		else { //$NON-NLS-1$ // this is the special case
			script.append(", \"multi-line:"); //$NON-NLS-1$
			script.append("false");
			script.append('\"'); //$NON-NLS-1$
		}
		
		
		//	no need to check if following attributes have values as defaults are automatically set if not on page
		script.append(", \""); //$NON-NLS-1$
		script.append(sFirstDay);
		script.append('\"'); //$NON-NLS-1$ 
		script.append("));"); //$NON-NLS-1$
		return script.toString();
	}
	
	/**
	 * Fetch, validate and transform a numeric component attribute embedded in a string
	 * @param component		- JSF component holding the requested attribute
	 * @param sAttInName	- name of JSF attribute, e.g. ShowTabs
	 * @param sAttOutName	- "show-tabs:"
	 * @param DefaultValue	- value to be assigned if nothing is found
	 * @param MinValue		- smallest valid value, if any
	 * @param MaxValue		- largest valid value, if any
	 * @return 				- e.g. "show-tabs:1"
	 */
	// TODO MWD: abstract to utility class so any renderer can use this
	private String getNumericAttribute(String sValue, String sAttOutName,String DefaultValue, Integer MinValue, Integer MaxValue) {

		if (sValue == null || sValue == "") //$NON-NLS-1$
			sValue = DefaultValue;
		else {
			try {
				int iValue = Integer.parseInt(sValue);
				if (MinValue != null && iValue < MinValue.intValue())
					sValue = DefaultValue;
				else if (MaxValue != null && iValue > MaxValue.intValue())
					sValue = DefaultValue;
				} catch (NumberFormatException e){
					sValue = DefaultValue;
			}
		}		
		sValue = sAttOutName + sValue; 
		return sValue;
	}
	
	/**
	 * Returns the id of the input field that keeps the value.
	 */
	public String getValueHolderClientId(FacesContext context, UIComponent component) {
		String id = component.getClientId(context);
		return id + _INPUT;
	}

}
