/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlProgressBar;

import com.ibm.faces.util.ScriptObject;
import com.ibm.faces.converter.NumberConverterEx;

/**
 * Renderer for JWL progress bar component. 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.InputHelper_DatePickerTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIInputHelper}
 */

public class ProgressBarRenderer extends Renderer  /*implements IScriptContributor*/ {
	
	final private String DEFAULT_TIME = "500";
	final private String DEFAULT_PERCENT = "5";
	final private String DEFAULT_START_PERCENT = "0";
	/**
	 * Renders the beginning of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}

		if(!(component.isRendered())) // if you're no meant to see anything at all.
			return;
		
		HxClientRenderUtil.addScriptLibrary("hxclient_pbar.js", component);
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		
		HtmlProgressBar prog_bar = null;
		if(component instanceof HtmlProgressBar)
			prog_bar = (HtmlProgressBar)component;
		
		String id = component.getClientId(context);
		
		boolean init_hidden = false;
		String dir = null;
		String lang= null;
		String title = null;
		String style = null;
		String styleClass = null;
		
		if(prog_bar != null)
		{
			init_hidden = prog_bar.isInitHidden();
			dir = prog_bar.getDir();
			lang = prog_bar.getLang();
			title = prog_bar.getTitle();
			style = prog_bar.getStyle(); 
			styleClass = prog_bar.getStyleClass(); 
		}
		else
		{
			Object a = component.getAttributes().get("initHidden");
			if(a instanceof String){
				if("true".equalsIgnoreCase(a.toString()))
					init_hidden = true;			
			}
			if(a instanceof Boolean){
				init_hidden = ((Boolean)a).booleanValue();
			}	
			
			dir = (String)component.getAttributes().get("dir");
			lang = (String)component.getAttributes().get("lang");
			title = (String)component.getAttributes().get("title");
			style = (String)component.getAttributes().get("style");
			styleClass = (String)component.getAttributes().get("styleClass");
		}
		
		
		
		ResponseWriter writer = context.getResponseWriter();
		
		/*
		 *  Rendered HTML should look like this
		 *  <div id="blaa_input" style="visibility:hidden"></div>
		 * 
		 * */
		
		
		writer.startElement("div",component);
		writer.writeAttribute("id", id, null);

		if(title != null && title.length() > 0)
			writer.writeAttribute("title", title, null);	
		
		if(dir != null && dir.length() > 0)
			writer.writeAttribute("dir", dir, null);	

		if(lang != null && lang.length() > 0)
			writer.writeAttribute("lang", lang, null);	

		
		if(init_hidden)
		{
			if((style != null) &&(style.length() > 0))
			{
				style = style +	";display:none;";
				writer.writeAttribute("style", style, null);
			}
			else
				writer.writeAttribute("style", "display:none;", null);
			
		}
		else
		{
			if((style != null) &&(style.length() > 0))
			{
				style = style +	";display:block;";
				writer.writeAttribute("style", style, null);
			}
			else
				writer.writeAttribute("style", "display:block;", null);			
			
		}
		
		if((styleClass != null) &&(styleClass.length() > 0))
			writer.writeAttribute("class", styleClass, null);		
		
		writer.endElement("div");
		
	}


	public String contributeScript(FacesContext context, UIComponent component, UIScriptCollector scriptCollector) {

		HtmlProgressBar prog_bar = null;
		
		if(component instanceof HtmlProgressBar)
			prog_bar = (HtmlProgressBar)component;
	
		boolean auto = true;
		//String mode = null;
		boolean outward = false;
		String sCSSPrefix = null;
		String timeInterval = null;		
		String message = null; 
		String proportion = null;
		String initvalue = null;

		
		if(prog_bar != null)
		{
			auto = prog_bar.isAuto();
			
			//mode = prog_bar.getMode();
			
			outward = prog_bar.isOutward();
			sCSSPrefix = prog_bar.getStyleClass();
			timeInterval = prog_bar.getTimeInterval();
			message = prog_bar.getMessage();
			proportion = prog_bar.getProportion();			
			initvalue = prog_bar.getInitValue();

		}
		else
		{
			Object a = component.getAttributes().get("auto");
			if(a instanceof String){
				if("true".equalsIgnoreCase(a.toString()))
					auto = true;			
			}
			if(a instanceof Boolean){
				auto = ((Boolean)a).booleanValue();
			}

			//mode = (String)component.getAttributes().get("mode");
			
			Object b = component.getAttributes().get("outward");
			if(b instanceof String){
				if("true".equalsIgnoreCase(b.toString()))
					outward = true;			
			}
			if(b instanceof Boolean){
				outward = ((Boolean)b).booleanValue();
			}			
			
			sCSSPrefix = (String)component.getAttributes().get("styleClass");
			timeInterval = (String)component.getAttributes().get("timeInterval");
			message = (String)component.getAttributes().get("message");
			proportion = (String)component.getAttributes().get("proportion");
			initvalue = (String)component.getAttributes().get("initValue");

		}
		
		
	/*
	 * example script
	 * hX_4.addComponent("form1:pb1", new hX_4.JSFProgressBar("css-prefix:progressBar", "normal:false", 
	 * "outward:true", "itu:900", "defaultmessage:is this going to work", "defaultpercent:20"));
	 * */
		
		StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME) //$NON-NLS-1$
		.append(".addComponent(\"")
		 .append(component.getClientId(context))
		  .append("\", new ")
			.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".JSFProgressBar(");  //$NON-NLS-1$//$NON-NLS-2$

		if (!auto) { //$NON-NLS-1$
		//if(mode == null || (mode != null && (mode.trim().length()==0 || "percent".equalsIgnoreCase(mode))))
		//{
			script.append("\"normal:"); //$NON-NLS-1$
			script.append("false");
			script.append('\"'); //$NON-NLS-1$
		}
		else
		{ //$NON-NLS-1$
			script.append("\"normal:"); //$NON-NLS-1$
			script.append("true");
			script.append('\"'); //$NON-NLS-1$
		}

		if (sCSSPrefix != null && sCSSPrefix != "") { //$NON-NLS-1$
			script.append(", \"css-prefix:"); //$NON-NLS-1$
			script.append(sCSSPrefix);
			script.append('\"'); //$NON-NLS-1$ 
		}		
		
		if (outward) { //$NON-NLS-1$
			script.append(", \"outward:"); //$NON-NLS-1$
			script.append(true);
			script.append('\"'); //$NON-NLS-1$
		}
		
		if (timeInterval != null && timeInterval != "") { //$NON-NLS-1$
			script.append(", \"itu:"); //$NON-NLS-1$
			if(auto && (timeInterval.trim() == ""))	
				timeInterval = DEFAULT_TIME;
			script.append(timeInterval);
			script.append('\"'); //$NON-NLS-1$
		}

		if (message != null && message != "") { //$NON-NLS-1$
			script.append(", \"defaultmessage:"); //$NON-NLS-1$
			script.append(message);
			script.append('\"'); //$NON-NLS-1$
		}			

		if(!auto)
		//if(mode == null || (mode != null && (mode.trim().length() == 0 || "percent".equalsIgnoreCase(mode))))
		{
			if (initvalue != null && initvalue != "") { //$NON-NLS-1$
				script.append(", \"startpercent:"); //$NON-NLS-1$
				if(!auto && (initvalue.trim() == ""))	
					initvalue = DEFAULT_START_PERCENT;	
				script.append(initvalue);
				script.append('\"'); //$NON-NLS-1$
			}
			
			NumberConverterEx numcon = HxClientRenderUtil.getDefaultNumberConverter();
			
			numcon.setPattern("##0%");
			
			ScriptObject converter = HxClientRenderUtil.buildJsNumberConverter(context,numcon);
						
			script.append(", \"numconverterid:");
			script.append(converter.getId());
			script.append('\"'); //$NON-NLS-1$
			
			scriptCollector.addScriptOnce(converter.getScript()); // so this so this converter could be reused
			
		}
		else
		{
			if (proportion != null && proportion != "") { //$NON-NLS-1$
				script.append(", \"defaultpercent:"); //$NON-NLS-1$
				if(auto && (proportion.trim() == ""))	
					proportion = DEFAULT_PERCENT;			
				script.append(proportion);
				script.append('\"'); //$NON-NLS-1$
			}		
		}	
		script.append("));\n"); //$NON-NLS-1$
		
		return script.toString();
	}
	
    
    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        //defect 280926
		if (context == null || component == null) {
			throw new NullPointerException();
		}

		if(!(component.isRendered())) // if you're no meant to see anything at all.
			return;
		
			
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		// we need to output our javascript library
		scriptCollector.addScript(contributeScript(context, component, scriptCollector));
    }

}
