/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import com.ibm.faces.component.UIPlayer;

/**
 * Renderer for {@link com.ibm.faces.component.UIPlayer UIPlayer} component, 
 * that creates a Real One control.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Player_RealPlayerTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIPlayer}
 */
public final class RealPlayerRenderer extends PlayerRenderer
{
	// Real parameters, as named in jsf attributes
	public static final String PARAM_BACKGROUNDCOLOR  	= "backgroundcolor"; //$NON-NLS-1$
	public static final String PARAM_CONTROLS  			= "controls"; //$NON-NLS-1$
	public static final String PARAM_CONSOLE  			= "console"; //$NON-NLS-1$
	public static final String PARAM_CENTER  			= "center"; //$NON-NLS-1$
	public static final String PARAM_LOOP	    		= "loop"; //$NON-NLS-1$
	public static final String PARAM_SHUFFLE	    	= "shuffle"; //$NON-NLS-1$
	
	//rendered as object tag parameters
	private final static Param[] objectParams = {
		new Param("src"						,"SRC"), //$NON-NLS-1$ //$NON-NLS-2$
		new Param(UIPlayer.ATTR_AUTOSTART	,"AUTOSTART"), //$NON-NLS-1$
		new Param(PARAM_SHUFFLE	    		,"SHUFFLE"), //$NON-NLS-1$
		new Param(PARAM_LOOP	    		,"LOOP"), //$NON-NLS-1$
		new Param(PARAM_CENTER	   			,"CENTER"), //$NON-NLS-1$
		new Param(PARAM_BACKGROUNDCOLOR 	,"BACKGROUNDCOLOR"), //$NON-NLS-1$
		new Param(PARAM_CONTROLS			,"CONTROLS"),//$NON-NLS-1$
		new Param(PARAM_CONSOLE				,"CONSOLE"),//$NON-NLS-1$
	};
	//rendered as enlcosed embed tag attributes
	private final static Param[] pluginParams = {
		new Param("src"						,"src"), //$NON-NLS-1$ //$NON-NLS-2$
		new Param(UIPlayer.ATTR_AUTOSTART	,"autostart"), //$NON-NLS-1$
		new Param(PARAM_BACKGROUNDCOLOR 	,"backgroundcolor"), //$NON-NLS-1$
		new Param(PARAM_CENTER	   			,"center"), //$NON-NLS-1$
		new Param(PARAM_SHUFFLE	    		,"shuffle"), //$NON-NLS-1$
		new Param(PARAM_LOOP	    		,"loop"), //$NON-NLS-1$
		new Param(PARAM_CONTROLS			,"controls"),//$NON-NLS-1$
		new Param(PARAM_CONSOLE				,"console"),//$NON-NLS-1$
	};
		
	private static final String DEFAULT_CLASSID = "clsid:CFCDAA03-8BE4-11CF-B84B-0020AFBBCCFA"; //$NON-NLS-1$
	private static final String DEFAULT_CODEBASE = null; //$NON-NLS-1$
	private static final String DEFAULT_PLUGINPAGE  = null; //$NON-NLS-1$
	private static final String DEFAULT_PLUGINTYPE = null; //$NON-NLS-1$

	/**
	 * Returns Class ID for Real control.
	 * 
	 * @return Class ID
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectClassId()
	 */	
	public String getObjectClassId() {
		return DEFAULT_CLASSID;
	}

	/**
	 * Returns Code Base for Real control.
	 * 
	 * @return Code Base
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectCodebase()
	 */
	public String getObjectCodebase() {
		return DEFAULT_CODEBASE;
	}

	/**
	 * Returns a list of parameters for Real control.
	 * 
	 * @return array of parameters
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectParams()
	 */
	protected Param[] getObjectParams() {
		return objectParams;
	}

	/**
	 * Returns URL to Real's web site
	 * 
	 * @return URL to Real's web site
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginPage()
	 */
	protected String getPluginPage() {
		return DEFAULT_PLUGINPAGE;
	}

	/**
	 * Returns type of Real plug-in
	 * 
	 * @return type of plugin
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginType()
	 */
	protected String getPluginType() {
		return DEFAULT_PLUGINTYPE;
	}

	/**
	 * Returns a list of parameters for Real plugin.
	 * 
	 * @return array of parameters
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginParams()
	 */
	protected Param[] getPluginParams() {
		return pluginParams;
	}

	/**
	 * Returns default control height.
	 * 
	 * @return height
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getDefaultHeight()
	 */
	protected String getDefaultHeight() {
		return "100"; //$NON-NLS-1$
	}

	/**
	 * Returns default control width.
	 * 
	 * @return width
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getDefaultWidth()
	 */
	protected String getDefaultWidth() {
		return "300"; //$NON-NLS-1$
	}

}
