/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.component.UIRequest;
import com.ibm.faces.component.html.HtmlRequestLink;


public class RequestLinkRenderer extends Renderer{
	
	

	public boolean getRendersChildren() {
		return true;
	}
	
	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {
		
		HtmlRequestLink lcomponent = null;
		if(component instanceof HtmlRequestLink){
			lcomponent = (HtmlRequestLink) component;
		}
		
		ResponseWriter writer = context.getResponseWriter();
		
		writer.startElement("a",component);
		writer.writeAttribute("id",component.getClientId(context),"id");
		
		//generate the url for href attribute
		String href = ((UIRequest)component).generateRequestURL(context);
		writer.writeURIAttribute("href",href,null);
		
		//write other attributes
		String style = null;
		if(lcomponent != null)
			style = lcomponent.getStyle();
		else		
			style = (String)component.getAttributes().get("style");				
		if(style!= null)
			writer.writeAttribute("style", style.trim(),"style");		

		String styleClass = null;
		if(lcomponent != null)
			styleClass = lcomponent.getStyleClass();
		else		
			styleClass = (String)component.getAttributes().get("styleClass");				
		if(styleClass!= null)
			writer.writeAttribute("class", styleClass,"class");	

		String dir = null;
		if(lcomponent != null)		
			dir= lcomponent.getDir();		
		else		
			dir = (String)component.getAttributes().get("dir");		
		if(dir != null)
			writer.writeAttribute("dir", dir,"dir");

		String lang = null;
		if(lcomponent != null)		
			lang= lcomponent.getLang();		
		else		
			lang = (String)component.getAttributes().get("lang");			
		if(lang != null)
			writer.writeAttribute("lang", lang,"lang");

		String title = null;
		if(lcomponent != null)		
			title = lcomponent.getTitle();		
		else		
			title = (String)component.getAttributes().get("title");			
		if(title != null)
			writer.writeAttribute("title", title,"title");

		String tabindex = null;
		if(lcomponent != null)		
			tabindex= lcomponent.getTabindex();		
		else		
			tabindex = (String)component.getAttributes().get("tabindex");			
		if(tabindex != null)
			writer.writeAttribute("tabindex", tabindex,"tabindex");

		String accesskey = null;
		if(lcomponent != null)		
			accesskey= lcomponent.getAccesskey();		
		else		
			accesskey = (String)component.getAttributes().get("accesskey");			
		if(accesskey != null)
			writer.writeAttribute("accesskey", accesskey,"accesskey");
									 

		String charset = null;
		if(lcomponent != null)		
			charset= lcomponent.getCharset();		
		else		
			charset = (String)component.getAttributes().get("charset");			
		if(charset != null)
			writer.writeAttribute("charset", charset,"charset");

		String hreflang = null;
		if(lcomponent != null)		
			hreflang= lcomponent.getHreflang();		
		else		
			hreflang = (String)component.getAttributes().get("hreflang");			
		if(hreflang != null)
			writer.writeAttribute("hreflang", hreflang,"hreflang");

		String target = null;
		if(lcomponent != null)		
			target = lcomponent.getTarget();		
		else		
			target = (String)component.getAttributes().get("target");			
		if(target != null)
			writer.writeAttribute("target", target,"target");

									
		//		 start of the event section
		String onblur = null;
		if(lcomponent != null)		
			onblur= lcomponent.getOnblur();		
		else		
			onblur = (String)component.getAttributes().get("onblur");			
		if(onblur != null)
			writer.writeAttribute("onblur", onblur,"onblur");

		String onclick = null;
		if(lcomponent != null)		
			onclick= lcomponent.getOnclick();
		else
			onclick = (String)component.getAttributes().get("onclick");		
		if(onclick != null)
			writer.writeAttribute("onclick", onclick,"onclick");
		
		String ondblclick = null;
		if(lcomponent != null)		
			ondblclick= lcomponent.getOndblclick();
		else
			ondblclick = (String)component.getAttributes().get("ondblclick");		
		if(ondblclick != null)
			writer.writeAttribute("ondblclick", ondblclick,"ondblclick");
		
		String onfocus = null;
		if(lcomponent != null)		
			onfocus= lcomponent.getOnfocus();		
		else		
			onfocus = (String)component.getAttributes().get("onfocus");
		if(onfocus != null)
			writer.writeAttribute("onfocus", onfocus,"onfocus");
		
		String onkeydown = null;
		if(lcomponent != null)		
			onkeydown= lcomponent.getOnkeydown();		
		else		
			onkeydown = (String)component.getAttributes().get("onkeydown");
		if(onkeydown != null)
			writer.writeAttribute("onkeydown", onkeydown,"onkeydown");
		
		String onkeypress = null;
		if(lcomponent != null)		
			onkeypress= lcomponent.getOnkeypress();		
		else		
			onkeypress = (String)component.getAttributes().get("onkeypress");
		if(onkeypress != null)
			writer.writeAttribute("onkeypress", onkeypress,"onkeypress");
		
		String onkeyup = null;
		if(lcomponent != null)		
			onkeyup= lcomponent.getOnkeyup();		
		else		
			onkeyup = (String)component.getAttributes().get("onkeyup");
		if(onkeyup != null)
			writer.writeAttribute("onkeyup", onkeyup,"onkeyup");
		
		String onmousedown = null;
		if(lcomponent != null)		
			onmousedown= lcomponent.getOnmousedown();		
		else		
			onmousedown = (String)component.getAttributes().get("onmousedown");		
		if(onmousedown != null)
			writer.writeAttribute("onmousedown", onmousedown,"onmousedown");
		
		String onmousemove = null;
		if(lcomponent != null)		
			onmousemove= lcomponent.getOnmousemove();		
		else		
			onmousemove = (String)component.getAttributes().get("onmousemove");
		if(onmousemove != null)
			writer.writeAttribute("onmousemove", onmousemove,"onmousemove");

		String onmouseout = null;
		if(lcomponent != null)		
			onmouseout= lcomponent.getOnmouseout();		
		else		
			onmouseout = (String)component.getAttributes().get("onmouseout");		
		if(onmouseout != null)
			writer.writeAttribute("onmouseout", onmouseout,"onmouseout");

		String onmouseover = null;
		if(lcomponent != null)		
			onmouseover= lcomponent.getOnmouseover();		
		else		
			onmouseover = (String)component.getAttributes().get("onmouseover");		
		if(onmouseover != null)
			writer.writeAttribute("onmouseover", onmouseover,"onmouseover");

		String onmouseup = null;
		if(lcomponent != null)		
			onmouseup= lcomponent.getOnmouseup();		
		else		
			onmouseup = (String)component.getAttributes().get("onmouseup");		
		if(onmouseup != null)
			writer.writeAttribute("onmouseup", onmouseup,"onmouseup");
								//		end of the event section
							// end of the attributes of the button		

		
		
	}

	public void encodeChildren(FacesContext context, UIComponent component)
			throws IOException {
		//check if a value has been set, if output as part of the link label
		String label = (String)((UIRequest)component).getValue();
		if(label != null){
			context.getResponseWriter().writeText(label,"value");
		}
		//write child components
		Utils.encodeChildren(context,component);
	}

	
	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {
		//just end anchor tag
		context.getResponseWriter().endElement("a");
	}
}
