/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.StringTokenizer;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.render.Renderer;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlBehaviorRightMouse;



public class RightMouseRenderer extends Renderer implements IScriptContributor {

	
	public void encodeEnd(FacesContext context, UIComponent component) throws IOException{

		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
		
		if(HxClientRenderUtil.isContainedInPanel(component)){
			//rendered by parent
			encode(context,component,false);
		} else {
			scriptCollector.register(this,component);
		}

	}
	
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		encode(context,component,true);
	}
	
	private void encode(FacesContext context, UIComponent component, boolean writeNow) {
		HtmlBehaviorRightMouse clickComponent = null;
		if ( component instanceof HtmlBehaviorRightMouse ){
			clickComponent = (HtmlBehaviorRightMouse)component;
		}
		
		
		String target, action, targetAction, function;
		if ( clickComponent != null ){
			target = clickComponent.getTarget();
			action = clickComponent.getBehaviorAction();
			targetAction = clickComponent.getTargetAction();
			function = clickComponent.getOnActionFunction();
		}else{
			target =  (String)component.getAttributes().get("target"); //$NON-NLS-1$
			action = (String)component.getAttributes().get("behaviorAction"); //$NON-NLS-1$
			targetAction = (String)component.getAttributes().get("targetAction"); //$NON-NLS-1$
			function = (String)component.getAttributes().get("onActionFunction"); //$NON-NLS-1$
		}
		
		UIComponent forComponent = null;
		if(target != null){
			if(!(target.startsWith("#"))){
				forComponent = component.findComponent(target);
				if(forComponent == null){
					//try from view root
					forComponent = Utils.findComponent(context.getViewRoot(),target);
				}
				if(forComponent != null){
					target = forComponent.getClientId(context);
				}
				//if null leave as specified in attribute, may refer to non-jsf html tag
			}
			
			
		} else {
			//target is null so apply to parent component
			forComponent = component.getParent();
			if(forComponent != null){
				target = forComponent.getClientId(context);
			} else {
				//we don't have a component to apply this click to, end it all
				return;
			}
		}
		
		if ( targetAction != null) {
			targetAction = HxClientRenderUtil.convertComponentIdsToClientIds(targetAction, component);
		}
		
		StringBuffer script = new StringBuffer();
		
		script.append("\n"); //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".addBehavior(\"");
		script.append(target);
		script.append("\", \"onmouse\", new "); //$NON-NLS-1$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".JSFBehaviorMouseclick(");
		script.append("\"scope:right\""); //$NON-NLS-1$
		if (null != action) {
			script.append(", \"action:"); //$NON-NLS-1$
			script.append(action);
			script.append("\"");
		}
		if (null != targetAction) {
			script.append(", \"target:"); //$NON-NLS-1$
			script.append(targetAction);
			script.append("\"");
		}
		if (null != function) {
			script.append(", \"function:"); //$NON-NLS-1$
			script.append(function);
			script.append("\"");
		}
		script.append("));"); //$NON-NLS-1$

		
		//add script
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		if(writeNow){
			scriptCollector.writeScriptOnce(script.toString(),context,false);
		} else {
			scriptCollector.addScriptOnce(script.toString());
		}

	}
}
