/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.io.Writer;
import java.util.Map;

import javax.faces.component.UICommand;
import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.event.ActionEvent;
import javax.faces.event.PhaseId;

import com.ibm.faces.component.UICommandEx;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.util.HtmlUtil;
import com.ibm.faces.util.LifecycleUtil;

/**
 * @author Alasdair McDowell
 */
public class RowActionRenderer extends AbstractRowRenderer{

    private final static String PAD_HIDDEN_ACTION = "__hidden_action"; //$NON-NLS-1$
    private final static String CLICKED = "true"; //$NON-NLS-1$
    private final static String UNCLICKED = "false"; //$NON-NLS-1$

    /**
     * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }

        String id = component.getClientId(context);

        //wrap contents in span for JS access
        ResponseWriter writer = context.getResponseWriter();
        HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
        HtmlUtil.writeTagStartCloseLn(writer, false);

        this.encodeParameters(context, component);

        //HtmlUtil.writeInputHidden(writer, id + PAD_HIDDEN_ACTION, UNCLICKED); //$NON-NLS-1$
        //RATLC01070102 - writing hidden input as disabled, javascript will enable when row clicked 
        writer.startElement("input",component);
        writer.writeAttribute("type","hidden",null);
        writer.writeAttribute("id",id + PAD_HIDDEN_ACTION,null);
        writer.writeAttribute("name",id + PAD_HIDDEN_ACTION,null);
        writer.writeAttribute("value",UNCLICKED,null);
        writer.writeAttribute("disabled",Boolean.TRUE,null);
        writer.endElement("input");
        //RATLC01070102 end 
        
        StringBuffer script = new StringBuffer();
		script.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".addBehavior(")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".getParentIdByType(\"")
		.append(id).append("\",\"TR")
		.append("\"),\"onclick\",new ")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".JSFBehaviorRowAction(\"input-id:")
		.append(id).append(PAD_HIDDEN_ACTION)
		.append("\",\"input-value:")
		.append(CLICKED)
		.append("\"));");
		
        //output script
        UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
        HxClientRenderUtil.addScriptLibrary("hxclient_table.js",component);
		scriptCollector.addScript(script.toString());
    }

    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        Writer writer = context.getResponseWriter();
        //close span
        HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
    }

    /**
     * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     */
    public void decode(FacesContext context, UIComponent component) {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }

        //1st, was this row clicked?
        String id = component.getClientId(context);
        String key = id + PAD_HIDDEN_ACTION;
        Map requestMap = context.getExternalContext().getRequestParameterMap();
        if (CLICKED.equals(requestMap.get(key))) {
            ActionEvent event = new ActionEvent(component);
            event.setPhaseId(PhaseId.INVOKE_APPLICATION);
            component.queueEvent(event);

            if (component instanceof UICommandEx) {
                UIData data = findUIData(component);
                if (data != null) {
                    ((UICommandEx) component).setRowIndex(data.getRowIndex());
                }
            }

            //2nd, if it was, decode any param attributes
            UIParameter[] params = LifecycleUtil.getParameters(component);
            if (params != null && params.length > 0) {
                if (component instanceof UICommand) {
                    UICommand command = (UICommand) component;
                    Object value = (Map) command.getValue();
                    Map map = null;
                    if (value instanceof Map) {
                        map = (Map) value;
                        map.clear();
                        map = this.decodeParameters(context, component, map);
                        if(map!=null) {
                            command.setValue(map);
                        }
                    }

                    if (component instanceof UICommandEx) {
                        if (map == null) {
                            map = this.decodeParameters(context, component);
                        }
                        ((UICommandEx) component).setRowParameters(map);
                    }
                }
            }
        }

    }
}
