/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2004 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
 
 
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.faces.component.UIColumn;
import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.component.UIOutput;
import javax.faces.component.UIParameter;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.base.PanelRowCategory;
import com.ibm.faces.component.html.HtmlPanelRowCategory;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;

/**
 * @author pandrews
 */

public class RowCategoryRenderer extends Renderer {

    public static final String CATEGORY_COLUMNS_ARRAY = "CategoryColumnsArray"; // $NON-NLS-1$
    public final String CATEGORY_DATA_KEY = "categoryData";
	private final String DEFAULT_STYLE="font-weight:bold"; //$NON-NLS-1$
	private final String NO_STYLE_CLASS="NO_STYLE_CLASS"; //$NON-NLS-1$
	private final String COLLAPSED_TRUE="display:none"; //$NON-NLS-1$
	private final String EXPANDED_MARKER = "_e"; //$NON-NLS-1$
	
	public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {
	
		if ((context == null) || (component == null)) {
			throw new NullPointerException();
		}
		
		//add scriptcollector
		UIScriptCollector scriptCollector=UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component); //$NON-NLS-1$
		HxClientRenderUtil.addScriptLibrary("hxclient_table.js", component); //$NON-NLS-1$
	}
    
	public void encodeEnd(FacesContext context, UIComponent component)
		throws IOException {
		
		Map attrsMap = component.getAttributes();
		
		HtmlPanelRowCategory rowComponent = null;
		if ( component instanceof HtmlPanelRowCategory ){
			rowComponent = (HtmlPanelRowCategory)component;
		}
		

//      get the number of columns, used to set colspan attributes
        int columnCount=getParentTableColumnCount(component);
//      get the index of the column this rowCategory component is belonged to.
        int columnIndex = getColumnIndex(component);
		String categoryString=null;
		UIData table=getParentTable(component);
        boolean firstCategoryColumn = isFirstCategoryColumn(columnIndex, table);
		
        CategoryData categoryData =((CategoryData)component.getAttributes().get(CATEGORY_DATA_KEY)); //$NON-NLS-1$
        if (categoryData == null){
        	categoryData = new CategoryData();
        	categoryData.init(table);
        	component.getAttributes().put(CATEGORY_DATA_KEY, categoryData);
        } else if (categoryData.allRowsProcessed
				&& table.getRowIndex() == table.getFirst()) {
			categoryData.init(table);
			if (firstCategoryColumn) {
				table.getAttributes().remove(CATEGORY_COLUMNS_ARRAY);
			}
		}

		String collapsed;
		String styleString;
		String styleClassString;
		String collapsedImageSrc;
		String expandedImageSrc;
		String tabindex;
		String alt, dir, lang, title;
        
		if ( rowComponent != null ){
			collapsed = rowComponent.getCollapsed();
			styleString = rowComponent.getStyle();
			styleClassString = rowComponent.getStyleClass();
			collapsedImageSrc = rowComponent.getCollapsedImage();
			expandedImageSrc = rowComponent.getExpandedImage();
			tabindex = rowComponent.getTabindex();
			if ( tabindex == null ){
				//check deprecated tabindexStartAt
				tabindex=rowComponent.getTabindexStartAt();
			}
			alt=rowComponent.getAlt();
			if(alt == null){
				alt=ResourceHandler.getBundle(context).getString("DatagridCategoryRenderer.twistieAlt"); //$NON-NLS-1$
				rowComponent.setAlt(alt);
			}
			dir = rowComponent.getDir();
			lang = rowComponent.getLang();
			title = rowComponent.getTitle();
		}else{
			collapsed = (String)attrsMap.get("collapsed"); //$NON-NLS-1$
			styleString = (String)attrsMap.get("style"); //$NON-NLS-1$
			styleClassString = (String)attrsMap.get("styleClass"); //$NON-NLS-1$
			collapsedImageSrc = (String)attrsMap.get("collapsedImage"); //$NON-NLS-1$
			expandedImageSrc = (String)attrsMap.get("expandedImage"); //$NON-NLS-1$
			tabindex = (String)attrsMap.get("tabindex"); //$NON-NLS-1$
			if ( tabindex == null ){
				//check deprecated tabindexStartAt
				tabindex = (String)attrsMap.get("tabindexStartAt"); //$NON-NLS-1$
			}
			alt = (String)attrsMap.get("alt"); //$NON-NLS-1$
			if(alt == null){
				alt=ResourceHandler.getBundle(context).getString("DatagridCategoryRenderer.twistieAlt"); //$NON-NLS-1$
				attrsMap.put("alt",alt); //$NON-NLS-1$
			}
			dir = (String)attrsMap.get("dir"); //$NON-NLS-1$
			lang = (String)attrsMap.get("lang"); //$NON-NLS-1$
			title = (String)attrsMap.get("title"); //$NON-NLS-1$

		}
		
		String collapsedStyle = null;
		
		if( collapsed == null){
			collapsedStyle=COLLAPSED_TRUE;
		}else if (("true").equalsIgnoreCase(collapsed)) { //$NON-NLS-1$
			collapsedStyle=COLLAPSED_TRUE;
		} else {
			collapsedStyle=""; //$NON-NLS-1$
		}
		
		if( styleString == null ){
			styleString=DEFAULT_STYLE;
		}
		
		if( styleClassString == null ){
			styleClassString=NO_STYLE_CLASS;
		}
		
		categoryString=getCategoryString(component);
		if(categoryString==null) {
			System.err.println("Cannot resolve row category value."); //$NON-NLS-1$
			return;
		}

		ResponseWriter writer=context.getResponseWriter();
        //Designer refactor (getExternalContext was being called a number of times, so using this instead
        ExternalContext externalContext = context.getExternalContext();

		
		//check if the categoryString already exists, if it doesn't add a new category
		//if it does put it under an already existing twistie, if not add it to the list
		boolean categoryExists=false;
			
        //Designer refactoring: moved the check for category exists out into a separate method.
        categoryExists = isCategoryExists(component, context, categoryString, categoryData.categoryList, table, columnCount, columnIndex);
        
		if(!categoryExists) {
        	categoryData.categoryList.add(categoryString);
		}
		
		//finish off row containing the category component, can't do anything with this!
		//need to hide the containing <tr> to keep consistent spacing between categories
		//id supplied for the script added in the scriptcollector to work on.
        String rowId=externalContext.encodeNamespace(component.getId()+"_hide_" + categoryData.currentRow); //$NON-NLS-1$

		
		writer.endElement("td"); //$NON-NLS-1$
		
		//RATLC01129389
		//for first row only write out cells with width set to maintain widths in split tables, spanning breaks layout
        String sizer = externalContext.encodeNamespace(component.getId() + "_catsize"); //$NON-NLS-1$

        if(externalContext.getRequestMap().get(sizer) == null){
			Iterator cols = getColumnList(component).iterator();
			//already rendered the first one so skip
			UIComponent col = null;
			String colwidth = null;
			Boolean idWritten = Boolean.FALSE;
			if(cols.hasNext()){
				col = (UIComponent) cols.next();	
			}
			while(cols.hasNext()){
				col = (UIComponent)cols.next();
				writer.startElement("td",component); //$NON-NLS-1$
				if(!idWritten.booleanValue()){
					writer.writeAttribute("id",sizer,null); //$NON-NLS-1$
					idWritten = Boolean.TRUE;
				}
				colwidth = (String)col.getAttributes().get("width"); //$NON-NLS-1$
				if(colwidth != null){
					writer.writeAttribute("width",colwidth,null); //$NON-NLS-1$
				}
				writer.endElement("td"); //$NON-NLS-1$
			}
            externalContext.getRequestMap().put(sizer,"set"); // $NON-NLS-1$
			
		} else {
			//render spanned row (will be hidden with script)
			writer.startElement("td",component); //$NON-NLS-1$
			writer.writeAttribute("id",rowId,null); //$NON-NLS-1$
			writer.writeAttribute("colspan",String.valueOf(columnCount-1),null); //$NON-NLS-1$
			writer.endElement("td"); //$NON-NLS-1$
		}
		
		writer.endElement("tr"); //$NON-NLS-1$
		
        //Designer Refactor: moved finding the script collector to above the big if block below
        //as its required for a Designer call further on (no point in getting it twice).
        UIScriptCollector scriptCollector = UIScriptCollector.find(component);
        
		//write the category rows, a new category required if it doesn't already exist
		if(categoryExists) {
			
			writer.startElement("tr",component); //$NON-NLS-1$

            if (categoryData.rowClasses.length > 0) {
                writer.writeAttribute("class", categoryData.rowClasses[categoryData.rowClassesCounter++],"rowClasses"); //$NON-NLS-1$ $NON-NLS-2$
                if (categoryData.rowClassesCounter >= categoryData.rowClasses.length) {
                	categoryData.rowClassesCounter = 0;
				}
			}
			
            writer.writeAttribute("style",collapsedStyle,"collapsed"); // $NON-NLS-2$ $NON-NLS-1$
			if (columnIndex > 0){
				for (int i = 0; i < columnIndex; i++) {
					writer.startElement("td",component); //$NON-NLS-1$
                    writer.writeAttribute("style","background-color:transparent;border-color:transparent",null); // $NON-NLS-2$ $NON-NLS-1$
					String cellId = getRowIdForColumn(i, table);
					if (cellId != null){
//                        if (i == columnIndex - 1){//this is the column
//                            writer.writeAttribute("id",cellId,null); //append s to signify this it can be collapsed but not expanded.
//                        }else{
                            writer.writeAttribute("id",cellId + EXPANDED_MARKER,null); //append s to signify this it can be collapsed but not expanded. $NON-NLS-1$
//                        }
					}
					writer.endElement("td"); //$NON-NLS-1$
				}
			}
			writer.startElement("td",component); //$NON-NLS-1$
            writer.writeAttribute("style","background-color:transparent;border-color:transparent",null); //$NON-NLS-1$ $NON-NLS-2$
            writer.writeAttribute("id",categoryData.idStr + categoryData.rowsUnderCategoryCount,null); //write id to td instead of tr, and have js function hide parent of found id's (the tr), this way, more than one column can hide a given row $NON-NLS-1$
			
			//increment row under category count
            ++categoryData.rowsUnderCategoryCount;
			
		} else {
			//create the category row
			
			//increment categoryIdNumber and set the row count under this category back to zero, this is used in row ids
			//then update id strings
			
            categoryData.idStr=component.getClientId(context)+"_" + categoryData.categoryIdNumber + "_"; //$NON-NLS-1$ $NON-NLS-2$
            ++categoryData.categoryIdNumber;
            categoryData.rowsUnderCategoryCount=0;
			
			StringBuffer script=new StringBuffer();
			
			//write category twistie row
			writer.startElement("tr",component); //$NON-NLS-1$
			if (!styleClassString.equals(NO_STYLE_CLASS)){
                writer.writeAttribute("class",styleClassString,"styleClass"); // $NON-NLS-2$ $NON-NLS-1$
			}
            if (firstCategoryColumn){
                writer.writeAttribute("style",styleString,null); //$NON-NLS-1$
            }else{
                writer.writeAttribute("style",styleString + "; " + collapsedStyle,null); //$NON-NLS-1$ $NON-NLS-2$
            }
			
			if (columnIndex > 0){
				for (int i = 0; i < columnIndex; i++) {
					writer.startElement("td",component); //$NON-NLS-1$
                    String cellId = getRowIdForColumn(i, table);
                    if (cellId != null){
                        writer.writeAttribute("id",cellId,null); //$NON-NLS-1$
                    }
                    writer.writeAttribute("style","background-color:transparent;border-color:transparent",null); // $NON-NLS-2$ $NON-NLS-1$
					writer.endElement("td"); //$NON-NLS-1$
				}
			}
			
			writer.startElement("td",component); //$NON-NLS-1$
			writer.writeAttribute("colspan",String.valueOf(columnCount - columnIndex),null); //$NON-NLS-1$
			
			//button
			writer.startElement("button",component); //$NON-NLS-1$
            writer.writeAttribute("id",categoryData.idStr+"button",null); // $NON-NLS-2$ $NON-NLS-1$
            writer.writeAttribute("type","button",null); // $NON-NLS-1$ $NON-NLS-2$
            writer.writeAttribute("style","border:none;background-color:transparent;cursor:pointer",null); //$NON-NLS-1$ $NON-NLS-2$
			
			if(tabindex!=null){
				writer.writeAttribute("tabindex",tabindex,null);	 //$NON-NLS-1$
			}
			if(dir != null) {
				writer.writeAttribute("dir", dir, null);	 //$NON-NLS-1$
			}
			if(title != null) {
				writer.writeAttribute("title", title, null);	 //$NON-NLS-1$
			}
			
			writer.startElement("img",component); //$NON-NLS-1$
            writer.writeAttribute("id",categoryData.idStr + "img",null); // $NON-NLS-1$ $NON-NLS-2$
			if(alt!=null){
				writer.writeAttribute("alt",alt,null); //$NON-NLS-1$
			}
			if(dir != null) {
				writer.writeAttribute("dir", dir, null);	 //$NON-NLS-1$
			}
			if(lang != null) {
				writer.writeAttribute("lang", lang, null);	 //$NON-NLS-1$
			}

			writer.endElement("img"); //$NON-NLS-1$
			
			// JSFBehaviorRowCategory js looks after images completley now.
			//writer.writeAttribute("src",collapsedStyle.equals(COLLAPSED_TRUE)? collapsedImageSrc : expandedImageSrc,null);
			
			//encode twistie urls if set, if none set script will add default twisties
			if(collapsedImageSrc != null){
				collapsedImageSrc = Utils.getEncodedResourceURL(context, collapsedImageSrc);	
			}
			if(expandedImageSrc != null){
				expandedImageSrc = Utils.getEncodedResourceURL(context, expandedImageSrc);	
			}
			
			writer.endElement("button"); //$NON-NLS-1$
			//add onlclick behavior to button
			script.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".addBehavior(\"") //$NON-NLS-1$
            .append(categoryData.idStr).append("button\",\"onclick\",new ") //$NON-NLS-1$
			.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".JSFBehaviorRowCategory(") //$NON-NLS-1$
            .append("\"cat-id:").append(categoryData.idStr).append("\","); // $NON-NLS-1$
			if(expandedImageSrc != null){
                script.append("\"twistie-exp-img:").append(expandedImageSrc).append("\","); // $NON-NLS-1$
			}
			if(collapsedImageSrc != null){
                script.append("\"twistie-col-img:").append(collapsedImageSrc).append("\","); // $NON-NLS-1$
			}
            script.append("\"first-row-num:").append(table.getFirst()).append("\",") // $NON-NLS-1$
            .append("\"first-last-num:").append(((table.getRows() <= 0)? table.getRowCount():(table.getFirst() + table.getRows()))).append("\",") // $NON-NLS-1$
            .append("\"component-id:").append(component.getId()).append("\",") // $NON-NLS-1$
			.append("\"init-img:") //$NON-NLS-1$
            .append((collapsed != null && collapsed.equalsIgnoreCase("false"))?"exp":"col") // $NON-NLS-1$ $NON-NLS-2$ $NON-NLS-3$
			.append("\"));\n"); //$NON-NLS-1$
			
			
			//link
            writer.startElement("a",component);
            writer.writeAttribute("id",categoryData.idStr+"link",null); // $NON-NLS-1$ $NON-NLS-2$
            writer.writeAttribute("style","cursor:pointer;",null); // $NON-NLS-2$ $NON-NLS-1$
			if (!styleClassString.equals(NO_STYLE_CLASS)){
                writer.writeAttribute("class",styleClassString,"styleClass"); // $NON-NLS-2$ $NON-NLS-1$
			}			
			writer.writeText(categoryString,null);
			writer.endElement("a"); //$NON-NLS-1$
			//add onclick behaviour to link
			script.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".addBehavior(\"") //$NON-NLS-1$
            .append(categoryData.idStr).append("link\",\"onclick\",new ") //$NON-NLS-1$
			.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".JSFBehaviorRowCategory(") //$NON-NLS-1$
            .append("\"cat-id:").append(categoryData.idStr).append("\","); //$NON-NLS-1$ //$NON-NLS-2$
			if(expandedImageSrc != null){
				script.append("\"twistie-exp-img:").append(expandedImageSrc).append("\","); //$NON-NLS-1$ //$NON-NLS-2$
			}
			if(collapsedImageSrc != null){
				script.append("\"twistie-col-img:").append(collapsedImageSrc).append("\","); //$NON-NLS-1$ //$NON-NLS-2$
			}
			script.append("\"first-row-num:").append(table.getFirst()).append("\",") //$NON-NLS-1$ //$NON-NLS-2$
			.append("\"first-last-num:").append(((table.getRows() <= 0)? table.getRowCount():(table.getFirst() + table.getRows()))).append("\",") //$NON-NLS-1$ //$NON-NLS-2$
			.append("\"component-id:").append(component.getId()).append("\"") //$NON-NLS-1$ //$NON-NLS-2$
			.append("));\n"); //$NON-NLS-1$

			writer.endElement("td"); //$NON-NLS-1$
			writer.endElement("tr"); //$NON-NLS-1$
			
            //Designer Refactoring, added a method call here (that does nothing for this renderer but does in Designers Extended renderer)
            //************** added to create listener for link and buttons getting fired (used to restore response document states)
            //in this renderer the call does nothing, but in the XSP sub class, it will.
            addButtonAndLinkListener(context, component, script, categoryData.idStr, columnIndex, expandedImageSrc);
            
			//write row
			writer.startElement("tr",component); //$NON-NLS-1$
            if (categoryData.rowClasses.length > 0) {
                writer.writeAttribute("class", categoryData.rowClasses[categoryData.rowClassesCounter++],"rowClasses"); //$NON-NLS-1$ $NON-NLS-2$
                if (categoryData.rowClassesCounter >= categoryData.rowClasses.length) {
                	categoryData.rowClassesCounter = 0;
				}
			}
//			writer.writeAttribute("id",idStr + rowsUnderCategoryCount,null);
            writer.writeAttribute("style",collapsedStyle,"collapsed"); //$NON-NLS-2$ $NON-NLS-1$
			if (columnIndex > 0){
				for (int i = 0; i < columnIndex; i++) {
					writer.startElement("td",component); //$NON-NLS-1$
                    writer.writeAttribute("style","background-color:transparent;border-color:transparent",null); //$NON-NLS-1$ $NON-NLS-2$
                    String cellId = getRowIdForColumn(i, table);
                    if (cellId != null){
                        writer.writeAttribute("id",cellId + EXPANDED_MARKER,null); //$NON-NLS-1$
                    }
					writer.endElement("td"); //$NON-NLS-1$
				}
			}
			writer.startElement("td",component); //$NON-NLS-1$
            writer.writeAttribute("style","background-color:transparent;border-color:transparent",null); //$NON-NLS-2$ $NON-NLS-1$
            writer.writeAttribute("id",categoryData.idStr + categoryData.rowsUnderCategoryCount,null); //$NON-NLS-1$
			
            ++categoryData.rowsUnderCategoryCount;
			

            if (scriptCollector != null){
                scriptCollector.addScript(script.toString());
            }
			
		}
		
        lastRowCheck(context, scriptCollector, component, table, categoryData);
    }
    
    //Designer Refactor: moved this out to a separate method as Designer do something different.
    protected void lastRowCheck(FacesContext context, UIScriptCollector scriptCollector, UIComponent component, UIData table, CategoryData categoryData) {
		// check if last row in table
        if(categoryData.currentRow == categoryData.lastRow) {
            //add to script collector.
            //previously the attributes created and used by this renderer were reset here, but for processing of other categories, this information is still needed, so a flag was set up
            //to tell the renderer to reset them when its next called (which will be for another request).
            //allRowsProcessed = Boolean.TRUE; //no point in setting this - on the local variable, but it does need to be set on the attribute map.
        	categoryData.allRowsProcessed = true;
            UIScriptCollector.find(table).addScript(getHideRundandantRowScript(component, table, context));
        }else{
			//increment currentRow
        	categoryData.currentRow++;
		}
        //intended for Designers extended renderer.
//		if (scriptCollector != null) {
//			scriptCollector.register(this, component);
//		}
	}
    
    
    protected void addButtonAndLinkListener(FacesContext context, UIComponent component, StringBuffer script, String idStr, int columnIndex, String expandedImageSrc) {
        //nothing implemented here, Used by Designer to add additional functionality.
    }


    protected boolean isCategoryExists(UIComponent component, FacesContext context, String categoryString, Vector categoryList, UIData dataTable, int columnCount,  int columnIndex) {
        boolean categoryExists = false;
        //srenwick, changed to find the last entry in the list added (sub-categories may have multiple categories of the same name in the over all column)
        //eg, if col1 has 2 categories (AA and BB) and each category as 2 sub categories (aa and bb), this would mean in col2 (where the subcategories are)
        //we would have two aa and 2 bb entries (one for AA and one for BB). So we need to just check the last category created for this column and use that
        //to establish if this is a new category. 
        //Another check that need to be made is if a master category closes off, all of its sub categories need to close off (regardless of whether or not the
        //sub categories value is changing or not)
        
        //first check if there is a column Category Entry in the dataTable for this column.
        //  if there is no entry in the array for our column index, then add one stating we're an new category
        
        CategoryState[] categoryColumns = (CategoryState[])dataTable.getAttributes().get(CATEGORY_COLUMNS_ARRAY);
        CategoryState categoryState = null;
        if (categoryColumns == null){
            categoryColumns = new CategoryState[columnCount];
            categoryState = new CategoryState();
            categoryColumns[columnIndex] = categoryState;
            dataTable.getAttributes().put(CATEGORY_COLUMNS_ARRAY, categoryColumns);
            categoryExists = false;
        }else{
            categoryState = categoryColumns[columnIndex];
            if (categoryState == null){
                categoryState = new CategoryState();
                categoryColumns[columnIndex] = new CategoryState();
                categoryExists = false;
            }else{
                categoryExists = categoryState.isCategoryExists();
                categoryState.setCategoryExists(true);
            }
        }

        if (categoryExists){
            String currentCategory = (String)categoryList.get(categoryList.size() - 1 );
            if (categoryString.equals(currentCategory)){
                categoryExists = true;
                categoryState.setCategoryExists(true);
            }else{
                categoryExists = false;
                //reset the category state for all categories that follow this one (subcategories to this category)
                for (int i = columnIndex + 1; i < categoryColumns.length; i++) {
                    
                    CategoryState state = categoryColumns[i];
                    if (state != null){
                        state.setCategoryExists(false);
                    }
                }
            }
        }

        return categoryExists;
    }

    
    private boolean isFirstCategoryColumn(int columnIndex, UIData table){
        Iterator columns =table.getChildren().iterator();
        int count = 0;
        while(columns.hasNext()) {
            UIComponent tableChild=(UIComponent)columns.next();
            if(tableChild instanceof UIColumn && tableChild.isRendered()) {
                if (count == columnIndex){
                    break; //we've hit the current column no point in looking further - this is the first category column
                }
                Iterator children = tableChild.getChildren().iterator();
                while(children.hasNext()) {
                    UIComponent child = (UIComponent)children.next();
                    
                    if (child instanceof PanelRowCategory ){
                        return false; //we've found a column containing a PanelRowCategory component preciding the current column (so we're not the first)
                    }
                }                    
                count++;
            }
        }
        return true;
    }
	
	private String getRowIdForColumn(int columnIndex, UIData table){
		String idString = null;
		Iterator columns =table.getChildren().iterator();
		UIColumn column = null;
		int count = 0;
		while(columns.hasNext()) {
			UIComponent tableChild=(UIComponent)columns.next();
			if(tableChild instanceof UIColumn && tableChild.isRendered()) {
				if (count == columnIndex){
					column = (UIColumn)tableChild;
					break;
				}
				count++;
			}
		}
		if (column != null){
			//ok, we have found the column for the specified index, now we need to check if it contains a PanelRowCategory component
			//and get its categoryIdNumber
			List columnChildren = column.getChildren();
			for (Iterator iter = columnChildren.iterator(); iter.hasNext();) {
				UIComponent colChild = (UIComponent) iter.next();

				if (colChild instanceof PanelRowCategory ){
					
                    CategoryData categoryData = (CategoryData)colChild.getAttributes().get(CATEGORY_DATA_KEY); 
                    
                    if (categoryData != null){
                        int rowsUnderCat = categoryData.rowsUnderCategoryCount;
                        idString=categoryData.idStr + rowsUnderCat;
                        categoryData.rowsUnderCategoryCount++;
					}
					break;
				}
			}
		}
		return idString;
	}

    protected String getHideRundandantRowScript(UIComponent component, UIData table, FacesContext context) {
		StringBuffer script=new StringBuffer();
		
		String id = context.getExternalContext().encodeNamespace(component.getId());
		
		//cleans up the unwanted rows by setting their display to none
		// old code. 
//		script.append("for(var cat_hide_rows=" + (table.getFirst()) + ";cat_hide_rows <= " + getLastRowIndex(table) + ";cat_hide_rows++) {"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
//		script.append("\nhX_temp=").append(HxClientRenderUtil.HX_VAR_NAME).append(".getElementById('").append(id).append("_hide_'+cat_hide_rows+'\');"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
//		script.append("\nif(hX_temp!=null)hX_temp.parentNode.style.display='none';}"); //$NON-NLS-1$
		// new code
		// do not use getElementById to find each row. Instead find the table and then iterate over all rows.
		// do it only once for the entire table
		if(getColumnIndex(component) == 0) {
			String tableClientId = table.getClientId(context);
			String tableId = table.getId();
			tableClientId = tableClientId.substring(0, tableClientId.lastIndexOf(tableId) + tableId.length());
			script.append("\nhX_temp=").append(HxClientRenderUtil.HX_VAR_NAME).append(".getElementById('").append(tableClientId).append("');\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			script.append("if(hX_temp!=null){\n");
			script.append("for(i=0;i<hX_temp.rows.length;i++){\n");
			script.append("for(j=0;j<hX_temp.rows[i].cells.length;j++){\n");
			script.append("id=hX_temp.rows[i].cells[j].id;\n");
			script.append("if(id!=null && id.indexOf('_hide_')!=-1){\n");
			script.append("hX_temp.rows[i].style.display='none';\n");
			script.append("break;\n");
			script.append("}}}}\n");
		}		

		//RATLC01129389 fix section - Hide cells rendered for first row (used for sizing)
		//note : display:none breaks layout in mozilla/firefox
		script.append("\nhX_temp=").append(HxClientRenderUtil.HX_VAR_NAME).append(".getElementById('").append(id).append("_catsize');\n"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		script.append("if(hX_temp!=null)hX_temp.parentNode.style.emptyCells='hide';\n"); //$NON-NLS-1$
		

		
		return script.toString();
	}

	private int getParentTableColumnCount(UIComponent component) {
		
		int count=0;
		Iterator columns;
		UIComponent data=getParentTable(component);
		//populate iterator with table children and count all that are rendered UIColumn components..
		columns=((UIData)data).getChildren().iterator();
		while(columns.hasNext()) {
			UIComponent tableChild=(UIComponent)columns.next();
			if(tableChild instanceof UIColumn && tableChild.isRendered()) {
				count++;
			}
		}
		return count;
	}
	
	private ArrayList getColumnList(UIComponent component){
		ArrayList list = new ArrayList();
		UIComponent table = getParentTable(component);
		Iterator children = table.getChildren().iterator();
		while(children.hasNext()){
			UIComponent child = (UIComponent)children.next();
			if(child instanceof UIColumn && child.isRendered()){
				list.add(child);
			}
		}
		return list;
	}
	
	/**
	 * 
	 * @param component
	 * @return the zero based index of the the column this rowCategory belongs to. We need this to know where to 
	 * place this cell (it won't always be the first cell in a row).
	 */
	private int getColumnIndex(UIComponent component) {
		//assume that parent of this component is of type UIColumn
		UIColumn thisColumn = (UIColumn)component.getParent();
		
		
		int count=0;
		Iterator columns;
		UIComponent data=getParentTable(component);
		//populate iterator with table children and count all that are rendered UIColumn components..
		columns=((UIData)data).getChildren().iterator();
		while(columns.hasNext()) {
			UIComponent tableChild=(UIComponent)columns.next();
			if(tableChild instanceof UIColumn && tableChild.isRendered()) {
				if (tableChild.equals(thisColumn)){
					break;
				}
				count++;
			}
		}
		return count;
	}
	
    protected UIData getParentTable(UIComponent component) {

		UIComponent comp=component.getParent();
		
		while(!(comp instanceof UIData) && comp!=null) {
			comp=comp.getParent();
		}
		//comp should be the UIData containing component, check...
		if (!(comp instanceof UIData) || comp==null) {
			System.err.println("Cannot find Categorized Row's containing table"); //$NON-NLS-1$
			return null;
		} 

		return (UIData) comp;
	}
	
	private String[] getRowClasses(UIData data) {

		String values = (String) data.getAttributes().get("rowClasses"); //$NON-NLS-1$
		if (values == null) {
			return (new String[0]);
		}
		values = values.trim();
		ArrayList list = new ArrayList();
		while (values.length() > 0) {
			int comma = values.indexOf(","); //$NON-NLS-1$
			if (comma >= 0) {
				list.add(values.substring(0, comma).trim());
				values = values.substring(comma + 1);
			} else {
				list.add(values.trim());
				values = ""; //$NON-NLS-1$
			}
		}
		String results[] = new String[list.size()];
		return ((String[]) list.toArray(results));

	}	

	private String getCategoryString(UIComponent component) {
		
		Iterator childrenIterator;
		UIComponent child;
		String categoryString=null;
		
		//check if for an outputFormat child component use its value if its not null
		childrenIterator=component.getChildren().iterator();
		Object val = null;
		while(childrenIterator.hasNext()) {
			child=(UIComponent)childrenIterator.next();
			if( child instanceof UIOutput ){
				
				ArrayList parameterList = new ArrayList();

				// get UIParameter children...
				Iterator kids = child.getChildren().iterator();
				while (kids.hasNext()) {
					UIComponent kid = (UIComponent) kids.next();

					if (!(kid instanceof UIParameter)) {
						continue;
					}
					parameterList.add(((UIParameter) kid).getValue());
				}
				if (parameterList.size() > 0) {
					val = ((UIOutput)child).getValue();
					if(val != null)
						categoryString =  MessageFormat.format
							(val.toString(), parameterList.toArray
							(new Object[parameterList.size()]));
				} else {
					//no parameters so get UIOutput value if any
					val = child.getAttributes().get("value"); //$NON-NLS-1$
					if(val != null)
						categoryString = val.toString();
				}
				
			}
		}
		//if no value set by children get the value specified in the rowCategory component
		if(categoryString==null){
			val = component.getAttributes().get("value"); //$NON-NLS-1$
			if(val != null)
				categoryString = val.toString();
		}
		return categoryString;
	}

	private int getLastRowIndex(UIData table,int currentRow){
		//returned int need to -1 because of equality check on currentRow that starts at 0	
		if(table.getRows()<=0){
			return (table.getRowCount() - 1);
		} 
		int lastRowNum;
		lastRowNum=currentRow+(table.getRows());
		if(lastRowNum > table.getRowCount()) lastRowNum=table.getRowCount();
		return lastRowNum - 1;			
	}
	

	public boolean getRendersChildren() {
		return true;
	}
    
    public void encodeChildren(FacesContext arg0, UIComponent arg1) throws IOException {
	}

    //Designer refactoring, extracted this from the getHideRedundantRowScript() as Designer can have many more rows that this does not cater for.
    protected int getLastRowIndex(UIData table){
    	return (table.getRows() <= 0)? table.getRowCount():(table.getFirst() + table.getRows());
    }
    
    
	protected class CategoryState{
        boolean categoryExists;
        public CategoryState() {
            this.categoryExists = true; //when a CategoryState object is created, it will always be signifying a new Category
        }
        boolean isCategoryExists(){
            return categoryExists;
        }
        void setCategoryExists(boolean categoryExists){
            this.categoryExists = categoryExists;
        }
    }
    
    //use this instead of all the variables above (hold one object in the attribute map, get it once instead of whats currently happening above.
    protected class CategoryData{
    	//wouldn't even bother with accessors
        public boolean  allRowsProcessed;
        public int categoryIdNumber;
        public int rowsUnderCategoryCount;
        public int currentRow;
        public int lastRow;
        public Vector categoryList;
        public String rowClasses[];
        public int rowClassesCounter;
        public String idStr;
        
        void init(UIData table){
        	allRowsProcessed = false;
        	categoryIdNumber = 0;
        	rowsUnderCategoryCount = 0;
        	currentRow = table.getFirst();
        	lastRow = getLastRowIndex(table,currentRow);
        	categoryList = new Vector();
        	rowClasses = getRowClasses(table); //maybe do this in the constructor not sure it needs to be done more than once - ever.
        	rowClassesCounter = 0;
        	idStr = new String();
        }
    }
}