/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.component.UICommand;
import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.event.ActionEvent;
import javax.faces.event.PhaseId;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UICommandEx;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.util.LifecycleUtil;

import com.ibm.faces.component.html.HtmlCommandExRowEdit;

/**
 * @author Alasdair McDowell
 * TODO - description
 */
public class RowEditRenderer extends AbstractRowRenderer implements IScriptContributor{

    private static final String PAD_EDIT = "__editCtrl"; //$NON-NLS-1$
    private static final String PAD_PANEL = "__editPnl"; //$NON-NLS-1$
    private static final String PAD_SAVE = "__saveCtrl"; //$NON-NLS-1$
    private static final String PAD_CANCEL = "__cancelCtrl"; //$NON-NLS-1$
    private static final String FIELD_SAVE = "__saveFld"; //$NON-NLS-1$

    /**
     * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     */
    public void decode(FacesContext context, UIComponent component) {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        String id = component.getClientId(context);
        String idFieldSave = id + FIELD_SAVE;
        Map requestMap = context.getExternalContext().getRequestParameterMap();
        if ("clicked".equals(requestMap.get(idFieldSave))) { //$NON-NLS-1$
            ActionEvent event = new ActionEvent(component);
            event.setPhaseId(PhaseId.INVOKE_APPLICATION);
            component.queueEvent(event);

            if (component instanceof UICommandEx) {
                UIData data = findUIData(component);
                if (data != null) {
                    ((UICommandEx) component).setRowIndex(data.getRowIndex());
                }
            }

            //2nd, if it was, decode any param attributes
            UIParameter[] params = LifecycleUtil.getParameters(component);
            if (params != null && params.length > 0) {
                if (component instanceof UICommand) {
                    UICommand command = (UICommand) component;
                    Object value = (Map) command.getValue();
                    Map map = null;
                    if (value instanceof Map) {
                        map = (Map) value;
                        map.clear();
                        map = this.decodeParameters(context, component, map);
                        if (map != null) {
                            command.setValue(map);
                        }
                    }

                    if (component instanceof UICommandEx) {
                        if (map == null) {
                            map = this.decodeParameters(context, component);
                        }
                        ((UICommandEx) component).setRowParameters(map);
                    }
                }
            }
        }
    }

    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }

        ResponseWriter writer = context.getResponseWriter();
        ResourceBundle bundle = ResourceHandler.getBundle(context);

        String id = component.getClientId(context);
        String idSave = id + PAD_SAVE;
        String idFieldSave = id + FIELD_SAVE;
        String idCancel = id + PAD_CANCEL;
        String idEdit = id + PAD_EDIT;
        String idPanel = id + PAD_PANEL;
        String idGrid = this.findUIDataId(context, component);

		HtmlCommandExRowEdit rowComponent = null;
				if ( component instanceof HtmlCommandExRowEdit ){
					rowComponent = (HtmlCommandExRowEdit)component;
				}
				
		String editStyleClass;
		if ( rowComponent != null ){
			editStyleClass = rowComponent.getEditStyleClass();
		}else{
			editStyleClass = (String) component.getAttributes().get("editStyleClass");
		}
		
		String saveLabel;
		if(rowComponent != null){
			saveLabel = rowComponent.getSaveLabel();
		} else {
			saveLabel = (String)component.getAttributes().get("saveLabel");
		}
		if(saveLabel == null){
			saveLabel = ResourceHandler.getString(bundle, "DataRendererEdit.save"); //$NON-NLS-1$
		}
		
		String cancelLabel;
		if(rowComponent != null){
			cancelLabel = rowComponent.getCancelLabel();
		} else {
			cancelLabel = (String)component.getAttributes().get("cancelLabel");
		}
		if(cancelLabel == null){
			cancelLabel = ResourceHandler.getString(bundle, "DataRendererEdit.cancel"); //$NON-NLS-1$
		}
		
		writer.startElement("input",component);
		
		writer.writeAttribute("id",idFieldSave,null);
		writer.writeAttribute("name",idFieldSave,null);
		writer.writeAttribute("type","hidden",null);
		writer.endElement("input");
		
		writer.startElement("table",component);
		writer.startElement("tr",component);
		writer.startElement("td",component);
		writer.startElement("button",component);
		writer.writeAttribute("id",idSave,null); //$NON-NLS-1$
		writer.writeAttribute("type","button",null); //$NON-NLS-1$
		if(editStyleClass!=null){
			writer.writeAttribute("class",editStyleClass,null); //$NON-NLS-1$
		}
		writer.writeText(saveLabel,null);
		writer.endElement("button");
		writer.endElement("td");
		
		writer.startElement("td",component);
		writer.startElement("button",component);
		writer.writeAttribute("id",idCancel,null); //$NON-NLS-1$
		writer.writeAttribute("type","button",null); //$NON-NLS-1$
		if(editStyleClass!=null){
			writer.writeAttribute("class",editStyleClass,null); //$NON-NLS-1$
		}
		writer.writeText(cancelLabel,null);
		writer.endElement("button");
		writer.endElement("td");
		writer.endElement("tr");
		writer.endElement("table");
		
		writer.endElement("div"); //$NON-NLS-1$

        StringBuffer script = new StringBuffer();

		script.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".addBehavior(\"")
		.append(idEdit).append("\",\"onclick\",new ")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".JSFBehaviorRowEditShowHide(")
		.append("\"div-id:").append(idPanel).append("\",")
		.append("\"show:").append("true").append("\"")
		.append("));\n");

		script.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".addBehavior(\"")
		.append(idCancel).append("\",\"onclick\",new ")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".JSFBehaviorRowEditShowHide(")
		.append("\"div-id:").append(idPanel).append("\",")
		.append("\"show:").append("false").append("\"")
		.append("));\n");
        
        
        
        if (idGrid != null) {
			script.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".addBehavior(\"")
			.append(idSave).append("\",\"onclick\",new ")
			.append(HxClientRenderUtil.HX_VAR_NAME)
			.append(".JSFBehaviorRowEditSave(")
			.append("\"div-id:").append(idPanel).append("\",")
			.append("\"table-id:").append(idGrid).append("\",")
			.append("\"hidden-id:").append(idFieldSave).append("\"")
			.append("));\n");
            
        }

        UIScriptCollector scriptCollector = UIScriptCollector.find(component);
        writeJavaScript(context, scriptCollector, script.toString());
        
        if(component instanceof UICommandEx) {
            ((UICommandEx) component).setRowIndex(-1);
        }
    }

    /**
     * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }

		HtmlCommandExRowEdit rowComponent = null;
		if ( component instanceof HtmlCommandExRowEdit ){
			rowComponent = (HtmlCommandExRowEdit)component;
		}
		HxClientRenderUtil.initJSLibraries(component, context);
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
		HxClientRenderUtil.addScriptLibrary("hxclient_table.js", component);
		

        ResponseWriter writer = context.getResponseWriter();
        ResourceBundle bundle = ResourceHandler.getBundle(context);

        String id = component.getClientId(context);
        String idEdit = id + PAD_EDIT;
        String idPanel = id + PAD_PANEL;
        
        String style,styleClass,editStyle,editStyleClass,title;
        
        if ( rowComponent != null ){
        	style = rowComponent.getStyle();
        	styleClass = rowComponent.getStyleClass();
        	editStyle = rowComponent.getEditStyle();
        	editStyleClass = rowComponent.getEditStyleClass();
        	title = rowComponent.getTitle();
        }else{
			style = (String) component.getAttributes().get("style"); //$NON-NLS-1$
			styleClass = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
			editStyle = (String) component.getAttributes().get("editStyle"); //$NON-NLS-1$
			editStyleClass = (String) component.getAttributes().get("editStyleClass");
			title = (String) component.getAttributes().get("title");
        }
    
		String editLabel;
		if(rowComponent != null){
			editLabel = rowComponent.getEditLabel();
		} else {
			editLabel = (String)component.getAttributes().get("editLabel");
		}
		if(editLabel == null){
			editLabel = ResourceHandler.getString(bundle, "DataRendererEdit.edit"); //$NON-NLS-1$
		}
		
		writer.startElement("span",component);
		writer.writeAttribute("id",id,"id"); //$NON-NLS-1$
		if(style!=null){
			writer.writeAttribute("style",style,"style"); //$NON-NLS-1$
		}
		if(styleClass!=null){
			writer.writeAttribute("class",styleClass,"styleClass"); //$NON-NLS-1$
		}
		
		writer.startElement("button", component); //$NON-NLS-1$
		writer.writeAttribute("id", idEdit,null); //$NON-NLS-1$
		writer.writeAttribute("type","button",null); //$NON-NLS-1$
		if(styleClass!=null){
			writer.writeAttribute("class",styleClass,"styleClass"); //$NON-NLS-1$
		}
		if(title != null)
			writer.writeAttribute("title",title,"title");
		writer.writeText(editLabel,null);
		writer.endElement("button"); //$NON-NLS-1$
		writer.endElement("span"); //$NON-NLS-1$

        if (editStyle == null) {
            editStyle = "position: absolute; z-index: 100; visibility: hidden;"; //$NON-NLS-1$
            if (editStyleClass == null) {
                editStyle += " background-color: white; border-color: black; border-style: solid; border-width: thin;"; //$NON-NLS-1$
            }
        }
		writer.startElement("div",component); //$NON-NLS-1$
		writer.writeAttribute("id",idPanel,"id"); //$NON-NLS-1$
		writer.writeAttribute("style", editStyle,"style"); //$NON-NLS-1$
		
		if(editStyleClass!=null){
			writer.writeAttribute("class", editStyleClass,"styleClass"); //$NON-NLS-1$
		}
    }


	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		// TODO Auto-generated method stub
		
	}

}
