/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;
import javax.faces.el.ValueBinding;

import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlPanelSection;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
import com.ibm.faces.renderkit.html_extended.Utils;
//import com.ibm.faces.util.LifecycleUtil;


/**
 * Renderer for JWL progress bar component. 
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.InputHelper_DatePickerTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIInputHelper}
 */

public class SectionRenderer extends Renderer{
	
	// theses are the names of the facets (icons and text) to look for
	final private String HEADING_OPEN = "opened";
	final private String HEADING_CLOSED = "closed";
	/*
	final private String ICON_OPEN = "icon-open";
	final private String ICON_CLOSED = "icon-closed";
	final private String TEXT_OPEN = "text-open";
	final private String TEXT_CLOSED = "text-closed";
	*/
	
	// there are for element id's
	final private String _INITCLOSED = "_initClosed";
	final private String _SECTIONHEADER = "_SECTIONHEADER";
	final private String _CONTENTS = "_CONTENTS";
	final private String _OPENED = "_OPENED";
	final private String _CLOSED = "_CLOSED";
	//final private String _IMG = "_IMG";
	final private String _TITLE = "_TITLE";

	
	// default style class 
	final private String _STYLECLASS = "panelSection";
	final private String _HEADER = "-header";
	final private String _BODY = "-body";
	final private String _DISPLAY_NONE = "display: none;";
	final private String _DISPLAY_BLOCK = "display: block;";
	
	
	
	
	public void decode(FacesContext context, UIComponent component) {
		if (context == null || component == null) {
			throw new NullPointerException();
		}
		
		HtmlPanelSection sec_comp = null;
		if(component instanceof HtmlPanelSection)
			sec_comp = (HtmlPanelSection)component;
		
		String id = component.getClientId(context);

		// see if the uistate has comeup in the request map
		// if it has set the value of initClosed to it in the component
		// This shouldn't happen when the page is rendered the very first time 
		
		String it_closed = (String)context.getExternalContext().getRequestParameterMap().get(id + _INITCLOSED);
		
		boolean open_close_overridable = getstateoverrideable(context, component); // defaults to false
	
		if(it_closed != null && it_closed.trim().length() > 0)
		{
			// having the value binding like this allows the state to be overidden if required
			if(open_close_overridable)	
			{
				ValueBinding vb = component.getValueBinding("initClosed");			
				if(vb != null && !vb.isReadOnly(context))
				{				
					if("true".equalsIgnoreCase(it_closed)) // it_closed means section should be closed when rendered
						vb.setValue(context, Boolean.TRUE);
					else
						vb.setValue(context, Boolean.FALSE);
					
					return;
				}
			}
			else
			{
				// this should only get run if literal strings initClosed="true"/initClosed="false" are used on the tag
				if(sec_comp != null)
				{
					if("true".equalsIgnoreCase(it_closed)) // it_closed means section should be closed when rendered
						sec_comp.setInitClosed(true);
					else
						sec_comp.setInitClosed(false);
				}
				else
				{
					if("true".equalsIgnoreCase(it_closed)) // it_closed means section should be closed when rendered
						component.getAttributes().put("initClosed","true");
					else
						component.getAttributes().put("initClosed","false");
				}
			}
		}
				
	}

	private boolean getstateoverrideable(FacesContext context, UIComponent component)
	{
		
		boolean open_close_overridable = false; // defaults to false
		
		
		HtmlPanelSection sec_comp = null;
		if(component instanceof HtmlPanelSection)
			sec_comp = (HtmlPanelSection)component;
		
		if(sec_comp != null)
		{
			open_close_overridable = sec_comp.isOverRideState(); // defaults to false
		}
		else
		{
			Object oco = component.getAttributes().get("overRideState");
			
			if(oco != null)
			{
				if(oco instanceof String)
				{
					open_close_overridable = (Boolean.valueOf(((String)oco))).booleanValue();
				}
				
				if(oco instanceof Boolean)
					open_close_overridable = ((Boolean)oco).booleanValue();
				
			}
			
		}
		return open_close_overridable;
	}
	
	/**
	 * Renders the beginning of the component.
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 *
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void encodeBegin(FacesContext context, UIComponent component) throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}

		if(!(component.isRendered())) // if you're no meant to see anything at all.
			return;
	
		HxClientRenderUtil.addScriptLibrary("hxclient_sec.js", component);
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		// we need to output our javascript library		
				HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		
		HtmlPanelSection sec_comp = null;
		if(component instanceof HtmlPanelSection)
			sec_comp = (HtmlPanelSection)component;
		
		String id = component.getClientId(context);
		
		boolean init_closed = true; // assume it is open by default
		String section_title = null;
		String sCSSPrefix = null;
		String tabindex = null; 
		String dir = null;
		String lang = null;
		String accesskey = null;
		boolean disabled = false;
		String style= null;
				
		if(sec_comp != null)
		{
			init_closed = sec_comp.isInitClosed();
			section_title = sec_comp.getTitle();
			sCSSPrefix = sec_comp.getStyleClass();	
			tabindex = sec_comp.getTabindex();
			dir = sec_comp.getDir();
			lang = sec_comp.getLang();
			accesskey = sec_comp.getAccesskey();
			disabled = sec_comp.isDisabled();
			style = sec_comp.getStyle();
			
		}
		else
		{
			Object a = component.getAttributes().get("initClosed");
			if(a instanceof String){
				if("true".equalsIgnoreCase(a.toString()))
					init_closed = true;
				else
					init_closed = false;
			}
			if(a instanceof Boolean){
				init_closed = ((Boolean)a).booleanValue();
			}
			
			section_title = (String)component.getAttributes().get("title");
			sCSSPrefix = (String)component.getAttributes().get("styleClass");
			tabindex = (String)component.getAttributes().get("tabindex");
			dir = (String)component.getAttributes().get("dir");
			lang = (String)component.getAttributes().get("lang");
			accesskey = (String)component.getAttributes().get("accesskey");
			style = (String)component.getAttributes().get("style");
			Object dis = component.getAttributes().get("disabled");	
			
			if(dis instanceof String) // if not set give default
				disabled = Boolean.valueOf((String)dis).booleanValue();

			if(dis instanceof Boolean)
				disabled = ((Boolean)dis).booleanValue();

		}
		
		
		/*if(sCSSPrefix == null)
			sCSSPrefix = _STYLECLASS;*/
		
		ResponseWriter writer = context.getResponseWriter();
		
		/*
		 *  Rendered HTML should look like this
		 *  <DIV id="section1_container" class="section">
		 * 		<DIV id="section1_title_contents" class="section-header">
		 *			<a id="section1"> contents of facets </a>
		 *		</DIV>
		 * 		<DIV id="section1_contents" style="section-body">
		 *			contents of section go here.....
		 * 		</DIV>
		 * 	<DIV>
		 * 
		 * FORMERLY would have been
		 *  <DIV id="section1_container" >
		 *		<a id="section1"><img id="section1_img" src=""/><spanid="section1_title">Your_title</span></a>
		 * 		<DIV id="section1_contents">
		 *			contents of section go here.....
		 * 		</DIV>
		 * 	<DIV>
		 * 
		 * */
		
		
		writer.startElement("div",component); // open container div
		writer.writeAttribute("id", id, null);
		if(sCSSPrefix != null && sCSSPrefix.trim().length()>0)
			writer.writeAttribute("class", sCSSPrefix, null);
		if(section_title != null && section_title.trim().length() > 0)
			writer.writeAttribute("title", section_title, null); // for tooltip and accessible text
		if(dir != null && dir.trim().length() > 0)
			writer.writeAttribute("dir", dir, null); 
		if(lang != null && lang.trim().length() > 0)
			writer.writeAttribute("lang", lang, null); 			
		if(disabled )
			writer.writeAttribute("disabled", "disabled", null); 
	
		if(style != null && style.trim().length() > 0)
			writer.writeAttribute("style", style, null); 		
		
		//START EVENTS
		String onclick = null; 
		if(sec_comp != null)		
			onclick= sec_comp.getOnclick();		
		else		
			onclick = (String)component.getAttributes().get("onclick");		
		if(onclick != null)
			writer.writeAttribute("onclick", onclick,"onclick");

		String ondblclick = null;
		if(sec_comp != null)		
			ondblclick= sec_comp.getOndblclick();
		else
			ondblclick = (String)component.getAttributes().get("ondblclick");		
		if(ondblclick != null)
			writer.writeAttribute("ondblclick", ondblclick,"ondblclick");
		
		String onkeydown = null;
		if(sec_comp != null)		
			onkeydown= sec_comp.getOnkeydown();		
		else		
			onkeydown = (String)component.getAttributes().get("onkeydown");
		if(onkeydown != null)
			writer.writeAttribute("onkeydown", onkeydown,"onkeydown");
		
		String onkeypress = null;
		if(sec_comp != null)		
			onkeypress= sec_comp.getOnkeypress();		
		else		
			onkeypress = (String)component.getAttributes().get("onkeypress");
		if(onkeypress != null)
			writer.writeAttribute("onkeypress", onkeypress,"onkeypress");
		
		String onkeyup = null;
		if(sec_comp != null)		
			onkeyup= sec_comp.getOnkeyup();		
		else		
			onkeyup = (String)component.getAttributes().get("onkeyup");
		if(onkeyup != null)
			writer.writeAttribute("onkeyup", onkeyup,"onkeyup");
		
		String onmousedown = null;
		if(sec_comp != null)		
			onmousedown= sec_comp.getOnmousedown();		
		else		
			onmousedown = (String)component.getAttributes().get("onmousedown");		
		if(onmousedown != null)
			writer.writeAttribute("onmousedown", onmousedown,"onmousedown");
		
		String onmousemove = null;
		if(sec_comp != null)		
			onmousemove= sec_comp.getOnmousemove();		
		else		
			onmousemove = (String)component.getAttributes().get("onmousemove");
		if(onmousemove != null)
			writer.writeAttribute("onmousemove", onmousemove,"onmousemove");

		String onmouseout = null;
		if(sec_comp != null)		
			onmouseout= sec_comp.getOnmouseout();		
		else		
			onmouseout = (String)component.getAttributes().get("onmouseout");		
		if(onmouseout != null)
			writer.writeAttribute("onmouseout", onmouseout,"onmouseout");

		String onmouseover = null;
		if(sec_comp != null)		
			onmouseover= sec_comp.getOnmouseover();		
		else		
			onmouseover = (String)component.getAttributes().get("onmouseover");		
		if(onmouseover != null)
			writer.writeAttribute("onmouseover", onmouseover,"onmouseover");

		String onmouseup = null;
		if(sec_comp != null)		
			onmouseup= sec_comp.getOnmouseup();		
		else		
			onmouseup = (String)component.getAttributes().get("onmouseup");		
		if(onmouseup != null)
			writer.writeAttribute("onmouseup", onmouseup,"onmouseup");	

		// START EVENTS
		
		
		// get the components
		UIComponent opened = component.getFacet(HEADING_OPEN);
		UIComponent closed = component.getFacet(HEADING_CLOSED);
		
		
		writer.startElement("div",component); // open section header div
		writer.writeAttribute("id", id+_TITLE, null);
		if(sCSSPrefix != null && sCSSPrefix.trim().length() > 0)
			writer.writeAttribute("class", sCSSPrefix+_HEADER, null);
		
		
		if(dir != null && dir.trim().length() > 0)
			writer.writeAttribute("dir", dir, null); 
		if(lang != null && lang.trim().length() > 0)
			writer.writeAttribute("lang", lang, null); 			


		writer.startElement("div",component); // open open section header anchor div
		writer.writeAttribute("id", id+_TITLE+_OPENED, null);

		
		if(dir != null && dir.trim().length() > 0)
			writer.writeAttribute("dir", dir, null); 
		if(lang != null && lang.trim().length() > 0)
			writer.writeAttribute("lang", lang, null); 		

		if(init_closed) // hide the open stuff
			writer.writeAttribute("style", _DISPLAY_NONE, null); 
		else
			writer.writeAttribute("style", _DISPLAY_BLOCK, null); 
		
		writer.startElement("a",component); // open the open anchor
		writer.writeAttribute("id", id+_SECTIONHEADER+_OPENED, null); // this has the		

		writer.writeAttribute("href", "javascript:;", null); 		

		
		if(tabindex != null && tabindex.trim().length() > 0)
			writer.writeAttribute("tabindex", tabindex, null); // tabindex
		if(accesskey != null && accesskey.trim().length() > 0)
			writer.writeAttribute("accesskey", accesskey, null); // tabindex		
		if(dir != null && dir.trim().length() > 0)
			writer.writeAttribute("dir", dir, null); 
		if(lang != null && lang.trim().length() > 0)
			writer.writeAttribute("lang", lang, null); 		
		
		// output contents of the open header
		if(opened != null)
			Utils.encodeComponent(context,opened);
		
		writer.endElement("a"); // close the open anchor 
		
		writer.endElement("div");  // close open section header anchor div

		writer.startElement("div",component); // open close section header anchor div
		writer.writeAttribute("id", id+_TITLE+_CLOSED, null);

		
		if(dir != null && dir.trim().length() > 0)
			writer.writeAttribute("dir", dir, null); 
		if(lang != null && lang.trim().length() > 0)
			writer.writeAttribute("lang", lang, null); 		

		if(init_closed) // show the closed stuff
			writer.writeAttribute("style", _DISPLAY_BLOCK, null); 
		else
			writer.writeAttribute("style", _DISPLAY_NONE, null); 		

		writer.startElement("a",component); // open the close anchor
		writer.writeAttribute("id", id+_SECTIONHEADER+_CLOSED, null); // this has the		

		writer.writeAttribute("href", "javascript:;", null); 

		
		if(tabindex != null && tabindex.trim().length() > 0)
			writer.writeAttribute("tabindex", tabindex, null); // tabindex
		if(accesskey != null && accesskey.trim().length() > 0)
			writer.writeAttribute("accesskey", accesskey, null); // tabindex		
		if(dir != null && dir.trim().length() > 0)
			writer.writeAttribute("dir", dir, null); 
		if(lang != null && lang.trim().length() > 0)
			writer.writeAttribute("lang", lang, null); 				
		
		// output contents of the closed header
		if(closed != null) 
			Utils.encodeComponent(context,closed);
		
		writer.endElement("a"); // close the close anchor

		writer.endElement("div");  // close open section header anchor div		
		
		
		
		writer.endElement("div"); // close the section header div
		
		writer.startElement("div",component);  // open contents div
		writer.writeAttribute("id", id+_CONTENTS, null);
		if(sCSSPrefix != null && sCSSPrefix.trim().length() > 0)
			writer.writeAttribute("class", sCSSPrefix+_BODY, null); 		

		if(dir != null && dir.trim().length() > 0)
			writer.writeAttribute("dir", dir, null); 
		if(lang != null && lang.trim().length() > 0)
			writer.writeAttribute("lang", lang, null); 			
		
		if(init_closed)
			writer.writeAttribute("style", _DISPLAY_NONE, null);
		else
			writer.writeAttribute("style", _DISPLAY_BLOCK, null);
			// contents go here
	
	}

	/*
	 * 
	 * 
	 * */
	public boolean getRendersChildren() {
		return true;
	}
	
	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public String doScript(FacesContext context, UIComponent component) {
				
		HtmlPanelSection sec_comp = null;
		
		if(component instanceof HtmlPanelSection)
			sec_comp = (HtmlPanelSection)component;
	
		boolean closed = true;
		boolean disabled = false;
		String sCSSPrefix = null;

			
		if(sec_comp != null)
			sCSSPrefix = sec_comp.getStyleClass();
		else
			sCSSPrefix = (String)component.getAttributes().get("styleClass");	
		
		if(sCSSPrefix == null) // if not set give default
			sCSSPrefix = _STYLECLASS;
		else
		{
			if("".equalsIgnoreCase(sCSSPrefix.trim())) // if blank set give default
				sCSSPrefix = _STYLECLASS;
		}
		
		
		if(sec_comp != null)
		{
			closed = sec_comp.isInitClosed();
			disabled = sec_comp.isDisabled();
		}
		else
		{
			Object ex = component.getAttributes().get("initClosed");	
		
			if(ex instanceof String) // if not set give default
				closed = Boolean.valueOf((String)ex).booleanValue();

			if(ex instanceof Boolean)
				closed = ((Boolean)ex).booleanValue();
		
			Object dis = component.getAttributes().get("disabled");	
		
			if(dis instanceof String) // if not set give default
				disabled = Boolean.valueOf((String)dis).booleanValue();

			if(dis instanceof Boolean)
				disabled = ((Boolean)dis).booleanValue();
		}		

		
	/*
	 * example script
	 * hX_4.addComponent("form1:section13", new hX_4.JSFSection("css-prefix:section", "expanded:true", 
	 * 		"diabled:false", "imgopened:form1:image1", "imgclosed:form1:image2", 
	 * 		"textopened:form1:text", "textclosed:form1:text2"));
	 * */
		
		StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME); //$NON-NLS-1$
		script.append(".addComponent(\"");  //$NON-NLS-1$//$NON-NLS-2$
		script.append(component.getClientId(context));
		script.append("\", new "); 	//$NON-NLS-1$//$NON-NLS-2$
		script.append(HxClientRenderUtil.HX_VAR_NAME);
		script.append(".JSFSection(");  //$NON-NLS-1$//$NON-NLS-2$


		if(closed) // done like this to make sure there is a first parameter passed to 
		{												// JSFSection constructor.
			//script.append(", \"expanded:"); //$NON-NLS-1$
			script.append("\"expanded:"); //$NON-NLS-1$
			script.append("false");
			script.append('\"'); //$NON-NLS-1$
		}
		else
		{
			//script.append(", \"expanded:"); //$NON-NLS-1$
			script.append("\"expanded:"); //$NON-NLS-1$
			script.append("true");
			script.append('\"'); //$NON-NLS-1$
		}
		
		if(disabled)
		{
			script.append(", \"disabled:"); //$NON-NLS-1$
			script.append("true");
			script.append('\"'); //$NON-NLS-1$
		}
		
	/*	script.append(", \"css-prefix:"); //$NON-NLS-1$
		script.append(sCSSPrefix);
		script.append('\"'); //$NON-NLS-1$  // NO CSS STUFF NEEDS TO BE ADDED TO JAVASCRIPT CURRENTLY
		*/
		
		script.append("));\n"); //$NON-NLS-1$
		
		return script.toString();
		
	}
	
	/** 
	 * Renders component's children.
	 * <p>
	 * Renders all child components as it see them.
	 * <p>
	 * 
	 * @param context {@link javax.faces.context.FacesContext} for the request we are processing
	 * @param component {@link javax.faces.component.UIComponent} to be rendered
	 * 
	 * @exception IOException if an input/output error occurs while rendering
	 *
	 * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */ 
	public void encodeChildren(FacesContext context, UIComponent component)throws IOException 
	{

		if(context == null || component == null)
			throw new NullPointerException();
		

		List children = component.getChildren();
		for (int i = 0; i < children.size(); i++) {
			UIComponent child = (UIComponent) children.get(i);
			Utils.encodeComponent(context, child);
		}
		
	}
	
    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
		if (context == null || component == null) {
			throw new NullPointerException();
		}

		if(!(component.isRendered())) // if you're no meant to see anything at all.
			return;
    	
		ResponseWriter writer = context.getResponseWriter();
		
    	//contents went here
    	
		writer.endElement("div"); // close the contents div
		
		writer.endElement("div"); // close the container div
    	
		String id = component.getClientId(context);
		
		String closed = "false"; // assume it is open
		
		Object a = component.getAttributes().get("initClosed");
		if(a instanceof String){
			if("true".equalsIgnoreCase(a.toString()))
				closed = "true";			
		}
		if(a instanceof Boolean){
			closed = ((Boolean)a).toString().toLowerCase();
		}
		
		writer.startElement("input",component);  // this is for the uistate
		writer.writeAttribute("id",id + _INITCLOSED,null);
		writer.writeAttribute("name",id + _INITCLOSED,null);
		writer.writeAttribute("type","hidden",null);
		writer.writeAttribute("value",closed,null);		// closed = true means section will be closed
		writer.endElement("input");
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		scriptCollector.addScript(doScript(context, component));
		
    }
    
    
    
    /*
     * return a fully (WSRP if on portal and is begin used) URL to the IMAGE
     * 
     * if param openimage (boolean) is true we are looking for the opened image
     * else we are looking for the closed image.
     * */
   /* private String getIcon(FacesContext context, UIComponent component, boolean openimage)
    {
    	String iconPath = null;
    	
    	String iconPath_sub;
    	
		HtmlPanelSection sec_comp = null;
		if(component instanceof HtmlPanelSection)
			sec_comp = (HtmlPanelSection)component;
    	
		 // try get image path from the component
		if(sec_comp != null)
		{
			if(openimage)
				iconPath_sub = sec_comp.getImgOpened();
			else
				iconPath_sub = sec_comp.getImgClosed();
		}
		else
		{
			if(openimage)
				iconPath_sub = (String)component.getAttributes().get("imgOpened");
			else
				iconPath_sub = (String)component.getAttributes().get("imgClosed");
		}
			//No image path set in component, so use the default    	
		if(iconPath_sub == null) 
			iconPath_sub = _IMG_OPENED;
		
		//get a full url path for the image
		String theContextPath = LifecycleUtil.getResourceServletContextPath(context);
		iconPath = context.getExternalContext().encodeResourceURL(theContextPath + iconPath_sub);
		
    	return iconPath;
    }*/
    
    
   
    /*
     * return a string for the anchor toptip (title) and acessible text.
     * 
     * if param opentext (boolean) is true we are looking for the opened text
     * else we are looking for the closed text.
     * */
 /*   private String getText(FacesContext context, UIComponent component, boolean opentext)
    {
    	String text_tip = null;
    	
		HtmlPanelSection sec_comp = null;
		if(component instanceof HtmlPanelSection)
			sec_comp = (HtmlPanelSection)component;
    	
		 // try get image path from the component
		if(sec_comp != null)
		{
			if(opentext)
				text_tip = sec_comp.getTextOpened();
			else
				text_tip = sec_comp.getTextClosed();
		}
		else
		{
			if(opentext)
				text_tip = (String)component.getAttributes().get("textOpened");
			else
				text_tip = (String)component.getAttributes().get("textClosed");
		}
		
    	return text_tip;
    }
 */
}
