/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.io.Writer;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.util.HtmlUtil;
import com.ibm.faces.util.JavaScriptUtil;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.html.HtmlOutputSelecticons;

/**
 * @author Alasdair McDowell
 * TODO - description
 */
public class SelecticonsRenderer extends AbstractSelectRenderer implements IScriptContributor{

    private static final String SELCTL_ICONS_ALL = "__selectCtrlIconsAll"; //$NON-NLS-1$
    private static final String SELCTL_ICONS_NONE = "__selectCtrlIconsNone"; //$NON-NLS-1$
    private static final String SELCTL_ICONS_INVERSE = "__selectCtrlIconsInverse"; //$NON-NLS-1$

    private static final String SELECT_ALL_IMG = Utils.versionizeFilename("datagrid_select_all.gif");
    private static final String SELECT_NONE_IMG = Utils.versionizeFilename("datagrid_select_none.gif");
    private static final String SELECT_INVERSE_IMG = Utils.versionizeFilename("datagrid_select_inverse.gif");
    
    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        HxClientRenderUtil.initJSLibraries(component, context);
        HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
        HxClientRenderUtil.addScriptLibrary("hxclient_table.js" , component);

		HtmlOutputSelecticons selectComponent = null;
		if ( component instanceof HtmlOutputSelecticons ){
			selectComponent = (HtmlOutputSelecticons)component;			
		}

        Writer writer = context.getResponseWriter();
        ResourceBundle bundle = ResourceHandler.getBundle(context);

        String localizedSelectAll = ResourceHandler.getString(bundle, "DatagridSelectionRenderer.Select_All"); //$NON-NLS-1$
        String localizedSelectNone = ResourceHandler.getString(bundle, "DatagridSelectionRenderer.Select_None"); //$NON-NLS-1$
        String localizedSelectInverse = ResourceHandler.getString(bundle, "DatagridSelectionRenderer.Select_Inverse"); //$NON-NLS-1$

        String id = component.getClientId(context);

        //note: rely on the fact rowselect ctrl will init UIScriptCollector
        String jsObjSelGroup =
            this.getJsSelectionGroupVariable(context, component);
        StringBuffer buffer = new StringBuffer();
        String onclickSelectAll = this.makeOnclickCode(buffer, jsObjSelGroup, "selectAll"); //$NON-NLS-1$
        String onclickSelectNone = this.makeOnclickCode(buffer, jsObjSelGroup, "selectNone"); //$NON-NLS-1$
        String onclickSelectInverse = this.makeOnclickCode(buffer, jsObjSelGroup, "selectInverse"); //$NON-NLS-1$

		String style,styleClass;
		Object useButtonsVal;

		if ( selectComponent != null ){
			style = selectComponent.getStyle();
			styleClass = selectComponent.getStyleClass();
			useButtonsVal = selectComponent.getUseButtons();
		}else{
			style = (String) component.getAttributes().get("style"); //$NON-NLS-1$
			styleClass = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
			useButtonsVal = component.getAttributes().get("useButtons");
		}

        boolean useButtons = "true".equals(useButtonsVal) || Boolean.TRUE.equals(useButtonsVal); //$NON-NLS-1$

        HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "style", style); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
        HtmlUtil.writeTagStartCloseLn(writer, false);

        if (useButtons) {
            this.writeInputSubmit(
                writer,
                id + SELCTL_ICONS_ALL,
                localizedSelectAll,
                onclickSelectAll,
                styleClass);
            this.writeInputSubmit(
                writer,
                id + SELCTL_ICONS_NONE,
                localizedSelectNone,
                onclickSelectNone,
                styleClass);
            this.writeInputSubmit(
                writer,
                id + SELCTL_ICONS_INVERSE,
                localizedSelectInverse,
                onclickSelectInverse,
                styleClass);
        }
        else {
        	
        	String srcAll,srcNone,srcInverse;
        	
        	if ( selectComponent != null ){
        		srcAll = selectComponent.getSrcAll();
        		srcNone = selectComponent.getSrcNone();
        		srcInverse = selectComponent.getSrcInverse();
        	}else{
				srcAll = (String) component.getAttributes().get("srcAll"); //$NON-NLS-1$
				srcNone = (String) component.getAttributes().get("srcNone"); //$NON-NLS-1$
				srcInverse = (String) component.getAttributes().get("srcInverse"); //$NON-NLS-1$ 
        	}
        	
            String imgAll = encodeResource(context, srcAll, SELECT_ALL_IMG);
            String imgNone = encodeResource(context, srcNone, SELECT_NONE_IMG);
            String imgInverse = encodeResource(context, srcInverse, SELECT_INVERSE_IMG);

                this.writeInputImage(
                    writer,
                    id + SELCTL_ICONS_ALL,
                    localizedSelectAll,
                    imgAll,
                    onclickSelectAll,
                    styleClass);

                this.writeInputImage(
                    writer,
                    id + SELCTL_ICONS_NONE,
                    localizedSelectNone,
                    imgNone,
                    onclickSelectNone,
                    styleClass);

                this.writeInputImage(
                    writer,
                    id + SELCTL_ICONS_INVERSE,
                    localizedSelectInverse,
                    imgInverse,
                    onclickSelectInverse,
                    styleClass);
        }

        HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
    }

    private String makeOnclickCode(
        StringBuffer buffer,
        String jsObjSelectGroup,
        String method) {
        buffer.setLength(0);
        buffer.append(jsObjSelectGroup);
        buffer.append("."); //$NON-NLS-1$
        buffer.append(method);
        buffer.append("(); return false;"); //$NON-NLS-1$
        return buffer.toString();
    }

    private void writeInputImage(
        Writer writer,
        String id,
        String alt,
        String src,
        String onclick,
        String css)
        throws IOException {
        HtmlUtil.writeTagStartOpen(writer, "input", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "type", "image"); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "name", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "alt", alt); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "title", alt); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "src", src); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "onclick", onclick); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", css); //$NON-NLS-1$
        HtmlUtil.writeTagStartCloseLn(writer, true);
    }

    private String encodeResource(FacesContext context, String customImage, String defaultImage) {
    	String imageURL = null;
        if (customImage != null) {
        	imageURL =  Utils.getEncodedResourceURL(context, customImage);
        } else {
        	imageURL = context.getExternalContext().encodeResourceURL(
        			             JavaScriptUtil.getImagePath(context, defaultImage));
        }
        return imageURL;
    }


	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		// TODO Auto-generated method stub
		
	}
}
