/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.io.Writer;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.util.HtmlUtil;
import com.ibm.faces.util.JavaScriptUtil;

import com.ibm.faces.component.html.HtmlOutputSelectmenu;

/**
 * @author Alasdair McDowell
 * TODO - description
 */
public class SelectmenuRenderer extends AbstractSelectRenderer {

    private static final String SELCTL_SIMPLE_CHOOSE = "__selectCtrlSimpleChoose"; //$NON-NLS-1$
    private static final String SELCTL_SIMPLE_SELECT = "__selectCtrlSimpleSelect"; //$NON-NLS-1$
    private static final String SELCTL_SIMPLE_SELECT_JS = "__selectCtrlSimpleSelectJs"; //$NON-NLS-1$

    private static final String SELCTL_ALL = "__selectAll"; //$NON-NLS-1$
    private static final String SELCTL_NONE = "__selectNone"; //$NON-NLS-1$
    private static final String SELCTL_INVERSE = "__selectInverse"; //$NON-NLS-1$

    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }

		HtmlOutputSelectmenu menuComponent = null;
		if ( component instanceof HtmlOutputSelectmenu ){
			menuComponent = (HtmlOutputSelectmenu)component;
		}

        Writer writer = context.getResponseWriter();
        ResourceBundle bundle = ResourceHandler.getBundle(context);

        String localizedSelect = ResourceHandler.getString(bundle, "DatagridSelectionRenderer.Select"); //$NON-NLS-1$
        String localizedSelectAll = ResourceHandler.getString(bundle, "DatagridSelectionRenderer.Select_All"); //$NON-NLS-1$
        String localizedSelectNone = ResourceHandler.getString(bundle, "DatagridSelectionRenderer.Select_None"); //$NON-NLS-1$
        String localizedSelectInverse = ResourceHandler.getString(bundle, "DatagridSelectionRenderer.Select_Inverse"); //$NON-NLS-1$

        String id = component.getClientId(context);

        String js = JavaScriptUtil.makeVariableName(id + SELCTL_SIMPLE_SELECT_JS);
        String selId = id + SELCTL_SIMPLE_CHOOSE;

		String style,styleClass, title;
		if ( menuComponent != null ){
			style = menuComponent.getStyle();
			styleClass = menuComponent.getStyleClass();
			title = menuComponent.getTitle();
		}else{
			style = (String) component.getAttributes().get("style"); //$NON-NLS-1$
			styleClass = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
			title = (String) component.getAttributes().get("title"); //$NON-NLS-1$
		}

        HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "style", style); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
        if(title != null)
        	HtmlUtil.writeTagAttribute(writer, "title", title); //$NON-NLS-1$
        HtmlUtil.writeTagStartCloseLn(writer, false);

        HtmlUtil.writeTagStartOpen(writer, "select", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", id + SELCTL_SIMPLE_CHOOSE); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "name", id + SELCTL_SIMPLE_CHOOSE); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "size", "1"); //$NON-NLS-1$ //$NON-NLS-2$
        HtmlUtil.writeTagAttribute(writer, "label", localizedSelect); //$NON-NLS-1$
        HtmlUtil.writeTagStartCloseLn(writer, false);

        HtmlUtil.writeOption(writer, localizedSelectAll, SELCTL_ALL);
        HtmlUtil.writeOption(writer, localizedSelectNone, SELCTL_NONE);
        HtmlUtil.writeOption(writer, localizedSelectInverse, SELCTL_INVERSE);

        HtmlUtil.writeTagEndLn(writer, "select"); //$NON-NLS-1$

        String jsObjGroup = this.getJsSelectionGroupVariable(context, component);
        StringBuffer onclick = new StringBuffer();
        if (jsObjGroup != null) {
            onclick.append("var "); //$NON-NLS-1$
            onclick.append(js);
            onclick.append(" = document.getElementById('"); //$NON-NLS-1$
            onclick.append(selId);
            onclick.append("'); "); //$NON-NLS-1$

            onclick.append("if("); //$NON-NLS-1$
            onclick.append(js);
            onclick.append(".options[0].selected) "); //$NON-NLS-1$
            onclick.append(jsObjGroup);
            onclick.append(".selectAll(); "); //$NON-NLS-1$

            onclick.append("if("); //$NON-NLS-1$
            onclick.append(js);
            onclick.append(".options[1].selected) "); //$NON-NLS-1$
            onclick.append(jsObjGroup);
            onclick.append(".selectNone(); "); //$NON-NLS-1$

            onclick.append("if("); //$NON-NLS-1$
            onclick.append(js);
            onclick.append(".options[2].selected) "); //$NON-NLS-1$
            onclick.append(jsObjGroup);
            onclick.append(".selectInverse(); "); //$NON-NLS-1$
        }
        onclick.append("return false;"); //$NON-NLS-1$

        this.writeInputSubmit(
            writer,
            id + SELCTL_SIMPLE_SELECT,
            localizedSelect,
            onclick.toString(),
            null);

        HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
    }

}
