/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;

import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlInputHelperSetFocus;
import javax.faces.render.Renderer;


/*
 * 
 * <p>SetFocusRenderer</p>
 * 
 */

public class SetFocusRenderer extends Renderer implements IScriptContributor {
	
    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {

        if (context == null || component == null) {
            throw new NullPointerException();
        }

		UIComponent parent = component.getParent();
		if (parent == null || !(parent instanceof UIForm)) {
			return; // no point in going any further if we don't have these
		}
	
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		scriptCollector.register(this, component);
    }

	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {

		// Check if the parent has a validation tag, and if yes - get validation attributes
		ResponseWriter writer = context.getResponseWriter();
		writer.write("\n"); //$NON-NLS-1$
		writer.write(HxClientRenderUtil.HX_VAR_NAME + ".addBehavior(\"#body\", \"onload\", new " + HxClientRenderUtil.HX_VAR_NAME + ".JSFBehaviorFocus("); //$NON-NLS-1$
		String target = null;
		if(component instanceof HtmlInputHelperSetFocus){
			target = ((HtmlInputHelperSetFocus)component).getTarget();
		} else {
			target = (String)component.getAttributes().get("target");
		}
		
		if(null != target){
			writer.write("\"focus-id:"); //$NON-NLS-1$
			writer.write(HxClientRenderUtil.convertComponentIdToClientId(target, component));
			writer.write("\"");
		}
		//fix for stephen - xfaces
		boolean selected = false;
		if(component instanceof HtmlInputHelperSetFocus){
			selected = ((HtmlInputHelperSetFocus)component).isSelect();
		} else {
			Object o = component.getAttributes().get("select");
			if(o instanceof String && ((String)o).equalsIgnoreCase("true")){
				selected = true;
			}
			if(o instanceof Boolean && ((Boolean)o).booleanValue()){
				selected = true;
			}
		}
		if(selected){
			//RATLC01085889
			if(target != null){
				writer.write(",");
			}
			writer.write("\"select\""); //$NON-NLS-1$
		}
		writer.write("));"); //$NON-NLS-1$
	}

	/**
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 * @param context
	 * @param component
	 * @throws IOException
	 */
	public void encodeEnd(FacesContext context, UIComponent component)
		throws IOException {
		//defect 280926
	}
	

}
