/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;


/**
 * Renderer for {@link com.ibm.faces.component.UIPlayer UIPlayer} component, 
 * that creates a Micromedia Shockwave control.
 * <p> 
 * <b>Tags:</b> {@link com.ibm.faces.taglib.html_extended.Player_ShockwaveTag}<br>
 * <b>Components:</b> {@link com.ibm.faces.component.UIPlayer}
 */

public final class ShockwaveRenderer extends PlayerRenderer
{
	// Shockwave parameters	
	public static final String PARAM_SWREMOTE			= "swRemote"; //$NON-NLS-1$
	public static final String PARAM_SWSTRETCHSTYLE		= "swStretchStyle"; //$NON-NLS-1$
	public static final String PARAM_BGCOLOR			= "bgColor"; //$NON-NLS-1$
	public static final String PARAM_SWSTRETCHHALIGN	= "swStretchHAlign"; //$NON-NLS-1$
	public static final String PARAM_SWSTRETCHVALIGN	= "swStretchVAlign"; //$NON-NLS-1$
	
	private static final Param[] objectParams = {
		new Param("src"  	  				, "src"), //$NON-NLS-1$ //$NON-NLS-2$
		new Param(PARAM_SWREMOTE			, "swRemote"), //$NON-NLS-1$
		new Param(PARAM_SWSTRETCHSTYLE  	, "swStretchStyle"), //$NON-NLS-1$
		new Param(PARAM_BGCOLOR  			, "bgColor"), //$NON-NLS-1$
		new Param(PARAM_SWSTRETCHHALIGN  	, "swStretchHAlign"), //$NON-NLS-1$
		new Param(PARAM_SWSTRETCHVALIGN  	, "swStretchVAlign"), //$NON-NLS-1$
	};

	private static final Param[] pluginParams = {
		new Param("src"  	  		, "src"), //$NON-NLS-1$ //$NON-NLS-2$
		new Param(PARAM_SWREMOTE			, "swRemote"), //$NON-NLS-1$
		new Param(PARAM_SWSTRETCHSTYLE  	, "swStretchStyle"), //$NON-NLS-1$
		new Param(PARAM_BGCOLOR  			, "bgColor"), //$NON-NLS-1$
		new Param(PARAM_SWSTRETCHHALIGN  	, "swStretchHAlign"), //$NON-NLS-1$
		new Param(PARAM_SWSTRETCHVALIGN  	, "swStretchVAlign"), //$NON-NLS-1$
	};
	
	private static final String DEFAULT_CLASSID = "clsid:166B1BCA-3F9C-11CF-8075-444553540000"; //$NON-NLS-1$
	private static final String DEFAULT_CODEBASE = "http://download.macromedia.com/pub/shockwave/cabs/director/sw.cab#version=8,0,0,0"; //$NON-NLS-1$

	private static final String PLUGIN_PAGE  = "http://www.macromedia.com/shockwave/download/"; //$NON-NLS-1$
	private static final String PLUGIN_TYPE = "application/x-director"; //$NON-NLS-1$
	

    
	/**
	 * Returns Class ID for Shockwave control.
	 * 
	 * @return Class ID
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectClassId()
	 */	
	public String getObjectClassId() {
		return DEFAULT_CLASSID;
	}

	/**
	 * Returns Code Base for Shockwave control.
	 * 
	 * @return Code Base
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectCodebase()
	 */
	public String getObjectCodebase() {
		return DEFAULT_CODEBASE;
	}

	/**
	 * Returns a list of parameters for Shockwave control.
	 * 
	 * @return array of parameters
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getObjectParams()
	 */
	protected Param[] getObjectParams() {
		return objectParams;
	}

	/**
	 * Returns URL to Shockwave's web site
	 * 
	 * @return URL to Shockwave's web site
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginPage()
	 */
	protected String getPluginPage() {
		return PLUGIN_PAGE;
	}

	/**
	 * Returns type of Shockwave plug-in
	 * 
	 * @return type of plugin
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginType()
	 */
	protected String getPluginType() {
		return PLUGIN_TYPE;
	}

	/**
	 * Returns a list of parameters for Shockwave plugin.
	 * 
	 * @return array of parameters
	 * @see com.ibm.faces.renderkit.html_extended.PlayerRenderer#getPluginParams()
	 */
	protected Param[] getPluginParams() {
		return pluginParams;
	}
}
