/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.util.HtmlUtil;


import com.ibm.faces.component.UIPager;
import com.ibm.faces.component.UIPager.PagerDataWrapper;
import com.ibm.faces.component.html.HtmlPagerSimple;

/**
 * <p>Simple pager renderer</p>
 * @author Alasdair McDowell/Paul Andrews
 */
public class SimplePagerRenderer extends AbstractPagerRenderer {

    private static final String PAGER_SIMPLE_CHOOSE = "___pagerSimple"; //$NON-NLS-1$
    private static final String PAGER_SIMPLE_GO = "___pagerSimpleGo"; //$NON-NLS-1$

    private static final String PAGER_PREVIOUS = "__pagerPrevious"; //$NON-NLS-1$
    private static final String PAGER_NEXT = "__pagerNext"; //$NON-NLS-1$
    private static final String PAGER_FIRST = "__pagerFirst"; //$NON-NLS-1$
    private static final String PAGER_LAST = "__pagerLast"; //$NON-NLS-1$

    private static final String PAGER_LAST_SELECTED = "__lastSelected"; //$NON-NLS-1$

    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        
        HtmlPagerSimple simpleComponent = null;
        if ( component instanceof HtmlPagerSimple ){
        	simpleComponent = (HtmlPagerSimple)component;
        }
        PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
        int lastPage = ((UIPager)component).getLastPage(pdw);

        ResponseWriter writer = context.getResponseWriter();
        ResourceBundle bundle = ResourceHandler.getBundle(context);

        //String localizedPager = ResourceHandler.getString(bundle, "PagerRenderer.pager"); //$NON-NLS-1$
        String localizedNext = ResourceHandler.getString(bundle, "PagerRenderer.nextPage"); //$NON-NLS-1$
        String localizedPrevious = ResourceHandler.getString(bundle, "PagerRenderer.previousPage"); //$NON-NLS-1$
        String localizedFirst = ResourceHandler.getString(bundle, "PagerRenderer.firstPage"); //$NON-NLS-1$
        String localizedLast = ResourceHandler.getString(bundle, "PagerRenderer.lastPage"); //$NON-NLS-1$
        String localizedGo = ResourceHandler.getString(bundle, "PagerRenderer.go"); //$NON-NLS-1$

        String id = component.getClientId(context);
        String selectId = id + PAGER_SIMPLE_CHOOSE;
        String inputId = id + PAGER_SIMPLE_GO;
        
        String style;
        String styleClass;
        String title;
        String tabindex;
        String imageSrc;
        
        if ( simpleComponent != null ){
        	style = simpleComponent.getStyle();
        	styleClass = simpleComponent.getStyleClass();
        	title = simpleComponent.getTitle();
        	tabindex = simpleComponent.getTabindex();
        	imageSrc = simpleComponent.getButtonImageSrc();
        }else{
			style = (String) component.getAttributes().get("style"); //$NON-NLS-1$
			styleClass = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
			title = (String) component.getAttributes().get("title"); //$NON-NLS-1$
			tabindex = (String) component.getAttributes().get("tabindex"); //$NON-NLS-1$
			imageSrc = (String) component.getAttributes().get("buttonImageSrc"); //$NON-NLS-1$
        }

        if(imageSrc != null){
        	imageSrc =  Utils.getEncodedResourceURL(context, imageSrc);
        }        


        HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$ 
        HtmlUtil.writeTagAttribute(writer, "style", style); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
        HtmlUtil.writeTagStartCloseLn(writer, false);

        HtmlUtil.writeTagStartOpen(writer, "select", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", selectId); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "name", selectId); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "size", "1"); //$NON-NLS-1$ //$NON-NLS-2$
        if(tabindex != null){
        	writer.writeAttribute("tabindex",tabindex,null);	
        }
        HtmlUtil.writeTagStartCloseLn(writer, false);

        //decide whether next or previous should be default selection
      
        String lastSelected =
            (String) component.getAttributes().get(PAGER_LAST_SELECTED);
        if (PAGER_PREVIOUS.equals(lastSelected)) {
            HtmlUtil.writeOption(writer, localizedPrevious, PAGER_PREVIOUS);
            HtmlUtil.writeOption(writer, localizedNext, PAGER_NEXT);
        }
        else {
            HtmlUtil.writeOption(writer, localizedNext, PAGER_NEXT);
            HtmlUtil.writeOption(writer, localizedPrevious, PAGER_PREVIOUS);
        }
        HtmlUtil.writeOption(writer, localizedFirst, PAGER_FIRST);
        if (lastPage >= 0) {
            HtmlUtil.writeOption(writer, localizedLast, PAGER_LAST);
        }

        HtmlUtil.writeTagEndLn(writer, "select"); //$NON-NLS-1$

        writer.write("\n");
        writer.startElement("input",component);
        if(imageSrc != null){
        	writer.writeAttribute("type","image",null);
        	writer.writeAttribute("src",imageSrc,null);
        } else {
        	writer.writeAttribute("type","submit",null);
        }
        writer.writeAttribute("id",inputId,null);
        writer.writeAttribute("name",inputId,null);
        writer.writeAttribute("value",localizedGo,null);
        if(style != null){
        	writer.writeAttribute("style",style,null);	
        }
        if(styleClass != null){
        	writer.writeAttribute("class",styleClass,null);	
        }
        if(title != null){
        	writer.writeAttribute("title",title,null);	
        	writer.writeAttribute("alt",title,null);
        }
        if(tabindex != null){
        	writer.writeAttribute("tabindex",tabindex,null);	
        }
        
        writer.endElement("input");
        writer.write("\n");
        
        HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
        
    }

    /**
     * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     */
    public void decode(FacesContext context, UIComponent component) {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        	
        PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
        if (pdw == null) {
            return;
        }

        if (!((UIPager)component).isPaged(pdw)) {
            return;
        }

        int newPage = -1;
        int currentPage = ((UIPager)component).getCurrentPage(pdw);
        int lastPage = ((UIPager)component).getLastPage(pdw);

        String keyGo = component.getClientId(context) + PAGER_SIMPLE_GO;
        if (context.getExternalContext().getRequestParameterMap().get(keyGo) != null ||
        	context.getExternalContext().getRequestParameterMap().get(keyGo + ".x") != null
			) {
            String keyChoice =
                component.getClientId(context) + PAGER_SIMPLE_CHOOSE;
            String valueChoice =
                (String) context
                    .getExternalContext()
                    .getRequestParameterMap()
                    .get(
                    keyChoice);

            if (valueChoice.equals(PAGER_NEXT)) {
                component.getAttributes().put(PAGER_LAST_SELECTED, PAGER_NEXT);
                newPage = currentPage + 1;
            }
            else if (valueChoice.equals(PAGER_PREVIOUS)) {
                component.getAttributes().put(
                    PAGER_LAST_SELECTED,
                    PAGER_PREVIOUS);
                newPage = currentPage - 1;
            }
            else if (valueChoice.equals(PAGER_FIRST)) {
                component.getAttributes().put(PAGER_LAST_SELECTED, PAGER_NEXT);
                newPage = 0;
            }
            else if ((valueChoice.equals(PAGER_LAST)) && (lastPage >= 0)) {
                component.getAttributes().put(
                    PAGER_LAST_SELECTED,
                    PAGER_PREVIOUS);
                newPage = lastPage >= 0 ? lastPage : currentPage + 1;
            }

            this.queueEvent(component, currentPage, newPage, lastPage);
        }
    }

}
