/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.Iterator;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.MethodBinding;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.UIRequest;
import com.ibm.faces.component.UISortHeader;
import com.ibm.faces.component.html.HtmlSortHeader;
import com.ibm.faces.util.JavaScriptUtil;
import com.ibm.faces.util.ResponseWriterBuffer;


public class SortHeaderRenderer extends RequestLinkRenderer {

	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {

		HtmlSortHeader tcomponent = null;
		if (component instanceof HtmlSortHeader) // check if this is a HtmlSortHeader
			tcomponent = (HtmlSortHeader)component;

		//get current state for the sort header if one exists
		String state = (String)context.getExternalContext().getRequestParameterMap().get("sort_order");
		String idClicked = (String)context.getExternalContext().getRequestParameterMap().get("sort_id");
		String isSortedId = (String)context.getExternalContext().getRequestParameterMap().get("sort_hidden_id");
		String isSortedOrder = (String)context.getExternalContext().getRequestParameterMap().get("sort_hidden_order");
		
		//get id
		String id = "";
		if(tcomponent != null)		
			id = tcomponent.getId();
		else		
			id = (String)component.getAttributes().get("id");

		//get style
		String style;
		if(tcomponent != null)		
			style = tcomponent.getStyleClass();
		else		
			style = (String)component.getAttributes().get("styleClass");

		//get initial state for the sort header
		String defaultState;
		if(tcomponent != null)		
			defaultState = tcomponent.getDefaultSortOrder();
		else		
			defaultState = (String)component.getAttributes().get("defaultSortOrder");
		
		//set up sort order
		String newSortOrder = "";
		String currentSortOrder= "";
		
		//if paging has occurred - need to set up sorting again if something is sorted
		if(state == null && isSortedId != null && isSortedId.equals(id)) {
			newSortOrder = getNewSortOrder(isSortedOrder);
			currentSortOrder = isSortedOrder;
			tcomponent.setStyleClass(style + "_" + isSortedOrder);
		
			//invoke action
			UIRequest requestComponent  = (UIRequest)tcomponent;
			MethodBinding method = requestComponent.getAction();
			if(method != null) {
				method.invoke(context, null);
			}
		}
		//else a sort header has been clicked on, update sort order
		else {

			//get styleClass for the component
			if(idClicked != null && idClicked.trim().equals(id.trim())) {
				
				//set style class for new sort order
				if(state != null){
					if(state.equals(UISortHeader.SORT_ASC)) {
						tcomponent.setStyleClass(style + "_" + UISortHeader.SORT_ASC);
					}
					else if(state.equals(UISortHeader.SORT_ASCON)) {
						tcomponent.setStyleClass(style + "_" + UISortHeader.SORT_ASCON);
					} 
					else if(state.equals(UISortHeader.SORT_DESC)) {
						tcomponent.setStyleClass(style + "_" + UISortHeader.SORT_DESC);
					}
					else if(state.equals(UISortHeader.SORT_DESCON)) {
						tcomponent.setStyleClass(style + "_" + UISortHeader.SORT_DESCON);
					}
					else if(state.equals(UISortHeader.SORT_BI)) {
						tcomponent.setStyleClass(style + "_" + UISortHeader.SORT_BI);
					}
					else if(state.equals(UISortHeader.SORT_BIUP)) {
						tcomponent.setStyleClass(style + "_" + UISortHeader.SORT_BIUP);
					}
					else if(state.equals(UISortHeader.SORT_BIDOWN)) {
						tcomponent.setStyleClass(style + "_" + UISortHeader.SORT_BIDOWN);
					}
				}
				else {
					tcomponent.setStyleClass(style + "_" + defaultState);
				}
			}
			else {
				tcomponent.setStyleClass(style + "_" + defaultState);
			}
			
			//set up request parameters on this component
			currentSortOrder = tcomponent.getStyleClass().substring(tcomponent.getStyleClass().lastIndexOf('_')+1, tcomponent.getStyleClass().length());
			newSortOrder = getNewSortOrder(currentSortOrder);
		}
		
		//remove current sort order and sort id parameters
		UIParameter p;
		
		for (Iterator it = component.getChildren().iterator(); it.hasNext(); ) {
			Object param = it.next();
			if(param instanceof UIParameter){
				p = (UIParameter)param;
				if(p.getName().equals("sort_order") || p.getName().equals("sort_id")){
					it.remove();
				}
			}
	    }
		
		//set up new sort order parameter
		UIParameter p2 = new UIParameter();
		p2.setId(component.getId()+"sort_order");
		p2.setName("sort_order");
		p2.setValue(newSortOrder);
		component.getChildren().add(p2);
		
		//set up new sort id parameter
		UIParameter p3 = new UIParameter();
		p3.setId(component.getId()+"sort_id");
		p3.setName("sort_id");
		p3.setValue(id);
		component.getChildren().add(p3);

		//set up id and value of hidden input fields if this column is sorted
		if(		currentSortOrder.equals(UISortHeader.SORT_ASCON) || 
				currentSortOrder.equals(UISortHeader.SORT_DESCON) || 
				currentSortOrder.equals(UISortHeader.SORT_BIUP) || 
				currentSortOrder.equals(UISortHeader.SORT_BIDOWN) ) {
			
			ResponseWriterBuffer sb = new ResponseWriterBuffer();
			ResponseWriter writer = context.getResponseWriter(); // get a sb
			
			//write out hidden input fields to store information
			writer.startElement("input", component);
	        writer.writeAttribute("type", "hidden", null);
	        writer.writeAttribute("id", id + "_sort_hidden_id", null);
	        writer.writeAttribute("name", "sort_hidden_id", null);
	        writer.writeAttribute("value", id, null);
	        
			writer.startElement("input", component);
	        writer.writeAttribute("type", "hidden", null);
	        writer.writeAttribute("id", id + "_sort_hidden_order", null);
	        writer.writeAttribute("name", "sort_hidden_order", null);
	        writer.writeAttribute("value", currentSortOrder, null);
		}
		
		//start the <a> tag
		super.encodeBegin(context, component);
	}
	
	public void encodeChildren(FacesContext context, UIComponent component)
	throws IOException {
		super.encodeChildren(context, component);
	}
	
	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {

		HtmlSortHeader tcomponent = null;
		if (component instanceof HtmlSortHeader) // check if this is a HtmlSortHeader
			tcomponent = (HtmlSortHeader)component;
		
		ResponseWriterBuffer sb = new ResponseWriterBuffer();
		ResponseWriter writer = context.getResponseWriter(); // get a sb	
		
		writer.startElement("img",component);

		String stateImagePath = null;

		String state = tcomponent.getStyleClass().substring(tcomponent.getStyleClass().lastIndexOf('_')+1, tcomponent.getStyleClass().length());
		String className = tcomponent.getStyleClass() + "-img";
		String alt = null;
		
		if(state.equals(UISortHeader.SORT_ASC)) {
			String sortasc = null;
			if(tcomponent != null)		
				sortasc = tcomponent.getSrcAscending();
			else		
				sortasc = (String)component.getAttributes().get("srcAscending");
			
			if(sortasc != null) {
				stateImagePath = sortasc;
				className = null;
			}
			alt = "SortHeaderRenderer.sortasc";
		}
		else if(state.equals(UISortHeader.SORT_ASCON)) {
			String sortascon = null;
			if(tcomponent != null)		
				sortascon = tcomponent.getSrcAscendingOn();
			else		
				sortascon = (String)component.getAttributes().get("srcAscendingOn");
			
			if(sortascon != null) {
				stateImagePath = sortascon;
				className = null;
			}
			alt = "SortHeaderRenderer.sortascon";
		}
		else if (state.equals(UISortHeader.SORT_DESC)) {
			String sortdesc = null;
			if(tcomponent != null)		
				sortdesc = tcomponent.getSrcDescending();
			else		
				sortdesc = (String)component.getAttributes().get("srcDescending");
			
			if(sortdesc != null) {
				stateImagePath = sortdesc;
				className = null;
			}
			alt = "SortHeaderRenderer.sortdesc";
		}
		else if (state.equals(UISortHeader.SORT_DESCON)) {
			String sortdescon = null;
			if(tcomponent != null)		
				sortdescon = tcomponent.getSrcDescendingOn();
			else		
				sortdescon = (String)component.getAttributes().get("srcDescendingOn");
			
			if(sortdescon != null) {
				stateImagePath = sortdescon;
				className = null;
			}
			alt = "SortHeaderRenderer.sortdescon";
		}
		else if (state.equals(UISortHeader.SORT_BI)) {
			String sortbi = null;
			if(tcomponent != null)		
				sortbi = tcomponent.getSrcBi();
			else		
				sortbi = (String)component.getAttributes().get("srcBi");
			
			if(sortbi != null) {
				stateImagePath = sortbi;
				className = null;
			}
			alt = "SortHeaderRenderer.sortbi";
		}
		else if (state.equals(UISortHeader.SORT_BIUP)) {
			String sortbiup = null;
			if(tcomponent != null)		
				sortbiup = tcomponent.getSrcBiUp();
			else		
				sortbiup = (String)component.getAttributes().get("srcBiUp");
			
			if(sortbiup != null) {
				stateImagePath = sortbiup;
				className = null;
			}
			alt = "SortHeaderRenderer.sortbiup";
		}
		else if (state.equals(UISortHeader.SORT_BIDOWN)) {
			String sortbidown = null;
			if(tcomponent != null)		
				sortbidown = tcomponent.getSrcBiDown();
			else		
				sortbidown = (String)component.getAttributes().get("srcBiDown");
			
			if(sortbidown != null) {
				stateImagePath = sortbidown;
				className = null;
			}
			alt = "SortHeaderRenderer.sortbidown";
		}

		if (null != stateImagePath) {
			writer.writeAttribute("src", Utils.getEncodedResourceURL(context, stateImagePath), null);
		} else {
			writer.writeAttribute("src", 
					context.getExternalContext().encodeResourceURL(JavaScriptUtil.getImagePath(context, "1x1.gif")), null);
		}
		
		if(className != null)
			writer.writeAttribute("class", className, null);
		if(alt != null){
			ResourceBundle bundle = ResourceHandler.getBundle(context);
			String altText = ResourceHandler.getString(bundle, alt);
			writer.writeAttribute("title", altText, null);
			writer.writeAttribute("alt", altText, null);
		}
		writer.endElement("img");
		
		sb.checkTagOpenState();
		writer.write(sb.toString());
		sb.clearBuffer();
		
		super.encodeEnd(context, component);
		
		//reset style class
		String style = tcomponent.getStyleClass().substring(0, tcomponent.getStyleClass().lastIndexOf('_'));
		tcomponent.setStyleClass(style);
	}
	
	public String getNewSortOrder(String state) {
		if(state.equals(UISortHeader.SORT_ASC)) {
			return UISortHeader.SORT_ASCON;
		}
		else if(state.equals(UISortHeader.SORT_ASCON)) {
			return UISortHeader.SORT_ASC;
		} 
		else if(state.equals(UISortHeader.SORT_DESC)) {
			return UISortHeader.SORT_DESCON;
		}
		else if(state.equals(UISortHeader.SORT_DESCON)) {
			return UISortHeader.SORT_DESC;
		}
		else if(state.equals(UISortHeader.SORT_BI)) {
			return UISortHeader.SORT_BIUP;
		}
		else if(state.equals(UISortHeader.SORT_BIUP)) {
			return UISortHeader.SORT_BIDOWN;
		}
		else if(state.equals(UISortHeader.SORT_BIDOWN)) {
			return UISortHeader.SORT_BI;
		}
		
		return "";
	}
}