/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.io.Writer;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.context.FacesContext;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.util.HtmlUtil;
import com.ibm.faces.util.JavaScriptUtil;
import javax.faces.render.Renderer;

import com.ibm.faces.component.html.HtmlOutputStatistics;

/**
 * @author Alasdair McDowell
 * TODO - description
 */
public class StatisticsRenderer extends Renderer implements IScriptContributor{

    private static final String PAD_STATISTICS = "__stats"; //$NON-NLS-1$
    private static final String JS_STATISTICS = "__jsStatistics"; //$NON-NLS-1$

    /*
     *  (non-Javadoc)
     * @see javax.faces.render.Renderer#encodeBegin(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     */
    public void encodeBegin(FacesContext context, UIComponent component)  throws IOException 
    {
    	HxClientRenderUtil.initJSLibraries(component, context);
    	HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
		HxClientRenderUtil.addScriptLibrary("hxclient_table.js", component);
    }
    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        
    	if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
		HtmlOutputStatistics statComponent = null;
		if ( component instanceof HtmlOutputStatistics ){
			statComponent = (HtmlOutputStatistics)component;
		}
		
        UIData data = this.findUIData(component);
        if (data == null) {
            return;
        }

        Writer writer = context.getResponseWriter();
        ResourceBundle bundle = ResourceHandler.getBundle(context);

        String id = component.getClientId(context);

        //Set style information
        String style;
        String styleClass;
        
    	String pagingOn;
		String selectedOn;
		String totalOn;
		String shownOn;
		String filteredOn;
		String statusTextOn;
		String title;
        
        if ( statComponent != null ){
        	style = statComponent.getStyle();
        	styleClass = statComponent.getStyleClass();
        	pagingOn = statComponent.getPaging();
			selectedOn = statComponent.getSelected();
			totalOn = statComponent.getTotal();
			shownOn = statComponent.getShown();
			filteredOn = statComponent.getFiltered();
			statusTextOn = statComponent.getStatusText();
			title = statComponent.getTitle();
        }else{
			style = (String) component.getAttributes().get("style");
			styleClass = (String) component.getAttributes().get("styleClass");
			pagingOn = (String) component.getAttributes().get("paging");
			selectedOn = (String) component.getAttributes().get("selected");
			totalOn = (String) component.getAttributes().get("total");
			shownOn = (String) component.getAttributes().get("shown");
			filteredOn = (String) component.getAttributes().get("filtered");
			statusTextOn = (String) component.getAttributes().get("statusText");
			title = (String) component.getAttributes().get("title");
        }

        //if nothing is set to true or false - possibly turn on paging and selected information to be backward compatible 
    	if(pagingOn == null && selectedOn == null && totalOn == null && shownOn ==null && filteredOn == null && statusTextOn == null) {
    		pagingOn = "true";
    		selectedOn = "true";
    	}
        
        //get information first
        //PAGING
        int currentPage = this.getCurrentPage(data);
        int lastPage = this.getLastPage(data);

        //SELECT
        String jsStatistics = id + JS_STATISTICS;
        jsStatistics = JavaScriptUtil.makeVariableName(jsStatistics);
        int numSelected = -99;
        Job job = new Job(jsStatistics);
        // data(1) = the UIData, data(2) = recursively search this for select component, job = job to run
        boolean supportsSelection = this.getSelectedRows(data, data, job);
        if(supportsSelection)
			numSelected = job.getSelected();
        
        //TOTAL
        int totalRows = this.getTotalRows(data);
        
        //SHOWN
        int shownRows = this.getShownRows(data);
        
        //FILTERED
        String filteredRows = "-99";
        if(filteredOn !=null && filteredOn.trim().length() > 0) {
        	filteredRows = statComponent.getFiltered();
        }
        
        //check if using statusText or customised information
        if(statusTextOn == null || statusTextOn.trim().equals("")) {

        	//no statusText set - using resource Strings instead
	        HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
	        if(id != null)
	        	HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
	        if(title != null)
	        	HtmlUtil.writeTagAttribute(writer, "title", title); //$NON-NLS-1$
	        if(style != null)
	        	HtmlUtil.writeTagAttribute(writer, "style", style); //$NON-NLS-1$
	        if(styleClass != null)
	        	HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
	        HtmlUtil.writeTagStartCloseLn(writer, false);
	        
        	if(pagingOn!=null && pagingOn.trim().equalsIgnoreCase("true")) {
                //Print out PAGING information
                if (lastPage >= 0) {
                	HtmlUtil.writeTagStartOpen(writer, "span", true, false); //$NON-NLS-1$
                	Object[] args = { new Integer(currentPage + 1), new Integer(lastPage + 1)};
        	        String localizedPageOf = ResourceHandler.getString(bundle, "PagerRenderer.pageof", args); //$NON-NLS-1$
        	        HtmlUtil.writeText(writer, localizedPageOf);
        	        HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
                }
        	}
        	
        	if(selectedOn!=null && selectedOn.equalsIgnoreCase("true")) {
                
                String localizedSelected = ResourceHandler.getString(bundle, "DatagridStatisticsRenderer.Selected"); //$NON-NLS-1$
                
        		//Print out SELECT information
        		if (supportsSelection) {
                    writer.write(" "); //$NON-NLS-1$
                    HtmlUtil.writeTagStartOpen(writer, "span", true, false); //$NON-NLS-1$
                    HtmlUtil.writeText(writer, localizedSelected);
                    HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
                    HtmlUtil.writeTagAttribute(writer, "id", id + PAD_STATISTICS); //$NON-NLS-1$
                    HtmlUtil.writeTagStartClose(writer, false);
                    writer.write("" + numSelected); //$NON-NLS-1$
                    HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
                    HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
                }

                if (supportsSelection) {
                    StringBuffer script = new StringBuffer();
                    script.append("var ") //$NON-NLS-1$
                    .append(jsStatistics).append(" = new ") //$NON-NLS-1$
        			.append(HxClientRenderUtil.HXG_FUNCTION_NAME)
        			.append(".prototype.HxStatisticsListener(\"")
                    .append(id + PAD_STATISTICS).append("\", ") //$NON-NLS-1$
                    .append(job.getSelected()).append(");\r\n"); //$NON-NLS-1$
                    this.writeJavaScript(context, component, script.toString());

                    this.writeJavaScript(
                        context,
                        component,
                        job.getScript().toString());
                }      		
        	}
        	
        	if(totalOn!=null && totalOn.trim().equalsIgnoreCase("true")) {
                //print the TOTAL number of rows
        		HtmlUtil.writeTagStartOpen(writer, "span", true, false); //$NON-NLS-1$
        		Object[] args1 = { new Integer(totalRows)};
                String localizedTotal = ResourceHandler.getString(bundle, "DatagridStatisticsRenderer.Total", args1); //$NON-NLS-1$
                HtmlUtil.writeText(writer, localizedTotal);
                HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$        		
        	}
        	
        	if(shownOn!=null && shownOn.trim().equalsIgnoreCase("true")) {
                //print the SHOWN number of rows
        		HtmlUtil.writeTagStartOpen(writer, "span", true, false); //$NON-NLS-1$
                Object[] args2 = { new Integer(shownRows)};
                String localizedShown = ResourceHandler.getString(bundle, "DatagridStatisticsRenderer.Shown", args2); //$NON-NLS-1$
                HtmlUtil.writeText(writer, localizedShown);
                HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
        	}
        	
            //print the FILTERED number of rows
            if(filteredOn!=null && filteredOn.trim().length() > 0) {
            	HtmlUtil.writeTagStartOpen(writer, "span", true, false); //$NON-NLS-1$
                Object[] args3 = { new Integer(filteredRows)};
                String localizedFiltered = ResourceHandler.getString(bundle, "DatagridStatisticsRenderer.Filtered", args3); //$NON-NLS-1$
                HtmlUtil.writeText(writer, localizedFiltered);
                HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
            }

            HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
            
        }
        else {
        	//output to the statistics footer
	        HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
	        if(id != null)
	        	HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
	        if(title != null)
	        	HtmlUtil.writeTagAttribute(writer, "title", title); //$NON-NLS-1$
	        if(style != null)
	        	HtmlUtil.writeTagAttribute(writer, "style", style); //$NON-NLS-1$
	        if(styleClass != null)
	        	HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
	        HtmlUtil.writeTagStartCloseLn(writer, false);
            
        	//use status text
        	String statusText = statusTextOn;
        	
        	//{0} = currentPage, {1} = last Page, {2} = numSelected
        	//{3} = totalRows, {4} = shownRows, {5} = filteredRows
        	Integer filter = new Integer(filteredRows);
        	int args[] = {currentPage+1, lastPage+1, numSelected, totalRows, shownRows, filter.intValue()};

        	//add values into the String (apart from selection)
        	for(int i=0; i<6; i++) {
        		if(i!=2){
		        	while(statusText.indexOf("{" + i + "}") >= 0) {
		        		statusText = statusText.substring(0,statusText.indexOf("{" + i + "}")) + args[i] + statusText.substring(statusText.indexOf("{" + i + "}")+3, statusText.length());
		        	}
        		}
        	}

        	//check if selection is needed
        	int x = 0;
        	while(statusText.indexOf("{2}") >= 0 && supportsSelection) {
    			
    			HtmlUtil.writeText(writer, statusText.substring(0,statusText.indexOf("{2}")));
    			HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
    	        HtmlUtil.writeTagAttribute(writer, "id", id + PAD_STATISTICS + x); //$NON-NLS-1$
    	        HtmlUtil.writeTagStartClose(writer, false);
    			writer.write("" + numSelected); //$NON-NLS-1$
                HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$

                StringBuffer script = new StringBuffer();
                script.append("var ") //$NON-NLS-1$
                .append(jsStatistics).append(" = new ") //$NON-NLS-1$
    			.append(HxClientRenderUtil.HXG_FUNCTION_NAME)
    			.append(".prototype.HxStatisticsListener(\"")
                .append(id + PAD_STATISTICS + x).append("\", ") //$NON-NLS-1$
                .append(job.getSelected()).append(");\r\n"); //$NON-NLS-1$
                this.writeJavaScript(context, component, script.toString());

                this.writeJavaScript(
                    context,
                    component,
                    job.getScript().toString());
                
                //update statusText String
                statusText = statusText.substring(statusText.indexOf("{2}")+3, statusText.length());
                
                x++;
        	}       		
        	
        	HtmlUtil.writeText(writer, statusText);
        	HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$
        }

    }

    private UIData findUIData(UIComponent component) {
		
    	HtmlOutputStatistics statComponent = null;
		if ( component instanceof HtmlOutputStatistics ){
			statComponent = (HtmlOutputStatistics)component;
		}
		
		String _for;
		if ( statComponent != null ){
			_for = statComponent.getFor();
		}else{
			_for = (String) component.getAttributes().get("for");
		}
        
        if (_for == null) {
            return getUIDataParent(component);
        }
        UIComponent c = component.findComponent(_for);
        if ((c == null) || (!(c instanceof UIData))) {
            return null;
        }
        return (UIData) c;
    }

    private UIData getUIDataParent(UIComponent component) {
        
    	component = component.getParent();
        if (component == null) {
            return null;
        }
        if (component instanceof UIData) {
            return (UIData) component;
        }
        return getUIDataParent(component);
    }

    private boolean getSelectedRows(UIData data, UIComponent component, Job job) {
        
    	if (data == null) {
            return false;
        }

        //String must match renderer name in faces config
        if ("com.ibm.faces.RowSelect".equals(component.getRendererType())) { //$NON-NLS-1$
            int start = data.getFirst();
            int len;
            int rows = data.getRows();
            if (rows > 0) {
                len = start + rows;
            }
            else {
                len = data.getRowCount();
            }
            for (int i = start; i < len; i++) {
                data.setRowIndex(i);
                if (!data.isRowAvailable()) {
                    break;
                }
                job.processRow(component);
            }
            data.setRowIndex(-1);
            return true;
        }

        List children = component.getChildren();
        for (int i = 0, len = children.size(); i < len; i++) {
            UIComponent child = (UIComponent) children.get(i);
            if (getSelectedRows(data, child, job)) {
                return true;
            }
        }
        return false;
    }

    private void writeJavaScript(FacesContext context, UIComponent component, String script) throws IOException {

        UIScriptCollector scriptCollector = UIScriptCollector.find(component);

        if (scriptCollector != null) {
            HxClientRenderUtil.initJSLibraries(scriptCollector, context);
            scriptCollector.addScript(script);
        }
        else {
            //failsafe
            Writer writer = context.getResponseWriter();
            writer.write(JavaScriptUtil.getStartTag());
            writer.write(script);
            writer.write(JavaScriptUtil.getEndTag());
        }
    }

    private class Job {

        private int selected = 0;
        private int total = 0;
        private int shown = 0;

        private String jsStatistics;
        private StringBuffer script = new StringBuffer();

        Job(String jsStatistics) {
            this.jsStatistics = jsStatistics;
        }

        void processRow(UIComponent component) {
            boolean bSelected = false;
            if (RowSelectRenderer.isRowSelected(component)) {
                selected++;
                bSelected = true;
            }
            
            String selectionVar =
                RowSelectRenderer.getJsSelectionVariable(
                    component.getClientId(FacesContext.getCurrentInstance()));

            script.append("if(!") //$NON-NLS-1$
            .append(selectionVar).append(") var ") //$NON-NLS-1$
            .append(selectionVar).append(" = new ") //$NON-NLS-1$
			.append(HxClientRenderUtil.HXG_FUNCTION_NAME)
			.append(".prototype.HxSelection(")
            .append(bSelected).append("); \r\n"); //$NON-NLS-1$
            script.append(selectionVar).append(".addSelectionListener(") //$NON-NLS-1$
            .append(jsStatistics).append(");\r\n"); //$NON-NLS-1$
            
        }

        StringBuffer getScript() {
            return script;
        }

        int getSelected() {
            return selected;
        }
    }

    /**
     * Calculates the current page of data.
     * @param data
     * @return  the current page or 0 if not paged
     */
    private int getCurrentPage(UIData data) {
        if (data == null) {
            return 0;
        }

        int rows = data.getRows();
        if (rows <= 0) {
            return 0;
        }

        return data.getFirst() / rows;
    }

    /**
     * Calculates the number of rows on the current page of data.
     * @param data
     * @return  the number of rows
     */
    private int getShownRows(UIData dataComponent) {
        if (dataComponent == null) {
            return 0;
        }

        int processedRowCount = 0;
        Boolean processRow = Boolean.TRUE;
        int rowIndex = dataComponent.getFirst();
        int rowsToDisplay = dataComponent.getRows();
        
		while(processRow.booleanValue()){
			++processedRowCount;
			dataComponent.setRowIndex(++rowIndex);
			if(!dataComponent.isRowAvailable() || (rowsToDisplay > 0 && processedRowCount >= rowsToDisplay)){
				processRow = Boolean.FALSE;
			}
		}

		dataComponent.setRowIndex(-1);
        return processedRowCount;
    }
    
    /**
     * Calculates the size of the total dataset
     * @param data
     * @return  size of the dataset
     */
    private int getTotalRows(UIData data) {
        
    	if (data == null) {
            return 0;
        }

        int rows = data.getRows();
        int max = data.getRowCount();
        
        if (rows <= 0 && max <= 0) {
            if (max < 0) {
                return -1;
            }
        	
        	return 0;
        }
        
        return max;
    }
	
    /**
     * Calculates the last page of the data.
     * @param data
     * @return  the last page (== number of pages) or -1 if unknown
     */
    private int getLastPage(UIData data) {
        if (data == null) {
            return 0;
        }

        int rows = data.getRows();
        if (rows <= 0) {
            return 0;
        }
        int max = data.getRowCount();
        if (max < 0) {
            return -1;
        }
        else if (max > 0) {
            int lastPage = (max / rows);
            if (max % rows == 0) {
                lastPage--;
            }
            return lastPage;
        }
        else {
            return 0;
        }
    }


	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		// TODO Auto-generated method stub
		
	}
}