/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.ValueBinding;

import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlInputHelperTypeahead;
import com.ibm.faces.util.AjaxUtil;



public class TypeaheadRenderer extends AjaxBaseRenderer{
   
	// parameter constants from incoming request
	public static final String PARAM_KEYNAME 	= "__searchKey__";			//$NON-NLS-1$	
	public static final String PARAM_VBEXPR 	= "__vbExpression__";		//$NON-NLS-1$
	private static final String PARAM_MAXRES 	= "__maxResults__";			//$NON-NLS-1$
	
	// attributes for Typeahead component
	private final static String ATTR_VALUE 		= "value"; 				//$NON-NLS-1$
	
	// Markup for constructing XML Doc
	private final static String XML_HEADER 		="<?xml version='1.0' encoding='ISO-8859-1'?>";	//$NON-NLS-1$
	private final static String XML_TA_OPEN 	= "<taresponse>"; 		//$NON-NLS-1$
	private final static String XML_TA_CLOSE 	= "</taresponse>"; 		//$NON-NLS-1$
	private final static String XML_URL_OPEN 	= "<url>"; 				//$NON-NLS-1$
	private final static String XML_URL_CLOSE 	= "</url>"; 				//$NON-NLS-1$
	private final static String XML_LIST_OPEN 	= "<suggestlist>"; 		//$NON-NLS-1$
	private final static String XML_LIST_CLOSE 	= "</suggestlist>"; 	//$NON-NLS-1$
	private final static String XML_ITEM_OPEN 	= "<suggestion>"; 		//$NON-NLS-1$
	private final static String XML_ITEM_CLOSE 	= "</suggestion>"; 		//$NON-NLS-1$
	
	
	
   public void encodeBegin(FacesContext context, UIComponent component)
         throws IOException {

      if(context == null || component == null) 
         throw new NullPointerException();

      if (!(component.getParent() instanceof UIInput)) {
         return;
      }

      if(!AjaxUtil.isAjaxRequest(context) || (AjaxUtil.isAjaxRequest(context) && AjaxUtil.isRendering(context)) 
      											&& !(component.getClientId(context)).equalsIgnoreCase(AjaxUtil.getAjaxComponentId(context)))
      {	// a normal request || an ajax request when your ancestor is to be rendered and you are no teh target
	      
	      HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
	      HxClientRenderUtil.addScriptLibrary("hxclient_bc.js", component);
	      HxClientRenderUtil.addScriptLibrary("hxclient_xml.js", component);
	      HxClientRenderUtil.addScriptLibrary("hxclient_ta.js", component);
			
	      //ResponseWriter writer = context.getResponseWriter(); // create a writer
	     
	      UIScriptCollector scriptCollector = UIScriptCollector.find(component);
	      //scriptCollector.register(this, component);
	      HxClientRenderUtil.initJSLibraries(scriptCollector, context);
	      
	      contributeScript(context, component);
      }
      else // this is an ajax request
      {		// and should only run if this component is targeted
      		if((component.getClientId(context)).equalsIgnoreCase(AjaxUtil.getAjaxComponentId(context)))
      		{
      			doAnswer(context, component);
      			//ajaxReRendering(context, component);
      		}
      }
   }

   public void encodeEnd(FacesContext context, UIComponent component)
         throws IOException {
      
      //do nothing
   }

   /* (non-Javadoc)
    * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
    */
   public void contributeScript(FacesContext context, UIComponent component) throws IOException {

   	
   	// Get the attributes from the tag / component
    String binding, css, id, inprogress, oncomplete, onstart, type,
		matchwidth, size, chars, delay, maxsugg;
	if (component != null) {
		binding = ((HtmlInputHelperTypeahead)component).getBinding();
		css = ((HtmlInputHelperTypeahead)component).getStyleClass();
		inprogress = ((HtmlInputHelperTypeahead)component).getInProgress();
		oncomplete = ((HtmlInputHelperTypeahead)component).getOncomplete();
		onstart = ((HtmlInputHelperTypeahead)component).getOnstart();
		type = ((HtmlInputHelperTypeahead)component).getType();
		matchwidth = ((HtmlInputHelperTypeahead)component).getMatchWidth();
		size = ((HtmlInputHelperTypeahead)component).getSize();
		maxsugg = ((HtmlInputHelperTypeahead)component).getMaxSuggestions();
		chars = ((HtmlInputHelperTypeahead)component).getStartCharacters();
		delay = ((HtmlInputHelperTypeahead)component).getStartDelay();
	} else {
		binding = (String)component.getAttributes().get("binding"); 
		css = (String)component.getAttributes().get("styleClass");
		inprogress = (String)component.getAttributes().get("inProgress");
		oncomplete = (String)component.getAttributes().get("oncomplete");
		onstart = (String)component.getAttributes().get("onstart");
		type = (String)component.getAttributes().get("type");
		matchwidth = (String)component.getAttributes().get("matchWidth");
		size = (String)component.getAttributes().get("size");
		maxsugg = (String)component.getAttributes().get("maxSuggestions");
		chars = (String)component.getAttributes().get("startCharacters");
		delay = (String)component.getAttributes().get("startDelay");         
	}	

      UIInput parent = (UIInput) component.getParent();
      
      UIScriptCollector scriptCollector = UIScriptCollector.find(component);
      
      
      StringBuffer script = new StringBuffer(HxClientRenderUtil.HX_VAR_NAME);
      script.append(".addBehavior(\"")
            .append(parent.getClientId(context))
            .append("\", \"onkeydown\", new ")
            .append(HxClientRenderUtil.HX_VAR_NAME)
            .append(".JSFBehaviorTypeahead(");
      boolean needsComma = true;
      script.append("\"id:")
      	.append(component.getClientId(context))
		.append("\"");
      if (css != null && css.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"css:")
              .append(css)
              .append("\"");
        needsComma = true;
      }
            
      
      String href = buildURL(context, component, component.getClientId(context), false);
		
      if (href != null && href.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"href:")
              .append(href)
              .append("\"");
        needsComma = true;
      }     
      
      
      
      
      if (oncomplete != null && oncomplete.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"oncomplete:")
              .append(oncomplete)
              .append("\"");
        needsComma = true;
      }
      if (onstart != null && onstart.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"onstart:")
              .append(onstart)
              .append("\"");
      }
      if (type != null && type.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"type:")
              .append(type)
              .append("\"");
      }
      if (matchwidth != null && matchwidth.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"matchwidth:")
              .append(matchwidth)
              .append("\"");
      }
      if (size != null && size.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"size:")
              .append(size)
              .append("\"");
      }
      if (maxsugg != null && maxsugg.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"maxsugg:")
              .append(maxsugg)
              .append("\"");
      }
      if (chars != null && chars.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"chars:")
              .append(chars)
              .append("\"");
      }
      if (delay != null && delay.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"delay:")
              .append(delay)
              .append("\"");
      }
      
      //append Value Binding - we need to pass this back up on an AJAX request in Portal
      ValueBinding vb = component.getValueBinding("value");
      String value = (null != vb) ? vb.getExpressionString() : null;
      
      if (value != null && value.length() > 0) {
      	if (needsComma) {
      		script.append(",");
      	}
      	script.append("\"vbexpression:")
              .append(value)
              .append("\"");
      }
      
      script.append("));\n");
     // context.getResponseWriter().write(script.toString());
      scriptCollector.addScript(script.toString());
      
   }

   /*
    * This writes the answer string to the response writer
    * */
   public void doAnswer(FacesContext context, UIComponent component) throws IOException
   {
   		ExternalContext extContext = context.getExternalContext();
		
		//changing for Portal testing
		Map parameters = extContext.getRequestParameterMap();

		int max_res = Integer.parseInt((String)parameters.get(PARAM_MAXRES));
		String key = (String)parameters.get(PARAM_KEYNAME);
		
		String valueExpr = (String)parameters.get(PARAM_VBEXPR);
		ValueBinding vb;
		
		vb = component.getValueBinding(ATTR_VALUE); //$NON-NLS-1$
		
		if(vb == null)
		{
			vb = context.getApplication().createValueBinding(valueExpr);
		}
		
		
		Object search_obj =  vb.getValue(context);
		Object search_return = ((Map)search_obj).get(key);
		
		/* so the object returned, should be null in the case of no matches.
		 * a string in the case of one hit and an arraylist of strings in teh case of mulitple hits
		*/
		
		String xml_result = process(search_return, max_res, buildURL(context, component, component.getClientId(context), false));

		ResponseWriter writer = context.getResponseWriter(); // create a writer
		
		writer.write(xml_result);				
	
   	
   }
   
	protected void ajaxReRendering(FacesContext context, UIComponent component) throws IOException {
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);

		String target = null;
	
		//get attribute values from the component
		UIInput parent = (UIInput) component.getParent();
		target = parent.getClientId(context);
		
		String href = buildURL(context, component, component.getClientId(context), false);
		
		StringBuffer ajaxScript = new StringBuffer("if(");
		// construct JavaScript String
		ajaxScript.append(HxClientRenderUtil.HX_VAR_NAME) 
					.append(".getBehaviorById(\"")					
					.append(target);
		
		ajaxScript.append("\", \"keydown\"");
					
        ajaxScript.append(", \"onkeydown\"))") 
					.append(HxClientRenderUtil.HX_VAR_NAME)		
					.append(".getBehaviorById(\"")					
					.append(target);
        
		ajaxScript.append("\", \"keydown\"");
		
		ajaxScript.append(", \"onkeydown\").setAttribute(\"href:") ;

		ajaxScript.append(href);
		ajaxScript.append("\"");			
	
		ajaxScript.append(");\n");
		
		// render JavaScript to page
		scriptCollector.addScript(ajaxScript.toString());
		
	}
   
	/*
	 * this methods takes the object returned and creates the appropriate xml string to return to
	 * the client
	 * The response sends back an XML document with the search results:
	 * 
	 * <?xml..>
	 * <taresponse>
	 * 	<suggestlist>
	 * 		<suggestion>aardvark</suggestion>
	 * 		...
	 * 		<suggestions>aardwolf</suggestion>
	 * 	</suggestlist>
	 * </taresponse>
	 * 
	 * If there are no suggestions returned, there are no child tags of the suggestlist tag
	 * The suggestions are followed by a url delimitted by a tag (not part of above xml document):
	 * 
	 * <url>new/Typahead/url.jsp</url> 
	 */
	private String process(Object result, int max_res, String target_url)
	{
		StringBuffer responseXml = new StringBuffer(XML_HEADER); 
		responseXml.append(XML_TA_OPEN);
		responseXml.append(XML_LIST_OPEN);

		if(result != null) 
		{
	
			if(result instanceof String)
			{
				responseXml.append(XML_ITEM_OPEN);
				responseXml.append(((String)result));
				responseXml.append(XML_ITEM_CLOSE);
			}
			else if(result instanceof ArrayList)
			{
				ArrayList list = ((ArrayList)result);
				
				if(list.size() == 0)
				{
					//do nothing
				}
				else if(list.get(0) instanceof String)
				{
					for(int i = 0; (i < list.size()) && (i < max_res); i++)
					{
						String val = (String) list.get(i);
						if (null != val) {
							responseXml.append(XML_ITEM_OPEN);
							responseXml.append(val);
							responseXml.append(XML_ITEM_CLOSE);
						}
					}
				}
			}
			else 
			{
				// return ((String)result);
			}
			
		}
					
		// close XML Document
		responseXml.append(XML_LIST_CLOSE);
		responseXml.append(XML_TA_CLOSE);
		
		// add delimited URL
		responseXml.append(XML_URL_OPEN);
		responseXml.append(target_url);
		responseXml.append(XML_URL_CLOSE);

		// write to response
		return responseXml.toString();
	}
	
}