/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2004 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.faces.component.NamingContainer;
import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;

import com.ibm.faces.component.UIDataIterator;

import java.text.DateFormat;


public class Utils {
	
	//common strings
	public static final String DATESTYLE_SHORT = "short"; //$NON-NLS-1$
	public static final String DATESTYLE_MEDIUM = "medium"; //$NON-NLS-1$
	public static final String DATESTYLE_LONG = "long"; //$NON-NLS-1$
	public static final String DATESTYLE_FULL = "full"; //$NON-NLS-1$
	
	public static final String NUMBERSTYLE_NUMBER = "number"; //$NON-NLS-1$
	public static final String NUMBERSTYLE_CURRENCY = "currency"; //$NON-NLS-1$
	public static final String NUMBERSTYLE_PERCENT = "percent"; //$NON-NLS-1$
	
	/**
	 * Parse the key expression.
	 * Parse result, modKey and Key, is available by using getModKey() and getKey()
	 *
	 */
	private String _modKey = ""; //$NON-NLS-1$
	private String _key = ""; //$NON-NLS-1$
	
	public Utils(String attr){
		parseKeyExpression(attr);
	}

	private void parseKeyExpression(String attr) {
		_modKey = ""; //$NON-NLS-1$
		_key = ""; //$NON-NLS-1$

		if (attr == null)
			return ;

		int x = attr.indexOf('+');
		if (x == -1) {
			_key = attr;
		}
		else {
			_modKey = attr.substring(0, x);
			_key = attr.substring(x+1);
		}
	}

	public String getModKey() { return _modKey; }
	public String getKey() { return _key; }
	public String getKeyCode() { 
		if (_key.length() > 0 && Character.isDigit(_key.charAt(0))) {
			return _key;
		}
		return getKeyCode(_key);
	}
	private Map _keyTable = null;

	private void _createKeyTable () {
		_keyTable = new HashMap();

		_keyTable.put("tab", 		"9"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("enter", 		"13"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("space", 		"32"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f1",			"112"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f2",			"113"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f3",			"114"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f4",			"115"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f5",			"116"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f6",			"117"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f7",			"118"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f8",			"119"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f9",			"120"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f10",		"121"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f11",		"122"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f12",		"123"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("a",			"65"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("b",			"66"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("c",			"67"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("d",			"68"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("e",			"69"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("f",			"70"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("g",			"71"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("h",			"72"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("i",			"73"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("j",			"74"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("k",			"75"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("l",			"76"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("m",			"77"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("n",			"78"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("o",			"79"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("p",			"80"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("q",			"81"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("r",			"82"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("s",			"83"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("t",			"84"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("u",			"85"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("v",			"86"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("w",			"87"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("x",			"88"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("y",			"89"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("z",			"90"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("arrow_up",	"38"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("arrow_down",	"40"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("arrow_left",	"37"); //$NON-NLS-1$ //$NON-NLS-2$
		_keyTable.put("arrow_right","39"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	public String getKeyCode(String keyname) {
		if (_keyTable == null) {
			_createKeyTable();
		}
		return (String)_keyTable.get(keyname.toLowerCase());
	}

	static public String qq(String s) {
		if (s != null) {
		  char ca[] = s.toCharArray();
		  StringBuffer sb = new StringBuffer();
		  sb.append("\"");									//$NON-NLS-1$
		  for (int i=0; i<ca.length; i++) {
			if (ca[i] == '\\' || ca[i] == '\"')			//$NON-NLS-1$ //$NON-NLS-2$
			  sb.append('\\');								//$NON-NLS-1$
			sb.append(ca[i]);
		  }
		  sb.append("\"");									//$NON-NLS-1$
		  s = sb.toString();								
		}
		return s; 			
	}
	static public String escapeBackslash(String s) {
		if (s != null) {
		  char ca[] = s.toCharArray();
		  StringBuffer sb = new StringBuffer();
		  for (int i=0; i<ca.length; i++) {
			if (ca[i] == '\\')			//$NON-NLS-1$ //$NON-NLS-2$
			  sb.append('\\');								//$NON-NLS-1$
			sb.append(ca[i]);
		  }
		  s = sb.toString();								
		}
		return s;
	}
	
	public static void encodeChildren(FacesContext context, UIComponent component)
	throws IOException {
		if (context == null || component == null){
			throw new NullPointerException(); 
		}

		Iterator children = component.getChildren().iterator();
		
		while (children.hasNext()) {
			UIComponent child = (UIComponent) children.next();
			Utils.encodeComponent(context,child);
		}
	}
	
	
	public static void encodeComponent(FacesContext context, UIComponent component)
	throws IOException {
		if(!component.isRendered()){
			return;
		}
		component.encodeBegin(context);
		if(component.getRendersChildren()){
			component.encodeChildren(context);
		} else {
			Iterator children = component.getChildren().iterator();
			while(children.hasNext()){
				UIComponent child = (UIComponent)children.next();
				encodeComponent(context,child);
			}
		}
		component.encodeEnd(context);
	}
	
	public static UIComponent findComponent(UIComponent component, String target){
		UIComponent found  = null;
		Iterator children = component.getFacetsAndChildren();
		while(children.hasNext()){
			UIComponent c = (UIComponent)children.next();
			if(c.getId() != null && c.getId().equals(target)) {
				return c;
			}
			if(c instanceof NamingContainer){
				found = c.findComponent(target);
			}
			if(found == null && c.getChildCount() > 0){
				found = findComponent(c,target);
			}
			if(found != null){
				break;
			}
		}
		return found;
	}

	/**
	 * Finds a component with a given ID as close to the current component as possible.
	 * 
	 * This method first searches for a 'target' component within the same NamingContainer.
	 * If nothing is found, the search is repeated for a NamingContainer above.
	 * And so on, until the search is performed for the entire tree starting with UIViewRoot.
	 * 
	 * @param component UIComponent to start the search from
	 * @param target ID of the component to be found
	 * @return UIComponent with the given ID or null if no such component could be found
	 */
	public static UIComponent findNearestComponent(UIComponent component, String target) {
		UIComponent found  = null;
		
		UIComponent parentNamingContainer = getParentNamingContainer(component); 
		while(parentNamingContainer != null) {
			found = findComponent(parentNamingContainer, target);
			if(found != null) {
				break;
			}
			parentNamingContainer = getParentNamingContainer(parentNamingContainer);
		}
		
		if(found == null) {
			found = findComponent(FacesContext.getCurrentInstance().getViewRoot(), target);
		}
		
		return found;
	}

	/**
	 * Finds the nearest parent NamingContainer for a given component.
	 * 
	 * @param component UIComponent for which to find a nearest NamingContainer
	 * @return UIComponent NamingContainer or null if no NamingContainer could be found
	 */
	public static UIComponent getParentNamingContainer(UIComponent component) {
		if(component == null) {
			return null;
		}
		UIComponent parent = component.getParent();
		if(parent instanceof NamingContainer) {
			return parent;
		}
		return getParentNamingContainer(parent);
	}
	
	public static String versionizeFilename(String filename){
		if(filename == null){
			return filename;
		}
		if(filename.lastIndexOf(".") == -1){
			return filename + "_" + HxClientRenderUtil.HX_VERSION;
		}
		
		StringBuffer newname = new StringBuffer(filename.substring(0,filename.lastIndexOf(".")))
		.append("_")
		.append(HxClientRenderUtil.HX_VERSION)
		.append(filename.substring(filename.lastIndexOf(".")));
		
		return newname.toString();
	}
	
	/*
	 * <p>recursively check up the tree for a UIData instance, returns null if not found</p>
	 */
    public static UIData findParentUIData(UIComponent component) {
        if (component == null) {
            return null;
        }
        if (component instanceof UIData) {
            return (UIData) component;
        }
        return findParentUIData(component.getParent());
    }
    
	/*
	 * <p>recursively check up the tree for a UIDataIterator instance, returns null if not found</p>
	 */
    public static UIDataIterator findParentUIDataIterator(UIComponent component) {
        if (component == null) {
            return null;
        }
        if (component instanceof UIDataIterator) {
            return (UIDataIterator) component;
        }
        return findParentUIDataIterator(component.getParent());
    }
    
	/*
	 * <p>Attempts to get the current viewid from the request.</p>
	 */
	public static String getViewIdFromRequest(FacesContext context){
		String viewId = null;
		Map reqMap = context.getExternalContext().getRequestMap();
		viewId = (String)reqMap.get("javax.servlet.include.path_info");
        if (viewId == null) {
            viewId = context.getExternalContext().getRequestPathInfo();
        }
        if (viewId == null) {
            viewId = (String)reqMap.get("javax.servlet.include.servlet_path");
        }
        if (viewId == null) {
            if (context.getExternalContext().getRequest() instanceof HttpServletRequest) {
                viewId = ((HttpServletRequest) context.getExternalContext().getRequest()).getServletPath();
            }
        }
		return viewId;
	}
	
	/*
	 * <p>Append string arrays elements into a comma separated string.</p>
	 */
	public static String appendStrings(String[] values){
		
		StringBuffer buf = new StringBuffer();
		Boolean first = Boolean.TRUE;
		
		for (int i = 0; i < values.length; i++){
			if(first.booleanValue()){
				buf.append("\"");
				first = Boolean.FALSE;
			} else {
				buf.append(",\"");
			}
			buf.append(values[i])
			.append("\"");
		}
		return buf.toString();
	}
	
	/*
	 * <p>Get date style.</p>
	 */
	public static int getDateStyle(String s) {
		int dateCode = DateFormat.DEFAULT;
		if (s != null && s != "") { //$NON-NLS-1$
			if (Utils.DATESTYLE_SHORT.equalsIgnoreCase(s)) {
				dateCode = DateFormat.SHORT;
			} else if (Utils.DATESTYLE_MEDIUM.equalsIgnoreCase(s)) {
				dateCode = DateFormat.MEDIUM;
			} else if (Utils.DATESTYLE_LONG.equalsIgnoreCase(s)) {
				dateCode = DateFormat.LONG;
			} else if (Utils.DATESTYLE_FULL.equalsIgnoreCase(s)) {
				dateCode = DateFormat.FULL;
			}
		}
		return dateCode;
	}
	
	/**
	 * @param context
	 * @param src
	 * @return Return the adjusted (context path added if necessary) and context encoded url 
	 */
	public static String getEncodedResourceURL(FacesContext context, String src){
		// we could add a check here for src already starting with the context path
        src = context.getApplication().getViewHandler().getResourceURL(
				context, src);
		return (context.getExternalContext().encodeResourceURL(src));
	}

	/**
	 * Returns the first CSS class name from the string of space-separated class names.
	 * Example:
	 * "class1 class2 class3" -> "class1"
	 * "class1" -> "class1"
	 * 
	 * @param classes space separated list of CSS classes
	 * @return first class
	 */
	public static String getFirstCSSClassName(String classes) {
		if(classes == null) {
			return null;
		}
		int firstSpace = classes.indexOf(' ');
		if(firstSpace != -1) {
			return classes.substring(0, firstSpace);
		}
		return classes;
	}
	

	/**
	 * checks a string to see if a pure number or not
	 * 
	 * @param measurement this is the string you want to check
	 * @return measurement if it has a metric at the end ('%', 'px', etc.)
	 * 			otherwise it returns measurement +"px"
	 */
	public static String checkMeasurementMetric(String measurement){
		
		String retMeasurement = measurement;
		
		if(retMeasurement != null)
		{
			retMeasurement = retMeasurement.trim();
									
			if(Character.isDigit(measurement.charAt(measurement.length()-1)))
				retMeasurement = retMeasurement+"px";			
		}
		
		return retMeasurement;
	}
}
