/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.io.Writer;
import java.util.Arrays;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.component.UIForm;
import javax.faces.context.FacesContext;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIPager;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.UIPager.PagerDataWrapper;
import com.ibm.faces.component.html.HtmlPagerWeb;
import com.ibm.faces.util.HtmlUtil;
import com.ibm.faces.util.JavaScriptUtil;

/**
 * @author Alasdair McDowell
 */
public class WebPagerRenderer extends AbstractPagerRenderer implements IScriptContributor{

    private static final String PAGER_SIMPLE_NUMBERLIST = "__pagerWeb"; //$NON-NLS-1$
    
    
	public void encodeBegin(FacesContext context, UIComponent component)
		throws IOException {
	
		if ((context == null) || (component == null)) {
			throw new NullPointerException();
		}
		//add scriptcollector
		UIScriptCollector scriptCollector=UIScriptCollector.find(component);
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		
		HxClientRenderUtil.addScriptLibrary("hxclient_be.js", component);
		HxClientRenderUtil.addScriptLibrary("hxclient_table.js", component);
		
	}    

    /**
     * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     * @throws IOException
     */
    public void encodeEnd(FacesContext context, UIComponent component)
        throws IOException {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
		HtmlPagerWeb webComponent = null;
		if ( component instanceof HtmlPagerWeb ){
			webComponent = (HtmlPagerWeb)component;
		}

        Writer writer = context.getResponseWriter();
        String id = component.getClientId(context);
        String idHidden = id + PAGER_SIMPLE_NUMBERLIST;
        String formId = this.getFormId(context, component);

		String style;
		String styleClass;
		String title;
		String tabindex;
		if ( webComponent != null ){
			style = webComponent.getStyle();
			styleClass = webComponent.getStyleClass();
			title = webComponent.getTitle();
			tabindex = webComponent.getTabindex();
		}else{
			style = (String) component.getAttributes().get("style"); //$NON-NLS-1$
			styleClass = (String) component.getAttributes().get("styleClass"); //$NON-NLS-1$
			title = (String) component.getAttributes().get("title"); //$NON-NLS-1$
			tabindex = (String) component.getAttributes().get("tabindex"); //$NON-NLS-1$
		}
		
        boolean isBinaryNavigation = this.isAttributeTrue(component, "binaryNavigation"); //$NON-NLS-1$
		
        HtmlUtil.writeTagStartOpen(writer, "span", false, false); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "id", id); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "style", style); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "class", styleClass); //$NON-NLS-1$
        HtmlUtil.writeTagStartCloseLn(writer, false);

        StringBuffer buffer = new StringBuffer();
        
        PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
        int currentPage = ((UIPager)component).getCurrentPage(pdw);

        HtmlUtil.writeInputHidden(writer, idHidden, "" + currentPage); //$NON-NLS-1$

        int[] pageList = null;
        if (isBinaryNavigation) {
            pageList = this.getBinaryTraversalPageList(component);
        }
        else {
            pageList = this.getSimplePageList(component);
            if ((pageList.length > 0) && (pageList[0] > 0)) {
                ResourceBundle bundle = ResourceHandler.getBundle(context);
                String localizedNext = ResourceHandler.getString(bundle, "PagerRenderer.previous"); //$NON-NLS-1$
                this.writePagerWebElement(/*component,*/
                        writer,
                        localizedNext,
                        currentPage - 1,
                        idHidden,
                        formId,
                        buffer,
						"_prev",
						title,
						tabindex);

            }
        }

        for (int i = 0, len = pageList.length; i < len; i++) {
            if (pageList[i] >= 0) {
                if (pageList[i] == currentPage) {
                    HtmlUtil.writeTagStartOpen(writer, "strong", false, false); //$NON-NLS-1$
                    HtmlUtil.writeTagStartClose(writer, false);
                    writer.write("" + (currentPage + 1)); //$NON-NLS-1$
                    HtmlUtil.writeTagEndLn(writer, "strong"); //$NON-NLS-1$
                }
                else {
                    //index is zero-index, so add 1 to display
                    String indexDisplay = "" + (pageList[i] + 1); //$NON-NLS-1$
                    this.writePagerWebElement(/*component,*/
                        writer,
                        indexDisplay,
                        pageList[i],
                        idHidden,
                        formId,
                        buffer,
						null,
						title,
						tabindex);
                }
            }
        }
        if (!isBinaryNavigation) {
            int lastPage = ((UIPager)component).getLastPage(pdw);
        	
            if ((pageList.length > 0)
                && (pageList[pageList.length - 1] < lastPage)) {
                ResourceBundle bundle = ResourceHandler.getBundle(context);
                String localizedPrevious = ResourceHandler.getString(bundle, "PagerRenderer.next"); //$NON-NLS-1$
                this.writePagerWebElement(/*component,*/
                        writer,
                        localizedPrevious,
                        currentPage + 1,
                        idHidden,
                        formId,
                        buffer,
						"_next",
						title,
						tabindex);
                
            }
        }

        HtmlUtil.writeTagEndLn(writer, "span"); //$NON-NLS-1$

        //need to import library for JS
        if (UIScriptCollector.find(component) == null) {
            writer.write(JavaScriptUtil.getStartTag(JavaScriptUtil.getJavaScriptPath(context) + Utils.versionizeFilename("hxclient_table.js")));
            writer.write(JavaScriptUtil.getEndTag());
        }

    }

    /**
     * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
     * @param context
     * @param component
     */
    public void decode(FacesContext context, UIComponent component) {
        if ((context == null) || (component == null)) {
            throw new NullPointerException();
        }
        
        PagerDataWrapper pdw = ((UIPager)component).getPagerDataWrapper();
        
        if (pdw == null) {
            return;
        }

         if ( !((UIPager)component).isPaged(pdw) ) {
            return;
        }
        
        String key = component.getClientId(context) + PAGER_SIMPLE_NUMBERLIST;
        String value =
            (String) context.getExternalContext().getRequestParameterMap().get(
                key);

        if (value != null) {
            int currentPage = ((UIPager)component).getCurrentPage(pdw);
            int lastPage = ((UIPager)component).getLastPage(pdw);
            int newPage = -1;
            try {
                Integer intObj = new Integer(value);
                newPage = intObj.intValue();
            }
            catch (NumberFormatException e) {
                //FAILSAFE
            }

            this.queueEvent(component, currentPage, newPage, lastPage);
        }
    }
    //RATLC01080768 - added string to supply suffix for id
    private void writePagerWebElement(/*UIComponent component,*/
        Writer writer,
        String text,
        int index,
        String id,
        String formId,
        StringBuffer buffer,
		String idSuffix,
		String title,
		String tabindex)
        throws IOException {
        if (formId == null) {
            formId = "notNestedInUIForm_csdfvfbdbfdfbdwbwrbrbgwrgtrwgwvwvfvfw"; //$NON-NLS-1$
        }
		
		
		String linkId=id+"__"+index;
		if(idSuffix != null){
			linkId+=idSuffix;
		}
		
		buffer.setLength(0);
		
		/* RATLC00296089
		 * There are issues in some combinations of browser/server with the pager behavior where intermittently
		 * page links clicks do not function - restoring back to previous method of adding false returning 
		 * onclick event to the pager links, this resolves the problem.
		 * 
		buffer.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".addBehavior(\"")
		.append(linkId).append("\",\"onclick\",new ")
		.append(HxClientRenderUtil.HX_VAR_NAME)
		.append(".JSFBehaviorPager(")
		.append("\"input-id:").append(id).append("\",")
		.append("\"index:").append(index).append("\",")
		.append("\"form-id:").append(formId).append("\",")
		.append("\"action:").append("submit\"")
		.append("));\n");
		*/
		
		 buffer.append(HxClientRenderUtil.HX_VAR_NAME)
        .append(".JSFWebPagerSubmit('") //$NON-NLS-1$
        .append(id).append("', ") //$NON-NLS-1$
        .append(index).append(", '") //$NON-NLS-1$
        .append(formId).append("'); return false;"); //$NON-NLS-1$
		
		
        HtmlUtil.writeTagStartOpen(writer, "a", false, false); //$NON-NLS-1$
		HtmlUtil.writeTagAttribute(writer, "id", linkId); //$NON-NLS-1$
        HtmlUtil.writeTagAttribute(writer, "href", "#"); //$NON-NLS-1$ //$NON-NLS-2$
        if(title != null)
        	HtmlUtil.writeTagAttribute(writer, "title", title); //$NON-NLS-1$ //$NON-NLS-2$
        if(tabindex != null)
        	HtmlUtil.writeTagAttribute(writer, "tabindex", tabindex); //$NON-NLS-1$ //$NON-NLS-2$        
        HtmlUtil.writeTagAttribute(writer, "onclick", buffer.toString()); //$NON-NLS-1$
        
        HtmlUtil.writeTagStartClose(writer, false);
        writer.write(text); //$NON-NLS-1$

        HtmlUtil.writeTagEndLn(writer, "a"); //$NON-NLS-1$
        
        /* for use with JSFBehaviorPager */ 
		//UIScriptCollector.find(component).addScript(buffer.toString());
    }

    /**
     * Produces a list of number that contains:
     * current page;
     * list of pages prior/after the current page;
     * pages between this list and the first/last page for binary traversal;
     * first page;
     * last page;
     * @param context
     * @param pager
     * @param data
     * @return
     */
   private int[] getBinaryTraversalPageList(UIComponent pager) {

        int numberOfPages = this.getAttributeInteger(pager, "numberOfPages", 10); //$NON-NLS-1$
        if (numberOfPages < 7) {
            numberOfPages = 7;
        }

        int[] numberList = new int[numberOfPages];
        int len = numberList.length;
        Arrays.fill(numberList, -1);
        
        int firstPage = 0;
        
        PagerDataWrapper pdw = ((UIPager)pager).getPagerDataWrapper();
        int lastPage = ((UIPager)pager).getLastPage(pdw);
        
        int page = ((UIPager)pager).getCurrentPage(pdw);

        if (lastPage < 0) {
            //then we don't know how many pages
            //just add one to current + hope for the best
            lastPage = page + 1;
        }

        //add first & last pages to array
        int idx = 0;
        numberList[idx++] = firstPage;
        if (page > firstPage) {
            numberList[idx++] = page;
        }
        if (lastPage > page) {
            numberList[idx++] = lastPage;
        }

        //fill array with pages before/after current page
        int pre = page - 1;
        int post = page + 1;
        while (true) {
            boolean ok = false;
            if (pre > firstPage) {
                numberList[idx++] = pre--;
                if (idx >= len) {
                    break;
                }
                ok = true;
            }
            if (post < lastPage) {
                numberList[idx++] = post++;
                if (idx >= len) {
                    break;
                }
                ok = true;
            }
            if (!ok) {
                break;
            }
        }
        Arrays.sort(numberList);

        //add binary traversal pages
        if (numberList[1] - numberList[0] > 1) {
            int prev = ((numberList[1] - numberList[0]) / 2) + numberList[0];
            if (prev != numberList[0]) {
                numberList[1] = prev;
            }
        }
        int lenMinusOne = len - 1;
        int lenMinusTwo = len - 2;
        if (numberList[lenMinusOne] - numberList[lenMinusTwo] > 1) {
            int next =
                numberList[lenMinusOne]
                    - ((numberList[lenMinusOne] - numberList[lenMinusTwo]) / 2);
            if (next != numberList[lenMinusTwo]) {
                numberList[lenMinusTwo] = next;
            }
        }

        return numberList;
    }

    private int[] getSimplePageList(UIComponent pager) {
    	PagerDataWrapper pdw = ((UIPager)pager).getPagerDataWrapper();
    	int currentPage = ((UIPager)pager).getCurrentPage(pdw);
    	
        int firstPage = 0;
       int lastPage = ((UIPager)pager).getLastPage(pdw);
        if (lastPage < 0) {
            //then we have no idea how big the list is; allow it though
            lastPage = Integer.MAX_VALUE;
        }

        //boolean currentPageLast = this.isAttributeTrue(pager, "currentPageLast"); //$NON-NLS-1$

        int numberOfPages = this.getAttributeInteger(pager, "numberOfPages", 5); //$NON-NLS-1$
        if (numberOfPages < 2) {
            numberOfPages = 2;
        }
        
        //RATLC01080768
        int midPos = numberOfPages / 2;
        //if odd number
        if(numberOfPages % 2 > 0) ++midPos;
        int first = currentPage - (midPos - 1);
        int last = currentPage + (numberOfPages - midPos);
        //RATLC01080768 end
        
        if (last > lastPage) {
            int delta = last - lastPage;
            first -= delta;
            last -= delta;
        }
        if (first < firstPage) {
            int delta = firstPage - first;
            first += delta;
            last += delta;
        }
        if (last > lastPage) {
            last = lastPage;
        }

        int[] range = new int[last - first + 1];
        for (int i = 0, len = range.length; i < len; i++) {
            range[i] = first++;
        }

        return range;
    }

    private boolean isAttributeTrue(UIComponent component, String key) {
        Object value = component.getAttributes().get(key);
        if (value == null) {
            return false;
        }

        if (Boolean.FALSE.equals(value)) {
            return false;
        }

        return Boolean.TRUE.equals(value) || ("true").equalsIgnoreCase(value.toString()); //$NON-NLS-1$
    }

    private int getAttributeInteger(
        UIComponent component,
        String key,
        int defaultValue) {
        Object value = component.getAttributes().get(key);
        if (value == null) {
            return defaultValue;
        }
        try {
            String str = value.toString();
            return Integer.parseInt(str);
        }
        catch (NumberFormatException e) {
            return defaultValue;
        }
    }

    /**
     * Method for finding the form id; useful when doing a JavaScript submit.
     * @param component
     * @return String
     */
    private String getFormId(FacesContext context, UIComponent component) {
        if (component == null) {
            return null;
        }
        if (component instanceof UIForm) {
            return component.getClientId(context);
        }
        return getFormId(context, component.getParent());
    }


	/* (non-Javadoc)
	 * @see com.ibm.faces.component.IScriptContributor#contributeScript(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		// TODO Auto-generated method stub
		
	}
}
