/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.renderkit.html_extended;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.faces.component.UIColumn;
import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.component.html.HtmlOutputText;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.el.ValueBinding;
import javax.faces.render.Renderer;

import com.ibm.faces.component.IScriptContributor;
import com.ibm.faces.component.UIScriptCollector;
import com.ibm.faces.component.html.HtmlDataXml;
import com.ibm.faces.util.ContextParams;
import com.ibm.faces.util.XMLResponseWriterBuffer;

public class XmlDataRenderer extends Renderer implements IScriptContributor {

	public void encodeBegin(FacesContext context, UIComponent component)
			throws IOException {
		
		if (context == null || component == null)
			throw new NullPointerException();
		
		UIScriptCollector scriptCollector = UIScriptCollector.find(component);
		scriptCollector.register(this, component);
		
		
		
		HxClientRenderUtil.initJSLibraries(scriptCollector, context);
		
		HxClientRenderUtil.addScriptLibrary("hxclient_xml.js", component);		
	}

	/**
	 * @param context
	 * @param component
	 * @param sb
	 * 
	 * This doesn't actually do anything: the child rendering occurs in the script stuff.
	 */
	public void encodeChildren(FacesContext context, UIComponent component)
			throws IOException {
		
		if (context == null || component == null)
			throw new NullPointerException();
	}
	
	public void contributeScript(FacesContext context, UIComponent component) throws IOException {
		//
		XMLResponseWriterBuffer sb = new XMLResponseWriterBuffer();
		String jsComponentId = component.getClientId(context).replaceAll(":","_");
		
		String encode = context.getExternalContext().getInitParameter(ContextParams.ENCODE_DATA);
		if(encode == null || !encode.equalsIgnoreCase("false"))
		{		
			jsComponentId = context.getExternalContext().encodeNamespace(jsComponentId);
		}
		
		
		
		sb.write("var " + jsComponentId + "= new "
			             + HxClientRenderUtil.HX_VAR_NAME + ".XMLData(\"");

		
		UIData dataComponent = (UIData)component;
		// If the data is more than one row we need a root tag for the xml
		// else the tag of the one row be used as the root tag.
		
		String startTag; 
		if (component.getAttributes().get("DataGraphRoot") != null)
		{
			startTag = (String)component.getAttributes().get("DataGraphRoot");
		}
		else
		{
			startTag  = component.getClientId(context).replaceAll(":", "_");
		}
		
		if (dataComponent.getRowCount() > 1)
		{	
			sb.startElement(startTag, component);
		}
		//	Get the current values for the data backend
		int rowIndex = ((UIData)component).getFirst();
		int rowsToDisplay = ((UIData) component).getRows();

		sb = renderMarkup(context, component, sb, rowIndex, rowsToDisplay);
		
		
		// Terminate the script
		if (dataComponent.getRowCount() > 1)
		{
			sb.endElement(startTag);
		}
		sb.write("\");");
	
		// Write to the document
		ResponseWriter writer = context.getResponseWriter();
		writer.write(sb.toString());
	}

	/**
	 * @param context
	 * @param component
	 * @param sb
	 * @return
	 * @throws IOException
	 */
	private XMLResponseWriterBuffer renderMarkup(FacesContext context, UIComponent component, XMLResponseWriterBuffer str, int rowStart, int rowsToDisplay) throws IOException {
		XMLResponseWriterBuffer sb = str;
		UIData dataComponent = (UIData)component;

	
		//	write the table body
		int rowIndex = rowStart;
		int processedRowCount = 0;
		
		Boolean processRow = Boolean.TRUE;
		
	//	dataComponent.processUpdates(context);
		
		if(dataComponent.getValueBinding("value")!=null)
	    {
	      dataComponent.setValue(null);
	    }
		
		dataComponent.setRowIndex(rowIndex);
		if (!dataComponent.isRowAvailable()) {
			processRow = Boolean.FALSE;
		}
			
		Iterator columns;
		Iterator columnChildren;
		String tagValue = "row";
		
		while (processRow.booleanValue()) {
			
		//	this.getTagValue(context, component);
		//	sb.startElement("row",component);
			
			
			if (component.getAttributes().get("javaClassName") != null)
			{
				tagValue = (String)component.getAttributes().get("javaClassName");
			}
			else {
				tagValue = this.getTagValue(context, component);
			}
			
			sb.startElement(tagValue,component);
			//process columns for row
			columns = getColumnList(dataComponent).iterator();
			while (columns.hasNext()) {
				UIColumn column = (UIColumn)columns.next();

				// Pick up the name from the facet
				String nodeName = getComponentValue(context, (UIComponent)column.getFacet("header"));
				
				
	/*
				if (nodeName == null) {
					//  nodeName = column.getClientId(context).replaceAll(":", "_");
					nodeName = getTagAttribute(context,(UIComponent)column.getChildren().get(0));
					System.out.println("nodename is "+nodeName);
				} 
				
			
				
				sb.startElement(nodeName,component); 
		*/		
				
					
	            columnChildren = column.getChildren().iterator();
	            String value = null; 
	            while (columnChildren.hasNext()) {
	            	
	            	
	            	
					UIComponent child = (UIComponent)columnChildren.next();
						
					if (nodeName == null) {
						//  nodeName = column.getClientId(context).replaceAll(":", "_");
						nodeName = getTagAttribute(context,child);
		
					} 
					
					
					if (child.getChildCount() == 0) {
						sb.startElement(nodeName,component); 
					}
					
						
					if (child instanceof HtmlDataXml) {
						
						ValueBinding vb  = child.getValueBinding("value");
						
						
						sb.checkTagOpenState();
						sb = renderMarkup(context, child, sb, ((UIData)child).getFirst(), ((UIData)child).getRows());
					//	sb = renderMarkup(context, child, sb, processedRowCount, ((UIData)child).getRows());
						
					} else {
						value = getComponentValue(context, child);
						if (value != null) {
							sb.writeText(value, null);
						}
					}
					if (child.getChildCount() == 0) {
						sb.endElement(nodeName);
					}
					
				}
	
	        //    sb.endElement(nodeName);
			}
			sb.endElement(tagValue);
			++processedRowCount;

			dataComponent.setRowIndex(++rowIndex);
			if (!dataComponent.isRowAvailable() || (rowsToDisplay > 0 && processedRowCount >= rowsToDisplay)) {
				processRow = Boolean.FALSE;
			}
		}
		
		dataComponent.setRowIndex(-1);
		sb.checkTagOpenState();
		return sb;
	}


	private String getValueData(FacesContext context, UIComponent component) {
		String value = null;
		ValueBinding vb = component.getValueBinding("value"); //$NON-NLS-1$
		if (vb != null) {
			Object obj = vb.getValue(context);
			
			if(obj != null)
			{
					
				// Not going to do this check for now, as some data can be 
				// floats and doubles etc. So leaving just going to use toString for now.
			//	if (obj instanceof String) {
					value = obj.toString();
				
			//	}
			}
		} else {
			value = (String) component.getAttributes().get("value"); //$NON-NLS-1$
		}
		return value;
	}
	
	private String getTagValue(FacesContext context, UIComponent component)
	{
		String tag = null;
		
		ValueBinding vb = component.getValueBinding("value");
		
		if (vb != null)
		{
			Class type = vb.getType(context);
			String tagClass = type.getName();	
			
			String simpleTag = tagClass.charAt(tagClass.length()-1) == ';'?tagClass.substring(tagClass.lastIndexOf(".")+1,tagClass.length()-1):tagClass.substring(tagClass.lastIndexOf(".")+1);
			
			
			return simpleTag;
			
		}
		
		
		return null;
	}
	
	private String getTagAttribute(FacesContext context, UIComponent component)
	{
		
		
		ValueBinding vb = component.getValueBinding("value");
		
			
		if (vb != null)
		{
		
			String expression = vb.getExpressionString(); 
			
			String tagAttr =  expression.substring(expression.lastIndexOf(".")+1,expression.length()-1);
			
			return tagAttr;
			
		}
		
		
		return null;
	}

	/**
	 * @param context
	 * @param component
	 * 
	 * @return
	 */
	private String getComponentValue(FacesContext context, UIComponent component) {
		String value = null;
		if (component != null) {
			if (component instanceof HtmlOutputText) {
				value = getValueData(context, component);
			} else {
				Iterator children = component.getChildren().iterator();
				while (children.hasNext()) {
					UIComponent child = (UIComponent)children.next();
					if (child instanceof HtmlOutputText) {
						value = getValueData(context, child);
					}
				}
			}
		}
		return value;
	}

	public void encodeEnd(FacesContext context, UIComponent component)
			throws IOException {

		if (context == null || component == null)
			throw new NullPointerException();
	}

	public boolean getRendersChildren() {
		return true;
	}

	private ArrayList getColumnList(UIData table) {
		ArrayList list = new ArrayList();
		Iterator children = table.getChildren().iterator();
		while (children.hasNext()) {
			UIComponent child = (UIComponent)children.next();
			if (child instanceof UIColumn && child.isRendered()) {
				list.add(child);
			}
		}
		return list;
	}
	
}
