/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.sdo;

import javax.faces.el.PropertyNotFoundException;
import javax.faces.el.PropertyResolver;
import com.ibm.websphere.sdo.DataObjectAccessBean;


/**
 * SdoPropertyResolver
 * 
 * @author xnjiang
 * @author kgshank@us.ibm.com
 */

public class SdoPropertyResolver extends PropertyResolver {

	private PropertyResolver oldPropResolver=null;
	private boolean sdoAvailable;

	public SdoPropertyResolver(PropertyResolver propResolver) throws Exception {
		//RATLC00969582 fix : sdoAvailable set initially to true or else it will always be false
		//cannot use this field in the instanceof test or, again, it will always be false.
		sdoAvailable=true;		
		// Check if SDO jars are installed and available
		try{
			boolean test = this instanceof DataObjectAccessBean;
		}catch( NoClassDefFoundError ncdfe ){
			sdoAvailable = false;
		}
		oldPropResolver = propResolver;
	}
	

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getValue(java.lang.Object, java.lang.Object)
	 */
	// TODO: Check that this is the correct behavior - the method has been changed to use
	//       and Object as thesecond param from a String.
	public Object getValue(Object base, Object property)
		throws PropertyNotFoundException {

//		if ((base == null) || (property == null)) {
//			String className =
//				base == null ? "null" : base.getClass().getName();
//			throw new PropertyNotFoundException(
//				"Error getting property '"
//					+ property
//					+ "' in bean of type "
//					+ className);
//		}

		if ( sdoAvailable && (base instanceof DataObjectAccessBean)) {
			return (((DataObjectAccessBean) base).get(property));
		}
		
		if (oldPropResolver != null) {
			return oldPropResolver.getValue(base, property);
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getValue(java.lang.Object, int)
	 */
	public Object getValue(Object base, int index)
		throws PropertyNotFoundException {

		if (oldPropResolver != null) {
			return oldPropResolver.getValue(base, index);
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#setValue(java.lang.Object, java.lang.Object, java.lang.Object)
	 */
	// TODO: Check that this is the correct behavior - the method has been changed to use
	//       and Object as thesecond param from a String.
	public void setValue(Object base, Object property, Object value)
		throws PropertyNotFoundException {

		if ( sdoAvailable && (base instanceof DataObjectAccessBean)) {
			if ( property == null ) {
				String className = base.getClass().getName();
				throw new PropertyNotFoundException(
					"Error setting property '"
						+ property
						+ "' in bean of type "
						+ className);
			}
			((DataObjectAccessBean) base).put(property, value);
			return;
		}
		
		if (oldPropResolver != null) {
			oldPropResolver.setValue(base, property, value);
			return;
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#setValue(java.lang.Object, int, java.lang.Object)
	 */
	public void setValue(Object base, int index, Object value)
		throws PropertyNotFoundException {

		if (oldPropResolver != null) {
			oldPropResolver.setValue(base, index, value);
			return;
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#isReadOnly(java.lang.Object, java.lang.Object)
	 */
	// TODO: Check that this is the correct behavior - the method has been changed to use
	//       and Object as thesecond param from a String.
	public boolean isReadOnly(Object base, Object property)
		throws PropertyNotFoundException {
		
		if ( sdoAvailable && (base instanceof DataObjectAccessBean) ) {
			try {
			
				if (property instanceof String) {
					return ((DataObjectAccessBean) base).isReadOnly(
						(String) property);
				} else {
					String className = base.getClass().getName();
					throw new PropertyNotFoundException(
						"Property '"
							+ property
							+ "' is not a String in bean of type "
							+ className);
				}
			
			} catch (Exception e) {
				//TODO - behavior on error? cannot directly catch MediatorException (causes ncdfe if no wdo)
				throw new PropertyNotFoundException(e);
			}
		}

		if (oldPropResolver != null) {
			return oldPropResolver.isReadOnly(base, property);
		}
		throw new PropertyNotFoundException();
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#isReadOnly(java.lang.Object, int)
	 */
	public boolean isReadOnly(Object base, int index)
		throws PropertyNotFoundException {

		if (base == null) {
			throw new NullPointerException();
		}
		
		if ( sdoAvailable && (base instanceof DataObjectAccessBean)) {
			throw new PropertyNotFoundException( "Property index not supported" );
		}

		if (oldPropResolver != null) {
			return oldPropResolver.isReadOnly(base, index);
		}
	
		throw new PropertyNotFoundException();
		
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getType(java.lang.Object, java.lang.Object)
	 */
	// TODO: Check that this is the correct behavior - the method has been changed to use
	//       and Object as thesecond param from a String.
	public Class getType(Object base, Object property)
		throws PropertyNotFoundException {

		if ( sdoAvailable && (base instanceof DataObjectAccessBean) ) {
			try {
				if (property instanceof String) {
					return ((DataObjectAccessBean) base).getType(
						(String) property);
				} else {
					String className = base.getClass().getName();
					throw new PropertyNotFoundException(
						"Error testing property '"
							+ property
							+ "' in bean of type "
							+ className);
				}
			} catch (Exception e) {
				//TODO - behavior on error? cannot directly catch MediatorException (causes ncdfe if no wdo)
				throw new PropertyNotFoundException(e);
			}
		}
		
		if (oldPropResolver != null) {
			return oldPropResolver.getType(base, property);
		}
		throw new PropertyNotFoundException();
		
	}

	/* (non-Javadoc)
	 * @see javax.faces.el.PropertyResolver#getType(java.lang.Object, int)
	 */
	public Class getType(Object base, int index)
		throws PropertyNotFoundException {

		if (base == null) {
			throw new NullPointerException();
		}
		
		if ( sdoAvailable && (base instanceof DataObjectAccessBean)) {
			throw new PropertyNotFoundException( "Property index not supported" );
		}
		
		if(oldPropResolver!=null){
			return oldPropResolver.getType(base, index);
		}
		
		throw new PropertyNotFoundException();
		
	}

}
