/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.taglib;

import javax.faces.component.UIComponent;
import javax.faces.el.ValueBinding;
import javax.faces.webapp.UIComponentTag;

import com.ibm.faces.component.UIColumnEx;
import com.ibm.faces.util.TagUtil;

/**
 * <p>Tag class for hx:column tag</p>
 * @author pandrews
 */
public class ColumnExTag extends UIComponentTag {
	
	private String footerClass;
	private String headerClass;
	private String width;
	private String dir;
	private String scope;
	private String axis;
	private String height;
	private String nowrap;
	private String align;
	private String valign;
	private String bgcolor;
	private String background;
	private String style;
	private String title;
	
	/**
	 * @param align The align to set.
	 */
	public void setAlign(String align) {
		this.align = align;
	}
	/**
	 * @param axis The axis to set.
	 */
	public void setAxis(String axis) {
		this.axis = axis;
	}
	/**
	 * @param background The background to set.
	 */
	public void setBackground(String background) {
		this.background = background;
	}
	/**
	 * @param bgcolor The bgcolor to set.
	 */
	public void setBgcolor(String bgcolor) {
		this.bgcolor = bgcolor;
	}
	/**
	 * @param dir The dir to set.
	 */
	public void setDir(String dir) {
		this.dir = dir;
	}
	/**
	 * @param footerClass The footerClass to set.
	 */
	public void setFooterClass(String footerClass) {
		this.footerClass = footerClass;
	}
	/**
	 * @param headerClass The headerClass to set.
	 */
	public void setHeaderClass(String headerClass) {
		this.headerClass = headerClass;
	}
	/**
	 * @param height The height to set.
	 */
	public void setHeight(String height) {
		this.height = height;
	}
	/**
	 * @param nowrap The nowrap to set.
	 */
	public void setNowrap(String nowrap) {
		this.nowrap = nowrap;
	}
	/**
	 * @param scope The scope to set.
	 */
	public void setScope(String scope) {
		this.scope = scope;
	}
	/**
	 * @param style The style to set.
	 */
	public void setStyle(String style) {
		this.style = style;
	}
	/**
	 * @param valign The valign to set.
	 */
	public void setValign(String valign) {
		this.valign = valign;
	}
	/**
	 * @param width The width to set.
	 */
	public void setWidth(String width) {
		this.width = width;
	}
	/**
	 * @param width The title to set.
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/* (non-Javadoc)
	 * @see javax.faces.webapp.UIComponentTag#getComponentType()
	 */
	public String getComponentType() {
		return "com.ibm.faces.Column";
	}

	/* (non-Javadoc)
	 * @see javax.faces.webapp.UIComponentTag#getRendererType()
	 */
	public String getRendererType() {
		return null;
	}

	
	
	/* (non-Javadoc)
	 * @see javax.faces.webapp.UIComponentTag#setProperties(javax.faces.component.UIComponent)
	 */
	protected void setProperties(UIComponent component) {
		super.setProperties(component);
		UIColumnEx col = (UIColumnEx)component;
		
		if(footerClass != null){
			if(isValueReference(footerClass)){
				ValueBinding vb = TagUtil.getValueBinding(footerClass);
				col.setValueBinding("footerClass",vb);
			} else {
				col.setFooterClass(footerClass);
			}
		}
		if(headerClass != null){
			if(isValueReference(headerClass)){
				ValueBinding vb = TagUtil.getValueBinding(headerClass);
				col.setValueBinding("headerClass",vb);
			} else {
				col.setHeaderClass(headerClass);
			}
		}
		if(width != null){
			if(isValueReference(width)){
				ValueBinding vb = TagUtil.getValueBinding(width);
				col.setValueBinding("width",vb);
			} else {
				col.setWidth(width);
			}
		}
		if(dir != null){
			if(isValueReference(dir)){
				ValueBinding vb = TagUtil.getValueBinding(dir);
				col.setValueBinding("dir",vb);
			} else {
				col.setDir(dir);
			}
		}
		if(scope != null){
			if(isValueReference(scope)){
				ValueBinding vb = TagUtil.getValueBinding(scope);
				col.setValueBinding("scope",vb);
			} else {
				col.setScope(scope);
			}
		}
		if(axis != null){
			if(isValueReference(axis)){
				ValueBinding vb = TagUtil.getValueBinding(axis);
				col.setValueBinding("axis",vb);
			} else {
				col.setAxis(axis);
			}
		}
		if(height != null){
			if(isValueReference(height)){
				ValueBinding vb = TagUtil.getValueBinding(height);
				col.setValueBinding("height",vb);
			} else {
				col.setHeight(height);
			}
		}
		if(nowrap != null){
			if(isValueReference(nowrap)){
				ValueBinding vb = TagUtil.getValueBinding(nowrap);
				col.setValueBinding("nowrap",vb);
			} else {
				col.setNowrap(nowrap);
			}
		}
		if(align != null){
			if(isValueReference(align)){
				ValueBinding vb = TagUtil.getValueBinding(align);
				col.setValueBinding("align",vb);
			} else {
				col.setAlign(align);
			}
		}
		
		if(valign != null){
			if(isValueReference(valign)){
				ValueBinding vb = TagUtil.getValueBinding(valign);
				col.setValueBinding("valign",vb);
			} else {
				col.setValign(valign);
			}
		}
		
		if(bgcolor != null){
			if(isValueReference(bgcolor)){
				ValueBinding vb = TagUtil.getValueBinding(bgcolor);
				col.setValueBinding("bgcolor",vb);
			} else {
				col.setBgcolor(bgcolor);
			}
		}
		if(background != null){
			if(isValueReference(background)){
				ValueBinding vb = TagUtil.getValueBinding(background);
				col.setValueBinding("background",vb);
			} else {
				col.setBackground(background);
			}
		}
		if(style != null){
			if(isValueReference(style)){
				ValueBinding vb = TagUtil.getValueBinding(style);
				col.setValueBinding("style",vb);
			} else {
				col.setStyle(style);
			}
		}
		if(title != null){
			if(isValueReference(title)){
				ValueBinding vb = TagUtil.getValueBinding(title);
				col.setValueBinding("title",vb);
			} else {
				col.setTitle(title);
			}
		}
	}
}
