/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.taglib;

import java.io.IOException;
import java.io.Writer;

import javax.faces.FactoryFinder;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.RenderKit;
import javax.faces.render.RenderKitFactory;
import javax.faces.webapp.UIComponentTag;
import javax.servlet.jsp.JspException;

public class ViewFragmentTag extends UIComponentTag {
	private ResponseWriter orgWriter = null;

	public int doEndTag() throws JspException {
		int retValue = super.doEndTag();

		popResponseWriter();

		return retValue;
	}

	public int doStartTag() throws JspException {

        FacesContext context = FacesContext.getCurrentInstance();
        
        if (context == null) { 
            throw new JspException("Cannot find FacesContext");
        }
            
		orgWriter = context.getResponseWriter();
		pushResponseWriter();

		return super.doStartTag();
	}

	protected void pushResponseWriter() {
        FacesContext context = FacesContext.getCurrentInstance();
		RenderKitFactory renderFactory = (RenderKitFactory) FactoryFinder.getFactory(FactoryFinder.RENDER_KIT_FACTORY);
		RenderKit renderKit = renderFactory.getRenderKit(context, context.getViewRoot().getRenderKitId());

		ResponseWriter writer = renderKit.createResponseWriter(new Writer() {
			public void close() throws IOException {
				pageContext.getOut().close();
			}

			public void flush() throws IOException {
			}

			public void write(char[] cbuf, int off, int len) throws IOException {
				pageContext.getOut().write(cbuf, off, len);
			}

			public void write(int c) throws IOException {
				pageContext.getOut().write(c);
			}

			public void write(String str) throws IOException {
				pageContext.getOut().write(str);
			}

			public void write(String str, int off, int len) throws IOException {
				pageContext.getOut().write(str, off, len);
			}
		}, null, pageContext.getRequest().getCharacterEncoding());

		context.setResponseWriter(writer);
	}

	protected void popResponseWriter() {
        FacesContext context = FacesContext.getCurrentInstance();
		context.setResponseWriter(orgWriter);
	}

    public String getRendererType() {
        return null;
    }

    public String getComponentType() {
        return "javax.faces.NamingContainer";
    }
}
