/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIAjax;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class AjaxExternalRequestTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(AjaxExternalRequestTag.class);

    //
    // Instance Variables
    //

    private java.lang.String inProgress;
    private java.lang.String oncomplete;
    private java.lang.String onerror;
    private java.lang.String onstart;
    private java.lang.String target;
    private java.lang.String title;

    private java.lang.String charset;
    private java.lang.String href;
    private java.lang.String hreflang;
    private java.lang.String params;
    private java.lang.String source;

    //
    // Setter Methods
    //

    public void setInProgress(java.lang.String inProgress) {
        this.inProgress = inProgress;
    }

    public void setOncomplete(java.lang.String oncomplete) {
        this.oncomplete = oncomplete;
    }

    public void setOnerror(java.lang.String onerror) {
        this.onerror = onerror;
    }

    public void setOnstart(java.lang.String onstart) {
        this.onstart = onstart;
    }

    public void setTarget(java.lang.String target) {
        this.target = target;
    }

    public void setTitle(java.lang.String title) {
        this.title = title;
    }

    public void setCharset(java.lang.String charset) {
        this.charset = charset;
    }

    public void setHref(java.lang.String href) {
        this.href = href;
    }

    public void setHreflang(java.lang.String hreflang) {
        this.hreflang = hreflang;
    }

    public void setParams(java.lang.String params) {
        this.params = params;
    }

    public void setSource(java.lang.String source) {
        this.source = source;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.ExternalRequest";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlAjaxExternalRequest"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIAjax ajaxComponent = (UIAjax)component;

        if (inProgress != null) {
            if (isValueReference(inProgress)) {
                ValueBinding vb = TagUtil.getValueBinding(inProgress);
                ajaxComponent.setValueBinding("inProgress", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.setInProgress(inProgress);
            }
        }
        if (oncomplete != null) {
            if (isValueReference(oncomplete)) {
                ValueBinding vb = TagUtil.getValueBinding(oncomplete);
                ajaxComponent.setValueBinding("oncomplete", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.setOncomplete(oncomplete);
            }
        }
        if (onerror != null) {
            if (isValueReference(onerror)) {
                ValueBinding vb = TagUtil.getValueBinding(onerror);
                ajaxComponent.setValueBinding("onerror", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.setOnerror(onerror);
            }
        }
        if (onstart != null) {
            if (isValueReference(onstart)) {
                ValueBinding vb = TagUtil.getValueBinding(onstart);
                ajaxComponent.setValueBinding("onstart", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.setOnstart(onstart);
            }
        }
        if (target != null) {
            if (isValueReference(target)) {
                ValueBinding vb = TagUtil.getValueBinding(target);
                ajaxComponent.setValueBinding("target", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.setTarget(target);
            }
        }
        if (title != null) {
            if (isValueReference(title)) {
                ValueBinding vb = TagUtil.getValueBinding(title);
                ajaxComponent.setValueBinding("title", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.setTitle(title);
            }
        }
        if (charset != null) {
            if (isValueReference(charset)) {
                ValueBinding vb = TagUtil.getValueBinding(charset);
                ajaxComponent.setValueBinding("charset", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.getAttributes().put("charset", charset); //$NON-NLS-1$
            }
        }
        if (href != null) {
            if (isValueReference(href)) {
                ValueBinding vb = TagUtil.getValueBinding(href);
                ajaxComponent.setValueBinding("href", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.getAttributes().put("href", href); //$NON-NLS-1$
            }
        }
        if (hreflang != null) {
            if (isValueReference(hreflang)) {
                ValueBinding vb = TagUtil.getValueBinding(hreflang);
                ajaxComponent.setValueBinding("hreflang", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.getAttributes().put("hreflang", hreflang); //$NON-NLS-1$
            }
        }
        if (params != null) {
            if (isValueReference(params)) {
                ValueBinding vb = TagUtil.getValueBinding(params);
                ajaxComponent.setValueBinding("params", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.getAttributes().put("params", params); //$NON-NLS-1$
            }
        }
        if (source != null) {
            if (isValueReference(source)) {
                ValueBinding vb = TagUtil.getValueBinding(source);
                ajaxComponent.setValueBinding("source", vb); //$NON-NLS-1$
            } else {
                ajaxComponent.getAttributes().put("source", source); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
