/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIBackchannel;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class BackchannelHttpRequestTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(BackchannelHttpRequestTag.class);

    //
    // Instance Variables
    //

    private java.lang.String oncomplete;
    private java.lang.String onerror;
    private java.lang.String onstart;

    private java.lang.String asynchronous;
    private java.lang.String mode;
    private java.lang.String useiframe;

    //
    // Setter Methods
    //

    public void setOncomplete(java.lang.String oncomplete) {
        this.oncomplete = oncomplete;
    }

    public void setOnerror(java.lang.String onerror) {
        this.onerror = onerror;
    }

    public void setOnstart(java.lang.String onstart) {
        this.onstart = onstart;
    }

    public void setAsynchronous(java.lang.String asynchronous) {
        this.asynchronous = asynchronous;
    }

    public void setMode(java.lang.String mode) {
        this.mode = mode;
    }

    public void setUseiframe(java.lang.String useiframe) {
        this.useiframe = useiframe;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.HttpRequest";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlBackchannelHttpRequest"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIBackchannel backchannelComponent = (UIBackchannel)component;

        if (oncomplete != null) {
            if (isValueReference(oncomplete)) {
                ValueBinding vb = TagUtil.getValueBinding(oncomplete);
                backchannelComponent.setValueBinding("oncomplete", vb); //$NON-NLS-1$
            } else {
                backchannelComponent.setOncomplete(oncomplete);
            }
        }
        if (onerror != null) {
            if (isValueReference(onerror)) {
                ValueBinding vb = TagUtil.getValueBinding(onerror);
                backchannelComponent.setValueBinding("onerror", vb); //$NON-NLS-1$
            } else {
                backchannelComponent.setOnerror(onerror);
            }
        }
        if (onstart != null) {
            if (isValueReference(onstart)) {
                ValueBinding vb = TagUtil.getValueBinding(onstart);
                backchannelComponent.setValueBinding("onstart", vb); //$NON-NLS-1$
            } else {
                backchannelComponent.setOnstart(onstart);
            }
        }
        if (asynchronous != null) {
            if (isValueReference(asynchronous)) {
                ValueBinding vb = TagUtil.getValueBinding(asynchronous);
                backchannelComponent.setValueBinding("asynchronous", vb); //$NON-NLS-1$
            } else {
                boolean _asynchronous = new Boolean(asynchronous).booleanValue();
                backchannelComponent.getAttributes().put("asynchronous", _asynchronous ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (mode != null) {
            if (isValueReference(mode)) {
                ValueBinding vb = TagUtil.getValueBinding(mode);
                backchannelComponent.setValueBinding("mode", vb); //$NON-NLS-1$
            } else {
                backchannelComponent.getAttributes().put("mode", mode); //$NON-NLS-1$
            }
        }
        if (useiframe != null) {
            if (isValueReference(useiframe)) {
                ValueBinding vb = TagUtil.getValueBinding(useiframe);
                backchannelComponent.setValueBinding("useiframe", vb); //$NON-NLS-1$
            } else {
                boolean _useiframe = new Boolean(useiframe).booleanValue();
                backchannelComponent.getAttributes().put("useiframe", _useiframe ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
