/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UICommandEx;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class CommandExRowActionTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(CommandExRowActionTag.class);

    //
    // Instance Variables
    //

    private java.lang.String action;
    private java.lang.String actionListener;
    private java.lang.String immediate;


    //
    // Setter Methods
    //

    public void setAction(java.lang.String action) {
        this.action = action;
    }

    public void setActionListener(java.lang.String actionListener) {
        this.actionListener = actionListener;
    }

    public void setImmediate(java.lang.String immediate) {
        this.immediate = immediate;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.RowAction";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlCommandExRowAction"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UICommandEx commandexComponent = (UICommandEx)component;

        if (action != null) {
             if (isValueReference(action)) {
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(action, null);
                commandexComponent.setAction(vb);
            } else {
                final String outcome = action;
                MethodBinding vb = new StaticMethodBinding(action);
                commandexComponent.setAction(vb);
            }
        }
        if (actionListener != null) {
            if (isValueReference(actionListener)) {
                Class args[] = { ActionEvent.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(actionListener, args);
                commandexComponent.setActionListener(vb);
            } else {
              Object params [] = {actionListener};
              throw new javax.faces.FacesException();
            }
            }
        if (immediate != null) {
            if (isValueReference(immediate)) {
                ValueBinding vb = TagUtil.getValueBinding(immediate);
                commandexComponent.setValueBinding("immediate", vb); //$NON-NLS-1$
            } else {
                boolean _immediate = new Boolean(immediate).booleanValue();
                commandexComponent.setImmediate(_immediate);
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
