/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2004 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.taglib.html_extended;

import javax.faces.convert.Converter;
import javax.faces.webapp.ConverterTag;
import javax.faces.webapp.UIComponentTag;
import javax.servlet.jsp.JspException;

import com.ibm.faces.converter.DateTimeConverterExICU;
import com.ibm.faces.util.TagUtil;

import com.ibm.icu.util.Calendar;
import com.ibm.icu.util.BuddhistCalendar;
import com.ibm.icu.util.GregorianCalendar;
import com.ibm.icu.util.HebrewCalendar;
import com.ibm.icu.util.IslamicCalendar;
import com.ibm.icu.util.JapaneseCalendar;
import com.ibm.icu.util.TimeZone;

import java.util.Locale;

public class ConvertDateTimeTag extends ConverterTag {

	private String type; 		
	private String pattern;		
	private String timeZone;	
	private String calendar;
	private String locale;
	private String dateStyle;
	private String timeStyle;
	
	private TimeZone timeZoneSetUp = null;
		
	public ConvertDateTimeTag(){
		super();
		//super.setId("DateTime"); //$NON-NLS-1$
		super.setConverterId("com.ibm.faces.DateTime"); //$NON-NLS-1$
		/*init();*/
		 
	}
	
    public void release() {
        super.release();
        type = null;
        pattern = null;
        timeZone = null;
        calendar = null;
        locale = null;
        dateStyle = null;
        timeStyle = null;
    }
	
	/*public void release(){
		super.release();
		init();
	}*/
	
	/*private void init(){
		mask = "";
		//RATLC01088464
		asis = "false";
	}*/
	
	
	/* (non-Javadoc)
	 * @see javax.faces.webapp.ConverterTag#createConverter()
	 */
	protected Converter createConverter() throws JspException {		
		DateTimeConverterExICU converter = null;
		converter = (DateTimeConverterExICU)super.createConverter();
		//type
		if(type != null){
			if(UIComponentTag.isValueReference(type)){
				Object result = TagUtil.getValueBindingValue(type);
				if(result instanceof String){				
					converter.setType((String)result);					
				}	
			} else {
				converter.setType(type);	
			}
			
		}	
		//timezone
		if(timeZone != null){
			Object result = null;
			if(UIComponentTag.isValueReference(timeZone)){
				result = TagUtil.getValueBindingValue(timeZone);
				
				if(result instanceof String){
					result = TimeZone.getTimeZone((String)result);				
				}
				
				if(result instanceof TimeZone){
					converter.setTimeZoneEx((TimeZone)result);
				}
				
			} else {
				result = TimeZone.getTimeZone(timeZone);
				
				if(result instanceof TimeZone){
					converter.setTimeZoneEx((TimeZone)result);
				}
			}
			//set so getCalendarFromString can use it if set
			if(result instanceof TimeZone){
				timeZoneSetUp = (TimeZone)result;
			}
			
		}
		//calendar
		if(calendar != null){
			if(UIComponentTag.isValueReference(calendar)){
				Object result = TagUtil.getValueBindingValue(calendar);

				if(result instanceof String){
					converter.setCalendar(getCalendarFromString((String)result));
				}
				if(result instanceof Calendar){
					converter.setCalendar((Calendar)result);
				}			
			}else {
				converter.setCalendar(getCalendarFromString(calendar));
			}
		} else {
			if(pattern == null){
				//default to gregorian is pattern & calendar hasn't been set
				converter.setCalendar(getCalendarFromString("gregorian"));
			}
		}
		//pattern
		if(pattern != null){ 
			if(UIComponentTag.isValueReference(pattern)){
				Object result = TagUtil.getValueBindingValue(pattern);
				if(result instanceof String){
					converter.setPattern((String)result);
				}
			} else {
				converter.setPattern(pattern);	
			}
			
		}
		//locale
		if(locale != null){
			if(UIComponentTag.isValueReference(locale)){
				Object result = TagUtil.getValueBindingValue(locale);
				if(result instanceof String){
					converter.setLocale(new Locale((String)result,""));
				}
				if(result instanceof Locale){
					converter.setLocale((Locale)result);
				}			
			} else {
				converter.setLocale(new Locale(locale,""));
			}
		}
		//datestyle
		if(dateStyle != null){
			if(UIComponentTag.isValueReference(dateStyle)){
				Object result = TagUtil.getValueBindingValue(dateStyle);
				if(result instanceof String){
					converter.setDateStyle((String)dateStyle);
				}
			} else {
				converter.setDateStyle(dateStyle);
			}
		}		
		//timestyle
		if(timeStyle != null){
			if(UIComponentTag.isValueReference(timeStyle)){
				Object result = TagUtil.getValueBindingValue(timeStyle);
				if(result instanceof String){
					converter.setTimeStyle((String)timeStyle);
				}
			} else {
				converter.setTimeStyle(timeStyle);
			}
		}		

		/*
		 * NOTE : NEED TO BARE IN MIND BILL's IDEA THAT NO NEED FOR PATTERN IF CALENDAR IS SET
		 * THOUGH I DON'T THINK LOGIC FOR IT SHOULD BE IN TAG CLASS
		 */
		
		return converter;
	}


	public void setCalendar(String calendar) {
		this.calendar = calendar;
	}

	public void setPattern(String pattern) {
		this.pattern = pattern;
	}

	public void setTimeZone(String timeZone) {
		this.timeZone = timeZone;
	}

	public void setType(String type) {
		this.type = type;
	}	
	
	public void setDateStyle(String dateStyle) {
		this.dateStyle = dateStyle;
	}

	public void setLocale(String locale) {
		this.locale = locale;
	}

	public void setTimeStyle(String timeStyle) {
		this.timeStyle = timeStyle;
	}
	
	
	private Calendar getCalendarFromString(String cal_str)
	{
		Calendar cal = null;

		if(timeZoneSetUp != null)
		{
			if("gregorian".equalsIgnoreCase(cal_str)) // this is the default
				cal = (Calendar)( new GregorianCalendar(timeZoneSetUp));
			
			if("buddhist".equalsIgnoreCase(cal_str))
				cal = (Calendar)( new BuddhistCalendar(timeZoneSetUp));
			if("hebrew".equalsIgnoreCase(cal_str))
				cal = (Calendar)( new HebrewCalendar(timeZoneSetUp));
			if("islamic".equalsIgnoreCase(cal_str))
				cal = (Calendar)( new IslamicCalendar(timeZoneSetUp));
			if("japanese".equalsIgnoreCase(cal_str))
				cal = (Calendar)( new JapaneseCalendar(timeZoneSetUp));
			/*if("??????".equalsIgnoreCase(cal_str)) // not used
				cal = (Calendar) new CopticCalendar(timeZoneSetUp);
			if("???????".equalsIgnoreCase(cal_str))
				cal = (Calendar) new EthiopticCalendar(timeZoneSetUp);*/ // not used.
		}
		else
		{
			if("gregorian".equalsIgnoreCase(cal_str)) // this is the default
				cal = (Calendar)( new GregorianCalendar());
			
			if("buddhist".equalsIgnoreCase(cal_str))
				cal = (Calendar)( new BuddhistCalendar());
			if("hebrew".equalsIgnoreCase(cal_str))
				cal = (Calendar)( new HebrewCalendar());
			if("islamic".equalsIgnoreCase(cal_str))
				cal = (Calendar)( new IslamicCalendar());
			if("japanese".equalsIgnoreCase(cal_str))
				cal = (Calendar)( new JapaneseCalendar());
			/*if("??????".equalsIgnoreCase(cal_str)) // not used
				cal = (Calendar) new CopticCalendar();
			if("???????".equalsIgnoreCase(cal_str))
				cal = (Calendar) new EthiopticCalendar();*/ // not used.		
		}
		return cal;
	}
	
}
