/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2004 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.taglib.html_extended;

import javax.faces.convert.Converter;
import javax.faces.webapp.ConverterTag;
import javax.faces.webapp.UIComponentTag;
import javax.servlet.jsp.JspException;

import com.ibm.faces.converter.MaskConverter;
import com.ibm.faces.util.TagUtil;


public class ConvertMaskTag extends ConverterTag {
	private String mask;
	private boolean bAsis;
	private String asis;
	
	private String protectMask;
	private String char1RegEx;
	private String char2RegEx;
	private String char3RegEx;
	private String char4RegEx;
	
	public ConvertMaskTag(){
		super();
		super.setConverterId("Mask");
		init();
	}
	
	public void release(){
		super.release();
		init();
	}
	
	private void init(){
		mask = "";
		//RATLC01088464
		asis = "false";
	}
	
	
	/* (non-Javadoc)
	 * @see javax.faces.webapp.ConverterTag#createConverter()
	 */
	protected Converter createConverter() throws JspException {		
		MaskConverter converter = null;
		converter = (MaskConverter)super.createConverter();
		
		if(mask != null && UIComponentTag.isValueReference(mask)){ //This could be a VBL expression, if it is			
			Object result = TagUtil.getValueBindingValue(mask);	// convert it to a String
			if(result instanceof String){				
				setMask((String)result);					
			}
		}		
		converter.setMask(mask);
		
		if (asis != null) {
            if (UIComponentTag.isValueReference(asis)) {            	
            	Boolean booleanObj = (Boolean) TagUtil.getValueBindingValue(asis); 
            	bAsis = booleanObj.booleanValue();   
            } else {
                bAsis = new Boolean(asis).booleanValue();
            }
        }
		
		converter.setAsis(bAsis);
		
		if(protectMask != null && UIComponentTag.isValueReference(protectMask)){
			Object result = TagUtil.getValueBindingValue(protectMask);
			if(result instanceof String){
				setProtectMask((String)result);
			}
		}
		converter.setProtectMask(getProtectMask());
		
		if(char1RegEx != null && UIComponentTag.isValueReference(char1RegEx)){
			Object result = TagUtil.getValueBindingValue(char1RegEx);
			if(result instanceof String){
				setChar1RegEx((String)result);
			}
		}
		converter.setChar1RegEx(getChar1RegEx());
		
		if(char2RegEx != null && UIComponentTag.isValueReference(char2RegEx)){
			Object result = TagUtil.getValueBindingValue(char2RegEx);
			if(result instanceof String){
				setChar2RegEx((String)result);
			}
		}
		converter.setChar2RegEx(getChar2RegEx());

		if(char3RegEx != null && UIComponentTag.isValueReference(char3RegEx)){
			Object result = TagUtil.getValueBindingValue(char3RegEx);
			if(result instanceof String){
				setChar3RegEx((String)result);
			}
		}
		converter.setChar3RegEx(getChar3RegEx());

		if(char4RegEx != null && UIComponentTag.isValueReference(char4RegEx)){
			Object result = TagUtil.getValueBindingValue(char4RegEx);
			if(result instanceof String){
				setChar4RegEx((String)result);
			}
		}
		converter.setChar4RegEx(getChar4RegEx());

		return converter;
	}
	
	public String isAsis() {
		return asis;
	}

	public String getMask() {
		return mask;
	}

	public void setAsis(String b) {
		asis = b;
	}

	public void setMask(String string) {
		mask = string;
	}
	
	

	public String getChar1RegEx() {
		return char1RegEx;
	}
	public void setChar1RegEx(String char1RegEx) {
		this.char1RegEx = char1RegEx;
	}
	public String getChar2RegEx() {
		return char2RegEx;
	}
	public void setChar2RegEx(String char2RegEx) {
		this.char2RegEx = char2RegEx;
	}
	public String getChar3RegEx() {
		return char3RegEx;
	}
	public void setChar3RegEx(String char3RegEx) {
		this.char3RegEx = char3RegEx;
	}
	public String getChar4RegEx() {
		return char4RegEx;
	}
	public void setChar4RegEx(String char4RegEx) {
		this.char4RegEx = char4RegEx;
	}

	public String getProtectMask() {
		return protectMask;
	}
	public void setProtectMask(String protectMask) {
		this.protectMask = protectMask;
	}
}
