/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2004 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.taglib.html_extended;

import javax.faces.convert.Converter;
import javax.faces.webapp.ConverterTag;
import javax.faces.webapp.UIComponentTag;
import javax.servlet.jsp.JspException;
import com.ibm.faces.converter.NumberConverterExICU;
import com.ibm.faces.util.TagUtil;
import java.util.Locale;

public class ConvertNumberTag extends ConverterTag {

	private String currencyCode;
	private String currencySymbol;
	private String groupingUsed;
	private String integerOnly;
	private String locale;
	private String maxFractionDigits;
	private String maxIntegerDigits;
	private String minFractionDigits;
	private String minIntegerDigits;
	private String pattern;	
	private String type;

		
		
	public ConvertNumberTag(){
		super();
		super.setConverterId("com.ibm.faces.Number"); //$NON-NLS-1$
	}
	
    public void release() {
        super.release();
    	currencyCode = null;
    	currencySymbol = null;
    	groupingUsed = null;
    	integerOnly = null;
    	locale = null;
    	maxFractionDigits = null;
    	maxIntegerDigits = null;
    	minFractionDigits = null;
    	minIntegerDigits = null;
    	pattern = null;	
    	type = null;
    }
	
	/* (non-Javadoc)
	 * @see javax.faces.webapp.ConverterTag#createConverter()
	 */
	protected Converter createConverter() throws JspException {		
		NumberConverterExICU converter = null;
		converter = (NumberConverterExICU)super.createConverter();

		//currencyCode
		if(currencyCode != null)
		{
			if(UIComponentTag.isValueReference(currencyCode))
			{
				Object result = TagUtil.getValueBindingValue(currencyCode);
				if(result instanceof String)
				{				
					converter.setCurrencyCode((String)result);					
				}	
			} else 
			{
				converter.setCurrencyCode(currencyCode);	
			}			
		}		

		//currencySymbol
		if(currencySymbol != null)
		{
			if(UIComponentTag.isValueReference(currencySymbol))
			{
				Object result = TagUtil.getValueBindingValue(currencySymbol);
				if(result instanceof String)
				{				
					converter.setCurrencySymbol((String)result);					
				}	
			} else 
			{
				converter.setCurrencySymbol(currencySymbol);	
			}			
		}	
		
		//groupingUsed
		if(groupingUsed != null)
		{
			if(UIComponentTag.isValueReference(groupingUsed))
			{
				Object result = TagUtil.getValueBindingValue(groupingUsed);
				if(result instanceof Boolean)
				{				
					converter.setGroupingUsed(((Boolean)result).booleanValue());					
				}
				if(result instanceof String){
					converter.setGroupingUsed((Boolean.valueOf(((String)result).trim())).booleanValue());
				}
			} else 
			{
				converter.setGroupingUsed((Boolean.valueOf(groupingUsed.trim())).booleanValue());	
			}			
		}	
		
		//integerOnly
		if(integerOnly != null)
		{
			if(UIComponentTag.isValueReference(integerOnly))
			{
				Object result = TagUtil.getValueBindingValue(integerOnly);
				if(result instanceof Boolean)
				{				
					converter.setIntegerOnly(((Boolean)result).booleanValue());					
				}
				if(result instanceof String){
					converter.setIntegerOnly((Boolean.valueOf(((String)result).trim())).booleanValue());
				}
			} else 
			{
				converter.setIntegerOnly((Boolean.valueOf(integerOnly.trim())).booleanValue());	
			}			
		}			

		//locale
		if(locale != null)
		{
			if(UIComponentTag.isValueReference(locale))
			{
				Object result = TagUtil.getValueBindingValue(locale);
				if(result instanceof String)
				{
					converter.setLocale(new Locale((String)result,""));
				}
				if(result instanceof Locale)
				{
					converter.setLocale((Locale)result);
				}			
			} else 
			{
				converter.setLocale(new Locale(locale,""));
			}
		}

		//maxFractionDigits
		if(maxFractionDigits != null)
		{ 
			if(UIComponentTag.isValueReference(maxFractionDigits))
			{
				Object result = TagUtil.getValueBindingValue(maxFractionDigits);
				if(result instanceof Integer){
					converter.setMaxFractionDigits(((Integer)result).intValue());
				}
				if(result instanceof String)
				{										
					converter.setMaxFractionDigits(Integer.parseInt(((String)result).trim()));
				}
				
			} else 
			{
				converter.setMaxFractionDigits(Integer.parseInt(maxFractionDigits.trim()));	
			}
		}

		//maxIntegerDigits
		if(maxIntegerDigits != null)
		{ 
			if(UIComponentTag.isValueReference(maxIntegerDigits))
			{
				Object result = TagUtil.getValueBindingValue(maxIntegerDigits);
				if(result instanceof Integer){
					converter.setMaxIntegerDigits(((Integer)result).intValue());
				}
				if(result instanceof String)
				{										
					converter.setMaxIntegerDigits(Integer.parseInt(((String)result).trim()));
				}
			} else 
			{
				converter.setMaxIntegerDigits(Integer.parseInt(maxIntegerDigits.trim()));	
			}
		}		

		//minFractionDigits
		if(minFractionDigits != null)
		{ 
			if(UIComponentTag.isValueReference(minFractionDigits))
			{
				Object result = TagUtil.getValueBindingValue(minFractionDigits);
				if(result instanceof Integer){
					converter.setMinFractionDigits(((Integer)result).intValue());
				}
				if(result instanceof String)
				{										
					converter.setMinFractionDigits(Integer.parseInt(((String)result).trim()));
				}
			} else 
			{
				converter.setMinFractionDigits(Integer.parseInt(minFractionDigits.trim()));	
			}
		}

		//minIntegerDigits
		if(minIntegerDigits != null)
		{ 
			if(UIComponentTag.isValueReference(minIntegerDigits))
			{
				Object result = TagUtil.getValueBindingValue(minIntegerDigits);
				if(result instanceof Integer){
					converter.setMinIntegerDigits(((Integer)result).intValue());
				}
				if(result instanceof String)
				{										
					converter.setMinIntegerDigits(Integer.parseInt(((String)result).trim()));
				}
			} else 
			{
				converter.setMinIntegerDigits(Integer.parseInt(minIntegerDigits.trim()));	
			}
		}		
		
		//pattern
		if(pattern != null)
		{ 
			if(UIComponentTag.isValueReference(pattern))
			{
				Object result = TagUtil.getValueBindingValue(pattern);
				if(result instanceof String)
				{
					converter.setPattern((String)result);
				}
			} else 
			{
				converter.setPattern(pattern);	
			}			
		}
		
		//type
		if(type != null)
		{
			if(UIComponentTag.isValueReference(type))
			{
				Object result = TagUtil.getValueBindingValue(type);
				if(result instanceof String)
				{				
					converter.setType((String)result);					
				}	
			} else 
			{
				converter.setType(type);	
			}			
		}		
		
		return converter;
	}


	/**
	 * @param currencyCode The currencyCode to set.
	 */
	public void setCurrencyCode(String currencyCode) {
		this.currencyCode = currencyCode;
	}
	/**
	 * @param currencySymbol The currencySymbol to set.
	 */
	public void setCurrencySymbol(String currencySymbol) {
		this.currencySymbol = currencySymbol;
	}
	/**
	 * @param groupingUsed The groupingUsed to set.
	 */
	public void setGroupingUsed(String groupingUsed) {
		this.groupingUsed = groupingUsed;
	}
	/**
	 * @param integerOnly The integerOnly to set.
	 */
	public void setIntegerOnly(String integerOnly) {
		this.integerOnly = integerOnly;
	}
	/**
	 * @param locale The locale to set.
	 */
	public void setLocale(String locale) {
		this.locale = locale;
	}
	/**
	 * @param maxFractionDigits The maxFractionDigits to set.
	 */
	public void setMaxFractionDigits(String maxFractionDigits) {
		this.maxFractionDigits = maxFractionDigits;
	}
	/**
	 * @param maxIntegerDigits The maxIntegerDigits to set.
	 */
	public void setMaxIntegerDigits(String maxIntegerDigits) {
		this.maxIntegerDigits = maxIntegerDigits;
	}
	/**
	 * @param minFractionDigits The minFractionDigits to set.
	 */
	public void setMinFractionDigits(String minFractionDigits) {
		this.minFractionDigits = minFractionDigits;
	}
	/**
	 * @param minIntegerDigits The minIntegerDigits to set.
	 */
	public void setMinIntegerDigits(String minIntegerDigits) {
		this.minIntegerDigits = minIntegerDigits;
	}
	/**
	 * @param pattern The pattern to set.
	 */
	public void setPattern(String pattern) {
		this.pattern = pattern;
	}
	/**
	 * @param type The type to set.
	 */
	public void setType(String type) {
		this.type = type;
	}
}
