/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIDataIterator;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * Data iterator component, iterates over a DataModel providing data to its child components.
 */
public class DataIteratorTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(DataIteratorTag.class);

    //
    // Instance Variables
    //

    private java.lang.String first;
    private java.lang.String rows;
    private java.lang.String value;
    private java.lang.String _var;


    //
    // Setter Methods
    //

    public void setFirst(java.lang.String first) {
        this.first = first;
    }

    public void setRows(java.lang.String rows) {
        this.rows = rows;
    }

    public void setValue(java.lang.String value) {
        this.value = value;
    }

    public void setVar(java.lang.String _var) {
        this._var = _var;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.DataIterator";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlDataIterator"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIDataIterator dataiteratorComponent = (UIDataIterator)component;

        if (first != null) {
            if (isValueReference(first)) {
                ValueBinding vb = TagUtil.getValueBinding(first);
                dataiteratorComponent.setValueBinding("first", vb); //$NON-NLS-1$
            } else {
                int _first = new Integer(first).intValue();
                dataiteratorComponent.setFirst(_first);
            }
        }
        if (rows != null) {
            if (isValueReference(rows)) {
                ValueBinding vb = TagUtil.getValueBinding(rows);
                dataiteratorComponent.setValueBinding("rows", vb); //$NON-NLS-1$
            } else {
                int _rows = new Integer(rows).intValue();
                dataiteratorComponent.setRows(_rows);
            }
        }
        if (value != null) {
            if (isValueReference(value)) {
                ValueBinding vb = TagUtil.getValueBinding(value);
                dataiteratorComponent.setValueBinding("value", vb); //$NON-NLS-1$
            } else {
                dataiteratorComponent.setValue(value);
            }
        }
        if (_var != null) {
            if (isValueReference(_var)) {
                ValueBinding vb = TagUtil.getValueBinding(_var);
                dataiteratorComponent.setValueBinding("_var", vb); //$NON-NLS-1$
            } else {
                dataiteratorComponent.setVar(_var);
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
