/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIFormItem;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class FormItemTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(FormItemTag.class);

    //
    // Instance Variables
    //

    private java.lang.String errorText;
    private java.lang.String infoText;
    private java.lang.String showHelp;

    private java.lang.String accesskey;
    private java.lang.String escape;
    private java.lang.String label;
    private java.lang.String noClientValidation;
    private java.lang.String releaseMargin;
    private java.lang.String style;
    private java.lang.String styleClass;
    private java.lang.String title;

    //
    // Setter Methods
    //

    public void setErrorText(java.lang.String errorText) {
        this.errorText = errorText;
    }

    public void setInfoText(java.lang.String infoText) {
        this.infoText = infoText;
    }

    public void setShowHelp(java.lang.String showHelp) {
        this.showHelp = showHelp;
    }

    public void setAccesskey(java.lang.String accesskey) {
        this.accesskey = accesskey;
    }

    public void setEscape(java.lang.String escape) {
        this.escape = escape;
    }

    public void setLabel(java.lang.String label) {
        this.label = label;
    }

    public void setNoClientValidation(java.lang.String noClientValidation) {
        this.noClientValidation = noClientValidation;
    }

    public void setReleaseMargin(java.lang.String releaseMargin) {
        this.releaseMargin = releaseMargin;
    }

    public void setStyle(java.lang.String style) {
        this.style = style;
    }

    public void setStyleClass(java.lang.String styleClass) {
        this.styleClass = styleClass;
    }

    public void setTitle(java.lang.String title) {
        this.title = title;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.FormItem";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlFormItem"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIFormItem formitemComponent = (UIFormItem)component;

        if (errorText != null) {
            if (isValueReference(errorText)) {
                ValueBinding vb = TagUtil.getValueBinding(errorText);
                formitemComponent.setValueBinding("errorText", vb); //$NON-NLS-1$
            } else {
                formitemComponent.setErrorText(errorText);
            }
        }
        if (infoText != null) {
            if (isValueReference(infoText)) {
                ValueBinding vb = TagUtil.getValueBinding(infoText);
                formitemComponent.setValueBinding("infoText", vb); //$NON-NLS-1$
            } else {
                formitemComponent.setInfoText(infoText);
            }
        }
        if (showHelp != null) {
            if (isValueReference(showHelp)) {
                ValueBinding vb = TagUtil.getValueBinding(showHelp);
                formitemComponent.setValueBinding("showHelp", vb); //$NON-NLS-1$
            } else {
                formitemComponent.setShowHelp(showHelp);
            }
        }
        if (accesskey != null) {
            if (isValueReference(accesskey)) {
                ValueBinding vb = TagUtil.getValueBinding(accesskey);
                formitemComponent.setValueBinding("accesskey", vb); //$NON-NLS-1$
            } else {
                formitemComponent.getAttributes().put("accesskey", accesskey); //$NON-NLS-1$
            }
        }
        if (escape != null) {
            if (isValueReference(escape)) {
                ValueBinding vb = TagUtil.getValueBinding(escape);
                formitemComponent.setValueBinding("escape", vb); //$NON-NLS-1$
            } else {
                boolean _escape = new Boolean(escape).booleanValue();
                formitemComponent.getAttributes().put("escape", _escape ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (label != null) {
            if (isValueReference(label)) {
                ValueBinding vb = TagUtil.getValueBinding(label);
                formitemComponent.setValueBinding("label", vb); //$NON-NLS-1$
            } else {
                formitemComponent.getAttributes().put("label", label); //$NON-NLS-1$
            }
        }
        if (noClientValidation != null) {
            if (isValueReference(noClientValidation)) {
                ValueBinding vb = TagUtil.getValueBinding(noClientValidation);
                formitemComponent.setValueBinding("noClientValidation", vb); //$NON-NLS-1$
            } else {
                boolean _noClientValidation = new Boolean(noClientValidation).booleanValue();
                formitemComponent.getAttributes().put("noClientValidation", _noClientValidation ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (releaseMargin != null) {
            if (isValueReference(releaseMargin)) {
                ValueBinding vb = TagUtil.getValueBinding(releaseMargin);
                formitemComponent.setValueBinding("releaseMargin", vb); //$NON-NLS-1$
            } else {
                formitemComponent.getAttributes().put("releaseMargin", releaseMargin); //$NON-NLS-1$
            }
        }
        if (style != null) {
            if (isValueReference(style)) {
                ValueBinding vb = TagUtil.getValueBinding(style);
                formitemComponent.setValueBinding("style", vb); //$NON-NLS-1$
            } else {
                formitemComponent.getAttributes().put("style", style); //$NON-NLS-1$
            }
        }
        if (styleClass != null) {
            if (isValueReference(styleClass)) {
                ValueBinding vb = TagUtil.getValueBinding(styleClass);
                formitemComponent.setValueBinding("styleClass", vb); //$NON-NLS-1$
            } else {
                formitemComponent.getAttributes().put("styleClass", styleClass); //$NON-NLS-1$
            }
        }
        if (title != null) {
            if (isValueReference(title)) {
                ValueBinding vb = TagUtil.getValueBinding(title);
                formitemComponent.setValueBinding("title", vb); //$NON-NLS-1$
            } else {
                formitemComponent.getAttributes().put("title", title); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
