/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIFormMessagesArea;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class FormMessagesAreaTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(FormMessagesAreaTag.class);

    //
    // Instance Variables
    //


    private java.lang.String errorText;
    private java.lang.String escape;
    private java.lang.String infoText;
    private java.lang.String showErrors;
    private java.lang.String showHelp;
    private java.lang.String style;
    private java.lang.String styleClass;
    private java.lang.String title;

    //
    // Setter Methods
    //

    public void setErrorText(java.lang.String errorText) {
        this.errorText = errorText;
    }

    public void setEscape(java.lang.String escape) {
        this.escape = escape;
    }

    public void setInfoText(java.lang.String infoText) {
        this.infoText = infoText;
    }

    public void setShowErrors(java.lang.String showErrors) {
        this.showErrors = showErrors;
    }

    public void setShowHelp(java.lang.String showHelp) {
        this.showHelp = showHelp;
    }

    public void setStyle(java.lang.String style) {
        this.style = style;
    }

    public void setStyleClass(java.lang.String styleClass) {
        this.styleClass = styleClass;
    }

    public void setTitle(java.lang.String title) {
        this.title = title;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.FormMessagesArea";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlFormMessagesArea"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIFormMessagesArea formmessagesareaComponent = (UIFormMessagesArea)component;

        if (errorText != null) {
            if (isValueReference(errorText)) {
                ValueBinding vb = TagUtil.getValueBinding(errorText);
                formmessagesareaComponent.setValueBinding("errorText", vb); //$NON-NLS-1$
            } else {
                formmessagesareaComponent.getAttributes().put("errorText", errorText); //$NON-NLS-1$
            }
        }
        if (escape != null) {
            if (isValueReference(escape)) {
                ValueBinding vb = TagUtil.getValueBinding(escape);
                formmessagesareaComponent.setValueBinding("escape", vb); //$NON-NLS-1$
            } else {
                boolean _escape = new Boolean(escape).booleanValue();
                formmessagesareaComponent.getAttributes().put("escape", _escape ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (infoText != null) {
            if (isValueReference(infoText)) {
                ValueBinding vb = TagUtil.getValueBinding(infoText);
                formmessagesareaComponent.setValueBinding("infoText", vb); //$NON-NLS-1$
            } else {
                formmessagesareaComponent.getAttributes().put("infoText", infoText); //$NON-NLS-1$
            }
        }
        if (showErrors != null) {
            if (isValueReference(showErrors)) {
                ValueBinding vb = TagUtil.getValueBinding(showErrors);
                formmessagesareaComponent.setValueBinding("showErrors", vb); //$NON-NLS-1$
            } else {
                formmessagesareaComponent.getAttributes().put("showErrors", showErrors); //$NON-NLS-1$
            }
        }
        if (showHelp != null) {
            if (isValueReference(showHelp)) {
                ValueBinding vb = TagUtil.getValueBinding(showHelp);
                formmessagesareaComponent.setValueBinding("showHelp", vb); //$NON-NLS-1$
            } else {
                formmessagesareaComponent.getAttributes().put("showHelp", showHelp); //$NON-NLS-1$
            }
        }
        if (style != null) {
            if (isValueReference(style)) {
                ValueBinding vb = TagUtil.getValueBinding(style);
                formmessagesareaComponent.setValueBinding("style", vb); //$NON-NLS-1$
            } else {
                formmessagesareaComponent.getAttributes().put("style", style); //$NON-NLS-1$
            }
        }
        if (styleClass != null) {
            if (isValueReference(styleClass)) {
                ValueBinding vb = TagUtil.getValueBinding(styleClass);
                formmessagesareaComponent.setValueBinding("styleClass", vb); //$NON-NLS-1$
            } else {
                formmessagesareaComponent.getAttributes().put("styleClass", styleClass); //$NON-NLS-1$
            }
        }
        if (title != null) {
            if (isValueReference(title)) {
                ValueBinding vb = TagUtil.getValueBinding(title);
                formmessagesareaComponent.setValueBinding("title", vb); //$NON-NLS-1$
            } else {
                formmessagesareaComponent.getAttributes().put("title", title); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
