/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIInputHelper;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class InputHelperKeybindTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(InputHelperKeybindTag.class);

    //
    // Instance Variables
    //


    private java.lang.String cancelBubble;
    private java.lang.String key;
    private java.lang.String onpress;
    private java.lang.String target;
    private java.lang.String targetAction;

    //
    // Setter Methods
    //

    public void setCancelBubble(java.lang.String cancelBubble) {
        this.cancelBubble = cancelBubble;
    }

    public void setKey(java.lang.String key) {
        this.key = key;
    }

    public void setOnpress(java.lang.String onpress) {
        this.onpress = onpress;
    }

    public void setTarget(java.lang.String target) {
        this.target = target;
    }

    public void setTargetAction(java.lang.String targetAction) {
        this.targetAction = targetAction;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.Keybind";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlInputHelperKeybind"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIInputHelper inputhelperComponent = (UIInputHelper)component;

        if (cancelBubble != null) {
            if (isValueReference(cancelBubble)) {
                ValueBinding vb = TagUtil.getValueBinding(cancelBubble);
                inputhelperComponent.setValueBinding("cancelBubble", vb); //$NON-NLS-1$
            } else {
                inputhelperComponent.getAttributes().put("cancelBubble", cancelBubble); //$NON-NLS-1$
            }
        }
        if (key != null) {
            if (isValueReference(key)) {
                ValueBinding vb = TagUtil.getValueBinding(key);
                inputhelperComponent.setValueBinding("key", vb); //$NON-NLS-1$
            } else {
                inputhelperComponent.getAttributes().put("key", key); //$NON-NLS-1$
            }
        }
        if (onpress != null) {
            if (isValueReference(onpress)) {
                ValueBinding vb = TagUtil.getValueBinding(onpress);
                inputhelperComponent.setValueBinding("onpress", vb); //$NON-NLS-1$
            } else {
                inputhelperComponent.getAttributes().put("onpress", onpress); //$NON-NLS-1$
            }
        }
        if (target != null) {
            if (isValueReference(target)) {
                ValueBinding vb = TagUtil.getValueBinding(target);
                inputhelperComponent.setValueBinding("target", vb); //$NON-NLS-1$
            } else {
                inputhelperComponent.getAttributes().put("target", target); //$NON-NLS-1$
            }
        }
        if (targetAction != null) {
            if (isValueReference(targetAction)) {
                ValueBinding vb = TagUtil.getValueBinding(targetAction);
                inputhelperComponent.setValueBinding("targetAction", vb); //$NON-NLS-1$
            } else {
                inputhelperComponent.getAttributes().put("targetAction", targetAction); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
