/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIInputHelper;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class InputHelperSetFocusTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(InputHelperSetFocusTag.class);

    //
    // Instance Variables
    //


    private java.lang.String select;
    private java.lang.String target;

    //
    // Setter Methods
    //

    public void setSelect(java.lang.String select) {
        this.select = select;
    }

    public void setTarget(java.lang.String target) {
        this.target = target;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.SetFocus";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlInputHelperSetFocus"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIInputHelper inputhelperComponent = (UIInputHelper)component;

        if (select != null) {
            if (isValueReference(select)) {
                ValueBinding vb = TagUtil.getValueBinding(select);
                inputhelperComponent.setValueBinding("select", vb); //$NON-NLS-1$
            } else {
                boolean _select = new Boolean(select).booleanValue();
                inputhelperComponent.getAttributes().put("select", _select ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (target != null) {
            if (isValueReference(target)) {
                ValueBinding vb = TagUtil.getValueBinding(target);
                inputhelperComponent.setValueBinding("target", vb); //$NON-NLS-1$
            } else {
                inputhelperComponent.getAttributes().put("target", target); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
