/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class InputMiniCalendarTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(InputMiniCalendarTag.class);

    //
    // Instance Variables
    //

    private java.lang.String converter;
    private java.lang.String immediate;
    private java.lang.String required;
    private java.lang.String validator;
    private java.lang.String value;
    private java.lang.String valueChangeListener;

    private java.lang.String accesskey;
    private java.lang.String dir;
    private java.lang.String disabled;
    private java.lang.String lang;
    private java.lang.String multiLine;
    private java.lang.String onblur;
    private java.lang.String onchange;
    private java.lang.String onfocus;
    private java.lang.String style;
    private java.lang.String styleClass;
    private java.lang.String tabindex;
    private java.lang.String title;

    //
    // Setter Methods
    //

    public void setConverter(java.lang.String converter) {
        this.converter = converter;
    }

    public void setImmediate(java.lang.String immediate) {
        this.immediate = immediate;
    }

    public void setRequired(java.lang.String required) {
        this.required = required;
    }

    public void setValidator(java.lang.String validator) {
        this.validator = validator;
    }

    public void setValue(java.lang.String value) {
        this.value = value;
    }

    public void setValueChangeListener(java.lang.String valueChangeListener) {
        this.valueChangeListener = valueChangeListener;
    }

    public void setAccesskey(java.lang.String accesskey) {
        this.accesskey = accesskey;
    }

    public void setDir(java.lang.String dir) {
        this.dir = dir;
    }

    public void setDisabled(java.lang.String disabled) {
        this.disabled = disabled;
    }

    public void setLang(java.lang.String lang) {
        this.lang = lang;
    }

    public void setMultiLine(java.lang.String multiLine) {
        this.multiLine = multiLine;
    }

    public void setOnblur(java.lang.String onblur) {
        this.onblur = onblur;
    }

    public void setOnchange(java.lang.String onchange) {
        this.onchange = onchange;
    }

    public void setOnfocus(java.lang.String onfocus) {
        this.onfocus = onfocus;
    }

    public void setStyle(java.lang.String style) {
        this.style = style;
    }

    public void setStyleClass(java.lang.String styleClass) {
        this.styleClass = styleClass;
    }

    public void setTabindex(java.lang.String tabindex) {
        this.tabindex = tabindex;
    }

    public void setTitle(java.lang.String title) {
        this.title = title;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.MiniCalendar";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlInputMiniCalendar"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIInput inputComponent = (UIInput)component;

        if (converter != null) {
            if (isValueReference(converter)) {
                      ValueBinding vb = 
                          TagUtil.getValueBinding(converter);
                  inputComponent.setValueBinding("converter", vb); //$NON-NLS-1$
            } else {
                Converter _converter = FacesContext.getCurrentInstance().
                    getApplication().createConverter(converter);
                inputComponent.setConverter(_converter);
            }
        }

        if (immediate != null) {
            if (isValueReference(immediate)) {
                ValueBinding vb = TagUtil.getValueBinding(immediate);
                inputComponent.setValueBinding("immediate", vb); //$NON-NLS-1$
            } else {
                boolean _immediate = new Boolean(immediate).booleanValue();
                inputComponent.setImmediate(_immediate);
            }
        }
        if (required != null) {
            if (isValueReference(required)) {
                ValueBinding vb = TagUtil.getValueBinding(required);
                inputComponent.setValueBinding("required", vb); //$NON-NLS-1$
            } else {
                boolean _required = new Boolean(required).booleanValue();
                inputComponent.setRequired(_required);
            }
        }
        if (validator != null) {
            if (isValueReference(validator)) {
                Class args[] = { FacesContext.class, UIComponent.class, Object.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(validator, args);
                inputComponent.setValidator(vb);
            } else {
              Object params [] = {validator};
              throw new javax.faces.FacesException();
            }
            }
        if (value != null) {
            if (isValueReference(value)) {
                ValueBinding vb = TagUtil.getValueBinding(value);
                inputComponent.setValueBinding("value", vb); //$NON-NLS-1$
            } else {
                inputComponent.setValue(value);
            }
        }
        if (valueChangeListener != null) {
            if (isValueReference(valueChangeListener)) {
                Class args[] = { ValueChangeEvent.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(valueChangeListener, args);
                inputComponent.setValueChangeListener(vb);
            } else {
              Object params [] = {valueChangeListener};
              throw new javax.faces.FacesException();
            }
            }
        if (accesskey != null) {
            if (isValueReference(accesskey)) {
                ValueBinding vb = TagUtil.getValueBinding(accesskey);
                inputComponent.setValueBinding("accesskey", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("accesskey", accesskey); //$NON-NLS-1$
            }
        }
        if (dir != null) {
            if (isValueReference(dir)) {
                ValueBinding vb = TagUtil.getValueBinding(dir);
                inputComponent.setValueBinding("dir", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("dir", dir); //$NON-NLS-1$
            }
        }
        if (disabled != null) {
            if (isValueReference(disabled)) {
                ValueBinding vb = TagUtil.getValueBinding(disabled);
                inputComponent.setValueBinding("disabled", vb); //$NON-NLS-1$
            } else {
                boolean _disabled = new Boolean(disabled).booleanValue();
                inputComponent.getAttributes().put("disabled", _disabled ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (lang != null) {
            if (isValueReference(lang)) {
                ValueBinding vb = TagUtil.getValueBinding(lang);
                inputComponent.setValueBinding("lang", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("lang", lang); //$NON-NLS-1$
            }
        }
        if (multiLine != null) {
            if (isValueReference(multiLine)) {
                ValueBinding vb = TagUtil.getValueBinding(multiLine);
                inputComponent.setValueBinding("multiLine", vb); //$NON-NLS-1$
            } else {
                boolean _multiLine = new Boolean(multiLine).booleanValue();
                inputComponent.getAttributes().put("multiLine", _multiLine ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (onblur != null) {
            if (isValueReference(onblur)) {
                ValueBinding vb = TagUtil.getValueBinding(onblur);
                inputComponent.setValueBinding("onblur", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("onblur", onblur); //$NON-NLS-1$
            }
        }
        if (onchange != null) {
            if (isValueReference(onchange)) {
                ValueBinding vb = TagUtil.getValueBinding(onchange);
                inputComponent.setValueBinding("onchange", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("onchange", onchange); //$NON-NLS-1$
            }
        }
        if (onfocus != null) {
            if (isValueReference(onfocus)) {
                ValueBinding vb = TagUtil.getValueBinding(onfocus);
                inputComponent.setValueBinding("onfocus", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("onfocus", onfocus); //$NON-NLS-1$
            }
        }
        if (style != null) {
            if (isValueReference(style)) {
                ValueBinding vb = TagUtil.getValueBinding(style);
                inputComponent.setValueBinding("style", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("style", style); //$NON-NLS-1$
            }
        }
        if (styleClass != null) {
            if (isValueReference(styleClass)) {
                ValueBinding vb = TagUtil.getValueBinding(styleClass);
                inputComponent.setValueBinding("styleClass", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("styleClass", styleClass); //$NON-NLS-1$
            }
        }
        if (tabindex != null) {
            if (isValueReference(tabindex)) {
                ValueBinding vb = TagUtil.getValueBinding(tabindex);
                inputComponent.setValueBinding("tabindex", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("tabindex", tabindex); //$NON-NLS-1$
            }
        }
        if (title != null) {
            if (isValueReference(title)) {
                ValueBinding vb = TagUtil.getValueBinding(title);
                inputComponent.setValueBinding("title", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("title", title); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
