/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class InputRowSelectTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(InputRowSelectTag.class);

    //
    // Instance Variables
    //

    private java.lang.String converter;
    private java.lang.String immediate;
    private java.lang.String required;
    private java.lang.String validator;
    private java.lang.String value;
    private java.lang.String valueChangeListener;

    private java.lang.String columnClasses;
    private java.lang.String noscript;
    private java.lang.String onclick;
    private java.lang.String onmouseup;
    private java.lang.String selectOne;
    private java.lang.String srcSelected;
    private java.lang.String srcUnselected;
    private java.lang.String style;
    private java.lang.String styleClass;
    private java.lang.String title;
    private java.lang.String titleSelected;

    //
    // Setter Methods
    //

    public void setConverter(java.lang.String converter) {
        this.converter = converter;
    }

    public void setImmediate(java.lang.String immediate) {
        this.immediate = immediate;
    }

    public void setRequired(java.lang.String required) {
        this.required = required;
    }

    public void setValidator(java.lang.String validator) {
        this.validator = validator;
    }

    public void setValue(java.lang.String value) {
        this.value = value;
    }

    public void setValueChangeListener(java.lang.String valueChangeListener) {
        this.valueChangeListener = valueChangeListener;
    }

    public void setColumnClasses(java.lang.String columnClasses) {
        this.columnClasses = columnClasses;
    }

    public void setNoscript(java.lang.String noscript) {
        this.noscript = noscript;
    }

    public void setOnclick(java.lang.String onclick) {
        this.onclick = onclick;
    }

    public void setOnmouseup(java.lang.String onmouseup) {
        this.onmouseup = onmouseup;
    }

    public void setSelectOne(java.lang.String selectOne) {
        this.selectOne = selectOne;
    }

    public void setSrcSelected(java.lang.String srcSelected) {
        this.srcSelected = srcSelected;
    }

    public void setSrcUnselected(java.lang.String srcUnselected) {
        this.srcUnselected = srcUnselected;
    }

    public void setStyle(java.lang.String style) {
        this.style = style;
    }

    public void setStyleClass(java.lang.String styleClass) {
        this.styleClass = styleClass;
    }

    public void setTitle(java.lang.String title) {
        this.title = title;
    }

    public void setTitleSelected(java.lang.String titleSelected) {
        this.titleSelected = titleSelected;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.RowSelect";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlInputRowSelect"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIInput inputComponent = (UIInput)component;

        if (converter != null) {
            if (isValueReference(converter)) {
                      ValueBinding vb = 
                          TagUtil.getValueBinding(converter);
                  inputComponent.setValueBinding("converter", vb); //$NON-NLS-1$
            } else {
                Converter _converter = FacesContext.getCurrentInstance().
                    getApplication().createConverter(converter);
                inputComponent.setConverter(_converter);
            }
        }

        if (immediate != null) {
            if (isValueReference(immediate)) {
                ValueBinding vb = TagUtil.getValueBinding(immediate);
                inputComponent.setValueBinding("immediate", vb); //$NON-NLS-1$
            } else {
                boolean _immediate = new Boolean(immediate).booleanValue();
                inputComponent.setImmediate(_immediate);
            }
        }
        if (required != null) {
            if (isValueReference(required)) {
                ValueBinding vb = TagUtil.getValueBinding(required);
                inputComponent.setValueBinding("required", vb); //$NON-NLS-1$
            } else {
                boolean _required = new Boolean(required).booleanValue();
                inputComponent.setRequired(_required);
            }
        }
        if (validator != null) {
            if (isValueReference(validator)) {
                Class args[] = { FacesContext.class, UIComponent.class, Object.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(validator, args);
                inputComponent.setValidator(vb);
            } else {
              Object params [] = {validator};
              throw new javax.faces.FacesException();
            }
            }
        if (value != null) {
            if (isValueReference(value)) {
                ValueBinding vb = TagUtil.getValueBinding(value);
                inputComponent.setValueBinding("value", vb); //$NON-NLS-1$
            } else {
                inputComponent.setValue(value);
            }
        }
        if (valueChangeListener != null) {
            if (isValueReference(valueChangeListener)) {
                Class args[] = { ValueChangeEvent.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(valueChangeListener, args);
                inputComponent.setValueChangeListener(vb);
            } else {
              Object params [] = {valueChangeListener};
              throw new javax.faces.FacesException();
            }
            }
        if (columnClasses != null) {
            if (isValueReference(columnClasses)) {
                ValueBinding vb = TagUtil.getValueBinding(columnClasses);
                inputComponent.setValueBinding("columnClasses", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("columnClasses", columnClasses); //$NON-NLS-1$
            }
        }
        if (noscript != null) {
            if (isValueReference(noscript)) {
                ValueBinding vb = TagUtil.getValueBinding(noscript);
                inputComponent.setValueBinding("noscript", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("noscript", noscript); //$NON-NLS-1$
            }
        }
        if (onclick != null) {
            if (isValueReference(onclick)) {
                ValueBinding vb = TagUtil.getValueBinding(onclick);
                inputComponent.setValueBinding("onclick", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("onclick", onclick); //$NON-NLS-1$
            }
        }
        if (onmouseup != null) {
            if (isValueReference(onmouseup)) {
                ValueBinding vb = TagUtil.getValueBinding(onmouseup);
                inputComponent.setValueBinding("onmouseup", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("onmouseup", onmouseup); //$NON-NLS-1$
            }
        }
        if (selectOne != null) {
            if (isValueReference(selectOne)) {
                ValueBinding vb = TagUtil.getValueBinding(selectOne);
                inputComponent.setValueBinding("selectOne", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("selectOne", selectOne); //$NON-NLS-1$
            }
        }
        if (srcSelected != null) {
            if (isValueReference(srcSelected)) {
                ValueBinding vb = TagUtil.getValueBinding(srcSelected);
                inputComponent.setValueBinding("srcSelected", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("srcSelected", srcSelected); //$NON-NLS-1$
            }
        }
        if (srcUnselected != null) {
            if (isValueReference(srcUnselected)) {
                ValueBinding vb = TagUtil.getValueBinding(srcUnselected);
                inputComponent.setValueBinding("srcUnselected", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("srcUnselected", srcUnselected); //$NON-NLS-1$
            }
        }
        if (style != null) {
            if (isValueReference(style)) {
                ValueBinding vb = TagUtil.getValueBinding(style);
                inputComponent.setValueBinding("style", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("style", style); //$NON-NLS-1$
            }
        }
        if (styleClass != null) {
            if (isValueReference(styleClass)) {
                ValueBinding vb = TagUtil.getValueBinding(styleClass);
                inputComponent.setValueBinding("styleClass", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("styleClass", styleClass); //$NON-NLS-1$
            }
        }
        if (title != null) {
            if (isValueReference(title)) {
                ValueBinding vb = TagUtil.getValueBinding(title);
                inputComponent.setValueBinding("title", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("title", title); //$NON-NLS-1$
            }
        }
        if (titleSelected != null) {
            if (isValueReference(titleSelected)) {
                ValueBinding vb = TagUtil.getValueBinding(titleSelected);
                inputComponent.setValueBinding("titleSelected", vb); //$NON-NLS-1$
            } else {
                inputComponent.getAttributes().put("titleSelected", titleSelected); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
