/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2004 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.taglib.html_extended;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.component.html.HtmlOutputText;
import javax.faces.el.ValueBinding;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;

import com.ibm.faces.component.UIJspPanel;
import com.ibm.faces.util.TagUtil;


/**
 * <p>Tag that creates a {@link UIJspPanel} instance for using html for laying out jsf components.</p>
 */

public class JspPanelTag extends UIComponentBodyTag {

	public String getRendererType() { return "com.ibm.faces.JspPanel"; }
	public String getComponentType() { return "com.ibm.faces.HtmlJspPanel"; }

	protected void setProperties(UIComponent component) {
		super.setProperties(component);
		UIJspPanel jsppanel = (UIJspPanel)component;
		// give the component the jsp pagecontext for use in marking position of jsf
		// components within body content
		((UIJspPanel) component).setPageContext(this.pageContext);
		//component.setTransient(true); defect 280241 - need to be able to decode children

	}

	/**
	 * <p>Set the local value of this component to reflect the nested
	 * body content of this JSP tag.</p>
	 */
	public int doAfterBody() throws JspException {

		UIJspPanel component = (UIJspPanel) getComponentInstance();
		// stop building of tagged bodytext phase by clear pagecontext prop
		component.setPageContext(null);
		if (getBodyContent() != null) {
			String bodyContent = getBodyContent().getString().trim();
            boolean jsfKids = component.getChildCount() > 0;
            if(jsfKids) {
                if(bodyContent.indexOf(UIJspPanel.JSF_COMP_MARKER) < 0) {
                    return getDoAfterBodyValue();
                }
            } else {
                //no point keeping it around
                component.setTransient(true);
            }
			// we are going to rebuild the component's child list
			// save the current list of the jsf components and clear the child list
			// then add new text components and re-add jsf components as indicated 
			if (bodyContent != null) {
				Iterator jsfComponents;
				if (component.getChildCount() > 0){
					jsfComponents = (new ArrayList(component.getChildren())).iterator();
				} else {
					jsfComponents = java.util.Collections.EMPTY_LIST.iterator();
				}
				List childComponents = component.getChildren();
				childComponents.clear();
				HtmlOutputText outputText;
				String value;
				int markerIndex = 0;
				int valueIndex = 0;
				do {
					markerIndex = bodyContent.indexOf(UIJspPanel.JSF_COMP_MARKER, markerIndex);
					if (markerIndex >= 0){
						if (valueIndex == markerIndex){
							value = null;
						} else {
							value = bodyContent.substring(valueIndex, markerIndex);
						}
						markerIndex = markerIndex + UIJspPanel.JSF_COMP_MARKER.length();
					} else {
						value = bodyContent.substring(valueIndex);
					}
					valueIndex = markerIndex;
					if (null != value) {
						outputText = new HtmlOutputText();
						outputText.setEscape(false);
						outputText.setValue(value);
						childComponents.add(outputText);
					}
					if (jsfComponents.hasNext()) {
						childComponents.add(jsfComponents.next());
					}
				} while (markerIndex >= 0);

			}
		}
       
		return (getDoAfterBodyValue());
	}
}
