/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIPager;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class PagerDeluxeTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(PagerDeluxeTag.class);

    //
    // Instance Variables
    //

    private java.lang.String action;
    private java.lang.String actionListener;
    private java.lang.String immediate;

    private java.lang.String _for;
    private java.lang.String hideDisabledButtons;
    private java.lang.String showOnlyNextPrevious;
    private java.lang.String srcFirst;
    private java.lang.String srcLast;
    private java.lang.String srcNext;
    private java.lang.String srcPrevious;
    private java.lang.String style;
    private java.lang.String styleClass;
    private java.lang.String title;
    private java.lang.String useButtons;

    //
    // Setter Methods
    //

    public void setAction(java.lang.String action) {
        this.action = action;
    }

    public void setActionListener(java.lang.String actionListener) {
        this.actionListener = actionListener;
    }

    public void setImmediate(java.lang.String immediate) {
        this.immediate = immediate;
    }

    public void setFor(java.lang.String _for) {
        this._for = _for;
    }

    public void setHideDisabledButtons(java.lang.String hideDisabledButtons) {
        this.hideDisabledButtons = hideDisabledButtons;
    }

    public void setShowOnlyNextPrevious(java.lang.String showOnlyNextPrevious) {
        this.showOnlyNextPrevious = showOnlyNextPrevious;
    }

    public void setSrcFirst(java.lang.String srcFirst) {
        this.srcFirst = srcFirst;
    }

    public void setSrcLast(java.lang.String srcLast) {
        this.srcLast = srcLast;
    }

    public void setSrcNext(java.lang.String srcNext) {
        this.srcNext = srcNext;
    }

    public void setSrcPrevious(java.lang.String srcPrevious) {
        this.srcPrevious = srcPrevious;
    }

    public void setStyle(java.lang.String style) {
        this.style = style;
    }

    public void setStyleClass(java.lang.String styleClass) {
        this.styleClass = styleClass;
    }

    public void setTitle(java.lang.String title) {
        this.title = title;
    }

    public void setUseButtons(java.lang.String useButtons) {
        this.useButtons = useButtons;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.Deluxe";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlPagerDeluxe"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIPager pagerComponent = (UIPager)component;

        if (action != null) {
             if (isValueReference(action)) {
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(action, null);
                pagerComponent.setAction(vb);
            } else {
                final String outcome = action;
                MethodBinding vb = new StaticMethodBinding(action);
                pagerComponent.setAction(vb);
            }
        }
        if (actionListener != null) {
            if (isValueReference(actionListener)) {
                Class args[] = { ActionEvent.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(actionListener, args);
                pagerComponent.setActionListener(vb);
            } else {
              Object params [] = {actionListener};
              throw new javax.faces.FacesException();
            }
            }
        if (immediate != null) {
            if (isValueReference(immediate)) {
                ValueBinding vb = TagUtil.getValueBinding(immediate);
                pagerComponent.setValueBinding("immediate", vb); //$NON-NLS-1$
            } else {
                boolean _immediate = new Boolean(immediate).booleanValue();
                pagerComponent.setImmediate(_immediate);
            }
        }
        if (_for != null) {
            if (isValueReference(_for)) {
                ValueBinding vb = TagUtil.getValueBinding(_for);
                pagerComponent.setValueBinding("__for", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("for", _for); //$NON-NLS-1$
            }
        }
        if (hideDisabledButtons != null) {
            if (isValueReference(hideDisabledButtons)) {
                ValueBinding vb = TagUtil.getValueBinding(hideDisabledButtons);
                pagerComponent.setValueBinding("hideDisabledButtons", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("hideDisabledButtons", hideDisabledButtons); //$NON-NLS-1$
            }
        }
        if (showOnlyNextPrevious != null) {
            if (isValueReference(showOnlyNextPrevious)) {
                ValueBinding vb = TagUtil.getValueBinding(showOnlyNextPrevious);
                pagerComponent.setValueBinding("showOnlyNextPrevious", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("showOnlyNextPrevious", showOnlyNextPrevious); //$NON-NLS-1$
            }
        }
        if (srcFirst != null) {
            if (isValueReference(srcFirst)) {
                ValueBinding vb = TagUtil.getValueBinding(srcFirst);
                pagerComponent.setValueBinding("srcFirst", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("srcFirst", srcFirst); //$NON-NLS-1$
            }
        }
        if (srcLast != null) {
            if (isValueReference(srcLast)) {
                ValueBinding vb = TagUtil.getValueBinding(srcLast);
                pagerComponent.setValueBinding("srcLast", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("srcLast", srcLast); //$NON-NLS-1$
            }
        }
        if (srcNext != null) {
            if (isValueReference(srcNext)) {
                ValueBinding vb = TagUtil.getValueBinding(srcNext);
                pagerComponent.setValueBinding("srcNext", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("srcNext", srcNext); //$NON-NLS-1$
            }
        }
        if (srcPrevious != null) {
            if (isValueReference(srcPrevious)) {
                ValueBinding vb = TagUtil.getValueBinding(srcPrevious);
                pagerComponent.setValueBinding("srcPrevious", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("srcPrevious", srcPrevious); //$NON-NLS-1$
            }
        }
        if (style != null) {
            if (isValueReference(style)) {
                ValueBinding vb = TagUtil.getValueBinding(style);
                pagerComponent.setValueBinding("style", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("style", style); //$NON-NLS-1$
            }
        }
        if (styleClass != null) {
            if (isValueReference(styleClass)) {
                ValueBinding vb = TagUtil.getValueBinding(styleClass);
                pagerComponent.setValueBinding("styleClass", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("styleClass", styleClass); //$NON-NLS-1$
            }
        }
        if (title != null) {
            if (isValueReference(title)) {
                ValueBinding vb = TagUtil.getValueBinding(title);
                pagerComponent.setValueBinding("title", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("title", title); //$NON-NLS-1$
            }
        }
        if (useButtons != null) {
            if (isValueReference(useButtons)) {
                ValueBinding vb = TagUtil.getValueBinding(useButtons);
                pagerComponent.setValueBinding("useButtons", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("useButtons", useButtons); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
