/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIPager;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class PagerSimpleTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(PagerSimpleTag.class);

    //
    // Instance Variables
    //

    private java.lang.String action;
    private java.lang.String actionListener;
    private java.lang.String immediate;

    private java.lang.String buttonImageSrc;
    private java.lang.String _for;
    private java.lang.String style;
    private java.lang.String styleClass;
    private java.lang.String title;

    //
    // Setter Methods
    //

    public void setAction(java.lang.String action) {
        this.action = action;
    }

    public void setActionListener(java.lang.String actionListener) {
        this.actionListener = actionListener;
    }

    public void setImmediate(java.lang.String immediate) {
        this.immediate = immediate;
    }

    public void setButtonImageSrc(java.lang.String buttonImageSrc) {
        this.buttonImageSrc = buttonImageSrc;
    }

    public void setFor(java.lang.String _for) {
        this._for = _for;
    }

    public void setStyle(java.lang.String style) {
        this.style = style;
    }

    public void setStyleClass(java.lang.String styleClass) {
        this.styleClass = styleClass;
    }

    public void setTitle(java.lang.String title) {
        this.title = title;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.Simple";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlPagerSimple"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIPager pagerComponent = (UIPager)component;

        if (action != null) {
             if (isValueReference(action)) {
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(action, null);
                pagerComponent.setAction(vb);
            } else {
                final String outcome = action;
                MethodBinding vb = new StaticMethodBinding(action);
                pagerComponent.setAction(vb);
            }
        }
        if (actionListener != null) {
            if (isValueReference(actionListener)) {
                Class args[] = { ActionEvent.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(actionListener, args);
                pagerComponent.setActionListener(vb);
            } else {
              Object params [] = {actionListener};
              throw new javax.faces.FacesException();
            }
            }
        if (immediate != null) {
            if (isValueReference(immediate)) {
                ValueBinding vb = TagUtil.getValueBinding(immediate);
                pagerComponent.setValueBinding("immediate", vb); //$NON-NLS-1$
            } else {
                boolean _immediate = new Boolean(immediate).booleanValue();
                pagerComponent.setImmediate(_immediate);
            }
        }
        if (buttonImageSrc != null) {
            if (isValueReference(buttonImageSrc)) {
                ValueBinding vb = TagUtil.getValueBinding(buttonImageSrc);
                pagerComponent.setValueBinding("buttonImageSrc", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("buttonImageSrc", buttonImageSrc); //$NON-NLS-1$
            }
        }
        if (_for != null) {
            if (isValueReference(_for)) {
                ValueBinding vb = TagUtil.getValueBinding(_for);
                pagerComponent.setValueBinding("__for", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("for", _for); //$NON-NLS-1$
            }
        }
        if (style != null) {
            if (isValueReference(style)) {
                ValueBinding vb = TagUtil.getValueBinding(style);
                pagerComponent.setValueBinding("style", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("style", style); //$NON-NLS-1$
            }
        }
        if (styleClass != null) {
            if (isValueReference(styleClass)) {
                ValueBinding vb = TagUtil.getValueBinding(styleClass);
                pagerComponent.setValueBinding("styleClass", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("styleClass", styleClass); //$NON-NLS-1$
            }
        }
        if (title != null) {
            if (isValueReference(title)) {
                ValueBinding vb = TagUtil.getValueBinding(title);
                pagerComponent.setValueBinding("title", vb); //$NON-NLS-1$
            } else {
                pagerComponent.getAttributes().put("title", title); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
