/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import javax.faces.component.UIPanel;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class PanelFormBoxTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(PanelFormBoxTag.class);

    //
    // Instance Variables
    //


    private java.lang.String escape;
    private java.lang.String helpPosition;
    private java.lang.String interItemSpacing;
    private java.lang.String label;
    private java.lang.String labelPosition;
    private java.lang.String showInPopup;
    private java.lang.String style;
    private java.lang.String styleClass;
    private java.lang.String title;
    private java.lang.String widthContent;
    private java.lang.String widthHelp;
    private java.lang.String widthLabel;

    //
    // Setter Methods
    //

    public void setEscape(java.lang.String escape) {
        this.escape = escape;
    }

    public void setHelpPosition(java.lang.String helpPosition) {
        this.helpPosition = helpPosition;
    }

    public void setInterItemSpacing(java.lang.String interItemSpacing) {
        this.interItemSpacing = interItemSpacing;
    }

    public void setLabel(java.lang.String label) {
        this.label = label;
    }

    public void setLabelPosition(java.lang.String labelPosition) {
        this.labelPosition = labelPosition;
    }

    public void setShowInPopup(java.lang.String showInPopup) {
        this.showInPopup = showInPopup;
    }

    public void setStyle(java.lang.String style) {
        this.style = style;
    }

    public void setStyleClass(java.lang.String styleClass) {
        this.styleClass = styleClass;
    }

    public void setTitle(java.lang.String title) {
        this.title = title;
    }

    public void setWidthContent(java.lang.String widthContent) {
        this.widthContent = widthContent;
    }

    public void setWidthHelp(java.lang.String widthHelp) {
        this.widthHelp = widthHelp;
    }

    public void setWidthLabel(java.lang.String widthLabel) {
        this.widthLabel = widthLabel;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.FormBox";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlPanelFormBox"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIPanel panelComponent = (UIPanel)component;

        if (escape != null) {
            if (isValueReference(escape)) {
                ValueBinding vb = TagUtil.getValueBinding(escape);
                panelComponent.setValueBinding("escape", vb); //$NON-NLS-1$
            } else {
                boolean _escape = new Boolean(escape).booleanValue();
                panelComponent.getAttributes().put("escape", _escape ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (helpPosition != null) {
            if (isValueReference(helpPosition)) {
                ValueBinding vb = TagUtil.getValueBinding(helpPosition);
                panelComponent.setValueBinding("helpPosition", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("helpPosition", helpPosition); //$NON-NLS-1$
            }
        }
        if (interItemSpacing != null) {
            if (isValueReference(interItemSpacing)) {
                ValueBinding vb = TagUtil.getValueBinding(interItemSpacing);
                panelComponent.setValueBinding("interItemSpacing", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("interItemSpacing", interItemSpacing); //$NON-NLS-1$
            }
        }
        if (label != null) {
            if (isValueReference(label)) {
                ValueBinding vb = TagUtil.getValueBinding(label);
                panelComponent.setValueBinding("label", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("label", label); //$NON-NLS-1$
            }
        }
        if (labelPosition != null) {
            if (isValueReference(labelPosition)) {
                ValueBinding vb = TagUtil.getValueBinding(labelPosition);
                panelComponent.setValueBinding("labelPosition", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("labelPosition", labelPosition); //$NON-NLS-1$
            }
        }
        if (showInPopup != null) {
            if (isValueReference(showInPopup)) {
                ValueBinding vb = TagUtil.getValueBinding(showInPopup);
                panelComponent.setValueBinding("showInPopup", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("showInPopup", showInPopup); //$NON-NLS-1$
            }
        }
        if (style != null) {
            if (isValueReference(style)) {
                ValueBinding vb = TagUtil.getValueBinding(style);
                panelComponent.setValueBinding("style", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("style", style); //$NON-NLS-1$
            }
        }
        if (styleClass != null) {
            if (isValueReference(styleClass)) {
                ValueBinding vb = TagUtil.getValueBinding(styleClass);
                panelComponent.setValueBinding("styleClass", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("styleClass", styleClass); //$NON-NLS-1$
            }
        }
        if (title != null) {
            if (isValueReference(title)) {
                ValueBinding vb = TagUtil.getValueBinding(title);
                panelComponent.setValueBinding("title", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("title", title); //$NON-NLS-1$
            }
        }
        if (widthContent != null) {
            if (isValueReference(widthContent)) {
                ValueBinding vb = TagUtil.getValueBinding(widthContent);
                panelComponent.setValueBinding("widthContent", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("widthContent", widthContent); //$NON-NLS-1$
            }
        }
        if (widthHelp != null) {
            if (isValueReference(widthHelp)) {
                ValueBinding vb = TagUtil.getValueBinding(widthHelp);
                panelComponent.setValueBinding("widthHelp", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("widthHelp", widthHelp); //$NON-NLS-1$
            }
        }
        if (widthLabel != null) {
            if (isValueReference(widthLabel)) {
                ValueBinding vb = TagUtil.getValueBinding(widthLabel);
                panelComponent.setValueBinding("widthLabel", vb); //$NON-NLS-1$
            } else {
                panelComponent.getAttributes().put("widthLabel", widthLabel); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
