/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIProgress;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class ProgressBarTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(ProgressBarTag.class);

    //
    // Instance Variables
    //

    private java.lang.String dir;
    private java.lang.String lang;
    private java.lang.String title;

    private java.lang.String auto;
    private java.lang.String initHidden;
    private java.lang.String initValue;
    private java.lang.String message;
    private java.lang.String outward;
    private java.lang.String proportion;
    private java.lang.String style;
    private java.lang.String styleClass;
    private java.lang.String timeInterval;

    //
    // Setter Methods
    //

    public void setDir(java.lang.String dir) {
        this.dir = dir;
    }

    public void setLang(java.lang.String lang) {
        this.lang = lang;
    }

    public void setTitle(java.lang.String title) {
        this.title = title;
    }

    public void setAuto(java.lang.String auto) {
        this.auto = auto;
    }

    public void setInitHidden(java.lang.String initHidden) {
        this.initHidden = initHidden;
    }

    public void setInitValue(java.lang.String initValue) {
        this.initValue = initValue;
    }

    public void setMessage(java.lang.String message) {
        this.message = message;
    }

    public void setOutward(java.lang.String outward) {
        this.outward = outward;
    }

    public void setProportion(java.lang.String proportion) {
        this.proportion = proportion;
    }

    public void setStyle(java.lang.String style) {
        this.style = style;
    }

    public void setStyleClass(java.lang.String styleClass) {
        this.styleClass = styleClass;
    }

    public void setTimeInterval(java.lang.String timeInterval) {
        this.timeInterval = timeInterval;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.Bar";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlProgressBar"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIProgress progressComponent = (UIProgress)component;

        if (dir != null) {
            if (isValueReference(dir)) {
                ValueBinding vb = TagUtil.getValueBinding(dir);
                progressComponent.setValueBinding("dir", vb); //$NON-NLS-1$
            } else {
                progressComponent.setDir(dir);
            }
        }
        if (lang != null) {
            if (isValueReference(lang)) {
                ValueBinding vb = TagUtil.getValueBinding(lang);
                progressComponent.setValueBinding("lang", vb); //$NON-NLS-1$
            } else {
                progressComponent.setLang(lang);
            }
        }
        if (title != null) {
            if (isValueReference(title)) {
                ValueBinding vb = TagUtil.getValueBinding(title);
                progressComponent.setValueBinding("title", vb); //$NON-NLS-1$
            } else {
                progressComponent.setTitle(title);
            }
        }
        if (auto != null) {
            if (isValueReference(auto)) {
                ValueBinding vb = TagUtil.getValueBinding(auto);
                progressComponent.setValueBinding("auto", vb); //$NON-NLS-1$
            } else {
                boolean _auto = new Boolean(auto).booleanValue();
                progressComponent.getAttributes().put("auto", _auto ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (initHidden != null) {
            if (isValueReference(initHidden)) {
                ValueBinding vb = TagUtil.getValueBinding(initHidden);
                progressComponent.setValueBinding("initHidden", vb); //$NON-NLS-1$
            } else {
                boolean _initHidden = new Boolean(initHidden).booleanValue();
                progressComponent.getAttributes().put("initHidden", _initHidden ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (initValue != null) {
            if (isValueReference(initValue)) {
                ValueBinding vb = TagUtil.getValueBinding(initValue);
                progressComponent.setValueBinding("initValue", vb); //$NON-NLS-1$
            } else {
                progressComponent.getAttributes().put("initValue", initValue); //$NON-NLS-1$
            }
        }
        if (message != null) {
            if (isValueReference(message)) {
                ValueBinding vb = TagUtil.getValueBinding(message);
                progressComponent.setValueBinding("message", vb); //$NON-NLS-1$
            } else {
                progressComponent.getAttributes().put("message", message); //$NON-NLS-1$
            }
        }
        if (outward != null) {
            if (isValueReference(outward)) {
                ValueBinding vb = TagUtil.getValueBinding(outward);
                progressComponent.setValueBinding("outward", vb); //$NON-NLS-1$
            } else {
                boolean _outward = new Boolean(outward).booleanValue();
                progressComponent.getAttributes().put("outward", _outward ? Boolean.TRUE : Boolean.FALSE); //$NON-NLS-1$
            }
        }
        if (proportion != null) {
            if (isValueReference(proportion)) {
                ValueBinding vb = TagUtil.getValueBinding(proportion);
                progressComponent.setValueBinding("proportion", vb); //$NON-NLS-1$
            } else {
                progressComponent.getAttributes().put("proportion", proportion); //$NON-NLS-1$
            }
        }
        if (style != null) {
            if (isValueReference(style)) {
                ValueBinding vb = TagUtil.getValueBinding(style);
                progressComponent.setValueBinding("style", vb); //$NON-NLS-1$
            } else {
                progressComponent.getAttributes().put("style", style); //$NON-NLS-1$
            }
        }
        if (styleClass != null) {
            if (isValueReference(styleClass)) {
                ValueBinding vb = TagUtil.getValueBinding(styleClass);
                progressComponent.setValueBinding("styleClass", vb); //$NON-NLS-1$
            } else {
                progressComponent.getAttributes().put("styleClass", styleClass); //$NON-NLS-1$
            }
        }
        if (timeInterval != null) {
            if (isValueReference(timeInterval)) {
                ValueBinding vb = TagUtil.getValueBinding(timeInterval);
                progressComponent.setValueBinding("timeInterval", vb); //$NON-NLS-1$
            } else {
                progressComponent.getAttributes().put("timeInterval", timeInterval); //$NON-NLS-1$
            }
        }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
