/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
*/

package com.ibm.faces.taglib.html_extended;


import com.ibm.faces.util.StaticMethodBinding;
import com.ibm.faces.util.TagUtil;
import java.io.IOException;
import javax.faces.component.UIComponent;
import com.ibm.faces.component.UIScriptCollector;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.event.ValueChangeEvent;
import javax.faces.convert.Converter;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import javax.faces.webapp.UIComponentTag;
import javax.faces.webapp.UIComponentBodyTag;
import javax.servlet.jsp.JspException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


public class ScriptCollectorTag extends UIComponentTag {


    public static Log log = LogFactory.getLog(ScriptCollectorTag.class);

    //
    // Instance Variables
    //

    private java.lang.String decode;
    private java.lang.String postRender;
    private java.lang.String preRender;


    //
    // Setter Methods
    //

    public void setDecode(java.lang.String decode) {
        this.decode = decode;
    }

    public void setPostRender(java.lang.String postRender) {
        this.postRender = postRender;
    }

    public void setPreRender(java.lang.String preRender) {
        this.preRender = preRender;
    }


    //
    // General Methods
    //

    public String getRendererType() { return "com.ibm.faces.ScriptCollector";  } //$NON-NLS-1$
    public String getComponentType() { return "com.ibm.faces.HtmlScriptCollector"; } //$NON-NLS-1$

    protected void setProperties(UIComponent component) {
        super.setProperties(component);
        UIScriptCollector scriptcollectorComponent = (UIScriptCollector)component;

        if (decode != null) {
            if (isValueReference(decode)) {
                Class args[] = { FacesContext.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(decode, args);
                scriptcollectorComponent.setDecode(vb);
            } else {
              Object params [] = {decode};
              throw new javax.faces.FacesException();
            }
            }
        if (postRender != null) {
            if (isValueReference(postRender)) {
                Class args[] = { FacesContext.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(postRender, args);
                scriptcollectorComponent.setPostRender(vb);
            } else {
              Object params [] = {postRender};
              throw new javax.faces.FacesException();
            }
            }
        if (preRender != null) {
            if (isValueReference(preRender)) {
                Class args[] = { FacesContext.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(preRender, args);
                scriptcollectorComponent.setPreRender(vb);
            } else {
              Object params [] = {preRender};
              throw new javax.faces.FacesException();
            }
            }
    }

    //
    // Methods From TagSupport
    //

    public int doStartTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doStartTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public int doEndTag() throws JspException {
        int rc = 0;
        try {
            rc = super.doEndTag();
        } catch (JspException e) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), e);
            }
            throw e;
        } catch (Throwable t) {
            if (log.isDebugEnabled()) {
                log.debug(getDebugString(), t);
            }
            throw new JspException(t);
        }
        return rc;
    }

    public String getDebugString() {
        String result = "id: "+this.getId()+" class: "+this.getClass().getName(); //$NON-NLS-1$ $NON-NLS-2$
        return result;
    }

}
