/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.util;

import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;


/**
 * @author mark_wallace@ie.ibm.com
 */
public class AjaxUtil {

    public static final String AJAX_VIEWID = "$$viewid"; //$NON-NLS-1$
    public static final String AJAX_MODE   = "$$ajaxmode"; //$NON-NLS-1$
    public static final String AJAX_COMPID = "$$ajaxid"; //$NON-NLS-1$
    
    public static final String VIEW_PREFIX = "view"; //$NON-NLS-1$


    public static final String MODE_NORMAL = "axnone"; //$NON-NLS-1$
    public static final String MODE_AJAX_FULL = "axfull"; //$NON-NLS-1$
    public static final String MODE_AJAX_PARTIAL = "axpartial"; //$NON-NLS-1$
    
    public static final int AJAX_NONE = 0;
    public static final int AJAX_FULL = 1;
    public static final int AJAX_PARTIAL = 2;
    public static final int AJAX_OTHER = 3;
    
    
    private static final String com_ibm_faces_AJAXISRENDERING = "com.ibm.faces.AJAXISRENDERING";
    
    /**
	 * 
	 * @param context
	 * @return
	 */
	public static boolean isAjaxRequest(FacesContext context) {
        return getAjaxMode(context) != AjaxUtil.AJAX_NONE;
	}
	
	/**
	 * 
	 * @param modeStr
	 * @return
	 */
    public static int getAjaxMode(FacesContext context) {
        String modeStr = getAjaxModeValue(context);
    	if (modeStr == null || modeStr.equals(AjaxUtil.MODE_NORMAL)) {
    		return AjaxUtil.AJAX_NONE;
    	}
    	else if(modeStr.equals(AjaxUtil.MODE_AJAX_PARTIAL)) {
    		return AjaxUtil.AJAX_PARTIAL;
    	} 
    	else if(modeStr.equals(AjaxUtil.MODE_AJAX_FULL)) {
    		return AjaxUtil.AJAX_FULL;
    	}
        else if(modeStr.length() > 0 ){
            return AjaxUtil.AJAX_OTHER;
        }
    	return AjaxUtil.AJAX_NONE;
    }
	
    /**
     * @param context
     * @return
     */
    public static String getAjaxModeValue(FacesContext context) {
        Map parameterMap = context.getExternalContext().getRequestParameterMap();
        String modeStr = (String)parameterMap.get(AjaxUtil.AJAX_MODE);
        return modeStr;
    }
    
    /**
     * Return the Ajax conmponent id associated with this FacesContext.
     */
    public static String getAjaxComponentId(FacesContext facesContext) {
    	Object obj = facesContext.getExternalContext().getRequestMap().get(AJAX_COMPID);
    	if(obj==null) {
    		obj = facesContext.getExternalContext().getRequestParameterMap().get(AJAX_COMPID);
    		if(obj==null) {
    			return null;
    		} else {
    			facesContext.getExternalContext().getRequestMap().put(AJAX_COMPID, obj);
    		}
    	}
    	return obj.toString();
    }

    
    

	public static String computeComponentId(FacesContext context, UIComponent component) {
		return component.getClientId(context);
	}
	


	public static boolean isRendering(FacesContext context) {
		Object o = context.getExternalContext().getRequestMap().get(com_ibm_faces_AJAXISRENDERING);
		
		if(o == null)
			return false;
		else
		{
			return ((Boolean)o).booleanValue();
		}		
	}


	public static boolean isRendered(FacesContext context, UIComponent component) {
		String  _componentId = getAjaxComponentId(context);
        if (_componentId != null) {
            String ajaxComponentId = computeComponentId(context, component);
            if (_componentId.equals(ajaxComponentId)) {
                return true;
            }
        }
        return false;
	}


	public static void setRendering(FacesContext context, boolean rendering) {
		context.getExternalContext().getRequestMap().put(com_ibm_faces_AJAXISRENDERING, Boolean.valueOf(rendering));
		
	}
    
    
    
    
    
    
    
}
