/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2004 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.util;

/*
 * Base64.java v1.0 - Base64 Encoder/Decoder
 *
 * Licensed Materials - Property of IBM
 * Project Piri Reis - com.ibm.piri.util.Base64
 *
 * B. Bora Bali
 * Senior I/T Specialist
 * balibora@us.ibm.com
 *
 */

/**
 * <code>Base64</code> consists of static methods for encoding and decoding
 * data in Base64.
 *
 * @version 1.0 - Created on January 27, 2004.
 * @author B. Bora Bali<br />Senior I/T Specialist<br />balibora@us.ibm.com
 *
 */
public class Base64 {
    /**
     * Base64-decodes the specified text.
     *
     * @param base64Text a <code>String</code> to be decoded.
     * @return a byte array that represents the Base64 decoded text.
     *
     */
    public static byte[] decode(String base64Text) {
        char[] inbutBuffer = new char[4];
        byte[] outputBuffer = new byte[base64Text.length() / 4 * 3 + 3];

        int inputBufferCnt = 0;
        int outputBufferCnt = 0;

        for (int i = 0;  i < base64Text.length();  i ++) {
            char ch = base64Text.charAt(i);

            if (ch == base64Pad
                    || ch < decodeTable.length
                    && decodeTable[ch] != Byte.MAX_VALUE) {

                inbutBuffer[inputBufferCnt++] = ch;

                if (inputBufferCnt == inbutBuffer.length) {
                    inputBufferCnt = 0;
                    outputBufferCnt +=
                        decode(inbutBuffer, outputBuffer, outputBufferCnt);
                }
            }
        }

        if (outputBufferCnt == outputBuffer.length) {
            return outputBuffer;
        }

        byte[] returnBuffer = new byte[outputBufferCnt];

        System.arraycopy(outputBuffer, 0, returnBuffer, 0, outputBufferCnt);

        return returnBuffer;
    }


    /**
     * Base64-encodes the specified byte array.
     *
     * @param clearText a byte array to be encoded.
     * @return a <code>String</code> that represents the encoded text.
     *
     */
    public static String encode(byte[] clearText) {
        return encode(clearText, 0, clearText.length);
    }


    /**
     * Base64-encodes the specified byte array.
     *
     * @param clearText a byte array to be encoded.
     * @param offset an integer that represents the offset within <code>
     * clearText</code> o start encoding.
     * @param len an integer that represents the the length of the text
     * to encode.
     *
     * @return a byte array that represents the encoded text.
     *
     */
    private static String encode(byte[] clearText, int offset, int len) {
        if (len <= 0) {
            return "";
        }

        char[] out = new char[len / 3 * 4 + 4];
        int rNdx = offset;
        int wNdx = 0;
        int rest = len;

        while (rest >= 3) {
            int i = ((clearText[rNdx] & 0xff) << 16) +
                    ((clearText[rNdx + 1] & 0xff) << 8) +
                    (clearText[rNdx + 2] & 0xff);

            out[wNdx++] = base64Alphabet[i >> 18];
            out[wNdx++] = base64Alphabet[(i >> 12) & 0x3f];
            out[wNdx++] = base64Alphabet[(i >> 6) & 0x3f];
            out[wNdx++] = base64Alphabet[i & 0x3f];

            rNdx += 3;
            rest -= 3;
        }

        if (rest == 1) {
            int i = clearText[rNdx]&0xff;
            out[wNdx++] = base64Alphabet[i >> 2];
            out[wNdx++] = base64Alphabet[(i << 4) & 0x3f];
            out[wNdx++] = base64Pad;
            out[wNdx++] = base64Pad;
        } else if (rest == 2) {
            int i = ((clearText[rNdx] & 0xff) << 8) +
                    (clearText[rNdx + 1] & 0xff);

            out[wNdx++] = base64Alphabet[i >> 10];
            out[wNdx++] = base64Alphabet[(i >> 4) & 0x3f];
            out[wNdx++] = base64Alphabet[(i << 2) & 0x3f];
            out[wNdx++] = base64Pad;
        }

        return new String(out, 0, wNdx);
    }


    /**
     * Base64-decodes the specified character array.
     *
     * @param clearText a byte array to be encoded.
     * @param offset an integer that represents the offset within <code>
     * clearText</code> o start encoding.
     * @param len an integer that represents the the length of the text
     * to encode.
     *
     * @return a byte array that represents the encoded text.
     *
     */
    private static int decode(char[] base64Text, byte[] outputBuffer, int ptr) {
        int outlen = 3;

        if (base64Text[3] == base64Pad) {
            outlen = 2;
        }

        if (base64Text[2] == base64Pad) {
            outlen = 1;
        }

        int b0 = decodeTable[base64Text[0]];
        int b1 = decodeTable[base64Text[1]];
        int b2 = decodeTable[base64Text[2]];
        int b3 = decodeTable[base64Text[3]];

        switch (outlen) {
            case 1:
                outputBuffer[ptr] = (byte)(b0 << 2 & 0xfc | b1 >> 4 & 0x3);
                return 1;
            case 2:
                outputBuffer[ptr++] = (byte)(b0 << 2 & 0xfc | b1 >> 4 & 0x3);
                outputBuffer[ptr] = (byte)(b1 << 4 & 0xf0 | b2 >> 2 & 0xf);
                return 2;
            case 3:
                outputBuffer[ptr++] = (byte)(b0 << 2 & 0xfc | b1 >> 4 & 0x3);
                outputBuffer[ptr++] = (byte)(b1 << 4 & 0xf0 | b2 >> 2 & 0xf);
                outputBuffer[ptr] = (byte)(b2 << 6 & 0xc0 | b3 & 0x3f);
                return 3;
            default:
                throw new RuntimeException("Unable to Base64-decode");
        }
    }

    // Base64 alphabet..
    private static final char[] base64Alphabet = {
            'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J',
            'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
            'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd',
            'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
            'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x',
            'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7',
            '8', '9', '+', '/'
    };

    private static final char base64Pad = '=';

    // Construct the decoding table..
    private static final byte[] decodeTable = new byte[128];

    static {
        for (int i = 0; i < decodeTable.length; i++) {
            decodeTable[i] = Byte.MAX_VALUE;
        }

        for (int i = 0; i < base64Alphabet.length; i++) {
            decodeTable[base64Alphabet[i]] = (byte)i;
        }
    }
}

