/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


package com.ibm.faces.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;


/**
 * JSF utility methods.
 * 
 * @author amcdowell
 */
public class LifecycleUtil {

	public static final String THIS_APP_CONTEXT_PATH = "com.ibm.faces.THIS_APP_CONTEXT_PATH"; //$NON-NLS-1$
	//private static String contextPath;
	
    public static final String THIS_APP_SERVLET_URL_PATTERN= "com.ibm.faces.THIS_APP_SERVLET_URL_PATTERN";
	//private static String jsResourceURLPattern;
	//private static String contextPathWithURLPattern;

    /**
     * Gets the given request parameter from the external context.
     * @param context
     * @param requestParamName
     * @return
     */
    public static String getRequestParam(
        FacesContext context,
        String requestParamName) {
        Map requestMap = context.getExternalContext().getRequestParameterMap();
        return (String) requestMap.get(requestParamName);
    }

    /**
     * Gets instances of UIParameter from a component.
     * @param c
     * @return any children of type UIParameter
     */
    public static UIParameter[] getParameters(UIComponent c) {
        List list = new ArrayList();
        Iterator children = c.getChildren().iterator();
        while (children.hasNext()) {
            Object child = children.next();
            if (child instanceof UIParameter) {
                list.add(child);
            }
        }
        UIParameter[] ret = new UIParameter[list.size()];
        list.toArray(ret);
        return ret;
    }
    
	//RATLC00285398 addition
    /**
     * Get context path using com.ibm.faces.USE_UNENCODED_CONTEXT_PATH context-param if it exists, if 
     * not return context.getExternalContext().getRequestContextPath()
     * @param context
     * @return
     */
    public static String getContextPath(FacesContext context) {
    	String contextPath = (String)context.getExternalContext().getApplicationMap().get(THIS_APP_CONTEXT_PATH); 
    	if(contextPath == null) {
			String unencodedSetting = context.getExternalContext().getInitParameter(ContextParams.USE_UNENCODED_CONTEXT_PATH);
			if(unencodedSetting == null || unencodedSetting.trim().length() == 0){
				contextPath = context.getExternalContext().getRequestContextPath();
			} else {
				unencodedSetting = unencodedSetting.trim();
				//need to check if the setting is a context path or number
				Boolean isNumber = Boolean.TRUE;
				int len = unencodedSetting.length();
				for(int i = 0; i < len; i++) {
					if(!Character.isDigit(unencodedSetting.charAt(i))){
						isNumber = Boolean.FALSE;
						break;
					}
				}
				if(isNumber.booleanValue()) {
					//is an integer setting
					String path = context.getExternalContext().getRequestContextPath();
					StringBuffer s = new StringBuffer();
					if(path.lastIndexOf("/") > 0) {
						int nodes = Integer.parseInt(unencodedSetting);
						if(nodes < 1) nodes = 1;
						int pos1 = 0;
						int pos2 = 1;
						int pathLength = path.length();
						for(int x = 1; x <= nodes; x++){
							pos2 = path.indexOf("/",pos2);
							if(pos2 == -1) pos2 = pathLength; 
							s.append(path.substring(pos1,pos2));
							if (pos2 != pathLength) {
								pos1 = pos2;
								++pos2;	
							} else {
								break;
							}
						}
						contextPath = s.toString();
				 	} else {
				 		contextPath = path;
					}
				} else {
					//is a path setting
					contextPath = unencodedSetting;
				}
				
			}
			context.getExternalContext().getApplicationMap().put(THIS_APP_CONTEXT_PATH, contextPath);
    	}
		return contextPath;
    }
    
    //RATLC01083006
    public static String getJSResourceURLPattern(FacesContext context){
    	String resPattern = (String)context.getExternalContext().getApplicationMap().get(THIS_APP_SERVLET_URL_PATTERN); 
    	if(resPattern == null){
    		String patternParam = context.getExternalContext().getInitParameter(ContextParams.JS_RESOURCE_SERVLET_URL_PATTERN);
    		if(patternParam != null && !patternParam.equals("")){
    			resPattern = patternParam;
    			//assuming pattern is like /name/* or /other/name/*
    			if(resPattern.startsWith("/"))
    				resPattern = resPattern.substring(1,resPattern.length());
    			if(resPattern.endsWith("/*"))
    				resPattern = resPattern.substring(0,resPattern.length() - 2);
    			if(resPattern.endsWith("/"))
    				resPattern = resPattern.substring(0,resPattern.length() - 1);
    		} else {
    			resPattern = ".ibmjsfres";
    		}
    		context.getExternalContext().getApplicationMap().put(THIS_APP_SERVLET_URL_PATTERN,resPattern);
    	}
    	return resPattern;
    }
    
    public static String getJSResourceURLPattern(String patternParam){
    	String resPattern = null;
   		if(patternParam != null && !patternParam.equals("")){
   			resPattern = patternParam;
   			//assuming pattern is like /name/* or /other/name/*
   			if(resPattern.startsWith("/"))
   				resPattern = resPattern.substring(1,resPattern.length());
   			if(resPattern.endsWith("/*"))
   				resPattern = resPattern.substring(0,resPattern.length() - 2);
   			if(resPattern.endsWith("/"))
   				resPattern = resPattern.substring(0,resPattern.length() - 1);
   		} else {
   			resPattern = ".ibmjsfres";
   		}
    	return resPattern;
    }
    
    public static String getResourceServletContextPath(FacesContext context){
    	StringBuffer sb = new StringBuffer();
    	sb.append(getContextPath(context))
		.append("/")
		.append(getJSResourceURLPattern(context));
    	return sb.toString();
    }

}
