/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */


package com.ibm.faces.util;

import javax.faces.context.FacesContext;

import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
/**
 * 
 * @author ldh
 * @version
 */
public class StringUtil extends Object
{
   protected static int getXMLReservedCharIndex(char c)
   {
      for (int i = 0; i < XMLReservedChars.length; i++)
         if (XMLReservedChars[i] == c)
            return i;
      return -1;
   }

   // Adds double quotes around the string, and escapes '\' and '"' and other
   // XML reserved characters with a '\'
   public static final String QuoteDoubleAndEscapeForXML(String s,
         boolean nestedEscape)
   {
      return StringUtil.QuoteDoubleAndEscapeForXML(s, nestedEscape, false);
   }

   // Adds double quotes around the string, and escapes '\' and '"' and other
   // XML reserved characters with a '\'
   public static final String QuoteDoubleAndEscapeForXML(String S,
         boolean NestedEscape, boolean xmlOutput)
   {
      String EnclosingSequence = (NestedEscape == true) ? "\\\"" : "\"";

      if (S == null)
         return EnclosingSequence + EnclosingSequence;

      StringBuffer XXX = new StringBuffer(128);
      XXX.append(EnclosingSequence);

      int len = S.length();
      char[] s = S.toCharArray();

      int i = 0;
      int j = 0;
      while (j < len)
      {
         int index = -1;
         while (j < len && index == -1)
         {
            char c = s[j++];
            index = getXMLReservedCharIndex(c);
            if (index != -1)
            {
               //no need to escape "\" if the output goes into an xml file
               if (c == '\\' && xmlOutput)
               {
                  index = -1;
               } else
               {
                  j--;
               }
            }
         }

         XXX.append(s, i, j - i);

         if (j == len)
            break;

         if (index >= 0)
         {
            XXX.append(escapeEntities[index]);
         }

         ++j;
         i = j;
      }
      return XXX.append(EnclosingSequence).toString();
   }

   //public static final String COPYRIGHT = com.ibm.odcb.utilities.IBMCopyright.SHORT_COPYRIGHT;

   static final String[]      _PADS     = {
         "",
         " ",
         "  ",
         "   ",
         "    ",
         "     ",
         "      ",
         "       ",
         "        ",
         "         ",
         "          ",
         "           ",
         "            ",
         "             ",
         "              ",
         "               ",
         "                ",
         "                 ",
         "                  ",
         "                   ",
         "                    ",
         "                     ",
         "                      ",
         "                       ",
         "                        ",
         "                         ",
         "                          ",
         "                           ",
         "                            ",
         "                             ",
         "                              ",
         "                               ",
         "                                ",
         "                                 ",
         "                                  ",
         "                                   ",
         "                                    ",
         "                                     ",
         "                                      ",
         "                                       ",
         "                                        ",
         "                                         ",
         "                                          ",
         "                                           ",
         "                                            ",
         "                                             ",
         "                                              ",
         "                                               ",
         "                                                ",
         "                                                 ",
         "                                                  ",
         "                                                   ",
         "                                                    ",
         "                                                     ",
         "                                                      ",
         "                                                       ",
         "                                                        ",
         "                                                         ",
         "                                                          ",
         "                                                           ",
         "                                                            ",
         "                                                             ",
         "                                                              ",
         "                                                               ",
         "                                                                ",
         "                                                                 ",
         "                                                                  ",
         "                                                                   ",
         "                                                                    ",
         "                                                                     ",
         "                                                                      ",
         "                                                                       ",
         "                                                                        ",
         "                                                                         ",
         "                                                                          ",
         "                                                                           ",
         "                                                                            ",
         "                                                                             ",
         "                                                                              ", };

   public static final String getPad(int Padding)
   {
      if (Padding < _PADS.length)
         return _PADS[Padding];

      StringBuffer XXX = new StringBuffer(Padding + 1);
      XXX.append(_PADS[_PADS.length - 1]);
      for (int i = _PADS.length - 1; i < Padding; ++i)
      {
         XXX.append(" ");
      }
      return XXX.toString();
   }

   //Method to only trim while spaces at the end of a string.
   public static final String trailTrim(String aStr)
   {
      if (aStr == null || aStr.equals(""))
      {
         return aStr;
      }
      String newStr = null;
      int i = 0;
      //loop the string from the end to find the first non-white space character
      for (i = aStr.length() - 1; i > 0; i--)
      {
         //If we hit any non-white space, break out the loop
         //Here we follow String.trim() using \u0020 as a criteria
         if (aStr.charAt(i) > '\u0020' )
         {
            break;
         }
      }
      //The last character is a non-white space char, no substring
      if (i == aStr.length() - 1)
      {
         newStr = aStr;
      } 
      else
      {
         newStr = aStr.substring(0, i+1);
      }
      return newStr;
   }

   // Adds properly escaped (if needed) double quotes around the character
   public static final String QuoteDoubleAndEscape(char c, boolean NestedEscape)
   {
      return (NestedEscape == true) ? "\\\"" + c + "\\\"" : "\"" + c + "\"";
   }
   
   // Adds properly escaped (if needed) double quotes around the character
   public static final String QuoteDoubleAndEscape(char c, boolean NestedEscape, boolean nullIsNull)
   {
      return (NestedEscape == true) ? "\\\"" + c + "\\\"" : "\"" + c + "\"";
   }

   // Adds properly escaped (if needed) double quotes around the character
   public static final String QuoteDoubleAndEscape(Character c,
         boolean NestedEscape)
   {
      return (NestedEscape == true) ? "\\\"" + c.charValue() + "\\\"" : "\""
            + c.charValue() + "\"";
   }
   
   // Adds properly escaped (if needed) double quotes around the character
   public static final String QuoteDoubleAndEscape(Character c,
         boolean NestedEscape, boolean nullIsNull)
   {
      return (NestedEscape == true) ? "\\\"" + c.charValue() + "\\\"" : "\""
            + c.charValue() + "\"";
   }

   // Adds properly escaped (if needed) double quotes around the string, and
   // escapes '\' and '"' characters with a '\'
   public static final String QuoteDoubleAndEscape(Object S,
         boolean NestedEscape)
   {
      if (S == null)
         return QuoteDoubleAndEscape("null", NestedEscape);
      return QuoteDoubleAndEscape(S.toString(), NestedEscape);
   }

   // Adds properly escaped (if needed) double quotes around the string, and
   // escapes '\' and '"' characters with a '\'
   public static final String QuoteDoubleAndEscapeForXML(Object S,
         boolean NestedEscape)
   {
      if (S == null)
         return QuoteDoubleAndEscapeForXML("null", NestedEscape);
      return QuoteDoubleAndEscapeForXML(S.toString(), NestedEscape);
   }
   
   // Adds properly escaped (if needed) double quotes around the string, and
   // escapes '\' and '"' characters with a '\'
   public static final String QuoteDoubleAndEscape(Object S,
         boolean NestedEscape, boolean nullIsNull)
   {
      if (S == null)
         return QuoteDoubleAndEscape("null", NestedEscape);
      return QuoteDoubleAndEscape(S.toString(), NestedEscape, nullIsNull);
   }

   // Adds properly escaped (if needed) double quotes around the string, and
   // escapes '\' and '"' characters with a '\'
   public static final String QuoteDoubleAndEscapeForXML(Object S,
         boolean NestedEscape, boolean nullIsNull)
   {
      if (S == null)
         return QuoteDoubleAndEscapeForXML("null", NestedEscape);
      return QuoteDoubleAndEscapeForXML(S.toString(), NestedEscape, nullIsNull);
   }

   // Adds properly escaped (if needed) double quotes around the string, and
   // escapes '\' and '"' characters with a '\'
   public static final String QuoteDoubleAndEscape(String S,
         boolean NestedEscape, boolean nullIsNull)
   {
      String EscapeSequence = (NestedEscape == true) ? "\\\\\\\"" : "\\\"";
      String EnclosingSequence = (NestedEscape == true) ? "\\\"" : "\"";
      String escapeSlash = (NestedEscape == true) ? "\\\\\\\\" : "\\\\";

      if (S == null)
         return nullIsNull ? null : EnclosingSequence + EnclosingSequence;

      StringBuffer XXX = new StringBuffer(128);
      XXX.append(EnclosingSequence);

      int len = S.length();
      char[] s = S.toCharArray();

      int i = 0;
      int j = 0;
      while (j < len)
      {
         while (j < len && s[j] != '\"' && s[j] != '\\')
            ++j;
         XXX.append(s, i, j - i);
         if (j == len)
            break;
         XXX.append(s[j] == '\"' ? EscapeSequence : escapeSlash);
         ++j;
         i = j;
      }
      // Donogh: Fix for RATLC01085787 (String data containing 
      // new line characters cause client data not to display).
      String result = XXX.append(EnclosingSequence).toString();
      
      if (result.indexOf('\r') != -1 )
      	result = result.replaceAll("\r", "\\\\r");
      if (result.indexOf('\n') != -1 )
      	result = result.replaceAll("\n", "\\\\n");
      return result;
   }

   public static final String QuoteDoubleAndEscape(String S,
         boolean NestedEscape)
   {
      return QuoteDoubleAndEscape(S, NestedEscape, false);
   }

   // Adds double quotes around the string, and escapes '\' and '"' and other
   // XML reserved characters with a '\'
   public static final String QuoteDoubleAndEscapeForXML(char c,
         boolean NestedEscape)
   {
      return (NestedEscape == true) ? "\\\"" + c + "\\\"" : "\"" + c + "\"";
   }
   
   // Adds double quotes around the string, and escapes '\' and '"' and other
   // XML reserved characters with a '\'
   public static final String QuoteDoubleAndEscapeForXML(char c,
         boolean NestedEscape, boolean nullIsNull)
   {
      return (NestedEscape == true) ? "\\\"" + c + "\\\"" : "\"" + c + "\"";
   }

   // Adds double quotes around the string, and escapes '\' and '"' and other
   // XML reserved characters with a '\'
   public static final String QuoteDoubleAndEscapeForXML(Character c,
         boolean NestedEscape)
   {
      return (NestedEscape == true) ? "\\\"" + c.charValue() + "\\\"" : "\""
            + c.charValue() + "\"";
   }
   
   // Adds double quotes around the string, and escapes '\' and '"' and other
   // XML reserved characters with a '\'
   public static final String QuoteDoubleAndEscapeForXML(Character c,
         boolean NestedEscape, boolean nullIsNull)
   {
      return (NestedEscape == true) ? "\\\"" + c.charValue() + "\\\"" : "\""
            + c.charValue() + "\"";
   }

   protected static char[]   XMLReservedChars    = { '\"', '&', '<', '>', '\\' };

   protected static String[] escapeEntities      = { "&quot;", "&amp;", "&lt;",
         "&gt;", "\\\\"                         };
   
   //These are proprietary encoding to get around the portal default data encoding
   //In portal, when we submit a diff with <> for XML elements, such as   <Current>, 
   //it will be encoded as &lt;Current&gt;.  This will cause a problem of XMl parsing on server side.
   protected static String[] escapeEntitiesForPortalDiff      = { "&lt;'", "&gt;'"};
   protected static char[]   diffXMLReservedChars    = {'<', '>'};
   protected static int[]    diffEscapeCharSkipIndex = { 5, 5};

   protected static int[]    escapeCharSkipIndex = { 6, 5, 4, 4, 2 };

   // removes escaping for XML reserved characters so that the string is a valid
   // JS string
   public static final String convertXMLEscapingtoJS(String s)
   {
      if (s != null)
      {
         String s1 = s;
         //the last escaping in escapeEntities "\\\\" is used for JS output
         // only.
         //when converting xml output to js, we should not use it
         for (int i = 0; i < escapeEntities.length - 1; i++)
         {
            int index = s.indexOf(escapeEntities[i]);
            while (index != -1)
            {
               s1 = s.substring(0, index);
               s1 += XMLReservedChars[i];
               s1 += s.substring(index + escapeCharSkipIndex[i]);
               s = s1;
               index = s.indexOf(escapeEntities[i]);
            }
         }
         return s1;
      }
      return null;
   }
   
   //These are proprietary encoding to get around the portal default data encoding
   //In portal, when we submit a diff with <> for XML elements, such as   <Current>, 
   //it will be encoded as &lt;Current&gt;.  This will cause a problem of XMl parsing on server side.
   //So the solution to this problem is that we replace all < and > in a diff with "&lt;'" and "&gt;'" before
   //the diff is submitted back to server.  On server side, we restore it  by reserving the eoncoding.
   //This encoding is simply XML entities plus a '.
   public static final String restoreEncodedDiffToValidXML(String s)
   {
      if (s != null)
      {
         String s1 = s;
         //the last escaping in escapeEntities "\\\\" is used for JS output
         // only.
         //when converting xml output to js, we should not use it
         for (int i = 0; i < escapeEntitiesForPortalDiff.length; i++)
         {
            int index = s.indexOf(escapeEntitiesForPortalDiff[i]);
            while (index != -1)
            {
               s1 = s.substring(0, index);
               s1 += diffXMLReservedChars[i];
               s1 += s.substring(index + diffEscapeCharSkipIndex[i]);
               s = s1;
               index = s.indexOf(escapeEntitiesForPortalDiff[i]);
            }
         }
         return s1;
      }
      return null;
   }
   
//Implements functionality of split method in Java 1.4 can't use this method because it doesn't 
  //work with project run on WAS 5.0 server
   public static final String[] splitToArray(String splitList,char delimiter)
   {
	   	int delimCount=0,placeHolder=0,delimPos=0;;
	   	
	   	//find length of array
	   	for(int count=0;count<splitList.length();count++)
		{
	   		
	   			if(splitList.charAt(count)==delimiter)
	   				delimCount++;
		
				
		}
	   	int numElems=delimCount+1;
	   	
	   	//create array to contain the individual elements
	   	String[] splitArray=new String[numElems];
	   	
	   	//split string into individual strings in the array using delimiter provided
	   	for(int i=0; i<splitList.length();i++)
		{
	   		delimPos=splitList.indexOf(delimiter,placeHolder);
	   		if(delimPos==-1)
	   			delimPos=splitList.length();
	   		splitArray[i]=splitList.substring(placeHolder,delimPos);
			placeHolder=delimPos+1;
			if(placeHolder>=splitList.length())
				break;
		
		}
	
   	return splitArray;
   }
   
   //Encode string (only if it has a faces context)
   public static final String encodeString(String string,boolean isFaces)
   {
   	String localString;
   		
   	if(string==null)
   	   		return "";
   	   	else
   	   	{
   	   		localString="";
   	    		if(isFaces==true)
   	    		{
   	    			//Needed for encoding variables
   	    			FacesContext facesContext = FacesContext.getCurrentInstance();
   	    			//check to see if this is a portal project and if so encode the string using the viewId
   	    			//from the application map. 
   	    			boolean inPortal = HxClientRenderUtil.isPortal(facesContext.getExternalContext().getRequest());
   	    			if(inPortal)
   	    			{
   	    				String viewId = facesContext.getExternalContext().getApplicationMap().get("PortalEncoding").toString();
   	    		
   	    				//Encode variable for portal
   	    				localString= viewId + string;//facesContext.getExternalContext().encodeNamespace(string);
   	    			}
   	    			else
   	    			{
   	    				localString=string;
   	    			}
   	    		}
   	    		else
   	    			localString=string;
   	    	
   	    	return localString;
   			
   	   	}

   }
   
  
   //Encode resource URL (only if it has a faces context)
   public static final String encodeResourceURL(String aURL, boolean isFaces)
	{
   	if(aURL!=null)
   	{
	    	if(isFaces==true)
			{
	    		FacesContext facesContext = FacesContext.getCurrentInstance();
	    		facesContext.getExternalContext().encodeResourceURL(aURL);
			}
	   	}
   	return aURL;
	}
	
   //Encode actionURL (only if it has a faces context)
	public String encodeActionURL(String aURL, boolean isFaces)
	{
		if(aURL!=null)
   	{
	    	if(isFaces==true)
			{
	    		FacesContext facesContext = FacesContext.getCurrentInstance();
	    		facesContext.getExternalContext().encodeActionURL(aURL);
			}
   	}
		return aURL;
	}

   //Determine whether page has a faces context. If it does encoding should occur.
   public static final boolean checkEncoding()
   {
   	
   	FacesContext context= FacesContext.getCurrentInstance();
   	boolean isFaces=true;
   	if(context==null)
   		isFaces=false;
   	else 
   		isFaces = true;
   
   	return isFaces;
   }

   public static void main(String[] args)
   {
	  System.out.println();
	  System.out.println("----For JS----");
	  System.out.println();
	  System.out.println(QuoteDoubleAndEscape("This is \"toto\" at \\1\\.",
	        false));
	  System.out
	        .println("println(\""
	          + QuoteDoubleAndEscape("This is \"toto\" at \\1\\.", true)
	          + "\");");
	  System.out.println("\"This is \\\"toto\\\" at \\1\\.\"");
	  System.out.println("\"This is &quot;toto&quot; at \\1\\\"");
	  System.out.println("println(\""
	    + QuoteDoubleAndEscape(
	          ">This < is \"toto\" at \\1\\. And & Me > You", true)
	    + "\");");
	  System.out.println(QuoteDoubleAndEscape("", true));
	  System.out.println(QuoteDoubleAndEscape("", false));
	  System.out.println(QuoteDoubleAndEscape(
	        "This string has no escape chars", true));
	  System.out.println(QuoteDoubleAndEscape(
	        "This string has no escape chars", false));
	  System.out.println(QuoteDoubleAndEscape(
	        "&This string has two escape chars\"", true));
	  System.out.println(QuoteDoubleAndEscape(
	        "<This string has two escape chars>", false));
	  System.out.println();
	  System.out.println("----For XML----");
	  System.out.println();
	  System.out.println(QuoteDoubleAndEscapeForXML(
	        "This is \"toto\" at \\1\\.", false));
	  System.out.println("println(\""
	    + QuoteDoubleAndEscapeForXML("This is \"toto\" at \\1\\.", true)
	    + "\");");
	  System.out.println("\"This is \\\"toto\\\" at \\1\\.\"");
	  System.out.println("\"This is &quot;toto&quot; at \\1\\\"");
	  System.out.println("println(\""
	    + QuoteDoubleAndEscapeForXML(
	          ">This < is \"toto\" at \\1\\. And & Me > You", true)
	    + "\");");
	  System.out.println(QuoteDoubleAndEscapeForXML("", true));
	  System.out.println(QuoteDoubleAndEscapeForXML("", false));
	  System.out.println(QuoteDoubleAndEscapeForXML(
	        "This string has no escape chars", true));
	  System.out.println(QuoteDoubleAndEscapeForXML(
	        "This string has no escape chars", false));
	  System.out.println(QuoteDoubleAndEscapeForXML(
	        "&This string has two escape chars\"", true));
	  System.out.println(QuoteDoubleAndEscapeForXML(
	        "<This string has two escape chars>", false));
	  
	  String sample = "   my Sample   ";
	  System.out.println("Before trainTrim():"+ sample+ "|after tailTrim():" + trailTrim(sample)+"|");
	  
	  sample = "&lt;'Current&gt;'&lt;'root_Root  xmi:id=\"_2\"  ODCid=\"_2\"  name=\"All Users\"  stocks=\"_4\"&gt;'";
	  System.out.println("Before restoreEncodedDiffToValidXML():"+ sample+ "|after restoreEncodedDiffToValidXML():" + restoreEncodedDiffToValidXML(sample)+"|");
	  
	  //Test for splitToArray
	  String splitlist="word,poo,doo,bee,doo";
	  String[] result=splitToArray(splitlist, ',');
	  for (int i=0;i<result.length;i++)
		  System.out.println("string "+i+" is "+result[i]);
   }
   
   public static String removeTrailingDecimalZeros(String s)
   {
   	if (s != null)
   	{
   		int index = s.indexOf(".");
   		if (index == -1)
   			return s;
   		String decimal = s.substring(index);
   		decimal = decimal.trim();
   		while (decimal.endsWith("0"))
   			decimal = decimal.substring(0,decimal.length()-1);
   		if (decimal.equals("."))
   			s=s.substring(0,index);
   		else
   			s=s.substring(0,index)+decimal;
   	}
   	return s;
   }
   
   /** 
    * replaces all ocurrances of searchstr in orgstr with replacement
    * if searchstr does not occur in orgstr , orgstr is returned
    * 
    * @param String orgstr - the string you wish to do the replacement on.
    * @param String searchstr - the substring you wish to search for and replace.
    * @param String replacement - the string to replace searchstr with. 
    * 
    * @return String - a string with the ocurrances of searchstr (if any) in orgstr replaced with replacement 
    * 
    * */   
   public static String replaceAll(String orgstr, String searchstr, String replacement)
   {
    
		StringBuffer altered = new StringBuffer();
		int si = 0;
		int ei = 0;
		int regl = searchstr.length();
		boolean notfinished = true;
		
		si = orgstr.indexOf(searchstr);
		if(si == -1) // if there is nothing to replace just return string
		{
			altered.append(orgstr); 
		}
		else
		{	
			altered.append(orgstr.substring(0, si)); 
			altered.append(replacement);
			si = si + regl; // move index foward
			
	    	while( si < orgstr.length() && notfinished)
	    	{
	    		ei = orgstr.indexOf(searchstr, si);
	    		
	    		if(ei == -1)
	    		{
	    			altered.append(orgstr.substring(si));
	    			notfinished = false;
	    		}
	    		else
	    		{
	    			altered.append(orgstr.substring(si,ei));
	    			altered.append(replacement);
	    			si = ei + regl;
	    		}	// end if    		
	    	} // end while
		}// end if(si == -1)
   
		return altered.toString();
   }
	
   
   public static boolean isEmpty(String s) {
		return s==null || s.length()==0;
   }
}
   