/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.util;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.faces.webapp.UIComponentTag;

/**
 * Utility class for custom tags.
 */
public class TagUtil {

	/**
	 * Private singleton implementation so we can use isValueReference(String)
	 */
	private static final UtilityUIComponentTag UTILITY_TAG =
		new UtilityUIComponentTag();

	/**
	 * If <code>value</code> is a value reference as determined by
	 * {@link javax.faces.webapp.UIComponentTag}.isValueReference(String value)
	 * then the {@link javax.faces.el.ValueBinding} is set on the component;
	 * otherwise the value is set as as an attribute.
	 * @param component     a UIComponent
	 * @param key           the property key
	 * @param value         the property value
	 */
	public final static void overrideProperty(
		UIComponent component,
		String key,
		Object value) {
		if (value == null) {
			return;
		}
		if ((value instanceof String)
			&& (UTILITY_TAG.isValueRef(value.toString()))) {
			// TODO: PFD - getValueBinding is now createValueBinding
			ValueBinding vb =
				FacesContext
					.getCurrentInstance()
					.getApplication()
					.createValueBinding(
					value.toString());
			component.setValueBinding(key, vb);
		} else {
			component.getAttributes().put(key, value);
		}
	}
	
	public static Object getValueBindingValue(String exp){
		if(exp == null || exp.length() < 1 || !UIComponentTag.isValueReference(exp)){
			return exp;
		}
		return getValueBinding(exp).getValue(FacesContext.getCurrentInstance());
	}
	
	public static ValueBinding getValueBinding(String exp){
		FacesContext context = FacesContext.getCurrentInstance();
		return context.getApplication().createValueBinding(exp);
	}

	/**
	 * See UTILITY_TAG above
	 */
	private static class UtilityUIComponentTag extends UIComponentTag {
		public String getRendererType() {
			return null;
		}
		public String getComponentType() {
			return null;
		}
		boolean isValueRef(String value) {
			return UIComponentTag.isValueReference(value);
		}
	}

}
