/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2004 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.validator;

import java.util.ResourceBundle;
import java.util.regex.PatternSyntaxException;

import javax.faces.application.FacesMessage;
import javax.faces.component.StateHolder;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

import com.ibm.faces.util.InputAssistNames;
import com.ibm.faces.util.FacesMessageUtil;
import com.ibm.faces.ResourceHandler;

public class ConstraintValidator implements Validator, StateHolder {
	private String regex;
	private ResourceBundle rb;

	public ConstraintValidator() {
		super();
	}

	public ConstraintValidator(String regex) {
		super();
		setRegex(regex);
	}

	/* (non-Javadoc)
	 * @see javax.faces.validator.Validator#validate(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.Object)
	 */
	public void validate(FacesContext context, UIComponent component, Object value) throws ValidatorException {

		if (context == null || component == null) {
			throw new NullPointerException(ResourceHandler.getString(rb,"ConstraintValidator.FacesContext_or_UIComponent_is_null._5")); //$NON-NLS-1$
		}

		rb = ResourceHandler.getBundle(context);

		String convertedValue = ""; //$NON-NLS-1$

		if (value != null) {
			if (value instanceof String) {
				convertedValue = (String) value;
			} else {
				convertedValue = value.toString();
			}

			if (regex.equals(InputAssistNames.ATTR_VALUE_ALPHABETONLY)) {
				validateAlphabetOnly(convertedValue, context, (UIInput) component);
			} else if (regex.equals(InputAssistNames.ATTR_VALUE_DIGITONLY)) {
				validateDigitOnly(convertedValue, context, (UIInput) component);
			} else if (regex.equals(InputAssistNames.ATTR_VALUE_ALNUMONLY)) {
				validateAlnumOnly(convertedValue, context, (UIInput) component);
			} else {
				validateRegEx(convertedValue, context, (UIInput) component);
			}
		}
	}

	protected void validateAlphabetOnly(String convertedValue, FacesContext context, UIInput component) {
		char[] checkString = convertedValue.toCharArray();
		boolean isValid = true;
		int length = checkString.length;
		for (int i = 0; i < length; i++) {
			boolean b = Character.isLetter(checkString[i]);
			if (!b) {
				isValid = false;
				break;
			}
		}
		if (!isValid) {
			FacesMessage errMsg = getFacesMessage(rb,
					FacesMessage.SEVERITY_ERROR, 
					"ConstraintValidator.Validation_Error._8",  //$NON-NLS-1$
					"ConstraintValidator.This_field_may_only_contain_alphabetic_characters_between_A_to_Z._1", //$NON-NLS-1$
					component,
					convertedValue); 
			context.addMessage(component.getClientId(context), errMsg);
			component.setValid(false);
		}
	}

	protected void validateDigitOnly(String convertedValue, FacesContext context, UIInput component) {
		char[] checkString = convertedValue.toCharArray();
		boolean isValid = true;
		int length = checkString.length;
		for (int i = 0; i < length; i++) {
			boolean b = Character.isDigit(checkString[i]);
			if (!b) {
				isValid = false;
				break;
			}
		}
		if (!isValid) {
			FacesMessage errMsg = getFacesMessage(rb,
					FacesMessage.SEVERITY_ERROR, 
					"ConstraintValidator.Validation_Error._9",  //$NON-NLS-1$
					"ConstraintValidator.This_field_may_only_contain_digits_between_0_to_9._2", //$NON-NLS-1$
					component,
					convertedValue); 
			context.addMessage(component.getClientId(context), errMsg);
			component.setValid(false);
		}
	}

	protected void validateAlnumOnly(String convertedValue, FacesContext context, UIInput component) {
		char[] checkString = convertedValue.toCharArray();
		boolean isValid = true;
		int length = checkString.length;
		for (int i = 0; i < length; i++) {
			boolean b = Character.isLetterOrDigit(checkString[i]);
			if (!b) {
				isValid = false;
				break;
			}
		}
		if (!isValid) {
			FacesMessage errMsg =
				getFacesMessage(rb,
						FacesMessage.SEVERITY_ERROR, 
						"ConstraintValidator.Validation_Error._10",  //$NON-NLS-1$
						"ConstraintValidator.This_field_may_contain_alphabetic_characters_and_digits._3", //$NON-NLS-1$
						component,
						convertedValue); 
			context.addMessage(component.getClientId(context), errMsg);
			component.setValid(false);
		}
	}

	protected void validateRegEx(String convertedValue, FacesContext context, UIInput component) {
		try {
			if(!convertedValue.matches(regex)) {
				FacesMessage errMsg = getFacesMessage(rb,
										FacesMessage.SEVERITY_ERROR, 
										"ConstraintValidator.Validation_Error._10",  //$NON-NLS-1$
										"ConstraintValidator.MatchRegExp", //$NON-NLS-1$
										component,
										convertedValue); 
				context.addMessage(component.getClientId(context), errMsg);
				component.setValid(false);
			}
		}
		catch(PatternSyntaxException e) {
			FacesMessage errMsg = getFacesMessage(rb,
					FacesMessage.SEVERITY_ERROR, 
					"ConstraintValidator.Validation_Error._10",  //$NON-NLS-1$
					"ConstraintValidator.InvalidRegExp", //$NON-NLS-1$
					component,
					convertedValue); 
			context.addMessage(component.getClientId(context), errMsg);
			component.setValid(false);
		}
	}
	
	// ---------------------------------------------------- StateHolder Methods
	
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#saveState(javax.faces.context.FacesContext)
	 */
	public Object saveState(FacesContext context) {
		Object values[] = new Object[1];
		values[0] = regex;
		return (values);
	}

	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#restoreState(javax.faces.context.FacesContext, java.lang.Object)
	 */
	public void restoreState(FacesContext context, Object state) {
		Object values[] = (Object[])state;
		regex = (String)values[0];
	}

	private boolean transientFlag = false;
	
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#isTransient()
	 */
	public boolean isTransient() {
		return transientFlag;
	}

	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#setTransient(boolean)
	 */
	public void setTransient(boolean newTransientValue) {
		this.transientFlag = newTransientValue;
	}

	// --------------------------------------- Setter/Getter Methods

	/**
	 * @return
	 */
	public String getRegex() {
		return regex;
	}

	/**
	 * @param string
	 */
	public void setRegex(String regex) {
		this.regex = regex;
	}
	
	//------ a  method to get message -----------
	
	protected FacesMessage getFacesMessage(ResourceBundle rb, FacesMessage.Severity severity, String summaryKey, String detailKey, UIComponent component, Object value)
	{
		return FacesMessageUtil.getValidatorFacesMessage(rb, severity, summaryKey, detailKey, component, value);
	}
}
