/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2004 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.validator;

import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.component.StateHolder;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.util.FacesMessageUtil;

public class ExpressionValidator implements Validator, StateHolder {

	private static String _validatorMessage = "validatorMessage"; // JSF 1.2
	
	public static String ALPHABET_ONLY_MESSAGE;
	public static String DIGIT_ONLY_MESSAGE;
	public static String ALPHABET_NUMBER_ONLY_MESSAGE;
	public static String REGEX_IS_INVALID_MESSAGE;
	public static String NULL_MESSAGE;

	private String expression;
	
	private ResourceBundle rb;

	public ExpressionValidator() {
		super();
	}

	public ExpressionValidator(String exp) {
		super();
		setExpression(exp);
	}

	/* (non-Javadoc)
	 * @see javax.faces.validator.Validator#validate(javax.faces.context.FacesContext, javax.faces.component.UIComponent, java.lang.Object)
	 */
	public void validate(FacesContext context, UIComponent component, Object value) throws ValidatorException {
		
		if(value == null) {
			return;
		}
		if(expression == null || expression.length() == 0) {
			return;
		}

		rb = ResourceHandler.getBundle(context);

		if (context == null || component == null) {
			throw new NullPointerException();
		}
		
//		String convertedValue = ""; //$NON-NLS-1$
//
//		if (value instanceof String) {
//			convertedValue = (String) value;
//		} else {
//			convertedValue = value.toString();
//		}
		
		boolean bValid = true;
		FacesMessage errMsg = null;
		
		/* NO LONGER SUPPORTED IN JSF 1.2
		// Setup a temporary variable resolver so that 'value' inside the expression can be resolved
		VariableResolver oldResolver = context.getApplication().getVariableResolver();
		VariableResolver newResolver = new ExpressionValidatorVariableResolver(oldResolver, value);
		context.getApplication().setVariableResolver(newResolver);
		*/
		
		int i = expression.indexOf("value");
		if(i > -1){
			//add value to requestScope so can be retrieved with variable resolver
			String expressionValueKey = component.getClientId(context) + "_exp_val";
			//variable resolver doesn't like the colon
			expressionValueKey = expressionValueKey.replace(':','_');
			context.getExternalContext().getRequestMap().put(expressionValueKey,value);
			
			//replace value occurances in expression string with requestScope reference to value
			StringBuffer buf = new StringBuffer();
			int pos = 0;
			while(i > -1){
				//check if occurance is the "value" keyword
				if(		Character.isJavaIdentifierStart(expression.charAt(i - 1)) 
						|| Character.isJavaIdentifierPart(expression.charAt(i + 5))
						|| expression.charAt(i - 1) == '\''
						|| expression.charAt(i + 5) == '\''
				){
					//not keyword, don't replace...
					buf.append(expression.substring(pos,i + 5));
					pos = i + 5;
				} else {
					//is keyword, replace with requestScope var
					buf.append(expression.substring(pos,i))
					.append("requestScope.")
					.append(expressionValueKey);
					pos = i + 5;
				}
				i = expression.indexOf("value",pos);
			}
			//end rewritten expression after no more "value" occurences found
			buf.append(expression.substring(pos));
			expression = buf.toString();
		}
		
		Object result = null;
		try {
			result = context.getApplication().createValueBinding(expression).getValue(context);
		}
		catch(EvaluationException e) {
			errMsg = getFacesMessage(rb, FacesMessage.SEVERITY_ERROR, "ExpressionValidator.ValidationError", "ExpressionValidator.InvalidExpression", component, null); //$NON-NLS-1$ //$NON-NLS-2$
			bValid = false;
		}
		
		if(!(result instanceof Boolean)) {
			errMsg = getFacesMessage(rb, FacesMessage.SEVERITY_ERROR, "ExpressionValidator.ValidationError", "ExpressionValidator.NotBoolean", component, null); //$NON-NLS-1$ //$NON-NLS-2$
			bValid = false;
		}
		else {
			Boolean bResult = (Boolean) result;
			if(bResult != Boolean.TRUE) {
				errMsg = getFacesMessage(rb, FacesMessage.SEVERITY_ERROR, "ExpressionValidator.ValidationError", "ExpressionValidator.FalseExpression", component, null); //$NON-NLS-1$ //$NON-NLS-2$
				bValid = false;
			}
		}
		
		// NO LONGER SUPPORTED IN JSF 1.2
		//context.getApplication().setVariableResolver(oldResolver);

		if(!bValid) {
			context.addMessage(component.getClientId(context), errMsg);
			((UIInput)component).setValid(false);
		}		
		
	}

	
	// ---------------------------------------------------- StateHolder Methods
	
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#saveState(javax.faces.context.FacesContext)
	 */
	public Object saveState(FacesContext context) {
		Object values[] = new Object[1];
		values[0] = expression;
		return (values);
	}

	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#restoreState(javax.faces.context.FacesContext, java.lang.Object)
	 */
	public void restoreState(FacesContext context, Object state) {
		Object values[] = (Object[])state;
		expression = (String)values[0];
	}

	private boolean transientFlag = false;
	
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#isTransient()
	 */
	public boolean isTransient() {
		return transientFlag;
	}

	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#setTransient(boolean)
	 */
	public void setTransient(boolean newTransientValue) {
		this.transientFlag = newTransientValue;
	}

	
	public String getExpression() {
		return expression;
	}
	public void setExpression(String expression) {
		this.expression = expression;
	}

	//------ a  method to get message -----------

	protected FacesMessage getFacesMessage(ResourceBundle rb, FacesMessage.Severity severity, String summaryKey, String detailKey, UIComponent component, Object value)
	{
		return FacesMessageUtil.getValidatorFacesMessage(rb, severity, summaryKey, detailKey, component, value);
	}
}
