/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.validator;

import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.component.StateHolder;
import javax.faces.component.UIComponent;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

import com.ibm.faces.ResourceHandler;
import com.ibm.faces.util.FacesMessageUtil;


public class ModulusSelfCheckValidator implements Validator, StateHolder {
	
	private static String _validatorMessage = "validatorMessage"; // JSF 1.2
	
	private String modulus;
	
	public String getModulus() {
		return modulus;
	}
	public void setModulus(String modulus) {
		this.modulus = modulus;
	}
	public void validate(FacesContext context, UIComponent component, Object value)
			throws ValidatorException {
		if(value == null){
			return;
		}
		
		ResourceBundle rb = null;
		FacesMessage errMsg = null;
		
		if(modulus == null || !(modulus.equals("10") || modulus.equals("11"))){
			//validation error
			rb = ResourceHandler.getBundle(context);
			errMsg = getFacesMessage(rb,
					FacesMessage.SEVERITY_ERROR,
					"ValidationMessage.validation_error", 
					"ModulusSelfCheckValidator.incorrect_modulus_setting",
					component,
					value);
			setInvalid(context,component,errMsg);
			return;
		}
		
		String strValue = null;
		
		if(value instanceof Integer || value instanceof Long){
			strValue = value.toString();
		}
		
		if(value instanceof String){
			strValue = (String)value;
		}
		
		//remove non-numeric characters
		StringBuffer buf = new StringBuffer();
		for(int x = 0; x < strValue.length() ; x++){
			if((Character.isDigit(strValue.charAt(x)))){
				buf.append(strValue.charAt(x));
			}
		}
		strValue = buf.toString();
		
		if(strValue == null || strValue.length() < 2){
			//invalid value
			rb = ResourceHandler.getBundle(context);
			errMsg = getFacesMessage(rb,
					FacesMessage.SEVERITY_ERROR,
					ResourceHandler.getFacesMessageString(rb, "ValidationMessage.validation_error"), 
					ResourceHandler.getFacesMessageString(rb, "ModulusSelfCheckValidator.invalid_value"),
					component,
					value);
			setInvalid(context,component,errMsg);
			return;
		}
		
		char checkDigit = strValue.charAt(strValue.length() - 1);
		strValue = strValue.substring(0,strValue.length() -1);
		int sum = 0;
		int result = -1;
		int digit;
		
		if(modulus.equals("10")){
			boolean odd = true;
			for(int i = 0; i < strValue.length(); i++){
				digit = Character.getNumericValue(strValue.charAt(i));	
				if(odd){
					sum += (digit < 5)? digit * 2 : (((digit - 5)*2)+1);
					odd = false;
				} else {
					sum += digit;	
					odd = true;
				}
			}
			result = (10 - (sum % 10));
		}
		
		if(modulus.equals("11")){
			int weight = 2;
			for (int i = strValue.length(); i > 0 ; i--){
				digit = Character.getNumericValue(strValue.charAt(i - 1));	
				sum += digit * weight;
				weight = (weight >= 7)? 2 : weight + 1;
			}
			result = ((sum % 11)==0)? 0 : (11 - (sum % 11));
		}
		
		
		if(result != Character.getNumericValue(checkDigit)){
			//invalid value
			rb = ResourceHandler.getBundle(context);
			errMsg = getFacesMessage(rb,
					FacesMessage.SEVERITY_ERROR,
					"ValidationMessage.validation_error", 
					"ModulusSelfCheckValidator.invalid_value",
					component,
					value);
			setInvalid(context,component,errMsg);
			return;
		}
		
	}
	
	private void setInvalid(FacesContext context, UIComponent component, FacesMessage errorMessage){
		context.addMessage(component.getClientId(context),errorMessage);
		((UIInput)component).setValid(false);
		throw new ValidatorException(errorMessage);
	}
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#saveState(javax.faces.context.FacesContext)
	 */
	public Object saveState(FacesContext context) {
		Object values[] = new Object[1];
		values[0] = this.modulus;
		return (values);
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#restoreState(javax.faces.context.FacesContext, java.lang.Object)
	 */
	public void restoreState(FacesContext context, Object state) {
		Object values[] = (Object[])state;
		this.modulus = (String)values[0];
		
	}
	
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#isTransient()
	 */
	public boolean isTransient() {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see javax.faces.component.StateHolder#setTransient(boolean)
	 */
	public void setTransient(boolean arg0) {
		// TODO Auto-generated method stub
		
	}

	//------ a  method to get message -----------
	
	protected FacesMessage getFacesMessage(ResourceBundle rb, FacesMessage.Severity severity, String summaryKey, String detailKey, UIComponent component, Object value)
	{
		return FacesMessageUtil.getValidatorFacesMessage(rb, severity, summaryKey, detailKey, component, value);
	}
}
