/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2006 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.webapp;

import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;

import com.ibm.faces.util.AjaxUtil;

/**
 * AjaxPhaseListener is a PhaseListener which will cause the response to be rendered after the 
 * Apply request Values phase of the JSF lifecycle for Ajax requests that specified a partial
 * JSF life cycle should be executed.
 * 
 * @author mark_wallace@ie.ibm.com
 */
public class AjaxPhaseListener implements PhaseListener {

	/* (non-Javadoc)
	 * @see javax.faces.event.PhaseListener#afterPhase(javax.faces.event.PhaseEvent)
	 */
	public void afterPhase(PhaseEvent event) {
		// if this is an Ajax request and the mode is set to indicate partial
		// execution of the lifecycle then perform rendering after the 
		// Apply Request Values phase of the lifecycle.
		
		// must be after Apply Request Values
		if (event.getPhaseId() != PhaseId.APPLY_REQUEST_VALUES) {
			return;
		}
		// must be an Ajax request
		FacesContext facesContext = event.getFacesContext();
		if (!(AjaxUtil.isAjaxRequest(facesContext))) {
			return;
		}
				
		// must be partial mode		
		if (AjaxUtil.getAjaxMode(facesContext) != AjaxUtil.AJAX_PARTIAL) {
			return;
		}
		
		facesContext.renderResponse();
	}

	/* (non-Javadoc)
	 * @see javax.faces.event.PhaseListener#beforePhase(javax.faces.event.PhaseEvent)
	 */
	public void beforePhase(PhaseEvent event) {
	}

	/* (non-Javadoc)
	 * @see javax.faces.event.PhaseListener#getPhaseId()
	 */
	public PhaseId getPhaseId() {
		return PhaseId.APPLY_REQUEST_VALUES;
	}

}
