/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2000, 2003 - All Rights Reserved. 
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */
package com.ibm.faces.webapp;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.faces.util.ContextParams;
import com.ibm.faces.util.LifecycleUtil;

//TODO bill
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;
//TODO bill

/**
 * @author ykats
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class JSResourceServlet  extends HttpServlet implements Servlet {
	private static final long serialVersionUID = -4384407003696299574L;
	private ClassLoader classLoader = null;
	private HashMap cache = null;
	private long startTime = -1; 

	private static final String PATH_PREFIX = "ibmjsfres"; //$NON-NLS-1$

	private String URLPattern = null;
	private Long expires = null;

	public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {

		InputStream stream = null;
		byte[] buffer = new byte[10000];


		String uri = req.getRequestURI();
		// System.err.println("JSRServlet: Processing request for: " + uri); //$NON-NLS-1$
		
		//RATLC01083006
		//uri = uri.substring(uri.lastIndexOf(PATH_PREFIX));
		if(URLPattern == null)
			URLPattern = LifecycleUtil.getJSResourceURLPattern(getServletContext().getInitParameter(ContextParams.JS_RESOURCE_SERVLET_URL_PATTERN));
		
		uri = PATH_PREFIX + uri.substring(uri.lastIndexOf(URLPattern) + URLPattern.length());
		
		long ifModSince = -1;
		try {
			ifModSince = req.getDateHeader("If-Modified-Since"); //$NON-NLS-1$
		} catch (IllegalArgumentException e) {
		}
		if(ifModSince >= startTime ) {
			// System.err.println("JSRServlet: " + uri + " exists in client's cache. If-Modified-Since: " + ifModSince); //$NON-NLS-1$ //$NON-NLS-2$
			resp.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
			return;
		}
		
		try {
			resp.setDateHeader("Last-Modified", startTime); //$NON-NLS-1$
			resp.setHeader("Content-Type", getServletContext().getMimeType(uri));
			
			/*
			 * w3 http1.1 spec (http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html) states...
			 * 
			 * To mark a response as "never expires," an origin server sends an Expires date 
			 * approximately one year from the time the response is sent. HTTP/1.1 servers SHOULD NOT 
			 * send Expires dates more than one year in the future.
			 * 
			 * As resources are now filename versioned by default we want to set this to 'never expires', to do this
			 * we set this to one year after start time according to http1.1 spec (section above).
			 * 1 year  = 31536000000 milliseconds
			 * 
			 * Custom expire value can be set with com.ibm.faces.RESOURCE_EXPIRE_MS context-param set to milliseconds
			 * 
			 */
			if(expires == null){
				String expiresParam = getServletContext().getInitParameter(ContextParams.RESOURCE_EXPIRE_MS);
				if(expiresParam != null){
					expires = Long.valueOf(expiresParam);
					if(expires.longValue() > 31536000000L){
						expires = Long.valueOf("31536000000");
					}
				} else {
					expires = Long.valueOf("31536000000");
				}
			}
			resp.setDateHeader("Expires", startTime + expires.longValue()); //$NON-NLS-1$



			//load the generated calendar/date strings for the locale request when someone looks for the string file
			String extrafile = null;
			if(uri.indexOf("/hxclient_S_")>0)
			{
				
				Locale locale =  null;
				//use the view locale parameter value for locale if exists
				String localeStr = req.getParameter(HxClientRenderUtil.HX_VIEW_LOCALE);
				if(localeStr == null){
					//use locale on file name
					String resName = "hxclient_S_" + HxClientRenderUtil.HX_VERSION;
					if(uri.indexOf(resName + ".js") > -1 ){
						//no lang on filename - use english
						localeStr = "en";
					} else if(uri.indexOf(resName + "_") > -1){
						int i = uri.indexOf(resName);
						localeStr = uri.substring(i + (resName.length() + 1), uri.indexOf(".js",i ));
					}
				}
				
				if(localeStr != null && localeStr.length() > 1){
					localeStr = localeStr.trim();
					if(localeStr.indexOf('_') > -1){
						locale = new Locale(localeStr.substring(0,2),localeStr.substring(3));
					} else {
						locale = new Locale(localeStr,"");
					}	
				} else {
					//get the locale from the request
					locale = req.getLocale();
				}
				
				
				
				Map resMap = (Map)getServletContext().getAttribute(HxClientRenderUtil.HX_STRING_RESOURCES);//(Map)context.getExternalContext().getApplicationMap().get(HX_STRING_RESOURCES);
				
				if(resMap != null){
					extrafile = (String)resMap.get(locale.toString());			
				}
				
				if(extrafile == null)
				{
					extrafile = HxClientRenderUtil.generateStringResources(locale , resMap);
				}

				// This turns the caledar/date strigs into a byte array to be streamed out
				if(extrafile != null) {
					byte[] bfile = extrafile.getBytes();
					resp.getOutputStream().write(bfile);
					return;
				}

			}			
			
			
			if(cache != null) {
				boolean bInCache;
				synchronized(this) {
					bInCache = cache.containsKey(uri);
				}
				if(bInCache) {
					// System.err.println("JSRServlet: " + uri + " found in cache"); //$NON-NLS-1$ //$NON-NLS-2$
					byte[] bytes;
					synchronized(this) {
						bytes = (byte[])cache.get(uri);
					}
					resp.getOutputStream().write(bytes);
					return;
				}
			}

			
			stream = classLoader.getResourceAsStream(uri);
			if(stream != null) {
				if(cache == null) {
					// System.err.println("JSRServlet: Loading " + uri);//$NON-NLS-1$
					int read = 0;
					while((read = stream.read(buffer)) > 0) {
						resp.getOutputStream().write(buffer, 0, read);
					}
				}
				else {
					// System.err.println("JSRServlet: Loading and saving " + uri); //$NON-NLS-1$
					byte[] bytes = new byte[0];
					int read = 0;
					while((read = stream.read(buffer)) > 0) {
						int newSize = bytes.length + read;
						byte[] newBytes = new byte[newSize];
						System.arraycopy(bytes, 0, newBytes, 0, bytes.length);
						System.arraycopy(buffer, 0, newBytes, bytes.length, read);
						bytes = newBytes;
					}
					resp.getOutputStream().write(bytes);
					synchronized(this) {
						cache.put(uri, bytes);
					}
				}
			}
			else {
				throw new FileNotFoundException("JSRServlet: " + uri + " could not be found"); //$NON-NLS-1$ //$NON-NLS-2$
			}

		} catch (FileNotFoundException e) {
			// System.err.println(e.getMessage()); //$NON-NLS-1$
			throw e;
		} finally {
			if(stream != null) {
				stream.close();
			}
		}
				
	}

	/* (non-Javadoc)
	 * @see javax.servlet.Servlet#init(javax.servlet.ServletConfig)
	 */
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		// System.err.println("JSRServlet: Init"); //$NON-NLS-1$
		classLoader = getClass().getClassLoader();
		
		String cacheParam = config.getInitParameter(ContextParams.JS_RESOURCE_SERVLET_CACHE);
		if(cacheParam == null) {
			// try the old (deprecated) param name
			cacheParam = config.getInitParameter(ContextParams.PARAM_CACHE_OLD);
		}
		
		if(cacheParam != null && cacheParam.equalsIgnoreCase("true")) { //$NON-NLS-1$
			// System.err.println("JSRServlet: cache is on"); //$NON-NLS-1$
			cache = new HashMap();
		}
		else {
			// System.err.println("JSRServlet: cache is off"); //$NON-NLS-1$
		}
		// Round down to the nearest second since client headers are in seconds
		startTime = System.currentTimeMillis() / 1000 * 1000;
	}

	/* (non-Javadoc)
	 * @see javax.servlet.Servlet#destroy()
	 */
	public void destroy() {
		// System.err.println("JSRServlet: Destroy"); //$NON-NLS-1$
		if(cache != null) {
			cache.clear();
		}
		super.destroy();
	}

}
