/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.webapp;

import java.lang.reflect.Method;

import javax.faces.component.UICommand;
import javax.faces.context.FacesContext;
import javax.faces.el.MethodBinding;
import javax.faces.event.ActionEvent;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;

import com.ibm.faces.component.UIRequest;
import com.ibm.faces.renderkit.html_extended.HxClientRenderUtil;


/**
 * @author pandrews
 * <p>Listens for the stateless command parmeter on the request, if value is found ti attempts to execute the referenced action code</p> 
 */
public class RequestPhaseListener implements PhaseListener {


	public void beforePhase(PhaseEvent event) {
		//do nothing
	}

	public PhaseId getPhaseId() {
		return PhaseId.RESTORE_VIEW;
	}
	
	public void afterPhase(PhaseEvent event) {

		FacesContext context = event.getFacesContext();
		String actionStr = (String)context.getExternalContext().getRequestParameterMap().get(UIRequest.ACTION_PARAMETER);
		
		
		if(actionStr != null){
		/*
		 * process...
		 *  - no view will be restored from session, new instance will be created by RestoreViewPhase class
		 *  - set the id of the empty view root to the viewid this request came from, this will be rendered if no navigation happens
		 *  - process the action string to see what to do, it may be set to not process navigation, be a string, be a method binding
		 */
		
			// if in portal and mode has changed, don't do anything
			if(HxClientRenderUtil.isPortal(context.getExternalContext().getRequest())) {
				String portletMode = null;
	            try {
	            	Method method = context.getExternalContext().getRequest().getClass().getMethod("getPortletMode", null);
					portletMode = method.invoke(context.getExternalContext().getRequest(), null).toString();
				} catch (Exception e) {
				}
				
				String oldMode = (String)context.getExternalContext().getRequestParameterMap().get(UIRequest.PORTLET_MODE_PARAMETER);
				if(portletMode != null && !portletMode.equals(oldMode)) {
					return;
				}
			}
			
			
			//set the viewid to the viewid this uirequest came from - will fall back to this if no navigation happens
			String fromViewId = (String)context.getExternalContext().getRequestParameterMap().get(UIRequest.VIEW_ID_PARAMETER);
			event.getFacesContext().getViewRoot().setViewId(fromViewId);
			
			if(actionStr.equals(UIRequest.NO_NAVIGATION_VALUE)){
				//no navigation to be performed (action attribute was not set on the component)
				return;
			} else if(actionStr.startsWith(UIRequest.OUTCOME_PREFIX)){
				//is a string
				context.getApplication().getNavigationHandler().handleNavigation(context,null,actionStr.substring(UIRequest.OUTCOME_PREFIX.length()));
			} else {
				//is a method binding expression
				actionStr = "#{" + actionStr + "}";
				UICommand component = new UICommand();
				MethodBinding binding = context.getApplication().createMethodBinding(actionStr,null);
				component.setAction(binding);
				ActionEvent action = new ActionEvent(component);
				context.getApplication().getActionListener().processAction(action);
			}
		}
	}
}
