/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2003 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

package com.ibm.faces.webapp;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.SocketException;
import java.util.Map;
import javax.faces.FacesException;
import javax.faces.FactoryFinder;
import javax.faces.component.UIComponent;
import javax.faces.component.UIData;
import javax.faces.component.UIParameter;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.faces.lifecycle.Lifecycle;
import javax.faces.lifecycle.LifecycleFactory;
import javax.faces.model.DataModel;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import com.ibm.faces.component.UIDataIterator;
import com.ibm.faces.renderkit.html_extended.Utils;
import com.ibm.faces.util.Base64;
import com.ibm.faces.util.DataModelUtil;
import com.ibm.faces.util.LifecycleUtil;


/**
 * This is a phase listener that services file download. 
 * It performs checks of the params after the RENDER_VIEW phase
 * and returns binary data if needed.
 * 
 * @author eugene konstantinov
 * 
 *
 */
public class ValueResourcePhaseListener implements PhaseListener, IValueResourceListener {
	/*
	public static final String VIEWID_PARAM = "vrViewID"; //$NON-NLS-1$
	public static final String CLIENTID_PARAM = "vrClientId"; //$NON-NLS-1$
	public static final String PARENTID_PARAM = "vrParentId"; //$NON-NLS-1$
	public static final String CLIENTINDEX_PARAM = "vrClientIndex"; //$NON-NLS-1$
	*/
	private final static String MIMETYPE_ATTR = "mimeType"; //$NON-NLS-1$
//	private boolean doRender = true;
	
	
	public static String VALUE_PARAM = "_vrval";
	public static String TYPE_PARAM = "_vrct";
	public static String LITERAL_TYPE_PREFIX = "_l_";
	public static String DATA_VALUE = "_vrd";
	public static String ROW_INDEX = "_vri";
	//Dummy view id used for url, .jsp extension is required for portal server to encode the generated url...
	//public static String VIEWID = "_vrlink.jsp";
	
	private static IValueResourceListener valueResourceListener = null;

	/* (non-Javadoc)
	 * @see javax.faces.event.PhaseListener#afterPhase(javax.faces.event.PhaseEvent)
	 */
	public void beforePhase(PhaseEvent event) {
	}
	
	/* Checks for the param in the request that indicates the server-side resource is requested  
	 * @see javax.faces.event.PhaseListener#beforePhase(javax.faces.event.PhaseEvent)
	 */
	public void afterPhase(PhaseEvent event) {

		try {
			FacesContext context = event.getFacesContext();
			if (context == null)
				return;

			Map parameters = context.getExternalContext().getRequestParameterMap();

			String valueParam = (String)parameters.get(VALUE_PARAM);
			
			if (valueParam != null) {
				//incoming request for download, stopping lifecycle at this point
				context.responseComplete();
			} else {
				//regular request
				return;
			}
			
			String typeParam = (String)parameters.get(TYPE_PARAM);
			String dataParam = (String)parameters.get(DATA_VALUE);
			
			Object resourceValue = null;
			String contentType = null;
			
			//type maybe set as a literal value, if so need to reset it to correct contentType value
			if(typeParam != null){
				if(typeParam.startsWith(LITERAL_TYPE_PREFIX)){
					//literal mimeType value set so set contentType field here
					typeParam = typeParam.substring(LITERAL_TYPE_PREFIX.length(),typeParam.length());
					StringBuffer buffer = new StringBuffer(typeParam.substring(0,typeParam.indexOf("_")));
					buffer.append("/")
					.append(typeParam.substring(typeParam.indexOf("_") + 1,typeParam.length()));
					
					contentType = buffer.toString();
				}
			}
			
			//tihs will be the case if contained in data component and value expression begins with var.....
			if(dataParam != null){
				//get dataTable/Iterator DataModel value
				//if we're here the var is the value's binding prefix...
				String var = valueParam.substring(0,valueParam.indexOf("."));
				Object dataValue = null;
				DataModel dm = null;
				ValueBinding vb = null;
				String index = null;
				dataParam = "#{" + dataParam + "}";
				vb = context.getApplication().createValueBinding(dataParam);
				if(vb != null){
					dataValue = vb.getValue(context);
					if (dataValue != null){
						dm = DataModelUtil.getDataModel(dataValue);
						
					}
				}
				if(dm != null){
					//we have a datamodel now we need to retreive the value for the row
					index = (String)parameters.get(ROW_INDEX);
					dm.setRowIndex(Integer.parseInt(index));
					if(dm.isRowAvailable()){
						//put row on requestMap under var so that value can be resolved thru binding
						context.getExternalContext().getRequestMap().put(var,dm.getRowData());
					}
				}
				vb = context.getApplication().createValueBinding("#{" + valueParam + "}");
				if(vb != null) {
					resourceValue = vb.getValue(context);
				}
				//resoureValue cannot be resolved, no point in continuing
				if(resourceValue == null){
					return;
				}
				//contentType may have been set to a literal string
				if(contentType == null){
					//we presume that mimeType won't be bound to a data var when value is not
					//so so just getting this value same whether var or not (var will be available at this point
					//if value uses var)
					vb = context.getApplication().createValueBinding("#{" + typeParam + "}");
					if(vb != null){
						contentType = (String)vb.getValue(context);
						//null contentType will be ok
					}
				}
			} else {
				//not contained in dataTable...
				ValueBinding vb = null;
				
				valueParam = "#{" + valueParam + "}";
				vb = context.getApplication().createValueBinding(valueParam);
				if(vb != null){
					resourceValue = vb.getValue(context);
				}
				//resoureValue cannot be resolved, no point in continuing
				if(resourceValue == null){
					return;
				}
				if(contentType == null){
					//hasn't been set by the check for literal value so must be a binding
					typeParam = "#{" + typeParam + "}";
					vb = context.getApplication().createValueBinding(typeParam);
					if(vb != null){
						contentType = (String)vb.getValue(context);
					}
				}
			}

			HttpServletResponse response =	(HttpServletResponse) context.getExternalContext().getResponse();

			if (response == null) {
				return;
			}
			byte[] bytes = null;
			if (resourceValue != null) {
				if (resourceValue instanceof String) {
					bytes = Base64.decode((String)resourceValue);
				} else if (resourceValue instanceof byte[]) {
					bytes = (byte[]) resourceValue;
				} else if (resourceValue instanceof InputStream) {
					InputStream is = (InputStream) resourceValue;
					ByteArrayOutputStream baos =
						new ByteArrayOutputStream();
					//		try {
					while (is.available() > 0) {
						baos.write(is.read());
					}
					bytes = baos.toByteArray();
					//		} catch (Exception e) {
					//			return;
					//		}
				} else {
					return;
				}
				
				if (contentType != null) {
					response.setContentLength(bytes.length);
					response.setContentType(contentType);
				}
				if ((bytes != null) && (bytes.length > 0)) {
					ServletOutputStream out = response.getOutputStream();
					if (out != null) {
						out.write(bytes);
					} else {
						return;
					}
				}
			}

		} catch (SocketException se) {
			return;
		} catch (Exception e) {
			System.err.println(
				"Exception thrown in ValueResourcePhaseListener:" + e); //$NON-NLS-1$
			e.printStackTrace();
		}
	}

	/* Listen to the RESTORE_VIEW phase
	 * @see javax.faces.event.PhaseListener#getPhaseId()
	 */
	public PhaseId getPhaseId() {
		return PhaseId.RESTORE_VIEW;
	}

	/*
	 * <p>Checks that there is a ValueResourcePhaseListener running, if found uses it to generate and return special ValueResource Url string, 
	 * if not found returns null.</p>
	 */
	public static String getUrl(FacesContext context, UIComponent component) {
		if(valueResourceListener == null) {
			Lifecycle lifecycle = null;
			try {
				LifecycleFactory lifecycleFactory =
					(LifecycleFactory) FactoryFinder.getFactory(
						FactoryFinder.LIFECYCLE_FACTORY);
				String lifecycleId = LifecycleFactory.DEFAULT_LIFECYCLE;	
						lifecycle = lifecycleFactory.getLifecycle(lifecycleId);
	
			} catch (FacesException e) {
				System.out.println(e.getLocalizedMessage()); 
				return null;
			}
		
			if (lifecycle == null) {
				return null;
			} 
			
			PhaseListener[] listeners = lifecycle.getPhaseListeners();
			PhaseListener next = null;
			for (int i = 0; i < listeners.length; i++) {
				next = listeners[i];
				if (next instanceof IValueResourceListener) {
					valueResourceListener = (IValueResourceListener) next;
					break;
				}
			}
		}
		
		if(valueResourceListener != null){			
			return valueResourceListener.getResourceURL(context,component);
		}

		return null;
	}
	
	/*
	 * <p>Generates and returns special ValueResource Url string.</p>
	 */
	public String getResourceURL(FacesContext context, UIComponent component) {
		
		ValueBinding valueBinding = null;
		//if a value can't be resolved return null
		String value = null;
		valueBinding = component.getValueBinding("value");
		if(valueBinding != null){
			value = valueBinding.getExpressionString();
		} else {
			//could be literal string, if so just return the value as no need to generate a ValueResource url
			value = (String)component.getAttributes().get("value");
			if(value == null){
				value = "";
			}
			return value;
		}
		if(value == null){
			//no value, no point in carrying on
			return null;
		}
		
		String contentType = null;
		valueBinding = component.getValueBinding(MIMETYPE_ATTR);
		if(valueBinding != null){
			contentType = valueBinding.getExpressionString();
		} else {
			//can be literal string
			contentType = (String)component.getAttributes().get(MIMETYPE_ATTR);
		}
		
		//Create the url

		/*RATLC01128988 - SDO/PageCode code RAD generates, uses the View object to store filter arguments.
		 * So cannot make resource requests stateless, view needs to be restored, changes to class were made
		 * so that it would work using client state saving, this will still work restoring the view - we don't
		 * need it, we now have to do it just in case PageCodeBase added data to viewroot attributes.
		 */
		
		/*
		 String url = context.getApplication().getViewHandler().getActionURL(context,context.getViewRoot().getViewId());
		 
 
		//use special viewid to prevent an unncessary view being restored. 
		StringBuffer href = new StringBuffer(url.substring(0,url.lastIndexOf("/") + 1));
		href.append(VIEWID);
		String fext = ".faces";
		//check for faces suffix
		if(url.endsWith(fext)){
			href.append(fext);
		}
		*/
		
		StringBuffer href = new StringBuffer(context.getApplication().getViewHandler().getActionURL(context,context.getViewRoot().getViewId()));
		href.append("?");
		
		
		//value will not be null - wouldn't get here otherwise
		if(value.indexOf("#{") > -1  && value.lastIndexOf("}") > -1){
			//is binding
			value = value.substring(value.indexOf("#") + 2, value.lastIndexOf("}"));
			href.append(VALUE_PARAM)
			.append("=")
			.append(value);
		}
		
		if(contentType != null && contentType.length() > 0){
			if(contentType.indexOf("#{")> -1 && contentType.indexOf("}") > -1 ){
				//is binding
				contentType = contentType.substring(contentType.indexOf("#{") + 2, contentType.lastIndexOf("}"));
				
			} else {
				//so must be a literal string
				//need a special prefix to let listener know this is a literal
				//also strip out the forward slash as this will break url
				contentType = contentType.trim();
				StringBuffer buffer = new StringBuffer(LITERAL_TYPE_PREFIX);
				buffer.append(contentType.substring(0,contentType.indexOf("/")))
				.append("_")
				.append(contentType.substring(contentType.indexOf("/") + 1,contentType.length()));
				
				contentType = buffer.toString();
			}
			href.append("&")
			.append(TYPE_PARAM)
			.append("=")
			.append(contentType);
		}

		
		/*
		 * Check if contained in dataTable or dataIterator, if so check if value's binding expression uses
		 * the data component's var property, if it does we need to emit extra parameters to handle the
		 * value is a data component. (We assume var won't be used for mimetype if its not used for value)
		 */
		Boolean isVarBinding = Boolean.FALSE;
		String index = null;
		String dataValue = null;
		UIData data = Utils.findParentUIData(component);
		if(data != null){
			if(data.getVar() != null && value.startsWith(data.getVar() + "." )){
				isVarBinding = Boolean.TRUE;
				valueBinding = data.getValueBinding("value");
				if(valueBinding != null){
					dataValue = valueBinding.getExpressionString();
					index = String.valueOf(data.getRowIndex());
				}
			}
		} else {
			//contained in dataIterator
			UIDataIterator dataIterator = Utils.findParentUIDataIterator(component);
			if(dataIterator != null){
				if(dataIterator.getVar() != null && value.startsWith(dataIterator.getVar() + "." )){
					isVarBinding = Boolean.TRUE;
					valueBinding = dataIterator.getValueBinding("value");
					if(valueBinding != null){
						dataValue = valueBinding.getExpressionString();
						index = String.valueOf(dataIterator.getIndex());
					}
				}
			}
		}
		
		if(isVarBinding.booleanValue()){
			if(dataValue !=null && index != null){
				dataValue = dataValue.substring(dataValue.indexOf("#{") + 2, dataValue.lastIndexOf("}"));
				href.append("&")
				.append(DATA_VALUE)
				.append("=")
				.append(dataValue)
				.append("&")
				.append(ROW_INDEX)
				.append("=")
				.append(index);	
			} else {
				//TODO - maybe throw error here??, properties not retrieved from parent data component, these
				//properties should be present for a dataTable to work properly.
			}
			
		}
		
		//add parameters to URL if they exist
		UIParameter[] params = LifecycleUtil.getParameters(component);
		if(params != null && params.length > 0){
			for(int i=0; i < params.length; i++){
				href.append("&")
				.append(params[i].getName())
				.append("=")
				.append(params[i].getValue());
			}
		}
	
		//returning unencoded href string as this is what code previous to this done
		return href.toString();
	}

}
