/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

//==================================================================
//  Input State Control
//==================================================================
//  Construct and attach to a behavior.  
//
// 	Monitors basic keyboard events (e.g., down and up) to provide a browser-neutral 
//  "keystroke transaction".  
//	Commit the transaction if the keyboard event is acceptable (e.g.,
//  if validation of the keystroke succeeds).  
//	Rollback the event if it fails.
//
//  InputState provides basic, generic handling of keystrokes.  Each
//  event is re-dispatched into the assist handler's specific "sub-input-state"
//  handler.  This allows a specific kind of assist to add more specific
//  processing to the event.
//
//==================================================================
HxG_5.prototype.inputState = function (_obj, _behavior, _shortform) {

	var obj=_obj, behavior=_behavior, value=obj.value, orgvalue=obj.value;
	var pos=-1, sel=-1, orgpos=-1, autotab=0, skipC=false;
	var keydown=false, keypress=false, updating=false, autosubmit=false, focus=false, checkedComponent=false;
	var keypress_r=null, commitCallback=null, hasComponent=null, autotabtimer=null;

	this.rollback	= rollback;	// Do not use the updated value/caret
	this.commit		= commit;	// Do use the updated value/caret
	this.uirelease	= uirelease;// Release the UI (force an onblur to happen)
	this.destroy	= destroy;	// Destructor
	this.setAutoTab	 = function setAutoTab(x)	{ autotab = x;			}
	this.setAutoSubmit= function setAutoSubmit(){ autosubmit = true;	}
	this.setCommitCallback= function setCommitCallback(x) { commitCallback = x; }
	this.setValue	 = function setValue(c) 	{ value = c;			}
	this.getValue	 = function getValue()		{ return value;			}
	this.setPos		 = function setPos(c)		{ pos = c;				}
	this.getPos		 = function getPos()		{ return pos;			}
	this.hasSel		 = function hasSel()		{ return sel;			}
	this.dispatch	 = dispatch;		// private
	this.dispatchTimer = dispatchTimer;	// private

	// Body of constructor

	hX_5.attachInputStateEvent(obj, hX_5._onF);
	hX_5.attachInputStateEvent(obj, hX_5._onB);
	hX_5.attachInputStateEvent(obj, hX_5._onKD);
	hX_5.attachInputStateEvent(obj, hX_5._onKP);
	hX_5.attachInputStateEvent(obj, hX_5._onKU);
	// If we're not doing char-by-char handling, don't set up the mouse events
	if (_shortform == null) {
		hX_5.attachInputStateEvent(obj, hX_5._onMC);
		// Following is only available in IE -- so a mess happens in Mozilla
		if (hX_5.brw.isIE()) {
			hX_5.attachInputStateEvent(obj, hX_5._onCP);
			hX_5.attachInputStateEvent(obj, hX_5._onCT);
			hX_5.attachInputStateEvent(obj, hX_5._onPST);
		}
	}

	// end body of constructor

	// Private -- helper to remember state
	function saveState () {
		value = obj.value;
		pos = hX_5.sel.getCaretPos(obj);
		sel = hX_5.sel.hasSelection(obj);
		orgvalue= value;
		orgpos	= pos;
	}

	// Private -- Handle deferring an autotab until after a keypress event is processed
	function dispatchTimer (item, action) {
		autotabtimer = null;
		if (focus && action == "autotab" && (autotab > 0)) doAutoTab();
	}

	// Public -- destructor
	function destroy () {
		// Remove events on the object itself
		hX_5.removeAllEvents(obj, obj.id);
	}

	// Public -- force onblur like handling to close up a field
	function uirelease () {
		keydown = keypress = false;
		updating = true;
		if (autotabtimer) {
			hX_5.i.killTimerTimed(autotabtimer);
			autotabtimer = null;
		}
		// If the converter needs to "unprep" the field, do it
		// Note that we only do this if we "prep'd" the field via an onfocus event
		if (focus && behavior.assist && behavior.assist.onBlur) behavior.assist.onBlur(null);
		focus = false;
		hX_5.pendingPrompt = null;
	}

	// Public -- Dispatch the event sent to the input state (through the behavior's dispatch)
	// Here's an outline of the rules for handling the key sequence:
	//
	// In the simplest case, the browser fires <keydown><keypress><key-up>.  
	//
	// The keydown/up return KEY codes (which physical key is pressed) 
	// while keypress returns the character code.  Since there's no mechanism in JS to map
	// keycodes (e.g., the 54th key is down) to characters (e.g., "F"), the rule in Windows
	// is that you process FUNCTION keys on keydown (e.g., F1, CTRL+C) while you process
	// CHARACTERS on keypress.
	//
	// Now, the browser often will fire a much more complex seqence of events than down/press/up.
	// If someone presses "SHIFT" then A", you'll get DOWN DOWN PRESS UP UP.  If someone holds
	// the A key down, you'll get DOWN DOWN DOWN DOWN DOWN PRESS PRESS PRESS PRESS UP UP (note
	// asymmetric number of downs/ups).  If someone presses SHIFT then releases it then presses A
	// you'll get DOWN UP DOWN PRESS UP.
	// If someone presses SHIFT A TAB (release SHIFT), you'll get DOWN DOWN PRESS UP in one field
	// and an UP (nothing else) in the previous field (so a field's sequence can start with an UP).
	// 
	// In sum, there's no reliable way of pairing DOWNS/UPS.  So again, in Windows you
	// track KEY DOWN for control keys and KEY PRESS for characters.  And you do nothing
	// on UP (except some error checking).
	// 
	// To make things worse, browsers mis-handle the IME (Input Method Editor).  When the IME
	// is being used to enter double-byte characters, the browser gets DOWN and UP events as
	// a user types in the IME.  We get DOWN/UP events on each key as they are pressed/released.
	// We do not get ANY keypress events (i.e., notification of the formation of a character)
	// while the IME is actively composing.  When the IME is "done" composing (usually indicated
	// when the user presses ENTER), we'll get a keypress (sometimes) for the ENTER (but not
	// for the formed characters).  So with the IME, we can't track characters as they are entered.
	//
	// NOTE:  See blurb in dispatchBehavior for onBlur validation (hxclient_be) on focus/blur handling
	function dispatch (evt) {
		var r=true, newR;
		if (hX_5.pendingFocusReturn && hX_5.pendingFocusReturn != obj.id) return true;
		hX_5.pendingFocusReturn = null;
		if ((obj.readOnly != null && obj.readOnly == true) || (obj.disabled != null && obj.disabled == true)) return true;
		skipC=false;

		switch (evt.type) {
			// Prep the value
			case hX_5._evF:
				focus = true;
				hX_5.pendingPrompt = this;
				keydown = keypress = false;
				updating = true;
				autotabtimer = null;
				// See if this field has a component attached to it (we only do this once
				// and we can't do it until now cuz the component could be added at any time)
				if (!checkedComponent) {
					hasComponent = hX_5.getComponentById(obj.id);
					checkedComponent = true;
				}
				// If the converter needs to "prep" the field (e.g., add prompt chars), do it
				// Netscape will normally select the contents of the field on focus
				// so this call (which presets the cursor) doesn't work in NS.
				// There's no obvious way to turn this "feature" off in NS
				if (behavior.assist && behavior.assist.onFocus) behavior.assist.onFocus(evt);

				// If we are coming off an action that wanted a "full select", do it.
				if (hX_5.pendingSelect && obj.select) obj.select();
				hX_5.pendingSelect = false;

				// Re-init state
				saveState();

				// Yamato code to disable the IME when a mask is in force.  Note that this
				// call isn't reliable (it's in the onfocus handler).  It may not take
				// effect until the next use of the field so we "initially" set the
				// ime mode as soon as it's declared in hxclient_be.js
				if (behavior.assist && pos >= 0) obj.style.imeMode = hX_5._DSB;

				// Note that we don't stop the "onfocus" -- "r" is true
				break;

			// Mark that the field does not have focus and unprep the value
			case hX_5._evB:
				keydown = keypress = false;
				updating = true;
				if (autotabtimer) {
					hX_5.i.killTimerTimed(autotabtimer);
					autotabtimer = null;
				}
				// If the converter needs to "unprep" the field, do it
				// Note that we only do this if we "prep'd" the field via an onfocus event
				if (focus && behavior.assist && behavior.assist.onBlur) behavior.assist.onBlur(evt);
				focus = false;
				hX_5.pendingPrompt = null;
				// Note that we don't stop the blur - "r" is true
				break;

			// Keydown 
			case hX_5._evKD:
				if (!focus) break;	
				// Mark the start of a keystroke sequence
				keydown = true;
				keypress = updating = false;
				saveState();
				// If this is a function key, handle it as we might not get a keypress
				// Check if this is a disposable keystroke
				if (isDisposableKey(evt) && behavior.assist) return false;
				// Check for a "submit" key (enter) as this is special
				else if (isSubmitFcnKey(evt) && behavior.assist) {
					// If the converter needs to "unprep" the field, do it else the prompt chars will be submitted
					// Note that we only do this if we "prep'd" the field via an onfocus event
					if (focus && behavior.assist && behavior.assist.onBlur) behavior.assist.onBlur(evt);
					focus = false;
					// Note that we don't stop the blur - "r" is true
					break;
				// Handle the others
				} else if (hX_5.isFcnKey(evt) && behavior.assist) {
					// If a component wants to handle this key, let them have it
					if (hasComponent && hasComponent.isRedirectFcnKey && hasComponent.isRedirectFcnKey(evt)) {
						// The component is going to deal with the value "without the mask", so convert it out
						var newValue = (behavior.assist.fromAssist) ? behavior.assist.fromAssist(value) : obj.value;
						// Let the component get it
						newValue = (hasComponent.doRedirectFcnKey) ? hasComponent.doRedirectFcnKey(evt, newValue) : newValue;
						// Put it back in the right format
						newValue = (behavior.assist.toAssist) ? behavior.assist.toAssist(newValue) : newValue;
						// Update the value in the field
						value = (newValue.v) ? newValue.v : value;
						commit(evt);
						// This isn't quite right -- we could "spin" to a bigger value which should adjust the pos, but it's close enough
						if (pos >= 0) hX_5.sel.setCaretPos(obj, pos);
						r = false;	// don't let the component get this event again					
					// Now the normal case -- let the behavior handle the key
					} else if (behavior.assist.onChangeKeyCode && !isSkipFcnKey(evt)) {
						r = behavior.assist.onChangeKeyCode(evt);
						if (r == -1) r = true;
						else if (r) {
							newR = doChange(evt);
							r = (r == -2) ? false : newR;
						}
						if (!r) hX_5.i.cancelAction(evt, true);
					}
					// We've handled the key here so mark us as "pressed"
					// Note however, that Japanese (and perhaps other) keyboards treat some keystrokes
					// as being BOTH a function key and as a printable character
					// If we think we have one of these keys, let the keypress be processed
					if (!isPrintableFcnKey(evt)) {
						keypress = true;
						keypress_r = r;
					}
				}
				break;

			// Key press
			case hX_5._evKP:
				if (!focus) break;	
				updating = false;
				// There's a bug in Netscape where if you press the "/" key (or "?" key), they
				// emit a spurious keypress with charcode set to 0 (and keycode set to 191) and then
				// a second "correct" keypress.  We need to ignore the first one
				// Also, if Netscape, we might be able to detect an IME sequenc starting in which case, stop
				// so we'll process a keyup that happens when the IME closes
				if (hX_5.brw.isNavOrMoz() && evt.keyCode && (evt.keyCode == 191 || evt.keyCode == 229) && evt.charCode!=null && evt.charCode==0)
					return false;
				// IE doesn't emit this event for fcn keys, Mozilla does so we need
				// to tell NS AGAIN to cancel the event if we get here on a fcn key
				// that was handled above
				if (keypress)
					r = keypress_r;
				else {
					keypress = true;
					// If this really is a keypress, then handle it
					// If assist is on, assist will process the character and add
					// it to the field
					if (behavior.assist && behavior.assist.onChangeChrCode) {
						r = behavior.assist.onChangeChrCode(evt);
						newR = false;
						if (r > 1) {	// Handle NS letting the newline/tab key here
							r = true;
							newR = true;
						} else if (r) {
							newR = doChange(evt);
							r = (r == -2) ? false : newR;
						}
						// Handle autotab when we DO have char-by-char processing on 
						// Assist has added the character to the value
						if (newR && (autotab > 0)) doAutoTab();
					// If we don't have assist on, we still want to test autotabbing
					// here (see blurb above for why).  However, the character isn't
					// in the value yet -- browsers add it in the post-handler for
					// the event (i.e., after we return).  So we set a fast timer
					// to do the autotab test immediately after this event (or as
					// close to it as possible)
					} else if (autotab > 0) autotabtimer = hX_5.i.startTimerTimed (obj.id, 0, "autotab", 1, hX_5.BHR_assist);
				}
				break;

			// We do as little as possible on a key-up.  Maybe we missed a keystroke
			// in the above in which case, in which case we'll handle it
			// NOTE:  We could get a keyup (with no keydown or keypress) IF the user
			// had a modifier key pressed when a field gets focus and releases it
			// in there (also could get a keypress w/o a keydown if they were
			// in "typematic" mode and they overflowed into a second field
			case hX_5._evKU:
				if (!focus) break;
				// A function key was pressed and we didn't catch it in keydown
				updating = false;
				if (behavior.assist && behavior.assist.onChangeKeyCode && (keydown && !keypress)) {
					// CTRL+C (copy) comes through in IE (though it shouldn't) so ignore it
					if (hX_5.brw.isIE() && evt.keyCode == 67)
						r = false;
					else {
						r = behavior.assist.onChangeKeyCode(evt);
						if (r) {
							newR = doChange(evt);
							r = (r == -2) ? false : newR;
						}
					}
				}
				keydown = keypress = false;
				break;

			// Mouse click 
			case hX_5._evMC:
				if (!focus) break;	
				keydown = keypress = updating = false;
				saveState();
				if (!sel) {
					if (behavior.assist && behavior.assist.onClick) {
						r = behavior.assist.onClick(evt);
						if (r == -2 && pos >= 0) {
							hX_5.sel.setCaretPos(obj, pos);
							r = false;
						}
					}
				}
				break;

			// Cut
			case hX_5._evCT:
				if (!focus) break;	
				keydown = keypress = updating = false;
				saveState();
				if (sel) {
					if (behavior.assist && behavior.assist.onCut) {
						r = behavior.assist.onCut(evt);
						if (r == -1) r = true;
						else if (r) {
							r = doChange(evt);
							r = (r == -2) ? false : r;
						}
					}
				}
				break;

			case hX_5._evCP:
				// We only trap and handle copy to make a copy of the
				// clipboard -- we don't process anything else
				if (focus && sel && behavior.assist && behavior.assist.onCopy) behavior.assist.onCopy(evt);
				break;

			// Paste
			case hX_5._evPST:
				if (!focus) break;	
				keydown = keypress = updating = false;
				saveState();
				if (behavior.assist && behavior.assist.onPaste) {
					newR = false;
					r = behavior.assist.onPaste(evt);
					if (r) {
						newR = doChange(evt);
						r = (r == -2) ? false : newR;
					}
					// Handle autotab when we DO have char-by-char processing on
					var didTab = false; 
					if (newR && (autotab > 0)) didTab = doAutoTab();
					// There's a "bug" in IE where selection is set wrong 
					// when the paste returns to the OS, so reset it using a "defer" 
					if (hX_5.brw.isIE() && newR && !didTab && pos >= 0) hX_5.sel.setCaretPos(obj, pos, null, true);
				}
				break;
		}
	return r;
	}

	// Public -- See if we should accept a change
	function doChange(evt) {
		var r = true;
		if (behavior.assist && behavior.assist.onValidate) r = behavior.assist.onValidate(evt);
		if (r) commit(evt);
		else rollback(evt);
		return r;
	}

	// Public -- Revert back to the saved value/caret
	function rollback (evt) {
		value = orgvalue;
		pos = orgpos;
		update(evt);
	} 

	// Public -- Accept the changes we've made and mark the caret
	function commit (evt) {
		if (!skipC && commitCallback) commitCallback(obj);
		update (evt);
	}

	// Private -- Do the real work of a rollback and commit
	function update (evt) {
		if (!updating) {
			updating = true;
			if (obj.value != value) obj.value = value;
			if (pos >= 0) hX_5.sel.setCaretPos(obj, pos);
			updating = false;
		}
	}

	// Private -- If the field is filled, go on to the next field
	function doAutoTab() {
		if (isFilled(hX_5.sel.getCaretPos(obj))) {
			if (autosubmit) {
				var a=[], t=[];
				a.push(hX_5.ACT_SUBMIT);
				t.push((behavior.autoSubmit.length > 0) ? behavior.autoSubmit : obj.form.id);
				return (hX_5.action.processAction (obj, null, null, a, true, t, null, null, null, true));
			} else 
				hX_5.gotoNextPrev(obj, true, true);
			return true;
		}
		return false;
	}

	// private
	function isFilled(curpos) {
		return ((behavior && behavior.assist && behavior.assist.isFilled)?behavior.assist.isFilled(curpos):(obj.value.length >= autotab && (curpos >= autotab)));
	}

	// Private -- Is this a "newline" which can cause a submit?
	function isSubmitFcnKey (evt) {
		var code = (evt.keyCode && (evt.keyCode != 0)) ? evt.keyCode : -1;
		if (code == 13)	if (!hX_5.isKeyBound(obj, "13", evt, hX_5.ACT_SUBMIT)) return true;
		return false;
	}

	// Private -- Is this a no-op?
	function isDisposableKey (evt) {
		var code = (evt.keyCode && (evt.keyCode != 0)) ? evt.keyCode : -1;
		return (hX_5.isKeyBound(obj, code+hX_5._E, evt, hX_5.ACT_NOTHING, hX_5.ACT_NONE, hX_5.ACT_STOP));
	}

	// Private -- Is this a function key we always ignore here ?
	function isSkipFcnKey (evt) {
		var code = (evt.keyCode && (evt.keyCode != 0)) ? evt.keyCode : -1;
		if (code == 16 || code == 17 || code == 18 || 	// shift, ctrl and alt
			code == 145 || code == 166 || code == 167)	// scroll lock, "menu left", "menu right" 
				return true;
		return false;
	}

	// Private -- Is this a function key that may end up generating a printable character later
	// This is an odd case where on Japanese and perhaps other keyboards, a key (e.g., the "YEN"
	// or "RED SLASH" key triggers a non-printable function (e.g., invokes IME) but it then
	// generates a keypress event with a printable character (a slash).
	function isPrintableFcnKey (evt) {
		var code = (evt.keyCode && (evt.keyCode != 0)) ? evt.keyCode : -1;
		return (code >= 223);
	}
}
