/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// ---------------------------------------------------------
// Extensions to JS String, Number and Date classes
// ---------------------------------------------------------

//------------------------------------------------------------------------------------------------------------
// (Small Number of) Globally defined color functions
HxG_5.prototype.HxICol = function () {
}

// Public: Convert a CSS "color" property to a triplet - R,G,B (decimal values)
// NOTE: does not handle color-names such as "black"
HxG_5.prototype.HxICol.prototype.CSStoRGB = function (c) {
	var i, x, r=0, g=0, b=0;
	if (c.charAt(0)==hX_5._LB) {
		r = parseInt(hX_5._HEX+c.substr(1,2));
		g = parseInt(hX_5._HEX+c.substr(3,2));
		b = parseInt(hX_5._HEX+c.substr(5,2));
	} else if (c.substr(0,3).toLowerCase()=="rgb") {
		x = hX_5.replaceAll(hX_5.replaceAll(c.substr(3),"(",hX_5._E),")",hX_5._E);
		i = x.indexOf(","); r = parseInt(x.substr(0,i),10); x = x.substr(i+1);
		i = x.indexOf(","); g = parseInt(x.substr(0,i),10); b = parseInt(x.substr(i+1),10);
	} else return null;
	return {R:r, G:g, B:b};
}

// Public: See if a "CSS" property represents a dark color or not
// NOTE: Does not use a proper RGB->HSL algorithm to compute this (i.e., compute the actual brightness of a color)
HxG_5.prototype.HxICol.prototype.isDark = function (c) {
	var cR=this.CSStoRGB(c);
	return ((cR==null)?false:((cR.R+cR.G+cR.B)<=400));
}

//------------------------------------------------------------------------------------------------------------
// Globally defined calendar functions used with the JS Date class.
// These functions convert JS dates to/from "calendrical" dates and/or manipulate JS dates or "calendrical" dates.
// A calendrical date is a triplet (y,m,d) represented in one of the three supported calendar systems -- Gregorian, Hijiri or Hebrew.
// So the base usage is:
//
// hX_5.cal.setCalendar (calendar-type)						// Set the calendar type for a set of operations to follow
// hX_5.cal.dateToCalendar (JS date) returns calendardate 	// Convert from a JS date object to a calendrical date triplet (y,m,d)
// hX_5.cal.calendarToDate (y,m,d,h,mn,s,mil) returns dt 	// Convert from a calendrical date and time to a JS date object
//
// Note 1: In a gregorian calendrical date, a negative year is equivalent to "BC", eg. -1 means 1 B.C.
// While in a Javascript date object (which is gregorian), a negative year is not offset to take into account "0",
// that, dt.getFullYear() for a date containing 1 B.C. will return 0.
//
// Note 2: Calendar functions do take into accout the "gap" in the Gregorian calendar that Java
// uses to indicate the dates "deleted" from the calendar when the Christian calendar switched from Julian to Gregorian representation
//
// Main (potentially public) routines that perform various calculations on a date:
//
// hX_5.cal.calendarToJulian(calendricalYear, calendricalMonth, calendricalDate) returns Julian Date 
// hX_5.cal.julianToDatePc (julianDate) 		returns JS Date y,m,d pieces
// hX_5.cal.daysInYear (calendricalYear) 		returns days in the year
// hX_5.cal.monthsInYear (calendricalYear) 		returns months in the year
// hX_5.cal.maxDaysInMonth () 					returns max days in any month for this calendar
// hX_5.cal.minDaysInMonth () 					returns min days in any month for this calendar
// hX_5.cal.daysInMonth (calMonth, calYear) 	returns days in the month
// hX_5.cal.isLeapYear (calendricalYear)		returns whether this is a leap year or not
// hX_5.cal.nearestLeapYear (calendricalYear) 	returns "nearest" leap year relative to specified year
// hX_5.cal.getDay = (JSdate)					returns the day of week for the specified JS date
// hX_5.cal.getJulianDay = (JSdate) 			returns the julian day of year for the specified JS date
// hX_5.cal.getDayOfWeekInMonth = (JSdate) 		returns the julian day of week in a month for the specified JS date
// hX_5.cal.getJulianWeek = (JSdate, FDOW) 		returns the julian week of the year for a specified JS date
// hX_5.cal.getJulianWeekOfMonth = (JSdate, FDOW) 	returns the julian week of the year for a specified JS date
// hX_5.cal.julianToMonthDay  = (d, calendricalYr) 	converts a calendrical julian day of year/yr to a calendrical month/day for the year
// hX_5.cal.twoYearToFourYear  = (cyr, today, offset)rounds a 2 digit gregorian year to the appropriate four digit year
// hX_5.cal.adjustDate = (JSdate, iYearInc, iMonthInc, iDayDif, iHourDif, iMinuteDif, iSecondDif)
//												Adds years, months, days, ... to a JS date

//------------------------------------------------------------------------------------------------------------

// Public: Before using any of the following, you MUST call setCalendar to set the computations to the correct calendar system
HxG_5.prototype.HxICal = function () {
	this.bIsGregorian = false;
	this.bIsHebrew = false;
	this.bIsHijiri = false;
	// A couple of helper arrays that summarize information about days/months
	this.hebrewAccumulatedMonthsPerCycle=[0,12,24,37,49,61,74,86,99,111,123,136,148,160,173,185,197,210,222];
	this.hijiriAccumulatedDaysPerCycle=[0,354,709,1063,1417,1772,2126,2481,2835,3189,3544,3898,4252,4607,4961,5316,5670,6024,6379,6733,7087,7442,7796,8150,8505,8859,9214,9568,9922,10277];
}

// Public: Set the calendar type (Gregorian, Hebrew, Hijiri) (set only one of these at a time)
HxG_5.prototype.HxICal.prototype.setCalendar = function (bIsGreg, bIsHb, bIsHj) {
	this.bIsHebrew = bIsHb;
	this.bIsHijiri = bIsHj;
	this.bIsGregorian = (!this.bIsHebrew && !this.bIsHijiri);
}

// Public: Given a calendrical year, get the number of days in a year (returns the maximum days in any year if year omitted)
HxG_5.prototype.HxICal.prototype.daysInYear = function (cyr) {
	if (this.bIsGregorian) return ((cyr==null||this.isLeapYear(cyr))?366:365);
	else if (this.bIsHijiri) return ((cyr==null||this.isLeapYear(cyr))?355:354);
	else return (hX_5.cal.calcRoshHaShannah(cyr+1)-hX_5.cal.calcRoshHaShannah(cyr));
}

// Public: Given a calendrical year, get the number of months in a year (returns the maximum months in any year if year omitted)
HxG_5.prototype.HxICal.prototype.monthsInYear = function (cyr) {
	return ((!this.bIsHebrew)?12:(cyr==null||this.isLeapYear(cyr)?13:12));
}

// Public: Get the maximum days in any month
HxG_5.prototype.HxICal.prototype.maxDaysInMonth = function () {
	return ((this.bIsGregorian)?31:30);
}

// Public: Get the minimum days in any month
HxG_5.prototype.HxICal.prototype.minDaysInMonth = function () {
	return ((this.bIsGregorian)?28:29);
}

// Given a calendrical year/month, get the number of days in a specific month
HxG_5.prototype.HxICal.prototype.daysInMonth = function (cm,cyr) {
	// 30 days has September, April, June and November, all the rest have 31 except ...
	if (this.bIsGregorian) return ((cm>11)?99:((1==cm)?(this.isLeapYear(cyr)?29:28):((cm==3||cm==5||cm==8||cm==10)?30:31)));
	// Hijiri -- lunar months alternate between 30/29 days except in leap year
	if (this.bIsHijiri) return ((cm>11)?99:((11==cm)?(this.isLeapYear(cyr)?30:29):((cm%2==1)?29:30)));
	// Hebrew is more complex, it's basically the same alternation of 30/29 days except
	// for "deficient" and "complete" years where there is one less day in Kiselv and one more day in Heshvan
	// See table below for the details
	else {
		var dYr, l=this.isLeapYear(cyr), r=(cm%2==1)?29:30;
		if (cm>11) r=(!l)?99:((cm>12)?99:29);
		else if (cm>10) r=(l)?30:29;
		else if (cm==7 || cm==8) {
			dYr = this.daysInYear(cyr);
			r += (((cm==7) && (dYr==355 || dYr==385))?1:0);
			r -= (((cm==8) && (dYr==353 || dYr==383))?1:0);
		}
		return r;
	}
}

// Given a Gregorian date, convert it to a calendrical date (i.e., a lunar date if the calendar is non-Gregorian)
HxG_5.prototype.HxICal.prototype.dateToCalendar = function (dt) {
	var l,mx,yD,y,m,d;
	if (this.bIsGregorian) {
		y=dt.getAdjYear(); m=dt.getMonth(); d=dt.getDate();
	} else if (this.bIsHijiri) {
		yD = this.gregorianToJulian(dt)-1948440; // Julian date rebased to year of Haj = July 16 622 AD)
		if (yD < 0) return {year:1, month:0, date:1};
		// Compute years up to 30-year lunar cycle containing this date
		// Each 30 year cycle contains 354*30 + 11 days
		y = Math.floor(yD/10631);	// Lunar 30 year cycle expressed as days (354*30 + 11 leap days)
		y = (y*30)+1;
		// For the remaining days, calc the years they represent within this cycle
		l=yD%10631; mx=this.daysInYear(y);
		while (l>=mx) {
			y++; l-=mx;
			mx = this.daysInYear(y);
		}
		// Compute month represented by remaining days
		m=0; d=l; mx=this.daysInMonth(m,y);
		while (d>=mx) {
			m++; d-=mx;
			mx=this.daysInMonth(m,y);
		}
		d++; // 1-base the remaining days
	} else {
		// Estimate the Hebrew year that corresponds to the Gregorian year and calc it's Julian date
		yD = this.gregorianToJulian(dt);
		if (yD < 347997) return {year:1, month:6, date:1};
		y=dt.getFullYear()+3761; m=6; d=1;
		l = this.calendarToJulian(y,m,d);
		// The estimate could well be "beyond" the current Julian date, so back up one year at a time
		// until it's less than the current date
		while (yD < l && l >= 0) {
			y--; l=this.calendarToJulian(y,m,d);
		}
		// Now that we're before the current date, walk forward one year at a time until we 
		// are on the right year (that is there are fewer days remaining than the length of the year)
		yD = yD-l;
		mx = this.daysInYear(y);
		while (yD>=mx) {
			y++; yD-=mx;
			mx = this.daysInYear(y);
		}
		// Given the remaining days, compute the month.  Remember that New Year
		// is the 7th month not the first
		m=6; mx=this.daysInMonth(m,y);
		while (yD>=mx) {
			m++; yD-=mx;
			m = (m>=this.monthsInYear(y))?0:m;
			mx = this.daysInMonth(m,y);
		}
		// Now left with the days in month (1-base it)
		d = yD+1;
	}
	return {year:y, month:m, date:d};
}

// Given a calendrical date, convert a it to a Gregorian date
HxG_5.prototype.HxICal.prototype.calendarToDate = function (y,m,d,h,mn,s,mls) {
	var o,r;
	if (this.bIsGregorian) o=this.calendarToDatePc(((y<0)?y+1:y),m,d);
	else o=this.calendarToDatePc(y,m,d);
	r = new Date(o.year, o.month, o.date, h, mn, s);
	// If year is between -99 and 99, the constructor adds 1900 to it so y is always wrong, m/d are wrong in leap years
	r.setFullYear(o.year); r.setMonth(o.month); r.setDate(o.date);
	if (mls && mls>0) r.setMilliseconds(mls);
	return r;
}	

// Given a calendrical date, convert a it to Gregorian date pieces
HxG_5.prototype.HxICal.prototype.calendarToDatePc = function (y,m,d) {
	if (this.bIsGregorian) return {year:y, month:m, date:d};
	return this.julianToDatePc(this.calendarToJulian(y,m,d));
}


// Given a calendrical date, convert it to a Julian date
HxG_5.prototype.HxICal.prototype.calendarToJulian = function (y,m,d) {
	var jD,j,mX;
	if (this.bIsGregorian) jD = this.gregorianToJulian(y,m,d);
	else if (this.bIsHijiri) {
		if (y < 0) return 0;
		jD = Math.floor((y-1)/30)*10631;
		jD += this.hijiriAccumulatedDaysPerCycle[(y-1)%30];
		for (j=0; j<m; j++) jD += this.daysInMonth(j,y);
		jD += (d-1+1948440);
	}
	else {
		if (y < 0) return 0;
		// Calc the Julian date of Rosh HaShannah relative to 1 Anno Mundi
		jD = this.calcRoshHaShannah(y);
		// Now add in the days in this year, remember that "New year" is in the middle of the year, 
		mX = (m>=6)?m:(this.monthsInYear(y));
		for (j=6; j<mX; j++) jD += this.daysInMonth(j,y);
		if (mX!=m) for (j=0; j<m; j++) jD += this.daysInMonth(j,y);
		jD += (d+347995);
	}
	return jD;
}

// Convert a Julian date to a Gregorian one (returns year, month and day)
HxG_5.prototype.HxICal.prototype.julianToDatePc= function (jd) {
	// Julian Dates are relative to January 1, 4713 BCE.  Algorithm derived from http://aa.usno.navy.mil/data/docs/JulianDate.html
	var i,j,x,z,y,d,m;
	if (jd < 0) return {year:-4713, month:0, date:1};
	// Handle correction for the Gregorian change-over on 4 October 1582
	if (jd > 2299160) {
		y = Math.floor(((jd-1867216)-0.25)/36524.25);
		x = jd + y + 1 - Math.floor(0.25*y);
	} else x = jd;
	// Now do the grotty math
	y = x + 1524.0;
	z = Math.floor(6680.0 + ((y-2439870.0)-122.1)/365.25);
	i = Math.floor(z*365.25);
	j = Math.floor((y-i)/30.6001);
	d = Math.floor(y-i-Math.floor(j*30.6001));
	m = Math.floor(j-1.0);		if (m > 12) m -= 12;
	y = Math.floor(z-4715.0);	if (m > 2) y--;
	return {year:y, month:m-1, date:d};
}

// Given a gregorian date (or a y,m,d): return the Julian Date of the date (takes either a date or a Y/M/D)
HxG_5.prototype.HxICal.prototype.gregorianToJulian = function(dt, iM, iD) {
	// Julian Dates are relative to January 1, 4713 BCE.  Algorithm derived from http://aa.usno.navy.mil/data/docs/JulianDate.html
	// Includes Gregorian change-over on 4 October 1582
	var g=(15+31*(10+12*1582)), jd, xy, xm, x, y, m, d;
	if (typeof(dt)=="object") {y=dt.getFullYear(); m=dt.getMonth()+1; d=dt.getDate();} else {y=dt;(y<0)?y++:y;m=iM+1;d=iD;}
	if (y <= -4713) return 0;
	if (y==1582 && m==10 && d>4 && d<15) d=4;
    if (m > 2) {
        xy = y;
        xm = m + 1;
    } else {
        xy = y - 1;
        xm = m + 13;
    }
	jd = Math.floor(Math.floor(365.25*xy) + Math.floor(30.6001*xm) + d + 1720995);
    if ((d+31*(m+12*y)) >= g) {
        x = Math.floor(0.01*xy);
        jd += (2 - x + Math.floor(0.25*x));
    }
	return jd;
}

// Given a calendrical date, determine if it is a leap year or not
HxG_5.prototype.HxICal.prototype.isLeapYear = function (cyr) {
	var y = (cyr<0)?cyr+1:cyr;
	if (this.bIsGregorian) return (((0==y%4) && (y<1582 || 0!=(y%100))) || (0==y%400));
	else return (this.isLunarLeap(y%((this.bIsHijiri)?30:19)));
}

// Given a calendrical date, determine the 'nearest" leap year
HxG_5.prototype.HxICal.prototype.nearestLeapYear = function (cyr) {
	var y, yc;
	if (this.bIsGregorian) {
		y = (cyr<0)?cyr++:cyr;
		y = Math.floor(y/4)*4;
		y = ((y<1582) || (0!=y%100) || (0==y%400))?y:(y+4);
		return ((y==0)?((cyr<0)?-4:4):y);
	} else if (this.bIsHijiri) {
		y=cyr; yc = y%30;
		while (!this.isLunarLeap(yc)) {
			y++;yc++;
		}
		return y;
	} else {
		y=cyr; yc = y%19;
		while (!this.isLunarLeap(yc)) {
			y++;yc++;
		}
		return y;
	}
}

// Given a cycle in a lunar calendar, compute if it's a leap year or not
HxG_5.prototype.HxICal.prototype.isLunarLeap = function (c) {
	if (this.bIsHijiri)	return (c==2 || c==5 || c==7 || c==10 || c==13 || c==15 || c==18 || c==21 || c==24 || c==26 || c==29);
	return (c==0 || c==3 || c==6 || c==8 || c==11 || c==14 || c==17);
}

// Given a Gregorian date, get the day of week for the current calendar type
HxG_5.prototype.HxICal.prototype.getDay = function (dt) {
	// N.B. differs from doing a getDay on the date in that this matches Java's skipping of days in 1582 AD
	// Convert to Julian day, adjust as Julian Day 1 isn't a Sunday then modulo 7 is the day of week
	return (Math.abs((this.gregorianToJulian(dt)+1)%7));
}

// Given a Gregorian date, get the Julian day of year (for the current calendar)
HxG_5.prototype.HxICal.prototype.getJulianDay = function (dt) {
	var i, d1, d2, m=dt.getMonth(), d=dt.getDate(), y=dt.getAdjYear();
	if (this.bIsGregorian) {
		d1 = this.gregorianToJulian(y,0,1); d2 = this.gregorianToJulian(y,m,d);
	} else {
		var o = this.dateToCalendar(dt);
		d1 = this.calendarToJulian(o.year, 0, 1); d2 = this.calendarToJulian(o.year, o.month, o.date);
	}
	return (d2-d1+1);
}

// Given a Gregorian date, get the count of day of Week In Month (for the current calendar)
HxG_5.prototype.HxICal.prototype.getDayOfWeekInMonth = function (dt) {
	var j=0, d=(this.bIsGregorian)?(dt.getDate()):((this.dateToCalendar(dt)).date);
	var k = Math.floor((d-1)/7)+1;
	while (d > 0) {j++; d -= 7;}
	return (j);
}

// Given a Gregorian date, get the Julian week within the year (for the current calendar)
// Note that this returns week and year as dates at the beginning of the year may be in the "first week"
// of the next year.
// That is, this is the "Java" definition of Julian weeks = the first week ends on the first Saturday so may 
// be < 7 days in length relative to the current year.  This means Jan 1 is always in week 1 of this year, 
// but Dec 31 of the last year could be in week 1 of this year 
HxG_5.prototype.HxICal.prototype.getJulianWeek = function (dt, FDOW) {
	var ofd, off, fd, fdw=(FDOW!=null)?FDOW:0, o=this.dateToCalendar(dt), y=o.year, d=this.getJulianDay(dt)-1, yD=this.daysInYear(y)-1;
	// If this is one of the last days of the year, for the week calc, we could be part of the first week of the next year
	if (d>=yD-6) {
		ofd=this.calendarToJulian(y+1,0,1), fd=(Math.abs((ofd+1)%7))
		off = (fdw>fd)?((fd+7)-fdw):(fd-fdw);
		if (off > (yD-d)) return {yr:y+1, wk:1};
	}
	// Not in next year, so calc it using this year
	ofd=this.calendarToJulian(y,0,1), fd=(Math.abs((ofd+1)%7))
	off = (fdw>fd)?((fd+7)-fdw):(fd-fdw);
	d = d + off;
	if (d >= 0)
		return {yr:y, wk:(Math.floor(d/7)+1)};
	else
		return {yr:-1, wk:-1};
}

// Given a Gregorian date, get the Julian week within the month
HxG_5.prototype.HxICal.prototype.getJulianWeekOfMonth = function (dt, FDOW) {
	// N.B. This is the "Java" definition of Julian weeks in month that is
	// the first week ends on the first last-day-of-the-week (e.g., first Saturday) and thus
	// may be less than 7 days in length
	var o=this.dateToCalendar(dt), ofd=this.calendarToJulian(o.year,o.month,1), fd=(Math.abs((ofd+1)%7)), f=(FDOW==0)?7:FDOW, d=o.date-1-(f-fd);	
	return (Math.floor(d/7)+2);
}

// Given a calendrical day-of-year and year, convert it to a calendrical month/day pair
HxG_5.prototype.HxICal.prototype.julianToMonthDay = function (cjd, cyr) {
	var m=0, d=cjd, dInM=this.daysInMonth(m,cyr);
	while (d > dInM) {
		d-=dInM; m++;
		dInM = this.daysInMonth(m,cyr);
	}
	return {month:m, date:d};
}

// Convert a "two digit year" to a "four digit year" using calendrical years
// Input year is a calendrical year (not gregorian)
HxG_5.prototype.HxICal.prototype.twoYearToFourYear = function (cyr, today, offset) {
	var y=cyr, o=this.dateToCalendar(today);
	y += ((Math.floor(o.year/100)*100)-100);
	if (y <= offset) y += 100;
	return y;
}

// Date overload:  Adjust a date by the specified years, months, days, ...
// NOTE however, that you can only inc/dec the year and month by "1".  Day can be changed'd by up to a week (7 days)
HxG_5.prototype.HxICal.prototype.adjustDate = function (dt, iYearInc, iMonthInc, iDayDif, iHourDif, iMinuteDif, iSecondDif) {
	var o, x, msDif=0, bResetTime=false, iOrigHour=dt.getHours(), iOrigMinute=dt.getMinutes(), iOrigSecond=dt.getSeconds();
	if (!iHourDif && !iMinuteDif && !iSecondDif && iDayDif) {
		// Set the hours to 4AM or 8PM to try to avoid an off by day problem during DST shifts. 
		bResetTime = true;
		dt.setHours(dt.getTimezoneOffset < 0 ? 4 : 20);
	}
	if (iSecondDif) msDif += iSecondDif * 1000;
	if (iMinuteDif) msDif += iMinuteDif * 60000;
	if (iHourDif) msDif += iHourDif * 3600000;
	if (iDayDif) msDif += iDayDif * 86400000;
	if (msDif) dt.setTime(dt.getTime() + msDif);

	o = this.dateToCalendar(dt);
	if (iYearInc) o.year += iYearInc;
	if (iMonthInc) o.month += iMonthInc;
	x = this.monthsInYear(o.year);
	if (o.month >= x) {
		o.month = (o.month-x);
		o.year++;
	}
	else if (o.month < 0) {
		o.year--;
		x = this.monthsInYear(o.year);
		o.month = x+o.month;
	}
	x = this.daysInMonth(o.month, o.year);
	if (o.date > x) o.date = x;
	o = this.calendarToDatePc(o.year, o.month, o.date);
	// Set it twice -- otherwise the value gets messed up in cases where the current day is > x above
	dt.setFullYear(o.year);	dt.setMonth(o.month); dt.setDate(o.date);
	dt.setFullYear(o.year);	dt.setMonth(o.month); dt.setDate(o.date);
	if (bResetTime) {
		dt.setHours(iOrigHour);
		dt.setMinutes(iOrigMinute);
		dt.setSeconds(iOrigSecond);
	}
	return dt;
}

// Hebrew calendar: given a calendrical year, find Rosh Ha-Shanah (the first day of the year)
HxG_5.prototype.HxICal.prototype.calcRoshHaShannah = function (cyr) {
	// First find the date and time of the new moon (or molad) For this purpose, the lunar cycle is assumed to be 
	// 29 days 12 hours and 793 halakim. A heleq (singular form of halakim) is 1/1080th of an hour or 3 1/3 seconds. 
	//
	// The first molad of year 1 occurred on Sunday at 11:11:20 P.M. While sunset varies, the day is assumed to begin at 6:00 P.M. 
	// for calendar calculation purposes. So the first molad was 5 hours 204 halakim after the start of Tishri 1, 0001 
	// (which is September 7, 3761 B.C.) All subsequent molads can be calculated from this starting point by adding the 
	// length of a lunar cycle (19 year cycle)
	var dow, lunarC=Math.floor((cyr-1)/19), y=Math.floor((cyr-1)%19), om=this.calcMoladOfLunarCycle(lunarC);
	om.moladHalakim += 765433 * this.hebrewAccumulatedMonthsPerCycle[y];
	om.moladDay += Math.floor(om.moladHalakim/25920);
	om.moladHalakim = om.moladHalakim%25920;
	// y = year-in-cycle, om.moladDay = day (counting from day 1) of the molad, om.moladHalakim is the "time" on that day
	// The dehiyyot are a set of rules applied to the first new moon of the year which
	// after application result in the date of Rosh Ha Shannah.  Each rule can delay
	// the start of the new year by one day.  However, rule one applies "after" the
	// other rules, meaning it can be delayined two days.
	// 1) Tishri 1 must never be Sunday, Wednesday or Friday. 
	// 2) If the molad occurs on or after noon, Tishri 1 must be delayed. 
	// 3) If it is not a leap year year and the molad occurs on Tuesday at or after 3:11:20 A.M., Tishri 1 must be delayed. 
	// 4) If it is the year following a leap year and the molad occurs on Monday at or after 9:32:43 and 1/3 sec, 
	//    Tishri 1 must be delayed. 
	// Rule 2
	dow=(om.moladDay%7);
	if /* Rule 2 */ ((om.moladHalakim >= 19440) || /* Rule 3 */ (!(y==2 || y==5 || y==7 || y==10 || y==13 || y==16 || y==18) && dow == 2 && om.moladHalakim >= 9924) ||
   	   /* Rule 4 */ ((y==3 || y==6 || y==8 || y==11 || y==14 || y==17 || y==0) && dow==1 && om.moladHalakim >= 16789)) 
	/* Adjust by 1 */ om.moladDay++;
	// Now apply Rule 1
	dow=(om.moladDay%7); if (dow == 3 || dow == 5 || dow == 0) om.moladDay++;
	return om.moladDay;
}

// Hebrew calendar: Given a lunar cycle year number, calc the new moon that starts the cycle (molad)
HxG_5.prototype.HxICal.prototype.calcMoladOfLunarCycle = function (lunarCycle /* zero-based */) {
	// Each new moon occurs roughly every 29 1/2 days.  So if you know the date of the "first" new moon in this
	// calendar, then you can compute out the new moon of a given year.  The tricky bit is the "roughly" every 29 1/2 days.
	// In the Hebrew calendar, each 19 years is exactly 235 lunar months.  And a lunar month is  29 days 12 hours and 793 halakim. 
	// Where a heleq (one halakim) is 1/1080th of an hour or 3 1/3 seconds.  Thus each 19 year cycle is 179876755 halakim in length.
	// And the first molad of year 1 occurred on Sunday at 11:11:20 P.M. While sunset varies, the day is assumed to begin at 6:00 P.M. 
	// for calendar calculation purposes. So the first molad was 5 hours 204 halakim (=5604))after the start of Tishri 1, 0001 
	// (which is September 7, 3761 B.C.) 
	// In other words, the base (31524 halakim) + lunarCycle*179876755 is the total halakim denoting this new moon
	// We then break down the total halakim into days (halakim/25920
	var h=31524+(179876755*lunarCycle);
	return {moladDay:(Math.floor(h/25920)), moladHalakim:(h%25920)};
/*
	The above is could be subject to rounding error, the following may be more accurate:
	var d1,d2,h1=31524+(lunarCycle*(179876755 & 0xFFFF)), h2 = h1>>16;
	h2 += lunarCycle*((179876755 >> 16) & 0xFFFF);
	d2 = Math.floor(h2/25920);
	h2 -= (d2*25920);
	h1 = (h2 << 16) | (h1 & 0xFFFF);
	d1 = Math.floor(h1/25920);
	return {moladDay:((d2 << 16) | d1), moladHalakim:(h1-(d1*25920))};
*/	
}
/*	Note on the number of days in a month for the Hebrew calendar

                      COMMON YEAR        	  LEAP YEAR
               Deficient Reg.  Complete Deficient Reg.  Complete
         1 Nisan     30   30   30       	  30   30   30
         2 Iyyar     29   29   29       	  29   29   29
         3 Sivan     30   30   30       	  30   30   30
         4 Tammuz    29   29   29      		  29   29   29
         5 Av        30   30   30      	      30   30   30
         6 Elul      29   29   29             29   29   29

         6 Tishri    30   30   30             30   30   30
         7 Heshvan   29   29   30             29   29   30 (variable)
         8 Kislev    29   30   30             29   30   30 (variable)
         9 Tevet     29   29   29             29   29   29
         10 Shevat   30   30   30             30   30   30
         11 Adar I   29   29   29             30   30   30 (variable)
         12 Adar II  --   --   --             29   29   29 (optional)
                    ---  ---  ---            ---  ---  ---
                    353  354  355            383  384  385
*/    

//-------------------------------------------------------------------
// JS Date Object
//-------------------------------------------------------------------

// Date Overload:  Clone a new datetime object
//				   dt = 0 means clone only date portion (setting time to noon)
//				   dt = 1 means clone only time portion (setting date to 1/1/this year)
//				   dt = 2 means clone date and time
Date.prototype.getAdjYear = function(dt) {
	var y = this.getFullYear();
	return ((y<=0)?y-1:y);
}
// Date Overload:  Clone a new datetime object
//				   dt = 0 means clone only date portion (setting time to noon)
//				   dt = 1 means clone only time portion (setting date to 1/1/this year)
//				   dt = 2 means clone date and time
Date.prototype.Hx4Clone = function(dt) {
	var r, y = this.getFullYear();
	if (dt != null && dt >= 2)  {
		r = new Date (this.getFullYear(), this.getMonth(), this.getDate(), this.getHours(), this.getMinutes(), this.getSeconds());
	} else if (dt != null && dt == 1)
		r = new Date (this.getFullYear(), 0, 0, this.getHours(), this.getMinutes(), this.getSeconds());
	else
		r = new Date (this.getFullYear(), this.getMonth(), this.getDate(), 12, 0, 0);
	r.setFullYear(y);
	return (r);
}

// Date overload: Compare date against another date
// 				  Returns 0 if dates are equal
//						  1 if this date object > oDate
//						 -1 if this date object < oDate
Date.prototype.Hx4CompareDate = function( oDate, bCompareTime ) {
	var iDiff = this.getFullYear() - oDate.getFullYear();
	if (iDiff==0) iDiff = this.getMonth() - oDate.getMonth();
	if (iDiff==0) iDiff = this.getDate() - oDate.getDate();
	if (bCompareTime && iDiff==0) {
		iDiff = this.getHours() - oDate.getHours();
		if (iDiff==0) iDiff = this.getMinutes() - oDate.getMinutes();
		if (iDiff==0) iDiff = this.getSeconds() - oDate.getSeconds();
	}
	if (iDiff < 0) iDiff = -1;
	else if (iDiff > 0) iDiff = 1;
	return iDiff;
}
