/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2007 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// ------------------------------------------------
// Base Class for JSF Behaviors
//
// JSF Behaviors assign a "behavior" to a DOM event
// ------------------------------------------------

// Public: Constructor
HxG_5.prototype.JSFBehavior = function () {
	this.beforeOthers = true;
	this.suppressAttach = false;
}

// Public: Page level onLoad handler
HxG_5.prototype.JSFBehavior.prototype.onPageLoad = function () {
	// Set the HTML onto the page
	this.setEvent();
	return true;
}

// Public: Set an "attribute" on the behavior.
HxG_5.prototype.JSFBehavior.prototype.setAttribute = function(a) {
	hX_5.parseArg (this.attributeTable, this, (arguments.length>1)?arguments:a);
	if (this.setAttributeCode) this.setAttributeCode();
	// No need to re-emit the HTML. It's fine as is
	return true;
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.JSFBehavior.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public: Set the event on the object (if it hasn't already been set)
HxG_5.prototype.JSFBehavior.prototype.setEvent = function () {
	if (this.DOMobj != null) {
		if (!this.HTMLrendered && !(this.eventname==hX_5._onL || this.suppressAttach)) {
			if (!this.beforeOthers) hX_5.attachBehaviorEvent(this.DOMobj, this.type, this.eventname, hX_5._lN);
			else hX_5.attachBehaviorEvent(this.DOMobj, this.type, this.eventname, hX_5._lP);
		}
		if (this.setEventCode) this.setEventCode();
		this.HTMLrendered = true;
		return true;
	}
	return false;
}

// Public: Base-level destroyer for all behaviors
HxG_5.prototype.JSFBehavior.prototype.destroy = function () {
	var e,i,d = [this.actions,this.targets];
	if (this.destroyCode) this.destroyCode();
	if (this.extraEvents) {
		while (this.extraEvents.length > 0) {
			e = this.extraEvents.pop();
			hX_5.removeEvent(e[0],e[1],null,true);
		}
		delete this.extraEvents;
	}
	if (this.extraIds) {
		while (this.extraIds.length > 0) {
			e = this.extraIds.pop();
			hX_5.removeEvent(e[0],e[1],e[2]);
		}
		delete this.extraIds;
	}
	for (i=0; i<d.length; i++) {
		if (d[i]) {
			while (d[i].length>0) d[i].pop();
			delete d[i];
		}
	}
	delete d;
	if (this.attributeTable) delete this.attributeTable;
	return true;
}

// -------------------------------------------------------
// JSF Behavior - Generic
//
// Generic behavior that can be attached to any event handler
// Used to run "actions".
// 
// Arguments:
//
//	see documentation.
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorGeneric = function () {
	this.type = hX_5.BHR_generic;
	this.beforeOthers = false;

	// Arguments
	this.fcn	= null;	// func1() ... etc
	this.actions= [];	// "lock"/"unlock"/func1() ... etc
	this.targets= [];	// action target ID

	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['action']		= ['actions',	hX_5.ARG_TO_NAU,	false,	null,	null];
		this.attributeTable ['function']	= ['fcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['target']		= ['targets',	hX_5.ARG_TO_NAN,	false,	null,	null];
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorGeneric.prototype = new hX_5.JSFBehavior();

// Private:  Override of Set Event to handle special on load case
HxG_5.prototype.JSFBehaviorGeneric.prototype.setEventCode = function() {
	if (!this.HTMLrendered && this.eventname == hX_5._onL) {
		var evt;
		if (document.createEventObject!=undefined) {
			evt = document.createEventObject();
			evt.type = hX_5._onL;
		} else {
			evt = document.createEvent("HTMLEvents");
			evt.initEvent(hX_5._onL, true, true);
		}
		return hX_5.action.processAction(this.DOMobj, evt, this.fcn, this.actions, true, this.targets, null, null, null);
	}
	return true;
}

// Public: Execute the action
HxG_5.prototype.JSFBehaviorGeneric.prototype.dispatchBehavior = function (thisObj, evt) {
	return ((this.DOMobj != null) && (evt != null)) ? hX_5.action.processAction(this.DOMobj, evt, this.fcn, this.actions, true, this.targets, null, null, null) : true;
}

// -------------------------------------------------------
// JSF Behavior - Keybind
//
// Run a user-specified behavior when a specified key-sequence is typed
// 
// Arguments:
//
//	see documentation.
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorKeybind = function () {
	this.type = hX_5.BHR_keybind;

	// NOTE:  Since a component can be bound to multiple keystrokes
	// (and a keystroke can have multiple bindings, e.g., shifted, alt)
	// the keybind behavior object consists of an array of key description
	// subcomponents.  When a new behavior is added, if the keybind 
	// behavior already exists for this component, the behavior adds itself
	// as a subcomponent of the existing behavior
	this.keyArray = [];

	// Attributes set by the caller -- passed into the key array
	// Keycode, modifier such as "ALT", "CTRL", "SHIFT", func1() ... etc
	this.keyCode = this.modifier = this.fcn = null;
	this.actions= [];		// "lock"/"unlock"/func1() ... etc
	this.targets= [];		// action target ID
	this.cancelBubble = true;// cancel default action
	this.retainFocus = false;// put focus back after action
	this.coBehavior = -1;	// Is there an assist cobehavior?

	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['keycode']		= ['keyCode',	hX_5.ARG_TO_NUM,	false,	1,		999999];
		this.attributeTable ['modifier']	= ['modifier',	hX_5.ARG_TO_NAU,	false,	null,	null];
		this.attributeTable ['action']		= ['actions',	hX_5.ARG_TO_NAU,	false,	null,	null];
		this.attributeTable ['function']	= ['fcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['target']		= ['targets',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['cancel-bubble']	= ['cancelBubble',	hX_5.ARG_IS_TRUE,	false,	null,	null];
		this.attributeTable ['retain-focus']	= ['retainFocus',	hX_5.ARG_IS_TRUE,	false,	null,	null];
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorKeybind.prototype = new hX_5.JSFBehavior();

// Augment: Description of a single key action
HxG_5.prototype.JSFKey = function () {
	this.keyCode = null;		// Keycode
	// Modifiers (> 1 is allowed)
	this.modAlt = this.modShift = this.modCtrl = false;
	// Prebuilt actions like lock, unlock, select, ..., their targets the functions
	this.actions = this.targets = this.fcn = null;
	this.cancelBubble = true;	// cancel default action
	this.retainFocus  = false;	// retain focus after executing non-focus related actions or any function
}

// Augment: Used to mutate a behavior into a "sub-behavior", that is, a <tag><event> pair
// can have lots of keycode "sub-behaviors"
HxG_5.prototype.JSFBehaviorKeybind.prototype.addSubComponent = function(obj) {
	var sub = new hX_5.JSFKey();
	sub.keyCode	= obj.keyCode;
	if (obj.modifier) {
		obj.modifier = obj.modifier;
		sub.modAlt	= (obj.modifier.indexOf("ALT") >= 0);
		sub.modShift= (obj.modifier.indexOf("SHIFT") >= 0);
		sub.modCtrl	= (obj.modifier.indexOf("CTRL") >= 0);
	}
	if (obj.actions)		sub.actions	= obj.actions;
	if (obj.targets)		sub.targets	= obj.targets;
	if (obj.fcn) 			sub.fcn 	= obj.fcn;
	if (obj.cancelBubble)	sub.cancelBubble= obj.cancelBubble;
	if (obj.retainFocus)	sub.retainFocus	= obj.retainFocus;
	this.keyArray.push(sub);

	// This is scuzzy, but we'll do it here (and never remove it).
	// If this is binding to an F1 key, turn off help (across the board)
	if (obj.keyCode == 112) document.onhelp = function () { return false; }	
}

HxG_5.prototype.JSFBehaviorKeybind.prototype.setEventCode = function () {
	// We only do this once -- when behavior is first run
	if (!this.HTMLrendered)
		if (hX_5.brw.isNavOrMoz()) hX_5.attachBehaviorEvent(this.DOMobj, this.type, hX_5._onKP, hX_5._lN, hX_5._onKD);
}

// Override:  Attribute handling
HxG_5.prototype.JSFBehaviorKeybind.prototype.setAttributeCode = function() {
	var obj, i, found=false;
	for (i=0; i < this.keyArray.length; i++) {
		obj = this.keyArray[i];
		if (obj.keyCode==this.keyCode && (this.modifier==null || (this.modifier!= null && obj.modifier && obj.modifier==this.modifier))) {
			found=true;
			if (this.actions)		obj.actions	= this.actions;
			if (this.targets)		obj.targets	= this.targets;
			if (this.fcn) 			obj.fcn 	= this.fcn;
			if (this.cancelBubble)	obj.cancelBubble= this.cancelBubble;
			if (this.retainFocus)	obj.retainFocus	= this.retainFocus;
		}
	}
	if (!found) this.addSubComponent(this);
}

// NOTE: This is defined on hX_5 (not the keybind)
// Public: See if a key is bound
HxG_5.prototype.isKeyBound = function(obj, keyname, evt, action, action1, action2) {
	if (obj != null && keyname != null && this.isString(keyname)) {
		var a, b, c, s, i, j, k, objid;
		for (i=0; i < 2; i++) {
			objid = (i==0) ? ((obj.tagName != hX_5._FRM) ? obj.id : null) : ((obj.form != null && obj.form.tagName == hX_5._FRM) ? obj.form.id : null)
			b = (objid != null) ? (this.getBehaviorById(objid, hX_5.BHR_keybind, hX_5._evKD)) : null;
			// N.B. We transmogrified the Netscape onkeydown into onkeypress
			b = (b != null) ? b : this.getBehaviorById(objid, hX_5.BHR_keybind, hX_5._evKP);
			if (b != null) {
				for (k=0; k < b.keyArray.length; k++) {
					if (b.keyArray[k].keyCode == keyname)
						if (evt == null)
							return true;
						else {
							a = ((evt.altKey) ? true : false), c = ((evt.ctrlKey) ? true : false), s = ((evt.shiftKey) ? true : false);
							if ((a == b.keyArray[k].modAlt) && (s == b.keyArray[k].modShift) && (c == b.keyArray[k].modCtrl)) {
								if (action) {
									for (j=0;j<b.keyArray[k].actions.length;j++) {
										if (action==b.keyArray[k].actions[j]) return true;
										if (action1!=null && action1==b.keyArray[k].actions[j]) return true;
										if (action2!=null && action2==b.keyArray[k].actions[j]) return true;
									}
									return false;										
								}
								return true;
							}
						}
				}
			}
		}
	}
	return (false);
}

// Public: Destroyer 
HxG_5.prototype.JSFBehaviorKeybind.prototype.destroyCode = function () {
	var o;
	while (this.keyArray.length>0) {
		o = this.keyArray.pop();
		delete o;
	}
}

// Public: Execute the action
HxG_5.prototype.JSFBehaviorKeybind.prototype.dispatchBehavior = function (thisObj, evt) {
	var result = true;
	if ((this.DOMobj != null) && (evt != null) && (this.keyArray.length > 0)) {
		var i, j, t, b, a = ((evt.altKey) ? true : false), c = ((evt.ctrlKey) ? true : false), s = ((evt.shiftKey) ? true : false);
		if (this.coBehavior==-1) this.coBehavior = hX_5.getBehaviorById(this.DOMobj.id, hX_5.BHR_assist, hX_5._onIN);
		for (i=0; i < this.keyArray.length && result; i++) {
			if ((evt.keyCode == this.keyArray[i].keyCode) && (a == this.keyArray[i].modAlt) && (s == this.keyArray[i].modShift) && (c == this.keyArray[i].modCtrl)) {
				if ((this.keyArray[i].actions!=null && this.keyArray[i].actions.length>0 && this.keyArray[i].actions[0]!=null) || this.keyArray[i].fcn != null) {
					if (evt.type == hX_5._evKP)
						result = false;
					else {
						b = false;
						if (this.coBehavior!=null) for (j=0;!b&&j<this.keyArray[i].actions.length;j++) b=(this.keyArray[i].actions[j]==hX_5.ACT_SUBMIT);
						if (b && this.coBehavior.state && this.coBehavior.state.uirelease) this.coBehavior.state.uirelease();
						result = hX_5.action.processAction(thisObj, evt, this.keyArray[i].fcn, this.keyArray[i].actions, true, 
						  this.keyArray[i].targets, null, this.keyArray[i].cancelBubble, this.keyArray[i].retainFocus);
					}
				}
			}
		}
	}
	return result;
}

// -------------------------------------------------------
// JSF Behavior - Validate
//
// Validate a value (only for onblur or onfocus)
// 
// Arguments:
//
//	validator		(id)	 Validator to use (required)
//	converter 		(id)	 Converter to use (required if validator needs a converter)
//	successAction 	(string) Actions to take if validation succeeds
//	successTarget 	(string) Targets that go with the success actions (if none, current field is used)
//	successFunction	(string) Javascript to execute if validation succeeds (normal
//							 form is "return func(this, event);"
//	successClass	(string) Class (or space-spearated list of classes) to apply if
//							 validation succeeds
//	errorAction 	(string) Actions to take if validation fails
//	errorTarget 	(string) Targets that go with the error actions (if none, current field is used)
//	errorFunction	(string) Javascript to execute if validation fails (normal
//							 form is "return func(this, event);"
//	errorClass		(string) Class (or space-spearated list of classes) to apply if
//							 validation fails
//	defaultClass	(string) Class (or space-spearated list of classes) to apply if
//							 "normally" (no error, no success)
//	All the success/error arguments are optional
//  See the documentation for a description of how the formbox/formitem attributes work
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorValidate = function () {
	this.type = hX_5.BHR_validate;

	// Attributes set by the caller -- passed into the key array
	// validator, converter, label, Span id's
	this.validatorId = this.converterId = this.labelId = this.spanInfoId = this.spanErrorId = this.errorOnLoad = null;
	this.successActions = [];		// Function to perform on success
	this.errorActions	= [];		// Function to perform on error
	this.successTargets = [];		// Target for the success action
	this.errorTargets 	= [];		// Target for the error action
	// Action to perform on success, Action to perform on error, Class to apply on success, Class to apply on error, Default class, label and error classes
	this.successFunction= this.errorFunction = this.successClass = this.errorClass = this.defaultClass = this.labelSuccessClass = this.labelErrorClass = null;
	// Label handling stuff	
	this.spLabel = this.spInfo = this.spError = this.spanIsPopup = this.errorString = null;
	// Additional gunk
	this.infoSpanAlways = this.errorSpanAlways = this.hasLabels = false;
	
	// Implementation
	// Converter object, Validator object
	this.converter = this.validator = null;
	this.coBehavior = -1;			// "input assist" behavior if there is one
	this.extraEvents = null;
	this.isComponent = false;		// If true this is a component

	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['validator']		= ['validatorId',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['converter']		= ['converterId',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['success-function']= ['successFunction',hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['error-function']	= ['errorFunction',	hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['success-action']	= ['successActions',hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['success-target']	= ['successTargets',hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['error-action']	= ['errorActions',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['error-target']	= ['errorTargets',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['success-class']	= ['successClass',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['error-class']		= ['errorClass',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['default-class']	= ['defaultClass',	hX_5.ARG_TO_NAN,	false,	null,	null];
		/* Following are used by formItem */
		this.attributeTable ['label-id']			= ['labelId',			hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['info-span-id']		= ['spanInfoId',		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['error-span-id']		= ['spanErrorId',		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['label-success-class']	= ['labelSuccessClass',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['label-error-class']	= ['labelErrorClass',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['span-is-popup']		= ['spanIsPopup',		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['info-span-always']	= ['infoSpanAlways',	hX_5.ARG_IS_TRUE,	false,	null,	null];
		this.attributeTable ['error-span-always']	= ['errorSpanAlways',	hX_5.ARG_IS_TRUE,	false,	null,	null];
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorValidate.prototype = new hX_5.JSFBehavior();

// Helper -- accessor for other behaviors
HxG_5.prototype.JSFBehaviorValidate.prototype.clearError = function () {
	this.errorOnLoad = null;
}

// Helper -- Is a label visible
HxG_5.prototype.JSFBehaviorValidate.prototype.isVisible = function (o) {
	// Check if we think something is visible or not.
	// We do a "quick check" only for now -- we don't walk up the tree 
	if (o==null) return false;
	var d=hX_5.g.getP(o, hX_5._DSP), v=hX_5.g.getP(o, hX_5._VIS);
	return ((d!=hX_5._NO) && (v==hX_5._VS || v=="auto" || v=="inherit"));
}

// Override:  Set the validator/converter
HxG_5.prototype.JSFBehaviorValidate.prototype.setAttributeCode = function () {
	this.validator = this.converter = null;
	if (this.validatorId) this.validator = hX_5.getValidatorById(this.validatorId);
	if (this.converterId) this.converter = hX_5.getConverterById(this.converterId);
	if (this.validator) {
		if ((this.validator.type == hX_5.CVT_number || this.validator.type == hX_5.CVT_datetime) && this.converter == null)
			this.validator = null;
	}
	// Verify the span-is-popup
	if (this.spanIsPopup != null) {
		this.spanIsPopup = this.spanIsPopup.toUpperCase();
		if (this.spanIsPopup != "OVER" && this.spanIsPopup != "UNDER" && this.spanIsPopup != "LEFT" && this.spanIsPopup != "RIGHT") this.spanIsPopup = null;
	}
	// Mark if we're doing some kind of label manipulations
	this.hasLabels = (this.labelId!=null || this.spanInfoId!=null || this.spanErrorId!=null);
	// Cache references to these objects
	this.spLabel= (this.labelId!=null)?hX_5.getElementById(this.labelId):null;
	this.spInfo	= (this.spanInfoId!=null)?hX_5.getElementById(this.spanInfoId):null;
	this.spError= (this.spanErrorId!=null)?hX_5.getElementById(this.spanErrorId):null;
}

// Override:  When the page is emitted clean up label handling and make sure we don't have "bad" onblur events
HxG_5.prototype.JSFBehaviorValidate.prototype.setEventCode = function () {
	// We only do this once -- when behavior is first run
	if (!this.HTMLrendered) {
		var t,i,c,p,z,foundError=false, v=(this.spError!=null && this.isVisible(this.spError));
		this.isComponent = hX_5.getComponentById(this.DOMobj.id);

		// If this is validation is set on a checkbox and/or radio button group, it may be set on the containing table
		// we need events on each element
		if (this.DOMobj.tagName!=undefined && this.DOMobj.id!=undefined && this.DOMobj.tagName==hX_5._TAB) {
			p = this.DOMobj.parentNode;
			while (p!=null && p.tagName!=undefined && p.tagName!=hX_5._FRM) p=p.parentNode;
			if (p && p.tagName!=undefined && p.tagName==hX_5._FRM) {
				c = p.elements[this.DOMobj.id];
				c = (c.length==undefined)?[c]:c;
				for (i=0; i<c.length; i++) hX_5.attachEvent(c[i], this.eventname, hX_5._lL, this.subDispatch);
				// Following is set to facilitate destructing the behavior
				if (this.extraEvents==null) this.extraEvents=[];
				this.extraEvents.push([c[i], this.eventname]);
			}
		}
		// Similarly, if this validation is set on a minicalendar, it really should be set on the input field
		if (this.isComponent!=null && this.isComponent.type==hX_5.CMP_calendar) {
			c = hX_5.getElementById(this.DOMobj.id + "_INPUT");
			if (c) {
				hX_5.attachEvent(c, this.eventname, hX_5._lL, this.subDispatch1);
				if (this.extraEvents==null) this.extraEvents=[];
				this.extraEvents.push([c, this.eventname]);
			}
		}

		// Convert the spans to the proper kind of "div" if they are popup
		if (this.spanIsPopup!=null) {
			p=null;
			if (this.spError) {
				p = this.spError.parentNode;
				// We have to "rip" the span out of the run for it to size to fit -- it could be
				// inside of an element that would otherwise constrain it's size
				if (document.body) document.body.appendChild(this.spError);
				// Tidy it's properties
				if (hX_5.g.getP(this.spError, hX_5._POS)!=hX_5._ABS) hX_5.g.setP(this.spError, hX_5._POS, hX_5._ABS);
				if (v && hX_5.g.getP(this.spError, hX_5._DSP)!=hX_5._BLK) hX_5.g.setP(this.spError, hX_5._DSP, hX_5._BLK);
				z = parseInt(hX_5.g.getP(this.spError, hX_5._ZND),10);
				if (z!=99999) hX_5.g.setP(this.spError, hX_5._ZND,"99999");
			}
			if (this.spInfo) {
				p = this.spInfo.parentNode;
				if (document.body) document.body.appendChild(this.spInfo);
				if (hX_5.g.getP(this.spInfo, hX_5._POS)!=hX_5._ABS) hX_5.g.setP(this.spInfo, hX_5._POS, hX_5._ABS);
				if (this.isVisible(this.spInfo) && hX_5.g.getP(this.spInfo, hX_5._DSP)!=hX_5._BLK) hX_5.g.setP(this.spInfo, hX_5._DSP, hX_5._BLK);
				z = parseInt(hX_5.g.getP(this.spInfo, hX_5._ZND),10);
				if (z!=99999) hX_5.g.setP(this.spInfo, hX_5._ZND,"99999");
			}
// FIXME: If the popup overlies a combo-box it won't display right -- it needs an iframe "backing"			
			// If we left behind an empty table cell, put something "small" in it so its borders show
			if (p!=null && p.tagName!=undefined && p.tagName==hX_5._TD && p.firstChild==null)
				hX_5.setIHTML(p,"<SPAN style='font-size:1px'>&nbsp;</SPAN>");
			// Positioning the spans in case they are initially displayed
			hX_5.action.positionLabels (this.DOMobj, this.spanIsPopup, this.spInfo, this.spError);
		}
		// If we have an info-span or an error-span, clean up their display characteristics
		if ((this.spInfo!=null) && this.infoSpanAlways && !v) hX_5.g.setP(this.spInfo, hX_5._DSP, ((this.spanIsPopup!=null)?hX_5._BLK:hX_5._E));

		// If we have an error span, check to see if it's visible (during the onPageLoad)
		// If it is, the field is considered to be initially "in error"
		// Also, if we have an error span and this is "use this error always", pick up the error string
		if (this.spError!=null) {
			if (this.errorSpanAlways) this.errorString = hX_5.getIHTML(this.spError);
			this.errorOnLoad = (v) ? this.errorString : null;
			if (this.errorOnLoad!=null) {
				if (this.eventname==hX_5._onB) {
					if (this.errorClass!=null && this.DOMobj.className!=this.errorClass) this.DOMobj.className = this.errorClass;
				} else {
					if (this.spLabel && this.labelErrorClass!=null && this.spLabel.className!=this.labelErrorClass) this.spLabel.className = this.labelErrorClass;
				}
			}
		}

		// Now clean up bad blurs
		if (this.eventname==hX_5._onB && this.errorActions!=null && this.successActions!=null) {
			// If this is the onblur handler and the user has specified any variant of "setfocus(thisobj)"
			// for BOTH success and failure, the page will lock (focus cannot ever leave the field)
			// So scan for this error and remove the success action if it's found
			for (i=0; !foundError && i < this.errorActions.length; i++) {
				t = (this.errorActions[i]!=null) ? this.errorActions[i] : hX_5._E;
				if (t==hX_5.ACT_FOCUS || t==hX_5.ACT_SELECTED || t==hX_5.ACT_UNSELECTED)
					foundError = this.errorTargets == null || this.errorTargets.length < i+1 || this.errorTargets[i]==null || this.errorTargets[i].length==0 || this.errorTargets[i]==this.DOMobj.id;
			}
			if (foundError) {
				for (i=0; i < this.successActions.length; i++) {
					t = (this.successActions[i]!=null) ? this.successActions[i] : hX_5._E;
					if (t==hX_5.ACT_FOCUS || t==hX_5.ACT_SELECTED) {
						if (this.successTargets == null || this.successTargets.length < i+1 || this.successTargets[i]==null || this.successTargets[i].length==0 || this.successTargets[i]==this.DOMobj.id)
							this.successActions[i] = hX_5._E;
					}
				}
			}
		}
		// If this is either an onblur or an onfocus event, you can have only ONE focus/select
		// action (duh)
		foundError = false;
		for (i=0; this.errorActions != null && i < this.errorActions.length; i++) {
			t = (this.errorActions[i]!=null) ? this.errorActions[i] : hX_5._E;
			if (t==hX_5.ACT_FOCUS || t==hX_5.ACT_SELECTED || t==hX_5.ACT_UNSELECTED) {
				if (foundError) this.errorActions[i] = hX_5._E;
				else foundError = true;
			}
		}
		foundError = false;
		for (i=0; this.successActions != null && i < this.successActions.length; i++) {
			t = (this.successActions[i]!=null) ? this.successActions[i] : hX_5._E;
			if (t==hX_5.ACT_FOCUS || t==hX_5.ACT_SELECTED || t==hX_5.ACT_UNSELECTED) {
				if (foundError) this.successActions[i] = hX_5._E;
				else foundError = true;
			}
		}
	}
}

HxG_5.prototype.JSFBehaviorValidate.prototype.destroyCode = function () {
	var i,d = [this.spLabel,this.spInfo,this.spError,this.successActions,this.errorActions,this.successTargets,this.errorTargets];
	for (i=0; i<d.length; i++) {
		if (d[i]) {
			while (d[i].length>0) d[i].pop();
			delete d[i];
		}
	}
	delete d;
	return true;
}

// Public: For grouped input fields, convert to a call on the parent component
HxG_5.prototype.JSFBehaviorValidate.prototype.subDispatch = function (evt) {
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt) {
		var b,c=hX_5.evtGetTarget(evt);
		if (c!=null) {
			b = hX_5.getBehaviorById(c.name, hX_5.BHR_validate, evt.type);
			if (b!=null && b.DOMobj) return(b.dispatchBehavior (b.DOMobj, evt));
		}
	}
}

// Public: For grouped input fields, convert to a call on the parent component
HxG_5.prototype.JSFBehaviorValidate.prototype.subDispatch1 = function (evt) {
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt) {
		var b,c=hX_5.evtGetTarget(evt);
		if (c!=null) {
			b = c.id.lastIndexOf("_INPUT");
			if (b>=0) {
				c = c.id.substr(0,b);
				b = hX_5.getBehaviorById(c, hX_5.BHR_validate, evt.type);
				if (b!=null && b.DOMobj) return(b.dispatchBehavior (b.DOMobj, evt));
			}
		}
	}
}

// Public: Execute the behavior
HxG_5.prototype.JSFBehaviorValidate.prototype.dispatchBehavior = function (thisObj, evt) {
	var result = true;
	if ((this.DOMobj != null) && (evt != null)) {
		// We have an issue when an error occurs in an onBlur in that you can't cancel
		// the blur -- the next field will receive focus.  If the error raises the Select action
		// on the field itself, then we get the following event sequence which will likely infinitely loop:
		// 		onblur	-- 	fires on the first field
		//			  		field fails validation and action sets focus/selection back to itself
		// 		onfocus	-- 	fires on the field after it in the tab order
		//		onblur	-- 	fires on the field after it in the tab order
		//		onfocus --  fires on the first field
		//
		// An infinite loop occurs if the second onblur listed above ALSO fails validation and has an 
		// action set to focus/select.  It will set focus to itself.  But the first focus is already
		// in the event queue so it will fire, then onblur (because there's a second focus in the queue),
		// and loop ad nauseum.
		//
		// Goodman says this is prevented (the loop) by never validating on the onBlur event and instead
		// validating only in the onChange event (which is cancelable).  However, we can't validate in the
		// onChange event as we want to catch the case where the value is initially wrong.  For example,
		// if "required" is true and the field is initially blank, when someone tabs through the field
		// we want to stop them and force them to enter a value.  onChange won't have fired in this case.
		//
		// So what we do to prevent this: when we try to set focus/selection to "ourself" in an action, we set
		// a flag that causes us to ignore the focus/blur events that occur until a focus event
		// goes to the original field

		if (hX_5.pendingFocusReturn && hX_5.pendingFocusReturn != this.DOMobj.id)
			return true;
		if (evt.type==hX_5._evF && hX_5.pendingBlurRefocus != null && hX_5.pendingBlurRefocus == this.DOMobj.id) {
			hX_5.pendingBlurRefocus = null;
			return true;
		}
		hX_5.pendingFocusReturn = hX_5.pendingBlurRefocus = null;
		if ((this.DOMobj.readOnly != null && this.DOMobj.readOnly == true) || (this.DOMobj.disabled != null && this.DOMobj.disabled == true))
			return true;

		// If this is the first time through dispatch, see if we have a co-behavior we need to care about
		if (this.coBehavior == -1) this.coBehavior = hX_5.getBehaviorById(this.DOMobj.id, hX_5.BHR_assist, hX_5._onIN);

		// Handle blur/focus
		if (evt.type==hX_5._evB || evt.type==hX_5._evF) {
			var error=null, strValue=this.DOMobj.value, objValue=strValue;
			// We need to see if the value is in error or not
			if (evt.type==hX_5._evF) {
				// For focus, we don't validate, we only see if the converter works
				if (this.coBehavior && this.coBehavior.assist)
					// In the focus case, we can assume assist will force the value to be correct
					error = null;
				else if (this.converter && strValue!=null ) {
					if (this.converter && strValue.length > 0) {
						objValue = this.converter.stringToValue(strValue);
						if (objValue == null)
							error = this.converter.lastError();
					}
				}
				if (error != null && error == hX_5.L[this.lc].ERR_empty) error = null;
			} else {
				// This is a hack for now:  handle select, radio btns and checkbox tags specially.  
				// We really should have a converter/validator explicitly for them but for now, we do a hard-coded 
				// check here
				var spc = this.specialValidator();
				if (spc.handled) {
					if (spc.lastError) error = hX_5.L[this.lc].ERR_required;
					else objValue = spc.lastValue;
					delete spc;
				} else {
					if (this.converter) {
						if (strValue.length == 0 && (this.converter.type != hX_5.CVT_mask))
							objValue = hX_5._E;
						else {
							objValue = this.converter.stringToValue(strValue);
							if (objValue == null)
								error = this.converter.lastError();
							else {
								strValue = this.converter.valueToString(objValue);
								if (strValue == null) error = this.converter.lastError();
								else this.DOMobj.value = strValue;
							}
						}
					}
					if (error == null && this.validator) {
						if (!(this.validator.validate(objValue, this.converter)))
							error = this.validator.lastError();
					}
				}
			}

			// If there's no error, then treat the success function as a follow on validation function
			var fcn=this.successFunction, action, cssStyle, b=null, cssLabelStyle=null, tgt;
			if (error != null) {
				evt.errorMsg = error;
				evt.objValue = void(0);
			// Success case
			} else {
				if (fcn != null) {
					evt.errorMsg = void(0);
					evt.objValue = objValue;
					try {
						// NOTE:  fcn should already be a fcn.  It's really bad if it isn't, however,
						// there could be some old assist code floating around that hasn't already done the cast
						var realfcn = (hX_5.isFunction(fcn)) ? fcn : new Function ("event", fcn);
						// NOTE:  We could run the function using either "obj" (the original "this" pointer
						// when the event started) or "t" (the target object) as the "this" pointer in the
						// invoked function.  Not sure which a user would expect.  Using "obj" as it's
						// the originating this pointer.
						result = realfcn.call(thisObj, evt);
						result = (hX_5.isUnDef(typeof result)) ? true : ((result == false) ? false: true);
						if (evt) {
							if (evt.returnValue!=undefined && evt.returnValue == false) result = false;
							else if (evt.getPreventDefault && evt.getPreventDefault()) result = false;
						}
					}
					catch (e) {
						window.status = hX_5.INTERR_Dispatcher;
						if (evt && evt.errorMsg)
							evt.errorMsg = null;
					}
					if (!result) {
						if (evt.errorMsg!=undefined)
							error = evt.errorMsg;
						else 
							error = evt.errorMsg = hX_5.L[this.lc].ERR_generic
					}
				}
			}
			if (this.errorOnLoad!=null && this.hasLabels)
				evt.errorMsg = error = this.errorOnLoad;
			if (error && this.errorSpanAlways && this.errorString!=null)
				evt.errorMsg = error = this.errorString;
			
			// Error case
			if (error != null) {
				fcn = this.errorFunction;
				action = this.errorActions;
				tgt = (this.errorTargets == null) ? [this.DOMobj] : this.errorTargets;
				cssStyle = (this.errorClass!=null)?this.errorClass:((this.successClass) ? this.successClass : this.defaultClass);
			// Success case
			} else {
				fcn = null;
				action = this.successActions;
				tgt = (this.successTargets == null) ? [this.DOMobj] : this.successTargets;
				cssStyle = (this.successClass) ? this.successClass : this.defaultClass;
			}
			// Set up the labels.  Except when the page is initially displayed, we only swap labels onBlur
			var lab=null, inf=null, err=null, infoShow=false, errorShow=false;
			if (evt.type==hX_5._evB || (evt.type==hX_5._evF && error!=null)) {
				lab=this.spLabel; inf=this.spInfo; err=this.spError;
				if (error != null) {
					if (this.spLabel)	cssLabelStyle = ((this.labelErrorClass!=null)?this.labelErrorClass:((this.labelSuccessClass!=null)?this.labelSuccessClass:null));
					if (this.spInfo)	infoShow = false;
					if (this.spError)	errorShow = true;
				} else {
					if (this.spLabel)	cssLabelStyle = (this.labelSuccessClass!=null)?this.labelSuccessClass:null;
					if (this.spInfo)	infoShow = (this.infoSpanAlways);
					if (this.spError)	errorShow = false;
				}
				if (this.errorOnLoad!=null) {
					this.errorOnLoad = null;
					b = hX_5.getBehaviorById(this.DOMobj.id, this.type, ((this.eventname==hX_5._onF)?hX_5._onB:hX_5._onF));
					if (b) b.clearError();
				}
			} else {
				if (!this.infoSpanAlways && (this.spError==null || !this.isVisible(this.spError))) {
					inf = this.spInfo;
					infoShow = true;
				}
			}
			// Now process the action
			result = hX_5.action.processActionEx (this.DOMobj, evt, fcn, action, false, tgt, cssStyle, null, null, (error != null),
					 error, lab, cssLabelStyle, this.spanIsPopup,  inf, infoShow, err, errorShow);
		}
	}
	return result;
}

HxG_5.prototype.JSFBehaviorValidate.prototype.specialValidator = function () {
	var r=false, lErr=false, lVal=hX_5._E, sel=false, i,c,s,p=((this.DOMobj.form!=undefined)?this.DOMobj.form:this.DOMobj.parentNode);
	if (this.validator && this.validator.required && this.DOMobj!=null && this.DOMobj.tagName!=undefined) {

		// For select tags, something must be selected
		if (this.DOMobj.tagName==hX_5._SEL) {
			while (p!=null && p.tagName!=undefined && p.tagName!=hX_5._FRM) p=p.parentNode;
			if (p && p.tagName!=undefined && p.tagName==hX_5._FRM && this.DOMobj.id!=undefined) {
				r=true;
				c = p.elements[this.DOMobj.id];
				c = (c.length==undefined)?[c]:c;
				for (i=0; i<c.length; i++) {
					if (c[i].selected) {
						// For combo-boxes, make sure the value isn't 0 or spaces
						if (this.DOMobj.size==1) {
							s = hX_5.allTrim(c[i].value);
							if (s.length>0 && s!=hX_5._Z) {
								sel = true;
								lVal = lVal+((lVal.length>0)?hX_5._SM:hX_5._E)+c[i].value;
							}
						// For listboxes, it just needs a selection
						} else {
							sel = true;
							lVal = lVal+((lVal.length>0)?hX_5._SM:hX_5._E)+c[i].value;
						}
					}
				}
				if (!sel) lErr=true;
			}
		} else if (this.DOMobj.tagName==hX_5._TAB) {
			// For radios and checks, we don't care what is selected, only that at least one thing is
			while (p!=null && p.tagName!=undefined && p.tagName!=hX_5._FRM) p=p.parentNode;
			if (p && p.tagName!=undefined && p.tagName==hX_5._FRM && this.DOMobj.id!=undefined) {
				r=true;
				c = p.elements[this.DOMobj.id];
				c = (c.length==undefined)?[c]:c;
				for (i=0; i<c.length; i++) {
					if (c[i].checked) {	
						s = (c[i].value && c[i].value.length>0)?c[i].value:"on";
						lVal = lVal+((lVal.length>0)?hX_5._SM:hX_5._E)+s;
						sel = true;
					}
				}
				if (!sel) lErr=true;
			}
		} else if (this.DOMobj.tagName==hX_5._INP && this.DOMobj.type!=undefined) {
			// Simple Checkboxes just need to be selected
			if (this.DOMobj.type==hX_5._CHK) {
				r=true;
				if (!this.DOMobj.checked) lErr = true;
				else lVal = "on";
			// File upload needs a "value" (which means some file has been selected)
			} else if (this.DOMobj.type==hX_5._FIL) {
				r=true;
				if (this.DOMobj.value!=null && this.DOMobj.value.length>0) lVal = this.DOMobj.value;
				else lErr=true;
			}
		}
		// Might be a component
		if (!r && this.isComponent!=null) {
			if (this.isComponent.type==hX_5.CMP_colorpicker) {
				r = true;
				if (this.DOMobj.value==null || this.DOMobj.value.length==0) lErr = true;
				else lVal = this.DOMobj.value;
			} else if (this.isComponent.type==hX_5.CMP_calendar) {
				r = true;
				c = hX_5.getElementById(this.DOMobj.id + "_INPUT");
				if (c.value==null || c.value.length==0) lErr = true;
				else lVal = c.value;
			}
		}						
	}
	return {handled:r, lastError:lErr, lastValue:lVal};
}

// -------------------------------------------------------
// JSF Behavior - Base Assist
//
// Basic assist functions -- autotab and key-by-key validation
// 
// Arguments:
//
//	validator		(id)	 Required if doing auto-char
//	converter 		(id)	 Converter to use (required if validator needs a converter)
//	auto-tab		(number) Optional. If > 0, after this many chars are typed, auto tab to next field (number of characters)
//  auto-submit		(id)	 Optional. If true/valued, will auto-submit when field is complete
//							 If provided AND autoTab is set, then will submit instead of advancing
// 							 If value of autosubmit is empty string, submit is done using "first" submit button
// 							 in the enclosing form.  Else provide an id and submit is done on that id or first
// 							 submit button found within it
//  auto-char		(bool)	 Optional. If true, characters are validated as they are typed.
// 							 otherwise they are validated only on the behavior's event (e.g., on blur)
//	prompt-char		(str)	 Optional, if provided, we display the prompt char and turn on auto-char
//							 and fiddle with the users input
//	ime-mode		(string) Optional. DEPRECATED.  Set's style property of object to this valud
//							 Values are "active" "auto" "disabled" "inactive"
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorAssist = function () {
	this.type = hX_5.BHR_assist;
	this.suppressAttach = true;

	// Attributes set by the caller -- passed into the key array
	// Converter, Validator, When to auto-tab to next field (# of chars), mutate auto-tab into auto-submit
	this.converterId = this.validatorId = this.autoTab = this.autoSubmit = this.autoChar = this.promptChar = this.imeMode = null;

	// Implementation
	// Converter object, Validator object, Mask handler object, Keyboard input state object,Max number of characters in the field
	this.converter = this.validator = this.assist = this.state = this.maxChars = null;
	this.doAutoChar = false;		// True if autoChar true and there is a converter									

	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['converter']		= ['converterId',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['validator']		= ['validatorId',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable	['auto-char']		= ['autoChar',		hX_5.ARG_IS_TRUE,	false,	null,	null];
		this.attributeTable ['auto-tab']		= ['autoTab',		hX_5.ARG_TO_NUM,	false,	0,		999999];
		this.attributeTable	['auto-submit']		= ['autoSubmit',	hX_5.ARG_TO_NAS,	false,	null,	null];
		this.attributeTable ['prompt-char']		= ['promptChar',	hX_5.ARG_TO_NAS,	false,	null,	null];
		this.attributeTable ['ime-mode']		= ['imeMode',		hX_5.ARG_TO_NAN,	false,	null,	null];
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorAssist.prototype = new hX_5.JSFBehavior();

// Override:  Set the validator/converter
HxG_5.prototype.JSFBehaviorAssist.prototype.setAttributeCode = function () {
	this.validator = this.converter = null;
	if (this.validatorId) this.validator = hX_5.getValidatorById(this.validatorId);
	if (this.converterId) this.converter = hX_5.getConverterById(this.converterId);
	if (this.validator) {
		if ((this.validator.type==hX_5.CVT_number || this.validator.type==hX_5.CVT_datetime) && this.converter==null)
			this.validator = null;
	}
	if (this.promptChar) {
		if (this.promptChar == hX_5._DFT || this.promptChar.length == 0) this.promptChar = hX_5._U;
		if (this.promptChar.length > 1) this.promptChar = this.promptChar.charAt(0);
		this.autoChar = true;
	}
	if (this.autoChar && !this.promptChar) this.promptChar = hX_5._U;
	if (this.autoChar && this.converter && this.converter.type==hX_5.CVT_number && this.converter.isExponent) {
		this.promptChar = this.autoChar = null;
	}
	this.doAutoChar = (this.autoChar && this.converter);
}

// Override:  Turn on key-by-key validation or auto-tab
HxG_5.prototype.JSFBehaviorAssist.prototype.setEventCode = function () {

	// If we're auto-tabbing or auto-char'ing, then initialize the "assist" handlers
	if (!this.HTMLrendered) {
		// If we're auto-tabbing or auto-char'ing, attach an inputState manager
		// The input state manager controls the basic flow of keystroke events (e.g., 
		// key down, key up, ...
		if (this.doAutoChar) {
			this.state = new hX_5.inputState(this.DOMobj, this);
		// Create the mask handler to use with this value.  The mask handler controls the details
		// of keystrokes relative to the mask (if there is one).  Basically it "preprocesses" keystrokes
		// BEFORE they get handled by the input state keystroke handler.  We call this AFTER the
		// input state handler is set up.  That way it gets pushed onto the event handler stack
		// last (so it executes FIRST).
			if (this.converter.type == hX_5.CVT_number) 
				this.assist = new hX_5.NumberAssist(this.DOMobj, this.state, this.converter, this.validator, this.promptChar);
			else if (this.converter.type == hX_5.CVT_datetime) 
				this.assist = new hX_5.DateTimeAssist(this.DOMobj, this.state, this.converter, this.validator, this.promptChar);
			else if (this.converter.type == hX_5.CVT_mask) 
				this.assist = new hX_5.MaskAssist(this.DOMobj, this.state, this.converter, this.validator, this.promptChar);
		}
	}

	// Whether the HTML is out or not, we have a potential consistency problem
	// if autoTab is on using "one maxLength" and the field has a different one
	if (this.autoTab != null) {
		// If autotab is set, use it
		var maxlen = (this.autoTab > 0) ? this.autoTab : 0;
		// Else if max length is set, use it
		var domMaxLen = (this.DOMobj.maxLength == -1 || this.DOMobj.maxLength == 0x7fffffff || this.DOMobj.maxLength == void(0)) ? 0 : this.DOMobj.maxLength;
		maxlen = (maxlen <= 0) ? domMaxLen : maxlen;
		// Else compute the size of the field
		this.maxChars = (maxlen > 0) ? maxlen : ((this.converter && this.converter.getMaxLength) ? this.converter.getMaxLength(this.validator) : 0);
		if (this.maxChars <= 0)
			this.autoTab = null;
		else {
			if (domMaxLen > 0) this.DOMobj.maxLength = this.maxChars;
			if (!this.HTMLrendered && this.state == null) this.state = new hX_5.inputState(this.DOMobj, this, true);
			this.state.setAutoTab(this.maxChars);
			if (this.autoSubmit != null) this.state.setAutoSubmit(true);
		}
	}

	// Deprecated
	if (this.imeMode != null) {
		if (hX_5.brw.isIE()) {
			if (this.assist != null) this.DOMobj.style.imeMode = hX_5._DSB;
			else if (this.imeMode != null) this.DOMobj.style.imeMode = this.imeMode;
		}	
	}
}

// Public: Destroyer 
HxG_5.prototype.JSFBehaviorAssist.prototype.destroyCode = function () {
	var i,d = [this.state,this.assist];
	for (i=0; i<d.length; i++) {
		if (d[i]) {
			while (d[i].length>0) d[i].pop();
			delete d[i];
		}
	}
	delete d;
	return true;
}

// -------------------------------------------------------
// JSF Behavior - Alert/Prompt
//
// Display an "yes/no" prompt message or an alert message
// 
// Arguments:
//
//	message (string, required)	The message to display
//	prompt-type (int)			0 (default) means "alert", 1 means "prompt"
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorAlert = function () {
	this.type = hX_5.BHR_alert;

	// Attributes set by the caller
	this.message = hX_5._E;
	this.mType = 1;

	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['message']		= ['message',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['prompt-type']	= ['mType',		hX_5.ARG_TO_NUM,	false,	0,		2];
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorAlert.prototype = new hX_5.JSFBehavior();

// Public: Execute the action
HxG_5.prototype.JSFBehaviorAlert.prototype.dispatchBehavior = function (thisObj, evt) {
	if (this.DOMobj != null && this.eventname != hX_5._E)  {
		var a = [];
		var t = [this.message];
		if (this.mType == 2) {
			return false;
		} else if (this.mType == 1 && this.message != hX_5._E) {
			a.push(hX_5.ACT_CONFIRM);
			return hX_5.action.processAction(this.DOMobj, evt, null, a, true, t, null, null, null);
		} else if (this.message != hX_5._E) {
			a.push(hX_5.ACT_ALERT);
			hX_5.action.processAction(this.DOMobj, evt, null, a, true, t, null, null, null);
			return true;
		}
	}
	return true;
}

// -------------------------------------------------------
// JSF Behavior - Set Focus
//
// Set the focus to the specified element
//
// Arguments:
//
//	focus-id (string, required)	id of the field to get focus
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorFocus = function () {
	this.type = hX_5.BHR_focus;
	this.beforeOthers = false;
	this.suppressAttach = true;

	// Attributes set by the caller
	this.focusId = null;
	this.select = false;
	
	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['focus-id']	= ['focusId',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['select']		= ['select',	hX_5.ARG_IS_TRUE,	false,	null,	null];
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorFocus.prototype = new hX_5.JSFBehavior();

// Private:  Override of Set Event
HxG_5.prototype.JSFBehaviorFocus.prototype.setEventCode = function() {
	if (!this.HTMLrendered && this.eventname == hX_5._onL) this.dispatchBehavior(null, null);
	return true;
}

// Public: Execute the action
HxG_5.prototype.JSFBehaviorFocus.prototype.dispatchBehavior = function (thisObj, evt) {
	if (this.eventname != hX_5._E) {
		var n, obj = (this.focusId == null) ? null : hX_5.getElementById(this.focusId);
		// If no id provided OR the tag is a form tag, find the first focusable within it
		if (obj == null || (obj && obj.tagName && obj.tagName == HxG_5._FRM)) {
			n = hX_5.getTabElements();
			if (n && n.length > 0 && n[0].obj) hX_5.i.setFocus(n[0].obj, this.select, true);
		// Otherwise set focus to the supplied field
		} else
			hX_5.i.setFocus(obj, this.select, true);
	}
	return;
}

// -------------------------------------------------------
// JSF Behavior - Hide/Show
//
// Hide/Show an object or a tree of objects
//
// Arguments:
//
//	hide-object (string, required)	id of the field to hide/show
//	state (string)					If "true", object is shown.  If "false" it's hidden.
//	hide-field	(string)			optional id of a field on the page whose value
//									is kept up-to date with the values "visible" or "none"
//									reflecting the state of the hidden object
// 	hide-tree (string)				optional id of the TABLE containing the object where
//									it is assumed each row may contain a "table" representing
//									a subtree of objects.  When this object is hidden, all
//									subtrees are hidden as well.
//	redraw (string)					if "true", when a hide-show is done, the page is redrawn
//									this is needed if the object contains JSF components
//									(they need to be redrawn on a "show");
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorHS = function () {
	this.type = hX_5.BHR_hs;
	this.beforeOthers = false;

	// Attributes set by the caller
	this.inHideObject = this.inHideTree = this.inHideField = hX_5._E;
	this.state = null;

	// Values that go with the above
	this.hideObject = this.hideField = this.hideTree = null;

	// Does what we're hiding showing have a component in it?
	this.hasComponents = null;

	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['hide-object']	= ['inHideObject',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['hide-tree']	= ['inHideTree',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['hide-field']	= ['inHideField',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['hide-field-value'] = ['state',	hX_5.ARG_TO_NAN,	false,	null,	null];
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorHS.prototype = new hX_5.JSFBehavior();

// Private:  Override of Set Event
HxG_5.prototype.JSFBehaviorHS.prototype.setEventCode = function() {
	this.hasComponents = (this.hideObject) ? hX_5.getComponentsInId(this.hideObject) : null;
	this.hideShow();
	return true;
}

// Private:  Override of Set Attribute
HxG_5.prototype.JSFBehaviorHS.prototype.setAttributeCode = function() {
	// Find the object that's to be hidden/shown
	this.hideObject	= hX_5.getElementById(this.inHideObject);
	this.hideTree	= hX_5.getElementById(this.inHideTree);
	this.hideField	= hX_5.getElementById(this.inHideField);
	this.state		= (this.state==null || this.state==hX_5._E)?hX_5._NO:this.state.toLowerCase();
	if (this.HTMLrendered) this.hideShow();
	return true;
}

HxG_5.prototype.JSFBehaviorHS.prototype.dispatchBehavior = function (thisObj, evt) {
	if (this.DOMobj != null && this.hideObject != null) {
		this.state = (this.hideObject.style.display == hX_5._NO) ? hX_5._VS : hX_5._NO;
		this.hideShow();
	}
	return;
}

HxG_5.prototype.JSFBehaviorHS.prototype.hideShow = function () {
	if (this.DOMobj != null && this.hideObject != null) {
		var bHidden = (this.hideObject.style.display == hX_5._NO), bStateHidden = (!(this.state == hX_5._VS)), lD = '_display';
		if (bHidden != bStateHidden) {
			var r, child, idString, node;
			if (this.hideTree != null && this.hideTree.rows)
				for (r=0; r < this.hideTree.rows.length; r++) {
					if (r != 0) {
						child = hX_5.findNestedObject(this.hideTree.rows[r], hX_5._TAB);
						if (child != null && child.nodeName == hX_5._TAB) {
							child.style.display = hX_5._NO;
							idString = child.id;
							idString = idString.substring(0,idString.lastIndexOf(hX_5._U));
							node = hX_5.getElementById (idString+lD);
							if (node != null) node.value = hX_5._NO;
						}
					}
				}	
			if (this.state != hX_5._NO) {
				this.hideObject.style.display = hX_5._E;
				if (this.hasComponents)	hX_5.redraw();
			} else
				this.hideObject.style.display = hX_5._NO;
		}
		if (this.hideField && this.state != this.hideField.value) this.hideField.value = this.state;
	}
	return;
}

// -------------------------------------------------------
// JSF Behavior - Track
//
// Track which field was last used within a form (i.e., which field 
// last had focus before the form was submitted).
// 
// Arguments:
//
//	see documentation.
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorTrack = function () {
	this.type = hX_5.BHR_track;
	this.suppressAttach = true;

	// Where we store the results
	this.lastField = null;	// <form-id>:POST_LASTFIELD
	this.lastCursor = null;	// <form-id>::POST_LASTCURSOR

	this.isTrackable = function (obj) {
		if (obj.tagName) {
			var t, skip = (obj.tagName==hX_5._BTN || obj.tagName==hX_5._ANC);
			if (!skip && obj.tagName == hX_5._INP) {
				t = obj.type.toLowerCase();
				skip = (t==hX_5._SBM || t==hX_5._RES || t==hX_5._lIMG || t==hX_5._TBTN)
			}
			// For now ignore whether we are in the tab order or not, if focus got here
			// we'll assume we want to track it
//			if (!skip) skip = (obj.tabIndex!=undefined && obj.tabIndex < 0);
			return !skip;
		}
		return false;
	}
	
	// Attributes supported on this object
	this.attributeTable = [];
		// Currently no arguments
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorTrack.prototype = new hX_5.JSFBehavior();

// Override:  Attach the appropriate events when the page loads
HxG_5.prototype.JSFBehaviorTrack.prototype.setEventCode = function () {
	// We only do this once -- when behavior is first run
	if (!this.HTMLrendered) {
		// We currently only support this for forms
		if (this.DOMobj && this.DOMobj.id && this.DOMobj.id.length > 0 && this.DOMobj.tagName && this.DOMobj.tagName == hX_5._FRM) {
			// Check that the "return value" field is present
			this.lastField = document.getElementById(this.DOMobj.id + ":POST_LASTFIELD");
			this.lastCursor= document.getElementById(this.DOMobj.id + ":POST_LASTCURSOR");
			if (this.lastField != null && this.lastField.value!=undefined) {
				this.extraEvents=[];
				this.lastField.disabled = false;
				if (this.lastCursor != null) this.lastCursor.diabled = false;
				// For IE, we can trap all focus events directly from the form
				if (hX_5.brw.isIE()) {
					hX_5.attachBehaviorEvent(this.DOMobj, this.type, "onfocusin", hX_5._lF, hX_5._onIN);
					this.extraEvents.push([this.DOMobj,"onfocusin"]);
				// For Netscape, we have to add the events to each "tabbable item"					
				} else {
					var i, obj, tabElements = hX_5.getTabElements();
					for (i=0; i < tabElements.length; i++) {
						obj = tabElements[i].obj;
						if (obj && obj.id && obj.tagName && this.isTrackable(obj)) {
							hX_5.attachBehaviorEvent(obj, this.type, hX_5._onF, hX_5._lF, "oninput_FORM");
							this.extraEvents.push([obj,hX_5._onF]);
						}
					}
				}
				hX_5.attachBehaviorEvent(this.DOMobj, this.type, hX_5._onMU, hX_5._lF, hX_5._onIN);
				hX_5.attachBehaviorEvent(this.DOMobj, this.type, hX_5._onKU, hX_5._lF, hX_5._onIN);
				this.extraEvents.push([this.DOMobj, hX_5._onMU]);
				this.extraEvents.push([this.DOMobj, hX_5._onKU]);
				// If we manually set focus, we won't get a focus event
				// So set a flag to track this case
				if (hX_5.cursorTracking == null) hX_5.cursorTracking = [];
				hX_5.cursorTracking.push(this);
			}
		}
	}
}

// Public: Execute the action
HxG_5.prototype.JSFBehaviorTrack.prototype.dispatchBehavior = function (thisObj, evt) {
	var src, result = true;
	if ((this.DOMobj != null) && (evt != null)) {
		src = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : (hX_5.brw.isIE() ? document.body : null));
		this.markFocus(src, evt.type);
	}
	return result;
}

// Public: Mark a field as having focus
HxG_5.prototype.JSFBehaviorTrack.prototype.markFocus = function (src, etype) {
	if (src != null) {
		if (src.form!=undefined && src.form!=null && src.form==this.DOMobj) {
			var id = (src.id!=undefined && src.id!=null && src.id.length>0)?src.id:hX_5._E;
			id = (id.length>0) ? id : (src.name!=undefined && src.name!=null && src.name.length>0)?src.name:hX_5._E;
			// Make sure this is a field we want to track
			if (id.length > 0 && hX_5.isTabbable(src) && this.isTrackable(src)) {
				if (etype=="focusin" || etype==hX_5._evF) {
					this.lastField.value = id;
					if (this.lastCursor != null) this.lastCursor.value = -1;
				} else if (this.lastCursor != null && (etype==hX_5._evMU || etype==hX_5._evKU)) {
					if (this.lastField.value == id) {
						var t = (src.type) ? src.type.toLowerCase() : hX_5._E;
						this.lastCursor.value = (src.tagName==hX_5._TXT || (src.tagName==hX_5._INP && t==hX_5._XT )) ? hX_5.sel.getCaretPos(src) : -1;
					} 
				}
			}
		}
	}
}

// -------------------------------------------------------
// JSF Behavior - Mouseclick
//
// Catch right mouse button clicks or catch all mouse activity
// 
// Arguments:
//
//	see documentation.
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorMouseclick = function () {
	this.type = hX_5.BHR_mouseclick;
	this.suppressAttach = true;
	this.timer = null;
	
	// NOTE:  Since a component can be bound to multiple types of clicks (left and right)
	// when a new behavior is added, we create a subcomponent
	this.mouseArray = [];

	// Attributes set by the caller -- passed into the key array
	this.scope	= hX_5._RGT;	// "LEFT", "RIGHT" or "ALL"
	this.fcn	= null;			// func1() ... etc
	this.actions= [];			// "lock"/"unlock"/func1() ... etc
	this.targets= [];			// action target ID
	this.extraEvents = null;
	this.eventIds = null;

	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['scope']		= ['scope',		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['action']		= ['actions',	hX_5.ARG_TO_NAU,	false,	null,	null];
		this.attributeTable ['function']	= ['fcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['target']		= ['targets',	hX_5.ARG_TO_NAN,	false,	null,	null];
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorMouseclick.prototype = new hX_5.JSFBehavior();

// Augment: Description of a single click action
HxG_5.prototype.JSFMouse = function () {
	this.scope = hX_5._RGT;
	this.actions = this.fcn = this.targets = null;
}

// Augment: Used to mutate a behavior into a "sub-behavior", that is, a <tag><event> pair
// can have lots of mouseclick "sub-behaviors"
HxG_5.prototype.JSFBehaviorMouseclick.prototype.addSubComponent = function(obj) {
	var sub = new hX_5.JSFMouse();
	sub.scope = obj.scope;
	if (obj.actions)	sub.actions	= obj.actions;
	if (obj.targets)	sub.targets	= obj.targets;
	if (obj.fcn) 		sub.fcn 	= obj.fcn;
	this.mouseArray.push(sub);
}

// Override:  Attribute handling
HxG_5.prototype.JSFBehaviorMouseclick.prototype.setAttributeCode = function() {
	var o, i, found=false;
	if (this.scope) this.scope = this.scope.toLowerCase();
	for (i=0; i < this.mouseArray.length; i++) {
		o = this.mouseArray[i];
		if (o.scope==this.scope) {
			found=true;
			if (this.actions)	o.actions	= this.actions;
			if (this.targets)	o.targets	= this.targets;
			if (this.fcn) 		o.fcn 	= this.fcn;
		}
	}
	if (!found) this.addSubComponent(this);
}

// Override:  Destroyer
HxG_5.prototype.JSFBehaviorKeybind.prototype.destroyCode = function () {
	var o;
	while (this.mouseArraylength>0) {
		o = this.mouseArray.pop();
		delete o;
	}
}

// Override:  Attach the appropriate events when the page loads
HxG_5.prototype.JSFBehaviorMouseclick.prototype.setEventCode = function () {
	// Compute the merged scope (all clicks)
	var i, ms=null, eID;
	for (i=0; i<this.mouseArray.length; i++) {
		if (ms==null) ms = this.mouseArray[i].scope;
		else if (this.mouseArray[i].scope==hX_5._ALL) ms = hX_5._ALL;
		else if (this.mouseArray[i].scope==hX_5._LFT) ms = (ms==hX_5._LFT)?hX_5._LFT:hX_5._ALL;
		else ms = (ms==hX_5._LFT)?hX_5._LFT:hX_5._ALL;
		if (ms==hX_5._ALL) break;
	}

	// In IE, attach the event to the oncontextmenu handler
	if (document.body.setCapture) {
		if (this.extraEvents==null) this.extraEvents=[];
		// Following catches the right mouse click
		if (ms == hX_5._RGT || ms == hX_5._ALL) {
			hX_5.attachEvent(this.DOMobj, hX_5._onCM, hX_5._lL, "hX_5.dispatchMouse(this, event)");
			this.extraEvents.push([this.DOMobj, hX_5._onCM]);
		}
		// Following catches the left mouse click
		if (ms == hX_5._LFT || ms == hX_5._ALL) {
			hX_5.attachEvent(this.DOMobj, hX_5._onMC, hX_5._lL, "hX_5.dispatchMouse(this, event)");
			hX_5.attachEvent(this.DOMobj, hX_5._onMCC, hX_5._lL, "hX_5.dispatchMouse(this, event)");
			this.extraEvents.push([this.DOMobj, hX_5._onMC]);
			this.extraEvents.push([this.DOMobj, hX_5._onMCC]);
		}
		hX_5.attachEvent(this.DOMobj, hX_5._onMD, hX_5._lL, "hX_5.dispatchMouse(this, event)");
		hX_5.attachEvent(this.DOMobj, hX_5._onMU, hX_5._lL, "hX_5.dispatchMouse(this, event)");
		this.extraEvents.push([this.DOMobj, hX_5._onMD]);
		this.extraEvents.push([this.DOMobj, hX_5._onMU]);

	// In Mozilla, we catch all mouse events at the "rootmost container", and then we ignore them
	} else if (document.body.addEventListener) {
		this.eventIds=[];
		if (this.extraIds==null) this.extraIds=[];
		eID = hX_5.attachEvent(window, hX_5._onMD, hX_5._lL, hX_5.dispatchMouse1, null, null, true);
		this.eventIds.push([window, hX_5._onMD, eID]);
		eID = hX_5.attachEvent(window, hX_5._onMU, hX_5._lL, hX_5.dispatchMouse1, null, null, true);
		this.eventIds.push([window, hX_5._onMU, eID]);
		eID = hX_5.attachEvent(window, hX_5._onMC, hX_5._lL, hX_5.dispatchMouse1, null, null, true);
		this.eventIds.push([window, hX_5._onMC, eID]);
		eID = hX_5.attachEvent(window, hX_5._onMCC,hX_5._lL, hX_5.dispatchMouse1, null, null, true);
		this.eventIds.push([window, hX_5._onMCC, eID]);
	}
}

// Private -- Handle deferring an autotab until after a keypress event is processed
HxG_5.prototype.JSFBehaviorMouseclick.prototype.dispatchTimer = function (item, action) {
	document.body.releaseCapture();
	this.timer = null;
}

HxG_5.prototype.dispatchMouse1 = function (evt) {
	hX_5.dispatchMouse(this, evt);
}

// Private: Handle dispatching the right mouse click
HxG_5.prototype.dispatchMouse = function (thisp, evt) {
	if ((typeof hX_5)=="undefined") return true;
	evt = (evt) ? evt : ((event) ? event: null);
	var j, m, p, b=(evt.type==hX_5._evCM?2:((evt.button<=1)?0:2)), thisB=null, src = hX_5.i.dispatchID(thisp);
	// IE - the event is on the id, so see if we have a behavior for this id
	if (document.body.setCapture) {
		src = hX_5.i.dispatchID(thisp);
		if (evt && src) {
			m = hX_5.getBehaviorsById(src);
			for (j=0; j<m.length; j++) {
				if (m[j].type==hX_5.BHR_mouseclick && m[j].eventname==hX_5._onMS) {
					thisB = m[j]; break;
				}
			}
		}
	// Mozilla -- the event is on the window, see if we have a bind
	// starting at the scope of the target and walking up
	} else {
		p = evt.target;
		while (p && thisB==null) {
			if (p.id || p.tagName==hX_5._BBODY) {
				m = hX_5.getBehaviorsById(p.tagName==hX_5._BBODY?hX_5._BODY:p.id);
				for (j=0; j<m.length; j++) {
					if (m[j].type==hX_5.BHR_mouseclick && m[j].eventname==hX_5._onMS) {
						thisB = m[j]; break;
					}
				}
			}
			p = p.parentNode;
		}
	}
	if (thisB) {
		m = false;
		for (j=0; j<thisB.mouseArray.length; j++) {
	 		if (thisB.mouseArray[j].scope==hX_5._ALL || (thisB.mouseArray[j].scope==hX_5._LFT && b==0) || (thisB.mouseArray[j].scope==hX_5._RGT && b==2)) {
				if ((thisB.mouseArray[j].actions!=null && thisB.mouseArray[j].actions.length>0 && thisB.mouseArray[j].actions[0]!=null) || thisB.mouseArray[j].fcn != null) {
					if (evt.type==hX_5._evMD) {
						if (!m && document.body.setCapture) thisB.DOMobj.setCapture();
						m = true;
					} else if (evt.type==hX_5._evMU) {
						if (!m && document.body.setCapture && thisB.timer==null)
							thisB.timer = hX_5.i.startTimerTimed (thisB.id, 0, hX_5._onMS, 1, hX_5.BHR_mouseclick, hX_5._onMS);
						m = true;
					} else if (evt.type==hX_5._evMC || evt.type==hX_5._evCM) {
						hX_5.action.processAction(thisB.DOMobj, evt, thisB.mouseArray[j].fcn, thisB.mouseArray[j].actions, true, thisB.mouseArray[j].targets, null, null, null);
						m = true;
					} else if (evt.type==hX_5._evMCC) {
						m = true;
					}
				}
			}
		}
		if (m) {
			hX_5.i.cancelAction(evt, true);
			return false;
		}
	}
	return true;	// let the event continue if we didn't want it
}


// -------------------------------------------------------
// JSF Behavior - SubmitListener for ODC components
//
// Specialized behavior attached to the onsubmit event of forms
// to trap the event and cause ODC stuff to happen.
// 
// Arguments:
//
//	see documentation.
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorSubmitListener = function () {
	this.type = hX_5.BHR_submitlistener;
	this.beforeOthers = false;

	// Attributes supported on this object
	this.attributeTable = [];
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorSubmitListener.prototype = new hX_5.JSFBehavior();

// Public: Execute the action
HxG_5.prototype.JSFBehaviorSubmitListener.prototype.dispatchBehavior = function (thisObj, evt) {
	if (ODCRegistry!=undefined && ODCRegistry!=null && thisObj.tagName!=undefined && thisObj.tagName==hX_5._FRM) {
		ODCRegistry.saveControlsToForm(thisObj);
	}		
}


// -------------------------------------------------------
// AJAX Behavior - Common code
//

// FIXME: Disabling and enabling of the div
// FIXME: Adding keyboard and title handling
// FIXME: decoding hidden fields outside the div
// FIXME: getting iframes to work for post
// FIXME: CODE UP ALl these cases and make sure release is always called

HxG_5.prototype.JSFBehaviorAjax = function () {
	// Implementation
	this.backChannel=null;		// The associated back channel object
	this.DOMParser=null;		// Iframe used to do parsing
	this.deferScript = null;	// Block to be executed later
	this.deferScriptChar = null;// Charset of block to be executed later
	this.deferComp = null;		// Deferred component
	this.deferCompAtt = null;	// Deferred component attributes
	this.nowScript = false;		// Script to be processed now
	this.locked = false;		//
	this.saveId = null;
	this.saveEvt = null;
	this.pauseId = null;
}

// Public: Page level onLoad handler
HxG_5.prototype.JSFBehaviorAjax.prototype.onPageLoad = function () {
	this.saveId = this.DOMobj.id;
	hX_5.mustDestroy.push(this);
	return true;
}

// Public: Set an "attribute" on the behavior.
HxG_5.prototype.JSFBehaviorAjax.prototype.setAttribute = function(a) {
	hX_5.parseArg (this.attributeTable, this, (arguments.length>1)?arguments:a);
	return true;
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.JSFBehaviorAjax.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public: Destroyer
HxG_5.prototype.JSFBehaviorAjax.prototype.destroy = function () {
	if (this.backChannel) {
		if (this.backChannel.destroy) this.backChannel.destroy();
		delete (this.backChannel);
	}
	if (this.DOMParser) {
		this.DOMParser.parentNode.removeChild(this.DOMParser);
		delete this.DOMParser;
	}
	if (this.deferScript) delete this.deferScript;
	if (this.deferCompAtt) delete this.deferCompAtt;
	if (this.saveEvt) delete this.saveEvt;
	var i,d = [this.paramNames,this.paramFields,this.actions,this.targets];
	for (i=0; i<d.length; i++) {
		if (d[i]) {
			while (d[i].length>0) d[i].pop();
			delete d[i];
		}
	}
	delete d;
	if (this.attributeTable) delete this.attributeTable;
	return true;
}

// Public: Execute the action
HxG_5.prototype.JSFBehaviorAjax.prototype.dispatchBehavior = function (thisObj, evt) {
	var  r=true, m, t=this.DOMobj.tagName, url=((this.href==null || this.href.length==0)?window.location.pathname:hX_5.stripURL(this.href));
	url=hX_5.reEncodeURI(url);
	if (this.locked) return r;	// If we are in the midst of changing the DOM, then ignore a get that comes thru
	if (this.saveId) this.DOMobj = hX_5.getElementById (this.saveId);
	this.pauseId = null;

	// Do a quick check that we're working against an object that isn't blatantly wrong
	if (t==hX_5._INP || t==hX_5._TXT || t==hX_5._BTN || t==hX_5._SEL) {
		this.callbackError(hX_5.INTERR_bc_badpanel);
		return false;
	}
	
	// We should have a url to issue the request against
	if (url!=null) {

		// If there's a setup function, run it
		this.saveEvt = null;
		if (this.startFcn!=null) r=hX_5.action.processFunction(this.DOMobj, evt, this.startFcn);
		if (evt!=null) this.saveEvt = hX_5.i.duplicateEvent(evt);
		if (r) {

			// Handle "disabling" the UI of the panel
			this.uilock();
			this.pauseId = hX_5.action.pauseActions();
			m = (this.type==hX_5.BHR_ajaxget)?"axpartial":((this.type==hX_5.BHR_ajaxsubmit)?"axfull":"axnone");

			// We're going to make an AJAX call (vs. an iframe call).  We'll assume the
			// backchannel will "fail over" and try via an iframe if AJAX isn't available
			if (this.backChannel)
				this.backChannel.reset();
			else
				this.backChannel = new hX_5.HTTPBackchannel("backChannel", this.callbackBehavior, ((this.type==hX_5.BHR_ajaxsubmit)?true:false), true, this.DOMobj.id+"_AJAX_IFRAME", this.callbackError, true, hX_5.isPortal);
			// If this is a "submit" and we want JSF to run the action associated with it
			// Push a hidden field into the page with the name of the submit button.
			if (this.type==hX_5.BHR_ajaxsubmit && evt!=null) {
				var s = hX_5.evtGetTarget(evt), tagset = [];
				tagset.push(new hX_5.tagListItem(hX_5._INP,hX_5._SBM));
				tagset.push(new hX_5.tagListItem(hX_5._INP,hX_5._lIMG));
				tagset.push(new hX_5.tagListItem(hX_5._BTN,hX_5._SBM));
				s =(hX_5.isTag(s, tagset))?s.name:null;
				delete tagset;
			}
			this.backChannel.LoadEx(url, this, this.DOMobj, m, this.paramNames, this.paramFields, null, ((evt!=null)?hX_5.evtGetTarget(evt):null), s);
			// We're done here -- now wait for the request to come home
		}
	}
	return r;
}

// Public: Callback handler when the request completes
HxG_5.prototype.JSFBehaviorAjax.prototype.callbackBehavior = function (XMLData, StringData) {
	var evt=null, r=true;
	if (XMLData && XMLData.documentElement) {
/*	
		alert ("Not sure how to process");
		But it's something like:
		    response  = req.responseXML.documentElement;
	        method    =	response.getElementsByTagName('method')[0].firstChild.data;
	        result    = response.getElementsByTagName('result')[0].firstChild.data;
or
		myData = new hX_5.XMLData(XMLData);
*/
	} else if (StringData) {
// DEBUG POINT -- StringData contains entire content transmitted
// alert (StringData);		
		var b, c, spcO=false, o, o1=null, DOM_o1=null, oH, i, j, k, l, h, t, svParent, svPos, svId, sE, tB="</BODY", tH="</HEAD>", tPH ='<head title="PORTALcontent">', tPHX = "</head", isP=false, skip, firstTag=null, ch, ch1, chB=-1, w1, w2, BODY, lsrc="src", lcharset="charset", ltype = "type", aa, ad=[], ab=document.getElementsByTagName(hX_5._SCR), lab=((ab)?ab.length:0);
		this.deferScript = hX_5._E;
		if (this.deferCompAtt) delete this.deferCompAtt;
		this.deferComp = this.deferCompAtt = this.deferScriptChar = null;
		this.nowScript = false;
		if (this.saveId) this.DOMobj = hX_5.getElementById (this.saveId);

		// If this is refreshRequest or updateRequest and not portal, we should have no head or body tag in what's returned (unless it's in a portlet)
		// If this is an externalRequest or portal , we might have a head and body tag. Parse the string down to be just the body tag.  
		// We could do a "real" parse of the HTML but this is expensive and slow
		// Instead, look for the </HEAD> tag and toss everything before it.  Then parse for the <BODY> tag.
		// Now someone could do something strunky like have a comment or script block that has the string </HEAD>
		// and we'd fail in that case, but we'll just tell people that they shouldn't do that
		// If this is portal, there could be a "second" head tag (and close) marking the start of the content
		// of the portlet we're interested in.  In this case, after the <body> tag, skip down to the portlet head tag
		l = StringData.length;
		ch = StringData.indexOf(tH);
		if (ch<0) ch = StringData.indexOf(tH.toLowerCase());
		if (ch<0 && this.type!=hX_5.BHR_ajaxgetexternal && !hX_5.isPortal) {
			BODY = StringData;
		} else {
			if (ch<0) ch=0; else ch+=tH.length;
			// After the </HEAD> tag, look for a body tag
			while (ch!=null && ch<l) {
				// Look for next tag start
				while (ch<l && StringData.charAt(ch)!=hX_5._LT) ch++;
				// Parse the located tag
				ch1 = hX_5.htmlParseTag(StringData, ch, false);
				if (ch1==null) ch=null;
				else {
					ch = ch1.ch;
					if (ch1.tag==hX_5._BBODY) {chB=ch; ch=null; }
				}
			}
			// chB points to the char after the close of the <body> tag
			// if this is portal, see if there's a proprietary tag marking the portlet content we're interested in
			if (hX_5.isPortal) {
				ch = StringData.indexOf(tPH, chB);
				if (ch>=0) {
					chB = ch+tPH.length;
					isP = true;
				}
			}
			// chB points to the char after the close of the body  or char after the close of the portal head tag
			// Look for the portal close tag if this is portal otherwise do a dirty find for the end body tag
			if (isP) {
				ch = StringData.lastIndexOf(tPHX);
				if (ch<0) {
					this.callbackError(hX_5.INTERR_bc_baddata);
					return false;
				}
			} else {
				ch = StringData.lastIndexOf(tB);
				if (ch<0) ch = StringData.lastIndexOf(tB.toLowerCase());
			}
			if (chB<0) {
				this.callbackError(hX_5.INTERR_bc_baddata);
				return false;
			}
			BODY = (ch>0) ? StringData.substring(chB,ch) : StringData.substr(chB);
		}
// DEBUG POINT -- BODY should now contain only the contents of the body tag
// alert (BODY);

		// Parse the tags into a tree of DOM objects
		if (this.DOMParser==null) this.DOMParser = hX_5.ifr.create(document, this.DOMobj.id + "_DOMPARSER", true, false);
		b = hX_5.ifr.getDoc(this.DOMParser);
		// N.B. We use an iframe instead of a doc fragment so we have scoping of the ids of elements within the fragment
		// we want to do a getElementById on the frag.  If the Frag is part of the same doc, we'll just refind the
		// elements already in the doc
		if (b && b.body) {
			try {
				// It's possible we're missing a "container" tag that we must have to parse the HTML.  Appears this is 
				// only true of "TABLE".
				ch=0; l=BODY.length; w1=hX_5._SCR.toLowerCase(); w2=(hX_5._SL+hX_5._SCR).toLowerCase();
				while (ch<l && BODY.charAt(ch)!=hX_5._LT) ch++;
				// Parse for the first non-script, non-comment
				ch1 = hX_5.htmlParseTag(BODY, ch, false);
				while (ch1!=null && ch<l && (ch1.tag==null || ch1.tag==hX_5._E || ch1.tag.toLowerCase()==w1 || ch1.tag.toLowerCase()==w2)) {
					if (firstTag==null && ch1.tag!=hX_5._E) firstTag=ch1.tag.toLowerCase();
					ch = ch1.ch;
					while (ch<l && BODY.charAt(ch)!=hX_5._LT) ch++;
					ch1 = hX_5.htmlParseTag(BODY, ch, false);
				}
				if (ch1!=null && (ch1.tag==hX_5._TR || ch1.tag==hX_5._TBDY || ch1.tag==hX_5._THED))
					BODY = (hX_5._LT + hX_5._TAB + hX_5._GT) + BODY + (hX_5._LT + hX_5._SL + hX_5._TAB + hX_5._GT)
				// Not clear why this is needed, but IE has problem parsing script blocks without something like it
				if (hX_5.brw.isIE() && firstTag!=null && firstTag==w1)
					BODY = "<SPAN style='display:none'><BR></SPAN>"+BODY;

// DEBUG POINT -- BODY should now contain only the contents of the body tag
// alert (BODY);

				// Now set the run of tags into the iframe
				while (b.body.lastChild!=null) b.body.removeChild(b.body.lastChild);
				aa=b.getElementsByTagName(hX_5._SCR);
				hX_5.setIHTML(b.body, BODY, b);

				// Try to find the source-tag and failing that the id of the panel
				o = b.getElementById((this.source)?this.source:this.DOMobj.id);
				// If we can't find that, use the whole body
				o = (o==null)?b.body:o;
// DEBUG POINT -- See what we've located in the iframe
// alert (o.innerHTML);
				if (o) {

					// Is this a component which may need special treatment?
					c = (o.id==undefined)?null:hX_5.getComponentById(o.id);
					if (c!=null) spcO = (c.type==hX_5.CMP_dialog || c.type==hX_5.CMP_section || c.type==hX_5.CMP_menu);
					if (spcO) {
						// Save off the current attributes and their state of these components
						// as we're going to be rebuilding the whole component
						this.deferComp = o.id;
						this.deferCompAtt = [];
						for (i in c.attributeTable) {
							s = c.getAttribute(i);
							if (s!=null) this.deferCompAtt.push([i,s+hX_5._E]);
						}
						// A few components have multiple tags that we're interested in, identify them
						if (c.type==hX_5.CMP_menu) {
							i = o.nextSibling;
							if (i!=null && i.tagName!=undefined && i.tagName==hX_5._DIV) {
								o1 = i;
								i = this.DOMobj.nextSibling;
								if (i!=null && i.tagName!=undefined && i.tagName==hX_5._DIV) DOM_o1 = i;
							}
						}
					}

					// At this point we have a run of tags to slam into the document
					// However, if there's a JS include in the block, we have to 
					// process the include and then LATER (next message through)
					// slam the tags into the document
					// This is cuz the browsers handle JS includes differently (including
					// when the JS gets compiled).  The JS needs to be included and compiled
					// BEFORE any inline script (including event handlers) gets correctly executed

					// Pull the scripts out of the returned stuff -- note that we are looking at
					// script found ANYWHERE in the returned "body" 
					h=document.getElementsByTagName('head')[0]; aa=b.getElementsByTagName(hX_5._SCR);
					if (aa==null || aa.length==0) aa = b.getElementsByTagName(hX_5._SCR.toLowerCase());	// Mozilla sometimes likes lower case
					if (aa!=null) {
						for (i=0; i<aa.length; i++) {
							// Check for inline script blocks
							if (aa[i].src==null || aa[i].src.length==0) {
								// For get and submit, we want any tag in the body
								// For external, things are dicier.  Really we'd like to only take the scripts IN THE PANEL (or one level 
								// up if this is a special object).  However, if this is a faces jsp that we're referencing, the scripts
								// are rendered by script collector at the "end of the page". So for now, we take them all
//								if (this.type==hX_5.BHR_ajaxget || this.type==hX_5.BHR_ajaxsubmit || spcO || hX_5.isChildTag (aa[i], o)) {
									this.deferScript = this.deferScript + aa[i].text;
									if ((this.deferScriptChar==null||this.deferScriptChar.length==0) && aa[i].charset!=null) this.deferScriptChar = aa[i].charset;
									ad.push(aa[i]);
//								}
							// Check for script includes anywhere in the body
							} else {
								skip = false;
								// Skip the include if it's already in the page
								for (j=0; !skip && j<lab; j++) {
									// Mozilla will have the full pathname of the src in the .scripts[] list of
									// known scripts, but the relative pathanme in "src".
									k = ab[j].src.lastIndexOf(aa[i].src);
									if (k>=0 && (k+aa[i].src.length >= ab[j].src.length)) skip = true;
								}
								if (!skip) {
									sE = document.createElement(hX_5._SCR);
									sE.setAttribute(lsrc,aa[i].src);
									if (aa[i].type!=null && aa[i].type.length>0) sE.setAttribute(ltype,aa[i].type);
									if (aa[i].charset!=null && aa[i].charset.length>0) sE.setAttribute(lcharset,aa[i].charset);
									h.appendChild(sE);
									ad.push(aa[i]);
									this.nowScript = true;
								}
							}
						}
					}
					// We can't destroy scripts in the above block -- IE is unhappy and gets really wacked out in the array processing
					for (i=0; i<ad.length; i++) if (ad[i] && ad[i].parentNode) (ad[i].parentNode).removeChild(ad[i]);

// DEBUG POINT: BODY content has script tags removed
// alert (BODY);				

					// At this point we have the tags that are to go into the panel as children
					// Inline script has been removed from it
					// Script includes are in the document's head waiting to finish
					if (this.nowScript) hX_5.AJAX_prototypeloader();

					// Ensure the UI is released
					hX_5.redraw();

					// Destroy the existing content in the panel
					while (this.DOMobj.lastChild!=null) this.destroyNode(this.DOMobj.lastChild);
					if (spcO) {
						// If this is a special "component", we may need to destoy the component
						// as well as its children
						svPos = this.DOMobj.nextSibling;
						svId = this.DOMobj.id;
						svParent = this.DOMobj.parentNode;
						this.destroyNode(this.DOMobj);
						if (DOM_o1!=null) {
							svPos = DOM_o1.nextSibling;
							this.destroyNode(DOM_o1);
						}
					}
					// Replace the content of the DOM obj with the HTML that makes up
					// the object in the iframe
					if (this.DOMobj.tagName != hX_5._TAB) {
						if (!spcO) {
							var ihtml = hX_5.getIHTML(o);
							if (ihtml!=null) hX_5.setIHTML(this.DOMobj, ihtml);
						} else {
							// For special cases like dialog boxes, we're going to replace the ENTIRE dialog
							// not just hte content.  We've already destructed the dialog, so
							// we need to create a "new one" (or at least a placeholder) with the
							// charactersitics of the one in the DOMParser
							if (o.outerHTML) {
								// For IE, we do this by making an "empty" DIV and then 
								// replacing it with the DIV in the retrieved content
								t = document.createElement(hX_5._DIV);
								t.innerHTML = o.outerHTML;
								this.DOMobj = t.firstChild;
								if (svPos) svParent.insertBefore(this.DOMobj, svPos); else svParent.appendChild(this.DOMobj);
								if (o1!=null && DOM_o1!=null) {
									t.innerHTML = o1.outerHTML;									
									if (this.DOMobj.nextSibling) svParent.insertBefore(t.firstChild, this.DOMobj.nextSibling); else svParent.appendChild(t.firstChild);
								}
								delete t;
							} else {
								this.DOMobj = document.importNode(o,false);
								if (svPos) svParent.insertBefore(this.DOMobj, svPos); else svParent.appendChild(this.DOMobj);
								hX_5.setIHTML(this.DOMobj, hX_5.getIHTML(o));
								if (o1!=null && DOM_o1!=null) {
									DOM_o1 = document.importNode(o1,false);
									if (this.DOMobj.nextSibling) svParent.insertBefore(DOM_o1, this.DOMobj.nextSibling); else svParent.appendChild(DOM_o1);
									hX_5.setIHTML(DOM_o1, hX_5.getIHTML(o1));
								}
							}
						}
					} else {
						// innerHTML is "readonly" in IE, so this hack generally works
						if (o.outerHTML) {
							// If this is an external request, before we slam outerhtml into our DOM,
							// make sure we fix the id
							if (this.type==hX_5.BHR_ajaxgetexternal && o.id!=undefined && this.DOMobj.id != o.id)
								o.id = this.DOMobj.id;
							// fixme: There still might be a problem with this if a behavior is attached to the table (or anything else?)									
							this.DOMobj.outerHTML = o.outerHTML;
						} else {
							hX_5.setIHTML(this.DOMobj, hX_5.getIHTML(o));
						}
					}

					// If client side state saving is turned on, we have to find the state saving hidden field
					// and update the value
					if (this.type==hX_5.BHR_ajaxsubmit && b.body) {
						// If the field is present, it should be a hidden field at the "top" level of the returned data
						oH = b.body.firstChild;
						var f, pre1="com.ibm.faces", pre2="com.sun.faces", cases=["com.ibm.faces.JWLAJAX.VIEW","com.ibm.faces.ENHANCED_SERVER_STATE_SAVING", "com.sun.faces.VIEW"];
						while (oH!=null) {
							if (oH.tagName!=undefined && oH.tagName==hX_5._INP && oH.type==hX_5._HI && oH.name!=undefined) {
								if (oH.name.indexOf(pre1)>=0 || oH.name.indexOf(pre2)>=0) {
									for (i=0; i<cases.length; i++) {
										if (oH.name==cases[i]) {
											// Find the right form
											if (this.DOMobj.form!=undefined) f = obj.form;
											else {
												j = this.DOMobj;
												while (j!=null && j.tagName!=undefined && j.tagName!=hX_5._FRM) j=j.parentNode;
												f = (j!=null && j.tagName!=undefined && j.tagName==hX_5._FRM)?j:null;
												if (f==null) {
													j = document.getElementsByTagName(hX_5._FRM);
													f = (j && j.length>0) ? j[0] : null;
												}
											}
											if (f!=null) {
												j = f.elements[oH.name];
												if (j!=null) {
													if (j.length!=undefined) j = j[0];
													j.value = oH.value;
													break;
												}
											}
										}
									}
								}
							}
							oH = oH.nextSibling;
						}
					}

					// Done here, queue the script replace if needed, otherwise return
					if (this.deferScript.length>0)
						hX_5.i.startObjTimerTimed(this, 0, (hX_5.brw.isIE()?10:50));
					else {
						// Success: Call the end function
						if (this.endFcn!=null) r=hX_5.action.processFunction(this.DOMobj, this.saveEvt, this.endFcn);
						if (this.pauseId) hX_5.action.unPauseActions(this.pauseId);
						this.locked = false;
					}
				}
				// Some explicit clean up of large things				
				if (BODY!=null) delete BODY;
					
			} catch (e) {
				this.callbackError(hX_5.INTERR_bc_baddata + ((e.message!=undefined && e.message.length>0)?(hX_5._B + e.message):hX_5._E));
			}
		}
	}
	return r;
}

// Destroy a node (tag) and any associated hxclient gunk
HxG_5.prototype.JSFBehaviorAjax.prototype.destroyNode = function (node) {
	var a, i, k=null;
	// Destroy children
	while (node.lastChild!=null) this.destroyNode(node.lastChild);
	// Destroy JSF components associated with this tag
	if (node.id!=undefined) {
		i = hX_5.getComponentById(node.id);
		if (i!=null) hX_5.removeComponent(i);
		// Destroy JSF behaviors associated with this tag
		a = hX_5.getBehaviorsById(node.id);
		for (i=0; a!=null && i<a.length; i++) if (a[i] != this) hX_5.removeBehavior(a[i]);
	}
	// Remove the tag from the DOM
	if (node.tagName!=undefined && node.tagName==hX_5._TD) {
		// N.B. Mozilla and somtimes IE sometimes have out of bounds indices ESPECIALLY
		// when dealing with odc:trees and tabbed panels.  Both of these components
		// have bad code in them which sets display:none on table rows/cells (which
		// you shouldn't do).  The result is that the cellindex (rowindex) for the
		// item is wrong (the cellindex/rowindex reflects VISIBLE rows/cells)
		// We handle this by always removing the "last" cell/row and handling
		// a DOM failure when we do this (which can happen in IE)
		try {
			node.parentNode.deleteCell(node.parentNode.cells.length-1);
		} catch(e) {
			k = node.parentNode.removeChild(node);	
		}
	} else if (node.tagName!=undefined && node.tagName==hX_5._TR) {
		try {
			node.parentNode.deleteRow(node.parentNode.rows.length-1);
		} catch(e) {
			k = node.parentNode.removeChild(node);	
		}
	} else if (node.tagName!=undefined && (node.tagName.toLowerCase()==hX_5.TAG_OBJECT || node.tagName.toLowerCase()==hX_5.TAG_EMBED)) {
		// Perhaps as a consequence of how we (and others) have to work around the assorted legal issues with the OBJECT and EMBED
		// tags (using DOM APIs to construct them isntead of direct tags in a page), IE seems to have problems
		// destroying the windows associated with these tags (e.g., if it's a Media Player or Quicktime Player)
		// resulting in the DOM object being destroyed but not the activeX window
		// The following leaks memory but it seems to at least work
		if (hX_5.brw.isIE()) {
			try {
				node.stop();		// Many activeX controls support this but you CANNOT query for the interface
			} catch(e) {
									// failure will happen
			}
			node.style.display="none";	// Hide the control
		}
		k = node.parentNode.removeChild(node);	// Remove the node
	} else
		k = node.parentNode.removeChild(node);
	// If possible destroy the DOM object
	if (k!=null) delete k;
}

// Finish installing the HTML (specifically scripts) into the panel
HxG_5.prototype.JSFBehaviorAjax.prototype.dispatchTimer = function () {
	var o, i, s, spn, r = true;
	if (this.deferScript.length>0) {
		try {
			if (this.nowScript) hX_5.AJAX_prototypeloader();
			// Mark hxclient as being in "page loading" mode
			// This holds components/behaviors from loading until we've finished up
			// getting all script to compile
			// When the scripts are done loading, we'll then load all behaviors/components
			hX_5.bodyLoadDone = false;

			o = document.getElementById((this.source)?this.source:this.DOMobj.id);
			o = (o==null)?document.body:o;	
			if (hX_5.brw.isIE()) {
				s = document.createElement(hX_5._SCR);
				s.setAttribute("type", "text/javascript");
				s.setAttribute("text", this.deferScript);
				if (this.deferScriptChar!=null && this.deferScriptChar.length>0) s.setAttribute("charset", this.deferScriptChar);
				o.appendChild(s);
			} else {
				spn = document.createElement(hX_5._SPN);
				s = "<SCRIPT type='text/javascript'";
				if (this.deferScriptChar!=null && this.deferScriptChar.length>0) s = s + " charset='" + this.deferScriptChar + hX_5._SQT;
				s = s + hX_5._GT;
				spn.innerHTML = s+this.deferScript+"</SCRIPT>";
				document.body.appendChild(spn);
			}
		} catch(e) {
			this.callbackError(hX_5.INTERR_bc_baddata + ((e.message!=undefined && e.message.length>0)?(hX_5._B + e.message):hX_5._E));
			r = false;
		}
	}
	// Now that script has run, if we re-instantiated an entire component, reset its attributes
	if (this.deferComp) {
		o = hX_5.getComponentById(this.deferComp);
		if (o) for (i=0; i<this.deferCompAtt.length; i++) o.setAttribute(this.deferCompAtt[i][0]+":"+this.deferCompAtt[i][1]);
	}
	// Finish up
	if (!hX_5.bodyLoadDone) hX_5.pageReLoadAction();
	if (r) if (this.endFcn!=null) r=hX_5.action.processFunction(this.DOMobj, this.saveEvt, this.endFcn);
	if (this.pauseId) hX_5.action.unPauseActions(this.pauseId);
	this.locked = false;
}

// Public: Callback handler when the request fails
HxG_5.prototype.JSFBehaviorAjax.prototype.callbackError = function (err) {
	if (this && this.uirelease) {
		this.uirelease();
		if (this.errorFcn!=null) r=hX_5.action.processFunction(this.DOMobj, null, this.errorFcn, err);
	}
	if (this.pauseId) hX_5.action.unPauseActions(this.pauseId);
}

// Public: Callback handler when the request fails
HxG_5.prototype.JSFBehaviorAjax.prototype.uilock = function () {
	this.locked = true;
	hX_5.uirelease(); // Take care with this -- it will release all timers including AJAX timers
}

// Public: Callback handler when the request fails
HxG_5.prototype.JSFBehaviorAjax.prototype.uirelease = function () {
	this.locked = false;
	if (this.pauseId) hX_5.action.unPauseActions(this.pauseId);
	// fixme: to be coded;
}

// -------------------------------------------------------
// AJAX Behavior - GetExternal
//
// Replace the content of a tag with content retreived via AJAX
// (from an external URL)
//
// Arguments:
//
//	see documentation.
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorAjaxGetExternal = function () {
	this.type = hX_5.BHR_ajaxgetexternal;

	// Attributes set by caller
	this.href = null;			// The URL of the page to do a get against
	this.paramNames = [];		// If parameters are to be encoded on URL, the names to give the parameters
	this.paramFields = [];		// If parameters are to be encoded on URL, ids of the fields to encode for each of the parameter names
	this.title = null;			// Title text used while in progress
	this.actions = [];			// If "in progress actions" are provided, the actions to use while in progress
	this.targets= [];			// Dummy vector to go with the above
	this.source = null;			// ID in the target page to use as the content	
	this.startFcn=null;			// If provided, BEFORE the request is made the function is run (can stop the request if returns false)
	this.endFcn=null;			// If provided, AFTER the request is succesfully made the function is run (does no run if request failed)
	this.errorFcn=null;			// If provided, if the request fails this is used instead of AFTER 
// FIXME: the following need to be handled (if possible)
	this.a_hreflang=null;
	this.a_charset=null;


	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['href']		= ['href',			hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['progress-actions']	= ['actions',		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['params']		= ['paramNames',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['param-fields']= ['paramFields',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['source']		= ['source',		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['title']		= ['title',			hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['on-start']	= ['startFcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['on-complete']	= ['endFcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['on-error']	= ['errorFcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['hreflang']	= ['a_hreflang',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['charset']		= ['a_charset',		hX_5.ARG_TO_NAN,	false,	null,	null];
	this.setAttribute(arguments);
	
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorAjaxGetExternal.prototype = new hX_5.JSFBehaviorAjax();


// -------------------------------------------------------
// AJAX Behavior - Get
//
// Replace the content of a tag with content retreived via AJAX
// (from the same URL)
//
// Arguments:
//
//	see documentation.
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorAjaxGet = function () {
	this.type = hX_5.BHR_ajaxget;

	// Attributes set by caller
	this.paramNames = [];		// If parameters are to be encoded on URL, the names to give the parameters
	this.paramFields = [];		// If parameters are to be encoded on URL, ids of the fields to encode for each of the parameter names
	this.title = null;			// Title text used while in progress
	this.actions = [];			// If "in progress actions" are provided, the actions to use while in progress
	this.targets= [];			// Dummy vector to go with the above
	this.startFcn=null;			// If provided, BEFORE the request is made the function is run (can stop the request if returns false)
	this.endFcn=null;			// If provided, AFTER the request is succesfully made the function is run (does no run if request failed)
	this.errorFcn=null;			// If provided, if the request fails this is used instead of AFTER 

	// Implementation
	this.href = null;			// The URL of the page to do a get against (NULL for a get)
	this.source=null;
	
	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['progress-actions']	= ['actions',		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['href']		= ['href',			hX_5.ARG_TO_NAN,	false,	null,	null];// BILL
		this.attributeTable ['params']		= ['paramNames',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['param-fields']= ['paramFields',	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['title']		= ['title',			hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['on-start']	= ['startFcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['on-complete']	= ['endFcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['on-error']	= ['errorFcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
	this.setAttribute(arguments);
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorAjaxGet.prototype = new hX_5.JSFBehaviorAjax();

// -------------------------------------------------------
// AJAX Behavior - Submit
//
// Replace the content of a tag with content retreived via AJAX
// (from the same URL).  The enclosing form is submitted as part 
// of the process
//
// Arguments:
//
//	see documentation.
// -------------------------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorAjaxSubmit = function () {
	this.type = hX_5.BHR_ajaxsubmit;

	// Attributes set by caller
	this.title = null;			// Title text used while in progress
	this.actions = [];			// If "in progress actions" are provided, the actions to use while in progress
	this.targets= [];			// Dummy vector to go with the above
	this.startFcn=null;			// If provided, BEFORE the request is made the function is run (can stop the request if returns false)
	this.endFcn=null;			// If provided, AFTER the request is succesfully made the function is run (does no run if request failed)
	this.errorFcn=null;			// If provided, if the request fails this is used instead of AFTER 

	// Implementation
	this.href = null;			// The URL of the page to do a get against (not passed in by user
	this.paramNames = [];		// If parameters are to be encoded on URL, the names to give the parameters
	this.paramFields = [];		// If parameters are to be encoded on URL, ids of the fields to encode for each of the parameter names
	this.source=null;
	
	// Attributes supported on this object
	this.attributeTable = [];
		this.attributeTable ['progress-actions']	= ['actions',		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['href']		= ['href',			hX_5.ARG_TO_NAN,	false,	null,	null];// BILL
		this.attributeTable ['title']		= ['title',			hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['on-start']	= ['startFcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['on-complete']	= ['endFcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['on-error']	= ['errorFcn',		hX_5.ARG_TO_FCNS,	false,	null,	null];
	this.setAttribute(arguments);
	
}
// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorAjaxSubmit.prototype = new hX_5.JSFBehaviorAjax();
