/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2007 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// ----------------------------------
// JSF Sub-button object
// --------------------------
// Usage: 
// 1.  Construct the obhject and store it as a child of a JSF Component (that's based on an input type=text tag.
//	   Constructor emits HTML so it should be in the pageLoad code of the component
// 2.  Component's redraw handler should call the subbuttons.size() call
// 3.  Component's redraw handler should take (one) argument "resizeOnly" which
//	   it passes to the size() call.
// 4.  Component can manually depress/undepress a butotn using the depress() call
// 5.  Component should be prepared to handle standard dispatch callbacks (dispatchComponent)
HxG_5.prototype.JSFSubButton = function (component, btnOrg, btnBorder, btnColor, btn1label, btn2label, blurIsOn, hinky, inputIsButton) {
	if (!component || !component.DOMobj || !component.DOMobj.id || component.DOMobj.id.length <= 0 || !component.DOMobj.tagName || component.DOMobj.tagName != hX_5._INP) return;

	// Caller provided description of the sub-button
	this.c			= component;
	this.btnOrg 	= (btnOrg) 			? btnOrg : 1;				// Count of buttons (1 or 2)
	this.btnBorder 	= (btnBorder!=null) ? btnBorder : -1;			// Border width, if -1 then uses input fields border, else width of border (0 to 3)
	this.btnColor 	= (btnColor)		? btnColor : hX_5.CLR_BTN;	// Button face color.  Defaults to buttonface
	this.btn1label	= (btn1label)		? btn1label: hX_5._E;		// Alt text for button
	this.btn2label  = (this.btnOrg > 1 && btn2label) ? btn2label : hX_5._E;	// Alt text for 2nd button
	this.btngraphic = (hinky) 			? hinky : hX_5.R_IMG_HINKIES;	// Graphic for butotn (defaults to standard hinky triangles)
	this.blurIsOn	= (blurIsOn)		? blurIsOn : false;
	this.inputIsButton = (inputIsButton!=null)? inputIsButton : 0;

	// Commonly used browser-dependent literals
	this.spanD = (hX_5.brw.isIE()) ? hX_5._INB : hX_5._INL;
	// Following isn't accurate -- for no browser running XP does there seem to be a way to compute the system border color
	// This is as close as we can get
	this.borderSystemColor = function (obj) {return ("InactiveCaption"); }
	// Following figures out a border color given oddities in the browsers
	this.borderCalcColor = function (obj) {
		var c = hX_5.g.getP(obj, hX_5._BRCOR);
		return ((c.length > 0 && c.indexOf(hX_5._B) < 0 && c==hX_5.g.getP (obj, hX_5._COR)) ?  hX_5._E : hX_5.g.getP(obj, hX_5._BRRCL));
	}
	
	// State information established when buttons created (can't be changed later)
	this.stStaticPos =				// Tag initially has a "static" position in a table 
	this.stPosition =				// Position used on the outerspan
	this.stResizable = null;		// Tag initially had a resizable width (of this value)
	this.stIsXP = false;			// Tag initially described a "system-styled XP contol"
									// that is, border draws as 1 pixel though it's actually 2 pixels
	// State information recalc'd on each "size" call
	this.stBtnBorderStyle = 		// Border style used to draw the buttons
	this.stBtnBackColor =			// Ditto but background color
	this.stBtnBorderColor = 		// Ditto but border color
	this.stBtnBorderWidth = hX_5._E;// Ditto but border width
	this.stBtnOffset = this.stElemOffset = this.stMgnLeft = this.stMgnTop = 0;

	// Impelementation -- tags that make up the button
	this.dupe = this.btn1 = this.btn2 = this.img1 = this.img2 = this.iSpn = this.oSpn = null;
	this.valid = false;

	// Did we emit into the page yet?
	this.HTMLcopied = this.addedResizeMonitor = false;
	this.addedPropMonitor = null;

	// Following code is used to remember the value of margins/positional information
	// set on the input field.  This information is removed from the input field
	// and copied to the outerspan.  We need to be able to restore it
	this.styleProps = [];
	this.styleValues = [];
	this.calcValues = calcValues;
	this.saveValues = saveValues;
	this.getValue = function (v) { return this.styleValues[v]; }
	function saveValues (obj) {
		var p = hX_5.g.getP(obj, hX_5._POS);
		this.styleProps[hX_5._POS] = (p && p.length > 0 && p != hX_5._AUT) ? p : null;
		p = hX_5.g.getP(obj, hX_5._MGNL);
		this.styleProps[hX_5._MGNL] = (p && p.length > 0 && p != hX_5._AUT) ? p : null;
		p = hX_5.g.getP(obj, hX_5._MGNR);
		this.styleProps[hX_5._MGNR] = (p && p.length > 0 && p != hX_5._AUT) ? p : null;
		p = hX_5.g.getP(obj, hX_5._MGNT);
		this.styleProps[hX_5._MGNT] = (p && p.length > 0 && p != hX_5._AUT) ? p : null;
		p = hX_5.g.getP(obj, hX_5._MGNB);
		this.styleProps[hX_5._MGNB] = (p && p.length > 0 && p != hX_5._AUT) ? p : null;
		p = hX_5.g.getP(obj, hX_5._LFT);
		this.styleProps[hX_5._LFT] = (p && p.length > 0 && p != hX_5._AUT) ? p : null;
		p = hX_5.g.getP(obj, hX_5._RGT);
		this.styleProps[hX_5._RGT] = (p && p.length > 0 && p != hX_5._AUT) ? p : null;
		p = hX_5.g.getP(obj, hX_5._TOP);
		this.styleProps[hX_5._TOP] = (p && p.length > 0 && p != hX_5._AUT) ? p : null;
		p = hX_5.g.getP(obj, hX_5._BOT);
		this.styleProps[hX_5._BOT] = (p && p.length > 0 && p != hX_5._AUT) ? p : null;
	}
	function calcValues (cl) {
		this.styleValues[hX_5._POS] = (this.styleProps[hX_5._POS] != null) ? this.styleProps[hX_5._POS] : hX_5.getClassStyle(cl, hX_5._POS);
		this.styleValues[hX_5._MGNL]= (this.styleProps[hX_5._MGNL] != null) ? this.styleProps[hX_5._MGNL] : hX_5.getClassStyle(cl, hX_5._MGNL);
		this.styleValues[hX_5._MGNR]= (this.styleProps[hX_5._MGNR] != null) ? this.styleProps[hX_5._MGNR] : hX_5.getClassStyle(cl, hX_5._MGNR);
		this.styleValues[hX_5._MGNT]= (this.styleProps[hX_5._MGNT] != null) ? this.styleProps[hX_5._MGNT] : hX_5.getClassStyle(cl, hX_5._MGNT);
		this.styleValues[hX_5._MGNB]= (this.styleProps[hX_5._MGNB] != null) ? this.styleProps[hX_5._MGNB] : hX_5.getClassStyle(cl, hX_5._MGNB);
		this.styleValues[hX_5._LFT] = (this.styleProps[hX_5._LFT] != null) ? this.styleProps[hX_5._LFT] : hX_5.getClassStyle(cl, hX_5._LFT);
		this.styleValues[hX_5._RGT] = (this.styleProps[hX_5._RGT] != null) ? this.styleProps[hX_5._RGT] : hX_5.getClassStyle(cl, hX_5._RGT);
		this.styleValues[hX_5._TOP] = (this.styleProps[hX_5._TOP] != null) ? this.styleProps[hX_5._TOP] : hX_5.getClassStyle(cl, hX_5._TOP);
		this.styleValues[hX_5._BOT] = (this.styleProps[hX_5._BOT] != null) ? this.styleProps[hX_5._BOT] : hX_5.getClassStyle(cl, hX_5._BOT);
	}

	// ---- Start of constructor code ----

	// Add handler for keystrokes on the input field
	var v, attr, tag, c = this.c, parent = c.DOMobj.parentNode;
	hX_5.attachComponentEvent (c.DOMobj, hX_5._onKD, hX_5._lN);
	hX_5.attachComponentEvent (c.DOMobj, hX_5._onF, hX_5._lN);
	if (this.inputIsButton!=0) {
		hX_5.attachComponentEvent (c.DOMobj, hX_5._onB, hX_5._lN);
		hX_5.attachComponentEvent (c.DOMobj, hX_5._onKP, hX_5._lN);
	}

	// First dupe the input tag so we can use the duplicate to track changes in size
	// Currently only used in Mozilla when dealing with percentage sized fields
	if (hX_5.brw.isNavOrMoz()) {
		tag = document.createElement(hX_5._INP);
		tag.setAttribute(hX_5._TYP, hX_5._XT);
		tag.setAttribute(hX_5._ID, c.DOMobj.id+"_HXB_GHOST");
		tag.className = c.DOMobj.className;
		tag.dir = c.DOMobj.dir;
		tag.disabled = c.DOMobj.disabled;
		tag.readOnly = c.DOMobj.readOnly;
		tag.language = c.DOMobj.language;
		tag.tabIndex = -1;
		if (c.DOMobj.style.cssText && c.DOMobj.style.cssText.length > 0) tag.style.cssText = c.DOMobj.style.cssText;
		tag.style.display = hX_5._NO;
		this.dupe = parent.insertBefore(tag, c.DOMobj);
	}

	// Wrap the input tag in a span so that we can position things relative to it
	// We're going to end up with:
	// <outer-span><dupe /><DOMobj /><innerspan><button><button></innerspan><focus-div /></outerspan>
	tag = document.createElement(hX_5._SPN);
	tag.setAttribute (hX_5._ID, c.DOMobj.id + "_outerspan");
	tag.style.position = hX_5._REL;
	tag.style.display = this.spanD;
	this.oSpn = parent.insertBefore(tag, c.DOMobj);
	this.oSpn.appendChild(c.DOMobj);
	// Following the Input tag, add a <span> element to continue the border around the 
	// input field so it surrounds the button(s)			
	tag = document.createElement(hX_5._SPN);
	tag.style.position = hX_5._ABS;
	tag.style.padding = tag.style.margin = hX_5._ZPX;
	tag.style.display = this.spanD;
	tag.style.textIndent = hX_5._ZPX;	// This is inherited into the contents, so turn it off within the span
	this.iSpn = this.oSpn.appendChild(tag);
	// Add a button in the span (button and enclosing span are sized in separate call)
	tag = document.createElement(hX_5._BTN);
	tag.setAttribute(hX_5._ID, c.DOMobj.id+"_1_HXB_HPBTN");
	tag.setAttribute(hX_5._TYP, hX_5._TBTN);
	tag.setAttribute(hX_5._TTL, this.btn1label);
	tag.style.position = hX_5._ABS;
	tag.style.padding = tag.style.margin = hX_5._ZPX;
	tag.style.fontSize = hX_5._1PX;
	tag.onclick = tag.ondblclick = tag.onmousedown = tag.onmouseup = tag.onmouseout = tag.onkeydown = tag.onkeypress = tag.onkeyup = this.dispatchBtnComponent;
	if (this.blurIsOn) tag.onblur = this.dispatchBtnComponent;
	this.btn1 = this.iSpn.appendChild(tag);
	// Add the scalable image to the button content
	tag = document.createElement(hX_5._IMG);
	tag.setAttribute(hX_5._SRC, c.r.getURL(c.p.R_IMG_SPACER));
	tag.setAttribute(hX_5._BRD, hX_5._Z);
	tag.style.fontSize = hX_5._1PX;
	tag.style.verticalAlign = "baseline";
	tag.style.padding = tag.style.margin = hX_5._ZPX;
	tag.setAttribute(hX_5._ALT, this.btn1label);
	this.img1 =this.btn1.appendChild(tag);
	this.valid = (this.oSpn && this.iSpn && this.btn1 && this.img1);
	// If this is a two button control, then add another button and image
	if (this.btnOrg == 2) {
		tag = document.createElement(hX_5._BTN);
		tag.setAttribute(hX_5._ID, c.DOMobj.id+"_2_HXB_HPBTN");
		tag.setAttribute(hX_5._TYP, hX_5._TBTN);
		tag.setAttribute(hX_5._TTL, this.btn2label);
		tag.style.position = hX_5._ABS;
		tag.style.padding = tag.style.margin = hX_5._ZPX;
		tag.style.fontSize = hX_5._1PX;
		tag.onclick = tag.ondblclick = tag.onmousedown = tag.onmouseup = tag.onmouseout = tag.onkeydown = tag.onkeypress = tag.onkeyup = this.dispatchBtnComponent;
		if (this.blurIsOn) tag.onblur = this.dispatchBtnComponent;
		this.btn2 = this.iSpn.appendChild(tag);
		// Add the scalable image to the button content
		tag = document.createElement(hX_5._IMG);
		tag.setAttribute(hX_5._SRC, c.r.getURL(c.p.R_IMG_SPACER));
		tag.setAttribute(hX_5._BRD, hX_5._Z);
		tag.style.fontSize = hX_5._1PX;
		tag.style.padding = tag.style.margin = hX_5._ZPX;
		tag.style.verticalAlign = "baseline";
		tag.setAttribute(hX_5._ALT, this.btn2label);
		this.img2 = this.btn2.appendChild(tag);
		this.valid = this.valid || (this.btn2 && this.img2);
	}
	// Finally, if this is an input field being used as a button, we need to create a div
	// that will cover the input field content area
	this.inpDiv = this.inpSubDiv = null;
	if (this.inputIsButton!=0) {
		tag = document.createElement(hX_5._DIV);
		tag.style.position = hX_5._ABS;
		tag.style.padding = tag.style.margin = hX_5._ZPX;
		tag.style.borderWidth = hX_5._2PX;
		tag.style.borderColor = hX_5.CLR_BCK;
		tag.style.backgroundColor = hX_5.CLR_BCK;
		tag.style.borderStyle = hX_5._SLD;
		tag.style.display = hX_5._E;
		tag.style.fontSize= hX_5._1PX;
		tag.style.overflow=hX_5._HI;
		tag.style.whiteSpace=hX_5._NOW;
		tag.onclick = tag.ondblclick = tag.onmousedown = tag.onmouseup = tag.onmouseout = this.dispatchBtnComponent;
		if (this.blurIsOn) tag.onblur = this.dispatchBtnComponent;
		this.inpDiv = this.oSpn.appendChild(tag);
		tag = document.createElement(hX_5._DIV);
		tag.style.position = hX_5._ABS;
		tag.style.padding = tag.style.margin = hX_5._ZPX;
		tag.style.display = hX_5._E;
		tag.style.fontSize= tag.style.borderWidth = hX_5._1PX;
		tag.style.borderColor = hX_5.CLR_SHDW;
		tag.style.borderStyle = hX_5._SLD;
		tag.style.overflow=hX_5._HI;
		tag.style.whiteSpace=hX_5._NOW;
		tag.onclick = tag.ondblclick = tag.onmousedown = tag.onmouseup = tag.onmouseout = this.dispatchBtnComponent;
		if (this.blurIsOn) tag.onblur = this.dispatchBtnComponent;
		this.inpSubDiv = this.inpDiv.appendChild(tag);
		this.valid = this.valid || (this.inpDiv!=null && this.inpSubDiv!=null);
	}
}

// Public: Destroy large memory objects and all event handlers
HxG_5.prototype.JSFSubButton.prototype.destroy = function () {
	// Events attached to the primary dom object are destroyed by the caller except for onsize
	if (this.btn1)
		this.btn1.onblur = this.btn1.onclick = this.btn1.ondblclick = this.btn1.onmousedown = this.btn1.onmouseup = this.btn1.onmouseout = this.btn1.onkeydown = this.btn1.onkeypress = this.btn1.onkeyup = null;
	if (this.btn2)
		this.btn2.onblur = this.btn2.onclick = this.btn2.ondblclick = this.btn2.onmousedown = this.btn2.onmouseup = this.btn2.onmouseout = this.btn2.onkeydown = this.btn2.onkeypress = this.btn2.onkeyup = null;
	if (this.inpDiv)
		this.inpDiv.onblur = this.inpDiv.onclick = this.inpDiv.ondblclick = this.inpDiv.onmousedown = this.inpDiv.onmouseup = this.inpDiv.onmouseout = null;
	if (this.inpSubDiv)
		this.inpSubDiv.onblur = this.inpSubDiv.onclick = this.inpSubDiv.ondblclick = this.inpSubDiv.onmousedown = this.inpSubDiv.onmouseup = this.inpSubDiv.onmouseout = null;
	if (this.addedResizeMonitor) hX_5.i.removeResizeMonitor(this.oSpn);
	if (this.addedPropMonitor)  hX_5.i.removeMonitor(this.c.DOMobj);
	if (this.styleProps) while (this.styleProps.length > 0) this.styleProps.pop();	
	if (this.styleValues) while (this.styleValues.length > 0) this.styleValues.pop();
	return true;	
}

// Private:	Size/Position the buttons
HxG_5.prototype.JSFSubButton.prototype.size = function (resizeOnly) {

	// Memorize the style values of any DOMobj we're going to over-write
	var c=this.c, prop, elemH, elemW, i;
	if (!this.valid) return false;
	if (!this.HTMLcopied) this.saveValues(this.c.DOMobj);

	// In the rare case where someone has reparented the original DOM object, quick fix
	// up the outerspan which now wraps nothing instead of the DOM object
	if (this.oSpn && this.c.DOMobj.parentNode.id==undefined || this.c.DOMobj.parentNode.id != (this.c.DOMobj.id + "_outerspan")) {
		this.c.DOMobj.parentNode.insertBefore(this.oSpn, this.c.DOMobj);
		var ff = this.oSpn.firstChild;
		if (this.dupe) ff = ff.nextSibling;
		if (ff) this.oSpn.insertBefore(this.c.DOMobj, ff);
	}

	// If this is a call from a resize handler (that is, the object has a resize monitor on it),
	// Handle this directly without mucking with the HTML	
	if (resizeOnly) {
		elemW = this.oSpn.offsetWidth;
		hX_5.g.setP(c.DOMobj, hX_5._WD, ((elemW - this.stElemOffset) > 0) ? ((elemW - this.stElemOffset) + hX_5._PX) : (hX_5._ZPX));
		hX_5.g.setP(this.iSpn, hX_5._LFT, ((elemW - this.stBtnOffset) > 0) ? ((elemW - this.stBtnOffset-1) + hX_5._PX) : (hX_5._ZPX));
		this.sizeColorArea();
		return true;
	}

	// Copy over the relevant positioning information from the input to the enclosing span
	// If we change a value on the <input> tag, we need to track the change so we can know
	// if a user subsequently modifies the value
	var bDisabled 	= c.DOMobj.disabled;
	var bReadOnly 	= c.DOMobj.readOnly;
	var bHideFocus	= (c.DOMobj.hideFocus) ? c.DOMobj.hideFocus : false;
	var bNoSelect	= (c.DOMobj.unselectable) ? c.DOMobj.unselectable : false;
	var bDisplay 	= hX_5.g.getP(c.DOMobj, hX_5._DSP) != hX_5._NO;
	var bVisible 	= (hX_5.brw.isIE())?(hX_5.g.getP(c.DOMobj, hX_5._VIS)):c.DOMobj.style.visibility;
	var taborder 	= (c.DOMobj.tabIndex) ? c.DOMobj.tabIndex : 0;
	var zIndex	 	= hX_5.g.getP(c.DOMobj, hX_5._ZND);
	taborder = (hX_5.brw.isIE()) ? taborder : ((taborder < 0) ? 0 : taborder);
	zIndex = hX_5.isString(zIndex) ? (((zIndex==hX_5._AUT))?0:parseInt(zIndex)) : zIndex;
	elemH = c.DOMobj.offsetHeight;
	elemW = c.DOMobj.offsetWidth;

	// If the element isn't on display we can't work with it (it has no size)
	if (elemH==0 || elemW==0) {
		if (elemW==0 && !this.HTMLcopied) {
			// If the width is %, we could have a zero width
			prop = hX_5.g.getP(c.DOMobj, hX_5._WD);
			bDisplay = (prop.indexOf(hX_5._PCT) >= 0);
		} else bDisplay = false;
	}
	if (!bDisplay) {
		if (elemH!=0 && elemW!=0) this.oSpn.style.display = hX_5._NO;
		this.btn1.style.display = this.iSpn.style.display = hX_5._NO;
		if (this.btn2) this.btn2.style.display = hX_5._NO;
		return true;
	}

	// Copy the visibility info from the input field to the surrounding spans
	// N.B. on copy: Netscape isn't happy copying "bulk" properties (e.g., "margin") so always copy property by property
	hX_5.g.setP(this.iSpn, hX_5._DSP, this.spanD);
	hX_5.g.setP(this.iSpn, hX_5._VIS, bVisible);
	hX_5.g.setP(this.oSpn, hX_5._DSP, this.spanD);
	hX_5.g.setP(this.oSpn, hX_5._VIS, bVisible);
	// IE issue -- make sure the content of the outer span is left aligned.  IE will sometimes copy
	// the alignment of a container into the span (e.g., if the outerspan is in a centered table cell)
	hX_5.g.setP(this.oSpn, hX_5._TXA, hX_5._LFT);
	if (this.oSpn.hideFocus && this.oSpn.hideFocus != bHideFocus) this.oSpn.HideFocus = bHideFocus;
	if (this.oSpn.unselectable && this.oSpn.unselectable != bNoSelect) this.oSpn.unselectable = bNoSelect;
	if (hX_5.brw.isIE())
		if (this.oSpn.disabled != bDisabled) this.oSpn.disabled = bDisabled;

	// Set up the position of the spans
	this.calcValues(c.DOMobj.className);
	prop = this.getValue(hX_5._POS);
	this.stStaticPos = (this.stStaticPos != null) ? this.stStaticPos : ((hX_5.brw.isIE() && (prop=="static")));
	this.stPosition = (prop != hX_5._ABS) ? hX_5._REL : prop;
	hX_5.g.setP(this.oSpn, hX_5._POS, this.stPosition);

	// Copy margins, top, left, right and bottom to the outerspan
	hX_5.g.setP(this.oSpn, hX_5._MGNL,	this.getValue(hX_5._MGNL));
	hX_5.g.setP(this.oSpn, hX_5._MGNR,	this.getValue(hX_5._MGNR));
	hX_5.g.setP(c.DOMobj, hX_5._MGNL,	hX_5._ZPX);
	hX_5.g.setP(c.DOMobj, hX_5._MGNR,	hX_5._ZPX);
	if (hX_5.brw.isIE()) {
		hX_5.g.setP(this.oSpn, hX_5._MGNT,this.getValue(hX_5._MGNT));
		hX_5.g.setP(this.oSpn, hX_5._MGNB,this.getValue(hX_5._MGNB));
		hX_5.g.setP(c.DOMobj, hX_5._MGNT,hX_5._ZPX);
		hX_5.g.setP(c.DOMobj, hX_5._MGNB,hX_5._ZPX);
	}
	if (this.stStaticPos) {
		hX_5.g.setP(this.oSpn, hX_5._LFT, hX_5._E);
		hX_5.g.setP(this.oSpn, hX_5._RGT, hX_5._E);
		hX_5.g.setP(this.oSpn, hX_5._TOP, hX_5._E);
		hX_5.g.setP(this.oSpn, hX_5._BOT, hX_5._E);
	} else {
		hX_5.g.setP(this.oSpn, hX_5._LFT, this.getValue(hX_5._LFT));
		hX_5.g.setP(this.oSpn, hX_5._RGT, this.getValue(hX_5._RGT));
		hX_5.g.setP(this.oSpn, hX_5._TOP, this.getValue(hX_5._TOP));
		hX_5.g.setP(this.oSpn, hX_5._BOT, this.getValue(hX_5._BOT));
	}
	hX_5.g.setP(c.DOMobj, hX_5._POS, hX_5._REL);
	hX_5.g.setP(c.DOMobj, hX_5._LFT, hX_5._E);
	hX_5.g.setP(c.DOMobj, hX_5._RGT, hX_5._E);
	hX_5.g.setP(c.DOMobj, hX_5._TOP, hX_5._E);
	hX_5.g.setP(c.DOMobj, hX_5._BOT, hX_5._E);
	// There's a bug in IE (which shows up in portal all the time) where if the component is inside
	// a div (and apparently only a div or other block-level element) and the div has a margin set
	// on the left, the margin somehow is copied into the child element itself such that 
	// "clearing" the left position (above) doesn't actually align the element correctly.
	// The following seems to "fix" it but it is something of a hack
	if (hX_5.brw.isIE() && c.DOMobj.offsetLeft != 0) hX_5.g.setP(c.DOMobj, hX_5._LFT, hX_5._Y+c.DOMobj.offsetLeft+hX_5._PX);
	// Manage the zIndex
	if (zIndex!=null && zIndex > 0) {
		hX_5.g.setP(this.oSpn, hX_5._ZND, zIndex+hX_5._E);
		hX_5.g.setP(this.iSpn, hX_5._ZND, zIndex+hX_5._E);
	}
	if (this.inpDiv) hX_5.g.setP(this.inpDiv, hX_5._ZND, (zIndex==null || zIndex==0)?hX_5._1:(zIndex+1)+hX_5._E);

	// Copy over any remaining relevant style properties from the input to the span
	hX_5.g.copyP (c.DOMobj, this.iSpn, "direction");

	// Figure out the specifics of the button style and style the enclosing span
	// There are two "styles" of buttons:
	// a) this.btnBorder < 0 = 	Control has a border the whole width of the control and the
	//							buttons are "inside" the border (default emitted stuff)
	// b) this.btnBorder >=0  	Control has a border only around the input field and the
	//							buttons are "outside" the border
	var isNowXP =  hX_5.g.testXPBorder(c.DOMobj, hX_5._INS);
	if (!this.HTMLcopied) this.stIsXP = isNowXP;
	var outBtn = (this.btnBorder >= 0 || this.stIsXP);
	var brdL = hX_5.g.parseBorder(c.DOMobj, hX_5._LFT, 2), brdR = hX_5.g.parseBorder(c.DOMobj, hX_5._RGT, 2);
	var brdT = hX_5.g.parseBorder(c.DOMobj, hX_5._TOP, 2), brdB = hX_5.g.parseBorder(c.DOMobj, hX_5._BOT, 2);
	if (outBtn) {
		hX_5.g.setP(this.iSpn, hX_5._BCOR, hX_5._TRN);
		hX_5.g.setP(this.iSpn, hX_5._BRWID,hX_5._ZPX);
		this.stBtnBorderWidth = 1;
		this.stBtnBorderStyle = hX_5._SLD;
		this.stBtnBorderColor = ((isNowXP)) ? this.borderSystemColor(c.DOMobj) : this.borderCalcColor(c.DOMobj);
	} else {
		hX_5.g.setP(this.iSpn, hX_5._BCOR, this.btnColor);
		hX_5.g.copyP (c.DOMobj, this.iSpn, hX_5._BRLWD, hX_5._BRLSY, hX_5._BRRWD, hX_5._BRRSY, hX_5._BRTWD, hX_5._BRTSY, hX_5._BRBWD, hX_5._BRBSY);
		hX_5.g.setP(this.iSpn, hX_5._BRLWD, hX_5._ZPX);
		this.stBtnBorderWidth = (brdR > 3) ? 3 : brdR;
		this.stBtnBorderColor = this.borderCalcColor(c.DOMobj);
		if (this.stBtnBorderColor == hX_5._E)
			hX_5.g.setP(this.iSpn, hX_5._BRCOR, this.stBtnBorderColor);
		else
			hX_5.g.copyP(c.DOMobj, this.iSpn, hX_5._BRCOR, hX_5._BRRCL, hX_5._BRLCL, hX_5._BRTCL, hX_5._BRBCL);
		this.stBtnBorderStyle = hX_5.g.getP(c.DOMobj, hX_5._BRRSY);
	}
	// If the border is solid, the button has a solid border.  Otherwise, it has a default (outset) border
	this.stBtnBorderStyle = (this.stBtnBorderStyle == hX_5._SLD) ? hX_5._SLD : hX_5._E;
	// If the border is an "innie" button with a solid border, limit it to 1 pixel in width
	var intBtnBorderWidth = (!outBtn && this.stBtnBorderStyle == hX_5._SLD && this.stBtnBorderWidth > 1) ? 1 : this.stBtnBorderWidth;
	// Button background is colored (standard coloring) unless it is brdLess in which case it's colorless
	this.stBtnBackColor = (this.stBtnBorderWidth > 0) ? this.btnColor : hX_5._TRN;
	hX_5.g.setP(this.btn1, hX_5._BCOR, this.stBtnBackColor);
	hX_5.g.setP(this.btn1, hX_5._BRSTY, this.stBtnBorderStyle);
	hX_5.g.setP(this.btn1, hX_5._BRCOR, this.stBtnBorderColor);
	hX_5.g.setP(this.btn1, hX_5._BRWID, intBtnBorderWidth + hX_5._PX);
	hX_5.g.setP(this.btn1, hX_5._DSP, hX_5._INL);
	this.btn1.disabled = bDisabled || bReadOnly;
	this.btn1.hideFocus = (this.stBtnBorderWidth > 0) ? true : bHideFocus;
	this.btn1.tabIndex = (this.inputIsButton!=0)?-1:taborder;
	var interiorLine = 1, bSpecialCase = (this.stBtnBorderStyle == hX_5._SLD);
	// Visually there's a special case when the button border is solid and this is an "innie" button
	// we simplify the coloring
	if (bSpecialCase) {
		interiorLine = this.stBtnBorderWidth;
		if (!outBtn) {
			hX_5.g.setP(this.btn1, hX_5._BRCOR, this.stBtnBackColor);
			hX_5.g.setP(this.iSpn, hX_5._BCOR, this.stBtnBorderColor);
		}
	}
	if (this.btn2) {
		this.btn2.tabIndex = this.btn1.tabIndex;
		this.btn2.disabled = this.btn1.disabled; 
		this.btn2.hideFocus = this.btn1.hideFocus;
		hX_5.g.setP(this.btn2, hX_5._DSP, this.btn1.style.display);
		hX_5.g.setP(this.btn2, hX_5._BCOR, this.btn1.style.backgroundColor);
		hX_5.g.setP(this.btn2, hX_5._BRCOR, this.btn1.style.borderColor);
		hX_5.g.setP(this.btn2, hX_5._BRSTY, this.btn1.style.borderStyle);
		hX_5.g.setP(this.btn2, hX_5._BRWID, this.btn1.style.borderWidth);
	}

	// Size the buttons
	elemH	= c.DOMobj.offsetHeight;
	var halfheight, btnHeight, btnWidth, fullheight=((outBtn) ? elemH : elemH - (brdT + brdB));
	fullheight = (fullheight < 0) ? 0 : fullheight;
	halfheight = Math.floor(fullheight/2);
	halfheight = (interiorLine > 1) ? (halfheight - (interiorLine-1)) : halfheight;
	btnHeight = (this.btn2) ? halfheight : fullheight;
	btnWidth = (this.btn2) ? ((btnHeight*2)-2) : ((btnHeight*2)-2);
	btnWidth = (btnWidth < 10) ? 10 : ((btnWidth > 18) ? 18: btnWidth);
	hX_5.g.setP(this.btn1, hX_5._TOP, hX_5._ZPX);
	hX_5.g.setP(this.btn1, hX_5._LFT, interiorLine + hX_5._PX);
	hX_5.g.setP(this.btn1, hX_5._WD, (btnWidth - (interiorLine-1)) + hX_5._PX);
	hX_5.g.setP(this.btn1, hX_5._HT, btnHeight + hX_5._PX);
	if (this.btn2) {
		hX_5.g.setP(this.btn2, hX_5._TOP, (fullheight - halfheight) + hX_5._PX);
		hX_5.g.setP(this.btn2, hX_5._LFT, this.btn1.style.left);
		hX_5.g.setP(this.btn2, hX_5._WD, this.btn1.style.width);
		hX_5.g.setP(this.btn2, hX_5._HT,this.btn1.style.height);
		if (bSpecialCase && (fullheight != ((halfheight*2)+interiorLine))) {
			i = (interiorLine==1) ? -1 : 1;
			hX_5.g.setP(this.btn2, hX_5._HT, (parseInt(this.btn2.style.height,10) + i) + hX_5._PX);
			hX_5.g.setP(this.btn2, hX_5._TOP,(parseInt(this.btn2.style.top,   10) - i) + hX_5._PX);
		}
	}

	// Scale the hinky image to fit inside the button
	var imgWidth, imgHeight = btnHeight-(intBtnBorderWidth*2);
	imgHeight = (imgHeight > 3)	? imgHeight-2 : 1;
	if (this.btngraphic == hX_5.R_IMG_HINKIES) {
		imgHeight = Math.min(4, imgHeight);
		imgWidth = imgHeight*2;
	} else {
		imgHeight = Math.min(12, imgHeight);
		imgWidth = (imgHeight >= 6) ? imgHeight : imgHeight*2;
	}
	if (imgHeight > 1) {
		hX_5.g.setP(this.img1, hX_5._BIMG, c.r.getURI (this.btngraphic));
		hX_5.g.setP(this.img1, hX_5._BPOS, c.r.wellPosition(this.btngraphic, imgHeight-2, (bDisabled || bReadOnly) ? 1:0, (this.btn2) ? 1 : 0));
	} else
		hX_5.g.setP(this.img1, hX_5._BIMG, c.r.getURI (hX_5.R_IMG_SPACER));
	hX_5.g.setP(this.img1, hX_5._BREP, hX_5._NREP);
	hX_5.g.setP(this.img1, hX_5._HT, imgHeight + hX_5._PX);
	hX_5.g.setP(this.img1, hX_5._WD, imgWidth + hX_5._PX);
	if (hX_5.brw.isNavOrMoz()) {
		i = ((this.btngraphic != hX_5.R_IMG_HINKIES)?(-this.stBtnBorderWidth):(-(this.stBtnBorderWidth-1)));
		if (this.stBtnBorderWidth >= 1) hX_5.g.setP(this.img1, hX_5._MGNL, i+hX_5._PX);
		if (this.stBtnBorderWidth >= 1) hX_5.g.setP(this.img1, hX_5._MGNT, (i-1)+hX_5._PX);
	}
	this.stMgnLeft = parseInt(hX_5.g.getP(this.img1, hX_5._MGNL));
	this.stMgnTop = parseInt(hX_5.g.getP(this.img1, hX_5._MGNT));
	if (this.btn2) {
		hX_5.g.setP(this.img2, hX_5._HT, this.img1.style.height);
		hX_5.g.setP(this.img2, hX_5._WD, this.img1.style.width);
		hX_5.g.setP(this.img2, hX_5._BIMG, this.img1.style.backgroundImage);
		hX_5.g.setP(this.img2, hX_5._BPOS, c.r.wellPosition(this.btngraphic, imgHeight-2, (bDisabled || bReadOnly) ? 1:0, 0));
		hX_5.g.setP(this.img2, hX_5._BREP, this.img1.style.backgroundRepeat);
		if (hX_5.brw.isNavOrMoz()) {
			hX_5.g.setP(this.img2, hX_5._MGNB, this.img1.style.marginBottom);
			hX_5.g.setP(this.img2, hX_5._MGNL, this.img1.style.marginLeft);
			hX_5.g.setP(this.img2, hX_5._PADB, hX_5._1PX);
		}
			
	}
	btnWidth = (hX_5.brw.isW3Cstd()) ? btnWidth : (this.btn1.offsetWidth+(interiorLine-1));
	btnWidth = (btnWidth < 0) ? 10 : btnWidth;

	// Size the input field removing the space the buttons will take up
	// If this is the first time through, we'll look at the DOMobj's width to do this
	// After this first time, we'll use the width of the outerspan as the basis for
	// the calculation.  Note that this means you can only set the width INITIALLY.
	// It can't be modified later
	if (this.HTMLcopied) {
		if (hX_5.brw.isIE() || this.stPosition==hX_5._ABS)
			elemW = this.oSpn.offsetWidth;
		// In Mozilla, if the outerspan isn't in a table or other sized element (or even sometimes if it is) 
		// and it's resizable, NS isn't resizing it so the width is constant
		// We fix this by putting back the "original" input tag and measuring it
		else if (hX_5.brw.isNavOrMoz() && this.dupe) {
			this.oSpn.style.display = hX_5._NO;
			this.dupe.style.display = hX_5._E;
			elemW = this.dupe.offsetWidth;
			this.dupe.style.display = hX_5._NO;
			this.oSpn.style.display = hX_5._E;
		} else
			elemW = this.oSpn.offsetWidth + this.stBtnOffset;
	} else {
		// Netscape BUG: getP (below) doesn't return percentages
		// (it returns resolved widths in fractional pixels), so % only works if it's in an inline style
		prop = (hX_5.brw.isIE()) ? hX_5.g.getP(c.DOMobj, hX_5._WD) : c.DOMobj.style.width;
		this.stResizable = ((prop.indexOf(hX_5._PCT)) < 0) ? null : prop;
		elemW = c.DOMobj.offsetWidth;
	}

	var W3CadjustH = 0, W3CadjustW = 0, elemW2 = elemW, bR, bL;
	if (!hX_5.brw.isW3Cstd())
		W3CadjustW = brdR = ((outBtn) ? 0 : brdR);
	else {
		brdR = (outBtn && !hX_5.brw.isIE()) ? 0 : brdR;
		elemH -= (brdT + brdB);
		W3CadjustW = W3CadjustH = brdR;
		if (hX_5.brw.isIE()) {
			var pL = parseInt(hX_5.g.getP(c.DOMobj, hX_5._PADL), 10), s = hX_5.g.getP(c.DOMobj, hX_5._BRLSY);
			bL = (s==hX_5._NO) ? 2 : brdL;
			s = hX_5.g.getP(c.DOMobj, hX_5._BRRSY);
			bR = (s==hX_5._NO) ? 2 : brdR;
			// IE makes a % width field too "wide" in strict mode (e.g., an input field with a 
			// width of 100% is actually several pixels wider than the page).  It does not do this
			// on a <span> with the same width.  We could correct the span to have the same
			// error. The following appears to maintain this differenece.
			if (this.stResizable && !this.HTMLcopied) elemW2 -= ((this.stResizable && !this.HTMLcopied)?(2*((bR + bL + pL + ((outBtn) ? 1 : ((bSpecialCase)?1:0))))) :((bR + bL + pL + ((outBtn) ? 1 : ((bSpecialCase)?1:0)))));
			W3CadjustW = (outBtn) ? 0 : (bSpecialCase)?interiorLine:2;
		}
	}

	elemW2 = elemW2 - (btnWidth + 1);
	elemW2 = (elemW2 < 0) ? 0 : elemW2;
	// Now set the width of the outer span and the input field
	hX_5.g.setP(this.oSpn, hX_5._WD, (this.stResizable) ? this.stResizable : (elemW + hX_5._PX));
	hX_5.g.setP(c.DOMobj, hX_5._WD, elemW2 + hX_5._PX);
	// Override padding right (which is inside the input field) so there's space
	// before the buttons (cursor needs at least 1 pixel there)
	hX_5.g.setP(c.DOMobj, hX_5._PADR, hX_5._1PX);

	// Position the span -- for IE it's absolutely positioned next to the input field
	// For Netscape, we position it INSIDE the input field in the 'padding'
	hX_5.g.setP(this.iSpn, hX_5._WD, (btnWidth + brdR + 1 - W3CadjustH) + hX_5._PX);
	hX_5.g.setP(this.iSpn, hX_5._HT, elemH + hX_5._PX);
	hX_5.g.setP(this.iSpn, hX_5._LFT, (c.DOMobj.offsetWidth - W3CadjustW) + hX_5._PX);
	this.stBtnOffset = btnWidth + brdR;
	if (hX_5.brw.isNavOrMoz()) {
		hX_5.g.setP(this.iSpn, hX_5._TOP, -(1+brdT) + hX_5._PX);
		hX_5.g.setP(c.DOMobj, hX_5._MGNR, btnWidth + hX_5._PX);
		this.stBtnOffset += (-brdR+1);
		// Netscape has a problem with setting the "top" so force it to be right
		var elemtop = hX_5.g.getAbsPosTop(c.DOMobj), spantop = hX_5.g.getAbsPosTop(this.iSpn);
		if (elemtop != spantop) hX_5.g.setP(this.iSpn, hX_5._TOP, (parseInt(this.iSpn.style.top, 10)) + (elemtop-spantop) + hX_5._PX);
	} else
		hX_5.g.setP(this.iSpn, hX_5._TOP, hX_5._1PX);

	// If this is a button, position the "cover" over the input field
	this.saveBR = (isNowXP)?1:brdR; this.saveBL = (isNowXP)?1:brdL; this.saveBT = (isNowXP)?1:brdT; this.saveBB = (isNowXP)?1:brdB;
	if (this.inpDiv) this.sizeColorArea ();

	// Attach a monitor for the size of the field changing
	if (!this.HTMLcopied) {
		if (this.stResizable) {
			this.stElemOffset = elemW - c.DOMobj.offsetWidth;
			hX_5.i.attachResizeMonitor(this.oSpn, this.dispatchButtonMonitorResize);
			this.HTMLcopied = true;
			this.addedResizeMonitor = true;
			// IE needs a kickstart
			if (hX_5.brw.isIE()) hX_5.i.fireEvent(this.oSpn.id, hX_5._onR);
		}
		// Attach a monitor to watch for changes in the HTML
		this.addedPropMonitor = hX_5.i.attachMonitor(c.DOMobj, this.dispatchMonitorBtn, "className", "class", hX_5._DSB, hX_5._RDO);
	}

	// Mark the HTML as emitted
	this.HTMLcopied = true;
	return true;
}

// Private:	Set the color area up to be correctly sized
HxG_5.prototype.JSFSubButton.prototype.sizeColorArea = function () {
	hX_5.g.setP(this.inpSubDiv, hX_5._PADL, hX_5._2PX);
	var std=(hX_5.brw.isNavOrMoz() || hX_5.brw.isIE6_CSS()), xyOff = ((std)?4:0), iW = this.c.DOMobj.offsetWidth - (this.saveBL+this.saveBR), iH = this.c.DOMobj.offsetHeight - (this.saveBT+this.saveBB);
	hX_5.g.setP(this.inpDiv, hX_5._TOP,(this.c.DOMobj.offsetTop + this.saveBT)+hX_5._PX);
	hX_5.g.setP(this.inpDiv, hX_5._LFT,(this.c.DOMobj.offsetLeft + this.saveBL)+hX_5._PX);
	hX_5.g.setP(this.inpDiv, hX_5._WD, (iW-xyOff)+hX_5._PX);
	hX_5.g.setP(this.inpDiv, hX_5._HT, (iH-xyOff)+hX_5._PX);
	hX_5.g.setP(this.inpSubDiv, hX_5._TOP, hX_5._ZPX);
	hX_5.g.setP(this.inpSubDiv, hX_5._LFT, hX_5._ZPX);
	xyOff = ((std)?4:0);
	hX_5.g.setP(this.inpSubDiv, hX_5._WD, (iW-4-xyOff)+hX_5._PX);
	xyOff = ((std)?2:0);
	hX_5.g.setP(this.inpSubDiv, hX_5._HT, (iH-4-xyOff)+hX_5._PX);
}

// Public:	Style the button area of an input area (if inputIsButton is set)
HxG_5.prototype.JSFSubButton.prototype.styleColorArea = function (src) {
	if (src && this.inpDiv && this.inputIsButton==2) {
		try {
			hX_5.g.copyP(src, this.inpSubDiv, hX_5._COR, "font-family", hX_5._FSZ, "font-style", "font-variant", hX_5._TXA, "text-decoration", "text-transform", "letter-spacing", "direction");
		} catch (e) {
			 // Ignore failure -- it happens for some unknown reason in IE if the input field is hidden
		}
		this.stNormalColor = hX_5.g.getP(src, hX_5._COR);
	}
}

// Public:	Color the button area of an input area (if inputIsButton is set)
HxG_5.prototype.JSFSubButton.prototype.colorArea = function (clr, txt, fFlag) {
	if (this.inpDiv) {
		// If a color is provided, set the background-color
		if (clr!=null) hX_5.g.setP(this.inpSubDiv, hX_5._BCOR, clr);
		// If text is provided, set the text
		if (txt!=null) hX_5.setIHTML(this.inpSubDiv, ((txt.length==0)?"&nbsp;":txt));
		// If there is text in it, make sure it's a color that can be read
		if (this.inputIsButton==2) {
			var c=hX_5.g.getP(this.inpSubDiv, hX_5._BCOR);
			hX_5.g.setP(this.inpSubDiv, hX_5._COR, (hX_5.col.isDark(c)?hX_5.CLR_WHT:this.stNormalColor));
			// And we need to vertically align it -- we do this will an "old" CSS cheat
			if (this.inpSubDiv.offsetHeight!=null && this.inpSubDiv.offsetHeight>0)
				this.inpSubDiv.style.lineHeight=(this.inpSubDiv.offsetHeight-4)+hX_5._PX;
		}
		// If requested, draw/undraw the focus rectangle
		if (fFlag!=null) {
			if (fFlag) {
				hX_5.g.setP(this.inpSubDiv, hX_5._BRCOR, hX_5.CLR_BLK);
				hX_5.g.setP(this.inpSubDiv, hX_5._BRSTY, "dotted");
			} else {
				hX_5.g.setP(this.inpSubDiv, hX_5._BRCOR, hX_5.CLR_SHDW);
				hX_5.g.setP(this.inpSubDiv, hX_5._BRSTY, hX_5._SLD);
			}
		}
	}
}

// Public:	Depress/Undepress one of the buttons
HxG_5.prototype.JSFSubButton.prototype.depress = function (btn, downstate) {
	if (!this.valid) return false;
	var i=this.stBtnBorderStyle, o1=0, o2=0, s=hX_5._E, b=((btn=="btn1")?this.btn1:this.btn2), m=((btn=="btn1")?this.img1:this.img2);
	if (downstate==true) {
		// "depress" a normally bordered button
		if (i==hX_5._INS) s = hX_5._OUT;
		else if (i=="groove")s = "ridge";
		else if (i=="ridge") s = "groove";
		else s = hX_5._INS;		
		o1 = (hX_5.brw.isIE())?(((b.offsetWidth%2)==1)?1:2):(this.stMgnLeft+1);
		o2 = (hX_5.brw.isIE())?(((b.offsetHeight%2)==1)?1:2):(this.stMgnTop+2);
	} else {
		s = (this.stBtnBorderStyle == hX_5._E) ? hX_5._OUT : this.stBtnBorderStyle;
		o1 = this.stMgnLeft;
		o2 = this.stMgnTop;
	}
	hX_5.g.setP(b, hX_5._BRSTY, s);
	hX_5.g.setP(m, hX_5._MGNL, o1+hX_5._PX);
	hX_5.g.setP(m, hX_5._MGNT, o2+hX_5._PX);
	return true;
}

// Private: Dispatch a (DOM) event to a button attached to a component object
HxG_5.prototype.JSFSubButton.prototype.dispatchBtnComponent = function(evt) {
	if ((typeof hX_5)=="undefined") return true;
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt && this) {
		var src = ((this.nodeType == hX_5.ELEMENT_NODE || this.nodeType == hX_5.DOCUMENT_NODE) ? this : this.parentNode);
		if (src) {
			var r, sibling, obj, skip=false, item=hX_5._E;
			if (src.tagName==hX_5._IMG) src = src.parentNode;
			if (src.tagName==hX_5._BTN || src.tagName==hX_5._DIV) {
				// <outer-span><dupe /><DOMobj /><innerspan><button><button></innerspan><focus-div /></outerspan>
				// Events can come in on the "focus div"
				if (src.tagName == hX_5._DIV) {
					item = "btn1";
					if (src.parentNode.tagName==hX_5._DIV) src = src.parentNode;
					if (src.previousSibling && src.previousSibling.tagName==hX_5._SPN) src = (src.previousSibling).previousSibling;
				} else {
					// Events can come in on images etc. inside the button -- ignore them
					sibling = src.previousSibling;
					src = src.parentNode;
					if (src && src.tagName == hX_5._SPN) {
						item = (sibling && sibling.tagName == hX_5._BTN)?"btn2":"btn1";
						src = src.previousSibling;
					}
				}
				if (src && src.id && src.tagName==hX_5._INP) {
					if (evt.type==hX_5._evMCC) skip = true;
					else if (evt.type==hX_5._evMC || evt.type==hX_5._evMD || evt.type==hX_5._evMU) skip = evt.button > 1;
					if (!skip) {
						obj = hX_5.getComponentById(src.id);
						if (obj) {
							r =obj.dispatchComponent (item, evt, evt.type, ((evt.keyCode)?evt.keyCode:null));
							if (r!=null && r==false) hX_5.i.cancelAction(evt, true);
						}
					}
				}
			}
		}
	}
	return r;
}

// Implementation: Handle Dispatch a (DOM) resize event to subbutton
HxG_5.prototype.JSFSubButton.prototype.dispatchButtonMonitorResize = function(evt) {
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt && this) {
		var src = ((this.nodeType == hX_5.ELEMENT_NODE || this.nodeType == hX_5.DOCUMENT_NODE) ? this : this.parentNode);
		if (src) {
			// Netscape gives us the event on the body (only) so just redraw in this case
			if (!hX_5.brw.isIE())
				hX_5.redraw();
			// IE gives us the event on the individual tags
			else if (src.tagName == hX_5._SPN) {
				var obj, inputfld = src.firstChild;
				if (inputfld && inputfld.tagName == hX_5._INP) {
					obj = hX_5.getComponentById(inputfld.id);
					if (obj) {
						obj.redraw(true);
						return false;
					}
				}
			}
		}
	}
	return true;	// let the event continue
}

// Private: Specific sub-handler for property monitor on compound-button JSF components
HxG_5.prototype.JSFSubButton.prototype.dispatchMonitorBtn = function(obj, evt, args) {
	var c = hX_5.getComponentById(obj);
	// For IE we just draw
	if (hX_5.brw.isIE())
		c.redraw();
	// Not sure why, but for some properties (e.g., visibility), Netscape  doesn't see
	// changes (getP doesn't report the actual style) right away, so time the call
	else {
		hX_5.i.deferNSRedraw.push(obj);
		setTimeout(hX_5.i.NSRedraw, 10);						
	}
	return true;	// Let the event continue
}
