/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// ----------------------------------------------
// hxclient_dlg.js
// 
// Base class for JSF Panel Dialog
//
// - PanelDialog component is a container for other
//   widgets, and can be moved around in the browser.
// - It can be modal or modeless.
// - Users can associate Show/Hide dialog actions with
//   some other JavaScript event (e.g button click).
// -----------------------------------------------

// ------------------------------------------------
// Public:  Constructor
HxG_5.prototype.JSFDialog = function () {
	this.type = hX_5.CMP_dialog;

	// General display information set by caller

	// 'align' attribute specifies the horizontal alignment of the dialog with respect to the surrounding context. Possible values: 
	// 		- left: the left edge of the dialog is aligned with the left edge of the browser window 
	//		- right: the right edge of the dialog is aligned with the right edge of the browser window
	//		- center: dialog is horizontally centered relative to the browser window.  
	//		- relative: left edge of the dialog is aligned with the left edge of the object that caused the onShow event to fire OR it is relative to relativeTo if relativeTo is provided.
	//
	// 'valign' attribute specifies the horizontal alignment of the dialog with respect to the surrounding context. Possible values: 
	//		- top: the top edge of the dialog is aligned with the top edge of the browser window
	//		- bottom: the bottom edge of the dialog is aligned with the bottom edge of the browser window
	//		- middle: dialog is vertically centered relative to the browser window.  
	//		- relative: dialog's top edge is aligned with the BOTTOM edge of the whatever object caused the onShow event to fire  OR it is relative to relativeTo if relativeTo is provided.  	
	this.align = this.valign = null;
		
	// 'forId' 		is optional attribute. If specified, then when this tag is "clicked", the dialog is shown. 
	//         		It is also used to position the dialog if 'align' and/or 'valign' attributes set to 'relative'
	//         		and 'relativeTo' attribute is not specified. This is a convenience attribute.
	// 'relativeTo'	is optional attribute. If provided and one of the positioning attributes is set to relative, 
	//        		then positioning is relative to this ID instead of the ID of the tag which triggered the onshow event.
	this.forId = this.relativeTo = null;
    
	// 'initiallyShow' attribute specifieds whether dialog is shown when page is loaded.
	//                 If true, when the page loads, the dialog is shown (and onshow handler is fired).
	this.initiallyShow = false;
	
	// 'initialFocus' attribute specifies id of the field that gets focus when the dialog is shown.  
	//                If omitted, the "first" field is given focus.
	this.initialFocus = null;
	
	// 'movable' attribute indicates weather users can move the dialog within the browser window. 
	//           Default value is true.
	this.movable = true;
		
	// 'enableMoveKey'  attribute specifies whether move keybinding sequence 'Ctrl+Shift+Arrow_Key' performs MOVE action when dialog has focus.
	//                  Default value is true.
	this.enableMoveKey = true;

	// 'noKeybindings'  If true, no keybindings associated with the dialog buttons.
	//                  Default value is false.
	this.noKeybindings = false;
	
	// 'saveState' attribute. If true, the values all input, select and textAreas are remembered when the onShow is fired.
	//             Default value is true.
	this.saveState = true;

	// 'subsidiaryDiv' attribute specifies an ID of an HTML and/or JSF container(s) whose input controls should not be blocked once modal dialog is active. 
	//                 This attribute is ignored for modeless dialogs.
	this.subsidiaryDiv = null;
	
	// 'type' attribute specifies the type of the dialog box. Values are modal and modeless. Default is modal.
	this.dlgType = hX_5.DLG_modal;

	// 'onshow' attribute specifies the name of the function to run whenever the dialog is made visible. 
	//          Signature is onShow(sourceId, event). SourceId is the id of the triggering DOM object.
	// 'onhide' attribute specifies the name of the function to run whenever the dialog is hidden. 
	//          Signature is onHide(sourceId, event). 
	//          SourceId is the id of the triggering DOM object that caused dialog to hide. 
	//          SourceId is "CANCEL" if it was the CANCEL button or "OK" if it was the OK button.  
	//          If it's the CANCEL button and saveState is true, state of the input fields in the dialog is restored to when the dialog was shown (before the handler is called).
	this.onshow = this.onhide = null;
	
	// array of all tabbable DOM objects that are children of this dialog component
	this.allChildren = null;
	
	// 'first' dialog OK and CANCEL buttons as defined by the tab order
	this.firstOk = null, this.firstCancel = null;
		
	// Implementation
	this.titleBarDOMobj = this.closeButtonDOMobj = this.closeButtonImgDOMobj = this.contentDivDOMobj = this.iframeObj = this.saveEvt = null;
	this.explictL = this.explicitR = this.explicitT = this.explicitB = this.hadFocus = this.mouseMoveID = this.mouseUpID = this.mouseCMID = null;
	this.explicitTest = false;
	this.testRect = new hX_5.HxRect();

	// Attributes
	this.attributeTable = [];
		this.attributeTable ['align']			  		= ['align',	    	 	hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['valign']			  		= ['valign',    		hX_5.ARG_TO_NAN,	false,	null,	null];
		this.attributeTable ['for-id']	  		  		= ['forId',	 			hX_5.ARG_TO_NAN,	false,	null,	null];
    	this.attributeTable ['relative-to']		  		= ['relativeTo',		hX_5.ARG_TO_NAN,	false,	null,	null];
    	this.attributeTable ['initially-show']	  		= ['initiallyShow',	 	hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['initial-focus']	  		= ['initialFocus',	 	hX_5.ARG_TO_NAN,	false,	null,	null];
    	this.attributeTable ['movable']	  				= ['movable',	 		hX_5.ARG_IS_TRUE,	false,	null,	null];    	    	
    	this.attributeTable ['enable-movekey']  		= ['enableMoveKey',		hX_5.ARG_IS_TRUE,	false,	null,	null];
    	this.attributeTable ['no-keybindings'] 			= ['noKeybindings',		hX_5.ARG_IS_TRUE,	false,	null,	null];    	
    	this.attributeTable ['save-state']	 	 		= ['saveState',	 		hX_5.ARG_IS_TRUE,	false,	null,	null];
		this.attributeTable ['type']					= ['dlgType',			hX_5.ARG_TO_NAN,	false,	null,	null];       	 	    	
		this.attributeTable ['onshow']					= ['onshow',			hX_5.ARG_TO_FCNS,	false,	null,	null];
		this.attributeTable ['onhide']					= ['onhide',			hX_5.ARG_TO_FCNS,	false,	null,	null];
	this.setAttribute(arguments);	
}

// ==== Constants are defined in hxclient_c.js

// Private: child element in the allChildren array
HxG_5.prototype.JSFDialog.prototype.elem = function(obj) {
	this.obj = obj;
	this.value = null;
}

// Override:  Attach the appropriate events when the page loads
HxG_5.prototype.JSFDialog.prototype.onPageLoad = function (){

	if( this.id && this.DOMobj ) // we have valid enclosing DIV DOMobj
	{
		var i, o, f, l, ln, actions, target;
		this.titleBarDOMobj    = hX_5.getElementById(this.id + hX_5.DLG_TitleBarSuffix);
		this.closeButtonDOMobj = hX_5.getElementById(this.id + hX_5.DLG_CloseButtonSuffix);
		this.closeButtonImgDOMobj = hX_5.getElementById(this.id + hX_5.DLG_CloseButtonImgSuffix);
		this.contentDivDOMobj  = hX_5.getElementById(this.id + hX_5.DLG_ContentAreaSuffix);
		var baseClass = hX_5.getFirstClassName(this.DOMobj);

		if( this.titleBarDOMobj) // chrome has a title bar
		{
			// setup CSS class and cursor style of the title bar DIV element
			this.titleBarDOMobj.className = baseClass + hX_5.CLASS_DLG[0];			
			hX_5.g.setP(this.titleBarDOMobj, hX_5._CRS, hX_5._DFT);

			// if dialog is 'movable', then monitor for 'mousedown' events on the title bar DIV element.
			// 		<DIV id="dialog1_TitleBar" class="panelDialog_TitleBar">
			// This event will be dispatched in the dispatchDialog(...) handler.
			if( this.movable ) hX_5.attachComponentEvent(this.id, hX_5._onMD);
		} // end if(chrome has a title bar)


		// setup style 'class' attribute for the content <DIV>
		this.contentDivDOMobj.className = baseClass + hX_5.CLASS_DLG[3];

		// We need to generate an underlying <iframe> element for IE to cover <SELECT> controls
		actions = "action:" + hX_5.ACT_MOVE;
		targets = "target:" + this.id;
		if( hX_5.brw.isIE() ){
			this.iframeObj = hX_5.ifr.create(document, this.id + hX_5.DLG_IFrameSuffix, false, false);
			this.DOMobj.parentNode.insertBefore(this.iframeObj, this.DOMobj);
			actions = actions + ";" + hX_5.ACT_MOVE;			
			targets = targets + ";" + this.iframeObj.id;
		} // end if(create <iframe> for IE)

		// add SHOW+STOP actions for the 'onclick' event for the 'forId' object where actions' target is JSFDialog component
		if( this.forId )
			hX_5.addBehavior(this.forId, hX_5._onMC, new hX_5.JSFBehaviorGeneric("action:"+hX_5.ACT_SHOW+";"+hX_5.ACT_STOP, "target:"+this.id));

		// add JSFBehaviorKeybind for 'Ctrl+Shift+___Arrow' keys to perform MOVE action on this JSFDialog component (and underlying iFrame)
		if( this.enableMoveKey && this.movable ){ 
			hX_5.addBehavior(this.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:37", "modifier:Ctrl+Shift", actions, targets));
			hX_5.addBehavior(this.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:38", "modifier:Ctrl+Shift", actions, targets));
			hX_5.addBehavior(this.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:39", "modifier:Ctrl+Shift", actions, targets));
			hX_5.addBehavior(this.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:40", "modifier:Ctrl+Shift", actions, targets));
		} // end if(add JSFBehaviorKeybind for 'Ctrl+Shift+___Arrow' keys)

		// If state saving is on, or we need to figure out dialog button keybindings, 
		// then find all child controls who's state should be saved, and Ok, Cancel buttons.
		if (this.saveState || !this.noKeybindings) {
			this.allChildren = [];
			if (hX_5.tabNav == null) hX_5.tabNav = new hX_5.tabNavigator();
			f = hX_5.tabNav.getFirst(this.DOMobj, true);
			l = hX_5.tabNav.getLast(this.DOMobj, true);
			ln = hX_5.tabNav.getTabItems().length;
			i = 0;
			while( f!=null && i<ln ){
				if( !this.noKeybindings ){
					if( this.firstOk==null && f.type!=undefined && (f.type==hX_5._SBM || f.type==hX_5._lIMG) ) this.firstOk = f;
					else if( this.firstCancel==null && f.type!=undefined && (f.type==hX_5._RES) ) this.firstCancel = f;
				} // end if(set dialog button keybindings)
				if( this.saveState ) {
					if (hX_5.isValuable(f)) this.allChildren.push(new this.elem(f));
				} // end if(saveState is true)
				if( f==l ) break;
				f = hX_5.tabNav.getFocus(f, +1, null, this.DOMobj, true);
				i++;
			} // end while()
		} // end if( this.saveState || !this.noKeybindings )

		// Monitor for 'mouseclick' events on the closeButton object in the title bar.
        //	<a href="#" id="dialog1_cb"  class="panelDialog_TitleBar-CloseButton">
        //    	<img src="ibmjsfres/panelDialog_CloseButton.gif"></img>
        //  </a>		
		if( this.closeButtonDOMobj ){ // CLOSE button behavior is similar to that of ESC key
			// if there is a CANCEL button, then CLOSE button performs CLICK on the CANCEL button
			// if there is no CANCEL button, then CLOSE button performs DLG_CANCEL+STOP actions	on the dialog
			if( this.firstCancel!=null ){
				actions = "action:" + hX_5.ACT_CLICK;
				targets = "target:" + this.firstCancel.id;			
			} // end if()
			else {
				actions = "action:"+hX_5.ACT_HIDE+";"+hX_5.ACT_STOP;
				targets = "target:" + this.id;
			} // end else()
			
			hX_5.addBehavior(this.closeButtonDOMobj.id, hX_5._onMC, new hX_5.JSFBehaviorGeneric(actions, targets));
			hX_5.addBehavior(this.closeButtonDOMobj.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:13", actions, targets));
			hX_5.addBehavior(this.closeButtonDOMobj.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:32", actions, targets));
			this.closeButtonDOMobj.className = baseClass + hX_5.CLASS_DLG[1]; // setup styleClass for the close button

			// If the 'src' attribute of the <img> is not set, means 'closeButtonImage' wasn't set
			// on the <hx:panelDialog> component.			
			// In such case, we emit 1x1.gif as 'src' attribute of the <img> and set corresponding
			// style class so that 'background-image' property for the 1x1.gif takes effect.
	        // 		<img src=".ibmjsfres/1x1.gif" class="panelDialog_TitleBar-CloseButton-img"></img>
			if((this.closeButtonImgDOMobj.src==undefined || this.closeButtonImgDOMobj.src==null || this.closeButtonImgDOMobj.src==hX_5._E )){
				this.closeButtonImgDOMobj.setAttribute(hX_5._SRC, this.r.getURL(hX_5.R_IMG_SPACER));
				this.closeButtonImgDOMobj.className = baseClass + hX_5.CLASS_DLG[2];								
			} // end if(empty 'src' attribute)
		} // end if(this.closeButtonDOMobj)

		// Add behaviors to set up NEWLINE and ESC handling on fields in the dialog if button keybinding is ON
		if( !this.noKeybindings ){
			// if there is an OK button, then NEWLINE is mapped to a CLICK on the OK button
			// if there is no OK button, but there is a CANCEL button, then NEWLINE is mapped to a CLICK on the CANCEL button			
			// if there are no OK, or CANCEL buttons, but there is a CLOSE button, then NEWLINE is mapped to DLG_CANCEL+STOP actions
			// if there are no OK, CANCEL, or CLOSE buttons, NEWLINE does nothing		
			if(this.firstOk!=null) // there is OK button
				hX_5.addBehavior(this.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:13", "action:"+hX_5.ACT_CLICK, "target:"+this.firstOk.id));
			else if(this.firstCancel!=null) // there is no OK button, but there is a CANCEL button
				hX_5.addBehavior(this.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:13", "action:"+hX_5.ACT_CLICK, "target:"+this.firstCancel.id));
			else if(this.closeButtonDOMobj) // there are no OK or CANCLE buttons, but there is a CLOSE button
				hX_5.addBehavior(this.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:13", "action:"+hX_5.ACT_DLG_CANCEL+";"+hX_5.ACT_STOP, "target:"+this.id));
								
			// if there is a CANCEL button, then ESC key is mapped to a CLICK on the CANCEL button
			// if there is no CANCEL button, but there is a CLOSE button, then ESC key is mapped to DLG_CANCEL+STOP actions
			// if there are no CANCEL or CLOSE buttons, ESC does nothing
			if(this.firstCancel!=null) // there is a CANCEL button
				hX_5.addBehavior(this.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:27", "action:"+hX_5.ACT_CLICK, "target:"+this.firstCancel.id));
			else if(this.closeButtonDOMobj) // there is no CANCLE button, but there is a CLOSE button
				hX_5.addBehavior(this.id, hX_5._onKD, new hX_5.JSFBehaviorKeybind("keycode:27", "action:"+hX_5.ACT_DLG_CANCEL+";"+hX_5.ACT_STOP, "target:"+this.id));						
		} // end if(set up NEWLINE and ESC handling)
		
		this.visible = false; // by default mark dialog as not visible
		
		// Now run through any load / unload processing needed
		if( this.initiallyShow ){ // dialog should be 'visible' to begin with
			// Ensure onLoad() method is called, but do it off the timer to ensure
			// all dialog components have enough time to initialize before it is invoked.
			if (document.createEventObject!=undefined) {
				this.saveEvt = document.createEventObject();
				this.saveEvt.type = "onload";
			} else {
				this.saveEvt = document.createEvent("HTMLEvents");
				this.saveEvt.initEvent("onload", true, true);
			}
			hX_5.i.startObjTimerTimed(this, null, 1);
		} // end if(dialog should be 'visible' to begin with)
		
	} // end if(we have valid enclosing DIV DOMobj)
	return true;
} // end onPageLoad()

// Public:  Set an "attribute" on the dialog.  Attributes are passed as strings of the form "name:value"
HxG_5.prototype.JSFDialog.prototype.setAttribute = function(a) {
	hX_5.parseArg (this.attributeTable, this, a);

	if (this.dlgType) this.dlgType = this.dlgType.toLowerCase();
	if (this.align) this.align = this.align.toLowerCase();
	if (this.valign) this.valign = this.valign.toLowerCase();

	// Re-emit the HTML for the images (but not the dispatch)
	if (this.HTMLrendered) this.redraw();
	return true;
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.JSFDialog.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public: this methods can be called directly on JSFDialog component to 'show' this dialog
HxG_5.prototype.JSFDialog.prototype.show = function (evt) {
	return (hX_5.action.processAction (this.DOMobj, (evt)?evt:null, null, ["SHOW","STOP"], true, [this.DOMobj.id], null, false, false, false));
}

// Public: this methods can be called directly on JSFDialog component to 'hide' this dialog
HxG_5.prototype.JSFDialog.prototype.hide = function (evt) {
	return (hX_5.action.processAction (this.DOMobj, (evt)?evt:null, null, ["HIDE","STOP"], true, [this.DOMobj.id], null, false, false, false));
}

// Private: Handle a timer event 
HxG_5.prototype.JSFDialog.prototype.dispatchTimer = function (param) {
	hX_5.action.processAction (this.DOMobj, this.saveEvt, null, ["SHOW"], false, [this.DOMobj.id], null, null, null, null);
	return true;
}

// Private: this methods performs all required steps before dialog is 'shown'
//          - invoke user-defined onshow handler (if any)
//          - update DialogStack
//          - for modal dialogs, 'disable' all controls outside of this dialog
//          - deal with save/restore dialog state
//          - deal with 'showing' of dialog's HTML child tags (e.g. IFrame)
HxG_5.prototype.JSFDialog.prototype.onLoad = function (thisDOMobj, evt) {
	var o, j;
	if( this.visible ) { // if dialog is already visible, just set focus to the dialog
		this.setDialogFocus();
		return true; // NOOP: dialog is already visible
	} // end if()

	// If an onshow handler is provided, call it
	if (this.onshow) {	// onshow handler is supplied, call it
		if (!hX_5.action.processFunction(this.DOMobj, evt, this.onshow)) {
			// If the user supplied function returns false, "cancel" showing the dialog
			hX_5.g.setP(this.DOMobj, hX_5._DSP, hX_5._NO);
			return false;
		}
	}

    // if dialog width is not fixed already, then for 'dragging' purposes we need to fix
    // this.DOMobj.style.width property to the 'rendered' width of the dialog: this.DOMobj.offsetWidth
	if( !this.DOMobj.style.width)
		this.DOMobj.style.width = hX_5.g.getRendWidth(this.DOMobj)+hX_5._PX;

	// compute and/or just set dialog's location within browser window
	if( !this.positionDialog(evt) ){
		hX_5.g.setP(this.DOMobj, hX_5._DSP, hX_5._NO);
		return false;		// and return from the handler with an error indicator
	} // end if()

	// dialog state is saved AFTER user-defined onshow() handler completes
	if( this.saveState && this.allChildren){ // save values of all dialog's components
		for (j=0; j<this.allChildren.length; j++)
			this.allChildren[j].value = hX_5.getValuableValue(this.allChildren[j].obj);
	} // end if()

	// Push this JSFDialog onto the DialogStack and set DOMobj's z-index accordingly
	// This will "disable" the underlying window
	var dlg_z_index = hX_5.i.pushOnDialogStack(this, (this.dlgType == hX_5.DLG_modal));

	// Ensure we set focus to the dialog appropriately -- this must be done AFTER
	// pushing the dialog onto the stack so we don't try to push focus into a field that isn't visible
	o = this.setDialogFocus();

	// Make sure that size of underlying iFrame object is equal to that of the dialog.
	// Also, ensure that underlying iFrame object is visible.
	if( this.iframeObj ) {
		var offset=hX_5.g.getRelPos(this.DOMobj), l=hX_5.g.getAbsPosLeft(this.DOMobj), t=hX_5.g.getAbsPosTop(this.DOMobj);
		if (offset) {
			l = l-hX_5.g.getAbsPosLeft(offset); t = t-hX_5.g.getAbsPosTop(offset);
		}
		hX_5.g.setP(this.iframeObj, hX_5._WD,  hX_5.g.getRendWidth(this.DOMobj) + hX_5._PX);
		hX_5.g.setP(this.iframeObj, hX_5._HT,  hX_5.g.getRendHeight(this.DOMobj) + hX_5._PX);
		hX_5.g.setP(this.iframeObj, hX_5._LFT, l+ hX_5._PX);
		hX_5.g.setP(this.iframeObj, hX_5._TOP, t + hX_5._PX);
		hX_5.g.setP(this.iframeObj, hX_5._ZND, (dlg_z_index-1)+hX_5._E); // set z-index of the underlying iframe object
		hX_5.g.setP(this.iframeObj, hX_5._DSP, hX_5._E);
	} // end if()

	this.visible = true; // remember that dialog is now 'visible'
	return true;
}


// Private: this methods performs all required steps before dialog is 'hidden'
//          - invoke user-defined onhide handler (if any)
//          - update DialogStack
//          - for modal dialogs, 're-enable' all controls outside of this dialog (if needed)
//          - deal with save/restore dialog state
//          - deal with 'hiding' of dialog's HTML child tags (e.g. IFrame)
HxG_5.prototype.JSFDialog.prototype.unLoad = function (thisDOMobj, evt) {
	if (evt!=null ){ // very limited amount of processing to be done for initial page load
		if( !this.visible ) return true; // NOOP: dialog is already hidden
		// we need to figure out what caused the unLoad event (which button)
		var j, sourceId=null, srcObj = hX_5.evtGetTarget(evt), moreModal=null;
		if(this.saveState || this.onhide ) { // if either attribute is true, need to figure out what caused the unLoad event
			// When designers define 'dialog buttons' the rules are:
			// - "submit" and "image" type buttons are treated as dialog "Ok" button.
			//   we pass string "Ok" to the to the onhide() handler.
			// - "reset" type buttons are treated as "Cancel".
			//   we pass string "Cancel" to the to the onhide() handler.
			// - for "button" type buttons we pass 'id' of the button to the onhide() handler
			// - for "undefined" type buttons we pass "undefined" to the onhide() handler
			if( this.closeButtonDOMobj && ((srcObj.id == this.closeButtonDOMobj.id) || ((srcObj.parentNode) && (srcObj.parentNode.id == this.closeButtonDOMobj.id))) ) { // title bar close button click
				sourceId = hX_5.DLG_CANCEL; // 'close' title bar close button is euivalent to Cancel button
			} else if( srcObj.type == undefined ) { // some odd event called unLoad() handler
				sourceId = hX_5._UND;
			} else if( srcObj.type==hX_5._SBM || srcObj.type==hX_5._lIMG ) {
				sourceId = hX_5.DLG_OK; // "submit" or "image" buttons are "Ok" dialog buttons
			} else if( srcObj.type==hX_5._RES ) {
				sourceId = hX_5.DLG_CANCEL; // "reset" buttons are "Cancel" dialog buttons
			} else if (evt.submitFlag!=undefined) {
				sourceId = (evt.submitFlag==hX_5.ACT_DLG_OK)?hX_5.DLG_OK:hX_5.DLG_CANCEL;
			} else { // some other 'button' type
				sourceId = (srcObj.id != undefined) ? srcObj : hX_5._UND;
			}
		} // end if(this.saveState || this.onhide)

		// If an onhide handler is provided, call it
		if (this.onhide) {
			evt.dialogAction = sourceId;
			if (!hX_5.action.processFunction(this.DOMobj, evt, this.onhide)) {
				// If the user supplied function returns false, "cancel" the cancel
				hX_5.g.setP(this.DOMobj, hX_5._DSP, hX_5._E);
				this.setDialogFocus(); // and make sure dialog gets focus back				
				return false;
			} // end if()
		} // end if(user supplied onhide handler)

		// dialog state is restored AFTER user-defined onhide as user can "cancel" the cancel in their function
		if(this.saveState && (hX_5.DLG_CANCEL==sourceId) && this.allChildren){
			for (j=0; j<this.allChildren.length; j++)
				hX_5.setValuableValue(this.allChildren[j].obj, this.allChildren[j].value);
		} // end if()

		// Remove this JSFDialog off the DialogStack and restore the UI
		moreModal = hX_5.i.removeFromDialogStack(this);

	} // end if(!onPageLoad)

	// Take care of hiding underlying iFrame object
	if( this.iframeObj ) hX_5.g.setP(this.iframeObj, hX_5._DSP, hX_5._NO);
	this.visible = false; // remember that dialog is now 'hidden'

	// Restore focus if we can
// FIXME: Really if this is a modeless dialog, we should set focus back to the field that had focus BEFORE
// something in the modeless dialog got focus but this would putting an event handler (beforeactivate for IE, focus for Mozilla)
// on the body and monitoring whenever a field in a modeless dialog is about to get focus and see if focus currently is 
// "below" the dialog and if so, where it was (and recording it in hadFocus)
	if (this.hadFocus!=null && hX_5.i.isFocusable(this.hadFocus))
		hX_5.i.setFocus(this.hadFocus, false, true);
	else if (moreModal!=null)
		moreModal.setDialogFocus();

	return false;         // do not let event handler to continue (this will stop form submit)
}

// Private: this method sets focus to the dialog whenever a SHOW action occurs
//          - whenever dialog is being made visible
//          - whenver dialog is already visible
HxG_5.prototype.JSFDialog.prototype.setDialogFocus = function () {
	var focusObj = null;
	this.hadFocus = hX_5.i.guessFocus();
	if (this.hadFocus==null && this.forId!=null) this.hadFocus = hX_5.getElementById(this.forId);
	if( this.initialFocus ) // designer specified field that gets focus when dialog is shown
		focusObj = hX_5.getElementById(this.initialFocus);
	if (focusObj == null && this.initialFocus!="none") {
		if (hX_5.tabNav == null) hX_5.tabNav = new hX_5.tabNavigator();
		focusObj = hX_5.tabNav.getFirst(this.DOMobj);
		// Skip over close button
		if (this.closeButtonDOMobj) focusObj = hX_5.tabNav.getFocus(focusObj, +1);
	}
	// if actually found an object to set focus on, the do it!
	if( focusObj != null )
		hX_5.i.setFocus(focusObj, false, true); // set focus to this object

	return focusObj;
}


// Private: this method positions dialog object based on one ore more of the following attributes:
//          - CSS properties: left, right, top, bottom
//          - align/valign
//          - pos-id
//
//          Computations of the position should occur only once - the first time around.
//          Afterwards, computed values are remembered and re-used so that the dialog is
//          re-displayed at the same location every time it is made visible.
HxG_5.prototype.JSFDialog.prototype.positionDialog = function (evt) {
	// First of all make sure that dilog has non-zero size (width and height)
	// and the dialog itself is no larger then the browser window in either dimention.
	// Should either one of the conditions be true, dialog will not be shown because
	// this would trigger security violation in IE 6 on XP SP2.
	var winW = hX_5.g.getWindowClientWidth(),  winH = hX_5.g.getWindowClientHeight(),
	    dlgW = hX_5.g.getRendWidth(this.DOMobj), dlgH = hX_5.g.getRendHeight(this.DOMobj),
	    pageW = hX_5.g.getPageWidth(), pageH = hX_5.g.getPageHeight();

	if( (dlgW > Math.max(winW,pageW)) || (dlgH > Math.max(winH,pageH)) || (0 == dlgW) || (0 == dlgH) ) return false; // cannot display dialog

	// See if style was (initially) explicitly set to a left/right/top/bottom
	var offset=hX_5.g.getRelPos(this.DOMobj), l,r,t,b;
	if (!this.explicitTest) {
		l=this.DOMobj.style.left; r=this.DOMobj.style.right; t=this.DOMobj.style.top; b=this.DOMobj.style.bottom;
		if ((l!=null && l!=hX_5._E && l!=hX_5._AUT) || (r!=null && r!=hX_5._E && r!=hX_5._AUT)) {
			this.explicitL = l; this.explicitR = r;
		}
		if ((t!=null && t!=hX_5._E && t!=hX_5._AUT) || (b!=null && b!=hX_5._E && b!=hX_5._AUT)) {
			this.explicitT = t; this.explicitB = b;
		}
		this.explicitTest = true;
			
	}
	
	// 'align' attribute specifies the horizontal alignment of the dialog with respect to the surrounding context. Possible values:
	// 		- left: the left edge of the dialog is aligned with the left edge of the browser window
	//		- right: the right edge of the dialog is aligned with the right edge of the browser window
	//		- center: dialog is horizontally centered relative to the browser window.
	//		- relative: left edge of the dialog is aligned with the left edge of the object that caused the onShow event to fire OR it is relative to relativeTo if relativeTo is provided.
	if (this.explicitL==null  && this.explicitR==null) { 
		l = 0;
		if( this.align == hX_5._LFT ){ // left edge of the dialog is aligned with the left edge of the browser window
			l = hX_5.g.getWinScrollOffsetX();
		} // end if()
		else if( this.align == hX_5._RGT ){ // right edge of the dialog is aligned with the right edge of the browser window
			l = hX_5.g.getWinScrollOffsetX()+(winW-dlgW);
		} // end else if()
		// dialog should be horizontally centered relative to the browser window.		
		else if( (this.align == hX_5._CTR) || ((this.align == null) && (this.relativeTo == null)) ){ 
			l = hX_5.g.getWinScrollOffsetX()+((winW-dlgW)/2);
		} // end else if()
		// left edge of the dialog should be aligned with the left edge of the object that caused the onShow event to fire OR it is relative to relativeTo if relativeTo is provided.
		else if( (this.align == hX_5._REL) ||((this.align == null) && (this.relativeTo != null)) ){
			var anchorObj = null; // look for positional anchor for the dialog: relativeTo, or object that caused onLoad event to fire
			if( this.relativeTo ) anchorObj = hX_5.getElementById(this.relativeTo);			
			if((null==anchorObj)&&(null != evt)&&(evt.type!="onload")) anchorObj = hX_5.evtGetTarget(evt);
			if( null != anchorObj ){ // found a positional anchor for the dialog
				l = hX_5.g.getAbsPosLeft(anchorObj);
			} // end if()
		} // end else if()
		// Bounds check:
		l = ((l+dlgW)>(hX_5.g.getWinScrollOffsetX()+winW)) ? (hX_5.g.getWinScrollOffsetX()+winW-dlgW):l;
		l = (l<0)?0:l;
		// Rebase the position to be relative to the container
		if (offset)	l = l-hX_5.g.getAbsPosLeft(offset);
		hX_5.g.setP(this.DOMobj, hX_5._LFT, l+hX_5._PX);
	} // end if(horizontal position is not specified via CSSP)
	else {
		hX_5.g.setP(this.DOMobj, hX_5._LFT, this.explicitL);
		hX_5.g.setP(this.DOMobj, hX_5._RGT, this.explicitR);
	}

	// 'valign' attribute specifies the horizontal alignment of the dialog with respect to the surrounding context. Possible values:
	//		- top: the top edge of the dialog is aligned with the top edge of the browser window
	//		- bottom: the bottom edge of the dialog is aligned with the bottom edge of the browser window
	//		- middle: dialog is vertically centered relative to the browser window.
	//		- relative: dialog's top edge is aligned with the BOTTOM edge of the whatever object caused the onShow event to fire  OR it is relative to relativeTo if relativeTo is provided.
	if (this.explicitT==null  && this.explicitB==null) {
		t = 0;
		if( this.valign == hX_5._TOP ){ // top edge of the dialog is aligned with the left edge of the browser window
			t = hX_5.g.getWinScrollOffsetY();
		} // end if()
		else if( this.valign == hX_5._BOT ){ // bottom edge of the dialog is aligned with the bottom edge of the browser window
			t = hX_5.g.getWinScrollOffsetY()+(winH-dlgH);
		} // end else if()
		// dialog should be vertically centered relative to the browser window.		
		else if( (this.valign == hX_5._MID) || ((this.valign == null) && (this.relativeTo == null)) ){ 
			t = hX_5.g.getWinScrollOffsetY()+((winH-dlgH)/2);
		} // end else if()
		// top edge of the dialog should be aligned with the bottom edge of the object that caused the onShow event to fire OR it is relative to relativeTo if relativeTo is provided.		
		else if( (this.valign == hX_5._REL) ||((this.valign == null) && (this.relativeTo != null)) ){ 
			var anchorObj = null; // look for positional anchor for the dialog: relativeTo, or object that caused onLoad event to fire
			if( this.relativeTo ) anchorObj = hX_5.getElementById(this.relativeTo);
			if((null==anchorObj)&&(null != evt)&&(evt.type!="onload")) anchorObj = hX_5.evtGetTarget(evt);
			if( null != anchorObj ){ // found a positional anchor for the dialog
				t = hX_5.g.getAbsPosTop(anchorObj) + hX_5.g.getRendHeight(anchorObj);
			} // end if()
		} // end else if()
		// Bounds check:
		t = ((t+dlgH)>(hX_5.g.getWinScrollOffsetY()+winH)) ? (hX_5.g.getWinScrollOffsetY()+winH-dlgH):t;
		t = (t<0)?0:t;
		// Rebase the position to be relative to the container
		if (offset) t = t-hX_5.g.getAbsPosTop(offset);
		hX_5.g.setP(this.DOMobj, hX_5._TOP, t+hX_5._PX);
	} // end if(vertical position is not specified via CSSP)
	else {
		hX_5.g.setP(this.DOMobj, hX_5._TOP, this.explicitT);
		hX_5.g.setP(this.DOMobj, hX_5._BOT, this.explicitB);
	}

    return true; // dialog can be displayed
}

// Private:  Handle a DOM event dispatched by the JSFDialog event handler
//           srcDOMobj - the DOM object of the dialog component that recieved an event
HxG_5.prototype.JSFDialog.prototype.dispatchComponent = function (item, evt, action, keycode) {
	if (evt) {
		var src = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : (hX_5.brw.isIE() ? document.body : null));
		if (src) {
			src = (src.nodeType == hX_5.ELEMENT_NODE || src.nodeType == hX_5.DOCUMENT_NODE) ? src : src.parentNode;
			// if this is a 'mousedown' event on the dialog's title bar, user is trying to move dialog
			if( (src.tagName == hX_5._DIV) && (src.id == (this.id + hX_5.DLG_TitleBarSuffix)) && (evt.type == hX_5._evMD) ) {
				// Make sure the user didn't click on the "close box"
				this.testRect.setRect(this.closeButtonDOMobj);
				var pt = hX_5.g.toPoint(evt);
				if (!this.testRect.hitTest(pt.X, pt.Y)) return this.handleMouseDrag(this.DOMobj, evt, this.id);
			} // end if(initiate moving of the dialog)
		} // end if(src)
	} // end if(evt)


	return true;	// let event continue
} // end dispatchComponent()
	

// Private:  Handle dragging of the JSFDialog using mouse
HxG_5.prototype.JSFDialog.prototype.handleMouseDrag = function (mainDivDOMobj, evt, dialogId) {
	var basePt = hX_5.g.toPoint(evt);
	if( (mainDivDOMobj != null) && (evt.type == hX_5._evMD) )
	{
		basePt.X = basePt.X - hX_5.g.getAbsPosLeft(mainDivDOMobj);
		basePt.Y = basePt.Y - hX_5.g.getAbsPosTop(mainDivDOMobj);

		// Force all other components on the page to yield the mouse
		hX_5.uirelease();

		// Register the event handlers that will respond to the mousemove events
		// and the mouseup event that follow this mousedown event.
		if (hX_5.brw.isIE()) {
			this.mouseMoveID = hX_5.attachEvent(document.body, hX_5._onMM, hX_5._lF, moveHandler, null, null, false);
			this.mouseUpID = hX_5.attachEvent(document.body, hX_5._onMU, hX_5._lF, upHandler, null, null, false);
			this.mouseCMID = hX_5.attachEvent(document.body, hX_5._onCM, hX_5._lF, upHandler, null, null, false);
			document.body.setCapture();
		} else {
			// These events are being attached to DOCUMENT (instead of document.body)
			// We need to make a direct call on the document to do this
			// The events will be removed as soon as the drag stops, so event cleanup managed
			// by the hx event manager isn't needed
			document.addEventListener(hX_5._evMU, upHandler, true);
			document.addEventListener(hX_5._evMM, moveHandler, true);
		} // end else()

		// Cancel the event
		hX_5.i.cancelAction(evt, true);
		return false;
	} // end if((mainDivDOMobj != null) && (evt.type == "mousedown"))

	/**
	 * This is the handler that captures mousemove events when an element
	 * is being dragged.  It is responsible for moving the element.
	 **/
	function moveHandler(evt)
	{
		evt = (evt) ? evt : ((event) ? event: null);
 		var b=(evt.type==hX_5._evCM?2:((evt.button<=1)?0:2)), c=hX_5.g.getRelPos(mainDivDOMobj);
 		if (b==0) {
 			var pt = hX_5.g.toPoint(evt);
	 		var self = hX_5.getComponentById(mainDivDOMobj.id);
			var winW = hX_5.g.getWindowClientWidth(),      winH = hX_5.g.getWindowClientHeight(),
				dlgW = hX_5.g.getRendWidth(mainDivDOMobj), dlgH = hX_5.g.getRendHeight(mainDivDOMobj),
				sTop = Math.max(document.body.scrollTop,document.body.parentNode.scrollTop),
				sLft = Math.max(document.body.scrollLeft,document.body.parentNode.scrollLeft);
			var X = pt.X - basePt.X, Y = pt.Y - basePt.Y;
			if (X < sLft) X = sLft;							if (Y < sTop) Y = sTop;
			if (X > sLft+winW-dlgW) X = sLft+winW-dlgW;	if (Y > sTop+winH-dlgH) Y = sTop+winH-dlgH;
			if (c) {
				X -= hX_5.g.getAbsPosLeft(c);
				Y -= hX_5.g.getAbsPosTop(c);
			}
			mainDivDOMobj.style.left = X + hX_5._PX;
			mainDivDOMobj.style.top =  Y + hX_5._PX;
			if (self.iframeObj) {
				self.iframeObj.style.left = hX_5.g.getP(mainDivDOMobj, hX_5._LFT);
				self.iframeObj.style.top = hX_5.g.getP(mainDivDOMobj, hX_5._TOP);
			}

			// And don't let anyone else see this event.
			hX_5.i.cancelAction(evt, true);
			return false;
		}
	} // end moveHandler()

	/**
	 * This is the handler that captures the final mouseup event that
	 * occurs at the end of a drag.
	 **/
	function upHandler(evt) {
		evt = (evt) ? evt : ((event) ? event: null);
		var self = hX_5.getComponentById(mainDivDOMobj.id);
 		var b=(evt.type==hX_5._evCM?0:((evt.button<=1)?0:2));
 		if (b==0) {
			if (hX_5.brw.isIE()) {				
				document.body.releaseCapture();
				hX_5.removeEvent(document.body, hX_5._onMM, self.mouseMoveID);
				hX_5.removeEvent(document.body, hX_5._onMU, self.mouseUpID);
				hX_5.removeEvent(document.body, hX_5._onCM, self.mouseCMID);
			} else {
				document.removeEventListener(hX_5._evMU, upHandler, true);
				document.removeEventListener(hX_5._evMM, moveHandler, true);
			} // end else()
			hX_5.i.cancelAction(evt, true);
			return false;
		}
	} // end upHandler()

	return true;
} // end handleMouseDrag()
	
// Public:  Redraw the slider incorporating any changes to the HTML of the input or slider fields
HxG_5.prototype.JSFDialog.prototype.redraw = function (resizeOnly) {
	return true;
}

// Public:  Release the ui state of the slider
HxG_5.prototype.JSFDialog.prototype.uirelease = function (free) {
	return true;
}

// Public: Destroy the dialog
HxG_5.prototype.JSFDialog.prototype.destroy = function () {
	var o;
	if (this.iframeObj) {
		this.iframeObj.parentNode.removeChild(this.iframeObj);
		delete this.iframeObj;
	}
	if (this.allChildren!=null) {
		while (this.allChildren.length > 0) {
			o = this.allChildren.pop();
			delete o;
		}
		delete this.allChildren;
		
	}
	if (this.testRect) delete this.testRect;
	delete this.attributeTable;
	return true;
}