/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// --------------------------
// JSF Popup
// --------------------------

// Public:  Constructor
// Arguments:	parentid	(id or object) The DOM object which visullay parents this popup
//							that is, the popup is visually associated with this tag
//				ownerobj	The object which physically parents the popup.  If null, popup is parented to the document (body)
//							else it's parented as the first child of this object
//				style		(int, values defined below). 
//							If 0, "standard popup window", events are automatically set on the div which contains the popup and are dispatched into the component
//							represented by the parentObj.  Events are for "tracking" the mouse (as mouse moves, events are relayed for "dragging")
//							If 1,"standard popup window", events are automatically set on the div which contains the popup and are dispatched into the component
//							represented by the parentObj.  Events are for "clicking" the mouse within the popup.  Popup can be contain things like scrollbars, input,...
//							If 2, "div mode", mouse is managed by other code (e.g., popup mouse manager)
//							currently only needed by menus
//				altComp		if provided (not null), the object to dispatch component events to (only needed if parentID does not refer
//							to a DOMobj that has a component attached to it).

// Styles of popups
HxG_5.prototype.POP_STYLE_DRAG = 0;
HxG_5.prototype.POP_STYLE_CLICK= 1;
HxG_5.prototype.POP_STYLE_MANAGED= 2;
HxG_5.prototype.POP_STYLE_CLICK_SCROLLABLE= 3;
// Pop directions
HxG_5.prototype.POP_DIR_RIGHT = 1;
HxG_5.prototype.POP_DIR_CENTER = 2;

HxG_5.prototype.JSFPopup = function (parentid, ownerobj, style, altComp) {
	this.type = "popup";
	this.parentId = parentid;
	this.DOMobj = this.parentDOMobj = null;
	this.parentCOMPobj = (altComp!=null && altComp.type!=undefined)?altComp:null;
	this.ownerDoc = document;
	// Where to parent the tags
	this.ownerObj = (ownerobj != null) ? ownerobj : document.body;
	// How clicking works
	this.clickStyle = (style != null) ? style : hX_5.POP_STYLE_CLICK;
	this.fullStyle = (this.clickStyle==hX_5.POP_STYLE_MANAGED);
	this.sticky = false;
	// Array of items "under" the popup (for handling IE bug)
	this.hideArray = this.hideArrayV = null;
	// Drop shadow management
	this.dropShadowWidth = this.dropShadowColor = null;
	this.dropDivs = [];
	// Transparency management
	this.transparency = this.transDiv = null;
	// To access the contents of the popup use the following two fcns
	this.getElem = function getElem() { return this.DOMobj; }	// The "div" which is the popup
	this.getDoc  = function getDoc()  { return this.ownerDoc; } // The document that contains the div (may differ from the parent of the popup)
}

// Public:  construct the HTML for the popup when the page loads
HxG_5.prototype.JSFPopup.prototype.onPageLoad = function() {
	this.parentDOMobj = hX_5.getElementById(this.parentId);
	this.parentCOMPobj = (this.parentCOMPobj!=null)?this.parentCOMPobj:hX_5.getComponentById(this.parentId);

	if (this.parentDOMobj) {
		// Only do this if we haven't emitted the HTML yet
		if (!this.HTMLrendered) {
			var elem = document.createElement(hX_5._DIV);
			elem.setAttribute(hX_5._ID, this.id);
			elem.setAttribute(hX_5._NAM, this.id + "_HXPOPUP");
			elem.style.position = hX_5._ABS;
			elem.style.left = elem.style.top = hX_5._ZPX;
			elem.style.height = elem.style.width = "10" + hX_5._PX;
			elem.style.zIndex = "1000000";
			if (!this.fullStyle) elem.style.margin = elem.style.padding = hX_5._ZPX;
			// N.B. leave the popup's coloring "unstyled" so it can be manipulated via a style class
			elem.style.verticalAlign = hX_5._TOP;
			elem.style.overflow = hX_5._HI;
			elem.style.display = hX_5._NO;
			// Insert this div at the top of the page
			this.ownerObj.insertBefore (elem, (this.ownerObj.firstChild));				
			this.DOMobj = elem;
			this.HTMLrendered = true;
			return true;
		}
	}
	return false;
}

// Public:  Destroy a popup
HxG_5.prototype.JSFPopup.prototype.destroy = function() {
	if (this.DOMobj) {
		this.DOMobj.parentNode.removeChild(this.DOMobj);
		delete this.DOMobj;
	}
	if (this.transDiv) {
		this.transDiv.parentNode.removeChild(this.transDiv);
		delete this.transDiv;
	}
	if (this.dropDivs) {
		var e;
		while (this.dropDivs.length > 0) {
			e = this.dropDivs.pop();
			e.parentNode.removeChild(e);
			delete e;
		}
	}
	if (this.hideArray) {
		while (this.hideArray.length>0) this.hideArray.pop();
		delete this.hideArray;
	}
	if (this.hideArrayV) {
		while (this.hideArrayV.length>0) this.hideArrayV.pop();
		delete this.hideArrayV;
	}
	return true;
}

// Public:  Put the popup on display
// N.B. If Dir is null, popup is "left aligned under the parent"
HxG_5.prototype.JSFPopup.prototype.uivisible = function (dir, location, inline, absPt, overrideESC) {
	var pop  = this.DOMobj;
	if (this.isPopped()) this.uirelease();
	if (inline) {
		if (pop) {
			this.parentDOMobj = this.parentCOMobj = inline;
			this.parentDOMobj.insertBefore(pop, this.parentDOMobj.firstChild);
			pop.style.position = "static";
			pop.style.left = pop.style.top = hX_5._ZPX;		
			pop.style.visibility = hX_5._VS;
			pop.style.display = hX_5._E;
		}
	} else {	
		if (pop && this.parentDOMobj) {
			// Make sure we're still at the beginning of the document
			this.ownerObj.insertBefore (pop, this.ownerObj.firstChild);
			// Compute out the height
			var tagtop, tagleft, i, ds, off, poptop, popleft;
			var base = (location != null) ? location : this.parentDOMobj;
			var tagheight = hX_5.g.getRendHeight(base), tagwidth = hX_5.g.getRendWidth(base);
			if (absPt) {
				tagtop = poptop = absPt.Y; popleft = tagLeft = absPt.X;
			} else {
				tagtop = hX_5.g.getAbsPosTop(base); tagleft = hX_5.g.getAbsPosLeft(base);
				if (dir != null && dir == hX_5.POP_DIR_RIGHT) {
					poptop  = tagtop - 1;
					popleft = tagleft + tagwidth - 1;
				} else {
					poptop  = tagtop + tagheight - 1;
					popleft = tagleft;
				}
			}
			// Position the popup and display it
			pop.style.top = poptop + hX_5._PX;
			pop.style.left= popleft + hX_5._PX;
			pop.style.display = hX_5._BLK;
			pop.style.visibility = hX_5._HI;
			// We can't "bounds" check the position until the element is on display
			// as IE doesn't give an element a size if it isn't on display
			// N.B. if the right edge of some popups are "near" the right edge of the display
			// Mozilla will sometimes report back the wrong width for the popup
			var x, y, h=hX_5.g.getRendHeight(pop), w=hX_5.g.getRendWidth(pop);
			var winheight = hX_5.g.getWindowClientHeight(), winwidth = hX_5.g.getWindowClientWidth();
			var wintop = Math.max(document.body.scrollTop,document.body.parentNode.scrollTop), 
			    winleft = Math.max(document.body.scrollLeft,document.body.parentNode.scrollLeft);
			// Stay inside the window
			if (((poptop-wintop) + h > winheight) && (winheight > h))
				pop.style.top = (tagtop - h + 1) + hX_5._PX;
			if (dir != null && dir==hX_5.POP_DIR_CENTER) {
				popleft += ((tagwidth - w)/2);
				pop.style.left= popleft + hX_5._PX;
			}
			if (popleft < winleft) pop.style.left= winleft + hX_5._PX;
			if (popleft+w >= (winleft+winwidth-1)) pop.style.left= (tagleft + tagwidth - w) + hX_5._PX;
			y = hX_5.g.getAbsPosTop(pop); x = hX_5.g.getAbsPosLeft(pop);

			// Handle special effects.
			// 
			// 1.  We check each time the popup is opened as the CSS that controls the effects
			//	   can change between uses and the CSS is not in force yet when the popup
			//	   is created.
			//
			// 2.  Transparency is controlled by setting "-moz-opacity" (Mozilla) and 
			//     "opacity" (IE).  Both properties should be set.  If you set transparency
			//	   on the popup, we make only the "background" transparent (excluding borders).
			//	   If you want a different transparency effect (e.g., everything in the popup
			//	   to be transparent), you should build by inserting a div into the popup
			//	   and having the div use CSS transparency directly.  "Our" transparency
			//	   is a convenience so that users don't have to jump through CSS hoops
			//	   to make the most common transparency effect.
			//
			// 3.  Drop-shadows are controlled by setting the "moz-outline-width" (Mozilla)
			//	   and "outlineWidth" (IE) properties.  This sets the width of the drop shadow.
			//	   The color can optionally be specified with the "moz-outline-color" (Mozilla)
			//	   and "outlineColor" (IE) properties.  Note that "outline-style" is not used
			//     (the drop-shadow is always solid)

			// While this is usually pretty crappy, it's not unusual at least in IE
			// to apply a "transition" to the display of the popup.  If one is set
			// in the CSS for the div, then "play it"
			if (hX_5.brw.isIE() && pop.filters && pop.filters.length > 0) {
				for (i = 0; i < pop.filters.length; i++)
					if (hX_5.isDef(typeof pop.filters[i].Apply)) pop.filters[i].Apply();
			}
			pop.style.visibility = hX_5._VS;
			if (hX_5.brw.isIE() && pop.filters && pop.filters.length > 0) {
				for (i = 0; i < pop.filters.length; i++)
					if (hX_5.isDef(typeof pop.filters[i].Play)) pop.filters[i].Play();
			}

			// Transparency is a little tricky.  If we set it on the popup div, then
			// the contents are all transparent too which isn't terribly nice
			// So we'll create a second div "inside" the popup and make it transparent.
			// We'll give that div the "background" information of the popup and remove
			// the background information from the popup.  That is, the "transparency" popup
			// will have the opacity set as well as background color/image
			var t = hX_5.g.getP(pop, "opacity");
			this.transparency = (t < 1) ? t : null;
			if (this.transparency && this.transDiv == null) {
				ds = this.transDiv = document.createElement(hX_5._DIV);
				ds.style.position = hX_5._ABS;
				ds.style.display = hX_5._NO;
				ds.style.padding = this.transDiv.style.margin = this.transDiv.style.left = this.transDiv.style.top = hX_5._ZPX;
				ds.style.backgroundColor = hX_5.g.getP(pop, hX_5._BCOR);
				ds.style.backgroundImage = hX_5.g.getP(pop, hX_5._BIMG);
				ds.style.backgroundRepeat = hX_5.g.getP(pop, hX_5._BREP);
				ds.style.backgroundAttachment = hX_5.g.getP(pop, "background-attachment");
				if (hX_5.brw.isIE()) {
					ds.style.backgroundPositionX = hX_5.g.getP(pop, "background-position-x");
					ds.style.backgroundPositionY = hX_5.g.getP(pop, "background-position-y");
				} else {
					// FIXME:  This isn't working for Mozilla
					ds.style.backgroundPosition = hX_5.g.getP(pop, hX_5._BPOS);
				}			
				pop.insertBefore (ds, pop.firstChild);
			} 
			// If transparency is on, position the transparent div and style it
			if (this.transparency && this.transDiv ) {
				ds = this.transDiv;
				var lB = hX_5.g.parseBorder(pop, hX_5._LFT,0), rB = hX_5.g.parseBorder(pop, hX_5._RGT,0);
				var tB = hX_5.g.parseBorder(pop, hX_5._TOP,0),  bB = hX_5.g.parseBorder(pop, hX_5._BOT,0);
				ds.style.left = ds.style.top  = hX_5._ZPX;
				ds.style.width  = (w-(lB+rB)) + hX_5._PX;
				ds.style.height = (h-(tB+bB)) + hX_5._PX;
				ds.style.display = hX_5._BLK;
				ds.style.zIndex = hX_5.__1;
				// Clear the background image/color from the popup
				pop.style.backgroundColor = hX_5._TRN;
				pop.style.backgroundImage = hX_5._NO;
				if (hX_5.brw.isNavOrMoz()) pop.style.opacity = hX_5._1;
				this.addTransparency(ds, this.transparency);
			}

			// If we have a drop-shadow, we need to make it by positioning divs
			// behind (and slightly offset from) the popup
			if (hX_5.brw.isNavOrMoz()) {
				pop.style.MozOutlineStyle=hX_5._SLD;
				this.dropShadowWidth = parseInt(hX_5.g.getP(pop, "-moz-outline-width"),10);
				pop.style.MozOutlineStyle=hX_5._E;
				this.dropShadowColor = hX_5.g.getP(pop, "-moz-outline-color");
				off = -1;
			} else {
				this.dropShadowWidth = parseInt(hX_5.g.getP(pop, null, "outline-width"),10);
				this.dropShadowColor = hX_5.g.getP(pop, null, "outline-color");
				off = 0;
			}
			if (this.dropShadowWidth > 0) {
				// Compute the "gradient" scale for the drop shadow
				this.dropShadowWidth = (this.dropShadowWidth > 10) ? 10 : this.dropShadowWidth;
				var scale = (this.transparency) ? this.transparency : .8;
				var scaleStep = (scale-0.10)/this.dropShadowWidth;
				scaleStep = (scaleStep < 0) ? 0 : scaleStep;
				// Make the divs
				for (i = 1; i <= this.dropShadowWidth; i++) {
					if (i > this.dropDivs.length) {
						ds = document.createElement(hX_5._DIV);
						ds.style.position = hX_5._ABS;
						ds.style.display = hX_5._NO;
						ds.style.padding = ds.style.margin = ds.style.left = ds.style.top = hX_5._ZPX;
						ds.style.backgroundColor = hX_5._TRN;
						ds.style.borderStyle = hX_5._SLD;
						ds.style.borderWidth = hX_5._ZPX;
						ds.style.borderBottomWidth = ds.style.borderRightWidth = hX_5._1PX;
						this.dropDivs.push(ds);
						this.ownerObj.insertBefore (ds, (this.ownerObj.firstChild));				
					}
					ds = this.dropDivs[i-1];
					ds.style.borderColor = (this.dropShadowColor) ? this.dropShadowColor : hX_5.CLR_SHDW;
					this.addTransparency(ds, (scale-((i-1)*scaleStep)));
				}
			}
			// Show the drop shadow if it's there
			for (i = 0; i < this.dropShadowWidth; i++) {
				ds = this.dropDivs[i];
				if (ds) {
					ds.style.left = (x+i+2+off) + hX_5._PX;
					ds.style.top  = (y+i+2+off) + hX_5._PX;
					ds.style.width  = (w-1) + hX_5._PX;
					ds.style.height = (h-1) + hX_5._PX;
					ds.style.zIndex = hX_5._E+(1000000-11+i);
					ds.style.display = hX_5._BLK;
				}
			}

			// in IE, we need to hide any combo-boxes under the popup (can't do it until
			// after the popup is on display so we can see what it obscures)
			if (hX_5.brw.isIE() && document.elementFromPoint) {
				var x1, y1, test, sBase = (hX_5.brw.isIE6_CSS()) ? document.body.parentNode : document.body;
				x = x - sBase.scrollLeft;
				y = y - sBase.scrollTop;
				pop.style.display = hX_5._NO;
				for (x1 = x; x1 < x+w; x1 += 12) {
					for (y1 = y; y1 < y+h; y1 += 12) {
						test = document.elementFromPoint(x1, y1);
						if (test && test.tagName == hX_5._SEL && test.style.visibility != hX_5._HI && test.style.display != hX_5._NO) {
							if (!this.hideArray) {
								this.hideArray = [];
								this.hideArrayV = [];
							}
							this.hideArray.push(test);
							this.hideArrayV.push(test.style.visibility);
							test.style.visibility = hX_5._HI;
						}
					}
					// Recheck the "bottom" of the popup
					test = document.elementFromPoint(x1, y+h-1);
					if (test && test.tagName == hX_5._SEL && test.style.visibility != hX_5._HI && test.style.display != hX_5._NO) {
						if (!this.hideArray) {
							this.hideArray = [];
							this.hideArrayV = [];
						}
						this.hideArray.push(test);
						this.hideArrayV.push(test.style.visibility);
						test.style.visibility = hX_5._HI;
					}
				}
				pop.style.display = hX_5._BLK;
			}
		}
	}

	// Start managing events
	if (pop && this.parentCOMPobj && this.clickStyle != hX_5.POP_STYLE_MANAGED) {
		hX_5.popMgr.start (pop, this.parentCOMPobj, (this.clickStyle==hX_5.POP_STYLE_CLICK_SCROLLABLE));
		if (overrideESC && hX_5.brw.isNavOrMoz()) hX_5.popMgr.overrideESC = true;
	}

	return true;
}

// Public:  Take the popup off display
HxG_5.prototype.JSFPopup.prototype.uirelease = function (force) {
	if (this.DOMobj && (force || !this.sticky)) {
		var i;
		// Hide the drop-shadow
		for (i = 0; i < this.dropShadowWidth; i++)
			if (this.dropDivs[i]) this.dropDivs[i].style.display = hX_5._NO;
		// Hide the transparency and restore any values we overrode on the popup
		if (this.transparency && this.transDiv ) {
			this.transDiv.style.display = hX_5._NO;
			this.DOMobj.style.backgroundColor = this.DOMobj.style.backgroundImage = hX_5._E;
			if (hX_5.brw.isNavOrMoz()) this.DOMobj.style.opacity = hX_5._E;
		}
		// Hide the popup
		this.DOMobj.style.display = hX_5._NO;
		this.DOMobj.style.visibility = hX_5._HI;
		// Show any fields we hid
		if (this.hideArray) {
			for (i = 0; i < this.hideArray.length; i++) {
				if (this.hideArray[i] && this.hideArray[i].style)
					this.hideArray[i].style.visibility = this.hideArrayV[i];
			}
			this.hideArray = null;
		}
		return true;
	}
	return false;
}

// Public: Check if the popup is "popped" open
HxG_5.prototype.JSFPopup.prototype.isPopped = function() {
	return (this.DOMobj && this.DOMobj.style.display != hX_5._NO) ? true : false;
}

// Helper:  Add transparency to a div
HxG_5.prototype.JSFPopup.prototype.addTransparency = function(o,t) {
	if (o) {
		if (o.filters) {
			o.style.filter = "progid:DXImageTransform.Microsoft.Alpha(enabled=false)";
			o.filters.item("DXImageTransform.Microsoft.Alpha").opacity = t*100;
			o.filters.item("DXImageTransform.Microsoft.Alpha").enabled = true;
		} else if (o.style)
			o.style.MozOpacity = t;
	}
}

// Helper:  Remove transparency from a tree of elements
HxG_5.prototype.JSFPopup.prototype.removeTransparency = function(o) {
	if (o) {
		if (o.filters) {
			o.style.filter = "progid:DXImageTransform.Microsoft.Alpha(enabled=false)";
			o.filters.item("DXImageTransform.Microsoft.Alpha").opacity = 100;
			o.filters.item("DXImageTransform.Microsoft.Alpha").enabled = true;
		} else if (o.style)
			o.style.MozOpacity = 1;
	}
}
