/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// ----------
// JSF Slider
// ----------

// Public:  Constructor
HxG_5.prototype.JSFSlider = function () {
	this.type = hX_5.CMP_slider;

	// Attributes set by caller
	this.buttonBorder = -1;
	this.buttonColor = hX_5.CLR_BTN;
	this.interval = 1;
	this.scale_width = 300;
	this.show_labels = true;
	this.sliderClass = hX_5.CLASS_DEF + hX_5.CLASS_SL;
	this.sliderLinesClass = this.sliderClass + hX_5.CLASS_SL_LINES;
	this.labelInterval = this.CSSPrefix = null;

	// Converter and validator
	this.NPid = this.NVid = hX_5._E;
	this.NP = this.NV = null;

	// Remember the dom objs of our tags
	this.objPopup = this.objStylus = this.objScale = this.objTicks = this.objLabels = null;

	// Implementation
    this.scale_height = 2;			// All these values are recalced whenever
    this.stylus_width = 10;			// a redraw is needed
    this.stylus_height = 20;
    this.tick_height = 5;
    this.tick_width = 2;
    this.label_size = 10;
    this.ticks = this.start_tick = this.tick_tabs = 0;
	this.JSFBTN = null;
	this.dragging = false;

    // Attributes supported on this object
    this.attributeTable = [];
	 	this.attributeTable ['validator']			= ['NVid',			hX_5.ARG_TO_NAN,	false,	null,	null];
	 	this.attributeTable ['converter']			= ['NPid',			hX_5.ARG_TO_NAN,	false,	null,	null];
    	this.attributeTable ['scale-width'] 		= ['scale_width', 	hX_5.ARG_TO_NUM, 	false,	0,		9999];
	 	this.attributeTable ['show-labels'] 		= ['show_labels', 	hX_5.ARG_IS_TRUE,	false,	null,	null];
	 	this.attributeTable ['increment']			= ['interval',		hX_5.ARG_TO_NUM,	false,	0.01,	9999999];
	 	this.attributeTable ['label-increment']		= ['labelInterval',	hX_5.ARG_TO_NUM,	false,	0.01,	9999999];
		this.attributeTable	['css-prefix']			= ['CSSPrefix',		hX_5.ARG_TO_NAN,	false,	null,	null];
	 	this.attributeTable	['button-color']		= ['buttonColor',	hX_5.ARG_TO_NAN,	false,	null,	null];
	 	this.attributeTable	['button-border']		= ['buttonBorder',	hX_5.ARG_TO_NUM,	false,	-1,		3];
		// N.B.  The attributes for the NumberValidator and NumberConverter are allowed as well
	this.setAttribute(arguments);	
}

// Public:  Page level onLoad handler
HxG_5.prototype.JSFSlider.prototype.onPageLoad = function () {

	// Try refinding the converter if it didn't exist when we first parsed arguments
	// Create a default one if none is found
	if (this.NPid && this.NP == null) this.NP = hX_5.getConverterById(this.NPid);
	if (this.NP == null) this.NP = new hX_5.NumberConverter();
	// Try refinding the validator if it didn't exist when we first parsed arguments
	if (this.NVid && this.NV == null) this.NV = hX_5.getValidatorById(this.NVid);
	if (this.NV == null) this.NV = new hX_5.NumberValidator();
	if (this.NV != null) this.verifyValidator();

	// Add the slider button to the HTML on the page
	this.JSFBTN = new hX_5.JSFSubButton(this, 1, this.buttonBorder, this.buttonColor, hX_5.L[this.lc].STR_SLIDER_LABEL, null, true);
	// Add the popup to the HTML on the page
	this.objPopup = hX_5.addPopup (this.id + "_POPUP", this.JSFBTN.oSpn.id, null, hX_5.POP_STYLE_DRAG, this);

	// Redraw to make it all happen 
	this.redraw();
	return true;
}

// Public:  Redraw the slider incorporating any changes to the HTML of the input or slider fields
HxG_5.prototype.JSFSlider.prototype.redraw = function (resizeOnly) {
	// Rebuild the button
	if (this.JSFBTN) this.JSFBTN.size(resizeOnly);
	return true;
}

// Public:  Release the ui state of the slider
HxG_5.prototype.JSFSlider.prototype.uirelease = function (free) {
	this.dragging = false;
	// N.B. popup has its own uirelease code
	if (this.JSFBTN) this.JSFBTN.depress("btn1", false);
	// If we're active, then on release set the focus
	if (hX_5.uistate.getActiveComponent() == this && this.JSFBTN && this.JSFBTN.btn1)  hX_5.i.setFocus(this.JSFBTN.btn1);
	if (free!=null) hX_5.uirelease(true);
	return true;
}

// Public:  Destroy -- remove event handlers and large memory objects
HxG_5.prototype.JSFSlider.prototype.destroy = function () {
	if (this.objPopup) hX_5.removePopup(this.objPopup.id);
	if (this.JSFBTN) {
		if (this.JSFBTN.destroy) this.JSFBTN.destroy();
		delete this.JSFBTN;
	}
	delete this.attributeTable;
	return true;
}

// Public:  Set a value into the slider
HxG_5.prototype.JSFSlider.prototype.setValue = function (value, notify, returns) {
	var r = false;
	if (this.DOMobj) {
		// If a string is passed in, convert it to a number
		var was=this.getValue(), ivalue = (hX_5.isString(value))?this.NP.stringToValue(value):value;
		if (ivalue != null) {
			// N.B. we do not force the value to the interval, but we do force it to bounds
			ivalue =  this.NV.coerce (ivalue, this.NP);
			if (returns) {
				r = (ivalue != null) ? this.NP.valueToString(ivalue) : hX_5._E;
			} else {
				this.DOMobj.value = (ivalue != null) ? this.NP.valueToString(ivalue) : hX_5._E;
				// Fire an onBlur event to cause behavior checking
				if (notify && was!=this.DOMobj.value) {
					hX_5.i.markEvent (this.DOMobj, hX_5._onF);
					hX_5.i.fireEvent (this.DOMobj, hX_5._onC);
					hX_5.i.fireEvent (this.DOMobj, hX_5._onB);
				}
				r = true;
			}
		}	
	}
	return r;
}
	
// Public:  Get a value from a slider
HxG_5.prototype.JSFSlider.prototype.getValue = function(cast) {
	// If cast, then convert the value as a date
	return ((this.DOMobj==null)?null:((cast)?(this.NV.coerce(this.DOMobj.value, this.NP)):(this.DOMobj.value)));
}	

// Private: Increment or decrement the current value (within the bound, using the current increment
HxG_5.prototype.JSFSlider.prototype.reValue = function (b, notify, inValue) {

	if (this.DOMobj) {
		var absdelta, delta=0, value = ((inValue!=null)?this.NV.coerce(inValue, this.NP):this.getValue(true));
		if (value == null || isNaN(value)) value = new Number(0);
		if (b=="btn3")
			value= this.NV.minimum;
		else if (b=="btn4")
			value= this.NV.maximum;
		else {
			delta = (b == "btn1") ? this.interval : -(this.interval);
			absdelta = (delta < 0) ? -(delta) : delta;
			// JS has rounding problems when the delta is something like .1, so we
			// handle fractional delta's specially (can't just add/subtract it)
			if (absdelta < 1) {
				var power10, delta1, power = (absdelta < .01) ? 3 : ((absdelta < .1) ? 2 : 1);
				power10 = Math.pow(10,power);
				delta1 = delta*power10; value1 = value*power10;
				value = (value1 + delta1);
				delta1 = (delta1 < 0) ? -(delta1) : delta1;
				value = (this.NV.minimum*power10) + ((Math.round((value-(this.NV.minimum*power10))/delta1))* delta1);
				value = value/power10;
			} else {
				value = value + delta;
				value = this.NV.minimum + ((Math.round((value-this.NV.minimum)/absdelta))* absdelta);
			}
		}
		value =  this.NV.coerce (value, this.NP);
		if (inValue!=null) return (this.setValue(value, false, true));
		else this.setValue (value, notify);
	}
	return null;
}

// Public:  Set an "attribute" on the slider.  Attributes are passed as strings of the form "name:value"
HxG_5.prototype.JSFSlider.prototype.setAttribute = function(a) {

	var wasV = this.NVid, wasC = this.NPid;
	hX_5.parseArg (this.attributeTable, this, a);
	if (this.bDsc) {
		this.bDsc.btnColor = this.buttonColor
		this.bDsc.btnBorder = this.buttonBorder;
	}

	// Find the converter and validator
	if (this.NPid && this.NPid != wasC) this.NP = hX_5.getConverterById(this.NPid);
	if (this.NVid && this.NVid != wasV) this.NV = hX_5.getValidatorById(this.NVid);
	if (this.NV != null) this.verifyValidator();
	// 	If labelInterval is provided, it must be > 0
	this.labelInterval = (this.labelInterval==null)?null:(this.labelInterval<1?1:this.labelInterval);
	// Redraw the changes
	if (this.HTMLrendered) this.redraw();
	return true;
}

// Private:  Verify that the validator is sufficient for a slider
HxG_5.prototype.JSFSlider.prototype.verifyValidator = function () {
	var min = this.NV.getAttribute("minimum-bound"), max = this.NV.getAttribute("maximum-bound");
	if (min==null && max==null) {
		this.NV.setAttribute ("maximum-bound:10");
		this.NV.setAttribute ("minimum-bound:0");
	} else if (min==null) this.NV.setAttribute ("minimum-bound:"+(max-10));
	else if (max==null) this.NV.setAttribute ("maximum-bound:"+(min+10));
	else if (min>=max) {
		min = max-10;
		this.NV.setAttribute ("minimum-bound:"+min);
	}
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.JSFSlider.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Private -- Is this a keycode that is owned by a component that's in a mask??
HxG_5.prototype.JSFSlider.prototype.isRedirectFcnKey = function (evt) {
	if (evt.altKey || evt.ctrlKey) return false;
	var code = (evt.keyCode && (evt.keyCode != 0)) ? evt.keyCode : -1;
	if (code == hX_5.KEY_UPARROW || code == hX_5.KEY_DNARROW) return true;
	return false;
}
HxG_5.prototype.JSFSlider.prototype.doRedirectFcnKey = function (evt, newValue) {
	var r=null, code=(evt.keyCode && (evt.keyCode != 0)) ? evt.keyCode : -1;
	if (evt.altKey || evt.ctrlKey) return r;
	if (code == hX_5.KEY_UPARROW) r = this.reValue("btn1", false, newValue);
	else if (code == hX_5.KEY_DNARROW) r = this.reValue("btn2", false, newValue);
	return r;
}

// Private:  Popup callback -- handle a Popup cancel event coming from the popup manager
HxG_5.prototype.JSFSlider.prototype.dispatchCancel = function (evt) {
	this.dragging = false;
	if (this.objPopup.isPopped()) this.setValue(this.escapeToValue, true);
}

// Private:  Handle a DOM event dispatched from within the popup
HxG_5.prototype.JSFSlider.prototype.dispatchPopEvent = function (evt, src, pt) {
	var r=false;
	if (!this.objPopup.isPopped()) return r;

	// Handle mouse down
	if (evt.type == hX_5._evMD) {
		if (!this.dragging) {
			this.dragging = true;
			this.dragMove(pt.X, pt.Y);
		}
	// Handle mouse up
	} else if (evt.type==hX_5._evMU) {
		// Done tracking the mouse
		if (this.dragging) {
			this.dragMove(pt.X, pt.Y);
			this.setValue(this.getValue(), true);
		}
		// When you mouse up (in the popup), we defer closing a tad so users can change their minds
		this.dragging = false;
		hX_5.popMgr.endTimed();
	// Handle mouse move
	} else if (evt.type==hX_5._evMM) {
		if (this.dragging) this.dragMove(pt.X, pt.Y);
	// Handle a keyboard
	} else if (evt.type==hX_5._evKD) {
		switch (evt.keyCode) {
		// ESC is handled by popup code

		// Space and Enter = accept value
		case hX_5.KEY_SPACEBAR:
		case hX_5.KEY_ENTER:
			this.setValue(this.fValue, true);
			hX_5.popMgr.endTimed(true);
			break;
			
		case hX_5.KEY_LTARROW:
			this.reValue("btn2", true);
			this.setStylusFromValue();
			break;
			
		case hX_5.KEY_RTARROW:
			this.reValue("btn1", true);
			this.setStylusFromValue();
			break;
			
		case hX_5.KEY_HOME:
		case hX_5.KEY_PGUP:
			this.reValue("btn4", true);
			this.setStylusFromValue();
			break;

		case hX_5.KEY_HOME:
		case hX_5.KEY_PGDN:
			this.reValue("btn3", true);
			this.setStylusFromValue();
			break;

		default:
			break;
		}
	}
	return r;
}

// Private:  Popup callback -- Handle call back as a user drags (moves the mouse)
HxG_5.prototype.JSFSlider.prototype.dragMove = function (xOff, yOff) {
	// As the slider moves, update the value
	var power = power = (this.interval < .01) ? 3 : ((this.interval < .1) ? 2 : 1);
	var power10 = Math.pow(10,power);
    var xLeft = xOff - hX_5.g.getAbsPosLeft(this.objScale);
    xLeft = (xLeft < 0) ? 0 : xLeft;
    xLeft = (xLeft > this.scale_width) ? this.scale_width : xLeft;
	xLeft = this.NV.minimum + (((Math.round( xLeft / this.tick_tabs) *power10) * this.interval)/power10);
    this.setValue(xLeft);
	this.setStylusFromValue();
}

// Private:  Handle a DOM event dispatched by the JSF object event handler
HxG_5.prototype.JSFSlider.prototype.dispatchComponent = function (item, evt, action, keycode) {
	// Return values are important so that event bubbling works
	var r=true, dropit=false, undropit=false;
	if (this.objPopup != null) {
		if (item == "btn1") {
			switch (action) {
				case hX_5._evMD:  
					if (this.objPopup.isPopped()) undropit = true;
					else dropit = true;
					break;
				case hX_5._evKD:
					if (keycode == hX_5.KEY_ENTER || keycode == hX_5.KEY_SPACEBAR || ((!evt.ctrlKey) && (keycode == hX_5.KEY_UPARROW || keycode == hX_5.KEY_DNARROW))) {
						if (this.objPopup.isPopped()) undropit = true;
						else dropit = true;
						r = false;
					} else if (!evt.ctrlKey && (keycode >= hX_5.KEY_SPACEBAR && keycode <= hX_5.KEY_DNARROW)) {
						r = false;
					}
					break;
				case hX_5._evKP:
				case hX_5._evKU:
					if (keycode >= hX_5.KEY_SPACEBAR && keycode <= hX_5.KEY_DNARROW) r = false;
					break;
					
				case hX_5._evMC:
				case hX_5._evMCC:
				case hX_5._evMM:
				case hX_5._evMO:
				case hX_5._evMV:
				case hX_5._evMU:
				case hX_5._evB:		// not sure how we get blur and focus, but we do
				case hX_5._evF:
					// nothing to do
					break;
	
				default:
					// release is all we need to do
					this.uirelease(true);
					break;
			}
			
		// Provide keyboard access to the drop-down button from the input field
		// (Same as a spinner)
		} else if (item==hX_5._lINP) {
			if (action==hX_5._evKD) {
				if (!evt.altKey && !evt.ctrlKey) {
					if (keycode == hX_5.KEY_DNARROW) this.reValue("btn2", false);
					else if (keycode == hX_5.KEY_UPARROW) this.reValue("btn1", false);
				}
			}
		} else {
			// Faulty dispatch, release all
			this.uirelease(true);
		}
		
		// Drop the popup
		if (dropit) {
			this.uirelease(true);		// Ensures no other popup is on display and that input assist is off
			this.escapeToValue = this.getValue();
			hX_5.i.setFocus(this.JSFBTN.btn1);
			hX_5.uistate.setActiveComponent (this);
			this.JSFBTN.depress ("btn1", true);
			this.sizeSliderToInput();	// Ensure the popup reflects any changes in users current prefs
			this.setStylusFromValue();	// Position the stylus
			this.objPopup.uivisible();
			hX_5.i.setFocus(this.objPopup.DOMobj);
			r = false;
			
		// Release the popup
		} else if (undropit) {
			this.uirelease(true);
			r = false;
		}
	}
	if (!r)	hX_5.i.cancelAction(evt, true);
	return r;
}

// Private:  Rebuild the contents of the slider popup given the current attributes
HxG_5.prototype.JSFSlider.prototype.sizeSliderToInput = function () {
	if (this.objPopup) {
		var popElem=this.objPopup.getElem(), popDoc=this.objPopup.getDoc(), curValue,i,j,p;

		// Make sure the class names are correct
		this.sliderClass = ((this.CSSPrefix != null && this.CSSPrefix.length>0) ? this.CSSPrefix : hX_5.CLASS_DEF) + hX_5.CLASS_SL;
		this.sliderLinesClass = this.sliderClass + hX_5.CLASS_SL_LINES;

		// Compute the number of ticks and the start tick
		curValue = this.getValue(true);
	    this.ticks = (( this.NV.maximum  - this.NV.minimum ) / this.interval ) + 1;
	    this.start_tick = ( curValue - this.NV.minimum ) / this.interval;
	    // Compute the size of the stylus -- right now it's a fixed size
	    this.stylus_width = 10;
    	this.stylus_height = 20;
    	// We use a fixed height (pixels) for the label font, in future we may make this scalable
	    this.label_size = 10;
    	// Set the thickness of the scale/height of the tick
    	this.scale_height = 1;	//this.label_size/10;
	    this.tick_height  = 4;	//this.label_size/2;
	    this.tick_width   = 1;  //this.label_size/10;
	    // Tick spacing
	    this.tick_tabs = (this.scale_width-this.tick_width) / (this.ticks-1.0);

		// Rebuild the HTML in the slider

		// Ensure the popup div is the right size and has the right class to style it
		if (popElem.className != this.sliderClass)popElem.className = this.sliderClass;
		// We need to calc the borders to set the interior contents correctly
		var borderR = hX_5.g.parseBorder(popElem, hX_5._RGT, 1), borderL = hX_5.g.parseBorder(popElem, hX_5._LFT, 1), borderT = hX_5.g.parseBorder(popElem, hX_5._TOP, 1), borderB = hX_5.g.parseBorder(popElem, hX_5._BOT, 1);
		if (hX_5.brw.isW3Cstd()) borderR = borderL = borderT = borderB = 0;
		i = this.NP.valueToString (this.NV.minimum).length; j = this.NP.valueToString (this.NV.maximum).length;
		var maxlabWidth = Math.max(i,j)*(Math.floor(this.label_size*((this.NP.isPercent)?.7:.6)));
		var maxleftMargin = Math.floor((Math.max(maxlabWidth,this.stylus_width)+1)/2);
		maxlabWidth = (this.labelInterval) ? maxlabWidth*2:maxlabWidth;
		var popheight = (this.stylus_height + this.tick_height + this.label_size + 3 + borderT + borderB), popwidth = (this.scale_width+(maxleftMargin*2)+borderL+borderR);
		hX_5.g.setP(popElem, hX_5._HT, popheight+hX_5._PX);
		hX_5.g.setP(popElem, hX_5._WD, popwidth+hX_5._PX);
		borderR = borderL = borderT = borderB = 0;
		// Add the four divs to the popup that make up the scale line, the ticks, the labels, and stylus
		var sE = this.objScale, tE = this.objTicks, lE = this.objLabels, yE = this.objStylus;
		if (this.objScale == null) {
			sE = this.objScale = popDoc.createElement(hX_5._DIV);
			sE.setAttribute (hX_5._ID, this.id + "_scale");
			sE.className = this.sliderLinesClass;
			sE.style.position = hX_5._ABS;
			sE.style.overflow = hX_5._HI;
			sE.style.fontSize = sE.style.borderWidth = sE.style.margin = sE.style.padding = hX_5._ZPX;
			popElem.appendChild(sE);
			tE = this.objTicks = popDoc.createElement(hX_5._DIV);
			tE.setAttribute (hX_5._ID, this.id + "_ticks");
			tE.className = this.sliderLinesClass;
			tE.style.position = hX_5._ABS;
			tE.style.overflow = hX_5._HI;
			tE.style.fontSize = tE.style.borderWidth = tE.style.margin = tE.style.padding = hX_5._ZPX;
			tE.style.backgroundColor = hX_5._TRN;
			popElem.appendChild(tE);
			lE = this.objLabels = popDoc.createElement(hX_5._DIV);
			lE.setAttribute (hX_5._ID, this.id + "_label");
			lE.className = this.sliderLinesClass;
			lE.style.position = hX_5._ABS;
			lE.style.fontSize = lE.style.borderWidth = lE.style.margin = lE.style.padding = hX_5._ZPX;
			lE.style.backgroundColor = hX_5._TRN;
			popElem.appendChild(lE);
			yE = this.objStylus = popDoc.createElement(hX_5._DIV);
			yE.className = this.sliderLinesClass;
			yE.style.position = hX_5._ABS;
			yE.style.overflow = hX_5._HI;
			yE.style.zIndex = "1000001";
			yE.style.fontSize = yE.style.borderWidth = yE.style.margin = yE.style.padding = hX_5._ZPX;
			yE.style.backgroundColor = hX_5._TRN;
			popElem.appendChild(yE);
		}

		// Style the scale div to look like a scale
		if (sE.className != this.sliderLinesClass)	sE.className = this.sliderLinesClass;
		sE.style.left = (borderL+maxleftMargin) + hX_5._PX;
		sE.style.top  = ((this.stylus_height/2)+borderT) + hX_5._PX;
		sE.style.height = this.scale_height + hX_5._PX;
		sE.style.width =  this.scale_width + hX_5._PX;
		p = hX_5.g.getP(sE, hX_5._COR);
		if (sE.style.backgroundColor != p) 	sE.style.backgroundColor = p;

		// Add ticks (each a div) to the ticks div
		if (tE.className != this.sliderLinesClass)	tE.className = this.sliderLinesClass;
		tE.style.left = (borderL+maxleftMargin) + hX_5._PX;
		tE.style.top  = ((this.stylus_height/2)+this.scale_height+borderT) + hX_5._PX;
		tE.style.height = ((this.stylus_height/2)+this.tick_height) + hX_5._PX;
		tE.style.width =  this.scale_width + hX_5._PX;
		var curTick = tE.firstChild;
		for (i=0; i<this.ticks; i++) {
			if (curTick==null) {
				curTick = popDoc.createElement(hX_5._DIV);
				tE.appendChild(curTick);
				curTick.style.position = hX_5._ABS;
				curTick.style.overflow = hX_5._HI;
				curTick.style.display = hX_5._BLK;
				curTick.style.fontSize = curTick.style.borderWidth = curTick.style.padding  = curTick.style.margin = hX_5._ZPX;
			}
			if (curTick.style.backgroundColor != p) curTick.style.backgroundColor = p;
			curTick.style.left =(i*this.tick_tabs) + hX_5._PX;
			curTick.style.width  = this.tick_width + hX_5._PX;
			if (i==0 || i==(this.ticks-1)) {
				curTick.style.top = hX_5._ZPX;
				curTick.style.height = ((this.stylus_height/2)+this.tick_height) + hX_5._PX;
			} else {
				curTick.style.top = (this.stylus_height/2) + hX_5._PX;
				curTick.style.height = this.tick_height + hX_5._PX;
			}
			// On to the next tick
			curTick = curTick.nextSibling;
		}
		// Hide any left over ticks (can happen if bounds are changed between calls)
		// Note that browsers can't actually "delete" an element only unhook it from the tree
		// so we just hide any extras
		while (curTick != null) {
			curTick.style.display = hX_5._NO;
			curTick = curTick.nextSibling;
		}
		// Add labels (each a div) to the labels div
		if (lE.className != this.sliderClass) lE.className = this.sliderClass;
		lE.style.left = (borderL+maxleftMargin) + hX_5._PX;
		lE.style.top = (this.stylus_height+this.tick_height+borderT) + hX_5._PX;
		lE.style.height = this.label_size + hX_5._PX;
		lE.style.width =  (this.scale_width+this.stylus_width + 3) + hX_5._PX;
		var curLabel = lE.firstChild, label, txtNode, subCnt=this.labelInterval, p10;
		for (i=0; i<this.ticks; i++) {
			if ((this.show_labels || i == 0 || i == this.ticks - 1) && (!this.show_labels || this.labelInterval==null || this.labelInterval==subCnt)) {
				subCnt = 0;
				if (curLabel == null) {
					curLabel = popDoc.createElement(hX_5._DIV);
					curLabel.className = this.sliderClass;
					curLabel.style.position = hX_5._ABS;
					curLabel.style.display = hX_5._BLK;
					curLabel.style.verticalAlign = hX_5._TOP;
					curLabel.style.textAlign = hX_5._CTR;
					curLabel.style.top = hX_5._ZPX;
					curLabel.style.fontSize = this.label_size + hX_5._PX;
					curLabel.style.borderWidth = curLabel.style.margin = curLabel.style.padding = hX_5._ZPX;
					curLabel.style.backgroundColor = hX_5._TRN;
					if (hX_5.brw.isNavOrMoz() && hX_5.g.getP(curLabel, hX_5._CRS) == hX_5._AUT) curLabel.style.cursor = hX_5._DFT;
					lE.appendChild(curLabel);
				}
				if (curLabel.className != this.sliderClass)	curLabel.className = this.sliderClass;
				curLabel.style.left = ((i*this.tick_tabs)-(maxlabWidth/2)+1) + hX_5._PX;
				curLabel.style.width  = maxlabWidth + hX_5._PX;
				curLabel.style.height = (this.label_size) + hX_5._PX;
				txtNode = curLabel.firstChild;
				if (txtNode == null) {
					txtNode = popDoc.createTextNode(hX_5._B);
					curLabel.appendChild(txtNode);
				}
				if (this.interval < 1) {
					p10 = Math.pow(10,((this.interval < .01) ? 3 : ((this.interval < .1) ? 2 : 1)));
					label = ((this.NV.minimum*p10)  + (i * (this.interval*p10)))/p10;
				} else label = this.NV.minimum  + (i * this.interval);
				if (label > this.NV.maximum) label = this.NV.maximum;
				txtNode.nodeValue = this.NP.valueToString(label);
				// On to the next tick
				curLabel = curLabel.nextSibling;
			}		
			subCnt++;
		}
		// Hide any left over labels (can happen if bounds are changed between calls)
		while (curLabel != null) {
			curLabel.style.display = hX_5._NO;
			curLabel = curLabel.nextSibling;
		}

		// Style the stylus div
		if (yE.className != this.sliderLinesClass)	yE.className = this.sliderLinesClass;
		yE.style.height = this.stylus_height + hX_5._PX;
		yE.style.width  = this.stylus_width  + hX_5._PX;
		var borderWidth=((hX_5.brw.isW3Cstd())?2:0), borderHeight=((hX_5.brw.isW3Cstd())?1:0);
		
		// Now fill the div with divs to make the stylus
		var subE=yE.firstChild, triangleHeight=((this.stylus_width/2)-1), boxHeight=((this.stylus_height-triangleHeight-borderHeight));
		// first the "top" block
		if (!subE) {
			subE = popDoc.createElement(hX_5._DIV);
			subE.className = this.sliderLinesClass;
			subE.style.position = hX_5._ABS;
			subE.style.overflow = hX_5._HI;
			subE.style.display = hX_5._BLK;
			subE.style.fontSize = subE.style.margin = subE.style.padding = hX_5._ZPX;
			subE.style.borderStyle = hX_5._SLD;
			subE.style.borderWidth = hX_5._1PX;
			subE.style.borderBottomWidth = subE.style.left = subE.style.top = hX_5._ZPX;
			yE.appendChild(subE);
		}
		if (subE.className != this.sliderLinesClass) subE.className = this.sliderLinesClass;
		subE.style.height = boxHeight + hX_5._PX;
		subE.style.width  = (this.stylus_width-borderWidth) + hX_5._PX;
		for (i = 1; i <= triangleHeight; i++) {
			subE = subE.nextSibling;
			if (!subE) {
				subE = popDoc.createElement(hX_5._DIV);
				subE.className = this.sliderLinesClass;
				subE.style.position = hX_5._ABS;
				subE.style.overflow = hX_5._HI;
				subE.style.fontSize = subE.style.margin = subE.style.padding = hX_5._ZPX;
				subE.style.borderStyle = hX_5._SLD;
				subE.style.borderWidth = hX_5._1PX;
				subE.style.borderBottomWidth = subE.style.borderTopWidth = hX_5._ZPX;
				yE.appendChild(subE);
			}
			if (subE.className != this.sliderLinesClass)	subE.className = this.sliderLinesClass;
			subE.style.top = (boxHeight+i-1+borderHeight) + hX_5._PX;
			subE.style.left = i + hX_5._PX;
			subE.style.height = hX_5._1PX;
			subE.style.width  = (this.stylus_width-borderWidth-(2*i)) + hX_5._PX;
		}
		return true;		
	}
	return false;
}

// Private: Given the value, draw the stylus
HxG_5.prototype.JSFSlider.prototype.setStylusFromValue = function () {
	var xLeft, xOff, wasLeft, value = this.getValue(true);
	if (!value || value == null || isNaN(value)) value = new Number(0);
    wasLeft = xLeft = parseInt(this.objScale.style.left, 10);
    xOff  = Math.round((value - this.NV.minimum)/this.interval) * this.tick_tabs;
    xLeft = xLeft + xOff - (this.stylus_width/2);
    if (wasLeft != xLeft) this.objStylus.style.left = xLeft + hX_5._PX;
    return true;
}
