/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2007 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// class JSFBehaviorTypeahead
// The JSFBehaviorTypeahead class is used to manage the typeahead functionality of an input

// Public:  Constructor
HxG_5.prototype.JSFBehaviorTypeahead = function () {
   
   this.type = hX_5.BHR_typeahead;
   
   // Style Consts
   this.TA_LIST 		= 0;	// suggestion list
   this.TA_ITEM 		= 1;	// suggestion item
   this.TA_ITEM_SEL 	= 2;	// suggestion item (selected)
   
   // Other Consts
   this.TA_MODE_AC = "autocomplete";
   this.TA_MODE_SL = "suggest";
   
   // Properties
   this.CSSPrefix 		= null;
   this.taId 			= null;
   this.mode 			= null;
   this.matchWidth 		= null;
   this.maxSuggestions 	= null;
   this.maxSize 		= null;
   this.startChars 		= null;
   this.startDelay 		= null;
   this.vbExpression	= null;
   this.onStart			= null;
   this.onComplete		= null;  
   this.href			= null;
   
   this.attributeTable = new Array();
      this.attributeTable ['id'] 			= ['taId',hX_5.ARG_TO_NAN,false, null, null];
      this.attributeTable ['type'] 			= ['mode',hX_5.ARG_TO_NAN,false, null, null];
      this.attributeTable ['delay'] 		= ['startDelay',hX_5.ARG_TO_NUM,false, 100, 10000];
      this.attributeTable ['matchwidth'] 	= ['matchWidth',hX_5.ARG_IS_TRUE,false, null, null];
      this.attributeTable ['maxsugg'] 		= ['maxSuggestions', hX_5.ARG_TO_NUM,false, -1, 1000];
      this.attributeTable ['size'] 			= ['maxSize', hX_5.ARG_TO_NUM,false, -1, 1000];
      this.attributeTable ['chars'] 		= ['startChars', hX_5.ARG_TO_NUM,false, -1, 1000];
      this.attributeTable ['css'] 			= ['CSSPrefix', hX_5.ARG_TO_NAN,false, null, null];
      this.attributeTable ['vbexpression']	= ['vbExpression', hX_5.ARG_TO_NAN, false, null, null];
      this.attributeTable ['onstart']		= ['onStart', hX_5.ARG_TO_NAN, false, null, null];
      this.attributeTable ['oncomplete']	= ['onComplete', hX_5.ARG_TO_NAN, false, null, null];     
      this.attributeTable ['href']			= ['href', hX_5.ARG_TO_NAN, false, null, null];
      
   this.setAttribute(arguments); 	// inherited method from superclass hX_5.JSFBehavior()
   
   // defaults
   this.CSSPrefix 		= (this.CSSPrefix!=null) 		? this.CSSPrefix 			: hX_5.CLASS_DEF + hX_5.CLASS_TA;
   this.mode 			= (this.mode!=null) 			? this.mode.toLowerCase() 	: this.TA_MODE_SL;
   this.matchWidth 		= (this.matchWidth!=null) 		? this.matchWidth 			: true;
   this.maxSize 		= (this.maxSize!=null) 			? this.maxSize 				: 10;
   this.maxSuggestions	= (this.maxSuggestions!=null) 	? this.maxSuggestions 		: 10;
   this.startChars 		= (this.startChars!=null) 		? this.startChars 			: 1;
   this.startDelay 		= (this.startDelay!=null) 		? this.startDelay 			: 500;

   this.classes				= null;
   this.baseClass			= null;
   this.setClassNames();
      
   // params for form submit
   this.TA_PARAM_NAME		= "__searchKey__";
   this.TA_PARAM_MAX		= "__maxResults__";
   this.TA_PARAM_VB			= "__vbExpression__";
   
   //const for constructing Typeahead hidden input ids
	this.TA_HIDDEN_ID = "$$TA_AJAX_UTIL$$";
   
   this.formatCallback = this.setValues;
   
   // The list of suggestions
   this.suggestionTable = null;
   this.objPop				= null;
   this.backChannel 		= null;
   this.suggestionCells 	= null;
   this.currentSuggestion 	= this.displayIndex = -1;	// currentSuggestion is index of selected suggestion
   														// displayIndex is index of first displayed suggestion in popup div
   														// displayIndex is 0 unless popup div is scrolled
   this.typedLength 		= 0;
   this.timer 				= null;
   this.autoCompleteActive 	= true;
   this.escToValue 			= null;
   this.focusToValue		= null;
   this.focusValueUsed		= false;
   
}

// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorTypeahead.prototype = new hX_5.JSFBehavior();

HxG_5.prototype.JSFBehaviorTypeahead.prototype.onPageLoad = function () {
	this.DOMobj.setAttribute("autocomplete", "off");
	this.setEvent();
	return true;
}

// private: Function for managing the timers associated with delayed reaction to inputs
HxG_5.prototype.JSFBehaviorTypeahead.prototype.dispatchTimer = function (keyCode, action) {

   if (this.objPop == null) {
      this.objPop = this.createSuggestionArea(this.DOMobj);
   }

   if (action == "backspace") {
      if (this.DOMobj.value.length > 0) {
         if ( (this.typedLength - this.DOMobj.value.length) == 1 ) {
            //single character deleted
            this.autoCompleteActive = true;
         } else {
            this.autoCompleteActive = false;
         }
      } else {
         this.uirelease(true);
         return;
      }
   } else if (action == "charpress") {
      this.autoCompleteActive = true;
   }
   this.typedLength = this.DOMobj.value.length; // cache old value length
   
   if(this.typedLength >= this.startChars) {
		if (this.onStart)
		{
			if (!hX_5.action.processFunction(this.DOMobj, keyCode, this.onStart)) 
			{ 
				this.uirelease(true);
				return false;
			}	
		}
      this.getTypeaheadValues(this.DOMobj.value);
   } else {
      // make sure it's released if we are not getting results
      this.uirelease(true);
   }
}

// private: called by popup management code on exit
HxG_5.prototype.JSFBehaviorTypeahead.prototype.dispatchCancel = function (evt) {
	if (this.escToValue!=null) this.setText(this.escToValue);
	if (this.timer!=null) {
    	hX_5.i.killTimerTimed(this.timer);
    	this.timer = null;
	}
	return;
}

// private: manage Dom events dispatched by popup management
HxG_5.prototype.JSFBehaviorTypeahead.prototype.dispatchPopEvent = function (evt, src, pt) {

   if (this.objPop && (!this.objPop.isPopped()))
      return true;	// handled by dispatchBehavior

   if (this.DOMobj!=null && evt!=null) {
      if (evt.type==hX_5._evKD) {
         return this.doKeydown(evt);
      } else if (evt.type == hX_5._evKP && hX_5.getKeyCode(evt)==hX_5.KEY_ESC) {
         // Firefox is weird and it handles ESC on Keypress (and any overriding of it must be done on keypress)
         // So the popup manager is set to defer processing ESC to this handler
         this.dispatchCancel();
         hX_5.uirelease();
         hX_5.i.cancelAction(evt, true)
         return false;
      } else if (evt.type==hX_5._evMD || evt.type==hX_5._evMM || evt.type==hX_5._evMU) {
         // See what td we are over
         if (src.tagName) {
            while (src && src.tagName!=undefined && src.tagName!=hX_5._TD && src.tagName!=hX_5._TAB) src = src.parentNode;
            if (src && src.tagName!=undefined && src.tagName==hX_5._TD) {
               if (evt.type==hX_5._evMU)
                  this.clickSuggestion(src);
               else
                  this.hoverOnSuggestion(src);
               return false;
            }
         }
      }
   }
   return true;	// need to return true to ensure event is registered in input field
}

// private: manage dispatched keydown events
HxG_5.prototype.JSFBehaviorTypeahead.prototype.dispatchBehavior = function (thisObj, evt) {
   
   if (this.objPop && this.objPop.isPopped()) return true;	// handled by dispatchPopEvent
      
   if ( (this.DOMobj != null) && (evt != null)) {
      if (evt.type==hX_5._evKD) return this.doKeydown(evt);
      if (evt.type==hX_5._evF) {
      	this.focusToValue = this.DOMobj.value;
      	this.focusValueUsed = false;
      }
      if (evt.type==hX_5._evB) {
      	this.uirelease(true);
      	// If we mucked with selection in the input field, onchange may not fire
      	// We'll force it to fire in this case (thought it could fire twice)
      	if (this.focusValueUsed && this.focusToValue!=null && this.focusToValue!=this.DOMobj.value)
		  hX_5.i.fireEvent (this.DOMobj, hX_5._onC);
      }
   }
   return true;	// need to return true to ensure event is registered in input field
}

// private: handle keydown events
HxG_5.prototype.JSFBehaviorTypeahead.prototype.doKeydown = function(evt) {
   var result = true, curr;
   
   if ((this.DOMobj != null) && (evt != null) && (evt.type==hX_5._evKD)) 
   {
		if (evt.altKey||evt.ctrlKey)
		{
			// do not process this character
			return result;
		}
      var keyCode = hX_5.getKeyCode(evt);
      var keyValue   = hX_5.getChrCode(evt);
      
      if (this.objPop == null) 
      {
         this.objPop = this.createSuggestionArea(this.DOMobj);
      }
      
      if (keyCode == hX_5.KEY_DNARROW)
      {
         if (this.objPop.isPopped() || this.mode == this.TA_MODE_AC) {
            if (this.currentSuggestion < (this.suggestionCells.length - 1)) {
               this.setSelected(this.currentSuggestion+1);
               curr = this.getSelected();
               this.setText(curr,this.typedLength,curr.length);  //lengths req'd for partial highlighting autocomplete
               if (this.mode == this.TA_MODE_SL && (this.currentSuggestion >= (this.displayIndex + this.maxSize)) ){
					var pop = this.objPop.getElem();
					pop.scrollTop = pop.scrollTop + hX_5.g.getRendHeight(this.suggestionCells[0]);
					this.displayIndex++;
				}
            } else if ( this.currentSuggestion == (this.suggestionCells.length - 1) ) {
               curr = this.getSelected();
               this.setText(curr,this.typedLength,curr.length);  //lengths req'd for partial highlighting autocomplete
            }
         } else {
            if(this.typedLength >= this.startChars) {
               this.getTypeaheadValues(this.DOMobj.value);  
            }
         }
      } else if (keyCode == hX_5.KEY_UPARROW) {
      	if (this.objPop.isPopped() || this.mode == this.TA_MODE_AC) {
			if (this.currentSuggestion > 0) {
				this.setSelected(this.currentSuggestion-1);
				curr = this.getSelected();
				this.setText(curr,this.typedLength,curr.length); //lengths req'd for partial highlighting autocomplete
				if (this.mode == this.TA_MODE_SL && (this.currentSuggestion < this.displayIndex) ){
					var pop = this.objPop.getElem();
					pop.scrollTop = pop.scrollTop - hX_5.g.getRendHeight(this.suggestionCells[0]);
					this.displayIndex--;
				}
			} else if (this.currentSuggestion == 0) {
				curr = this.getSelected();
				this.setText(curr,this.typedLength,curr.length);  //lengths req'd for partial highlighting autocomplete
			}
		}
      } else if (keyCode == hX_5.KEY_RTARROW) {
         if (this.objPop.isPopped()) {
            this.getTypeaheadValues(this.getSelected());
         }
      }
      else if (keyCode == hX_5.KEY_LTARROW) {
      	// Don't do anything here?
      } else if (keyCode == hX_5.KEY_ENTER || keyCode==hX_5.KEY_TAB) { // does the return and tab key
         if (this.objPop.isPopped()) {
            if (keyCode==hX_5.KEY_ENTER) {
		        this.uirelease(true);
				hX_5.i.cancelAction(evt,true);
	            return false;
	        }
         }
        this.uirelease(true);
         
      } else if ( !hX_5.isFcnKey(evt) ) { // any printable character
         if(this.timer) {
            hX_5.i.killTimerTimed(this.timer);
            this.timer = null;
         }
         this.timer = hX_5.i.startTimerTimed(this.DOMobj.id, keyValue, "charpress", this.startDelay, hX_5.BHR_typeahead, hX_5._evKD);
      } else if (keyCode == hX_5.KEY_BS) {// backspace
         if(this.timer) {
            hX_5.i.killTimerTimed(this.timer);
            this.timer = null;
         }
         this.timer = hX_5.i.startTimerTimed(this.DOMobj.id, keyValue, "backspace", this.startDelay, hX_5.BHR_typeahead, hX_5._evKD);
      }
      return result;
   }
}

HxG_5.prototype.JSFBehaviorTypeahead.prototype.setEventCode = function () {
	// We only do this once -- when behavior is first run
	if (!this.HTMLrendered) {
		hX_5.attachBehaviorEvent(this.DOMobj, this.type, hX_5._onB, hX_5._lN, hX_5._onKD);
		hX_5.attachBehaviorEvent(this.DOMobj, this.type, hX_5._onF, hX_5._lN, hX_5._onKD);
		if (hX_5.brw.isNavOrMoz()) hX_5.attachBehaviorEvent(this.DOMobj, this.type, hX_5._onB, hX_5._lN, hX_5._onKP);
	}
}

// private: change selected suggestion item
HxG_5.prototype.JSFBehaviorTypeahead.prototype.setSelected = function (index) {
   this.suggestionCells[this.currentSuggestion].className = this.classes[this.TA_ITEM];
   this.currentSuggestion = index;
   this.suggestionCells[this.currentSuggestion].className = this.classes[this.TA_ITEM_SEL];
}

// private: set the text in the input field and caret position if necessary
HxG_5.prototype.JSFBehaviorTypeahead.prototype.setText = function(str, pos1, pos2) {
   this.DOMobj.value = str;
   if(!pos2)
   		pos2 = null;
   if (pos1 == null) {
      // default cursor to end
      this.setCaretPosition(this.DOMobj.value.length, null);
   } else {
      // caret fn will handle the case where pos2 is null
      this.setCaretPosition(pos1,pos2);
   }
   this.focusValueUsed = true;
}

// private: create the table to contain the suggestion list and containing popup object
HxG_5.prototype.JSFBehaviorTypeahead.prototype.createSuggestionArea = function(where) {
   
   //create popup
   var pop =  hX_5.addPopup (this.id + "_suggPop", this.id, null, hX_5.POP_STYLE_CLICK_SCROLLABLE, this);
   
   //get the popup div
   var el = pop.getElem();
   el.className = this.baseClass;
   this.suggestionTable = document.createElement(hX_5._TAB);
   this.suggestionTable.setAttribute(hX_5._ID, this.DOMobj.id + "_suggArea");
   this.suggestionTable.className = this.classes[this.TA_LIST];
   
   el.appendChild(this.suggestionTable);
	if (this.matchWidth || hX_5.getClassStyle(this.baseClass, hX_5._WD) != hX_5._AUT)
	{
		this.suggestionTable.style.width = "100%";
	}
	else
	{
		this.suggestionTable.style.width = "";
	}
	
   //return the popup
   return pop;
}

// private: get suggestion by index
HxG_5.prototype.JSFBehaviorTypeahead.prototype.getSuggestion = function(index) {
   return this.suggestionCells[index].firstChild.firstChild.nodeValue;
}

// private: get currently selected suggestion
HxG_5.prototype.JSFBehaviorTypeahead.prototype.getSelected = function() {
   return this.suggestionCells[this.currentSuggestion].firstChild.firstChild.nodeValue;
}

// private: set suggestion by index
HxG_5.prototype.JSFBehaviorTypeahead.prototype.setSuggestion = function(index, str) {
   this.suggestionCells[index].firstChild.firstChild.nodeValue = str;
}

// private: create <tr> and <td> for a new suggestion
HxG_5.prototype.JSFBehaviorTypeahead.prototype.createSuggestionCell = function (index, txt, selected) {
   
   // deprecated
   //var row = document.createElement(hX_5._TR),	// create table row
   //cell = document.createElement(hX_5._TD);	// create table data
   
   var row = this.suggestionTable.insertRow(index),
   		cell = row.insertCell(0);
   
   cell.setAttribute(hX_5._ID, this.DOMobj.id + "_sugg" + index);
   cell.className = (selected) ? this.classes[this.TA_ITEM_SEL] : this.classes[this.TA_ITEM];
   
   // append text node to contain suggestion string
   /*cell.appendChild(document.createTextNode(""));
   cell.firstChild.nodeValue = txt;*/
   
   // new method: create anchor tag and store that in the suggestion cell
   // ensures better screenreader compatability
   var anch = document.createElement(hX_5._ANC);
   anch.setAttribute("href", hX_5._LB);
   anch.appendChild(document.createTextNode(""));
   anch.firstChild.nodeValue = txt;
	cell.appendChild(anch);
	
   // store reference to cell
   this.suggestionCells[index] = cell;
   
	if (this.matchWidth || hX_5.getClassStyle(this.baseClass, hX_5._WD) != hX_5._AUT)
	{
		this.suggestionCells[index].style.width = "100%";
	}
	else
	{
		this.suggestionCells[index].style.width = "";
	}
   
}

// private: remove suggestion <tr> and <td>
HxG_5.prototype.JSFBehaviorTypeahead.prototype.removeSuggestion = function(index) {
   var row = this.suggestionCells[index].parentNode;
   row.deleteCell(0);
   this.suggestionTable.deleteRow(index);
   this.suggestionCells[index] = null;
}

// private: remove last suggestion <tr> and <td>
HxG_5.prototype.JSFBehaviorTypeahead.prototype.removeLastSuggestion = function() {

   var row = this.suggestionCells[this.suggestionCells.length-1].parentNode;
   row.deleteCell(0);
   this.suggestionTable.deleteRow(this.suggestionCells.length-1);
   this.suggestionCells[this.suggestionCells.length-1] = null;
   this.suggestionCells.length--;
}

// private: set suggestion String in suggestion area
// removes and adds cells as is needed
HxG_5.prototype.JSFBehaviorTypeahead.prototype.populateSuggestionArea = function(slist) {
   var i;
   if (this.suggestionCells == null) {
      this.suggestionCells = new Array();
      
      //create table cells to contain suggestions
      this.currentSuggestion = this.displayIndex = 0;
      for (i = 0; i < slist.length ;i++) {
         var sel = (i==0) ? true : false;
         this.createSuggestionCell(i, slist[i], sel);
      }
   }
   else
   {
      var old_len = this.suggestionCells.length;
      for (i = 0; i < slist.length; i++) {
         if (i<old_len) {
            this.setSuggestion(i,slist[i]);
         } else {
            this.createSuggestionCell(i, slist[i], sel);
         }
      }
      
      //remove any old cells no longer required
      if (slist.length < old_len)
      {
         for (i = slist.length; i < old_len; i++) {
            this.removeLastSuggestion();
         }
         //this.suggestionCells = this.suggestionCells.slice(0,slist.length);
      }
      
      if (this.currentSuggestion != 0)
      {
         //set the current suggestion to the first item
         if (slist.length >= old_len) {
            // only do this if the cell hasn't already been removed
            this.suggestionCells[this.currentSuggestion].className = this.classes[this.TA_ITEM];
         }
         this.currentSuggestion = 0;
         this.suggestionCells[0].className = this.classes[this.TA_ITEM_SEL];
      }
   }
   
   var sugg1 = this.getSuggestion(0);
   this.escToValue = this.DOMobj.value;
   if (this.DOMobj.value != sugg1 && this.autoCompleteActive) {
      this.setText(sugg1,this.typedLength, sugg1.length);
   }
   
	this.uivisible();
   
}

// private: handle mouse hover event
HxG_5.prototype.JSFBehaviorTypeahead.prototype.hoverOnSuggestion = function(elem) {
   
   var suggCellId = elem.id, sugg = "_sugg";

   var ip = hX_5.getElementById( suggCellId.substring(0, suggCellId.lastIndexOf('_')) ),
   		suggIndex = suggCellId.substring(suggCellId.lastIndexOf(sugg) + sugg.length),
   		ta = hX_5.getBehaviorById(ip.id,hX_5.BHR_typeahead,hX_5._evKD);

   if (suggIndex != ta.currentSuggestion) {
      ta.suggestionCells[ta.currentSuggestion].className = ta.classes[ta.TA_ITEM];;
      ta.suggestionCells[suggIndex].className = ta.classes[ta.TA_ITEM_SEL];
      ta.currentSuggestion = suggIndex;
   }
   
}

// private: handle mouse click on suggestion cell
HxG_5.prototype.JSFBehaviorTypeahead.prototype.clickSuggestion = function(elem) {

   var suggCellId = elem.id, sugg = "_sugg";
   
   var ip = hX_5.getElementById( suggCellId.substring(0, suggCellId.lastIndexOf('_')) ),
   		suggIndex = suggCellId.substring(suggCellId.lastIndexOf(sugg) + sugg.length),
   		ta = hX_5.getBehaviorById(ip.id,hX_5.BHR_typeahead,hX_5._evKD);
   
   if(!ta.suggestionCells) {
      return;
   } else if (suggIndex != ta.currentSuggestion) {
      ta.suggestionCells[ta.currentSuggestion].className = ta.classes[ta.TA_ITEM];
      ta.suggestionCells[suggIndex].className = ta.classes[ta.TA_ITEM_SEL];
      ta.currentSuggestion = suggIndex;
   }
   
   ta.uirelease(true);
   ta.setText(elem.firstChild.firstChild.nodeValue);
}	



	 /* private: Process the returned data. 
	 * The response sends back an XML document with the search results:
	 * 
	 * <?xml..>
	 * <taresponse>
	 * 	<suggestlist>
	 * 		<suggestion>aardvark</suggestion>
	 * 		...
	 * 		<suggestions>aardwolf</suggestion>
	 * 	</suggestlist>
	 * </taresponse>
	 * 
	 * If there are no suggestions returned, there are no child tags of the suggestlist tag
	 * The suggestions are followed by a url delimitted by a tag (not part of above xml document):
	 * 
	 * <url>new/Typahead/url.jsp</url> 
	 */
HxG_5.prototype.JSFBehaviorTypeahead.prototype.setValues = function(XMLData, StringData, param) {

 	// this is Typeahead object

   if (this != null && this != 'undefined' && (this.type == hX_5.BHR_typeahead)) 
   {
		
	    var myData;
		if (XMLData && XMLData.documentElement) 
		{
	         myData = new hX_5.XMLData(XMLData);
		} 
		else if (StringData) 
		{
			// get the response
			var bodyStr = this.parseBody(StringData);

			// constants needed for parsing
			var startXml = bodyStr.indexOf("<?xml"),
				delimResponseStart = "<taresponse>",delimResponseEnd = "</taresponse>", 
				delimUrlStart = "<url>", delimUrlEnd = "</url>",
				tagnameList = "suggestlist";
				
			// find the end of the suggestion list XML document
			var endXml = bodyStr.indexOf(delimResponseEnd) + delimResponseEnd.length, xmlStr;

			// extract the XML (<?xml.. to </taresponse>)
			xmlStr = bodyStr.substring( startXml, endXml );

			// parse the suggestions
			myData = new hX_5.XMLData(xmlStr);
			var localList = new Array(), iY = 0,
      			suggs = myData.child(tagnameList).children();
      		
      					//get Url string; do not include <url> tag delimeters
			var startUrl = bodyStr.indexOf(delimUrlStart,endXml) + delimUrlStart.length,
				endUrl = bodyStr.indexOf(delimUrlEnd,endXml);
			
			var urlStr = bodyStr.substring(startUrl, endUrl);
			
			// store the new Url in the typeahead behavior
			this.setAttribute("href:" + urlStr);      		

 		    for (var iX=0; iX<=suggs.length()-1; iX++) 
		    {
	    		localList[iY] = suggs.elementByIndex(iX).text().toString();
		         if (localList[iY].length > 0)
	    	        iY++;
	      	}

			// localList is list of suggestions
	      	if ( localList.length > 0 ) 
	      	{
				this.populateSuggestionArea(localList);
			} 
			
			if (!hX_5.action.processFunction(this.DOMobj, null, this.onComplete)) 
			{ 
				// If there were results shown and 'oncomplete' returned 'false', remove the popup
				if ( localList.length > 0 ) 
				{
					this.uirelease(true);
				}
			}	         
	

		} 
		else 
		{
			this.uirelease(true);
		}
		
   }
}


// private: make gui/popup visible
HxG_5.prototype.JSFBehaviorTypeahead.prototype.uivisible = function() {
		
	// need to set widths and heights here - cannot do this with hX_5.g.getRend*** before popup div is visible in IE
	if (this.suggestionCells.length > 0 && this.mode == this.TA_MODE_SL)
	{
		this.objPop.uivisible(null,null,null,null,true);	// put popup on display

		var pop = this.objPop.getElem();

		pop.style.overflowY = hX_5._AUT;

		 //override default popup height and width set in pop up onPageLoad
		if (this.matchWidth) 
		{
			if (hX_5.brw.isIE() && (!hX_5.brw.isW3Cstd()) )
			{
				// IE in transitional mode seems to include borders in width calculation for some reason
				pop.style.width = hX_5.g.getRendWidth(this.DOMobj) + hX_5._PX;
			}
			else
			{
				pop.style.width = (hX_5.g.getRendWidth(this.DOMobj) - hX_5.g.parseBorder(pop, hX_5._LFT, 1) - hX_5.g.parseBorder(pop, hX_5._RGT, 1)) + hX_5._PX;
			}
		} 
		else
		{
			var css_width = hX_5.getClassStyle(this.baseClass, hX_5._WD);
			// if there is no width set in the css for the popup enforce variable
			// width by setting style.width = "" on the div element
			if (css_width == hX_5._AUT)
			{
				// if there is no width set in the css for the popup enforce variable
				// width by setting style.width = "" on the div element
				pop.style.width = "";
			}
			else
			{
				// force css width
				pop.style.width = css_width;
			}
			
		}

		
		var row_height = hX_5.g.getRendHeight(this.suggestionCells[0]),
			num_rows = Math.min(this.maxSize, this.suggestionCells.length);
			
		if (hX_5.brw.isIE() && (!hX_5.brw.isW3Cstd()) )
		{
			//need to make more room in IE as IE includes the borders in the styled box height
			pop.style.height = (row_height * num_rows) + hX_5.g.parseBorder(pop, hX_5._TOP, 1) + hX_5.g.parseBorder(pop, hX_5._BOT, 1) + hX_5._PX;
		}
		else
		{
			pop.style.height = (row_height * num_rows) + hX_5._PX;
		}

		if ( !this.matchWidth 
			&& (this.suggestionCells.length > this.maxSize)	// scrolling should be on
			&& (hX_5.brw.isIE()))
		{
			// need to manually add scrollwidth in IE
			var w =(hX_5.g.getRendWidth(pop)+hX_5.getScrollbarWidth());
			pop.style.width = w + hX_5._PX;
		}
		
		// to enforce new style dimensions for popup visible rect for mouse event hit test
		this.objPop.uivisible(null,null,null,null,true);
	} 	
}

// private: release function for gui/popup
HxG_5.prototype.JSFBehaviorTypeahead.prototype.uirelease = function(free) {
     if (this.timer!=null) {
        hX_5.i.killTimerTimed(this.timer);
        this.timer = null;
     }
     if (free!=null)
      hX_5.uirelease(true);
   
   if (this.suggestionTable != null) {
      //remove rows till there are none left
      while (this.suggestionTable.childNodes.length > 0) {
         this.suggestionTable.removeChild(this.suggestionTable.childNodes[0]);
      }
   }
   
   if (this.suggestionCells != null) {
      for (var i =0; i < this.suggestionCells.length; i++)
         this.suggestionCells[i] = null;
      this.suggestionCells = null;
   }
   
   // reset vars:
   this.currentSuggestion = -1;
   this.typedLength = 0;
   this.autoCompleteActive = true;
}

// private: destructor
HxG_5.prototype.JSFBehaviorTypeahead.prototype.destroy = function(free) {
   var o;
   // Delete popup and backchannel
   if (this.objPop) hX_5.removePopup(this.objPop.id);
   if (this.backChannel) {
      if (this.backChannel.destroy) this.backChannel.destroy();
      delete (this.backChannel);
   }
   // Delete larger memory objects
   if (this.suggestionTable) delete this.suggestionTable;
   if (this.attributeTable) delete this.attributeTable;
   if (this.suggestionCells) {
      while (this.suggestionCells.length > 0) {
         o = this.suggestionCells.pop();
         delete o;
      }
      delete this.suggestionCells;
   }
   if (this.classes) {
   	  while (this.classes.length>0) this.classes.pop();
   	  delete this.classes;
   }
	return true;
}

// private: creates backchannel object, constructs request url and sends request
HxG_5.prototype.JSFBehaviorTypeahead.prototype.getTypeaheadValues = function(txt) {
	
   if (txt && (txt != "")) {

		//put together 2d array of params to be passed to server
		var ptr_arr = [[this.TA_PARAM_NAME,txt],
						[this.TA_PARAM_MAX,this.maxSuggestions],
						[this.TA_PARAM_VB,this.vbExpression]]; 
		
		if(hX_5.isPortal)
		{
			var arr_ptr = [this.TA_PARAM_NAME,this.TA_PARAM_MAX,this.TA_PARAM_VB]; 
			var arr_val = [txt, this.maxSuggestions, this.vbExpression];
			
			this.backChannel = null;
			this.backChannel = new hX_5.HTTPBackchannel("backChannel", this.formatCallback, true, true, null, this.onError, true ,hX_5.isPortal); 
			this.backChannel.LoadEx(this.href, this, null, null, arr_ptr, arr_val, true); 
   		
	   	}
	   	else
	   	{
   	   	
	      this.backChannel = null;
	      this.backChannel = new hX_5.HTTPBackchannel("backChannel", this.formatCallback, false, true, null, this.onError);
	      /* TODO: This should not be necessary!
	         if (this.backChannel == null) {
    		     this.backChannel = new hX_5.HTTPBackchannel("backChannel", this.formatCallback, true, true, null, this.onError);
	         } else {
	        	 this.backChannel.reset();
      		}
      		*/
     	// TODO: We need to add support for in-client state saving
      		var reqURL = hX_5.reEncodeURI(this.href); 
			if (ptr_arr!=null) reqURL = reqURL + hX_5.encodeURIParams(reqURL, ptr_arr);

      		this.backChannel.Load(reqURL, null, this);
      	}
   	}
   	
}

// private: error function
HxG_5.prototype.JSFBehaviorTypeahead.prototype.onError = function(msg) {
   //Adam commenting this out to avoid alert("OK") happening all the time
   //alert(msg);
}


// private: set cursor position / selection. if only one arg supplied, 2nd arg defaults to end
HxG_5.prototype.JSFBehaviorTypeahead.prototype.setCaretPosition = function(pos1, pos2) {
   
   this.typedLength = pos1;
   if (arguments.length > 1) {
      hX_5.sel.setCaretPos(this.DOMobj, pos1, pos2, hX_5.brw.isNavOrMoz());
   } else {
      hX_5.sel.setCaretPos(this.DOMobj, pos1, null, hX_5.brw.isNavOrMoz());
   }
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.JSFBehaviorTypeahead.prototype.getAttribute = function (attribute) {
   return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public:  Set an "attribute" on the typeahead.  Attributes are passed as strings of the form "name:value"
HxG_5.prototype.JSFBehaviorTypeahead.prototype.setAttribute = function(a) {
   
   hX_5.parseArg (this.attributeTable, this, a);
   return true;
}

// Implementation:  Given a prefix, generate the names of all classes we use
HxG_5.prototype.JSFBehaviorTypeahead.prototype.setClassNames = function () {
   
   if ( (this.CSSPrefix != null) && (this.CSSPrefix.length > 0) ) {
      this.baseClass = this.CSSPrefix;
   } else {
      this.baseClass = hX_5.CLASS_DEF + hX_5.CLASS_TA;	// default
   }

   var l = hX_5.CLASS_TAS.length;
   this.classes = [];
   for (var i=0; i<l; i++)
      this.classes.push(this.baseClass + hX_5.CLASS_TAS[i]);
   
}

HxG_5.prototype.JSFBehaviorTypeahead.prototype.parseBody = function (StringData, XMLData) {
	
	// This code is copied from the first portion of 
	// HxG_5.prototype.JSFBehaviorAjax.prototype.callbackBehavior = function (XMLData, StringData) {
	// need a way to consolidate
	
	var evt=null, r=true;
	if (XMLData && XMLData.documentElement) {
/*	
		alert ("Not sure how to process");
		But it's something like:
		    response  = req.responseXML.documentElement;
	        method    =	response.getElementsByTagName('method')[0].firstChild.data;
	        result    = response.getElementsByTagName('result')[0].firstChild.data;
or
		myData = new hX_5.XMLData(XMLData);
*/
	} else if (StringData) {
// DEBUG POINT -- StringData contains entire content transmitted
// alert (StringData);		
		var b, c, spcO=false, o, o1=null, DOM_o1=null, oH, i, j, k, l, h, svParent, svPos, svId, sE, tB="</BODY", tH="</HEAD>", tPH ='<head title="PORTALcontent">', tPHX = "</head", isP=false, skip, firstTag=null, ch, ch1, chB=-1, w1, w2, BODY, lsrc="src", lcharset="charset", ltype = "type", aa, ad=[], ab=document.getElementsByTagName(hX_5._SCR), lab=((ab)?ab.length:0);

		// If this is refreshRequest or updateRequest and not portal, we should have no head or body tag in what's returned (unless it's in a portlet)
		// If this is an externalRequest or portal , we might have a head and body tag. Parse the string down to be just the body tag.  
		// We could do a "real" parse of the HTML but this is expensive and slow
		// Instead, look for the </HEAD> tag and toss everything before it.  Then parse for the <BODY> tag.
		// Now someone could do something strunky like have a comment or script block that has the string </HEAD>
		// and we'd fail in that case, but we'll just tell people that they shouldn't do that
		// If this is portal, there could be a "second" head tag (and close) marking the start of the content
		// of the portlet we're interested in.  In this case, after the <body> tag, skip down to the portlet head tag
		l = StringData.length;
		ch = StringData.indexOf(tH);
		if (ch<0) ch = StringData.indexOf(tH.toLowerCase());
		if (ch<0 && this.type!=hX_5.BHR_ajaxgetexternal && !hX_5.isPortal) {
			BODY = StringData;
		} else {
			if (ch<0) ch=0; else ch+=tH.length;
			// After the </HEAD> tag, look for a body tag
			while (ch!=null && ch<l) {
				// Look for next tag start
				while (ch<l && StringData.charAt(ch)!=hX_5._LT) ch++;
				// Parse the located tag
				ch1 = hX_5.htmlParseTag(StringData, ch, false);
				if (ch1==null) ch=null;
				else {
					ch = ch1.ch;
					if (ch1.tag==hX_5._BBODY) {chB=ch; ch=null; }
				}
			}
			// chB points to the char after the close of the <body> tag
			// if this is portal, see if there's a proprietary tag marking the portlet content we're interested in
			if (hX_5.isPortal) {
				ch = StringData.indexOf(tPH, chB);
				if (ch>=0) {
					chB = ch+tPH.length;
					isP = true;
				}
			}
			// chB points to the char after the close of the body  or char after the close of the portal head tag
			// Look for the portal close tag if this is portal otherwise do a dirty find for the end body tag
			if (isP) {
				ch = StringData.lastIndexOf(tPHX);
				if (ch<0) {
					this.callbackError(hX_5.INTERR_bc_baddata);
					return false;
				}
			} else {
				ch = StringData.lastIndexOf(tB);
				if (ch<0) ch = StringData.lastIndexOf(tB.toLowerCase());
			}
			if (chB<0) {
				this.callbackError(hX_5.INTERR_bc_baddata);
				return false;
			}
			BODY = (ch>0) ? StringData.substring(chB,ch) : StringData.substr(chB);
		}
	}
	return (BODY ? BODY : "");
}