/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// ---------------------
// JSF Tab Frame Control
// ---------------------

// Public:  Constructor
HxG_5.prototype.JSFTabFrame = function () {
	this.type = "tabframe";

	// Attributes set by caller
	this.initialTab = 0;
	this.hideTabs = false;
	this.showAtMostTabs = -1;
	this.frameHeight = "200";
	this.frameWidth = "300";
	this.uniformTabs = false;
	this.slantActiveLeft = 0;
	this.slantActiveRight = 4;
	this.slantInactiveLeft = 0;
	this.slantInactiveRight = 4;
	this.CSSPrefix = hX_5.CLASS_TABPANEL;
	this.classAll = "";
	this.tabCount  = 0;
	
	this.hasButtons = false;
	this.buttonNextId = null;
	this.buttonPrevId = null;
	this.buttonOKId = null;
	this.buttonCancelId = null;
	
	// Configuration
	this.verifyStyleSheets = false;	// If true, we'll verify that the control has a valid stylesheet family
	
	// Implementation
    this.activeTab = 0;
    this.tabs = new Array();
    this.HTML = null;
    this.tabHeight = -1;
    this.tabWidth = -1;

    // Attributes supported on this object
    this.attributeTable = new Array();
    	this.attributeTable ['height'] 				= ['frameHeight', 	null, 				false,	null,	null];
	 	this.attributeTable ['width'] 				= ['frameWidth', 	null,				false,	null,	null];
	 	this.attributeTable ['hide-tabs']			= ['hideTabs',		hX_5.ARG_IS_TRUE,	true,	null,	null];
	 	this.attributeTable	['uniform-tabs']		= ['uniformTabs',	hX_5.ARG_IS_TRUE,	true,	null,	null];
	 	this.attributeTable	['initial-tab']			= ['initialTab',	hX_5.ARG_TO_NUM,	true,	0,		99];
	 	this.attributeTable	['show-tabs']			= ['showAtMostTabs',hX_5.ARG_TO_NUM,	true,	2,		99];
	 	this.attributeTable	['slant-inactive-left']	= ['slantInactiveLeft',	hX_5.ARG_TO_NUM,true,	0,		99];
	 	this.attributeTable	['slant-active-left']	= ['slantActiveLeft',	hX_5.ARG_TO_NUM,true,	0,		99];
	 	this.attributeTable	['slant-inactive-right']= ['slantInactiveRight',hX_5.ARG_TO_NUM,true,	0,		99];
	 	this.attributeTable	['slant-active-right']	= ['slantActiveRight',	hX_5.ARG_TO_NUM,true,	0,		99];
	 	this.attributeTable	['next-button']			= ['buttonNextId',	hX_5.ARG_TO_NAN,	true,	null,	null];
	 	this.attributeTable	['prev-button']			= ['buttonPrevId',	hX_5.ARG_TO_NAN,	true,	null,	null];
	 	this.attributeTable	['back-button']			= ['buttonPrevId',	hX_5.ARG_TO_NAN,	true,	null,	null];
	 	this.attributeTable	['finish-button']		= ['buttonOKId',	hX_5.ARG_TO_NAN,	true,	null,	null];
	 	this.attributeTable	['cancel-button']		= ['buttonCancelId',hX_5.ARG_TO_NAN,	true,	null,	null];
	
	this.setAttribute(arguments);	
}

// Public:  Page level onLoad handler
HxG_5.prototype.JSFTabFrame.prototype.onPageLoad = function () {
	if (hX_5.brw.isNavOrMoz()) this.DOMobj.style.visibility = "hidden";
	this.activeTab = (this.initialTab < 0) ? 0 : ((this.initialTab > this.tabCount-1) ? this.tabCount-1 : this.initialTab);
	this.redraw();
	if (hX_5.brw.isNavOrMoz()) this.DOMobj.style.visibility = "visible";
	return true;
}

// Public:  Redraw the tabframe incorporating any changes to the HTML of the input or tabframe fields
HxG_5.prototype.JSFTabFrame.prototype.redraw = function () {
	this.emitHTML();
	return true;
}

// Public:  Release the ui state of the tabframe
HxG_5.prototype.JSFTabFrame.prototype.uirelease = function () {
	// no-op
	return true;
}

// Public:  Add a subcomponent (a tab) to a tabframe
HxG_5.prototype.JSFTabFrame.prototype.addSubComponent = function (childId, childObj) {
	if (childId && childObj && childObj.type == "tab" && !this.HTMLrendered) {
		this.tabs[this.tabCount] = childObj;
		this.tabCount++;
		childObj.DOMobj.style.display = "none";
		childObj.fixupLabel();
	}
}

// Public:  Set the value of a tabframe (i.e., set which tab is active)
HxG_5.prototype.JSFTabFrame.prototype.setValue = function (tabid, notify, evt) {
	var rValue;
	var dir = 0;
	var tabnum = 0
	if (tabid == "+") {
		tabnum = this.nextTab(this.activeTab);
		dir = +1;
	} else if (tabid == "-") {
		tabnum = this.prevTab(this.activeTab);
		dir = -1;
	} else {
		tabnum = this.toTabNum(tabid);
	}
	if (tabnum >= 0 && tabnum < this.tabCount) {
		if (this.activeTab != tabnum) {
			// We're switching tabs, if the current tab has an "onExit" event, fire it and
			// see if we continue
			if (notify && this.tabs[this.activeTab].onExit) {
				var thisEvent = new Object;
				this.initEvent (thisEvent, evt, "exit", this.tabs[this.activeTab].DOMobj, null, this.tabs[tabnum].DOMobj, true);
				rValue = this.tabs[this.activeTab].onExit.call (this, thisEvent);
				if (!thisEvent.returnValue) rValue = false;
				if (hX_5.isDef(typeof(rValue)) && !rValue) {
					return rValue;
				}
			}
			// Switch to the new tab
			var was = this.activeTab;
			this.activeTab = tabnum;
			this.drawTabHTML(dir, false);
			// If the new tab has an "onEnter" event, fire it
			if (notify && this.tabs[this.activeTab].onEnter) {
				var thisEvent = new Object;
				this.initEvent (thisEvent, evt, "exit", this.tabs[this.activeTab].DOMobj, this.tabs[was].DOMobj, null, true);
				rValue = this.tabs[this.activeTab].onEnter.call(this, thisEvent);
				if (!thisEvent.returnValue) rValue = false;
			}
		}
	}
	return rValue;
}

// Public:  Get a value (the active tab) from a tabframe
HxG_5.prototype.JSFTabFrame.prototype.getValue = function() {
	return this.activeTab
}

// Public:  Hide a tab
HxG_5.prototype.JSFTabFrame.prototype.hideTab = function(tabid) {
	if (!this.HTMLrendered) return;
	tabnum = this.toTabNum(tabid);
	this.tabs[tabnum].visible = false;
	this.tabs[tabnum].tdObj.style.display = "none";
	if (tabnum == this.activeTab) {
		tabnum = this.prevTab(this.activeTab);
		this.setValue(tabnum, true);
	}
	this.redraw();
}

// Public:  Show a tab
HxG_5.prototype.JSFTabFrame.prototype.showTab = function(tabid) {
	if (!this.HTMLrendered) return;
	tabnum = this.toTabNum(tabid);
	this.tabs[tabnum].visible = true;
	this.tabs[tabnum].tdObj.style.display = "";
	this.redraw();
}

// Public:  Set an "attribute" or set of attributes on the tabframe.  Attributes are passed as strings of the form "name:value"
//			Call with one or more attributes (comma separated as arguments)
HxG_5.prototype.JSFTabFrame.prototype.setAttribute = function(a) {

	hX_5.parseArg (this.attributeTable, this, a);
	this.hasButtons = (this.buttonNextId || this.buttonPrevId || this.buttonOKId || this.buttonCancelId);
		
	// Redraw the changes
	if (this.HTMLrendered) this.redraw();

	return true;
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.JSFTabFrame.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

// Public: Retrieve the number of tabs
HxG_5.prototype.JSFTabFrame.prototype.getTabCount = function () {
	return (this.tabCount);
}

// Private:  Find the next visible tab
HxG_5.prototype.JSFTabFrame.prototype.nextTab = function(startTab) {
	var tabnum = startTab+1;
	while (tabnum < this.tabCount && !this.tabs[tabnum].visible) tabnum++;
	tabnum = (tabnum > this.tabCount-1) ? this.tabCount-1 : ((tabnum < 0) ? 0 : tabnum);
	while (tabnum > 0 && !this.tabs[tabnum].visible) tabnum--;
	tabnum = (tabnum > this.tabCount-1) ? this.tabCount-1 : ((tabnum < 0) ? 0 : tabnum);
	return tabnum;
}

// Private:  find the previous visible tab
HxG_5.prototype.JSFTabFrame.prototype.prevTab = function(startTab) {
	var tabnum = startTab-1;
	while (tabnum > 0 && !this.tabs[tabnum].visible) tabnum--;
	tabnum = (tabnum > this.tabCount-1) ? this.tabCount-1 : ((tabnum < 0) ? 0 : tabnum);
	while (tabnum < this.tabCount && !this.tabs[tabnum].visible) tabnum++;
	tabnum = (tabnum > this.tabCount-1) ? this.tabCount-1 : ((tabnum < 0) ? 0 : tabnum);
	return tabnum;
}

// Implementation:  Convert a "div-id" of a panel to a tabnumber
HxG_5.prototype.JSFTabFrame.prototype.toTabNum = function(value) {
	var	tabnum = value;
	if (hX_5.isString(value)) {
		for (tabnum = 0; tabnum < this.tabCount; tabnum++) {
			if (this.tabs[tabnum].id == value) {
				break;
			}
		}
	}
	return ((tabnum <= 0) ? 0 : ((tabnum >= this.tabCount-1) ? this.tabCount-1 : tabnum));
}

// Implementation:  Given a prefix, generate the names of all classes we use
HxG_5.prototype.JSFTabFrame.prototype.setClassNames = function () {
	if (this.classAll != "" && this.classAll == this.DOMobj.className) return;
	if (this.DOMobj.className && this.DOMobj.className.length > 0 && this.DOMobj.className != 'undefined' ) {
		this.classAll = this.DOMobj.className;
		var i = this.classAll.indexOf(" " );
		if (i < 0)
			this.CSSPrefix = this.classAll;
		else 
			this.CSSPrefix = this.classAll.substr(0,i);
		if (this.verifyStyleSheets) {			
			var sheets, rulelist, j;
			var validFamily = false;
			var testname = this.CSSPrefix + "-" + hX_5.CLASS_TAB_HEADER;
			for (i = 0; i < document.styleSheets.length; i++) {
				sheets = document.styleSheets[i];
				rulelist = (hX_5.isDef(typeof(sheets.cssRules))) ? sheets.cssRules :
					((hX_5.isDef(typeof(sheets.rules))) ? sheets.rules : null);
				if (rulelist) {
					for (j = 0; j < rulelist.length; j++) {
						if (rulelist[j].selectorText.indexOf(testname) >= 0) {
							validFamily = true;
							break;
						}
					}		
				}
			}
			if (!validFamily) {
				this.CSSPrefix = hX_5.CLASS_TABPANEL;
				alert ("Invalid stylesheet family");
			}
		}
	} else {
		this.classAll = hX_5.CLASS_TABPANEL;
		this.CSSPrefix = hX_5.CLASS_TABPANEL;
	}
	this.classHeader = this.CSSPrefix + "-" + hX_5.CLASS_TAB_HEADER;
	this.classBody   = this.CSSPrefix + "-" + hX_5.CLASS_TAB_BODY;
	this.classFooter = this.CSSPrefix + "-" + hX_5.CLASS_TAB_FOOTER;
	this.classButton = this.CSSPrefix + "-" + hX_5.CLASS_TAB_BUTTON;
	this.classTabActive = this.CSSPrefix + "-" + hX_5.CLASS_TAB_TABACTIVE;
	this.classTabInactive = this.CSSPrefix + "-" + hX_5.CLASS_TAB_TABINACTIVE;
}

// Private:  Handle a DOM event dispatched by by a tab button's onclick handler
HxG_5.prototype.JSFTabFrame.prototype.dispatchAction = function (evt) {
	// N.B. "this" is not valid
	var thisp = hX_5;
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt && evt.type=="click" || (evt.type=="keydown" && (evt.keyCode==32 || evt.keyCode==13))) {
		var item = 0;
		var src = hX_5.evtGetTarget(evt);
		if (src) {
			src = (src.nodeType == hX_5.ELEMENT_NODE || src.nodeType == hX_5.DOCUMENT_NODE) ? src : src.parentNode;
			if (src) {
				var focusable = src;
				src = thisp.geo.findContainer (src, "TD", "TR", "TBODY", "TABLE", "TD");
				if (src && src.id && src.id != "") {
					var divid = src.id;
					var j = divid.indexOf("-HX-TD-SUBC");
					if (j >= 0) 
						divid = divid.substring(0,j);
					srcobj = thisp.getParentComponentById(divid);
					if (srcobj) {
						srcobj.setValue(divid, true, evt);
						// odd IE bug -- you can't use "src" here to set focus,
						// you have to reget the dom obj
						if (focusable.id) {
							if (thisp.brw.isIE()) {
								var ieBugObj = thisp.getElementById(focusable.id);
								if (ieBugObj) ieBugObj.focus();
							} else {
								// fixme -- NS bug, can't focus a span
							}
						}
						thisp.imp.cancelAction(true);
						return false;
					}
				}
			}
		}
	}
	return;
}	

// Private:  Handle a DOM event dispatched by by a tab button's onclick handler
HxG_5.prototype.JSFTabFrame.prototype.dispatchBtnAction = function (evt) {
	// N.B. "this" is not valid
	var thisp = hX_5;
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt) {
		var item = 0;
		var src = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : null);
		if (src) {
			src = (src.nodeType == hX_5.ELEMENT_NODE || src.nodeType == hX_5.DOCUMENT_NODE) ? src : src.parentNode;
			if (src) {
				if (src.tagName == "INPUT" && src.id) {
					var ok = false;
					tabc = thisp.geo.findContainer (src, "TD", "TR", "TBODY", "TABLE");
					if (!tabc.id)
						tabc = thisp.geo.findContainer (tabc, "TD", "TR", "TBODY", "TABLE");
					tabc = (tabc) ? thisp.getComponentById(tabc.id) : null;
					if (tabc) {
						if (tabc.HTML.hdrbtn1 && src.id == tabc.HTML.hdrbtn1.id) {
							tabc.drawTabHTML(-1, true);
							ok = true;
						} else if (tabc.HTML.hdrbtn2 && src.id == tabc.HTML.hdrbtn2.id) {
							tabc.drawTabHTML(+1, true);
							ok = true;
						} else if (tabc.HTML.btn1 && src.id == tabc.HTML.btn1.id) {
							tabc.setValue("-", true, evt);
							if (tabc.HTML.btn1.disabled && tabc.HTML.btn2)
								tabc.HTML.btn2.focus();
							ok = true;
						} else if (tabc.HTML.btn2 && src.id == tabc.HTML.btn2.id) {
							tabc.setValue("+", true, evt);
							if (tabc.HTML.btn2.disabled && tabc.HTML.btn1)
								tabc.HTML.btn1.focus();
							ok = true;
						} else if (tabc.HTML.btn3 && src.id == tabc.HTML.btn3.id) {
							// no-op let the normal onclick fire
						} else if (tabc.HTML.btn4 && src.id == tabc.HTML.btn4.id) {
							// no-op let the normal onclick fire
						}
					}
					if (ok)	{
						thisp.imp.cancelAction(true);
						return false;
					}
				}
			}
		}
	}
	return;
}	

// Private: Construct an "event" object for use when a tabframe event is fired
HxG_5.prototype.JSFTabFrame.prototype.initEvent = function (thisEvent, evt, name, id, wasid, willbeid, cancelable) {
	var i;
	if (evt) {
		for (i in evt) thisEvent[i] = evt[i];
	}
	thisEvent.bubbles = false;
	thisEvent.cancelable = cancelable;
	thisEvent.cancelBubble = false;
	thisEvent.returnValue = true;
	thisEvent.eventPhase = 2;
	thisEvent.fromElement = wasid;
	thisEvent.toElement = willbeid;
	thisEvent.currentTarget = id;
	thisEvent.relatedTarget = (wasid) ? wasid : willbeid;
	thisEvent.srcElement = id;
	thisEvent.target = id;
	thisEvent.type = name;
}	

// Implementation:  Contents (objects) making up a tab control table
HxG_5.prototype.JSFtabset = function (inobj, R) {
	this.valid = false;
	
	// Make sure the containing table is spaced correctly
	inobj.DOMobj.border = "0";
	inobj.DOMobj.cellSpacing = "0";
	inobj.DOMobj.cellPadding = "0";
	
	// Basic table = three rows (header body footer), each row with one cell
	this.tr1 = this.tr2 = this.tr3 = this.td1 = this.td2 = this.td3 = null;
	// Header
	this.hdrtr = this.hdrTable = this.hdrspcer = this.hdrbtn1 = this.hdrbtn2 = null;
	// Footer 
	this.btn1 = this.btn2 = this.btn3 = this.btn4 = null;
	
	// Verify the basic structure of the table, should be a table of 3 rows each row containing a cell
	// If this isn't so, add the missing rows and cells.  We want to end up with:
	// <table> 
	//	<tr><td></td></tr>						<-- will later contain table of tabs in the one cell
	//  <tr><td>one div per tab panel</td></tr> <-- divs laid down by script collector, one per panel body
	//  <tr><td></td></tr>						<-- should already contain buttons
	// </table>
	var temp;
	if (inobj == null || inobj.DOMobj == null || inobj.DOMobj.tagName != "TABLE" ) return;
	if (inobj.DOMobj.rows.length < 3) {
		temp = inobj.DOMobj.insertRow(0);
		temp.insertCell(0);
	}
	if (inobj.DOMobj.rows.length < 3) {
		temp = inobj.DOMobj.insertRow(2);
		temp.insertCell(0);
	}
	this.tr1 = inobj.DOMobj.rows[0];
	this.tr2 = inobj.DOMobj.rows[1];		
	this.tr3 = inobj.DOMobj.rows[2];
	if (this.tr1 && this.tr2 && this.tr3) {
		this.td1 = this.tr1.cells[0];
		this.td2 = this.tr2.cells[0];
		this.td3 = this.tr3.cells[0];
	}
	
	// If the basic structure is OK, then create the content of the first tabs row (header)
	if (this.tr1 && this.tr2 && this.tr3 && this.td1 && this.td2 && this.td3) {
		this.td1.vAlign = "top";
//		this.td1.noWrap = "true";
		this.td2.vAlign = "top";
		this.td3.vAlign = "top";
		// The top row is a cell containing a table, that is:
		// <TR><TD><TABLE><TR></TR></TABLE></TD></TR>
		this.hdrTable = document.createElement("TABLE");
		this.hdrTable.border = "0";
		this.hdrTable.cellSpacing = "0";
		this.hdrTable.cellPadding = "0";
		this.hdrtr = this.hdrTable.insertRow(0);
		
		// The row in the header table has one cell for each tab, plus a "spacer" cell
		var thisTab, tab;
		for (thisTab = 0; thisTab < (inobj.tabCount+1); thisTab++) {
			tab = this.hdrtr.insertCell(thisTab);
			tab.noWrap = "true";
			// For all non-spacer cells, insert a "tab table" in the cell
			if (thisTab < inobj.tabCount) {
				inobj.emitTabHTML(tab);
				tab.setAttribute("id", inobj.tabs[thisTab].id  + "-HX-TD-SUBC");
				inobj.tabs[thisTab].tdObj = tab;
			// For the spacer cell, emit the next/prev buttons if needed
			} else {
				this.hdrspcer = tab;
				tab.style.backgroundColor = "transparent";
				if (inobj.showAtMostTabs >= 0) {
					this.hdrbtn1 = inobj.emitButtonHTML (inobj.DOMobj.id + "-HX-HDRBTN1", 1)
					this.hdrbtn1.style.marginLeft = "4px";
					this.hdrbtn1.style.backgroundPosition = R.wellPosition(hx.R_IMG_LG_HINKIES, 1,0);
					this.hdrbtn1.alt  = inobj.p.STR_TABPANEL_SCROLLLEFT;
					this.hdrbtn1.title= inobj.p.STR_TABPANEL_SCROLLLEFT;
					tab.appendChild(this.hdrbtn1);
					this.hdrbtn2 = inobj.emitButtonHTML (inobj.DOMobj.id + "-HX-HDRBTN2", 1)
					this.hdrbtn2.style.backgroundPosition = R.wellPosition(hx.R_IMG_LG_HINKIES, 0,0);
					this.hdrbtn2.alt  = hx.STR_TABPANEL_SCROLLRIGHT;
					this.hdrbtn2.title= hx.STR_TABPANEL_SCROLLRIGHT;
					tab.appendChild(this.hdrbtn2);
				}
			}
		}
		this.td1.appendChild(this.hdrTable);
		this.valid = (this.hdrtr.cells.length == inobj.tabCount+1);
	}
}

// Private:  Rebuild the contents of the tabframe given the current attributes
HxG_5.prototype.JSFTabFrame.prototype.emitHTML = function () {

	if (this.DOMobj && this.tabCount > 0) {
		var thisTab, tab, thisWidth;
	
		// If we haven't added an HTML descriptor add it
		if (!this.HTML) this.HTML = new this.p.JSFtabset(this, this.r);
		if (!this.HTML.valid) return;
		this.setClassNames();
		// At this point the table is structured, we want to make sure that all 
		// the width's and classes are correctly set, and things are shown/hidden

		// Style the table which is the frame around the tab control
		if (this.DOMobj.className != this.classAll)
			this.DOMobj.className = this.classAll;
		this.DOMobj.width = this.frameWidth;

		if (this.hideTabs) {
			this.HTML.tr1.style.display = "none";
		// Now clean up the table that contains the "tab" cells
		}  else {
			this.HTML.tr1.style.display = "";
			this.HTML.td1.className = this.classHeader;
			this.HTML.td1.width = "100%";
			this.HTML.hdrTable.width = "100%";
			
			// Clean up the spacer cell
			this.HTML.hdrspcer.width = "99%";
			
			// Style each of the "tab" cells
			var maxWidth = 0;
			for (thisTab = 0; thisTab < this.tabCount; thisTab++) {
				thisWidth = this.styleTabHTML (this.tabs[thisTab], this.HTML.hdrtr.cells[thisTab], (thisTab==0), this.classTabActive, this.slantActiveLeft, this.slantActiveRight, true);
				maxWidth = Math.max(Math.max(maxWidth, thisWidth), this.tabWidth);
			}
			if (this.uniformTabs) {
				this.tabWidth = maxWidth;
				for (thisTab = 0; thisTab < this.tabCount; thisTab++) {
					tab = this.getFirstChild(this.HTML.hdrtr.cells[thisTab]);
					tab.width = this.tabWidth+"";
				}
			}
		}
		
		// Style the second row of the table which contains the body divs
		this.HTML.td2.width = "100%";
		this.HTML.td2.className = this.classBody;
		
		// Make sure the tab content bodies are sized
		for (thisTab = 0; thisTab < this.tabCount; thisTab++) {
			this.tabs[thisTab].DOMobj = this.p.getElementById (this.tabs[thisTab].id);
			this.tabs[thisTab].DOMobj.style.height = this.frameHeight;
			this.tabs[thisTab].DOMobj.style.width = "100%";
		}

		// Handle having buttons on display -- they're kept in the cell in the third row
		if (!this.hasButtons) {
			// Ignore the row and its content
			this.HTML.tr3.style.display = "none";
		} else {
			this.HTML.td3.width = "100%";
			this.HTML.td3.className = this.classFooter;			
			this.HTML.btn1 = this.emitButtonHTML (this.buttonPrevId,  0, true);
			this.HTML.btn2 = this.emitButtonHTML (this.buttonNextId,  0, true);
			this.HTML.btn3 = this.emitButtonHTML (this.buttonOKId,    0, false);
			this.HTML.btn4 = this.emitButtonHTML (this.buttonCancelId,0, false);
		}
		
		// Draw the tabs as selected/unselected
		this.drawTabHTML(0, false);
	
		// Done		
		this.HTMLrendered = true;
		return true;		
	}
	return false;
}

// Private:  The first child of a cell/div can be a "text node" in Netscape
HxG_5.prototype.JSFTabFrame.prototype.getFirstChild = function (cell) {
	r = (cell) ? cell.firstChild : null;
	while (r && r.nodeType != hX_5.ELEMENT_NODE) r = r.nextSibling;
	return r;
}

// Private: Make/Fixup a button
HxG_5.prototype.JSFTabFrame.prototype.emitButtonHTML = function (id, typ, localHandler) {
	var btn =  null;
	if (id) {
		if (typ == 0) {
			btn = this.p.getElementById(id);
			if (btn && btn.tagName && btn.tagName == "INPUT" && btn.type && (btn.type == "submit" || btn.type == "button")) {
				if (localHandler) 	btn.onclick = this.dispatchBtnAction;
				if (btn.className.indexOf(this.classButton) < 0)
					btn.className = btn.className + " " + this.classButton;
			} else {
				btn = null;
			}
		} else {
			btn = document.createElement("INPUT");
			btn.setAttribute ("id", id);
			btn.setAttribute ("type", "image");
			btn.onclick = this.dispatchBtnAction;
			btn.style.width = btn.style.height = "12px";
			btn.style.margin = btn.style.padding = "0px";
			btn.style.borderStyle = "solid";
			btn.style.borderColor = "threedshadow";
			btn.style.borderWidth = "1px";
			btn.src = this.r.getURL (this.r.R_IMG_SPACER);
			btn.style.backgroundImage = this.r.getURI (this.p.R_IMG_LG_HINKIES);
			btn.style.backgroundRepeat = "no-repeat";
		}
	}
	return btn;
}

// Private:  Make a table used to draw a tab. Each table is a one row with 4 cells in it.
//			 Leftmost cell is a "spacer", next cell the "slant left", then the label cell, then the right slant cell
//			 i.e., ./label\
HxG_5.prototype.JSFTabFrame.prototype.emitTabHTML = function (td) {

	td.vAlign = "top";
	var table = document.createElement("table");
	table.border = "0";
	table.cellSpacing = "0";
	table.cellPadding = "0";
	table.onclick = this.dispatchAction;
	table.onkeydown = this.dispatchAction;
	table.style.cursor = "pointer";
	td.appendChild(table);
	var tr = table.insertRow(0);

	// First cell is "invisible" -- used only to position things
	var td = tr.insertCell(0);
	td.setAttribute("width", "0");
	td.vAlign = "top";
	td.noWrap = "true";
	td.style.padding="0px";
	td.style.margin="0px";
	td.style.borderWidth = "0px";

	// Second cell is the "left" slant
	td = tr.insertCell(1);
	td.vAlign = "top";
	td.noWrap = "true";
	td.style.padding="0px";
	td.style.margin="0px";
	td.style.borderRightWidth = "0px";

	// Third cell is the label
	td = tr.insertCell(2);
	td.vAlign = "top";
	td.noWrap = "true";
	td.style.margin ="0px";

	// Fourth cell is the "right" slant
	td = tr.insertCell(3);
	td.vAlign = "top";
	td.noWrap = "true";
	td.style.padding="0px";
	td.style.margin="0px";
	td.style.borderLeftWidth = "0px";
	
	return table;
}

// Private:  Style the HTML of a 4-cell table to make it look like a tab
HxG_5.prototype.JSFTabFrame.prototype.styleTabHTML = function (tabobj, td, bFirst, tabclass, nipLeft, nipRight, bottomline) {

	if (td.style.display == "none") return -1;
		
	var table = this.getFirstChild(td);
	var tr  = table.rows[0];
	var tdZ = tr.cells[0];
	var tdL = tr.cells[1];
	var tdM = tr.cells[2];
	var tdR = tr.cells[3];

	// Style the center td, first clearing anything we may have set previously
	tdM.style.borderLeftWidth = tdM.style.borderRightWidth = "";
	tdM.className = tabclass;
	if (tabobj.rewriteLabel) {	
		tdM.innerHTML = tabobj.wrappedLabel;
		tabobj.rewriteLabel = false;
	}
	var borderT = this.p.geo.parseBorder (tdM, "top", 1);
	var borderB = this.p.geo.parseBorder (tdM, "bottom", 1);
	var borderL = this.p.geo.parseBorder (tdM, "left", 1);
	var borderR = this.p.geo.parseBorder (tdM, "right", 1);
	var backColor  	     = this.p.geo.getEffectiveStyle(tdM, "background-color");
	var borderTopStyle	 = this.p.geo.getEffectiveStyle(tdM, "border-top-style");
	var borderTopColor   = this.p.geo.getEffectiveStyle(tdM, "border-top-color");
	var borderRightColor = this.p.geo.getEffectiveStyle(tdM, "border-right-color");
	var borderLeftColor  = this.p.geo.getEffectiveStyle(tdM, "border-left-color");
	var borderBottomColor= this.p.geo.getEffectiveStyle(tdM, "border-bottom-color");
	tdM.style.borderLeftWidth = tdM.style.borderRightWidth = "0px";
	if (bFirst)
		this.tabHeight = tdM.offsetHeight;
	if (tdL.height != this.tabHeight) {
		tdL.height = this.tabHeight+"";
		tdM.height = this.tabHeight+"";
		tdR.height = this.tabHeight+"";
	}
	var cssVOffset = 0;
	var cssHOffset = 0;
	if (this.p.brw.isNavOrMoz()) {
		cssVOffset = borderT + ((borderT > 1) ? 1 : 0);
		cssHOffset = 2;
	}
	
	// Style the left td
	var color, sdiv, div, nip, xtra;
	div = this.getFirstChild(tdL);
	if (!div) div = this.createDiv(tdL,0);
	if (tdL.className != tdM.className)	tdL.className = tdM.className;
	if (tdL.style.textAlign != "left")  tdL.style.textAlign = "left";
	sdiv = this.getFirstChild(div);
	if (nipLeft == 0) {
		if (tdL.width != "1") {
			tdL.setAttribute("width", "1");
			tdL.style.borderLeftWidth = "";
			tdL.style.borderTopWidth = "";
			tdL.style.backgroundColor = "";
		}
	} else {
		nip = nipLeft + borderL;
		xtra = (borderT > 1) ? 1 : 0;
		nip = (nip > this.tabHeight - borderB + xtra) ? this.tabHeight - borderB + xtra - cssVOffset: nip;
		if (tdL.width != nip+"") {
			tdL.width = nip+"";
			tdL.style.borderLeftWidth = "0px";
			tdL.style.borderTopWidth = "0px";
			tdL.style.backgroundColor = "transparent";
		}
		for (var k = 1; k <= nip; k++) {
			if (!sdiv) sdiv = this.createDiv(div,1);
			color = (k <= borderL) ? borderLeftColor : backColor;
			this.vertLine (sdiv, color, k-1, (nip-k+1-xtra), this.tabHeight - borderB - (nip-k+1) + xtra - cssVOffset, borderT+xtra, borderTopStyle, borderTopColor);
			sdiv = sdiv.nextSibling;
		}
	}
	while (sdiv) {
		sdiv.style.display = "none";
		sdiv = sdiv.nextSibling;
	}
	
	// Style the right td
	div = this.getFirstChild(tdR);
	if (!div) div = this.createDiv(tdR,0);
	if (tdR.className != tdM.className) tdR.className = tdM.className;
	if (tdR.style.textAlign != "left") tdR.style.textAlign = "left";
	sdiv = this.getFirstChild(div);
	if (nipRight == 0) {
		if (tdR.width != "1") {
			tdR.setAttribute("width", "1");
			tdR.style.borderRightWidth = "";
			tdR.style.borderTopWidth = "";
			tdR.style.backgroundColor = "";
		}
	} else {
		nip = nipRight + borderR;
		xtra = (borderT > 1) ? 1 : 0;
		nip = (nip > this.tabHeight - borderB + xtra) ? (this.tabHeight - borderB + xtra - cssVOffset) : nip;
		if (tdR.width != nip+"") {
			tdR.width = nip+"";
			tdR.style.borderRightWidth = "0px";
			tdR.style.borderTopWidth = "0px";
			tdR.style.backgroundColor = "transparent";
		}
		for (var k = 1; k <= nip; k++) {
			if (!sdiv) sdiv = this.createDiv(div,1);
			color = (k > nip-borderR) ? borderRightColor : backColor;
			this.vertLine (sdiv, color, k-1, k-xtra, this.tabHeight - borderB - k + xtra - cssVOffset, borderT+xtra, borderTopStyle, borderTopColor);
			sdiv = sdiv.nextSibling;
		}
	}
	while (sdiv) {
		sdiv.style.display = "none";
		sdiv = sdiv.nextSibling;
	}

	// If this is the active tab, we may need to "overwrite" the line at the bottom of the tab
	// Tab is active, its bottom border color is same as the tab color, and the border width is zero
	div = this.getFirstChild(tdZ);
	if (!div) div = this.createDiv(tdZ,1);
	sdiv = this.getFirstChild(div);
	if (!sdiv) sdiv = this.createDiv(div,1);
	if (div.style.display != "none") div.style.display = "none";
	if (bottomline) {
		if (borderBottomColor == backColor && borderB == 0) {
			var	w = (tr.offsetWidth) - (borderR+borderL) + "px";			
			var h = (this.p.geo.parseBorder (this.HTML.td1, "bottom", 1)) + "px";
			if (div.style.display != "block")		div.style.display = "block";
			if (div.style.overflow != "visible") 	div.style.overflow = "visible";
			if (div.style.zIndex != "100000") 	 	div.style.zIndex = "100000";
			if (sdiv.style.display != "block")		sdiv.style.display = "block";
			if (sdiv.style.overflow != "hidden")	sdiv.style.overflow = "hidden";
			if (sdiv.style.left != borderL + "px")	sdiv.style.left = borderL + "px";
			if (sdiv.style.top  != this.tabHeight + "px")	sdiv.style.top = this.tabHeight + "px";
			if (sdiv.style.width!= w)				sdiv.style.width = w;
			if (sdiv.style.height != h)				sdiv.style.height = h;
			sdiv.style.backgroundColor = backColor;
		}
	}
	return (tr.offsetWidth + (borderL+borderR) + cssHOffset);
}

// Implementation: Helper for creating divs used to paint pixels
HxG_5.prototype.JSFTabFrame.prototype.createDiv = function(p, t) {
	var d = document.createElement("div");
	if (t==0) {
		d.style.position = "relative";
		d.style.top = "0px";
		d.style.left = "0px";
	} else {
		d.style.position = "absolute";	
		d.style.overflow = "hidden";
	}
	d.style.backgroundColor = "transparent";
	d.style.fontSize= "0px";
	d.style.width = "1px";	d.style.height = "1px";
	d.style.borderWidth = "0px";
	d.style.padding = "0px";
	d.style.margin = "0px";			
	p.appendChild(d);
	return d;
}

// Implementation: Helper for "drawing" a line using a div to draw the line
HxG_5.prototype.JSFTabFrame.prototype.vertLine = function (sdiv, color, hOffset, vOffset, height, borderT, borderTopStyle, borderTopColor) {
	var h = ((height <= 1) ? 1 : height) + "px";
	sdiv.style.backgroundColor = color;
	sdiv.style.borderTopStyle = borderTopStyle;
	sdiv.style.borderTopColor = borderTopColor;
	sdiv.style.borderTopWidth = borderT;
	if (sdiv.style.display!= "block")		sdiv.style.display="block";
	if (sdiv.style.height != h)				sdiv.style.height = h;
	if (sdiv.style.width  != "1px")			sdiv.style.width  = "1px";
	if (sdiv.style.top    != vOffset+"px")	sdiv.style.top    = vOffset + "px";
	if (sdiv.style.left   != hOffset+"px")	sdiv.style.left	  = hOffset + "px";
}

// Private: Draw a tab as either selected or unselected
HxG_5.prototype.JSFTabFrame.prototype.drawTabHTML = function (dir, noscroll) {
	var div, tab, tabC, thisTab, state, lastVisibleTab, minVisibleTab;
	var firstVisibleTab = 0;
	this.setClassNames();
	if (this.showAtMostTabs >= 0) {
	//  If we're showing only "some" of the tabs (scrolling is enabled), then if you click "prev", and 
	//  tab you want is not on display, then we'll scroll the minimum (left) you need to get it on display.  
	//	Thus if you scroll, the tab will show up as the "last" tab (scrolling right)
	//  or "first tab" (scrolling left).
		var k, midVisibileTab, minVisibleTab, minFirstVisibleTab;
		for (firstVisibleTab = 0; firstVisibleTab < this.tabCount; firstVisibleTab++) {
			if (this.tabs[firstVisibleTab].visibleNow) break;
		}
		k = 0;
		minFirstVisibleTab = -1;
		for (minVisibleTab = 0; minVisibleTab < this.tabCount; minVisibleTab++) {
			if (this.tabs[minVisibleTab].visible) {
				minFirstVisibleTab = (minFirstVisibleTab < 0) ? minVisibleTab : minFirstVisibleTab;
				k++;
				if (k >= this.showAtMostTabs) break;
			}
		}
		k = 0;
		for (midVisibleTab = firstVisibleTab; midVisibleTab < this.tabCount; midVisibleTab++) {
			if (this.tabs[midVisibleTab].visible) {
				k++;
				if (k >= this.showAtMostTabs) break;
			}
		}
		k = 0;
		for (lastVisibleTab = this.tabCount-1; lastVisibleTab >= 0; lastVisibleTab--) {
			if (this.tabs[lastVisibleTab].visible) {
				k++;
				if (k >= this.showAtMostTabs) break;
			}
		}
		if (!noscroll) {
			if (this.activeTab < firstVisibleTab || this.activeTab > midVisibleTab) {
				firstVisibleTab = this.activeTab;
				if (firstVisibleTab < minVisibleTab)
					firstVisibleTab = minFirstVisibleTab;
				else if (firstVisibleTab > lastVisibleTab)
					firstVisibleTab = lastVisibleTab;
			}
		} else {
			k = 0;
			if (dir < 0) {
				for (firstVisibleTab = firstVisibleTab; firstVisibleTab >= 0; firstVisibleTab--) {
					if (this.tabs[firstVisibleTab].visible) {
						k++;
						if (k >= this.showAtMostTabs) break;
					}
				}
				firstVisibleTab = (firstVisibleTab < minFirstVisibleTab) ? minFirstVisibleTab : firstVisibleTab;
			} else {
				firstVisibleTab = midVisibleTab;
				firstVisibleTab = (firstVisibleTab > lastVisibleTab) ? lastVisibleTab : firstVisibleTab;
			}
		}
		
		// Go hide/show the tabs
		k = 0;
		for (thisTab = 0; thisTab < this.tabCount; thisTab++) {
			state = "";
			if (this.tabs[thisTab].visible && thisTab >= firstVisibleTab) {
				k++;
				state = (k <= this.showAtMostTabs) ? "" : "none";
			} else {
				state = "none";
			}
			this.tabs[thisTab].visibleNow = (state == "") ? true : false;
			tab = this.HTML.hdrtr.cells[thisTab];
			if (tab.style.display != state) tab.style.display = state;
		}
	}

	for (thisTab = 0; thisTab < this.tabCount; thisTab++) {
		// Hide/Show this tab and make sure it's either "active" or "inactive"
		if (!this.hideTabs && this.tabs[thisTab].visible && this.tabs[thisTab].visibleNow) {
			tab = this.HTML.hdrtr.cells[thisTab];
			tabC = this.getFirstChild(tab).rows[0].cells[0];
			if (tab && tabC) {
				// Make the tab show as active or inactive
				if (thisTab == this.activeTab) {
					if (tabC.className != this.classTabActive)
						this.styleTabHTML (this.tabs[thisTab], tab, false, this.classTabActive, this.slantActiveLeft, this.slantActiveRight, true);
				} else {
					if (tabC.className != this.classTabInactive)
						this.styleTabHTML (this.tabs[thisTab], tab, false, this.classTabInactive, this.slantInactiveLeft, this.slantInactiveRight, false);
				}
			}
		}
		// Hide/Show the divs
		div = this.tabs[thisTab].DOMobj;
		if (thisTab == this.activeTab) {
			if (div.style.display != "block")
				div.style.display = "block";
		} else {
			if (div.style.display != "none")
				div.style.display = "none";
		}
	}
	
	// Update the buttons to reflect the tabs on display
	if (this.hasButtons && !noscroll) {
		var prevTab = this.prevTab(this.activeTab);
		var nextTab = this.nextTab(this.activeTab);	
		this.HTML.btn1.disabled = (this.activeTab==prevTab) ? true : false;
		this.HTML.btn2.disabled = (this.activeTab==nextTab) ? true: false;
		if (this.HTML.btn3) {
			this.HTML.btn3.style.display = (this.tabs[this.activeTab].showFinish) ? "inline" : "none";
			this.HTML.btn3.disabled = (this.tabs[this.activeTab].enableFinish) ? false : true;
		}
		if (this.HTML.btn4) {
			this.HTML.btn4.style.display = (this.tabs[this.activeTab].showCancel) ? "inline" : "none";
			this.HTML.btn4.disabled = (this.tabs[this.activeTab].enableCancel) ? false : true;
		}
	}
	if (this.showAtMostTabs >= 0) {
		if (firstVisibleTab < minVisibleTab) {
			if (!this.HTML.hdrbtn1.disabled == true) {
				this.HTML.hdrbtn1.disabled = true;
				this.HTML.hdrbtn1.style.backgroundPosition = this.r.wellPosition(this.p.R_IMG_LG_HINKIES, 1,1);
			}
		} else {
			if (!this.HTML.hdrbtn1.disabled == false) {
				this.HTML.hdrbtn1.disabled = false;
				this.HTML.hdrbtn1.style.backgroundPosition = this.r.wellPosition(this.p.R_IMG_LG_HINKIES, 1,0);
			}
		}
		if (firstVisibleTab >= lastVisibleTab) {
			if (!this.HTML.hdrbtn2.disabled == true) {
				this.HTML.hdrbtn2.disabled = true;
				this.HTML.hdrbtn2.style.backgroundPosition = this.r.wellPosition(this.p.R_IMG_LG_HINKIES, 0,1);
			}
		} else {
			if (!this.HTML.hdrbtn2.disabled == false) {
				this.HTML.hdrbtn2.disabled = false;
				this.HTML.hdrbtn2.style.backgroundPosition = this.r.wellPosition(this.p.R_IMG_LG_HINKIES, 0,0);
			}
		}
		
	}
}


// ---------------
// JSF Tab Control
// ---------------

HxG_5.prototype.JSFTab = function () {
	this.type = "tab";
	
	this.visible = true;
	this.visibleNow = true;
	this.label = "";
	this.wrappedLabel = "";
	this.rewriteLabel = true;
	this.tdObj = null;
	this.onEnter = null;
	this.onExit = null;
	this.showFinish = false;
	this.enableFinish = true;
	this.showCancel = true;
	this.enableCancel = true;
	// this.id, this.p, this.DOMObj are set by addSubComponent
		
    // Attributes supported on this object
    this.attributeTable = new Array();
    	this.attributeTable ['label'] 				= ['label', 		null, 				false,	null,	null];
	 	this.attributeTable	['onenter-string']		= ['onEnter',		hX_5.ARG_TO_FCNS,	false,	null,	null];
	 	this.attributeTable	['onexit-string']		= ['onExit',		hX_5.ARG_TO_FCNS,	false,	null,	null];
	 	this.attributeTable	['onenter-function']	= ['onEnter',		hX_5.ARG_TO_FCN,	false,	null,	null];
	 	this.attributeTable	['onexit-function']		= ['onExit',		hX_5.ARG_TO_FCN,	false,	null,	null];
	 	this.attributeTable	['show-cancel']			= ['showCancel',	hX_5.ARG_IS_TRUE,	false,	null,	null];
	 	this.attributeTable	['enable-cancel']		= ['enableCancel',	hX_5.ARG_IS_TRUE,	false,	null,	null];
	 	this.attributeTable	['show-finish']			= ['showFinish',	hX_5.ARG_IS_TRUE,	false,	null,	null];
	 	this.attributeTable	['enable-finish']		= ['enableFinish',	hX_5.ARG_IS_TRUE,	false,	null,	null];
	this.setAttribute(arguments);
}

// Public:  Set an "attribute" on the tab.  Attributes are passed as strings of the form "name:value"
//			Attributes are "label", "onexit-function", "onexit-string", "onenter-function", "onenter-string:, "show-cancel", "show-finish"
HxG_5.prototype.JSFTab.prototype.setAttribute = function(a) {
	wasLabel = this.label;
	hX_5.parseArg (this.attributeTable, this, a);
	if (wasLabel != this.label)	this.fixupLabel();
}

// Implementation: To make a label a "hotspot" for tabbing, wrap it in a span with a tab index set
HxG_5.prototype.JSFTab.prototype.fixupLabel = function() {
	if (this.id) {
		var genid = this.id + "-HX-SPAN";
		this.wrappedLabel = "<SPAN unselectable='on' tabIndex='0' id='" + genid + "'>" + this.label + "</SPAN>";
		this.rewriteLabel = true;
	}
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.JSFTab.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}
