/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

////////////////////////////////////////////////////////////////////////////
// DATA TABLE component
////////////////////////////////////////////////////////////////////////////
//
// Controls behaviors and features added as script to a data table
// Currently, only manages vertical scrolling.  Over time, should
// coordinate the other behaviors directly added to a table.
//
///////////////////////////////////////////////////////////////////////////
// Public:  Constructor
HxG_5.prototype.JSFDataTable = function () {
	this.type = hX_5.CMP_datatable;

	// Attributes set by caller
	this.adjustColumns = false;
	this.percentageColumns = hX_5._E;

	// Implementation
	this.initialWidth=null;
	this.monitorSet=null;
	this.monitorTimer=null;

    // Attributes supported on this object
    this.attributeTable = [];
	 	this.attributeTable ['adjust-columns']		= ['adjustColumns',		hX_5.ARG_IS_TRUE,	true,	null,	null];
	this.setAttribute(arguments);
}

// Public:  Page level onLoad handler
HxG_5.prototype.JSFDataTable.prototype.onPageLoad = function () {
	// Set up handling columns with a percentage width
	this.percentageColumns = hX_5._E;
	var i, _T = 'T', _F = 'F', hasP=false;
	var table2 = hX_5.getElementById(this.DOMobj.id + "head2");
	if (table2==null) table2 = hX_5.getElementById(this.DOMobj.id + "head3");
	if (table2) {
		for (i=0; i<table2.rows[0].cells.length; i++) 
		 if (table2.rows[0].cells[i].width.indexOf(hX_5._PCT)>0) {
			 this.percentageColumns = this.percentageColumns+_T;
			 hasP = true;
		} else this.percentageColumns = this.percentageColumns+_F;
	}
	if (!hasP) this.percentageColumns = hX_5._E 
	// Firefox can misdraw a horizontal scroll bar (shouldn't have one) in some cases (it has a single pixel rounding error)
	// Unless it's specifically turned on, turn it off
	if (hX_5.brw.isNavOrMoz() && this.DOMobj.parentNode.tagName!=undefined && this.DOMobj.parentNode.tagName==hX_5._DIV && this.DOMobj.parentNode.style.overflowX!=undefined && this.DOMobj.parentNode.style.overflowX==hX_5._E) 
		this.DOMobj.parentNode.style.overflowX = hX_5._HI;
	this.redraw();
	return true;
}

// Public:  Redraw the table if something in it needs size adjustments
HxG_5.prototype.JSFDataTable.prototype.redraw = function (resizeOnly) {
	if (this.adjustColumns) this.adjustColumnWidths();
	return true;
}

HxG_5.prototype.JSFDataTable.prototype.adjustColumnWidths = function() {
	var width = hX_5.getScrollbarWidth();
	var table1 = hX_5.getElementById(this.DOMobj.id + "head1");
	var table2 = hX_5.getElementById(this.DOMobj.id + "head2");
	var table3 = hX_5.getElementById(this.DOMobj.id + "foot1");
	var table4 = hX_5.getElementById(this.DOMobj.id + "foot2");
	var i,currRow,lastCell,lastDiv,t1_width=t2_width=t3_width=t4_width=0;
	// For each table in a header or footer, set the size of the
	// "div" that represents the location of the scroll bar "under"/"over" it
	// to the size of the scroll bar.
	if(table1) {
		t1_width = hX_5.g.getRendWidth(table1);
		for(i=0; i<table1.rows.length; i++) {
			currRow = table1.rows[i];
			lastCell = currRow.cells[currRow.cells.length-1];
			lastDiv = lastCell.firstChild;
			lastDiv.style.width = width+hX_5._PX;								
			lastCell.width = width;
		}
	}

	if(table2) {
		t2_width = hX_5.g.getRendWidth(table2);
		for(i=0; i<table2.rows.length; i++) {
			currRow = table2.rows[i];
			lastCell = currRow.cells[currRow.cells.length-1];
			lastDiv = lastCell.firstChild;				
			lastDiv.style.width = width+hX_5._PX;								
			lastCell.width = width;
		}
	}
	if(table3) {
		t3_width = hX_5.g.getRendWidth(table3);
		for(i=0; i<table3.rows.length; i++) {
			currRow = table3.rows[i];
			lastCell = currRow.cells[currRow.cells.length-1];
			lastDiv = lastCell.firstChild;				
			lastDiv.style.width = width+hX_5._PX;								
			lastCell.width = width;
		}
	}
	if(table4) {
		t4_width = hX_5.g.getRendWidth(table4);
		for(i=0; i<table4.rows.length; i++) {
			currRow = table4.rows[i];
			lastCell = currRow.cells[currRow.cells.length-1];
			lastDiv = lastCell.firstChild;				
			lastDiv.style.width = width+hX_5._PX;								
			lastCell.width = width;
		}
	}

	// We usually render the footer with "layout:fixed" set in it.  This will break most footers, so disable it
	if (hX_5.brw.isIE() && table1!=null && table1.style.tableLayout!=undefined && table1.style.tableLayout=="fixed") table1.style.tableLayout=hX_5._E;
	if (hX_5.brw.isIE() && table3!=null && table3.style.tableLayout!=undefined && table3.style.tableLayout=="fixed") table3.style.tableLayout=hX_5._E;
	if (hX_5.brw.isIE() && table4!=null && table4.style.tableLayout!=undefined && table4.style.tableLayout=="fixed") table4.style.tableLayout=hX_5._E;

	// Finally, IE may (depends on version/DTD) add a scroll bar "outside" a div
	// making it wider.  We need to make it behave more like Mozilla where
	// the width of the div is the width including the scrollbar
	lastDiv = this.DOMobj.parentNode;
	if (lastDiv && lastDiv.tagName == hX_5._DIV) {
		if (this.initialWidth==null) this.initialWidth = lastDiv.style.width;
		var curW=hX_5.g.getRendWidth(lastDiv), maxW = Math.max(Math.max(t1_width,t2_width),Math.max(t3_width,t4_width));
		if (maxW<=0) maxW = parseInt(this.initialWidth,10);	
		// NOTE: this is not correct if the initial width is a percentage width and there is no header/footer OR
		// if someone changed the width and there is no header/footer
		if (maxW!=curW) {
			// if IE and strict mode...
			lastDiv.style.width=((hX_5.brw.isIE() && hX_5.brw.isW3Cstd())?(maxW-width):maxW)+hX_5._PX;
			// If this is a percentage width, we have to now track changes
			// to the head/foot and reflect them
			if (this.monitorSet==null && this.initialWidth.indexOf(hX_5._PCT)>0) {
				this.monitorSet = (table2)?table2:((table1)?table1:((table3)?table3:((table4)?table4:null)));
				if (this.monitorSet!=null) {
					hX_5.i.attachResizeMonitor(this.monitorSet, this.dispatchMonitorResize, true);
				}
			}
		}
	}

	//update the widths of any percentage columns
	if(this.percentageColumns != hX_5._E) {
		//get values to compare the percentage columns to
		if(table2 == null) table2 = table3;
		if(table2 != null) {
			var tr = (this.DOMobj.tBodies!=undefined && this.DOMobj.tBodies.length>0)?this.DOMobj.tBodies[0]:this.DOMobj;
			tr = (tr && tr.rows.length>0)?tr.rows[0]:null;
			var th = (table2 && table2.rows.length>0)?table2.rows[0]:null;
			var y, borderLeft, borderRight, paddingLeft, paddingRight, newWidth, _T = 'T';
			//we have some percentage columns
			for(y=0; th!=null && tr!=null && y<this.percentageColumns.length; y++) {

				if (this.percentageColumns.charAt(y) == _T) {
					// *** make columns same size as the headers ***
					newWidth = th.cells[y].offsetWidth;
					if(!hX_5.brw.isIE() || hX_5.brw.isW3Cstd()) {
						borderLeft = hX_5.g.parseBorder(tr.cells[y], hX_5._LFT, 1);
						borderRight = hX_5.g.parseBorder(tr.cells[y], hX_5._RGT, 1);
						paddingLeft = hX_5.g.getP(tr.cells[y], hX_5._PADL);
						if (paddingLeft.indexOf("px") > 0) paddingLeft = paddingLeft.substr(0,paddingLeft.length-2);
						else paddingLeft = hX_5.toPixels(tr.cells[y],paddingLeft);
						paddingRight = hX_5.g.getP(tr.cells[y], hX_5._PADR);
						if (paddingRight.indexOf("px") > 0) paddingRight = paddingRight.substr(0,paddingRight.length-2);
						else paddingRight = hX_5.toPixels(tr.cells[y],paddingRight);
						if(borderLeft!=null)	newWidth = newWidth - borderLeft;
						if(borderRight!=null)	newWidth = newWidth - borderRight;
						if(paddingLeft!=null)	newWidth = newWidth - paddingLeft;
						if(paddingRight!=null)	newWidth = newWidth - paddingRight;
					}
					tr.cells[y].width = newWidth;
					tr.cells[y].style.width = newWidth + hX_5._PX;
					// IE and sometimes FF in W3C mode needs to have the column head matched up as well
					if (hX_5.brw.isW3Cstd()) {
						th.cells[y].width = newWidth;
						th.cells[y].style.width = newWidth + hX_5._PX;
					}
				}
			}
			// If this is a percentage width, we have to now track changes
			// to the head/foot and reflect them
			if (this.monitorSet==null && this.initialWidth.indexOf(hX_5._PCT)>0) {
				this.monitorSet = table2;
				hX_5.i.attachResizeMonitor(this.monitorSet, this.dispatchMonitorResize, true);
			}
		}
	}
}

// Public:  Release the ui state of the spinner
HxG_5.prototype.JSFDataTable.prototype.uirelease = function (free) {
	if (this.monitorTimer) hX_5.i.killTimerTimed(this.monitorTimer);
	this.monitorTimer = null;
	if (free!=null) hX_5.uirelease(true);
	return true;
}

// Public:  Destroy -- remove event handlers and large memory objects
HxG_5.prototype.JSFDataTable.prototype.destroy = function () {
	if (this.monitorSet) hX_5.i.removeResizeMonitor(this.monitorSet);
	return true;
}

// Public:  Set a value (No-Op, currently tables have no value)
HxG_5.prototype.JSFDataTable.prototype.setValue = function (value, notify, returns) {
	var r = false;
	return r;
}

// Public:  Get a value from (No-Op, currently tables have no value)
HxG_5.prototype.JSFDataTable.prototype.getValue = function(cast) {
	return null;	
}

// Public:  Set an "attribute" 
HxG_5.prototype.JSFDataTable.prototype.setAttribute = function(a) {
	hX_5.parseArg (this.attributeTable, this, a);
	if (this.HTMLrendered) this.redraw();
	return true;
}

// Public: Retrieve the value of an attribute
HxG_5.prototype.JSFDataTable.prototype.getAttribute = function (attribute) {
	return (hX_5.getArg(this.attributeTable, this, attribute));
}

HxG_5.prototype.JSFDataTable.prototype.dispatchMonitorResize = function (evt) {
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt) {
		if (hX_5.brw.isIE()) {
			var obj, src = ((this.nodeType == hX_5.ELEMENT_NODE || this.nodeType == hX_5.DOCUMENT_NODE) ? this : this.parentNode);
			if (src) {
				obj = src.id;
				if (obj.lastIndexOf("head1")>0) obj = obj.substr(0,obj.length-5);
				else if (obj.lastIndexOf("head2")>0) obj = obj.substr(0,obj.length-5);
				else if (obj.lastIndexOf("foot1")>0) obj = obj.substr(0,obj.length-5);
				else if (obj.lastIndexOf("foot2")>0) obj = obj.substr(0,obj.length-5);
				obj = hX_5.getComponentById(obj);
				// Because resize events are issued as a user resizes, we'll reduce the number
				// we process by only processing them every 10 milliseconds
				if (hX_5.brw.isIE()) {
					if (obj.monitorTimer!=null) hX_5.i.killTimerTimed(obj.monitorTimer);
					obj.monitorTimer = hX_5.i.startObjTimerTimed(obj, "resize", 10);
				}
			}
		} else
			hX_5.redraw();
	}
}

/* Currently not needed 
// Private:  Handle an DOM event dispatched by the JSF object event handler
HxG_5.prototype.JSFDataTable.prototype.dispatchComponent = function (item, evt, action, keycode) {
// fixme: code if needed
}
*/
// Private:  Catch a timer event
HxG_5.prototype.JSFDataTable.prototype.dispatchTimer = function (item, action) {
	this.monitorTimer = null;
	this.redraw();
}

////////////////////////////////////////////////////////////////////////////
// NEW DATA TABLE BEHAVIORS TO REPLACE 512 SCRIPT - pandrews
////////////////////////////////////////////////////////////////////////////

//UTILITY :  get the id of a parent type, setting an id if none exists
HxG_5.prototype.getParentIdByType = function(id,type) {
	var parent=hX_5.getElementById(id);
	parent=parent.parentNode;
	while (parent != null) {
		if(parent.nodeName==type){
			break;
		}
		parent=parent.parentNode;
	}
	if (parent==null) {
		return null;
	}
	//assign id if not already set
	if (parent.id==null || parent.id=="") {
		parent.id=type + "_parentof_"+ id;
	}
	
	return parent.id;
}

//UTILITY : get the parent obj of certain type
HxG_5.prototype.getParentOfType = function(obj,type) {
	var parent=obj.parentNode;
	while(parent!=null){
		if(parent.nodeName==type) {
			return parent;
		}
		parent=parent.parentNode;
	}
	return null;
}

// --------------------------
// JSF data table Row Action Behavior
// --------------------------


// Public:  Constructor
HxG_5.prototype.JSFBehaviorRowAction = function () {
	this.type = hX_5.BHR_rowaction;
	
	// Attributes set by the caller
	this.hiddenInputId="";
	this.inputValue="";
	this.colClass=null;
	this.actionId="";
	this.url="";
	
	this.inputObj=null;
	this.formObj=null;
	this.trObj=null;
	
   // Attributes supported on this object
    this.attributeTable = new Array();
    	//commandRowAction
	 	this.attributeTable ['input-id']=['hiddenInputId',hX_5.ARG_TO_NAN,false,null,null];
	 	this.attributeTable ['input-value']	= ['inputValue',hX_5.ARG_TO_NAN,false,null,null];
	 	//for requestRowAction
	 	this.attributeTable ['action-id']=['actionId',hX_5.ARG_TO_NAN,false,null,null];
	 	this.attributeTable ['url']=['url',hX_5.ARG_TO_NAN,false,null,null];
	 	//not used yet, future plan to implement applying class to cells onmouseover
	 	//this.attributeTable ['col-class'] = ['colClass',hX_5.ARG_TO_NAN,false,null,null];
	 	
	this.setAttribute(arguments);
}

// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorRowAction.prototype = new hX_5.JSFBehavior();

// Private:  Override of Set Attribute
HxG_5.prototype.JSFBehaviorRowAction.prototype.setAttributeCode = function() {
	if(this.actionId){
		//requestRowAction
		this.inputObj = hX_5.getElementById(this.actionId);
		this.trObj=hX_5.getParentOfType(this.inputObj,hX_5._TR);
	} else {
		//command row action
		this.inputObj = hX_5.getElementById(this.hiddenInputId);
		this.formObj=hX_5.getParentOfType(this.inputObj, hX_5._FRM);
		this.trObj=hX_5.getParentOfType(this.inputObj,hX_5._TR);
	}
	//set cursor to pointer
	this.trObj.style.cursor="pointer";
	return true;
}

// Public: Execute the action
HxG_5.prototype.JSFBehaviorRowAction.prototype.dispatchBehavior = function(thisObj, evt) {
	if(this.actionId) { 
		// requestRowAction -- go to the request's url
	   	if (hX_5.JSFBehaviorRowActionCheck(evt)) {
			hX_5.action.processAction(this.DOMobj, evt, null, ["goto", "stop"], true, ["url('"+this.url+"')"], null, null, null);
	   	}
	   	return;
	}
	
	// commandRowAction -- submit the form
	if (this.DOMobj != null && this.inputObj != null && this.formObj != null) {
		if (hX_5.JSFBehaviorRowActionCheck(evt)) {
			//RATLC01070102 fix, set input for clicked row as enabled - then reset to disabled after submit
			this.inputObj.disabled=false;
			this.inputObj.value=this.inputValue;
			this.formObj.submit();
			this.inputObj.disabled=true;
		}
	}
	return;
}

//Method for checking if RowAction has been clicked or a link/button component inside it
//Returns true for RowAction click, false otherwise
HxG_5.prototype.JSFBehaviorRowActionCheck = function (evt) { 
	evt = (evt) ? evt : ((event) ? event: null);
	var r = true, tgt;
	if (evt) {
		tgt = hX_5.evtGetTarget(evt);
		while (r && tgt!=null && tgt.tagName!=undefined) {
			if (tgt.tagName == hX_5._TAB || tgt.tagName == hX_5._TD || tgt.tagName == hX_5._TH || tgt.tagName == hX_5._TR) {
				tgt = null;
			} else if (tgt.tagName == hX_5._ANC) {
				r = !(tgt.onclick!=null || tgt.onmousedown!=null || (tgt.href!=null && tgt.href!=hX_5._E && tgt.getAttribute("href",2)!=hX_5._LB));
				tgt = tgt.parentNode;
			} else if (tgt.tagName == hX_5._INP) {
				r = !(tgt.onclick!=null || tgt.onmousedown!=null || (tgt.type != null && tgt.type=="submit"));
				tgt = tgt.parentNode;
			} else {
				tgt = tgt.parentNode;
			}
		}
	}
	return r;
}


// --------------------------
// JSF data table Row Category Behavior
// --------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorRowCategory=function () {
this.type=hX_5.BHR_rowcategory;

	 this.catId="";
	 this.expImgSrc="";
	 this.colImgSrc="";
	 this.firstRowNum="";
	 this.lastRowNum="";
	 this.componentId="";
	 this.initImg="";
	 this.expImg=null;
	 this.colImg=null;
	 this.attributeTable=new Array();
	 this.attributeTable ['cat-id']=['catId',hX_5.ARG_TO_NAN,false,null,null];
	 this.attributeTable ['twistie-exp-img']=['expImgSrc',hX_5.ARG_TO_NAN,false,null,null];
	 this.attributeTable ['twistie-col-img']=['colImgSrc',hX_5.ARG_TO_NAN,false,null,null];
	 this.attributeTable ['first-row-num']=['firstRowNum',hX_5.ARG_TO_NAN,false,null,null];
	 this.attributeTable ['last-row-num']=['lastRowNum',hX_5.ARG_TO_NAN,false,null,null];
	 this.attributeTable ['component-id']=['componentId',hX_5.ARG_TO_NAN,false,null,null];
	 this.attributeTable ['init-img']=['initImg',hX_5.ARG_TO_NAN,false,null,null];
	 this.setAttribute(arguments);
}
HxG_5.prototype.JSFBehaviorRowCategory.prototype=new hX_5.JSFBehavior();

HxG_5.prototype.JSFBehaviorRowCategory.prototype.IMAGE_ARRAY=new Array();

HxG_5.prototype.JSFBehaviorRowCategory.prototype.setAttributeCode=function() {

		 //use default images if not set
		 if(!this.expImgSrc){
		 	this.expImgSrc = hX_5.currentResource().baseImgPath + 'datagrid_category_expanded_v3_0_4.gif';
		 }
		 if(!this.colImgSrc){
		 	this.colImgSrc = hX_5.currentResource().baseImgPath + 'datagrid_category_collapsed_v3_0_4.gif';
		 }

		 for(var i=0;i < this.IMAGE_ARRAY.length;i=i+2) {
		 		 if(this.IMAGE_ARRAY[i] == this.expImgSrc){
		 		 		 this.expImg=this.IMAGE_ARRAY[i+1];
		 		 }
		 		 if(this.IMAGE_ARRAY[i] == this.colImgSrc){
		 		 		 this.colImg=this.IMAGE_ARRAY[i+1];
		 		 }
		 }
		 if(this.expImg==null){
		 		 this.expImg=new Image();
		 		 this.expImg.src=this.expImgSrc;
		 		 this.IMAGE_ARRAY[this.IMAGE_ARRAY.length]=this.expImgSrc;
		 		 this.IMAGE_ARRAY[this.IMAGE_ARRAY.length]=this.expImg;
		 }
		 if(this.colImg==null){
		 		 this.colImg=new Image();
		 		 this.colImg.src=this.colImgSrc;
		 		 this.IMAGE_ARRAY[this.IMAGE_ARRAY.length]=this.colImgSrc;
		 		 this.IMAGE_ARRAY[this.IMAGE_ARRAY.length]=this.colImg;
		 }
		 if(this.initImg){
		 		 //initial twistie images
		 		 var twistie=hX_5.getElementById(this.catId+'img');
		 		 if(this.initImg == 'exp'){
		 		 	twistie.src=this.expImg.src;
		 		 } else {
		 		 	twistie.src=this.colImg.src;
		 		 } 
		 }
		 return true;
}

HxG_5.prototype.JSFBehaviorRowCategory.prototype.setEventCode=function() {
	hX_5.bodyLoadRedrawB = true;
	return true;
}

HxG_5.prototype.JSFBehaviorRowCategory.prototype.dispatchBehavior=function(thisObj, evt) {

		 var hideId = this.componentId + "_hide_";
		 
		 this.processRows(this.catId);
		 for(var cat_hide_rows=0;cat_hide_rows <= this.lastRowNum;cat_hide_rows++) {
			hide_row = hX_5.getElementById(hideId+cat_hide_rows+'');
	 		if(hide_row!=null) hide_row.parentNode.style.display='none';
		 }
		 hX_5.redraw();		 
		 return;
}

HxG_5.prototype.JSFBehaviorRowCategory.prototype.processRows=function(currentRow) {
		 var rowNum=0,row=null,cell=null,twistieId=currentRow+'img',cellId="";
		 var twistie=hX_5.getElementById(twistieId);
		 var collapsed = false;
		 if (twistie.src == this.colImg.src){
		 		 collapsed = true;
		 		 twistie.src=this.expImg.src;
		 }else{
		 		 twistie.src=this.colImg.src;
		 }

		 var u=0;
		 while(true) {
		 		 cellId=currentRow+rowNum;
		 		 if ((cell=hX_5.getElementById(cellId))!=null){
		 		 		 row = cell.parentNode;
		 		 		 this.changeRowState(row, collapsed, false);
		 		 		 //cell ids are appended with _e to save its expanded state
		 		 } else if ((cell=hX_5.getElementById(cellId + '_e'))!=null){
		 		 		 		 row = cell.parentNode;
		 		 		 		 this.changeRowState(row, collapsed, true);
		 		 } else {
		 		 		 break;
		 		 }
		 		 rowNum++;
		 }
		 return;
}

HxG_5.prototype.JSFBehaviorRowCategory.prototype.changeRowState=function(row, collapsed, subCategoryRow) {
		 if(collapsed == true){

		 		 //check siblings for a td with _hide_ in it
		 		 var children = row.childNodes;
		 		 var isHiddenRow = false;
		 		 for (var i = 0; i < children.length; i++){
		 		 		 var cell = children[i];
		 		 		 this.restoreStateOfSubCategories(cell);
		 		 		 var cellId = cell.id;
		 		 		 if (cellId != null){
		 		 		 		 if (cellId.indexOf("_hide_") > 0){
		 		 		 		 		 isHiddenRow = true;
		 		 		 		 		 break;
		 		 		 		 }
		 		 		 }
		 		 }
		 		 if (isHiddenRow == false && subCategoryRow == false){
		 		 
		 		 		 row.style.display='';
		 		 }
		 } else {
		 		 row.style.display='none';
		 }
}

HxG_5.prototype.JSFBehaviorRowCategory.prototype.restoreStateOfSubCategories=function(cell) {
		 if (cell != null && cell.tagName == "TD"){
		 		 //look for the button (we find a button image we can tell the state of the rows controled by that button)
		 		 var cellChildren = cell.childNodes;
		 		 for (var j = 0; j < cellChildren.length; j++){ 
		 		 		 var cellChild = cellChildren[j];
		 		 		 if (cellChild.tagName == HxG_5.prototype._BTN){
		 		 		 		 var buttonId = cellChild.id;
		 		 		 		 var buttonCategoryId = buttonId.substring(0, buttonId.indexOf("button"));
		 		 		 		 var twistie=hX_5.getElementById(buttonCategoryId+"img");
		 		 		 		 		 
		 		 		 		 if (twistie.src == this.expImg.src){
		 		 		 		 		 var subCellId;
		 		 		 		 		 var rowNum = 0;
		 		 		 		 		 while(true) {
		 		 		 		 		 		 subCellId=buttonCategoryId+rowNum;
		 		 		 		 		 		 if ((subCell=hX_5.getElementById(subCellId))!=null){
		 		 		 		 		 		 		 subCell.parentNode.style.display='';
		 		 		 		 		 		 }else{
		 		 		 		 		 		 		 break;
		 		 		 		 		 		 }
		 		 		 		 		 		 rowNum++;
		 		 		 		 		 }
		 		 		 		 }
		 		 		 }
		 		 }
		 }
}

// ---------------------------------------------------------------------------------------
// JSF data table Row Edit Show/Hide Behavior
// NOTE:  There are THREE behaviors for each "edit" button.
//		  One attached to the edit button, one to the "save" button
//	 	  and one attached to the "cancel" button.  They need to intercommunicate
//		  which they do by looking up the "other" related behaviors based on 
//		  the three buttons all having the same "id" with different "suffixes"
// ---------------------------------------------------------------------------------------

// Public:  Constructor
// This behavior is used on both the edit button and the cancel button (but there are two instances of it)
HxG_5.prototype.JSFBehaviorRowEditShowHide = function () {
	this.type = hX_5.BHR_roweditshowhide;
	
	// Attributes set by the caller
	this.divId=hX_5._E;
	this.show=hX_5._E;
	// Implementation
	this.divObj=null;
	this.trObj=null;
	this.cloneObj=null;	// Only exists on the "cancel" behavior
	
   // Attributes supported on this object
    this.attributeTable = new Array();
	 	this.attributeTable ['div-id']=['divId',hX_5.ARG_TO_NAN,false,null,null];
	 	this.attributeTable ['show']=['show',hX_5.ARG_TO_NAN,false,null,null];
	this.setAttribute(arguments);
}

// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype = new hX_5.JSFBehavior();

// Private:  Override of Set Attribute
HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.setAttributeCode = function() {
	this.divObj=hX_5.getElementById(this.divId);
	this.trObj=hX_5.getParentOfType(this.divObj,hX_5._TR);
	return true;
}

// Private:  destructor
HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.destroy = function() {
	if (this.cloneObj!=null) delete this.cloneObj;
	this.cloneObj = null;
	return true;
}

// Public: Execute the show/hide event
HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.dispatchBehavior = function(thisObj, evt) {
	if (this.divObj==null || !this.divObj.style || this.trObj==null) return;

	// Hiding the dialog (cancel clicked)
	if(this.show.toLowerCase()==hX_5._FLS) {
		//i.e. cancel - restore state to initial render
		this.restoreState(this.cloneObj, this.divObj);
		this.divObj.style.visibility = hX_5._HI;
		HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.LAST_SHOWN = null;
		HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.LAST_STATE = null;

	// Showing the dialog (edit clicked)
	} else {
		// If we have something on display, take it off display
		if(HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.LAST_SHOWN!=null && HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.LAST_SHOWN.style.visibility!=hX_5._HI) {
			var div = HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.LAST_SHOWN;
			var state = HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.LAST_STATE;
			this.restoreState(state, div);
			div.style.visibility = hX_5._HI;
		}
		// Showing requires saving off the state on the CANCEL behavior
		var o=null, s=hX_5._E, i=this.DOMobj.id.lastIndexOf("__editCtrl");
		if (i>=0) {
			s = this.DOMobj.id.substr(0,i)+"__cancelCtrl";
			o=hX_5.getBehaviorById(s,hX_5.BHR_roweditshowhide,"click");
		}
		if (o) {
			o.saveState();		
			HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.LAST_SHOWN=o.divObj;
			HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.LAST_STATE=o.cloneObj;
		}

		// Position the div and make it visible
		var pageT = Math.max(document.body.scrollTop,document.body.parentNode.scrollTop), 
		    pageL = Math.max(document.body.scrollLeft,document.body.parentNode.scrollLeft),
		    pageW = hX_5.g.getWindowClientWidth(),
		    pageH = hX_5.g.getWindowClientHeight(),
			rowW = hX_5.g.getRendWidth(this.trObj), rowH = hX_5.g.getRendHeight(this.trObj), 
		    rowL = hX_5.g.getAbsPosLeft(this.trObj), rowT = hX_5.g.getAbsPosTop(this.trObj), 
		    spanW = hX_5.g.getRendWidth(this.divObj), spanH = hX_5.g.getRendHeight(this.divObj);
		var spanL = rowL + (rowW/2) - (spanW/2);
		if (pageW>0 && spanW>0 && ((spanL+spanW) > (pageL+pageW)))
			spanL = (pageL+pageW)-spanW;
		var spanT = (rowT + (rowH/2)) - (spanH/2);
		if (pageH>0 && spanH>0 && ((spanT+spanH) > (pageT+pageH)))
			spanT = (pageT+pageH)-spanH;
		if(spanL<0) spanL = 0;
		if(spanT<0) spanT = 0;
		var a,f,c,pY = (this.divObj.offsetParent)?hX_5.g.getAbsPosTop(this.divObj.offsetParent):0, pX = (this.divObj.offsetParent)?hX_5.g.getAbsPosLeft(this.divObj.offsetParent):0;
		this.divObj.style.left = (spanL-pX) + hX_5._PX;
		this.divObj.style.top = (spanT-pY) + hX_5._PX;

		// The following should probably be added, though it doesn't seem to work??
		this.divObj.style.zIndex = (hX_5.BIG_ZINDEX-100)+hX_5._E;

		// Redraw needed as component values could have been "mucked with" either by a restorestate or as a consequence of going visible
		this.divObj.style.visibility = hX_5._VS;
		a = hX_5.getComponentsInId (this.divObj);
		if (a!=null) {
			for (i=0;i<a.length;i++) {
				c = hX_5.getComponentById(a[i]);
				if (c!=null) c.redraw();
			}
			delete a;
		}
		
		// Now set the focus to the first field in the div that can receive focus
		if (hX_5.tabNav == null) hX_5.tabNav = new hX_5.tabNavigator();
		f = hX_5.tabNav.getFirst(this.divObj);
		if (f) hX_5.i.setFocus(f, false, true);
	}
}
HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.LAST_SHOWN = null;
HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.LAST_STATE = null;

// Private: Helper that saves the state of the dialog before it's shown (so it can be undone)
HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.saveState = function() {
	// We should use the code in hxclient_o to determine save/restore the state of the 
	// objects in the row-edit div.  However, this code will be really really slow in this case
	// as there are many many "tabbable objects" in the page (each row-edit has it's own copy
	// of the "row-edit dialog" so for each row there are "n" objects).  The hxclient code
	// is "front-loaded" (it does the work when something is going to be displayed).
	// We use custom code here thats "back-loaded" -- it only has to do a lot of work if
	// the state needs to be restored
	if (this.cloneObj!=null) {
		delete this.cloneObj;
		this.cloneObj = null;
	}
	this.cloneObj = this.divObj.cloneNode(true);	// Clone the div (fairly fast)
	// Cloning a node doesn't correctly clone "selection" within a <select> tag (at least in IE)
	this.tidyClone(this.divObj, this.cloneObj);
}
	
// Private: Helper that restores the state of the dialog before it's hidden (does a "cancel")
HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.restoreState = function(objFrom, objTo) {
	var i,c;
	// Check that nothing bad has happened	
	if(objFrom.nodeName != objTo.nodeName || objFrom.childNodes.length != objTo.childNodes.length) return;
	// If this is a "valued" entity
	if (objFrom.value!=null) {
		// Select tags are odd (see above)
		if (objTo.tagName==hX_5._SEL) {
			if (objTo.options.length==objFrom.options.length) {
				for (i=0; i<objTo.options.length; i++)
					if (objTo.options[i].selected!=objFrom.options[i].selected) objTo.options[i].selected = objFrom.options[i].selected;
			}
			objTo.selectedIndex=objFrom.selectedIndex;
			objTo.value=objFrom.value;
		// Don't do "options" for all other valued items -- we did them above
		} else if (objTo.tagName!=hX_5._OPT) {
			if (objTo.value!=objFrom.value) objTo.value = objFrom.value;
			// Make sure to copy the "checked" state of a radio button or checkbox
			if (objTo.checked!=undefined && objTo.checked!=objFrom.checked) objTo.checked = objFrom.checked;
		}
	}
	// Traverse
	for(i=0; i<objFrom.childNodes.length; i++) this.restoreState(objFrom.childNodes[i], objTo.childNodes[i]);
}

// Private:  Helper for the above
HxG_5.prototype.JSFBehaviorRowEditShowHide.prototype.tidyClone = function(div, clone) {
	var dobj, cobj, i, j;
	if (div!=null && clone!=null) {
		if (div.tagName && clone.tagName && div.tagName==hX_5._SEL && clone.tagName==hX_5._SEL) {
			for (i=0; i<div.options.length; i++) clone.options.selected = div.options.selected;
			clone.value = div.value;
			clone.selectedIndex = div.selectedIndex;
		}
		dobj = div.firstChild; cobj = clone.firstChild;
		while (dobj!=null) {
			this.tidyClone(dobj,cobj);
			cobj = cobj.nextSibling;
			dobj = dobj.nextSibling;
		}
	}
}

// ----------------------------------------
// JSF data table Row Edit Save Behavior
// ----------------------------------------

// Public:  Constructor
HxG_5.prototype.JSFBehaviorRowEditSave = function () {
	this.type = hX_5.BHR_roweditsave;

	// Attributes set by the caller
	this.divId=hX_5._E;
	this.tableId=hX_5._E;
	this.hiddenId=hX_5._E;

	this.divObj=null;
	this.tableObj=null;
	this.formObj=null;
	this.hiddenObj=null;
	
   // Attributes supported on this object
    this.attributeTable = new Array();
	 	this.attributeTable ['div-id']=['divId',hX_5.ARG_TO_NAN,false,null,null];
	 	this.attributeTable ['table-id']=['tableId',hX_5.ARG_TO_NAN,false,null,null];
	 	this.attributeTable ['hidden-id']=['hiddenId',hX_5.ARG_TO_NAN,false,null,null];
	this.setAttribute(arguments);
}

// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorRowEditSave.prototype = new hX_5.JSFBehavior();

// Private:  Override of Set Attribute
HxG_5.prototype.JSFBehaviorRowEditSave.prototype.setAttributeCode = function() {
	this.divObj=hX_5.getElementById(this.divId);
	this.tableObj=hX_5.getElementById(this.tableId);
	this.formObj=hX_5.getParentOfType(this.tableObj,hX_5._FRM);
	this.hiddenObj=hX_5.getElementById(this.hiddenId);
	return true;
}

// Public: Execute the Category
HxG_5.prototype.JSFBehaviorRowEditSave.prototype.dispatchBehavior = function(thisObj, evt) {

	if(this.formObj==null) return;

	this.hiddenObj.value = "clicked";

	//submit the form
	this.formObj.submit();
}

// --------------------------
// JSF Web Pager Submit function
// --------------------------

/*RATLC00296089 
There are issues in some combinations of browser/server with the pager behavior where intermittently
 page links clicks do not function e.g IE + WPS5.1.0.3 - restoring back to previous method of adding 
 false returning onclick event to the pager links, this resolves the problem.
*/
HxG_5.prototype.JSFWebPagerSubmit = function(strId, nIndex, strFormId)
{
	if(strId==null) return;
	if(nIndex==null) return;
	if(strFormId==null) return;

	var objField = null;
	var objForm = null;
	var cur = null;

	//first, try the W3 way if available
	if(document.getElementById)
	{
		objField = document.getElementById(strId);
		objForm = document.getElementById(strFormId);
	}

	//if W3 way fails, then try the old fashioned way
	if((objField==null)||(objForm==null))
	{
		if((!document.forms[strFormId])||(!document.forms[strFormId].strId))
		{
			//then we can do no more
			return;
		}
		objField = document.forms[strFormId].strId;
		objForm = document.forms[strFormId];
	}

	if(objField != null && objForm != null) {
		cur = objField.value;
		objField.value = nIndex;
		
		//fire onSubmit event to save state of ODC components on the page
		hX_5.i.fireEvent(objForm, "onSubmit")
		
		objForm.submit();
		objField.value = cur;
	}
}

/*
// --------------------------
// JSF data table Pager Behavior
// --------------------------


// Public:  Constructor
HxG_5.prototype.JSFBehaviorPager = function () {
	this.type = "tablepager";
	
	// Attributes set by the caller
	this.inputId="";
	this.index="";
	this.formId="";
	this.action="";
	
	
	this.formObj=null;
	this.inputObj=null;
	
   // Attributes supported on this object
    this.attributeTable = new Array();
	 	this.attributeTable ['input-id']=['inputId',hX_5.ARG_TO_NAN,false,null,null];
	 	this.attributeTable ['index']=['index',hX_5.ARG_TO_NAN,false,null,null];
	 	this.attributeTable ['form-id']=['formId',hX_5.ARG_TO_NAN,false,null,null];
	 	this.attributeTable ['action']=['action',hX_5.ARG_TO_NAN,false,null,null];
	this.setAttribute(arguments);
}

// Inherit: Add in the superclass definition
HxG_5.prototype.JSFBehaviorPager.prototype = new hX_5.JSFBehavior();

// Private:  Override of Set Attribute
HxG_5.prototype.JSFBehaviorPager.prototype.setAttributeCode = function() {
	this.inputObj=hX_5.getElementById(this.inputId);
	this.formObj=hX_5.getElementById(this.formId);
	return true;
}

// Public: Execute the Category
HxG_5.prototype.JSFBehaviorPager.prototype.dispatchBehavior = function(thisObj, evt) {
	if(this.formObj==null || this.inputObj==null){
		return;
	}
	if(this.action=="submit"){
		this.submit();
	}
}

HxG_5.prototype.JSFBehaviorPager.prototype.submit= function(){
	
	//RATLC01071183 fix : get the current page, revert to this value after a pager submit in case back button is used
	var curPage = this.inputObj.value;
	this.inputObj.value=this.index;
	
	//fire onSubmit event to save state of ODC components on the page
	hX_5.i.fireEvent(this.formObj, "onSubmit")
	
	this.formObj.submit();
	this.inputObj.value = curPage;
	
}
*/

////////////////////////////////////////////////////////////////////////////
// DATAGRID CLASSES - START - A McDowell
////////////////////////////////////////////////////////////////////////////

////class HxSelection////

/**
*Constructor.
*Class for holding selection information and notifying selection listeners of events.
*/
HxG_5.prototype.HxSelection = function(bSelected)
{
	this.bSelected = bSelected;

	this.arrSelectionListeners = new Array();
}

/**
*Set selected true/false
*/
HxG_5.prototype.HxSelection.prototype.setSelected = function (bSelected)
{
	if(this.bSelected==bSelected)
	{
		return;
	}

	this.bSelected = bSelected;

	//notify listeners of change
	this.fireSelectionListeners();
}

/**
*Set selected true/false
*/
HxG_5.prototype.HxSelection.prototype.isSelected = function ()
{
	return this.bSelected;
}

/**
*Add a listener
*Must provide a method selectionChanged(objHxSelection)
*/
HxG_5.prototype.HxSelection.prototype.addSelectionListener = function (objListener)
{
	if(!objListener)
	{
		alert("RUNTIME ERROR: null SelectionListener added to HxSelection");
	}
	if(!objListener.selectionChanged)
	{
		alert("RUNTIME ERROR: SelectionListener does not proide selectionChanged method");
	}
	
	this.arrSelectionListeners[this.arrSelectionListeners.length] = objListener;
}

/**
*Fires any listeners listening to object changes
*/
HxG_5.prototype.HxSelection.prototype.fireSelectionListeners = function ()
{
	var len = this.arrSelectionListeners.length;
	for(var i=0; i<len; i++)
	{
		this.arrSelectionListeners[i].selectionChanged(this);
	}
}

////class HxStatisticsListener////

/**
*Constructor
*Class for listening to a HxSelection.
*Will update div tag contents with statistic info.
*/
HxG_5.prototype.HxStatisticsListener = function(strDivId, nSelCount)
{
	if(!hX_5.getElementById)
	{
		//then don't attempt to make live
		return;
	}

	this.nSelCount = nSelCount;

	var objDiv = hX_5.getElementById(strDivId);
	if(!objDiv)
	{
		alert("RUNTIME ERROR: "+strDivId+" not found.");
		return;
	}

	this.objText = objDiv.firstChild;
}

HxG_5.prototype.HxStatisticsListener.prototype.selectionChanged = function (objHxSelection)
{
	if(objHxSelection.isSelected())
	{
		this.nSelCount++;
	}
	else
	{
		this.nSelCount--;
	}

	this.objText.data = this.nSelCount;
}

////class HxCheckboxListener////

/**
*Constructor.
*Class for listening to a check box.
*Will notify the HxSelection object of changes.
*Will add itself as a listener for changes in the HxSelection; DO NOT REGISTER AS LISTENER MANUALLY.
*/
HxG_5.prototype.HxCheckboxListener = function(strCheckboxId, objSelection)
{
	if(!hX_5.getElementById)
	{
		//then this isn't going to work
		return;
	}

	this.objCheckbox = hX_5.getElementById(strCheckboxId);
	if(!this.objCheckbox)
	{
		alert("RUNTIME ERROR: checkbox "+strCheckboxId+" cannot be found");
		return;
	}

	this.objSelection = objSelection;

	this.bindToDom();
	
	//remember the selection object + add this as a Listener
	this.objSelection = objSelection;
	this.objSelection.addSelectionListener(this);
}

HxG_5.prototype.HxCheckboxListener.prototype.bindToDom = function ()
{
	var thisObj = this;
	this.objCheckbox.onclick = function()
	{
		thisObj.onclick();
	}
}

HxG_5.prototype.HxCheckboxListener.prototype.isSelected = function ()
{
	return this.objCheckbox.checked;
}

HxG_5.prototype.HxCheckboxListener.prototype.setSelected = function (bSelected)
{
	this.objCheckbox.checked = bSelected;
}

HxG_5.prototype.HxCheckboxListener.prototype.onclick = function ()
{
	this.objSelection.setSelected(this.isSelected());
}

HxG_5.prototype.HxCheckboxListener.prototype.selectionChanged = function (objHxSelection)
{
	if(objHxSelection.isSelected() != this.isSelected())
	{
		this.setSelected( objHxSelection.isSelected() );
	}
}

////class HxStyleListener////

/**
*Class for listening to a HxSelection + changing CSS styles according to changes
*/
/*HxG_5.prototype.HxStyleListener = function(strElementId, strUnselectedCss, strSelectedCss)
{
	if(!hX_5.getElementById)
	{
		return;
	}

	this.objElement = hX_5.getElementById(strElementId);
	if(!this.objElement)
	{
		alert("RUNTIME ERROR: element id="+strElementId+" cannot be found");
	}
	this.strUnselectedCss = strUnselectedCss;
	this.strSelectedCss = strSelectedCss;
}

HxG_5.prototype.HxStyleListener.prototype.setStyle = function (strStyleCss)
{
	this.objElement.className = strStyleCss;
}

HxG_5.prototype.HxStyleListener.prototype.selectionChanged = function (objHxSelection)
{
	if(objHxSelection.isSelected())
	{
		this.setStyle( this.strSelectedCss );
	}
	else
	{
		this.setStyle( this.strUnselectedCss );
	}
}*/

//////////class HxStyleRegister//////////////////

/**
*Adds stylesheet listeners to a row.
*varSelection		associated selection object
*strTableId			the id of the table
*strSpanId			the id associated with the select control
*strStyles			comma-delimited list of styles
*/
HxG_5.prototype.HxStyleRegister = function(varSelection, strTableId, strSpanId, strStyles)
{
	if(!hX_5.getElementById) {
		return;
	}
	
	this.objTable = hX_5.getElementById(strTableId);
	if(!this.objTable) {
		alert("RUNTIME ERROR: table not found: "+strTableId);
	}
	this.objSpan = hX_5.getElementById(strSpanId);
	if(!this.objSpan) {
		alert("RUNTIME ERROR: select element not found: "+strSpanId);
	}
	
	//this.styleIterator = new HxG_5.prototype.HxStyleIterator(strStyles);
	//don't know why its not instantiated using the hX_1 object, leaving it in comments in of problem - pandrews
	this.styleIterator = new hX_5.HxStyleIterator(strStyles);
	
	
	var cellArr = this.getCells();
	if(!cellArr) {
		return;
	}
	//document.write("<br>cells found: "+cellArr.length);
	for(var i=0; i<cellArr.length; i++) {
		//document.write("<br>"+cellArr[i].nodeName);
		var td = cellArr[i];
		var style = this.styleIterator.next();
		//var listener = new HxG_5.prototype.HxStyleListener(td, style);
		//don't know why its not instantiated using the hX_1 object, leaving it in comments in of problem - pandrews
		var listener = new hX_5.HxStyleListener(td, style);
		listener.selectionChanged(varSelection);
		varSelection.addSelectionListener(listener);
	}
}

/**Finds all the TD elements in the given row*/
HxG_5.prototype.HxStyleRegister.prototype.getCells = function()
{
	var tableBody = this.getChildOfType(this.objTable, hX_5._TBDY);
	if(!tableBody) {
		return null;
	}
	var trArr = this.getChildrenOfType(tableBody, hX_5._TR);
	for(var i=0; i<trArr.length; i++) {
		if( this.isDescendant(trArr[i], this.objSpan) ) {
			return this.getChildrenOfTypes(trArr[i], hX_5._TD, hX_5._TH);
		}
	}
	return null;
}

HxG_5.prototype.HxStyleRegister.prototype.getChildOfType = function(objParent, strNodeName)
{
	var objArr = this.getChildrenOfType(objParent, strNodeName);
	if(objArr.length<1) {
		return null;
	}
	return objArr[0];
}

HxG_5.prototype.HxStyleRegister.prototype.getChildrenOfType = function(objParent, strNodeName)
{
	var objArr = new Array();
	for(var i=0; i<objParent.childNodes.length; i++) {
		if(objParent.childNodes[i].nodeName == strNodeName) {
			objArr[objArr.length] = objParent.childNodes[i];
		}
	}
	return objArr;
}

HxG_5.prototype.HxStyleRegister.prototype.getChildrenOfTypes = function(objParent, strNodeName, strNodeName2)
{
	var objArr = new Array();
	for(var i=0; i<objParent.childNodes.length; i++) {
		if(objParent.childNodes[i].nodeName == strNodeName || objParent.childNodes[i].nodeName == strNodeName2) {
			objArr[objArr.length] = objParent.childNodes[i];
		}
	}
	return objArr;
}

/**Determines if an element is a parent of another element*/
HxG_5.prototype.HxStyleRegister.prototype.isDescendant = function(objParent, objChild)
{
	var n = objChild;
	while(true) {
		n = n.parentNode;
		if(!n) {
			return false;
		}
		if(n == objParent) {
			return true;
		}
	}
	return false;
}

///////////class HxStyleListener////////////////////

HxG_5.prototype.HxStyleListener = function(objTd, strStyle)
{
	this.objTd = objTd;
	this.strUnselectedCss = objTd.className;
	this.strSelectedCss = strStyle;
	
	//document.write("<br>node: "+objTd.nodeName+" u:"+this.strUnselectedCss+" s:"+this.strSelectedCss);
}

HxG_5.prototype.HxStyleListener.prototype.selectionChanged = function (objHxSelection)
{
	if(objHxSelection.isSelected())
	{
		this.objTd.className = this.strSelectedCss;
	}
	else
	{
		this.objTd.className = this.strUnselectedCss;
	}
}

///////////class HxStyleIterator//////////////////

HxG_5.prototype.HxStyleIterator = function(strStyles)
{
	this.styleList = new Array();
	this.index = 0;

	var str = new String(strStyles);
	var i, subs, n = 0;
	while(true)
	{
		i = str.indexOf(",", n);
		if(i<0) {
			subs = str.substring(n, str.length);
			this.styleList[this.styleList.length] = subs;
			break;
		}
		else {
			subs = str.substring(n, i);
			this.styleList[this.styleList.length] = subs;
			n = i+1;
		}
	}
}

HxG_5.prototype.HxStyleIterator.prototype.next = function()
{
	if(this.index >= this.styleList.length) {
		this.index = 0;
	}
	
	return this.styleList[this.index++];
}

////class HxCheckImageListener////

HxG_5.prototype.HxCheckImageListener = function(strImgId, strUnselectedUrl, strSelectedUrl, strUnselected, strSelected)
{
	if(!hX_5.getElementById)
	{
		return;
	}

	this.objImg = hX_5.getElementById(strImgId);
	if(this.objImg==null)
	{
		alert("RUNTIME ERROR: cannot find item to set background on: "+strImgId);
	}
	this.strUnselectedUrl = strUnselectedUrl;
	this.strSelectedUrl = strSelectedUrl;
	this.strUnselected = strUnselected;
	this.strSelected = strSelected;
}

HxG_5.prototype.HxCheckImageListener.prototype.selectionChanged = function (objHxSelection)
{
	if(objHxSelection.isSelected())
	{
		//this.objImg.style.backgroundImage = "url(\""+this.strSelectedUrl+"\")";
		this.objImg.src = this.strSelectedUrl;
		this.objImg.alt = this.strSelected;
		this.objImg.title = this.strSelected;
	}
	else
	{
		//this.objImg.style.backgroundImage = "url(\""+this.strUnselectedUrl+"\")";
		this.objImg.src = this.strUnselectedUrl;
		this.objImg.alt = this.strUnselected;
		this.objImg.title = this.strUnselected;		
	}
}

////class HxCheckCellListener/////

/**
*Class for handling Notes-style select (drag-select).
*/
HxG_5.prototype.HxCheckCellListener = function(strSpanId, strInputId, objGroup, objHxSelection){
	if(!hX_5.getElementById)
	{
		return;
	}

	this.objTd = hX_5.getElementById(strSpanId);
	if(!this.objTd)
	{
		alert("RUNTIME ERROR: SPAN element "+strSpanId+" cannot be found");
		return;
	}
	this.objTd = this.getParentOfTypeCell(this.objTd);
	
	if(!this.objTd)
	{
		alert("RUNTIME ERROR: TD element cannot be found");
		return;
	}
	
	this.objInput = hX_5.getElementById(strInputId);
	if(!this.objInput)
	{
		alert("RUNTIME ERROR: INPUT element "+strInputId+" cannot be found");
		return;
	}
	
	this.objGroup = objGroup;
	this.objHxSelection = objHxSelection;

	this.bindToDom();

	this.objGroup.addMember(this);
}

/**
*Binds to DOM object.
*/
HxG_5.prototype.HxCheckCellListener.prototype.bindToDom = function()
{
	var thisObj = this;
	this.objTd.onmousedown = function (evt)
	{
		if(!evt)
			evt = window.event;
		thisObj.onMouseDown(evt);

		return false;
	}

	this.objTd.onclick = function (evt)
	{
		return false;
	}

	this.objTd.onkeypress = function (evt)
	{
		if(!evt)
			evt = window.event;
		thisObj.onKey(evt);
		//return false;
	}
}

/**
*User pressed button on cell.
*/
HxG_5.prototype.HxCheckCellListener.prototype.onMouseDown = function(evt)
{
	this.objHxSelection.setSelected( !this.objHxSelection.isSelected() );

	this.setSelected(this.objHxSelection.isSelected());
	this.objGroup.fireMouseDown(evt, this.objHxSelection.isSelected());
}

/**
*User pressed key.
*/
HxG_5.prototype.HxCheckCellListener.prototype.onKey = function(evt)
{
	var nKey = evt.which | evt.keyCode;
	if(nKey!=32) //space
	{
		return;
	}
	this.objHxSelection.setSelected( !this.objHxSelection.isSelected() );
	this.setSelected(this.objHxSelection.isSelected());
}

/**
*Set selected/unselected on click (toggle).
*/
HxG_5.prototype.HxCheckCellListener.prototype.setSelected = function(bSelected)
{
	//reset all values if this if group set to radio
	if(bSelected && this.objGroup.isRadio){
		//clear all selections
		for(x in this.objGroup.arrayMembers){
			this.objGroup.arrayMembers[x].objHxSelection.setSelected(false);
			this.objGroup.arrayMembers[x].objInput.value = false;
		}
	}
	this.objHxSelection.setSelected(bSelected);
	this.objInput.value = bSelected;
}

/**
*Is selected?
*/
HxG_5.prototype.HxCheckCellListener.prototype.isSelected = function()
{
	return this.objHxSelection.isSelected();
}

/**
*Is the mouse inside this element?
*/
HxG_5.prototype.HxCheckCellListener.prototype.isInElementBoundary = function(pt)
{
	var y=hX_5.g.getAbsPosTop(this.objTd), x=hX_5.g.getAbsPosLeft(this.objTd);
	if(pt.X<x || pt.Y<y) return false;

	y += this.objTd.offsetHeight; x += this.objTd.offsetWidth;
	if (pt.Y>y || pt.X>x) return false;

	return true;
}

HxG_5.prototype.HxCheckCellListener.prototype.getParentOfType = function(node, strType)
{
	if((node == null) || (strType==null))
	{
		return null;
	}

	var parent = node.parentNode;
	while(parent!=null)
	{
		if(parent.nodeName==strType)
		{
			return parent;
		}
		parent = parent.parentNode;
	}
	
	return null;
}

HxG_5.prototype.HxCheckCellListener.prototype.getParentOfTypeCell = function(node)
{
	if(node == null)
	{
		return null;
	}

	var parent = node.parentNode;
	while(parent!=null)
	{
		if(parent.nodeName==hX_5._TD ||parent.nodeName==hX_5._TH)
		{
			return parent;
		}
		parent = parent.parentNode;
	}
	
	return null;
}

////class HxCheckCellGroup////

/**
*Constructor
*Class for holding a group of HxCheckCellListener objects + methods for their interaction.
*/
HxG_5.prototype.HxCheckCellGroup = function(radio)
{
	this.arrayMembers = new Array();
	this.bSelected = true;
	this.isRadio = false;

	if(radio){
		this.isRadio = true;
	}

	if(!hX_5.getElementById)
	{
		return;
	}

	/*this.objPointer = hX_5.getElementById(strPointerId)
	if(!this.objPointer)
	{
		alert("RUNTIME ERROR: pointer image id="+strPointerId+" could not be found");
	}*/
}

/**Static variable*/
HxG_5.prototype.HxCheckCellGroup.prototype.GROUP = null;

/**
*Adds a child to the group.
*/
HxG_5.prototype.HxCheckCellGroup.prototype.addMember = function (objHxCheckCellListener)
{
	this.arrayMembers[this.arrayMembers.length] = objHxCheckCellListener;
}

/**
*Returns the mode we're setting (selected/unselected).
*/
HxG_5.prototype.HxCheckCellGroup.prototype.isSelected = function()
{
	return this.bSelected;
}

/**
*Sets the mode we're going to pass on the drag.
*/
HxG_5.prototype.HxCheckCellGroup.prototype.setSelected = function(bSelected)
{
	this.bSelected = bSelected;
}

/**
*Fired by HxCheckCellListener child.
*/
HxG_5.prototype.HxCheckCellGroup.prototype.fireMouseDown = function(evt, bSelected)
{
	this.setSelected(bSelected);
	//HxG_5.prototype.HxCheckCellGroup.prototype.GROUP = this;
	hX_5.HxCheckCellGroup.prototype.GROUP = this;
	this.registerWindowListeners(evt);
}

/**
*Adds some global listeners to the document.
*/
HxG_5.prototype.HxCheckCellGroup.prototype.registerWindowListeners = function(evt)
{
	if(document.attachEvent)
	{
		document.attachEvent("onmousemove", HxG_5.prototype.HxCheckCellGroup.prototype.onMouseEnterStatic);
		document.attachEvent("onmouseup",   HxG_5.prototype.HxCheckCellGroup.prototype.onMouseUpStatic);
		document.attachEvent("onselectstart",   HxG_5.prototype.HxCheckCellGroup.prototype.onSelectStartStatic);
		window.event.cancelBubble = true;
		window.event.returnValue = false;
	}
	else if(document.addEventListener)
	{
		document.addEventListener("mousemove", HxG_5.prototype.HxCheckCellGroup.prototype.onMouseEnterStatic, true);
		document.addEventListener("mouseup",   HxG_5.prototype.HxCheckCellGroup.prototype.onMouseUpStatic, true);
		evt.preventDefault();
	}
}

/**
*Method for preventing ugly selection of text + other unwanted effects.
*/
HxG_5.prototype.HxCheckCellGroup.prototype.onSelectStartStatic = function(evt)
{
	return false;
}

/**
*Static method for handling drag events (mousemove/onmousemove).
*/
HxG_5.prototype.HxCheckCellGroup.prototype.onMouseEnterStatic = function(evt)
{
	if(!evt)
		evt = window.event;

	//HxG_5.prototype.HxCheckCellGroup.prototype.GROUP.onMouseEnter(evt);
	hX_5.HxCheckCellGroup.prototype.GROUP.onMouseEnter(evt);
}

/**
*Instance handling of mousemove/onmousemove events.
*/
HxG_5.prototype.HxCheckCellGroup.prototype.onMouseEnter = function(evt)
{
	var len = this.arrayMembers.length;
	for(var i=0; i<len; i++)
	{
		if(this.arrayMembers[i].isInElementBoundary( hX_5.g.toPoint(evt)))
		{
			this.arrayMembers[i].setSelected( this.isSelected() );
			return;
		}
	}
}

/**
*Mouse button lifted, kill the events.
*/
HxG_5.prototype.HxCheckCellGroup.prototype.onMouseUpStatic = function(evt)
{
	if(!evt)
		evt = window.event;

	if(document.detachEvent)
	{
		document.detachEvent("onmousemove", HxG_5.prototype.HxCheckCellGroup.prototype.onMouseEnterStatic);
		document.detachEvent("onmouseup", HxG_5.prototype.HxCheckCellGroup.prototype.onMouseUpStatic);
		document.detachEvent("onselectstart",   HxG_5.prototype.HxCheckCellGroup.prototype.onSelectStartStatic);
	}
	else if(document.removeEventListener)
	{
		document.removeEventListener("mousemove", HxG_5.prototype.HxCheckCellGroup.prototype.onMouseEnterStatic, true);
		document.removeEventListener("mouseup", HxG_5.prototype.HxCheckCellGroup.prototype.onMouseUpStatic, true);
	}

	//HxG_5.prototype.HxCheckCellGroup.prototype.GROUP = null;
	hX_5.HxCheckCellGroup.prototype.GROUP = null;
}

/**
*Select all method
*/
HxG_5.prototype.HxCheckCellGroup.prototype.selectAll = function()
{
	var len = this.arrayMembers.length;
	for(var i=0; i<len; i++)
	{
		this.arrayMembers[i].setSelected( true );
	}
}

/**
*Select none method
*/
HxG_5.prototype.HxCheckCellGroup.prototype.selectNone = function()
{
	var len = this.arrayMembers.length;
	for(var i=0; i<len; i++)
	{
		this.arrayMembers[i].setSelected( false );
	}
}

/**
*Select inverse method
*/
HxG_5.prototype.HxCheckCellGroup.prototype.selectInverse = function()
{
	var len = this.arrayMembers.length;
	for(var i=0; i<len; i++)
	{
		this.arrayMembers[i].setSelected( !this.arrayMembers[i].isSelected() );
	}
}

//////////////class HxFocusChanger/////////////

/**
*Class for adding focus traversal to an element.
*/
HxG_5.prototype.HxFocusChanger = function(element, nPrev, nNext)
{
	if(!hX_5.getElementById)
	{
		//failsafe
		return;
	}

	this.objThis = this.findFocusableElement(element);
	this.nNext = nNext;
	this.nPrev = nPrev;

	if(this.objThis==null)
	{
		return;
	}

	this.bindToDom(this.objThis);
}

/**
*Add our listener to the DOM element
*/
HxG_5.prototype.HxFocusChanger.prototype.bindToDom = function(objThis)
{
	if(objThis.onkeyup != null)
	{
		this.anotherKeyListener = objThis.onkeyup;
	}

	var thisObject = this;

	objThis.onkeyup = function(evt)
	{
		if(!evt)
			evt = window.event;

		thisObject.onkeyup(evt);

		if(thisObject.anotherKeyListener!=null)
		{
			var fn = objThis.onkeyup;
			objThis.onkeyup = thisObject.anotherKeyListener;
			objThis.onkeyup(evt);
			objThis.onkeyup = fn;
		}
	}
}

HxG_5.prototype.HxFocusChanger.prototype.onkeyup = function(evt)
{
	var nKey = evt.which | evt.keyCode;
	var objGoto = null;

	if(nKey == this.nNext)
	{
		objGoto = this.findPeer(this.objThis, true);
	}
	else if(nKey == this.nPrev)
	{
		objGoto = this.findPeer(this.objThis, false);
	}
	else
	{
		return;
	}

	if(objGoto!=null)
	{
		objGoto.focus();
	}
}

/**
*Finds the element that is going to take focus
*/
HxG_5.prototype.HxFocusChanger.prototype.findFocusableElement = function(el)
{
	if(el==null)
	{
		return null;
	}

	var obj = el;
	if( typeof(el) == "string" )
	{
		obj = hX_5.getElementById(el);
	}

	if(obj==null)
	{
		alert("RUNTIME ERROR: cannot find: "+strId);
	}

	if(!obj.focus)
	{
		//then this isn't going to work; fail
		return null;
	}

	//TODO - more checks?

	return obj;
}

HxG_5.prototype.HxFocusChanger.prototype.findPeer = function(objElement, bNext)
{
	var tdObj = this.getParentOfTypeCell(objElement);
	var trObj = this.getParentOfType(objElement, hX_5._TR);
	var objRowContainer = trObj.parentNode;
	
	if( (tdObj==null) || (trObj==null) || (objRowContainer==null) )
	{
		return null;
	}

	var nIdx = this.getIndexInParent(trObj);
	while(true)
	{
		if(bNext)
		{
			nIdx++;
		}
		else
		{
			nIdx--;
		}

		var objSibling = this.getChildAtIndex(objRowContainer, nIdx);
		if(objSibling==null)
		{
			return null;
		}

		if( !this.isVisible(objSibling) )
		{
			continue;
		}

		var objTdPeer = this.findTdPeer(tdObj, objSibling);
		if(objTdPeer==null)
		{
			continue;
		}

		var objLike = this.findLike(objElement, objTdPeer);
		if(objLike!=null)
		{
			return objLike;
		}
	}

	return null;
}

HxG_5.prototype.HxFocusChanger.prototype.findTdPeer = function(tdFindPeer, trIn)
{
	var nIdx = this.getIndexInParent(tdFindPeer);
	return this.getChildAtIndex(trIn, nIdx);
}

HxG_5.prototype.HxFocusChanger.prototype.findLike = function(objFindLike, objIn)
{
	var nIdx = this.getIndexInParent(objFindLike);
	var objPotentialPeer = this.getChildAtIndex(objIn, nIdx);
	if(objPotentialPeer!=null)
	{
		if( this.isViableTarget(objPotentialPeer) )
		{
			return objPotentialPeer;
		}
	}

	var iterator = new  HxDomIterator(objIn);
	while(iterator.hasNext())
	{
		var objNext = iterator.next();
		if( this.isViableTarget(objNext) )
		{
			return objNext;
		}
	}
	return null;
}

HxG_5.prototype.HxFocusChanger.prototype.getChildAtIndex = function(objElement, nIdx)
{
	if(nIdx<0)
	{
		return null;
	}
	if(objElement==null)
	{
		return null;
	}
	if(nIdx >= objElement.childNodes.length)
	{
		return null;
	}
	return objElement.childNodes[nIdx];
}

HxG_5.prototype.HxFocusChanger.prototype.getIndexInParent = function(objElement)
{
	var objParent = objElement.parentNode;
	for(var i=0; i<objParent.childNodes.length; i++)
	{
		if(objElement==objParent.childNodes[i])
		{
			return i;
		}
	}
	return -1;
}

HxG_5.prototype.HxFocusChanger.prototype.getParentOfType = function(node, strType)
{
	if((node == null) || (strType==null))
	{
		return null;
	}

	var parent = node.parentNode;
	while(parent!=null)
	{
		if(parent.nodeName==strType)
		{
			return parent;
		}
		parent = parent.parentNode;
	}
	
	return null;
}

HxG_5.prototype.HxFocusChanger.prototype.getParentOfTypeCell = function(node)
{
	if(node == null)
	{
		return null;
	}

	var parent = node.parentNode;
	while(parent!=null)
	{
		if(parent.nodeName==hX_5.TD || parent.nodeName==hX_5.TH)
		{
			return parent;
		}
		parent = parent.parentNode;
	}
	
	return null;
}

HxG_5.prototype.HxFocusChanger.prototype.isViableTarget = function(objElement)
{
	return this.isFocusable(objElement) && this.isVisible(objElement);
}

HxG_5.prototype.HxFocusChanger.prototype.isFocusable = function(objElement)
{
	if(objElement==null)
	{
		return false;
	}

	if(!objElement.focus)
	{
		return false;
	}

	var nodeName = objElement.nodeName;
	if(nodeName == hX_5._INP)
		return true;
	if(nodeName == hX_5._BTN)
		return true;
	if(nodeName == hX_5._ANC)
		return true;

	return false;
}

HxG_5.prototype.HxFocusChanger.prototype.isVisible = function(objElement)
{
	if(objElement.style)
	{
		if(objElement.style.visibility=="hidden")
		{
			return false;
		}
		if(objElement.style.display=="none")
		{
			return false;
		}
	}
	var parent = objElement.parentNode;
	if(parent==null)
	{
		return true;
	}
	return this.isVisible(parent);
}

//////class HxDomIterator/////////

/**
*Class for iterating throw an element and its children.
*Elements are returned in order:
*1. element
*2. children
*3. siblings
*@param	element	the DOM element or its String id attribute
*/
HxG_5.prototype.HxDomIterator = function(element)
{
	this.index = 0;
	this.root = this.findElement(element);
	if(this.root==null)
	{
		//failsafe
		return;
	}

	this.stack = new Array();
	
	this.pushToStack(this.root);
}

HxG_5.prototype.HxDomIterator.prototype.pushToStack = function(obj)
{
	//push not supported in IE pre 5.5
	//this.stack.push(this.root);
	this.stack[this.index++] = obj;
}

HxG_5.prototype.HxDomIterator.prototype.popFromStack = function()
{
	//return this.stack.pop();
	return this.stack[this.index--];
}

/**
*Init method for locating the element.
*/
HxG_5.prototype.HxDomIterator.prototype.findElement = function(element)
{
	var ret = element;

	if(typeof(element) == "string" )
	{
		if(!hX_5.getElementById)
		{
			return null;
		}
		ret = hX_5.getElementById(element);
		if(!ret)
		{
			alert("RUNTIMER ERROR: cannot find "+element);
		}
	}

	return ret;
}

/**
*Has next.
*/
HxG_5.prototype.HxDomIterator.prototype.hasNext = function()
{
	//return (this.stack.length!=0);
	return (this.index>0);
}

/**
*The next DOM element.
*/
HxG_5.prototype.HxDomIterator.prototype.next = function()
{
	var obj = this.popFromStack();

	if(obj==null)
	{
		return null;
	}

	if(obj.nextSibling)
	{
		this.pushToStack(obj.nextSibling);
	}
	if(obj.firstChild)
	{
		this.pushToStack(obj.firstChild);
	}

	return obj;
}

////////////class HxFocusAdder////////////////

HxG_5.prototype.HxFocusAdder = function(strTableId, nPrev, nNext)
{
	if(!hX_5.getElementById)
	{
		//failsafe
		return;
	}
	
	var objTable = hX_5.getElementById(strTableId);
	if(objTable==null)
	{
		return;
	}
	
	var i, iterator, element, arrElements = objTable.getElementsByTagName(hX_5._TD);
	var y, iterator2, element2, arrElements2 = objTable.getElementsByTagName(hX_5._TH);
	
	for(i=0; i<arrElements.length; i++)
	{
		iterator = new HxDomIterator(arrElements[i]);
		while(iterator.hasNext())
		{
			element = iterator.next();
			if(this.isFocusable(element))
			{
				new HxFocusChanger(element, nPrev, nNext);
			}
		}
	}
	
	for(y=0; y<arrElements2.length; y++)
	{
		iterator2 = new HxDomIterator(arrElements2[y]);
		while(iterator2.hasNext())
		{
			element2 = iterator2.next();
			if(this.isFocusable(element2))
			{
				new HxFocusChanger(element2, nPrev, nNext);
			}
		}
	}
}

HxG_5.prototype.HxFocusAdder.prototype.isFocusable = HxG_5.prototype.HxFocusChanger.prototype.isFocusable;




/**static cursor up keycode*/
HxG_5.prototype.HxFocusAdder.prototype.UP = 38;
/**static cursor down keycode*/
HxG_5.prototype.HxFocusAdder.prototype.DOWN = 40;


///////////class HxGridShow///////////////
			
/*function HxGridShow(strShowId, strHideId, strAId)
{
	if(!hX_5.getElementById)
	{
		return;
	}
	this.objShow = this.findElement(strShowId);
	this.objHide = this.findElement(strHideId);
	this.objA = this.findElement(strAId);

	this.bindToDOM();
}

HxGridShow.prototype.findElement = function(strId)
{
	var obj = hX_5.getElementById(strId);
	if(obj==null)
	{
		alert("RUNTIME ERROR: cannot find element "+strId);
	}
	return obj;
}

HxGridShow.prototype.bindToDOM = function()
{
	var thisObj = this;

	this.objA.onclick = function(evt)
	{
		if(!evt)
		{
			var evt = window.event;
		}
		thisObj.onClick(evt);
		return false;
	}
}

HxGridShow.prototype.onClick = function(evt)
{
	this.objShow.style.display = "inline";
	this.objHide.style.display = "none";
}*/

//////////class HxEditSave////////////////

/*function HxEditSave(strHidId, strAId, nIdx)
{
	if(!hX_5.getElementById)
	{
		return;
	}

	this.objHidden = this.findElement(strHidId);
	this.objA = this.findElement(strAId);
	this.nIdx = nIdx;
	this.objForm = this.getParentOfType(this.objA, "FORM");
	if(this.objForm==null)
	{
		return;
	}

	this.bindToDOM();
}

HxEditSave.prototype.findElement = function(strId)
{
	var obj = hX_5.getElementById(strId);
	if(obj==null)
	{
		alert("RUNTIME ERROR: cannot find element "+strId);
	}
	return obj;
}

HxEditSave.prototype.bindToDOM = function()
{
	var thisObj = this;

	this.objA.onclick = function(evt)
	{
		if(!evt)
		{
			var evt = window.evt;
		}
		thisObj.onClick(evt);
	}
}

HxEditSave.prototype.getParentOfType = HxRowActionListener.prototype.getParentOfType;

HxEditSave.prototype.onClick = function(evt)
{
	this.objHidden.value = this.nIdx;
	this.objForm.submit();
}*/

///////////////class HxGridEdit/////////////////////

/*function HxGridEdit()
{
	this.editRows = new Array();
	this.viewRows = new Array();
}

HxGridEdit.prototype.addRow = function(strTrId, strTrEditId, strAEditId, strACancelId)
{
	if(!hX_5.getElementById)
	{
		return;
	}

	this.bindToCancel(strACancelId);
	this.bindToEdit(strTrId, strTrEditId, strAEditId);

	this.viewRows[this.viewRows.length] = strTrId;
	this.editRows[this.editRows.length] = strTrEditId;
}

HxGridEdit.prototype.bindToCancel = function(strACancelId)
{
	var thisObj = this;
	var objA = this.findElement(strACancelId);

	objA.onclick = function(evt)
	{
		thisObj.resetRows();
	}
}

HxGridEdit.prototype.bindToEdit = function(strTrId, strTrEditId, strAEditId)
{
	var thisObj = this;
	var objA = this.findElement(strAEditId);

	objA.onclick = function(evt)
	{
		thisObj.editRow(strTrId, strTrEditId);
	}
}

HxGridEdit.prototype.editRow = function(strTrId, strTrEditId)
{
	var objTr = this.findElement(strTrId);
	var objTrEditId = this.findElement(strTrEditId);

	this.resetRows();
	
	objTr.style.display = "none";
	//objTrEditId.style.display = "inline";
	objTrEditId.style.display = "";
}

HxGridEdit.prototype.findElement = function(strId)
{
	var obj = hX_5.getElementById(strId);
	if(obj==null)
	{
		alert("RUNTIME ERROR: cannot find element "+strId);
	}
	return obj;
}

HxGridEdit.prototype.resetRows = function()
{
	for(var row in this.editRows)
	{
		var obj = this.findElement(this.editRows[row]);
		obj.style.display = "none";
	}
	for(var row in this.viewRows)
	{
		var obj = this.findElement(this.viewRows[row]);
		//obj.style.display = "inline";
		obj.style.display = "";
	}
}*/


////////////////////////////////////////////////////////////////////////////
// DATAGRID CLASSES - END
////////////////////////////////////////////////////////////////////////////
