/*
 * Licensed Material - Property of IBM 
 * (C) Copyright IBM Corp. 2002, 2006 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure 
 * restricted by GSA ADP Schedule Contract with IBM Corp. 
 */

// -----------------
// HxG Global Object
// -----------------

//----------------------------------------------------------------------------------------------------
// Public:	Add a component to the JSF global object
// 
// Arguments:	id		(string tagname or object) The DOM object to which the component is attached
//				obj		(object) The JSF Component to be attached (e.g., a JSFSpinner)
//
HxG_5.prototype.addComponent = function(id, obj) {
	if (id && obj && this.brw.isW3C() ) {
		var DOMobj = this.getElementById(id);
		if (DOMobj!=null && (this.Components[id]==undefined || this.Components[id]==null)) {
			this.Components[id] = obj;
			obj.id = id;
			obj.DOMobj = DOMobj;
			obj.HTMLrendered = false;
			obj.p = this;
			obj.r = hX_5.currentResource();
			obj.lc = hX_5.currentLocale();
			this.loadComponent(obj);
			return true;
		}
	}
	return false;
}
HxG_5.prototype.add = HxG_5.prototype.addComponent;

//----------------------------------------------------------------------------------------------------
// Public:	Remove a component from the JSF global object
// 
// Arguments:	obj		The component to be destroyed
//
// NOTE: This removes memory resources and event handlers.  It does NOT remove any HTML added
// to the page by the component
HxG_5.prototype.removeComponent = function(obj) {
	if (obj && obj.DOMobj && obj.DOMobj.id && this.brw.isW3C() ) {
		// NOTE: We assume the caller has already done a uirelease on the entire UI
		// We really can't do it here as the component may already be partially destroyed
		// (particularly if AJAX is calling this routine)

		// Ask the event manager to destroy any event handling so the component doesn't have
		// to destroy any events attached directly to the DOM object
		hX_5.removeAllEvents(obj.DOMobj, obj.DOMobj.id);
		hX_5.i.removeMonitor(obj.DOMobj);
		hX_5.i.removeResizeMonitor(obj.DOMobj);
		// Ask the component to destroy what it can
		if (obj.destroy) obj.destroy();
		// Remove from the array
		this.Components[obj.DOMobj.id] = null;
		// Destroy the object
		delete obj;
	}
	return true;
}

//----------------------------------------------------------------------------------------------------
// Public:	Add a subcomponent to a multi-part component
// 
// Arguments:	parentid (string) id of the JSF Component which is the parent
//				childid	 (string) id to give this subcomponent
//				obj		 (object) The JSF Component to be attached (e.g., a JSFSpinner)
//				position (number)(optional) Position (by default it's last)
// OR :
// 				parentid (string) id of the JSF Component which is the parent
//				array	 (array of items)
//						  The items are assigned the ids made by appending "-1" (-2, -3, -4, ...) to the parent id
HxG_5.prototype.addSubComponent = function (parentId, iChildId, iObj, iPosition) {
	if (parentId && this.brw.isW3C() ) {
		var theParent = this.getComponentById (parentId);
		theParent = (theParent == null) ? this.getSubComponentById (parentId) : theParent;
		if (theParent) {
			if (hX_5.isString(iChildId)) {
				if (iChildId && iObj) {
					this.SubComponents[iChildId] = iObj;
					iObj.id = iChildId;
					iObj.p = theParent;
					theParent.addSubComponent (iObj, iPosition);
				}
			} else if (iChildId.length) {
				var a=iChildId, childId, i;
				for (i=0; i<a.length; i++) {
					if (a[i]) {
						childId = parentId + hX_5._Y + (i+1);
						this.SubComponents[childId] = a[i];
						a[i].id = childId;
						a[i].p = theParent;
						theParent.addSubComponent(a[i], null);
					}
				}
			}
		}
		return true;
	}
	return false;
}
HxG_5.prototype.addS = HxG_5.prototype.addSubComponent;

//----------------------------------------------------------------------------------------------------
// Public:	Remove a subcomponent to a multi-part component
// Arguments:	parentid (string) id of the JSF Component which is the parent
//				childid	 (string) subcomponent to remove
HxG_5.prototype.removeSubComponent = function (parentId, childId) {
	if (parentId && this.brw.isW3C() ) {
		var theParent = this.getComponentById (parentId);
		theParent = (theParent == null) ? this.getSubComponentById (parentId) : theParent;
		if (theParent && childId) {
			this.SubComponents[childId] = null;
			theParent.removeSubComponent (childId);
			return true;
		}
	}
	return false;
}
HxG_5.prototype.remS = HxG_5.prototype.removeSubComponent;

//----------------------------------------------------------------------------------------------------
// Private:		Destroy a subcomponent 
// Arguments:	parentid (string) id of the JSF Component which is the parent
//				childid	 (string) subcomponent to remove
HxG_5.prototype.destroySubComponent = function (childId) {
	if (childId && this.brw.isW3C() ) {
		this.SubComponents[childId] = null;
		return true;
	}
	return false;
}

//----------------------------------------------------------------------------------------------------
// Public:	Add a behavior to the JSF global object 
// 
// Arguments:	id			(string tagname or object) The DOM object to which the behavior is attached
//				eventname	(string) The DOM event which triggers the behavior, all lowercase, IE spelling,
//							with the 'on'. For example, 'onclick' or 'onkeypress"'.
//				obj			(object) The JSF Behavior to be attached (e.g., a JSFBehaviorHS)
//
HxG_5.prototype.addBehavior = function(id, eventname, obj) {
	if (id && eventname && obj && obj.type && this.brw.isW3C() ) {
		var DOMobj=(id==hX_5._BODY)?document.body:this.getElementById(id);
		if (DOMobj) {
			var e=hX_5.getBehaviorEvent(obj.type, eventname), i=e.indexOf(hX_5._ON), thisid=id + hX_5._HX + obj.type + hX_5._U + ((i==0)?e.substr(2):e);
			if (this.Behaviors[thisid] && this.Behaviors[thisid].addSubComponent) {
				this.Behaviors[thisid].addSubComponent(obj);
			} else {
				this.Behaviors[thisid] = obj;
				obj.id = id;
				obj.eventname = e;
				obj.DOMobj = DOMobj;
				obj.HTMLrendered = false;
				obj.p = this;
				obj.r = hX_5.currentResource();
				obj.lc = hX_5.currentLocale();
				this.loadComponent(obj);
			}
			return true;
		}
	}
	return false;
}
HxG_5.prototype.addB = HxG_5.prototype.addBehavior;

//----------------------------------------------------------------------------------------------------
// Public:	Given an behavior object, destroy it
// 
// Arguments:	obj			(a behavior object (not an id)
HxG_5.prototype.removeBehavior = function(obj) {
	if (obj && obj.DOMobj && obj.DOMobj.id && this.brw.isW3C() ) {
		// NOTE: We assume the caller has already done a uirelease on the entire UI
		// We really can't do it here as the component may already be partially destroyed
		// (particularly if AJAX is calling this routine)

		// Remove all events we can from the object
		hX_5.removeAllEvents(obj.DOMobj, obj.DOMobj.id);
		// Ask the component to destroy what it can
		if (obj.destroy) obj.destroy();
		// Remove from the array
		var e=this.getBehaviorEvent(obj.type, obj.eventname),i=e.indexOf(hX_5._ON), thisid=obj.id + hX_5._HX + obj.type + hX_5._U + ((i==0)?e.substr(2):e);
		hX_5.Behaviors[thisid] = null;
		// Destroy the object
		delete obj;

	}
	return;
}

//----------------------------------------------------------------------------------------------------
// Public:	Add a converter to the JSF global object 
// 
// Arguments:	id		(string) The name to give this converter.
//				obj		(object) The JSF Converter to be attached (e.g., a JSFDateTimeConverter)
//
HxG_5.prototype.addConverter = function(id, obj) {
	if (id && obj && this.brw.isW3C() ) {
		this.Converters[id] = obj;
		obj.id = id;
		obj.p = this;
		return true;
	}
	return false;
}
HxG_5.prototype.addC = HxG_5.prototype.addConverter;

//----------------------------------------------------------------------------------------------------
// Public:	Add a validator to the JSF global object 
// 
// Arguments:	id		(string) The name to give this validator.
//				obj		(object) The JSF Validator to be attached (e.g., a JSFDateTimeValidator)
//
HxG_5.prototype.addValidator = function(id, obj) {
	if (id && obj && this.brw.isW3C() ) {
		this.Validators[id] = obj;
		obj.id = id;
		obj.p = this;
		return true;
	}
	return false;
}
HxG_5.prototype.addV = HxG_5.prototype.addValidator;

//----------------------------------------------------------------------------------------------------
// Public:	Add an HTML tag to the JSF global object
// 
// Arguments:	id		(string tagname or object) The HTML object to be added to the JSF global object
//				type	(string)  Reason for adding it (currently should be set to 'taborder');
HxG_5.prototype.addHTMLTag = function(id, type) {
	if (id && type=="taborder" && this.brw.isW3C() ) {
		var DOMobj = this.getElementById(id);
		if (DOMobj) {
			this.HTMLTags.push(DOMobj);
			return true;
		}
	}
	return false;
}
HxG_5.prototype.addH = HxG_5.prototype.addHTMLTag;

//----------------------------------------------------------------------------------------------------
// Private:	Add a popup to the JSF global object
// 
// Arguments:	id			(string) The id to give this popup.
//				parentid	(string tagname or object) The DOM object which parents this popup
//				srcvalue	(URL) Not currently supported, URL of content if not constructed
//				clickstyle	(int, value = 0,1,2). 
//							If 0, mouse is automatically put in track mode where we track the position of the mouse so it can be used to "drag" (e.g., slider)
//							If 1, mouse can be used to 'click' within the popup, i.e., the popup contents can be clicked on (e.g., date picker)
//							If 2, mouse is managed by other code (e.g., popup mouse manager)
// 				fullstyle	If true, the popup is left completely unstyled (usually for popup menu)
//							Otherwise, the margins/padding are cleared on it
//				altComp		if !null, the object to dispatch component events to (only needed if parentID does not refer
//							to a DOMobj that has a component attached to it).
HxG_5.prototype.addPopup = function(id, parentId, ownerObj, style, altComp) {
	if (id && this.brw.isW3C() ) {
		// Only add the popup if it doesn't exist
		if (this.Popups[id]) {
			// currently don't need to re-init the popup if it exists
			return this.Popups[id];
		} else {
			// Construct a new popup object
			var obj = new this.JSFPopup(parentId, ownerObj, style, altComp);
			this.Popups[id] = obj;
			obj.id = id;
			obj.HTMLrendered = false;
			obj.p = this;
			obj.r = hX_5.currentResource();
			obj.lc = hX_5.currentLocale();
			this.loadComponent(obj);
			return obj;
		}
	}
	return null;
}
HxG_5.prototype.addP = HxG_5.prototype.addPopup;

//----------------------------------------------------------------------------------------------------
// Public:	Destroy a popup
HxG_5.prototype.removePopup = function(id) {
	if (id && this.brw.isW3C() ) {
		var obj = this.Popups[id];
		if (obj) {
			if (obj.destroy) obj.destroy();
			delete obj;
		}
		this.Popups[id] = null;
	}
}

//----------------------------------------------------------------------------------------------------
// Public:	Cause all components being managed by the global object to be rendered
//			in the HTML on the page. (Actually call defers the rendering to body onload event)
HxG_5.prototype.onPageLoad = function () {
	if (!this.bodyLoadRewritten) this.bodyLoadEventNum = this.attachEvent(document.body, hX_5._onL, hX_5._lL, this.pageLoadAction);
	this.bodyLoadRewritten = true;
	return true;
}

//----------------------------------------------------------------------------------------------------
// Private:	load a component
HxG_5.prototype.loadComponent = function(c) {
	if (this.bodyLoadDone && c.onPageLoad) {
		if (!c.HTMLrendered) c.onPageLoad();
		c.HTMLrendered = true;
	}
}

//----------------------------------------------------------------------------------------------------
// Private:	Actual PageUnLoad code
HxG_5.prototype.pageUnloadAction= function(evt) {
	// First remove the event handler that got us here
	var i, varobj;
	// First remove the event handler that got us here
	if (typeof(hX_5) != 'undefined') hX_5.removeEvent(document.body, hX_5._onUL, this.bodyUnloadEventNum);
	// Release the UI
	hX_5.uirelease(null,true);
	// Destroy important things
	for (i=0; i<hX_5.mustDestroy.length; i++) if (hX_5.mustDestroy[i].destroy) hX_5.mustDestroy[i].destroy();
	// Destroy the tab order navigator
	if (hX_5.tabNav != null) delete (hX_5.tabNav);
	// Destroy the CSS cache
	if (hX_5.CSSSelectors != null) delete (hX_5.CSSSelectors);
	// If it has client data remove dom elements
	if((typeof ODCRegistry) != 'undefined' && ODCRegistry) if (typeof(unbindElements) != 'undefined') unbindElements();
	// done
	return true;
}

//----------------------------------------------------------------------------------------------------
// Private:	Actual onPageLoad code
HxG_5.prototype.pageLoadAction = function(evt) {
	var varobj, inBodyLoad=hX_5.bodyLoadDone;
	hX_5.bodyLoadDone = true;

	// First remove the event handler that got us here
	hX_5.removeEvent(document.body, hX_5._onL, hX_5.bodyLoadEventNum);

 	// Initialize components, behaviors and popups
	for (varobj in hX_5.Components) if (hX_5.Components[varobj]) hX_5.loadComponent(hX_5.Components[varobj]);
	for (varobj in hX_5.Behaviors) if (hX_5.Behaviors[varobj]) hX_5.loadComponent(hX_5.Behaviors[varobj]);
	for (varobj in hX_5.Popups) if (hX_5.Popups[varobj]) hX_5.loadComponent(hX_5.Popups[varobj]);

	// Set up an unload event handler that will do some cleanup if possible
	if (hX_5.brw.isIE() || ((typeof ODCRegistry) != 'undefined' && ODCRegistry))
		this.bodyUnloadEventNum = hX_5.attachEvent(document.body, "onunload", hX_5._lL, hX_5.pageUnloadAction);

	// Don't know why, but when we have an element that needs a resize tracker in Mozilla
	// it seems to sometimes need a redraw at this point.  We also redraw if requested by a loaded component
	if (!inBodyLoad && ((hX_5.bodyLoadRedrawA && hX_5.bodyLoadRedrawB) || (hX_5.globalResize && hX_5.brw.isNavOrMoz()))) hX_5.redraw();
	hX_5.bodyLoadRedrawA = hX_5.bodyLoadRedrawB = false;

	return;	// let the event continue
}

//----------------------------------------------------------------------------------------------------
// Private:	Page Reload -- same as above but used by AJAX to reload part of the page
HxG_5.prototype.pageReLoadAction = function() {
	var varobj;
	hX_5.bodyLoadDone = true;

 	// Initialize components, behaviors and popups
	for (varobj in hX_5.Components) if (hX_5.Components[varobj]) hX_5.loadComponent(hX_5.Components[varobj]);
	for (varobj in hX_5.Behaviors) if (hX_5.Behaviors[varobj]) hX_5.loadComponent(hX_5.Behaviors[varobj]);
	for (varobj in hX_5.Popups) if (hX_5.Popups[varobj]) hX_5.loadComponent(hX_5.Popups[varobj]);

	// Don't know why, but when we have an element that needs a resize tracker in Mozilla
	// it seems to sometimes need a redraw at this point.  We also redraw if requested by a loaded component
	if ((hX_5.bodyLoadRedrawA && hX_5.bodyLoadRedrawB) || (hX_5.globalResize && hX_5.brw.isNavOrMoz())) hX_5.redraw();
	hX_5.bodyLoadRedrawA = hX_5.bodyLoadRedrawB = false;
	
	return;	
}

//----------------------------------------------------------------------------------------------------
// Public:	Subscribe an any "event" broadcast using the broadcast function
//			eventName:	The name of the event you want to subscribe to
//						NOTE:  the event "redraw" is special in that all hx components
//						are automatically subscribed to it
//			handler:	The handler to call -- must be of type "function"
//			(returns):  subscription id -- use this to "unsubscribe"
HxG_5.prototype.subscribe = function(eventName, handler, c) {
	if ((c!=null && c.type!=undefined) || (c==null && hX_5.isFunction(handler))) {
		this.topicUnq++;
		if (this.topicUnq>10000) this.topicUnq=99;
		if (c!=null || hX_5.isFunction(handler))
		if(!hX_5.Topics[eventName])
			hX_5.Topics[eventName] = [[this.topicUnq,handler]];
		else
			hX_5.Topics[eventName].push([this.topicUnq,handler]);
		return this.topicUnq;
	} else return null;
}

//----------------------------------------------------------------------------------------------------
// Public:	Unsubscribe
//			id	-- id returned by a previous "subscribe" call
HxG_5.prototype.unsubscribe = function(id) {
	var varobj, i, done=false;
	for (varobj in this.Topics) {
		for (i=0; i< hX_5.Topics[varobj].length; i++) {
			if (hX_5.Topics[varobj][i][0]==id) {
				delete (hX_5.Topics[varobj][i]);
				hX_5.Topics[varobj][i]=null;
				done=true;
				break;
			}
		}
		if (done) {
			for (i=hX_5.Topics[varobj].length-1; i>0; i--) {
				if (hX_5.Topics[varobj][i]==null)
					hX_5.Topics[varobj].pop();
				else
					break;
			}
			break;
		}
	}
}

//----------------------------------------------------------------------------------------------------
// Public:	Broadcast an event -- all subscribers will be notified of the event
//			eventName:	The name of the event you want to broadcast
//			param:		A parameter to pass to the handler(s) that are called
HxG_5.prototype.broadcast = function(eventName, param) {
	if (hX_5.Topics[eventName]) {
		for (var i=0; i< hX_5.Topics[eventName].length; i++) {
			if (hX_5.Topics[eventName][i]!=null && hX_5.Topics[eventName][i][1])
				hX_5.Topics[eventName][i][1](param);
		}
		if (eventName=="redraw") hX_5.redraw(param, true);
	}
}

//----------------------------------------------------------------------------------------------------
// Public:	Resize and redraw all JSF controls and their attendant buttons, ... 
//		    Extended version that finds and redraws parent ODC-style components
HxG_5.prototype.redrawWithODC = function (thisC, param, noloop) {
	this.redraw(param,noloop);
	if (thisC.DOMobj && this.brw.isNavOrMoz() && hX_5.isDef(ODCRegistry) && ODCRegistry!=null && ODCRegistry.Elements) {
		for (var e in ODCRegistry.Elements) {
			// Tabbed Panel
			if (ODCRegistry.Elements[e] && ODCRegistry.Elements[e].JSObject && ODCRegistry.Elements[e].JSObject.type && ODCRegistry.Elements[e].JSObject.type.toLowerCase()=="tabpanel") {
				// If this component is a child of the tabbed panel, redraw the tabbed panel
				var p = thisC.DOMobj.parentNode;
				var t = ODCRegistry.Elements[e].JSObject;
				var tp = (t.getDOMobj)?t.getDOMobj():null;
				if (tp) {
					while (p && p!=tp) p = p.parentNode;
					if (p && p==tp) {
						if (t.redraw) t.redraw();
						break;
					}
				}
			}
		}
	}
}

//----------------------------------------------------------------------------------------------------
// Public:	Resize and redraw all JSF controls and their attendant buttons, ... 
HxG_5.prototype.redraw = function (param, noloop) {
	var varobj,i;
	// For components, release any state that's in play and then redraw them
	// the redraw call will resize them
	for (varobj in this.Components) {
		if (this.Components[varobj] && this.Components[varobj].uirelease) this.Components[varobj].uirelease();
		if (this.Components[varobj] && this.Components[varobj].redraw) this.Components[varobj].redraw(null, param);
	}
	// For popups, just release them. They always redraw themselves whenever
	// they are popped open
	for (varobj in this.Popups) if (this.Popups[varobj] && this.Popups[varobj].uirelease) this.Popups[varobj].uirelease();
	// Call any user registered redraw topics
	if (!noloop && hX_5.Topics.length>0 && hX_5.Topics["redraw"]) {
		varobj = hX_5.Topics["redraw"];
		for (var i=0; i< varobj.length; i++)
			if (varobj[i]!=null && varobj[i][1]) varobj[i][1](param);
	}
	// Destroy the tab order navigator
	if (hX_5.tabNav != null) {
		delete (hX_5.tabNav);
		hX_5.tabNav = null;
	}
	// Destroy the CSS cache
	if (hX_5.CSSSelectors !=null) {
		delete (hX_5.CSSSelectors)
		hX_5.CSSSelectors = null;
	}

	if (!(this.isUnDef(typeof hX_1)) && (hX_1 != null) && (hX_1.redraw)) hX_1.redraw();
	if (!(this.isUnDef(typeof hX_2)) && (hX_2 != null) && (hX_2.redraw)) hX_2.redraw();
	if (!(this.isUnDef(typeof hX_3)) && (hX_3 != null) && (hX_3.redraw)) hX_3.redraw();
	if (!(this.isUnDef(typeof hX_4)) && (hX_4 != null) && (hX_4.redraw)) hX_4.redraw();

	return true;
}

//----------------------------------------------------------------------------------------------------
// Public: Release any popups that are visible or any other UI'isms that we want to clear out
HxG_5.prototype.uirelease = function (freeOnly, allTimers) {
	this.popMgr.end();
	this.mouser.killMouse();
	if (hX_5.pendingPrompt != null) hX_5.pendingPrompt.uirelease();
	if (freeOnly == null) {
		var o = this.uistate.getActiveComponent();
		if (o) o.uirelease();	// release it first so it can handle focus
		this.uistate.setActiveComponent (hX_5._E);
	}
	this.i.killAllTimers(allTimers);
	if (freeOnly == null) {
		for (varobj in this.Components)
			if (this.Components[varobj] && this.Components[varobj].uirelease) this.Components[varobj].uirelease();
	}
	for (varobj in this.Popups)
		if (this.Popups[varobj] && this.Popups[varobj].uirelease) this.Popups[varobj].uirelease();
	if (freeOnly == null) {
		if (!(this.isUnDef(typeof hX_1)) && (hX_1 != null) && (hX_1.uirelease)) hX_1.uirelease();
		if (!(this.isUnDef(typeof hX_2)) && (hX_2 != null) && (hX_2.uirelease)) hX_2.uirelease()
		if (!(this.isUnDef(typeof hX_3)) && (hX_3 != null) && (hX_3.uirelease)) hX_3.uirelease()
		if (!(this.isUnDef(typeof hX_4)) && (hX_4 != null) && (hX_4.uirelease)) hX_4.uirelease()
	}
	return true;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given an object's DOM id (a string from 'id=') look up the component object.
// 
// Arguments:	obj		(string tagname or object) Find the component for this object.
//
HxG_5.prototype.getComponentById = function(obj) {
	return (obj != null && this.isString(obj)) ? ((this.Components[obj]==undefined)?null:this.Components[obj]) : obj;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given an object's DOM id (a string from "id=") return the id's of any components 
// within it
// 
// Arguments:	obj			(string tagname or object) Find the components within this object.
//				(returns)	array of component ids (null if none)
//
HxG_5.prototype.getComponentsInId = function(inobj, a) {
	var c, obj=hX_5.getElementById(inobj), r=((a)?a:null);
	if (obj) {
		if (obj.id!=undefined && obj.id != null && obj.id.length > 0) {
			c = hX_5.getComponentById(obj.id);
			if (c) {
				if (r==null) r=[];
				r.push(c.id);
			}
		}
		for (obj = obj.firstChild; obj != null; obj = obj.nextSibling) r = hX_5.getComponentsInId(obj, r);
	}
	return r;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given an object's DOM id (a string from "id=") return the id of the DOM object
//          (and equally of the component) of the first parent object that has a component
//			of the specified type associated with it
// 
// Arguments:	obj			(string tagname or object) Find the components within this object.
//				type		The type of the component being searched for, e.g., "paneldialog"
//							See hxclient_c.js for a list of known types
//				(returns)	id
//
HxG_5.prototype.getParentComponentOfId = function(inobj, type) {
	var c, t=type.toLowerCase(), obj=hX_5.getElementById(inobj);
	while (t && obj!=null) {
		if (obj.id!=undefined && obj.id != null && obj.id.length > 0) {
			c = hX_5.getComponentById(obj.id);
			if (c && c.type==t) return obj.id;
		}
		obj = obj.parentNode;
	}
	return null;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given a subcomponent's DOM id (a string from 'id=') look up it's component
// 
// Arguments:	obj		(string tagname or object) Find the subcomponent for this object.
//
HxG_5.prototype.getSubComponentById = function(obj) {
	return (obj != null && this.isString(obj)) ? ((this.SubComponents[obj]==undefined)?null:this.SubComponents[obj]) : obj;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given a component's id retrieve an array of subcomponent ids
// 
// Arguments:	obj			(string tagname or object) Find the subcomponents for this id.
//				(returns)	array of subcomponent ids
HxG_5.prototype.getSubComponentsInId = function(obj) {
	var v,x,p=hX_5.getComponentById(obj);
	p = (p==null)?hX_5.getSubComponentById(obj):p;
	if (p && p.id!=undefined) {
		var r = [];
		for (v in hX_5.SubComponents) {
			x = hX_5.SubComponents[v];
			if (x && x.p!=undefined && x.p.id!=undefined && x.p.id==p.id) {
				r.push(x.id);			
			}
		}
		return ((r.length>0)?r:null);
	}
	return null;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given a subcomponent's DOM id (a string from 'id=') look up its parent component
// 
// Arguments:	obj		(string tagname or object) Find the component for this object.
//
HxG_5.prototype.getParentComponentById = function(obj) {
	return (obj != null && this.isString(obj)) ? ((this.SubComponents[obj].p==undefined)?null:this.SubComponents[obj].p) : obj;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given a converter's id (a string from 'id=') get the converter
// 
// Arguments:	obj		(string tagname or object) Find the converter with this id.
//
HxG_5.prototype.getConverterById = function(obj) {
	return (obj != null && this.isString(obj)) ? ((this.Converters[obj]==undefined)?null:this.Converters[obj]) : obj;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given a validator's id (a string from 'id=') get the validator
// 
// Arguments:	obj		(string tagname or object) Find the validator with this id.
//
HxG_5.prototype.getValidatorById = function(obj) {
	return (obj != null && this.isString(obj)) ? ((this.Validators[obj]==undefined)?null:this.Validators[obj]) : obj;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given an object's DOM id (a string from 'id=') look up the behavior object.
// 
// Arguments:	obj			(string tagname) Find the behavior for this object
//				behaviorname(string) For this behavior-name
//				eventname	(string) For this event
//							e.g., find the 'assist' behavior for 'onclick' on 'form1:text1'
//				returns		behavior object
HxG_5.prototype.getBehaviorById = function(obj, behaviorname, eventname) {
	if (obj != null && this.isString(obj)) {
		var e=this.getBehaviorEvent(behaviorname, eventname),i=e.indexOf(hX_5._ON), thisid=obj + hX_5._HX + behaviorname + hX_5._U + ((i==0)?e.substr(2):e);
		return ((hX_5.Behaviors[thisid]==undefined)?null:hX_5.Behaviors[thisid]);
	}
	return (obj);
}

// Private:	Given a behavior name and eventname, we may need to change the eventname
HxG_5.prototype.getBehaviorEvent = function(bname, ename) {
	return (ename);
	// We don't do this anymore, but below is an example of what we did in earlier versions
//	return ((this.brw.isNavOrMoz() && ename==hX_5._onKD && bname==hX_5.BHR_keybind)?hX_5._onKP:ename);
}

//----------------------------------------------------------------------------------------------------
// Public:	Given an object's DOM id look up all behaviors attached to it
// 
// Arguments:	obj			(string tagname) Find the behavior for this object
//				returns		array of behavior objects (or null)
HxG_5.prototype.getBehaviorsById = function(obj) {
	var behavior, r=[], o=(obj==null)?null:(this.isString(obj)?obj: ((obj.id!=undefined)?obj.id:null));
	if (o!=null) for (behavior in hX_5.Behaviors) {
		if (hX_5.Behaviors[behavior] && hX_5.Behaviors[behavior].id==o) r.push(hX_5.Behaviors[behavior]);
	}
	return (r);
}

//----------------------------------------------------------------------------------------------------
// Public:	Given a popup's DOM id, look up the object.
// 
// Arguments:	obj	(string tagname or object) Find the popup for this object
//
HxG_5.prototype.getPopupById = function(obj) {
	return (obj != null && this.isString(obj)) ? ((this.Popups[obj]==undefined)?null:this.Popups[obj]) : obj;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given an object's DOM id (a string from 'id=') look up the DOM object.
//			This is a safe wrapper for the DOM's document.getElementById call.
// 
// Arguments:	obj		(string tagname or object) Find the DOM object for this string (no-ops
//						if passed a DOM object)
//
HxG_5.prototype.getElementById = function(obj) {
	var theObj = obj;
	if (obj != null && this.isString(obj)) {
		if (window.document.getElementById != null) theObj = window.document.getElementById(obj);
		else if (window.document.all != null) theObj = window.document.all(obj);
	}
	return theObj;
}

//----------------------------------------------------------------------------------------------------
// Public:	Mark a page as being a portal-managed page (i.e., the page contains portlets)
// 
HxG_5.prototype.setPortalEncoding = function() {
	hX_5.isPortal = true;
}

//----------------------------------------------------------------------------------------------------
// Public: attach an event handler to a JSF Component.  Events are sent to component.dispatchComponent function
//
// Arguments:	objid		The object to which a handler is to be attached
//				eventname	The event to handle (e.g., onclick). All lower case, with the 'on'
//				position	See below.
HxG_5.prototype.attachComponentEvent = function (objid, eventname, position) {
	hX_5.attachEvent (objid, eventname, position, hX_5.i.dispatchComponent);
}

//----------------------------------------------------------------------------------------------------
// Public: attach an event handler to a JSF Behavior.  Events are sent to component.dispatchBehavior function
//
// Arguments:	objid		The object to which a handler is to be attached
//				eventname	The event to handle (e.g., onclick). All lower case, with the 'on'
//				position	See below.
HxG_5.prototype.attachBehaviorEvent = function (objid, behaviorname, eventname, position, alias) {
	if (alias) hX_5.attachEvent(objid, eventname, position, "hX_5.i.dispatchBehavior(this, event, '" + behaviorname + "', '" + alias + "')");
	else hX_5.attachEvent(objid, eventname, position, "hX_5.i.dispatchBehavior(this, event, '" + behaviorname + "')");
}

//----------------------------------------------------------------------------------------------------
// Public: attach an event handler to an inputState of JSF Behavior.  Events are dispatched into the state handler
//
// Arguments:	objid		The object to which a handler is to be attached
//				eventname	The event to handle (e.g., onclick). All lower case, with the 'on'
HxG_5.prototype.attachInputStateEvent = function (objid, eventname) {
	hX_5.attachEvent(objid, eventname, hX_5._lF, hX_5.i.dispatchInputState);
}

//----------------------------------------------------------------------------------------------------
// Public: attach an event handler to a DOM object preserving the handler already on the object
// NOTE: this routine should be used carefully.  If possible, use one of the above wrapper routines instead.
//
//
// Arguments:	objid		The object to which a handler is to be attached
//				eventname	The event to handle (e.g., onclick). All lower case, with the 'on'
//				position	As there may already be event handlers attached to this event on this object,
//							the position of this handler relative to the others.  Values are:
//							'f' or 'First'		-- make it the first handler to run
//							'l' or 'Last'		-- make it the last handler to run
//							'p' or 'Previous'	-- make it run before any handler the user had
//							'n' or 'Next'		-- make it run after any handler the user had
//							Note that the above are relative.  If you make your handler first then
//							someone else makes their handler first after you, their handler is 'first'
//				handler		The handler to run when the event fires, can be a function (preferred)
//							or a string containing the code to run (which will be wrapped into a anonymous
//							function).  Note that neither allow arguments to be passed to the handler
//							beyond the normal this and event arguments a handler gets by default.
//				The following two args can currently only be used on property monitors
//				Note that only one monitor can be set per object
//				handler1	Available to the "handler".  Allows the handler to be generic code which
//							checks for certain things and then if all is OK, it can redispatch to
//							this handler.
//				args		An array of arbitrary size/content passed to handler.
//				The following must be used with care:
//				MozDown		Instead of adding a regular handler in Mozilla, add the event handler
//							so that the event is caught on the "bubble down" phase.  Param
//							is ignored in IE (where only a "regular" handler is installed).
HxG_5.prototype.attachEvent = function (objid, eventname, position, handler, handler1, args, MozDown) {
	if (objid && eventname && handler) {
		var rValue = null, obj = this.getElementById(objid);
		var mozD = (document.addEventListener!=undefined && (MozDown || eventname==hX_5._onPC));
		if (obj) {
			// See if we already have an event object for this guy
			var srcid = this.i.dispatchToID(obj), userObj = hX_5.lookupEvent (hX_5._uR, srcid, eventname), skip=false;
			// No event handler exists for this object, so create one
			if (userObj == null) userObj = hX_5.addNewEvent (srcid, obj, eventname, hX_5._uR, handler1, args, mozD);
			// Add the event to the right place in the stack
			if (mozD) {
				skip   = userObj.chkEvtL (MozDown,  handler);
				rValue = userObj.addEvtL (MozDown,  handler);
			} else if (position == hX_5._lF || position == "first")	rValue = userObj.addPre  (hX_5._lF, handler);
			else if (position == hX_5._lP || position == "previous")rValue = userObj.addPre  (hX_5._lL, handler);
			else if (position == hX_5._lN || position == "next")	rValue = userObj.addPost (hX_5._lF, handler);
			else													rValue = userObj.addPost (hX_5._lL, handler);
			// Reslam the event handler in the object to redirect to our handler
			userObj.tgtObj = (srcid == hX_5._WDW)? window:((srcid == hX_5._BODY) ? document.body : obj);
			if (this.brw.isIE())
				userObj.tgtObj.setAttribute(eventname, hX_5.runEvent);
			else {
				// Always slam the event in Mozilla
				if (userObj.tgtObj!=window)
					userObj.tgtObj.setAttribute(eventname, "return(hX_5.runEvent(this, event));");
				// ALSO, for some cases, we need an event listener as well.  Note that we are directly
				// overwriting any user-supplied listener in this case as there is no way to find out
				// if the user already had one attached
				if (!skip)	{ // listener isn't attached yet
					if (eventname == hX_5._onPC) {
					// If we're monitoring a value, Mozilla uses a different monitor
					// Note that this switch only works if only ONE thing is monitored -- the value
						userObj.tgtObj.addEventListener((args && args.length && args[0]==hX_5._VL)?hX_5._onDOMVPC:hX_5._onDOMPC, hX_5.runEvent1, false);
					} else if (MozDown) {
						var eName = (eventname.indexOf(hX_5._ON) == 0) ? eventname.substr(2) : eventname;
						userObj.tgtObj.addEventListener(eName, hX_5.runEvent2, true);
					}
				}
			}
			return rValue;
		}
	}
	return null;
}

//----------------------------------------------------------------------------------------------------
// Public:	Remove an event handler from a DOM object
//
// Arguments:	obj			(string tagname or object) Identifies the object
//				eventname	(string) which event handler to remove
//				which	 	If omitted or null , all JSF handlers are removed, otherwise
//							the id returned by attachEvent
//				destroy		If which not set, then if true, also destroy
//							the user event and all other handling for this eventname
HxG_5.prototype.removeEvent = function (objid, eventname, which, destroy) {
	if (objid && eventname) {
		var obj = this.getElementById(objid);
		if (obj) {
			var srcid = this.i.dispatchToID(obj), userObj = hX_5.lookupEvent (hX_5._uR, srcid, eventname);
			// If we found the object, remove the event(s)
			if (userObj != null) {
				if (arguments.length > 2 && which!=null) {
					if (hX_5.isNumber(which)) userObj.remove(which);
				} else {
					userObj.remove(0, destroy);
				}
				return true;
			}
		}
	}
	return false;
}

// --------------------------------------------------------
// Public: HxBrowserMgr -- Browser capability sniffer
// --------------------------------------------------------

//----------------------------------------------------------------------------------------------------
// Public:	Browser object -- use the functions below to query what browser this is and/or what
// 			capabilities it has
HxG_5.prototype.HxBrowserMgr = function () {

	this.m_browserLevel		= 3;
	this.m_browserName		= 'Netscape';
	this.m_browserVersion	= 'Unknown';
	this.m_clientPlatform	= 'Unknown';
	this.m_JSVersion		= 1.0;

	this.m_hasStyleSheet	= false;
	this.m_hasActiveX		= true;
	this.m_hasIECacheBug	= false;
	this.m_hasDOM			= false;
	this.m_hasDOM1			= false;
	this.m_hasDOM2			= false;
	this.m_hasLayers		= false;
	this.m_hasPersistentStore= false;
	this.m_hasInnerHTML		= true;

	this.m_isIE				= false;
	this.m_isNavigator		= false;
	this.m_isFirefox		= false;
	this.m_isOpera			= false;
	this.m_isKonqueror		= false;
	this.m_isMozilla		= false;
	
	this.m_isW3C 			= false;
	this.m_isW3C_STD 		= false;
	this.m_isIE6_CSS 		= false;
	this.m_isIE_XP			= false;
	this.m_isIXP			= false;

	this.isBrowserLevel			= function(neededBrowserLevel) { return (neededBrowserLevel < (this.m_browserLevel+1)); }
	this.getRealAppVersion		= function() { return this.m_realAppVersion; }
	this.hasIEZindexLimitation	= function() { return (this.m_isIE && this.m_realAppVersion < 5.5); }

	this.getJSVersion		= function() { return this.m_JSVersion; }

	this.hasActiveX			= function() { return this.m_hasActiveX; }
	this.hasIECacheBug		= function() { return this.m_hasIECacheBug; }
	this.hasStyleSheet		= function() { return this.m_hasStyleSheet; }
	this.hasDOM				= function() { this.m_isIE && this.isBrowserLevel(4); }
	this.hasDOM1			= function() { return this.m_hasDOM1; }
	this.hasDOM2			= function() { return this.m_hasDOM2; } 
	this.hasInnerHTML		= function() { return this.m_hasInnerHTML; }

	this.isIE				= function() { return this.m_isIE; }
	this.isNetscape			= function() { return this.m_isNavigator; }
	this.isFirefox			= function() { return this.m_isFirefox; }
	this.isOpera			= function() { return this.m_isOpera; }
	this.isKonqueror		= function() { return this.m_isKonqueror; }
	this.isMozilla			= function() { return this.m_isMozilla; }
	this.isNavOrMoz			= function() { return (this.m_isMozilla || this.m_isNavigator); }

	this.isNetscape4		= function() { return (this.m_isNavigator && this.m_realAppVersion >= 4.0); }
	this.hasLayers			= function() { return this.m_hasLayers; }

	this.isIE5				= function() { return (this.m_isIE && this.m_realAppVersion >= 5.0); }
	this.isIE5_5			= function() { return (this.m_isIE && this.m_realAppVersion >= 5.5); }
	this.isIE6_CSS			= function() { return (this.m_isIE6_CSS); }

	this.isW3C				= function() { return (this.m_isW3C); }
	this.isW3Cstd			= function() { return (this.m_isW3C_STD); }
	
	this.isPlatformMac		= function() { return ((this.m_clientPlatform.indexOf("Mac") != -1) ? true : false); }
	this.isPlatformWin		= function() { return ((this.m_clientPlatform.indexOf("Win") != -1) ? true : false); }
	this.isPlatformIEWinXP	= function() { return (this.m_isIE_XP); }
	this.isPlatformWinXP	= function() { return (this.m_isXP); }
	
	this.inited = false;
}

//----------------------------------------------------------------------------------------------------
// Implementation
HxG_5.prototype.HxBrowserMgr.prototype.initClientBrowserObject = function (parent) {
	
	if (this.inited)
		return;
	this.p = parent;
	this.m_browserLevel	= parseInt (navigator.appVersion.charAt(0), 10);

	if (navigator.appName!=undefined)
		this.m_browserName = navigator.appName;
	if (navigator.userAgent!=undefined)
		this.m_browserVersion = navigator.userAgent;

	if (navigator.platform!=undefined)
		this.m_clientPlatform = navigator.platform;

	this.m_isIE = (this.m_browserName == "Microsoft Internet Explorer");
	this.m_isNavigator = (this.m_browserName == "Netscape");
	this.m_isOpera = (this.m_browserName == "Opera");
	this.m_isKonqueror = (this.m_browserName == "Konqueror");
	this.m_isMozilla = (window.navigator != null ) ? ( window.navigator.userAgent.indexOf("ecko") != -1 ) : false;
	if (this.m_isMozilla) {
		this.m_isNavigator = false;
		this.m_isFirefox = (window.navigator != null ) ? ( window.navigator.userAgent.indexOf("irefox") != -1 ) : false;
	}
	this.m_realAppVersion = parseFloat( this.m_isIE ? navigator.appVersion.substr(navigator.appVersion.indexOf("MSIE") + 4) : navigator.appVersion);

	this.m_hasLayers = this.m_isNavigator && this.m_browserLevel >= 4 && this.m_browserLevel < 5;

	if( window.gCookie && gCookie.bs && gCookie.bs == hX_5._E ) {
		this.m_isIE = true;
		this.m_isNavigator = false;
		this.m_browserLevel = 5;
		this.m_realAppVersion = 5.0;
	}
	if( window.gCookie && gCookie.bs && gCookie.bs == "_NN4" ) {
		this.m_isIE = false;
		this.m_isNavigator = true;
		this.m_browserLevel = 4;
		this.m_realAppVersion = 4.7;
	}

	this.m_hasIECacheBug		= this.m_isIE;
	this.m_hasStyleSheet		= this.isBrowserLevel(4);
	this.m_hasActiveX			= this.m_isIE && (navigator.appVersion.indexOf( "Win") != -1);
	this.m_hasPersistentStore	= this.m_isIE && this.m_realAppVersion > 5 && this.isPlatformWin();
	
	if ( window.document.implementation != null) {
		this.m_hasDOM1 	=	window.document.implementation.hasFeature("HTML","1.0");
		this.m_hasDOM2 	=	window.document.implementation.hasFeature("HTML","2.0") &&
							window.document.implementation.hasFeature("Events","2.0") &&
							window.document.implementation.hasFeature("Core","2.0") &&
							window.document.implementation.hasFeature("CSS2","2.0");
	}

	// See if innerHTML is supported or not
	if (!this.isIE()) {
		var testIH = document.createElement("span");
		if (testIH && testIH.innerHTML!=undefined) {
			try {
				testIH.innerHTML = "foo";
				this.m_hasInnerHTML = (testIH.innerHTML == "foo");
			} catch (e) {
				this.m_hasInnerHTML = false;
			}
		} else
			this.m_hasInnerHTML = false;
	}

	if (this.isPlatformWin() && (navigator.appVersion!=undefined) && (navigator.userAgent!=undefined) ) {
		var str=((this.m_isIE) ? navigator.appVersion : navigator.userAgent), ix=str.indexOf("Windows NT ");
		if (ix >= 0 && ix+11 < str.length) {
			str = str.substr(ix+11,3);
			if (str.length > 0) {
				var snum = new Number(str);
				this.m_isXP = (snum >= 5.1);
				this.m_isIE_XP = (this.m_isIE && this.m_isXP);
			}
		}
	}

	if (document.images) {
		this.m_isW3C = (document.body && document.body.style && document.getElementById) ? true : false;
		if (this.m_isW3C) {
			this.m_isW3C_STD = (this.isNavOrMoz()) ? true : ((document.compatMode && document.compatMode.indexOf("CSS1") >= 0) ? true : false);
			this.m_isIE6_CSS = (this.m_isIE && document.compatMode && document.compatMode.indexOf("CSS1") >= 0)? true : false;
		}
	}
}

// --------------------------------------------------------
// Public HxGeometryMgr -- Manage geometry calcuations
// --------------------------------------------------------

HxG_5.prototype.HxGeometryMgr = function (thisparent) {
	this.p = thisparent;
	this.p.geo = this;		// For older versions of hxclient
}

//----------------------------------------------------------------------------------------------------
// Public: Get the absolute Top corner of an element (in pixels)
HxG_5.prototype.HxGeometryMgr.prototype.getAbsPosTop = function(obj) {
	var rValue = 0, elem = this.p.getElementById(obj);
	if (hX_5.brw.isIE6_CSS()) {
		while (elem) {
			rValue += elem.offsetTop;
			if (elem.tagName!=hX_5._BBODY && elem.tagName!=hX_5._HHTML) rValue -= elem.scrollTop;
			elem = elem.offsetParent;
		}
	} else if (elem.offsetParent) {
		while(elem.offsetParent) {
			rValue += elem.offsetTop;
			// Firefox is a bit screwy here.  At least for the case where we embed something like
			// a table inside a scrolling div, the div doesn't show up as "offsetparent".
			// So we never get it's scroll offset.  The following hack is sufficient
			// for scrolling data tables
			if (hX_5.brw.isNavOrMoz) {
				if (elem.parentNode != elem.offsetParent && elem.parentNode.scrollTop!=0)
					rValue -= elem.parentNode.scrollTop;
				else
					rValue -= elem.scrollTop;
			} else
				rValue -= elem.scrollTop;
			elem = elem.offsetParent;
		}
	} else if (elem.y)
		rValue += elem.y;
	return rValue;
}

// Public: Get the absolute Left corner of an element (in pixels)
HxG_5.prototype.HxGeometryMgr.prototype.getAbsPosLeft = function(obj) {
	var rValue = 0, elem = this.p.getElementById(obj);
	if (hX_5.brw.isIE6_CSS()) {
		while (elem) {
			rValue += elem.offsetLeft;
			if (elem.tagName!=hX_5._BBODY && elem.tagName!=hX_5._HHTML) rValue -= elem.scrollLeft;
			elem = elem.offsetParent;
		}
	} else if (elem.offsetParent) {
		while(elem.offsetParent) {
			rValue += elem.offsetLeft;
			// Firefox is a bit screwy here -- see above.
			if (hX_5.brw.isNavOrMoz) {
				if (elem.parentNode != elem.offsetParent && elem.parentNode.scrollLeft!=0)
					rValue -= elem.parentNode.scrollLeft;
				else
					rValue -= elem.scrollLeft;
			} else {
				rValue -= elem.scrollLeft;
			}
			elem = elem.offsetParent;
		}
	} else if (elem.x)
		rValue += elem.x;
	return rValue;
}

//----------------------------------------------------------------------------------------------------
// Public: Get the relative positioning context for an absolutely positioned element
HxG_5.prototype.HxGeometryMgr.prototype.getRelPos = function(obj) {
	return ((obj.offsetParent)?obj.offsetParent:null);
}

//----------------------------------------------------------------------------------------------------
// Public: Get the relative LEFT corner of an element (in pixels)
HxG_5.prototype.HxGeometryMgr.prototype.getRelPosLeft = function(obj) {
	var elem = this.p.getElementById(obj);
	return (elem.offsetLeft);
}	

//----------------------------------------------------------------------------------------------------
// Public: Get the relative Top corner of an element (in pixels)
HxG_5.prototype.HxGeometryMgr.prototype.getRelPosTop = function(obj) {
	var elem = this.p.getElementById(obj);
	return (elem.offsetTop);
}

//----------------------------------------------------------------------------------------------------
// Public:	Get the rendered size of an element. NOTE: Result is browser specific
HxG_5.prototype.HxGeometryMgr.prototype.getRendHeight = function (obj) {
	var elem = this.p.getElementById(obj);
	return parseInt(elem.offsetHeight, 10);
}

HxG_5.prototype.HxGeometryMgr.prototype.getRendWidth = function (obj) {
	var elem = this.p.getElementById(obj);
	return parseInt(elem.offsetWidth, 10);
}

//----------------------------------------------------------------------------------------------------
// Public:	Parse a border style statement and convert keywords to pixels
HxG_5.prototype.HxGeometryMgr.prototype.parseBorder = function (obj, which, defwidth) {
	var width = defwidth, bname = "border-" + which + "-style";
	if ((this.getP(obj, bname)) == hX_5._NO) {
		width = 0;
	} else {
		bname = "border-" + which + "-width";
		var value = this.getP(obj, bname);
		width = (value == hX_5._E) ? defwidth : ((value == "thin") ? 2 : ((value == "medium") ? 3 : ((value == "thick") ? 4 : hX_5.toPixels(obj,value))));
	}
	return width;
}

//----------------------------------------------------------------------------------------------------
// Public:	Check for the oddball Win XP definition of a "system" border on IE only
HxG_5.prototype.HxGeometryMgr.prototype.testIEXPBorder = function (obj) {
	// There is no truly reliable way of testing this. Essentially, the user sets
	// no background color, border width or border style
	// XP reports back the background color resolved to the value of "buttonface"
	// the style as "none" and the width as "2px".
	// We can't resolve "buttonface" ourselves so we just test the other pattern
	return ((hX_5.brw.isPlatformIEWinXP())?((hX_5.g.getP(obj, hX_5._BRSTY) == hX_5._NO && hX_5.g.getP(obj, hX_5._BRWID) == hX_5._2PX)):false);
}

//----------------------------------------------------------------------------------------------------
// Public:	Check for the oddball Win XP definition of a system border (IE and Mozilla)
HxG_5.prototype.HxGeometryMgr.prototype.testXPBorder = function (obj, style) {
	var rvalue = false;
	// There is no truly reliable way of testing this. Essentially, the user sets
	// no background color, border width or border style
	// XP reports back the background color resolved to the value of "buttonface"
	// the style as "none" and the width as "2px".
	// We can't resolve "buttonface" ourselves so we just test the other pattern
	if (hX_5.brw.isPlatformWinXP()) {
		if (hX_5.brw.isIE()) {
			rvalue = (hX_5.g.getP(obj, hX_5._BRSTY) == hX_5._NO && hX_5.g.getP(obj, hX_5._BRWID) == hX_5._2PX);
		} else {
			var a=hX_5.g.getP(obj,hX_5._BRLSY),  b=hX_5.g.getP(obj,hX_5._BRRSY), c=hX_5.g.getP(obj,hX_5._BRTSY), d=hX_5.g.getP(obj,hX_5._BRBSY);
			rvalue = (a==b && b==c && c==d && a==style);
			if (rvalue) {
				a = hX_5.g.getP(obj,hX_5._BRLWD);
				b = hX_5.g.getP(obj,hX_5._BRRWD);
				c = hX_5.g.getP(obj,hX_5._BRTWD);
				d = hX_5.g.getP(obj,hX_5._BRBWD);
				rvalue = (a==b && b==c && c==d && a==hX_5._2PX);
			}
			if (rvalue) {
				a = hX_5.g.getP(obj,hX_5._BRLCL);
				b = hX_5.g.getP(obj,hX_5._BRRCL);
				c = hX_5.g.getP(obj,hX_5._BRTCL);
				d = hX_5.g.getP(obj,hX_5._BRBCL);
				var e = hX_5.getEvalStyle(obj,hX_5._COR,hX_5.CLR_BTN);
				rvalue = (a==b && b==c && c==d && a==e);
			}
		}
	}
	return rvalue;
}

//----------------------------------------------------------------------------------------------------
// Public:	Get width of the browser window (interior) in pixels
HxG_5.prototype.HxGeometryMgr.prototype.getWindowClientWidth = function () {
	if (self.innerWidth) {
		// This is Mozilla.  Mozilla has mucked around with getting "innerWidth" right across multiple versions
		// We'll return innerWidth, but if we think it includes the scroll bar, we'll subtract it out
		var sb = hX_5.getScrollbarWidth()
		if (document.body.clientWidth!=undefined && document.body.clientWidth!=null && document.body.clientWidth > 0 && document.body.clientWidth==self.innerWidth-sb)
			return self.innerWidth-sb;
		else
			return self.innerWidth;
	} else if (document.documentElement && document.documentElement.clientWidth)
		return document.documentElement.clientWidth;
	else if (this.p.brw.isIE6_CSS() && document.body.parentElement.clientWidth)
		return document.body.parentElement.clientWidth;
	else if (document.body && document.body.clientWidth)
		return document.body.clientWidth;
	return 0;
}

//----------------------------------------------------------------------------------------------------
// Public:	Get height of the browser window (interior) in pixels
HxG_5.prototype.HxGeometryMgr.prototype.getWindowClientHeight = function () {
	if (self.innerHeight) {
		// This is Mozilla.  Mozilla has mucked around with getting "innerHeight" right across multiple versions
		// We'll return "innerHeight" but we'll see if we think there's a scroll bar on in which case
		// we'll return reduce it by the scroll bar size
		var sb = hX_5.getScrollbarWidth()
		if (document.body.clientHeight!=undefined && document.body.clientHeight!=null && document.body.clientHeight > 0 && document.body.clientHeight==self.innerHeight-sb)
			return self.innerHeight-sb;
		else
			return self.innerHeight;
	} else if (document.documentElement && document.documentElement.clientHeight)
		return document.documentElement.clientHeight;
	else if (this.p.brw.isIE6_CSS() && document.body.parentElement.clientHeight)
		return document.body.parentElement.clientHeight;
	else if (document.body && document.body.clientHeight)
		return document.body.clientHeight;
	return 0;
}

//----------------------------------------------------------------------------------------------------
// Public:	Get how much the page has scrolled (scrolling offset) - horizontally
HxG_5.prototype.HxGeometryMgr.prototype.getWinScrollOffsetX = function () {
	if (self.pageXOffset)
		return self.pageXOffset;
	else if (document.documentElement && document.documentElement.scrollLeft)
		return document.documentElement.scrollLeft;
	else if (document.body)
		return document.body.scrollLeft;
	return 0;
}

//----------------------------------------------------------------------------------------------------
// Public:	Get how much the page has scrolled (scrolling offset) - vertically
HxG_5.prototype.HxGeometryMgr.prototype.getWinScrollOffsetY = function () {
	if (self.pageYOffset)
		return self.pageYOffset;
	else if (document.documentElement && document.documentElement.scrollTop)
		return document.documentElement.scrollTop;
	else if (document.body)
		return document.body.scrollTop;
	return 0;
}

//----------------------------------------------------------------------------------------------------
// Public:	Get the width of the page (the content)
HxG_5.prototype.HxGeometryMgr.prototype.getPageWidth = function () {
	if (hX_5.brw.isIE() || (document.body.scrollWidth > document.body.offsetWidth))
		return document.body.scrollWidth;
	else 
		return document.body.offsetWidth;
}

//----------------------------------------------------------------------------------------------------
// Public:	Get the width of the page (the content)
HxG_5.prototype.HxGeometryMgr.prototype.getPageHeight = function () {
	if (hX_5.brw.isIE() || (document.body.scrollHeight > document.body.offsetHeight))
		return document.body.scrollHeight;
	else 
		return document.body.offsetHeight;
}

//----------------------------------------------------------------------------------------------------
// Public: Get the actual (effective) value of a style property, that is, style after CSS is applied to the object
HxG_5.prototype.HxGeometryMgr.prototype.getP = function (obj, prop1, prop2) {
	var p, s, r=null, o=this.p.getElementById(obj);
	if (o) {
		// IE
		if (o.currentStyle) {
			p = (prop2) ? prop2 : this.CSStagToDOMtag (prop1);
			r = o.currentStyle.getAttribute(p);
		// W3C
		} else if (window.getComputedStyle) {
			s = window.getComputedStyle(o, hX_5._E);
			// "z-index" may have been misnamed in some versions of Mozilla
//			p = (prop1==hX_5._ZND)?this.CSStagToDOMtag(prop1):prop1;
			r = s.getPropertyValue(prop1);
		}
	}
	return (r);
}

//----------------------------------------------------------------------------------------------------
// Public: Set the value of a style property if the new value differs from the old
HxG_5.prototype.HxGeometryMgr.prototype.setP = function (obj, prop, newV) {
	var p, o=hX_5.getElementById(obj);
	if (o) {
		if (o.currentStyle) {
			p = this.CSStagToDOMtag(prop);
			if (o.style.getAttribute(p) != newV) o.style.setAttribute(p, newV);
		} else {
			// "z-index" may have been misnamed in some versions of Mozilla
//			p = (prop==hX_5._ZND)?this.CSStagToDOMtag(prop):prop;
			if (o.style.getPropertyValue(prop) != newV) o.style.setProperty(prop, newV, hX_5._E);
		}
	}
}

// Public: Copy the style property value from one object to another
// N.B. Copies the effective style, not the dom obj style (i.e., the style computed by
// combining the DOM obj's style with any CSS in force).
HxG_5.prototype.HxGeometryMgr.prototype.copyP = function (src, dst, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9) {
	var x, s, l=arguments.length;
	for (x=2; x<l; x++) {
		s = hX_5.g.getP(src, arguments[x]);
		hX_5.g.setP(dst, arguments[x], s);
	}
}

//----------------------------------------------------------------------------------------------------
// Helper Implementation: Convert a CSS tag name to a DOM tag name
HxG_5.prototype.HxGeometryMgr.prototype.CSStagToDOMtag = function(value) {
	var w=value, q=w.indexOf(hX_5._Y);
	while (q >= 0) {
		w = w.substring(0, q) + (w.substring(q+1, q+2)).toUpperCase() + w.substring(q+2);
		q = w.indexOf(hX_5._Y);
	}
	return w;
}

// Public: Given a DOM object, find a parent object that matches a containment pattern
HxG_5.prototype.HxGeometryMgr.prototype.findContainer = function (obj, p0, p1, p2, p3, p4, p5, p6, p7, p8, p9) {
	var i, p=obj;
	// skip 'up' until we find something that matches p0
	while (p && p.tagName && p.tagName.toLowerCase() != p0.toLowerCase()) 
		p = (p.parentNode) ? p.parentNode : null;
	// Match the rest of the pattern
	while (i <= arguments.length-2 && p && p.tagName && p.tagName.toLowerCase() == arguments[i].toLowerCase()) {
		p = (p.parentNode) ? p.parentNode : null;
		i++;
	}
	p = (p && p.tagName && p.tagName.toLowerCase() == arguments[arguments.length-1].toLowerCase()) ? p : null;
	return p;
}

// Public: Given a DOM object, determine if it is disabled (directly or indirectly)
HxG_5.prototype.HxGeometryMgr.prototype.isDisabled = function (obj) {
	var o=hX_5.getElementById(obj), p=o.parentNode, r=o.disabled;
	if (o.isDisabled!=undefined)
		r = o.isDisabled;
	else {
		while (!r && p!=null) {
			r = (p.disabled!=undefined)?p.disabled:false;
			p = p.parentNode;
		}
	}
	return r;
}

// --------------------------------------------------------
// Private: HxResourceMgr -- Manages accessing resources
// --------------------------------------------------------

// Small cache of calculated sizes and stuff
HxG_5.prototype.HxCache = function () {
	this.L = [];				// Lengths of date strings (by locale)
	this.BIGNUM_converter		= null;
	this.STDDTP_converter		= null;
}
HxG_5.prototype.HxLCache = function () {
	// Max lengths of calendarstrings (one for each main calendar type - Gregorian, Hijiri, Hebrew)
	this.MAX_STR_DATE_MONTHS	= [];
	this.MAX_STR_DATE_SHMONTHS	= [];
	this.MAX_STR_DATE_DAYS		= [];
	this.MAX_STR_DATE_SHDAYS	= [];
}

HxG_5.prototype.HxCache.prototype.strInit = function(l) {
	if (hX_5.c.L[l]==null) hX_5.c.L[l] = new hX_5.HxLCache();
	if (hX_5.c.L[l].MAX_STR_DATE_MONTHS.length==0) {
		var i,j,k;
		// Calc the max length of day and month names and the minimum unique length of a (long) month name
		for (i=0;i<3;i++) {
			hX_5.c.L[l].MAX_STR_DATE_SHDAYS[i] = hX_5.c.L[l].MAX_STR_DATE_DAYS[i] = hX_5.c.L[l].MAX_STR_DATE_SHMONTHS[i] = hX_5.c.L[l].MAX_STR_DATE_MONTHS[i] = 0;
			for (j=0;j<hX_5.L[l].STRA_DATE_SHDAYS[i].length;j++) hX_5.c.L[l].MAX_STR_DATE_SHDAYS[i] = Math.max(hX_5.c.L[l].MAX_STR_DATE_SHDAYS[i], hX_5.L[l].STRA_DATE_SHDAYS[i][j].length);
			for (j=0;j<hX_5.L[l].STRA_DATE_DAYS[i].length;j++) hX_5.c.L[l].MAX_STR_DATE_DAYS[i] = Math.max(hX_5.c.L[l].MAX_STR_DATE_DAYS[i], hX_5.L[l].STRA_DATE_DAYS[i][j].length);
			for (j=0;j<hX_5.L[l].STRA_DATE_SHMONTHS[i].length;j++) hX_5.c.L[l].MAX_STR_DATE_SHMONTHS[i] = Math.max(hX_5.c.L[l].MAX_STR_DATE_SHMONTHS[i], hX_5.L[l].STRA_DATE_SHMONTHS[i][j].length);
			for (j=0;j<hX_5.L[l].STRA_DATE_MONTHS[i].length;j++) hX_5.c.L[l].MAX_STR_DATE_MONTHS[i] = Math.max(hX_5.c.L[l].MAX_STR_DATE_MONTHS[i], hX_5.L[l].STRA_DATE_MONTHS[i][j].length);
		}
	}
}

// Return the resource manager at the bottom of the stack
// Create it if the stack is empty
HxG_5.prototype.currentResource = function () {
	if (hX_5.res != null) {
		if (hX_5.res.length <= 0) {
			hX_5.res[0] = new hX_5.HxResourceMgr(hX_5);
			hX_5.res[0].setAbsolutePaths();
		}
		return (hX_5.res[hX_5.res.length-1]);
	}
	return null;
}

HxG_5.prototype.HxResourceMgr = function (thisparent) {
	this.p = thisparent;
	
	this.bPathSet		= false;
	this.jsfRootToken	= "/faces";
	this.protocol		= "http://";
	this.baseLibPath	= ".ibmjsfres/";
	this.baseImgPath	= this.baseLibPath + "img/";
	this.baseCssPath	= this.baseLibPath + "css/";
	this.baseDlgPath	= this.baseLibPath + "dlg/";

	this.getImgPath = function() { return this.baseImgPath; }
	this.getCssPath = function() { return this.baseCssPath; }
	this.getDlgPath = function() { return this.baseDlgPath; }	
	
	// Standard image cache -- constants used to access it
	this.p.R_IMG_CNT = 7;
	this.p.R_IMG_SPACER = 0;
	this.p.R_IMG_HINKIES = 1;
	this.p.R_IMG_LG_HINKIES = 2;
	this.p.R_IMG_VLG_HINKIES = 3;
	this.p.R_IMG_CAL_HINKIES = 4;
	this.p.R_IMG_COL_HINKIES = 5;
	this.p.R_IMG_TIME_HINKIES = 6;
	// Definition of images
	this.RF_IMGS = [["1x1.gif", 1, 1, 0, 0, 0],
					["hinkies_v3_0_4.gif", 16, 96, 2, 3, 4],
					["hinkies_lg_v3_0_4.gif", 24, 24, 2, 2, 1],
					["hinkies_vlg_v3_0_4.gif", 24, 24, 2, 2, 1],
					["HinkiesC_v3_0_4.gif", 24, 156, 2, 12, 1],
					["HinkiesCl_v3_0_4.gif", 24, 156, 2, 12, 1],
					["HinkiesT_v3_0_4.gif", 24, 156, 2, 12, 1]];
	// Array of cached images
	this.images	= [];
}

//----------------------------------------------------------------------------------------------------
// Private: recache all images, ... using the current pathname(s)
HxG_5.prototype.HxResourceMgr.prototype.recache = function () {
	for (var i = 0; i < this.p.R_IMG_CNT; i++) {
		this.images[i] = new Image(this.RF_IMGS[i][1], this.RF_IMGS[i][2]);
		this.images[i].src = this.getImgPath() + this.RF_IMGS[i][0];
	}
}

//----------------------------------------------------------------------------------------------------
// Private: Return URI/URL to a cached image
HxG_5.prototype.HxResourceMgr.prototype.getURI = function (img) {
	return hX_5._URL + hX_5._SQT + this.images[img].src + hX_5._SQT + ")";
}
HxG_5.prototype.HxResourceMgr.prototype.getURL = function (img) {
	return this.images[img].src;
}

//----------------------------------------------------------------------------------------------------
// Private:	Given a (background) image made up of multiple images, return the offset information
// needed to put one of the images on display in the background
HxG_5.prototype.HxResourceMgr.prototype.wellPosition = function (id, row, col, grp) {
	var cellX = (this.RF_IMGS[id][1]/this.RF_IMGS[id][3]), cellY=(this.RF_IMGS[id][2]/(this.RF_IMGS[id][4]*this.RF_IMGS[id][5]));
	return 	((-(col * cellX)) + hX_5._PX + hX_5._B + (-((((grp)?grp:0)*(cellY*this.RF_IMGS[id][4]))+(row*cellY))) + hX_5._PX);
}

//----------------------------------------------------------------------------------------------------
// Public:	Create a new instance of a resource object using an absolute path to a resource server
HxG_5.prototype.setResourceServer = function(path) {
	var lchar, cr = this.res.length;
	this.res[cr] = new this.HxResourceMgr(this);
	lchar = path.charAt(path.length-1);
	path = (lchar == "/") ? path : path + "/";
	this.res[cr].baseImgPath = path + "img/";
	this.res[cr].baseCssPath = path + "css/";
	this.res[cr].baseDlgPath = path + "dlg/";
	this.res[cr].bPathSet = true;
	this.res[cr].recache();
}

//----------------------------------------------------------------------------------------------------
// Helper Implementation: Set the absolute path for all resources
HxG_5.prototype.HxResourceMgr.prototype.setAbsolutePaths = function() {
	if (this.bPathSet == false) {
		var sPathPrefix = this.getAbsolutePathPrefix(this.jsfRootToken);
		this.baseImgPath = sPathPrefix + this.baseLibPath + "img/";
		this.baseCssPath = sPathPrefix + this.baseLibPath + "css/";
		this.baseDlgPath = sPathPrefix + this.baseLibPath + "dlg/";
		this.bPathSet = true;
		this.recache();
	}
}

/* Currently not used
HxG_5.prototype.HxResourceMgr.prototype.setRelativePaths = function() {
	if (this.bPathSet == false) {
		var sPage2Root = this.getRelativePathPrefix(this.jsfRootToken);
		this.baseImgPath = sPage2Root + this.baseImgPath;
		this.baseCssPath = sPage2Root + this.baseCssPath;
		this.baseDlgPath = sPage2Root + this.baseDlgPath;
		this.bPathSet = true;
		this.recache();
	}
}

// Helper Implementation:	Build the (relative) path to the JSF stuff
HxG_5.prototype.HxResourceMgr.prototype.getRelativePathPrefix = function (sToken) {
	if (this.p.isUnDef(typeof sToken) || sToken == "")
		sToken = this.jsfRootToken;
	
	var sLocation	= new String(document.location);
	var sPage2Root	= "";
	var iCursor	= sLocation.indexOf(sToken, 0);
	
	if (iCursor != -1) {
		iCursor += sToken.length;
		var sTailPath = sLocation.slice(iCursor+1);
		for (var i = 0; i < sTailPath.length; i++) {
			if (sTailPath.charAt(i) == "/") {
				sPage2Root += "../";
			}
		}
	}
	else {
		iCursor = sLocation.indexOf(this.protocol, 0);
		if (iCursor != -1) {
			iCursor += this.protocol.length;
			var i,j;
			for (i = iCursor, j= 0; i < sLocation.length; i++) {
				if (sLocation.charAt(i) == "/")
					j++;
			}
			if (j > 2) {
				while (j-- > 2)
					sPage2Root += "../";
			}	
		}	
	}
	return sPage2Root;
}
*/

//----------------------------------------------------------------------------------------------------
// Helper Implementation:	Build the absolute path to the JSF stuff
HxG_5.prototype.HxResourceMgr.prototype.getAbsolutePathPrefix = function (sToken) {

	if (this.p.isUnDef(typeof sToken) || sToken == hX_5._E) sToken = this.jsfRootToken;
	
	var sLocation=new String(document.location), sPathPrefix=hX_5._E, iCursor=sLocation.indexOf(sToken, 0) - 1;
	var i, sl="/";
	
	if (iCursor < 0) {
		/* Didn't find token - no "faces" in location */
		iCursor = sLocation.indexOf(this.protocol, 0);
		if (iCursor != -1) {
			var sTailPath = sLocation.slice(iCursor + this.protocol.length);
			for (i = 0; i < sTailPath.length; i++) {
				if (sTailPath.charAt(i) == sl) {
					sTailPath = sTailPath.slice(i);
					for (i = 1; i < sTailPath.length; i++) {
						if (sTailPath.charAt(i) == sl) {
							sPathPrefix = sTailPath.substring(0, i + 1);
							break;
						}
					}
				}
			}
		}
	} else {
		/* rewind thru the location string for the start of the project dir */
		var bFoundProjectDir = false;
		for (i = iCursor; i >= 0; i--) {
			if (sLocation.charAt(i) == sl) {
				bFoundProjectDir = true;
				break;
			}
		}
		if (bFoundProjectDir == true) {
			/* for "http://servername:port#/project_name/.ibmjsfres/..." do next line */
			// sPathPrefix = sLocation.substring(0, i + (iCursor - i) + 1 + 1);

			/* for "/project_name/.ibmjsfres/..." do next line */
			sPathPrefix = sLocation.substring(i, i + (iCursor - i) + 1 + 1);

			/* for "/project_name/faces/.ibmjsfres/..." do next line */
			// sPathPrefix = sLocation.substring(i, i + (iCursor - i) + sToken.length + 1 + 1);
		}
	}
	return sPathPrefix;
}

//----------------------------------------------------------------------------------------------------
// Public:	Set the locale that will be used when creating new components/behaviors/converters, ...
// 			This locale will be used until another call is made.
//			If no call is made, the page's locale will be used
//
// Arguments:	l	(string) Locale identifier (ISO form), e.g., "en" or "en-GB"
HxG_5.prototype.setLocale = function(l) {
	if (hX_5.locales!=null && l!=null && l.length && l.length>0) hX_5.locales.push(l);
}

//----------------------------------------------------------------------------------------------------
// Public:	Return the last locale that was set, if no locale was set, return the identifier
// 			of the first locale that was set.
//			If no set of locale strings has been established in the page, see if an "old style"
//			non-locale-based set of strings exists in the page and set up those strings so
//			they can be used as the locale
//
HxG_5.prototype.currentLocale = function () {
	if (hX_5.locales != null && hX_5.L!=null) {
		var l = null;
		for (l in hX_5.L) { break; }
		if (hX_5.locales.length <= 0 || l==null) hX_5.createDefaultLocale();
		return (hX_5.locales[hX_5.locales.length-1]);
	}
	return null;
}

//----------------------------------------------------------------------------------------------------
// Private:	For compatibility with "old" page renderers that render strings without a locale
//			index to them, see if we can convert the old strings to a locale based one
HxG_5.prototype.createDefaultLocale = function () {
	if (hX_5.locales!=null && hX_5.L!=null) {
		// If there are locale strings on the page, but no setLocale call was made, make the call for the page
		var q=null, l=null;
		if (hX_5.locales.length<=0) {
			for (l in hX_5.L) {
				hX_5.locales.push(l);
				break;
			}
		}
		if (l==null) {
			l = "default";
			hX_5.locales.push(l);
		}
		// If no locale-based strings exist, see if we have non-locale-based strings
		for (q in hX_5.L) { break; }
		if (q==null && hX_5.STR_TIMEZONES!=undefined) {
			hX_5.L[l] = new Object();
			if (hX_5.STR_SPINNER_INCREMENT!=undefined) hX_5.L[l].STR_SPINNER_INCREMENT = hX_5.STR_SPINNER_INCREMENT;
			if (hX_5.STR_SPINNER_DECREMENT!=undefined) hX_5.L[l].STR_SPINNER_DECREMENT = hX_5.STR_SPINNER_DECREMENT;
			if (hX_5.STR_TABPANEL_SCROLLLEFT!=undefined) hX_5.L[l].STR_TABPANEL_SCROLLLEFT = hX_5.STR_TABPANEL_SCROLLLEFT;
			if (hX_5.STR_TABPANEL_SCROLLRIGHT!=undefined) hX_5.L[l].STR_TABPANEL_SCROLLRIGHT = hX_5.STR_TABPANEL_SCROLLRIGHT;
			if (hX_5.STR_SLIDER_LABEL!=undefined) hX_5.L[l].STR_SLIDER_LABEL = hX_5.STR_SLIDER_LABEL;
			if (hX_5.STR_COLOR_DN_TOGGLE!=undefined) hX_5.L[l].STR_COLOR_DN_TOGGLE = hX_5.STR_COLOR_DN_TOGGLE;
			if (hX_5.STR_TIME_DN_TOGGLE!=undefined) hX_5.L[l].STR_TIME_DN_TOGGLE = hX_5.STR_TIME_DN_TOGGLE;
			if (hX_5.STR_DATE_DN_TOGGLE!=undefined) hX_5.L[l].STR_DATE_DN_TOGGLE = hX_5.STR_DATE_DN_TOGGLE;
			if (hX_5.STR_DATE_PREVIOUS_YEAR!=undefined) hX_5.L[l].STR_DATE_PREVIOUS_YEAR = hX_5.STR_DATE_PREVIOUS_YEAR;
			if (hX_5.STR_DATE_NEXT_YEAR!=undefined) hX_5.L[l].STR_DATE_NEXT_YEAR = hX_5.STR_DATE_NEXT_YEAR;
			if (hX_5.STR_DATE_PREVIOUS_MONTH!=undefined) hX_5.L[l].STR_DATE_PREVIOUS_MONTH = hX_5.STR_DATE_PREVIOUS_MONTH;
			if (hX_5.STR_DATE_NEXT_MONTH!=undefined) hX_5.L[l].STR_DATE_NEXT_MONTH = hX_5.STR_DATE_NEXT_MONTH;
			if (hX_5.STR_TIMEZONES!=undefined) hX_5.L[l].STR_TIMEZONES = hX_5.STR_TIMEZONES;
			if (hX_5.ERR_datetime_parse_fail!=undefined) hX_5.L[l].ERR_datetime_parse_fail = hX_5.ERR_datetime_parse_fail;
			if (hX_5.ERR_epoch_name!=undefined) hX_5.L[l].ERR_epoch_name = hX_5.ERR_epoch_name;
			if (hX_5.ERR_day_name!=undefined) hX_5.L[l].ERR_day_name = hX_5.ERR_day_name;
			if (hX_5.ERR_month_name!=undefined) hX_5.L[l].ERR_month_name = hX_5.ERR_month_name;
			if (hX_5.ERR_month_range!=undefined) hX_5.L[l].ERR_month_range = hX_5.ERR_month_range;
			if (hX_5.ERR_month_rangelimit!=undefined) hX_5.L[l].ERR_month_rangelimit = hX_5.ERR_month_rangelimit;
			if (hX_5.ERR_epoch_range!=undefined) hX_5.L[l].ERR_epoch_range = hX_5.ERR_epoch_range;
			if (hX_5.ERR_hour_range_long!=undefined) hX_5.L[l].ERR_hour_range_long = hX_5.ERR_hour_range_long;
			if (hX_5.ERR_hour_range_short!=undefined) hX_5.L[l].ERR_hour_range_short = hX_5.ERR_hour_range_short;
			if (hX_5.ERR_minute_range!=undefined) hX_5.L[l].ERR_minute_range = hX_5.ERR_minute_range;
			if (hX_5.ERR_second_range!=undefined) hX_5.L[l].ERR_second_range = hX_5.ERR_second_range;
			if (hX_5.ERR_ampm_name!=undefined) hX_5.L[l].ERR_ampm_name = hX_5.ERR_ampm_name;
			if (hX_5.ERR_tzone_name!=undefined) hX_5.L[l].ERR_tzone_name = hX_5.ERR_tzone_name;
			if (hX_5.ERR_generic!=undefined) hX_5.L[l].ERR_generic = hX_5.ERR_generic;
			if (hX_5.ERR_empty!=undefined) hX_5.L[l].ERR_empty = hX_5.ERR_empty;
			if (hX_5.ERR_minimum!=undefined) hX_5.L[l].ERR_minimum = hX_5.ERR_minimum;
			if (hX_5.ERR_maximum!=undefined) hX_5.L[l].ERR_maximum = hX_5.ERR_maximum;
			if (hX_5.ERR_required!=undefined) hX_5.L[l].ERR_required = hX_5.ERR_required;
			if (hX_5.ERR_constraint!=undefined) hX_5.L[l].ERR_constraint = hX_5.ERR_constraint;
			if (hX_5.ERR_number_fail!=undefined) hX_5.L[l].ERR_number_fail = hX_5.ERR_number_fail;
			if (hX_5.ERR_number_char!=undefined) hX_5.L[l].ERR_number_char = hX_5.ERR_number_char;
			if (hX_5.STRA_DATE_MONTHS!=undefined) hX_5.L[l].STRA_DATE_MONTHS = hX_5.STRA_DATE_MONTHS;
			if (hX_5.STRA_DATE_SHMONTHS!=undefined) hX_5.L[l].STRA_DATE_SHMONTHS = hX_5.STRA_DATE_SHMONTHS;
			if (hX_5.STRA_DATE_DAYS!=undefined) hX_5.L[l].STRA_DATE_DAYS = hX_5.STRA_DATE_DAYS;
			if (hX_5.STRA_DATE_SHDAYS!=undefined) hX_5.L[l].STRA_DATE_SHDAYS = hX_5.STRA_DATE_SHDAYS;
			if (hX_5.STRA_DATE_SHSHDAYS!=undefined) hX_5.L[l].STRA_DATE_SHSHDAYS = hX_5.STRA_DATE_SHSHDAYS;
			if (hX_5.STRA_DATE_EPOCHS!=undefined) hX_5.L[l].STRA_DATE_EPOCHS = hX_5.STRA_DATE_EPOCHS;
			if (hX_5.STR_AM_SUFFIX!=undefined) hX_5.L[l].STR_AM_SUFFIX = hX_5.STR_AM_SUFFIX;
			if (hX_5.STR_PM_SUFFIX!=undefined) hX_5.L[l].STR_PM_SUFFIX = hX_5.STR_PM_SUFFIX;
		}
	}
}

// --------------------------------------------------------
// Private: HxUIState -- Object that saves/remembers UI State
// --------------------------------------------------------

HxG_5.prototype.HxUIState = function () {
	this.activecomponent = null;		/* Active component (one with focus/popup up) */
	this.activecomponentkind = "";		/* Kind of active component */
}

// Private: Set which component has a popup on display
HxG_5.prototype.HxUIState.prototype.setActiveComponent = function(obj) {
	this.activecomponent = (obj) ? obj : "";
	return true;
}

// Private: Get which component has a popup on display
HxG_5.prototype.HxUIState.prototype.getActiveComponent = function(kind) {
	return (this.activecomponent && hX_5.isObject(this.activecomponent))?this.activecomponent:null;
}

// --------------------------------------------------------
// Private: JSFargument and JSFargument table management
// --------------------------------------------------------

//----------------------------------------------------------------------------------------------------
// Private:	Make an argument object by parsing the string 'name:value' into two pieces
HxG_5.prototype.JSFargument = function (s) {
	this.value = this.attribute = hX_5._E;
	if (s && (hX_5.isString(s))) {
		var q = s.indexOf(hX_5._CL);
		q = (q < 0) ? s.indexOf(hX_5._EQ) : q;
		if (q > 0) {
			this.attribute = s.substring(0,q);
			this.value = s.substring(q+1);
		} else {
			this.attribute = s;
		}
		this.attribute = hX_5.Trim(this.attribute);
		this.attribute = this.attribute.toLowerCase();
	}
}

//----------------------------------------------------------------------------------------------------
// Private:	Given a line in an attribute descriptor table, parse (and validate) an argument
// and convert the result into the correct attribute of the object
HxG_5.prototype.ARG_IS_TRUE = 1;	// Argument should be "true" or "false" (i.e., boolean)
HxG_5.prototype.ARG_TO_NUM = 2;		// Argument should be a string that can be converted to a number.
									// Really should be an string that represents an integer
HxG_5.prototype.ARG_TO_NAN = 3;		// Argument should be a string or null.  Leading/Trailing spaces are ignored.
HxG_5.prototype.ARG_TO_FCN = 4;		// Argument is a function
HxG_5.prototype.ARG_TO_FCNS= 5;		// Argument is a string that is the "body" of a function
HxG_5.prototype.ARG_TO_NAS = 6;		// Argument should be a string or null.  Leading/Trailing spaces are preserved.
HxG_5.prototype.ARG_IS_TRUEV = 7;	// Argument should be "true" or "visible" OR "false" or "none" (matches "visible" CSS property)
HxG_5.prototype.ARG_TO_BIGNUM = 8;	// Argument should be a string that can be converted to a number.
									// Should be used if string represents a "big number", i.e., has a decimal point
HxG_5.prototype.ARG_TO_NAU = 9;		// Argument should be a string or null.  Leading/Trailing spaces are ignored.
									// Argument is converted to upper case
HxG_5.prototype.ARG_TO_NAB = 10;	// Argument should be a string or null.  If null, it's converted to an empty string

HxG_5.prototype.JSFargument.prototype.parseArg = function (thisp, type, arg1, arg2) {
	if (type != hX_5.ARG_TO_NAS)
		this.value = hX_5.Trim(this.value);
	if (type == hX_5.ARG_IS_TRUE)
		return (!(this.value == hX_5._FLS || this.value == hX_5._OFF || this.value == hX_5._Z));
	else if (type == hX_5.ARG_IS_TRUEV)
		return ( (this.value == hX_5._FLS || this.value == hX_5._OFF || this.value == hX_5._Z || this.value == hX_5._NO) ? hX_5._NO:hX_5._VS);
	else if (type == hX_5.ARG_TO_NUM || type == hX_5.ARG_TO_BIGNUM) {
		var num;
		if (hX_5.isString(this.value)) {
			if (type == hX_5.ARG_TO_NUM)
				num = new Number(this.value);
			else {
				if (hX_5.c.BIGNUM_converter == null) hX_5.c.BIGNUM_converter = new hX_5.NumberConverter();
				num = hX_5.c.BIGNUM_converter.constantToValue(this.value);
			}
		} else
			num = this.value;
		num = Math.min(Math.max(arg1, num), arg2);
		return num;
	} else if (type == hX_5.ARG_TO_NAN || type == hX_5.ARG_TO_NAU ||type == hX_5.ARG_TO_NAS)
		return ((this.value.length <= 0)?null:((type==hX_5.ARG_TO_NAU) ? this.value.toUpperCase():this.value));
	else if (type == hX_5._ARG_TO_NAB)
		return (this.value.length <= 0)?hX_5._E:this.value;
	else if (type == hX_5.ARG_TO_FCN)
		return eval(this.value);
	else if (type == hX_5.ARG_TO_FCNS)
		return (new Function ("event", this.value));
	else
	 	return this.value;
}

HxG_5.prototype.parseArg = function (attributeTable, thisp, a) {
	var i, j, v, v1, arg, desc, b = (hX_5.isString(a)) ? [a] : a;
	for (i=0; i<b.length; i++) {
		arg = new this.JSFargument(b[i]);
		if (arg.attribute.length >= 0) {
			desc = attributeTable[arg.attribute];
			if (desc) {
				if (!(desc[2]) || (desc[2] && !thisp.HTMLrendered)) {
					v = (desc[1] == null) ? (arg.value) : (arg.parseArg(thisp, desc[1], desc[3], desc[4]));
					if (thisp[desc[0]]!=null && hX_5.isArray(thisp[desc[0]])) {
						while ((thisp[desc[0]]).length>0) (thisp[desc[0]]).pop();
						j = (v==null) ? -1 : v.indexOf(hX_5._SM);
						while (j >= 0) {
							v1 = v.substr(0,j);
							v = v.substr(j+1);
							(thisp[desc[0]]).push(v1);
							j = (v==null) ? -1 : v.indexOf(hX_5._SM);
						}
						(thisp[desc[0]]).push(v);
					} else
						thisp[desc[0]] = v;
				}
			}
		}
	}
}

HxG_5.prototype.getArg = function (attributeTable, thisp, arg) {
	var i, r=null;
	if (arg.length >= 0) {
		var argdesc = attributeTable[arg];
		if (argdesc) r = thisp[argdesc[0]];
		if (r!=null) {
			if (argdesc[1]==hX_5.ARG_TO_FCNS) {
				r = r+hX_5._E;
				i = r.indexOf("{");
				r = r.substr(i+1);
				i = r.lastIndexOf("}");
				r = r.substring(0,i-1);
			} else if (hX_5.isArray(r)) {
				var r1=hX_5._E;
				for (i=0; i<r.length; i++) r1=r1+((i>0)?hX_5._SM:hX_5._E)+r[i];
				r = r1;
			}
		}
	}
	return r;
}

// --------------------------------------------------------------
// Private: HxCompImpMgr -- Manage common component functions
// --------------------------------------------------------------

HxG_5.prototype.HxCompImpMgr = function (thisparent) {
	this.p = thisparent;
	this.p.imp = this;		// For older versions of hxclient
	this.deferSetFocus = this.deferSetSelect = this.deferSetSelect1 = null;
	this.deferNSRedraw = [];	
}

// Private: Within an eventhandler, compute the object that dispatched the event
HxG_5.prototype.HxCompImpMgr.prototype.dispatchID = function(obj) {
	var src = null;
	if (obj && obj.id) {
		src = (obj.nodeType == hX_5.ELEMENT_NODE || obj.nodeType == hX_5.DOCUMENT_NODE) ? obj : obj.parentNode;
		src = src.id;
	} else if (obj == document || obj == document.body)
		src = hX_5._BODY;
	return src;
}

// Private: Within an eventhandler, compute the object to dispatch to
HxG_5.prototype.HxCompImpMgr.prototype.dispatchToID = function(obj) {
	return (obj.navigator) ? hX_5._WDW : (((obj.nodeName == hX_5._DDOC || obj.nodeName == hX_5._BBODY || obj.nodeName.toLowerCase() == hX_5._BODY || obj.nodeName.toLowerCase() == hX_5._DOC) ? hX_5._BODY : obj.id));
}

// Private: Standard event dispatcher for a component (goes with attachComponentEvent above)
//			The event is relayed to the function "dispatchComponent" in the component object for this DOM object
HxG_5.prototype.HxCompImpMgr.prototype.dispatchComponent = function(evt) {
	if ((typeof hX_5)=="undefined") return true;
	evt = (evt) ? evt : ((event) ? event: null);
	var src = hX_5.i.dispatchID(this);
	if (evt && src) {
		var obj = hX_5.getComponentById(src);
		if (obj) {
			var item = obj.DOMobj.tagName.toLowerCase();
			return (obj.dispatchComponent (item, evt, evt.type, ((evt.keyCode)?evt.keyCode:null)));
		}
	}
	return true;	// let the event continue
}

// Private: Standard event dispatcher for a behavior (goes with attachBehaviorEvent above)
//			The event is relayed to the function "dispatchBehavior" in the behavior object for this DOM object
HxG_5.prototype.HxCompImpMgr.prototype.dispatchBehavior = function(thisp, evt, name, alias) {
	if ((typeof hX_5)=="undefined") return true;
	evt = (evt) ? evt : ((event) ? event: null);
	var o,x,src = hX_5.i.dispatchID(thisp);
	if (evt && name && src) {
		if (alias && alias == "oninput_FORM") {
			x = document.getElementById(src);
			if (x && x.form) {
				src = x.form.id;
				alias = hX_5._onIN;
			}
		}
		o=hX_5.getBehaviorById(src,name,((alias!=null)?alias:evt.type));
		if (o!=null) return(o.dispatchBehavior (thisp, evt));
	}
	return true;	// let the event continue
}

// Private: Standard event dispatcher for an inputState (goes with attachInputStateEvent above)
HxG_5.prototype.HxCompImpMgr.prototype.dispatchInputState = function(evt) {
	if ((typeof hX_5)=="undefined") return true;
	evt = (evt) ? evt : ((event) ? event: null);
	var src = hX_5.i.dispatchID(this);
	if (evt && src) {
		var obj = hX_5.getBehaviorById(src, hX_5.BHR_assist, hX_5._onIN);
		if (obj != null && obj.state) return obj.state.dispatch(evt);
	}
	return true;
}

// Private: Attach a property-change monitor to a DOM object. This calls the specified monitor
// whenever an identified CSS property of the object is changed (the computed/actual property).
// 
// Arguments:	obj			(string tagname or object) Identifies the DOM object to monitor
//				monitor		(function) Function to call when a property is changed
//				property1	(string) Name of the property to be monitored (e.g., "color")
//							Name is in "DOM" syntax (not CSS syntax), e.g., "backgroundColor" not "background-color"
//							You can provide multiple properties, e.g., "attachMonitor(obj, myfunction, "width", "height", "color", "backgroundColor");
// NOTE: Only one monitor can be attached on an object (this isn't enforced, but attaching > 1 will fail)
HxG_5.prototype.HxCompImpMgr.prototype.attachMonitor = function(obj, monitor, property1, property2) {
	var x, proparray = [];
	for (x = 2; x < arguments.length; x++) proparray.push(arguments[x]);
	return this.p.attachEvent (obj, hX_5._onPC, hX_5._lL, this.dispatchMonitor, monitor, proparray);
}

HxG_5.prototype.HxCompImpMgr.prototype.removeMonitor = function(obj, which) {
	this.p.removeEvent(obj, hX_5._onPC, which, true);
}

// Private: Generic handler for property monitors
HxG_5.prototype.HxCompImpMgr.prototype.dispatchMonitor = function(evt) {
	if ((typeof hX_5)=="undefined") return;
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt && this) {
		var src = ((this.nodeType == hX_5.ELEMENT_NODE || this.nodeType == hX_5.DOCUMENT_NODE) ? this : this.parentNode);
		if (src) {
			var srcid=hX_5.i.dispatchToID(src), obj=hX_5.getComponentById(srcid), prop=(hX_5.brw.isIE()) ? ((evt.propertyName) ? evt.propertyName:null) : ((evt.type==hX_5._onDOMVPC)?hX_5._VL:((evt.attrName) ? evt.attrName : null));
			// Find the event object
			var j, userObj = hX_5.lookupEvent (hX_5._uR, srcid, hX_5._onPC);
			// See if we're looking for this argument
			if (userObj != null && userObj.args && userObj.args.length) {
				for (j=0; j < userObj.args.length; j++) {
					if (userObj.args[j] == prop) {
						// N.B. on monitors we always let the monitor continue (i.e., ignore the result)
						if (hX_5.isString(userObj.handler1) && ((userObj.handler1 == "standard") || (userObj.handler1 == "standard1"))) {
							// For IE we just draw
							if (hX_5.brw.isIE() || userObj.handler1 == "standard1") {
								obj.redraw();
							// For Mozilla, if the changed values may not be visible within this
							// event handler, so we redraw off a timer so we see the changed values
							} else {
								hX_5.i.deferNSRedraw.push(obj);
								hX_5.i.startDspTimerTimed("hX_5.i.NSRedraw", 10);
							}
						} else
							userObj.handler1.call (this, obj, evt, userObj.args);
						break;
					}
				}				
			}
		}
	}
	return	// Let the event continue
}

// Private: Handle redraw when a NS prop change found
HxG_5.prototype.HxCompImpMgr.prototype.NSRedraw = function (unq) {
	if ((typeof hX_5)=="undefined") return;
	hX_5.i.qKillTimer(unq);
	while (hX_5.i.deferNSRedraw.length > 0) {
		hX_5.i.deferNSRedraw[hX_5.i.deferNSRedraw.length-1].redraw();
		hX_5.i.deferNSRedraw.pop();
	}
}

// Private: Attach a monitor for "resize" events to an object.  Default action is to redraw the
// object when the event occurs
// 
// Arguments:	obj			(string tagname or object) Identifies the DOM object to monitor
//
HxG_5.prototype.HxCompImpMgr.prototype.attachResizeMonitor = function(obj, type) {
	// In IE, we just need to just let the object do the resize handling
	if (this.p.brw.isIE()) {
		// Note hardcoding of the event -- there are problems if we let user code handle the event
		obj.onresize = (!type) ? this.dispatchMonitorResize : type;
	} else {
		// In Netscape, we have to monitor the entire window resizing (we use only
		// one event to do this).
		if (!hX_5.globalResize) {
			this.p.attachEvent(document, hX_5._onR, hX_5._lL, ((!type) ? this.dispatchMonitorResize : type));
			hX_5.globalResize = true;
		}
	}
}

// Private: Remove a monitor for "resize" events to an object.  
// 
// Arguments:	obj			(string tagname or object) Identifies the DOM object to monitor
//
HxG_5.prototype.HxCompImpMgr.prototype.removeResizeMonitor = function(obj) {
	// In IE, we just need to just let the object do the resize handling
	if (this.p.brw.isIE()) {
		obj.onresize = null;
	} else {
		// Netscape is using a global monitor which stays put
	}
}

// Private: Specific sub-handler for resize monitor 
HxG_5.prototype.HxCompImpMgr.prototype.dispatchMonitorResize = function(evt) {
	if ((typeof hX_5)=="undefined") return;
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt) {
		// For IE, just handle this object
		if (hX_5.brw.isIE()) {
			if (this) {
				var src = ((this.nodeType == hX_5.ELEMENT_NODE || this.nodeType == hX_5.DOCUMENT_NODE) ? this : this.parentNode);
				if (src) {
					var srcid = hX_5.i.dispatchToID(src);
					var obj = hX_5.getComponentById(srcid);
					if (obj) obj.redraw();
				}
			}
		} else {
			// For Netscape, redraw the page
			hX_5.redraw();
		}
	}
	return	// Let the event continue
}

// Private: Cancel an event bubble
HxG_5.prototype.HxCompImpMgr.prototype.cancelAction = function(evt, preventDefault) {
	if (evt!=null) {
		evt.cancelBubble = true;
		if (preventDefault) {
			if (evt.preventDefault) evt.preventDefault();
			else if (evt.returnValue) evt.returnValue = false;
		}
	}
}

// Private:	Fire an event
HxG_5.prototype.HxCompImpMgr.prototype.fireEvent = function(obj, evtname, evttag) {
	var elem=this.p.getElementById(obj), ename=evtname.toLowerCase();
	if (elem) {
		if (elem.fireEvent) {
			return (elem.fireEvent(ename));
		} else if (elem.dispatchEvent) {
			var newEvt, shName=((ename.indexOf(hX_5._ON)==0)?ename.substr(2):ename), eType=hX_5.i.typeEvent(shName);
			newEvt = document.createEvent(eType);
			newEvt.initEvent(shName, true, true);
			return (elem.dispatchEvent(newEvt));	
		}
	}
	return false;
}

// Private: Given an event name (e.g., "click") figure out the DOM Level 3 event type
HxG_5.prototype.HxCompImpMgr.prototype.typeEvent = function(eName) {
	r = "HTMLEvents";
	if (eName == hX_5._evMD || eName == hX_5._evMU || eName == hX_5._evMC || eName == hX_5._evMCC ||
		eName == hX_5._evMM || eName == hX_5._evMO || eName == hX_5._evMV)
		r = "MouseEvents";
	// As this isn't fully supported yet, it seems OK to just distinguish Mouse from HTML events for now
	return r;
}

// Private:	Mark an event as having occurred in the case where we are firing the event
// and thus the event handler may not be invoked (notably focus and submit events)
HxG_5.prototype.HxCompImpMgr.prototype.markEvent = function(obj, evtname) {
	if (hX_5.cursorTracking && evtname==hX_5._evF) {
		for (var c=0; c < hX_5.cursorTracking.length; c++) hX_5.cursorTracking[c].markFocus(obj,hX_5._evF);
		return true;
	}
	return false;
}

//----------------------------------------------------------------------------------------------------
// Private:	Make a partial copy of an event object
//
// NOTE: For IE, this is a full copy of an existing "global" event object.
// For Mozilla, we copy a reasonable number of attributes (but not all)
//
// Arguments:	evt			(event object)	The event object to copy
//				(return)	The copied event object
HxG_5.prototype.HxCompImpMgr.prototype.duplicateEvent = function (evt) {
	var newEvt=null;
	if (evt!=null) {
		if (document.createEventObject!=undefined)
			newEvt = document.createEventObject(evt);
		else {
			var varobj, f="function", s;
			newEvt = new Object();
			newEvt = document.createEvent(hX_5.i.typeEvent(evt.type));
			newEvt.initEvent(evt.type, true, true);
			for (varobj in evt) {
				if (varobj.charAt(0)==varobj.charAt(0).toLowerCase() && typeof(evt[varobj])!=f) 
				try { 
					newEvt[varobj] = evt[varobj];
				} catch (e) {
					// Not sure why this works, but if fixes probs with some assignments above
					s=newEvt[varobj];
				}
			}
		}
	}
	return newEvt;
}

// Private:	Give an object focus
HxG_5.prototype.HxCompImpMgr.prototype.setFocus = function(obj, select, defer) {
	var c, o = hX_5.getElementById(obj);
	// JSF sets the id of a checkbox/radio button GROUP to the containing table
	// See if we have that case, in which case, set focus to the first checkbox/radio button within it
	if (o && o.tagName && o.tagName == hX_5._TAB && o.rows && o.rows.length > 0) {
		if (o.rows[0].cells && o.rows[0].cells.length > 0) {
			c = hX_5.getFirstChildInCell(o.rows[0].cells[0]);
			if (c && c.tagName && c.tagName == hX_5._LAB) {
				c = c.firstChild;
				if (c && c.tagName && c.tagName == hX_5._INP && (c.type.toLowerCase() == hX_5._RAD || c.type.toLowerCase() == hX_5._CHK))
					o = c;
			}
		}
	}
	// If cursor tracking is on, mark the fact that we're trying to set focus manually (and so no onfocus event will be fired)
	hX_5.imp.markEvent(o, hX_5._evF);
	// Now do the work
	hX_5.i.deferSetFocus = o;
	if (hX_5.i.deferSetFocus) {
		hX_5.i.deferSetSelect = ((arguments.length > 1) && select);
		// Deferred handler -- used because if we try to set focus when a "window change"
		// is potentially going to happen (e.g., an alert box), the deferrer seems to fix
		// up IE so it happens at the right point in time
		if (arguments.length > 2 && defer)
		// N.B.  There's a bug in Firefox that causes the focus handler to throw
		// an exception when the focusSetter runs off a timer.  It shows
		// a security error in the debug window (presumably another of their
		// bugs relating to incorrectly trying to catch security errors when doing
		// work across frames).  The focusSetter does work correctly, it just
		// causes a spurious error message to be displayed (only in the debug window)
			hX_5.i.startDspTimerTimed("hX_5.i.focusSetter", 10);
		else 
			hX_5.i.focusSetter();
		return true;
	}
	return false;
}
HxG_5.prototype.HxCompImpMgr.prototype.focusSetter = function(unq) {
	if ((typeof hX_5)=="undefined") return;
	hX_5.i.qKillTimer(unq);
	// There are several bugs in IE relative to focus setting.  
	//  1)  If you set focus to a checkbox or radio button or link, and the document window has not
	//      been "activated" via the keyboard (in particular via the Tab key), focus is set correctly
	//		but the focus rectangle doesn't draw.  There appears to be no known workaround for this.
	//	2)  If you set focus to a field while another one has a selection in it, IE doesn't "clear"
	//		the selection in the field before moving focus to a new place.  This appears to be 
	//		a feature rather than a bug (they clear it if you move focus to another input text field,
	//		but not in any other case, presumably so you can copy/paste a slice of the page).
	//		We try to detect this case and we'll clear the selection if we think this is an example of this.
	if (document.hasFocus && document.hasFocus() && document.activeElement && document.activeElement.id) {
		if (hX_5.sel.hasSelection(document.activeElement)) {
			var was=document.activeElement, to=hX_5.i.deferSetFocus;
			if (to.id == null || to.id != was.id) hX_5.sel.setSelection(was, 0, 0);
		}
	}
	// Now set focus
	if (hX_5.i.deferSetFocus!=null) try {
		if (hX_5.i.deferSetFocus.focus && hX_5.i.isFocusable(hX_5.i.deferSetFocus)) {
			hX_5.i.deferSetFocus.focus();
			// And if requested, select contents
			if (hX_5.i.deferSetSelect && hX_5.i.deferSetFocus.select!=undefined) hX_5.i.deferSetFocus.select();
		} else {
			hX_5.pendingFocusReturn = hX_5.pendingBlurRefocus = null;
		}
	}
	catch(e) {
		hX_5.pendingFocusReturn = hX_5.pendingBlurRefocus = null;
	}
}

HxG_5.prototype.HxCompImpMgr.prototype.isFocusable = function(obj) {

	// Hidden fields aren't focusable as are disabled fields
	if ((obj.type && obj.type.toLowerCase()==hX_5._HI) || obj.disabled) return false;
		
	// Fields with a style of "hidden" are excluded 
	// OR, fields with a parent who is hidden/display are as well
	var root = obj;
	while (root != null && root.tagName != hX_5._BBODY) {
		if (root.nodeType == hX_5.ELEMENT_NODE) {
			if (hX_5.g.getP (root, hX_5._VIS) == hX_5._HI || hX_5.g.getP (root, hX_5._DSP) == hX_5._NO)		
				return false;
		}
		root = root.parentNode;
	}
	return (obj.focus != null);
}

// Public: Guess which element (if any) has focus -- this is really just an educated guess
// as there is no reliable way of determining this in a browser
HxG_5.prototype.HxCompImpMgr.prototype.guessFocus = function() {
	if (document.hasFocus && document.hasFocus() && document.activeElement && document.activeElement.tagName!=hX_5._BBODY) return document.activeElement;
	return null;
}

// -------------------------------------------------------------------------------------------
// Private: Timer management
// When the callback for a timer is invoked, the this pointer isn't useful (it's set to "window")
// The following timer calls can be used to set a timer that will invoke a callback on an
// object AND the this pointer will be set up so it references the object being called.
// All timers are killed whenever a uirelease is called on hX_5
// -------------------------------------------------------------------------------------------

// Private:	Start a timer of type "timeout" that dispatches into a component or behavior object
//	ID		= the id of the component or behavior
//  item	= a parameter to be passed to the callback as the first argument
//	action	= a second parameter to be passed to the callback as the second argument
//			  NOTE: parameters are evaluated when the timer is SET and are passed as strings
//  period	= period of the timer (in milliseconds)
//  bname	= IF a behavior, the behavior name
//  ename	= IF a behavior, the event name
// 	returns = id of this timer instance.  May be used to kill the timer prematurely (see KillTimer below)
// 
//  Name and signature of callback (invariant) is 'dispatchTimer(item, action)'
HxG_5.prototype.HxCompImpMgr.prototype.startTimerTimed = function(id, item, action, period, bname, ename, permanent) {
	// N.B. the scope in which timers work don't know what "this" is, use the global object directly
	hX_5.timerUnq = (hX_5.timerUnq < 32000)?(hX_5.timerUnq+1):1;
	var d = (bname != null) ? "hX_5.i.dispatchTimerBehavior('" : "hX_5.i.dispatchTimer('";
	var t = window.setTimeout(d + hX_5.timerUnq + "', '" + id + "', '" + item + "', '" + action + ((bname==null)?"":("', '" + bname)) + ((ename==null)?"":("', '" + ename)) + "')", period);
	this.addTimer(t, "t", hX_5.timerUnq, null, permanent);
	return t;
}

// Private:	Start a timer of type "interval" that dispatches into a component
//	ID		= the id of the component
//  item	= a parameter to be passed to the callback as the first argument
//	action	= a second parameter to be passed to the callback as the second argument
//			  NOTE: parameters are evaluated when the timer is SET and are passed as strings
//  period	= period of the timer (in milliseconds)
// 	returns = id of this timer instance.  May be used to kill the timer prematurely (see KillTimer below)
// 
//  Name and signature of callback (invariant) is 'dispatchTimer(item, action)'
HxG_5.prototype.HxCompImpMgr.prototype.startTimerInterval = function(id, item, action, period, permanent) {
	hX_5.timerUnq = (hX_5.timerUnq < 32000)?(hX_5.timerUnq+1):1;
	var t = window.setInterval("hX_5.i.dispatchITimer('" + hX_5.timerUnq + "', '" + id + "', '" + item + "', '" + action + "')", period);
	this.addTimer(t, "i", hX_5.timerUnq, null, permanent);
	return t;
}

// Private:	Start a timer of type "Timeout" that dispatches into any JS object
//  obj		= the object to be dispatched to 
//  item	= a parameter to be passed to the callback as the first (only) argument
//			  NOTE: parameters are evaluated when the timer is SET and are passed as strings
//  period	= period of the timer (in milliseconds)
// 	returns = id of this timer instance.  May be used to kill the timer prematurely (see KillTimer below)
// 
//  Name and signature of callback (invariant) is 'dispatchTimer(item, action)'
HxG_5.prototype.HxCompImpMgr.prototype.startObjTimerTimed = function(obj, param, period, permanent) {
	hX_5.timerUnq = (hX_5.timerUnq < 32000)?(hX_5.timerUnq+1):1;
	var t = window.setTimeout("hX_5.i.dispatchOTimer('" + hX_5.timerUnq + "', '" + param + "')", period);
	this.addTimer(t, "t", hX_5.timerUnq, obj, permanent);
	return t;
}

// Private:	Start a timer of type "timeout" that directly dispatches to a known (singleton) object
// This call should be used extremely sparingly (really only within this file)
// The this pointer is NOT established in the callback.  No arguments can be passed.
// The callback must remove the timer from the timer list within the callback
HxG_5.prototype.HxCompImpMgr.prototype.startDspTimerTimed = function(strBack, period, arg0, permanent) {
	hX_5.timerUnq = (hX_5.timerUnq < 32000)?(hX_5.timerUnq+1):1;
	var t = window.setTimeout(strBack + "('" + hX_5.timerUnq + ((arg0==null)?hX_5._E:("'"+hX_5._C+"'"+arg0)) + "')", period);
	this.addTimer(t, "t", hX_5.timerUnq, null, permanent);
	return t;
}

// Private:	Kill all outstanding timers
HxG_5.prototype.HxCompImpMgr.prototype.killAllTimers = function(force) {
	var i, timerid, perm, iS="i", l=hX_5.Timers.length;
	for (i=0; i<l; i++) {
		timerid = hX_5.Timers[i][0];
		perm =hX_5.Timers[i][4];
		if (timerid != 0 && (force!=null || !perm)) {
			if (hX_5.Timers[i][1]==iS) window.clearInterval(timerid);
			else window.clearTimeout(timerid);
			hX_5.Timers[i][0] = 0;
		}
	}
}

// Private:	Kill a timer of type "timeout"
HxG_5.prototype.HxCompImpMgr.prototype.killTimerTimed = function(timerid) {
	if (timerid != null && timerid != 0) {
		window.clearTimeout(timerid);
		this.tKillTimer(timerid,"t");
	}
}

// Private:	Kill a timer of type "interval"
HxG_5.prototype.HxCompImpMgr.prototype.killTimerInterval = function(timerid) {
	if (timerid != null && timerid != 0) {
		window.clearInterval(timerid);
		this.tKillTimer(timerid,"i");
	}
}

// INTERNAL USE ONLY
HxG_5.prototype.HxCompImpMgr.prototype.tKillTimer = function(timerid, typ) {
	var i, l=hX_5.Timers.length;
	for (i=0; i<l; i++) {
		if ((timerid==hX_5.Timers[i][0]) && (typ==hX_5.Timers[i][1])) {
			hX_5.Timers[i][0] = 0;
			break;
		}
	}
}
// INTERNAL USE ONLY
HxG_5.prototype.HxCompImpMgr.prototype.timerUnqToId = function(unq) {
	var i, l=hX_5.Timers.length;
	for (i=0; i<l; i++) {
		if (hX_5.Timers[i][0]!=0 && hX_5.Timers[i][2]==unq) {
			return(hX_5.Timers[i][0]);
		}
	}
	return null;
}
HxG_5.prototype.HxCompImpMgr.prototype.qKillTimer = function(unq) {
	if (unq==null) return;
	var i, l=hX_5.Timers.length;
	for (i=0; i<l; i++) {
		if (unq==hX_5.Timers[i][2]) {
			hX_5.Timers[i][0] = 0;
			break;
		}
	}
}

// Helper Implementation:	Add a timer to the list of known timers
HxG_5.prototype.HxCompImpMgr.prototype.addTimer = function(timerid, type, objUnq, obj, permanent) {
	var i, added=false, perm=((permanent!=null&&permanent)?true:false), l=hX_5.Timers.length, e=[timerid, type, objUnq, obj, perm];
	for (i=0; i<l; i++) {
		if (hX_5.Timers[i][0] == 0) {
			hX_5.Timers[i] = e;
			added = true;
			break;
		}
	}
	if (!added) hX_5.Timers[l] = e;
}


// Private: Dispatch a timed timer to a JS object
HxG_5.prototype.HxCompImpMgr.prototype.dispatchOTimer = function(unq, param) {
	if ((typeof hX_5)!="undefined" && unq!=null) {
		var t=null, i, l=hX_5.Timers.length;
		for (i=0; i<l; i++) {
			if (unq==hX_5.Timers[i][2]) {
				t = hX_5.Timers[i][3];
				break;
			}
		}
		hX_5.i.qKillTimer(unq);
		if (t && t.dispatchTimer) t.dispatchTimer(param);
	}
	return false;
}

// Private: Dispatch a timed timer to a component object
HxG_5.prototype.HxCompImpMgr.prototype.dispatchTimer = function(unq, id, item, action) {
	if ((typeof hX_5)!="undefined" && id) {
		hX_5.i.qKillTimer(unq);
		var elem = hX_5.getComponentById(id);
		if (elem) {
			elem.dispatchTimer(item, action);
			return true;
		}
	}
	return false;
}

// Private: Dispatch an interval timer to a component object
HxG_5.prototype.HxCompImpMgr.prototype.dispatchITimer = function(unq, id, item, action) {
	if ((typeof hX_5)!="undefined" && id) {
		var elem = hX_5.getComponentById(id);
		if (elem) {
			elem.dispatchTimer(item, action);
			return true;
		}
	}
	return false;
}

// Private: Dispatch a timed timer to an behavior and/or input State
HxG_5.prototype.HxCompImpMgr.prototype.dispatchTimerBehavior = function(unq, src, item, action, bname, ename) {
	if ((typeof hX_5)!="undefined" && src) {
		hX_5.i.qKillTimer(unq);
		var aa=(bname==hX_5.BHR_assist), obj=hX_5.getBehaviorById(src,bname,((aa)?hX_5._onIN:ename));
		if (obj != null) {
			if (aa) {
				if (obj.state && obj.state.dispatchTimer) return obj.state.dispatchTimer(item, action);
			} else {
				if (obj.dispatchTimer) return obj.dispatchTimer(item, action);
			}
		}
	}
	return false;
}

// -------------------------------------------------------------------------------------------
// Private: DialogStack management
// -------------------------------------------------------------------------------------------
// Private: dialogs always added onto the top of the DialogStack.
//          to ensure that top dialog always has the highest z-index, we generate
//          z-index value based on the current size of DialogStack, namely
//          z-index = (DialogStack.length + BIG_ZINDEX
// 			When a modal dialog is pushed onto the stack, the UI "under" it is "disabled"
//			When it's popped from the stack the UI "under" it is "enabled" (note that
//		    modal dialogs can be "stacked" in which case only part of the UI underneath may now be enabled)
// @vadimAdd
HxG_5.prototype.HxCompImpMgr.prototype.dlgEntry = function (dlg,isModal) {
	this.dlg = dlg;
	this.isModal = (isModal!=null)?isModal:false;
	this.saveDisableSelects = null;
	this.keyboard = null;
	this.keyboard2 = null;
	this.mouse = null;
	this.iframe = null;
	this.resize = null;
}

HxG_5.prototype.HxCompImpMgr.prototype.dialogIframeStack = null;
HxG_5.prototype.HxCompImpMgr.prototype.pushOnDialogStack = function (dlg, isModal) {
	// Create a new entry in the dialog stack
	var b, obj, e=new hX_5.i.dlgEntry(dlg, isModal), newLength=hX_5.DialogStack.push(e), z=(newLength*3)+hX_5.BIG_ZINDEX;
	// Set the z-index on the dialog object
	hX_5.g.setP(dlg.DOMobj, hX_5._POS, hX_5._ABS);
	hX_5.g.setP(dlg.DOMobj, hX_5._ZND, z+hX_5._E);

	// Disable the UI if this is modal
	if (isModal) {
		var i, a, d, o, firstModal=true, lastModal=null;
		for (i=0; firstModal && i<hX_5.DialogStack.length-1; i++) if (hX_5.DialogStack[i]!=null && hX_5.DialogStack[i].isModal) firstModal=false;
		for (i=hX_5.DialogStack.length-2; lastModal==null && i>=0; i--) if (hX_5.DialogStack[i]!=null && hX_5.DialogStack[i].isModal) lastModal=hX_5.DialogStack[i];

		// Create (or refetch) an iframe
		if (hX_5.dialogIframeStack==null) hX_5.dialogIframeStack=[];
		while (hX_5.dialogIframeStack.length < hX_5.DialogStack.length) {
			o = document.createElement(hX_5._DIV);
			o.setAttribute (hX_5._ID, "HX_DLG_SCRATCH_"+hX_5.dialogIframeStack.length);
			o.setAttribute (hX_5._NAM, o.id);
			o.style.display = hX_5._NO;
			o.style.zIndex = hX_5.BIG_ZINDEX-1;
			o.style.position = hX_5._ABS;
			o.style.top = hX_5._ZPX;
			o.style.left = hX_5._ZPX;
			o.style.width = hX_5._ZPX;
			o.style.height = hX_5._ZPX;
			o.style.borderWidth = hX_5._ZPX;
			o.style.borderStyle = hX_5._NO;	
			o.style.padding = hX_5._ZPX;
			o.style.margin = hX_5._ZPX;
			o.style.fontSize = hX_5._1PX;
			document.body.appendChild(o);
			hX_5.dialogIframeStack.push(o);
		}
		o = e.iframe = hX_5.dialogIframeStack[hX_5.DialogStack.length-1];

		// Set the zindex of the iframe to be just "under" the dialog and size appropriately
		if (firstModal) {
			// First modal dialog covers the screen (or the document -- whichever is bigger)
			document.body.appendChild(o);
			hX_5.g.setP(o, hX_5._ZND, (z-2)+hX_5._E);
			hX_5.g.setP(o, hX_5._WD, Math.max(hX_5.g.getWindowClientWidth(),hX_5.g.getPageWidth()) + hX_5._PX);
			hX_5.g.setP(o, hX_5._HT, Math.max(hX_5.g.getWindowClientHeight(),hX_5.g.getPageHeight()) + hX_5._PX);
			e.resize = hX_5.attachEvent(document.body, hX_5._onR, hX_5._lF, hX_5.dispatchDlgSkip);
		} else {
			// Subsequent modal dialogs only need to cover whatever may have been "popped up" by the
			// dialog below it
			d = lastModal.dlg.DOMobj;
			d.insertBefore(o, d.firstChild);
			hX_5.g.setP(o, hX_5._ZND, (z-2)+hX_5._E);
			hX_5.g.setP(o, hX_5._WD, hX_5.g.getRendWidth(d)+hX_5._PX);
			hX_5.g.setP(o, hX_5._HT, hX_5.g.getRendHeight(d)+hX_5._PX);
		}

		// Make the iframe "opaque through a gray background"
		hX_5.g.setP(o, hX_5._DSP, hX_5._E);
		hX_5.g.setP(o, hX_5._BCOR, "#E2E2E2");
		if (o.filters) {
			o.style.filter = "progid:DXImageTransform.Microsoft.Alpha(enabled=false)";
			o.filters.item("DXImageTransform.Microsoft.Alpha").opacity = 50;
			o.filters.item("DXImageTransform.Microsoft.Alpha").enabled = true;
		} else if (o.style) {
			o.style.MozOpacity = .4;
		}

		// At this point there's a cover over all items in the window (blocking mouse activity)
		// If this is the first (only) modal dialog in the stack, we need to put a keyboard trap
		// on the first/last controls in the underlying window to ensure it doesn't get focus
		for (i=0; firstModal && i<hX_5.DialogStack.length-1; i++) if (hX_5.DialogStack[i]!=null && hX_5.DialogStack[i].isModal) firstModal=false;
		if (hX_5.brw.isIE()) {
			if (firstModal) e.keyboard = hX_5.attachEvent(document.body, "onbeforeactivate", hX_5._lF, hX_5.dispatchDlgSkip);
		} else {
			if (hX_5.tabNav == null) hX_5.tabNav = new hX_5.tabNavigator();
			obj = hX_5.tabNav.getLast(dlg.DOMobj);
			e.keyboard = hX_5.attachEvent(obj, hX_5._onKD, hX_5._lF, hX_5.dispatchDlgSkip, null, null, true);
			obj = hX_5.tabNav.getFirst(dlg.DOMobj);
			e.keyboard2 = hX_5.attachEvent(obj, hX_5._onKD, hX_5._lF, hX_5.dispatchDlgSkip, null, null, true);
			e.mouse =  hX_5.attachEvent(o, hX_5._onMD, hX_5._lF, hX_5.dispatchDlgSkip, null, null, true);
		}			

		// Finally, there's a bug in IE that must be handled.  If an iframe covers a <select> tag it will either
		// hide the select tag or keep it "on top of everything" depending on how/when "transparency" is set
		// on the iframe.  We'll let the select tags show, and disable any that need it are visible
		if (hX_5.brw.isIE()) {
			// Re-enable anything "under" us
			if (lastModal!=null) {
				a = lastModal.saveDisabledSelects;
				for (i=0; i < a.length; i++) a[i].disabled=false;
			}
			if (e.saveDisabledSelects) delete (e.saveDisabledSelects);
			e.saveDisabledSelects = [];
			if (hX_5.tabNav == null) hX_5.tabNav = new hX_5.tabNavigator();
			a = hX_5.tabNav.getTabItems();
			for (i=0; i<a.length; i++) {
				if (a[i]!=null && a[i].obj.tagName==hX_5._SEL && !a[i].obj.disabled && !hX_5.isChildTag(a[i].obj, e.dlg.DOMobj)) {
					e.saveDisabledSelects.push(a[i].obj);
					a[i].obj.disabled=true;
				}
			}	
		}
	}

	// Return the z-index used
    return (z);
}

HxG_5.prototype.HxCompImpMgr.prototype.removeFromDialogStack = function (dlg) {
	var i, e, obj, lastModal=null, l=hX_5.DialogStack.length, z=(parseInt(hX_5.g.getP(dlg.DOMobj.id, hX_5._ZND),10)-hX_5.BIG_ZINDEX)/3;

	// Remove the dialog from the stack
	if (l==z)	// dlg is on the top of DialogStack
		e = hX_5.DialogStack.pop();
	else { // dlg is in the middle of the DialogStack (only modeless case)
		i = z-1;
		e = hX_5.DialogStack[i];
		hX_5.DialogStack[i] = null;
	}

	// Set the z-index of the dialog
	hX_5.g.setP(e.dlg.DOMobj, hX_5._ZND, -1);
	if (e.iframe) {
		document.body.appendChild(e.iframe);
		e.iframe.style.display = hX_5._NO;
		e.iframe.style.width = hX_5._ZPX;
		e.iframe.style.height = hX_5._ZPX;
		e.iframe.style.zIndex = -1;
	}

	// Clean up the stack if there's empty dreck at the bottom
	while (hX_5.DialogStack.length>0 && hX_5.DialogStack[hX_5.DialogStack.length-1]==null) hX_5.DialogStack.pop();

	// Release the UI if we disabled it
	if (e.isModal) {
		for (i=hX_5.DialogStack.length-1; lastModal==null && i>=0; i--) if (hX_5.DialogStack[i]!=null && hX_5.DialogStack[i].isModal) lastModal=hX_5.DialogStack[i];

		// For IE, re-enable any selects we disabled
		if (hX_5.brw.isIE() && e.saveDisabledSelects!=null && e.saveDisabledSelects.length>0)
			for (i=0; i<e.saveDisabledSelects.length; i++) e.saveDisabledSelects[i].disabled=false;
		if (e.saveDisabledSelects!=null) delete (e.saveDisabledSelects);

		// If there's still a modal dialog on display -- make the UI disabled relative to it
		if (lastModal!=null) {
			// For IE, re-enable any selects we disabled
			if (hX_5.brw.isIE() && lastModal.saveDisabledSelects!=null && lastModal.saveDisabledSelects.length>0)
				for (i=0; i<lastModal.saveDisabledSelects.length; i++) lastModal.saveDisabledSelects[i].disabled=true;
		}
		
		// Remove the keyboard traps
		if (hX_5.brw.isIE()) {
			if (lastModal==null && e.keyboard) hX_5.removeEvent(document.body, "onbeforeactivate", e.keyboard);
			if (lastModal==null && e.resize) hX_5.removeEvent(document.body, hX_5._onR, e.resize);
		} else {
			if (hX_5.tabNav == null) hX_5.tabNav = new hX_5.tabNavigator();
			obj = hX_5.tabNav.getLast(e.dlg.DOMobj);
			hX_5.removeEvent(obj, hX_5._onKD, e.keyboard);
			obj = hX_5.tabNav.getFirst(e.dlg.DOMobj);
			hX_5.removeEvent(obj, hX_5._onKD, e.keyboard2);
			hX_5.removeAllEvents(e.iframe, e.iframe.id);
			if (lastModal==null && e.resize) hX_5.removeEvent(document.body, hX_5._onR, e.resize);
		}
	}
	if (e) delete e;
	return ((lastModal!=null)?lastModal.dlg:null);
}

// Handle keyboard when UI is disabled
HxG_5.prototype.dispatchDlgSkip = function(evt) {
	evt = (evt) ? evt : ((event) ? event: null);
	var d, tgt = hX_5.evtGetTarget(evt), r=true, obj=null, d=hX_5.DialogStack.length-1;
	while (d>=0 && !hX_5.DialogStack[d].isModal) d--;
	if (d>=0) {
		d = hX_5.DialogStack[d];
		if (evt.type==hX_5._evKD) {
			if (evt.keyCode == hX_5.KEY_TAB) {
				if (hX_5.tabNav == null) hX_5.tabNav = new hX_5.tabNavigator();
				if (tgt==hX_5.tabNav.getLast(d.dlg.DOMobj)) {
					if (!evt.shiftKey) obj = hX_5.tabNav.getFirst(d.dlg.DOMobj);
				} else {
					if (evt.shiftKey) obj = hX_5.tabNav.getLast(d.dlg.DOMobj);
				}
				if (obj) {
					hX_5.i.setFocus(obj);
					r = false;
				}
			}
		} else if (evt.type==hX_5._evR) {
			d = 0;
			while (d < hX_5.DialogStack.length && !hX_5.DialogStack[d].isModal) d++;
			// Resize the iframe to the window (in Firefox this will handle scroll bars better if we do this)
			hX_5.g.setP(hX_5.DialogStack[d].iframe, hX_5._WD, hX_5.g.getWindowClientWidth());
			hX_5.g.setP(hX_5.DialogStack[d].iframe, hX_5._HT, hX_5.g.getWindowClientHeight());
			// Now resize the iframe to the window OR the document (whichever is bigger)
			hX_5.g.setP(hX_5.DialogStack[d].iframe, hX_5._WD, Math.max(hX_5.g.getWindowClientWidth(),document.body.scrollWidth));
			hX_5.g.setP(hX_5.DialogStack[d].iframe, hX_5._HT, Math.max(hX_5.g.getWindowClientHeight(),document.body.scrollHeight));
		} else if (evt.type==hX_5._evMD) {
			r = false;
		} else {
			if (!hX_5.isChildTag(tgt, d.dlg.DOMobj, "_HXPOPUP")) {
				if (hX_5.tabNav == null) hX_5.tabNav = new hX_5.tabNavigator();
				obj = (evt.shiftKey) ? hX_5.tabNav.getLast(d.dlg.DOMobj) : hX_5.tabNav.getFirst(d.dlg.DOMobj);
				// If we set focus immediately, focus moves "halfway" to the field we want 
				// This seems to "stop" focus events from going to the field, but it's still 
				// visually "partly" in focus, so we defer the focus set and we cancel the action
				// This works in MOST but not all cases
				if (obj) hX_5.i.setFocus(obj, false, true);
				r = false;
			}
		}
	}
	if (!r) hX_5.i.cancelAction(evt,true);
	return r;
}

// -----------------------------------------------
// Private: HxPopMgr -- Manage events for popup
// -----------------------------------------------
// Manages the mouse so that clicking outside the popup cancels it
// Also manages deferring closing the popup a tad so users can change their minds
// Private: Construct the event manager
HxG_5.prototype.popEvents = null;
HxG_5.prototype.HxPopMgr = function (thisparent) {
	this.p = thisparent;
	this.popObj = this.ownObj = null;
	this.setEvents = this.popswap = this.popcap = this.buttondown = false;
	this.timerid = 0;
	this.rect = null;
	this.overrideESC = false;
}

// Public: Start managing events while a popup is on display
HxG_5.prototype.HxPopMgr.prototype.start = function (popObject, ownObject, noMouseCapture) {
	if (popObject && ownObject) {
		this.popObj = popObject;
		this.ownObj = ownObject;
		this.overrideESC = this.popswap = this.popcap = this.buttondown = false;
		// Remember the coords of the popup is located
		if (this.rect!=null) delete this.rect;
		this.rect = new hX_5.HxRect(this.popObj);
		if (hX_5.popEvents==null) hX_5.popEvents = hX_5.brw.isIE() ? [hX_5._onMD, hX_5._onMU, hX_5._onMM, hX_5._onKP, hX_5._onKD, hX_5._onKU, hX_5._onMC, hX_5.onSC, hX_5._onCM] : [hX_5._evMD, hX_5._evMU, hX_5._evMM, hX_5._evKP, hX_5._evKD, hX_5._evKU, hX_5._evMC, hX_5._evSC];
		// If we haven't attached events to the popup, do it now
		if (!this.setEvents) for (var j=0; j<hX_5.popEvents.length; j++) {
			if (hX_5.brw.isIE()) {
				if (noMouseCapture)
					this.popObj.setAttribute(hX_5.popEvents[j], hX_5.popMgr.dispatchPopEvent);
				else
					this.popObj.setAttribute(hX_5.popEvents[j], hX_5.popMgr.dispatchPopEvent);
			} else
				this.popObj.addEventListener(hX_5.popEvents[j], hX_5.popMgr.dispatchPopEvent, true);

		}
		// Set up trap for events outside the popup
		if (hX_5.brw.isIE()) {
			if (noMouseCapture) {
				hX_5.swapPopEvent(true, null, hX_5.popEvents, true);
				this.popswap = true;
			} else {
				this.popObj.setCapture(false);
				this.popcap = true;
			}
		} else {
			hX_5.swapPopEvent(true, null, hX_5.popEvents);
			this.popswap = true;
		}
		return true;
	}
	return false;
}

// Public: Reset the size of the area covered by the popup. Use sparingly.
HxG_5.prototype.HxPopMgr.prototype.resetSize = function() {
	this.rect.height = hX_5.g.getRendHeight(this.popObj);
	this.rect.width	 = hX_5.g.getRendWidth(this.popObj);
}

// Public: Start a "timed" end of the popup
HxG_5.prototype.HxPopMgr.prototype.endTimed = function (fast) {
	if (hX_5.popMgr && hX_5.popMgr.ownObj)
		hX_5.popMgr.timerid = hX_5.i.startDspTimerTimed("hX_5.popMgr.dispatchTimer", (fast)?100:(hX_5.brw.isIE()?500:400));
}

// Public: End managing the popup (only call from uirelease)
HxG_5.prototype.HxPopMgr.prototype.end = function () {
	if (this.timerid != 0) hX_5.i.killTimerTimed (this.timerid);
	if (this.popcap) this.popObj.releaseCapture();
	if (this.popswap) hX_5.swapPopEvent(false, null, hX_5.popEvents);
	this.popObj = this.ownObj = null;
	this.popswap = this.popcap = false;
	this.timerid = 0;
}

// Helper implementation: callback used by timer to end the manager
HxG_5.prototype.HxPopMgr.prototype.dispatchTimer = function (unq) {
	if ((typeof hX_5)=="undefined") return;
	hX_5.i.qKillTimer(unq);
	if (hX_5.popMgr && hX_5.popMgr.ownObj && hX_5.popMgr.ownObj.uirelease) hX_5.popMgr.ownObj.uirelease(true);
}

// Helper Implementation: dispatch callback used to dispatch an event into a popup (or cancel it)
HxG_5.prototype.HxPopMgr.prototype.dispatchPopEvent = function(evt) {
	var r = true;
	if ((typeof hX_5)=="undefined" || !hX_5.popMgr || !hX_5.popMgr.ownObj) return r;
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt) {
		var src = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : (hX_5.brw.isIE() ? document.body : null));
		if (src) {
			src = (src.nodeType == hX_5.ELEMENT_NODE || src.nodeType == hX_5.DOCUMENT_NODE) ? src : src.parentNode;
			// Compute the coordinates
			var d=hX_5.popMgr, o=d.ownObj, pt=hX_5.g.toPoint(evt), isInPopup=(d.rect && d.rect.hitTest(pt));
			// If a timer is in force, mouse down and key down kill it so user can try again
			if ((evt.type==hX_5._evMD || evt.type==hX_5._evKD) && d.timerid != 0) {
				hX_5.i.killTimerTimed (d.timerid);
				d.timerid = 0;
			}
			// If user presses ESC or clicks outside the popup, cancel or ignore it 
			if ((evt.type==hX_5._evKD && evt.keyCode == hX_5.KEY_ESC) || ((evt.type==hX_5._evMD || evt.type==hX_5._evMU) && !isInPopup)) {
				if (!(evt.type==hX_5._evMU && !d.buttondown) && !(evt.type==hX_5._evKD && evt.keyCode == hX_5.KEY_ESC && d.overrideESC)) {
					if (o.dispatchCancel) o.dispatchCancel(evt);
					if (o.uirelease) o.uirelease(true);
					r = false;
				}
			// Any other event is sent to the popup's contents
			} else {
				if (evt.type==hX_5._evMD || evt.type==hX_5._evKD) d.buttondown = true;
				if (o.dispatchPopEvent && (evt.type != hX_5._evMM || isInPopup)) r = o.dispatchPopEvent(evt, src, pt);
			}
		}
	}
	if (!r)	hX_5.i.cancelAction(evt, true);
	return r;
}

// -----------------------------------------------
// Private: JSFMenuMouse -- Handle the mouse
// -----------------------------------------------

// Public: Construct the mouse manager (done once when hx is initialized)
HxG_5.prototype.mouserMouseEvents = null;
HxG_5.prototype.HxMouseMgr = function (thisparent) {
	this.p = thisparent;
	this.root = this.basespot = this.baserect = this.savehotrect = this.basehotrect = null;
	if (this.hotspots) delete (this.hotspots);
	this.hotspots = [];
	this.popswap = false;
	this.timerid = 0;
	this.maxDelay = 750;
}

// Public: Define the "base" for a chain of hotspots.  
// The mouse is tracked across this chain of hotspots.  The base consists of
// a div that's always on display (e.g., a root menu) and an optional sub-spot
// within that div (that's root of a chain of hotspots).
// Root is a JSF object with a dispatch method.  Location is a DOM object within it.
HxG_5.prototype.HxMouseMgr.prototype.setBaseSpot = function (root, iLocation, longDelay) {
	this.root = root;
	var location = (iLocation) ? iLocation : root.DOMobj;
	this.basespot = root.DOMobj;
	if (this.baserect == null) this.baserect = new hX_5.HxRect(root.DOMobj);
	else this.baserect.setRect(root.DOMobj);
	if (this.basehotrect == null) this.basehotrect = new hX_5.HxRect(location);
	else this.basehotrect.setRect(location);
	if (this.savehotrect == null) this.savehotrect = new hX_5.HxRect(location);
	else this.savehotrect.setRect(location);
	// Set the delay for this stack o'hotspots
	this.maxDelay = (longDelay != null && longDelay) ? 5750:500;		
	// If we're adding a hotspot, we don't want a kill timer coming in
	if (this.timerid != 0) {
		hX_5.i.killTimerTimed (this.timerid);
		this.timerid = 0;
	}
}

// Add a hotspot to the mouse manager
HxG_5.prototype.HxMouseMgr.prototype.addHotSpot = function (obj) {

	// Descriptor of this hotspot
	function sub (obj) {
		this.obj = obj;
		this.id = obj.id;
		this.rect = new hX_5.HxRect(obj);
	}
	// If we're adding a hotspot, we don't want a kill timer coming in
	if (this.timerid != 0) {
		hX_5.i.killTimerTimed (this.timerid);
		this.timerid = 0;
	}
	// Ensure we don't already have this hotspot defined
	for (var i = 0; i < this.hotspots.length; i++) {
		if (this.hotspots[i].id == obj.id) return;
	}
	// If the hotspot isn't defined, add it and make sure we're tracking the mouse
	this.hotspots.push(new sub(obj));
	if (this.hotspots.length == 1) this.tidyBaseRect();
	// Start tracking
	this.trackMouse();
}

// Public: Remove a hotspot from the mouse manager
HxG_5.prototype.HxMouseMgr.prototype.removeHotSpot = function (obj) {
	for (var i = 0; i < this.hotspots.length; i++) {
		if (this.hotspots[i].id  == obj.id) {
			this.hotspots.splice(i,1);
			break;
		}
	}
	if (this.hotspots.length <= 0) this.killMouse();
}

// Private:  Expand the base hotrect so there's no "dead" space between
// it and a popup
HxG_5.prototype.HxMouseMgr.prototype.tidyBaseRect = function (obj) {
	if (this.hotspots[0]) {
		var r = this.hotspots[0].rect;
		this.basehotrect.setRectR(this.savehotrect);
		var b = this.basehotrect;
		if (r.X > b.X+b.width)
			b.width = r.X-b.X;
		else if (r.X+r.width < b.X) {
			b.width += ((r.X+r.width)-b.X);
			b.X = r.X+r.width;
		}
		if (r.Y > b.Y+b.height)
			b.height = r.Y-b.Y;
		else if (r.Y+r.height < b.Y) {
			b.height += ((r.y+r.height)-b.Y);
			b.Y = r.Y+r.height;
		}
	}
}

// Private: Start tracking the mouse over a set of hotspots
HxG_5.prototype.HxMouseMgr.prototype.trackMouse = function () {
	if (!this.popswap) {
		hX_5.mouserMouseEvents = hX_5.brw.isIE() ? [hX_5._onMD, hX_5._onMU, hX_5._onMM, hX_5._onMV, hX_5._onMO, hX_5._onKD, hX_5._onKU, hX_5._onCM] : [hX_5._evMD, hX_5._evMU, hX_5._evMM, hX_5._evMC, hX_5._evMV, hX_5._evMO, hX_5._evKD, hX_5._evKU, hX_5._evKP, hX_5._evSC];
		hX_5.swapPopEvent(true, hX_5.mouser, hX_5.mouserMouseEvents);
		this.popswap = true;
	}
}

// Private: Stop tracking the mouse
HxG_5.prototype.HxMouseMgr.prototype.killMouse = function () {
	if (this.timerid != 0) hX_5.i.killTimerTimed (this.timerid);
	if (this.popswap) hX_5.swapPopEvent(false, hX_5.mouser, hX_5.mouserMouseEvents);
	this.popswap = false;
	this.timerid = 0;
	while (this.hotspots.length > 0) this.hotspots.pop();
	this.basespot = null;
}

// Private: dispatch mouse/keyboard events that occur over hotspots
HxG_5.prototype.HxMouseMgr.prototype.dispatchPopEvent = function(evt) {
	if ((typeof hX_5)=="undefined") return true;
	var r = false;
	evt = (evt) ? evt : ((event) ? event: null);
	if (evt) {
		var src = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : (hX_5.brw.isIE() ? document.body : null));
		if (src) {
			src = (src.nodeType == hX_5.ELEMENT_NODE || src.nodeType == hX_5.DOCUMENT_NODE) ? src : src.parentNode;

			// Dispatch keystrokes to the popup
			if (evt.type==hX_5._evKD || evt.type==hX_5._evKU || evt.type==hX_5._evKP) {
				// Key actions kill any timers
				if (hX_5.mouser.timerid != 0) {
					hX_5.i.killTimerTimed (hX_5.mouser.timerid);
					hX_5.mouser.timerid = 0;
				}
				if (hX_5.mouser.root && hX_5.mouser.root.dispatchPopEvent) r = hX_5.mouser.root.dispatchPopEvent(src, evt);

			// Mouseout and Mouseover aren't used by the mouse-management code, but if we pass them
			// through, things inside the managed areas might want these events
			} else if (evt.type==hX_5._evMO || evt.type==hX_5._evMV) {
				r = true;
				// In IE, we have mouse capture on and so we should "push the event along" in some cases
				if (hX_5.brw.isIE() && src.tagName!=undefined && src.id!=undefined && src.tagName==hX_5._INP) {
					var obj = hX_5.getComponentById(src.id);
					r = (obj && obj.dispatchComponent) ? (obj.dispatchComponent(src, evt, evt.type, null)):true;
				}

			// Handle the mouse events we handle on the mouser
			} else if (evt.type==hX_5._evMD || evt.type==hX_5._evMU || evt.type==hX_5._evMM) {
				// Compute the coordinates
				var i, isBase = false, point = hX_5.g.toPoint(evt), isInPopup = hX_5.mouser.basehotrect.hitTest(point);
				if (isInPopup)
					isBase = true;
				else {
					for (i=0; (!isInPopup) && (i<hX_5.mouser.hotspots.length); i++) {
						if (hX_5.mouser.hotspots[i]) isInPopup = hX_5.mouser.hotspots[i].rect.hitTest(point);
					}
				}	
				// Dispatch mouse events to the popup if they occur inside it
				if (isInPopup) {
					// If we're back in a popup, stop the kill timer
					if (hX_5.mouser.timerid != 0) {
						hX_5.i.killTimerTimed (hX_5.mouser.timerid);
						hX_5.mouser.timerid = 0;
					}
					// Yes -- dispatch the event to the popup
					// But curiously, in the case of input type=button (and the like) if the mouse
					// is over the label part of the button, the event is listed as "on body"
					if (src && src.tagName == hX_5._BBODY && hX_5.brw.isIE()) src = document.elementFromPoint(point.X, point.Y);
					if (hX_5.mouser.root && hX_5.mouser.root.dispatchPopEvent) {
						r = hX_5.mouser.root.dispatchPopEvent(src, evt, isBase);
					}
				} else {
					// clicks outside the hotspots kill the menu immediately
					if (evt.type!=hX_5._evMM) hX_5.mouser.root.dispatchTimer("popup", "MouseOut");
					// "moves" outside the hotspots are relayed to the owning event to handle
					// after a delay -- either "short" or "long"
					else if (hX_5.mouser.root.dispatchTimer) {
						if (hX_5.mouser.baserect.hitTest(point)) hX_5.mouser.root.dispatchTimer("popup", "MouseOut");
						else if (hX_5.mouser.timerid <= 0) hX_5.mouser.timerid = hX_5.i.startTimerTimed (hX_5.mouser.root.id, "popup", "MouseOut", hX_5.mouser.maxDelay);
					}	
				}
			}
		}
	}
	// Cancel the action in most cases
	if (r==false) hX_5.i.cancelAction(evt, true);
	return r;
}

// -----------------------------------------------------------------------------
// Private: HxEventMgr -- Manage assigning events to DOM tags 
// -----------------------------------------------------------------------------
// Structure of the manager
// this.userEvents[]	= An array of JSFEventObjects.  One object for each DOM object
//					  	  we have an event attached to
// JSFEventObject		= For each object, an array of JSFEvent objects, one object
//						  for each eventname, e.g. onclick
// JSFEvent				= For each event, contains ordered lists of the event handlers
//						  we've assigned to this object/eventname.  
//						  The object's handler is now set to run our own "master handler"
//						  which in turn runs each handler listed in the JSFEvent object

// See if we have assigned an event handler for this event name to this object declared in hX = hX.userEvents
HxG_5.prototype.lookupEvent = function (kind, id, name) {
	var j,l1,eObj=(id!=null && this.isString(id) && this.userEvents[id]!=undefined)?this.userEvents[id]:null;
	if (eObj!=null) {
		l1 = eObj.events.length;
		for (j=0; j<l1; j++) {
			if (eObj.events[j].etype==kind && eObj.events[j].eventname==name) {
				return (eObj.events[j]);
			}
		}
	}
	return null;
}

// Implementation:  Add a new obj/eventname object to the list of events we're managing
// The caller should have checked that one doesn't exist (using lookupEvent)
HxG_5.prototype.addNewEvent = function (id, obj, eventname, kind, handler1, args, MozDown) {
	var uObj, eObj = (this.userEvents[id]!=undefined)?this.userEvents[id]:null;
	if (eObj==null) {
		eObj = new hX_5.JSFEventObject(id);
		this.userEvents[id] = eObj;
	}
	uObj = new hX_5.JSFEvent(obj, eventname, kind, handler1, args, MozDown);
	eObj.events.push(uObj);
	return uObj;
}

// Implementation:  Remove all (of our) event handlers attached to an object
// NOTE: Should not be used on window, document or body; only on elements within body
HxG_5.prototype.removeAllEvents = function (obj, id) {
	var evtObj, eObj = (this.userEvents[id]!=undefined)?this.userEvents[id]:null;
	if (eObj!=null) {
		while (eObj.events.length > 0) {
			evtObj = eObj.events.pop();
			evtObj.remove(0,true);
			delete evtObj;
		}
		delete eObj;
		this.userEvents[id] = null;
	}
}

// Implementation:	Constructor for the master event object, one per object
HxG_5.prototype.JSFEventObject = function (id) {
	this.id = id;
	this.events = [];
}

// Implementation:	Constructor for one of our event objects
HxG_5.prototype.JSFEvent = function (obj, eventname, etype, handler1, args, MozDown) {
	if (obj && eventname) {
		this.eventCounter = 0;
		this.eventname = eventname;
		this.User	= null;
		this.preUser= [];
		this.postUser= [];
		this.eventL=[];
		this.etype	= etype;
		this.obj	= obj;
		this.tgtObj	= null;
		this.args	= args;
		this.handler1= handler1;
		this.id = null;
		var f = null;
		if (etype == hX_5._uR) {
			if (MozDown)
				this.id = (hX_5._WDW == hX_5.i.dispatchToID(obj)) ? hX_5._WDW : obj.id;
			else if ((hX_5._BODY == hX_5.i.dispatchToID(obj))) {
				f = document.body.getAttribute(eventname);
				if (f==null) f = document.body.getAttribute[eventname];
				this.id = hX_5._BODY;
			} else {
				f = obj.getAttribute(eventname);
				if (f==null) f = obj[eventname];
				this.id = obj.id;
			}
		} else this.id = obj;
		if (f != null) this.User = (hX_5.isFunction(f)) ? f : new Function("event", f);
	}
}

HxG_5.prototype.JSFEventEntry = function (id, handler, moz) {
	this.id = id;
	this.moz = moz;
	if (hX_5.isFunction(handler)) this.handler = handler;
	else if (hX_5.isString(handler)) this.handler = new Function("event", "return " + handler + ";");
	else this.handler = hX_5.JSFNoOp;
}

HxG_5.prototype.JSFNoOp = function (event) {
	return;
}

// Add an event handler to the list of handlers assigned to this eventname
HxG_5.prototype.JSFEvent.prototype.addPost = function (pos, handler) {
	this.eventCounter++;
	var entry = new hX_5.JSFEventEntry(this.eventCounter, handler);
	if (pos == hX_5._lF || pos == "first") this.postUser.unshift (entry);
	else this.postUser.push (entry);
	return (this.eventCounter);
}
HxG_5.prototype.JSFEvent.prototype.addPre = function(pos, handler) {
	this.eventCounter++;
	var entry = new hX_5.JSFEventEntry(this.eventCounter, handler);
	if (pos == hX_5._lF || pos == "first") this.preUser.unshift (entry);
	else this.preUser.push (entry);
	return (this.eventCounter);
}
HxG_5.prototype.JSFEvent.prototype.chkEvtL = function(pos, handler) {
	if (this.eventL.length>0) {
		for (var i=0; i< this.eventL.length; i++)
			if (this.eventL[i]!=null && this.eventL[i].moz!=null && this.eventL[i].moz==pos) return true;
	}
	return false;
}
HxG_5.prototype.JSFEvent.prototype.addEvtL = function(pos, handler) {
	this.eventCounter++;
	var entry = new hX_5.JSFEventEntry(this.eventCounter, handler, pos);
	this.eventL.push(entry);
	return (this.eventCounter);
}

// Remove event handlers assigned to this event handler
HxG_5.prototype.JSFEvent.prototype.remove = function (id, destroy) {
	var i, e, removed=false, l=this.postUser.length, moz1=false, moz2=false, rmoz=null, ename;
	if (id <= 0) {
		// Empty arrays and delete all handlers
		while (this.postUser.length > 0) {
			e = this.postUser.pop();
			if (destroy) delete (e);
		}
		while (this.preUser.length > 0) {
			e = this.preUser.pop();
			if (destroy) delete (e);
		}
		while (this.eventL.length > 0) {
			e = this.eventL.pop();
			if (e.moz) moz1 = true; else moz2 = true;
			if (destroy) delete (e);
		}
		if (destroy) {
			// Remove our "run" handler
			this.tgtObj.setAttribute(this.eventname, null);
			// Remove any special mozilla event listeners
			if (document.addEventListener && (moz1||moz2)) {
				eName = (this.eventname.indexOf(hX_5._ON) == 0) ? this.eventname.substr(2) : this.eventname;
				if (moz1) this.tgtObj.removeEventListener(eName, hX_5.runEvent2, true);
				if (moz2) this.tgtObj.removeEventListener((this.args && this.args.length && this.args[0]==hX_5._VL)?hX_5._onDOMVPC:hX_5._onDOMPC, hX_5.runEvent1, false);
			}	
		}
	} else {
		for (i = 0; ((i < l) && (!removed)); i++) {
			if (this.postUser[i] && this.postUser[i].id == id) {
				this.postUser[i] = null;
				removed = true;
				while (this.postUser.length>0 && this.postUser[this.postUser.length-1]==null) this.postUser.pop();
			}
		}
		l = this.preUser.length;
		for (i = 0; ((i < l) && (!removed)); i++) {
			if (this.preUser[i] && this.preUser[i].id == id) {
				this.preUser[i] = null;
				removed = true;
				while (this.preUser.length>0 && this.preUser[this.preUser.length-1]==null) this.preUser.pop();
			}
		}
		if (!removed) {
			l = this.eventL.length; moz1=0; moz2=0;
			for (i = 0; (i < l); i++) {
				if (this.eventL[i]) {
					if (!removed && this.eventL[i].id == id) {
						rmoz = this.eventL[i].moz;
						this.eventL[i] = null;
						removed = true;
						while (this.eventL.length>0 && this.eventL[this.eventL.length-1]==null) this.eventL.pop();
					} else {
						if (this.eventL[i].moz) moz1++; else moz2++;
					}
				}
			}
			if (rmoz!=null) {
				eName = (this.eventname.indexOf(hX_5._ON) == 0) ? this.eventname.substr(2) : this.eventname;
				if (rmoz && moz1==0) this.tgtObj.removeEventListener(eName, hX_5.runEvent2, true);
				if (!rmoz && mox2==0) this.tgtObj.removeEventListener((this.args && this.args.length && this.args[0]==hX_5._VL)?hX_5._onDOMVPC:hX_5._onDOMPC, hX_5.runEvent1, false);
			}
		}
	}
}

// Private: Swap in/out global events used for popup managment
HxG_5.prototype.swapPopEvent = function (addevent, iHandler, mouseEvents, noCapture) {
	var d = (iHandler) ? iHandler: hX_5.popMgr;
	// Dragging uses events defined by the owner of the drag object
	var userObj, j, l=mouseEvents.length, dd="#docdrag", ff, ss="S";
	for (j=0; j < l; j++) {
		// See if we have a slot for one of these
		userObj = hX_5.lookupEvent (ss, dd, mouseEvents[j]);
		// Swap out the jsfevent and put back the original one
		if (!addevent) {
			if (userObj) {
				if (this.brw.isIE()) {
					document.body.parentNode.setAttribute(mouseEvents[j], userObj.User);
				} else {
					ff =  hX_5._ON + mouseEvents[j];
					window.removeEventListener(mouseEvents[j], d.dispatchPopEvent, true);
					window[ff] = userObj.User;
				}
			}
		// Swap out the original event and put in the jsf one
		} else {
			if (userObj == null) userObj = hX_5.addNewEvent (dd, dd, mouseEvents[j], ss);
			if (this.brw.isIE()) {
				userObj.User = document.body.parentNode.getAttribute(mouseEvents[j]);
				document.body.parentNode.setAttribute(mouseEvents[j], d.dispatchPopEvent);
			} else {
				ff =  hX_5._ON + mouseEvents[j];
				userObj.User = window[ff];
				window.addEventListener(mouseEvents[j], d.dispatchPopEvent, true);
			}
		}
	}
	if (this.brw.isIE() && noCapture==null) {
		if (addevent) document.body.parentNode.setCapture();
		else document.body.parentNode.releaseCapture();
	}
	return true;
}

// Implementation:	Substitute event handler that runs all the code attached to an event
HxG_5.prototype.runEvent1 = function (evt) {
	hX_5.runEvent(this, evt);
}
HxG_5.prototype.runEvent2 = function (evt) {
	hX_5.runEvent(this, evt, true);
}
HxG_5.prototype.runEvent = function (ethis, evt, MozDown) {
	// Get the event object from IE/Netscape
	evt = (evt) ? evt : ((event) ? event: null);
	// IE sometimes messes up and doesn't define the return value (e.g., on a KeyDown up/down arrow)
	if (evt.returnValue!=undefined && evt.returnValue==false) evt.returnValue = true;
	// Get the this pointer which refers to the object with the event handler
	ethis = (!ethis) ? this : ethis;
	// Netscape doesn't like providing valid this pointers for 'body' or 'document' it conflates them with window
	// There's a problem here in that we allow window to be used as well so we don't know -- is this window or document or body?
	// Right now "window" is used only internally and only when MozDown is set, so in that one case
	// we'll use window, otherwise we'll use body.  This hack will eventually need to be reworked
	ethis = (ethis == window || ethis == document.window) ? ((MozDown)?window:window.document.body) : ethis;
	// Find the event handler for this object
	if (ethis && evt) {
		var srcid, userObj, evtname = (evt.type==hX_5._onDOMPC || evt.type==hX_5._onDOMVPC) ? hX_5._onPC : (hX_5._ON + evt.type);
		srcid = (ethis == window) ? hX_5._WDW : ((ethis == document || ethis == document.body) ? hX_5._BODY : ethis.id);
		userObj = hX_5.lookupEvent (hX_5._uR, srcid, evtname);
		// return the result of running the event
		if (userObj != null) return ((hX_5.brw.isIE()) ? (userObj.run(ethis)) : (userObj.run(ethis, evt)));
	}
	// If we don't have the event in the list, let processing continue	
	return true;
}

// Implementation:	Run the script(s) associated with the event handler
HxG_5.prototype.JSFEvent.prototype.run = function (ethis, evt) {
	// After each script runs, check the "return result" to see if the script
	// wants to allow processing to continue or not
	var i, r = true;
	evt = (evt) ? evt : ((event) ? event: null);
	for (i = 0; r && i < this.eventL.length; i++) {
		if (this.eventL[i] != null) {
			r = (hX_5.brw.isIE()) ? ((this.eventL[i]).handler.call(ethis)) : ((this.eventL[i]).handler.call(ethis, evt));
			r = (r == false) ? false: true;
			if (evt.returnValue!=undefined && evt.returnValue == false) r = false;
			else if (evt.getPreventDefault && evt.type!=hX_5._onDOMPC && evt.type!=hX_5._onDOMVPC && evt.getPreventDefault()) r = false;
		}
	}
	for (i = 0; r && i < this.preUser.length; i++) {
		if (this.preUser[i] != null) {
			r = (hX_5.brw.isIE()) ? ((this.preUser[i]).handler.call(ethis)) : ((this.preUser[i]).handler.call(ethis, evt));
			r = (r == false) ? false: true;
			if (evt.returnValue!=undefined && evt.returnValue == false) r = false;
			else if (evt.getPreventDefault && evt.getPreventDefault()) r = false;
		}
	}
	if (r && this.User != null) {
		r = (hX_5.brw.isIE()) ? (this.User.call(ethis)) : (this.User.call(ethis, evt));
		r = (r == false) ? false: true;
		if (evt.returnValue!=undefined && evt.returnValue == false) r = false;
		else if (evt.getPreventDefault && evt.getPreventDefault()) r = false;
	}
	for (i = 0; r && i < this.postUser.length; i++) {
		if (this.postUser[i] != null) {
			r = (hX_5.brw.isIE()) ? ((this.postUser[i]).handler.call(ethis)) : ((this.postUser[i]).handler.call(ethis, evt));
			r = (r == false) ? false: true;
			if (evt.returnValue!=undefined && evt.returnValue == false) r = false;
			else if (evt.getPreventDefault && evt.getPreventDefault()) r = false;
		}
	}
	return r;
}

// -----------------------------------------------------------------------------
// Private: Constraint management
// -----------------------------------------------------------------------------
HxG_5.prototype.HxConstraints = function () {
	// There are two types of constraints:
	// 1)  Regular expressions (composed of Javascript regular expressions)
	// 2)  "EL" expressions = a line of javascript that is a "conditional expression" (e.g., value > "a" && value < "c"
	this.regexp = [];		// Array of compiled regular expressions, referenced by "name"
	this.ELexp  = [];		// Array of "compiled" EL expressions, again referenced by "name"
	this.regexpstr = [];	// Array of regular expressions that have been used (string version of expression)
	this.ELexpstr  = [];	// Array of EL expressions that have been used (string version of expression)
	
	// Register a set of "canned" regular expressions
	this.staticConstraint("AlphabetExOnly","^[A-Za-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF]+$");
	this.staticConstraint("AlphabetOnly",  "^[A-Za-z]+$");
	this.staticConstraint("DigitOnly",     "^[0-9]+$");
	this.staticConstraint("AlnumExOnly",   "^[A-Za-z0-9\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u00FF]+$");
	this.staticConstraint("AlnumOnly",     "^[A-Za-z0-9]+$");
	this.staticConstraint("UpperCaseOnly", "^[A-Z]+$");
	this.staticConstraint("LowerCaseOnly", "^[a-z]+$");
//	this.staticConstraint("Name", 		   "^[$#@A-Z][$#@_A-Z0-9]+$");
}

HxG_5.prototype.HxConstraints.prototype.getRegExpByName = function (name) {
	return this.regexp[name];
}

HxG_5.prototype.HxConstraints.prototype.testRegExp = function (test, exp) {
	return ((test.length==0) ? true : (!(test.match(exp) == null)));
}

HxG_5.prototype.HxConstraints.prototype.staticConstraint = function (name, x) {
	try {
		this.regexp[name]= new RegExp(x);
		this.regexp[name].compile(x);
	} 
	catch (e) {
		window.status = hX_5.INTERR_Constraint;
	}
}
HxG_5.prototype.HxConstraints.prototype.registerConstraint = function (x) {
	var name, i, hxc = "hxConstraint";
	// See if this constraint already exists
	for (i = 0; i < this.regexpstr.length; i++) {
		// Return the existing one
		if (this.regexpstr[i] == x) {
			name = hxc + i;
			return this.regexp[name];
		}
	}
	name = hxc + this.regexpstr.length;
	this.regexpstr.push(x);

	// Need to register a new one
	try {
		this.regexp[name]= new RegExp(x);
		this.regexp[name].compile(x);
		return this.regexp[name];
	} 
	catch (e) {
		window.status = hX_5.INTERR_Constraint;
	}
	return null;
}

HxG_5.prototype.HxConstraints.prototype.registerELConstraint = function (x, type) {
	var name, i, hxe = "hxExpression";
	// See if this expression already exists
	for (i = 0; i < this.ELexpstr.length; i++) {
		// Return the existing one
		if (this.ELexpstr[i] == x) {
			name = hxe + i;
			return this.ELexp[name];
		}
	}
	name = hxe + this.ELexpstr.length;
	this.ELexpstr.push(x);

	// Need to register a new one
	try {
		var exp = hX_5.replaceAll(x, "@testValue", "testValue");
		if (type=="date") {
			exp = hX_5.replaceAll(exp, "testValue.date",	"testValue.getDate()");
			exp = hX_5.replaceAll(exp, "testValue.day",		"testValue.getDay()");
			exp = hX_5.replaceAll(exp, "testValue.hours", 	"testValue.getHours()");
			exp = hX_5.replaceAll(exp, "testValue.minutes",	"testValue.getMinutes()");
			exp = hX_5.replaceAll(exp, "testValue.month",	"testValue.getMonth()");
			exp = hX_5.replaceAll(exp, "testValue.seconds", "testValue.getSeconds()");
			exp = hX_5.replaceAll(exp, "testValue.timezoneOffset", "testValue.getTimezoneOffset()");
			exp = hX_5.replaceAll(exp, "testValue.time",	"testValue.getTime()");
			exp = hX_5.replaceAll(exp, "testValue.year", 	"(testValue.getFullYear()-1900)");
		}
		exp = " return ( " + exp +  "); ";
		this.ELexp[name] = new Function("testValue", exp);
		return this.ELexp[name];
	} 
	catch (e) {
		window.status = hX_5.INTERR_Constraint;
	}
	return null;
}

HxG_5.prototype.HxConstraints.prototype.testELExp = function (test, exp) {
	return (exp.call(this, test));
}

// ------------------------------------------------
// JSF Action (Sub-Behaviors)
// ------------------------------------------------
// The following is the "public" (for end user) API call that corresponds to "JSFGenericBehavior"
HxG_5.prototype.processBehavior = function (id, evt, actions, targets) {
	var obj = hX_5.getElementById(id);
	if (obj && evt && (actions!=null || fcn!=null)) {
		action = [];
		target = [];
		var i, j, z, t=actions;
		for (j=0; j<2; j++) {
			if (j==0) {t=actions; z=action;} else {t=targets; z=target;}
			while (t.length > 0) {
				i=t.indexOf(hX_5._SM);
				if (i>=0) {
					z.push(t.substring(0,i));
					t = t.substr(i+1);
				} else {
					z.push(t);
					t = hX_5._E;
				}
			}
		}
		hX_5.action.processAction(obj, evt, null, action, true, target, null, null, null, null);
	}
}

// NOTE:  The JSF Action object is instantiated once as part
// of the hxclient object.  To "do" an action, you call hX.action.(args);
HxG_5.prototype.HxAction = function () {

	//--- INTERFACE ----
	
	// Public functions
	this.processAction = processAction;
	this.processActionEx = processActionEx;
	this.processFunction = processFunction;
	this.positionLabels = positionLabels;
	this.doClick = doClick;
	this.doHideShow = doHideShow;
	this.doHideShowCB = doHideShowCB;
	this.doContinueActions = doContinueActions;
	this.pauseActions = pauseActions;
	this.unPauseActions = unPauseActions;
	this.isPaused = isPaused;
	this.pauseBlock = pauseBlock;
	this.pauseStateBlock = pauseStateBlock;
	this.pActions = [];
	this.pBlocks = [];
	this.sBlocks = [];
	this.pRunningCount = 0;
	this.whileMOTrapOn = null;
	this.whileMOTrapper = whileMOTrapper;
	this.whileBTrapOn = null;
	this.whileBTrapper = whileBTrapper;

	// ---- IMPLEMENTATION ----

	// Public: Pause the running of a string of actions
	function pauseActions() {
		var i, empty=null, p;
		p = new this.pauseBlock();
		for (i=0; empty==null && i<this.pActions.length; i++) if (this.pActions[i]==null) empty = i;
		if (empty==null)
			this.pActions.push(p);
		else
			this.pActions[empty]=p;
		return p.id;
	}
	
	// Public: Unpause the running of a string of actions
	function unPauseActions(ID) {
		var i, obj=null;
		for (i=0; obj==null && i<this.pActions.length; i++) if (this.pActions[i]!=null && this.pActions[i].id==ID) obj=i;
		if (obj!=null) {
			delete this.pActions[obj];
			this.pActions[obj]=null;
			while (this.pActions.length > 0 && this.pActions[this.pActions.length-1]==null) this.pActions.pop();
		}
	}

	// Private: See if this action run is paused
	function isPaused(ID) {
		for (var i=0; i<this.pActions.length; i++) {
			if (this.pActions[i]!=null && this.pActions[i].id==ID) return this.pActions[i];
		}
		return null;
	}

	// Private: A Pause Block object
	function pauseBlock() {
		this.id = hX_5.action.pRunningCount;
		this.retryCount = 0;
		this.defer = defer;
			// Private: Actually pause the action
			function defer (timeout, obj, evt, fcn, action, globalAction, target, cssStyle, cancelBubble, retainFocus, isError,
							error, label, cssLabelStyle, popLabels, infoSpan, infoShow, errorSpan, errorShow, k) {
				this.retryCount = 0;
				var i, empty=null;
				var p = new hX_5.action.pauseStateBlock(this.id, obj, evt, fcn, action, globalAction, target, cssStyle, cancelBubble, retainFocus, isError,
							error, label, cssLabelStyle, popLabels, infoSpan, infoShow, errorSpan, errorShow, k);
				for (i=0; empty==null && i<hX_5.action.pBlocks.length; i++) if (hX_5.action.pBlocks[i]==null) empty = i;
				if (empty==null)
					hX_5.action.pBlocks.push(p);
				else
					hX_5.action.pBlocks[empty] = p;
				hX_5.i.startDspTimerTimed("hX_5.action.doContinueActions", timeout, (this.id+hX_5._E), true);
			}
		return;
	}

	// Pause state block
	function pauseStateBlock (id, obj, evt, fcn, action, globalAction, target, cssStyle, cancelBubble, retainFocus, isError,
							  error, label, cssLabelStyle, popLabels, infoSpan, infoShow, errorSpan, errorShow, k) {
		this.id = id;
		this.obj = obj;
		// FIXME: EVTS will be screwed UP, so doc which things will work and which things won't
		if (document.createEventObject!=undefined) 
			this.evt = document.createEventObject(evt);
		else
			// In the w3c world, you can't actually create a "copy" of an event object
			// (there's no way to set the target), so we'll just make a regular
			// object with a few things in it
			this.evt = new dummyEvent(evt.type, evt.target, evt);
		this.fcn = fcn;
		this.action = action;
		this.globalAction = globalAction;
		this.target = target;
		this.cssStyle = cssStyle;
		this.cancelBubble = cancelBubble;
		this.retainFocus = retainFocus;
		this.isError = isError;
		this.error = error;
		this.label = label;
		this.cssLabelStyle = cssLabelStyle;
		this.popLabels = popLabels;
		this.infoSpan = infoSpan;
		this.infoShow = infoShow;
		this.errorSpan = errorSpan;
		this.errorShow = errorShow;
		this.actionK = k;
		this.blockCancelled = false;
	}

	// Timer callback: pause action
	function doContinueActions (unq, ID) {
		if ((typeof hX_5)=="undefined") return;
		hX_5.i.qKillTimer(unq);
		var i, p=hX_5.action.isPaused(ID);
		// Continue only after the "get" callback has completed
		if (p!=null) {
			p.retryCount++;
			// Fixme: not clear what the interval should be -- this will pause up to a few minutes by iterating 25 times each time with a larger interval
			if (p.retryCount <= 25 && !p.blockCancelled) {
				hX_5.i.startDspTimerTimed("hX_5.action.doContinueActions", 100+((p.retryCount-1)*200), (p.id+hX_5._E), true);
			} else {
				if (p.blockCancelled) {
					if (hX_5.action.whileMOTrapOn!=null) hX_5.removeEvent(document.body, hX_5._onMO, hX_5.action.whileMOTrapOn);
					if (hX_5.action.whileBTrapOn!=null) hX_5.removeEvent(document.body, hX_5._onB, hX_5.action.whileBTrapOn);
					hX_5.action.whileBTrapOn = hX_5.action.whileMOTrapOn = null;
				}
				// Not much we can do if we time out on the retry, so we'll just never finish the sequence
				p = null;
				for (i=0; p==null && i<hX_5.action.pActions.length; i++) if (hX_5.action.pActions[i]!=null && hX_5.action.pActions[i].id==ID) p=i;
				if (p!=null) {
				    delete hX_5.action.pActions[p];
				    hX_5.action.pActions[p]=null;
					while (hX_5.action.pActions.length > 0 && hX_5.action.pActions[hX_5.action.pActions.length-1]==null) hX_5.action.pActions.pop();
				}
				p = null;
				for (i=0; p==null && i<hX_5.action.pBlocks.length; i++) if (hX_5.action.pBlocks[i]!=null && hX_5.action.pBlocks[i].id==ID) p=i;
				if (p!=null) {
				    delete hX_5.action.pBlocks[p];
				    hX_5.action.pBlocks[p]=null;
					while (hX_5.action.pBlocks.length > 0 && hX_5.action.pBlocks[hX_5.action.pBlocks.length-1]==null) hX_5.action.pBlocks.pop();
				}
			}
		} else {
			for (i=0; p==null && i<hX_5.action.pBlocks.length; i++) if (hX_5.action.pBlocks[i]!=null && hX_5.action.pBlocks[i].id==ID) p=i;
			if (p!=null) {
				i = p;
				p = hX_5.action.pBlocks[i];
				// If we came here and we don't think we have the mouse anymore, DON'T continue doing actions
				if (p.blockCancelled) {
					if (hX_5.action.whileMOTrapOn!=null) hX_5.removeEvent(document.body, hX_5._onMO, hX_5.action.whileMOTrapOn);
					if (hX_5.action.whileBTrapOn!=null) hX_5.removeEvent(document.body, hX_5._onB, hX_5.action.whileBTrapOn);
					hX_5.action.whileBTrapOn = hX_5.action.whileMOTrapOn = null;
				} else {
					var r = hX_5.action.processActionEx(
						p.obj, p.evt, p.fcn, p.action, p.globalAction, p.target, p.cssStyle, p.cancelBubble, p.retainFocus, p.isError,
					    p.error, p.label, p.cssLabelStyle, p.popLabels, p.infoSpan, p.infoShow, p.errorSpan, p.errorShow, p.actionK);
				}
			    delete hX_5.action.pBlocks[i];
			    hX_5.action.pBlocks[i]=null;
				while (hX_5.action.pBlocks.length > 0 && hX_5.action.pBlocks[hX_5.action.pBlocks.length-1]==null) hX_5.action.pBlocks.pop();
			}			
		}
		return false;
	}

	/* Privste: trap the "cancel" events that can come in (e.g., that focus/mouse have left the object the actions are running against)	 */
	function whileMOTrapper(evt) {
		if (hX_5.action.pBlocks && hX_5.action.pBlocks.length>0) {
			evt = (evt) ? evt : ((event) ? event: null);
			var i, t = hX_5.evtGetTarget(evt);
			// See if we can find a pause block started on a mouseover for this object
			for (i=0; i< hX_5.action.pBlocks.length; i++) {
				if (hX_5.action.pBlocks[i]!=null && hX_5.action.pBlocks[i].evt!=null && 
					hX_5.action.pBlocks[i].evt.type==hX_5._evMV && hX_5.isChildTag(t,hX_5.action.pBlocks[i].obj)) { 
					hX_5.action.pBlocks[i].blockCancelled=true;
				}
			}
		}
	}
	function whileBTrapper(evt) {
		if (hX_5.action.pBlocks && hX_5.action.pBlocks.length>0) {
			evt = (evt) ? evt : ((event) ? event: null);
			var i, t = hX_5.evtGetTarget(evt);
			// See if we can find a pause block started on a mouseover for this object
			for (i=0; i< hX_5.action.pBlocks.length; i++) {
				if (hX_5.action.pBlocks[i] && hX_5.action.pBlocks[i].evt && 
				  hX_5.action.pBlocks[i].evt.type==hX_5._evF && hX_5.isChildTag(t,hX_5.action.pBlocks[i].obj)) { 
					hX_5.action.pBlocks[i].blockCancelled=true;
				}
			}
		}
	}

	// Private: Apply a css style class
	function doCssStyleClass (inobj, styleClass, inlabel, styleLabelClass) {
		var obj=hX_5.getElementById(inobj);
		if ((obj != null) && (styleClass != null) && (obj.className != styleClass))
			obj.className = styleClass;
		obj = (inlabel!=null) ? hX_5.getElementById(inlabel) : null;
		if ((obj != null) && (styleLabelClass != null) && (obj.className != styleLabelClass))
			obj.className = styleLabelClass;
	}

	// Private: Show/Hide labels
	function doHideShowLabels (pop, infoSpan, infoShow, errorSpan, errorShow, error) {
		var d, dsp=((pop==null)?hX_5._E:hX_5._BLK), obj=(infoSpan!=null)?hX_5.getElementById(infoSpan):null;
		if (obj != null) {
			d = hX_5.g.getP(obj, hX_5._DSP);
			if (infoShow && d==hX_5._NO) hX_5.g.setP(obj, hX_5._DSP, dsp);
			if (!infoShow && d!=hX_5._NO) hX_5.g.setP(obj, hX_5._DSP, hX_5._NO);
		}
		obj = (errorSpan!=null)?hX_5.getElementById(errorSpan) : null;
		if (obj != null) {
			d = hX_5.g.getP(obj, hX_5._DSP);
			if (errorShow) {
				if (error!=hX_5.getIHTML(obj)) hX_5.setIHTML(obj,error);
				if (errorShow && d==hX_5._NO) hX_5.g.setP(obj, hX_5._DSP, dsp);
			}
			if (!errorShow && d!=hX_5._NO) hX_5.g.setP(obj, hX_5._DSP, hX_5._NO);
		}
	}

	// Public: Position labels
	function positionLabels (o, pop, spInfo, spError) {
		if (o && (spInfo || spError)) {
			var d, p,l,t,h, rel=false, top=hX_5.g.getAbsPosTop(o), left=hX_5.g.getAbsPosLeft(o), sTop=top, sLeft=left;
			if (pop=="RIGHT") left += hX_5.g.getRendWidth(o);
			else if (pop=="UNDER") top += hX_5.g.getRendHeight(o);
			else rel = true;
			sLeft = left; sTop = top;
			// Absolutely position the elements. NOTE: We are not bounding these to fit inside the window at the momemt.
			// It's not clear that we can do this without putting the popup right on the input field
			if (spError) {
				l = hX_5.g.getAbsPosLeft(spError); t = hX_5.g.getAbsPosTop(spError);
				if (rel) {
					d = hX_5.g.getP(spError, hX_5._DSP);
					if (d==hX_5._NO) hX_5.g.setP(spError, hX_5._DSP, hX_5._BLK);
					if (pop=="LEFT") left -= hX_5.g.getRendWidth(spError);
					else top -= hX_5.g.getRendHeight(spError);
					if (d!=hX_5.g.getP(spError, hX_5._DSP)) hX_5.g.setP(spError, hX_5._DSP, d);
				}
				if (l!= left) hX_5.g.setP(spError, hX_5._LFT, left+hX_5._PX);
				if (t!= top) hX_5.g.setP(spError, hX_5._TOP, top+hX_5._PX);
			}
			if (spInfo) {
				l = hX_5.g.getAbsPosLeft(spInfo); t = hX_5.g.getAbsPosTop(spInfo);
				if (rel) {
					d = hX_5.g.getP(spInfo, hX_5._DSP);
					if (d==hX_5._NO) hX_5.g.setP(spInfo, hX_5._DSP, hX_5._BLK);
					if (pop=="LEFT") sLeft -= hX_5.g.getRendWidth(spInfo);
					else sTop -= hX_5.g.getRendHeight(spInfo);
					if (d!=hX_5.g.getP(spInfo, hX_5._DSP)) hX_5.g.setP(spInfo, hX_5._DSP, d);
				}
				if (l!= sLeft) hX_5.g.setP(spInfo, hX_5._LFT, sLeft+hX_5._PX);
				if (t!= sTop) hX_5.g.setP(spInfo, hX_5._TOP, sTop+hX_5._PX);
			}
		}
	}

	// Timer callback: Hide/Show fields
	function doHideShowCB (unq) {
		if ((typeof hX_5)=="undefined") return true;
		var i,blk,r=false,tid=hX_5.i.timerUnqToId(unq);
		hX_5.i.qKillTimer(unq);
		for (i=0; i<hX_5.action.sBlocks.length; i++) {
			blk = hX_5.action.sBlocks[i];
			if (blk!=null && blk[0]==tid) {
				r = hX_5.action.doHideShow(blk[1], blk[2], blk[3], true);
				delete hX_5.action.sBlocks[i];
				hX_5.action.sBlocks[i]=null;
				break;
			}
		}
		while (hX_5.action.sBlocks.length>0 && hX_5.action.sBlocks[hX_5.action.sBlocks.length-1]==null) hX_5.action.sBlocks.pop();
		return r;
	}

	// Private: Hide/Show fields dealing with nested stuff, redrawing components and handling load/unload handlers					
	function doHideShow (doload, t, evt, pended) {
		var r=true, s, i=true, once=false;
		if (pended && (hX_5.pendingBlurRefocus!=null || hX_5.pendingFocusReturn!=null))
			r = false;
		else {
			s=(t.id!=undefined)?hX_5.getComponentById(t.id):null;
			if (s && s.onPreLoad!=undefined) i = s.onPreLoad(s.DOMobj, evt);
			if (!i)
				r = false;
			else {
				if (doload==1)
					t.style.display = hX_5._E;
				else if (doload==2)
					t.style.display = hX_5._NO;
				else if (doload==3)
					hX_5.applyNestedProperty(t, hX_5._VIS, hX_5._VS);
				else
					hX_5.applyNestedProperty(t, hX_5._VIS, hX_5._HI);
				// Go through the children and redraw/load them
				a = hX_5.getComponentsInId(t);
				for (i=0; a!=null && i<a.length; i++) {
					s = hX_5.getComponentById(a[i]);
					if (s) {
						if (s.redraw) s.redraw();
						if (!once) {
							if ((doload==1 || doload==3) && s.onLoad!=undefined) s.onLoad(s.DOMobj, evt);
							if ((doload==2 || doload==4) && s.unLoad!=undefined) s.unLoad(s.DOMobj, evt);
							once=true;
						}
					}
				}
			}
		}
		return r;
	}

	// Private: Clear from the current position of the cursor to the end of a field
	function doClear (obj) {
		if (obj && obj.tagName && obj.value && obj.value.length > 0 && (obj.tagName == hX_5._TXT || (obj.tagName == hX_5._INP && (obj.type.toLowerCase() == hX_5._XT ||obj.type.toLowerCase()==hX_5._PWD)))) {
			var p = hX_5.sel.getCaretPos(obj);
			if (p>=0 && p<obj.value.length) {
				if (hX_5.brw.isIE()) hX_5.skipVCMonitor = true;	// Mark for value changed monitor
				obj.value = obj.value.substr(0,p);
			}
		}
	}

	// Private: Make a cheap dummy event object for Mozilla
	function dummyEvent (typ, tgt, evt) {
		this.type = typ;
		this.target = tgt;
		if (evt.pageX!=undefined) {
			this.pageX = evt.pageX;
			this.pageY = evt.pageY;
		}
	}

	// Private: Execute an action
	function doAction (srcobj, action, globalAction, target, evt, hasSubmit, hasStop) {
		var r=true, doload=0, redraw=false, redrawHS=false, a, b, s=null, i, t=target, domMaxLen, a=(action)?action.toUpperCase():null;
		if (srcobj != null && t != null && a != null) {
			hX_5.skipVCMonitor=false;
			switch (a) {
				case hX_5.ACT_NOTHING:
				case hX_5.ACT_NONE:
				case hX_5.ACT_STOP:
					r = false;
					break;
				case hX_5.ACT_WHILEEVENT:
					break;
				case hX_5.ACT_GOTO:
					if (t) window.location.href = t;
					break;
				case hX_5.ACT_ALERT:
					if (t && t.toUpperCase()==hX_5.TGT_ATERROR) t = (evt.errorMsg)?evt.errorMsg:hX_5.TGT_ISERROR;
					if (t) alert (t);
					break;
				case hX_5.ACT_CONFIRM:
					if (t && t.toUpperCase()==hX_5.TGT_ATERROR) t = (evt.errorMsg)?evt.errorMsg:hX_5.TGT_ISERROR;
					if (t) {
						s = confirm(t);
						r = (s)?true:false;
					}
					break;
				case hX_5.ACT_LOCKED:
					hX_5.applyNestedAttribute(t, hX_5._DSB, true, true);
					redraw = true;
					break;
				case hX_5.ACT_UNLOCKED:
					hX_5.applyNestedAttribute(t, hX_5._DSB, false, true);
					redraw = true;
					break;
				case hX_5.ACT_INVERTLOCKED:
					hX_5.applyNestedAttribute(t, hX_5._DSB, ((t.disabled)?false:true), true);
					redraw = true;
					break;
				case hX_5.ACT_SHOW:
					redrawHS = true;
					doload = 1;
					break;
				case hX_5.ACT_HIDE:
					redrawHS = true;
					doload = 2;
					break;
				case hX_5.ACT_INVERTHIDE:
					redrawHS = true;
					doload = (hX_5.g.getP(t,hX_5._DSP)==hX_5._NO)?1:2;
					break;
				case hX_5.ACT_VISIBLE:
					redrawHS = true;
					doload = 3;
					break;
				case hX_5.ACT_INVISIBLE:
					redrawHS = true;
					doload = 4;
					break;
				case hX_5.ACT_INVERTVISIBLE:
					redrawHS = true;
					doload = (hX_5.g.getP(t,hX_5._VIS)==hX_5._HI)?3:4;
					break;
				case hX_5.ACT_NEXTTAB:
					hX_5.gotoNextPrev (t, true);
					break;
				case hX_5.ACT_NEXTTABCLEAR:
					if (evt) {
						s = hX_5.evtGetTarget(evt);				
						doClear(s);
					}
					hX_5.gotoNextPrev (t, true);
					break;
				case hX_5.ACT_PREVTAB:
					hX_5.gotoNextPrev (t, false);
					break;
				case hX_5.ACT_PREVTABCLEAR:
					if (evt) {
						s = hX_5.evtGetTarget(evt);				
						doClear(s);
					}
					hX_5.gotoNextPrev (t, false);
					break;
				case hX_5.ACT_CLICK:
					doClick(t);
					r = false;
					break;
				case hX_5.ACT_GET:
					while (s==null && t!=null) {
						b = hX_5.getBehaviorsById(t);
						if (b && b.length > 0) for (i=0; i<b.length; i++) {
							if (b[i].type==hX_5.BHR_ajaxgetexternal || b[i].type==hX_5.BHR_ajaxget || b[i].type==hX_5.BHR_ajaxsubmit) {
								s = b[i];
								break;
							}
						}
						t = t.parentNode;
					}
					if (s && s.dispatchBehavior) s.dispatchBehavior(null, evt);
					break;
				case hX_5.ACT_SUBMIT:
					var tagset = [];
					tagset.push(new hX_5.tagListItem(hX_5._INP,hX_5._SBM));
					tagset.push(new hX_5.tagListItem(hX_5._INP,hX_5._lIMG));
					tagset.push(new hX_5.tagListItem(hX_5._BTN,hX_5._SBM));
					if (hX_5.isTag(t, tagset))
						doClick(t);
					else {
						var submit, form = (t.tagName==hX_5._FRM) ? t : t.form;
						if (form) {
							submit = hX_5.findFirstTag(form, tagset);
							if (submit) doClick(submit);			
						}
					}
					return false;
					break;
				case hX_5.ACT_PADLEFT:
					if (t && t.value && t.maxLength) {
						domMaxLen = (t.maxLength == -1 || t.maxLength == 0x7fffffff || t.maxLength == void(0)) ? 0 : t.maxLength;
						if (domMaxLen > 0) {
							if (hX_5.brw.isIE()) hX_5.skipVCMonitor = true;	// Mark for value changed monitor
							t.value = hX_5.padL(t.value, domMaxLen, hX_5._B);
						}
					}
					break;
				case hX_5.ACT_PADRIGHT:
					if (t && t.value && t.maxLength) {
						domMaxLen = (t.maxLength == -1 || t.maxLength == 0x7fffffff || t.maxLength == void(0)) ? 0 : t.maxLength;
						if (domMaxLen > 0) {
							if (hX_5.brw.isIE()) hX_5.skipVCMonitor = true;	// Mark for value changed monitor
							t.value = hX_5.padR(t.value, domMaxLen, hX_5._B);
						}
					}
					break;
				case hX_5.ACT_TRIM:
					if (t && t.value) {
						if (hX_5.brw.isIE()) hX_5.skipVCMonitor = true;	// Mark for value changed monitor
						t.value = hX_5.Trim(t.value);
					}
					break;
				// NOTE: The following isn't public
				case hX_5.ACT_MOVE:
					if (evt && evt.keyCode!=undefined && t) {
						if (evt.keyCode==hX_5.KEY_LTARROW || evt.keyCode==hX_5.KEY_RTARROW) {
							s = hX_5.g.getAbsPosLeft(t)+((evt.keyCode==hX_5.KEY_LTARROW)?-5:5);
							i = hX_5.g.getWindowClientWidth()-hX_5.g.getRendWidth(t);
							s = (s<0)?0:((s>i)?i:s);
							if (hX_5.g.getRelPos(t)) s = s - hX_5.g.getAbsPosLeft(hX_5.g.getRelPos(t));
							t.style.left = s+hX_5._PX;
						} else if (evt.keyCode==hX_5.KEY_UPARROW || evt.keyCode==hX_5.KEY_DNARROW) {
							s = hX_5.g.getAbsPosTop(t)+((evt.keyCode==hX_5.KEY_UPARROW)?-5:5);
							i = hX_5.g.getWindowClientHeight()-hX_5.g.getRendHeight(t);
							s = (s<0)?0:((s>i)?i:s);
							if (hX_5.g.getRelPos(t)) s = s - hX_5.g.getAbsPosTop(hX_5.g.getRelPos(t));
							t.style.top = s+hX_5._PX;
						}
					}
	    			break;
				// NOTE: The following isn't public
				case hX_5.ACT_DLG_OK:
					redrawHS = true;
					doload = 2;
					evt.submitFlag = hX_5.ACT_DLG_OK;
					break;
				case hX_5.ACT_DLG_CANCEL: 
					redrawHS = true;
					doload = 2;
					evt.submitFlag = hX_5.ACT_DLG_CANCEL;
					break;
				case hX_5.ACT_FOCUS:
				case hX_5.ACT_SELECTED:
				case hX_5.ACT_UNSELECTED:
					// Handling setting focus/selection is tricky, 
					// Take care before changing this handler
					r = false;
					// First, if this field is generating the focus event, don't
					// reset focus from within the handler
					if (evt && evt.type==hX_5._evF && t.id==srcobj.id) {
						if (a == hX_5.ACT_SELECTED)
							// Just do a selection
							hX_5.sel.setCaretPos(srcobj, 0, srcobj.value.length, hX_5.brw.isNavOrMoz());
						else {
							// If Focus, unselect and leave cursor at start of field
							// Note that we have to explicitly set the caret for reasons that are unclear
							// in IE (caret isn't set anywhere if we don't do this) and in Netscape
							// where it will always select all otherwise)
							// Also note that if "assist" is on, it will be setting the cursor so we'll
							// skip doing it here in that case
							var bev = hX_5.getBehaviorById(srcobj.id, hX_5.BHR_assist, hX_5._onIN);
							if ((a==hX_5.ACT_UNSELECTED) || (bev==null) || (bev!=null && bev.assist==null))
								hX_5.sel.setCaretPos(srcobj, 0, 0, hX_5.brw.isNavOrMoz());
						}
						break;
					}
					// Otherwise we are either setting focus to a different field in the onfocus event
					// OR we are setting focus to the same field from its onblur event
					// OR we are setting focus to a different field from this onblur event
					//
					// If we are "moving" focus to a different field in this onFocus event, there can be problems
					// as the OS isn't very happy when someone does this.  We let it happen
					// and let the user see if it works ok or not.
					//
					// If we are in the onblur and we are setting focus back into the same field
					// we're effectively trying to "block" the blur from occurring.  This normally
					// happens when there's an error in the field and you want to keep the cursor
					// in the field until the error has been stopped.  In this case, we really
					// want the action to be "Don't advance to next field" as opposed to 
					// "refire focus event here".  (We really just want the blur to return "false"
					// and have the OS stop the advance to the next field.) Since you cannot
					// actually stop onblur from processing and a suceeding onfocus from
					// happening, we'll set focus back to this field and set some flags that
					// will try to prevent other hxclient code from doing extra processing
					// of the onfocus events that fire.
					
					// Set focus to the field and either select/not the contents
					hX_5.i.setFocus(t, (a==hX_5.ACT_SELECTED), true);
					// If we're coming off a blur, flag this case so that in the validation
					// dispatcher above, we won't immediately process the focus IF we're trying
					// to keep focus in the same field.
					if (evt && evt.type==hX_5._evB) hX_5.pendingBlurRefocus = t.id;
					// If this is not a global action (i.e. this is called from a field validator),
					// we will ASSUME the field has focus and only set selection OTHERWISE the field
					// will get an onFocus event which will want to "reset" the field when we want
					// it to stay in it's "error state"
					if (!globalAction) {
						hX_5.pendingFocusReturn = t.id;
						hX_5.pendingSelect = (a==hX_5.ACT_SELECTED);
					}
					break;
	
				default:
					break;
			}
		}

		// Redraw may need deferring
		if (redrawHS) {
			var se,tid,isDlg=false;
			s=(t.id!=undefined)?hX_5.getComponentById(t.id):null;
			// If we're doing a "hide" on a dialog, we can't defer if we think we might be doing a submit
			// we need to "un-disable" all controls in the form immediately as the submit will not work right
			// if we defer and disable them later
			if (s && s.type==hX_5.CMP_dialog && (doload==2 || doload==4) && hasSubmit!=null && hasStop!=null) {
				// If we see something that is likely a submit in the action stream
				isDlg = hasSubmit;
				// If the action is on a "submit" button and there is no Stop
				isDlg = (isDlg)?isDlg:((!hasStop && srcobj.tagName && srcobj.tagName==hX_5._INP && (srcobj.type.toLowerCase()==hX_5._SBM || srcobj.type.toLowerCase()==hX_5._RES || srcobj.type.toLowerCase()==hX_5._lIMG)));
				isDlg = (isDlg)?isDlg:(srcobj.tagName && srcobj.tagName==hX_5._INP && srcobj.type.toLowerCase()==hX_5._RES);
				// If we're not going thru the defer code, we have to kill off anything that might try to validate/set focus after the dialog is gone
				if (isDlg) {
					hX_5.pendingBlurRefocus = null;
					hX_5.pendingFocusReturn = null;
				}
			}
			if (s && globalAction && !isDlg && (s.onLoad || s.onPreLoad)) {
				// Defer processing until after any focus activity has cleared
				// We need an event object -- to pass on to the load code
				if (document.createEventObject!=undefined) 
					se = document.createEventObject(evt);
				else
					// In the w3c world, you can't actually create a "copy" of an event object
					// (there's no way to set the target), so we'll just make a regular
					// object with a few things in it
					se = new dummyEvent(evt.type, evt.target, evt);
				if (evt.submitFlag) se.submitFlag=evt.submitFlag;
				tid = hX_5.i.startDspTimerTimed("hX_5.action.doHideShowCB", 1, null, true);
				hX_5.action.sBlocks.push([tid,doload,t,se]);
				r = true;
			} else {
				r = doHideShow (doload, t, evt, false);
			}
		}
		if (redraw) {
			a = hX_5.getComponentsInId(t);
			for (s=0; a!=null && s<a.length; s++) {
				s = hX_4.getComponentById(a[s]);
				if (s) s.redraw();
			}
		}
		
		// Done
		if (!r) hX_5.i.cancelAction(evt, true);
		return r;
	}

	// Public:  click a button.  Note that if we fire the onclick event on a button
	// the button gets the event but it doesn't "do all" that it should (e.g., it doesn't
	// do a submit which is important for us!).  If we directly call the click event
	// the button gets the event (onclick fires), if onclick continues then a submit
	// happens including the firing of an onsubmit event.  So we try to click whenever
	// possible
	function doClick (inObj, evttag) {
		var o = hX_5.getElementById(inObj);
		if (o.click && hX_5.brw.isIE())
			return (o.click());
		else {
			var r = hX_5.i.fireEvent (o, hX_5._onMC, evttag);
			if (r == null || r == true && (!hX_5.brw.isIE() && o.tagName && o.tagName==hX_5._ANC)) {
				// Firing a click on an a tag in mozilla doesn't work
				// unless the tag has a click handler that actually does it (go figure)
				if (o.href) window.location.href = o.href;
			}
			return r;
		}
	}

	// Public:  Process an action
	// NOTE: action and target are ARRAYS of actions and corresponding targets
	function processAction (obj, evt, fcn, action, globalAction, target, cssStyle, cancelBubble, retainFocus, isError) {
		return this.processActionEx (obj, evt, fcn, action, globalAction, target, cssStyle, cancelBubble, retainFocus, isError, 
		null, null, null, false, null, false, null, false, null);
	}
	function processActionEx (obj, evt, fcn, action, globalAction, target, cssStyle, cancelBubble, retainFocus, isError,
							  error, label, cssLabelStyle, popLabels, infoSpan, infoShow, errorSpan, errorShow, reEntry) {

		// Assign this execution of actions an id
		if (reEntry==null) {
			this.pRunningCount++;
			if (this.pRunningCount > 10000) this.pRunningCount=1;
		}
		var id = this.pRunningCount;

		// Execute the function (if present)
		var r = (reEntry!=null)?true:this.processFunction(obj,evt,fcn);

		// Execute the action(s) (if present)
		var k, u, t, a, initialr=r, pause=null, hasSubmit=false, hasStop=false, whileFound=null, whileTime=null, whileName=null;
		if (r && action != null) {
			for (k = ((reEntry!=null)?reEntry:0); r && pause==null && k < action.length; k++) {
				u = action[k].toUpperCase();
				hasSubmit = (u==hX_5.ACT_SUBMIT || u==hX_5.ACT_CLICK || u==hX_5.ACT_DLG_OK || u==hX_5.ACT_DLG_CANCEL);
				hasStop = (u==hX_5.ACT_NOTHING || u==hX_5.ACT_NONE || u==hX_5.ACT_STOP);
				whileFound = (reEntry==null && u==hX_5.ACT_WHILEEVENT)?k:whileFound;
			}
			// If we have a "WhileEvent" in the sequence (and we aren't re-entering)
			// Then defer processing the events for the specified time and see what event will cause this
			// action sequence to "cancel".  Note that we only "cancel" when a timer of some sort goes off
			// when we can examine whether to continue or not.  Normally this happens when the initial "pause"
			// timer goes off or when the timers checking for an AJAX get go off
			if (whileFound!=null) {
				// How long should we "pause" before doing the actions?
				whileTime = (target[whileFound]!=null)?parseInt(target[whileFound],10):20;
				// Which event do we track?  It's the "opposite" of the event that triggered this
				// That is, if this is a mouseover, we look for mouseout.  If this is a focus, we look for a blur.
				// Currently these are the only two pairs of events we handle
				if (evt!=null) whileName = (evt.type==hX_5._evMV)?hX_5._onMO:((evt.type==hX_5._evF)?hX_5._onB:null);
				// We need to track the event mouse so we know if we're still in/out of the object
				// We can only do this if we're in a MoveOver at the moment
				if (whileName==hX_5._onMO)
					this.whileMOTrapOn = hX_5.attachEvent(obj, whileName, hX_5._lF, hX_5.action.whileMOTrapper);
				if (whileName==hX_5._onB)
					this.whileBTrapOn = hX_5.attachEvent(obj, whileName, hX_5._lF, hX_5.action.whileBTrapper);

				// Pause the action sequence
				var pauseFid = hX_5.action.pauseActions(), pauseF = this.isPaused(pauseFid);
				pauseF.defer (whileTime, obj, evt, fcn, action, globalAction, target, cssStyle, cancelBubble, retainFocus, isError, error, label, cssLabelStyle, popLabels, infoSpan, infoShow, errorSpan, errorShow, 0);
				hX_5.action.unPauseActions(pauseFid);
				return false;
			}

			// Now run the actions						
			for (k = ((reEntry!=null)?reEntry:0); r && pause==null && k < action.length; k++) {
				// Compute the target
				a = action[k].toUpperCase();
				t = target[k];
				if (!(a==hX_5.ACT_CONFIRM || a==hX_5.ACT_ALERT)) {
					if (hX_5.isDef(typeof t) && t != null) {
						if (t.toLowerCase() == hX_5._NO) t = document.body;
						else if (t.substr(0,4).toLowerCase() == hX_5._URL || t.substr(0,4).toLowerCase() == hX_5._URI) t = hX_5.stripURL(t);
						else t = hX_5.getElementById(t);
					} else if (evt != null) t = hX_5.evtGetTarget(evt);
				}
				// Do the action
				r = doAction (obj, a, globalAction, t, evt, hasSubmit, hasStop);
				r = (hX_5.isUnDef(typeof r)) ? true : ((r == false) ? false: true);
				if (evt && evt.returnValue!=undefined && evt.returnValue == false) r = false;
				else if (evt && evt.getPreventDefault && evt.getPreventDefault()) r = false;
				if (r && a==hX_5.ACT_GET) {
					pause = this.isPaused(id);
					if (pause!=null) pause.defer (20, obj, evt, fcn, action, globalAction, target, cssStyle, cancelBubble, retainFocus, isError, error, label, cssLabelStyle, popLabels, infoSpan, infoShow, errorSpan, errorShow, k+1);
					// If we pause, we have to cancel running this action -- otherwise things get messy if this is a submit button -- it will submit while we are paused
					r = false;
				}
			}
		}

		if (!pause) {	
			// Kill the mouse tracker if it's on
			if (this.whileBTrapOn!=null || hX_5.action.whileMOTrapOn!=null) {
				if (hX_5.action.whileMOTrapOn!=null) hX_5.removeEvent(document.body, hX_5._onMO, hX_5.action.whileMOTrapOn);
				if (hX_5.action.whileBTrapOn!=null) hX_5.removeEvent(document.body, hX_5._onB, hX_5.action.whileBTrapOn);
				hX_5.action.whileBTrapOn = hX_5.action.whileMOTrapOn = null;
			}
			// Set the CSS style and hide/show labels
			if (initialr) {
				doCssStyleClass (obj, cssStyle, label, cssLabelStyle);
				if (popLabels!=null) this.positionLabels(obj, popLabels, infoSpan, errorSpan);
				doHideShowLabels (popLabels, infoSpan, infoShow, errorSpan, errorShow, error);
				// Reposition since the above call could have changed the error text
				if (popLabels!=null) this.positionLabels(obj, popLabels, infoSpan, errorSpan);
			}
		
			// Work out if we stop event processing/propagation
			if (cancelBubble != null || (isError && !r)) {
				if (evt) {
					if (evt.preventDefault) {
						evt.stopPropagation();
						evt.preventDefault();
					} else {
					// At least in IE, we need to clear the keycode for function keys
					// otherwise, stopping propagation doesn't stop the function key
					// from doing something like "help"
						if (hX_5.brw.isIE() && evt.type==hX_5._evKD) {
							var code = (evt.keyCode && (evt.keyCode != 0)) ? evt.keyCode : -1;
							if ((code >= 91 && code <= 93) || (code >= 112 && code <= 123))		// fcn keys && fcn keys on keypad
							try {
							// There are some wack cases where this doesn't work so catch it
								evt.keyCode = 0;
							} catch (e) {
							}
						}						
						evt.cancelBubble = true;
						evt.returnValue = false;
					}
				}
				r = false;
			}
		
			// And if we try to retain focus
			// Note that this is an "old" v5.1x feature and it's STRONGLY DEPRECATED
			if (retainFocus) hX_5.i.setFocus(t, false, true);
		}
		return r;
	}

	// Public:  Process a JS function provided by a user
	function processFunction (obj, evt, fcn, arg0) {

		// Execute the function (if present)
		var r = true;
		if (fcn != null && obj!=null) {
			try {
				// NOTE:  fcn should already be a fcn.  It's really bad if it isn't, however,
				// there could be some old assist code floating around that hasn't already done the cast
				var realfcn = (hX_5.isFunction(fcn)) ? fcn : new Function ("event", fcn);
				// NOTE:  We could run the function using either "obj" (the original "this" pointer
				// when the event started) or "t" (the target object) as the "this" pointer in the
				// invoked function.  Not sure which a user would expect.  Using "obj" as it's
				// the originating this pointer.
				r = (arg0==null) ? realfcn.call(obj, evt) : realfcn.call(obj, arg0, evt);
				r = (hX_5.isUnDef(typeof r)) ? true : ((r == false) ? false: true);
				if (evt) {
					if (evt.returnValue!=undefined && evt.returnValue == false) r = false;
					else if (evt.getPreventDefault && evt.getPreventDefault()) r = false;
				}
			}
			catch (e) {
				window.status = hX_5.INTERR_Dispatcher;
				if (evt && evt.errorMsg) evt.errorMsg = null;
			}
		}
		return r;
	}
}

// -----------------------------------------------------------------------------
// Private: Simple selection management
// -----------------------------------------------------------------------------

HxG_5.prototype.HxCaretSelection = function () {
}

HxG_5.prototype.HxCaretSelection.prototype.getSelection = function (obj) {
	var r, sel = null;
	// NS
	if (obj.selectionEnd) {
		sel = obj.value.substring(obj.selectionStart,obj.selectionEnd);
	// IE
	} else if (document.selection) {
		r = document.selection.createRange();
		sel = r.text;
	}
	return sel;
}

HxG_5.prototype.HxCaretSelection.prototype.getSelectionR = function (obj) {
	var s=0, e=0, l;
	// NS
	if (obj.selectionEnd) {
		s = obj.selectionStart;
		e = obj.selectionEnd;
	// IE
	} else if (document.selection) {
		s = this.getCaretPos(obj);
		l = this.getSelection(obj);
		e = s+l.length;
	}
	s = (s < 0) ? 0 : s;
	e = (e > obj.value.length) ? obj.value.length : e;
	return{start: s, end: e};
}

HxG_5.prototype.HxCaretSelection.prototype.hasSelection = function (obj) {
	var sel = hX_5.sel.getSelection(obj);
	return (sel != null && sel.length > 0);
}

HxG_5.prototype.HxCaretSelection.prototype.setSelection = function (obj, start, end) {
	// IE
	if (obj.createTextRange) {
		// Some objects list createTextRage as a function, though it will CRASH IE if you call
		// it (because they really don't support it).  Let's verify the function using
		// Goodman's list of "OK" cases.
		var t=((obj.tagName) ? obj.tagName : hX_5._E), y=((obj.type) ? obj.type.toLowerCase() : hX_5._E);
		if (t==hX_5._BBODY || t==hX_5.TXT || t==hX_5.__BTN || 
		   (t==hX_5._INP && (y==hX_5._TBTN || y==hX_5._HI || y==hX_5._PWD || y==hX_5._RES || y==hX_5._SBM || y==hX_5._XT)))
		{
			var r = obj.createTextRange();
			r.collapse(true);
			r.moveEnd('character', end);
			r.moveStart('character', start);
			r.select();
		}
	// NS
	} else if (obj.setSelectionRange) {
		try {
			obj.setSelectionRange(start, (end==null)?start:end);
		} catch (e) {
			// Unclear why we sometimes get an error here, but ignore it when we do
		}
	}
}

HxG_5.prototype.HxCaretSelection.prototype.getCaretPos = function (obj, hasFocus) {
	if (hasFocus != null && !hasFocus)
		if (obj.focus) obj.focus();
	// NS
	if (obj.selectionStart) {
		return obj.selectionStart;
	// IE
	} else if (document.selection) {
		var n, r = document.selection.createRange();
		r.moveEnd("textedit");
		if (obj != r.parentElement())
			n = -1;		
		else
			n = obj.value.length - r.text.length;
		return n;
	}
	return 0;
}

HxG_5.prototype.HxCaretSelection.prototype.setCaretPos = function (obj, pos, pos1, defer) {
	// When handling "commands" (e.g., paste), we need to defer setting the caret pos
	// until this "dispatch" is done.  This is the simplest defer we can do (so use it sparingly)
	if (arguments.length > 3 && defer != null && defer) {
		hX_5.i.deferSetFocus = obj;
		hX_5.i.deferSetSelect = pos;
		hX_5.i.deferSetSelect1 = (arguments.length > 2 && pos1 != null) ? pos1 : null;
		hX_5.i.startDspTimerTimed("hX_5.i.selSetter", 1);
	} else
		hX_5.sel.setSelection(obj, pos, pos1);
}
HxG_5.prototype.HxCompImpMgr.prototype.selSetter = function(unq) {
	if ((typeof hX_5)=="undefined") return;
	hX_5.i.qKillTimer(unq);
	hX_5.sel.setSelection(hX_5.i.deferSetFocus, hX_5.i.deferSetSelect, ((hX_5.i.deferSetSelect1 != null)?hX_5.i.deferSetSelect1:hX_5.i.deferSetSelect));
}

// -----------------------------------------------------------------------------
// Private: Simple rectangle handling
// -----------------------------------------------------------------------------

// Construct a rectangle.
// 		Arguments:		(none)	construct an empty rectangle
//						(1)		construct a rectangle from the coords of this object
HxG_5.prototype.HxRect = function (obj) {
	this.X = this.Y = this.width = this.height = null;
	if (arguments.length > 0) this.setRect(obj);
}

// Set a value into the rectangle
HxG_5.prototype.HxRect.prototype.setRect = function (obj) {
	if (obj) {
		this.X 	= hX_5.g.getAbsPosLeft(obj);
		this.Y 	= hX_5.g.getAbsPosTop(obj);
		this.width	= hX_5.g.getRendWidth(obj);
		this.height	= hX_5.g.getRendHeight(obj);
	}
}
// Set a value into the rectangle
HxG_5.prototype.HxRect.prototype.setRectR = function (r) {
	if (r) {
		this.X 	= r.X;
		this.Y 	= r.Y;
		this.width	= r.width;
		this.height	= r.height;
	}
}
// Set a value into the rectangle
HxG_5.prototype.HxRect.prototype.incRect = function (i) {
	this.X 	+= i;
	this.Y 	+= i;
	this.width	+= (i*2);
	this.height	+= (i*2);
}
// Test if a point is in a rectangle
// Arguments:	(1 arg)	 A point (see next routine)
//				(2 args) X and Y coordinates in pixels adjusted for scrolling (see next routine)
HxG_5.prototype.HxRect.prototype.hitTest = function (iX, iY) {
	if (this.X == null || this.Y == null || this.width == null || this.height == null)
		return false;
	var x, y;
	if (arguments.length == 1) {
		x = iX.X; y = iX.Y;
	} else {
		x = iX; y = iY;
	}
	return ((x >= this.X) && (x < (this.X+this.width)) &&
			(y >= this.Y) && (y < (this.Y+this.height)));
}

// Convert an "event" xy mouse coordinate to absolute coords a rect uses
// Arguments:	(1 arg)	 An event object
HxG_5.prototype.HxGeometryMgr.prototype.toPoint = function (evt) {
	var cX, cY;
	if (hX_5.brw.isIE()) {
		var sBase = (hX_5.brw.isIE6_CSS()) ? document.body.parentNode : document.body;
		cX = evt.clientX; cY = evt.clientY;
		cX = cX + sBase.scrollLeft;
		cY = cY + sBase.scrollTop;
	} else {
		cX = evt.pageX;	cY = evt.pageY;
	}
	return {X: cX, Y: cY};
}

// -----------------------------------------------------------------------------
// Private: Constructor for our fake clipboard (see inputState)
// -----------------------------------------------------------------------------

HxG_5.prototype.pseudoClipboard = function () {
	this.valueAsString = null;
	this.valueAsNumber = null;
}

// -----------------------------------------------------------------------------
// Public: General purpose helper for  managing iframes
// -----------------------------------------------------------------------------
HxG_5.prototype.HxIFrameMgr = function (thisparent) {
	this.p = thisparent;
}

// -----------------------------------------------------------------------------
// Public: Create an iframe
// 	doc			The document that's the parent of the iframe
//	id			The id to give the iframe
//	invisible	If true, the iframe will be made non-visible by making it small
//  body		If true, an (empty) body will be put in the iframe
// -----------------------------------------------------------------------------
HxG_5.prototype.HxIFrameMgr.prototype.create = function (doc, id, invisible, transparent) {
	var o=null, d;
	if (id && doc.createElement) {
		o = doc.createElement(hX_5._IFRM);
		o.setAttribute (hX_5._ID, id);
		o.setAttribute (hX_5._NAM, id);
		o.src = "javascript:'';";
		// Styling is a bit tricky, if this is not "invisible", we'll let the caller
		// set the style on the returned object.  If it is "invisible" we'll see
		// what we can do to make it non-visible
		if (invisible) {
			o.style.position = hX_5._ABS;
			o.style.top = 	hX_5._ZPX;
			o.style.left = 	hX_5._ZPX;
			o.style.width  = hX_5._1PX;
			o.style.height = hX_5._1PX;
			o.style.borderWidth = hX_5._ZPX;
			o.style.borderStyle = hX_5._NO;
			o.style.display = hX_5._NO;
		} else {
			o.frameBorder = hX_5._Z;
			o.style.display = hX_5._NO;
			o.style.zIndex = hX_5.BIG_ZINDEX-1;
			o.style.position = hX_5._ABS;
			o.style.top = hX_5._ZPX;
			o.style.left = hX_5._ZPX;
			o.style.borderWidth = hX_5._ZPX;
			o.style.borderStyle = hX_5._NO;
		}
		// By convention, we add it to the end of the document
		doc.body.appendChild(o);
		if (transparent) o.allowTransparency = true;		// Oddness for IE and must be done AFTER attaching to document
		// Add a body to the iframe
		d = (o.contentDocument) ? o.contentDocument : o.Document;
		if (d.body==undefined || d.body==null) d.write("<body></body>");
	}
	return o;
}

// -----------------------------------------------------------------------------
// Public: Get the document in an iframe
// -----------------------------------------------------------------------------
HxG_5.prototype.HxIFrameMgr.prototype.getDoc = function (obj) {
	return ((obj==null) ? null : ((obj.contentDocument) ? obj.contentDocument : ((obj.Document)? obj.Document: null)));
}

// -----------------------------------------------------------------------------
// Public: Get the window (global namespace) an iframe
// -----------------------------------------------------------------------------
HxG_5.prototype.HxIFrameMgr.prototype.getWin = function (obj) {
	var d = hX_5.ifr.getDoc(obj);
	return ((d==null) ? null : ((d.defaultView) ? d.defaultview : ((d.parentWindow) ? d.parentWindow : null)));
}

// -----------------------------------------------------------------------------
// Public: General purpose helper methods
// -----------------------------------------------------------------------------

// Public: Add up to "n" leading characters to a string or number.  Zero used by default as pad.
HxG_5.prototype.padL = function (theNumber, toFill, pad) {
	var s = theNumber.toString(), i, n = toFill - s.length;
	for (i = 0; i < n; i++) s = ((pad) ? pad:hX_5._Z) + s;
	return s;
}

// Public: Add up to "n" trailing characters to a string or number (converted to string if need be).
HxG_5.prototype.padR = function (theNumber, toFill, pad) {
	var s = theNumber.toString(), i, n = toFill - s.length;
	for (i=0; i<n; i++) s = s + ((pad) ? pad:hX_5._Z);
	return s;
}

// Public:	Truncates a string of digits, rounding the last digit if need be
// Rounding mode to round towards the "nearest neighbor" unless both neighbors are 
// equidistant, in which case, round towards the even neighbor. Behaves as for 
// ROUND_HALF_UP if the digit to the left of the discarded fraction is odd; 
// behaves as for ROUND_HALF_DOWN if it's even
HxG_5.prototype.truncateRoundHalfEven = function (str, chars) {
	var ostr = str, oflow = false;
	if (ostr.length > chars) {
		var tstr = ostr.substr(chars);
		ostr = ostr.substr(0, chars);
		// We'll do only a limited cascading round (first 3 discarded digits only)
		var r = (tstr.length > 2) ? (new Number(tstr.substr(0,3))) : ((tstr.length > 1) ? (new Number(tstr.substr(0,2))*10) : ( new Number(tstr.charAt(0))*100));
		var r1 = new Number (ostr.charAt(ostr.length-1));
		if (r > 500 || (r == 500 && ((Math.floor(r1/2)*2) != r1))) {
			if (r1+1 == 10) {
				var n = new Number(ostr) +1, nn = new Number(ostr);
				n = n.toString(); nn = nn.toString();
				if (n.length > nn.length && str.charAt(0) != hX_5._Z) {
					ostr = hX_5._Z;
					oflow = true;
				} else {
					ostr = hX_5.padL(n, chars);
				}
			} else
				ostr = ostr.substr(0,ostr.length-1) + (r1+1);
		}
	}
	return {value: ostr, overflow: oflow};
}

// Public: Replace all occurrences of one pattern of chars with another
HxG_5.prototype.replaceAll = function (s, ch1, ch2) {
	//RATLC00971888 if the find and replace patterns are the same return
	if(ch1==ch2)return s;
	var o = s, t = ch1.length, i = o.indexOf(ch1);
	while (i >= 0 && o.length > 0) {
		o = (i==0) ? (ch2 + o.substr(ch1.length)) : (o.substr(0,i) + ch2 + o.substr(i+t));
		i = o.indexOf(ch1);
	}
	return o;
}

// Public: Replace a character in a string with another character
HxG_5.prototype.rplOne = function (s, pos, ch1) {
	return (pos < s.length)?(s.substr(0,pos)+ch1+s.substr(pos+1)):s;
}

// Public: Strip leading, trailing spaces, replace multiple spaces with a single space
HxG_5.prototype.allTrim = function (s) {
	var o = hX_5.replaceAll(s, "  ", hX_5._B);
	if (o.charAt(0) == hX_5._B) o = o.substr(1);
	if (o.charAt(o.length-1) == hX_5._B) o = o.substr(0,o.length-1);
	return o;
}

// Public: Strip leading, trailing spaces from a string
HxG_5.prototype.Trim = function (s) {
	if (null == s || 0 == s.length || !hX_5.isString(s)) return s;
	var o, q = 0;
	while (q < s.length && s.substr(q,1) == hX_5._B) q++;
	o = s.substr(q);
	q = o.length-1;
	while (q >= 0 && o.substr(q,1) == hX_5._B) q--;
	return (o.slice(0,q+1));
}

// Public: Strip leading spaces (or specified character) from a string
HxG_5.prototype.trimL = function (s, iChar) {
	if (null==s || 0==s.length) return s;
	var q = 0, ch = (iChar == null) ? hX_5._B:iChar;
	while (q < s.length && s.substr(q,1) == ch) q++;
	return (s.substr(q));
}

// Public: Strip trailing spaces (or specified character) from a string
HxG_5.prototype.trimR = function (s, iChar) {
	if (null==s || 0==s.length) return s;
	var q = s.length-1, ch = (iChar == null) ? hX_5._B:iChar;
	while (q >= 0 && s.substr(q,1) == ch) q--;
	return (s.substr(0,q+1));
}

// Public: Does a character match one of a set of chars
HxG_5.prototype.isChar = function (c, set) {
	return (set.indexOf(c) >= 0) ? true : false;
}

// Public: Given a URL passed to us via various means (e.g., as a param) ensure the params are encoded correctly
HxG_5.prototype.reEncodeURI = function (uri) {
	var r=uri, l=r.indexOf(hX_5._QM);
	if (l>=0) {
		var s, p, p1, params=r.substr(l+1);
		r = r.substr(0,l+1);
		while (params.length > 0) {
			l = params.indexOf(hX_5._AMP);
			if (l>=0) {
				s = hX_5._AMP;
				p = params.substr(0,l);
				params = params.substr(l+1);
			} else {
				p = params;
				s = hX_5._E;
				params = hX_5._E;
			}
			l = p.indexOf(hX_5._EQ);
			if (l>=0) {
				p1 = p.substr(l+1);
				p = p.substr(0,l);
				p1 = decodeURIComponent(p1);
				p1 = hX_5._EQ+this.encode(p1);
			} else {
				p1=hX_5._E;
			}
			p = decodeURIComponent(p);
			r=r+this.encode(p)+p1+s;
		}
	}
	return r;
}

// Public: Encode 'search' (query) name/value pairs for use in a URI
// Returns the query part of a URI, e.g., ?name=value&type=fish
// nv is an array of name/value pairs (e.g., element of the array consists of [name,value]
// "n" is the name, "v" is a value.  They can be a single pair of strings or a pair of arrays of strings
// The base url is required so we can test if there already is a query on it
HxG_5.prototype.encodeURIParams = function (base, nv) {
	var r=hX_5._E, i, hasQ=(base!=null && (base.indexOf(hX_5._QM)>=0));
	for (i=0; i<nv.length; i++) {
		r = r+((!hasQ) ? hX_5._QM:hX_5._AMP);
		r = r+this.encode(nv[i][0]);
		r = r + ((nv[i][1])?(hX_5._EQ+this.encode(nv[i][1])):hX_5._E);
		hasQ = true;
	}
	return r;
}
// Encoder for a URI component
HxG_5.prototype.encode = function (s) {
	return encodeURIComponent(s);
}

// Public: Given a message, format it replacing {0} {1} ... in the message with the args in the supplied array
HxG_5.prototype.NLSFormatMsg = function (template, args) {
	var i, r = hX_5._E, l1="{", l2="}";
	if (template != null) {
		r = template;
		if (args != null) {
			for (i = 0; i < args.length; i++)  r = hX_5.replaceAll (r, l1 + i + l2, args[i]);
		}
	}
	return r;
}

// Public:	Given a DOM Object, see if another object is a child of it
HxG_5.prototype.findNestedObject = function(obj, ichildtag) {

	if(obj == null || obj.childNodes == null || obj.childNodes.length == 0)
		return null;

	var i, nested_child, children = obj.childNodes, childtag = ichildtag.toLowerCase();
	for(i = 0; i < children.length; i++) {
		if(children[i].nodeName.toLowerCase() == childtag)
			return children[i];
	}
	for(i = 0; i < children.length; i++) {
		nested_child = hX_5.findNestedObject(children[i], childtag);
		if (nested_child != null) {
			return nested_child;
		}
	}
	return null;
}

// Public:  Apply an attribute value to all objects within a container
HxG_5.prototype.applyNestedAttribute = function(root, attributeName, attributeValue, force) {
	if (root.nodeType == hX_5.ELEMENT_NODE) {
		if ((root.getAttribute(attributeName) != null) || force)
			// Netscape bungs this up through setAttribute
			if (attributeName == hX_5._DSB) root.disabled = attributeValue;
			else root.setAttribute(attributeName, attributeValue);
	}
	for (var child = root.firstChild; child != null; child = child.nextSibling)
		this.applyNestedAttribute (child, attributeName, attributeValue, force);
}

// Public:  Apply a property value to all objects within a container
HxG_5.prototype.applyNestedProperty = function(root, propertyName, propertyValue) {
	if (root.nodeType == hX_5.ELEMENT_NODE)  hX_5.g.setP(root, propertyName, propertyValue);
	for (var child = root.firstChild; child != null; child = child.nextSibling)
		this.applyNestedProperty (child, propertyName, propertyValue);
}

// Public: Given a table cell, get the first child in it
HxG_5.prototype.getFirstChildInCell = function (cell) {
	var r = (cell) ? cell.firstChild : null;
	while (r && r.nodeType != hX_5.ELEMENT_NODE) r = r.nextSibling;
	return r;
}

//----------------------------------------------------------------------------------------------------
// Public:	Advance to the next/previous field in the tab order of the page
// 			See hxclient_nav for a detailed description
//			NOTE:  There's a known problem in IE where it doesn't draw the focus rectangle around
//				   radio buttons, checkboxes and some anchor tags if the page has not been "manually"
//				   focussed using the TAB key.  The right field has focus, it's just not obvious it has focus.
//
// Arguments:	t		(string tagname or object)	Reference object (i.e., move forward/back from this object)
//				forward	(bool)						If true move forward, else move backward
HxG_5.prototype.gotoNextPrev = function (t, forward, skipHelper) {
	// Figure out the next object to get focus
	if (hX_5.tabNav == null) hX_5.tabNav = new hX_5.tabNavigator();
	var o = hX_5.tabNav.getFocus(t, forward, skipHelper);
	if (o) hX_5.i.setFocus(o, (o.tagName==hX_5._INP && o.type.toLowerCase()==hX_5._XT), true);
}

// Public: Get the tabbable elements in a page
HxG_5.prototype.getTabElements = function () {
	// If we don't have a tab order navigator, make one
	if (hX_5.tabNav == null) hX_5.tabNav = new hX_5.tabNavigator();
	return hX_5.tabNav.getTabItems();
}

// Helper: make an array of "tabbable" tags
HxG_5.prototype.tabbableElements = null;
HxG_5.prototype.getTabbableElements = function () {
	if (hX_5.tabbableElements == null)
		hX_5.tabbableElements = [
			new hX_5.tagListItem(hX_5._INP,null),
			new hX_5.tagListItem(hX_5._SEL,null),
			new hX_5.tagListItem(hX_5._TXT,null),
			new hX_5.tagListItem(hX_5._BTN,null),
			new hX_5.tagListItem(hX_5._ANC,null),
			new hX_5.tagListItem(hX_5._EMB,null),
			new hX_5.tagListItem(hX_5._OBJ,null),
			new hX_5.tagListItem(hX_5._APP,null)];
	// 	Currently we don't correctly handle 'area'
	return hX_5.tabbableElements;	
}

// Public: Check if this tag is tabbable because it has a taborder set
HxG_5.prototype.isTabbable = function (node) {
	if (hX_5.isTag(node, hX_5.getTabbableElements())) {
		if (node.tagName==hX_5._INP) {
			if (node.type.toLowerCase()==hX_5._HI) return false;
			if (node.id && node.id.length>9 && node.id.substr(node.id.length-10)=="_HXB_GHOST") return false;
		}
		return true;
	}
	if (node && node.nodeType!=undefined && node.nodeType == hX_5.ELEMENT_NODE) {
		if (node.tabIndex!=undefined && node.tabIndex > 0)
			return true;
	}
	return false;
}

// Helper: make an array of "valued" tags
HxG_5.prototype.valuableElements = null;
HxG_5.prototype.getValuableElements = function () {
	if (hX_5.valuableElements == null)
		hX_5.valuableElements = [
			new hX_5.tagListItem(hX_5._INP,null),
			new hX_5.tagListItem(hX_5._SEL,null),
			new hX_5.tagListItem(hX_5._TXT,null)];
	// 	Currently we don't correctly handle objects or file uploads
	return hX_5.valuableElements;	
}

// Public: Check if this tag is "stateful" that is has a value we can manipulate
// If extended is set, then we count hidden fields and JSF "groups of radio/checks"
HxG_5.prototype.isValuable = function (o, extended) {
	var lc, node=hX_5.getElementById(o);
	if (node!=null) {
		if (hX_5.isTag(node, hX_5.getValuableElements())) {
			if (node.tagName==hX_5._INP) {
				lc = node.type.toLowerCase();
				if (!(lc==hX_5._HI && extended)) if (lc!=hX_5._XT && lc!=hX_5._RAD && lc!=hX_5._CHK && lc!=hX_5._PWD) return false;
				if (node.id && node.id.length>9 && node.id.substr(node.id.length-10)=="_HXB_GHOST") return false;
				if (extended && node.disabled!=undefined && node.disabled) return false;
			}
			return true;
		// JSF radio button and checkbox groups are odd
		} else if (extended && node.tagName!=undefined && node.tagName==hX_5._TAB && node.id!=undefined && node.id!=null) {
			var c, p=node.parentNode;
			while (p!=null && p.tagName!=undefined && p.tagName!=hX_5._FRM) p=p.parentNode;
			if (p && p.tagName!=undefined && p.tagName==hX_5._FRM) {
				c = p.elements[node.id];
				c = (c.length==undefined)?[c]:c;
				if (hX_5.isObject(c) && c.length!=undefined) {
					lc = c[0].type.toLowerCase();
					if (lc==hX_5._RAD || lc==hX_5._CHK) return true;
				}
			}
		}
	}
	return false;
}

// Public: For a supported stateful field, get it's value.  For <select> tags the value is the index or an array of indexes
// If submittable is set, we package values for "form submission"
// If extended is set, we'll handle JSF groups of checks/radios
HxG_5.prototype.getValuableValue = function (o, submittable, extended) {
	var r=null, lc, i, j, p, c, node=hX_5.getElementById(o);
	if (node!=null) {
		// JSF radio button and checkbox groups are odd in that the id COULD be the table id
		if (extended && node.tagName!=undefined && node.tagName==hX_5._TAB && node.id!=undefined && node.id!=null) {
			if (!submittable || !node.disabled) {
				p=node.parentNode;
				while (p!=null && p.tagName!=undefined && p.tagName!=hX_5._FRM) p=p.parentNode;
				if (p && p.tagName!=undefined && p.tagName==hX_5._FRM) {
					c = p.elements[node.id];
					c = (c.length==undefined)?[c]:c;
					if (hX_5.isObject(c) && c.length!=undefined) {
						lc = c[0].type.toLowerCase();
						if (lc==hX_5._RAD || lc==hX_5._CHK) node = c[0];
					}
				}
			}
		}
		// JSF has several cases IN ADDITION to radio buttons, where more than one element may be
		// on the page with the same "name" (and may have no id).
		// If this node has a name, then see if this is an array case, otherwise treat as a singleton
		c = (node.name!=undefined && node.name!=hX_5._E && node.form!=undefined) ? node.form.elements[node.name] : [node];
		c = (c==null)?[node]:((c.length==undefined)?[c]:c);
		for (j=0; j<c.length; j++) {
			node = c[j];
			if (!submittable || !node.disabled) {
				// <input> and <textarea> tags
				if (node.tagName==hX_5._TXT || node.tagName==hX_5._INP) {
					lc = node.type.toLowerCase();
					// Type is text, password or hidden
					if (node.tagName==hX_5._TXT || lc==hX_5._XT || lc==hX_5._PWD || lc==hX_5._HI) {
						r=node.value;
						break;	// we don't handle multiple values for these types
					// Checkboxes
					} else if (lc==hX_5._CHK) {
						// We haven't seen this checkbox before
						if (r==null) {
							r = ((node.checked)?((node.value!=undefined && node.value.length>0)?node.value:"on"):null);
						} else {
							if (hX_5.isString(r)) r = [r];
							if (node.checked) r.push((node.value!=undefined && node.value.length>0)?node.value:"on");
						}
					// Radio buttons
					} else if (lc==hX_5._RAD) {
						// Only one radio can be selected, so if r is set, don't change it
						if (r==null) r = (node.checked) ? ((node.value!=undefined && node.value.length>0)?node.value:"on") : null;
					}
				// <select> turned into <option>
				} else if (node.tagName==hX_5._OPT) {
					// We haven't seen this option before
					if (!submittable || !node.parentNode.disabled) {
						if (r==null) {
							r = ((node.selected)?((node.value!=undefined && node.value.length>0)?node.value:"on"):null);
						} else {
							if (hX_5.isString(r)) r = [r];
							if (node.selected) r.push((node.value!=undefined && node.value.length>0)?node.value:"on");
						}
					}
				}
			}
		}
	}
	return r;
}

// Public: For a supported stateful field, set its value.  For <select> tags the value is the index or an array of indexes
HxG_5.prototype.setValuableValue = function (o, v) {
	var r=false, lc, i, j, p, c, node=hX_5.getElementById(o);
	if (node!=null) {
		// JSF radio button and checkbox groups are odd in that the id COULD be the table id
		if (node.tagName!=undefined && node.tagName==hX_5._TAB && node.id!=undefined && node.id!=null) {
			p=node.parentNode;
			while (p!=null && p.tagName!=undefined && p.tagName!=hX_5._FRM) p=p.parentNode;
			if (p && p.tagName!=undefined && p.tagName==hX_5._FRM) {
				c = p.elements[node.id];
				c = (c.length==undefined)?[c]:c;
				if (hX_5.isObject(c) && c.length!=undefined) {
					lc = c[0].type.toLowerCase();
					if (lc==hX_5._RAD || lc==hX_5._CHK) node = c[0];
				}
			}
		}
		// JSF has several cases IN ADDITION to radio buttons, where more than one element may be
		// on the page with the same "name" (and may have no id).
		// If this node has a name, then see if this is an array case, otherwise treat as a singleton
		c = (node.name!=undefined && node.name!=hX_5._E && node.form!=undefined) ? node.form.elements[node.name] : [node];
		c = (c.length==undefined)?[c]:c;
		for (j=0; j<c.length; j++) {
			node = c[j];
			// <input> and <textarea> tags
			if (node.tagName==hX_5._TXT || node.tagName==hX_5._INP) {
				lc = node.type.toLowerCase();
					if (node.tagName==hX_5._TXT || lc==hX_5._XT || lc==hX_5._PWD || lc==hX_5._HI) {
					node.value = (v==null)?hX_5._E:v;
					r = true;
				} else if (lc==hX_5._CHK) {
					node.checked = false;
					if (v!=null) {
						if (hX_5.isString(v)) {
							node.checked = (v=="on" || (node.value!=undefined && node.value.length>0 && node.value==v));
						} else {
							if (node.value!=undefined && node.value.length>0) {
								for (i=0; !node.checked && i<v.length; i++) if (node.value == v[i]) node.checked=true;
							}
						}
					}
					r = true;
				} else if (lc==hX_5._RAD) {
					node.checked = (node.value!=undefined && node.value==v);
					r = true;
				}
			} else if (node.tagName==hX_5._OPT) {
				node.selected = false;
				if (v!=null) {
					if (hX_5.isString(v)) {
						node.selected = (node.value!=undefined &&  node.value==v);
					} else {
						if (node.value!=undefined && node.value.length>0) {
							for (i=0; !node.selected && i<v.length; i++) if (node.value == v[i]) node.selected=true;
						}
					}
				}
				r = true;
			}
		}
	}
	return r;
}

//----------------------------------------------------------------------------------------------------
// Public:	Check if a object is of a type (e.g., is this tag an <input type=text> or <input type=button>)
//
// Arguments:	inobj		(string tagname or object)	Identifies the object
//				tagset		(array of tagListItems)		Array of allowed tag types.  Each element
//														consists of a tagname (e.g., 'input') and an optional
//														tag type (e.g., 'text').  Type is used for tags like <input>
//														only.
//				(return)	(bool)						True if this object is in the list of tags
HxG_5.prototype.tagListItem = function (tag, type) {
	this.tag = tag.toLowerCase();
	this.type = (type != null) ? type.toLowerCase() : null;
}
HxG_5.prototype.isTag = function (inobj, tagset) {
	var i, hit=false, obj=hX_5.getElementById(inobj);
	if (obj && obj.nodeType!=undefined && obj.nodeType == hX_5.ELEMENT_NODE) {
		for (i = 0; i < tagset.length && !hit; i++) {
			if (tagset[i].tag == obj.tagName.toLowerCase()) {
				if ((tagset[i].type == null) || (obj.getAttribute(hX_5._TYP).toLowerCase() == tagset[i].type))
					hit = true;
			}
		}
	}
	return hit;
}

//----------------------------------------------------------------------------------------------------
// Public:	Find the first occurence of an object of a type within a scope.  For example, find the first
//			"span" within a table cell.
//
// Arguments:	inroot		(string tagname or object)	Identifies the object to search in
//				tagset		(array of tagList items)	As above, array of allowed types
//				(return)	(object)					The located object (null if none located)
HxG_5.prototype.findFirstTag = function (inroot, tagset) {
	var obj, root = hX_5.getElementById(inroot);
	if (hX_5.isTag(root, tagset))
		return root;
	if (root && root.firstChild!=undefined) {
		for (var child = root.firstChild; child != null; child = child.nextSibling) {
			obj = this.findFirstTag (child, tagset);
			if (obj != null)
				return obj;
		}
	}
	return null;
}

//----------------------------------------------------------------------------------------------------
// Public:	Check if an object is a child of another object
//
// Arguments:	child		(string tagname or object)	Identifies the putative child object
//				parent		(string tagname or object)	Identifies the putative parent object
//				parentname	(string)					optional, usually "_HXPOPUP" if provided
//														if provided also checks if object is is a child
//														of a tag with this in it's name attribute
//														(e.g., this a child of tag FORM or of a POPUP)
//				(return)	(boolean)					True if the child is indeed a child of the parent
HxG_5.prototype.isChildTag = function (child, parent, parentname) {
	var c = hX_5.getElementById(child), p = hX_5.getElementById(parent);
	while (c!=null) {
		if (c==p) return true;
		if (parentname!=null && c.name!=undefined && c.name.indexOf(parentname)>=0) return true;
		c = c.parentNode;
	}
	return false;
}

//----------------------------------------------------------------------------------------------------
// Public:	Check if a object is of a JS Type (e.g., is this a "string" object)
//
// These four are passed in "object" as their argument
HxG_5.prototype.isObject = function(obj) {
	return (typeof(obj) == "object");
}
HxG_5.prototype.isFunction = function(obj) {
	return (typeof(obj) == "function");
}
HxG_5.prototype.isString = function(obj) {
	return (typeof(obj) == "string");
}
HxG_5.prototype.isNumber = function(obj) {
	return (typeof(obj) == "number");
}
HxG_5.prototype.isArray = function(obj) {
	return (typeof(obj) == "object" && obj.length!=undefined && obj.push!=undefined);
}
// These two are passed in "typeof (object)" as their argument
// You need to test variables with these tests (e.g., hX_5.isDef(typeof Foo)
// You can test properties/attributes of an object with foo.moo==undefined (no quotes)
// In other words, if there's no period in the thing being tested, use one of these routines
HxG_5.prototype.isUnDef = function(objt) {
	return (objt == "undefined");
}
HxG_5.prototype.isDef = function(objt) {
	return (objt != "undefined");
}

//----------------------------------------------------------------------------------------------------
// Public:	Pull a keycode from an event
// 			N.B.  the use of "which" below is for pre 6.0 Netscape.
//
// Arguments:	evt			(object)	The event object
//				(return)	int			The keycode (if there is one)
HxG_5.prototype.getKeyCode = function(ev) {
	return ((ev.keyCode) ? ev.keyCode : ev.which);
}

//----------------------------------------------------------------------------------------------------
// Public:	Is a keycode probably a function key?
HxG_5.prototype.isFcnKey = function(evt) {
	var code = hX_5.getKeyCode(evt);
	code = (code && (code != 0)) ? code : -1;
	if (((code < 48) && (code != 32 && code != 9 && code != 13))	// Basic keycodes like arrows/end/home (9=tab, 13=enter, 32=space)
	// 65-90 = alphanumerics, 96-111 = numeric keypad, 91-95 = ?
	|| (code >= 91 && code <= 93)				// fcn keys on keypad
	|| (code >= 112 && code <= 123)				// fcn keys
	|| (code >= 124 && code <= 185)				// not sure, but 145 = scroll lock
	|| (code >= 223) )							// not sure
		return true;
	return false;
}

//----------------------------------------------------------------------------------------------------
// Public:	Pull a character code from an event
//
// Arguments:	evt			(object)	The event object
//				(return)	int			The keycode (if there is one)
HxG_5.prototype.getChrCode = function(ev) {
	return (String.fromCharCode ((ev.charCode) ? ev.charCode : ev.keyCode));
}
HxG_5.prototype.setChrCode = function(ev, c) {
	if (ev.charCode) ev.charCode = c.charCodeAt(0);
	else if (ev.keyCode) ev.keyCode = c.charCodeAt(0);
}

//----------------------------------------------------------------------------------------------------
// Public:	Test if a keyCode represents a function key or not
//
// Arguments:	keycode		(keycode)	The keycode (from an event) to test
//				(return)	(bool) 		True if it's a function key
HxG_5.prototype.isFunctionKey = function(keyCode) {
	return (112 <= keyCode && keyCode << 123);
}

//----------------------------------------------------------------------------------------------------
// Public:	Return the target of the event
//
// Arguments:	evt			(event object)	The event object
//				(return)	(DOM object) the target of the event
HxG_5.prototype.evtGetTarget = function (evt) {
	return ((evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : null));
}

//----------------------------------------------------------------------------------------------------
// Public:	Return the first (or only) class associated with an object
//
// Arguments:	inobj		(string tagname or object) Identifies the object
//				(return)	(string) the class name (null if none)
HxG_5.prototype.getFirstClassName = function (inobj) {
	var r = null, obj = hX_5.getElementById(inobj);
	if (obj && obj.className) {
		r = hX_5.allTrim(obj.className);
		if (r.indexOf(hX_5._B) > 0)
			r = r.substr(0,r.indexOf(hX_5._B));
	}
	return r;
}

//----------------------------------------------------------------------------------------------------
// Public:	Append a class to the (list of) classes used by an object, optionally removing one or more
//			classes before appending
//
// Arguments:	inobj		(string tagname or object) Identifies the object
//				name		(string)	class (name of class) to add to the object
//				remove		(string)	first remove all classes from the object that START WITH this string
//				prepend		(bool)		Do a prepend instead
HxG_5.prototype.appendClassName = function (inobj, name, remove, prepend) {
	var obj = hX_5.getElementById(inobj), c = hX_5._B + hX_5.allTrim(obj.className);
	if (remove && remove.length > 0 && c.length > 0) {
		var nc, r = hX_5._B + remove, k = c.indexOf(r);
		while (k >= 0) {
			nc = c.substr(k+1);
			c = c.substr(0,k);
			k = nc.indexOf(hX_5._B);
			nc = (k<0) ? hX_5._E : nc.substr(k);
			c = c + hX_5._B + nc;
			c = hX_5._B + hX_5.allTrim(c);
			k = c.indexOf(r);
		}
	}
	c = hX_5.allTrim(c);
	if (prepend!=null)
		c = (c.length > 0) ? (name + hX_5._B + c) : name;
	else
		c = (c.length > 0) ? (c + hX_5._B + name) : name;
	if (obj.className != c) obj.className = c;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given a class name, remove a suffix from it (if one is found)
// Arguments:	c		(string)	class name
//				suff	(array)		array of possible suffixes to be removed
HxG_5.prototype.stripClassSuffix = function (c, suff) {
	if (c && suff && suff.length && suff.length > 0) {
		var i, r = c, l = suff.length, ll = c.length;
		for (i = 0; i < l; i++) {
			if (suff[i].length < c.length && c.substr(ll-suff[i].length) == suff[i])
				return c.substr(0,ll-suff[i].length);
		}
	}
	return r;
}

//----------------------------------------------------------------------------------------------------
// Public:	Is a class defined in a stylesheet of this page
//
// Arguments:	inobj		(string)	Class to test for (no leading period)
//				(return)	(object) 	Null if fails, else the style object representing this class
HxG_5.prototype.isClassDefined = function (testname) {
	if (hX_5.CSSSelectors == null) {
		if (hX_5.CSSSelectors) delete (hX_5.CSSSelectors);
		hX_5.CSSSelectors = [];
		var i, l=document.styleSheets.length;
		for (i = 0; i < l; i++) hX_5.readSheet(document.styleSheets[i]);
	}
	if (hX_5.CSSSelectors[hX_5._P + testname]!=null) return true;
	return false;
}
// Private helper for above
HxG_5.prototype.readSheet = function (sheets) {
	var rulelist, rule, selector, j, k, l;
	// IE puts imported stylesheets into "imports" (or it's supposed to -- this isn't really documented)
	if (sheets.imports!=null && sheets.imports.length) {
		l = sheets.imports.length;
		for (j=0; j<l; j++) {
			try {
				hX_5.readSheet(sheets.imports[j]);
			} catch(e) {
				// Ignore sheet (could be in a different namespace and can't be read)
			}
		}
	}
	rulelist = (sheets.cssRules!=undefined) ? sheets.cssRules : ((sheets.rules!=undefined) ? sheets.rules : null);
	if (rulelist != null) {
		l = rulelist.length
		for (j = 0; j < l; j++) {
			// Firefox (W3C browsers) puts @imported stylesheets in the rule list as a rule
			// If styleSeet is defined, then this is a (nested) stylesheet not a rule
			if (rulelist[j].styleSheet!=undefined) {
				try {
					hX_5.readSheet(rulelist[j].styleSheet);
				} catch (e) {
					// Ignore sheet (could be in a different namespace and can't be read)
				}
			} else {
				if (rulelist[j].selectorText!=null) {
					rule = rulelist[j].selectorText;
					while (rule && rule.length > 0) {
						k = rule.indexOf(hX_5._C);
						if (k < 0) {
							selector = rule;
							rule = hX_5._E;
						} else {
							selector = rule.substr(0,k);
							rule = rule.substr(k+1);
						}
						selector = hX_5.allTrim(selector);
						if (selector.length > 0 && selector.indexOf(hX_5._B) < 0 && selector.charAt(0) ==  hX_5._P)
							hX_5.CSSSelectors[selector] = selector;
					}
				}
			}
		}		
	}
}

//----------------------------------------------------------------------------------------------------
// Public:	Given a class name, return the value of a property in the class
//
// Arguments:	cl			(string)	a class name
//				prop		(string)	Property to look up (long form name, e.g., "text-align"
//				(return)	(string)	Value of property (or null)
//----------------------------------------------------------------------------------------------------
// Public:	Given a class (name), read a style in it and return the value
//			Special handling is provided for background-images
HxG_5.prototype.scratchImg	= null;
HxG_5.prototype.scratchImg1	= null;
HxG_5.prototype.scratchImg2	= null;
HxG_5.prototype.scratchImg3 = null;
HxG_5.prototype.scratchImg4 = null;
HxG_5.prototype.getClassStyle = function (cl, prop) {
	if (cl && cl.length > 0) {
		// If we don't have a temporary tag to try out styles, then make it
		if (this.scratchImg == null) {
			var img = hX_5.getElementById("HX_SCRATCH_IMG");
			if (!img) {
				img = document.createElement(hX_5._DIV);
				img.setAttribute(hX_5._ID, "HX_SCRATCH_IMG");
				img.style.display = hX_5._NO;
				img.style.width = hX_5._ZPX;
				img = document.body.appendChild(img);
			}
			this.scratchImg = img;
		}
		this.scratchImg.style.width = hX_5._E;
		this.scratchImg.className = cl;
		var bi = hX_5.g.getP(this.scratchImg, prop);
		this.scratchImg.style.width = hX_5._ZPX;
		if (prop == hX_5._BIMG) {
			if (bi && bi.length && bi.length > 0 && bi != hX_5._NO)
				return (this.stripURL(bi));
		} else
			return bi;
	}
	return null;
}

HxG_5.prototype.getEvalStyle = function (obj, prop, value) {
	if (obj && prop && value) {
		// If we don't have a temporary tag to try out styles, then make it
		if (this.scratchImg1 == null) {
			var img = hX_5.getElementById("HX_SCRATCH_IMG1");
			if (!img) {
				img = document.createElement(hX_5._DIV);
				img.setAttribute(hX_5._ID, "HX_SCRATCH_IMG1");
				img.style.display = hX_5._NO;
				img.style.width = hX_5._ZPX;
				img = document.body.appendChild(img);
			}
			this.scratchImg1 = img;
		}
		hX_5.g.setP(this.scratchImg1, prop, value);
		return (hX_5.g.getP(this.scratchImg1, prop));
	}
	return null;
}

HxG_5.prototype.getItemSize = function (obj, doRow) {
	if (obj) {
		// If we don't have a temporary tag to try out styles, then make it
		var img, w, h, o;
		if (this.scratchImg2 == null) {
			img = hX_5.getElementById("HX_SCRATCH_IMG2");
			if (!img) {
				img = document.createElement(hX_5._DIV);
				img.setAttribute(hX_5._ID, "HX_SCRATCH_IMG2");
				img.style.display = hX_5._NO;
				img.style.width = hX_5._ZPX;
				img = document.body.appendChild(img);
			}
			this.scratchImg2 = img;
		}
		this.scratchImg2.appendChild(obj);
		this.scratchImg2.style.width = hX_5._E;
		this.scratchImg2.style.height = hX_5._E;
		this.scratchImg2.style.display = hX_5._E;
		o = (doRow)?((obj.rows!=undefined && obj.rows.length>0)?obj.rows[0]:null):obj;
		if (o) {
			w = parseInt(o.offsetWidth,10); 
			h = parseInt(o.offsetHeight,10);
		}
		this.scratchImg2.style.display = hX_5._NO;
		this.scratchImg2.style.width =  hX_5._ZPX;
		this.scratchImg2.removeChild(obj);
		return (o) ? {H:h, W:w} : null;
	}
	return null;
}

//----------------------------------------------------------------------------------------------------
// Public:	Given a (style property) size value (e.g., 10pt or 12px) associated with an object, convefrt the size string
// to pixels
//
// Arguments:	size		(string)	Valu to look up and convert to pixels, e.g., "10px" or "32pt" or "12in"
//				(return)	(string)	Value of property in pixels or (null if not translatable (e.g., if "auto" or "thin")
//----------------------------------------------------------------------------------------------------
// Public:	Given a class (name), read a style in it and return the value
//			Special handling is provided for background-images
HxG_5.prototype.toPixels = function (obj,sz) {
	var r=null, img,div;
	if (obj!=null) obj = hX_5.getElementById(obj);
	if (obj!= null && sz && sz.length!=undefined && sz.length > 0) {
		// If we don't have a temporary tag to try out styles, then make it
		if (this.scratchImg4 == null) {
			img = hX_5.getElementById("HX_SCRATCH_IMG4");
			if (!img) {
				img = document.createElement(hX_5._DIV);
				img.setAttribute(hX_5._ID, "HX_SCRATCH_IMG4");
				img.style.display = hX_5._NO;
				img.style.margin = hX_5._ZPX;
				img.style.padding = hX_5._ZPX;
				img.style.borderWidth = hX_5._ZPX;
				img = document.body.appendChild(img);
				div = document.createElement(hX_5._INP);
				div.setAttribute("type","button");
				div.style.margin = hX_5._ZPX;
				div.style.padding = hX_5._ZPX;
				div.style.borderWidth = hX_5._ZPX;
				div = img.appendChild(div);
			}
			this.scratchImg4 = img;
		}
		// Make sure size is OK
		r = parseInt(sz,10);
		if (r==null || isNaN(r) || !hX_5.isNumber(r) || sz.indexOf(hX_5._PCT)>=0) r=null;
		else {
			div = this.scratchImg4; img = div.firstChild;
			div.style.fontFamily=hX_5.g.getP(obj,"font-family");
			div.style.fontSize=hX_5.g.getP(obj,"font-size");
			try {
				img.style.width = sz;
				div.style.display = hX_5._E;
				r = hX_5.g.getRendWidth(img);
				div.style.display = hX_5._NO;
			} catch(e) {
				r = null;
			}
		}
	}
	return r;
}

// Determine the width of a scroll bar (system-provided) associated with an object
HxG_5.prototype.getScrollbarWidth = function() {
	var img=null, ta=null, w=0;
	if (this.scratchImg3 == null) {
		img = hX_5.getElementById("HX_SCRATCH_IMG3");
		if (!img) {
			img = document.createElement(hX_5._DIV);
			img.setAttribute(hX_5._ID, "HX_SCRATCH_IMG3");
			img.style.display = hX_5._NO;
			img.style.width = hX_5._ZPX;
			this.scratchImg3 = document.body.appendChild(img);
		}
		if (img.firstChild==null) {
			ta = document.createElement(hX_5._TXT);
			ta.cols="20";
			ta.rows="2";
			img.appendChild(ta);
		}
	}
	img = this.scratchImg3;
	ta = img.firstChild;
	ta.style.overflow="hidden";
	img.style.width="300"+hX_5._PX;
	img.style.display=hX_5._BLK;
	w = ta.clientHeight;
	ta.style.overflow="scroll";
	w -= ta.clientHeight;
	img.style.width=hX_5._E;
	img.style.display=hX_5._NO;
	return w;
}

//----------------------------------------------------------------------------------------------------
// Public:	Remove an "url ()" wrapper from an image property
//
// Arguments:	url			(string)	property to tidy up
//				(return)	(string)	cleaned URL
HxG_5.prototype.stripURL = function (url) {
	var u = url;
	if ((u.substr(0,4).toLowerCase() == hX_5._URL) || (u.substr(0,4).toLowerCase() == hX_5._URI)) {
		u = u.substr(4);
		u = u.substr(0,u.length-1);
		u = hX_5.replaceAll(u, hX_5._DQT,hX_5._E);
		u = hX_5.replaceAll(u, hX_5._SQT,hX_5._E);
	}
	return u;
}

//----------------------------------------------------------------------------------------------------
// Public:	"W3C" conformant getter for innerHTML
// 			Note that all major browsers support the IE way of doing this (non conformant)
//			The "official" way (should it ever be needed) is to do it by creating a "range" object
//
// Arguments:	obj			DOMobj
//				(return)	innerHTML
HxG_5.prototype.getIHTML = function (obj) {
	return ((obj && obj.innerHTML) ? obj.innerHTML : null);
}

//----------------------------------------------------------------------------------------------------
// Public:	"W3C" conformant setter for innerHTML
// 			Note that all major browsers support the IE way of doing this (non conformant)
//			The "official" way is to do it by creating a "range" object and setting a doc fragment in it
//			HOWEVER, when Mozilla is in XML doc type mode, the createContextualFragment call (on the Range object)
//			always returns an error (and by extension, innerHTML returns an error as it's a wrapper around this call).
//			They don't consider this a bug (though almost every other human does consider it a bug).
//			So when in this mode, we need to provide an alternative implementation.
//
// Arguments:	obj			DOMobj
//				html		the html to set
HxG_5.prototype.setIHTML = function (obj, html, doc) {
	if (!obj) return;
	if (hX_5.brw.isNavOrMoz() && !hX_5.brw.hasInnerHTML()) {
		// Remove any existing nodes
		while (obj.lastChild) obj.removeChild(obj.lastChild);
		hX_5.htmlParseRun(obj, html, false, doc);
	} else
		obj.innerHTML = html;
}

// Private: Interpret a run of HTML and populate the DOM (as children of root) with the tags
// This routine is not perfect -- it does not handle ill-formed HTML well, nor does it handle
// some of the wack tags like <pre>
HxG_5.prototype.htmlParseRun = function(root, s, nohead, Doc) {
var doc=((Doc!=null)?Doc:window.document), ch=0, chL, ch1, ch2, ch3, i, j, txt, tx1, l=s.length, sN, sV, aV, node, sscr="</Script>", sty="style", obj=root, objStack=[root], inHead=((nohead!=null) && nohead);
try {
	// Run through all the text in the string
	while (ch!=null && ch<l) {
		chL = ch;
		// Look for next tag start
		while (ch<l && s.charAt(ch)!=hX_5._LT) ch++;
		// Push out anything found before that tag as text
		if (ch!=chL) {
			// When we create a test node, ampersanded special characters aren't handled, so we'll handle the common ones here
			txt=s.substring(chL,ch);
			if (txt.indexOf(hX_5._AMP)>=0) {
				ch1=0; tx1=hX_5._E;
				while (ch1<txt.length) {
					if (txt.charAt(ch1)==hX_5._AMP) {
						ch1++; ch2=hX_5._E;
						while (txt.charAt(ch1)!=hX_5._SM) { ch2 = ch2+txt.charAt(ch1); ch1++; }
						if (ch2.length>0) {
							if (ch2.charAt(0)==hX_5._LB) {
								ch2 = parseInt(ch2.substr(1),10);
								tx1 = tx1 + String.fromCharCode(ch2)
							} else {
								// We can't handle all the possible abbrev for chars.  We have a table
								// of some that we handle
								ch2 = ch2.toLowerCase();
								for (ch3=0;ch3<hX_5.AMP_ABBS.length;ch3++) {
									if (ch2==hX_5.AMP_ABBS[ch3][0]) {
										tx1 = tx1 + hX_5.AMP_ABBS[ch3][1];
										ch3=-1;
										break;
									}
								}
								if (ch3>=0) tx1 = tx1 + hX_5.AMP_ABBS[0][1];
							}
						}
					} else {
						tx1 = tx1+txt.charAt(ch1);
					}
					ch1++;
				}
				txt = tx1;				
			}
			if (!inHead) obj.appendChild(doc.createTextNode(txt));
		}

		// Parse the located tag
		chV = hX_5.htmlParseTag(s, ch, true);
		if (chV==null) {
			// Done
			ch=null;
		} else {
			ch = chV.ch;
			// It's a comment, so ignore it
			if (chV.tag.length==0) {
			} else {
				// It's an end tag, so "pop" a level off the stack
				if (chV.tag.charAt(0)==hX_5._SL) {
					if (!inHead) {
						if (hX_5.htmlTagIsContainer(chV.tag)) {
							if (objStack.length>1) {
								objStack.pop();
								obj = objStack[objStack.length-1];
							} else {
								throw new Error(hX_5._E);
							}
						}
					}
				// It's a new tag
				} else {
					if (!inHead) {
						if (chV.tag==hX_5._SCR) {
							// Find the end script tag
							ch1 = s.indexOf(sscr.toUpperCase(),ch);
							ch2 = s.indexOf(sscr.toLowerCase(),ch);
							ch3 = s.indexOf(sscr,ch);
							ch2 = (ch3<0)?ch2:((ch2<0)?ch3:Math.min(ch2,ch3));
							ch1 = (ch2<0)?ch1:((ch1<0)?ch2:Math.min(ch2,ch1));
							if (ch1<0) {
								ch = null;
							} else {
								node = doc.createElement(chV.tag);
								for (i=0; i< chV.attN.length; i++)
									node.setAttribute(chV.attN[i], chV.attV[i]);
								node.text = s.substring(ch,ch1);
								obj.appendChild(node);
								ch = ch1+sscr.length;
							}							
						} else {
							node = doc.createElement(chV.tag);
							for (i=0; i< chV.attN.length; i++) {
								// Add the attribute, style needs special handling
								if (chV.attN[i]==sty) {
									aV=chV.attV[i];
									node.setAttribute(chV.attN[i], aV);
									while (aV.length > 0) {
										j = aV.indexOf(hX_5._SM);
										if (j >= 0) {
											sN = hX_5.Trim(aV.substr(0,j));
											aV = hX_5.Trim(aV.substr(j+1));
										} else {
											sN = aV; 
											aV = hX_5._E;
										}
										if (sN.length > 0) {
											sV = hX_5._E;
											j = sN.indexOf(hX_5._CL);
											if (j >= 0) {
												sV = hX_5.Trim(sN.substr(j+1));
												sN = hX_5.Trim(sN.substr(0,j));
											}
											if (doc.body.currentStyle) {
												node.style.setAttribute(hX_5.g.CSStagToDOMtag(sN), sV);
											} else {
												node.style.setProperty(sN, sV, hX_5._E);
											}
										}
									}
								} else {
									if ((chV.attV[i])==hX_5._E)
										node.setAttribute(chV.attN[i], true);
									else
										node.setAttribute(chV.attN[i], chV.attV[i]);
								}				
							}
							obj.appendChild(node);
							if (hX_5.htmlTagIsContainer(chV.tag) && !chV.ended) {
								objStack.push(node);
								obj = node;
							}
						}
					} else if (chV.tag==hX_5._BBODY) inHead=false;
				}
			}
		}
	}
} catch (e) {
	if (!inHead && root && doc) root.appendChild(doc.createTextNode("ERROR"));
}
}
// Skip over white space in a string
HxG_5.prototype.htmlTagIsContainer = function(t) {
	return (!(t == hX_5._BR || t==hX_5._HR || t == hX_5._INP || t == hX_5._IMG));
}
// Skip over white space in a string
HxG_5.prototype.htmlSkipWS = function(s, ch) {
	// tab, newline, space and linefeed are the HTML white space characters, a run of which collapses to one space
	var c=s.charCodeAt(ch), l=s.length;
	while (ch<l && (c==32 || c==9 || c==10 || c==13)) {ch++;c=s.charCodeAt(ch);}
	return ch;
}
// Skip over non-white space in a string
HxG_5.prototype.htmlSkipNotWS = function(s, ch, eq1, eq2, eq3) {
	var c=s.charCodeAt(ch), cc1=((eq1!=null)?eq1.charCodeAt(0):32), cc2=((eq2!=null)?eq2.charCodeAt(0):32), cc3=((eq3!=null)?eq3.charCodeAt(0):32), l=s.length;
	while (ch<l && c!=32 && c!=9 && c!=10 && c!=13 && c!=cc1 && c!=cc2 && c!=cc3) {ch++;c=s.charCodeAt(ch);}
	return ch;
}
// With ch pointing at a < character, parse the string until a > is hit.  ch is advanced to the end of the tag
// and a structure describing the tag is returned
HxG_5.prototype.htmlParseTag = function(s, ch, saveArgs) {
	var a,v, chL, tag=null, attN=null, attV=null, l=s.length, ended=false;
	if (saveArgs!=null && saveArgs) {attN=[]; attV=[];}
	if (ch>=l || s.charAt(ch)!=hX_5._LT) return null;
	ch++;
	if (s.charAt(ch)==hX_5._EX) {
		while (ch<l && s.charAt(ch)!=hX_5._GT) ch++;
		tag=hX_5._E;
	} else {
		chL = ch;
		if (s.charAt(ch)==hX_5._SL) ch++;
		ch = hX_5.htmlSkipNotWS(s,ch,hX_5._SL,hX_5._GT);
		tag = s.substring(chL,ch);
		ch = hX_5.htmlSkipWS(s,ch);
		while (ch<l && s.charAt(ch)!=hX_5._GT && s.charAt(ch)!=hX_5._SL) {
			chL = ch;
			ch = hX_5.htmlSkipNotWS(s, ch, hX_5._EQ, hX_5._SL, hX_5._GT);
			a = s.substring(chL,ch);
			v = hX_5._E;
			if (s.charAt(ch)!=hX_5._SL && s.charAt(ch)!=hX_5._GT) {
				ch = hX_5.htmlSkipWS(s,ch);
				if (s.charAt(ch)==hX_5._EQ) ch++;
				chL = ch = hX_5.htmlSkipWS(s,ch);
				if (s.charAt(ch)==hX_5._SQT) {
					chL++;ch++;
					while (ch<l && s.charAt(ch)!=hX_5._SQT) ch++;
					v = s.substring(chL,ch);
					ch++;
				} else if (s.charAt(ch)==hX_5._DQT) {
					chL++;ch++;
					while (ch<l && s.charAt(ch)!=hX_5._DQT) ch++;
					v = s.substring(chL,ch);
					ch++;
				} else {
					ch = hX_5.htmlSkipNotWS(s, ch, hX_5._SL, hX_5._GT);
					v = s.substring(chL,ch);
				}
			}
			if (saveArgs!=null && saveArgs) {
				attN.push(a);
				attV.push(v);
			}			
			ch = hX_5.htmlSkipWS(s,ch);
		}
	}
	if (s.charAt(ch)==hX_5._SL) {ended=true; ch++};
	ch = hX_5.htmlSkipWS(s,ch);
	if (s.charAt(ch)==hX_5._GT) ch++;
	// Package the return
	if (saveArgs!=null && saveArgs) {
		return{ch:ch, tag:hX_5.tagCase(tag), ended:ended, attN:attN, attV:attV};
	} else {
		return{ch:ch, tag:hX_5.tagCase(tag), ended:ended};
	}
}		

// pass in the dom object you want to check the dir of, 
// if dir not set on it, it will check the parents' dir until it find one set 
// or there are no more parents. Returns the first dir value hit. 
// returns hX_5._RTL or hX_5._LTR. If not found it always returns hX_5._LTR.
HxG_5.prototype.findHonourableDir = function(obj) {
	 if(obj == null) return null;
	 var curr = obj, curr_dir = null, thedir = hX_5._LTR, notfound = true;
	 do{
	 	 if(curr.dir != null) curr_dir = curr.dir;
		 if(curr_dir!= null && curr_dir != hX_5._E)
		 {
		 	 if(curr_dir.toLowerCase() == hX_5._LTR)
		 	 {
		 	 	thedir = hX_5._LTR;
		 	 	notfound = false;
		 	 }		 	 
		 	 else if(curr_dir.toLowerCase() == hX_5._RTL)
		 	 {
		 	 	thedir = hX_5._RTL;
		 	 	notfound = false;
		 	 }		 		 			 	
		 }
		 curr = curr.parentNode;	
	 }while(notfound && (curr != null))
	 return thedir;	 
}

// --------------------------------------------------------
// CORE:	Instantiate a JSF Global Client Object
// --------------------------------------------------------
// Instantiate one and only one instance -- this is the only object in the global namespace
if ((typeof hX_5) == 'undefined' || !hX_5) { var hX_5 = new HxG_5(); }
// Assistance for end-user programmers: add an "hX" variable to the page that's
// tied to the current version
if ((typeof hX) == 'undefined' || !hX) { var hX = hX_5; }
